!*****************************************************************************
!                   Time-stamp: <2023-07-05 17:03:10 sander>
!      Author: Rolf Sander, Max-Planck Institute Mainz, Germany
!*****************************************************************************

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or (at
! your option) any later version. This program is distributed in the
! hope that it will be useful, but WITHOUT ANY WARRANTY; without even
! the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
! PURPOSE. See the GNU General Public License for more details. You
! should have received a copy of the GNU General Public License along
! with this program. If not, see: https://www.gnu.org/licenses/

!*****************************************************************************

MODULE henry_psql

  USE henry_mem
  IMPLICIT NONE
  SAVE
  PUBLIC

CONTAINS

  !---------------------------------------------------------------------------

  SUBROUTINE psql_initialize
    IMPLICIT NONE

    OPEN (IO_PSQL, FILE="tmp_henry_header.sql")
    WRITE(IO_PSQL,"(A)") "-- -*- mode: sql; sql-product: postgres; -*-"
    WRITE(IO_PSQL,*)
    WRITE(IO_PSQL,"(A)") "DROP TABLE IF EXISTS henry_notes   CASCADE;"
    WRITE(IO_PSQL,"(A)") "DROP TABLE IF EXISTS henry         CASCADE;"
    WRITE(IO_PSQL,"(A)") "DROP TABLE IF EXISTS species       CASCADE;"
    WRITE(IO_PSQL,"(A)") "DROP TABLE IF EXISTS subcategories CASCADE;"
    WRITE(IO_PSQL,"(A)") "DROP TABLE IF EXISTS categories    CASCADE;"
    WRITE(IO_PSQL,"(A)") "DROP TABLE IF EXISTS notes         CASCADE;"
    WRITE(IO_PSQL,"(A)") "DROP TABLE IF EXISTS literature    CASCADE;"
    WRITE(IO_PSQL,"(A)") "CREATE TABLE notes (id SERIAL PRIMARY KEY,"
    WRITE(IO_PSQL,"(A)") "    notelabel TEXT,"
    WRITE(IO_PSQL,"(A)") "    noterefs  TEXT,"
    WRITE(IO_PSQL,"(A)") "    notetext  TEXT);"
    WRITE(IO_PSQL,"(A)") "CREATE TABLE literature (id SERIAL PRIMARY KEY,"
    WRITE(IO_PSQL,"(A)") "    bibtexkey    TEXT,"
    WRITE(IO_PSQL,"(A)") "    quotation    TEXT,"
    WRITE(IO_PSQL,"(A)") "    bibliography TEXT,"
    WRITE(IO_PSQL,"(A)") "    doi          TEXT);"
    WRITE(IO_PSQL,"(A)") "CREATE TABLE categories (id SERIAL PRIMARY KEY,"
    WRITE(IO_PSQL,"(A)") "    catname TEXT);"
    WRITE(IO_PSQL,"(A)") "CREATE TABLE subcategories (id SERIAL PRIMARY KEY,"
    WRITE(IO_PSQL,"(A)") "    subcatname TEXT,"
    WRITE(IO_PSQL,"(A)") "    cat_id INT REFERENCES categories(id));"
    WRITE(IO_PSQL,"(A)") "CREATE TABLE species (id SERIAL PRIMARY KEY,"
    WRITE(IO_PSQL,"(A)") "    iupac    TEXT,"
    WRITE(IO_PSQL,"(A)") "    formula  TEXT,"
    WRITE(IO_PSQL,"(A)") "    trivial  TEXT,"
    WRITE(IO_PSQL,"(A)") "    casrn    TEXT,"
    WRITE(IO_PSQL,"(A)") "    inchikey TEXT,"
    WRITE(IO_PSQL,"(A)") "    subcat_id INT REFERENCES subcategories(id));"
    WRITE(IO_PSQL,"(A)") "CREATE TABLE henry (id SERIAL PRIMARY KEY,"
    WRITE(IO_PSQL,"(A)") "    Hominus TEXT,"
    WRITE(IO_PSQL,"(A)") "    mindHR  TEXT,"
    WRITE(IO_PSQL,"(A)") "    htype   TEXT,"
    WRITE(IO_PSQL,"(A)") "    species_id    INT REFERENCES species(id),"
    WRITE(IO_PSQL,"(A)") "    literature_id INT REFERENCES literature(id));"
    WRITE(IO_PSQL,"(A)") "CREATE TABLE henry_notes (id SERIAL,"
    WRITE(IO_PSQL,"(A)") "    henry_id INT REFERENCES henry(id) ON UPDATE CASCADE,"
    WRITE(IO_PSQL,"(A)") "    notes_id INT REFERENCES notes(id) ON UPDATE CASCADE,"
    WRITE(IO_PSQL,"(A)") "    CONSTRAINT henry_notes_pkey PRIMARY KEY (henry_id, notes_id));"
    WRITE(IO_PSQL,*)
    CLOSE(IO_PSQL)
    OPEN (IO_PSQL, FILE="tmp_henry1.sql")
    WRITE(IO_PSQL,"(A)") "-- -*- mode: sql; sql-product: postgres; -*-"
    WRITE(IO_PSQL,"(A)") "-- created by henry_psql.f90, do not edit!"
    WRITE(IO_PSQL,*)

  END SUBROUTINE psql_initialize

  !---------------------------------------------------------------------------

  SUBROUTINE psql_henrydata (henry, mindHR, htype, iupac, ref)

    IMPLICIT NONE
    CHARACTER(LEN=*), INTENT(IN) :: henry
    CHARACTER(LEN=*), INTENT(IN) :: mindHR
    CHARACTER(LEN=*), INTENT(IN) :: htype
    CHARACTER(LEN=*), INTENT(IN) :: iupac
    CHARACTER(LEN=*), INTENT(IN) :: ref

    WRITE(IO_PSQL,"(A)") "INSERT INTO henry" // &
      "(Hominus, mindHR, htype, species_id, literature_id) VALUES ("
    WRITE(IO_PSQL,"(A)") "    '[ltx+]" // TRIM(henry)  // "[ltx-]',"
    WRITE(IO_PSQL,"(A)") "    '[ltx+]" // TRIM(mindHR) // "[ltx-]',"
    WRITE(IO_PSQL,"(A)") "    '" // TRIM(htype) // "',"
    WRITE(IO_PSQL,"(A)") "    (SELECT id from species WHERE iupac='[ltx+]" // &
      TRIM(iupac) // "[ltx-]'),"
    WRITE(IO_PSQL,"(A)") "    (SELECT id from literature WHERE bibtexkey='" // &
      TRIM(ref) // "'));"
  END SUBROUTINE psql_henrydata

  !---------------------------------------------------------------------------

  SUBROUTINE psql_species(iupac, formula, trivial, casrn, inchikey)

    IMPLICIT NONE
    CHARACTER(LEN=*), INTENT(IN) :: iupac, formula, trivial, casrn, inchikey

    WRITE(IO_PSQL,"(A)") "INSERT INTO species " // &
      "(iupac, formula, trivial, casrn, inchikey, subcat_id) VALUES ("
    WRITE(IO_PSQL,"(A)") "    '[ltx+]" // TRIM(iupac) // "[ltx-]',"
    WRITE(IO_PSQL,"(A)") "    '[ltx+]\chem{" // TRIM(formula)   // "}[ltx-]',"
    WRITE(IO_PSQL,"(A)") "    '[ltx+]" // TRIM(trivial) // "[ltx-]',"
    WRITE(IO_PSQL,"(A)") "    '" // TRIM(casrn)     // "',"
    WRITE(IO_PSQL,"(A)") "    '" // TRIM(inchikey)  // "',"
    WRITE(IO_PSQL,"(A)") "    (SELECT id from subcategories WHERE subcatname='[ltx+]" // &
      TRIM(current_subcatname) // "[ltx-]'));"

  END SUBROUTINE psql_species

  !---------------------------------------------------------------------------

  SUBROUTINE psql_speciesgroup(level, groupname)

    IMPLICIT NONE
    INTEGER,          INTENT(IN) :: level
    CHARACTER(LEN=*), INTENT(IN) :: groupname

    SELECT CASE(level)
    CASE (1)
      current_catname    = groupname
      WRITE(IO_PSQL,"(A)") "INSERT INTO categories (catname) VALUES ("
      WRITE(IO_PSQL,"(A)") "    '[ltx+]" // TRIM(current_catname) // "[ltx-]');"
    CASE (2)
      current_subcatname = groupname
      WRITE(IO_PSQL,"(A)") "INSERT INTO subcategories (subcatname, cat_id) VALUES ("
      WRITE(IO_PSQL,"(A)") "    '[ltx+]" // TRIM(current_subcatname) // "[ltx-]',"
      WRITE(IO_PSQL,"(A)") "    (SELECT id from categories WHERE catname='[ltx+]" &
        // TRIM(current_catname) // "[ltx-]'));"
    END SELECT

  END SUBROUTINE psql_speciesgroup

  !---------------------------------------------------------------------------

  SUBROUTINE psql_note(notelabel, notetext)

    IMPLICIT NONE
    CHARACTER(LEN=*), INTENT(IN) :: notelabel, notetext

    WRITE(IO_PSQL,"(A)") "INSERT INTO notes (notelabel, noterefs, notetext) VALUES ("
    WRITE(IO_PSQL,"(A)") "    '[ltx+]" // TRIM(notelabel)    // "[ltx-]',"
    WRITE(IO_PSQL,"(A)") "    '[bibtexkeylist]', '[ltx+]" // TRIM(notetext) // "[ltx-]');"

  END SUBROUTINE psql_note

  !---------------------------------------------------------------------------

  SUBROUTINE psql_henry_notes(notelabel)

    IMPLICIT NONE
    CHARACTER(LEN=*), INTENT(IN) :: notelabel

    WRITE(IO_PSQL,"(A)") "INSERT INTO henry_notes (henry_id, notes_id) VALUES ("
    WRITE(IO_PSQL,"(A)") "    (SELECT CURRVAL('henry_id_seq')),"
    WRITE(IO_PSQL,"(A)") "    (SELECT id from notes WHERE notelabel='" // &
      TRIM(notelabel) // "'));"

  END SUBROUTINE psql_henry_notes

  !---------------------------------------------------------------------------

!!$  SUBROUTINE psql_refs
!!$
!!$    IMPLICIT NONE
!!$    WRITE(IO_PSQL,"(A)") "INSERT INTO literature (bibtexkey) VALUES ("
!!$    WRITE(IO_PSQL,"(A)") "    '" // TRIM(ref) // "');"
!!$
!!$  END SUBROUTINE psql_refs

  !---------------------------------------------------------------------------

  SUBROUTINE psql_finalize

    IMPLICIT NONE
    CLOSE(IO_PSQL)

  END SUBROUTINE psql_finalize

  !---------------------------------------------------------------------------

END MODULE henry_psql

!*****************************************************************************
!                                  end of file
!*****************************************************************************
