#!/usr/bin/env python3
# -*- coding: utf-8 -*- Time-stamp: <2023-02-13 11:32:06 sander>

# https://www.datatechnotes.com/2020/09/curve-fitting-with-curve-fit-function-in-python.html

import os, sys
from numpy import exp, log
import pandas as pd
from scipy.optimize import curve_fit
import matplotlib.pyplot as plt
import warnings

from sklearn.metrics import r2_score # for calc of r^2
# https://stackoverflow.com/questions/19189362/getting-the-r-squared-value-using-curve-fit

T0 = 298.15

def func_ABC(T, A, B, C):
    return exp(A + B/T + C*log(T))

def fit_data(filename, verbose=False):
    # read csv data file with pandas:
    df   = pd.read_csv(f'../output/Tdep_data/{filename}.dat', header=None, comment='#')
    T    = df.iloc[:,0] # 1st column
    Hscp = df.iloc[:,1] # 2nd column
    # 3-parameter fit:
    with warnings.catch_warnings():
        # ignore the warning "Covariance of the parameters could not be estimated"
        # which always occurs when there are only 3 data points:
        if (len(T)==3):
            warnings.simplefilter('ignore')
        [A, B, C], _ = curve_fit(func_ABC, T, Hscp)
    HscpT  = func_ABC(T, A, B, C)
    HscpT0 = func_ABC(T0, A, B, C)
    r2 = r2_score(Hscp, HscpT)
    mindHR = B - C*(T0)
    if (verbose):
        print(f'{filename}: HscpT0 = {HscpT0:.4f}, mindHR = {mindHR:9.4f}, ' +
              f'r2 = {r2:7.5f}, A = {A:9.4f}, B = {B:10.4f}, C = {C:7.4f}')
    # write regression results into the input directory for henry.f90:
    with open(f'../input/{filename}-ABC.dat', 'w') as outfile:
        print(f'{A} {B} {C} {r2}', file=outfile)
    
if __name__ == '__main__':
    fit_data(sys.argv[1]) # e.g.: ./Tdep_ABC.py ref3781-1333-74-0
