#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import numpy as np
import pandas as pd
import re
import os
from subprocess import PIPE, run

df = pd.read_csv('ref3409-sp.csv', skiprows=1)

def print_database_info():
    print(df.shape)   # -> (3414, 46)
    print(df.columns)
    # -> ['ID', 'MCM_ID', 'SMILES', 'structure', 'MF', 'MW', 'PRECURSOR',
    # 'oxidation genration', 'C', 'H', 'N', 'O', 'Cl', 'S', 'Br',
    # 'Number of Atoms', 'O:C', 'average carbon oxidation state', 'AROMATIC',
    # 'ROH', 'RCHO', 'RCOR'', 'RCOOH', 'RCOOR'', 'ROR'', 'NO2', 'NO3', 'PAN',
    # 'RCOOOH', 'ROOH& ROOR'', 'Number of functional groups', 'Pi2 (S)',
    # 'LogL16 (L)', 'McGowan Volume (V)', 'BetaH2 (B)', 'AlphaH2 (A)',
    # 'R2 (E)', 'ppLFER_logKWIOM/G', 'ppLFER_logKW/G', 'SPARC_logKWIOM/G',
    # 'SPARC_logKW/G', 'COSMOtherm_logKWIOM/G', 'COSMOtherm_logKW/G',
    # 'COSMOtherm vapor pressure (kPa)',
    # 'COSMOtherm activity coefficients in WIOM',
    # 'COSMOtherm activity coefficient in water']
    # ID:
    print(df[["MCM_ID", "SMILES"]])
    # chemical composition:
    print(df[["MF", "MW", "C", "H", "N", "O", "Cl", "S", "Br", "Number of Atoms"]])
    # chemical functional groups:
    print(df[["AROMATIC", "ROH", "RCHO", "RCOR'", "RCOOH", "RCOOR'", "ROR'", "NO2",
             "NO3", "PAN", "RCOOOH", "ROOH& ROOR'"]])
    # calculated Henry's law constants (KW/G = c_w/c_g -> Hscc):
    print(df[["ppLFER_logKW/G", "SPARC_logKW/G", "COSMOtherm_logKW/G"]])
    # https://pandas.pydata.org/pandas-docs/stable/reference/api/pandas.DataFrame.loc.html
    print(df.loc[0:2, 'C':'Br'])       # loc = locate [lines, cols]

def grep_i(searchstring, filename):
    # suffix _i indicates grep option '-i' for case insensitive
    results = []
    with open(filename) as f:
        for line in f:
            if (searchstring in line):
                results.append(line.strip())
    return results
    
def print_info():
    print('\nMCM:                  %s' % (mcmname))
    print('Formula:              %s' % (formula))
    print('SMILES:               %s' % (row['SMILES']))
    print('Inchi:                %s' % (inchi))
    print('Inchikey:             %s' % (inchikey))
    print('Picture:              http://mcm.york.ac.uk/pics/species/%s.png' % (
        row['MCM_ID']))        
    print('lg(Hscc(ppLFER)):     %g' % (row["ppLFER_logKW/G"]))
    print('lg(Hscc(SPARC)):      %g' % (row["SPARC_logKW/G"]))
    print('lg(Hscc(COSMOtherm)): %g' % (row["COSMOtherm_logKW/G"]))

def get_casrn():
    result = grep_i(inchikey, '../../henry.f90')
    if (result):
        return re.sub('"([^"]*)".*', r'\1', result[0], flags=re.IGNORECASE)
    else:
        return ''
        
def get_inchikey():
    command = 'echo "%s" | babel -xw -ismi -oinchikey' % (row['SMILES'])
    output = run(command, shell=True, stdout=PIPE, stderr=PIPE, universal_newlines=True)
    inchikey = output.stdout.rstrip('\n')
    # correct some inchikeys that were not calculated correctly because
    # SMILES in csv do not include stereoisomers:
    if(mcmname=='CBUT2ENE'):
        inchikey = 'IAQRGUVFOMOMEM-ARJAWSKDSA-N'
    if(mcmname=='TBUT2ENE'):
        inchikey = 'IAQRGUVFOMOMEM-ONEGZZNKSA-N'
    if(mcmname=='CPENT2ENE'):
        inchikey = 'QMMOXUPEWRXHJS-HYXAFXHYSA-N'
    if(mcmname=='TPENT2ENE'):
        inchikey = 'QMMOXUPEWRXHJS-HWKANZROSA-N'
    if(mcmname=='CDICLETH'):
        inchikey = 'KFUSEUYYWQURPO-UPHRSURJSA-N'
    if(mcmname=='TDICLETH'):
        inchikey = 'KFUSEUYYWQURPO-OWOJBTEDSA-N'
    if(mcmname=='CHEX2ENE'):
        inchikey = 'RYPKRALMXUUNKS-HYXAFXHYSA-N' # 7688-21-3     
    if(mcmname=='THEX2ENE'):
        inchikey = 'RYPKRALMXUUNKS-HWKANZROSA-N' # 4050-45-7  
    if(mcmname=='BCARY'):
        inchikey = 'NPNUFJAVOOONJE-IOMPXFEGSA-N' # 87-44-5
    return inchikey
        
if __name__ == '__main__':

    f_spc  = open('ref3409-species-raw.f90', 'w')
    f_dat  = open('ref3409.dat',             'w')
    f_html = open('ref3409-raw.html',        'w')
    # print code to activate CALL Species commands in emacs later:
    print(r'!0000000  (query-replace-regexp ".*!!!" "  "     nil nil nil nil)', file=f_spc)
    print(r'!0000000  (query-replace        "&&&"   "&\n   " nil nil nil nil)', file=f_spc)

    # print_database_info()

    for index, row in df.iterrows():
        #if (index>30): break #qqq only a few lines for testing
        mcmname = row['MCM_ID']
        # skip duplicate entries:
        if (mcmname=='CHOMOHCO3H'): continue # same as COHM2CO3H  
        if (mcmname=='CHOMOHPAN'):  continue # same as COHM2PAN   
        if (mcmname=='CO2N3CHO'):   continue # same as C4CONO3CO  
        if (mcmname=='IEB1OOH'):    continue # same as C57OOH     
        if (mcmname=='IEC1OOH'):    continue # same as C59OOH     
        if (mcmname=='MACRNBCO3H'): continue # same as MACRNBCO2H 
        if (mcmname=='MACROHNO3'):  continue # same as MACRNB     
        formula = row['MF']
        # run babel to get inchi:
        command = 'echo "%s" | babel -xw -ismi -oinchi' % (row['SMILES'])
        output = run(command, shell=True, stdout=PIPE, stderr=PIPE, universal_newlines=True)
        inchi = output.stdout.rstrip('\n')
        # run babel to get inchikey:
        inchikey = get_inchikey()
        # get casrn, if species is already in henry.f90:
        casrn = get_casrn()
        # H values:
        lfer  = row["ppLFER_logKW/G"]
        sparc = row["SPARC_logKW/G"]
        cosmo = row["COSMOtherm_logKW/G"]

        # elemental composition:
        n_C  = row["C"]
        n_H  = row["H"]
        n_O  = row["O"]
        n_N  = row["N"]
        n_Cl = row["Cl"]
        n_Br = row["Br"]
        n_S  = row["S"]
        #print(f' S{n_S:02} Br{n_Br:02} Cl{n_Cl:02} N{n_N:02} O{n_O:02} C{n_C:02}')
        
        # logical variables: is element included (1) or not (0)?
        l_C  = np.sign(n_C) 
        l_H  = np.sign(n_H) 
        l_O  = np.sign(n_O) 
        l_N  = np.sign(n_N) 
        l_Cl = np.sign(n_Cl)
        l_Br = np.sign(n_Br)
        l_S  = np.sign(n_S) 
        
        ethr = np.sign(row["ROR'"])
        estr = np.sign(row["RCOOR'"])
        acid = np.sign(row["RCOOH"])
        keto = np.sign(row["RCOR'"])
        alde = np.sign(row["RCHO"])
        alco = np.sign(row["ROH"])
        arom = np.sign(row["AROMATIC"])

        # create sum formula in TeX format, based on the elemental composition:
        formula_tex = ''
        for element in ['C', 'H', 'N', 'O', 'Cl', 'Br', 'S']:
            n_element = int(row[element])
            if (n_element>0):
                formula_tex += element
                if (n_element>1):
                    if (n_element<10):
                        formula_tex += '_' + str(n_element)
                    else:
                        formula_tex += '_{' + str(n_element) + '}'

        # check internal consistency of row['MF'] and rows with elemental composition:
        formula3 = ''
        for element in ['C', 'H', 'Cl', 'Br', 'N', 'O', 'S']:
            n_element = int(row[element])
            if (n_element>0):
                formula3 += element
                if (n_element>1):
                    formula3 += str(n_element)
        if (formula != formula3):
            print(f'{mcmname:15} {formula:10} {formula3:10}')
 
        sortstring = f'{l_S}{l_Br}{l_Cl}{l_N}{l_O}{l_C} ' + \
              f'{ethr}{estr}{acid}{keto}{alde}{alco}{arom} {n_C:02}'
        if ((casrn) or (mcmname=='IEPOXB')):
            print('!' + sortstring +
                  f' MCM:{mcmname:15}  {casrn:12}    {formula:10}', file=f_spc)
        else:
            # create "CALL Species" command for new species:
            quoted_mcmname     = f'"MCM:'+mcmname+'",'
            quoted_formula_tex = f'"'+formula_tex+'",'
            print(' ' + sortstring +
                  f' !!!CALL Species({quoted_mcmname:20} {quoted_formula_tex:25} "", &&&' +
                  f' "", "{inchikey:27}")', file=f_spc)
            # create web page for visualizing the new species:
            print('<TR><TD>' + sortstring +
                  f'</TD><TD>{mcmname}</TD><TD>{formula}</TD><TD>{inchikey}</TD><TD>' +
                  f'<img src="http://mcm.york.ac.uk/pics/species/{mcmname}.png"></TD></TR>',
                  file=f_html)

        # print 3 calculated Henry's law constants to data file:
        print(f'{lfer:8} {sparc:8} {cosmo:8} "{inchikey}" "MCM:{mcmname}"', file=f_dat)
  
    #print_info()
    f_spc.close()
    f_dat.close()
    f_html.close()

    # create f90 file that define the MCM species with "CALL Species":
    os.system('sort ref3409-species-raw.f90 > ref3409-species.f90')
    os.remove('ref3409-species-raw.f90')

    # create html file to show pictures of the species:
    os.system('echo "<HTML><BODY><TABLE border=1 cellpadding=2 cellspacing=2>"' +
              ' > ref3409.html')
    os.system('sort ref3409-raw.html >> ref3409.html')
    os.system('echo "</TABLE></BODY></HTML>" >> ref3409.html')
    os.remove('ref3409-raw.html')
