#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import matplotlib.pyplot as plt
import numpy as np

a = 0.04145
b = 0.06583
T0 = 298.15

def H(T):
    return 1E-3 * a * np.exp(b*(T-273.15))

if __name__ == '__main__':

    # 100 linearly spaced numbers from 273 K to 323 K:
    T = np.linspace(273, 323, 100)

    # The analytical derivative mindHR = d lnH / d (1/T) at T0 is:
    mindHR = -b*T0**2
    
    # make plot:
    plt.figure(figsize=(11.69,8.27), dpi=100)
    # the fuction as given by ref3659 in red:
    plt.plot(T, H(T), 'r')
    # the normal temperature-dependence with -Delta H/R in blue:
    print(H(T0), mindHR)
    plt.plot(T, H(T0)*np.exp(mindHR*(1/T-1/T0)), 'b')
    # add marker at T0:
    plt.scatter(T0, H(T0), c='black', marker='o')
    # save fig:
    plt.savefig(f'ref3659.pdf')

##############################################################################
