#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import matplotlib.pyplot as plt
import numpy as np

# 100 linearly spaced numbers:
T = np.linspace(273, 363, 100)

def H(param,T):
    A = param[0]
    B = param[1]
    C = param[2]
    return np.exp(A - B/T + C*np.log(T)) # np.log = ln

def makeplot(chem, litreview, combined, measured, Traw, Hraw):
    CtoK = 273.15
    plt.figure(figsize=(11.69,8.27), dpi=100)
    # red line,   Tab. 1, literature review:
    plt.plot(T, H(litreview, T), 'r')
    # blue line,  Tab. 1: combined with data measured in this study:
    plt.plot(T, H(combined,  T), 'b')
    # green line, Tab. 2, calculated regression parameters A, B, C:
    plt.plot(T, H(measured,  T), 'g')
    # green dots, Tab. 2, measured in this study from 10...90 C:
    plt.scatter(np.array(Traw)+CtoK, np.array(Hraw), color='g')
    plt.title(chem)
    plt.savefig(f'ref3681-{chem}.pdf')

if __name__ == '__main__':

    makeplot('112-trichloroethane',
             [196.111,  12330.0,  -27.737],
             [111.318,  8598.1,   -15.053],
             [-295.178, -11638.3, 44.461 ],
             [40.,  60.,  80.,  90. ], 
             [0.08, 0.14, 0.23, 0.37]) 

    makeplot('1112-tetrachloroethane',
             [-627.459, -24549.8, 95.262 ],
             [643.414,  34195.7,  -93.240],
             [650.032,  34898.2,  -94.026],
             [10.,  20.,  40.,  50.,  60.,  70.,  80.,  90. ], 
             [0.01, 0.05, 0.22, 0.28, 0.37, 0.42, 0.75, 0.77]) 

    makeplot('chloroethene',
             [-17.713,  1196.2,   3.813 ],
             [-16.658,  1145.5,   3.597 ],
             [-314.560, -13701.6, 47.195],
             [40.,  60.,  70.,  80.,  90. ], 
             [1.43, 2.47, 2.29, 2.5,  4.52]) 

    makeplot('11-dichloroethene',
             [357.062, 18641.4, -51.677],
             [272.503, 14818.7, -39.091],
             [217.685, 12299.0, -30.963],
             [10.,  20.,  30.,  40.,  50.,  60.,  70.,  80.,  90. ],
             [0.57, 0.81, 1.27, 1.64, 2.08, 2.43, 2.83, 3.37, 3.72])

    makeplot('trans-12-dichloroethene',
             [49.275,  5027.6,  -5.850 ],
             [177.760, 10748.7, -25.030],
             [66.774,  5275.5,  -8.767 ],
             [40.,  60.,  80.,  90. ], 
             [0.64, 0.99, 1.54, 1.73]) 

    makeplot('trichloroethene',
             [88.308,  7242.0,  -11.411],
             [110.411, 8218.0,  -14.715],
             [249.668, 14406.5, -35.500],
             [10.,  20.,  30.,  40.,  50.,  60.,  70.,  80.,  90. ], 
             [0.19, 0.31, 0.51, 0.72, 0.96, 1.24, 1.49, 1.88, 2.08]) 

    makeplot('tetrachloroethene',
             [269.309, 15787.9, -38.035],
             [306.515, 17408.7, -43.611],
             [310.103, 17321.9, -44.295],
             [10.,  20.,  30.,  40.,  50.,  60.,  70.,  80.,  90.], 
             [0.31, 0.53, 0.91, 1.27, 1.76, 2.23, 2.64, 3.32, 3.7]) 

    makeplot('ethylbenzene', # using 13454.3 instead of -13454.3
             [571.229,  29076.8, -83.347],
             [220.665,  13454.3, -31.024],
             [107.956,   8193.7, -14.329],
             [10.,  20.,  30.,  40.,  50.,  60.,  70.,  80.,  90. ],
             [0.15, 0.24, 0.44, 0.59, 0.81, 1.05, 1.45, 1.97, 2.67])

    makeplot('chlorobenzene',
             [322.326, 17338.1,  -46.683],
             [190.288, 11451.5,  -26.972], # instead of [190.288, 11451.5,  -45.382],
             [191.729, 11472.0, -27.188],  # instead of [710.418, 36201.3, -103.629],
             [10., 20.,  30.,  40.,  50.,  60.,  70.,  80.,  90. ],
             [0.1, 0.17, 0.24, 0.28, 0.47, 0.53, 0.63, 0.72, 0.92])

    makeplot('14-dimethylbenzene', # using -12575.3 instead of 12575.3
             [690.635,  34608.8,  -101.065],
             [325.315,  18501.7,  -46.436 ],
             [-305.829, -12575.3, 46.139  ],
             [40.,  60.,  80.,  90. ], 
             [0.58, 0.99, 1.45, 2.32]) 


