!*****************************************************************************
!                   Time-stamp: <2023-09-19 11:10:55 sander>
!      Author: Rolf Sander, Max-Planck Institute Mainz, Germany
!*****************************************************************************

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or (at
! your option) any later version. This program is distributed in the
! hope that it will be useful, but WITHOUT ANY WARRANTY; without even
! the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
! PURPOSE. See the GNU General Public License for more details. You
! should have received a copy of the GNU General Public License along
! with this program. If not, see: https://www.gnu.org/licenses/

!*****************************************************************************

PROGRAM Henry

  USE henry_util,   ONLY: Initialize, PrintGeneralNotes, &
                          SpeciesGroup, Species,         &
                          Middle, Finalize
  USE henry_ref0500 ! references 0001-0500
  USE henry_ref1000 ! references 0501-1000
  USE henry_ref1500 ! references 1001-1500
  USE henry_ref2000 ! references 1501-2000
  USE henry_ref2500 ! references 2001-2500
  USE henry_ref3000 ! references 2501-3000
  USE henry_ref3500 ! references 3001-3500
  USE henry_ref4000 ! references 3501-4000
  IMPLICIT NONE

  !---------------------------------------------------------------------------

  CALL Initialize
  CALL PrintGeneralNotes

  !---------------------------------------------------------------------------

  ! uncomment next line to activate a quick testrun:
  !l_testrun = .TRUE. !qqq
  IF (l_testrun) THEN
    ! for a quick testrun, process only one or a few references:
    CALL CleanUp ; CALL ref3955
  ELSE
    ! go through individual references (ref....), convert their data to the
    ! SI unit [mol m-3 Pa-1], and store the results in the array henry_data:
    CALL CleanUp ; CALL ref_thiswork_BrCl
    CALL CleanUp ; CALL ref_thiswork_Cl2O
    ! (2023)
    CALL CleanUp ; CALL ref3765
    CALL CleanUp ; CALL ref3766
    CALL CleanUp ; CALL ref3769
    ! (2022)
    CALL CleanUp ; CALL ref3676
    CALL CleanUp ; CALL ref3682
    CALL CleanUp ; CALL ref3719
    CALL CleanUp ; CALL ref3726
    CALL CleanUp ; CALL ref3728
    CALL CleanUp ; CALL ref3730
    CALL CleanUp ; CALL ref3776
    CALL CleanUp ; CALL ref3939
    CALL CleanUp ; CALL ref3962
    ! (2021)
    CALL CleanUp ; CALL ref3539
    CALL CleanUp ; CALL ref3506
    CALL CleanUp ; CALL ref3507
    CALL CleanUp ; CALL ref3521
    CALL CleanUp ; CALL ref3300
    CALL CleanUp ; CALL ref3556
    CALL CleanUp ; CALL ref3575
    CALL CleanUp ; CALL ref3582
    CALL CleanUp ; CALL ref3681
    CALL CleanUp ; CALL ref3695
    CALL CleanUp ; CALL ref3753
    CALL CleanUp ; CALL ref3963
    ! (2020)
    CALL CleanUp ; CALL ref3517
    CALL CleanUp ; CALL ref3563
    CALL CleanUp ; CALL ref3586
    CALL CleanUp ; CALL ref3723
    ! (2019)
    CALL CleanUp ; CALL ref3424
    CALL CleanUp ; CALL ref3500
    CALL CleanUp ; CALL ref3525
    CALL CleanUp ; CALL ref3570
    CALL CleanUp ; CALL ref3627
    CALL CleanUp ; CALL ref3632
    CALL CleanUp ; CALL ref3771
    CALL CleanUp ; CALL ref3788
    ! (2018)
    CALL CleanUp ; CALL ref3407
    CALL CleanUp ; CALL ref3578
    CALL CleanUp ; CALL ref3680
    CALL CleanUp ; CALL ref3964
    ! (2017)
    CALL CleanUp ; CALL ref3248
    CALL CleanUp ; CALL ref3271
    CALL CleanUp ; CALL ref3272
    CALL CleanUp ; CALL ref3404
    CALL CleanUp ; CALL ref3409
    CALL CleanUp ; CALL ref3426
    CALL CleanUp ; CALL ref3678
    CALL CleanUp ; CALL ref3683
    ! (2016)
    CALL CleanUp ; CALL ref3246
    CALL CleanUp ; CALL ref3306
    CALL CleanUp ; CALL ref3411
    CALL CleanUp ; CALL ref3413
    CALL CleanUp ; CALL ref3508
    CALL CleanUp ; CALL ref3684
    CALL CleanUp ; CALL ref3732
    ! (2015)
    CALL CleanUp ; CALL ref3088
    CALL CleanUp ; CALL ref3111
    CALL CleanUp ; CALL ref3215
    CALL CleanUp ; CALL ref3216
    CALL CleanUp ; CALL ref3245
    CALL CleanUp ; CALL ref3400
    CALL CleanUp ; CALL ref3423
    CALL CleanUp ; CALL ref3550
    CALL CleanUp ; CALL ref3562
    CALL CleanUp ; CALL ref3581
    CALL CleanUp ; CALL ref3685
    CALL CleanUp ; CALL ref3686
    ! (2014)
    CALL CleanUp ; CALL ref3006
    CALL CleanUp ; CALL ref3022
    CALL CleanUp ; CALL ref3024
    CALL CleanUp ; CALL ref3025
    CALL CleanUp ; CALL ref3032
    CALL CleanUp ; CALL ref3034
    CALL CleanUp ; CALL ref3064
    CALL CleanUp ; CALL ref3067
    CALL CleanUp ; CALL ref3093
    CALL CleanUp ; CALL ref3094
    CALL CleanUp ; CALL ref3410
    CALL CleanUp ; CALL ref3689
    CALL CleanUp ; CALL ref3760
    ! (2013)
    CALL CleanUp ; CALL ref3518
    CALL CleanUp ; CALL ref2891
    CALL CleanUp ; CALL ref2921
    CALL CleanUp ; CALL ref2926
    CALL CleanUp ; CALL ref2939
    CALL CleanUp ; CALL ref2987
    CALL CleanUp ; CALL ref2999
    CALL CleanUp ; CALL ref3007
    CALL CleanUp ; CALL ref3008
    CALL CleanUp ; CALL ref3009
    CALL CleanUp ; CALL ref3033
    CALL CleanUp ; CALL ref3089
    CALL CleanUp ; CALL ref3010
    CALL CleanUp ; CALL ref3583
    CALL CleanUp ; CALL ref3584
    CALL CleanUp ; CALL ref3655
    ! (2012)
    CALL CleanUp ; CALL ref2824
    CALL CleanUp ; CALL ref2828
    CALL CleanUp ; CALL ref2927
    CALL CleanUp ; CALL ref2990
    CALL CleanUp ; CALL ref3014
    CALL CleanUp ; CALL ref3015
    CALL CleanUp ; CALL ref3087
    CALL CleanUp ; CALL ref3096
    CALL CleanUp ; CALL ref3098
    CALL CleanUp ; CALL ref3117
    CALL CleanUp ; CALL ref3406
    CALL CleanUp ; CALL ref3770
    ! (2011)
    CALL CleanUp ; CALL ref2626
    CALL CleanUp ; CALL ref2805
    CALL CleanUp ; CALL ref2817
    CALL CleanUp ; CALL ref2823
    CALL CleanUp ; CALL ref2991
    CALL CleanUp ; CALL ref3013
    CALL CleanUp ; CALL ref3262
    CALL CleanUp ; CALL ref3524
    CALL CleanUp ; CALL ref3549
    CALL CleanUp ; CALL ref3564
    CALL CleanUp ; CALL ref3666
    CALL CleanUp ; CALL ref3951
    ! (2010)
    CALL CleanUp ; CALL ref2546
    CALL CleanUp ; CALL ref2822
    CALL CleanUp ; CALL ref2827
    CALL CleanUp ; CALL ref2982
    CALL CleanUp ; CALL ref2989
    CALL CleanUp ; CALL ref2994
    CALL CleanUp ; CALL ref2995
    CALL CleanUp ; CALL ref3097
    CALL CleanUp ; CALL ref3523
    CALL CleanUp ; CALL ref3526
    CALL CleanUp ; CALL ref3533
    CALL CleanUp ; CALL ref3572
    CALL CleanUp ; CALL ref3579
    CALL CleanUp ; CALL ref3580
    CALL CleanUp ; CALL ref3633
    CALL CleanUp ; CALL ref3665
    CALL CleanUp ; CALL ref3691
    CALL CleanUp ; CALL ref3729
    ! (2009)
    CALL CleanUp ; CALL ref2401
    CALL CleanUp ; CALL ref2563
    CALL CleanUp ; CALL ref2821
    CALL CleanUp ; CALL ref2826
    CALL CleanUp ; CALL ref2645
    CALL CleanUp ; CALL ref3134
    ! (2008)
    CALL CleanUp ; CALL ref2360
    CALL CleanUp ; CALL ref2521
    CALL CleanUp ; CALL ref2571
    CALL CleanUp ; CALL ref2577
    CALL CleanUp ; CALL ref2601
    CALL CleanUp ; CALL ref2815
    CALL CleanUp ; CALL ref2816
    CALL CleanUp ; CALL ref2820
    CALL CleanUp ; CALL ref2925
    CALL CleanUp ; CALL ref2955
    CALL CleanUp ; CALL ref2970
    CALL CleanUp ; CALL ref3116
    CALL CleanUp ; CALL ref3135
    CALL CleanUp ; CALL ref3302
    CALL CleanUp ; CALL ref3392
    CALL CleanUp ; CALL ref3393
    CALL CleanUp ; CALL ref3555
    CALL CleanUp ; CALL ref3654
    CALL CleanUp ; CALL ref3724
    ! (2007)
    CALL CleanUp ; CALL ref1855
    CALL CleanUp ; CALL ref2183
    CALL CleanUp ; CALL ref2371
    CALL CleanUp ; CALL ref2406
    CALL CleanUp ; CALL ref2524
    CALL CleanUp ; CALL ref2526
    CALL CleanUp ; CALL ref2530
    CALL CleanUp ; CALL ref2566
    CALL CleanUp ; CALL ref2580
    CALL CleanUp ; CALL ref2597
    CALL CleanUp ; CALL ref2600
    CALL CleanUp ; CALL ref2630
    CALL CleanUp ; CALL ref2804
    CALL CleanUp ; CALL ref2829
    CALL CleanUp ; CALL ref2932
    CALL CleanUp ; CALL ref2954
    CALL CleanUp ; CALL ref2997
    CALL CleanUp ; CALL ref3115
    CALL CleanUp ; CALL ref3118
    CALL CleanUp ; CALL ref3305
    CALL CleanUp ; CALL ref3527
    CALL CleanUp ; CALL ref3554
    ! (2006)
    CALL CleanUp ; CALL ref1831
    CALL CleanUp ; CALL ref1945
    CALL CleanUp ; CALL ref2122
    CALL CleanUp ; CALL ref2123
    CALL CleanUp ; CALL ref2492
    CALL CleanUp ; CALL ref2564
    CALL CleanUp ; CALL ref2565
    CALL CleanUp ; CALL ref2599
    CALL CleanUp ; CALL ref2611_2614
    CALL CleanUp ; CALL ref2814
    CALL CleanUp ; CALL ref2819
    CALL CleanUp ; CALL ref2992
    CALL CleanUp ; CALL ref2993
    CALL CleanUp ; CALL ref3065
    CALL CleanUp ; CALL ref3112
    CALL CleanUp ; CALL ref3304
    CALL CleanUp ; CALL ref3589
    CALL CleanUp ; CALL ref3621
    CALL CleanUp ; CALL ref3694
    CALL CleanUp ; CALL ref3948
    CALL CleanUp ; CALL ref3950
    ! (2005)
    CALL CleanUp ; CALL ref1571
    CALL CleanUp ; CALL ref1917
    CALL CleanUp ; CALL ref1929
    CALL CleanUp ; CALL ref1936
    CALL CleanUp ; CALL ref1937
    CALL CleanUp ; CALL ref2124
    CALL CleanUp ; CALL ref2125
    CALL CleanUp ; CALL ref2169
    CALL CleanUp ; CALL ref2212
    CALL CleanUp ; CALL ref2285
    CALL CleanUp ; CALL ref2321
    CALL CleanUp ; CALL ref2447
    CALL CleanUp ; CALL ref2468
    CALL CleanUp ; CALL ref2496
    CALL CleanUp ; CALL ref2598
    CALL CleanUp ; CALL ref2629
    CALL CleanUp ; CALL ref2802
    CALL CleanUp ; CALL ref2825
    CALL CleanUp ; CALL ref2915
    CALL CleanUp ; CALL ref2948
    CALL CleanUp ; CALL ref3214
    CALL CleanUp ; CALL ref3303
    CALL CleanUp ; CALL ref3512
    CALL CleanUp ; CALL ref3610
    CALL CleanUp ; CALL ref3618
    CALL CleanUp ; CALL ref3690
    CALL CleanUp ; CALL ref3780
    ! (2004)
    CALL CleanUp ; CALL ref1658
    CALL CleanUp ; CALL ref1721
    CALL CleanUp ; CALL ref1932
    CALL CleanUp ; CALL ref1942
    CALL CleanUp ; CALL ref2469
    CALL CleanUp ; CALL ref2483
    CALL CleanUp ; CALL ref2488
    CALL CleanUp ; CALL ref2489
    CALL CleanUp ; CALL ref2554
    CALL CleanUp ; CALL ref2646
    CALL CleanUp ; CALL ref2647
    CALL CleanUp ; CALL ref2803
    CALL CleanUp ; CALL ref2818
    CALL CleanUp ; CALL ref2913
    CALL CleanUp ; CALL ref2936
    CALL CleanUp ; CALL ref2965
    CALL CleanUp ; CALL ref2978
    CALL CleanUp ; CALL ref2996
    CALL CleanUp ; CALL ref3129
    CALL CleanUp ; CALL ref3616
    CALL CleanUp ; CALL ref3671
    ! (2003)
    CALL CleanUp ; CALL ref1773
    CALL CleanUp ; CALL ref1856
    CALL CleanUp ; CALL ref1922
    CALL CleanUp ; CALL ref1923
    CALL CleanUp ; CALL ref1931
    CALL CleanUp ; CALL ref1940
    CALL CleanUp ; CALL ref1943
    CALL CleanUp ; CALL ref2105
    CALL CleanUp ; CALL ref2216
    CALL CleanUp ; CALL ref2432
    CALL CleanUp ; CALL ref2438
    CALL CleanUp ; CALL ref2535
    CALL CleanUp ; CALL ref2536
    CALL CleanUp ; CALL ref2540
    CALL CleanUp ; CALL ref2808
    CALL CleanUp ; CALL ref2897
    CALL CleanUp ; CALL ref2899
    CALL CleanUp ; CALL ref2923
    CALL CleanUp ; CALL ref3532
    CALL CleanUp ; CALL ref3612
    CALL CleanUp ; CALL ref3613
    CALL CleanUp ; CALL ref3642
    CALL CleanUp ; CALL ref3650
    CALL CleanUp ; CALL ref3812
    ! (2002)
    CALL CleanUp ; CALL ref1513
    CALL CleanUp ; CALL ref1776
    CALL CleanUp ; CALL ref1935
    CALL CleanUp ; CALL ref2472
    CALL CleanUp ; CALL ref2487
    CALL CleanUp ; CALL ref2494
    CALL CleanUp ; CALL ref2497
    CALL CleanUp ; CALL ref2940
    CALL CleanUp ; CALL ref2949
    CALL CleanUp ; CALL ref2953
    CALL CleanUp ; CALL ref2963
    CALL CleanUp ; CALL ref3528
    CALL CleanUp ; CALL ref3536
    CALL CleanUp ; CALL ref3605
    CALL CleanUp ; CALL ref3606
    CALL CleanUp ; CALL ref3609
    CALL CleanUp ; CALL ref3643
    CALL CleanUp ; CALL ref3670
    CALL CleanUp ; CALL ref3693
    CALL CleanUp ; CALL ref3731
    ! (2001)
    CALL CleanUp ; CALL ref1323
    CALL CleanUp ; CALL ref1514
    CALL CleanUp ; CALL ref1516
    CALL CleanUp ; CALL ref1524
    CALL CleanUp ; CALL ref1525
    CALL CleanUp ; CALL ref1930
    CALL CleanUp ; CALL ref2512
    CALL CleanUp ; CALL ref2522
    CALL CleanUp ; CALL ref2532
    CALL CleanUp ; CALL ref2543
    CALL CleanUp ; CALL ref2610
    CALL CleanUp ; CALL ref2969
    CALL CleanUp ; CALL ref3081
    CALL CleanUp ; CALL ref3132
    CALL CleanUp ; CALL ref3611
    CALL CleanUp ; CALL ref3617
    CALL CleanUp ; CALL ref3636
    CALL CleanUp ; CALL ref3674
    CALL CleanUp ; CALL ref3940
    ! (2000)
    CALL CleanUp ; CALL ref1026
    CALL CleanUp ; CALL ref1217
    CALL CleanUp ; CALL ref1442
    CALL CleanUp ; CALL ref1502
    CALL CleanUp ; CALL ref1573
    CALL CleanUp ; CALL ref1574
    CALL CleanUp ; CALL ref1928
    CALL CleanUp ; CALL ref2165
    CALL CleanUp ; CALL ref2338
    CALL CleanUp ; CALL ref2340
    CALL CleanUp ; CALL ref2837
    CALL CleanUp ; CALL ref2896
    CALL CleanUp ; CALL ref2910
    CALL CleanUp ; CALL ref2929
    CALL CleanUp ; CALL ref2957
    CALL CleanUp ; CALL ref2961
    CALL CleanUp ; CALL ref3073
    CALL CleanUp ; CALL ref3261
    CALL CleanUp ; CALL ref3412
    CALL CleanUp ; CALL ref3414
    CALL CleanUp ; CALL ref3531
    CALL CleanUp ; CALL ref3534
    CALL CleanUp ; CALL ref3644
    CALL CleanUp ; CALL ref3672
    CALL CleanUp ; CALL ref3673
    CALL CleanUp ; CALL ref3692
    ! (1999)
    CALL CleanUp ; CALL ref0897
    CALL CleanUp ; CALL ref1004
    CALL CleanUp ; CALL ref1018
    CALL CleanUp ; CALL ref1038
    CALL CleanUp ; CALL ref1096
    CALL CleanUp ; CALL ref1107
    CALL CleanUp ; CALL ref1143
    CALL CleanUp ; CALL ref1182
    CALL CleanUp ; CALL ref1195
    CALL CleanUp ; CALL ref1257
    CALL CleanUp ; CALL ref1996
    CALL CleanUp ; CALL ref2440
    CALL CleanUp ; CALL ref2462
    CALL CleanUp ; CALL ref2477
    CALL CleanUp ; CALL ref2914
    CALL CleanUp ; CALL ref2935
    CALL CleanUp ; CALL ref2938
    CALL CleanUp ; CALL ref2960
    CALL CleanUp ; CALL ref2964
    CALL CleanUp ; CALL ref3003
    CALL CleanUp ; CALL ref3402
    CALL CleanUp ; CALL ref3436
    CALL CleanUp ; CALL ref3653
    CALL CleanUp ; CALL ref3811
    ! (1998)
    CALL CleanUp ; CALL ref0796
    CALL CleanUp ; CALL ref0895
    CALL CleanUp ; CALL ref0896
    CALL CleanUp ; CALL ref0910
    CALL CleanUp ; CALL ref0935
    CALL CleanUp ; CALL ref0937
    CALL CleanUp ; CALL ref0981
    CALL CleanUp ; CALL ref0982
    CALL CleanUp ; CALL ref0985
    CALL CleanUp ; CALL ref0992
    CALL CleanUp ; CALL ref0998
    CALL CleanUp ; CALL ref1024
    CALL CleanUp ; CALL ref1046
    CALL CleanUp ; CALL ref1056
    CALL CleanUp ; CALL ref1110
    CALL CleanUp ; CALL ref1112
    CALL CleanUp ; CALL ref1113
    CALL CleanUp ; CALL ref1114
    CALL CleanUp ; CALL ref1230
    CALL CleanUp ; CALL ref1330
    CALL CleanUp ; CALL ref1481
    CALL CleanUp ; CALL ref2171
    CALL CleanUp ; CALL ref2241
    CALL CleanUp ; CALL ref2286
    CALL CleanUp ; CALL ref2464
    CALL CleanUp ; CALL ref2838
    CALL CleanUp ; CALL ref2950
    CALL CleanUp ; CALL ref3091
    CALL CleanUp ; CALL ref3121
    CALL CleanUp ; CALL ref3571
    CALL CleanUp ; CALL ref3645
    CALL CleanUp ; CALL ref3646
    CALL CleanUp ; CALL ref3764
    ! (1997)
    CALL CleanUp ; CALL ref0536
    CALL CleanUp ; CALL ref0769
    CALL CleanUp ; CALL ref0797
    CALL CleanUp ; CALL ref0837
    CALL CleanUp ; CALL ref0844
    CALL CleanUp ; CALL ref0893
    CALL CleanUp ; CALL ref0921
    CALL CleanUp ; CALL ref0977
    CALL CleanUp ; CALL ref0983
    CALL CleanUp ; CALL ref0984
    CALL CleanUp ; CALL ref1060
    CALL CleanUp ; CALL ref1115
    CALL CleanUp ; CALL ref1200
    CALL CleanUp ; CALL ref2339
    CALL CleanUp ; CALL ref2389
    CALL CleanUp ; CALL ref2450
    CALL CleanUp ; CALL ref2451
    CALL CleanUp ; CALL ref2572
    CALL CleanUp ; CALL ref2806
    CALL CleanUp ; CALL ref2836
    CALL CleanUp ; CALL ref2904
    CALL CleanUp ; CALL ref2905
    CALL CleanUp ; CALL ref2958
    CALL CleanUp ; CALL ref3095
    CALL CleanUp ; CALL ref3124
    CALL CleanUp ; CALL ref3126
    CALL CleanUp ; CALL ref3573
    CALL CleanUp ; CALL ref3657
    CALL CleanUp ; CALL ref3791
    CALL CleanUp ; CALL ref3961
    ! (1996)
    CALL CleanUp ; CALL ref0271
    CALL CleanUp ; CALL ref0446
    CALL CleanUp ; CALL ref0511
    CALL CleanUp ; CALL ref0516
    CALL CleanUp ; CALL ref0548
    CALL CleanUp ; CALL ref0585
    CALL CleanUp ; CALL ref0588
    CALL CleanUp ; CALL ref0619
    CALL CleanUp ; CALL ref0625
    CALL CleanUp ; CALL ref0669
    CALL CleanUp ; CALL ref0670
    CALL CleanUp ; CALL ref0671
    CALL CleanUp ; CALL ref0672
    CALL CleanUp ; CALL ref0747
    CALL CleanUp ; CALL ref0892
    CALL CleanUp ; CALL ref0995
    CALL CleanUp ; CALL ref1536
    CALL CleanUp ; CALL ref1597
    CALL CleanUp ; CALL ref1925
    CALL CleanUp ; CALL ref2431
    CALL CleanUp ; CALL ref2456
    CALL CleanUp ; CALL ref2463
    CALL CleanUp ; CALL ref2930
    CALL CleanUp ; CALL ref2972
    CALL CleanUp ; CALL ref2973
    CALL CleanUp ; CALL ref3590
    CALL CleanUp ; CALL ref3651
    CALL CleanUp ; CALL ref3662
    ! (1995)
    CALL CleanUp ; CALL ref0276
    CALL CleanUp ; CALL ref0315
    CALL CleanUp ; CALL ref0477
    CALL CleanUp ; CALL ref0493
    CALL CleanUp ; CALL ref0498
    CALL CleanUp ; CALL ref0538
    CALL CleanUp ; CALL ref0542
    CALL CleanUp ; CALL ref0583
    CALL CleanUp ; CALL ref0584
    CALL CleanUp ; CALL ref0586
    CALL CleanUp ; CALL ref0617
    CALL CleanUp ; CALL ref0624
    CALL CleanUp ; CALL ref0636
    CALL CleanUp ; CALL ref0923
    CALL CleanUp ; CALL ref1073
    CALL CleanUp ; CALL ref1125
    CALL CleanUp ; CALL ref1484
    CALL CleanUp ; CALL ref1775
    CALL CleanUp ; CALL ref1907
    CALL CleanUp ; CALL ref2013
    CALL CleanUp ; CALL ref2121
    CALL CleanUp ; CALL ref2840
    CALL CleanUp ; CALL ref3569
    CALL CleanUp ; CALL ref3647
    CALL CleanUp ; CALL ref3664
    CALL CleanUp ; CALL ref3947
    ! (1994)
    CALL CleanUp ; CALL ref0311
    CALL CleanUp ; CALL ref0389
    CALL CleanUp ; CALL ref0514
    CALL CleanUp ; CALL ref0622
    CALL CleanUp ; CALL ref0638
    CALL CleanUp ; CALL ref0644
    CALL CleanUp ; CALL ref0997
    CALL CleanUp ; CALL ref1124
    CALL CleanUp ; CALL ref1126
    CALL CleanUp ; CALL ref1127
    CALL CleanUp ; CALL ref1501
    CALL CleanUp ; CALL ref2485
    CALL CleanUp ; CALL ref2542
    CALL CleanUp ; CALL ref2895
    CALL CleanUp ; CALL ref2931
    CALL CleanUp ; CALL ref2967
    CALL CleanUp ; CALL ref3614
    CALL CleanUp ; CALL ref3631
    CALL CleanUp ; CALL ref3869
    ! (1993)
    CALL CleanUp ; CALL ref0599
    CALL CleanUp ; CALL ref0635
    CALL CleanUp ; CALL ref0637
    CALL CleanUp ; CALL ref0799
    CALL CleanUp ; CALL ref0858
    CALL CleanUp ; CALL ref0903
    CALL CleanUp ; CALL ref1155
    CALL CleanUp ; CALL ref1229
    CALL CleanUp ; CALL ref1342
    CALL CleanUp ; CALL ref1912
    CALL CleanUp ; CALL ref2457
    CALL CleanUp ; CALL ref2493
    CALL CleanUp ; CALL ref2527
    CALL CleanUp ; CALL ref2538
    CALL CleanUp ; CALL ref2937
    CALL CleanUp ; CALL ref2945
    CALL CleanUp ; CALL ref3071
    CALL CleanUp ; CALL ref3530
    CALL CleanUp ; CALL ref3544
    ! CALL CleanUp ; CALL ref3790 ! NOTUSED
    ! (1992)
    CALL CleanUp ; CALL ref0171
    CALL CleanUp ; CALL ref0288
    CALL CleanUp ; CALL ref0379
    CALL CleanUp ; CALL ref0507
    CALL CleanUp ; CALL ref0510
    CALL CleanUp ; CALL ref0515
    CALL CleanUp ; CALL ref0541
    CALL CleanUp ; CALL ref0633
    CALL CleanUp ; CALL ref0731
    CALL CleanUp ; CALL ref0789
    CALL CleanUp ; CALL ref0800
    CALL CleanUp ; CALL ref0855
    CALL CleanUp ; CALL ref0856
    CALL CleanUp ; CALL ref0857
    CALL CleanUp ; CALL ref1063
    CALL CleanUp ; CALL ref1142
    CALL CleanUp ; CALL ref1146
    CALL CleanUp ; CALL ref1148
    CALL CleanUp ; CALL ref1162
    CALL CleanUp ; CALL ref2230
    CALL CleanUp ; CALL ref2231
    CALL CleanUp ; CALL ref2435
    CALL CleanUp ; CALL ref2439
    CALL CleanUp ; CALL ref2475
    CALL CleanUp ; CALL ref2525
    CALL CleanUp ; CALL ref2983
    CALL CleanUp ; CALL ref3648
    CALL CleanUp ; CALL ref3675
    CALL CleanUp ; CALL ref3679
    ! (1991)
    CALL CleanUp ; CALL ref0031
    CALL CleanUp ; CALL ref0183
    CALL CleanUp ; CALL ref0205
    CALL CleanUp ; CALL ref0443
    CALL CleanUp ; CALL ref0495
    CALL CleanUp ; CALL ref0502
    CALL CleanUp ; CALL ref0520
    CALL CleanUp ; CALL ref0631
    CALL CleanUp ; CALL ref1145
    CALL CleanUp ; CALL ref1152
    CALL CleanUp ; CALL ref1153
    CALL CleanUp ; CALL ref2014
    CALL CleanUp ; CALL ref2016
    CALL CleanUp ; CALL ref2974
    CALL CleanUp ; CALL ref2977
    CALL CleanUp ; CALL ref3603
    ! (1990)
    CALL CleanUp ; CALL ref0598
    CALL CleanUp ; CALL ref0630
    CALL CleanUp ; CALL ref1144
    CALL CleanUp ; CALL ref1194
    CALL CleanUp ; CALL ref1500
    CALL CleanUp ; CALL ref2270
    CALL CleanUp ; CALL ref2460
    CALL CleanUp ; CALL ref2839
    CALL CleanUp ; CALL ref2966
    CALL CleanUp ; CALL ref2975
    CALL CleanUp ; CALL ref3070
    CALL CleanUp ; CALL ref3090
    CALL CleanUp ; CALL ref3232
    CALL CleanUp ; CALL ref3668
    CALL CleanUp ; CALL ref3866
    CALL CleanUp ; CALL ref4023
    ! (1989)
    CALL CleanUp ; CALL ref0034
    CALL CleanUp ; CALL ref0076
    CALL CleanUp ; CALL ref0530
    CALL CleanUp ; CALL ref0533
    CALL CleanUp ; CALL ref0615
    CALL CleanUp ; CALL ref0620
    CALL CleanUp ; CALL ref0901
    CALL CleanUp ; CALL ref1014
    CALL CleanUp ; CALL ref1141
    CALL CleanUp ; CALL ref1995
    CALL CleanUp ; CALL ref2229
    CALL CleanUp ; CALL ref2436
    CALL CleanUp ; CALL ref2570
    CALL CleanUp ; CALL ref2631
    CALL CleanUp ; CALL ref2901
    CALL CleanUp ; CALL ref2924
    CALL CleanUp ; CALL ref3005
    CALL CleanUp ; CALL ref3076
    ! (1988)
    CALL CleanUp ; CALL ref0067
    CALL CleanUp ; CALL ref0160
    CALL CleanUp ; CALL ref0260
    CALL CleanUp ; CALL ref0484
    CALL CleanUp ; CALL ref0592
    CALL CleanUp ; CALL ref0643
    CALL CleanUp ; CALL ref0854
    CALL CleanUp ; CALL ref0900
    CALL CleanUp ; CALL ref0941
    CALL CleanUp ; CALL ref1140
    CALL CleanUp ; CALL ref1156
    CALL CleanUp ; CALL ref1157
    CALL CleanUp ; CALL ref1498
    CALL CleanUp ; CALL ref2235
    CALL CleanUp ; CALL ref2478
    CALL CleanUp ; CALL ref2482
    CALL CleanUp ; CALL ref2906
    CALL CleanUp ; CALL ref2928
    CALL CleanUp ; CALL ref3078
    CALL CleanUp ; CALL ref3079
    CALL CleanUp ; CALL ref3638
    CALL CleanUp ; CALL ref3649
    CALL CleanUp ; CALL ref3863
    ! (1987)
    CALL CleanUp ; CALL ref0532
    CALL CleanUp ; CALL ref0715
    CALL CleanUp ; CALL ref0759
    CALL CleanUp ; CALL ref0824
    CALL CleanUp ; CALL ref0853
    CALL CleanUp ; CALL ref2243
    CALL CleanUp ; CALL ref2292
    CALL CleanUp ; CALL ref2448
    CALL CleanUp ; CALL ref2449
    CALL CleanUp ; CALL ref2452
    CALL CleanUp ; CALL ref2479
    ! CALL CleanUp ; CALL ref2625 ! NOTUSED
    CALL CleanUp ; CALL ref3545
    CALL CleanUp ; CALL ref3861
    CALL CleanUp ; CALL ref3862
    CALL CleanUp ; CALL ref3946
    ! (1986)
    CALL CleanUp ; CALL ref0019
    CALL CleanUp ; CALL ref0046
    CALL CleanUp ; CALL ref0088
    CALL CleanUp ; CALL ref0189
    CALL CleanUp ; CALL ref0485
    CALL CleanUp ; CALL ref0487
    CALL CleanUp ; CALL ref0508
    CALL CleanUp ; CALL ref0523
    CALL CleanUp ; CALL ref0524
    CALL CleanUp ; CALL ref0629
    CALL CleanUp ; CALL ref0695
    CALL CleanUp ; CALL ref0852
    CALL CleanUp ; CALL ref1071
    CALL CleanUp ; CALL ref2291
    CALL CleanUp ; CALL ref2302
    CALL CleanUp ; CALL ref2903
    CALL CleanUp ; CALL ref2946
    CALL CleanUp ; CALL ref3085
    CALL CleanUp ; CALL ref3535
    CALL CleanUp ; CALL ref3626
    CALL CleanUp ; CALL ref3793
    CALL CleanUp ; CALL ref3860
    CALL CleanUp ; CALL ref3929
    CALL CleanUp ; CALL ref3943
    ! (1985)
    CALL CleanUp ; CALL ref0140
    CALL CleanUp ; CALL ref0167
    CALL CleanUp ; CALL ref0483
    CALL CleanUp ; CALL ref0488
    CALL CleanUp ; CALL ref0582
    CALL CleanUp ; CALL ref0850
    CALL CleanUp ; CALL ref1068
    CALL CleanUp ; CALL ref1139
    CALL CleanUp ; CALL ref1331
    CALL CleanUp ; CALL ref2232
    CALL CleanUp ; CALL ref2240
    CALL CleanUp ; CALL ref2269
    CALL CleanUp ; CALL ref2279
    CALL CleanUp ; CALL ref2290
    CALL CleanUp ; CALL ref2531
    CALL CleanUp ; CALL ref2537
    CALL CleanUp ; CALL ref2576
    CALL CleanUp ; CALL ref2830
    CALL CleanUp ; CALL ref2902
    CALL CleanUp ; CALL ref2912
    CALL CleanUp ; CALL ref2947
    CALL CleanUp ; CALL ref2976
    CALL CleanUp ; CALL ref3110
    CALL CleanUp ; CALL ref3635
    CALL CleanUp ; CALL ref3667
    CALL CleanUp ; CALL ref3734
    ! (1984)
    CALL CleanUp ; CALL ref0074
    CALL CleanUp ; CALL ref0087
    CALL CleanUp ; CALL ref0103
    CALL CleanUp ; CALL ref0209
    CALL CleanUp ; CALL ref0287
    CALL CleanUp ; CALL ref0512
    CALL CleanUp ; CALL ref0513
    CALL CleanUp ; CALL ref0527
    CALL CleanUp ; CALL ref0531
    CALL CleanUp ; CALL ref0601
    CALL CleanUp ; CALL ref0632
    CALL CleanUp ; CALL ref1497
    CALL CleanUp ; CALL ref2433
    CALL CleanUp ; CALL ref2434
    CALL CleanUp ; CALL ref3640
    ! (1983)
    CALL CleanUp ; CALL ref0082
    CALL CleanUp ; CALL ref0203
    CALL CleanUp ; CALL ref0480
    CALL CleanUp ; CALL ref0522
    CALL CleanUp ; CALL ref0608
    CALL CleanUp ; CALL ref0641
    CALL CleanUp ; CALL ref0730
    CALL CleanUp ; CALL ref1076
    CALL CleanUp ; CALL ref2233
    CALL CleanUp ; CALL ref2301
    CALL CleanUp ; CALL ref2444
    CALL CleanUp ; CALL ref2455
    CALL CleanUp ; CALL ref2481
    CALL CleanUp ; CALL ref2533
    CALL CleanUp ; CALL ref2534
    CALL CleanUp ; CALL ref2556
    CALL CleanUp ; CALL ref2911
    CALL CleanUp ; CALL ref2944
    CALL CleanUp ; CALL ref2971
    CALL CleanUp ; CALL ref3084
    CALL CleanUp ; CALL ref3587
    CALL CleanUp ; CALL ref3661
    CALL CleanUp ; CALL ref3794
    CALL CleanUp ; CALL ref3797
    CALL CleanUp ; CALL ref3857
    ! (1982)
    CALL CleanUp ; CALL ref0489
    CALL CleanUp ; CALL ref0834
    CALL CleanUp ; CALL ref1926
    CALL CleanUp ; CALL ref2234
    CALL CleanUp ; CALL ref2288
    CALL CleanUp ; CALL ref2453
    CALL CleanUp ; CALL ref2486
    CALL CleanUp ; CALL ref2529
    CALL CleanUp ; CALL ref2557
    CALL CleanUp ; CALL ref2909
    CALL CleanUp ; CALL ref3080
    CALL CleanUp ; CALL ref3083
    CALL CleanUp ; CALL ref3119
    CALL CleanUp ; CALL ref3529
    CALL CleanUp ; CALL ref3591
    CALL CleanUp ; CALL ref3855
    CALL CleanUp ; CALL ref3856
    ! (1981)
    CALL CleanUp ; CALL ref0043
    CALL CleanUp ; CALL ref0059
    CALL CleanUp ; CALL ref0449
    CALL CleanUp ; CALL ref0479
    CALL CleanUp ; CALL ref0491
    CALL CleanUp ; CALL ref0492
    CALL CleanUp ; CALL ref0849
    CALL CleanUp ; CALL ref1150
    CALL CleanUp ; CALL ref1910
    CALL CleanUp ; CALL ref2220
    CALL CleanUp ; CALL ref2225
    CALL CleanUp ; CALL ref2236
    CALL CleanUp ; CALL ref2445
    CALL CleanUp ; CALL ref2465
    CALL CleanUp ; CALL ref2549
    CALL CleanUp ; CALL ref2968
    CALL CleanUp ; CALL ref2980
    CALL CleanUp ; CALL ref2984
    CALL CleanUp ; CALL ref3601
    CALL CleanUp ; CALL ref3852
    CALL CleanUp ; CALL ref3853
    CALL CleanUp ; CALL ref3854
    ! (1980)
    CALL CleanUp ; CALL ref0559
    CALL CleanUp ; CALL ref0201
    CALL CleanUp ; CALL ref2219
    CALL CleanUp ; CALL ref2271
    CALL CleanUp ; CALL ref2454
    CALL CleanUp ; CALL ref2832
    CALL CleanUp ; CALL ref2900
    CALL CleanUp ; CALL ref2951
    CALL CleanUp ; CALL ref2952
    CALL CleanUp ; CALL ref2956
    CALL CleanUp ; CALL ref2979
    CALL CleanUp ; CALL ref3120
    CALL CleanUp ; CALL ref3600
    CALL CleanUp ; CALL ref3628
    CALL CleanUp ; CALL ref3669
    CALL CleanUp ; CALL ref3751
    CALL CleanUp ; CALL ref3851
    CALL CleanUp ; CALL ref3945
    ! (1979)
    CALL CleanUp ; CALL ref0482
    CALL CleanUp ; CALL ref0490
    CALL CleanUp ; CALL ref0628
    CALL CleanUp ; CALL ref0729
    CALL CleanUp ; CALL ref1137
    CALL CleanUp ; CALL ref2226
    CALL CleanUp ; CALL ref2378
    CALL CleanUp ; CALL ref2437
    CALL CleanUp ; CALL ref2459
    CALL CleanUp ; CALL ref2484
    CALL CleanUp ; CALL ref2917
    CALL CleanUp ; CALL ref2918
    CALL CleanUp ; CALL ref2922
    CALL CleanUp ; CALL ref2942
    CALL CleanUp ; CALL ref3659
    CALL CleanUp ; CALL ref3700
    CALL CleanUp ; CALL ref3777
    CALL CleanUp ; CALL ref3849
    CALL CleanUp ; CALL ref3850
    ! (1978)
    CALL CleanUp ; CALL ref0540
    CALL CleanUp ; CALL ref0821
    CALL CleanUp ; CALL ref1138
    CALL CleanUp ; CALL ref2218
    CALL CleanUp ; CALL ref2381
    CALL CleanUp ; CALL ref2388
    CALL CleanUp ; CALL ref3547
    CALL CleanUp ; CALL ref3926
    ! (1977)
    CALL CleanUp ; CALL ref0190
    CALL CleanUp ; CALL ref0545
    CALL CleanUp ; CALL ref2458
    CALL CleanUp ; CALL ref3782
    ! (1976)
    CALL CleanUp ; CALL ref2224
    CALL CleanUp ; CALL ref2379
    CALL CleanUp ; CALL ref3082
    ! (1975)
    CALL CleanUp ; CALL ref0544
    CALL CleanUp ; CALL ref0642
    CALL CleanUp ; CALL ref0728
    CALL CleanUp ; CALL ref1916
    CALL CleanUp ; CALL ref2280
    CALL CleanUp ; CALL ref2289
    CALL CleanUp ; CALL ref2471
    CALL CleanUp ; CALL ref3540
    ! (1974)
    CALL CleanUp ; CALL ref0505
    CALL CleanUp ; CALL ref0563
    CALL CleanUp ; CALL ref1492
    CALL CleanUp ; CALL ref1779
    CALL CleanUp ; CALL ref2203
    CALL CleanUp ; CALL ref2376
    CALL CleanUp ; CALL ref2377
    CALL CleanUp ; CALL ref2981
    CALL CleanUp ; CALL ref3781
    ! (1973)
    CALL CleanUp ; CALL ref0640
    CALL CleanUp ; CALL ref0727
    CALL CleanUp ; CALL ref0792
    CALL CleanUp ; CALL ref0794
    CALL CleanUp ; CALL ref1149
    CALL CleanUp ; CALL ref2441
    CALL CleanUp ; CALL ref3696
    CALL CleanUp ; CALL ref3701
    CALL CleanUp ; CALL ref3806
    ! (1972)
    CALL CleanUp ; CALL ref0725
    CALL CleanUp ; CALL ref0726
    CALL CleanUp ; CALL ref2375
    CALL CleanUp ; CALL ref2467
    CALL CleanUp ; CALL ref2833
    CALL CleanUp ; CALL ref2959
    ! (1971)
    CALL CleanUp ; CALL ref0722
    CALL CleanUp ; CALL ref0723
    CALL CleanUp ; CALL ref0724
    CALL CleanUp ; CALL ref0755
    CALL CleanUp ; CALL ref2578
    CALL CleanUp ; CALL ref2807
    CALL CleanUp ; CALL ref3568
    CALL CleanUp ; CALL ref3697
    CALL CleanUp ; CALL ref3698
    CALL CleanUp ; CALL ref3808
    CALL CleanUp ; CALL ref3917
    ! (1970)
    CALL CleanUp ; CALL ref0763
    CALL CleanUp ; CALL ref0791
    CALL CleanUp ; CALL ref3086
    CALL CleanUp ; CALL ref3567
    CALL CleanUp ; CALL ref3608
    CALL CleanUp ; CALL ref3786
    CALL CleanUp ; CALL ref3787
    ! (1969)
    CALL CleanUp ; CALL ref0754
    CALL CleanUp ; CALL ref0795
    CALL CleanUp ; CALL ref3565
    CALL CleanUp ; CALL ref3566
    CALL CleanUp ; CALL ref3785
    ! (1968)
    CALL CleanUp ; CALL ref0627
    CALL CleanUp ; CALL ref2387
    CALL CleanUp ; CALL ref2831
    ! (1967)
    CALL CleanUp ; CALL ref0721
    CALL CleanUp ; CALL ref1911
    CALL CleanUp ; CALL ref3122
    ! (1966)
    CALL CleanUp ; CALL ref0525
    CALL CleanUp ; CALL ref3710
    CALL CleanUp ; CALL ref3784
    CALL CleanUp ; CALL ref3924
    ! (1965)
    CALL CleanUp ; CALL ref0714
    CALL CleanUp ; CALL ref0753
    CALL CleanUp ; CALL ref0766
    CALL CleanUp ; CALL ref2550
    CALL CleanUp ; CALL ref3075
    ! (1964)
    CALL CleanUp ; CALL ref0752
    CALL CleanUp ; CALL ref2442
    CALL CleanUp ; CALL ref2446
    CALL CleanUp ; CALL ref3703
    CALL CleanUp ; CALL ref3799
    CALL CleanUp ; CALL ref3956
    ! (1963)
    CALL CleanUp ; CALL ref0496
    CALL CleanUp ; CALL ref3077
    CALL CleanUp ; CALL ref3699
    CALL CleanUp ; CALL ref3923
    CALL CleanUp ; CALL ref4017
    ! (1962)
    CALL CleanUp ; CALL ref0735
    CALL CleanUp ; CALL ref0750
    CALL CleanUp ; CALL ref1340
    CALL CleanUp ; CALL ref2551
    CALL CleanUp ; CALL ref3658
    ! (1961)
    CALL CleanUp ; CALL ref1151
    CALL CleanUp ; CALL ref3100
    CALL CleanUp ; CALL ref3123
    ! (1960)
    CALL CleanUp ; CALL ref0529
    CALL CleanUp ; CALL ref1992
    CALL CleanUp ; CALL ref3431
    ! (1958)
    CALL CleanUp ; CALL ref2552
    CALL CleanUp ; CALL ref3592
    CALL CleanUp ; CALL ref3705
    CALL CleanUp ; CALL ref3965
    ! (1957)
    CALL CleanUp ; CALL ref3066
    ! (1956)
    CALL CleanUp ; CALL ref0626
    CALL CleanUp ; CALL ref0776
    CALL CleanUp ; CALL ref3593
    ! (1955)
    CALL CleanUp ; CALL ref3955
    ! (1954)
    CALL CleanUp ; CALL ref0719
    CALL CleanUp ; CALL ref0764
    CALL CleanUp ; CALL ref3801
    ! (1953)
    CALL CleanUp ; CALL ref2222
    ! (1952)
    CALL CleanUp ; CALL ref3750
    CALL CleanUp ; CALL ref3783
    ! (1951)
    CALL CleanUp ; CALL ref0790
    ! (1950)
    CALL CleanUp ; CALL ref3660
    CALL CleanUp ; CALL ref3957
    ! (1949)
    CALL CleanUp ; CALL ref3966
    ! (1947)
    CALL CleanUp ; CALL ref3967
    ! (1944)
    CALL CleanUp ; CALL ref3969
    ! (1943)
    CALL CleanUp ; CALL ref2835
    ! (1942)
    CALL CleanUp ; CALL ref3949
    ! (1941)
    CALL CleanUp ; CALL ref3541
    CALL CleanUp ; CALL ref3807
    ! (1939)
    CALL CleanUp ; CALL ref0528
    CALL CleanUp ; CALL ref3809
    ! (1938)
    CALL CleanUp ; CALL ref2491
    ! (1937)
    CALL CleanUp ; CALL ref3702
    CALL CleanUp ; CALL ref3925
    ! (1935)
    CALL CleanUp ; CALL ref0712
    CALL CleanUp ; CALL ref0713
    ! (1934)
    CALL CleanUp ; CALL ref1072
    CALL CleanUp ; CALL ref3707
    ! (1932)
    CALL CleanUp ; CALL ref1913
    CALL CleanUp ; CALL ref1914
    CALL CleanUp ; CALL ref1915
    ! (1930)
    CALL CleanUp ; CALL ref3789
    ! (1929)
    CALL CleanUp ; CALL ref3715
    ! (1928)
    CALL CleanUp ; CALL ref0732
    ! (1927)
    CALL CleanUp ; CALL ref3641
    ! (1926)
    CALL CleanUp ; CALL ref3748
    ! (1925)
    CALL CleanUp ; CALL ref0481
    CALL CleanUp ; CALL ref0486
    ! (1922)
    CALL CleanUp ; CALL ref3548
    CALL CleanUp ; CALL ref3709
    ! (1919)
    ! CALL CleanUp ; CALL ref3706 ! NOTUSED
    ! (1918)
    CALL CleanUp ; CALL ref3942
    ! (1917)
    CALL CleanUp ; CALL ref2933
    CALL CleanUp ; CALL ref3594
    ! (1913)
    CALL CleanUp ; CALL ref3749
    ! (1911)
    CALL CleanUp ; CALL ref3546
    CALL CleanUp ; CALL ref3934
    ! (1910)
    CALL CleanUp ; CALL ref3714
    ! (1909)
    CALL CleanUp ; CALL ref3704
    CALL CleanUp ; CALL ref3716
    ! (1907)
    CALL CleanUp ; CALL ref3718
    ! (1906)
    CALL CleanUp ; CALL ref0862
    CALL CleanUp ; CALL ref0863
    ! (1905)
    CALL CleanUp ; CALL ref3595
    ! (1904)
    CALL CleanUp ; CALL ref3711
    ! (1902)
    CALL CleanUp ; CALL ref3747
    ! (1901)
    CALL CleanUp ; CALL ref0869
    CALL CleanUp ; CALL ref2834
    ! (1900)
    CALL CleanUp ; CALL ref3795
    CALL CleanUp ; CALL ref3937
    ! (1899)
    CALL CleanUp ; CALL ref0871
    CALL CleanUp ; CALL ref2841
    ! (1897)
    CALL CleanUp ; CALL ref3944
    ! (1894)
    CALL CleanUp ; CALL ref3596
    ! (1891)
    CALL CleanUp ; CALL ref3026
    CALL CleanUp ; CALL ref3027
    CALL CleanUp ; CALL ref3713
    ! (1890)
    CALL CleanUp ; CALL ref3712
    ! (1884)
    CALL CleanUp ; CALL ref3938
    ! (1883)
    CALL CleanUp ; CALL ref3935
    ! (1873)
    CALL CleanUp ; CALL ref3597
    ! (1855)
    CALL CleanUp ; CALL ref3708
    CALL CleanUp ; CALL ref3717
    CALL CleanUp ; CALL ref3933
  ENDIF ! if l_testrun

  !---------------------------------------------------------------------------

  CALL middle

  !---------------------------------------------------------------------------

  ! Go through individual species and select all data enties in
  ! henry_data that belong to it by using the CASRN or InChIKey to
  ! identify the species. The results are written to henry_data.tex.

  ! The order of halogen species is as in MECCA. For organics, the order is:
  ! 1) alkanes, 2) alkenes, 3) alkynes, 4) aromatics
  ! within each group, sorted by increasing size: C_1, C_2, ...
  ! within each size, sorted by increasing number of halogen atoms

  ! if there is more than 1 trivial name, they must be separated with a
  ! semicolon, not with a comma.

  ! To identify a species, the CASRN and the InChIKey can be used. At
  ! least one of them must be defined.

  ! Some restrictions because of makeindex:
  ! - The name must not contain either "@" or "!" or "|".
  ! - Currently, only the Greek letters alpha, beta, gamma and delta are
  !   treated correctly. Add more to SUBROUTINE Species, if necessary.

  !***************************************************************************
  CALL SpeciesGroup(1,"Inorganic species")
  !***************************************************************************
  CALL SpeciesGroup(2,"Oxygen (\chem{O})")
  CALL Species("O_2",                                 "7782-44-7", "MYMOFIZGZYHOMD-UHFFFAOYSA-N", "oxygen", "")
  CALL Species("O_3",                                "10028-15-6", "CBENFWSGALASAD-UHFFFAOYSA-N", "ozone", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Hydrogen (\chem{H})")
  CALL Species("H",                                  "12385-13-6", "YZCKVEUIGOORGS-UHFFFAOYSA-N", "hydrogen atom", "")
  CALL Species("H_2",                                 "1333-74-0", "UFHFLCQGNIYNRP-UHFFFAOYSA-N", "hydrogen", "")
  CALL Species("D_2",                                 "7782-39-0", "UFHFLCQGNIYNRP-VVKOMZTBSA-N", "deuterium", "")
  CALL Species("OH",                                  "3352-57-6", "TUJKJAMUKRIRHC-UHFFFAOYSA-N", "hydroxyl radical", "")
  CALL Species("HO_2",                                "3170-83-0", "MHAJPDPJQMAIIY-UHFFFAOYSA-M", "hydroperoxy radical", "")
  CALL Species("H_2O_2",                              "7722-84-1", "MHAJPDPJQMAIIY-UHFFFAOYSA-N", "hydrogen peroxide", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Nitrogen (\chem{N})")
  ! H, N:
  CALL Species("N_2",                                 "7727-37-9", "IJGRMHOSHXDMSA-UHFFFAOYSA-N", "nitrogen", "")
  CALL Species("NH_3",                                "7664-41-7", "QGZKDVFQNNGYKY-UHFFFAOYSA-N", "ammonia", "")
  CALL Species("HN_3",                                "7782-79-8", "JUINSXZKUKVTMD-UHFFFAOYSA-N", "hydrazoic acid", "")
  CALL Species("H_4N_2",                               "302-01-2", "OAKJQQAXSVQMHS-UHFFFAOYSA-N", "hydrazine", "")
  ! H, O, N:
  CALL Species("N_2O",                               "10024-97-2", "GQPLMRYTRLFLPF-UHFFFAOYSA-N", "dinitrogen monoxide", "nitrous oxide; laughing gas")
  CALL Species("NO",                                 "10102-43-9", "MWUXSHHQAYIFBG-UHFFFAOYSA-N", "nitrogen monoxide", "nitric oxide")
  CALL Species("NO_2",                               "10102-44-0", "JCXJVPUVTGWSNB-UHFFFAOYSA-N", "nitrogen dioxide", "")
  CALL Species("NO_3",                               "12033-49-7", "YPJKMVATUPSWOH-UHFFFAOYSA-N", "nitrogen trioxide", "nitrate radical")
  CALL Species("N_2O_3",                             "10544-73-7", "LZDSILRDTDCIQT-UHFFFAOYSA-N", "dinitrogen trioxide", "")
  CALL Species("N_2O_4",                             "10544-72-6", "WFPZPJSADLPSON-UHFFFAOYSA-N", "dinitrogen tetroxide", "")
  CALL Species("N_2O_5",                             "10102-03-1", "ZWWCURLKEXEFQT-UHFFFAOYSA-N", "dinitrogen pentoxide", "nitric anhydride")
  CALL Species("H_3NO",                               "7803-49-8", "AVXURJPOCDRRFD-UHFFFAOYSA-N", "hydroxylamine", "")
  CALL Species("HNO_2",                               "7782-77-6", "IOVCWXUNBOPUCH-UHFFFAOYSA-N", "nitrous acid", "")
  CALL Species("HNO_3",                               "7697-37-2", "GRYLNZFGIOXLOG-UHFFFAOYSA-N", "nitric acid", "")
  CALL Species("HNO_4",                              "26404-66-0", "UUZZMWZGAZGXSF-UHFFFAOYSA-N", "pernitric acid", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Fluorine (\chem{F})")
  ! F, H, O:
  CALL Species("F",                                  "14762-94-8", "YCKRFDGAMUMZLT-UHFFFAOYSA-N", "fluorine atom", "")
  CALL Species("HF",                                  "7664-39-3", "KRHYYFGTRYWZRS-UHFFFAOYSA-N", "hydrogen fluoride", "")
  CALL Species("F_2O",                                "7783-41-7", "UJMWVICAENGCRF-UHFFFAOYSA-N", "difluorine monoxide", "")
  ! F, H, O, N:
  CALL Species("NF_3",                                "7783-54-2", "GVGCUCJTUSOZKP-UHFFFAOYSA-N", "nitrogen trifluoride", "")
  CALL Species("N_2F_4",                             "10036-47-2", "GFADZIUESKAXAK-UHFFFAOYSA-N", "dinitrogen tetrafluoride", "tetrafluorohydrazine")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Chlorine (\chem{Cl})")
  ! Cl:
  CALL Species("Cl_2",                                "7782-50-5", "KZBUYRJDOAKODT-UHFFFAOYSA-N", "chlorine (molecular)", "")
  CALL Species("Cl",                                 "22537-15-1", "ZAMOUSCENKQFHK-UHFFFAOYSA-N", "chlorine atom", "")
  ! Cl, H:
  CALL Species("HCl",                                 "7647-01-0", "VEXZGXHMUGYJMC-UHFFFAOYSA-N", "hydrogen chloride", "")
  ! Cl, H, O:
  CALL Species("HOCl",                                "7790-92-3", "QWPPOHNGKGFGJK-UHFFFAOYSA-N", "hypochlorous acid", "")
  CALL Species("HClO_4",                              "7601-90-3", "VLTRZXGMWDSKGL-UHFFFAOYSA-N", "perchloric acid", "")
  CALL Species("ClO",                                "14989-30-1", "MLWGAEVSWJXOQJ-UHFFFAOYSA-N", "monochlorine monoxide", "")
  CALL Species("Cl_2O",                               "7791-21-1", "RCJVRSBWZCNNQT-UHFFFAOYSA-N", "dichlorine monoxide", "")
  CALL Species("ClO_2",                              "10049-04-4", "OSVXSBDYLRYLIG-UHFFFAOYSA-N", "chlorine dioxide", "")
  CALL Species("ClO_3F",                              "7616-94-6", "XHFXMNZYIKFCPN-UHFFFAOYSA-N", "perchloryl fluoride", "")
  ! Cl, H, O, N:
  CALL Species("NOCl",                                "2696-92-6", "VPCDQGACGWYTMC-UHFFFAOYSA-N", "nitrosyl chloride", "")
  CALL Species("ClNO_2",                             "13444-90-1", "HSSFHZJIMRUXDM-UHFFFAOYSA-M", "nitryl chloride", "")
  CALL Species("ClNO_3",                             "14545-72-3", "XYLGPCWDPLOBGP-UHFFFAOYSA-N", "chlorine nitrate", "")
  CALL Species("NH_2Cl",                             "10599-90-3", "QDHHCQZDFGDHMP-UHFFFAOYSA-N", "chloramine", "chloramide")
  CALL Species("NHCl_2",                              "3400-09-7", "JSYGRUBHOCKMGQ-UHFFFAOYSA-N", "dichloramine", "chlorimide")
  CALL Species("NCl_3",                              "10025-85-1", "QEHKBHWEUPXBCW-UHFFFAOYSA-N", "nitrogen trichloride", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Bromine (\chem{Br})")
  ! Br:
  CALL Species("Br_2",                                "7726-95-6", "GDTBXPJZTBHREO-UHFFFAOYSA-N", "bromine (molecular)", "")
  CALL Species("Br",                                 "10097-32-2", "WKBOTKDWSSQWDR-UHFFFAOYSA-N", "bromine atom", "")
  ! Br, H:
  CALL Species("HBr",                                "10035-10-6", "CPELXLSAUQHCOX-UHFFFAOYSA-N", "hydrogen bromide", "")
  ! Br, H, O:
  CALL Species("HOBr",                               "13517-11-8", "CUILPNURFADTPE-UHFFFAOYSA-N", "hypobromous acid", "")
  ! Br, H, O, N:
  CALL Species("BrNO_2",                             "13536-70-4", "SEYAFCXQVVHRPY-UHFFFAOYSA-M", "nitryl bromide", "")
  CALL Species("BrNO_3",                             "40423-14-1", "RRTWEEAEXPZMPY-UHFFFAOYSA-N", "bromine nitrate", "")
  ! Br, Cl:
  CALL Species("BrCl",                               "13863-41-7", "CODNYICXDISAEA-UHFFFAOYSA-N", "bromine chloride", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Iodine (\chem{I})")
  ! I:
  CALL Species("I_2",                                 "7553-56-2", "PNDPGZBMCMUPRI-UHFFFAOYSA-N", "iodine (molecular)", "")
  CALL Species("I",                                  "14362-44-8", "ZCYVEMRRCGMTRW-UHFFFAOYSA-N", "iodine atom", "")
  ! I, H:
  CALL Species("HI",                                 "10034-85-2", "XMBWDFGMSWQBCA-UHFFFAOYSA-N", "hydrogen iodide", "")
  ! I, H, O:
  CALL Species("IO",                                 "14696-98-1", "AFSVSXMRDKPOEW-UHFFFAOYSA-N", "iodine monoxide", "")
  CALL Species("OIO",                                "13494-92-3", "WXDJHDMIIZKXSK-UHFFFAOYSA-N", "iodine dioxide", "")
  CALL Species("I_2O_2",                            "215239-62-6", "IELAHHPSAVYAOC-UHFFFAOYSA-N", "diiodine dioxide", "")
  CALL Species("I_2O_3",                             "11085-17-9", "NMNCVPKLBXOKQA-UHFFFAOYSA-N", "diiodine trioxide", "")
  CALL Species("I_2O_4",                           "1024652-24-1", "XHTWXUOEQMOFEJ-UHFFFAOYSA-N", "diiodine tetroxide", "")
  CALL Species("HOI",                                "14332-21-9", "GEOVEUCEIQCBKH-UHFFFAOYSA-N", "hypoiodous acid", "")
  ! I, H, O, N:
  CALL Species("INO_2",                              "15465-40-4", "PSZTWRLUACEPOW-UHFFFAOYSA-N", "iodine nitrite", "")
  CALL Species("INO_3",                              "14696-81-2", "CCJHDZZUWZIVJF-UHFFFAOYSA-N", "iodine nitrate", "")
  ! I, Cl:
  CALL Species("ICl",                                 "7790-99-0", "QZRGKCOWNLSUDK-UHFFFAOYSA-N", "iodine chloride", "")
  ! I, Br:
  CALL Species("IBr",                                 "7789-33-5", "CBEQRNSPHCCXSH-UHFFFAOYSA-N", "iodine bromide", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Sulfur (\chem{S})")
  CALL Species("S",                                   "7704-34-9", "NINIDFKCEFEMDL-UHFFFAOYSA-N", "sulfur", "")
  CALL Species("H_2S",                                "7783-06-4", "RWSOTUBLDIXVET-UHFFFAOYSA-N", "hydrogen sulfide", "")
  CALL Species("D_2S",                               "13536-94-2", "RWSOTUBLDIXVET-ZSJDYOACSA-N", "deuterium sulfide", "")
  CALL Species("SO_2",                                "7446-09-5", "RAHZWNYVWXNFOC-UHFFFAOYSA-N", "sulfur dioxide", "")
  CALL Species("SO_3",                                "7446-11-9", "AKEJUJNQAAGONA-UHFFFAOYSA-N", "sulfur trioxide", "")
  CALL Species("H_2SO_4",                             "7664-93-9", "QAOWNCQODCNURD-UHFFFAOYSA-N", "sulfuric acid", "")
  CALL Species("SF_6",                                "2551-62-4", "SFZCNBIFKDRMGX-UHFFFAOYSA-N", "sulfur hexafluoride", "")
  CALL Species("SO_2F_2",                             "2699-79-8", "OBTWBSRJZRCYQV-UHFFFAOYSA-N", "sulfuryl fluoride", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Rare gases (\chem{He}, \chem{Ne}, \chem{Ar}, \chem{Kr}, \chem{Xe}, \chem{Rn})")
  CALL Species("He",                                  "7440-59-7", "SWQJXJOGLNCZEY-UHFFFAOYSA-N", "helium", "")
  CALL Species("Ne",                                  "7440-01-9", "GKAOGPIIYCISHV-UHFFFAOYSA-N", "neon", "")
  CALL Species("Ar",                                  "7440-37-1", "XKRFYHLGVUSROY-UHFFFAOYSA-N", "argon", "")
  CALL Species("Kr",                                  "7439-90-9", "DNNSSWSSYDEUBZ-UHFFFAOYSA-N", "krypton", "")
  CALL Species("Xe",                                  "7440-63-3", "FHNFHKCVQCLJFQ-UHFFFAOYSA-N", "xenon", "")
  CALL Species("^{133}Xe",                           "14932-42-4", "FHNFHKCVQCLJFQ-NJFSPNSNSA-N", "xenon-133", "")
  CALL Species("Rn",                                 "10043-92-2", "SYUHGPGVQRZVTB-UHFFFAOYSA-N", "radon", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Other elements (\chem{B}, \chem{Se}, \chem{P}, \chem{As}, \chem{Hg})")
  CALL Species("H_3BO_3",                            "10043-35-3", "KGBXLFKZBHKPEV-UHFFFAOYSA-N", "boric acid", "")
  CALL Species("H_2Se",                               "7783-07-5", "BUGBHKTXTAQXES-UHFFFAOYSA-N", "selenium hydride", "")
  CALL Species("PH_3",                                "7803-51-2", "XYFCBTPGUUZFHI-UHFFFAOYSA-N", "phosphorus trihydride", "phosphine")
  CALL Species("AsH_3",                               "7784-42-1", "RQNWIZPPADIBDY-UHFFFAOYSA-N", "arsenic hydride", "arsine")
  CALL Species("Hg",                                  "7439-97-6", "QSHDDOUJBYECFT-UHFFFAOYSA-N", "mercury", "")
  CALL Species("HgO",                                "21908-53-2", "UKWHYYKOEPRTIC-UHFFFAOYSA-N", "mercury(II) oxide", "")
  CALL Species("Hg(OH)_2",                           "12135-13-6", "VLKKXDVIWIBHHS-UHFFFAOYSA-L", "mercury dihydroxide", "")
  CALL Species("HgCl_2",                              "7487-94-7", "LWJROJCJINYWOX-UHFFFAOYSA-L", "mercury dichloride", "")
  CALL Species("HgBr_2",                              "7789-47-1", "NGYIMTKLQULBOO-UHFFFAOYSA-L", "mercury dibromide", "")
  CALL Species("HgI_2",                               "7774-29-0", "YFDLHELOZYVNJE-UHFFFAOYSA-L", "mercury diiodide", "")
  !***************************************************************************
  CALL SpeciesGroup(1,"Hydrocarbons (\chem{C}, \chem{H})")
  !***************************************************************************
  CALL SpeciesGroup(2,"Alkanes")
  ! C1:
  CALL Species("CH_4",                                  "74-82-8", "VNWKTOKETHGBQD-UHFFFAOYSA-N", "methane", "")
  ! C2:
  CALL Species("C_2H_6",                                "74-84-0", "OTMSDBZUPAUEDD-UHFFFAOYSA-N", "ethane", "")
  ! C3:
  CALL Species("C_3H_8",                                "74-98-6", "ATUOYWHBWRKTHZ-UHFFFAOYSA-N", "propane", "")
  ! C4:
  CALL Species("C_4H_{10}",                            "106-97-8", "IJDNQMDRQITEOD-UHFFFAOYSA-N", "butane", "")
  CALL Species("HC(CH_3)_3",                            "75-28-5", "NNPPMTNAJDCUHE-UHFFFAOYSA-N", "2-methylpropane", "isobutane")
  ! C5:
  CALL Species("C_5H_{12}",                            "109-66-0", "OFBQJSOFQDEBGM-UHFFFAOYSA-N", "pentane", "")
  CALL Species("C_5H_{12}",                             "78-78-4", "QWTDNUCVQCZILF-UHFFFAOYSA-N", "2-methylbutane", "isopentane")
  CALL Species("C(CH_3)_4",                            "463-82-1", "CRSOQBOWXPBRES-UHFFFAOYSA-N", "dimethylpropane", "neopentane")
  ! C6:
  CALL Species("C_6H_{14}",                            "110-54-3", "VLKZOEOYAKHREP-UHFFFAOYSA-N", "hexane", "")
  CALL Species("C_6H_{14}",                            "107-83-5", "AFABGHUZZDYHJO-UHFFFAOYSA-N", "2-methylpentane", "isohexane")
  CALL Species("C_6H_{14}",                             "96-14-0", "PFEOZHBOMNWTJB-UHFFFAOYSA-N", "3-methylpentane", "")
  CALL Species("C_6H_{14}",                             "75-83-2", "HNRMPXKDFBEGFZ-UHFFFAOYSA-N", "2,2-dimethylbutane", "")
  CALL Species("C_6H_{14}",                             "79-29-8", "ZFFMLCVRJBZUDZ-UHFFFAOYSA-N", "2,3-dimethylbutane", "")
  ! C7:
  CALL Species("C_7H_{16}",                            "142-82-5", "IMNFDUFMRHMDMM-UHFFFAOYSA-N", "heptane", "")
  CALL Species("C_7H_{16}",                            "591-76-4", "GXDHCNNESPLIKD-UHFFFAOYSA-N", "2-methylhexane", "isoheptane")
  CALL Species("C_7H_{16}",                            "589-34-4", "VLJXXKKOSFGPHI-UHFFFAOYSA-N", "3-methylhexane", "") ! other CAS: 78918-91-9 for (R)-enantiomer
  CALL Species("C_7H_{16}",                            "590-35-2", "CXOWYJMDMMMMJO-UHFFFAOYSA-N", "2,2-dimethylpentane", "")
  CALL Species("C_7H_{16}",                            "565-59-3", "WGECXQBGLLYSFP-UHFFFAOYSA-N", "2,3-dimethylpentane", "")
  CALL Species("C_7H_{16}",                            "108-08-7", "BZHMBWZPUJHVEE-UHFFFAOYSA-N", "2,4-dimethylpentane", "")
  CALL Species("C_7H_{16}",                            "562-49-2", "AEXMKKGTQYQZCS-UHFFFAOYSA-N", "3,3-dimethylpentane", "")
  CALL Species("C_7H_{16}",                            "617-78-7", "AORMDLNPRGXHHL-UHFFFAOYSA-N", "3-ethylpentane", "")
  CALL Species("C_7H_{16}",                            "464-06-2", "ZISSAWUMDACLOM-UHFFFAOYSA-N", "2,2,3-trimethylbutane", "")
  ! C8:
  CALL Species("C_8H_{18}",                            "111-65-9", "TVMXDCGIABBOFY-UHFFFAOYSA-N", "octane", "")
  CALL Species("C_8H_{18}",                            "592-27-8", "JVSWJIKNEAIKJW-UHFFFAOYSA-N", "2-methylheptane", "")
  CALL Species("C_8H_{18}",                            "589-81-1", "LAIUFBWHERIJIH-UHFFFAOYSA-N", "3-methylheptane", "")
  CALL Species("C_8H_{18}",                            "589-53-7", "CHBAWFGIXDBEBT-UHFFFAOYSA-N", "4-methylheptane", "")
  CALL Species("C_8H_{18}",                            "590-73-8", "FLTJDUOFAQWHDF-UHFFFAOYSA-N", "2,2-dimethylhexane", "")
  CALL Species("C_8H_{18}",                            "584-94-1", "JXPOLSKBTUYKJB-UHFFFAOYSA-N", "2,3-dimethylhexane", "")
  CALL Species("C_8H_{18}",                            "589-43-5", "HDGQICNBXPAKLR-UHFFFAOYSA-N", "2,4-dimethylhexane", "")
  CALL Species("C_8H_{18}",                            "592-13-2", "UWNADWZGEHDQAB-UHFFFAOYSA-N", "2,5-dimethylhexane", "")
  CALL Species("C_8H_{18}",                            "563-16-6", "KUMXLFIBWFCMOJ-UHFFFAOYSA-N", "3,3-dimethylhexane", "")
  CALL Species("C_8H_{18}",                            "583-48-2", "RNTWWGNZUXGTAX-UHFFFAOYSA-N", "3,4-dimethylhexane", "")
  CALL Species("C_8H_{18}",                            "619-99-8", "SFRKSDZMZHIISH-UHFFFAOYSA-N", "3-ethylhexane", "")
  CALL Species("C_8H_{18}",                            "564-02-3", "XTDQDBVBDLYELW-UHFFFAOYSA-N", "2,2,3-trimethylpentane", "")
  CALL Species("C_8H_{18}",                            "540-84-1", "NHTMVDHEPJAVLT-UHFFFAOYSA-N", "2,2,4-trimethylpentane", "isooctane")
  CALL Species("C_8H_{18}",                            "560-21-4", "OKVWYBALHQFVFP-UHFFFAOYSA-N", "2,3,3-trimethylpentane", "")
  CALL Species("C_8H_{18}",                            "565-75-3", "RLPGDEORIPLBNF-UHFFFAOYSA-N", "2,3,4-trimethylpentane", "")
  CALL Species("C_8H_{18}",                            "609-26-7", "DUPUVYJQZSLSJB-UHFFFAOYSA-N", "3-ethyl-2-methylpentane", "")
  CALL Species("C_8H_{18}",                           "1067-08-9", "GIEZWIDCIFCQPS-UHFFFAOYSA-N", "3-ethyl-3-methylpentane", "")
  CALL Species("C_8H_{18}",                            "594-82-1", "OMMLUKLXGSRPHK-UHFFFAOYSA-N", "2,2,3,3-tetramethylbutane", "")
  ! C9:
  CALL Species("C_9H_{20}",                            "111-84-2", "BKIMMITUMNQMOS-UHFFFAOYSA-N", "nonane", "")
  CALL Species("C_9H_{20}",                           "3221-61-2", "ZUBZATZOEPUUQF-UHFFFAOYSA-N", "2-methyloctane", "")
  CALL Species("C_9H_{20}",                           "2216-33-3", "SEEOMASXHIJCDV-UHFFFAOYSA-N", "3-methyloctane", "")
  CALL Species("C_9H_{20}",                           "2216-34-4", "DOGIHOCMZJUJNR-UHFFFAOYSA-N", "4-methyloctane", "")
  CALL Species("C_9H_{20}",                           "3074-71-3", "WBRFDUJXCLCKPX-UHFFFAOYSA-N", "2,3-dimethylheptane", "")
  CALL Species("C_9H_{20}",                           "1071-26-7", "PSABUFWDVWCFDP-UHFFFAOYSA-N", "2,2-dimethylheptane", "")
  CALL Species("C_9H_{20}",                           "2213-23-2", "AUKVIBNBLXQNIZ-UHFFFAOYSA-N", "2,4-dimethylheptane", "")
  CALL Species("C_9H_{20}",                           "2216-30-0", "HQZHQNKZOYIKQC-UHFFFAOYSA-N", "2,5-dimethylheptane", "")
  CALL Species("C_9H_{20}",                           "1072-05-5", "KBPCCVWUMVGXGF-UHFFFAOYSA-N", "2,6-dimethylheptane", "")
  CALL Species("C_9H_{20}",                           "4032-86-4", "BVAKDOXCVSMKHE-UHFFFAOYSA-N", "3,3-dimethylheptane", "")
  CALL Species("C_9H_{20}",                            "922-28-1", "MAKRYGRRIKSDES-UHFFFAOYSA-N", "3,4-dimethylheptane", "")
  CALL Species("C_9H_{20}",                            "926-82-9", "DZJTZGHZAWTWGA-UHFFFAOYSA-N", "3,5-dimethylheptane", "")
  CALL Species("C_9H_{20}",                           "1068-19-5", "WSOKFYJGNBQDPW-UHFFFAOYSA-N", "4,4-dimethylheptane", "")
  CALL Species("C_9H_{20}",                          "15869-80-4", "PSVQKOKKLWHNRP-UHFFFAOYSA-N", "3-ethylheptane", "")
  CALL Species("C_9H_{20}",                           "2216-32-2", "XMROPFQWHHUFFS-UHFFFAOYSA-N", "4-ethylheptane", "")
  CALL Species("C_9H_{20}",                          "16747-25-4", "CBVFSZDQEHBJEQ-UHFFFAOYSA-N", "2,2,3-trimethylhexane", "")
  CALL Species("C_9H_{20}",                          "16747-26-5", "AFTPEBDOGXRMNQ-UHFFFAOYSA-N", "2,2,4-trimethylhexane", "")
  CALL Species("C_9H_{20}",                           "3522-94-9", "HHOSMYBYIHNXNO-UHFFFAOYSA-N", "2,2,5-trimethylhexane", "")
  CALL Species("C_9H_{20}",                          "16747-28-7", "DJYSEQMMCZAKGT-UHFFFAOYSA-N", "2,3,3-trimethylhexane", "")
  CALL Species("C_9H_{20}",                            "921-47-1", "RUTNOQHQISEBGT-UHFFFAOYSA-N", "2,3,4-trimethylhexane", "")
  CALL Species("C_9H_{20}",                           "1069-53-0", "ODGLTLJZCVNPBU-UHFFFAOYSA-N", "2,3,5-trimethylhexane", "")
  CALL Species("C_9H_{20}",                          "16747-30-1", "SVEMKBCPZYWEPH-UHFFFAOYSA-N", "2,4,4-trimethylhexane", "")
  CALL Species("C_9H_{20}",                          "16747-31-2", "ARWOOWBJJKVYOV-UHFFFAOYSA-N", "3,3,4-trimethylhexane", "")
  CALL Species("C_9H_{20}",                          "16789-46-1", "MVLOWDRGPHBNNF-UHFFFAOYSA-N", "3-ethyl-2-methylhexane", "")
  CALL Species("C_9H_{20}",                           "3074-75-7", "KYCZJIBOPKRSOV-UHFFFAOYSA-N", "4-ethyl-2-methylhexane", "")
  CALL Species("C_9H_{20}",                           "3074-76-8", "CYWROHZCELEGSE-UHFFFAOYSA-N", "3-ethyl-3-methylhexane", "")
  CALL Species("C_9H_{20}",                           "3074-77-9", "OKCRKWVABWILDR-UHFFFAOYSA-N", "3-ethyl-4-methylhexane", "")
  CALL Species("C_9H_{20}",                           "7154-79-2", "QUKOJKFJIHSBKV-UHFFFAOYSA-N", "2,2,3,3-tetramethylpentane", "")
  CALL Species("C_9H_{20}",                           "1186-53-4", "VZFMYOCAEQDWDY-UHFFFAOYSA-N", "2,2,3,4-tetramethylpentane", "")
  CALL Species("C_9H_{20}",                           "1070-87-7", "GUMULFRCHLJNDY-UHFFFAOYSA-N", "2,2,4,4-tetramethylpentane", "")
  CALL Species("C_9H_{20}",                          "16747-38-9", "JLCYYQOQSAMWTA-UHFFFAOYSA-N", "2,3,3,4-tetramethylpentane", "")
  CALL Species("C_9H_{20}",                          "16747-32-3", "CLZCPQKGOAXOJT-UHFFFAOYSA-N", "3-ethyl-2,2-dimethylpentane", "")
  CALL Species("C_9H_{20}",                          "16747-33-4", "MMASVVOQIKCFJZ-UHFFFAOYSA-N", "3-ethyl-2,3-dimethylpentane", "")
  CALL Species("C_9H_{20}",                           "1068-87-7", "VLHAGZNBWKUMRW-UHFFFAOYSA-N", "3-ethyl-2,4-dimethylpentane", "")
  CALL Species("C_9H_{20}",                           "1067-20-5", "BGXXXYLRPIRDHJ-UHFFFAOYSA-N", "3,3-diethylpentane", "")
  ! C10:
  CALL Species("C_{10}H_{22}",                         "124-18-5", "DIOQZVSQGTUSAI-UHFFFAOYSA-N", "decane", "")
  CALL Species("C_{10}H_{22}",                         "871-83-0", "SGVYKUFIHHTIFL-UHFFFAOYSA-N", "2-methylnonane", "")
  CALL Species("C_{10}H_{22}",                        "5911-04-6", "PLZDDPSCZHRBOY-UHFFFAOYSA-N", "3-methylnonane", "")
  CALL Species("C_{10}H_{22}",                       "17301-94-9", "IALRSQMWHFKJJA-UHFFFAOYSA-N", "4-methylnonane", "")
  CALL Species("C_{10}H_{22}",                       "15869-85-9", "TYSIILFJZXHVPU-UHFFFAOYSA-N", "5-methylnonane", "")
  CALL Species("C_{10}H_{22}",                       "15869-87-1", "GPBUTTSWJNPYJL-UHFFFAOYSA-N", "2,2-dimethyloctane", "")
  CALL Species("C_{10}H_{22}",                        "7146-60-3", "YPMNDMUOGQJCLW-UHFFFAOYSA-N", "2,3-dimethyloctane", "")
  CALL Species("C_{10}H_{22}",                        "4032-94-4", "IXAVTTRPEXFVSX-UHFFFAOYSA-N", "2,4-dimethyloctane", "")
  CALL Species("C_{10}H_{22}",                       "15869-89-3", "HOAAQUNESXYFDT-UHFFFAOYSA-N", "2,5-dimethyloctane", "")
  CALL Species("C_{10}H_{22}",                        "2051-30-1", "ZALHPSXXQIPKTQ-UHFFFAOYSA-N", "2,6-dimethyloctane", "")
  CALL Species("C_{10}H_{22}",                        "1072-16-8", "KEVMYFLMMDUPJE-UHFFFAOYSA-N", "2,7-dimethyloctane", "")
  CALL Species("C_{10}H_{22}",                        "4110-44-5", "DBULLUBYDONGLT-UHFFFAOYSA-N", "3,3-dimethyloctane", "")
  CALL Species("C_{10}H_{22}",                       "15869-92-8", "QQCWGAMGBCGAQJ-UHFFFAOYSA-N", "3,4-dimethyloctane", "")
  CALL Species("C_{10}H_{22}",                       "15869-93-9", "VRHRGVJOUHJULC-UHFFFAOYSA-N", "3,5-dimethyloctane", "")
  CALL Species("C_{10}H_{22}",                       "15869-94-0", "JEEQUUSFXYRPRK-UHFFFAOYSA-N", "3,6-dimethyloctane", "")
  CALL Species("C_{10}H_{22}",                       "15869-95-1", "ZMEDGZAGMLTROM-UHFFFAOYSA-N", "4,4-dimethyloctane", "")
  CALL Species("C_{10}H_{22}",                       "15869-96-2", "DOYJTLUPPPUSMD-UHFFFAOYSA-N", "4,5-dimethyloctane", "")
  CALL Species("C_{10}H_{22}",                        "5881-17-4", "OEYGTUAKNZFCDJ-UHFFFAOYSA-N", "3-ethyloctane", "")
  CALL Species("C_{10}H_{22}",                       "15869-86-0", "NRJUFUBKIFIKFI-UHFFFAOYSA-N", "4-ethyloctane", "")
  CALL Species("C_{10}H_{22}",                       "52896-92-1", "ACYHSTUWOQNWCX-UHFFFAOYSA-N", "2,2,3-trimethylheptane", "")
  CALL Species("C_{10}H_{22}",                       "14720-74-2", "IIYGOARYARWJBO-UHFFFAOYSA-N", "2,2,4-trimethylheptane", "")
  CALL Species("C_{10}H_{22}",                       "20291-95-6", "GZJFAWOTMWATOS-UHFFFAOYSA-N", "2,2,5-trimethylheptane", "")
  CALL Species("C_{10}H_{22}",                        "1190-83-6", "FHJCGIUZJXWNET-UHFFFAOYSA-N", "2,2,6-trimethylheptane", "")
  CALL Species("C_{10}H_{22}",                       "52896-93-2", "QACXEXNKLFWKLK-UHFFFAOYSA-N", "2,3,3-trimethylheptane", "")
  CALL Species("C_{10}H_{22}",                       "52896-95-4", "UVVYAKOLFKEZEE-UHFFFAOYSA-N", "2,3,4-trimethylheptane", "")
  CALL Species("C_{10}H_{22}",                       "20278-85-7", "YKPNYFKOKKKGNM-UHFFFAOYSA-N", "2,3,5-trimethylheptane", "")
  CALL Species("C_{10}H_{22}",                        "4032-93-3", "IHPXJGBVRWFEJB-UHFFFAOYSA-N", "2,3,6-trimethylheptane", "")
  CALL Species("C_{10}H_{22}",                        "4032-92-2", "QALGVLROELGEEM-UHFFFAOYSA-N", "2,4,4-trimethylheptane", "")
  CALL Species("C_{10}H_{22}",                       "20278-84-6", "YMBNRMDSLJNNPF-UHFFFAOYSA-N", "2,4,5-trimethylheptane", "")
  CALL Species("C_{10}H_{22}",                        "2613-61-8", "YNLBBDHDNIXQNL-UHFFFAOYSA-N", "2,4,6-trimethylheptane", "")
  CALL Species("C_{10}H_{22}",                        "1189-99-7", "SOYLPZSOEXZMLE-UHFFFAOYSA-N", "2,5,5-trimethylheptane", "")
  CALL Species("C_{10}H_{22}",                       "20278-87-9", "WRBHKVWLEIYLDZ-UHFFFAOYSA-N", "3,3,4-trimethylheptane", "")
  CALL Species("C_{10}H_{22}",                        "7154-80-5", "VRVRZZWPKABUOE-UHFFFAOYSA-N", "3,3,5-trimethylheptane", "")
  CALL Species("C_{10}H_{22}",                       "20278-88-0", "BLNBSBLKPFFJKQ-UHFFFAOYSA-N", "3,4,4-trimethylheptane", "")
  CALL Species("C_{10}H_{22}",                       "20278-89-1", "LJIIBBYARMPSMT-UHFFFAOYSA-N", "3,4,5-trimethylheptane", "")
  CALL Species("C_{10}H_{22}",                       "14676-29-0", "NKMJCVVUYDKHAV-UHFFFAOYSA-N", "3-ethyl-2-methylheptane", "")
  CALL Species("C_{10}H_{22}",                       "52896-88-5", "OJDKRASKNKPYDH-UHFFFAOYSA-N", "4-ethyl-2-methylheptane", "")
  CALL Species("C_{10}H_{22}",                       "13475-78-0", "DGEMPTLPTFNEHJ-UHFFFAOYSA-N", "5-ethyl-2-methylheptane", "")
  CALL Species("C_{10}H_{22}",                       "17302-01-1", "HSOMNBKXPGCNBH-UHFFFAOYSA-N", "3-ethyl-3-methylheptane", "")
  CALL Species("C_{10}H_{22}",                       "52896-89-6", "BTGGSWBKRYMHQK-UHFFFAOYSA-N", "4-ethyl-3-methylheptane", "")
  CALL Species("C_{10}H_{22}",                       "52896-90-9", "VXARVYMIZCGZGG-UHFFFAOYSA-N", "3-ethyl-5-methylheptane", "")
  CALL Species("C_{10}H_{22}",                       "52896-91-0", "JZBKRUIGSVOOIC-UHFFFAOYSA-N", "3-ethyl-4-methylheptane", "")
  CALL Species("C_{10}H_{22}",                       "17302-04-4", "MPYQJQDSICRCJJ-UHFFFAOYSA-N", "4-ethyl-4-methylheptane", "")
  CALL Species("C_{10}H_{22}",                        "3178-29-8", "ABYGSZMCWVXFCQ-UHFFFAOYSA-N", "4-propylheptane", "")
  CALL Species("C_{10}H_{22}",                       "52896-87-4", "AZLAWGCUDHUQDB-UHFFFAOYSA-N", "4-(1-methylethyl)-heptane", "4-isopropylheptane")
  CALL Species("C_{10}H_{22}",                       "13475-81-5", "RMQHJMMCLSJULX-UHFFFAOYSA-N", "2,2,3,3-tetramethylhexane", "")
  CALL Species("C_{10}H_{22}",                       "52897-08-2", "MHPSPNGWFAGBNH-UHFFFAOYSA-N", "2,2,3,4-tetramethylhexane", "")
  CALL Species("C_{10}H_{22}",                       "52897-09-3", "GCFKTDRTZYDRBI-UHFFFAOYSA-N", "2,2,3,5-tetramethylhexane", "")
  CALL Species("C_{10}H_{22}",                       "51750-65-3", "PXHNHTBJHHSVPT-UHFFFAOYSA-N", "2,2,4,4-tetramethylhexane", "")
  CALL Species("C_{10}H_{22}",                       "16747-42-5", "KDRZICOOQNIJDN-UHFFFAOYSA-N", "2,2,4,5-tetramethylhexane", "")
  CALL Species("C_{10}H_{22}",                        "1071-81-4", "HXQDUXXBVMMIKL-UHFFFAOYSA-N", "2,2,5,5-tetramethylhexane", "")
  CALL Species("C_{10}H_{22}",                       "52897-10-6", "HIHSOGFAVTVMCY-UHFFFAOYSA-N", "2,3,3,4-tetramethylhexane", "")
  CALL Species("C_{10}H_{22}",                       "52897-11-7", "GCGFXFIPOBRMQT-UHFFFAOYSA-N", "2,3,3,5-tetramethylhexane", "")
  CALL Species("C_{10}H_{22}",                       "52897-12-8", "XDRDDPSGUQMOBO-UHFFFAOYSA-N", "2,3,4,4-tetramethylhexane", "")
  CALL Species("C_{10}H_{22}",                       "52897-15-1", "BHGNYYIOYPFWKC-UHFFFAOYSA-N", "2,3,4,5-tetramethylhexane", "")
  CALL Species("C_{10}H_{22}",                        "5171-84-6", "MCEYLFHKATVXLN-UHFFFAOYSA-N", "3,3,4,4-tetramethylhexane", "")
  CALL Species("C_{10}H_{22}",                       "20291-91-2", "XYDYODCWVCBIOQ-UHFFFAOYSA-N", "3-ethyl-2,2-dimethylhexane", "")
  CALL Species("C_{10}H_{22}",                       "52896-99-8", "QHLDBFLIDFTHQI-UHFFFAOYSA-N", "4-ethyl-2,2-dimethylhexane", "")
  CALL Species("C_{10}H_{22}",                       "52897-00-4", "PJIFKODHGMUPFH-UHFFFAOYSA-N", "3-ethyl-2,3-dimethylhexane", "")
  CALL Species("C_{10}H_{22}",                       "52897-01-5", "RHMRCCBCYFAZIK-UHFFFAOYSA-N", "4-ethyl-2,3-dimethylhexane", "")
  CALL Species("C_{10}H_{22}",                        "7220-26-0", "OSKIMJMPFNLVOU-UHFFFAOYSA-N", "3-ethyl-2,4-dimethylhexane", "")
  CALL Species("C_{10}H_{22}",                       "52897-03-7", "SIKFMUYSQCEQOO-UHFFFAOYSA-N", "4-ethyl-2,4-dimethylhexane", "")
  CALL Species("C_{10}H_{22}",                       "52897-04-8", "UJEUVDLASLOZIV-UHFFFAOYSA-N", "3-ethyl-2,5-dimethylhexane", "")
  CALL Species("C_{10}H_{22}",                       "52897-05-9", "ZRTXVJYJVBTXHE-UHFFFAOYSA-N", "4-ethyl-3,3-dimethylhexane", "")
  CALL Species("C_{10}H_{22}",                       "52897-06-0", "ZGJCTUKRTSBTIQ-UHFFFAOYSA-N", "3-ethyl-3,4-dimethylhexane", "")
  CALL Species("C_{10}H_{22}",                       "17302-02-2", "WWNGLKDLYKNGGT-UHFFFAOYSA-N", "3,3-diethylhexane", "")
  CALL Species("C_{10}H_{22}",                       "19398-77-7", "VBZCRMTUDYIWIH-UHFFFAOYSA-N", "3,4-diethylhexane", "")
  CALL Species("C_{10}H_{22}",                       "62016-13-1", "YBOXGRMAQIYMGV-UHFFFAOYSA-N", "2-methyl-3-(1-methylethyl)-hexane", "3-isopropyl-2-methylhexane")
  CALL Species("C_{10}H_{22}",                       "16747-44-7", "WKQBIIUOSATALN-UHFFFAOYSA-N", "2,2,3,3,4-pentamethylpentane", "")
  CALL Species("C_{10}H_{22}",                       "16747-45-8", "OWFKEHICSVOVAC-UHFFFAOYSA-N", "2,2,3,4,4-pentamethylpentane", "")
  CALL Species("C_{10}H_{22}",                       "52897-17-3", "AJDIFHIHSYVDGP-UHFFFAOYSA-N", "3-ethyl-2,2,3-trimethylpentane", "")
  CALL Species("C_{10}H_{22}",                       "52897-18-4", "VLIZIVHXZXQRDE-UHFFFAOYSA-N", "3-ethyl-2,2,4-trimethylpentane", "")
  CALL Species("C_{10}H_{22}",                       "52897-19-5", "OHZNMGSGEFVFTI-UHFFFAOYSA-N", "3-ethyl-2,3,4-trimethylpentane", "")
  CALL Species("C_{10}H_{22}",                       "52897-16-2", "DSSAZLXYIQIXGW-UHFFFAOYSA-N", "3,3-diethyl-2-methylpentane", "")
  CALL Species("C_{10}H_{22}",                       "13475-79-1", "YVYHOOYMDHZALB-UHFFFAOYSA-N", "2,4-dimethyl-3-(1-methylethyl)-pentane", "2,4-dimethyl-3-isopropylpentane")
  ! C11:
  CALL Species("C_{11}H_{24}",                        "1120-21-4", "RSJKGSCJYJTIGS-UHFFFAOYSA-N", "undecane", "")
  CALL Species("C_{11}H_{24}",                        "6975-98-0", "CNPVJWYWYZMPDS-UHFFFAOYSA-N", "2-methyldecane", "")
  CALL Species("C_{11}H_{24}",                       "13151-34-3", "JJRUZTXRDDMYGM-UHFFFAOYSA-N", "3-methyldecane", "")
  CALL Species("C_{11}H_{24}",                        "2847-72-5", "DVWZNKLWPILULD-UHFFFAOYSA-N", "4-methyldecane", "")
  CALL Species("C_{11}H_{24}",                       "13151-35-4", "QUYFPNWYGLFQQU-UHFFFAOYSA-N", "5-methyldecane", "")
  CALL Species("C_{11}H_{24}",                       "17302-14-6", "WDSBVMLUILIJOW-UHFFFAOYSA-N", "2,2-dimethylnonane", "")
  CALL Species("C_{11}H_{24}",                        "2884-06-2", "IGJRNTLDWLTHCQ-UHFFFAOYSA-N", "2,3-dimethylnonane", "")
  CALL Species("C_{11}H_{24}",                       "17302-24-8", "JZUUOAUSQCXSTN-UHFFFAOYSA-N", "2,4-dimethylnonane", "")
  CALL Species("C_{11}H_{24}",                       "17302-27-1", "NQUMJENPNGXAIH-UHFFFAOYSA-N", "2,5-dimethylnonane", "")
  CALL Species("C_{11}H_{24}",                       "17302-28-2", "MGNGOEWESNDQAN-UHFFFAOYSA-N", "2,6-dimethylnonane", "")
  CALL Species("C_{11}H_{24}",                       "17302-29-3", "QYQSPINNJUXEDY-UHFFFAOYSA-N", "2,7-dimethylnonane", "")
  CALL Species("C_{11}H_{24}",                       "17302-30-6", "FZFRYQHIQUEAAV-UHFFFAOYSA-N", "2,8-dimethylnonane", "")
  CALL Species("C_{11}H_{24}",                       "17302-15-7", "HTRYNYZYFGHKDV-UHFFFAOYSA-N", "3,3-dimethylnonane", "")
  CALL Species("C_{11}H_{24}",                       "17302-22-6", "PAXLEVPLFYBSQJ-UHFFFAOYSA-N", "3,4-dimethylnonane", "")
  CALL Species("C_{11}H_{24}",                       "17302-25-9", "BAFVBVRBYKSWCE-UHFFFAOYSA-N", "3,5-dimethylnonane", "")
  CALL Species("C_{11}H_{24}",                       "17302-31-7", "YHLBUWVGPXILSW-UHFFFAOYSA-N", "3,6-dimethylnonane", "")
  CALL Species("C_{11}H_{24}",                       "17302-32-8", "YGPVLXJHRFZYJJ-UHFFFAOYSA-N", "3,7-dimethylnonane", "")
  CALL Species("C_{11}H_{24}",                       "17302-18-0", "HARRKRKVTQABSF-UHFFFAOYSA-N", "4,4-dimethylnonane", "")
  CALL Species("C_{11}H_{24}",                       "17302-23-7", "JDNGDDOTBYZAGS-UHFFFAOYSA-N", "4,5-dimethylnonane", "")
  CALL Species("C_{11}H_{24}",                       "17302-26-0", "JZKWOUUZMBTBDO-UHFFFAOYSA-N", "4,6-dimethylnonane", "")
  CALL Species("C_{11}H_{24}",                        "6414-96-6", "BEPKYSJVUZWKMP-UHFFFAOYSA-N", "5,5-dimethylnonane", "")
  CALL Species("C_{11}H_{24}",                       "17302-11-3", "FKJSIWPQZCKMIL-UHFFFAOYSA-N", "3-ethylnonane", "")
  CALL Species("C_{11}H_{24}",                        "5911-05-7", "UGCQDCMVAKKTQG-UHFFFAOYSA-N", "4-ethylnonane", "")
  CALL Species("C_{11}H_{24}",                       "17302-12-4", "QPOVZYGNFWRMJE-UHFFFAOYSA-N", "5-ethylnonane", "")
  CALL Species("C_{11}H_{24}",                       "62016-26-6", "XZPXMWMVWDSEMY-UHFFFAOYSA-N", "2,2,3-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "18932-14-4", "IKMGZPRUMVFYBK-UHFFFAOYSA-N", "2,2,4-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-27-7", "CSVNISDVBYGXME-UHFFFAOYSA-N", "2,2,5-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-28-8", "NBIHFQKVSFKHGH-UHFFFAOYSA-N", "2,2,6-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-29-9", "QGQAHVJOBYNMFN-UHFFFAOYSA-N", "2,2,7-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-30-2", "KFYWDCDWVPXXCA-UHFFFAOYSA-N", "2,3,3-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-31-3", "SJQAEXCGOLTHPG-UHFFFAOYSA-N", "2,3,4-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-32-4", "CEOHXVQAHFSSEG-UHFFFAOYSA-N", "2,3,5-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-33-5", "MNFBUNLEXRMWOY-UHFFFAOYSA-N", "2,3,6-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-34-6", "XJKKSYAVEVAGFX-UHFFFAOYSA-N", "2,3,7-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-35-7", "CJXXTMYWNCYKHU-UHFFFAOYSA-N", "2,4,4-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-36-8", "UJYGOBMOHSFJQP-UHFFFAOYSA-N", "2,4,5-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-37-9", "XHNIFDXYGLPJLP-UHFFFAOYSA-N", "2,4,6-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-38-0", "DUHKHXHIPOGMOW-UHFFFAOYSA-N", "2,4,7-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-39-1", "DZKJZWAIBSEZKB-UHFFFAOYSA-N", "2,5,5-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-14-2", "VHBZECSWMWWTMQ-UHFFFAOYSA-N", "2,5,6-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "54166-32-4", "RUPXAIGHLDMSOL-UHFFFAOYSA-N", "2,6,6-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-40-4", "IJVIFYHTPQSPSW-UHFFFAOYSA-N", "3,3,4-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-41-5", "BEFCOJKHEMBPSK-UHFFFAOYSA-N", "3,3,5-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-42-6", "CFESHXNQRRYSED-UHFFFAOYSA-N", "3,3,6-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-43-7", "WWCBWKZBTINVDE-UHFFFAOYSA-N", "3,4,4-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-44-8", "BPFOUTQNEVQSFR-UHFFFAOYSA-N", "3,4,5-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-45-9", "RMGAENUTYIFEJO-UHFFFAOYSA-N", "3,4,6-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "61868-94-8", "FVYNWISHVUNIOZ-UHFFFAOYSA-N", "3,5,5-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "61868-95-9", "QGCXVKCDWOOWTQ-UHFFFAOYSA-N", "4,4,5-trimethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-16-4", "ITDXDXCBDQBZEB-UHFFFAOYSA-N", "2-methyl-3-ethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-17-5", "ZYEQSLLHBOITAM-UHFFFAOYSA-N", "2-methyl-4-ethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-18-6", "CQCKNPUKBOITAX-UHFFFAOYSA-N", "2-methyl-5-ethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-19-7", "AZXGABNJUBNOHW-UHFFFAOYSA-N", "2-methyl-6-ethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "17302-16-8", "DQNINFLTCGTQGU-UHFFFAOYSA-N", "3-methyl-3-ethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-20-0", "BTRAURWAFYPYMW-UHFFFAOYSA-N", "3-methyl-4-ethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-21-1", "LZFCEXJTLLKGPW-UHFFFAOYSA-N", "3-methyl-5-ethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-22-2", "FTQLPWORENXYAZ-UHFFFAOYSA-N", "3-methyl-6-ethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-23-3", "LXRSBMFQRZTMNK-UHFFFAOYSA-N", "4-methyl-3-ethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "17302-19-1", "IXZULXYHNRHENR-UHFFFAOYSA-N", "4-methyl-4-ethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-24-4", "DTSHQAHKLUTGAR-UHFFFAOYSA-N", "4-methyl-5-ethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-25-5", "NXUUVZOGBPCPDV-UHFFFAOYSA-N", "4-methyl-6-ethyloctane", "")
  CALL Species("C_{11}H_{24}",                       "17302-13-5", "VFAMBAFLNKONTN-UHFFFAOYSA-N", "4-propyloctane", "")
  CALL Species("C_{11}H_{24}",                       "62016-15-3", "VSJAVEFYQMREHJ-UHFFFAOYSA-N", "4-isopropyloctane", "")
  CALL Species("C_{11}H_{24}",                       "61868-40-4", "YPGGGWSLWOECQF-UHFFFAOYSA-N", "2,2,3,3-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-41-5", "HPKDRGPKZGWSND-UHFFFAOYSA-N", "2,2,3,4-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-42-6", "KXNFNEAZTWCHIL-UHFFFAOYSA-N", "2,2,3,5-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-43-7", "AQHGQPHAJYTMDJ-UHFFFAOYSA-N", "2,2,3,6-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-44-8", "JMRHVFQTDSCACD-UHFFFAOYSA-N", "2,2,4,4-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-45-9", "MPDBIZNQOUSUIN-UHFFFAOYSA-N", "2,2,4,5-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-46-0", "XSMHBEAIQLABAO-UHFFFAOYSA-N", "2,2,4,6-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-47-1", "VQJNSRPWQMPDDO-UHFFFAOYSA-N", "2,2,5,5-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-48-2", "ZYJXKOJVCNGFSL-UHFFFAOYSA-N", "2,2,5,6-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "40117-45-1", "GKNMBVVJQTWDRT-UHFFFAOYSA-N", "2,2,6,6-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-49-3", "OHIUILFTBGSFAG-UHFFFAOYSA-N", "2,3,3,4-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-50-6", "XGCFPCZVDOZHLT-UHFFFAOYSA-N", "2,3,3,5-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-51-7", "ZVRALCAHDAJURU-UHFFFAOYSA-N", "2,3,3,6-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-52-8", "SXHQWTUVNZIMTP-UHFFFAOYSA-N", "2,3,4,4-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-53-9", "LGUFKOOYNDNVNP-UHFFFAOYSA-N", "2,3,4,5-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-54-0", "WOHJQOFSYZPITE-UHFFFAOYSA-N", "2,3,4,6-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-55-1", "QSBIXVGCKCYBBL-UHFFFAOYSA-N", "2,3,5,5-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "52670-32-3", "RAHGVMDMAJFLTP-UHFFFAOYSA-N", "2,3,5,6-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-56-2", "QALMSCCHYDFGLF-UHFFFAOYSA-N", "2,4,4,5-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-57-3", "DTMSZIVVBLZFEV-UHFFFAOYSA-N", "2,4,4,6-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-58-4", "JVYVKEOAYQAABU-UHFFFAOYSA-N", "2,4,5,5-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-59-5", "CFNIIBCZVLNKPV-UHFFFAOYSA-N", "3,3,4,4-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-60-8", "QVCSKDHUHWRVTA-UHFFFAOYSA-N", "3,3,4,5-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-61-9", "GALDMESQGNRJGE-UHFFFAOYSA-N", "3,3,5,5-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-62-0", "CXIAIMLBUJNNJR-UHFFFAOYSA-N", "3,4,4,5-tetramethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61869-03-2", "SBWMIVQVGGUMCE-UHFFFAOYSA-N", "2,2-dimethyl-3-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "62016-46-0", "BJOUMNMMNSZESV-UHFFFAOYSA-N", "2,2-dimethyl-4-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "62016-47-1", "HVVFIKMZXFNXKA-UHFFFAOYSA-N", "2,2-dimethyl-5-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-21-1", "KQUHXOYVQZYOOF-UHFFFAOYSA-N", "2,3-dimethyl-3-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-22-2", "CNEPAGDSOHLVRW-UHFFFAOYSA-N", "2,3-dimethyl-4-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-23-3", "BFCKNSGKYWMUNP-UHFFFAOYSA-N", "2,3-dimethyl-5-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-24-4", "OBCGEYFMJYXFJV-UHFFFAOYSA-N", "2,4-dimethyl-3-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-25-5", "OETKSNKIFXLJDO-UHFFFAOYSA-N", "2,4-dimethyl-4-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-26-6", "RZHNOEUMKYRHIX-UHFFFAOYSA-N", "2,4-dimethyl-5-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-27-7", "DEZOXTLNRYCKNK-UHFFFAOYSA-N", "2,5-dimethyl-3-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-28-8", "CUHCBWKQRDWQMN-UHFFFAOYSA-N", "2,5-dimethyl-4-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-29-9", "GPZDEXDVXFJVLH-UHFFFAOYSA-N", "2,5-dimethyl-5-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-30-2", "BFXHSWDHLPKFSH-UHFFFAOYSA-N", "2,6-dimethyl-3-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-31-3", "GBSGJLJGXHBTQQ-UHFFFAOYSA-N", "2,6-dimethyl-4-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-32-4", "QXJXAHFNWHEQEU-UHFFFAOYSA-N", "3,3-dimethyl-4-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-33-5", "FLLHBJKGZIOYAA-UHFFFAOYSA-N", "3,3-dimethyl-5-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-34-6", "HKSCZRRMBTUZLO-UHFFFAOYSA-N", "3,4-dimethyl-3-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-35-7", "RMVWUAZCGGWZKA-UHFFFAOYSA-N", "3,4-dimethyl-4-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-36-8", "PCRCVFLPWIQGNJ-UHFFFAOYSA-N", "3,4-dimethyl-5-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-37-9", "FQOCKNRDCVODKW-UHFFFAOYSA-N", "3,5-dimethyl-3-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-38-0", "IKVNPWSSYMJDKE-UHFFFAOYSA-N", "3,5-dimethyl-4-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-39-1", "YMWPSGKMCMMFSS-UHFFFAOYSA-N", "4,4-dimethyl-3-ethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-96-0", "AXOUMHTUGUFOKF-UHFFFAOYSA-N", "2-methyl-4-propylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-97-1", "WAMROSASDOIVDE-UHFFFAOYSA-N", "3-methyl-4-propylheptane", "")
  CALL Species("C_{11}H_{24}",                       "17302-20-4", "RDZFGBZTNOYNAH-UHFFFAOYSA-N", "4-methyl-4-propylheptane", "")
  CALL Species("C_{11}H_{24}",                        "6876-18-2", "GECZBVJYUAPYFD-UHFFFAOYSA-N", "2-methyl-3-isopropylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-98-2", "ADLZFTVQXMDTCY-UHFFFAOYSA-N", "2-methyl-4-isopropylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-99-3", "SRECVVXQFXRBNI-UHFFFAOYSA-N", "3-methyl-4-isopropylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61869-00-9", "LSJZMBVEVGYTAG-UHFFFAOYSA-N", "4-methyl-4-isopropylheptane", "")
  CALL Species("C_{11}H_{24}",                       "17302-17-9", "XATIZWAWQAIMQJ-UHFFFAOYSA-N", "3,3-diethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61869-01-0", "UWOPVDFSBGDRJR-UHFFFAOYSA-N", "3,4-diethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61869-02-1", "OBHTWTQZINRNAB-UHFFFAOYSA-N", "3,5-diethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "17302-21-5", "WDTMGYSKSAMSPF-UHFFFAOYSA-N", "4,4-diethylheptane", "")
  CALL Species("C_{11}H_{24}",                       "60302-21-8", "MDOHZJRFNISIIH-UHFFFAOYSA-N", "4-{\it tert}-butylheptane", "")
  CALL Species("C_{11}H_{24}",                       "61868-85-7", "RSEBTOONCGVAKN-UHFFFAOYSA-N", "2,2,3,3,4-pentamethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-86-8", "QGVXFIIZUSICQO-UHFFFAOYSA-N", "2,2,3,3,5-pentamethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-87-9", "HBVQLCKOFAUQGY-UHFFFAOYSA-N", "2,2,3,4,4-pentamethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-88-0", "LDUJDGROUWHEKU-UHFFFAOYSA-N", "2,2,3,4,5-pentamethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "14739-73-2", "UKYAYXXQXKPIBL-UHFFFAOYSA-N", "2,2,3,5,5-pentamethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "60302-23-0", "JQFZWBZEZYOCQL-UHFFFAOYSA-N", "2,2,4,4,5-pentamethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-89-1", "GHRPCKCGHFDDOH-UHFFFAOYSA-N", "2,3,3,4,4-pentamethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "52670-33-4", "STMIMOZUYGUOQZ-UHFFFAOYSA-N", "2,3,3,4,5-pentamethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-72-2", "HUAYVWMWWLHZAB-UHFFFAOYSA-N", "2,2,3-trimethyl-3-ethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-73-3", "ZSUMTZIXJNZKBI-UHFFFAOYSA-N", "2,2,3-trimethyl-4-ethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-74-4", "UDUTWNYTAVENNO-UHFFFAOYSA-N", "2,2,4-trimethyl-3-ethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-75-5", "YFQDCXGMLYJDSG-UHFFFAOYSA-N", "2,2,4-trimethyl-4-ethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-76-6", "GJVFOLLEVKEKMU-UHFFFAOYSA-N", "2,2,5-trimethyl-3-ethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-77-7", "UKTBERWYLDSZDQ-UHFFFAOYSA-N", "2,2,5-trimethyl-4-ethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-78-8", "DNDCULLMOLSZER-UHFFFAOYSA-N", "2,3,3-trimethyl-4-ethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-79-9", "XCNOKZUAQAZTAJ-UHFFFAOYSA-N", "2,3,4-trimethyl-3-ethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-80-2", "HNNSQDJQJRTMMW-UHFFFAOYSA-N", "2,3,4-trimethyl-4-ethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-81-3", "VYVPEEFLIYVOTH-UHFFFAOYSA-N", "2,3,5-trimethyl-3-ethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-82-4", "LDKMZLRIVWGQRW-UHFFFAOYSA-N", "2,3,5-trimethyl-4-ethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-83-5", "MTYPKDRKJRWXKB-UHFFFAOYSA-N", "2,4,4-trimethyl-3-ethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-84-6", "ADDYWUNUHVKQGT-UHFFFAOYSA-N", "3,3,4-trimethyl-4-ethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-63-1", "VKABOTNMFKIVDN-UHFFFAOYSA-N", "2,2-dimethyl-3-isopropylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-64-2", "XVWGUCNHEXRNJU-UHFFFAOYSA-N", "2,3-dimethyl-3-isopropylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-65-3", "QRNSQXWPYPLHTL-UHFFFAOYSA-N", "2,4-dimethyl-3-isopropylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-66-4", "QKICRGDGSLQROM-UHFFFAOYSA-N", "2,5-dimethyl-3-isopropylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-67-5", "QNRHKXCWHYUEKP-UHFFFAOYSA-N", "2-methyl-3,3-diethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-68-6", "MWHIPTVSAPEYPC-UHFFFAOYSA-N", "2-methyl-3,4-diethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-69-7", "UNPXTJSCIHOMKB-UHFFFAOYSA-N", "2-methyl-4,4-diethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-70-0", "ZFMWIWWLOKHXHC-UHFFFAOYSA-N", "3-methyl-3,4-diethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "61868-71-1", "CUHCCZRBXHOAMW-UHFFFAOYSA-N", "3-methyl-4,4-diethylhexane", "")
  CALL Species("C_{11}H_{24}",                       "60302-27-4", "JKJQSSSRSKPVEM-UHFFFAOYSA-N", "2,2,3,3,4,4-hexamethylpentane", "")
  CALL Species("C_{11}H_{24}",                       "61868-93-7", "HBSINJYUKWVYBC-UHFFFAOYSA-N", "2,2,3,4-tetramethyl-3-ethylpentane", "")
  CALL Species("C_{11}H_{24}",                        "3178-30-1", "FNWCWFQEEKFGHB-UHFFFAOYSA-N", "2,2,4,4-tetramethyl-3-ethylpentane", "")
  CALL Species("C_{11}H_{24}",                       "61868-90-4", "SZPCILWZDHUKLW-UHFFFAOYSA-N", "2,2,4-trimethyl-3-isopropylpentane", "")
  CALL Species("C_{11}H_{24}",                       "61868-91-5", "FHBFVPOXLRZZPP-UHFFFAOYSA-N", "2,3,4-trimethyl-3-isopropylpentane", "")
  CALL Species("C_{11}H_{24}",                       "60302-28-5", "KCQSYGZVYTYSLG-UHFFFAOYSA-N", "2,2-dimethyl-3,3-diethylpentane", "")
  CALL Species("C_{11}H_{24}",                       "61868-92-6", "VVQOQVXGUMJNET-UHFFFAOYSA-N", "2,4-dimethyl-3,3-diethylpentane", "")
  ! C12:
  CALL Species("C_{12}H_{26}",                         "112-40-3", "SNRUBQQJIBEYMU-UHFFFAOYSA-N", "dodecane", "")
  CALL Species("C_{12}H_{26}",                       "31807-55-3", "GTJOHISYCKPIMT-UHFFFAOYSA-N", "2-methylundecane", "")
  CALL Species("C_{12}H_{26}",                        "1002-43-3", "HTZWVZNRDDOFEI-UHFFFAOYSA-N", "3-methylundecane", "")
  CALL Species("C_{12}H_{26}",                        "2980-69-0", "KNMXZGDUJVOTOC-UHFFFAOYSA-N", "4-methylundecane", "")
  CALL Species("C_{12}H_{26}",                        "1632-70-8", "QULNVKABFWNUCW-UHFFFAOYSA-N", "5-methylundecane", "")
  CALL Species("C_{12}H_{26}",                       "17302-33-9", "VPYZCUCKYWHJGX-UHFFFAOYSA-N", "6-methylundecane", "")
  CALL Species("C_{12}H_{26}",                       "17302-37-3", "WBWYXWILSHQILH-UHFFFAOYSA-N", "2,2-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                       "17312-44-6", "ZCTGYLNFWOQVHV-UHFFFAOYSA-N", "2,3-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                        "2801-84-5", "OJAFXEXESSNPMH-UHFFFAOYSA-N", "2,4-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                       "17312-50-4", "DQHKBYZSYRJBMD-UHFFFAOYSA-N", "2,5-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                       "13150-81-7", "DHJGXZWEQBKLNV-UHFFFAOYSA-N", "2,6-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                       "17312-51-5", "RVQIXUWWPOTVNP-UHFFFAOYSA-N", "2,7-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                       "17312-52-6", "KSRGGHUVCVWVDW-UHFFFAOYSA-N", "2,8-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                        "1002-17-1", "HWISDPDDDUZJAW-UHFFFAOYSA-N", "2,9-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                       "17302-38-4", "URERYYDSQOIHQK-UHFFFAOYSA-N", "3,3-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                       "17312-45-7", "NRBMEEDORZDRIT-UHFFFAOYSA-N", "3,4-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                       "17312-48-0", "XXSUEVGKGOUJMD-UHFFFAOYSA-N", "3,5-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                       "17312-53-7", "NQWFSCYWTXQNGG-UHFFFAOYSA-N", "3,6-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                       "17312-54-8", "VDAVEASVPZDNQB-UHFFFAOYSA-N", "3,7-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                       "17312-55-9", "KMAHIPNGGSOJSM-UHFFFAOYSA-N", "3,8-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                       "17312-39-9", "WZCACTKWHXCWFZ-UHFFFAOYSA-N", "4,4-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                       "17312-46-8", "QZFIIEYSHODCSV-UHFFFAOYSA-N", "4,5-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                       "17312-49-1", "TVGNRLXIJXKVGD-UHFFFAOYSA-N", "4,6-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                       "17312-56-0", "GCKWUFQALHAZDH-UHFFFAOYSA-N", "4,7-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                       "17453-92-8", "RNXSOUOIPAWOAC-UHFFFAOYSA-N", "5,5-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                        "1636-43-7", "NCJIZIYQFWXMFZ-UHFFFAOYSA-N", "5,6-dimethyldecane", "")
  CALL Species("C_{12}H_{26}",                       "17085-96-0", "ZBDDVSBBCGZQDV-UHFFFAOYSA-N", "3-ethyldecane", "")
  CALL Species("C_{12}H_{26}",                        "1636-44-8", "IGTKVLJTIZALGL-UHFFFAOYSA-N", "4-ethyldecane", "")
  CALL Species("C_{12}H_{26}",                       "17302-36-2", "BCQLAHKMMOGIIS-UHFFFAOYSA-N", "5-ethyldecane", "")
  CALL Species("C_{12}H_{26}",                       "55499-04-2", "XXNUJUNKYOZLAJ-UHFFFAOYSA-N", "2,2,3-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-50-3", "YYYAICGYFLHBIN-UHFFFAOYSA-N", "2,2,4-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-51-4", "ZIXRFABUOOUIMA-UHFFFAOYSA-N", "2,2,5-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-52-5", "BUTMZMIZSWINDK-UHFFFAOYSA-N", "2,2,6-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-53-6", "PXOURNMYVAOUML-UHFFFAOYSA-N", "2,2,7-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-54-7", "WXUAVABOZJOELJ-UHFFFAOYSA-N", "2,2,8-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-55-8", "RSUFJUIYLXRYFG-UHFFFAOYSA-N", "2,3,3-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-56-9", "RRZRZLRUYPZYEH-UHFFFAOYSA-N", "2,3,4-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-57-0", "IESSJVKAXJWJFD-UHFFFAOYSA-N", "2,3,5-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-58-1", "AUXUELQPRQTQBE-UHFFFAOYSA-N", "2,3,6-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-59-2", "GGUGBCMHEULHJS-UHFFFAOYSA-N", "2,3,7-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-60-5", "GQHCJRDJABETBJ-UHFFFAOYSA-N", "2,3,8-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-61-6", "RRLKOZCPQXMIIJ-UHFFFAOYSA-N", "2,4,4-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-62-7", "NZAPYIGNIYRJNJ-UHFFFAOYSA-N", "2,4,5-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-10-5", "QCQMRLDBNMVKOZ-UHFFFAOYSA-N", "2,4,6-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-11-6", "OHBDIVOYUFLOTO-UHFFFAOYSA-N", "2,4,7-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "49542-74-7", "PZXMOJBYRFSNJH-UHFFFAOYSA-N", "2,4,8-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-12-7", "ZRRKWMYBHUMSTG-UHFFFAOYSA-N", "2,5,5-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-13-8", "KXHDWXIQHWSOHI-UHFFFAOYSA-N", "2,5,6-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-14-9", "KMBPYTMBXFKXON-UHFFFAOYSA-N", "2,5,7-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "49557-09-7", "YYZRNFMGAQMLJJ-UHFFFAOYSA-N", "2,5,8-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-15-0", "MTPVPWZHWOHMQT-UHFFFAOYSA-N", "2,6,6-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-16-1", "PGLYQCHTCUBASV-UHFFFAOYSA-N", "2,6,7-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-17-2", "RCTMUTWYOMEASD-UHFFFAOYSA-N", "2,7,7-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-18-3", "RNCRKMFELJSFRA-UHFFFAOYSA-N", "3,3,4-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-19-4", "KRWQPUYFIGHZNF-UHFFFAOYSA-N", "3,3,5-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-20-7", "GPSMOBMYMZBKGT-UHFFFAOYSA-N", "3,3,6-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-21-8", "NZYGFHSHEQXPFT-UHFFFAOYSA-N", "3,3,7-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-22-9", "UYNWXBDPBKCTRS-UHFFFAOYSA-N", "3,4,4-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-23-0", "PFVDUOFPBLPHEQ-UHFFFAOYSA-N", "3,4,5-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-24-1", "VSRUKIVBXBGRDU-UHFFFAOYSA-N", "3,4,6-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "27802-85-3", "HQIBDLHPQGDJEW-UHFFFAOYSA-N", "3,4,7-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-25-2", "MUVSDRJGHVLJQX-UHFFFAOYSA-N", "3,5,5-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-26-3", "HMCPPWBQUPZETF-UHFFFAOYSA-N", "3,5,6-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-27-4", "LLVWEFHXQBAEJX-UHFFFAOYSA-N", "3,5,7-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-28-5", "CBDYLMAJISNHBH-UHFFFAOYSA-N", "3,6,6-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-29-6", "QLJZSYCAASGFGU-UHFFFAOYSA-N", "4,4,5-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-30-9", "KBTTWWLCACXOSV-UHFFFAOYSA-N", "4,4,6-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-31-0", "QVDLUCCOJXIINS-UHFFFAOYSA-N", "4,5,5-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62211-85-2", "BBEKERHGCOZOIX-UHFFFAOYSA-N", "4,5,6-trimethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-73-0", "NVCDNLWOUSWNRQ-UHFFFAOYSA-N", "2-methyl-3-ethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-37-6", "GRYPXYDXCQOVAY-UHFFFAOYSA-N", "2-methyl-4-ethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-38-7", "WZWWJMDOQQIPJZ-UHFFFAOYSA-N", "2-methyl-5-ethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-39-8", "GBMUYXDAAUUNLY-UHFFFAOYSA-N", "2-methyl-6-ethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-40-1", "PHPGTUZXJVCCKP-UHFFFAOYSA-N", "2-methyl-7-ethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "17302-39-5", "MJZXPSBAAXOUJF-UHFFFAOYSA-N", "3-methyl-3-ethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-41-2", "ZELDGOVWPZHVJS-UHFFFAOYSA-N", "3-methyl-4-ethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-42-3", "GDUZVOIDJDZQBC-UHFFFAOYSA-N", "3-methyl-5-ethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-43-4", "WSMCXXNPFLZSMY-UHFFFAOYSA-N", "3-methyl-6-ethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-44-5", "FSGFZSLYTXQNCW-UHFFFAOYSA-N", "3-methyl-7-ethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-45-6", "AUMKRFNJSTUECA-UHFFFAOYSA-N", "4-methyl-3-ethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "17312-40-2", "LXQCGRFCISXKCQ-UHFFFAOYSA-N", "4-methyl-4-ethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-46-7", "BDJIXOFXGLDYLN-UHFFFAOYSA-N", "4-methyl-5-ethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-47-8", "SULZKBWLMOBACQ-UHFFFAOYSA-N", "4-methyl-6-ethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-48-9", "HYIBWPIYSSTZLB-UHFFFAOYSA-N", "4-methyl-7-ethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-49-0", "ZBALIHDUORTBQR-UHFFFAOYSA-N", "5-methyl-3-ethylnonane", "")
  CALL Species("C_{12}H_{26}",                        "1632-71-9", "HKUUNAOHRKAJIY-UHFFFAOYSA-N", "5-methyl-4-ethylnonane", "")
  CALL Species("C_{12}H_{26}",                       "14531-16-9", "DZSBQMNPPIKBNF-UHFFFAOYSA-N", "5-methyl-5-ethylnonane", "")
  CALL Species("C_{12}H_{26}",                        "6165-37-3", "QLKPGWNXMMBQMG-UHFFFAOYSA-N", "4-propylnonane", "")
  CALL Species("C_{12}H_{26}",                         "998-35-6", "ISHSSTRAYNPQFX-UHFFFAOYSA-N", "5-propylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-71-8", "BXYKZQOGHMLDIE-UHFFFAOYSA-N", "4-isopropylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62184-72-9", "ALZCRHWYCQKCQK-UHFFFAOYSA-N", "5-isopropylnonane", "")
  CALL Species("C_{12}H_{26}",                       "62183-74-8", "UXQAEOWCSOPBLF-UHFFFAOYSA-N", "2,2,3,3-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-75-9", "HEJULKHHKGYPNT-UHFFFAOYSA-N", "2,2,3,4-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-76-0", "MRBTZFDZTJYCAO-UHFFFAOYSA-N", "2,2,3,5-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-77-1", "YRSBZJIBIXVOME-UHFFFAOYSA-N", "2,2,3,6-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-78-2", "CPEHPCZXBBEYOL-UHFFFAOYSA-N", "2,2,3,7-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-79-3", "TYUFTNSABIBNRY-UHFFFAOYSA-N", "2,2,4,4-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-80-6", "SNJCVPBJFQJMFW-UHFFFAOYSA-N", "2,2,4,5-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-81-7", "ODZRFEPMTHWMBK-UHFFFAOYSA-N", "2,2,4,6-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-82-8", "QHDQKPZLRKHEHV-UHFFFAOYSA-N", "2,2,4,7-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-83-9", "RJJMZKVKQLGGDK-UHFFFAOYSA-N", "2,2,5,5-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-84-0", "BQUVPJLOJUCQGM-UHFFFAOYSA-N", "2,2,5,6-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-19-3", "ISTINYWNHRBEDQ-UHFFFAOYSA-N", "2,2,5,7-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-20-6", "BANZIZHQGFRTMI-UHFFFAOYSA-N", "2,2,6,6-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-21-7", "ZSXJNQURTCJFGP-UHFFFAOYSA-N", "2,2,6,7-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                        "1071-31-4", "QZUFNKONEPLWBC-UHFFFAOYSA-N", "2,2,7,7-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-22-8", "ITTRGXLKGDMYDX-UHFFFAOYSA-N", "2,3,3,4-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-23-9", "RVOIBAZQSHFKMM-UHFFFAOYSA-N", "2,3,3,5-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-24-0", "MBJHOSHYIFXEPL-UHFFFAOYSA-N", "2,3,3,6-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-25-1", "WWQBXZREQWIXFF-UHFFFAOYSA-N", "2,3,3,7-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-26-2", "MELCVUZPHQRNTK-UHFFFAOYSA-N", "2,3,4,4-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-27-3", "AMIZVDCCXDQOHL-UHFFFAOYSA-N", "2,3,4,5-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-28-4", "OYZWSGWTBSWCLN-UHFFFAOYSA-N", "2,3,4,6-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-29-5", "YUCNZDFGMLKYFT-UHFFFAOYSA-N", "2,3,4,7-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-30-8", "RQOLUXQWYAKKTG-UHFFFAOYSA-N", "2,3,5,5-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-31-9", "IPYLOMHOPWIUFA-UHFFFAOYSA-N", "2,3,5,6-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-32-0", "AIUMUUQTYGQPMV-UHFFFAOYSA-N", "2,3,5,7-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-33-1", "CVRMZIOMFWYAPV-UHFFFAOYSA-N", "2,3,6,6-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "52670-34-5", "FZCGYGCYZRXLDY-UHFFFAOYSA-N", "2,3,6,7-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-34-2", "OBWRJGVWBWXXAO-UHFFFAOYSA-N", "2,4,4,5-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-35-3", "UQCGTRPILZWACX-UHFFFAOYSA-N", "2,4,4,6-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "35866-96-7", "QYUWMFZODNLLRJ-UHFFFAOYSA-N", "2,4,4,7-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-36-4", "OMFWIGPPTXTQHH-UHFFFAOYSA-N", "2,4,5,5-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-37-5", "YJMGKHXCSVEVKG-UHFFFAOYSA-N", "2,4,5,6-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                        "2217-17-6", "PWYLDJLMYGOPSR-UHFFFAOYSA-N", "2,4,5,7-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-38-6", "VVEPRQWGSASPPP-UHFFFAOYSA-N", "2,4,6,6-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-39-7", "DLCPMTDXNVOOLU-UHFFFAOYSA-N", "2,5,5,6-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-40-0", "SQOVOQLOSFZJGI-UHFFFAOYSA-N", "2,5,6,6-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-41-1", "VXEUNQVVVNDDTL-UHFFFAOYSA-N", "3,3,4,4-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-42-2", "FATMBSLXTUXZBP-UHFFFAOYSA-N", "3,3,4,5-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-43-3", "VCXMKRQQDPQALS-UHFFFAOYSA-N", "3,3,4,6-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-44-4", "UJLDJWNVZROTLS-UHFFFAOYSA-N", "3,3,5,5-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-45-5", "AQPSWWWUVHSZJK-UHFFFAOYSA-N", "3,3,5,6-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-46-6", "PAEUGKMMSQUAGH-UHFFFAOYSA-N", "3,3,6,6-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62199-47-7", "HWNTXBYEGXKWKG-UHFFFAOYSA-N", "3,4,4,5-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62185-19-7", "LNRFUGUGPGLBLV-UHFFFAOYSA-N", "3,4,4,6-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62185-20-0", "NCNZUEBQNGRARF-UHFFFAOYSA-N", "3,4,5,5-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62185-21-1", "NADJQGPTQSFIHB-UHFFFAOYSA-N", "3,4,5,6-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62185-22-2", "INTYEXUWOYUVGJ-UHFFFAOYSA-N", "4,4,5,5-tetramethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-95-3", "LIZQPVRBTUZZCL-UHFFFAOYSA-N", "2,2-dimethyl-3-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-96-4", "PEGJIJFSOTZJFH-UHFFFAOYSA-N", "2,2-dimethyl-4-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-97-5", "ACWGGQUVVQOZPJ-UHFFFAOYSA-N", "2,2-dimethyl-5-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-98-6", "SHNILIMSRTWRKG-UHFFFAOYSA-N", "2,2-dimethyl-6-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-99-7", "NDFHPGVSCVJNFR-UHFFFAOYSA-N", "2,3-dimethyl-3-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62184-00-3", "DCBIYDZULXWXTQ-UHFFFAOYSA-N", "2,3-dimethyl-4-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62184-01-4", "NPUNZBBVZZNRNS-UHFFFAOYSA-N", "2,3-dimethyl-5-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62184-02-5", "BWSXPPUMTZVKPA-UHFFFAOYSA-N", "2,3-dimethyl-6-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62184-03-6", "QYHDQDAUJCPVFJ-UHFFFAOYSA-N", "2,4-dimethyl-3-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62184-04-7", "XAKSPVTVTMAMDT-UHFFFAOYSA-N", "2,4-dimethyl-4-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62184-05-8", "JHDZOLPKXSWZKD-UHFFFAOYSA-N", "2,4-dimethyl-5-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62184-06-9", "VGDSKYYXZZCASQ-UHFFFAOYSA-N", "2,4-dimethyl-6-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62184-07-0", "BCQWMBQACXNHGP-UHFFFAOYSA-N", "2,5-dimethyl-3-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62184-08-1", "IMMDGMKEESKIQT-UHFFFAOYSA-N", "2,5-dimethyl-4-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62184-09-2", "BVAIFAWSJOPLJD-UHFFFAOYSA-N", "2,5-dimethyl-5-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-50-0", "CYTDZGRTFHXLQS-UHFFFAOYSA-N", "2,5-dimethyl-6-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-51-1", "WAFCPUFGSNTFSP-UHFFFAOYSA-N", "2,6-dimethyl-3-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-52-2", "JIOQMQLJPQDNFA-UHFFFAOYSA-N", "2,6-dimethyl-4-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-53-3", "ZBMWKURVBLDBAH-UHFFFAOYSA-N", "2,6-dimethyl-5-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-54-4", "DGFWFJRTTJTVEQ-UHFFFAOYSA-N", "2,6-dimethyl-6-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-55-5", "XEMFRSYZKNPRTA-UHFFFAOYSA-N", "2,7-dimethyl-3-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-56-6", "KSVMIUVYLYZDMT-UHFFFAOYSA-N", "2,7-dimethyl-4-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-57-7", "BKNJDUXGYCILJT-UHFFFAOYSA-N", "3,3-dimethyl-4-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-58-8", "GQTCKTISADWVNV-UHFFFAOYSA-N", "3,3-dimethyl-5-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-59-9", "DDRNTRVXHMVVLA-UHFFFAOYSA-N", "3,3-dimethyl-6-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62212-28-6", "AOQVLRSMPNHJPF-UHFFFAOYSA-N", "3,4-dimethyl-3-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-60-2", "WBKPODGAYHVUMM-UHFFFAOYSA-N", "3,4-dimethyl-4-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-61-3", "LEPWFUNEXVKPDO-UHFFFAOYSA-N", "3,4-dimethyl-5-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-62-4", "DPDGQUDZYGOBAM-UHFFFAOYSA-N", "3,4-dimethyl-6-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-63-5", "QYJKERUDQYOWRC-UHFFFAOYSA-N", "3,5-dimethyl-3-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-64-6", "OMAJEXDENFMCIN-UHFFFAOYSA-N", "3,5-dimethyl-4-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-65-7", "OLBJDMSGRVKJMF-UHFFFAOYSA-N", "3,5-dimethyl-5-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-66-8", "LLHGAWREVJAITE-UHFFFAOYSA-N", "3,5-dimethyl-6-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-67-9", "MNXPIOFJFYQCKE-UHFFFAOYSA-N", "3,6-dimethyl-3-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-68-0", "RTVRGBUFZDOPIH-UHFFFAOYSA-N", "3,6-dimethyl-4-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-69-1", "RHSANRLUOZFJDW-UHFFFAOYSA-N", "4,4-dimethyl-3-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-70-4", "BETVNWPSZXTJLK-UHFFFAOYSA-N", "4,4-dimethyl-5-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-71-5", "UGEWSKWSWHZABT-UHFFFAOYSA-N", "4,4-dimethyl-6-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-72-6", "MNDPOEACQSQPCJ-UHFFFAOYSA-N", "4,5-dimethyl-3-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-73-7", "MOEUNMMPAYEJOZ-UHFFFAOYSA-N", "4,5-dimethyl-4-ethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "17302-40-8", "DGJISSKLLWWXTG-UHFFFAOYSA-N", "3,3-diethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-92-0", "HILZBOWVGZXYGQ-UHFFFAOYSA-N", "3,4-diethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-93-1", "DSYIMNMBWBEOHY-UHFFFAOYSA-N", "3,5-diethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-94-2", "UTCTYSTYZOAAOS-UHFFFAOYSA-N", "3,6-diethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "17312-42-4", "FTEQKVNFZNDQJG-UHFFFAOYSA-N", "4,4-diethyloctane", "")
  CALL Species("C_{12}H_{26}",                        "1636-41-5", "XRVCHXCONYJHLU-UHFFFAOYSA-N", "4,5-diethyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62184-33-2", "FJPVBTBEYVNWCA-UHFFFAOYSA-N", "2-methyl-4-propyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62184-34-3", "NYBDGQCIJCAIJL-UHFFFAOYSA-N", "2-methyl-5-propyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62184-35-4", "CTXPEFTVFFFIBQ-UHFFFAOYSA-N", "3-methyl-4-propyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62184-36-5", "VOSHMCRSZWNTLJ-UHFFFAOYSA-N", "3-methyl-5-propyloctane", "")
  CALL Species("C_{12}H_{26}",                       "17312-41-3", "RNKCZSBPRRHTOZ-UHFFFAOYSA-N", "4-methyl-4-propyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-85-1", "MVUWSCFPURGAJF-UHFFFAOYSA-N", "4-methyl-5-propyloctane", "")
  CALL Species("C_{12}H_{26}",                       "13287-19-9", "KBIUKRVYBYJVSS-UHFFFAOYSA-N", "2-methyl-3-isopropyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-86-2", "NRRICFUUPCDSKW-UHFFFAOYSA-N", "2-methyl-4-isopropyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-87-3", "OPKVPKXVSDEXOZ-UHFFFAOYSA-N", "2-methyl-5-isopropyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-88-4", "UXTHXSNAZBLVOD-UHFFFAOYSA-N", "3-methyl-4-isopropyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-89-5", "VEJPAYNPRZYJNN-UHFFFAOYSA-N", "3-methyl-5-isopropyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-90-8", "KCNISWXEVSJYRS-UHFFFAOYSA-N", "4-methyl-4-isopropyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62183-91-9", "HEFVKUUZAXCMQX-UHFFFAOYSA-N", "4-methyl-5-isopropyloctane", "")
  CALL Species("C_{12}H_{26}",                       "62184-32-1", "RWILNGDOCMDMJW-UHFFFAOYSA-N", "4-{\it tert}-butyloctane", "")
  CALL Species("C_{12}H_{26}",                       "13475-82-6", "VKPSKYDESGTTFR-UHFFFAOYSA-N", "2,2,4,6,6-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-80-5", "NTDYBHYVUCYILC-UHFFFAOYSA-N", "2,2,3,3,4-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-81-6", "ZLCPPJIFAFIYPV-UHFFFAOYSA-N", "2,2,3,3,5-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-82-7", "LYTFNNQFUUFUDH-UHFFFAOYSA-N", "2,2,3,3,6-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-83-8", "SUFPUVGFEWPIFA-UHFFFAOYSA-N", "2,2,3,4,4-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-84-9", "XYDYIKVIGCXDLV-UHFFFAOYSA-N", "2,2,3,4,5-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-85-0", "XRJGBYMRUIWAJP-UHFFFAOYSA-N", "2,2,3,4,6-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-86-1", "OSZLORKBJVWZNT-UHFFFAOYSA-N", "2,2,3,5,5-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-87-2", "MHJUEZDXDYXCPA-UHFFFAOYSA-N", "2,2,3,5,6-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-88-3", "SLGYOGBODYIQGW-UHFFFAOYSA-N", "2,2,3,6,6-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-61-5", "XRGPEBQZXVFCRE-UHFFFAOYSA-N", "2,2,4,4,5-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-62-6", "NOFQKTWPZFUCOO-UHFFFAOYSA-N", "2,2,4,4,6-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-63-7", "JGOKXUSMFJOVQT-UHFFFAOYSA-N", "2,2,4,5,5-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-64-8", "JVESFBBBMJIZLO-UHFFFAOYSA-N", "2,2,4,5,6-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-65-9", "FMJMMNPKKCFHTH-UHFFFAOYSA-N", "2,2,5,5,6-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-66-0", "VBOBCKMZHQJDQA-UHFFFAOYSA-N", "2,3,3,4,4-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-67-1", "OLZJEASBDLLWCY-UHFFFAOYSA-N", "2,3,3,4,5-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-68-2", "YENAHMOHMFYKPC-UHFFFAOYSA-N", "2,3,3,4,6-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-69-3", "DTLBRJVRUQMSQN-UHFFFAOYSA-N", "2,3,3,5,5-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "52670-35-6", "SZQDOCSTKXRXAR-UHFFFAOYSA-N", "2,3,3,5,6-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-70-6", "CHHJLGDSEYTKKE-UHFFFAOYSA-N", "2,3,4,4,5-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-71-7", "DHXWMADSONSWTL-UHFFFAOYSA-N", "2,3,4,4,6-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-72-8", "BDFJISGXUJUYCX-UHFFFAOYSA-N", "2,3,4,5,5-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "27574-98-7", "YIKFFSNUOIJPSI-UHFFFAOYSA-N", "2,3,4,5,6-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-73-9", "TXGUYEKSMHZHSB-UHFFFAOYSA-N", "2,4,4,5,5-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-74-0", "YFXNIDIHYOGTEN-UHFFFAOYSA-N", "3,3,4,4,5-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-75-1", "NZIFKVSORPBQSX-UHFFFAOYSA-N", "3,3,4,5,5-pentamethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-04-6", "CVXGCQXACVZXQR-UHFFFAOYSA-N", "2,2,3-trimethyl-3-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-05-7", "COHRBFNOOUABCT-UHFFFAOYSA-N", "2,2,3-trimethyl-4-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-06-8", "OOIGDPTZPWRGHN-UHFFFAOYSA-N", "2,2,3-trimethyl-5-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-07-9", "TYGUUZVWCBYDKW-UHFFFAOYSA-N", "2,2,4-trimethyl-3-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-08-0", "XMUXASPVOBQLPC-UHFFFAOYSA-N", "2,2,4-trimethyl-4-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-09-1", "QADOHJKDWMLCQT-UHFFFAOYSA-N", "2,2,4-trimethyl-5-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-10-4", "SUDWTPMZYLBGKW-UHFFFAOYSA-N", "2,2,5-trimethyl-3-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-11-5", "GDPLKYYKURLRKJ-UHFFFAOYSA-N", "2,2,5-trimethyl-4-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-12-6", "KJWMMFYTYSUWMH-UHFFFAOYSA-N", "2,2,5-trimethyl-5-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-13-7", "RIIDJOQRWGTQEN-UHFFFAOYSA-N", "2,2,6-trimethyl-3-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-14-8", "QGYSVVWKEDKPTP-UHFFFAOYSA-N", "2,2,6-trimethyl-4-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-15-9", "LYZRWXZZVRWJHX-UHFFFAOYSA-N", "2,2,6-trimethyl-5-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-16-0", "VIIXOXRHJWHIBP-UHFFFAOYSA-N", "2,3,3-trimethyl-4-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-17-1", "JHGOLBUJAIJLCX-UHFFFAOYSA-N", "2,3,3-trimethyl-5-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-18-2", "YZXMDESZKHUODO-UHFFFAOYSA-N", "2,3,4-trimethyl-3-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-55-4", "QDWZIZQKVLESDK-UHFFFAOYSA-N", "2,3,4-trimethyl-4-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-56-5", "URFFXJPJMCMXFR-UHFFFAOYSA-N", "2,3,4-trimethyl-5-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-57-6", "QHACFORXSGHTJG-UHFFFAOYSA-N", "2,3,5-trimethyl-3-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-58-7", "UEZAASCWBAVHRO-UHFFFAOYSA-N", "2,3,5-trimethyl-4-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-59-8", "LTSLSBVPXXAMBM-UHFFFAOYSA-N", "2,3,5-trimethyl-5-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-60-1", "KOVQAKVRXODFNL-UHFFFAOYSA-N", "2,3,6-trimethyl-3-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-61-2", "RBVKVRIWSQSXPO-UHFFFAOYSA-N", "2,3,6-trimethyl-4-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-62-3", "JXCQTSFHNKTOLJ-UHFFFAOYSA-N", "2,3,6-trimethyl-5-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-63-4", "BNFUBMCCIACHFX-UHFFFAOYSA-N", "2,4,4-trimethyl-3-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-64-5", "YSCJXRHNKUXHLH-UHFFFAOYSA-N", "2,4,4-trimethyl-5-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-65-6", "JFKSTTDMBQAPCT-UHFFFAOYSA-N", "2,4,5-trimethyl-3-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-66-7", "HSVLOEWHJNAWLC-UHFFFAOYSA-N", "2,4,5-trimethyl-4-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-67-8", "JAUBFUIZENVVAG-UHFFFAOYSA-N", "2,4,5-trimethyl-5-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-68-9", "KQBJYFJWQOUHCI-UHFFFAOYSA-N", "2,4,6-trimethyl-3-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-69-0", "LPNPQCHEMXFBSW-UHFFFAOYSA-N", "2,4,6-trimethyl-4-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-70-3", "MARFNXCIPCRAJU-UHFFFAOYSA-N", "2,5,5-trimethyl-3-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-71-4", "HGYOEMXZYDBOKQ-UHFFFAOYSA-N", "2,5,5-trimethyl-4-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-72-5", "QLOXTBGCIUOWIG-UHFFFAOYSA-N", "3,3,4-trimethyl-4-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-73-6", "CUWOOQVXUDOHOC-UHFFFAOYSA-N", "3,3,4-trimethyl-5-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-74-7", "QXCHZKHHIUTBOU-UHFFFAOYSA-N", "3,3,5-trimethyl-4-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-75-8", "GCWLDOMARVHVOZ-UHFFFAOYSA-N", "3,3,5-trimethyl-5-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-76-9", "SIIFYRMARKBLER-UHFFFAOYSA-N", "3,4,4-trimethyl-3-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-77-0", "XSLWCLUAFIVTLD-UHFFFAOYSA-N", "3,4,4-trimethyl-5-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-78-1", "ZHILUIHUWKDTTM-UHFFFAOYSA-N", "3,4,5-trimethyl-3-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-79-2", "SZOKDASFURUJDF-UHFFFAOYSA-N", "3,4,5-trimethyl-4-ethylheptane", "")
  CALL Species("C_{12}H_{26}",                                 "", "DWEXNJRCPKKJBQ-UHFFFAOYSA-N", "2,2-dimethyl-4-propylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-30-2", "AZUDVOIXPNEOLR-UHFFFAOYSA-N", "2,3-dimethyl-4-propylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-31-3", "FDVJFPZZPSPQAO-UHFFFAOYSA-N", "2,4-dimethyl-4-propylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-32-4", "PJXVVWKRCASATD-UHFFFAOYSA-N", "2,5-dimethyl-4-propylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-33-5", "UFJZYHSURMZROL-UHFFFAOYSA-N", "2,6-dimethyl-4-propylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-34-6", "XVWBLUKOUJTCDO-UHFFFAOYSA-N", "3,3-dimethyl-4-propylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-35-7", "UGSAJWKYCFFZRD-UHFFFAOYSA-N", "3,4-dimethyl-4-propylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-36-8", "GKVRDSIZILZQJE-UHFFFAOYSA-N", "3,5-dimethyl-4-propylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-37-9", "XPOSVKCTMHOWTN-UHFFFAOYSA-N", "2,2-dimethyl-3-isopropylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-38-0", "HFRVMJCEWUXCMP-UHFFFAOYSA-N", "2,2-dimethyl-4-isopropylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-39-1", "IAWDNHPMOJTEEN-UHFFFAOYSA-N", "2,3-dimethyl-3-isopropylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-40-4", "GEQDAUMUCVOARW-UHFFFAOYSA-N", "2,3-dimethyl-4-isopropylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-41-5", "MWVRZCOXSIAHLW-UHFFFAOYSA-N", "2,4-dimethyl-3-isopropylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-42-6", "MTPOSRQPOBWFQC-UHFFFAOYSA-N", "2,4-dimethyl-4-isopropylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-43-7", "XSKMFUBSJXETFW-UHFFFAOYSA-N", "2,5-dimethyl-3-isopropylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-44-8", "OOYWQZAQFKOCKA-UHFFFAOYSA-N", "2,5-dimethyl-4-isopropylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-45-9", "NOAGKIODSFDOTA-UHFFFAOYSA-N", "2,6-dimethyl-3-isopropylheptane", "")
  CALL Species("C_{12}H_{26}",                       "35866-89-8", "BIBHNAJLMRUJLS-UHFFFAOYSA-N", "2,6-dimethyl-4-isopropylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-46-0", "MACIOLXBYDNTTI-UHFFFAOYSA-N", "3,3-dimethyl-4-isopropylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-47-1", "QFKXRROMVRXZIF-UHFFFAOYSA-N", "3,4-dimethyl-4-isopropylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-89-4", "XJRRTENBDSNHQB-UHFFFAOYSA-N", "3,5-dimethyl-4-isopropylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-90-7", "VBKOVPDJLQIMSZ-UHFFFAOYSA-N", "2-methyl-3,3-diethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-91-8", "PRSHREOBBOOUSO-UHFFFAOYSA-N", "2-methyl-3,4-diethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-92-9", "KZTGLPPJUZINLQ-UHFFFAOYSA-N", "2-methyl-3,5-diethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-93-0", "VJTVFYMTOXDIID-UHFFFAOYSA-N", "2-methyl-4,4-diethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-94-1", "SRLQJPCYGKMUSM-UHFFFAOYSA-N", "2-methyl-4,5-diethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-95-2", "XPAWKGLXSPHNAN-UHFFFAOYSA-N", "2-methyl-5,5-diethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-96-3", "VRVVWJIPKQWWJB-UHFFFAOYSA-N", "3-methyl-3,4-diethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-97-4", "RGRSPNWZJFXFAN-UHFFFAOYSA-N", "3-methyl-3,5-diethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-98-5", "WSBKWCXGZHVUHW-UHFFFAOYSA-N", "3-methyl-4,4-diethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62198-99-6", "NVBVLQDXCRDBBU-UHFFFAOYSA-N", "3-methyl-4,5-diethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-00-2", "JOJSCBROPYJHJX-UHFFFAOYSA-N", "3-methyl-5,5-diethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-01-3", "KGSWPEXUAXYDMO-UHFFFAOYSA-N", "4-methyl-3,3-diethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-02-4", "RAFABRWSQNCWEY-UHFFFAOYSA-N", "4-methyl-3,4-diethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62199-03-5", "XUVBJZPYCGZQDK-UHFFFAOYSA-N", "4-methyl-3,5-diethylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-23-3", "HIHQJZLWJQHTEJ-UHFFFAOYSA-N", "2-methyl-4-{\it tert}-butylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-24-4", "IKKVFRPBQQTTKP-UHFFFAOYSA-N", "3-methyl-4-{\it tert}-butylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-25-5", "JABLMQZBFVBNBS-UHFFFAOYSA-N", "4-methyl-4-{\it tert}-butylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-26-6", "FPODIYZXVCXXQE-UHFFFAOYSA-N", "3-ethyl-4-propylheptane", "")
  CALL Species("C_{12}H_{26}",                       "17312-43-5", "JTZJHTZCYBYCSZ-UHFFFAOYSA-N", "4-ethyl-4-propylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-27-7", "VENQCMDSFBOTMS-UHFFFAOYSA-N", "3-ethyl-4-isopropylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-28-8", "IKRHMKIYGHBTGZ-UHFFFAOYSA-N", "4-ethyl-4-isopropylheptane", "")
  CALL Species("C_{12}H_{26}",                       "62185-11-9", "ACZDZCRQHVWGEF-UHFFFAOYSA-N", "2,2,3,3,4,4-hexamethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62185-12-0", "IFHVEHXJQMYGHH-UHFFFAOYSA-N", "2,2,3,3,4,5-hexamethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "60302-24-1", "YFGLJPQISFJQSO-UHFFFAOYSA-N", "2,2,3,3,5,5-hexamethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62185-13-1", "QJVUKMHIEDVCDT-UHFFFAOYSA-N", "2,2,3,4,4,5-hexamethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62185-14-2", "NXZMBOYMQLHOPD-UHFFFAOYSA-N", "2,2,3,4,5,5-hexamethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "52670-36-7", "GRVNDHRZJGSRDC-UHFFFAOYSA-N", "2,3,3,4,4,5-hexamethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62184-98-9", "CRTTYRRLFPCOTC-UHFFFAOYSA-N", "2,2,3,3-tetramethyl-4-ethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62184-99-0", "SYWRGACXWPDDMM-UHFFFAOYSA-N", "2,2,3,4-tetramethyl-3-ethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62185-00-6", "WVLMCSYUMSZJQJ-UHFFFAOYSA-N", "2,2,3,4-tetramethyl-4-ethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62185-01-7", "CHURHVBNTYCJDL-UHFFFAOYSA-N", "2,2,3,5-tetramethyl-3-ethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62185-02-8", "GYAMSLKQDQYJGT-UHFFFAOYSA-N", "2,2,3,5-tetramethyl-4-ethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62185-03-9", "HWHADLCAKCOTRI-UHFFFAOYSA-N", "2,2,4,4-tetramethyl-3-ethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62185-04-0", "KZHIDSBQFJFWDV-UHFFFAOYSA-N", "2,2,4,5-tetramethyl-3-ethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62185-05-1", "HAVVVIZEMKFOFL-UHFFFAOYSA-N", "2,2,4,5-tetramethyl-4-ethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62185-06-2", "GJJUHGLRQQDIFN-UHFFFAOYSA-N", "2,2,5,5-tetramethyl-3-ethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62185-07-3", "YVKQGHGRZXUTNK-UHFFFAOYSA-N", "2,3,3,4-tetramethyl-4-ethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62185-08-4", "AMMSTTMNZHWREI-UHFFFAOYSA-N", "2,3,3,5-tetramethyl-4-ethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62185-09-5", "VYCKEOSFJUAFGK-UHFFFAOYSA-N", "2,3,4,4-tetramethyl-3-ethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62185-10-8", "IBPLCPRMAQRHHK-UHFFFAOYSA-N", "2,3,4,5-tetramethyl-3-ethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62199-79-5", "UTCSLOLKGOKPGW-UHFFFAOYSA-N", "2,2,3-trimethyl-3-isopropylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62199-80-8", "VEMACPZMOMAUAN-UHFFFAOYSA-N", "2,2,4-trimethyl-3-isopropylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62199-81-9", "GZFAXOCUKPGWEA-UHFFFAOYSA-N", "2,2,5-trimethyl-3-isopropylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62199-82-0", "QFLYTOVYLBZBJC-UHFFFAOYSA-N", "2,2,5-trimethyl-4-isopropylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62199-83-1", "SXELEUFFBZGXIC-UHFFFAOYSA-N", "2,3,4-trimethyl-3-isopropylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62199-84-2", "GUMUKRVQSSIBJO-UHFFFAOYSA-N", "2,3,5-trimethyl-3-isopropylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62199-85-3", "VBRDSJSWPGXVIR-UHFFFAOYSA-N", "2,3,5-trimethyl-4-isopropylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62199-86-4", "UGPHXQWIHGYHOH-UHFFFAOYSA-N", "2,4,4-trimethyl-3-isopropylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62199-89-7", "ZAKPEOVUOZWBNO-UHFFFAOYSA-N", "2,2-dimethyl-3,4-diethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62184-89-8", "CYPRMBGQWCLJBN-UHFFFAOYSA-N", "2,2-dimethyl-4,4-diethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62184-90-1", "BUUBMKJJLVCTKU-UHFFFAOYSA-N", "2,3-dimethyl-3,4-diethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62184-91-2", "YNHBDJNMYMZVLM-UHFFFAOYSA-N", "2,3-dimethyl-4,4-diethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62184-92-3", "OXAROVXPKXMTIV-UHFFFAOYSA-N", "2,4-dimethyl-3,3-diethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62184-93-4", "AUVJTUXGNKLTIN-UHFFFAOYSA-N", "2,4-dimethyl-3,4-diethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62184-94-5", "XHDPVTDRZGCNFV-UHFFFAOYSA-N", "2,5-dimethyl-3,3-diethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62184-95-6", "WCPAKUFSQFCWTQ-UHFFFAOYSA-N", "2,5-dimethyl-3,4-diethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62184-96-7", "XFUAJHJJXMVZCQ-UHFFFAOYSA-N", "3,3-dimethyl-4,4-diethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62184-97-8", "NTQKWVJMJSYRDX-UHFFFAOYSA-N", "3,4-dimethyl-3,4-diethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62199-76-2", "VZAYJUSYQVNGBC-UHFFFAOYSA-N", "2,2-dimethyl-3-{\it tert}-butylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62199-77-3", "TXYYDQJNRQJTOW-UHFFFAOYSA-N", "2-methyl-3-ethyl-3-isopropylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62199-78-4", "JXHSGKIMXDNKIS-UHFFFAOYSA-N", "2-methyl-4-ethyl-3-isopropylhexane", "")
  CALL Species("C_{12}H_{26}",                       "62199-87-5", "YGNQREHYYJPYLX-UHFFFAOYSA-N", "3,3,4-triethylhexane", "")
  CALL Species("C_{12}H_{26}",                       "66576-21-4", "IZNAUALMSCRPBS-UHFFFAOYSA-N", "2,2,3,4,4-pentamethyl-3-ethylpentane", "")
  CALL Species("C_{12}H_{26}",                       "62185-17-5", "OKGFCJOAXUYCBG-UHFFFAOYSA-N", "2,2,3,4-tetramethyl-3-isopropylpentane", "")
  CALL Species("C_{12}H_{26}",                       "62185-18-6", "BYIFHEDQKBAFSW-UHFFFAOYSA-N", "2,2,4,4-tetramethyl-3-isopropylpentane", "")
  CALL Species("C_{12}H_{26}",                       "62185-15-3", "GIKXQMPMOJVRDN-UHFFFAOYSA-N", "2,2,4-trimethyl-3,3-diethylpentane", "")
  CALL Species("C_{12}H_{26}",                       "62185-16-4", "ASDDXPPGGPFNIL-UHFFFAOYSA-N", "2,4-dimethyl-3-ethyl-3-isopropylpentane", "")
  ! C13:
  CALL Species("C_{13}H_{28}",                         "629-50-5", "IIYFAKIEWZDVMP-UHFFFAOYSA-N", "tridecane", "")
  CALL Species("C_{13}H_{28}",                        "1560-97-0", "HGEMCUOAMCILCP-UHFFFAOYSA-N", "2-methyldodecane", "")
  CALL Species("C_{13}H_{28}",                       "17312-57-1", "GRJUENNHVNYCHD-UHFFFAOYSA-N", "3-methyldodecane", "")
  CALL Species("C_{13}H_{28}",                       "17312-64-0", "QDKSGHXRHXVMPF-UHFFFAOYSA-N", "2,2-dimethylundecane", "")
  CALL Species("C_{13}H_{28}",                       "17312-77-5", "QSSUTSOGIQHRIU-UHFFFAOYSA-N", "2,3-dimethylundecane", "")
  CALL Species("C_{13}H_{28}",                       "17312-80-0", "WMZNFELFMFOGCC-UHFFFAOYSA-N", "2,4-dimethylundecane", "")
  ! C14:
  CALL Species("C_{14}H_{30}",                         "629-59-4", "BGHCVCJVXZWKCC-UHFFFAOYSA-N", "tetradecane", "")
  CALL Species("C_{14}H_{30}",                        "1560-96-9", "CJBFZKZYIPBBTO-UHFFFAOYSA-N", "2-methyltridecane", "")
  CALL Species("C_{14}H_{30}",                        "6418-41-3", "NLHRRMKILFRDGV-UHFFFAOYSA-N", "3-methyltridecane", "")
  CALL Species("C_{14}H_{30}",                       "49598-54-1", "ATWISEHEXAEGKB-UHFFFAOYSA-N", "2,2-dimethyldodecane", "")
  CALL Species("C_{14}H_{30}",                        "6117-98-2", "QBIXLGJCVGNDBJ-UHFFFAOYSA-N", "2,3-dimethyldodecane", "")
  CALL Species("C_{14}H_{30}",                        "6117-99-3", "AFELDWXNIFIYOC-UHFFFAOYSA-N", "2,4-dimethyldodecane", "")
  ! C15:
  CALL Species("C_{15}H_{32}",                         "629-62-9", "YCOZIPAWZNQLMR-UHFFFAOYSA-N", "pentadecane", "")
  CALL Species("C_{15}H_{32}",                        "1560-95-8", "KUVMKLCGXIYSNH-UHFFFAOYSA-N", "2-methyltetradecane", "")
  CALL Species("C_{15}H_{32}",                       "18435-22-8", "HXUYUZCPGPKNGS-UHFFFAOYSA-N", "3-methyltetradecane", "")
  CALL Species("C_{15}H_{32}",                       "61869-04-3", "NVEUWWMNWPNXOC-UHFFFAOYSA-N", "2,2-dimethyltridecane", "")
  CALL Species("C_{15}H_{32}",                       "18435-20-6", "SWUJSLXRUPXTQB-UHFFFAOYSA-N", "2,3-dimethyltridecane", "")
  CALL Species("C_{15}H_{32}",                       "61868-05-1", "JDFJCABQSZLDMZ-UHFFFAOYSA-N", "2,4-dimethyltridecane", "")
  ! C16:
  CALL Species("C_{16}H_{34}",                         "544-76-3", "DCAYPVUWAIABOU-UHFFFAOYSA-N", "hexadecane", "")
  CALL Species("C_{16}H_{34}",                        "1560-93-6", "BANXPJUEBPWEOT-UHFFFAOYSA-N", "2-methylpentadecane", "")
  CALL Species("C_{16}H_{34}",                        "2882-96-4", "FWXKCXJPHSAYMK-UHFFFAOYSA-N", "3-methylpentadecane", "")
  CALL Species("C_{16}H_{34}",                       "59222-86-5", "VCAMXEBNASZVEZ-UHFFFAOYSA-N", "2,2-dimethyltetradecane", "")
  CALL Species("C_{16}H_{34}",                       "18435-23-9", "ZQRVEWIJXCRTFW-UHFFFAOYSA-N", "2,3-dimethyltetradecane", "")
  CALL Species("C_{16}H_{34}",                       "61868-06-2", "NWTLFDYNCLTSAR-UHFFFAOYSA-N", "2,4-dimethyltetradecane", "")
  ! >= C17:
  CALL Species("C_{17}H_{36}",                         "629-78-7", "NDJKXXJCMXVBJW-UHFFFAOYSA-N", "heptadecane", "")
  CALL Species("C_{18}H_{38}",                         "593-45-3", "RZJRJXONCZWCBN-UHFFFAOYSA-N", "octadecane", "")
  CALL Species("C_{19}H_{40}",                         "629-92-5", "LQERIDTXQFOHKA-UHFFFAOYSA-N", "nonadecane", "")
  CALL Species("C_{20}H_{42}",                         "112-95-8", "CBFCDTFDPHXCNY-UHFFFAOYSA-N", "eicosane", "")
  CALL Species("C_{21}H_{44}",                         "629-94-7", "FNAZRRHPUDJQCJ-UHFFFAOYSA-N", "heneicosane", "")
  CALL Species("C_{22}H_{46}",                         "629-97-0", "HOWGUJZVBDQJKV-UHFFFAOYSA-N", "docosane", "")
  CALL Species("C_{23}H_{48}",                         "638-67-5", "FIGVVZUWCLSUEI-UHFFFAOYSA-N", "tricosane", "")
  CALL Species("C_{24}H_{50}",                         "646-31-1", "POOSGDOYLQNASK-UHFFFAOYSA-N", "tetracosane", "")
  CALL Species("C_{25}H_{52}",                         "629-99-2", "YKNWIILGEFFOPE-UHFFFAOYSA-N", "pentacosane", "")
  CALL Species("C_{26}H_{54}",                         "630-01-3", "HMSWAIKSFDFLKN-UHFFFAOYSA-N", "hexacosane", "")
  CALL Species("C_{27}H_{56}",                         "593-49-7", "BJQWYEJQWHSSCJ-UHFFFAOYSA-N", "heptacosane", "")
  CALL Species("C_{28}H_{58}",                         "630-02-4", "ZYURHZPYMFLWSH-UHFFFAOYSA-N", "octacosane", "")
  CALL Species("C_{29}H_{60}",                         "630-03-5", "IGGUPRCHHJZPBS-UHFFFAOYSA-N", "nonacosane", "")
  CALL Species("C_{30}H_{62}",                         "638-68-6", "JXTPJDDICSTXJX-UHFFFAOYSA-N", "triacontane", "")
  CALL Species("C_{32}H_{66}",                         "544-85-4", "QHMGJGNTMQDRQA-UHFFFAOYSA-N", "dotriacontane", "")
  CALL Species("C_{35}H_{72}",                         "630-07-9", "VHQQPFLOGSTQPC-UHFFFAOYSA-N", "pentatriacontane", "")
  CALL Species("C_{36}H_{74}",                         "630-06-8", "YDLYQMBWCWFRAI-UHFFFAOYSA-N", "hexatriacontane", "")
  CALL Species("C_{38}H_{78}",                        "7194-85-6", "BVKCQBBZBGYNOP-UHFFFAOYSA-N", "octatriacontane", "")

  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Cycloalkanes")
  CALL Species("C_3H_6",                                "75-19-4", "LVZWSLJZHVFIQJ-UHFFFAOYSA-N", "cyclopropane", "")
  CALL Species("C_4H_8",                               "287-23-0", "PMPVIKIVABFJJI-UHFFFAOYSA-N", "cyclobutane", "")
  CALL Species("C_5H_{10}",                            "287-92-3", "RGSFGYAAUTVSQA-UHFFFAOYSA-N", "cyclopentane", "")
  CALL Species("C_6H_{12}",                            "110-82-7", "XDTMQSROBMDMFD-UHFFFAOYSA-N", "cyclohexane", "")
  CALL Species("C_5H_9CH_3",                            "96-37-7", "GDOPTJXRTPNYNR-UHFFFAOYSA-N", "methylcyclopentane", "")
  CALL Species("C_7H_{14}",                            "291-64-5", "DMEGYFMYUHOHGS-UHFFFAOYSA-N", "cycloheptane", "")
  CALL Species("C_6H_{11}CH_3",                        "108-87-2", "UAEPNZWRGJTJPN-UHFFFAOYSA-N", "methylcyclohexane", "")
  CALL Species("C_6D_{11}CD_3",                      "10120-28-2", "UAEPNZWRGJTJPN-OBYKGMMLSA-N", "methylcyclohexane-d14", "")
  CALL Species("C_7H_{14}",                           "1640-89-7", "IFTRQJLVEBNKJK-UHFFFAOYSA-N", "ethylcyclopentane", "")
  CALL Species("C_7H_{14}",                           "1638-26-2", "QWHNJUXXYKPLQM-UHFFFAOYSA-N", "1,1-dimethylcyclopentane", "")
  CALL Species("C_7H_{14}",                           "1192-18-3", "RIRARCHMRDHZAR-KNVOCYPGSA-N", "{\it cis}-1,2-dimethylcyclopentane", "")
  CALL Species("C_7H_{14}",                           "2532-58-3", "XAZKFISIRYLAEE-KNVOCYPGSA-N", "{\it cis}-1,3-dimethylcyclopentane", "")
  CALL Species("C_7H_{14}",                            "822-50-4", "RIRARCHMRDHZAR-RNFRBKRXSA-N", "{\it trans}-1,2-dimethylcyclopentane", "")
  CALL Species("C_7H_{14}",                           "1759-58-6", "XAZKFISIRYLAEE-RNFRBKRXSA-N", "{\it trans}-1,3-dimethylcyclopentane", "")
  CALL Species("C_8H_{16}",                            "292-64-8", "WJTCGQSWYFHTAC-UHFFFAOYSA-N", "cyclooctane", "")
  CALL Species("C_8H_{16}",                           "4126-78-7", "GYNNXHKOJHMOHS-UHFFFAOYSA-N", "methylcycloheptane", "")
  CALL Species("C_8H_{16}",                            "590-66-9", "QEGNUYASOUJEHD-UHFFFAOYSA-N", "1,1-dimethylcyclohexane", "")
  CALL Species("C_6H_{10}(CH_3)_2",                    "583-57-3", "KVZJLSYJROEPSQ-UHFFFAOYSA-N", "1,2-dimethylcyclohexane", "")
  CALL Species("C_6H_{10}(CH_3)_2",                   "2207-01-4", "KVZJLSYJROEPSQ-OCAPTIKFSA-N", "{\it cis}-1,2-dimethylcyclohexane", "")
  CALL Species("C_6H_{10}(CH_3)_2",                   "6876-23-9", "KVZJLSYJROEPSQ-HTQZYQBOSA-N", "{\it trans}-1,2-dimethylcyclohexane", "") ! "KVZJLSYJROEPSQ-YUMQZZPRSA-N" for 1S2S enantiomer
  CALL Species("C_8H_{16}",                            "638-04-0", "SGVUHPSBDNVHKL-OCAPTIKFSA-N", "{\it cis}-1,3-dimethylcyclohexane", "")
  CALL Species("C_8H_{16}",                           "2207-03-6", "SGVUHPSBDNVHKL-HTQZYQBOSA-N", "{\it trans}-1,3-dimethylcyclohexane", "")
  CALL Species("C_6H_{10}(CH_3)_2",                    "589-90-2", "QRMPKOFEUHIBNM-UHFFFAOYSA-N", "1,4-dimethylcyclohexane", "")
  CALL Species("C_8H_{16}",                            "624-29-3", "QRMPKOFEUHIBNM-OCAPTIKFSA-N", "{\it cis}-1,4-dimethylcyclohexane", "")
  CALL Species("C_6H_{10}(CH_3)_2",                   "2207-04-7", "QRMPKOFEUHIBNM-ZKCHVHJHSA-N", "{\it trans}-1,4-dimethylcyclohexane", "")
  CALL Species("C_8H_{16}",                           "1678-91-7", "IIEWJVIFRVWJOD-UHFFFAOYSA-N", "ethylcyclohexane", "")
  CALL Species("C_5H_7(CH_3)_3",                      "4259-00-1", "WINCSBAYCULVDU-UHFFFAOYSA-N", "1,1,2-trimethylcyclopentane", "")
  CALL Species("C_5H_7(CH_3)_3",                      "4516-69-2", "OBKHYUIZSOIEPG-UHFFFAOYSA-N", "1,1,3-trimethylcyclopentane", "")
  CALL Species("C_8H_{16}",                           "2613-69-6", "VCWNHOPGKQCXIQ-RNLVFQAGSA-N", "1,{\it cis}-2,{\it cis}-3-trimethylcyclopentane", "")
  CALL Species("C_8H_{16}",                          "15890-40-1", "VCWNHOPGKQCXIQ-JIGDXULJSA-N", "1,{\it cis}-2,{\it trans}-3-trimethylcyclopentane", "")
  CALL Species("C_8H_{16}",                          "19374-46-0", "VCWNHOPGKQCXIQ-RNFRBKRXSA-N", "1,{\it trans}-2,{\it cis}-3-trimethylcyclopentane", "")
  CALL Species("C_8H_{16}",                           "2613-72-1", "PNUFYSGVPVMNRN-RNLVFQAGSA-N", "1,{\it cis}-2,{\it cis}-4-trimethylcyclopentane", "")
  CALL Species("C_8H_{16}",                           "4850-28-6", "PNUFYSGVPVMNRN-WHUPJOBBSA-N", "1,{\it cis}-2,{\it trans}-4-trimethylcyclopentane", "")
  CALL Species("C_8H_{16}",                          "16883-48-0", "PNUFYSGVPVMNRN-HTQZYQBOSA-N", "1,{\it trans}-2,{\it cis}-4-trimethylcyclopentane", "")
  CALL Species("C_8H_{16}",                          "16747-50-5", "LETYIFNDQBJGPJ-UHFFFAOYSA-N", "1-methyl-1-ethylcyclopentane", "")
  CALL Species("C_8H_{16}",                            "930-89-2", "BSKOLJVTLRLTHE-SFYZADRCSA-N", "1-methyl-{\it cis}-2-ethylcyclopentane", "")
  CALL Species("C_8H_{16}",                            "930-90-5", "BSKOLJVTLRLTHE-HTQZYQBOSA-N", "1-methyl-{\it trans}-2-ethylcyclopentane", "")
  CALL Species("C_8H_{16}",                           "2613-66-3", "PQXAPVOKLYINEI-SFYZADRCSA-N", "1-methyl-{\it cis}-3-ethylcyclopentane", "")
  CALL Species("C_8H_{16}",                           "2613-65-2", "PQXAPVOKLYINEI-HTQZYQBOSA-N", "1-methyl-{\it trans}-3-ethylcyclopentane", "")
  CALL Species("C_5H_9C_3H_7",                        "2040-96-2", "KDIAMAVWIJYWHN-UHFFFAOYSA-N", "propylcyclopentane", "")
  CALL Species("C_8H_{16}",                           "3875-51-2", "TVSBRLGQVHJIKT-UHFFFAOYSA-N", "isopropylcyclopentane", "")
  CALL Species("C_8H_{14}",                                    "", "WZXMVFMYGAOEAT-UHFFFAOYSA-N", "5,5-dimethylbicyclo[2.1.1]hexane", "MCM:C8BC")
  CALL Species("C_9H_{18}",                           "7094-26-0", "MEBONNVPKOBPEA-UHFFFAOYSA-N", "1,1,2-trimethylcyclohexane", "")
  CALL Species("C_9H_{18}",                           "3073-66-3", "PYOLJOJPIPCRDP-UHFFFAOYSA-N", "1,1,3-trimethylcyclohexane", "")
  CALL Species("C_9H_{18}",                           "7094-27-1", "UIWORXHEVNIOJG-UHFFFAOYSA-N", "1,1,4-trimethylcyclohexane", "")
  CALL Species("C_9H_{18}",                           "1839-88-9", "DQTVJLHNWPRPPH-AYMMMOKOSA-N", "1,{\it cis}-2,{\it cis}-3-trimethylcyclohexane", "")
  CALL Species("C_9H_{18}",                           "7667-55-2", "DQTVJLHNWPRPPH-BRPSZJMVSA-N", "1,{\it cis}-2,{\it trans}-3-trimethylcyclohexane", "")
  CALL Species("C_9H_{18}",                           "1678-80-4", "VCJPCEVERINRSG-YIZRAAEISA-N", "1,{\it cis}-2,{\it cis}-4-trimethylcyclohexane", "")
  CALL Species("C_9H_{18}",                           "7667-58-5", "VCJPCEVERINRSG-HLTSFMKQSA-N", "1,{\it cis}-2,{\it trans}-4-trimethylcyclohexane", "")
  CALL Species("C_9H_{18}",                           "1795-27-3", "ODNRTOSCFYDTKF-AYMMMOKOSA-N", "{\it cis},{\it cis}-1,3,5-trimethylcyclohexane", "")
  CALL Species("C_9H_{18}",                           "1795-26-2", "ODNRTOSCFYDTKF-FBJIGQNJSA-N", "{\it cis},{\it trans}-1,3,5-trimethylcyclohexane", "")
  CALL Species("C_9H_{18}",                           "1678-81-5", "DQTVJLHNWPRPPH-HTQZYQBOSA-N", "1,{\it trans}-2,{\it cis}-3-trimethylcyclohexane", "")
  CALL Species("C_9H_{18}",                           "7667-59-6", "VCJPCEVERINRSG-DJLDLDEBSA-N", "1,{\it trans}-2,{\it cis}-4-trimethylcyclohexane", "")
  CALL Species("C_9H_{18}",                           "7667-60-9", "VCJPCEVERINRSG-IWSPIJDZSA-N", "1,{\it trans}-2,{\it trans}-4-trimethylcyclohexane", "")
  CALL Species("C_9H_{18}",                           "4926-90-3", "YPJRYQGOKHKNKZ-UHFFFAOYSA-N", "1-methyl-1-ethylcyclohexane", "")
  CALL Species("C_9H_{18}",                           "4923-77-7", "XARGIVYWQPXRTC-BDAKNGLRSA-N", "1-methyl-{\it cis}-2-ethylcyclohexane", "")
  CALL Species("C_9H_{18}",                          "19489-10-2", "UDDVMPHNQKRNNS-BDAKNGLRSA-N", "1-methyl-{\it cis}-3-ethylcyclohexane", "")
  CALL Species("C_9H_{18}",                           "4926-78-7", "CYISMTMRBPPERU-DTORHVGOSA-N", "1-methyl-{\it cis}-4-ethylcyclohexane", "")
  CALL Species("C_9H_{18}",                           "4923-78-8", "XARGIVYWQPXRTC-RKDXNWHRSA-N", "1-methyl-{\it trans}-2-ethylcyclohexane", "")
  CALL Species("C_9H_{18}",                           "4926-76-5", "UDDVMPHNQKRNNS-RKDXNWHRSA-N", "1-methyl-{\it trans}-3-ethylcyclohexane", "")
  CALL Species("C_9H_{18}",                           "6236-88-0", "CYISMTMRBPPERU-KYZUINATSA-N", "1-methyl-{\it trans}-4-ethylcyclohexane", "")
  CALL Species("C_9H_{18}",                           "1678-92-8", "DEDZSLCZHWTGOR-UHFFFAOYSA-N", "propylcyclohexane", "")
  CALL Species("C_9H_{18}",                            "696-29-7", "GWESVXSMPKAFAS-UHFFFAOYSA-N", "isopropylcyclohexane", "")
  CALL Species("C_9H_{18}",                          "52688-89-8", "YXDMSFJDVHXFCV-UHFFFAOYSA-N", "1,1,2,2-tetramethylcyclopentane", "")
  CALL Species("C_9H_{18}",                                    "", "CXCBKSYSKZEEJB-YUMQZZPRSA-N", "{\it cis}-1,1,2,3-tetramethylcyclopentane", "") ! also CXCBKSYSKZEEJB-JGVFFNPUSA-N ?
  CALL Species("C_9H_{18}",                          "62016-70-0", "CXCBKSYSKZEEJB-SFYZADRCSA-N", "{\it trans}-1,1,2,3-tetramethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "62016-71-1", "AVBGIJNNMIBMQG-SFYZADRCSA-N", "{\it cis}-1,1,2,4-tetramethylcyclopentane", "")
  CALL Species("C_9H_{18}",                                    "", "AVBGIJNNMIBMQG-YUMQZZPRSA-N", "{\it trans}-1,1,2,4-tetramethylcyclopentane", "") ! also AVBGIJNNMIBMQG-HTQZYQBOSA-N ?
  CALL Species("C_9H_{18}",                          "50876-33-0", "YWYCGTZNHWYQBD-UHFFFAOYSA-N", "1,1,3,3-tetramethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "53907-60-1", "OWHFMVURUNNXMJ-OCAPTIKFSA-N", "{\it cis}-1,1,3,4-tetramethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "20309-77-7", "OWHFMVURUNNXMJ-HTQZYQBOSA-N", "{\it trans}-1,1,3,4-tetramethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "18938-68-6", "DHLYDHNCPUAVHP-OCAPTIKFSA-N", "{\it cis}-1,2,2,3-tetramethylcyclopentane", "")
  CALL Species("C_9H_{18}",                                    "", "DHLYDHNCPUAVHP-YUMQZZPRSA-N", "{\it trans}-1,2,2,3-tetramethylcyclopentane", "")
  CALL Species("C_9H_{18}",                           "2532-65-2", "INYXDKODFMWKER-FNCVBFRFSA-N", "1,{\it cis}-2,{\it cis}-3,{\it cis}-4-tetramethylcyclopentane", "")
  CALL Species("C_9H_{18}",                           "2532-69-6", "INYXDKODFMWKER-BGZDPUMWSA-N", "1,{\it cis}-2,{\it cis}-3,{\it trans}-4-tetramethylcyclopentane", "")
  CALL Species("C_9H_{18}",                           "2532-68-5", "INYXDKODFMWKER-BZNPZCIMSA-N", "1,{\it cis}-2,{\it trans}-3,{\it cis}-4-tetramethylcyclopentane", "")
  CALL Species("C_9H_{18}",                           "2532-67-4", "INYXDKODFMWKER-HXFLIBJXSA-N", "1,{\it trans}-2,{\it cis}-3,{\it trans}-4-tetramethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "19907-40-5", "INYXDKODFMWKER-OJOKCITNSA-N", "1,{\it trans}-2,{\it trans}-3,{\it cis}-4-tetramethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "54549-80-3", "RXPIHZJWAFCHEJ-UHFFFAOYSA-N", "1,1-dimethyl-2-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "62016-61-9", "WXHYOGXBESCULU-UHFFFAOYSA-N", "1,1-dimethyl-3-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "62016-63-1", "DRWDWIMYUGHJBR-DTWKUNHWSA-N", "1,{\it cis}-2-dimethyl-1-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "62016-62-0", "DRWDWIMYUGHJBR-RKDXNWHRSA-N", "1,{\it trans}-2-dimethyl-1-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                                    "", "UMUGNPFWQJAOJI-HLTSFMKQSA-N", "1,{\it cis}-2-dimethyl-{\it cis}-3-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                                    "", "UMUGNPFWQJAOJI-HRDYMLBCSA-N", "1,{\it trans}-2-dimethyl-{\it trans}-3-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                                    "", "UMUGNPFWQJAOJI-IWSPIJDZSA-N", "1,{\it cis}-2-dimethyl-{\it trans}-3-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                                    "", "UMUGNPFWQJAOJI-VGMNWLOBSA-N", "1,{\it trans}-2-dimethyl-{\it cis}-3-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "62016-64-2", "QKXQNQVXTYSGKS-BRPSZJMVSA-N", "1,{\it cis}-2-dimethyl-{\it cis}-4-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "62016-65-3", "QKXQNQVXTYSGKS-AYMMMOKOSA-N", "1,{\it cis}-2-dimethyl-{\it trans}-4-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "62016-66-4", "QKXQNQVXTYSGKS-HTQZYQBOSA-N", "1,{\it trans}-2-dimethyl-{\it cis}-4-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "62016-68-6", "OAWOMHRJVJMDLZ-DTWKUNHWSA-N", "1,{\it cis}-3-dimethyl-1-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "62016-67-5", "OAWOMHRJVJMDLZ-RKDXNWHRSA-N", "1,{\it trans}-3-dimethyl-1-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "19903-00-5", "JREISGVVJTVFBL-AYMMMOKOSA-N", "1,{\it cis}-3-dimethyl-{\it cis}-2-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "19902-98-8", "JREISGVVJTVFBL-BRPSZJMVSA-N", "1,{\it cis}-3-dimethyl-{\it trans}-2-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "19902-99-9", "JREISGVVJTVFBL-HTQZYQBOSA-N", "1,{\it trans}-3-dimethyl-{\it cis}-2-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                                    "", "VMCXXGFUCWAIIN-YIZRAAEISA-N", "1,{\it cis}-3-dimethyl-{\it cis}-4-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                                    "", "VMCXXGFUCWAIIN-DJLDLDEBSA-N", "1,{\it cis}-3-dimethyl-{\it trans}-4-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                                    "", "VMCXXGFUCWAIIN-CIUDSAMLSA-N", "1,{\it trans}-3-dimethyl-{\it cis}-4-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                                    "", "VMCXXGFUCWAIIN-XHNCKOQMSA-N", "1,{\it trans}-3-dimethyl-{\it trans}-4-ethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "16631-63-3", "HICYLMKNNFKEMK-UHFFFAOYSA-N", "1-methyl-1-propylcyclopentane", "")
  CALL Species("C_9H_{18}",                            "932-43-4", "ADQJFBQXLAAVQA-BDAKNGLRSA-N", "1-methyl-{\it cis}-2-propylcyclopentane", "")
  CALL Species("C_9H_{18}",                            "932-44-5", "ADQJFBQXLAAVQA-RKDXNWHRSA-N", "1-methyl-{\it trans}-2-propylcyclopentane", "")
  CALL Species("C_9H_{18}",                           "2443-04-1", "HRSBIYASWAILIF-BDAKNGLRSA-N", "1-methyl-{\it cis}-3-propylcyclopentane", "")
  CALL Species("C_9H_{18}",                           "2443-03-0", "HRSBIYASWAILIF-RKDXNWHRSA-N", "1-methyl-{\it trans}-3-propylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "61828-00-0", "XFMQGDWBNOQLEG-UHFFFAOYSA-N", "1-methyl-1-isopropylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "61868-01-7", "CGWXYEIWDQDFIU-RKDXNWHRSA-N", "1-methyl-{\it cis}-2-isopropylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "61828-01-1", "CGWXYEIWDQDFIU-BDAKNGLRSA-N", "1-methyl-{\it trans}-2-isopropylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "61828-02-2", "CDTDMKCVKCGRPD-BDAKNGLRSA-N", "1-methyl-{\it cis}-3-isopropylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "61828-03-3", "CDTDMKCVKCGRPD-RKDXNWHRSA-N", "1-methyl-{\it trans}-3-isopropylcyclopentane", "")
  CALL Species("C_9H_{18}",                           "2721-38-2", "DPGQSDLGKGLNHC-UHFFFAOYSA-N", "1,1-diethylcyclopentane", "")
  CALL Species("C_9H_{18}",                            "932-39-8", "JKMYLSLBFNMSFP-DTORHVGOSA-N", "1,{\it cis}-2-diethylcyclopentane", "")
  CALL Species("C_9H_{18}",                            "932-40-1", "JKMYLSLBFNMSFP-RKDXNWHRSA-N", "1,{\it trans}-2-diethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "62016-59-5", "RUUVWUNHERVOAY-DTORHVGOSA-N", "1,{\it cis}-3-diethylcyclopentane", "")
  CALL Species("C_9H_{18}",                          "62016-60-8", "RUUVWUNHERVOAY-RKDXNWHRSA-N", "1,{\it trans}-3-diethylcyclopentane", "")
  CALL Species("C_9H_{18}",                           "2040-95-1", "ZAGHKONXGGSVDV-UHFFFAOYSA-N", "butylcyclopentane", "")
  CALL Species("C_9H_{18}",                           "3788-32-7", "DPUYDFJBHDYVQM-UHFFFAOYSA-N", "isobutylcyclopentane", "")
  CALL Species("C_9H_{18}",                           "4850-32-2", "DCEHVBLXWODXCW-UHFFFAOYSA-N", "{\it sec}-butylcyclopentane", "")
  CALL Species("C_9H_{18}",                           "3875-52-3", "BFWVYBVSRYIDHI-UHFFFAOYSA-N", "{\it tert}-butylcyclopentane", "")
  CALL Species("C_9H_{16}",                            "496-10-6", "BNRNAKTVFSZAFA-UHFFFAOYSA-N", "octahydro-1H-indene", "")
  CALL Species("C_{10}H_{20}",                        "1678-93-9", "GGBJHURWWWLEQH-UHFFFAOYSA-N", "butylcyclohexane", "")
  CALL Species("C_{10}H_{20}",                        "3741-00-2", "HPQURZRDYMUHJI-UHFFFAOYSA-N", "pentylcyclopentane", "")
  CALL Species("C_{10}H_{18}",                          "91-17-8", "NNBZCPXTIHJBJL-UHFFFAOYSA-N", "decahydronaphthalene", "decalin")
  CALL Species("C_{10}H_{18}",                         "493-01-6", "NNBZCPXTIHJBJL-AOOOYVTPSA-N", "($Z$)-bicyclo[4.4.0]decane", "{\it cis}-decahydronaphthalene; {\it cis}-decalin")
  CALL Species("C_{10}H_{18}",                         "493-02-7", "NNBZCPXTIHJBJL-MGCOHNPYSA-N", "($E$)-bicyclo[4.4.0]decane", "{\it trans}-decahydronaphthalene; {\it trans}-decalin")
  CALL Species("C_{10}H_{18}",                         "473-55-2", "XOKSLPVRUOBDEW-UHFFFAOYSA-N", "2,6,6-trimethylbicyclo[3.1.1]heptane", "dihydropinene")
  CALL Species("C_{10}H_{16}",                         "508-32-7", "RRBYUSWBLVXTQN-UHFFFAOYSA-N", "tricyclene", "")
  CALL Species("C_{11}H_{22}",                        "4457-00-5", "LKHGKBBAJAFMSQ-UHFFFAOYSA-N", "hexylcyclopentane", "")
  CALL Species("C_{11}H_{22}",                        "4292-92-6", "HLTMUYBTNSVOFY-UHFFFAOYSA-N", "pentylcyclohexane", "")
  CALL Species("C_{12}H_{24}",                         "294-62-2", "DDTBPAQBQHZRDW-UHFFFAOYSA-N", "cyclododecane", "")
  CALL Species("C_{12}H_{24}",                        "5617-42-5", "BOFNAOHMSHEKQL-UHFFFAOYSA-N", "heptylcyclopentane", "")
  CALL Species("C_{12}H_{24}",                        "4292-75-5", "QHWAQXOSHHKCFK-UHFFFAOYSA-N", "hexylcyclohexane", "")
  CALL Species("C_{12}H_{22}",                          "92-51-3", "WVIIMZNLDWSIRH-UHFFFAOYSA-N", "1,1'-bicyclohexyl", "")
  CALL Species("C_{13}H_{26}",                        "5617-41-4", "MSTLSCNJAHAQNU-UHFFFAOYSA-N", "heptylcyclohexane", "")
  CALL Species("C_{13}H_{26}",                        "1795-20-6", "HARXDULSBROLME-UHFFFAOYSA-N", "octylcyclopentane", "")
  CALL Species("C_{14}H_{28}",                        "1795-15-9", "FBXWCEKQCVOOLT-UHFFFAOYSA-N", "octylcyclohexane", "")
  CALL Species("C_{14}H_{28}",                        "2882-98-6", "GDCYEUOAZVKNHT-UHFFFAOYSA-N", "nonylcyclopentane", "")
  CALL Species("C_{14}H_{26}",                       "33704-60-8", "TUALLFJCLUYJEN-UHFFFAOYSA-N", "octahydro-1,1,2,3,3-pentamethyl-1H-indene", "")
  CALL Species("C_{15}H_{30}",                        "2883-02-5", "CLMFECCMAVQYQA-UHFFFAOYSA-N", "nonylcyclohexane", "")
  CALL Species("C_{15}H_{30}",                        "1795-21-7", "WOUVLFFZQCUYOL-UHFFFAOYSA-N", "decylcyclopentane", "")
  CALL Species("C_{16}H_{32}",                        "1795-16-0", "STWFZICHPLEOIC-UHFFFAOYSA-N", "decylcyclohexane", "")
  CALL Species("C_{18}H_{34}",                       "38970-72-8", "XUVKLBIJXLIPDZ-UHFFFAOYSA-N", "1,1'-(2-methylpentane-2,4-diyl)dicyclohexane", "")
  CALL Species("C_{18}H_{32}",                        "1706-50-9", "JBQRJHVXZMSLNH-UHFFFAOYSA-N", "1,1':3',1''-tercyclohexane", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Aliphatic alkenes and cycloalkenes")
  ! non-cyclic, one double bond:
  CALL Species("C_2H_4",                                "74-85-1", "VGGSQFUCUMXWEO-UHFFFAOYSA-N", "ethene", "ethylene")
  CALL Species("C_3H_6",                               "115-07-1", "QQONPFPTGQHPMA-UHFFFAOYSA-N", "propene", "propylene")
  CALL Species("C_4H_8",                               "106-98-9", "VXNZUUAINFGPBY-UHFFFAOYSA-N", "1-butene", "")
  CALL Species("C_4H_8",                               "107-01-7", "IAQRGUVFOMOMEM-UHFFFAOYSA-N", "2-butene", "")
  CALL Species("C_4H_8",                               "590-18-1", "IAQRGUVFOMOMEM-ARJAWSKDSA-N", "{\it cis}-2-butene", "")
  CALL Species("C_4H_8",                               "624-64-6", "IAQRGUVFOMOMEM-ONEGZZNKSA-N", "{\it trans}-2-butene", "")
  CALL Species("C_4H_8",                               "115-11-7", "VQTUBCCKSQIDNK-UHFFFAOYSA-N", "2-methylpropene", "isobutene")
  CALL Species("C_5H_{10}",                            "109-67-1", "YWAKXRMUMFPDSH-UHFFFAOYSA-N", "1-pentene", "")
  CALL Species("C_5H_{10}",                            "109-68-2", "QMMOXUPEWRXHJS-UHFFFAOYSA-N", "2-pentene", "")
  CALL Species("C_5H_{10}",                            "627-20-3", "QMMOXUPEWRXHJS-HYXAFXHYSA-N", "{\it cis}-2-pentene", "")
  CALL Species("C_5H_{10}",                            "646-04-8", "QMMOXUPEWRXHJS-HWKANZROSA-N", "{\it trans}-2-pentene", "")
  CALL Species("C_5H_{10}",                            "563-46-2", "MHNNAWXXUZQSNM-UHFFFAOYSA-N", "2-methyl-1-butene", "")
  CALL Species("C_5H_{10}",                            "513-35-9", "BKOOMYPCSUNDGP-UHFFFAOYSA-N", "2-methyl-2-butene", "")
  CALL Species("C_5H_{10}",                            "563-45-1", "YHQXBTXEYZIYOV-UHFFFAOYSA-N", "3-methyl-1-butene", "")
  CALL Species("C_6H_{12}",                            "592-41-6", "LIKMAJRDDDTEIG-UHFFFAOYSA-N", "1-hexene", "")
  CALL Species("C_6H_{12}",                           "7688-21-3", "RYPKRALMXUUNKS-HYXAFXHYSA-N", "{\it cis}-2-hexene", "")
  CALL Species("C_6H_{12}",                           "4050-45-7", "RYPKRALMXUUNKS-HWKANZROSA-N", "{\it trans}-2-hexene", "")
  CALL Species("C_6H_{12}",                           "7642-09-3", "ZQDPJFUHLCOCRG-WAYWQWQTSA-N", "{\it cis}-3-hexene", "")
  CALL Species("C_6H_{12}",                          "13269-52-8", "ZQDPJFUHLCOCRG-AATRIKPKSA-N", "{\it trans}-3-hexene", "")
  CALL Species("C_6H_{12}",                            "763-29-1", "WWUVJRULCWHUSA-UHFFFAOYSA-N", "2-methyl-1-pentene", "")
  CALL Species("C_6H_{12}",                            "760-20-3", "LDTAOIUHUHHCMU-UHFFFAOYSA-N", "3-methyl-1-pentene", "")
  CALL Species("C_6H_{12}",                            "691-37-2", "WSSSPWUEQFSQQG-UHFFFAOYSA-N", "4-methyl-1-pentene", "")
  CALL Species("C_6H_{12}",                            "625-27-4", "JMMZCWZIJXAGKW-UHFFFAOYSA-N", "2-methyl-2-pentene", "")
  CALL Species("C_6H_{12}",                            "922-62-3", "BEQGRRJLJLVQAQ-XQRVVYSFSA-N", "{\it cis}-3-methyl-2-pentene", "")
  CALL Species("C_6H_{12}",                            "616-12-6", "BEQGRRJLJLVQAQ-GQCTYLIASA-N", "{\it trans}-3-methyl-2-pentene", "")
  CALL Species("C_6H_{12}",                            "691-38-3", "LGAQJENWWYGFSN-PLNGDYQASA-N", "{\it cis}-4-methyl-2-pentene", "")
  CALL Species("C_6H_{12}",                            "674-76-0", "LGAQJENWWYGFSN-SNAWJCMRSA-N", "{\it trans}-4-methyl-2-pentene", "")
  CALL Species("C_6H_{12}",                            "760-21-4", "RYKZRKKEYSRDNF-UHFFFAOYSA-N", "3-methylenepentane", "2-ethyl-1-butene")
  CALL Species("C_6H_{12}",                            "563-78-0", "OWWIWYDDISJUMY-UHFFFAOYSA-N", "2,3-dimethyl-1-butene", "")
  CALL Species("C_6H_{12}",                            "558-37-2", "PKXHXOTZMFCXSH-UHFFFAOYSA-N", "3,3-dimethyl-1-butene", "")
  CALL Species("C_6H_{12}",                            "563-79-1", "WGLLSSPDPJPLOR-UHFFFAOYSA-N", "2,3-dimethyl-2-butene", "")
  CALL Species("C_7H_{14}",                            "592-76-7", "ZGEGCLOFRBLKSE-UHFFFAOYSA-N", "1-heptene", "")
  CALL Species("C_7H_{14}",                            "592-77-8", "OTTZHAVKAVGASB-UHFFFAOYSA-N", "2-heptene", "")
  CALL Species("C_7H_{14}",                           "6443-92-1", "OTTZHAVKAVGASB-HYXAFXHYSA-N", "{\it cis}-2-heptene", "")
  CALL Species("C_7H_{14}",                          "14686-13-6", "OTTZHAVKAVGASB-HWKANZROSA-N", "{\it trans}-2-heptene", "")
  CALL Species("C_7H_{14}",                           "7642-10-6", "WZHKDGJSXCTSCK-ALCCZGGFSA-N", "{\it cis}-3-heptene", "")
  CALL Species("C_7H_{14}",                          "14686-14-7", "WZHKDGJSXCTSCK-FNORWQNLSA-N", "{\it trans}-3-heptene", "")
  CALL Species("C_7H_{14}",                           "6094-02-6", "IRUDSQHLKGNCGF-UHFFFAOYSA-N", "2-methyl-1-hexene", "")
  CALL Species("C_7H_{14}",                           "2738-19-4", "BWEKDYGHDCHWEN-UHFFFAOYSA-N", "2-methyl-2-hexene", "")
  CALL Species("C_7H_{14}",                          "15840-60-5", "IQANHWBWTVLDTP-WAYWQWQTSA-N", "2-methyl-{\it cis}-3-hexene", "")
  CALL Species("C_7H_{14}",                            "692-24-0", "IQANHWBWTVLDTP-AATRIKPKSA-N", "2-methyl-{\it trans}-3-hexene", "")
  CALL Species("C_7H_{14}",                           "3404-61-3", "RITONZMLZWYPHW-UHFFFAOYSA-N", "3-methyl-1-hexene", "")
  CALL Species("C_7H_{14}",                          "10574-36-4", "JZMUUSXQSKCZNO-ALCCZGGFSA-N", "3-methyl-{\it cis}-2-hexene", "")
  CALL Species("C_7H_{14}",                          "20710-38-7", "JZMUUSXQSKCZNO-FNORWQNLSA-N", "3-methyl-{\it trans}-2-hexene", "")
  CALL Species("C_7H_{14}",                           "4914-89-0", "FHHSSXNRVNXTBG-SREVYHEPSA-N", "3-methyl-{\it cis}-3-hexene", "")
  CALL Species("C_7H_{14}",                           "3899-36-3", "FHHSSXNRVNXTBG-VOTSOKGWSA-N", "3-methyl-{\it trans}-3-hexene", "")
  CALL Species("C_7H_{14}",                           "3769-23-1", "SUWJESCICIOQHO-UHFFFAOYSA-N", "4-methyl-1-hexene", "")
  CALL Species("C_7H_{14}",                           "3683-19-0", "MBNDKEPQUVZHCM-XQRVVYSFSA-N", "4-methyl-{\it cis}-2-hexene", "")
  CALL Species("C_7H_{14}",                           "3683-22-5", "MBNDKEPQUVZHCM-GQCTYLIASA-N", "4-methyl-{\it trans}-2-hexene", "")
  CALL Species("C_7H_{14}",                           "3524-73-0", "JIUFYGIESXPUPL-UHFFFAOYSA-N", "5-methyl-1-hexene", "")
  CALL Species("C_7H_{14}",                          "13151-17-2", "GHBKCPRDHLITSE-PLNGDYQASA-N", "5-methyl-{\it cis}-2-hexene", "")
  CALL Species("C_7H_{14}",                           "7385-82-2", "GHBKCPRDHLITSE-SNAWJCMRSA-N", "5-methyl-{\it trans}-2-hexene", "")
  CALL Species("C_7H_{14}",                           "3404-72-6", "LIMAEKMEXJTSNI-UHFFFAOYSA-N", "2,3-dimethyl-1-pentene", "")
  CALL Species("C_7H_{14}",                          "10574-37-5", "WFHALSLYRWWUGH-UHFFFAOYSA-N", "2,3-dimethyl-2-pentene", "")
  CALL Species("C_7H_{14}",                           "2213-32-3", "LXQPBCHJNIOMQU-UHFFFAOYSA-N", "2,4-dimethyl-1-pentene", "")
  CALL Species("C_7H_{14}",                            "625-65-0", "VVCFYASOGFVJFN-UHFFFAOYSA-N", "2,4-dimethyl-2-pentene", "")
  CALL Species("C_7H_{14}",                           "3404-73-7", "TXBZITDWMURSEF-UHFFFAOYSA-N", "3,3-dimethyl-1-pentene", "")
  CALL Species("C_7H_{14}",                           "7385-78-6", "WFHXQNMTMDKVJG-UHFFFAOYSA-N", "3,4-dimethyl-1-pentene", "")
  CALL Species("C_7H_{14}",                           "4914-91-4", "PPBWEVVDSRKEIK-ALCCZGGFSA-N", "3,4-dimethyl-{\it cis}-2-pentene", "")
  CALL Species("C_7H_{14}",                           "4914-92-5", "PPBWEVVDSRKEIK-FNORWQNLSA-N", "3,4-dimethyl-{\it trans}-2-pentene", "")
  CALL Species("C_7H_{14}",                            "762-62-9", "KLCNJIQZXOQYTE-UHFFFAOYSA-N", "4,4-dimethyl-1-pentene", "")
  CALL Species("C_7H_{14}",                            "762-63-0", "BIDIHFPLDRSAMB-WAYWQWQTSA-N", "4,4-dimethyl-{\it cis}-2-pentene", "")
  CALL Species("C_7H_{14}",                            "690-08-4", "BIDIHFPLDRSAMB-AATRIKPKSA-N", "4,4-dimethyl-{\it trans}-2-pentene", "")
  CALL Species("C_7H_{14}",                           "3404-71-5", "TWCRBBJSQAZZQB-UHFFFAOYSA-N", "2-ethyl-1-pentene", "")
  CALL Species("C_7H_{14}",                           "4038-04-4", "YPVPQMCSLFDIKA-UHFFFAOYSA-N", "3-ethyl-1-pentene", "")
  CALL Species("C_7H_{14}",                            "816-79-5", "XMYFZAWUNVHVGI-UHFFFAOYSA-N", "3-ethyl-2-pentene", "")
  CALL Species("C_7H_{14}",                           "7357-93-9", "ADHCYQWFCLQBFG-UHFFFAOYSA-N", "3-methyl-2-ethyl-1-butene", "")
  CALL Species("C_7H_{14}",                            "594-56-9", "AUYRUAVCWOAHQN-UHFFFAOYSA-N", "2,3,3-trimethyl-1-butene", "")
  CALL Species("C_8H_{16}",                            "111-66-0", "KWKAKUADMBZCLK-UHFFFAOYSA-N", "1-octene", "")
  CALL Species("C_8H_{16}",                           "7642-04-8", "ILPBINAXDRFYPL-HYXAFXHYSA-N", "{\it cis}-2-octene", "")
  CALL Species("C_8H_{16}",                          "13389-42-9", "ILPBINAXDRFYPL-HWKANZROSA-N", "{\it trans}-2-octene", "")
  CALL Species("C_8H_{16}",                          "14850-22-7", "YCTDZYMMFQCTEO-ALCCZGGFSA-N", "{\it cis}-3-octene", "")
  CALL Species("C_8H_{16}",                          "14919-01-8", "YCTDZYMMFQCTEO-FNORWQNLSA-N", "{\it trans}-3-octene", "")
  CALL Species("C_8H_{16}",                           "7642-15-1", "IRUCBBFNLDIMIK-FPLPWBNLSA-N", "{\it cis}-4-octene", "")
  CALL Species("C_8H_{16}",                          "14850-23-8", "IRUCBBFNLDIMIK-BQYQJAHWSA-N", "{\it trans}-4-octene", "")
  CALL Species("C_8H_{16}",                          "15870-10-7", "RCBGGJURENJHKV-UHFFFAOYSA-N", "2-methyl-1-heptene", "")
  CALL Species("C_8H_{16}",                            "627-97-4", "WEPNJTDVIIKRIK-UHFFFAOYSA-N", "2-methyl-2-heptene", "")
  CALL Species("C_8H_{16}",                          "20488-34-0", "CYEZJYAMLNTSKN-SREVYHEPSA-N", "2-methyl-{\it cis}-3-heptene", "")
  CALL Species("C_8H_{16}",                            "692-96-6", "CYEZJYAMLNTSKN-VOTSOKGWSA-N", "2-methyl-{\it trans}-3-heptene", "")
  CALL Species("C_8H_{16}",                           "4810-09-7", "QDMFTFWKTYXBIW-UHFFFAOYSA-N", "3-methyl-1-heptene", "")
  CALL Species("C_8H_{16}",                          "22768-19-0", "OFKLSPUVNMOIJB-YVMONPNESA-N", "3-methyl-{\it cis}-2-heptene", "")
  CALL Species("C_8H_{16}",                          "22768-20-3", "OFKLSPUVNMOIJB-VMPITWQZSA-N", "3-methyl-{\it trans}-2-heptene", "")
  CALL Species("C_8H_{16}",                          "22768-17-8", "AAUHUDBDDBJONC-FPLPWBNLSA-N", "3-methyl-{\it cis}-3-heptene", "")
  CALL Species("C_8H_{16}",                          "22768-18-9", "AAUHUDBDDBJONC-BQYQJAHWSA-N", "3-methyl-{\it trans}-3-heptene", "")
  CALL Species("C_8H_{16}",                          "13151-05-8", "BFGOGLKYJXQPJZ-UHFFFAOYSA-N", "4-methyl-1-heptene", "")
  CALL Species("C_8H_{16}",                          "66225-16-9", "SVGLFIBXFVQUQY-XQRVVYSFSA-N", "4-methyl-{\it cis}-2-heptene", "")
  CALL Species("C_8H_{16}",                          "66225-17-0", "SVGLFIBXFVQUQY-GQCTYLIASA-N", "4-methyl-{\it trans}-2-heptene", "")
  CALL Species("C_8H_{16}",                          "14255-24-4", "KKVVJQGDNYIIMN-VURMDHGXSA-N", "4-methyl-{\it cis}-3-heptene", "")
  CALL Species("C_8H_{16}",                          "13714-85-7", "KKVVJQGDNYIIMN-SOFGYWHQSA-N", "4-methyl-{\it trans}-3-heptene", "")
  CALL Species("C_8H_{16}",                          "13151-04-7", "WNEYWVBECXCQRT-UHFFFAOYSA-N", "5-methyl-1-heptene", "")
  CALL Species("C_8H_{16}",                          "24608-84-2", "VIHUHUGDEZCPDK-XQRVVYSFSA-N", "5-methyl-{\it cis}-2-heptene", "")
  CALL Species("C_8H_{16}",                          "24608-85-3", "VIHUHUGDEZCPDK-GQCTYLIASA-N", "5-methyl-{\it trans}-2-heptene", "")
  CALL Species("C_8H_{16}",                          "50422-80-5", "YMNTZRCUPAYGLG-SREVYHEPSA-N", "5-methyl-{\it cis}-3-heptene", "")
  CALL Species("C_8H_{16}",                          "53510-18-2", "YMNTZRCUPAYGLG-VOTSOKGWSA-N", "5-methyl-{\it trans}-3-heptene", "")
  CALL Species("C_8H_{16}",                           "5026-76-6", "DFVOXRAAHOJJBN-UHFFFAOYSA-N", "6-methyl-1-heptene", "")
  CALL Species("C_8H_{16}",                          "66225-18-1", "LXBJRNXXTAWCKU-PLNGDYQASA-N", "6-methyl-{\it cis}-2-heptene", "")
  CALL Species("C_8H_{16}",                          "51065-65-7", "LXBJRNXXTAWCKU-SNAWJCMRSA-N", "6-methyl-{\it trans}-2-heptene", "")
  CALL Species("C_8H_{16}",                          "66225-19-2", "PMPISKBGRHSPEE-WAYWQWQTSA-N", "6-methyl-{\it cis}-3-heptene", "")
  CALL Species("C_8H_{16}",                          "66225-20-5", "PMPISKBGRHSPEE-AATRIKPKSA-N", "6-methyl-{\it trans}-3-heptene", "")
  CALL Species("C_8H_{16}",                            "690-92-6", "JPLZSSHKQZJYTJ-SREVYHEPSA-N", "2,2-dimethyl-{\it cis}-3-hexene", "")
  CALL Species("C_8H_{16}",                            "690-93-7", "JPLZSSHKQZJYTJ-VOTSOKGWSA-N", "2,2-dimethyl-{\it trans}-3-hexene", "")
  CALL Species("C_8H_{16}",                          "16746-86-4", "LVLXQRZPKUFJJQ-UHFFFAOYSA-N", "2,3-dimethyl-1-hexene", "")
  CALL Species("C_8H_{16}",                           "7145-20-2", "RGYAVZGBAJFMIZ-UHFFFAOYSA-N", "2,3-dimethyl-2-hexene", "")
  CALL Species("C_8H_{16}",                          "59643-75-3", "PRTXQHCLTIKAAJ-VURMDHGXSA-N", "2,3-dimethyl-{\it cis}-3-hexene", "")
  CALL Species("C_8H_{16}",                          "66225-30-7", "PRTXQHCLTIKAAJ-SOFGYWHQSA-N", "2,3-dimethyl-{\it trans}-3-hexene", "")
  CALL Species("C_8H_{16}",                          "16746-87-5", "PKVDGQHNRICJLA-UHFFFAOYSA-N", "2,4-dimethyl-1-hexene", "")
  CALL Species("C_8H_{16}",                          "14255-23-3", "IZSBIPQYBGDXJZ-UHFFFAOYSA-N", "2,4-dimethyl-2-hexene", "")
  CALL Species("C_8H_{16}",                          "37549-89-6", "VFCHHMABGOYOQI-VURMDHGXSA-N", "2,4-dimethyl-{\it cis}-3-hexene", "")
  CALL Species("C_8H_{16}",                          "61847-78-7", "VFCHHMABGOYOQI-SOFGYWHQSA-N", "2,4-dimethyl-{\it trans}-3-hexene", "")
  CALL Species("C_8H_{16}",                           "6975-92-4", "ISZWTVCVSJVEOL-UHFFFAOYSA-N", "2,5-dimethyl-1-hexene", "")
  CALL Species("C_8H_{16}",                           "3404-78-2", "VFZIUYUUQFYZBR-UHFFFAOYSA-N", "2,5-dimethyl-2-hexene", "")
  CALL Species("C_8H_{16}",                          "10557-44-5", "KNCMKWVOMRUHKZ-WAYWQWQTSA-N", "2,5-dimethyl-{\it cis}-3-hexene", "")
  CALL Species("C_8H_{16}",                            "692-70-6", "KNCMKWVOMRUHKZ-AATRIKPKSA-N", "2,5-dimethyl-{\it trans}-3-hexene", "")
  CALL Species("C_8H_{16}",                           "3404-77-1", "RXYYKIMRVXDSFR-UHFFFAOYSA-N", "3,3-dimethyl-1-hexene", "")
  CALL Species("C_8H_{16}",                          "16745-94-1", "OWWRMMIWAOBBFK-UHFFFAOYSA-N", "3,4-dimethyl-1-hexene", "")
  CALL Species("C_8H_{16}",                          "19550-81-3", "FMNLVKMLDPGPRY-ALCCZGGFSA-N", "3,4-dimethyl-{\it cis}-2-hexene", "")
  CALL Species("C_8H_{16}",                          "19550-82-4", "FMNLVKMLDPGPRY-FNORWQNLSA-N", "3,4-dimethyl-{\it trans}-2-hexene", "")
  CALL Species("C_8H_{16}",                          "19550-87-9", "XTUXVDJHGIEBAA-FPLPWBNLSA-N", "3,4-dimethyl-{\it cis}-3-hexene", "")
  CALL Species("C_8H_{16}",                          "19550-88-0", "XTUXVDJHGIEBAA-BQYQJAHWSA-N", "3,4-dimethyl-{\it trans}-3-hexene", "")
  CALL Species("C_8H_{16}",                           "7423-69-0", "FEZKAPRRVNNJTK-UHFFFAOYSA-N", "3,5-dimethyl-1-hexene", "")
  CALL Species("C_8H_{16}",                          "66225-31-8", "BQAZYKYBFAMHPG-YVMONPNESA-N", "3,5-dimethyl-{\it cis}-2-hexene", "")
  CALL Species("C_8H_{16}",                          "66225-12-5", "BQAZYKYBFAMHPG-VMPITWQZSA-N", "3,5-dimethyl-{\it trans}-2-hexene", "")
  CALL Species("C_8H_{16}",                           "1647-08-1", "SUJVAMIXNUAJEY-UHFFFAOYSA-N", "4,4-dimethyl-1-hexene", "")
  CALL Species("C_8H_{16}",                                    "", "OQEVAISXHCRQGF-ALCCZGGFSA-N", "4,4-dimethyl-{\it cis}-2-hexene", "")
  CALL Species("C_8H_{16}",                          "19550-83-5", "OQEVAISXHCRQGF-FNORWQNLSA-N", "4,4-dimethyl-{\it trans}-2-hexene", "")
  CALL Species("C_8H_{16}",                          "16106-59-5", "UFWIBUBEFUNVNI-UHFFFAOYSA-N", "4,5-dimethyl-1-hexene", "")
  CALL Species("C_8H_{16}",                          "65036-71-7", "OAVNNZUEVHDCKP-WAYWQWQTSA-N", "4,5-dimethyl-{\it cis}-2-hexene", "")
  CALL Species("C_8H_{16}",                          "66225-14-7", "OAVNNZUEVHDCKP-AATRIKPKSA-N", "4,5-dimethyl-{\it trans}-2-hexene", "")
  CALL Species("C_8H_{16}",                           "7116-86-1", "KZJIOVQKSAOPOP-UHFFFAOYSA-N", "5,5-dimethyl-1-hexene", "")
  CALL Species("C_8H_{16}",                          "39761-61-0", "NWZJLSKAFZXSQH-WAYWQWQTSA-N", "5,5-dimethyl-{\it cis}-2-hexene", "")
  CALL Species("C_8H_{16}",                          "39782-43-9", "NWZJLSKAFZXSQH-AATRIKPKSA-N", "5,5-dimethyl-{\it trans}-2-hexene", "")
  CALL Species("C_8H_{16}",                           "1632-16-2", "XTVRLCUJHGUXCP-UHFFFAOYSA-N", "2-ethyl-1-hexene", "")
  CALL Species("C_8H_{16}",                           "3404-58-8", "OLGHJTHQWQKJQQ-UHFFFAOYSA-N", "3-ethyl-1-hexene", "")
  CALL Species("C_8H_{16}",                          "16789-51-8", "AUJLDZJNMXNESO-UHFFFAOYSA-N", "3-ethyl-3-hexene", "")
  CALL Species("C_8H_{16}",                          "36880-72-5", "QEMJIDSLEPYRLM-YVMONPNESA-N", "3-ethyl-{\it cis}-2-hexene", "")
  CALL Species("C_8H_{16}",                          "66225-15-8", "QEMJIDSLEPYRLM-VMPITWQZSA-N", "3-ethyl-{\it trans}-2-hexene", "")
  CALL Species("C_8H_{16}",                          "16746-85-3", "OPMUAJRVOWSBTP-UHFFFAOYSA-N", "4-ethyl-1-hexene", "")
  CALL Species("C_8H_{16}",                          "54616-49-8", "STHONQMAWQLWLX-DAXSKMNVSA-N", "4-ethyl-{\it cis}-2-hexene", "")
  CALL Species("C_8H_{16}",                          "19781-63-6", "STHONQMAWQLWLX-QPJJXVBHSA-N", "4-ethyl-{\it trans}-2-hexene", "")
  CALL Species("C_8H_{16}",                            "560-23-6", "TUSBCMPNIOJUBX-UHFFFAOYSA-N", "2,3,3-trimethyl-1-pentene", "")
  CALL Species("C_8H_{16}",                            "565-76-4", "FAWUHEYSSPPNSH-UHFFFAOYSA-N", "2,3,4-trimethyl-1-pentene", "")
  CALL Species("C_8H_{16}",                            "565-77-5", "SZFRZEBLZFTODC-UHFFFAOYSA-N", "2,3,4-trimethyl-2-pentene", "")
  CALL Species("C_8H_{16}",                            "107-39-1", "FXNDIJDIPNCZQJ-UHFFFAOYSA-N", "2,4,4-trimethyl-1-pentene", "")
  CALL Species("C_8H_{16}",                            "107-40-4", "LAAVYEUJEMRIGF-UHFFFAOYSA-N", "2,4,4-trimethyl-2-pentene", "")
  CALL Species("C_8H_{16}",                            "560-22-5", "LLFHCOGPDCJMKY-UHFFFAOYSA-N", "3,3,4-trimethyl-1-pentene", "")
  CALL Species("C_8H_{16}",                            "564-03-4", "BOPVNOMJIDZBQB-UHFFFAOYSA-N", "3,4,4-trimethyl-1-pentene", "")
  CALL Species("C_8H_{16}",                            "598-96-9", "FZQMZRXKWHQJAG-UHFFFAOYSA-N", "3,4,4-trimethyl-2-pentene", "")
  CALL Species("C_8H_{16}",                          "39761-64-3", "FZQMZRXKWHQJAG-SREVYHEPSA-N", "3,4,4-trimethyl-{\it cis}-2-pentene", "")
  CALL Species("C_8H_{16}",                          "39761-57-4", "FZQMZRXKWHQJAG-VOTSOKGWSA-N", "3,4,4-trimethyl-{\it trans}-2-pentene", "")
  CALL Species("C_8H_{16}",                          "19780-66-6", "HPHHYSWOBXEIRG-UHFFFAOYSA-N", "2-methyl-3-ethyl-1-pentene", "")
  CALL Species("C_8H_{16}",                          "19780-67-7", "FQYUGAXHZSQHMU-UHFFFAOYSA-N", "2-methyl-3-ethyl-2-pentene", "")
  CALL Species("C_8H_{16}",                           "3404-67-9", "YXLCVBVDFKWWRW-UHFFFAOYSA-N", "3-methyl-2-ethyl-1-pentene", "")
  CALL Species("C_8H_{16}",                           "6196-60-7", "PHHHEKOJKDYRIN-UHFFFAOYSA-N", "3-methyl-3-ethyl-1-pentene", "")
  CALL Species("C_8H_{16}",                           "3404-80-6", "TVBQWTDYXVGWJL-UHFFFAOYSA-N", "4-methyl-2-ethyl-1-pentene", "")
  CALL Species("C_8H_{16}",                          "61847-80-1", "DTNALCAUPPLROB-UHFFFAOYSA-N", "4-methyl-3-ethyl-1-pentene", "")
  CALL Species("C_8H_{16}",                          "42067-48-1", "DSTFDBMUTNIZGD-YVMONPNESA-N", "4-methyl-3-ethyl-{\it cis}-2-pentene", "")
  CALL Species("C_8H_{16}",                          "42067-49-2", "DSTFDBMUTNIZGD-VMPITWQZSA-N", "4-methyl-3-ethyl-{\it trans}-2-pentene", "")
  CALL Species("C_8H_{16}",                          "15918-08-8", "FYUUBXZYRPRIHC-UHFFFAOYSA-N", "2-propyl-1-pentene", "")
  CALL Species("C_8H_{16}",                          "16746-02-4", "QOUCFWFZPKWYRE-UHFFFAOYSA-N", "2-isopropyl-1-pentene", "")
  CALL Species("C_8H_{16}",                          "18231-53-3", "KQRPZENSXZIOQO-UHFFFAOYSA-N", "3,3-dimethyl-2-ethyl-1-butene", "")
  CALL Species("C_8H_{16}",                         "111823-35-9", "GIBPFTYFAOOKOV-UHFFFAOYSA-N", "3-methyl-2-isopropyl-1-butene", "")
  CALL Species("C_9H_{18}",                            "124-11-8", "JRZJOMJEPLMPRA-UHFFFAOYSA-N", "1-nonene", "")
  CALL Species("C_9H_{18}",                           "4588-18-5", "FBEDQPGLIKZGIN-UHFFFAOYSA-N", "2-methyl-1-octene", "")
  CALL Species("C_{10}H_{20}",                         "872-05-9", "AFFLGGQVNFXPEV-UHFFFAOYSA-N", "1-decene", "")
  CALL Species("C_{10}H_{20}",                        "2980-71-4", "YLZQHQUVNZVGOK-UHFFFAOYSA-N", "2-methyl-1-nonene", "")
  CALL Species("C_{11}H_{22}",                         "821-95-4", "DCTOHCCUXLBQMS-UHFFFAOYSA-N", "1-undecene", "")
  CALL Species("C_{11}H_{22}",                       "13151-27-4", "HLMACKQLXSEXIY-UHFFFAOYSA-N", "2-methyl-1-decene", "")
  CALL Species("C_{12}H_{24}",                         "112-41-4", "CRSBERNSMYQZNG-UHFFFAOYSA-N", "1-dodecene", "")
  CALL Species("C_{12}H_{24}",                       "18516-37-5", "SJVKHZYVCVKEGM-UHFFFAOYSA-N", "2-methyl-1-undecene", "")
  CALL Species("C_{12}H_{24}",                         "123-48-8", "NBUMCEJRJRRLCA-UHFFFAOYSA-N", "2,2,4,6,6-pentamethyl-3-heptene", "")
  CALL Species("C_{13}H_{26}",                        "2437-56-1", "VQOXUMQBYILCKR-UHFFFAOYSA-N", "1-tridecene", "")
  CALL Species("C_{13}H_{26}",                       "16435-49-7", "PWRBDKMPAZFCSV-UHFFFAOYSA-N", "2-methyl-1-dodecene", "")
  CALL Species("C_{14}H_{28}",                        "1120-36-1", "HFDVRLIODXPAHB-UHFFFAOYSA-N", "1-tetradecene", "")
  CALL Species("C_{14}H_{28}",                         "638-60-8", "OBDUMNZXAIUUTH-UHFFFAOYSA-N", "2-tetradecene", "")
  CALL Species("C_{14}H_{28}",                       "18094-01-4", "VNBHQOHLCULRDN-UHFFFAOYSA-N", "2-methyl-1-tridecene", "")
  CALL Species("C_{15}H_{30}",                       "13360-61-7", "PJLHTVIBELQURV-UHFFFAOYSA-N", "1-pentadecene", "")
  CALL Species("C_{15}H_{30}",                       "52254-38-3", "WSNMNSLVXDWAFZ-UHFFFAOYSA-N", "2-methyl-1-tetradecene", "")
  CALL Species("C_{16}H_{32}",                         "629-73-2", "GQEZCXVZFLOKMC-UHFFFAOYSA-N", "1-hexadecene", "")
  CALL Species("C_{16}H_{32}",                       "29833-69-0", "FWQJRKLMXMTXDY-UHFFFAOYSA-N", "2-methyl-1-pentadecene", "")
  CALL Species("C_{17}H_{36}",                       "61868-07-3", "TXGHMPJBDYZJLF-UHFFFAOYSA-N", "2,4-dimethylpentadecane", "")
  ! non-cyclic, two or more double bonds:
  CALL Species("C_4H_6",                               "590-19-2", "QNRMTGGDHLBXQZ-UHFFFAOYSA-N", "1,2-butadiene", "")
  CALL Species("C_4H_6",                               "106-99-0", "KAKZBPTYRLMSJV-UHFFFAOYSA-N", "1,3-butadiene", "")
  CALL Species("C_5H_8",                               "598-25-4", "PAKGDPSCXSUALC-UHFFFAOYSA-N", "3-methyl-1,2-butadiene", "2-methyl-2,3-butadiene")
  CALL Species("C_5H_8",                                "78-79-5", "RRHGJUQNOFWUDK-UHFFFAOYSA-N", "2-methyl-1,3-butadiene", "isoprene")
  CALL Species("C_5H_8",                               "591-95-7", "LVMTVPFRTKXRPH-UHFFFAOYSA-N", "1,2-pentadiene", "")
  CALL Species("C_5H_8",                               "504-60-9", "PMJHHCWVYXUKFD-UHFFFAOYSA-N", "1,3-pentadiene", "")
  CALL Species("C_5H_8",                              "2004-70-8", "PMJHHCWVYXUKFD-SNAWJCMRSA-N", "{\it trans}-1,3-pentadiene", "")
  CALL Species("C_5H_8",                              "1574-41-0", "PMJHHCWVYXUKFD-PLNGDYQASA-N", "{\it cis}-1,3-pentadiene", "")
  CALL Species("C_5H_8",                               "591-93-5", "QYZLKGVUSQXAMU-UHFFFAOYSA-N", "1,4-pentadiene", "")
  CALL Species("C_5H_8",                               "591-96-8", "PODAMDNJNMAKAZ-UHFFFAOYSA-N", "2,3-pentadiene", "")
  CALL Species("C_6H_{10}",                            "592-44-9", "XIAJQOBRHVKGSP-UHFFFAOYSA-N", "1,2-hexadiene", "")
  CALL Species("C_6H_{10}",                          "14596-92-0", "AHAREKHAZNPPMI-WAYWQWQTSA-N", "{\it cis}-1,3-hexadiene", "")
  CALL Species("C_6H_{10}",                          "20237-34-7", "AHAREKHAZNPPMI-AATRIKPKSA-N", "{\it trans}-1,3-hexadiene", "")
  CALL Species("C_6H_{10}",                            "592-45-0", "PRBHEGAFLDMLAL-UHFFFAOYSA-N", "1,4-hexadiene", "")
  CALL Species("C_6H_{10}",                           "7318-67-4", "PRBHEGAFLDMLAL-XQRVVYSFSA-N", "{\it cis}-1,4-hexadiene", "")
  CALL Species("C_6H_{10}",                           "7319-00-8", "PRBHEGAFLDMLAL-GQCTYLIASA-N", "{\it trans}-1,4-hexadiene", "")
  CALL Species("C_6H_{10}",                            "592-42-7", "PYGSKMBEVAICCR-UHFFFAOYSA-N", "1,5-hexadiene", "")
  CALL Species("C_6H_{10}",                            "592-49-4", "DPUXQWOMYBMHRN-UHFFFAOYSA-N", "2,3-hexadiene", "")
  CALL Species("C_6H_{10}",                           "6108-61-8", "APPOKADJQUIAHP-GLIMQPGKSA-N", "{\it cis}-2,{\it cis}-4-hexadiene", "")
  CALL Species("C_6H_{10}",                           "5194-50-3", "APPOKADJQUIAHP-CIIODKQPSA-N", "{\it cis}-2,{\it trans}-4-hexadiene", "")
  CALL Species("C_6H_{10}",                           "5194-51-4", "APPOKADJQUIAHP-GGWOSOGESA-N", "{\it trans}-2,{\it trans}-4-hexadiene", "")
  CALL Species("C_6H_{10}",                            "763-30-4", "DRWYRROCDFQZQF-UHFFFAOYSA-N", "2-methyl-1,4-pentadiene", "")
  CALL Species("C_6H_{10}",                           "1501-60-6", "RCJMVGJKROQDCB-PLNGDYQASA-N", "2-methyl-1,{\it cis}-3-pentadiene", "")
  CALL Species("C_6H_{10}",                            "926-54-5", "RCJMVGJKROQDCB-SNAWJCMRSA-N", "2-methyl-1,{\it trans}-3-pentadiene", "")
  CALL Species("C_6H_{10}",                           "3043-33-2", "JWMDOGMKTRMFDS-UHFFFAOYSA-N", "2-methyl-2,3-pentadiene", "")
  CALL Species("C_6H_{10}",                           "7417-48-3", "INFFCVIZNSUFGK-UHFFFAOYSA-N", "3-methyl-1,2-pentadiene", "")
  CALL Species("C_6H_{10}",                           "2787-45-3", "BOGRNZQRTNVZCZ-WAYWQWQTSA-N", "3-methyl-1,{\it cis}-3-pentadiene", "")
  CALL Species("C_6H_{10}",                           "2787-43-1", "BOGRNZQRTNVZCZ-AATRIKPKSA-N", "3-methyl-1,{\it trans}-3-pentadiene", "")
  CALL Species("C_6H_{10}",                           "1115-08-8", "IKQUUYYDRTYXAP-UHFFFAOYSA-N", "3-methyl-1,4-pentadiene", "")
  CALL Species("C_6H_{10}",                          "13643-05-5", "CAAAXQFHDYHTTC-UHFFFAOYSA-N", "4-methyl-1,2-pentadiene", "")
  CALL Species("C_6H_{10}",                            "926-56-7", "CJSBUWDGPXGFGA-UHFFFAOYSA-N", "4-methyl-1,3-pentadiene", "")
  CALL Species("C_6H_{10}",                           "3404-63-5", "IGLWCQMNTGCUBB-UHFFFAOYSA-N", "2-ethyl-1,3-butadiene", "")
  CALL Species("C_6H_{10}",                            "513-81-5", "SDJHPPZKZZWAKF-UHFFFAOYSA-N", "2,3-dimethyl-1,3-butadiene", "")
  CALL Species("C_7H_{12}",                           "3070-53-9", "GEAWFZNTIFJMHR-UHFFFAOYSA-N", "1,6-heptadiene", "")
  CALL Species("C_8H_{14}",                            "764-13-6", "DZPCYXCBXGQBRN-UHFFFAOYSA-N", "2,5-dimethyl-2,4-hexadiene", "")
  ! cyclic, one double bond:
  CALL Species("C_4H_6",                              "3100-04-7", "SHDPRTQPPWIEJG-UHFFFAOYSA-N", "1-methylcyclopropene", "")
  CALL Species("C_5H_8",                               "142-29-0", "LPIQUOYDBNQMRZ-UHFFFAOYSA-N", "cyclopentene", "")
  CALL Species("C_6H_{10}",                            "110-83-8", "HGCIXCUEYOPUTN-UHFFFAOYSA-N", "cyclohexene", "")
  CALL Species("C_6H_{10}",                            "693-89-0", "ATQUFXWBVZUTKO-UHFFFAOYSA-N", "1-methylcyclopentene", "")
  CALL Species("C_7H_{12}",                            "628-92-2", "ZXIJMRYMVAMXQP-UHFFFAOYSA-N", "cycloheptene", "")
  CALL Species("C_6H_9CH_3",                           "591-49-1", "CTMHWPIWNRWQEG-UHFFFAOYSA-N", "1-methylcyclohexene", "")
  CALL Species("C_8H_{14}",                            "931-88-4", "URYYVOIYTNXXBN-UPHRSURJSA-N", "cyclooctene", "")
  CALL Species("C_{14}H_{24}",                       "33704-59-5", "CDEGOUYLXTUDAU-UHFFFAOYSA-N", "1,1,2,3,3-pentamethyl-2,3,4,5,6,7-hexahydro-1H-indene", "")
  ! cyclic, more than one double bond:
  CALL Species("C_5H_6",                               "542-92-7", "ZSWFCLXCOIISFI-UHFFFAOYSA-N", "1,3-cyclopentadiene", "")
  CALL Species("C_6H_8",                               "592-57-4", "MGNZXYYWBUKAII-UHFFFAOYSA-N", "1,3-cyclohexadiene", "")
  CALL Species("C_6H_8",                               "628-41-1", "UVJHQYIOXKWHFD-UHFFFAOYSA-N", "1,4-cyclohexadiene", "1,4-dihydrobenzene")
  CALL Species("C_7H_{10}",                           "4054-38-0", "GWYPDXLJACEENP-UHFFFAOYSA-N", "1,3-cycloheptadiene", "")
  CALL Species("C_7H_8",                               "544-25-2", "CHVJITGCYZJHLR-UHFFFAOYSA-N", "1,3,5-cycloheptatriene", "")
  CALL Species("C_8H_{12}",                            "111-78-4", "VYXHVRARDIDEHS-QGTKBVGQSA-N", "1,5-cyclooctadiene", "")
  CALL Species("C_8H_{12}",                           "2622-21-1", "SDRZFSPCVYEJTP-UHFFFAOYSA-N", "1-ethenylcyclohexene", "")
  CALL Species("C_8H_{12}",                            "100-40-3", "BBDKZWKEPDTENS-UHFFFAOYSA-N", "4-ethenylcyclohexene", "4-vinylcyclohexene")
  CALL Species("C_8H_8",                               "629-20-9", "KDUIUFJBNGTBMD-DLMDZQPMSA-N", "1,3,5,7-cyclooctatetraene", "")
  CALL Species("C_{10}H_{12}",                          "77-73-6", "HECLRDQVFMWTQS-UHFFFAOYSA-N", "3a,4,7,7a-tetrahydro-4,7-methano-1H-indene", "dicyclopentadiene")
  CALL Species("C_{12}H_{18}",                        "4904-61-4", "ZOLLIQAKMYWTBR-UHFFFAOYSA-N", "1,5,9-cyclododecatriene", "")
  CALL Species("C_{12}H_{18}",                         "706-31-0", "ZOLLIQAKMYWTBR-RYMQXAEESA-N", "($E$,$E$,$Z$)-1,5,9-cyclododecatriene", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Aliphatic alkynes")
  CALL Species("C_2H_2",                                "74-86-2", "HSFWRNGVRCDJHI-UHFFFAOYSA-N", "ethyne", "acetylene")
  CALL Species("CH_3CCH",                               "74-99-7", "MWWATHDPGQKSAR-UHFFFAOYSA-N", "propyne", "")
  CALL Species("C_2H_5CCH",                            "107-00-6", "KDKYADYSIPSCCQ-UHFFFAOYSA-N", "1-butyne", "ethylacetylene")
  CALL Species("C_4H_6",                               "503-17-3", "XNMQEEKYCVKGBD-UHFFFAOYSA-N", "2-butyne", "dimethylacetylene")
  CALL Species("C_3H_7CCH",                            "627-19-0", "IBXNCJKFFQIKKY-UHFFFAOYSA-N", "1-pentyne", "")
  CALL Species("C_5H_8",                               "627-21-4", "NKTDTMONXHODTI-UHFFFAOYSA-N", "2-pentyne", "")
  CALL Species("C_5H_8",                               "598-23-2", "USCSRAJGJYMJFZ-UHFFFAOYSA-N", "3-methyl-1-butyne", "")
  CALL Species("C_4H_9CCH",                            "693-02-7", "CGHIBGNXEGJPQZ-UHFFFAOYSA-N", "1-hexyne", "")
  CALL Species("C_6H_{10}",                            "764-35-2", "MELUCTCJOARQQG-UHFFFAOYSA-N", "2-hexyne", "")
  CALL Species("C_6H_{10}",                            "928-49-4", "DQQNMIPXXNPGCV-UHFFFAOYSA-N", "3-hexyne", "")
  CALL Species("C_6H_{10}",                            "922-59-8", "PLHJCCHSCFNKCC-UHFFFAOYSA-N", "3-methyl-1-pentyne", "")
  CALL Species("C_6H_{10}",                           "7154-75-8", "OXRWICUICBZVAE-UHFFFAOYSA-N", "4-methyl-1-pentyne", "")
  CALL Species("C_6H_{10}",                          "21020-27-9", "SLMFWJQZLPEDDU-UHFFFAOYSA-N", "4-methyl-2-pentyne", "")
  CALL Species("C_6H_{10}",                            "917-92-0", "PPWNCLVNXGCGAF-UHFFFAOYSA-N", "3,3-dimethyl-1-butyne", "")
  CALL Species("C_5H_{11}CCH",                         "628-71-7", "YVXHZKKCZYLQOP-UHFFFAOYSA-N", "1-heptyne", "")
  CALL Species("C_7H_{12}",                           "1119-65-9", "AMSFEMSYKQQCHL-UHFFFAOYSA-N", "2-heptyne", "")
  CALL Species("C_7H_{12}",                           "2586-89-2", "KLYHSJRCIZOUHE-UHFFFAOYSA-N", "3-heptyne", "")
  CALL Species("C_7H_{12}",                          "36566-80-0", "POBOUPFSQKXZFZ-UHFFFAOYSA-N", "2-methyl-3-hexyne", "")
  CALL Species("C_7H_{12}",                          "40276-93-5", "OPZULQHRFNTFFZ-UHFFFAOYSA-N", "3-methyl-1-hexyne", "")
  CALL Species("C_7H_{12}",                          "52713-81-2", "YFZSGTDENCTWGW-UHFFFAOYSA-N", "4-methyl-1-hexyne", "")
  CALL Species("C_7H_{12}",                          "20198-49-6", "ABEXZFRJQJSEBW-UHFFFAOYSA-N", "4-methyl-2-hexyne", "")
  CALL Species("C_7H_{12}",                           "2203-80-7", "HKNANEMUCJGPMS-UHFFFAOYSA-N", "5-methyl-1-hexyne", "")
  CALL Species("C_7H_{12}",                          "53566-37-3", "SVGAHRUSRQTQES-UHFFFAOYSA-N", "5-methyl-2-hexyne", "")
  CALL Species("C_7H_{12}",                            "918-82-1", "KQIIKSQUHGGYCU-UHFFFAOYSA-N", "3,3-dimethyl-1-pentyne", "")
  CALL Species("C_7H_{12}",                          "61064-08-2", "JDQKSTIAVKXRSK-UHFFFAOYSA-N", "3,4-dimethyl-1-pentyne", "")
  CALL Species("C_7H_{12}",                          "13361-63-2", "KHBYKPSFBHWBJQ-UHFFFAOYSA-N", "4,4-dimethyl-1-pentyne", "")
  CALL Species("C_7H_{12}",                            "999-78-0", "FOALCTWKQSWRST-UHFFFAOYSA-N", "4,4-dimethyl-2-pentyne", "")
  CALL Species("C_7H_{12}",                          "21020-26-8", "WGWGXWSBPXLXTA-UHFFFAOYSA-N", "3-ethyl-1-pentyne", "")
  CALL Species("C_6H_{13}CCH",                         "629-05-0", "UMIPWJGWASORKV-UHFFFAOYSA-N", "1-octyne", "")
  CALL Species("C_8H_{14}",                           "2809-67-8", "QCQALVMFTWRCFI-UHFFFAOYSA-N", "2-octyne", "")
  CALL Species("C_8H_{14}",                          "15232-76-5", "UDEISTCPVNLKRJ-UHFFFAOYSA-N", "3-octyne", "")
  CALL Species("C_8H_{14}",                           "4911-60-8", "XYBFBXTUWDPXLK-UHFFFAOYSA-N", "2,2-dimethyl-3-hexyne", "")
  CALL Species("C_7H_{15}CCH",                        "3452-09-3", "OSSQSXOTMIGBCF-UHFFFAOYSA-N", "1-nonyne", "")
  CALL Species("C_9H_{16}",                          "19447-29-1", "LXKRETAGISZJAD-UHFFFAOYSA-N", "2-nonyne", "")
  CALL Species("C_9H_{16}",                          "20184-89-8", "SRRDSRCWRHKEKU-UHFFFAOYSA-N", "3-nonyne", "")
  CALL Species("C_9H_{16}",                          "17530-23-3", "DKFPMSNUJIRAMR-UHFFFAOYSA-N", "2,2,5-trimethyl-3-hexyne", "")
  CALL Species("C_{10}H_{18}",                       "17530-24-4", "FXVDWKZNFZMSOU-UHFFFAOYSA-N", "2,2,5,5-tetramethyl-3-hexyne", "")
  CALL Species("C_{10}H_{18}",                         "764-93-2", "ILLHQJIJCRNRCJ-UHFFFAOYSA-N", "1-decyne", "")
  CALL Species("C_{10}H_{18}",                        "2384-70-5", "RWDDSTHSVISBEA-UHFFFAOYSA-N", "2-decyne", "")
  CALL Species("C_{10}H_{18}",                        "2384-85-2", "JUWXVJKQNKKRLD-UHFFFAOYSA-N", "3-decyne", "")
  CALL Species("C_{11}H_{20}",                        "2243-98-3", "YVSFLVNWJIEJRV-UHFFFAOYSA-N", "1-undecyne", "")
  CALL Species("C_{11}H_{20}",                       "60212-29-5", "XZSXEDPHMIFYOS-UHFFFAOYSA-N", "2-undecyne", "")
  CALL Species("C_{11}H_{20}",                       "60212-30-8", "DPWGJNPCPLQVKQ-UHFFFAOYSA-N", "3-undecyne", "")
  CALL Species("C_{12}H_{22}",                         "765-03-7", "ZVDBUOGYYYNMQI-UHFFFAOYSA-N", "1-dodecyne", "")
  CALL Species("C_{12}H_{22}",                         "629-49-2", "NDIJGAGRSOPRNJ-UHFFFAOYSA-N", "2-dodecyne", "")
  CALL Species("C_{12}H_{22}",                        "6790-27-8", "ZFAGQZXKTQFQLE-UHFFFAOYSA-N", "3-dodecyne", "")
  CALL Species("C_{13}H_{24}",                       "26186-02-7", "GZEDKDBFUBPZNG-UHFFFAOYSA-N", "1-tridecyne", "")
  CALL Species("C_{13}H_{24}",                       "28467-75-6", "ZGKKGWBQPYIOBH-UHFFFAOYSA-N", "2-tridecyne", "")
  CALL Species("C_{13}H_{24}",                       "60186-78-9", "HDLJOUCNTFYRPD-UHFFFAOYSA-N", "3-tridecyne", "")
  CALL Species("C_{14}H_{26}",                         "765-10-6", "DZEFNRWGWQDGTR-UHFFFAOYSA-N", "1-tetradecyne", "")
  CALL Species("C_{14}H_{26}",                       "60212-32-0", "DWWMJIDVLGUCDE-UHFFFAOYSA-N", "3-tetradecyne", "")
  CALL Species("C_{15}H_{28}",                         "765-13-9", "DONJGKADZJEXRJ-UHFFFAOYSA-N", "1-pentadecyne", "")
  CALL Species("C_{15}H_{28}",                       "52112-25-1", "VWWAVTXUUYIIEE-UHFFFAOYSA-N", "2-pentadecyne", "")
  CALL Species("C_{15}H_{28}",                       "61886-61-1", "RUHQUJVXUGQUNY-UHFFFAOYSA-N", "3-pentadecyne", "")
  CALL Species("C_{16}H_{30}",                         "629-74-3", "UCIDYSLOTJMRAM-UHFFFAOYSA-N", "1-hexadecyne", "")
  CALL Species("C_{16}H_{30}",                       "61886-62-2", "HRFPRVYNMNOAQO-UHFFFAOYSA-N", "3-hexadecyne", "")
  CALL Species("CH_2CHCCH",                            "689-97-4", "WFYPICNXBKQZGB-UHFFFAOYSA-N", "3-buten-1-yne", "vinylacetylene")
  CALL Species("C_4H_2",                               "460-12-8", "LLCSWKVOHICRDD-UHFFFAOYSA-N", "butadiyne", "biacetylene")
  CALL Species("C_7H_8",                              "2396-63-6", "RSPZSDWVQWRAEF-UHFFFAOYSA-N", "1,6-heptadiyne", "")
  CALL Species("C_9H_{12}",                           "2396-65-8", "DMOVPHYFYSASTC-UHFFFAOYSA-N", "1,8-nonadiyne", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Mononuclear aromatics")
  ! saturated substituents, sorted be increasing number of C atoms:
  CALL Species("C_6H_6",                                "71-43-2", "UHOVQNZJYSORNB-UHFFFAOYSA-N", "benzene", "")
  CALL Species("C_6D_6",                              "1076-43-3", "UHOVQNZJYSORNB-MZWXYZOWSA-N", "benzene-d6", "")
  CALL Species("C_6H_5CH_3",                           "108-88-3", "YXFVVABEGXRONW-UHFFFAOYSA-N", "methylbenzene", "toluene")
  CALL Species("C_6D_5CD_3",                          "2037-26-5", "YXFVVABEGXRONW-JGUCLWPXSA-N", "methylbenzene-d8", "toluene-d8")
  CALL Species("C_6H_4(CH_3)_2",                        "95-47-6", "CTQNGGLPUBDAKN-UHFFFAOYSA-N", "1,2-dimethylbenzene", "$o$-xylene")
  CALL Species("C_6D_4(CD_3)_2",                     "56004-61-6", "CTQNGGLPUBDAKN-ZGYYUIRESA-N", "1,2-dimethylbenzene-d10", "$o$-xylene-d10")
  CALL Species("C_6H_4(CH_3)_2",                       "108-38-3", "IVSZLXZYQVIEFR-UHFFFAOYSA-N", "1,3-dimethylbenzene", "$m$-xylene")
  CALL Species("C_6H_4(CH_3)_2",                       "106-42-3", "URLKBWYHVLBVBO-UHFFFAOYSA-N", "1,4-dimethylbenzene", "$p$-xylene")
  CALL Species("C_6H_3(CH_3)_3",                       "526-73-8", "FYGHSUNMUKGBRK-UHFFFAOYSA-N", "1,2,3-trimethylbenzene", "")
  CALL Species("C_6H_3(CH_3)_3",                        "95-63-6", "GWHJZXXIDMPWGX-UHFFFAOYSA-N", "1,2,4-trimethylbenzene", "")
  CALL Species("C_6H_3(CH_3)_3",                       "108-67-8", "AUHZEENZYGFFBQ-UHFFFAOYSA-N", "1,3,5-trimethylbenzene", "mesitylene")
  CALL Species("C_{10}H_{14}",                         "488-23-3", "UOHMMEJUHBCKEE-UHFFFAOYSA-N", "1,2,3,4-tetramethylbenzene", "")
  CALL Species("C_{10}H_{14}",                         "527-53-7", "BFIMMTCNYPIMRN-UHFFFAOYSA-N", "1,2,3,5-tetramethylbenzene", "")
  CALL Species("C_{10}H_{14}",                          "95-93-2", "SQNZJJAZBFDUTD-UHFFFAOYSA-N", "1,2,4,5-tetramethylbenzene", "")
  CALL Species("C_6H_5C_2H_5",                         "100-41-4", "YNQLUTRBYVCPMQ-UHFFFAOYSA-N", "ethylbenzene", "")
  CALL Species("C_6D_5C_2D_5",                       "25837-05-2", "YNQLUTRBYVCPMQ-CFTAVCBPSA-N", "ethylbenzene-d10", "")
  CALL Species("C_{10}H_{14}",                         "135-01-3", "KVNYFPKFSJIPBJ-UHFFFAOYSA-N", "1,2-diethylbenzene", "$o$-diethylbenzene")
  CALL Species("C_{10}H_{14}",                         "141-93-5", "AFZZYIJIWUTJFO-UHFFFAOYSA-N", "1,3-diethylbenzene", "$m$-diethylbenzene")
  CALL Species("C_{10}H_{14}",                         "105-05-5", "DSNHSQKRULAAEI-UHFFFAOYSA-N", "1,4-diethylbenzene", "$p$-diethylbenzene")
  CALL Species("C_6H_5C_3H_7",                         "103-65-1", "ODLMAHJVESYWTB-UHFFFAOYSA-N", "propylbenzene", "")
  CALL Species("C_6H_5C_3H_7",                          "98-82-8", "RWGFKTVRMDUZSP-UHFFFAOYSA-N", "(2-propyl)-benzene", "isopropylbenzene; cumene")
  CALL Species("C_6H_4CH_3C_2H_5",                     "611-14-3", "HYFLWBNQFMXCPA-UHFFFAOYSA-N", "1-ethyl-2-methylbenzene", "$o$-ethyltoluene")
  CALL Species("C_6H_4CH_3C_2H_5",                     "620-14-4", "ZLCSFXXPPANWQY-UHFFFAOYSA-N", "1-ethyl-3-methylbenzene", "$m$-ethyltoluene")
  CALL Species("C_6H_4CH_3C_2H_5",                     "622-96-8", "JRLPEMVDPFPYPJ-UHFFFAOYSA-N", "1-ethyl-4-methylbenzene", "$p$-ethyltoluene")
  CALL Species("C_6H_5C_4H_9",                         "104-51-8", "OCKPCBLVNKHBMX-UHFFFAOYSA-N", "butylbenzene", "")
  CALL Species("C_6H_5C_4H_9",                         "135-98-8", "ZJMWRROPUADPEA-UHFFFAOYSA-N", "(1-methylpropyl)-benzene", "{\it sec}-butylbenzene") ! other CAS: 36383-15-0
  CALL Species("C_6H_5C_4H_9",                         "538-93-2", "KXUHSQYYJYAXGZ-UHFFFAOYSA-N", "(2-methylpropyl)-benzene", "isobutylbenzene")
  CALL Species("C_6H_5C_4H_9",                          "98-06-6", "YTZKOQUCBOVLHL-UHFFFAOYSA-N", "(1,1-dimethylethyl)-benzene", "{\it tert}-butylbenzene")
  CALL Species("C_{10}H_{14}",                        "1074-17-5", "YQZBFMJOASEONC-UHFFFAOYSA-N", "1-methyl-2-propylbenzene", "")
  CALL Species("C_{10}H_{14}",                        "1074-43-7", "QUEBYVKXYIKVSO-UHFFFAOYSA-N", "1-methyl-3-propylbenzene", "")
  CALL Species("C_{10}H_{14}",                        "1074-55-1", "JXFVMNFKABWTHD-UHFFFAOYSA-N", "1-methyl-4-propylbenzene", "")
  CALL Species("C_{10}H_{14}",                        "2870-04-4", "CHIKRULMSSADAF-UHFFFAOYSA-N", "2-ethyl-$m$-xylene", "")
  CALL Species("C_{10}H_{14}",                        "1758-88-9", "AXIUBBVSOWPLDA-UHFFFAOYSA-N", "2-ethyl-$p$-xylene", "")
  CALL Species("C_{10}H_{14}",                         "933-98-2", "QUBBAXISAHIDNM-UHFFFAOYSA-N", "3-ethyl-$o$-xylene", "")
  CALL Species("C_{10}H_{14}",                         "934-80-5", "SBUYFICWQNHBCM-UHFFFAOYSA-N", "4-ethyl-$o$-xylene", "")
  CALL Species("C_{10}H_{14}",                         "874-41-9", "MEMBJMDZWKVOTB-UHFFFAOYSA-N", "4-ethyl-$m$-xylene", "")
  CALL Species("C_{10}H_{14}",                         "934-74-7", "LMAUULKNZLEMGN-UHFFFAOYSA-N", "5-ethyl-$m$-xylene", "1-ethyl-3,5-dimethylbenzene")
  CALL Species("C_{10}H_{14}",                         "527-84-4", "WWRCMNKATXZARA-UHFFFAOYSA-N", "1-methyl-2-(1-methylethyl)-benzene", "$o$-cymene")
  CALL Species("C_{10}H_{14}",                         "535-77-3", "XCYJPXQACVEIOS-UHFFFAOYSA-N", "1-methyl-3-(1-methylethyl)-benzene", "$m$-cymene")
  CALL Species("C_{10}H_{14}",                          "99-87-6", "HFPZCAJZSCWRBC-UHFFFAOYSA-N", "1-methyl-4-(1-methylethyl)-benzene", "$p$-cymene; $p$-isopropyltoluene")
  CALL Species("C_{11}H_{16}",                          "98-51-1", "QCWXDVFBZVHKLV-UHFFFAOYSA-N", "4-{\it tert}-butyltoluene", "")
  CALL Species("C_6H_5C_5H_{11}",                      "538-68-1", "PWATWSYOIIXYMA-UHFFFAOYSA-N", "pentylbenzene", "")
  CALL Species("C_{11}H_{16}",                         "700-12-9", "BEZDDPMMPIDMGJ-UHFFFAOYSA-N", "pentamethylbenzene", "")
  CALL Species("C_{11}H_{16}",                       "17059-44-8", "IRUSTUOJENXLMN-UHFFFAOYSA-N", "1,2-dimethyl-3-propylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "3982-66-9", "FZJVYOOQGFZCSY-UHFFFAOYSA-N", "1,2-dimethyl-4-propylbenzene", "")
  CALL Species("C_{11}H_{16}",                       "17059-45-9", "POVRSTNZQPBWAS-UHFFFAOYSA-N", "1,3-dimethyl-2-propylbenzene", "")
  CALL Species("C_{11}H_{16}",                       "61827-85-8", "HPAXKQMKDWCLGU-UHFFFAOYSA-N", "1,3-dimethyl-4-propylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "3982-64-7", "NBICXWXPZRNMPF-UHFFFAOYSA-N", "1,3-dimethyl-5-propylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "3042-50-0", "PWEDYOIWLPZSRP-UHFFFAOYSA-N", "1,4-dimethyl-2-propylbenzene", "")
  CALL Species("C_{11}H_{16}",                       "22539-65-7", "GDEQPEBFOWYWSA-UHFFFAOYSA-N", "1,2-dimethyl-3-isopropylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "4132-77-8", "MGMSKQZIAGFMRU-UHFFFAOYSA-N", "1,2-dimethyl-4-isopropylbenzene", "")
  CALL Species("C_{11}H_{16}",                       "14411-75-7", "IVCIQLTVLDOHKA-UHFFFAOYSA-N", "1,3-dimethyl-2-isopropylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "4706-89-2", "AADQFNAACHHRLT-UHFFFAOYSA-N", "1,3-dimethyl-4-isopropylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "4706-90-5", "RMKJTYPFCFNTGQ-UHFFFAOYSA-N", "1,3-dimethyl-5-isopropylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "4132-72-3", "CLSBTDGUHSQYTO-UHFFFAOYSA-N", "1,4-dimethyl-2-isopropylbenzene", "")
  CALL Species("C_{11}H_{16}",                       "16021-20-8", "DMUVQFCRCMDZPW-UHFFFAOYSA-N", "1-ethyl-2-propylbenzene", "")
  CALL Species("C_{11}H_{16}",                       "20024-91-3", "QCYGXOCMWHSXSU-UHFFFAOYSA-N", "1-ethyl-3-propylbenzene", "")
  CALL Species("C_{11}H_{16}",                       "20024-90-2", "ADQDTIAWIXUACV-UHFFFAOYSA-N", "1-ethyl-4-propylbenzene", "")
  CALL Species("C_{11}H_{16}",                       "18970-44-0", "ZAJYARZMPOEGLK-UHFFFAOYSA-N", "1-ethyl-2-isopropylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "4920-99-4", "GSLSBTNLESMZTN-UHFFFAOYSA-N", "1-ethyl-3-isopropylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "4218-48-8", "GUUDUUDWUWUTPD-UHFFFAOYSA-N", "1-ethyl-4-isopropylbenzene", "")
  CALL Species("C_{11}H_{16}",                       "13632-93-4", "LRJOXARIJKBUFE-UHFFFAOYSA-N", "1-methyl-2,3-diethylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "1758-85-6", "PZMJNJDRDKPVLB-UHFFFAOYSA-N", "1-methyl-2,4-diethylbenzene", "")
  CALL Species("C_{11}H_{16}",                       "13632-94-5", "ZEHGGUIGEDITMM-UHFFFAOYSA-N", "1-methyl-2,5-diethylbenzene", "")
  CALL Species("C_{11}H_{16}",                       "13632-95-6", "XZZNTLNFQVAKFD-UHFFFAOYSA-N", "1-methyl-2,6-diethylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "2050-24-0", "HILAULICMJUOLK-UHFFFAOYSA-N", "1-methyl-3,5-diethylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "1595-11-5", "NUJILYKLNKQOOX-UHFFFAOYSA-N", "1-methyl-2-butylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "1595-04-6", "OAPCPUDMDJIBOQ-UHFFFAOYSA-N", "1-methyl-3-butylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "1595-05-7", "SBBKUBSYOVDBBC-UHFFFAOYSA-N", "1-methyl-4-butylbenzene", "")
  CALL Species("C_{11}H_{16}",                       "36301-29-8", "XNMPJDZAHSMAMN-UHFFFAOYSA-N", "1-methyl-2-isobutylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "5160-99-6", "SDHYGAUOCHFYSR-UHFFFAOYSA-N", "1-methyl-3-isobutylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "5161-04-6", "VCGBZXLLPCGFQM-UHFFFAOYSA-N", "1-methyl-4-isobutylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "1595-06-8", "AMBAWAHKHZZAAY-UHFFFAOYSA-N", "1-methyl-2-{\it sec}-butylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "1772-10-7", "RMNILBOMCXQZFC-UHFFFAOYSA-N", "1-methyl-3-{\it sec}-butylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "1595-16-0", "LWCFXYMSEGQWNB-UHFFFAOYSA-N", "1-methyl-4-{\it sec}-butylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "1074-92-6", "AXHVNJGQOJFMHT-UHFFFAOYSA-N", "1-methyl-2-{\it tert}-butyl benzene", "")
  CALL Species("C_{11}H_{16}",                        "1075-38-3", "JTIAYWZZZOZUTK-UHFFFAOYSA-N", "1-methyl-3-{\it tert}-butylbenzene", "")
  CALL Species("C_{11}H_{16}",                       "13732-80-4", "MJQJAQGFUBIGIK-UHFFFAOYSA-N", "1-methyl-3,4-diethylbenzene", "")
  CALL Species("C_{11}H_{16}",                        "1007-26-7", "CJGXJKVMUHXVHL-UHFFFAOYSA-N", "1-phenyl-2,2-dimethylpropane", "")
  CALL Species("C_{11}H_{16}",                        "3968-85-2", "IFDLFCDWOFLKEB-UHFFFAOYSA-N", "1-phenyl-2-methylbutane", "")
  CALL Species("C_{11}H_{16}",                        "2049-94-7", "XNXIYYFOYIUJIW-UHFFFAOYSA-N", "1-phenyl-3-methylbutane", "")
  CALL Species("C_{11}H_{16}",                        "4481-30-5", "NQRMTOKLHZNAQH-UHFFFAOYSA-N", "2-phenyl-3-methylbutane", "")
  CALL Species("C_{11}H_{16}",                        "2719-52-0", "LTHAIAJHDPJXLG-UHFFFAOYSA-N", "2-phenylpentane", "")
  CALL Species("C_{11}H_{16}",                        "1196-58-3", "PBWHJRFXUPLZDS-UHFFFAOYSA-N", "3-phenylpentane", "")
  CALL Species("C_6H_5C_5H_{11}",                     "2049-95-8", "QHTJSSMHBLGUHV-UHFFFAOYSA-N", "(1,1-dimethylpropyl)-benzene", "{\it tert}-amylbenzene")
  CALL Species("C_6H_5C_6H_{13}",                     "1077-16-3", "LTEQMZWBSYACLV-UHFFFAOYSA-N", "hexylbenzene", "")
  CALL Species("C_{12}H_{18}",                          "87-85-4", "YUWFEBAXEOLKSG-UHFFFAOYSA-N", "hexamethylbenzene", "")
  CALL Species("C_{12}H_{18}",                       "42205-08-3", "VIDOPANCAUPXNH-UHFFFAOYSA-N", "1,2,3-triethylbenzene", "")
  CALL Species("C_{12}H_{18}",                         "877-44-1", "WNLWIOJSURYFIB-UHFFFAOYSA-N", "1,2,4-triethylbenzene", "")
  CALL Species("C_{12}H_{18}",                         "102-25-0", "WJYMPXJVHNDZHD-UHFFFAOYSA-N", "1,3,5-triethylbenzene", "")
  CALL Species("C_{12}H_{18}",                        "7397-06-0", "QRPPSTNABSMSCS-UHFFFAOYSA-N", "4-{\it tert}-butyl-$o$-xylene", "")
  CALL Species("C_{12}H_{18}",                          "98-19-1", "FZSPYHREEHYLCB-UHFFFAOYSA-N", "1-(1,1-dimethylethyl)-3,5-dimethylbenzene", "")
  CALL Species("C_{12}H_{18}",                       "25321-09-9", "OKIRBHVFJGXOIS-UHFFFAOYSA-N", "$o$-diisopropylbenzene", "")
  CALL Species("C_{12}H_{18}",                          "99-62-7", "UNEATYXSUBPPKP-UHFFFAOYSA-N", "$m$-diisopropylbenzene", "")
  CALL Species("C_{12}H_{18}",                         "100-18-5", "SPPWGCYEYAMHDT-UHFFFAOYSA-N", "$p$-diisopropylbenzene", "")
  CALL Species("C_{12}H_{14}",                        "4994-16-5", "XWCWNUSFQVJNDI-UHFFFAOYSA-N", "4-phenylcyclohexene", "")
  CALL Species("C_6H_5C_7H_{15}",                     "1078-71-3", "LBNXAWYDQUGHGX-UHFFFAOYSA-N", "heptylbenzene", "")
  CALL Species("C_{13}H_{20}",                          "98-23-7", "ZQVJKYPEIPJEIP-UHFFFAOYSA-N", "5-{\it tert}-butyl-1,2,3-trimethylbenzene", "")
  CALL Species("C_6H_5C_8H_{17}",                     "2189-60-8", "CDKDZKXSXLNROY-UHFFFAOYSA-N", "octylbenzene", "")
  CALL Species("C_{15}H_{24}",                        "1081-77-2", "LIXVMPBOGDCSRM-UHFFFAOYSA-N", "nonylbenzene", "")
  CALL Species("C_{15}H_{24}",                       "15181-11-0", "WIXDSJRJFDWTNY-UHFFFAOYSA-N", "3,5-di-{\it tert}-butyltoluene", "")
  CALL Species("C_{15}H_{24}",                         "717-74-8", "VUMCUSHVMYIRMB-UHFFFAOYSA-N", "1,3,5-tris(1-methylethyl)benzene", "")
  CALL Species("C_{16}H_{18}",                       "64800-83-5", "BDEIYMXBPHSOSG-UHFFFAOYSA-N", "ethyl(phenylethyl)-benzene", "")
  CALL Species("C_{16}H_{18}",                        "6165-52-2", "JOUBGGHXBLOLFY-UHFFFAOYSA-N", "4-(1-phenylethyl)-$m$-xylene", "")
  CALL Species("C_{16}H_{26}",                         "104-72-3", "UZILCZKGXMQEQR-UHFFFAOYSA-N", "decylbenzene", "1-phenyldecane")
  CALL Species("C_{16}H_{26}",                        "4537-13-7", "DDTJIQUCOLHYDL-UHFFFAOYSA-N", "2-phenyldecane", "")
  CALL Species("C_{16}H_{26}",                        "4621-36-7", "PYVIFMPVFLOTLN-UHFFFAOYSA-N", "3-phenyldecane", "")
  CALL Species("C_{16}H_{26}",                        "4537-12-6", "QTDBKYLPIZUTFN-UHFFFAOYSA-N", "4-phenyldecane", "")
  CALL Species("C_{16}H_{26}",                        "4537-11-5", "CDOBABYRHNVZQG-UHFFFAOYSA-N", "5-phenyldecane", "")
  CALL Species("C_{17}H_{28}",                        "6742-54-7", "XBEADGFTLHRJRB-UHFFFAOYSA-N", "undecylbenzene", "1-phenylundecane")
  CALL Species("C_{17}H_{28}",                        "4536-88-3", "YHJBCRBYDBNEIK-UHFFFAOYSA-N", "2-phenylundecane", "")
  CALL Species("C_{17}H_{28}",                        "4536-87-2", "NVHBFHMWJJMQTG-UHFFFAOYSA-N", "3-phenylundecane", "")
  CALL Species("C_{17}H_{28}",                        "4536-86-1", "NSQAXMRLBNXEHK-UHFFFAOYSA-N", "4-phenylundecane", "")
  CALL Species("C_{17}H_{28}",                        "4537-15-9", "RRPCXIBGXYGQNC-UHFFFAOYSA-N", "5-phenylundecane", "")
  CALL Species("C_{17}H_{28}",                        "4537-14-8", "WCABIRIFXVXGQH-UHFFFAOYSA-N", "6-phenylundecane", "")
  CALL Species("C_{18}H_{30}",                         "123-01-3", "KWKXNDCHNDYVRT-UHFFFAOYSA-N", "dodecylbenzene", "1-phenyldodecane")
  CALL Species("C_{18}H_{30}",                        "2719-61-1", "VRPRIAVYSREHAN-UHFFFAOYSA-N", "2-phenyldodecane", "")
  CALL Species("C_{18}H_{30}",                        "2400-00-2", "PGVOXXHNGYYHHB-UHFFFAOYSA-N", "3-phenyldodecane", "")
  CALL Species("C_{18}H_{30}",                        "2719-64-4", "RHDHXBLZBVAPTL-UHFFFAOYSA-N", "4-phenyldodecane", "")
  CALL Species("C_{18}H_{30}",                        "2719-63-3", "NPAWGLOPXKCTCV-UHFFFAOYSA-N", "5-phenyldodecane", "")
  CALL Species("C_{18}H_{30}",                        "2719-62-2", "ZYHJQFMTTFCBKH-UHFFFAOYSA-N", "6-phenyldodecane", "")
  CALL Species("C_{19}H_{32}",                         "123-02-4", "MCVUKOYZUCWLQQ-UHFFFAOYSA-N", "tridecylbenzene", "1-phenyltridecane")
  CALL Species("C_{19}H_{32}",                        "4534-53-6", "FCXPVFLEDIQLLO-UHFFFAOYSA-N", "2-phenyltridecane", "")
  CALL Species("C_{19}H_{32}",                        "4534-52-5", "VZZMNLVGDGMQQV-UHFFFAOYSA-N", "3-phenyltridecane", "")
  CALL Species("C_{19}H_{32}",                        "4534-51-4", "RZGVZPAWCGDMCK-UHFFFAOYSA-N", "4-phenyltridecane", "")
  CALL Species("C_{19}H_{32}",                        "4534-50-3", "MZTIRLOLMGVVEK-UHFFFAOYSA-N", "5-phenyltridecane", "")
  CALL Species("C_{19}H_{32}",                        "4534-49-0", "OTSYFFDVDLHIKX-UHFFFAOYSA-N", "6-phenyltridecane", "")
  CALL Species("C_{20}H_{34}",                        "1459-10-5", "JZALLXAUNPOCEU-UHFFFAOYSA-N", "tetradecylbenzene", "1-phenyltetradecane")
  CALL Species("C_{20}H_{34}",                        "4534-59-2", "GDFUGKICRHMMOT-UHFFFAOYSA-N", "2-phenyltetradecane", "")
  CALL Species("C_{20}H_{34}",                        "4534-58-1", "ILOABSZOHZMWLD-UHFFFAOYSA-N", "3-phenyltetradecane", "")
  CALL Species("C_{20}H_{34}",                        "4534-57-0", "YXLRHYCRTUAPMC-UHFFFAOYSA-N", "4-phenyltetradecane", "")
  CALL Species("C_{20}H_{34}",                        "4534-56-9", "RAWFVRXBPRHWLJ-UHFFFAOYSA-N", "5-phenyltetradecane", "")
  CALL Species("C_{20}H_{34}",                        "4534-55-8", "MTDIFFBFSKQPIA-UHFFFAOYSA-N", "6-phenyltetradecane", "")
  !CALL Species("C_{20}H_{34}",                        "4534-54-7", "ALMICIFPGWBKNR-UHFFFAOYSA-N", "7-phenyltetradecane", "")
  CALL Species("C_{21}H_{36}",                        "2131-18-2", "JIRNEODMTPGRGV-UHFFFAOYSA-N", "pentadecylbenzene", "")
  ! unsaturated substituents:
  CALL Species("C_8H_8",                               "100-42-5", "PPBRXRYQALVLMV-UHFFFAOYSA-N", "ethenylbenzene", "styrene")
  CALL Species("C_9H_{10}",                            "873-66-5", "QROGIFZRVHSFLM-QHHAFSJGSA-N", "($E$)-1-propenylbenzene", "")
  CALL Species("C_9H_{10}",                            "637-50-3", "QROGIFZRVHSFLM-UHFFFAOYSA-N", "1-propenylbenzene", "")
  CALL Species("C_9H_{10}",                            "300-57-2", "HJWLCRVIBGQPNF-UHFFFAOYSA-N", "2-propenylbenzene", "allylbenzene")
  CALL Species("C_9H_{10}",                            "100-80-1", "JZHGRUMIRATHIU-UHFFFAOYSA-N", "1-ethenyl-3-methylbenzene", "$m$-methylstyrene")
  CALL Species("C_9H_{10}",                            "622-97-9", "JLBJTVDPSNHSKJ-UHFFFAOYSA-N", "1-ethenyl-4-methylbenzene", "$p$-methylstyrene")
  CALL Species("C_9H_{10}",                             "98-83-9", "XYLMUPLGERFSHI-UHFFFAOYSA-N", "(1-methylethenyl)-benzene", "$\alpha$-methyl styrene")
  CALL Species("C_8H_6",                               "536-74-3", "UEXCJVNBTNXOEH-UHFFFAOYSA-N", "phenylacetylene", "")
  CALL Species("C_{18}H_{20}",                        "6144-04-3", "ATSCZMXECPRPEA-BUHFOSPRSA-N", "$\alpha$-methylstyrene dimer", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Terpenes and terpenoids")
  ! monoterpenes (C10):
  CALL Species("C_{10}H_{20}",                          "99-82-1", "CFJYNSNXFXLKNS-UHFFFAOYSA-N", "1-methyl-4-(1-methylethyl)-cyclohexane", "$p$-menthane")
  CALL Species("C_{10}H_{16}",                          "80-56-8", "GRWFGVWFFZKLTI-UHFFFAOYSA-N", "$\alpha$-pinene", "")
  CALL Species("C_{10}H_{16}",                         "127-91-3", "WTARULDDTDQWMU-UHFFFAOYSA-N", "$\beta$-pinene", "")
  CALL Species("C_{10}H_{16}",                          "99-86-5", "YHQGMYUVUMAZJR-UHFFFAOYSA-N", "1-methyl-4-(1-methylethyl)-1,3-cyclohexadiene", "$\alpha$-terpinene")
  CALL Species("C_{10}H_{16}",                          "99-85-4", "YKFLAYDHMOASIY-UHFFFAOYSA-N", "1-methyl-4-(1-methylethyl)-1,4-cyclohexadiene", "$\gamma$-terpinene")
  CALL Species("C_{10}H_{16}",                         "138-86-3", "XMGQYMWWDOXHJM-UHFFFAOYSA-N", "1-methyl-4-(1-methylethenyl)-cyclohexene", "limonene")
  CALL Species("C_{10}H_{16}",                        "5989-27-5", "XMGQYMWWDOXHJM-SNVBAGLBSA-N", "($R$)-1-methyl-4-(1-methylethenyl)-cyclohexene", "$R$-(+)-limonene; $D$-limonene")
  CALL Species("C_{10}H_{16}",                        "5989-54-8", "XMGQYMWWDOXHJM-JTQLQIEISA-N", "($S$)-1-methyl-4-(1-methylethenyl)-cyclohexene", "$S$-(-)-limonene")
  CALL Species("C_{10}H_{16}",                       "13466-78-9", "BQOFWKZOCNGFEC-UHFFFAOYSA-N", "3,7,7-trimethyl-bicyclo[4.1.0]hept-3-ene", "3-carene")
  CALL Species("C_{10}H_{16}",                         "123-35-3", "UAHWPYUMFXYFJY-UHFFFAOYSA-N", "7-methyl-3-methylene-1,6-octadiene", "myrcene")
  CALL Species("C_{10}H_{16}",                         "586-62-9", "MOYAFQVGZZPNRA-UHFFFAOYSA-N", "1-methyl-4-(1-methylethylidene)-cyclohexene", "$\alpha$-terpinolene")
  CALL Species("C_{10}H_{16}",                          "99-83-2", "OGLDWXZKYODSOB-UHFFFAOYSA-N", "$\alpha$-phellandrene", "")
  CALL Species("C_{10}H_{16}",                        "4221-98-1", "OGLDWXZKYODSOB-SNVBAGLBSA-N", "($R$)-($-$)-$\alpha$-phellandrene", "")
  CALL Species("C_{10}H_{16}",                         "555-10-2", "LFJQCDVYDGGFCH-UHFFFAOYSA-N", "$\beta$-phellandrene", "")
  CALL Species("C_{10}H_{16}",                       "13877-91-3", "IHPKGUQCSIINRJ-UHFFFAOYSA-N", "3,7-dimethyl-1,3,6-octatriene", "$\beta$-ocimene")
  CALL Species("C_{10}H_{16}",                        "3338-55-4", "IHPKGUQCSIINRJ-NTMALXAHSA-N", "($Z$)-3,7-dimethyl-1,3,6-octatriene", "{\it cis}-$\beta$-ocimene")
  CALL Species("C_{10}H_{16}",                        "3779-61-1", "IHPKGUQCSIINRJ-CSKARUKUSA-N", "($E$)-3,7-dimethyl-1,3,6-octatriene", "{\it trans}-$\beta$-ocimene")
  CALL Species("C_{10}H_{16}",                          "79-92-5", "CRPUJAZIXJMDBK-UHFFFAOYSA-N", "2,2-dimethyl-3-methylene-bicyclo[2.2.1]heptane", "camphene")
  CALL Species("C_{10}H_{16}",                        "3387-41-5", "NDVASEGYNIMXJL-UHFFFAOYSA-N", "4-methylene-1-(1-methylethyl)-bicyclo[3.1.0]hexane", "sabinene")
  CALL Species("C_{10}H_{16}",                         "281-23-2", "ORILYTVJVMAKLC-UHFFFAOYSA-N", "tricyclo[3.3.1.1(3,7)]decane", "adamantane")
  ! sesquiterpenes (C15):
  CALL Species("C_{15}H_{24}",                          "87-44-5", "NPNUFJAVOOONJE-IOMPXFEGSA-N", "$\beta$-caryophyllene", "")
  CALL Species("C_{15}H_{24}",                         "469-61-4", "IRAQOCYXUMOFCW-KYEXWDHISA-N", "$\alpha$-cedrene", "")
  CALL Species("C_{15}H_{24}",                         "502-61-4", "CXENHBSYCFFKJS-VDQVFBMKSA-N", "$\alpha$-farnesene", "")
  CALL Species("C_{15}H_{24}",                        "6753-98-6", "FAMPSKZZVDUYOS-HRGUGZIWSA-N", "$\alpha$-humulene", "")
  CALL Species("C_{15}H_{24}",                       "22567-17-5", "DUYRYUZIBGFLDD-UHFFFAOYSA-N", "$\gamma$-gurjunene", "")
  CALL Species("C_{15}H_{24}",                       "24959-83-9", "CGZBLYYTRIVVTD-UHFFFAOYSA-N", "isosativene", "")
  CALL Species("C_{15}H_{24}",                        "5989-08-2", "HICYDYJTCDBHMZ-UHFFFAOYSA-N", "$\alpha$-longipinene", "")
  CALL Species("C_{15}H_{24}",                        "4545-68-0", "ZCJQJJWNFDNQGZ-UHFFFAOYSA-N", "$\alpha$-neoclovene", "")
  CALL Species("C_{15}H_{24}",                       "56684-96-9", "BUDWHMNUSAOQBI-UHFFFAOYSA-N", "$\beta$-neoclovene", "")
  CALL Species("C_{15}H_{24}",                                 "", "ZYKFQRCKKGFKQR-UHFFFAOYSA-N", "$\gamma$-neoclovene", "")
  CALL Species("C_{15}H_{24}",                        "4630-07-3", "QEBNYNLSCGVZOH-UHFFFAOYSA-N", "valencene", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Polynuclear aromatics")
  ! separate rings:
  CALL Species("C_{18}H_{22}",                       "36876-13-8", "NUEUMFZLNOCRCQ-UHFFFAOYSA-N", "bis(1-methylethyl)-1,1'-biphenyl", "")
  CALL Species("C_{18}H_{22}",                        "1742-14-9", "NCSVCMFDHINRJE-UHFFFAOYSA-N", "1,1-bis(3,4-dimethylphenyl)ethane", "")
  CALL Species("C_{21}H_{20}",                      "100404-06-6", "SMOZJCPUZPHHKQ-UHFFFAOYSA-N", "1-benzyl-2-(2-methylbenzyl)benzene", "")
  CALL Species("C_{21}H_{20}",                       "56310-11-3", "INLIRICAUXXHSB-UHFFFAOYSA-N", "2,5-dibenzyltoluene", "")
  CALL Species("(C_6H_5)_2",                            "92-52-4", "ZUOUZKKEUPVFJK-UHFFFAOYSA-N", "biphenyl", "")
  CALL Species("C_{13}H_{12}",                         "643-58-3", "ALLIZEAXNXSFGD-UHFFFAOYSA-N", "2-methyl-1,1'-biphenyl", "") ! other CAS: 28652-72-4
  CALL Species("C_{13}H_{12}",                         "643-93-6", "NPDIDUXTRAITDE-UHFFFAOYSA-N", "3-methyl-1,1'-biphenyl", "")
  CALL Species("C_{13}H_{12}",                         "644-08-6", "ZZLCFHIKESPLTH-UHFFFAOYSA-N", "4-methyl-1,1'-biphenyl", "")
  CALL Species("C_{13}H_{12}",                         "101-81-5", "CZZYITDELCSZES-UHFFFAOYSA-N", "diphenylmethane", "1,1'-methylenebisbenzene")
  CALL Species("C_{14}H_{14}",                         "103-29-7", "QWUWMCYKGHVNAV-UHFFFAOYSA-N", "1,2-diphenylethane", "dibenzyl")
  CALL Species("C_{15}H_{16}",                       "20282-30-8", "LIWRTHVZRZXVFX-UHFFFAOYSA-N", "3-isopropyl-1,1'-biphenyl", "")
  CALL Species("C_{15}H_{16}",                        "7116-95-2", "KWSHGRJUSUJPQD-UHFFFAOYSA-N", "4-isopropyl-1,1'-biphenyl", "")
  CALL Species("C_{18}H_{14}",                          "84-15-1", "OIAQMFOKAXHPNH-UHFFFAOYSA-N", "$o$-terphenyl", "")
  CALL Species("C_{18}H_{14}",                          "92-06-8", "YJTKZCDBKVTVBY-UHFFFAOYSA-N", "$m$-terphenyl", "")
  CALL Species("C_{18}H_{14}",                          "92-94-4", "XJKSTNDFUHDPQJ-UHFFFAOYSA-N", "$p$-terphenyl", "")
  ! fused rings:
  ! C9:
  CALL Species("C_9H_8",                                "95-13-6", "YBYIRNPNPLQARY-UHFFFAOYSA-N", "indene", "")
  CALL Species("C_9H_{12}",                          "16219-75-3", "OJOWICOBYCXEKR-KRXBUXKQSA-N", "5-ethylidene-2-norbornene", "")
  CALL Species("C_9H_{10}",                            "496-11-7", "PQNFLJBBNBOBRQ-UHFFFAOYSA-N", "indane", "")
  ! C10:
  CALL Species("C_{10}H_8",                            "275-51-4", "CUFNKYGDVFVPHO-UHFFFAOYSA-N", "azulene", "")
  CALL Species("C_{10}H_8",                             "91-20-3", "UFWIBTONFRDIAS-UHFFFAOYSA-N", "naphthalene", "")
  CALL Species("C_{10}D_8",                           "1146-65-2", "UFWIBTONFRDIAS-PGRXLJNUSA-N", "naphthalene-d8", "")
  CALL Species("C_{10}H_7CH_3",                         "90-12-0", "QPUYECUOLPXSFR-UHFFFAOYSA-N", "1-methylnaphthalene", "")
  CALL Species("C_{10}D_7CD_3",                      "38072-94-5", "QPUYECUOLPXSFR-UZHHFJDZSA-N", "1-methylnaphthalene-d10", "")
  CALL Species("C_{10}H_7CH_3",                         "91-57-6", "QIMMUPPBPVKWKM-UHFFFAOYSA-N", "2-methylnaphthalene", "")
  CALL Species("C_{10}H_7C_2H_5",                     "1127-76-0", "ZMXIYERNXPIYFR-UHFFFAOYSA-N", "1-ethylnaphthalene", "")
  CALL Species("C_{10}H_7C_2H_5",                      "939-27-5", "RJTJVVYSTUQWNI-UHFFFAOYSA-N", "2-ethylnaphthalene", "")
  CALL Species("C_{10}H_{12}",                         "119-64-2", "CXWXQJXEFPUFDZ-UHFFFAOYSA-N", "1,2,3,4-tetrahydronaphthalene", "tetralin")
  ! C12:
  CALL Species("C_{12}H_{12}",                         "573-98-8", "QNLZIZAQLLYXTC-UHFFFAOYSA-N", "1,2-dimethylnaphthalene", "")
  CALL Species("C_{12}H_{12}",                         "575-41-7", "QHJMFSMPSZREIF-UHFFFAOYSA-N", "1,3-dimethylnaphthalene", "")
  CALL Species("C_{12}H_{12}",                         "571-58-4", "APQSQLNWAIULLK-UHFFFAOYSA-N", "1,4-dimethylnaphthalene", "")
  CALL Species("C_{12}H_{12}",                         "571-61-9", "SDDBCEWUYXVGCQ-UHFFFAOYSA-N", "1,5-dimethylnaphthalene", "")
  CALL Species("C_{12}H_{12}",                         "575-43-9", "CBMXCNPQDUJNHT-UHFFFAOYSA-N", "1,6-dimethylnaphthalene", "")
  CALL Species("C_{12}H_{12}",                         "575-37-1", "SPUWFVKLHHEKGV-UHFFFAOYSA-N", "1,7-dimethylnaphthalene", "")
  CALL Species("C_{12}H_{12}",                         "569-41-5", "XAABPYINPXYOLM-UHFFFAOYSA-N", "1,8-dimethylnaphthalene", "")
  CALL Species("C_{12}H_{12}",                         "581-40-8", "WWGUMAYGTYQSGA-UHFFFAOYSA-N", "2,3-dimethylnaphthalene", "")
  CALL Species("C_{12}H_{12}",                         "581-42-0", "YGYNBBAUIYTWBF-UHFFFAOYSA-N", "2,6-dimethylnaphthalene", "")
  CALL Species("C_{12}H_{12}",                         "582-16-1", "LRQYSMQNJLZKPS-UHFFFAOYSA-N", "2,7-dimethylnaphthalene", "")
  CALL Species("C_{12}H_{10}",                          "83-32-9", "CWRYPZZKDGJXCA-UHFFFAOYSA-N", "acenaphthene", "")
  CALL Species("C_{12}H_8",                            "208-96-8", "HXGDTGSAIMULJN-UHFFFAOYSA-N", "acenaphthylene", "")
  ! C13:
  CALL Species("C_{13}H_{14}",                        "2717-42-2", "JCNGSJUYPCVGAM-UHFFFAOYSA-N", "1,2,4-trimethylnaphthalene", "")
  CALL Species("C_{13}H_{14}",                        "2131-38-6", "HXDVFWJRDVUZFT-UHFFFAOYSA-N", "1,3,7-trimethylnaphthalene", "")
  CALL Species("C_{13}H_{14}",                       "17057-91-9", "XYTKCJHHXQVFCK-UHFFFAOYSA-N", "1,3,8-trimethylnaphthalene", "")
  CALL Species("C_{13}H_{14}",                        "2131-41-1", "FSAWRQYDMHSDRN-UHFFFAOYSA-N", "1,4,5-trimethylnaphthalene", "")
  CALL Species("C_{13}H_{14}",                        "2131-42-2", "VGKRZAKNKJAKDN-UHFFFAOYSA-N", "1,4,6-trimethylnaphthalene", "")
  CALL Species("C_{13}H_{14}",                         "829-26-5", "UNBZRJCHIWTUHB-UHFFFAOYSA-N", "2,3,6-trimethylnaphthalene", "")
  CALL Species("C_{13}H_{14}",                        "6158-45-8", "PMPBFICDXLLSRM-UHFFFAOYSA-N", "1-(1-methylethyl)naphthalene", "1-isopropylnaphthalene")
  CALL Species("C_{13}H_{14}",                        "2027-17-0", "TVYVQNHYIHAJTD-UHFFFAOYSA-N", "2-(1-methylethyl)naphthalene", "2-isopropylnaphthalene")
  CALL Species("C_{13}H_{14}",                        "7372-86-3", "ZOYUJOHRFWIQTH-UHFFFAOYSA-N", "2-ethyl-6-methylnaphthalene", "")
  CALL Species("C_{13}H_{10}",                          "86-73-7", "NIHNNTQXNPWCJQ-UHFFFAOYSA-N", "2,3-benzindene", "fluorene")
  ! C14:
  CALL Species("C_{14}H_{12}",                        "1730-37-6", "GKEUODMJRFDLJY-UHFFFAOYSA-N", "1-methyl-9H-fluorene", "")
  CALL Species("C_{14}H_{12}",                        "1430-97-3", "RKJHJMAZNPASHY-UHFFFAOYSA-N", "2-methyl-9H-fluorene", "")
  CALL Species("C_{14}H_{12}",                        "2523-37-7", "ZVEJRZRAUYJYCO-UHFFFAOYSA-N", "9-methyl-9H-fluorene", "")
  CALL Species("C_{14}H_{16}",                        "2131-43-3", "ONIJFQFZCKJNDH-UHFFFAOYSA-N", "1,2,5,6-tetramethylnaphthalene", "")
  CALL Species("C_{14}H_{16}",                       "13764-18-6", "VPSPONOBLZCLIU-UHFFFAOYSA-N", "1,4,6,7-tetramethylnaphthalene", "")
  CALL Species("C_{14}H_{16}",                         "490-65-3", "UZVVHZJKXPCUQU-UHFFFAOYSA-N", "1-methyl-7-(1-methylethyl)-naphthalene", "")
  CALL Species("C_{14}H_{12}",                         "103-30-0", "PJANXHGTPQOBST-VAWYXSNFSA-N", "($E$)-stilbene", "{\it trans}-1,2-diphenylethene")
  CALL Species("C_{14}H_{10}",                          "85-01-8", "YNPNZTXNASCQKK-UHFFFAOYSA-N", "phenanthrene", "")
  CALL Species("C_{14}H_{12}",                         "776-35-2", "XXPBFNVKTVJZKF-UHFFFAOYSA-N", "9,10-dihydrophenanthrene", "")
  CALL Species("C_{14}H_{10}",                         "120-12-7", "MWPLVEDNUUSJAV-UHFFFAOYSA-N", "anthracene", "")
  CALL Species("C_{14}H_{12}",                         "613-31-0", "WPDAVTSOEQEGMS-UHFFFAOYSA-N", "9,10-dihydroanthracene", "")
  CALL Species("C_{14}H_{20}",                          "81-03-8", "NNXHDILUOAXSPU-UHFFFAOYSA-N", "2,3-dihydro-1,1,3,3,5-pentamethyl-1H-indene", "")
  ! C15:
  CALL Species("C_{15}H_{12}",                         "610-48-0", "KZNJSFHJUQDYHE-UHFFFAOYSA-N", "1-methylanthracene", "")
  CALL Species("C_{15}H_{12}",                         "613-12-7", "GYMFBYTZOGMSQJ-UHFFFAOYSA-N", "2-methylanthracene", "")
  CALL Species("C_{15}H_{12}",                         "779-02-2", "CPGPAVAKSZHMBP-UHFFFAOYSA-N", "9-methylanthracene", "")
  CALL Species("C_{15}H_{12}",                         "883-20-5", "DALBHIYZSZZWBS-UHFFFAOYSA-N", "9-methyl-phenanthrene", "")
  CALL Species("C_{15}H_{14}",                        "2294-82-8", "QBBCCEYJCKGWIK-UHFFFAOYSA-N", "9-ethylfluorene", "")
  CALL Species("C_{15}H_{14}",                         "442-66-0", "NHPVHXMCRWRSNW-UHFFFAOYSA-N", "1,7-dimethylfluorene", "")
  CALL Species("C_{15}H_{12}",                         "832-69-9", "DOWJXOHBNXRUOD-UHFFFAOYSA-N", "1-methylphenanthrene", "")
  ! C16:
  CALL Species("C_{16}H_{20}",                       "57122-16-4", "JDBFIFNXALGSOF-UHFFFAOYSA-N", "1,3-diisopropylnaphthalene", "")
  CALL Species("C_{16}H_{20}",                       "27351-96-8", "GFZWCYSEPXDDRI-UHFFFAOYSA-N", "1,5-diisopropylnaphthalene", "")
  CALL Species("C_{16}H_{20}",                       "94133-80-9", "XNSUVOOWWSAQMZ-UHFFFAOYSA-N", "1,7-diisopropylnaphthalene", "")
  CALL Species("C_{16}H_{20}",                       "24157-81-1", "GWLLTEXUIOFAFE-UHFFFAOYSA-N", "2,6-diisopropylnaphthalene", "")
  CALL Species("C_{16}H_{20}",                       "40458-98-8", "YGDMAJYAQCDTNG-UHFFFAOYSA-N", "2,7-diisopropylnaphthalene", "")
  CALL Species("C_{16}H_{14}",                       "52251-71-5", "ZXAGXLDEMUNQSH-UHFFFAOYSA-N", "2-ethylanthracene", "")
  CALL Species("C_{16}H_{14}",                       "16664-45-2", "UJUUQKOWULNFNG-UHFFFAOYSA-N", "1,3-dimethylphenanthrene", "")
  CALL Species("C_{16}H_{14}",                        "1576-67-6", "OMIBPZBOAJFEJS-UHFFFAOYSA-N", "3,6-dimethylphenanthrene", "")
  CALL Species("C_{16}H_{14}",                         "613-06-9", "OGVRJXPGSVLDRD-UHFFFAOYSA-N", "2,3-dimethylanthracene", "")
  CALL Species("C_{16}H_{14}",                         "781-43-1", "JTGMTYWYUZDRBK-UHFFFAOYSA-N", "9,10-dimethylanthracene", "")
  CALL Species("C_{16}H_{16}",                        "4037-45-0", "ZTBWLZNGULENBG-UHFFFAOYSA-N", "9-propyl-9H-fluorene", "")
  CALL Species("C_{16}H_{10}",                         "129-00-0", "BBEAQIROQSPTKN-UHFFFAOYSA-N", "pyrene", "")
  CALL Species("C_{16}H_{10}",                         "206-44-0", "GVEPBJHOBDJJJI-UHFFFAOYSA-N", "benzo[$jk$]fluorene", "fluoranthene")
  CALL Species("C_{16}H_{24}",                        "2084-69-7", "JIVANURAWUCQIG-UHFFFAOYSA-N", "1,2,3,4-tetrahydro-1,1,3,4,4,6-hexamethylnaphthalene", "")
  CALL Species("C_{16}H_{16}",                        "1633-22-3", "OOLUVSIJOMLOCB-UHFFFAOYSA-N", "[2.2]paracyclophane", "")
  ! C17:
  CALL Species("C_{17}H_{12}",                         "238-84-6", "HKMTVMBEALTRRR-UHFFFAOYSA-N", "benzo[$a$]fluorene", "")
  CALL Species("C_{17}H_{12}",                        "2381-21-7", "KBSPJIWZDWBDGM-UHFFFAOYSA-N", "1-methylpyrene", "")
  CALL Species("C_{17}H_{12}",                        "3442-78-2", "VIRFPLJXRDHVEI-UHFFFAOYSA-N", "2-methylpyrene", "")
  CALL Species("C_{17}H_{12}",                       "30997-39-8", "VVRCMNWZFPMXQZ-UHFFFAOYSA-N", "2-methylfluoranthene", "") ! other CAS: 33543-31-6
  CALL Species("C_{17}H_{16}",                       "17024-03-2", "PIWHTUVAMGYSIC-UHFFFAOYSA-N", "9-n-propylphenanthrene", "")
  CALL Species("C_{17}H_{16}",                       "30436-55-6", "MYWOJODOMFBVCB-UHFFFAOYSA-N", "1,2,6-trimethylphenanthrene", "")
  CALL Species("C_{17}H_{18}",                        "3952-42-9", "RBDADLSAYYPJAN-UHFFFAOYSA-N", "9-butyl-9H-fluorene", "")
  CALL Species("C_{17}H_{12}",                         "243-17-4", "HAPOJKSPCGLOOD-UHFFFAOYSA-N", "11H-benzo[$b$]fluorene", "")
  ! C18:
  CALL Species("C_{18}H_{14}",                       "15679-24-0", "GSKHIRFMTJUBSM-UHFFFAOYSA-N", "2,7-dimethylpyrene", "")
  CALL Species("C_{18}H_{12}",                         "218-01-9", "WDECIBYCCFPHNR-UHFFFAOYSA-N", "chrysene", "")
  CALL Species("C_{18}H_{12}",                          "92-24-0", "IFLREYGFSNHWGE-UHFFFAOYSA-N", "naphthacene", "2,3-benzanthracene")
  CALL Species("C_{18}H_{12}",                         "217-59-4", "SLGBZMMZGDRARJ-UHFFFAOYSA-N", "triphenylene", "benzo[$l$]phenanthrene")
  CALL Species("C_{18}H_{12}",                          "56-55-3", "DXBHBZVCASKNBY-UHFFFAOYSA-N", "benz[$a$]anthracene", "")
  CALL Species("C_{18}H_{14}",                       "15679-25-1", "MCZKPUHBCIHFBZ-UHFFFAOYSA-N", "4,5-dimethylpyrene", "")
  CALL Species("C_{18}H_{14}",                       "20496-16-6", "JXUOLJXLPIIRTP-UHFFFAOYSA-N", "3-ethylfluoranthene", "")
  CALL Species("C_{18}H_{18}",                      "204256-39-3", "XFJWRZAPBOJONX-UHFFFAOYSA-N", "1,2,6,9-tetramethylphenanthrene", "")
  CALL Species("C_{18}H_{18}",                                 "", "MDZYWNWRMJIOPN-UHFFFAOYSA-N", "1,9-dimethyl-5-ethylphenanthrene", "")
  CALL Species("C_{18}H_{18}",                                 "", "NFFCDFHAMXFVEK-UHFFFAOYSA-N", "1,9-dimethyl-7-ethylphenanthrene", "")
  CALL Species("C_{18}H_{20}",                       "60466-61-7", "TXOHWLOHKUPUKO-UHFFFAOYSA-N", "1,2,3,4-tetrahydro-5-(1-phenylethyl)-naphthalene", "")
  ! C19:
  CALL Species("C_{19}H_{14}",                        "3351-30-2", "BLVHWJCLSMYFMT-UHFFFAOYSA-N", "4-methylchrysene", "")
  CALL Species("C_{19}H_{14}",                        "3697-24-3", "GOHBXWHNJHENRX-UHFFFAOYSA-N", "5-methylchrysene", "")
  CALL Species("C_{19}H_{14}",                        "1705-85-7", "ASVDRLYVNFOSCI-UHFFFAOYSA-N", "6-methylchrysene", "")
  CALL Species("C_{19}H_{16}",                       "42211-33-6", "HIOZPFGGCXEPAP-UHFFFAOYSA-N", "1-propylpyrene", "")
  CALL Species("C_{19}H_{14}",                        "2541-69-7", "DIIFUCUPDHMNIV-UHFFFAOYSA-N", "7-methylbenz[$a$]anthracene", "")
  CALL Species("C_{19}H_{14}",                        "2381-15-9", "WUMGYHICFXGLAB-UHFFFAOYSA-N", "10-methylbenz[$a$]anthracene", "")
  CALL Species("C_{19}H_{14}",                        "2422-79-9", "ACYOLKMEHHTLAB-UHFFFAOYSA-N", "12-methylbenz[$a$]anthracene", "")
  ! C20:
  CALL Species("C_{20}H_{16}",                        "2732-58-3", "ZJSYTTGSPQNXKT-UHFFFAOYSA-N", "6-ethylchrysene", "")
  CALL Species("C_{20}H_{16}",                         "316-51-8", "DBPDWIZRMPUWRH-UHFFFAOYSA-N", "3,9-dimethylbenz[$a$]anthracene", "")
  CALL Species("C_{20}H_{18}",                       "35980-18-8", "UFOTZLIYHMGVAV-UHFFFAOYSA-N", "1-butylpyrene", "")
  CALL Species("C_{20}H_{16}",                          "57-97-6", "ARSRBNBHOADGJU-UHFFFAOYSA-N", "7,12-dimethyl-benz[$a$]anthracene", "")
  CALL Species("C_{20}H_{16}",                       "58429-99-5", "GKVUDAZMZLMNJQ-UHFFFAOYSA-N", "9,10-dimethyl-benz[$a$]anthracene", "")
  CALL Species("C_{20}H_{12}",                         "205-99-2", "FTOVXSOBNPWTSH-UHFFFAOYSA-N", "benzo[$b$]fluoranthene", "")
  CALL Species("C_{20}H_{12}",                         "207-08-9", "HAXBIWFMXWRORI-UHFFFAOYSA-N", "benzo[$k$]fluoranthene", "")
  CALL Species("C_{20}H_{12}",                          "50-32-8", "FMMWHPNWAFZXNH-UHFFFAOYSA-N", "benzo[$a$]pyrene", "benz[$a$]pyrene")
  CALL Species("C_{20}H_{12}",                         "192-97-2", "TXVHTIQJNYSSKO-UHFFFAOYSA-N", "benzo[$e$]pyrene", "")
  CALL Species("C_{20}H_{12}",                         "198-55-0", "CSHWQDPOILHKBI-UHFFFAOYSA-N", "perylene", "dibenz[$de,kl$]anthracene")
  CALL Species("C_{20}H_{12}",                         "203-33-8", "OQDXASJSCOTNQS-UHFFFAOYSA-N", "1,2-benzfluoranthene", "")
  CALL Species("C_{20}H_{12}",                         "205-82-3", "KHNYNFUTFKJLDD-UHFFFAOYSA-N", "benzo[$j$]fluoranthene", "")
  ! C21:
  CALL Species("C_{21}H_{18}",                     "1586755-28-3", "UGLPREDFUVNEMJ-UHFFFAOYSA-N", "1,3,6-trimethylchrysene", "")
  CALL Species("C_{21}H_{18}",                        "6910-41-4", "PGBSMHMYMMBHFF-UHFFFAOYSA-N", "6-propylchrysene", "")
  CALL Species("C_{21}H_{16}",                          "56-49-5", "PPQNQXQZIWHJRB-UHFFFAOYSA-N", "20-methylcholanthrene", "")
  ! C22:
  CALL Species("C_{22}H_{20}",                        "6901-71-9", "XJQKMLMKECCWEO-UHFFFAOYSA-N", "6-butylchrysene", "")
  CALL Species("C_{22}H_{14}",                          "53-70-3", "LHRCREOYAASXPZ-UHFFFAOYSA-N", "dibenz[$a,h$]anthracene", "")
  CALL Species("C_{22}H_{12}",                         "193-39-5", "SXQBHARYMNFBPS-UHFFFAOYSA-N", "indeno[1,2,3-$cd$]pyrene", "")
  CALL Species("C_{22}H_{12}",                         "191-24-2", "GYFAGKUZYNFMBN-UHFFFAOYSA-N", "benzo[$ghi$]perylene", "")
  CALL Species("C_{22}H_{14}",                         "215-58-7", "RAASUWZPTOJQAY-UHFFFAOYSA-N", "benzo[$b$]triphenylene", "dibenz[$a,c$]anthracene")
  CALL Species("C_{22}H_{14}",                         "224-41-9", "KLIHYVJAYWCEDM-UHFFFAOYSA-N", "dibenz[$a,j$]anthracene", "")
  CALL Species("C_{22}H_{14}",                         "213-46-7", "GBROPGWFBFCKAG-UHFFFAOYSA-N", "picene", "")
  CALL Species("C_{22}H_{14}",                         "194-69-4", "YZWGEMSQAMDWEM-UHFFFAOYSA-N", "benzo[$c$]chrysene", "")
  CALL Species("C_{22}H_{14}",                         "196-78-1", "JZOIZKBKSZMVRV-UHFFFAOYSA-N", "benzo[$g$]chrysene", "")
  ! C24:
  CALL Species("C_{24}H_{14}",                         "192-65-4", "KGHMWBNEMFNJFZ-UHFFFAOYSA-N", "dibenzo[$a,e$]pyrene", "")
  CALL Species("C_{24}H_{14}",                         "189-64-0", "RXUSYFJGDZFVND-UHFFFAOYSA-N", "dibenzo[$a,h$]pyrene", "")
  CALL Species("C_{24}H_{14}",                         "189-55-9", "TUGYIJVAYAHHHM-UHFFFAOYSA-N", "dibenzo[$a,i$]pyrene", "")
  CALL Species("C_{24}H_{12}",                         "191-07-1", "VPUGDVKSAQVFFS-UHFFFAOYSA-N", "coronene", "")
  CALL Species("C_{24}H_{14}",                        "5385-75-1", "JHOWUOKQHJHGMU-UHFFFAOYSA-N", "dibenz[$a,e$]aceanthrylene", "")
  ! C26:
  CALL Species("C_{26}H_{16}",                         "217-54-9", "DHCSBRKYHMINPB-UHFFFAOYSA-N", "dibenzo[$b,k$]chrysene", "")
  !***************************************************************************
  CALL SpeciesGroup(1,"Organic species with oxygen (\chem{O})")
  !***************************************************************************
  CALL SpeciesGroup(2,"Carbon oxides")
  CALL Species("CO",                                   "630-08-0", "UGFAIRIUMAVXCW-UHFFFAOYSA-N", "carbon monoxide", "")
  CALL Species("CO_2",                                 "124-38-9", "CURLTUGMZLYLDI-UHFFFAOYSA-N", "carbon dioxide", "")
  CALL Species("C_3O_2",                               "504-64-3", "GNEVIACKFGQMHB-UHFFFAOYSA-N", "carbon suboxide", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Alcohols (\chem{ROH})")
  ! aliphatic, saturated:
  ! C1:
  CALL Species("CH_3OH",                                "67-56-1", "OKKJLVBELUTLKV-UHFFFAOYSA-N", "methanol", "")
  ! C2:
  CALL Species("C_2H_5OH",                              "64-17-5", "LFQSCWFLJHTTHZ-UHFFFAOYSA-N", "ethanol", "")
  ! C3:
  CALL Species("C_3H_7OH",                              "71-23-8", "BDERNNFJNOPAEC-UHFFFAOYSA-N", "1-propanol", "")
  CALL Species("C_3H_7OH",                              "67-63-0", "KFZMGEQAYNKOFK-UHFFFAOYSA-N", "2-propanol", "isopropanol")
  CALL Species("C_3H_6O_2",                            "556-52-5", "CTKINSOISVBQLD-UHFFFAOYSA-N", "glycidol", "")
  ! C4:
  CALL Species("C_4H_9OH",                              "71-36-3", "LRHPLDYGYMQRHN-UHFFFAOYSA-N", "1-butanol", "")
  CALL Species("C_4H_{10}O",                            "78-92-2", "BTANRVKWQNVYAZ-UHFFFAOYSA-N", "2-butanol", "{\it sec}-butanol")
  CALL Species("C_4H_{10}O",                            "78-83-1", "ZXEKIIBDNHEJCQ-UHFFFAOYSA-N", "2-methyl-1-propanol", "isobutanol")
  CALL Species("C_4H_{10}O",                            "75-65-0", "DKGAVHZHDRPRBM-UHFFFAOYSA-N", "2-methyl-2-propanol", "{\it tert}-butanol")
  ! C5:
  CALL Species("C_5H_{11}OH",                           "71-41-0", "AMQJEAYHLZJPGS-UHFFFAOYSA-N", "1-pentanol", "amyl alcohol")
  CALL Species("C_5H_{12}O",                          "6032-29-7", "JYVLIDXNZAXMDK-UHFFFAOYSA-N", "2-pentanol", "{\it sec}-pentanol")
  CALL Species("C_5H_{12}O",                           "584-02-1", "AQIXEPGDORPWBJ-UHFFFAOYSA-N", "3-pentanol", "")
  CALL Species("C_5H_{12}O",                           "137-32-6", "QPRQEDXDYOZYLA-UHFFFAOYSA-N", "2-methyl-1-butanol", "")
  CALL Species("C_5H_{12}O",                          "1565-80-6", "QPRQEDXDYOZYLA-RXMQYKEDSA-N", "($S$)-2-methyl-1-butanol", "")
  CALL Species("C_5H_{12}O",                           "123-51-3", "PHTQWCKDNZKARW-UHFFFAOYSA-N", "3-methyl-1-butanol", "isopentanol")
  CALL Species("C_5H_{12}O",                            "75-85-4", "MSXVEPNJUHWQHW-UHFFFAOYSA-N", "2-methyl-2-butanol", "{\it tert}-pentanol")
  CALL Species("C_5H_{12}O",                           "598-75-4", "MXLMTQWGSQIYOW-UHFFFAOYSA-N", "3-methyl-2-butanol", "")
  CALL Species("C_5H_{12}O",                            "75-84-3", "KPSSIOMAKSHJJG-UHFFFAOYSA-N", "2,2-dimethyl-1-propanol", "")
  ! C6:
  CALL Species("C_6H_{14}O",                           "111-27-3", "ZSIAUFGUXNUGDI-UHFFFAOYSA-N", "1-hexanol", "")
  CALL Species("C_6H_{14}O",                           "626-93-7", "QNVRIHYSUZMSGM-UHFFFAOYSA-N", "2-hexanol", "")
  CALL Species("C_6H_{14}O",                           "623-37-0", "ZOCHHNOQQHDWHG-UHFFFAOYSA-N", "3-hexanol", "")
  CALL Species("C_6H_{14}O",                           "105-30-6", "PFNHSEQQEPMLNI-UHFFFAOYSA-N", "2-methyl-1-pentanol", "")
  CALL Species("C_6H_{14}O",                           "589-35-5", "IWTBVKIGCDZRPL-UHFFFAOYSA-N", "3-methyl-1-pentanol", "")
  CALL Species("C_6H_{14}O",                           "590-36-3", "WFRBDWRZVBPBDO-UHFFFAOYSA-N", "2-methyl-2-pentanol", "")
  CALL Species("C_6H_{14}O",                           "565-60-6", "ZXNBBWHRUSXUFZ-UHFFFAOYSA-N", "3-methyl-2-pentanol", "")
  CALL Species("C_6H_{14}O",                           "108-11-2", "WVYWICLMDOOCFB-UHFFFAOYSA-N", "4-methyl-2-pentanol", "")
  CALL Species("C_6H_{14}O",                           "565-67-3", "ISTJMQSHILQAEC-UHFFFAOYSA-N", "2-methyl-3-pentanol", "")
  CALL Species("C_6H_{14}O",                            "77-74-7", "FRDAATYAJDYRNW-UHFFFAOYSA-N", "3-methyl-3-pentanol", "")
  CALL Species("C_6H_{14}O",                            "97-95-0", "TZYRSLHNPKPEFV-UHFFFAOYSA-N", "2-ethyl-1-butanol", "")
  CALL Species("C_6H_{14}O",                          "1185-33-7", "XRMVWAKMXZNZIL-UHFFFAOYSA-N", "2,2-dimethyl-1-butanol", "")
  CALL Species("C_6H_{14}O",                         "19550-30-2", "SXSWMAUXEHKFGX-UHFFFAOYSA-N", "2,3-dimethyl-1-butanol", "") ! other CAS: 20281-85-0, 54206-54-1, 79956-98-2
  CALL Species("C_6H_{14}O",                           "594-60-5", "IKECULIHBUCAKR-UHFFFAOYSA-N", "2,3-dimethyl-2-butanol", "")
  CALL Species("C_6H_{14}O",                           "624-95-3", "DUXCSEISVMREAX-UHFFFAOYSA-N", "3,3-dimethyl-1-butanol", "")
  CALL Species("C_6H_{14}O",                           "464-07-3", "DFOXKPDFWGNLJU-UHFFFAOYSA-N", "3,3-dimethyl-2-butanol", "")
  CALL Species("C_6H_{14}O_3",                          "77-99-6", "ZJCCRDAZUWHFQH-UHFFFAOYSA-N", "2,2-bis(hydroxymethyl)-1-butanol", "")
  ! C7:
  CALL Species("C_7H_{16}O",                           "111-70-6", "BBMCTIGTTCKYKF-UHFFFAOYSA-N", "1-heptanol", "")
  CALL Species("C_7H_{16}O",                           "543-49-7", "CETWDUZRCINIHU-UHFFFAOYSA-N", "2-heptanol", "")
  CALL Species("C_7H_{16}O",                           "589-82-2", "RZKSECIXORKHQS-UHFFFAOYSA-N", "3-heptanol", "")
  CALL Species("C_7H_{16}O",                           "589-55-9", "YVBCULSIZWMTFY-UHFFFAOYSA-N", "4-heptanol", "")
  CALL Species("C_7H_{16}O",                           "624-22-6", "LCFKURIJYIJNRU-UHFFFAOYSA-N", "2-methyl-1-hexanol", "")
  CALL Species("C_7H_{16}O",                         "13231-81-7", "YGZVAQICDGBHMD-UHFFFAOYSA-N", "3-methyl-1-hexanol", "")
  CALL Species("C_7H_{16}O",                           "818-49-5", "YNPVNLWKVZZBTM-UHFFFAOYSA-N", "4-methyl-1-hexanol", "")
  CALL Species("C_7H_{16}O",                           "627-98-5", "ZVHAANQOQZVVFD-UHFFFAOYSA-N", "5-methyl-1-hexanol", "")
  CALL Species("C_7H_{16}O",                           "625-23-0", "KRIMXCDMVRMCTC-UHFFFAOYSA-N", "2-methyl-2-hexanol", "")
  CALL Species("C_7H_{16}O",                          "2313-65-7", "IRLSKJITMWPWNY-UHFFFAOYSA-N", "3-methyl-2-hexanol", "")
  CALL Species("C_7H_{16}O",                          "2313-61-3", "KZUBXUKRWLMPIO-UHFFFAOYSA-N", "4-methyl-2-hexanol", "")
  CALL Species("C_7H_{16}O",                           "627-59-8", "ZDVJGWXFXGJSIU-UHFFFAOYSA-N", "5-methyl-2-hexanol", "")
  CALL Species("C_7H_{16}O",                           "617-29-8", "RGRUUTLDBCWYBL-UHFFFAOYSA-N", "2-methyl-3-hexanol", "")
  CALL Species("C_7H_{16}O",                           "597-96-6", "KYWJZCSJMOILIZ-UHFFFAOYSA-N", "3-methyl-3-hexanol", "")
  CALL Species("C_7H_{16}O",                           "615-29-2", "NZPGYIBESMMUFU-UHFFFAOYSA-N", "4-methyl-3-hexanol", "")
  CALL Species("C_7H_{16}O",                           "623-55-2", "RGCZULIFYUPTAR-UHFFFAOYSA-N", "5-methyl-3-hexanol", "")
  CALL Species("C_7H_{16}O",                         "27522-11-8", "UKFQWAVMIMCNEH-UHFFFAOYSA-N", "2-ethyl-1-pentanol", "")
  CALL Species("C_7H_{16}O",                         "66225-51-2", "DVEFUHVVWJONKR-UHFFFAOYSA-N", "3-ethyl-1-pentanol", "")
  CALL Species("C_7H_{16}O",                           "609-27-8", "NEHRITNOSGFGGS-UHFFFAOYSA-N", "3-ethyl-2-pentanol", "")
  CALL Species("C_7H_{16}O",                           "597-49-9", "XKIRHOWVQWCYBT-UHFFFAOYSA-N", "3-ethyl-3-pentanol", "")
  CALL Species("C_7H_{16}O",                          "2370-12-9", "QTOMCRXZFDHJOL-UHFFFAOYSA-N", "2,2-dimethyl-1-pentanol", "")
  CALL Species("C_7H_{16}O",                         "10143-23-4", "MIBBFRQOCRYDDB-UHFFFAOYSA-N", "2,3-dimethyl-1-pentanol", "")
  CALL Species("C_7H_{16}O",                          "6305-71-1", "OVOVDHYEOQJKMD-UHFFFAOYSA-N", "2,4-dimethyl-1-pentanol", "")
  CALL Species("C_7H_{16}O",                         "19264-94-9", "IFDHMBHPLKHMOY-UHFFFAOYSA-N", "3,3-dimethyl-1-pentanol", "")
  CALL Species("C_7H_{16}O",                          "6570-87-2", "SVJNECJVNWTYQG-UHFFFAOYSA-N", "3,4-dimethyl-1-pentanol", "")
  CALL Species("C_7H_{16}O",                          "3121-79-7", "OWCNPTHAWPMOJU-UHFFFAOYSA-N", "4,4-dimethyl-1-pentanol", "")
  CALL Species("C_7H_{16}O",                          "4911-70-0", "YRSIFCHKXFKNME-UHFFFAOYSA-N", "2,3-dimethyl-2-pentanol", "") ! other CAS: 50819-06-2
  CALL Species("C_7H_{16}O",                           "625-06-9", "FMLSQAUAAGVTJO-UHFFFAOYSA-N", "2,4-dimethyl-2-pentanol", "")
  CALL Species("C_7H_{16}O",                         "19781-24-9", "FCUOIGTWJDBTHR-UHFFFAOYSA-N", "3,3-dimethyl-2-pentanol", "")
  CALL Species("C_7H_{16}O",                         "64502-86-9", "SODKTKJZYBOMMT-UHFFFAOYSA-N", "3,4-dimethyl-2-pentanol", "")
  CALL Species("C_7H_{16}O",                          "6144-93-0", "OIBKGNPMOMMSSI-UHFFFAOYSA-N", "4,4-dimethyl-2-pentanol", "")
  CALL Species("C_7H_{16}O",                          "3970-62-5", "HMSVXZJWPVIVIV-UHFFFAOYSA-N", "2,2-dimethyl-3-pentanol", "")
  CALL Species("C_7H_{16}O",                           "595-41-5", "RFZHJHSNHYIRNE-UHFFFAOYSA-N", "2,3-dimethyl-3-pentanol", "")
  CALL Species("C_7H_{16}O",                           "600-36-2", "BAYAKMPRFGNNFW-UHFFFAOYSA-N", "2,4-dimethyl-3-pentanol", "")
  CALL Species("C_7H_{16}O",                         "18371-13-6", "KMWHQWJVSALJAW-UHFFFAOYSA-N", "2-ethyl-2-methyl-1-butanol", "")
  CALL Species("C_7H_{16}O",                         "32444-34-1", "OXFFPTMSBXBVLZ-UHFFFAOYSA-N", "2-ethyl-3-methyl-1-butanol", "")
  CALL Species("C_7H_{16}O",                         "55505-23-2", "FAMOAJMEFGYSNY-UHFFFAOYSA-N", "2,2,3-trimethyl-1-butanol", "")
  CALL Species("C_7H_{16}O",                         "36794-64-6", "IWWVOLGZHKFWKK-UHFFFAOYSA-N", "2,3,3-trimethyl-1-butanol", "")
  CALL Species("C_7H_{16}O",                           "594-83-2", "OKXVARYIKDXAEO-UHFFFAOYSA-N", "2,3,3-trimethyl-2-butanol", "")
  ! C8:
  CALL Species("C_8H_{18}O",                           "111-87-5", "KBPLFHHGFOOTCA-UHFFFAOYSA-N", "1-octanol", "")
  CALL Species("C_8H_{18}O",                           "123-96-6", "SJWFXCIHNDVPSH-UHFFFAOYSA-N", "2-octanol", "")
  CALL Species("C_8H_{18}O",                           "589-98-0", "NMRPBPVERJPACX-UHFFFAOYSA-N", "3-octanol", "") ! other CAS: 20296-29-1
  CALL Species("C_8H_{18}O",                           "589-62-8", "WOFPPJOZXUTRAU-UHFFFAOYSA-N", "4-octanol", "")
  CALL Species("C_8H_{18}O",                         "60435-70-3", "QZESEQBMSFFHRY-UHFFFAOYSA-N", "2-methyl-1-heptanol", "")
  CALL Species("C_8H_{18}O",                          "1070-32-2", "MUPPEBVXFKNMCI-UHFFFAOYSA-N", "3-methyl-1-heptanol", "")
  CALL Species("C_8H_{18}O",                           "817-91-4", "LLUQZGDMUIMPTC-UHFFFAOYSA-N", "4-methyl-1-heptanol", "")
  CALL Species("C_8H_{18}O",                          "7212-53-5", "KFARNLMRENFOHE-UHFFFAOYSA-N", "5-methyl-1-heptanol", "")
  CALL Species("C_8H_{18}O",                          "1653-40-3", "BWDBEAQIHAEVLV-UHFFFAOYSA-N", "6-methyl-1-heptanol", "") ! other CAS: 26952-21-6
  CALL Species("C_8H_{18}O",                           "625-25-2", "ACBMYYVZWKYLIP-UHFFFAOYSA-N", "2-methyl-2-heptanol", "")
  CALL Species("C_8H_{18}O",                         "31367-46-1", "SZERMVMTUUAYML-UHFFFAOYSA-N", "3-methyl-2-heptanol", "")
  CALL Species("C_8H_{18}O",                         "56298-90-9", "GUHWHNUGIGOSCN-UHFFFAOYSA-N", "4-methyl-2-heptanol", "")
  CALL Species("C_8H_{18}O",                         "54630-50-1", "FYMBAYNKBWGEIK-UHFFFAOYSA-N", "5-methyl-2-heptanol", "")
  CALL Species("C_8H_{18}O",                          "4730-22-7", "FCOUHTHQYOMLJT-UHFFFAOYSA-N", "6-methyl-2-heptanol", "")
  CALL Species("C_8H_{18}O",                         "18720-62-2", "QGVFLDUEHSIZIG-UHFFFAOYSA-N", "2-methyl-3-heptanol", "")
  CALL Species("C_8H_{18}O",                          "5582-82-1", "PQOSNJHBSNZITJ-UHFFFAOYSA-N", "3-methyl-3-heptanol", "")
  CALL Species("C_8H_{18}O",                         "14979-39-6", "BKQICAFAUMRYLZ-UHFFFAOYSA-N", "4-methyl-3-heptanol", "")
  CALL Species("C_8H_{18}O",                         "18720-65-5", "SECKOSOTZOBWEI-UHFFFAOYSA-N", "5-methyl-3-heptanol", "")
  CALL Species("C_8H_{18}O",                         "18720-66-6", "MNBIBGDICHMQFN-UHFFFAOYSA-N", "6-methyl-3-heptanol", "")
  CALL Species("C_8H_{18}O",                         "21570-35-4", "QXPLZEKPCGUWEM-UHFFFAOYSA-N", "2-methyl-4-heptanol", "")
  CALL Species("C_8H_{18}O",                          "1838-73-9", "JMRDKKYZLXDPLN-UHFFFAOYSA-N", "3-methyl-4-heptanol", "")
  CALL Species("C_8H_{18}O",                           "598-01-6", "IQXKGRKRIRMQCQ-UHFFFAOYSA-N", "4-methyl-4-heptanol", "")
  CALL Species("C_8H_{18}O",                           "104-76-7", "YIWUKEYIRIRTPP-UHFFFAOYSA-N", "2-ethyl-1-hexanol", "")
  CALL Species("C_8H_{18}O",                         "41065-95-6", "LWWJDXKGQVEZKT-UHFFFAOYSA-N", "3-ethyl-1-hexanol", "")
  CALL Species("C_8H_{18}O",                         "66576-32-7", "RLGDVTUGYZAYIX-UHFFFAOYSA-N", "4-ethyl-1-hexanol", "")
  CALL Species("C_8H_{18}O",                          "2370-13-0", "GSSDZVRLQDXOPL-UHFFFAOYSA-N", "2,2-dimethyl-1-hexanol", "")
  CALL Species("C_8H_{18}O",                                   "", "HIYTVAULOHUDEC-UHFFFAOYSA-N", "2,3-dimethyl-1-hexanol", "")
  CALL Species("C_8H_{18}O",                          "3965-59-1", "GDRBQWCGBCJTLB-UHFFFAOYSA-N", "2,4-dimethyl-1-hexanol", "")
  CALL Species("C_8H_{18}O",                          "6886-16-4", "OBOHUKJIPIBYTA-UHFFFAOYSA-N", "2,5-dimethyl-1-hexanol", "")
  CALL Species("C_8H_{18}O",                         "10524-70-6", "RAKYQWCHMSXUEG-UHFFFAOYSA-N", "3,3-dimethyl-1-hexanol", "")
  CALL Species("C_8H_{18}O",                         "66576-57-6", "QVFKMROLPSPCIX-UHFFFAOYSA-N", "3,4-dimethyl-1-hexanol", "")
  CALL Species("C_8H_{18}O",                         "13501-73-0", "WETBJXIDTZXCBL-UHFFFAOYSA-N", "3,5-dimethyl-1-hexanol", "")
  CALL Species("C_8H_{18}O",                          "6481-95-4", "VZWIUKUDEGQHIO-UHFFFAOYSA-N", "4,4-dimethyl-1-hexanol", "")
  CALL Species("C_8H_{18}O",                         "60564-76-3", "QABJATQYUASJEM-UHFFFAOYSA-N", "4,5-dimethyl-1-hexanol", "")
  CALL Species("C_8H_{18}O",                          "2768-18-5", "QYFVEEMPFRRFNN-UHFFFAOYSA-N", "5,5-dimethyl-1-hexanol", "")
  CALL Species("C_8H_{18}O",                         "24448-19-9", "YBQZSEZVMFENOM-UHFFFAOYSA-N", "3-ethyl-2-hexanol", "")
  CALL Species("C_8H_{18}O",                                   "", "ALAPJJMGVQNJIU-UHFFFAOYSA-N", "4-ethyl-2-hexanol", "")
  CALL Species("C_8H_{18}O",                         "19550-03-9", "BFKOEFFCVFMWPF-UHFFFAOYSA-N", "2,3-dimethyl-2-hexanol", "")
  CALL Species("C_8H_{18}O",                         "42328-76-7", "RTRZHEPXWQXCCI-UHFFFAOYSA-N", "2,4-dimethyl-2-hexanol", "")
  CALL Species("C_8H_{18}O",                          "3730-60-7", "JPUIYNHIEXIFMV-UHFFFAOYSA-N", "2,5-dimethyl-2-hexanol", "")
  CALL Species("C_8H_{18}O",                         "22025-20-3", "LTTHCWFPLXSQOD-UHFFFAOYSA-N", "3,3-dimethyl-2-hexanol", "")
  CALL Species("C_8H_{18}O",                         "19550-05-1", "LBJWJHMCZHKLQU-UHFFFAOYSA-N", "3,4-dimethyl-2-hexanol", "")
  CALL Species("C_8H_{18}O",                         "66576-27-0", "VGGMROOHXNLMTP-UHFFFAOYSA-N", "3,5-dimethyl-2-hexanol", "")
  CALL Species("C_8H_{18}O",                                   "", "CJGWLHDFPXIKQX-UHFFFAOYSA-N", "4,4-dimethyl-2-hexanol", "")
  CALL Species("C_8H_{18}O",                                   "", "KHSOOQUMPPYGAW-UHFFFAOYSA-N", "4,5-dimethyl-2-hexanol", "")
  CALL Species("C_8H_{18}O",                         "31841-77-7", "NMSUCVXVCHZTEH-UHFFFAOYSA-N", "5,5-dimethyl-2-hexanol", "")
  CALL Species("C_8H_{18}O",                           "597-76-2", "WNDLTOTUHMHNOC-UHFFFAOYSA-N", "3-ethyl-3-hexanol", "")
  CALL Species("C_8H_{18}O",                         "19780-44-0", "BOJLCKCCKQMGKD-UHFFFAOYSA-N", "4-ethyl-3-hexanol", "")
  CALL Species("C_8H_{18}O",                          "4209-90-9", "PFHLGQKVKALLMD-UHFFFAOYSA-N", "2,2-dimethyl-3-hexanol", "")
  CALL Species("C_8H_{18}O",                          "4166-46-5", "CGWJMIUMPDDHQC-UHFFFAOYSA-N", "2,3-dimethyl-3-hexanol", "")
  CALL Species("C_8H_{18}O",                         "13432-25-2", "UCRQJBCLZKHOGX-UHFFFAOYSA-N", "2,4-dimethyl-3-hexanol", "")
  CALL Species("C_8H_{18}O",                         "19550-07-3", "SNKTZHPOKPYBPT-UHFFFAOYSA-N", "2,5-dimethyl-3-hexanol", "")
  CALL Species("C_8H_{18}O",                         "19550-08-4", "FJXOYCIYKQJAAF-UHFFFAOYSA-N", "3,4-dimethyl-3-hexanol", "")
  CALL Species("C_8H_{18}O",                          "4209-91-0", "INMGJWCKWKKMPN-UHFFFAOYSA-N", "3,5-dimethyl-3-hexanol", "")
  CALL Species("C_8H_{18}O",                         "19550-09-5", "BARKHVPKOOSMNW-UHFFFAOYSA-N", "4,4-dimethyl-3-hexanol", "")
  CALL Species("C_8H_{18}O",                                   "", "UAVZZCGGBCLHIP-UHFFFAOYSA-N", "4,5-dimethyl-3-hexanol", "")
  CALL Species("C_8H_{18}O",                         "66576-31-6", "NPZRPUOKIPAIEL-UHFFFAOYSA-N", "5,5-dimethyl-3-hexanol", "")
  CALL Species("C_8H_{18}O",                         "58175-57-8", "LASHFHLFDRTERB-UHFFFAOYSA-N", "2-propyl-1-pentanol", "")
  CALL Species("C_8H_{18}O",                          "5970-63-8", "VZCOFGHOKLEMPL-UHFFFAOYSA-N", "2-ethyl-2-methyl-1-pentanol", "")
  CALL Species("C_8H_{18}O",                                   "", "ALPFTHGDLMTYRM-UHFFFAOYSA-N", "2-ethyl-3-methyl-1-pentanol", "")
  CALL Species("C_8H_{18}O",                           "106-67-2", "QCHSJPKDWOFACC-UHFFFAOYSA-N", "2-ethyl-4-methyl-1-pentanol", "")
  CALL Species("C_8H_{18}O",                                   "", "VWQDMMBTEFDWNM-UHFFFAOYSA-N", "3-ethyl-2-methyl-1-pentanol", "")
  CALL Species("C_8H_{18}O",                         "10524-71-7", "XYUCPYMMHKJJSS-UHFFFAOYSA-N", "3-ethyl-3-methyl-1-pentanol", "")
  CALL Species("C_8H_{18}O",                         "38514-13-5", "RWIFVESHBHTZEM-UHFFFAOYSA-N", "3-ethyl-4-methyl-1-pentanol", "")
  CALL Species("C_8H_{18}O",                         "57409-53-7", "GBONVOIRPAJSLA-UHFFFAOYSA-N", "2,2,3-trimethyl-1-pentanol", "")
  CALL Species("C_8H_{18}O",                           "123-44-4", "CWPPDTVYIJETDF-UHFFFAOYSA-N", "2,2,4-trimethyl-1-pentanol", "")
  CALL Species("C_8H_{18}O",                                   "", "LFRFDOWJBYDJGD-UHFFFAOYSA-N", "2,3,3-trimethyl-1-pentanol", "")
  CALL Species("C_8H_{18}O",                          "6570-88-3", "PAZDSSMTPLLLIR-UHFFFAOYSA-N", "2,3,4-trimethyl-1-pentanol", "")
  CALL Species("C_8H_{18}O",                         "16325-63-6", "ZNRVRWHPZZOTIE-UHFFFAOYSA-N", "2,4,4-trimethyl-1-pentanol", "")
  CALL Species("C_8H_{18}O",                                   "", "OIPMJJVCOSACDS-UHFFFAOYSA-N", "3,3,4-trimethyl-1-pentanol", "")
  CALL Species("C_8H_{18}O",                         "16325-64-7", "FQNYRSRCONKFCN-UHFFFAOYSA-N", "3,4,4-trimethyl-1-pentanol", "")
  CALL Species("C_8H_{18}O",                         "19780-63-3", "FRUMTAZIGSJVOJ-UHFFFAOYSA-N", "3-ethyl-2-methyl-2-pentanol", "")
  CALL Species("C_8H_{18}O",                         "66576-23-6", "PFRFNYYIOSQSDD-UHFFFAOYSA-N", "3-ethyl-4-methyl-2-pentanol", "")
  CALL Species("C_8H_{18}O",                         "23171-85-9", "FBWWGYIEJGQWJP-UHFFFAOYSA-N", "2,3,3-trimethyl-2-pentanol", "")
  CALL Species("C_8H_{18}O",                         "66576-26-9", "FTPXXARYDFWPGE-UHFFFAOYSA-N", "2,3,4-trimethyl-2-pentanol", "")
  CALL Species("C_8H_{18}O",                           "690-37-9", "BSYJHYLAMMJNRC-UHFFFAOYSA-N", "2,4,4-trimethyl-2-pentanol", "")
  CALL Species("C_8H_{18}O",                         "19411-41-7", "BOBYQFFCQPQLOB-UHFFFAOYSA-N", "3,3,4-trimethyl-2-pentanol", "")
  CALL Species("C_8H_{18}O",                         "10575-56-1", "MXDGSCZQIMQLII-UHFFFAOYSA-N", "3,4,4-trimethyl-2-pentanol", "")
  CALL Species("C_8H_{18}O",                           "597-05-7", "DMHIJUVUPKCGLJ-UHFFFAOYSA-N", "3-ethyl-2-methyl-3-pentanol", "")
  CALL Species("C_8H_{18}O",                         "66576-22-5", "SLBLSROGXMXPPF-UHFFFAOYSA-N", "3-ethyl-3-methyl-2-pentanol", "")
  CALL Species("C_8H_{18}O",                          "7294-05-5", "KLIHWYNSISMOMR-UHFFFAOYSA-N", "2,2,3-trimethyl-3-pentanol", "")
  CALL Species("C_8H_{18}O",                          "5162-48-1", "AXINNNJHLJWMTC-UHFFFAOYSA-N", "2,2,4-trimethyl-3-pentanol", "")
  CALL Species("C_8H_{18}O",                          "3054-92-0", "PLSMHHUFDLYURK-UHFFFAOYSA-N", "2,3,4-trimethyl-3-pentanol", "")
  CALL Species("C_8H_{18}O",                         "18593-92-5", "IDGDEUZERZKYHG-UHFFFAOYSA-N", "3-methyl-2-(1-methylethyl)-1-butanol", "")
  CALL Species("C_8H_{18}O",                         "66576-24-7", "OCTRJUXIYWWVAR-UHFFFAOYSA-N", "2,2,3,3-tetramethyl-1-butanol", "")
  CALL Species("C_8H_{18}O",                         "13023-60-4", "CBHXDVOSUKFRBE-UHFFFAOYSA-N", "2,2-diethyl-1-butanol", "")
  CALL Species("C_8H_{18}O",                                   "", "YPIJDMPDVZUWJO-UHFFFAOYSA-N", "2,3-dimethyl-2-ethyl-1-butanol", "")
  CALL Species("C_8H_{18}O",                         "66576-56-5", "WOVMYJCSCISMAU-UHFFFAOYSA-N", "3,3-dimethyl-2-ethyl-1-butanol", "")
  CALL Species("C_8H_{18}O_2",                         "112-25-4", "UPGSWASWQBLSKZ-UHFFFAOYSA-N", "2-(hexyloxy)-ethanol", "")
  ! C9:
  CALL Species("C_9H_{20}O",                           "143-08-8", "ZWRUINPWMLAQRD-UHFFFAOYSA-N", "1-nonanol", "") ! other CAS: 28473-21-4
  CALL Species("C_9H_{20}O",                           "628-99-9", "NGDNVOAEIVQRFH-UHFFFAOYSA-N", "2-nonanol", "")
  CALL Species("C_9H_{20}O",                           "624-51-1", "GYSCXPVAKHVAAY-UHFFFAOYSA-N", "3-nonanol", "")
  CALL Species("C_9H_{20}O",                          "5932-79-6", "IXUOEGRSQCCEHB-UHFFFAOYSA-N", "4-nonanol", "")
  CALL Species("C_9H_{20}O",                           "623-93-8", "FCBBRODPXVPZAH-UHFFFAOYSA-N", "5-nonanol", "")
  CALL Species("C_9H_{20}O",                           "818-81-5", "IGVGCQGTEINVOH-UHFFFAOYSA-N", "2-methyl-1-octanol", "")
  CALL Species("C_9H_{20}O",                         "38514-02-2", "CLFSZAMBOZSCOS-UHFFFAOYSA-N", "3-methyl-1-octanol", "")
  CALL Species("C_9H_{20}O",                         "38514-03-3", "MWWKESKJRHQWEF-UHFFFAOYSA-N", "4-methyl-1-octanol", "")
  CALL Species("C_9H_{20}O",                         "38514-04-4", "CGCDFYUPZXVGIX-UHFFFAOYSA-N", "5-methyl-1-octanol", "")
  CALL Species("C_9H_{20}O",                         "38514-05-5", "WWRGKAMABZHMCN-UHFFFAOYSA-N", "6-methyl-1-octanol", "")
  CALL Species("C_9H_{20}O",                          "2430-22-0", "QDTDKYHPHANITQ-UHFFFAOYSA-N", "7-methyl-1-octanol", "")
  CALL Species("C_9H_{20}O",                           "628-44-4", "KBCNUEXDHWDIFX-UHFFFAOYSA-N", "2-methyl-2-octanol", "")
  CALL Species("C_9H_{20}O",                         "27644-49-1", "WQADSKJNOTZWML-UHFFFAOYSA-N", "3-methyl-2-octanol", "")
  CALL Species("C_9H_{20}O",                         "66793-81-5", "IWFICVXFFNDWOJ-UHFFFAOYSA-N", "5-methyl-2-octanol", "")
  CALL Species("C_9H_{20}O",                         "66793-83-7", "NOEKZKTXHKNMAQ-UHFFFAOYSA-N", "7-methyl-2-octanol", "")
  CALL Species("C_9H_{20}O",                         "26533-34-6", "DIVBBSLQUDHECU-UHFFFAOYSA-N", "2-methyl-3-octanol", "")
  CALL Species("C_9H_{20}O",                          "5340-36-3", "JEWXYDDSLPIBBO-UHFFFAOYSA-N", "3-methyl-3-octanol", "")
  CALL Species("C_9H_{20}O",                         "66793-80-4", "XPWBBDBJJRZCPW-UHFFFAOYSA-N", "4-methyl-3-octanol", "")
  CALL Species("C_9H_{20}O",                         "40225-75-0", "MFYHIHFYDULUQP-UHFFFAOYSA-N", "6-methyl-3-octanol", "")
  CALL Species("C_9H_{20}O",                         "66793-84-8", "IDCFJIMYNKBKMB-UHFFFAOYSA-N", "7-methyl-3-octanol", "")
  CALL Species("C_9H_{20}O",                         "40575-41-5", "BIAVIOIDPRPYJK-UHFFFAOYSA-N", "2-methyl-4-octanol", "")
  CALL Species("C_9H_{20}O",                         "26533-35-7", "MJOKZMZDONULOD-UHFFFAOYSA-N", "3-methyl-4-octanol", "")
  CALL Species("C_9H_{20}O",                         "23418-37-3", "RXSIKQJQLQRQQY-UHFFFAOYSA-N", "4-methyl-4-octanol", "")
  CALL Species("C_9H_{20}O",                         "59734-23-5", "YLTHHPQUTLMNIF-UHFFFAOYSA-N", "5-methyl-4-octanol", "")
  CALL Species("C_9H_{20}O",                         "66793-82-6", "KFRCBGHGDZSOJV-UHFFFAOYSA-N", "6-methyl-4-octanol", "")
  CALL Species("C_9H_{20}O",                         "33933-77-6", "KJMBBHZOLRRVMV-UHFFFAOYSA-N", "7-methyl-4-octanol", "")
  CALL Species("C_9H_{20}O",                           "817-60-7", "QNJAZNNWHWYOEO-UHFFFAOYSA-N", "2-ethyl-1-heptanol", "")
  CALL Species("C_9H_{20}O",                          "3525-25-5", "VRZRVMXNGMZLDB-UHFFFAOYSA-N", "3-ethyl-1-heptanol", "")
  CALL Species("C_9H_{20}O",                           "998-65-2", "GOJBFVJUMYSMJJ-UHFFFAOYSA-N", "5-ethyl-1-heptanol", "")
  CALL Species("C_9H_{20}O",                         "19780-39-3", "MMQDVLWWGWJSFS-UHFFFAOYSA-N", "3-ethyl-2-heptanol", "")
  CALL Species("C_9H_{20}O",                         "19780-41-7", "XKRZDNKKANUBPV-UHFFFAOYSA-N", "3-ethyl-3-heptanol", "")
  CALL Species("C_9H_{20}O",                           "597-90-0", "GNROHGFUVTWFNG-UHFFFAOYSA-N", "4-ethyl-4-heptanol", "")
  CALL Species("C_9H_{20}O",                         "14250-79-4", "WENIXZFPXMQPQQ-UHFFFAOYSA-N", "2,2-dimethyl-1-heptanol", "")
  CALL Species("C_9H_{20}O",                           "820-05-3", "GCBXGQPBCBPHSP-UHFFFAOYSA-N", "4,6-dimethyl-1-heptanol", "")
  CALL Species("C_9H_{20}O",                         "65769-10-0", "IOFUAVGBFVXDAO-UHFFFAOYSA-N", "6,6-dimethyl-1-heptanol", "")
  CALL Species("C_9H_{20}O",                         "66794-00-1", "PQSMEVPHTJECDZ-UHFFFAOYSA-N", "2,3-dimethyl-2-heptanol", "")
  CALL Species("C_9H_{20}O",                         "65822-93-7", "VORBOMMQBCSRQF-UHFFFAOYSA-N", "2,4-dimethyl-2-heptanol", "")
  CALL Species("C_9H_{20}O",                          "1561-18-8", "LVIFBEPHIBJBEU-UHFFFAOYSA-N", "2,5-dimethyl-2-heptanol", "")
  CALL Species("C_9H_{20}O",                         "13254-34-7", "HGDVHRITTGWMJK-UHFFFAOYSA-N", "2,6-dimethyl-2-heptanol", "")
  CALL Species("C_9H_{20}O",                         "51079-52-8", "YYUGBYFBCFRGNZ-UHFFFAOYSA-N", "4,6-dimethyl-2-heptanol", "")
  CALL Species("C_9H_{20}O",                         "58795-24-7", "ONBXNGYZXZBRJA-UHFFFAOYSA-N", "5,6-dimethyl-2-heptanol", "")
  CALL Species("C_9H_{20}O",                         "19549-70-3", "QENWAAAGDINHGE-UHFFFAOYSA-N", "2,2-dimethyl-3-heptanol", "")
  CALL Species("C_9H_{20}O",                         "19549-71-4", "JIEGVNXCNNWVPH-UHFFFAOYSA-N", "2,3-dimethyl-3-heptanol", "")
  CALL Species("C_9H_{20}O",                         "19549-73-6", "XZDMJRIWJSNEGC-UHFFFAOYSA-N", "2,6-dimethyl-3-heptanol", "")
  CALL Species("C_9H_{20}O",                         "19549-74-7", "NOSOEGQGQOMHBF-UHFFFAOYSA-N", "3,5-dimethyl-3-heptanol", "")
  CALL Species("C_9H_{20}O",                         "66793-99-5", "ZIKNXBYJTDTRQA-UHFFFAOYSA-N", "2,2-dimethyl-4-heptanol", "")
  CALL Species("C_9H_{20}O",                         "19549-77-0", "QKRRAXACNUNGCF-UHFFFAOYSA-N", "2,4-dimethyl-4-heptanol", "")
  CALL Species("C_9H_{20}O",                                   "", "NXLQTSHQSAVKJB-UHFFFAOYSA-N", "2,5-dimethyl-4-heptanol", "")
  CALL Species("C_9H_{20}O",                           "108-82-7", "HXQPUEQDBSPXTE-UHFFFAOYSA-N", "2,6-dimethyl-4-heptanol", "")
  CALL Species("C_9H_{20}O",                         "19549-78-1", "UGAZGYFCTFGQQH-UHFFFAOYSA-N", "3,3-dimethyl-4-heptanol", "")
  CALL Species("C_9H_{20}O",                         "19549-79-2", "ZKXITRNXHWEQJU-UHFFFAOYSA-N", "3,5-dimethyl-4-heptanol", "")
  CALL Species("C_9H_{20}O",                          "1573-28-0", "HVPGGLNDHUWMLS-UHFFFAOYSA-N", "3,6-dimethyl-3-heptanol", "")
  CALL Species("C_9H_{20}O",                         "66794-04-5", "DXXCMAAACWSRPE-UHFFFAOYSA-N", "2-ethyl-3-methyl-1-hexanol", "")
  CALL Species("C_9H_{20}O",                         "66794-06-7", "NRZVENBFUFCASY-UHFFFAOYSA-N", "2-ethyl-4-methyl-1-hexanol", "")
  CALL Species("C_9H_{20}O",                         "66794-07-8", "KVJOUCNJVQBRAO-UHFFFAOYSA-N", "2-ethyl-5-methyl-1-hexanol", "")
  CALL Species("C_9H_{20}O",                                   "", "YNZQSDTXBBBFQH-UHFFFAOYSA-N", "3-ethyl-1-methyl-1-hexanol", "")
  CALL Species("C_9H_{20}O",                         "66794-01-2", "UDZCUJOKBNABRS-UHFFFAOYSA-N", "3-ethyl-2-methyl-1-hexanol", "")
  CALL Species("C_9H_{20}O",                          "1484-87-3", "HRXNWQMMTLLJJQ-UHFFFAOYSA-N", "3,3,5-trimethyl-1-hexanol", "")
  CALL Species("C_9H_{20}O",                         "66793-73-5", "OLRPKAJSAOYYCY-UHFFFAOYSA-N", "3,4,4-trimethyl-1-hexanol", "")
  CALL Species("C_9H_{20}O",                          "3452-97-9", "BODRLKRKPXBDBN-UHFFFAOYSA-N", "3,5,5-trimethyl-1-hexanol", "")
  CALL Species("C_9H_{20}O",                         "66793-75-7", "IECJUMYEWJOKHM-UHFFFAOYSA-N", "4,5,5-trimethyl-1-hexanol", "")
  CALL Species("C_9H_{20}O",                           "817-46-9", "JSUXZEJWGVYJJG-UHFFFAOYSA-N", "2-propyl-1-hexanol", "")
  CALL Species("C_9H_{20}O",                         "66793-85-9", "MHJIVMDBOGBUHH-UHFFFAOYSA-N", "3-propyl-1-hexanol", "")
  CALL Species("C_9H_{20}O",                         "66794-02-3", "HTOYXTGJEOUKPM-UHFFFAOYSA-N", "3-ethyl-2-methyl-2-hexanol", "")
  CALL Species("C_9H_{20}O",                         "21102-13-6", "JTOONBKGYWVPOY-UHFFFAOYSA-N", "2,3,4-trimethyl-2-hexanol", "")
  CALL Species("C_9H_{20}O",                         "66793-91-7", "QMEHJTXRYVSFES-UHFFFAOYSA-N", "2,4,4-trimethyl-2-hexanol", "")
  CALL Species("C_9H_{20}O",                         "66793-93-9", "CIVZBWYMPDUPSQ-UHFFFAOYSA-N", "2,4,5-trimethyl-2-hexanol", "")
  CALL Species("C_9H_{20}O",                         "66793-71-3", "ASOCSFZHADJKFU-UHFFFAOYSA-N", "2,5,5-trimethyl-2-hexanol", "")
  CALL Species("C_9H_{20}O",                         "66794-03-4", "VUMFTWNZYFBYEB-UHFFFAOYSA-N", "3-ethyl-2-methyl-3-hexanol", "")
  CALL Species("C_9H_{20}O",                         "51200-80-7", "YOCLWWFBEJQTBM-UHFFFAOYSA-N", "3-ethyl-4-methyl-3-hexanol", "")
  CALL Species("C_9H_{20}O",                           "597-77-3", "RMBUNHOAKTUXTC-UHFFFAOYSA-N", "3-ethyl-5-methyl-3-hexanol", "")
  CALL Species("C_9H_{20}O",                         "33943-21-4", "GBUYIUMHXYBONU-UHFFFAOYSA-N", "4-ethyl-2-methyl-3-hexanol", "")
  CALL Species("C_9H_{20}O",                         "66794-05-6", "DUGZCTGTSJAPCV-UHFFFAOYSA-N", "4-ethyl-3-methyl-3-hexanol", "")
  CALL Species("C_9H_{20}O",                          "5340-41-0", "DHNHHYWBZNQHGN-UHFFFAOYSA-N", "2,2,3-trimethyl-3-hexanol", "")
  CALL Species("C_9H_{20}O",                         "66793-89-3", "YMBVVPXERVAJTB-UHFFFAOYSA-N", "2,2,4-trimethyl-3-hexanol", "")
  CALL Species("C_9H_{20}O",                          "3970-60-3", "UCLXUUYVWJKXIK-UHFFFAOYSA-N", "2,2,5-trimethyl-3-hexanol", "")
  CALL Species("C_9H_{20}O",                         "66793-90-6", "LTKHKGAPVGSECA-UHFFFAOYSA-N", "2,3,4-trimethyl-3-hexanol", "")
  CALL Species("C_9H_{20}O",                         "65927-60-8", "ZVKQNIUBLJYSGN-UHFFFAOYSA-N", "2,3,5-trimethyl-3-hexanol", "")
  CALL Species("C_9H_{20}O",                         "66793-92-8", "PDIVDRVTPZNRDZ-UHFFFAOYSA-N", "2,4,4-trimethyl-3-hexanol", "")
  CALL Species("C_9H_{20}O",                         "66793-72-4", "NPUIBFBNZSKEAP-UHFFFAOYSA-N", "2,5,5-trimethyl-3-hexanol", "")
  CALL Species("C_9H_{20}O",                         "66793-74-6", "QWJDXMFJOMPTCV-UHFFFAOYSA-N", "3,4,4-trimethyl-3-hexanol", "")
  CALL Species("C_9H_{20}O",                         "66810-87-5", "HGGJNDQMXCTKCR-UHFFFAOYSA-N", "3,5,5-trimethyl-3-hexanol", "")
  CALL Species("C_9H_{20}O",                         "54004-41-0", "IGSWOIOCVJEQRH-UHFFFAOYSA-N", "4-methyl-2-propyl-1-pentanol", "")
  CALL Species("C_9H_{20}O",                         "55505-24-3", "CBHPZSADBWMIGE-UHFFFAOYSA-N", "4-methyl-2-(1-methylethyl)-1-pentanol", "")
  CALL Species("C_9H_{20}O",                         "66793-98-4", "JIZZVMCXCCJUHI-UHFFFAOYSA-N", "2-ethyl-2,4-dimethyl-1-pentanol", "")
  CALL Species("C_9H_{20}O",                         "66793-95-1", "HEQYMZTUSVDQBW-UHFFFAOYSA-N", "3-ethyl-2,2-dimethyl-1-pentanol", "")
  CALL Species("C_9H_{20}O",                         "66793-86-0", "HLWCPAJEXXTJHY-UHFFFAOYSA-N", "2,3,3,4-tetramethyl-2-pentanol", "")
  CALL Species("C_9H_{20}O",                         "66793-87-1", "DBCCRNKDJIOKIO-UHFFFAOYSA-N", "2,3,4,4-tetramethyl-2-pentanol", "")
  CALL Species("C_9H_{20}O",                         "66793-88-2", "AIQOKYGLKGGWOI-UHFFFAOYSA-N", "3,3,4,4-tetramethyl-2-pentanol", "")
  CALL Species("C_9H_{20}O",                         "66793-94-0", "IENXYHLNFHBNRR-UHFFFAOYSA-N", "3,3-diethyl-2-pentanol", "")
  CALL Species("C_9H_{20}O",                         "66793-97-3", "AVQHNUMJCWRCBA-UHFFFAOYSA-N", "2,3-dimethyl-3-ethyl-2-pentanol", "")
  CALL Species("C_9H_{20}O",                         "21102-09-0", "YGQDLQQRBCOMKJ-UHFFFAOYSA-N", "4,4-dimethyl-3-ethyl-2-pentanol", "")
  CALL Species("C_9H_{20}O",                         "66793-96-2", "CFWIFHZJKFFDFU-UHFFFAOYSA-N", "3-ethyl-2,2-dimethyl-3-pentanol", "")
  CALL Species("C_9H_{20}O",                          "3970-59-0", "PZPYIRQFCOFLGJ-UHFFFAOYSA-N", "3-ethyl-2,4-dimethyl-3-pentanol", "")
  CALL Species("C_9H_{20}O",                         "29772-39-2", "KNTFAYXQHRNXSM-UHFFFAOYSA-N", "2,2,3,4-tetramethyl-3-pentanol", "")
  CALL Species("C_9H_{20}O",                         "14609-79-1", "WFJSIIHYYLHRHB-UHFFFAOYSA-N", "2,2,4,4-tetramethyl-3-pentanol", "")
  ! C10:
  CALL Species("C_{10}H_{22}O",                        "112-30-1", "MWKFXSUHUHTGQN-UHFFFAOYSA-N", "1-decanol", "") ! other CAS: 36729-58-5
  CALL Species("C_{10}H_{22}O",                       "1120-06-5", "ACUZDYFTRHEKOS-UHFFFAOYSA-N", "2-decanol", "")
  CALL Species("C_{10}H_{22}O",                       "1565-81-7", "ICEQLCZWZXUUIJ-UHFFFAOYSA-N", "3-decanol", "")
  CALL Species("C_{10}H_{22}O",                       "2051-31-2", "DTDMYWXTWWFLGJ-UHFFFAOYSA-N", "4-decanol", "")
  CALL Species("C_{10}H_{22}O",                       "5205-34-5", "SZMNDOUFZGODBR-UHFFFAOYSA-N", "5-decanol", "")
  CALL Species("C_{10}H_{22}O",                      "40589-14-8", "BEGNRPGEHZBNKK-UHFFFAOYSA-N", "2-methyl-1-nonanol", "")
  CALL Species("C_{10}H_{22}O",                      "10297-57-1", "VREDNSVJXRJXRI-UHFFFAOYSA-N", "2-methyl-2-nonanol", "")
  CALL Species("C_{10}H_{22}O",                      "26533-33-5", "OFIYMUXECPHIPZ-UHFFFAOYSA-N", "2-methyl-3-nonanol", "")
  CALL Species("C_{10}H_{22}O",                      "26533-31-3", "IBHHTADZYDLHPM-UHFFFAOYSA-N", "2-methyl-4-nonanol", "")
  CALL Species("C_{10}H_{22}O",                      "29843-62-7", "RLQVUGAVOCBRNQ-UHFFFAOYSA-N", "2-methyl-5-nonanol", "")
  CALL Species("C_{10}H_{22}O",                      "22663-64-5", "BXQPYGLPOMTAPU-UHFFFAOYSA-N", "3-methyl-1-nonanol", "")
  CALL Species("C_{10}H_{22}O",                      "60671-32-1", "QCBDLZKDRUKOFS-UHFFFAOYSA-N", "3-methyl-2-nonanol", "")
  CALL Species("C_{10}H_{22}O",                      "21078-72-8", "VZBFPIMCUSPDLS-UHFFFAOYSA-N", "3-methyl-3-nonanol", "")
  CALL Species("C_{10}H_{22}O",                                "", "QRZAYHZIHWIJHD-UHFFFAOYSA-N", "3-methyl-5-nonanol", "")
  CALL Species("C_{10}H_{22}O",                       "1489-47-0", "HECVGHIJHFNAIL-UHFFFAOYSA-N", "4-methyl-1-nonanol", "")
  CALL Species("C_{10}H_{22}O",                      "23418-38-4", "GDCOAKPWVJCNGI-UHFFFAOYSA-N", "4-methyl-4-nonanol", "")
  CALL Species("C_{10}H_{22}O",                       "2768-16-3", "DBJSFYCKLLBKGB-UHFFFAOYSA-N", "5-methyl-1-nonanol", "")
  CALL Species("C_{10}H_{22}O",                      "66731-95-1", "KZTLXVUOEODDMR-UHFFFAOYSA-N", "5-methyl-2-nonanol", "")
  CALL Species("C_{10}H_{22}O",                      "66719-43-5", "CCSZLOPBDJWYHV-UHFFFAOYSA-N", "5-methyl-3-nonanol", "")
  CALL Species("C_{10}H_{22}O",                      "66719-44-6", "BFYIAYZULSQBFQ-UHFFFAOYSA-N", "5-methyl-4-nonanol", "")
  CALL Species("C_{10}H_{22}O",                      "33933-78-7", "AGSIGVZAVLOKLP-UHFFFAOYSA-N", "5-methyl-5-nonanol", "")
  CALL Species("C_{10}H_{22}O",                      "66256-60-8", "OYTRLQQQIHEJFE-UHFFFAOYSA-N", "6-methyl-2-nonanol", "")
  CALL Species("C_{10}H_{22}O",                      "33234-93-4", "BJKXZCGJAOWNTN-UHFFFAOYSA-N", "7-methyl-1-nonanol", "")
  CALL Species("C_{10}H_{22}O",                      "66256-61-9", "GCOBJHFUTQUABB-UHFFFAOYSA-N", "7-methyl-2-nonanol", "")
  CALL Species("C_{10}H_{22}O",                      "26981-98-6", "KLZAWUYSXUGWGQ-UHFFFAOYSA-N", "7-methyl-4-nonanol", "")
  CALL Species("C_{10}H_{22}O",                      "25339-17-7", "PLLBRTOLHQQAQQ-UHFFFAOYSA-N", "8-methyl-1-nonanol", "isodecanol")
  CALL Species("C_{10}H_{22}O",                      "14779-92-1", "ZVZKLBCGIYWGOU-UHFFFAOYSA-N", "8-methyl-2-nonanol", "")
  CALL Species("C_{10}H_{22}O",                       "2370-14-1", "KEXGXAGJHHCTKD-UHFFFAOYSA-N", "2,2-dimethyl-1-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "19841-72-6", "AZIWNYAWODSERA-UHFFFAOYSA-N", "2,2-dimethyl-3-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "66719-52-6", "IVLDCRDTRQCTIY-UHFFFAOYSA-N", "2,2-dimethyl-4-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "19781-10-3", "AWHYRPPRRQITHX-UHFFFAOYSA-N", "2,3-dimethyl-3-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "18675-20-2", "WHJCLXRXPLJSSV-UHFFFAOYSA-N", "2,4-dimethyl-2-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "33933-79-8", "VRFMFHBPJUSGFK-UHFFFAOYSA-N", "2,4-dimethyl-4-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "66719-53-7", "VIHAZZPWGKHJQE-UHFFFAOYSA-N", "2,5-dimethyl-4-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "62417-08-7", "ZKWASVBZSAGDBI-UHFFFAOYSA-N", "2,6-dimethyl-1-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "18479-57-7", "WRFXXJKURVTLSY-UHFFFAOYSA-N", "2,6-dimethyl-2-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "66719-54-8", "ZFOGJEKQQNVCFB-UHFFFAOYSA-N", "2,6-dimethyl-4-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "42007-73-8", "JJZZVQICFJITMO-UHFFFAOYSA-N", "2,7-dimethyl-2-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "66719-55-9", "VFUOFVONEDBLIJ-UHFFFAOYSA-N", "2,7-dimethyl-3-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "19781-11-4", "FRJOBNOTOHIMIH-UHFFFAOYSA-N", "2,7-dimethyl-4-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "66719-30-0", "JEBZCASOCDXENQ-UHFFFAOYSA-N", "3,4-dimethyl-4-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "56065-42-0", "DMIBTMSBSULJMT-UHFFFAOYSA-N", "3,5-dimethyl-3-octanol", "")
  CALL Species("C_{10}H_{22}O",                        "151-19-9", "NPHCXUPGMINOPP-UHFFFAOYSA-N", "3,6-dimethyl-3-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "66719-31-1", "XFUFFAQPWAGSDK-UHFFFAOYSA-N", "3,6-dimethyl-4-octanol", "")
  CALL Species("C_{10}H_{22}O",                        "106-21-8", "PRNCMAKCNVRZFX-UHFFFAOYSA-N", "3,7-dimethyl-1-octanol", "pelargol")
  CALL Species("C_{10}H_{22}O",                      "15340-96-2", "XCWMPEYBKUYTLZ-UHFFFAOYSA-N", "3,7-dimethyl-2-octanol", "")
  CALL Species("C_{10}H_{22}O",                         "78-69-3", "DLHQZZUEERVIGQ-UHFFFAOYSA-N", "3,7-dimethyl-3-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "66719-32-2", "UQKHXPAWOMZBLN-UHFFFAOYSA-N", "4,5-dimethyl-1-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "66719-33-3", "JOWDWXPTKVUHBV-UHFFFAOYSA-N", "4,6-dimethyl-1-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "56065-43-1", "QJVDAQVYZBSYCB-UHFFFAOYSA-N", "4,6-dimethyl-4-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "66719-34-4", "UWAGZVFFZJGLJU-UHFFFAOYSA-N", "4,7-dimethyl-1-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "19781-13-6", "OTISWHNJCUMBFI-UHFFFAOYSA-N", "4,7-dimethyl-4-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "66719-35-5", "RYFZXYQQFYLUHM-UHFFFAOYSA-N", "7,7-dimethyl-1-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "20592-10-3", "HTRVTKUOKQWGMO-UHFFFAOYSA-N", "2-ethyl-1-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "66719-36-6", "VXOCEIUJVCHLRR-UHFFFAOYSA-N", "3-ethyl-1-octanol", "")
  CALL Species("C_{10}H_{22}O",                       "2051-32-3", "NPQPNSNHYJTUSA-UHFFFAOYSA-N", "3-ethyl-3-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "63126-48-7", "WXJDPIWUPZLVSF-UHFFFAOYSA-N", "3-ethyl-4-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "38395-42-5", "OYBUBRUQIKTRET-UHFFFAOYSA-N", "4-ethyl-4-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "19781-27-2", "JWZFCOOWAQHCBP-UHFFFAOYSA-N", "6-ethyl-3-octanol", "")
  CALL Species("C_{10}H_{22}O",                      "29772-40-5", "GCLFPNQSLCWZAQ-UHFFFAOYSA-N", "2,2,3-trimethyl-3-heptanol", "")
  CALL Species("C_{10}H_{22}O",                      "57233-31-5", "KNACVESXMPHLNM-UHFFFAOYSA-N", "2,2,4-trimethyl-4-heptanol", "")
  CALL Species("C_{10}H_{22}O",                      "66256-42-6", "JHHGGYCQTXZCTL-UHFFFAOYSA-N", "2,2,5-trimethyl-4-heptanol", "")
  CALL Species("C_{10}H_{22}O",                      "66256-43-7", "AXUPPSSNJCYJOX-UHFFFAOYSA-N", "2,2,6-trimethyl-3-heptanol", "")
  CALL Species("C_{10}H_{22}O",                      "66256-44-8", "ITTQYDBLGDGLMB-UHFFFAOYSA-N", "2,2,6-trimethyl-4-heptanol", "")
  CALL Species("C_{10}H_{22}O",                      "58046-40-5", "BAZNZYUOJWCTIQ-UHFFFAOYSA-N", "2,3,6-trimethyl-3-heptanol", "")
  CALL Species("C_{10}H_{22}O",                      "66256-46-0", "RCSCLTAIPNDFGC-UHFFFAOYSA-N", "2,4,5-trimethyl-4-heptanol", "")
  CALL Species("C_{10}H_{22}O",                      "66256-47-1", "VUZHXHGZVRACMM-UHFFFAOYSA-N", "2,4,6-trimethyl-2-heptanol", "")
  CALL Species("C_{10}H_{22}O",                      "60836-07-9", "QSVYJSJPLCSACO-UHFFFAOYSA-N", "2,4,6-trimethyl-4-heptanol", "")
  CALL Species("C_{10}H_{22}O",                      "66256-48-2", "LQQYOLMGDBGEBC-UHFFFAOYSA-N", "2,5,6-trimethyl-2-heptanol", "")
  CALL Species("C_{10}H_{22}O",                                "", "LTDMOZOPTDNTLG-UHFFFAOYSA-N", "3,3,6-trimethyl-4-heptanol", "")
  CALL Species("C_{10}H_{22}O",                      "66256-50-6", "WOUVARFKMJIYBY-UHFFFAOYSA-N", "3,5,5-trimethyl-3-heptanol", "")
  CALL Species("C_{10}H_{22}O",                      "51079-79-9", "FHQUDZUTAZYJRH-UHFFFAOYSA-N", "4,6,6-trimethyl-2-heptanol", "")
  CALL Species("C_{10}H_{22}O",                      "38514-15-7", "NTGBCQFKKOCODF-UHFFFAOYSA-N", "3-(1-methylethyl)-1-heptanol", "")
  CALL Species("C_{10}H_{22}O",                      "51200-82-9", "OHSMKBPEDBXYDU-UHFFFAOYSA-N", "4-(1-methylethyl)-4-heptanol", "")
  CALL Species("C_{10}H_{22}O",                      "66719-37-7", "GYCRUUSYQLIIBA-UHFFFAOYSA-N", "3-ethyl-2-methyl-3-heptanol", "")
  CALL Species("C_{10}H_{22}O",                                "", "FGJMKJVRSBVZPS-UHFFFAOYSA-N", "3-ethyl-5-methyl-3-heptanol", "")
  CALL Species("C_{10}H_{22}O",                      "66731-94-0", "XVSHXHABYNUHNI-UHFFFAOYSA-N", "5-ethyl-4-methyl-3-heptanol", "")
  CALL Species("C_{10}H_{22}O",                      "10042-59-8", "YLQLIQIAXYRMDL-UHFFFAOYSA-N", "2-propyl-1-heptanol", "")
  CALL Species("C_{10}H_{22}O",                       "2198-72-3", "SJTPBRMACCDJPZ-UHFFFAOYSA-N", "4-propyl-4-heptanol", "")
  CALL Species("C_{10}H_{22}O",                      "66256-63-1", "SAFMWGCEQYLRLS-UHFFFAOYSA-N", "2,2,3,4-tetramethyl-3-hexanol", "")
  CALL Species("C_{10}H_{22}O",                      "66256-65-3", "LKAPNEGVXBMYKE-UHFFFAOYSA-N", "2,2,4,4-tetramethyl-3-hexanol", "")
  CALL Species("C_{10}H_{22}O",                      "55073-86-4", "CFEYPBVKHMZCFR-UHFFFAOYSA-N", "2,2,5,5-tetramethyl-3-hexanol", "")
  CALL Species("C_{10}H_{22}O",                      "66256-66-4", "IELWGTFQRBDDCZ-UHFFFAOYSA-N", "2,3,4,4-tetramethyl-2-hexanol", "")
  CALL Species("C_{10}H_{22}O",                      "66256-67-5", "ZQFIWHOYWCJIHU-UHFFFAOYSA-N", "2,3,4,4-tetramethyl-3-hexanol", "")
  CALL Species("C_{10}H_{22}O",                       "5396-09-8", "KTSYBVKVNWGHBC-UHFFFAOYSA-N", "2,3,5,5-tetramethyl-3-hexanol", "")
  CALL Species("C_{10}H_{22}O",                      "66256-68-6", "VVRHXTVKOHXZJX-UHFFFAOYSA-N", "2,4,4,5-tetramethyl-3-hexanol", "")
  CALL Species("C_{10}H_{22}O",                      "66256-69-7", "USWOQOMRDCYQMW-UHFFFAOYSA-N", "3,3,5,5-tetramethyl-2-hexanol", "")
  CALL Species("C_{10}H_{22}O",                      "66256-39-1", "UHJJSHWWLQIIOU-UHFFFAOYSA-N", "3,4,4,5-tetramethyl-3-hexanol", "")
  CALL Species("C_{10}H_{22}O",                      "66256-40-4", "LBLUJHQRJVUQJR-UHFFFAOYSA-N", "3,4,5,5-tetramethyl-3-hexanol", "")
  CALL Species("C_{10}H_{22}O",                      "66719-48-0", "NLGGBBOPZSANIC-UHFFFAOYSA-N", "2,4-dimethyl-4-ethyl-3-hexanol", "")
  CALL Species("C_{10}H_{22}O",                       "2768-15-2", "LAPPDPWPIZBBJY-UHFFFAOYSA-N", "2-butyl-1-hexanol", "")
  CALL Species("C_{10}H_{22}O",                      "66719-47-9", "UOYRABPBVGDJLW-UHFFFAOYSA-N", "4-ethyl-2,2-dimethyl-3-hexanol", "")
  CALL Species("C_{10}H_{22}O",                      "66719-41-3", "VHGVOODVCUKVCL-UHFFFAOYSA-N", "4-methyl-2-(1-methylethyl)-1-hexanol", "")
  CALL Species("C_{10}H_{22}O",                      "66256-62-0", "VZXWJVFQXZUFQS-UHFFFAOYSA-N", "4-methyl-2-propyl-1-hexanol", "")
  CALL Species("C_{10}H_{22}O",                                "", "DXDJMJLAABGHOH-UHFFFAOYSA-N", "5,5-dimethyl-2-ethyl-1-hexanol", "")
  CALL Species("C_{10}H_{22}O",                       "5340-62-5", "FVVBGHIXYRKNGT-UHFFFAOYSA-N", "5,5-dimethyl-3-ethyl-3-hexanol", "")
  CALL Species("C_{10}H_{22}O",                       "2051-33-4", "SFIQHFBITUEIBP-UHFFFAOYSA-N", "5-methyl-2-(1-methylethyl)-1-hexanol", "")
  CALL Species("C_{10}H_{22}O",                      "51200-81-8", "GGOIYPAZTVTJBE-UHFFFAOYSA-N", "2-methyl-3-(1-methylethyl)-3-hexanol", "2,4-dimethyl-3-propyl-3-pentanol")
  CALL Species("C_{10}H_{22}O",                       "5857-69-2", "UYFZQUABCZILAI-UHFFFAOYSA-N", "2,2,3,4,4-pentamethyl-3-pentanol", "")
  CALL Species("C_{10}H_{22}O",                      "81931-81-9", "WQRJEFGWAMKEBO-UHFFFAOYSA-N", "2,3-dimethyl-2-{\it tert}-butyl-1-butanol", "")
  CALL Species("C_{10}H_{22}O",                      "51200-83-0", "IFXNWIUSZUHIDG-UHFFFAOYSA-N", "2,4-dimethyl-3-(1-methylethyl)-3-pentanol", "")
  CALL Species("C_{10}H_{22}O",                      "66719-50-4", "DNBJFVXHMLZKHP-UHFFFAOYSA-N", "3,4-dimethyl-3-isopropyl-2-pentanol", "")
  CALL Species("C_{10}H_{22}O",                      "66719-51-5", "DQWFOEIWLQBIFY-UHFFFAOYSA-N", "4,4-dimethyl-3-isopropyl-1-pentanol", "")
  CALL Species("C_{10}H_{22}O",                      "66256-41-5", "CHZVHXPAKQDRSE-UHFFFAOYSA-N", "3-ethyl-2,2,4-trimethyl-3-pentanol", "")
  CALL Species("C_{10}H_{22}O",                      "22417-45-4", "FXXCTCCAZGTNNO-UHFFFAOYSA-N", "4-methyl-2-(2-methylpropyl)-1-pentanol", "")
  ! >C10:
  CALL Species("C_{11}H_{24}O",                        "112-42-5", "KJIOQYGWTQBHNH-UHFFFAOYSA-N", "1-undecanol", "")
  CALL Species("C_{11}H_{24}O",                       "1653-30-1", "XMUJIPOFTAHSOK-UHFFFAOYSA-N", "2-undecanol", "")
  CALL Species("C_{11}H_{24}O",                       "6929-08-4", "HCARCYFXWDRVBZ-UHFFFAOYSA-N", "3-undecanol", "")
  CALL Species("C_{12}H_{26}O",                        "112-53-8", "LQZZUXJYWNFBMV-UHFFFAOYSA-N", "1-dodecanol", "")
  CALL Species("C_{12}H_{26}O",                      "10203-30-2", "OKDGZLITBCRLLJ-UHFFFAOYSA-N", "3-dodecanol", "")
  CALL Species("C_{12}H_{26}O",                        "123-17-1", "LFEHSRSSAGQWNI-UHFFFAOYSA-N", "2,6,8-trimethyl-4-nonanol", "")
  CALL Species("C_{13}H_{28}O",                        "112-70-9", "XFRVVPUIAFSTFO-UHFFFAOYSA-N", "1-tridecanol", "")
  CALL Species("C_{14}H_{30}O",                        "112-72-1", "HLZKNKRTKFSKGZ-UHFFFAOYSA-N", "1-tetradecanol", "") ! other CAS: 27196-00-5
  CALL Species("C_{14}H_{30}O",                       "4706-81-4", "BRGJIIMZXMWMCC-UHFFFAOYSA-N", "2-tetradecanol", "")
  CALL Species("C_{15}H_{32}O",                        "629-76-5", "REIUXOLGHVXAEO-UHFFFAOYSA-N", "1-pentadecanol", "")
  CALL Species("C_{16}H_{34}O",                        "124-29-8", "BXWNKGSJHAJOGX-UHFFFAOYSA-N", "1-hexadecanol", "cetyl alcohol") ! other CAS: 36653-82-4
  CALL Species("C_{16}H_{34}O",                      "14852-31-4", "FVDRFBGMOWJEOR-UHFFFAOYSA-N", "2-hexadecanol", "")
  CALL Species("C_{17}H_{36}O",                       "1454-85-9", "GOQYKNQRPGWPLP-UHFFFAOYSA-N", "1-heptadecanol", "")
  CALL Species("C_{17}H_{36}O",                      "16813-18-6", "ZNYQHFLBAPNPRC-UHFFFAOYSA-N", "2-heptadecanol", "")
  CALL Species("C_{18}H_{38}O",                        "112-92-5", "GLDOVTGHNKAZLK-UHFFFAOYSA-N", "1-octadecanol", "")
  CALL Species("C_{19}H_{40}O",                       "1454-84-8", "XGFDHKJUZCCPKQ-UHFFFAOYSA-N", "1-nonadecanol", "")
  CALL Species("C_{20}H_{42}O",                        "629-96-9", "BTFJIXJJCSYFAL-UHFFFAOYSA-N", "1-eicosanol", "")
  CALL Species("C_{22}H_{46}O",                        "661-19-8", "NOPFSRXAKWQILS-UHFFFAOYSA-N", "1-docosanol", "behenic alcohol")
  CALL Species("C_{24}H_{50}O",                        "506-51-4", "TYWMIZZBOVGFOV-UHFFFAOYSA-N", "1-tetracosanol", "")
  ! cyclic:
  CALL Species("C_5H_9OH",                              "96-41-3", "XCIXKGXIYUWCLL-UHFFFAOYSA-N", "cyclopentanol", "")
  CALL Species("C_6H_{11}OH",                          "108-93-0", "HPXRVTGHNJAIIH-UHFFFAOYSA-N", "cyclohexanol", "")
  CALL Species("C_7H_{13}OH",                          "502-41-0", "QCRFMSUKWRQZEM-UHFFFAOYSA-N", "cycloheptanol", "")
  CALL Species("C_7H_{13}OH",                          "589-91-3", "MQWCXKGKQLNYQG-UHFFFAOYSA-N", "4-methylcyclohexanol", "")
  CALL Species("C_7H_{14}O",                           "583-59-5", "NDVWOBYBJYUSMF-UHFFFAOYSA-N", "2-methylcyclohexanol", "")
  CALL Species("C_7H_{14}O",                           "591-23-1", "HTSABYAWKQAHBT-UHFFFAOYSA-N", "3-methylcyclohexanol", "")
  CALL Species("C_8H_{14}O",                                   "", "CEOBYJRLFASWKF-UHFFFAOYSA-N", "MCM:C8BCOH", "")
  CALL Species("C_8H_{16}O",                         "34885-03-5", "OSINZLLLLCUKJH-UHFFFAOYSA-N", "(4-methylcyclohexyl)methanol", "")
  CALL Species("C_{10}H_{18}O",                        "507-70-0", "DTGKSKDOIYIVQL-UHFFFAOYSA-N", "borneol", "")
  CALL Species("C_{10}H_{18}O",                       "1632-73-1", "IAIHUHQCLTYTSF-UHFFFAOYSA-N", "fenchol", "fenchyl alcohol")
  CALL Species("C_{10}H_{20}O",                       "1490-04-6", "NOOLISFMXDJSKH-UHFFFAOYSA-N", "menthol", "")
  CALL Species("C_{11}H_{20}O",                       "2371-42-8", "LFYXNXGVLGKVCJ-BOBPJJCASA-N", "2-methylisoborneol", "")
  CALL Species("C_{12}H_{22}O",                      "19700-21-1", "JLPUXFOGCDVKGO-GRYCIOLGSA-N", "geosmin", "")
  CALL Species("C_{12}H_{24}O",                       "1724-39-6", "SFVWPXMPRCIVOK-UHFFFAOYSA-N", "cyclododecanol", "")
  CALL Species("C_{15}H_{26}O",                         "77-53-2", "SVURIXNDRWRAFU-OGMFBOKVSA-N", "cedrol", "")
  CALL Species("C_{15}H_{26}O",                       "5986-55-0", "GGHMUJBZYLPWFD-CUZKYEQNSA-N", "patchoulol", "")
  CALL Species("C_{15}H_{28}O_2",                       "80-04-6", "CDBAMNGURPMUTG-UHFFFAOYSA-N", "perhydrobisphenol a", "")
  CALL Species("C_{16}H_{28}O",                       "3407-42-9", "BWVZAZPLUTUBKD-UHFFFAOYSA-N", "3-(5,5,6-trimethyl-2-norbornyl)cyclohexanol", "")
  CALL Species("C_{16}H_{28}O",                      "66068-84-6", "PCFHYANYPQEMPU-UHFFFAOYSA-N", "4-(5,5,6-trimethylbicyclo[2.2.1]hept-2-yl)cyclohexan-1-ol", "")
  CALL Species("C_{16}H_{28}O",                      "66072-32-0", "LFHQKYSBKVWWOS-UHFFFAOYSA-N", "4-((1R,2R,4R)-born-2-yl)cyclohexanol", "")
  ! unsaturated:
  CALL Species("C_2H_4O",                              "557-75-5", "IMROMDMJAWUWLK-UHFFFAOYSA-N", "ethenol", "vinyl alcohol")
  CALL Species("C_3H_5OH",                             "107-18-6", "XXROGKLTLUQVRX-UHFFFAOYSA-N", "2-propen-1-ol", "allyl alcohol")
  CALL Species("C_3H_4O",                              "107-19-7", "TVDSBUOJIPERQY-UHFFFAOYSA-N", "2-propyn-1-ol", "propargyl alcohol")
  CALL Species("CH_3CHCHCH_2OH",                      "6117-91-5", "WCASXYBKJHWFMY-UHFFFAOYSA-N", "2-buten-1-ol", "")
  CALL Species("C_4H_8O",                              "627-27-0", "ZSPTYLOMNJNZNG-UHFFFAOYSA-N", "3-buten-1-ol", "")
  CALL Species("C_5H_{10}O",                           "821-09-0", "LQAVWYMTUMSFBE-UHFFFAOYSA-N", "4-penten-1-ol", "")
  CALL Species("C_5H_8O",                            "10229-10-4", "IDYNOORNKYEHHO-UHFFFAOYSA-N", "3-pentyn-1-ol", "")
  CALL Species("C_5H_9O",                              "115-18-4", "HNVRRHSXBLFLIG-UHFFFAOYSA-N", "2-methyl-3-buten-2-ol", "")
  CALL Species("C_5H_8O",                              "115-19-5", "CEBKHWWANWSNTI-UHFFFAOYSA-N", "2-methyl-3-butyn-2-ol", "")
  CALL Species("C_6H_{10}O",                            "77-75-8", "QXLPXWSKPNOQLE-UHFFFAOYSA-N", "3-methyl-1-pentyn-3-ol", "meparfynol; methyl pentynol")
  CALL Species("C_6H_{10}O",                           "822-67-3", "PQANGXXSEABURG-UHFFFAOYSA-N", "2-cyclohexen-1-ol", "")
  CALL Species("C_7H_{12}O",                          "1632-68-4", "ZQTYQMYDIHMKQB-UHFFFAOYSA-N", "bicyclo[2.2.1]heptan-2-ol", "norborneol")
  CALL Species("C_8H_{16}O",                         "22104-78-5", "AYQPVPFZWIQERS-UHFFFAOYSA-N", "2-octen-1-ol", "")
  CALL Species("C_8H_{16}O",                          "3391-86-4", "VSMOENVRRABVKN-UHFFFAOYSA-N", "1-octen-3-ol", "")
  CALL Species("C_{10}H_{20}O",                        "106-22-9", "QMVPMAAFGQKVCJ-UHFFFAOYSA-N", "3,7-dimethyl-6-octen-1-ol", "citronellol")
  CALL Species("C_{10}H_{18}O",                         "78-70-6", "CDOSHBSSFJOMGT-UHFFFAOYSA-N", "3,7-dimethyl-1,6-octadien-3-ol", "linalool")
  CALL Species("C_{10}H_{18}O_2",                     "1365-19-1", "BXOURKNXQXLKRK-UHFFFAOYSA-N", "linalool oxide", "")
  CALL Species("C_{10}H_{18}O",                        "106-24-1", "GLZPCOQZEFWAFX-JXMROGBWSA-N", "($E$)-3,7-dimethyl-2,6-octadien-1-ol", "geraniol")
  CALL Species("C_{10}H_{18}O",                        "106-25-2", "GLZPCOQZEFWAFX-YFHOEESVSA-N", "($Z$)-3,7-dimethyl-2,6-octadien-1-ol", "")
  CALL Species("C_{10}H_{16}O",                        "768-95-6", "VLLNJDMHDJRNFK-UHFFFAOYSA-N", "tricyclo[3.3.1.1(3,7)]decan-1-ol", "1-adamantanol")
  CALL Species("C_{15}H_{26}O",                       "4602-84-0", "CRDAMVZIKSXKFV-YFVJMOTDSA-N", "3,7,11-trimethyl-2,6,10-dodecatrien-1-ol", "farnesol")
  CALL Species("C_{18}H_{36}O",                        "143-28-2", "ALSTYHKOOCGGFT-KTKRTIGZSA-N", "($Z$)-9-octadecen-1-ol", "oleyl alcohol")
  CALL Species("C_{18}H_{34}O",                      "66410-28-4", "QBNCGBJHGBGHLS-IYUNJCAYSA-N", "(3$E$,13$Z$)-octadeca-3,13-dien-1-ol", "")
  CALL Species("C_{20}H_{34}O",                      "26266-77-3", "FLMIYUXOBAUKJM-UHFFFAOYSA-N", "dihydroabietyl alcohol", "")
  CALL Species("C_{20}H_{32}O",                        "965-90-2", "AOXRBFRFYPMWLR-UHFFFAOYSA-N", "ethylestrenol", "")
  CALL Species("C_{20}H_{40}O",                        "150-86-7", "BOTWFXYSPFMFNR-PYDDKJGSSA-N", "{\it trans}-phytol", "")
  ! aromatic:
  CALL Species("C_6H_5OH",                             "108-95-2", "ISWSIDIOOBJBQZ-UHFFFAOYSA-N", "hydroxybenzene", "phenol")
  CALL Species("C_6H_5CH_2OH",                         "100-51-6", "WVDDGKGOMKODPV-UHFFFAOYSA-N", "(hydroxymethyl)-benzene", "benzyl alcohol")
  CALL Species("HOC_6H_4CH_3",                          "95-48-7", "QWVGKYWNOKOFNN-UHFFFAOYSA-N", "1-hydroxy-2-methylbenzene", "2-cresol; $o$-cresol")
  CALL Species("HOC_6H_4CH_3",                         "108-39-4", "RLSSMJSEOOYNOY-UHFFFAOYSA-N", "1-hydroxy-3-methylbenzene", "3-cresol; $m$-cresol")
  CALL Species("HOC_6H_4CH_3",                         "106-44-5", "IWDCLRJOBJJRNH-UHFFFAOYSA-N", "1-hydroxy-4-methylbenzene", "4-cresol; $p$-cresol")
  CALL Species("C_8H_{10}O",                           "526-75-0", "QWBBPBRQALCEIZ-UHFFFAOYSA-N", "1-hydroxy-2,3-dimethylbenzene", "2,3-xylenol; 2,3-dimethylphenol")
  CALL Species("C_8H_{10}O",                           "105-67-9", "KUFFULVDNCHOFZ-UHFFFAOYSA-N", "1-hydroxy-2,4-dimethylbenzene", "2,4-xylenol; 2,4-dimethylphenol")
  CALL Species("C_8H_{10}O",                            "95-87-4", "NKTOLZVEWDHZMU-UHFFFAOYSA-N", "1-hydroxy-2,5-dimethylbenzene", "2,5-xylenol; 2,5-dimethylphenol")
  CALL Species("C_8H_{10}O",                           "576-26-1", "NXXYKOUNUYWIHA-UHFFFAOYSA-N", "1-hydroxy-2,6-dimethylbenzene", "2,6-xylenol; 2,6-dimethylphenol")
  CALL Species("C_8H_{10}O",                            "95-65-8", "YCOXTKKNXUZSKD-UHFFFAOYSA-N", "1-hydroxy-3,4-dimethylbenzene", "3,4-xylenol; 3,4-dimethylphenol")
  CALL Species("C_8H_{10}O",                           "108-68-9", "TUAMRELNJMMDMT-UHFFFAOYSA-N", "1-hydroxy-3,5-dimethylbenzene", "3,5-xylenol; 3,5-dimethylphenol")
  CALL Species("C_8H_{10}O",                            "89-95-2", "XPNGNIFUDRPBFJ-UHFFFAOYSA-N", "2-methylbenzenemethanol", "")
  CALL Species("C_8H_{10}O",                           "587-03-1", "JJCKHVUTVOPLBV-UHFFFAOYSA-N", "3-methylbenzenemethanol", "")
  CALL Species("C_8H_{10}O",                           "589-18-4", "KMTDMTZBNYGUNX-UHFFFAOYSA-N", "4-methylbenzenemethanol", "")
  CALL Species("C_8H_{10}O",                            "98-85-1", "WAPNOHKVXSQRPX-UHFFFAOYSA-N", "$\alpha$-methylbenzyl alcohol", "")
  CALL Species("C_8H_{10}O",                            "90-00-6", "IXQGCWUGDFDQMF-UHFFFAOYSA-N", "1-hydroxy-2-ethylbenzene", "2-ethylphenol")
  CALL Species("C_8H_{10}O",                           "620-17-7", "HMNKTRSOROOSPP-UHFFFAOYSA-N", "1-hydroxy-3-ethylbenzene", "3-ethylphenol")
  CALL Species("C_8H_{10}O",                           "123-07-9", "HXDOZKJGKXYMEW-UHFFFAOYSA-N", "1-hydroxy-4-ethylbenzene", "4-ethylphenol")
  CALL Species("C_9H_{12}O",                           "526-85-2", "XRUGBBIQLIVCSI-UHFFFAOYSA-N", "2,3,4-trimethylphenol", "")
  CALL Species("C_9H_{12}O",                           "697-82-5", "OGRAOKJKVGDSFR-UHFFFAOYSA-N", "2,3,5-trimethylphenol", "")
  CALL Species("C_9H_{12}O",                          "2416-94-6", "QQOMQLYQAXGHSU-UHFFFAOYSA-N", "2,3,6-trimethylphenol", "")
  CALL Species("C_9H_{12}O",                           "527-60-6", "BPRYUXCVCCNUFE-UHFFFAOYSA-N", "2,4,6-trimethylphenol", "")
  CALL Species("C_9H_{12}O",                           "527-54-8", "FDQQNNZKEJIHMS-UHFFFAOYSA-N", "3,4,5-trimethylphenol", "")
  CALL Species("C_9H_{12}O",                           "645-56-7", "KLSLBUSXWBJMEC-UHFFFAOYSA-N", "1-hydroxy-4-propylbenzene", "4-propylphenol")
  CALL Species("C_9H_{12}O",                            "88-69-7", "CRBJBYGJVIBWIY-UHFFFAOYSA-N", "2-(1-methylethyl)-phenol", "")
  CALL Species("C_9H_{12}O",                           "617-94-7", "BDCFWIDZNLCTMF-UHFFFAOYSA-N", "2-phenylisopropanol", "")
  CALL Species("C_9H_{12}O",                          "6161-62-2", "OCKYMBMCPOAFLL-UHFFFAOYSA-N", "2-ethyl-3-methylphenol", "")
  CALL Species("C_9H_{12}O",                          "1687-61-2", "LTRVUFFOMIUCPJ-UHFFFAOYSA-N", "2-ethyl-5-methylphenol", "")
  CALL Species("C_9H_{12}O",                          "1687-64-5", "CIRRFAQIWQFQSS-UHFFFAOYSA-N", "2-ethyl-6-methylphenol", "")
  CALL Species("C_9H_{12}O",                           "698-71-5", "XTCHLXABLZQNNN-UHFFFAOYSA-N", "5-ethyl-3-methylphenol", "")
  CALL Species("C_9H_{12}O",                          "7287-82-3", "SDCBYRLJYGORNK-UHFFFAOYSA-N", "1-(2-methylphenyl)ethanol", "")
  CALL Species("C_9H_{12}O",                           "644-35-9", "LCHYEKKJCUJAKN-UHFFFAOYSA-N", "2-propylphenol", "")
  CALL Species("C_9H_{12}O",                            "93-54-9", "DYUQAZSOFZSPHD-UHFFFAOYSA-N", "1-phenyl-1-propanol", "")
  CALL Species("C_9H_{12}O",                         "13651-14-4", "ZQQIVMXQYUZKIQ-UHFFFAOYSA-N", "2,3-dimethylbenzyl alcohol", "")
  CALL Species("C_9H_{12}O",                          "6966-10-5", "OKGZCXPDJKKZAP-UHFFFAOYSA-N", "3,4-dimethylbenzyl alcohol", "")
  CALL Species("C_9H_{12}O",                         "27129-87-9", "IQWWTJDRVBWBEL-UHFFFAOYSA-N", "3,5-dimethylbenzyl alcohol", "")
  CALL Species("C_{10}H_{14}O",                         "88-18-6", "WJQOZHYUIDYNHM-UHFFFAOYSA-N", "2-(1,1-dimethylethyl)-phenol", "")
  CALL Species("C_{10}H_{14}O",                        "585-34-2", "CYEKUDPFXBLGHH-UHFFFAOYSA-N", "3-(1,1-dimethylethyl)-phenol", "")
  CALL Species("C_{10}H_{14}O",                         "89-72-5", "NGFPWHGISWUQOI-UHFFFAOYSA-N", "2-(1-methylpropyl)phenol", "")
  CALL Species("C_{10}H_{14}O",                         "99-71-8", "ZUTYZAFDFLLILI-UHFFFAOYSA-N", "4-(1-methylpropyl)-phenol", "4-{\it sec}-butylphenol")
  CALL Species("C_{10}H_{14}O",                         "98-54-4", "QHPQWRBYOIRBIT-UHFFFAOYSA-N", "4-{\it tert}-butylphenol", "")
  CALL Species("C_{10}H_{14}O",                        "499-75-2", "RECUKUPTGUEGMW-UHFFFAOYSA-N", "2-methyl-5-(1-methylethyl)-phenol", "carvacrol")
  CALL Species("C_{10}H_{14}O",                         "89-83-8", "MGSRCZKZVOBKFT-UHFFFAOYSA-N", "5-methyl-2-(1-methylethyl)-phenol", "thymol")
  CALL Species("C_{10}H_{14}O",                       "5379-18-0", "RHBAJFPGUNNLFA-UHFFFAOYSA-N", "1-(3,5-dimethylphenyl)ethanol", "")
  CALL Species("C_{10}H_{14}O",                       "2219-79-6", "MXHAHSBTOVFDBK-UHFFFAOYSA-N", "6-ethyl-2,4-xylenol", "")
  CALL Species("C_{11}H_{16}O",                       "2409-55-4", "IKEHOXWJQXIQAG-UHFFFAOYSA-N", "2-(1,1-dimethylethyl)-4-methylphenol", "")
  CALL Species("C_{11}H_{16}O",                         "80-46-6", "NRZWYNLTFLDQQX-UHFFFAOYSA-N", "4-(1,1-dimethylpropyl)phenol", "")
  CALL Species("C_{11}H_{16}O",                                "", "OZZOZHVUMWFILP-UHFFFAOYSA-N", "MCM:DE35TOH", "")
  CALL Species("C_{11}H_{16}O",                                "", "VUDWBUZLGAXSDX-UHFFFAOYSA-N", "MCM:DEMPHOH", "")
  CALL Species("C_{11}H_{14}O",                      "56138-10-4", "WWHATUWNHYNMNW-UHFFFAOYSA-N", "josenol", "2-methyl-3-(4-methylphenyl)-2-propen-1-ol")
  CALL Species("C_{12}H_{16}O",                     "285977-85-7", "UWSPWQQZFOSTHS-UHFFFAOYSA-N", "lilyflore", "")
  CALL Species("C_{13}H_{12}O",                         "91-01-0", "QILSFLSDHQAZET-UHFFFAOYSA-N", "benzhydrol", "")
  CALL Species("C_{14}H_{22}O",                       "1806-26-4", "NTDQQZYCCIDJRK-UHFFFAOYSA-N", "1-hydroxy-4-octylbenzene", "4-octylphenol")
  CALL Species("C_{14}H_{22}O",                        "140-66-9", "ISAVYTVYFVQUDY-UHFFFAOYSA-N", "4-(1,1,3,3-tetramethylbutyl)-phenol", "$p$-{\it tert}-octylphenol")
  CALL Species("C_{15}H_{24}O",                        "104-40-5", "IGFHQQFPSIBGKE-UHFFFAOYSA-N", "1-hydroxy-4-nonylbenzene", "4-nonylphenol")
  CALL Species("C_{15}H_{24}O",                        "128-37-0", "NLZUEZXRPGMBCV-UHFFFAOYSA-N", "2,6-bis(1,1-dimethylethyl)-4-methylphenol", "butylated hydroxytoluene; BHT")
  CALL Species("C_{15}H_{24}O",                                "", "RYIHVIPUIXFRNI-IUODEOHRSA-N", "4-(3',5'-dimethyl-3'-heptyl)-phenol(+)", "") ! +/- assignment may be incorrect
  CALL Species("C_{15}H_{24}O",                                "", "RYIHVIPUIXFRNI-SWLSCSKDSA-N", "4-(3',5'-dimethyl-3'-heptyl)-phenol(-)", "") ! +/- assignment may be incorrect
  CALL Species("C_8H_{10}O",                            "60-12-8", "WRMNZCZEMHIOCP-UHFFFAOYSA-N", "2-phenylethanol", "")
  CALL Species("C_9H_{12}O",                           "122-97-4", "VAJVDSVGBWFCLW-UHFFFAOYSA-N", "3-phenyl-1-propanol", "")
  CALL Species("C_{10}H_{14}O",                       "3360-41-6", "LDZLXQFDGRCELX-UHFFFAOYSA-N", "4-phenyl-1-butanol", "")
  CALL Species("C_{10}H_8O",                            "90-15-3", "KJCVRFUGPWSIIH-UHFFFAOYSA-N", "1-naphthalenol", "1-naphthol")
  CALL Species("C_{10}H_8O",                           "135-19-3", "JWAZRIHNYRIHIV-UHFFFAOYSA-N", "2-naphthalenol", "2-naphthol")
  CALL Species("C_{12}H_{10}O",                         "90-43-7", "LLEMOWNGBBNAJR-UHFFFAOYSA-N", "$o$-hydroxybiphenyl", "")
  CALL Species("C_{12}H_{10}O",                         "92-69-3", "YXVFYQXJAXKLAK-UHFFFAOYSA-N", "$p$-hydroxybiphenyl", "")
  CALL Species("C_{18}H_{30}O",                        "732-26-3", "PFEFOYRSMXVNEL-UHFFFAOYSA-N", "2,4,6-tris(1,1-dimethylethyl)phenol", "")
  CALL Species("C_{20}H_{30}O",                       "3772-55-2", "WSKGRAGZAQRSED-IOJLRTSASA-N", "dehydroabietol", "")
  CALL Species("C_{23}H_{32}O_2",                      "119-47-1", "KGRVJHAUYBGFFP-UHFFFAOYSA-N", "2,2'-methylenebis(6-(1,1-dimethylethyl)-4-methylphenol)", "")
  CALL Species("C_{24}H_{42}O",                        "137-99-5", "FDAJTLLBHNHECW-UHFFFAOYSA-N", "2,4-dinonylphenol", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Polyols (\chem{R(OH)_n})")
  ! aliphatic, saturated:
  CALL Species("HOCH_2OH",                             "463-57-0", "CKFGINPQOCXMAZ-UHFFFAOYSA-N", "methanediol", "")
  CALL Species("CH_3CH_2(OH)_2",                      "4433-56-1", "AZHSSKPUVBVXLK-UHFFFAOYSA-N", "1,1-ethanediol", "")
  CALL Species("HO(CH_2)_2OH",                         "107-21-1", "LYCAIKOWRPUZTN-UHFFFAOYSA-N", "1,2-ethanediol", "ethylene glycol")
  CALL Species("C_3H_8O_2",                             "57-55-6", "DNIAPMSPPWPWGF-UHFFFAOYSA-N", "1,2-propanediol", "propylene glycol")
  CALL Species("C_3H_8O_2",                            "504-63-2", "YPFDHNVEDLHUCE-UHFFFAOYSA-N", "1,3-propanediol", "")
  CALL Species("C_3H_8O_3",                             "56-81-5", "PEDCQBHIVMGVHV-UHFFFAOYSA-N", "1,2,3-propanetriol", "glycerol")
  CALL Species("C_4H_{10}O_2",                         "584-03-2", "BMRWNKZVCUKKSR-UHFFFAOYSA-N", "1,2-butanediol", "")
  CALL Species("C_4H_{10}O_2",                         "107-88-0", "PUPZLCDOIYMWBV-UHFFFAOYSA-N", "1,3-butanediol", "")
  CALL Species("C_4H_{10}O_2",                         "110-63-4", "WERYXYBDKMZEQL-UHFFFAOYSA-N", "1,4-butanediol", "")
  CALL Species("C_4H_{10}O_2",                         "513-85-9", "OWBTYPJTUOEWEK-UHFFFAOYSA-N", "2,3-butanediol", "")
  CALL Species("C_4H_{10}O_2",                        "5341-95-7", "OWBTYPJTUOEWEK-ZXZARUISSA-N", "{\it meso}-2,3-butanediol", "")
  CALL Species("C_4H_{10}O_2",                         "558-43-0", "BTVWZWFKMIUSGS-UHFFFAOYSA-N", "2-methylpropane-1,2-diol", "")
  CALL Species("C_4H_{10}O_2",                        "2163-42-0", "QWGRWMMWNDWRQN-UHFFFAOYSA-N", "2-methylpropane-1,3-diol", "")
  CALL Species("C_4H_{10}O_3",                        "4435-50-1", "YAXKTBLXMTYWDQ-UHFFFAOYSA-N", "1,2,3-butanetriol", "")
  CALL Species("C_4H_{10}O_3",                        "3068-00-6", "ARXKVVRQIIOZGF-UHFFFAOYSA-N", "1,2,4-butanetriol", "")
  CALL Species("C_4H_{10}O_4",                        "7541-59-5", "UNXHWFMMPAWVPI-UHFFFAOYSA-N", "1,2,3,4-butanetetrol", "1,2,3,4-tetrahydroxybutane; tetritol")
  CALL Species("C_4H_{10}O_4",                         "149-32-6", "UNXHWFMMPAWVPI-ZXZARUISSA-N", "2($R$),3($S$)-1,2,3,4-butanetetrol", "erythritol")
  CALL Species("C_5H_{12}O_2",                        "5343-92-0", "WCVRQHFDJLLWFE-UHFFFAOYSA-N", "1,2-pentanediol", "")
  CALL Species("C_5H_{12}O_2",                        "3174-67-2", "RUOPINZRYMFPBF-UHFFFAOYSA-N", "1,3-pentanediol", "")
  CALL Species("C_5H_{12}O_2",                         "626-95-9", "GLOBUAZSRIOKLN-UHFFFAOYSA-N", "1,4-pentanediol", "")
  CALL Species("C_5H_{12}O_2",                         "111-29-5", "ALQSHHUCVQOPAS-UHFFFAOYSA-N", "1,5-pentanediol", "")
  CALL Species("C_5H_{12}O_2",                       "42027-23-6", "XLMFDCKSFJWJTP-UHFFFAOYSA-N", "2,3-pentanediol", "")
  CALL Species("C_5H_{12}O_2",                         "625-69-4", "GTCCGKPBSJZVRZ-UHFFFAOYSA-N", "2,4-pentanediol", "")
  CALL Species("C_5H_{12}O_2",                       "50468-22-9", "HJJZIMFAIMUSBW-UHFFFAOYSA-N", "3-methylbutane-1,2-diol", "")
  CALL Species("C_5H_{12}O_2",                        "2568-33-4", "XPFCZYUVICHKDS-UHFFFAOYSA-N", "3-methylbutane-1,3-diol", "")
  CALL Species("C_5H_{12}O_2",                       "41051-72-3", "DOPZLYNWNJHAOS-UHFFFAOYSA-N", "2-methylbutane-1,2-diol", "")
  CALL Species("C_5H_{12}O_2",                         "684-84-4", "GNBPEYCZELNJMS-UHFFFAOYSA-N", "2-methylbutane-1,3-diol", "")
  CALL Species("C_5H_{12}O_2",                        "2938-98-9", "MWCBGWLCXSUTHK-UHFFFAOYSA-N", "2-methylbutane-1,4-diol", "")
  CALL Species("C_5H_{12}O_2",                        "5396-58-7", "IDEOPBXRUBNYBN-UHFFFAOYSA-N", "2-methylbutane-2,3-diol", "")
  CALL Species("C_5H_{12}O_2",                         "126-30-7", "SLCVBVWXLSEKPL-UHFFFAOYSA-N", "2,2-dimethylpropane-1,3-diol", "")
  CALL Species("C_5H_{12}O_3",                          "77-85-0", "QXJQHYBHAIHNGG-UHFFFAOYSA-N", "2-(hydroxymethyl)-2-methyl-1,3-propanediol", "")
  CALL Species("C_5H_{12}O_3",                       "62875-07-4", "XYHGSPUTABMVOC-UHFFFAOYSA-N", "2-methylbutane-1,2,4-triol", "")
  CALL Species("C_5H_{12}O_3",                                 "", "RXEJCNRKXVSXDJ-UHFFFAOYSA-N", "MCM:C56OH", "")
  CALL Species("C_5H_{12}O_3",                                 "", "MOIOWCZVZKHQIC-UHFFFAOYSA-N", "MCM:HO124C5", "")
  CALL Species("C_5H_{12}O_3",                                 "", "ANUUQAHHEZMTAS-UHFFFAOYSA-N", "MCM:HO134C5", "")
  CALL Species("C_5H_{12}O_3",                                 "", "QFZITDCVRJQLMZ-UHFFFAOYSA-N", "MCM:MBOAOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "MTMASQITBHTGFW-UHFFFAOYSA-N", "2-pentene-2,4,5-triol", "ME1TRIOL") ! SMILES: C(O)C(O)C=C(C)(O)
  CALL Species("C_5H_{10}O_3",                                 "", "YXPJRFSYSVRDQI-UHFFFAOYSA-N", "2-methyl-3-butene-1,2,3-triol", "")
  CALL Species("C_5H_{10}O_3",                                 "", "FVYHKVMWBMOFGS-DUXPYHPUSA-N", "(E)-2-methyl-1-butene-1,3,4-triol", "")
  CALL Species("C_5H_{10}O_3",                                 "", "FVYHKVMWBMOFGS-RQOWECAXSA-N", "(Z)-2-methyl-1-butene-1,3,4-triol", "")
  CALL Species("C_5H_{12}O_4",                                 "", "HGVJFBSSLICXEM-WHFBIAKZSA-N", "2-methylthreitol", "")
  CALL Species("C_5H_{12}O_4",                       "93921-83-6", "HGVJFBSSLICXEM-CRCLSJGQSA-N", "2-methylerythritol", "")
  CALL Species("C_5H_{12}O_4",                       "42933-13-1", "HGVJFBSSLICXEM-UHFFFAOYSA-N", "2-Methylbutane-1,2,3,4-tetrol", "MeBuTETROL") ! SMILES: C(O)C(O)C(C)(O)C(O)
  CALL Species("C_5H_{12}O_4",                         "115-77-5", "WXZMFSXDPGVJKK-UHFFFAOYSA-N", "2,2-bis(hydroxymethyl)1,3-propanediol", "pentaerythritol")
  CALL Species("C_5H_{12}O_5",                        "7643-75-6", "HEBKCHPVOIAQTA-UHFFFAOYSA-N", "1,2,3,4,5-pentanepentol", "")
  CALL Species("C_5H_{12}O_5",                          "87-99-0", "HEBKCHPVOIAQTA-SCDXWVJYSA-N", "(2R,3R,4S)-pentane-1,2,3,4,5-pentol", "xylitol")
  CALL Species("C_5H_{12}O_5",                         "488-81-3", "HEBKCHPVOIAQTA-ZXFHETKHSA-N", "(2R,3S,4S)-pentane-1,2,3,4,5-pentol", "adonitol; ribitol")
  CALL Species("C_5H_{12}O_5",                        "2152-56-9", "HEBKCHPVOIAQTA-QWWZWVQMSA-N", "(2R,4R)-pentane-1,2,3,4,5-pentol", "arabitol; arabinitol") ! also includes (2S,4S) enatiomer = HEBKCHPVOIAQTA-IMJSIDKUSA-N
  CALL Species("C_6H_{10}O_5",                         "498-07-7", "TWNIBLMWSKIRAT-VFUOTHLCSA-N", "levoglucosan", "")
  CALL Species("C_6H_{12}O_6",                       "30237-26-4", "BJHIKXHVCXFQLS-UYFOZJQFSA-N", "fructose", "")
  CALL Species("C_6H_{12}O_6",                        "3458-28-4", "GZCGUPFRVQAUEE-KVTDHHQDSA-N", "mannose", "")
  CALL Species("C_6H_{12}O_6",                          "50-99-7", "GZCGUPFRVQAUEE-SLPGGIOYSA-N", "glucose", "")
  CALL Species("C_6H_{14}O_2",                        "6920-22-5", "FHKSXSQHXQEMOK-UHFFFAOYSA-N", "1,2-hexanediol", "")
  CALL Species("C_6H_{14}O_2",                       "16432-53-4", "QVTWBMUAJHVAIJ-UHFFFAOYSA-N", "1,4-hexanediol", "")
  CALL Species("C_6H_{14}O_2",                         "629-11-8", "XXMIOPMDWAUFGU-UHFFFAOYSA-N", "1,6-hexanediol", "")
  CALL Species("C_6H_{14}O_2",                         "617-30-1", "QCIYAEYRVFUFAP-UHFFFAOYSA-N", "2,3-hexanediol", "")
  CALL Species("C_6H_{14}O_2",                        "2935-44-6", "OHMBHFSEKCCCBW-UHFFFAOYSA-N", "2,5-hexanediol", "")
  CALL Species("C_6H_{12}O_2",                         "931-17-9", "PFURGBBHAOXLIO-UHFFFAOYSA-N", "1,2-cyclohexanediol", "")
  CALL Species("C_6H_{14}O_2",                         "149-31-5", "SPXWGAHNKXLXAP-UHFFFAOYSA-N", "2-methyl-1,3-pentanediol", "")
  CALL Species("C_6H_{14}O_2",                        "6287-17-8", "PNJNLCNHYSWUPT-UHFFFAOYSA-N", "2-methyl-1,4-pentanediol", "")
  CALL Species("C_6H_{14}O_2",                         "107-41-5", "SVTBMSDMJJWYQN-UHFFFAOYSA-N", "2-methyl-2,4-pentanediol", "")
  CALL Species("C_6H_{14}O_2",                       "33879-72-0", "HIYKOZFIVZIBFO-UHFFFAOYSA-N", "3-methyl-1,3-pentanediol", "")
  CALL Species("C_6H_{14}O_2",                       "26787-63-3", "WOHXXIWTEHLCQK-UHFFFAOYSA-N", "3-methyl-1,4-pentanediol", "")
  CALL Species("C_6H_{14}O_2",                        "1462-10-8", "HAIVWDGLCRYQMC-UHFFFAOYSA-N", "4-methyl-1,4-pentanediol", "")
  CALL Species("C_6H_{14}O_2",                          "76-09-5", "IVDFJHOHABJVEH-UHFFFAOYSA-N", "2,3-dimethyl-2,3-butanediol", "")
  CALL Species("C_6H_{14}O_2",                       "57716-80-0", "SKQUTIPQJKQFRA-UHFFFAOYSA-N", "2,3-dimethyl-1,4-butanediol", "")
  CALL Species("C_6H_{14}O_2",                       "32812-23-0", "GQSZUUPRBBBHRI-UHFFFAOYSA-N", "2,2-dimethyl-1,4-butanediol", "")
  CALL Species("C_6H_{14}O_3",                         "106-69-4", "ZWVMLYRJXORSEP-UHFFFAOYSA-N", "1,2,6-hexanetriol", "")
  CALL Species("C_6H_{14}O_3",                                 "", "XPUHQLFLUQJINS-UHFFFAOYSA-N", "MCM:H134M3C5", "")
  CALL Species("C_6H_{14}O_6",                       "45007-61-2", "FBPFZTCFMRRESA-UHFFFAOYSA-N", "1,2,3,4,5,6-hexahydroxy hexane", "")
  CALL Species("C_6H_{14}O_6",                          "50-70-4", "FBPFZTCFMRRESA-JGWLITMVSA-N", "(2S,3R,4R,5R)-hexane-1,2,3,4,5,6-hexol", "sorbitol")
  CALL Species("C_6H_{14}O_6",                          "69-65-8", "FBPFZTCFMRRESA-KVTDHHQDSA-N", "(2R,3R,4R,5R)-hexane-1,2,3,4,5,6-hexol", "mannitol")
  CALL Species("C_6H_{14}O_6",                         "608-66-2", "FBPFZTCFMRRESA-GUCUJZIJSA-N", "(2R,3S,4R,5S)-hexane-1,2,3,4,5,6-hexol", "dulcitol; galactitol")
  CALL Species("C_6H_{12}O_4",                       "35652-37-0", "RDIDGZFQASQXBU-UHFFFAOYSA-N", "1,2,4,5-cyclohexanetetrol", "1,2,4,5-tetrahydroxycyclohexane")
  CALL Species("C_6H_{12}O_6",                          "87-89-8", "CDAISMWEOUEBRE-UHFFFAOYSA-N", "1,2,3,4,5,6-hexahydroxycyclohexane", "")
  CALL Species("C_7H_{16}O_2",                         "629-30-1", "SXCBDZAEHILGLM-UHFFFAOYSA-N", "1,7-heptanediol", "")
  CALL Species("C_7H_{16}O_2",                       "20748-86-1", "XVEOUOTUJBYHNL-UHFFFAOYSA-N", "2,4-heptanediol", "")
  CALL Species("C_7H_{16}O_2",                       "70444-25-6", "XTVHTJKQKUOEQA-UHFFFAOYSA-N", "2,5-heptanediol", "")
  CALL Species("C_7H_{16}O_2",                       "40646-08-0", "ZRCYHJNCCBSTSZ-UHFFFAOYSA-N", "4-methyl-1,4-hexanediol", "")
  CALL Species("C_7H_{16}O_2",                       "29044-06-2", "KZWQVDXGMOSSNY-UHFFFAOYSA-N", "2-methyl-2,5-hexanediol", "")
  CALL Species("C_7H_{16}O_2",                                 "", "PMORVUNYTOZCSE-UHFFFAOYSA-N", "3-methyl-2,5-hexanediol", "")
  CALL Species("C_7H_{16}O_2",                         "115-76-4", "XRVCFZPJAHWYTB-UHFFFAOYSA-N", "2,2-diethyl-1,3-propanediol", "")
  CALL Species("C_7H_{16}O_5",                                 "", "HUYORHVLGRMTGF-UHFFFAOYSA-N", "1,2,3,4,5-pentahydroxyheptane", "")
  CALL Species("C_7H_{16}O_5",                                 "", "HBKKLPCMKBGRJH-UHFFFAOYSA-N", "1,2,3,4,6-pentahydroxyheptane", "")
  CALL Species("C_7H_{16}O_5",                                 "", "NSOOVWLSFZLKQX-UHFFFAOYSA-N", "1,2,3,5,7-pentahydroxyheptane", "")
  CALL Species("C_7H_{16}O_6",                      "688007-16-1", "YMEXGEAJNZRQEH-VIFPVBQESA-N", "1,2,3,4,5,6-hexahydroxyheptane", "1-deoxy-heptitol")
  CALL Species("C_8H_{18}O_2",                       "24434-09-1", "BCKOQWWRTRBSGR-UHFFFAOYSA-N", "3,6-octanediol", "")
  CALL Species("C_8H_{16}O_2",                         "105-08-8", "YIMQCDZDWXUDCA-UHFFFAOYSA-N", "1,4-cyclohexanedimethanol", "")
  CALL Species("C_8H_{18}O_2",                          "94-96-2", "RWLALWYNXFYRGW-UHFFFAOYSA-N", "2-ethyl-1,3-hexanediol", "")
  CALL Species("C_8H_{18}O_2",                         "144-19-4", "JCTXKRPTIMZBJT-UHFFFAOYSA-N", "2,2,4-trimethyl-1,3-pentanediol", "")
  CALL Species("C_8H_{18}O_2",                         "110-03-2", "ZWNMRZQYWRLGMM-UHFFFAOYSA-N", "2,5-dimethyl-2,5-hexanediol", "")
  CALL Species("C_9H_{20}O_2",                        "3937-56-2", "ALVZNPYWJMLXKV-UHFFFAOYSA-N", "1,9-nonanediol", "")
  CALL Species("C_9H_{20}O_2",                        "4469-85-6", "YKEWIFGQPMVBEF-UHFFFAOYSA-N", "3,6-nonanediol", "")
  CALL Species("C_{10}H_{22}O_2",                      "112-47-0", "FOTKYAAJKYLFFN-UHFFFAOYSA-N", "1,10-decanediol", "")
  CALL Species("C_{10}H_{22}O_2",                              "", "RBFBEPALGOYNGK-UHFFFAOYSA-N", "3,6-decanediol", "")
  CALL Species("C_{10}H_{18}O_2",                              "", "MOILFCKRQFQVFS-UHFFFAOYSA-N", "MCM:APINBOH", "")
  CALL Species("C_{10}H_{18}O_2",                              "", "OMDMTHRBGUBUCO-UHFFFAOYSA-N", "MCM:APINCOH", "")
  CALL Species("C_{10}H_{18}O_2",                              "", "VXXAKGMRLUXFQH-UHFFFAOYSA-N", "MCM:BPINAOH", "")
  CALL Species("C_{10}H_{18}O_2",                              "", "XYKGEKWHBMLSGS-UHFFFAOYSA-N", "MCM:BPINCOH", "")
  CALL Species("C_{11}H_{24}O_2",                              "", "VCLRIEYYOZNUNU-UHFFFAOYSA-N", "MCM:HO36C11", "")
  CALL Species("C_{12}H_{26}O_2",                              "", "CLQOYXKLYNGAHW-UHFFFAOYSA-N", "MCM:HO36C12", "")
  ! aliphatic, unsaturated:
  CALL Species("C_4H_8O_2",                            "110-64-5", "ORTVZLZNOYNASJ-UHFFFAOYSA-N", "2-butene-1,4-diol", "")
  CALL Species("C_4H_8O_2",                            "497-06-3", "ITMIAZBRRZANGB-UHFFFAOYSA-N", "3-butene-1,2-diol", "")
  CALL Species("C_4H_6O_2",                            "110-65-6", "DLDJFQGPPSQZKI-UHFFFAOYSA-N", "2-butyne-1,4-diol", "1,4-dihydroxy-2-butyne")
  CALL Species("C_5H_{10}O_3",                                 "", "PGARYUHLQUORKU-UHFFFAOYSA-N", "MCM:C524OH", "")
  CALL Species("C_5H_{10}O_2",                                 "", "FLXLJBCLEUWWCG-UHFFFAOYSA-N", "MCM:ISOPAOH", "")
  CALL Species("C_5H_{10}O_2",                                 "", "XZRGYMKUQMPDQH-UHFFFAOYSA-N", "MCM:ISOPBOH", "")
  CALL Species("C_5H_{10}O_2",                                 "", "HBHXLSPUXXXICF-UHFFFAOYSA-N", "MCM:ISOPDOH", "")
  CALL Species("C_6H_{12}O_2",                                 "", "VCGCHTQYUUAIGQ-UHFFFAOYSA-N", "MCM:C622OH", "")
  CALL Species("C_6H_{12}O_2",                                 "", "BZQZWOYZOVQJLT-UHFFFAOYSA-N", "MCM:C624OH", "")
  CALL Species("C_7H_{12}O_2",                                 "", "JRLLENLHZCUEBJ-UHFFFAOYSA-N", "MCM:C720OH", "")
  CALL Species("C_{10}H_{18}O_2",                              "", "WKZWTZTZWGWEGE-UHFFFAOYSA-N", "MCM:LIMAOH", "")
  CALL Species("C_{10}H_{18}O_2",                              "", "ZJALAEQNHJQSTN-UHFFFAOYSA-N", "MCM:LIMCOH", "")
  CALL Species("C_{12}H_{22}O",                      "33956-49-9", "CSWBSLXBXRFNST-MQQKCMAXSA-N", "($E$,$E$)-8,10-dodecadien-1-ol", "codlemone")
  CALL Species("C_{15}H_{26}O_2",                              "", "XZTGVWRBZDBQLP-UHFFFAOYSA-N", "MCM:BCAOH", "")
  CALL Species("C_{15}H_{26}O_2",                              "", "ALNVAIGKARRJOT-UHFFFAOYSA-N", "MCM:BCCOH", "")
  ! aromatic:
  CALL Species("C_6H_4(OH)_2",                         "120-80-9", "YCIMNLLNPGFGHC-UHFFFAOYSA-N", "1,2-dihydroxybenzene", "pyrocatechol")
  CALL Species("C_6H_4(OH)_2",                         "108-46-3", "GHMLBKRAJCXXBS-UHFFFAOYSA-N", "1,3-dihydroxybenzene", "resorcinol")
  CALL Species("C_6H_4(OH)_2",                         "123-31-9", "QIGBRXMKCJKVMJ-UHFFFAOYSA-N", "1,4-dihydroxybenzene", "hydroquinone")
  CALL Species("C_6H_6O_3",                             "87-66-1", "WQGWDDDVZFFDIG-UHFFFAOYSA-N", "1,2,3-benzenetriol", "pyrogallic acid")
  CALL Species("C_7H_8O_2",                            "488-17-5", "PGSWEKYNAOWQDF-UHFFFAOYSA-N", "1,2-dihydroxy-3-methylbenzene", "")
  CALL Species("C_8H_{10}O_2",                                 "", "UUCQGNWZASKXNN-UHFFFAOYSA-N", "MCM:ECATECHOL", "")
  CALL Species("C_8H_{10}O_2",                                 "", "YGLVLWAMIJMBPF-UHFFFAOYSA-N", "MCM:MXYCATECH", "")
  CALL Species("C_8H_{10}O_2",                                 "", "RYHGQTREHREIBC-UHFFFAOYSA-N", "MCM:OXYCATECH", "")
  CALL Species("C_8H_{10}O_2",                                 "", "RGUZWBOJHNWZOK-UHFFFAOYSA-N", "MCM:PXYCATECH", "")
  CALL Species("C_9H_{12}O_2",                                 "", "XLZHGKDRKSKCAU-UHFFFAOYSA-N", "MCM:IPCATECHOL", "")
  CALL Species("C_9H_{12}O_2",                                 "", "IFERDVOUSBGAOD-UHFFFAOYSA-N", "MCM:METCATECH", "")
  CALL Species("C_9H_{12}O_2",                                 "", "QTXIMHKRKLVXRS-UHFFFAOYSA-N", "MCM:OETCATECH", "")
  CALL Species("C_9H_{12}O_2",                                 "", "GOZVFLWHGAXTPA-UHFFFAOYSA-N", "MCM:PCATECHOL", "")
  CALL Species("C_9H_{12}O_2",                                 "", "WVRWBYUJPWRFQO-UHFFFAOYSA-N", "MCM:PETCATECH", "")
  CALL Species("C_9H_{12}O_2",                                 "", "DEIKGXRMQUHZJD-UHFFFAOYSA-N", "MCM:T123CATECH", "")
  CALL Species("C_9H_{12}O_2",                                 "", "NZEZVJPYSAXNTR-UHFFFAOYSA-N", "MCM:T124CATECH", "")
  CALL Species("C_{10}H_8O_2",                          "92-44-4", "JRNGUTKWMSBIBF-UHFFFAOYSA-N", "2,3-dihydroxynaphthalene", "")
  CALL Species("C_{12}H_{18}O_2",                      "136-77-6", "WFJIVOKAWHGMBH-UHFFFAOYSA-N", "hexylresorcinol", "")
  CALL Species("C_{14}H_{22}O",                        "128-39-2", "DKCPKDPYUFEZCP-UHFFFAOYSA-N", "2,6-bis(1,1-dimethylethyl)phenol", "")
  CALL Species("C_{15}H_{16}O",                        "599-64-4", "QBDSZLJBMIMQRS-UHFFFAOYSA-N", "4-(1-methyl-1-phenylethyl)phenol", "")
  CALL Species("C_{21}H_{24}O_2",                     "1568-80-5", "SICLLPHPVFCNTJ-UHFFFAOYSA-N", "2,2',3,3'-tetrahydro-3,3,3',3'-tetramethyl-1,1'-spirobi(1H-indene)-6,6'-diol", "")
  CALL Species("C_{21}H_{26}O_2",                   "129188-99-4", "UMPGNGRIGSEMTC-UHFFFAOYSA-N", "4,4'-(3,3,5-trimethylcyclohexane-1,1-diyl)diphenol", "")
  CALL Species("C_{21}H_{24}O_4",                       "77-08-7", "POFMQEVZKZVAPQ-UHFFFAOYSA-N", "3,3,3',3'-tetramethyl-1,1'-spirobi(indan)-5,5',6,6'-tetrol", "")
  CALL Species("C_{25}H_{18}O_2",                     "3236-71-3", "YWFPGFJLYRKYJZ-UHFFFAOYSA-N", "9,9-bis(4-hydroxyphenyl)fluorene", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Peroxides (\chem{ROOH}) and peroxy radicals (\chem{ROO})")
  CALL Species("CH_3OOH",                             "3031-73-0", "MEUKEBNAABNAEX-UHFFFAOYSA-N", "methyl hydroperoxide", "methyl peroxide")
  CALL Species("C_2H_5OOH",                           "3031-74-1", "ILHIHKRJJMKBEE-UHFFFAOYSA-N", "ethyl hydroperoxide", "ethyl peroxide")
  CALL Species("HOCH_2OOH",                          "15932-89-5", "NEZWFWIACBUQMN-UHFFFAOYSA-N", "hydroxymethyl hydroperoxide", "HMHP; HMP")
  CALL Species("HOCH_2OOCH_2OH",                     "17088-73-2", "JLJXMZMKMRQOLN-UHFFFAOYSA-N", "bis-(hydroxymethyl)-peroxide", "BHMP")
  CALL Species("C_4H_{10}O_2",                          "75-91-2", "CIHOLLKRGTVIJN-UHFFFAOYSA-N", "{\it tert}-butyl hydroperoxide", "")
  CALL Species("C_8H_{18}O_2",                         "110-05-4", "LSXWFXONGKSEMY-UHFFFAOYSA-N", "di-{\it tert}-butylperoxide", "")
  CALL Species("C_9H_{12}O_2",                          "80-15-9", "YQHLDYVWEZKEOX-UHFFFAOYSA-N", "1-methyl-1-phenylethylhydroperoxide", "")
  CALL Species("C_{18}H_{22}O_2",                       "80-43-3", "XMNIXWIUMCBBBL-UHFFFAOYSA-N", "dicumyl peroxide", "")
  CALL Species("CH_3OO",                              "2143-58-0", "WTFNSXYULBQCQV-UHFFFAOYSA-N", "methylperoxy radical", "")
  CALL Species("HOCH_2OO",                           "27828-51-9", "OLHGCLQZCFQBGO-UHFFFAOYSA-N", "hydroxymethylperoxy radical", "")
  CALL Species("CH_3C(O)O_2",                        "36709-10-1", "ZBQKPDHUDKSCRS-UHFFFAOYSA-N", "peroxyacetyl radical", "")
  CALL Species("C_3H_4O_3",                                    "", "AZIQALWHRUQPHV-UHFFFAOYSA-N", "MCM:ACO3H", "")
  CALL Species("C_3H_8O_2",                                    "", "SGJUFIMCHSLMRJ-UHFFFAOYSA-N", "MCM:IC3H7OOH", "")
  CALL Species("C_3H_8O_2",                                    "", "TURGQPDWYFJEDY-UHFFFAOYSA-N", "MCM:NC3H7OOH", "")
  CALL Species("C_3H_6O_3",                                    "", "CZPZWMPYEINMCF-UHFFFAOYSA-N", "MCM:PERPROACID", "")
  CALL Species("C_4H_6O_3",                                    "", "TXNBRZBEFMJQMW-UHFFFAOYSA-N", "MCM:C3DBCO3H", "")
  CALL Species("C_4H_{10}O_2",                                 "", "FUHWWEDRJKHMKK-UHFFFAOYSA-N", "MCM:IC4H9OOH", "")
  CALL Species("C_4H_6O_3",                                    "", "OELQSSWXRGADDE-UHFFFAOYSA-N", "MCM:MACO3H", "")
  CALL Species("C_4H_6O_5",                                    "", "DJPWYFXNRMHUNU-UHFFFAOYSA-N", "IEPOX1CO3H", "") ! SMILES: CC1(C(=O)OO)OC1O
  CALL Species("C_4H_{10}O_2",                                 "", "AKUNSTOMHUXJOZ-UHFFFAOYSA-N", "MCM:NC4H9OOH", "")
  CALL Species("C_4H_8O_3",                                    "", "LBAYFEDWGHXMSM-UHFFFAOYSA-N", "MCM:PERBUACID", "")
  CALL Species("C_4H_8O_3",                                    "", "LVQKOPBJHBWELS-UHFFFAOYSA-N", "MCM:PERIBUACID", "")
  CALL Species("C_4H_{10}O_2",                                 "", "SPQMVUPFYWDFCB-UHFFFAOYSA-N", "MCM:SC4H9OOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "LWBFPIVRZGBOFS-UHFFFAOYSA-N", "MCM:BUT2CO3H", "")
  CALL Species("C_5H_{10}O_3",                                 "", "GTLLKMCVRPVGBK-UHFFFAOYSA-N", "MCM:C3ME3CO3H", "")
  CALL Species("C_5H_{12}O_2",                                 "", "HIHRAMNMOMKJDG-UHFFFAOYSA-N", "MCM:IPEAOOH", "")
  CALL Species("C_5H_{12}O_2",                                 "", "VIUWCQFVAFABHL-UHFFFAOYSA-N", "MCM:IPEBOOH", "")
  CALL Species("C_5H_{12}O_2",                                 "", "XRXANEMIFVRKLN-UHFFFAOYSA-N", "MCM:IPECOOH", "")
  CALL Species("C_5H_{12}O_2",                                 "", "UEQURRFROJBOLG-UHFFFAOYSA-N", "MCM:NEOPOOH", "")
  CALL Species("C_5H_{12}O_2",                                 "", "KCHNMIKAMRQBHD-UHFFFAOYSA-N", "MCM:PEAOOH", "")
  CALL Species("C_5H_{12}O_2",                                 "", "XRIRVAYMWUMXBR-UHFFFAOYSA-N", "MCM:PEBOOH", "")
  CALL Species("C_5H_{12}O_2",                                 "", "RLBQWRSRDVXRTJ-UHFFFAOYSA-N", "MCM:PECOOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "UQGPCEVQKLOLLM-UHFFFAOYSA-N", "MCM:PERPENACID", "")
  CALL Species("C_5H_{10}O_3",                                 "", "YVAACGXAZGGQSM-UHFFFAOYSA-N", "MCM:TBUTCO3H", "")
  CALL Species("C_5H_{10}O_7",                                 "", "GRPDYNZWDDUILX-UHFFFAOYSA-N", "C520OOH", "") ! SMILES: O=C(CO)C(CO)(COO)OO
  CALL Species("C_5H_8O_4",                                    "", "PQEISBQMLHJHJQ-UHFFFAOYSA-N", "C518OOH", "") ! SMILES: C=C(COO)C(=O)CO
  CALL Species("C_6H_{12}O_3",                                 "", "JTSRVXQXWWOTER-UHFFFAOYSA-N", "MCM:C54CO3H", "")
  CALL Species("C_6H_{12}O_3",                                 "", "NQUPKCJGWCPODR-UHFFFAOYSA-N", "MCM:C5H11CO3H", "")
  CALL Species("C_6H_{12}O_4",                                 "", "UPAWKBDHGPWUCG-UHFFFAOYSA-N", "ROO6R7OOH", "") ! SMILES: CC(=O)CCC(CO)OO
  CALL Species("C_6H_{12}O_5",                                 "", "QZPUZDFJKAEVLL-UHFFFAOYSA-N", "ROO6R6OOH", "") ! SMILES: CC(=O)CCC(COO)OO
  CALL Species("C_6H_{12}O_6",                                 "", "FUTSLQULSAKEJA-UHFFFAOYSA-N", "C624OOH", "") ! SMILES: C=C(COO)C(CO)(CO)OO
  CALL Species("C_6H_{12}O_2",                                 "", "FGGJBCRKSVGDPO-UHFFFAOYSA-N", "MCM:CHEXOOH", "")
  CALL Species("C_6H_{14}O_2",                                 "", "RZICEOJUAFHYFO-UHFFFAOYSA-N", "MCM:HEXAOOH", "")
  CALL Species("C_6H_{14}O_2",                                 "", "XWXUHAUZCICPHE-UHFFFAOYSA-N", "MCM:HEXBOOH", "")
  CALL Species("C_6H_{14}O_2",                                 "", "NMNOLZYZNVYLRJ-UHFFFAOYSA-N", "MCM:HEXCOOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "RQLGMECUXRQENU-UHFFFAOYSA-N", "MCM:M22C3CO3H", "")
  CALL Species("C_6H_{14}O_2",                                 "", "XDWQBMWWYOVNSC-UHFFFAOYSA-N", "MCM:M22C43OOH", "")
  CALL Species("C_6H_{14}O_2",                                 "", "VWOCCXGIWJLCIT-UHFFFAOYSA-N", "MCM:M22C4OOH", "")
  CALL Species("C_6H_{14}O_2",                                 "", "UPTQBPSUWRBZRD-UHFFFAOYSA-N", "MCM:M23C43OOH", "")
  CALL Species("C_6H_{14}O_2",                                 "", "MCTQHWIBGOKRHO-UHFFFAOYSA-N", "MCM:M23C4OOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "KGAQRBNLHLEGET-UHFFFAOYSA-N", "MCM:M2C43CO3H", "")
  CALL Species("C_6H_{14}O_2",                                 "", "OTKKOLXSCMINFN-UHFFFAOYSA-N", "MCM:M2PEAOOH", "")
  CALL Species("C_6H_{14}O_2",                                 "", "XHHITPKXZQXJIC-UHFFFAOYSA-N", "MCM:M2PEBOOH", "")
  CALL Species("C_6H_{14}O_2",                                 "", "JBJSMFBSVBRLAK-UHFFFAOYSA-N", "MCM:M2PECOOH", "")
  CALL Species("C_6H_{14}O_2",                                 "", "BZGMEGUFFDTCNP-UHFFFAOYSA-N", "MCM:M2PEDOOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "IHLWWYDUPGXTHJ-UHFFFAOYSA-N", "MCM:M33C3CO3H", "")
  CALL Species("C_6H_{14}O_2",                                 "", "OUMLJDCULPJYHU-UHFFFAOYSA-N", "MCM:M33C4OOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "CHSVHHKRTQKZCN-UHFFFAOYSA-N", "MCM:M3C4CO3H", "")
  CALL Species("C_6H_{14}O_2",                                 "", "AOZFMIRWQXSLHO-UHFFFAOYSA-N", "MCM:M3PEAOOH", "")
  CALL Species("C_6H_{14}O_2",                                 "", "XCSSHHOKPSJZJS-UHFFFAOYSA-N", "MCM:M3PEBOOH", "")
  CALL Species("C_6H_{14}O_2",                                 "", "BWMWVYKKWYNWMF-UHFFFAOYSA-N", "MCM:M3PECOOH", "")
  CALL Species("C_7H_{12}O_6",                                 "", "RAASFOSVQCCDRJ-UHFFFAOYSA-N", "ROO6R5OOH", "") ! SMILES: CC(=O)CCC(CC(=O)OO)OO
  CALL Species("C_7H_{14}O_3",                                 "", "GLAYRDYYNSCSPO-UHFFFAOYSA-N", "MCM:C6H13CO3H", "")
  CALL Species("C_7H_{16}O_2",                                 "", "RCPMMXGHMBPBLI-UHFFFAOYSA-N", "MCM:HEPTOOH", "")
  CALL Species("C_7H_{16}O_2",                                 "", "MZRXBNCEXTUOSL-UHFFFAOYSA-N", "MCM:M2HEXAOOH", "")
  CALL Species("C_7H_{16}O_2",                                 "", "VERXWMXNIBWXFV-UHFFFAOYSA-N", "MCM:M2HEXBOOH", "")
  CALL Species("C_7H_{16}O_2",                                 "", "SLXXGGKZGKGOKQ-UHFFFAOYSA-N", "MCM:M3HEXAOOH", "")
  CALL Species("C_7H_{16}O_2",                                 "", "YDBMAIVJCSJSKQ-UHFFFAOYSA-N", "MCM:M3HEXBOOH", "")
  CALL Species("C_7H_{16}O_2",                         "762-46-9", "FWELUXZVATZEMI-UHFFFAOYSA-N", "1-methylhexyl hydroperoxide", "C7H15O2H") ! SMILES: CC(OO)CCCCC
  CALL Species("C_7H_{16}O_3",                                 "", "KHWIBANUCNWWGW-UHFFFAOYSA-N", "C7OHOOH", "") ! SMILES: CCC(OO)CCC(O)C
  CALL Species("C_8H_{14}O_2",                                 "", "TWXDGALDAWYAQL-UHFFFAOYSA-N", "MCM:C8BCOOH", "")
  CALL Species("C_8H_{18}O_2",                                 "", "PXUMFRDGKQLDGL-UHFFFAOYSA-N", "MCM:OCTOOH", "")
  CALL Species("C_8H_{18}O_2",                                 "", "NAXZMRYIZGEALQ-UHFFFAOYSA-N", "1-methylheptl hydroperoxide", "C8H17O2H") ! SMILES: CC(OO)CCCCCC
  CALL Species("C_8H_{18}O_3",                                 "", "QQTSMKQYSKGCOU-UHFFFAOYSA-N", "C8OHOOH", "") ! SMILES: CCCC(OO)CCC(O)C
  CALL Species("C_9H_{20}O_2",                                 "", "BXPIMWLPPYUCSQ-UHFFFAOYSA-N", "MCM:NONOOH", "")
  CALL Species("C_{10}H_{22}O_2",                              "", "MPMVWDJTKHXCAN-UHFFFAOYSA-N", "MCM:DECOOH", "")
  CALL Species("C_{10}H_{22}O_4",                              "", "AXIHYONNAPPFSO-UHFFFAOYSA-N", "C5H112O2H", "") ! SMILES: CC(OO)CCCCC(OO)CCC
  CALL Species("C_{10}H_{20}O_6",                              "", "IRUZCTPAAZATQM-UHFFFAOYSA-N", "LIMAB15OOH2", "") ! SMILES: CC(CO)(OO)C1CCC(C)(OO)C(O)C1
  CALL Species("C_{10}H_{18}O_4",                              "", "NSOHXIGSWKLSHV-UHFFFAOYSA-N", "RO5R1O2H", "") ! SMILES: CC1(C)OC2CC1CCC2(CO)OO
  CALL Species("C_{10}H_{18}O_5",                              "", "MFXJOKCSTDMZRM-UHFFFAOYSA-N", "RO5R2O2H", "") ! SMILES: CC1(C)OC(OO)CC1CCC(=O)CO
  CALL Species("C_{10}H_{18}O_5",                              "", "GBBPAXKBVMQHIG-UHFFFAOYSA-N", "ROO6R1OOH", "") ! SMILES: CC1(CO)OOC2CC1CCC2(C)OO
  CALL Species("C_{10}H_{18}O_6",                              "", "IEPFJXDIWPQIRC-UHFFFAOYSA-N", "RO5R3O2H", "") ! SMILES: CC(C)(OC=O)C(CCC(=O)CO)COO
  CALL Species("C_{11}H_{24}O_2",                              "", "PPEGYVQDERYKNI-UHFFFAOYSA-N", "MCM:UDECOOH", "")
  CALL Species("C_{12}H_{26}O_2",                              "", "ONWAGJMEBPWHEG-UHFFFAOYSA-N", "MCM:DDECOOH", "")
  CALL Species("C_6H_6O_2",                                    "", "JYINMLPNDRBKKZ-UHFFFAOYSA-N", "MCM:C6H5OOH", "")
  CALL Species("C_7H_8O_2",                                    "", "YVJRCWCFDJYONJ-UHFFFAOYSA-N", "MCM:C6H5CH2OOH", "")
  CALL Species("C_7H_6O_3",                                    "", "XCRBXWCUXJNEFX-UHFFFAOYSA-N", "MCM:C6H5CO3H", "")
  CALL Species("C_7H_8O_2",                                    "", "VCTGBWYTEXZDNM-UHFFFAOYSA-N", "MCM:MXYL1OOH", "")
  CALL Species("C_7H_8O_2",                                    "", "OCSKWYCRTWOMLG-UHFFFAOYSA-N", "MCM:OXYL1OOH", "")
  CALL Species("C_7H_8O_2",                                    "", "ZCUUEQJCOSBJOR-UHFFFAOYSA-N", "MCM:PXYL1OOH", "")
  CALL Species("C_8H_8O_3",                                    "", "BXGXGTXWGGOFSP-UHFFFAOYSA-N", "MCM:C6H5C2CO3H", "")
  CALL Species("C_8H_{10}O_2",                                 "", "VZQOBPXGQJXYGY-UHFFFAOYSA-N", "MCM:C6H5C2OOH", "")
  CALL Species("C_8H_{10}O_2",                                 "", "FPXURWFXMHYDGM-UHFFFAOYSA-N", "MCM:DM123OOH", "")
  CALL Species("C_8H_{10}O_2",                                 "", "UQJVEYMRRMOLPI-UHFFFAOYSA-N", "MCM:DM124OOH", "")
  CALL Species("C_8H_{10}O_2",                                 "", "AGQLZSPEKWKXRP-UHFFFAOYSA-N", "MCM:DMPHOOH", "")
  CALL Species("C_8H_{10}O_2",                                 "", "SULLUHFYVRLICT-UHFFFAOYSA-N", "MCM:EBENZOOH", "")
  CALL Species("C_8H_{10}O_2",                                 "", "LDNGPDBEDBLXJS-UHFFFAOYSA-N", "MCM:MXY1OOH", "")
  CALL Species("C_8H_8O_3",                                    "", "INFFZTZRGNVBOC-UHFFFAOYSA-N", "MCM:MXYLCO3H", "")
  CALL Species("C_8H_{10}O_2",                                 "", "UTKFWOZXPFIBPQ-UHFFFAOYSA-N", "MCM:MXYLOOH", "")
  CALL Species("C_8H_8O_3",                                    "", "DNEXRQSSNZPAJJ-UHFFFAOYSA-N", "MCM:OXYLCO3H", "")
  CALL Species("C_8H_{10}O_2",                                 "", "JTJQZHCBMBJJMV-UHFFFAOYSA-N", "MCM:OXYLOOH", "")
  CALL Species("C_8H_{10}O_2",                                 "", "QBGYVNGBYXOUAG-UHFFFAOYSA-N", "MCM:PXY1OOH", "")
  CALL Species("C_8H_8O_3",                                    "", "IFDPVSBDNAQBRQ-UHFFFAOYSA-N", "MCM:PXYLCO3H", "")
  CALL Species("C_8H_{10}O_2",                                 "", "VCEHMDXCOGIOJJ-UHFFFAOYSA-N", "MCM:PXYLOOH", "")
  CALL Species("C_9H_{12}O_2",                                 "", "MKWUSWBHCPKVBI-UHFFFAOYSA-N", "MCM:EMPHOOH", "")
  CALL Species("C_9H_{12}O_2",                                 "", "DWPOLKIWKCGPAM-UHFFFAOYSA-N", "MCM:ETOLOOH", "")
  CALL Species("C_9H_{12}O_2",                                 "", "CZNJIWCDRJWKLG-UHFFFAOYSA-N", "MCM:IPBENZOOH", "")
  CALL Species("C_9H_{12}O_2",                                 "", "HUAUUZDXRNWOSS-UHFFFAOYSA-N", "MCM:MET1OOH", "")
  CALL Species("C_9H_{12}O_2",                                 "", "KVZSMDLRNLVTLK-UHFFFAOYSA-N", "MCM:OET1OOH", "")
  CALL Species("C_9H_{12}O_2",                                 "", "BFUPNGCSYHWOOH-UHFFFAOYSA-N", "MCM:PBENZOOH", "")
  CALL Species("C_9H_{12}O_2",                                 "", "IKLGQZOVXYBNEV-UHFFFAOYSA-N", "MCM:PET1OOH", "")
  CALL Species("C_9H_{12}O_2",                                 "", "JYLUDNGUBXOJPX-UHFFFAOYSA-N", "MCM:PHC3OOH", "")
  CALL Species("C_9H_{10}O_3",                                 "", "NQSDUJYDMWXHMR-UHFFFAOYSA-N", "MCM:TM123BCO3H", "")
  CALL Species("C_9H_{12}O_2",                                 "", "HSKYDUVZGAHSEO-UHFFFAOYSA-N", "MCM:TM123BOOH", "")
  CALL Species("C_9H_{12}O_2",                                 "", "QIIZMGNSXSJCNA-UHFFFAOYSA-N", "MCM:TM123OOH", "")
  CALL Species("C_9H_{10}O_3",                                 "", "HPLBJCPAWPQKDU-UHFFFAOYSA-N", "MCM:TM124BCO3H", "")
  CALL Species("C_9H_{12}O_2",                                 "", "ZUEBLWOBXMLPMV-UHFFFAOYSA-N", "MCM:TM124BOOH", "")
  CALL Species("C_9H_{12}O_2",                                 "", "DSGKIBNJTLHQCN-UHFFFAOYSA-N", "MCM:TM124OOH", "")
  CALL Species("C_9H_{10}O_3",                                 "", "APDYULOMODUXLL-UHFFFAOYSA-N", "MCM:TMBCO3H", "")
  CALL Species("C_9H_{12}O_2",                                 "", "GOKTXPLVCSLYGI-UHFFFAOYSA-N", "MCM:TMBOOH", "")
  CALL Species("C_{10}H_{14}O_2",                              "", "NSJZLSFVHPJVDL-UHFFFAOYSA-N", "MCM:DM35EBOOH", "")
  CALL Species("C_{10}H_{12}O_3",                              "", "GYXFVTDASBFNQD-UHFFFAOYSA-N", "MCM:EMPHCO3H", "")
  CALL Species("C_{11}H_{16}O_2",                              "", "HPTJZRKCFIXEOP-UHFFFAOYSA-N", "MCM:DE35TOOH", "")
  CALL Species("C_2H_4O_4",                                    "", "IUEZWLCUORJBDZ-UHFFFAOYSA-N", "MCM:HOCH2CO3H", "")
  CALL Species("C_2H_6O_3",                                    "", "FKPAKAOEHCFKLE-UHFFFAOYSA-N", "MCM:HYETHO2H", "")
  CALL Species("C_3H_6O_5",                                    "", "GTTVGDRVLZEEEW-UHFFFAOYSA-N", "MCM:C2OHOCOOH", "")
  CALL Species("C_3H_8O_4",                                    "", "XSLBWJPPWWFTQY-UHFFFAOYSA-N", "MCM:C3DIOLOOH", "")
  CALL Species("C_3H_8O_3",                                    "", "LTKFKDLZMYUGKU-UHFFFAOYSA-N", "MCM:HO1C3OOH", "")
  CALL Species("C_3H_6O_4",                                    "", "HPIZQAHZLDXVRM-UHFFFAOYSA-N", "MCM:HOC2H4CO3H", "")
  CALL Species("C_3H_8O_3",                                    "", "CGHALRGHFXEMJB-UHFFFAOYSA-N", "MCM:HYPROPO2H", "")
  CALL Species("C_3H_8O_3",                                    "", "LGTUXDWECPSIQO-UHFFFAOYSA-N", "MCM:IPROPOLO2H", "")
  CALL Species("C_3H_6O_4",                                    "", "LLWVBKQQPPRNEX-UHFFFAOYSA-N", "MCM:IPROPOLPER", "")
  CALL Species("C_4H_{10}O_3",                                 "", "VRQPOZSYDWHVQK-UHFFFAOYSA-N", "MCM:BUT2OLOOH", "")
  CALL Species("C_4H_8O_3",                                    "", "ATNKTSDVMCPWQK-UHFFFAOYSA-N", "MCM:BUTDAOOH", "")
  CALL Species("C_4H_8O_3",                                    "", "VQGAWAMXUXLZCZ-UHFFFAOYSA-N", "MCM:BUTDBOOH", "")
  CALL Species("C_4H_8O_3",                                    "", "TVUDLONEIXNOQY-UHFFFAOYSA-N", "MCM:BUTDCOOH", "")
  CALL Species("C_4H_6O_4",                                    "", "ZSVMRAYUVFDTKB-UHFFFAOYSA-N", "MCM:HC3CCO3H", "")
  CALL Species("C_4H_6O_4",                                    "", "AVWVUWGQXKTDKD-UHFFFAOYSA-N", "MCM:HC3CO3H", "")
  CALL Species("C_4H_6O_4",                                    "", "IZGOVIKWSGSSGB-UHFFFAOYSA-N", "MCM:HMACO3H", "")
  CALL Species("C_4H_8O_5",                                    "", "SAVJJQFYFPVKTL-UHFFFAOYSA-N", "MCM:HO13C3CO3H", "")
  CALL Species("C_4H_{10}O_4",                                 "", "WTIWPVYZJGYFOG-UHFFFAOYSA-N", "MCM:HO13C4OOH", "")
  CALL Species("C_4H_{10}O_3",                                 "", "RCPPCOTVBKCJGX-UHFFFAOYSA-N", "MCM:HO1C4OOH", "")
  CALL Species("C_4H_8O_4",                                    "", "WSXMRWGMMYXEOT-UHFFFAOYSA-N", "MCM:HO2C3CO3H", "")
  CALL Species("C_4H_{10}O_3",                                 "", "VJLLYXNZUMWUFH-UHFFFAOYSA-N", "MCM:HO2C4OOH", "")
  CALL Species("C_4H_8O_4",                                    "", "BPXTXQLLUZFFNC-UHFFFAOYSA-N", "MCM:HO3C3CO3H", "")
  CALL Species("C_4H_{10}O_3",                                 "", "RMYCTZOJULYKEL-UHFFFAOYSA-N", "MCM:HO3C4OOH", "")
  CALL Species("C_4H_8O_4",                                    "", "CAGKPIQTEWLESW-UHFFFAOYSA-N", "MCM:HOC3H6CO3H", "")
  CALL Species("C_4H_8O_4",                                    "", "XMJMNOOOSJCSST-UHFFFAOYSA-N", "MCM:HOIPRCO3H", "")
  CALL Species("C_4H_{10}O_3",                                 "", "NCECNUIKMRTTTQ-UHFFFAOYSA-N", "MCM:IBUTOLBO2H", "")
  CALL Species("C_4H_{10}O_3",                                 "", "IYIYQILWOLBTBM-UHFFFAOYSA-N", "MCM:IBUTOLCO2H", "")
  CALL Species("C_4H_8O_4",                                    "", "VAHIZEUROSIQMC-UHFFFAOYSA-N", "MCM:IPRHOCO3H", "")
  CALL Species("C_4H_{10}O_3",                                 "", "KPUCJKMFRKKADT-UHFFFAOYSA-N", "MCM:NBUTOLAOOH", "")
  CALL Species("C_4H_{10}O_3",                                 "", "UKXKMFPUVLQYHV-UHFFFAOYSA-N", "MCM:NBUTOLBOOH", "")
  CALL Species("C_4H_{10}O_3",                                 "", "OLMJQPVPWWPTBW-UHFFFAOYSA-N", "MCM:TBUTOLOOH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "OPXDDGDNASPZKO-UHFFFAOYSA-N", "MCM:C3M3OHCO3H", "")
  CALL Species("C_5H_{10}O_5",                                 "", "ZMIMRJOKAFWNST-UHFFFAOYSA-N", "MCM:C42CO3H", "")
  CALL Species("C_5H_8O_4",                                    "", "VWNMYJVYTYSNQP-UHFFFAOYSA-N", "MCM:C46CO3H", "")
  CALL Species("C_5H_{10}O_5",                                 "", "WQUIEUVQOLAPHC-UHFFFAOYSA-N", "MCM:C4OH2CO3H", "")
  CALL Species("C_5H_{10}O_4",                                 "", "OQVSVCDIVLTQNI-UHFFFAOYSA-N", "MCM:C4OHCO3H", "")
  CALL Species("C_5H_{12}O_3",                                 "", "YEVLRKKUVOPQLT-UHFFFAOYSA-N", "MCM:C51OH2OOH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "CMYVSBWTXHBCDK-UHFFFAOYSA-N", "MCM:C524OOH", "")
  CALL Species("C_5H_{12}O_3",                                 "", "UCQFCLCUMHUGLJ-UHFFFAOYSA-N", "MCM:C52OH1OOH", "")
  CALL Species("C_5H_{12}O_3",                                 "", "HJJMDLGJAJYCSL-UHFFFAOYSA-N", "MCM:C52OH3OOH", "")
  CALL Species("C_5H_{12}O_3",                                 "", "ZPHQYWRKWRKGNS-UHFFFAOYSA-N", "MCM:C52OOH", "")
  CALL Species("C_5H_{12}O_3",                                 "", "AJVYEIIRONFKES-UHFFFAOYSA-N", "MCM:C53OH2OOH", "")
  CALL Species("C_5H_{12}O_4",                                 "", "YUSGXJKDHCBGQE-UHFFFAOYSA-N", "MCM:C54OOH", "")
  CALL Species("C_5H_{12}O_4",                                 "", "ZJCLGCDNIIJGCW-UHFFFAOYSA-N", "MCM:C56OOH", "")
  CALL Species("C_5H_{10}O_5",                                 "", "AHAADFOLDXPWKJ-UHFFFAOYSA-N", "MCM:H13C43CO3H", "")
  CALL Species("C_5H_{10}O_4",                                 "", "WNBRWGLXVYJADJ-UHFFFAOYSA-N", "MCM:H2M2C3CO3H", "")
  CALL Species("C_5H_{12}O_3",                                 "", "ZUPZYIDTDVLJQO-UHFFFAOYSA-N", "MCM:H2M3C4OOH", "")
  CALL Species("C_5H_8O_4",                                    "", "RKJIARBOZVRICG-UHFFFAOYSA-N", "MCM:HC4ACO3H", "")
  CALL Species("C_5H_8O_4",                                    "", "BKBSBVFBLVOVGK-UHFFFAOYSA-N", "MCM:HC4CCO3H", "")
  CALL Species("C_5H_{12}O_3",                                 "", "YTVLHPDOOITGNI-UHFFFAOYSA-N", "MCM:HM22C3OOH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "JLDURWTXWMLEQQ-UHFFFAOYSA-N", "MCM:HM22CO3H", "")
  CALL Species("C_5H_{12}O_3",                                 "", "HSLVQIVCJSWUOZ-UHFFFAOYSA-N", "MCM:HM2C43OOH", "")
  CALL Species("C_5H_{12}O_3",                                 "", "WHBWUNCJQHRHKD-UHFFFAOYSA-N", "MCM:HM33C3OOH", "")
  CALL Species("C_5H_{12}O_4",                                 "", "MPQCFHWMMIVYQV-UHFFFAOYSA-N", "MCM:HO13C5OOH", "")
  CALL Species("C_5H_{12}O_3",                                 "", "NOIFGIIKGVCIDM-UHFFFAOYSA-N", "MCM:HO1C5OOH", "")
  CALL Species("C_5H_{10}O_5",                                 "", "BPXJXZFDALRNCD-UHFFFAOYSA-N", "MCM:HO24C4CO3H", "")
  CALL Species("C_5H_{12}O_4",                                 "", "LXCUFKDAZLTFBK-UHFFFAOYSA-N", "MCM:HO24C5OOH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "JJSNUFSNLCXQDD-UHFFFAOYSA-N", "MCM:HO2C43CO3H", "")
  CALL Species("C_5H_{10}O_4",                                 "", "WEYKGJBTAJHRDZ-UHFFFAOYSA-N", "MCM:HO2C4CO3H", "")
  CALL Species("C_5H_{12}O_3",                                 "", "NQEGDBNDYHCERX-UHFFFAOYSA-N", "MCM:HO2C54OOH", "")
  CALL Species("C_5H_{12}O_3",                                 "", "GSRFGQMXWRCOMF-UHFFFAOYSA-N", "MCM:HO2C5OOH", "")
  CALL Species("C_5H_{12}O_3",                                 "", "BPHNXEOBBWBBGO-UHFFFAOYSA-N", "MCM:HO2M2C4OOH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "OFXAQZFPCOWEPU-UHFFFAOYSA-N", "MCM:HO3C4CO3H", "")
  CALL Species("C_5H_{12}O_3",                                 "", "AMTQAQGRUCFWLQ-UHFFFAOYSA-N", "MCM:HO3C5OOH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "HHXRGEGWJUDFIW-UHFFFAOYSA-N", "MCM:HOBUT2CO3H", "")
  CALL Species("C_5H_{10}O_4",                                 "", "JVGUWDNKCJEDLO-UHFFFAOYSA-N", "MCM:HOIBUTCO3H", "")
  CALL Species("C_5H_{10}O_3",                                 "", "PEJOQVDRQFVMLE-UHFFFAOYSA-N", "MCM:ISOPAOOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "QTGGFXPTAQEODO-UHFFFAOYSA-N", "MCM:ISOPBOOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "FCLDANRCFPUKED-UHFFFAOYSA-N", "MCM:ISOPCOOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "HYHMYONEYJINON-UHFFFAOYSA-N", "MCM:ISOPDOOH", "")
  CALL Species("C_5H_{12}O_3",                                 "", "YWZMEYHQKKHRJF-UHFFFAOYSA-N", "MCM:M2BU2OLOOH", "")
  CALL Species("C_5H_{12}O_3",                                 "", "KLOVYRXVQOACLZ-UHFFFAOYSA-N", "MCM:M2BUOL2OOH", "")
  CALL Species("C_5H_{12}O_3",                                 "", "AFEDOYRLNKBWPZ-UHFFFAOYSA-N", "MCM:M3BU2OLOOH", "")
  CALL Species("C_5H_{12}O_4",                                 "", "VJEOLZUQMDEVMO-UHFFFAOYSA-N", "MCM:MBOAOOH", "")
  CALL Species("C_5H_{12}O_4",                                 "", "GXJWPCHTRDIVHR-UHFFFAOYSA-N", "MCM:MBOBOOH", "")
  CALL Species("C_5H_{12}O_3",                                 "", "SAXHQAUHEBEDAO-UHFFFAOYSA-N", "MCM:ME2BUOLOOH", "")
  CALL Species("C_5H_{12}O_3",                                 "", "MUYQYONVNPPJGI-UHFFFAOYSA-N", "MCM:ME3BUOLOOH", "")
  CALL Species("C_5H_{12}O_3",                                 "", "UVDGWSPEOVCOFS-UHFFFAOYSA-N", "MCM:PROL11MOOH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "KTQROZYFUGXCGA-UHFFFAOYSA-N", "MCM:PROL1MCO3H", "")
  CALL Species("C_6H_8O_4",                                    "", "BVNIDXNYDXJAFS-UHFFFAOYSA-N", "MCM:BZBIPER2OH", "")
  CALL Species("C_6H_8O_5",                                    "", "MXPMOVDECINDPN-UHFFFAOYSA-N", "MCM:BZBIPEROOH", "")
  CALL Species("C_6H_{14}O_3",                                 "", "HOGFDWYRBLIMGM-UHFFFAOYSA-N", "MCM:C4ME2OHOOH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "HLKDNTMYLJJNIF-UHFFFAOYSA-N", "MCM:C518CO3H", "")
  CALL Species("C_6H_{12}O_3",                                 "", "GWTGWFHICNOSTQ-UHFFFAOYSA-N", "MCM:C622OOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "DDEGWHJCSBWJSM-UHFFFAOYSA-N", "MCM:C624OOH", "")
  CALL Species("C_6H_{14}O_3",                                 "", "ZEOCDIHFKNFYIT-UHFFFAOYSA-N", "MCM:C64OH5OOH", "")
  CALL Species("C_6H_{14}O_3",                                 "", "PVVIDEIOKNUTFV-UHFFFAOYSA-N", "MCM:C65OH4OOH", "")
  CALL Species("C_6H_{14}O_3",                                 "", "HRDCBAWVKIWIOZ-UHFFFAOYSA-N", "MCM:C6OH5OOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "UBIBQRDHLFTCKX-UHFFFAOYSA-N", "MCM:CYHXOLAOOH", "")
  CALL Species("C_6H_{14}O_4",                                 "", "KEIPMCKLLYLETP-UHFFFAOYSA-N", "MCM:H13M3C5OOH", "")
  CALL Species("C_6H_{14}O_3",                                 "", "NRUVZELSEXXOEO-UHFFFAOYSA-N", "MCM:H1MC5OOH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "NRXKHRBCIJLLIW-UHFFFAOYSA-N", "MCM:H2M2C4CO3H", "")
  CALL Species("C_6H_{12}O_4",                                 "", "BSSSRXSGDLZQTD-UHFFFAOYSA-N", "MCM:H2M3C4CO3H", "")
  CALL Species("C_6H_{14}O_3",                                 "", "VFABJBQUVDJTMS-UHFFFAOYSA-N", "MCM:H2MC5OOH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "BXUACBWFNFVFRM-UHFFFAOYSA-N", "MCM:H3M2C4CO3H", "")
  CALL Species("C_6H_{12}O_4",                                 "", "QREFYDOYYWEEBY-UHFFFAOYSA-N", "MCM:H3M3C4CO3H", "")
  CALL Species("C_6H_{14}O_3",                                 "", "SFWIETIRFCEPBV-UHFFFAOYSA-N", "MCM:H3M3C5OOH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "CFTUKLJUHGJIMT-UHFFFAOYSA-N", "MCM:HM22C3CO3H", "")
  CALL Species("C_6H_{14}O_3",                                 "", "RBKQZGUZSABUAQ-UHFFFAOYSA-N", "MCM:HM22C4OOH", "")
  CALL Species("C_6H_{14}O_3",                                 "", "OEDZXCCMDSIVQE-UHFFFAOYSA-N", "MCM:HM23C4OOH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "AAGQWOGOWVBCOC-UHFFFAOYSA-N", "MCM:HM2C43CO3H", "")
  CALL Species("C_6H_{12}O_4",                                 "", "CJLFHOYFCBYSAF-UHFFFAOYSA-N", "MCM:HM33C3CO3H", "")
  CALL Species("C_6H_{14}O_3",                                 "", "IRDKGYBPQHNNSI-UHFFFAOYSA-N", "MCM:HM33C4OOH", "")
  CALL Species("C_6H_{14}O_3",                                 "", "AODLLEVTDATZKZ-UHFFFAOYSA-N", "MCM:HO1C6OOH", "")
  CALL Species("C_6H_{14}O_3",                                 "", "XFJAYUAZOYAZNW-UHFFFAOYSA-N", "MCM:HO1MC5OOH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "HFXNQHIGYFOTHL-UHFFFAOYSA-N", "MCM:HO2C54CO3H", "")
  CALL Species("C_6H_{14}O_3",                                 "", "BVOYWPYSMMRYPX-UHFFFAOYSA-N", "MCM:HO2C6OOH", "")
  CALL Species("C_6H_{14}O_3",                                 "", "XDJWSJNLVSIJHD-UHFFFAOYSA-N", "MCM:HO2M2C5OOH", "")
  CALL Species("C_6H_{14}O_3",                                 "", "AGNBGPUCVJMPGZ-UHFFFAOYSA-N", "MCM:HO2MC5OOH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "NIDIDJCFEJYKBD-UHFFFAOYSA-N", "MCM:HO3C5CO3H", "")
  CALL Species("C_6H_{14}O_3",                                 "", "WSWZWPBQPMRENX-UHFFFAOYSA-N", "MCM:HO3C6OOH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "YRMQNGAQYQCFSB-UHFFFAOYSA-N", "MCM:HO4C5CO3H", "")
  CALL Species("C_6H_{12}O_4",                                 "", "ZOLITBGWKBFVQG-UHFFFAOYSA-N", "MCM:HO5C5CO3H", "")
  CALL Species("C_6H_{14}O_3",                                 "", "VKWHJFIJUJODAG-UHFFFAOYSA-N", "MCM:HO5C6OOH", "")
  CALL Species("C_6H_8O_5",                                    "", "JRROYJLQPIXCRD-UHFFFAOYSA-N", "MCM:PHENOH", "")
  CALL Species("C_6H_8O_6",                                    "", "WQYYYMQUZFKHHO-UHFFFAOYSA-N", "MCM:PHENOOH", "")
  CALL Species("C_7H_{12}O_4",                                 "", "JSEAPEOJJJOJEL-UHFFFAOYSA-N", "MCM:C622CO3H", "")
  CALL Species("C_7H_{12}O_4",                                 "", "VELNAHZLHYPPOS-UHFFFAOYSA-N", "MCM:C624CO3H", "")
  CALL Species("C_7H_{12}O_3",                                 "", "LPMHECUYQWFTDP-UHFFFAOYSA-N", "MCM:C720OOH", "")
  CALL Species("C_7H_{10}O_5",                                 "", "UMHUAUCNJCQCRC-UHFFFAOYSA-N", "MCM:CRESOH", "")
  CALL Species("C_7H_{10}O_6",                                 "", "POYQJFWARUGMBT-UHFFFAOYSA-N", "MCM:CRESOOH", "")
  CALL Species("C_7H_{16}O_3",                                 "", "MDRLNEUVCFZYIZ-UHFFFAOYSA-N", "MCM:H2M2C65OOH", "")
  CALL Species("C_7H_{16}O_3",                                 "", "HGFYTCXFDJYCQF-UHFFFAOYSA-N", "MCM:H2M4C65OOH", "")
  CALL Species("C_7H_{16}O_3",                                 "", "NIKXNAUPHRWTEG-UHFFFAOYSA-N", "MCM:H2M5C65OOH", "")
  CALL Species("C_7H_{14}O_4",                                 "", "DTTYYTHTMWEWDN-UHFFFAOYSA-N", "MCM:H3M3C5CO3H", "")
  CALL Species("C_7H_{16}O_3",                                 "", "IWLJNMJTDMGLQI-UHFFFAOYSA-N", "MCM:H3M3C6OOH", "")
  CALL Species("C_7H_{16}O_3",                                 "", "YDKPOQVDDHNRNK-UHFFFAOYSA-N", "MCM:HO3C76OOH", "")
  CALL Species("C_7H_{14}O_4",                                 "", "KSOZMYOJLUVTBJ-UHFFFAOYSA-N", "MCM:HO5C6CO3H", "")
  CALL Species("C_7H_{16}O_3",                                 "", "YGTQIECBHSKCOV-UHFFFAOYSA-N", "MCM:HO6C7OOH", "")
  CALL Species("C_7H_{10}O_4",                                 "", "CXEVRKZODGVABW-UHFFFAOYSA-N", "MCM:TLBIPER2OH", "")
  CALL Species("C_7H_{10}O_5",                                 "", "JAKOWCUSWIMTAF-UHFFFAOYSA-N", "MCM:TLBIPEROOH", "")
  CALL Species("C_8H_{12}O_5",                                 "", "BUWSXDLGJSAPEF-UHFFFAOYSA-N", "MCM:EBENZOLOH", "")
  CALL Species("C_8H_{12}O_6",                                 "", "RHHRSXIXEXUGSZ-UHFFFAOYSA-N", "MCM:EBENZOLOOH", "")
  CALL Species("C_8H_{12}O_4",                                 "", "AUMASFBINMOOAH-UHFFFAOYSA-N", "MCM:EBZBPER2OH", "")
  CALL Species("C_8H_{12}O_5",                                 "", "SCDRKAWXSVOIOT-UHFFFAOYSA-N", "MCM:EBZBPEROOH", "")
  CALL Species("C_8H_{18}O_3",                                 "", "SPJZDLLIJATZHB-UHFFFAOYSA-N", "MCM:HO3C86OOH", "")
  CALL Species("C_8H_{16}O_4",                                 "", "UKHVIUJOBBSNIJ-UHFFFAOYSA-N", "MCM:HO6C7CO3H", "")
  CALL Species("C_8H_{18}O_3",                                 "", "NDXYTTLTSMWWGD-UHFFFAOYSA-N", "MCM:HO7C8OOH", "")
  CALL Species("C_8H_{12}O_4",                                 "", "HJWPGRMMUKJUOW-UHFFFAOYSA-N", "MCM:MXYBPER2OH", "")
  CALL Species("C_8H_{12}O_5",                                 "", "JLBHNTWNBASURB-UHFFFAOYSA-N", "MCM:MXYBPEROOH", "")
  CALL Species("C_8H_{12}O_5",                                 "", "HYWULKMMJLAVGS-UHFFFAOYSA-N", "MCM:MXYOLOH", "")
  CALL Species("C_8H_{12}O_6",                                 "", "GRFXDNYNQVBVKX-UHFFFAOYSA-N", "MCM:MXYOLOOH", "")
  CALL Species("C_8H_{12}O_4",                                 "", "VCLGCGXZSIRZEA-UHFFFAOYSA-N", "MCM:OXYBPER2OH", "")
  CALL Species("C_8H_{12}O_5",                                 "", "AMPOFIFBYJNSOH-UHFFFAOYSA-N", "MCM:OXYBPEROOH", "")
  CALL Species("C_8H_{12}O_5",                                 "", "CAIZPBLFMULAII-UHFFFAOYSA-N", "MCM:OXYOLOH", "")
  CALL Species("C_8H_{12}O_6",                                 "", "QYYRJAPHRBSJOE-UHFFFAOYSA-N", "MCM:OXYOLOOH", "")
  CALL Species("C_8H_{12}O_4",                                 "", "IOEDFCIBQPWSMP-UHFFFAOYSA-N", "MCM:PXYBPER2OH", "")
  CALL Species("C_8H_{12}O_5",                                 "", "LZVOCZWDZPUJRQ-UHFFFAOYSA-N", "MCM:PXYBPEROOH", "")
  CALL Species("C_8H_{12}O_5",                                 "", "YSBIHFSZRPVTOU-UHFFFAOYSA-N", "MCM:PXYOLOH", "")
  CALL Species("C_8H_{12}O_6",                                 "", "ROOFTVJQSDLVOJ-UHFFFAOYSA-N", "MCM:PXYOLOOH", "")
  CALL Species("C_9H_{20}O_3",                                 "", "GGTTUTGCXALAOI-UHFFFAOYSA-N", "MCM:HO3C96OOH", "")
  CALL Species("C_9H_{18}O_4",                                 "", "LZPDGAGYXQEGIA-UHFFFAOYSA-N", "MCM:HO7C8CO3H", "")
  CALL Species("C_9H_{20}O_3",                                 "", "DDKPTZKAQLYQQK-UHFFFAOYSA-N", "MCM:HO8C9OOH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "MXOPSFWTSRHQGT-UHFFFAOYSA-N", "MCM:IPBENZOLOH", "")
  CALL Species("C_9H_{14}O_4",                                 "", "ZYFWWFFTFWUTQU-UHFFFAOYSA-N", "MCM:IPBZBPR2OH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "PFZQQVVUSVRBJH-UHFFFAOYSA-N", "MCM:IPBZBPROOH", "")
  CALL Species("C_9H_{14}O_6",                                 "", "CHCZSTHNGRHXBM-UHFFFAOYSA-N", "MCM:IPBZOLOOH", "")
  CALL Species("C_9H_{14}O_4",                                 "", "WBMKIPDFTCRXQQ-UHFFFAOYSA-N", "MCM:METLBPR2OH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "JDOROKVPYHJTEM-UHFFFAOYSA-N", "MCM:METLBPROOH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "JGSZPUTWQXTPGZ-UHFFFAOYSA-N", "MCM:METOLOH", "")
  CALL Species("C_9H_{14}O_6",                                 "", "WHWNZVSDBMWGPT-UHFFFAOYSA-N", "MCM:METOLOOH", "")
  CALL Species("C_9H_{14}O_4",                                 "", "HUTNYAIOWRBWAA-UHFFFAOYSA-N", "MCM:OETLBPR2OH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "JWHYGBDQJAPXFD-UHFFFAOYSA-N", "MCM:OETLBPROOH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "RKSMHAQFVDQZCV-UHFFFAOYSA-N", "MCM:OETOLOH", "")
  CALL Species("C_9H_{14}O_6",                                 "", "HMIPOFAAVJMIHR-UHFFFAOYSA-N", "MCM:OETOLOOH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "PPOCUJCGHLIXAB-UHFFFAOYSA-N", "MCM:PBENZOLOH", "")
  CALL Species("C_9H_{14}O_6",                                 "", "YYEAFIOCCPRRFC-UHFFFAOYSA-N", "MCM:PBENZOLOOH", "")
  CALL Species("C_9H_{14}O_4",                                 "", "NXHGWQBANLWNAY-UHFFFAOYSA-N", "MCM:PBZBPER2OH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "BDTBCPZZIFVKES-UHFFFAOYSA-N", "MCM:PBZBPEROOH", "")
  CALL Species("C_9H_{14}O_4",                                 "", "ZJSMOJBPUXMKGO-UHFFFAOYSA-N", "MCM:PETLBPR2OH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "HYHORQQVDRWXSH-UHFFFAOYSA-N", "MCM:PETLBPROOH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "WWMQWQMHOYSCPT-UHFFFAOYSA-N", "MCM:PETOLOH", "")
  CALL Species("C_9H_{14}O_6",                                 "", "NWKQTACFFZGFMV-UHFFFAOYSA-N", "MCM:PETOLOOH", "")
  CALL Species("C_9H_{14}O_4",                                 "", "XELNTWYHIBVYRB-UHFFFAOYSA-N", "MCM:TM123BP2OH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "VYRKWTIFXGFOKI-UHFFFAOYSA-N", "MCM:TM123BPOOH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "CUPCKHHMKYENPF-UHFFFAOYSA-N", "MCM:TM123OLOH", "")
  CALL Species("C_9H_{14}O_6",                                 "", "QKUAZMPWBBDUFL-UHFFFAOYSA-N", "MCM:TM123OLOOH", "")
  CALL Species("C_9H_{14}O_4",                                 "", "ALTMQDHYGSUKTA-UHFFFAOYSA-N", "MCM:TM124BP2OH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "WAGRVRZIKOHUQN-UHFFFAOYSA-N", "MCM:TM124BPOOH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "BDZMWXRIFUJLAS-UHFFFAOYSA-N", "MCM:TM124OLOH", "")
  CALL Species("C_9H_{14}O_6",                                 "", "BWEFFYCDYKVJMY-UHFFFAOYSA-N", "MCM:TM124OLOOH", "")
  CALL Species("C_9H_{14}O_4",                                 "", "UOFFRVMQLZOQAS-UHFFFAOYSA-N", "MCM:TM135BP2OH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "JMJJTOHEWPZJQK-UHFFFAOYSA-N", "MCM:TM135BPOOH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "QLKJHZCZIQIKHZ-UHFFFAOYSA-N", "MCM:TM135OLOH", "")
  CALL Species("C_9H_{14}O_6",                                 "", "BLJZZZVPECPSDH-UHFFFAOYSA-N", "MCM:TM135OLOOH", "")
  CALL Species("C_{10}H_{18}O_3",                              "", "RYIWEMFTAFVTLU-UHFFFAOYSA-N", "MCM:APINAOOH", "")
  CALL Species("C_{10}H_{18}O_3",                              "", "USRGRUOHOYDFID-UHFFFAOYSA-N", "MCM:APINBOOH", "")
  CALL Species("C_{10}H_{18}O_3",                              "", "PONCTWLYBMTTOF-UHFFFAOYSA-N", "MCM:APINCOOH", "")
  CALL Species("C_{10}H_{18}O_3",                              "", "YIWAKBLZYMRLPO-UHFFFAOYSA-N", "MCM:BPINAOOH", "")
  CALL Species("C_{10}H_{18}O_3",                              "", "LNOABRBKZGHYLB-UHFFFAOYSA-N", "MCM:BPINBOOH", "")
  CALL Species("C_{10}H_{18}O_3",                              "", "ONJNLKNHLLSXIR-UHFFFAOYSA-N", "MCM:BPINCOOH", "")
  CALL Species("C_{10}H_{16}O_4",                              "", "NCSYYALRPIRWRM-UHFFFAOYSA-N", "MCM:C918CO3H", "")
  CALL Species("C_{10}H_{16}O_4",                              "", "JNPNKLHEXQFLDW-UHFFFAOYSA-N", "MCM:DMEBPR2OH", "")
  CALL Species("C_{10}H_{16}O_5",                              "", "FGEXZPMCUOXGAY-UHFFFAOYSA-N", "MCM:DMEBPROOH", "")
  CALL Species("C_{10}H_{16}O_5",                              "", "KIALIUKUKYONFN-UHFFFAOYSA-N", "MCM:DMEPHOLOH", "")
  CALL Species("C_{10}H_{16}O_6",                              "", "NQWKUASPEMANIL-UHFFFAOYSA-N", "MCM:DMEPHOLOOH", "")
  CALL Species("C_{10}H_{22}O_3",                              "", "FUGKCLXVXWNPCD-UHFFFAOYSA-N", "MCM:HO3C106OOH", "")
  CALL Species("C_{10}H_{20}O_4",                              "", "UIPJRQQFQSAZPI-UHFFFAOYSA-N", "MCM:HO8C9CO3H", "")
  CALL Species("C_{10}H_{18}O_3",                              "", "FPMDSGWWEURSNO-UHFFFAOYSA-N", "MCM:LIMAOOH", "")
  CALL Species("C_{10}H_{18}O_3",                              "", "KUZZIHAARVNWSI-UHFFFAOYSA-N", "MCM:LIMBOOH", "")
  CALL Species("C_{10}H_{18}O_3",                              "", "VVHMRXSQYYKNLC-UHFFFAOYSA-N", "MCM:LIMCOOH", "")
  CALL Species("C_{11}H_{18}O_5",                              "", "ZTCJTCVANLKQAR-UHFFFAOYSA-N", "MCM:DEMPHOLOH", "")
  CALL Species("C_{11}H_{18}O_6",                              "", "MGZZNFCOXGVYJN-UHFFFAOYSA-N", "MCM:DEMPHOLOOH", "")
  CALL Species("C_{11}H_{18}O_4",                              "", "DLRWBBGTDDBDMY-UHFFFAOYSA-N", "MCM:DETLBPR2OH", "")
  CALL Species("C_{11}H_{18}O_5",                              "", "XWTOJCWARUYVPA-UHFFFAOYSA-N", "MCM:DETLBPROOH", "")
  CALL Species("C_{11}H_{24}O_3",                              "", "XCBVDNPGCYSIMS-UHFFFAOYSA-N", "MCM:HO3C116OOH", "")
  CALL Species("C_{12}H_{26}O_3",                              "", "FENUAUYSPLOZDX-UHFFFAOYSA-N", "MCM:HO3C126OOH", "")
  CALL Species("C_{15}H_{26}O_3",                              "", "UHQMVBPMTRQRFN-UHFFFAOYSA-N", "MCM:BCAOOH", "")
  CALL Species("C_{15}H_{26}O_3",                              "", "LZWWDZWHHIWFFS-UHFFFAOYSA-N", "MCM:BCBOOH", "")
  CALL Species("C_{15}H_{26}O_3",                              "", "LYZHLRDUTRINQK-UHFFFAOYSA-N", "MCM:BCCOOH", "")
  CALL Species("C_6H_6O_3",                                    "", "GQLFYNLDVCYDKN-UHFFFAOYSA-N", "MCM:CATEC1OOH", "")
  CALL Species("C_7H_8O_3",                                    "", "XVDDPRZRYAGIKB-UHFFFAOYSA-N", "MCM:MCATEC1OOH", "")
  CALL Species("C_8H_{10}O_3",                                 "", "WXEIAPGLSOOSMJ-UHFFFAOYSA-N", "MCM:ECATEC1OOH", "")
  CALL Species("C_8H_{10}O_3",                                 "", "OTSRNDWSIFNBPX-UHFFFAOYSA-N", "MCM:MXCTEC1OOH", "")
  CALL Species("C_8H_{10}O_3",                                 "", "FFFASTPMSYAWLR-UHFFFAOYSA-N", "MCM:OCATEC1OOH", "")
  CALL Species("C_8H_{10}O_3",                                 "", "XEPXMQBYFZNYBF-UHFFFAOYSA-N", "MCM:PCATEC1OOH", "")
  CALL Species("C_8H_{10}O_3",                                 "", "PRSNAZGJNCSLPW-UHFFFAOYSA-N", "MCM:STYRENOOH", "")
  CALL Species("C_9H_{12}O_3",                                 "", "LUGXJULPVOELPW-UHFFFAOYSA-N", "MCM:IPCATC1OOH", "")
  CALL Species("C_9H_{12}O_3",                                 "", "RXVYKZMASOKYNM-UHFFFAOYSA-N", "MCM:MTCTEC1OOH", "")
  CALL Species("C_9H_{12}O_3",                                 "", "BNYRTZAXTAHDLR-UHFFFAOYSA-N", "MCM:OTCTEC1OOH", "")
  CALL Species("C_9H_{12}O_3",                                 "", "WDHGVTVSFNORIO-UHFFFAOYSA-N", "MCM:PRCATC1OOH", "")
  CALL Species("C_9H_{12}O_3",                                 "", "SXYSFXQBYZNKPP-UHFFFAOYSA-N", "MCM:PTCTEC1OOH", "")
  CALL Species("C_9H_{12}O_3",                                 "", "XFULZEJTRUOVEV-UHFFFAOYSA-N", "MCM:T123CT1OOH", "")
  CALL Species("C_9H_{12}O_3",                                 "", "XVSFIBYJDMNABQ-UHFFFAOYSA-N", "MCM:T124CT1OOH", "")
  CALL Species("C_2H_4O_3",                                    "", "TUJPSEFVDYHSJJ-UHFFFAOYSA-N", "MCM:HCOCH2OOH", "")
  CALL Species("C_2H_2O_4",                                    "", "JOALXJIWVKUVBR-UHFFFAOYSA-N", "MCM:HCOCO3H", "")
  CALL Species("C_3H_4O_4",                                    "", "VUPDPJIDXKCVGY-UHFFFAOYSA-N", "MCM:C3DIALOOH", "")
  CALL Species("C_3H_6O_3",                                    "", "XSASRUDTFFBDDK-UHFFFAOYSA-N", "MCM:CHOC2H4OOH", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Aldehydes (\chem{RCHO})")
  ! aliphatic:
  CALL Species("HCHO",                                  "50-00-0", "WSFSSNUMVMOOMR-UHFFFAOYSA-N", "methanal", "formaldehyde")
  CALL Species("CH_3CHO",                               "75-07-0", "IKHGUXGNUITLKF-UHFFFAOYSA-N", "ethanal", "acetaldehyde")
  CALL Species("OHCCHO",                               "107-22-2", "LEQAOMBKQFMDFZ-UHFFFAOYSA-N", "ethanedial", "glyoxal")
  CALL Species("C_2H_5CHO",                            "123-38-6", "NBBJYMSMWIIQGU-UHFFFAOYSA-N", "propanal", "propionaldehyde")
  CALL Species("C_3H_4O_2",                            "542-78-9", "WSMYVTOQOOLQHP-UHFFFAOYSA-N", "propanedial", "malonaldehyde")
  CALL Species("C_3H_4O_3",                                    "", "NVXLIZQNSVLKPO-UHFFFAOYSA-N", "MCM:C32OH13CO", "")
  CALL Species("C_3H_6O_2",                                    "", "BSABBBMNWQWLLU-UHFFFAOYSA-N", "MCM:CH3CHOHCHO", "")
  CALL Species("C_3H_6O_2",                                    "", "AKXKFZDCRYJKTF-UHFFFAOYSA-N", "MCM:HOC2H4CHO", "")
  CALL Species("C_3H_7CHO",                            "123-72-8", "ZTQSAGDEMFDKMZ-UHFFFAOYSA-N", "butanal", "butyraldehyde")
  CALL Species("C_4H_8O",                               "78-84-2", "AMIMRNSIRUDHCM-UHFFFAOYSA-N", "2-methylpropanal", "isobutyraldehyde")
  CALL Species("C_4H_6O_2",                          "16002-19-0", "VXYSFSCCSQAYJV-UHFFFAOYSA-N", "2-methylpropanedial", "")
  CALL Species("C_4H_4O_2",                                    "", "JGEMYUOFGVHXKV-UHFFFAOYSA-N", "MCM:MALDIAL", "")
  CALL Species("C_4H_6O_3",                                    "", "SQHUBVCIVAIUAB-UHFFFAOYSA-N", "MCM:C3MDIALOH", "")
  CALL Species("C_4H_8O_4",                                    "", "YTBSYETUWUMLBZ-UHFFFAOYSA-N", "MCM:C41OH", "")
  CALL Species("C_4H_8O_5",                                    "", "ROHPNOOUQDXFHZ-UHFFFAOYSA-N", "MCM:C41OOH", "")
  CALL Species("C_4H_8O_3",                                    "", "DFFAMJFPVBTTBX-UHFFFAOYSA-N", "MCM:C4OCCOHCOH", "")
  CALL Species("C_4H_8O_3",                                    "", "CQSYGAZTCJHVFE-UHFFFAOYSA-N", "MCM:CHOC3DIOL", "")
  CALL Species("C_4H_6O_2",                                    "", "PPNVQCFSKPIRKK-UHFFFAOYSA-N", "MCM:HC3CCHO", "")
  CALL Species("C_4H_6O_2",                                    "", "FXCMZPXXCRHRNK-UHFFFAOYSA-N", "MCM:HC3CHO", "")
  CALL Species("C_4H_8O_4",                                    "", "OUEKYUKJSLEOIU-UHFFFAOYSA-N", "MCM:HMACROH", "")
  CALL Species("C_4H_6O_2",                                    "", "QVBICLGJAQXLSA-UHFFFAOYSA-N", "MCM:HMACR", "")
  CALL Species("C_4H_8O_3",                                    "", "NKVLMFFGFYHDNE-UHFFFAOYSA-N", "MCM:HO13C3CHO", "")
  CALL Species("C_4H_8O_2",                                    "", "HSJKGGMUJITCBW-UHFFFAOYSA-N", "MCM:HO2C3CHO", "")
  CALL Species("C_4H_8O_2",                                    "", "UIKQNMXWCYQNCS-UHFFFAOYSA-N", "MCM:HO3C3CHO", "")
  CALL Species("C_4H_8O_2",                                    "", "PIAOXUVIBAKVSP-UHFFFAOYSA-N", "MCM:HOC3H6CHO", "")
  CALL Species("C_4H_6O_4",                                    "", "UUWVJXZLAWXQBU-UHFFFAOYSA-N", "MCM:HOHOC4DIAL", "")
  CALL Species("C_4H_8O_2",                                    "", "JTMCAHGCWBGWRV-UHFFFAOYSA-N", "MCM:HOIPRCHO", "")
  CALL Species("C_4H_8O_2",                                    "", "HNVAGBIANFAIIL-UHFFFAOYSA-N", "MCM:IBUTALOH", "")
  CALL Species("C_4H_8O_3",                                    "", "JBCPUXACCOWZEB-UHFFFAOYSA-N", "MCM:MACROH", "")
  CALL Species("C_4H_9CHO",                            "110-62-3", "HGBOYTHUEUWSSQ-UHFFFAOYSA-N", "pentanal", "valeraldehyde")
  CALL Species("C_5H_{10}O",                            "96-17-3", "BYGQBDHUGHBGMD-UHFFFAOYSA-N", "2-methylbutanal", "")
  CALL Species("C_5H_{10}O",                           "590-86-3", "YGHRJJRRZDOVPD-UHFFFAOYSA-N", "3-methylbutanal", "isovaleraldehyde")
  CALL Species("C_5H_6O_2",                                    "", "USBJWIKCHJDWPF-UHFFFAOYSA-N", "MCM:C4MDIAL", "")
  CALL Species("C_5H_8O_2",                                    "", "WNBFTLCNQKKVHC-UHFFFAOYSA-N", "MCM:CO1M22CHO", "")
  CALL Species("C_5H_{10}O",                           "630-19-3", "FJJYHTVHBVXEEQ-UHFFFAOYSA-N", "2,2-dimethylpropanal", "pivaldehyde")
  CALL Species("C_5H_8O_3",                                    "", "QTDYRJFEJDMBDI-UHFFFAOYSA-N", "MCM:C3EDIALOH", "")
  CALL Species("C_5H_{10}O_2",                                 "", "VKYKDJZVZBURQF-UHFFFAOYSA-N", "MCM:C3M3OH2CHO", "")
  CALL Species("C_5H_{10}O_3",                                 "", "NGVSSTUGOBBIBC-UHFFFAOYSA-N", "MCM:C42CHO", "")
  CALL Species("C_5H_8O_4",                                    "", "NUHRTSFMSDZSTE-UHFFFAOYSA-N", "MCM:C4M2AL2OH", "")
  CALL Species("C_5H_{10}O_2",                                 "", "SUTLBTHMXYSMSZ-UHFFFAOYSA-N", "MCM:C4OHCHO", "")
  CALL Species("C_5H_8O_3",                                    "", "UIVALZXFWDRKKE-UHFFFAOYSA-N", "MCM:C514OH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "ZKDSJDSEVDWBAC-UHFFFAOYSA-N", "MCM:C57OH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "HTPZSALIZDTBIL-UHFFFAOYSA-N", "MCM:C58OH", "")
  CALL Species("C_5H_6O_3",                                    "", "KBKQEUDBRUGUIQ-UHFFFAOYSA-N", "MCM:C5DIALOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "KJLUILOCNALPNM-UHFFFAOYSA-N", "MCM:H13C43CHO", "")
  CALL Species("C_5H_{10}O_2",                                 "", "FXFBPKDQLDOIRG-UHFFFAOYSA-N", "MCM:H2M2C3CHO", "")
  CALL Species("C_5H_8O_2",                                    "", "BSHDRMLUCYMQOP-UHFFFAOYSA-N", "MCM:HC4ACHO", "")
  CALL Species("C_5H_8O_2",                                    "", "GCHJBJOOADXJFT-UHFFFAOYSA-N", "MCM:HC4CCHO", "")
  CALL Species("C_5H_{10}O_2",                                 "", "JJMOMMLADQPZNY-UHFFFAOYSA-N", "MCM:HM22CHO", "")
  CALL Species("C_5H_{10}O_3",                                 "", "WDKLWOBHKQJYSU-UHFFFAOYSA-N", "MCM:HO24C4CHO", "")
  CALL Species("C_5H_{10}O_2",                                 "", "ZURZPPULRFXVLF-UHFFFAOYSA-N", "MCM:HO2C43CHO", "")
  CALL Species("C_5H_{10}O_2",                                 "", "HFZMJAMTNAAZQE-UHFFFAOYSA-N", "MCM:HO2C4CHO", "")
  CALL Species("C_5H_{10}O_2",                                 "", "WRWLNLBWBJEUPI-UHFFFAOYSA-N", "MCM:HO3C4CHO", "")
  CALL Species("C_5H_{10}O_2",                                 "", "PLBZJQQQFIOXRU-UHFFFAOYSA-N", "MCM:HOBUT2CHO", "")
  CALL Species("C_5H_{10}O_2",                                 "", "RWXBAXNFTXQJAY-UHFFFAOYSA-N", "MCM:HOIBUTCHO", "")
  CALL Species("C_5H_{10}O_3",                                 "", "YDXYYBJRCIQQSF-UHFFFAOYSA-N", "MCM:MBOBCO", "")
  CALL Species("C_5H_{10}O_2",                                 "", "MEHIGMLPKIJWEA-UHFFFAOYSA-N", "MCM:PROL1MCHO", "")
  CALL Species("C_5H_{11}CHO",                          "66-25-1", "JARKCYVAAOWBJS-UHFFFAOYSA-N", "hexanal", "")
  CALL Species("OHC(CH_2)_3CHO",                       "111-30-8", "SXRSQZLOMIGNAQ-UHFFFAOYSA-N", "pentanedial", "glutaraldehyde")
  CALL Species("C_6H_{12}O",                           "123-15-9", "FTZILAQGHINQQR-UHFFFAOYSA-N", "2-methylpentanal", "2-methylvaleraldehyde")
  CALL Species("C_6H_{10}O_2",                        "1072-21-5", "UMHJEEQLYBKSAN-UHFFFAOYSA-N", "hexanedial", "adipaldehyde")
  CALL Species("C_6H_{12}O",                                   "", "LTNUSYNQZJZUSY-UHFFFAOYSA-N", "MCM:M22C3CHO", "")
  CALL Species("C_6H_{12}O",                                   "", "AKUUEDVRXOZTBF-UHFFFAOYSA-N", "MCM:M2C43CHO", "")
  CALL Species("C_6H_{12}O",                                   "", "QYPLKDUOPJZROX-UHFFFAOYSA-N", "MCM:M33C3CHO", "")
  CALL Species("C_6H_{12}O",                         "15877-57-3", "YJWJGLQYQJGEEP-UHFFFAOYSA-N", "3-methylpentanal", "")
  CALL Species("C_6H_{12}O",                          "1119-16-0", "JGEGJYXHCFUMJF-UHFFFAOYSA-N", "4-methylpentanal", "")
  CALL Species("C_6H_{12}O",                            "97-96-1", "UNNGUFMVYQJGTD-UHFFFAOYSA-N", "2-ethylbutanal", "")
  CALL Species("C_6H_{10}O_2",                                 "", "XEAYIEUKFACAKS-UHFFFAOYSA-N", "MCM:C518CHO", "")
  CALL Species("C_6H_{10}O_3",                                 "", "LOUHJSYYTNSRBN-UHFFFAOYSA-N", "MCM:C615OH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "ASGSKBSLHTVURJ-UHFFFAOYSA-N", "MCM:C623OH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "SIBVVDHXFPINCX-UHFFFAOYSA-N", "MCM:C67OH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "RJKLRJTWLKKQSE-UHFFFAOYSA-N", "MCM:C68OH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "GATFIJYWERJMGU-UHFFFAOYSA-N", "MCM:C6DIALOH", "")
  CALL Species("C_6H_{12}O_2",                                 "", "FPFTWHJPEMPAGE-UHFFFAOYSA-N", "MCM:CO1C6OH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "TWSXDPXQZVBPJU-UHFFFAOYSA-N", "MCM:CO1H63OH", "")
  CALL Species("C_6H_{12}O_2",                                 "", "NEJZWIVQOAKZHY-UHFFFAOYSA-N", "MCM:H2M2C4CHO", "")
  CALL Species("C_6H_{12}O_2",                                 "", "GUXDGBBNSMURMG-UHFFFAOYSA-N", "MCM:H2M3C4CHO", "")
  CALL Species("C_6H_{12}O_2",                                 "", "HKXRCBUXVFAKDA-UHFFFAOYSA-N", "MCM:H3M3C4CHO", "")
  CALL Species("C_6H_{12}O_2",                                 "", "VOISQCUJWIZDHK-UHFFFAOYSA-N", "MCM:HM22C3CHO", "")
  CALL Species("C_6H_{12}O_2",                                 "", "QQZFDCJFBSAIBQ-UHFFFAOYSA-N", "MCM:HM2C43CHO", "")
  CALL Species("C_6H_{12}O_2",                                 "", "YHMMFPKTSXQULG-UHFFFAOYSA-N", "MCM:HM33C3CHO", "")
  CALL Species("C_6H_{12}O_2",                                 "", "ZHZOUKIXPREHLS-UHFFFAOYSA-N", "MCM:HO2C54CHO", "")
  CALL Species("C_6H_{12}O_2",                                 "", "HAPLZPWSLJPZBM-UHFFFAOYSA-N", "MCM:HO3C5CHO", "")
  CALL Species("C_6H_{12}O_2",                                 "", "BRZMRZVKWQWYPJ-UHFFFAOYSA-N", "MCM:HO5C5CHO", "")
  CALL Species("C_6H_{13}CHO",                         "111-71-7", "FXHGMKSSBGDXIY-UHFFFAOYSA-N", "heptanal", "")
  CALL Species("C_7H_{12}O_2",                                 "", "JWVBPJJNIHELFZ-UHFFFAOYSA-N", "MCM:C622CHO", "")
  CALL Species("C_7H_{12}O_2",                                 "", "AIMYSSDKEKDSEU-UHFFFAOYSA-N", "MCM:C624CHO", "")
  CALL Species("C_7H_{14}O_4",                                 "", "RQDWEXZQVQBWBC-UHFFFAOYSA-N", "MCM:C728OH", "")
  CALL Species("C_7H_{14}O_4",                                 "", "ROYLBPWMPALPRM-UHFFFAOYSA-N", "MCM:C730OH", "")
  CALL Species("C_7H_{14}O_2",                                 "", "PUMVOOYIPAYIQQ-UHFFFAOYSA-N", "MCM:H3M3C5CHO", "")
  CALL Species("C_7H_{15}CHO",                         "124-13-0", "NUJGJRNETVAIRJ-UHFFFAOYSA-N", "octanal", "")
  CALL Species("C_8H_{16}O",                           "123-05-7", "LGYNIFWIKSEESD-UHFFFAOYSA-N", "2-ethylhexanal", "")
  CALL Species("C_8H_{12}O_2",                                 "", "MEGRLTONSLAOOM-UHFFFAOYSA-N", "MCM:C729CHO", "")
  CALL Species("C_8H_{14}O_3",                                 "", "WBGXLUPQADVOIR-UHFFFAOYSA-N", "MCM:C810OH", "")
  CALL Species("C_8H_{14}O_2",                                 "", "CCTLWIOCAVSPLW-UHFFFAOYSA-N", "MCM:C822OH", "")
  CALL Species("C_8H_{14}O_3",                                 "", "RBBQERVBTKJTLF-UHFFFAOYSA-N", "MCM:C824OH", "")
  CALL Species("C_8H_{14}O_4",                                 "", "LLAUXDLXTPQIPI-UHFFFAOYSA-N", "MCM:C826OH", "")
  CALL Species("C_8H_{14}O_2",                                 "", "KODLEZWYEGWLGP-UHFFFAOYSA-N", "MCM:C830OH", "")
  CALL Species("C_8H_{14}O_3",                                 "", "YTEDFRLZFCPFBH-UHFFFAOYSA-N", "MCM:C831OH", "")
  CALL Species("C_8H_{14}O_2",                                 "", "KMZPPEYMMZMCIQ-UHFFFAOYSA-N", "MCM:C89OH", "")
  CALL Species("C_8H_{17}CHO",                         "124-19-6", "GYHFUZHODSMOHU-UHFFFAOYSA-N", "nonanal", "")
  CALL Species("C_9H_{18}O",                          "5435-64-3", "WTPYRCJDOZVZON-UHFFFAOYSA-N", "3,5,5-trimethylhexanal", "")
  CALL Species("C_9H_{19}CHO",                         "112-31-2", "KSMVZQYAVGTKIV-UHFFFAOYSA-N", "decanal", "")
  CALL Species("C_{10}H_{16}O_2",                              "", "GWMMOZICECYZLL-UHFFFAOYSA-N", "MCM:C918CHO", "")
  CALL Species("C_{10}H_{20}O_2",                      "107-75-5", "WPFVBOQKRVRMJB-UHFFFAOYSA-N", "hydroxycitronellal", "")
  CALL Species("C_{11}H_{22}O",                        "112-44-7", "KMPQYAYAQWNLME-UHFFFAOYSA-N", "undecanal", "")
  CALL Species("C_{12}H_{24}O",                        "112-54-9", "HFJRKMMYBMWEAD-UHFFFAOYSA-N", "dodecanal", "")
  CALL Species("C_{13}H_{26}O",                      "10486-19-8", "BGEHHAVMRVXCGR-UHFFFAOYSA-N", "1-tridecanal", "")
  CALL Species("C_{14}H_{28}O",                        "124-25-4", "UHUFTBALEZWWIH-UHFFFAOYSA-N", "1-tetradecanal", "")
  CALL Species("C_{12}H_{18}O",                    "1339119-15-1", "BFTAXJRKNXWMMX-UHFFFAOYSA-N", "aquaflora", "")
  CALL Species("C_{13}H_{24}O",                    "1254940-85-6", "XUTZHJNXAOHUTM-UHFFFAOYSA-N", "starfleur", "3-(4-isobutylcyclohexyl)propanal")
  CALL Species("C_{12}H_{18}O_2",                              "", "IWMJDJFLMSAAES-UHFFFAOYSA-N", "MCM:C126CO", "")
  CALL Species("C_{12}H_{20}O_2",                              "", "PQYIIYBUHROKIY-UHFFFAOYSA-N", "MCM:C126OH", "")
  CALL Species("C_{12}H_{20}O_3",                              "", "QHKXYVNBHFONML-UHFFFAOYSA-N", "MCM:C127OH", "")
  CALL Species("C_{13}H_{20}O_2",                              "", "ZDIXDRBTLRZYNM-UHFFFAOYSA-N", "MCM:C126CHO", "")
  CALL Species("C_{13}H_{22}O_3",                              "", "CBUCUSJJCJUTML-UHFFFAOYSA-N", "MCM:C1311OH", "")
  CALL Species("C_{13}H_{22}O_4",                              "", "XIWFBNXBOAYHAJ-UHFFFAOYSA-N", "MCM:C1313OH", "")
  CALL Species("C_{13}H_{22}O_2",                              "", "GEHGNVSPJQGFQG-UHFFFAOYSA-N", "MCM:C136OH", "")
  CALL Species("CH_2CHCHO",                            "107-02-8", "HGINCPLSRVDWNT-UHFFFAOYSA-N", "propenal", "acrolein")
  CALL Species("C_4H_6O",                               "78-85-3", "STNJBCKSHOAVAJ-UHFFFAOYSA-N", "2-methylpropenal", "methacrolein")
  CALL Species("CH_3CHCHCHO",                         "4170-30-3", "MLUCVPSAIODCQM-UHFFFAOYSA-N", "2-butenal", "crotonaldehyde")
  CALL Species("CH_3CHCHCHO",                          "123-73-9", "MLUCVPSAIODCQM-NSCUHMNNSA-N", "($E$)-2-butenal", "{\it trans}-crotonaldehyde")
  CALL Species("C_3H_7CHCHCHO",                        "505-57-7", "MBDOYVRWFFCFHM-UHFFFAOYSA-N", "2-hexenal", "")
  CALL Species("C_3H_7CHCHCHO",                       "6728-26-3", "MBDOYVRWFFCFHM-SNAWJCMRSA-N", "($E$)-2-hexenal", "{\it trans}-2-hexenal")
  CALL Species("CH_3CHCHCHCHCHO",                      "142-83-6", "BATOPAZDIZEVQF-MQQKCMAXSA-N", "($E$,$E$)-2,4-hexadienal", "{\it trans}-{\it trans}-2,4-hexadienal") ! other CAS: 4488-48-6
  CALL Species("C_7H_{12}O",                          "2463-63-0", "NDFKTBCGKNOHPJ-UHFFFAOYSA-N", "2-heptenal", "")
  CALL Species("C_7H_{12}O",                         "18829-55-5", "NDFKTBCGKNOHPJ-AATRIKPKSA-N", "($E$)-2-heptenal", "{\it trans}-2-heptenal")
  CALL Species("C_7H_{12}O",                          "6728-31-0", "VVGOCOMZRGWHPI-ARJAWSKDSA-N", "($Z$)-4-heptenal", "{\it cis}-4-heptenal")
  CALL Species("C_8H_{14}O",                          "2363-89-5", "LVBXEMGDVWVTGY-UHFFFAOYSA-N", "2-octenal", "")
  CALL Species("C_8H_{14}O",                          "2548-87-0", "LVBXEMGDVWVTGY-VOTSOKGWSA-N", "($E$)-2-octenal", "{\it trans}-2-octenal")
  CALL Species("C_8H_{14}O",                           "645-62-5", "PYLMCYQHBRSDND-UHFFFAOYSA-N", "2-ethyl-2-hexenal", "")
  CALL Species("C_9H_{16}O",                         "18829-56-6", "BSAIUMLZVGUGKX-BQYQJAHWSA-N", "($E$)-2-nonenal", "{\it trans}-2-nonenal")
  CALL Species("C_9H_{14}O",                           "557-48-2", "HZYHMHHBBBSGHB-ODYTWBPASA-N", "{\it trans}-2,{\it cis}-6-nonadienal", "")
  CALL Species("C_{10}H_{18}O",                        "106-23-0", "NEHNMFOYXAPHSD-UHFFFAOYSA-N", "3,7-dimethyl-6-octenal", "citronellal")
  CALL Species("C_{10}H_{16}O",                       "5392-40-5", "WTEVQBCEXWBHNA-UHFFFAOYSA-N", "3,7-dimethyl-2,6-octadienal", "citral")
  CALL Species("C_{11}H_{18}O",                     "850997-10-3", "IHMKWBJKOWYASH-UHFFFAOYSA-N", "tillenal", "3-(4,4-dimethyl-1-cyclohexen-1-yl)propanal")
  CALL Species("C_{12}H_{20}O",                    "1378867-81-2", "VZZSYXAVGYODQG-UHFFFAOYSA-N", "lilybelle", "")
  CALL Species("C_{13}H_{22}O",                                "", "WGWWNPFQPHLSIM-UHFFFAOYSA-N", "mugoxal", "3-(4-{\it tert}-butylcyclohexen-1-yl)propanal")
  CALL Species("C_{15}H_{22}O",                       "8028-48-6", "NOPLRNXKHZRXHT-UHFFFAOYSA-N", "orange oil", "") ! other CAS: 60066-88-8
  ! aromatic:
  CALL Species("C_6H_5CHO",                            "100-52-7", "HUMNYLRZRPPJDN-UHFFFAOYSA-N", "benzaldehyde", "")
  CALL Species("C_6H_5CH_2CHO",                        "122-78-1", "DTUQWGWMVIHBKE-UHFFFAOYSA-N", "phenylacetaldehyde", "")
  CALL Species("C_8H_8O",                              "529-20-4", "BTFQKIATRPGRBS-UHFFFAOYSA-N", "2-methylbenzaldehyde", "$o$-tolualdehyde")
  CALL Species("C_8H_8O",                              "620-23-5", "OVWYEQOVUDKZNU-UHFFFAOYSA-N", "3-methylbenzaldehyde", "$m$-tolualdehyde")
  CALL Species("C_8H_8O",                              "104-87-0", "FXLOVSHXALFLKQ-UHFFFAOYSA-N", "4-methylbenzaldehyde", "$p$-tolualdehyde")
  CALL Species("C_8H_6O_2",                            "623-27-8", "KUCOHFSKRZZVRO-UHFFFAOYSA-N", "terephthaldialdehyde", "")
  CALL Species("C_6H_4(OH)CHO",                         "90-02-8", "SMQUZDBALVYZAC-UHFFFAOYSA-N", "2-hydroxybenzaldehyde", "2-formylphenol; salicylaldehyde")
  CALL Species("C_6H_4(OH)CHO",                        "100-83-4", "IAVREABSGIHHMO-UHFFFAOYSA-N", "3-hydroxybenzaldehyde", "3-formylphenol")
  CALL Species("C_6H_4(OH)CHO",                        "123-08-0", "RGHHSNMVTDWUBI-UHFFFAOYSA-N", "4-hydroxybenzaldehyde", "4-formylphenol")
  CALL Species("C_8H_8O_2",                            "135-02-4", "PKZJLOCLABXVMC-UHFFFAOYSA-N", "2-methoxybenzaldehyde", "")
  CALL Species("C_8H_8O_2",                            "591-31-1", "WMPDAIZRQDCGFH-UHFFFAOYSA-N", "3-methoxybenzaldehyde", "")
  CALL Species("C_8H_8O_2",                            "123-11-5", "ZRSNZINYAWTAHE-UHFFFAOYSA-N", "4-methoxybenzaldehyde", "")
  CALL Species("C_9H_8O",                              "104-55-2", "KJPRLNWUNMBNBZ-QPJJXVBHSA-N", "3-phenyl-2-propenal", "cinnamaldehyde")
  CALL Species("C_9H_{10}O",                          "5779-93-1", "UIFVCPMLQXKEEU-UHFFFAOYSA-N", "2,3-dimethylbenzaldehyde", "")
  CALL Species("C_9H_{10}O",                          "5973-71-7", "POQJHLBMLVTHAU-UHFFFAOYSA-N", "3,4-dimethylbenzaldehyde", "")
  CALL Species("C_9H_{10}O",                          "5779-95-3", "NBEFMISJJNGCIZ-UHFFFAOYSA-N", "3,5-dimethylbenzaldehyde", "")
  CALL Species("C_{11}H_8O",                            "66-99-9", "PJKVFARRVXDXAD-UHFFFAOYSA-N", "2-naphthaldehyde", "")
  CALL Species("C_{12}H_{16}O",                       "7775-00-0", "RLEFOSDUWZYGOS-UHFFFAOYSA-N", "cyclemax", "4-(1-methylethyl)-benzenepropanal")
  CALL Species("C_{13}H_{18}O",                      "18127-01-0", "FZJUFJKVIYFBSY-UHFFFAOYSA-N", "bourgeonal", "")
  CALL Species("C_{13}H_{18}O",                     "125109-85-5", "OHRBQTOZYGEWCJ-UHFFFAOYSA-N", "florhydral", "")
  CALL Species("C_{13}H_{18}O",                        "103-95-7", "ZFNVDHOSLNRHNN-UHFFFAOYSA-N", "cyclamen aldehyde", "")
  CALL Species("C_{13}H_{16}O",                    "1226911-69-8", "LBKHGAIELUNYML-ZRDIBKRKSA-N", "mimosal", "4-methyl-5-(4-methylphenyl)pent-4-enal")
  CALL Species("C_{14}H_{20}O",                         "80-54-6", "SDQFDHOLCGWZPU-UHFFFAOYSA-N", "lilial", "")
  CALL Species("C_{14}H_{18}O",                     "300371-33-9", "GEPCDOWRWODJEY-UHFFFAOYSA-N", "hivernal", "")
  CALL Species("C_{14}H_{20}O",                       "6658-48-6", "YLIXVKUWWOQREC-UHFFFAOYSA-N", "silvial", "")
  CALL Species("C_{14}H_{20}O",                      "62518-65-4", "GLZRHVTZLDNUQP-UHFFFAOYSA-N", "mefloral", "")
  CALL Species("C_{14}H_{20}O",                    "1637294-12-2", "UKZXPOJABTXLMK-UHFFFAOYSA-N", "nympheal", "3-(4-isobutyl-2-methylphenyl)propanal")
  CALL Species("C_{15}H_{20}O",                        "101-86-0", "GUUHFMWKWLOQMM-UHFFFAOYSA-N", "hexyl cinnamic aldehyde", "")
  CALL Species("C_{14}H_{18}O",                        "122-40-7", "HMKKIXGYKWDQSV-KAMYIIQDSA-N", "$\alpha$-amyl cinnamaldehyde", "")
  ! peroxides:
  CALL Species("C_3H_4O_4",                                    "", "CFPRXLWZTODTLN-UHFFFAOYSA-N", "MCM:HCOCH2CO3H", "")
  CALL Species("C_3H_6O_3",                                    "", "UTLVMFCPERHVKU-UHFFFAOYSA-N", "MCM:PROPALOOH", "")
  CALL Species("C_4H_8O_3",                                    "", "UBMPIIYSSVMXGT-UHFFFAOYSA-N", "MCM:BUTALAOOH", "")
  CALL Species("C_4H_8O_3",                                    "", "HFNSAVKBHMZYBR-UHFFFAOYSA-N", "MCM:BUTALO2H", "")
  CALL Species("C_4H_6O_4",                                    "", "WJWKLOFSCXWKQD-UHFFFAOYSA-N", "MCM:C3MDIALOOH", "")
  CALL Species("C_4H_6O_4",                                    "", "ZCKJJZIUFVHIST-UHFFFAOYSA-N", "MCM:CHOC2CO3H", "")
  CALL Species("C_4H_8O_3",                                    "", "OSYMKIVGDGNGKG-UHFFFAOYSA-N", "MCM:IBUTALBO2H", "")
  CALL Species("C_4H_8O_3",                                    "", "UINKREZTGVLFAU-UHFFFAOYSA-N", "MCM:IBUTALO2H", "")
  CALL Species("C_4H_4O_4",                                    "", "QFBRACYMKXGFRM-UHFFFAOYSA-N", "MCM:MALDALCO3H", "")
  CALL Species("C_4H_6O_4",                                    "", "IGHWWGZEWMXWCB-UHFFFAOYSA-N", "MCM:PRPAL2CO3H", "")
  CALL Species("C_5H_8O_4",                                    "", "IOKXWRCVGKGMQG-UHFFFAOYSA-N", "MCM:C3EDIALOOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "FJYKIGXUSFRBER-UHFFFAOYSA-N", "MCM:C3M3CHOOOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "ZCMMOPXACREGKB-UHFFFAOYSA-N", "MCM:C4CHOAOOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "JELNOURJPXUYRE-UHFFFAOYSA-N", "MCM:C4CHOBOOH", "")
  CALL Species("C_5H_6O_4",                                    "", "JLZDPUBRQDSTHL-UHFFFAOYSA-N", "MCM:C4CODBCO3H", "")
  CALL Species("C_5H_8O_4",                                    "", "PUFIIBDSTKVRQR-UHFFFAOYSA-N", "MCM:C514OOH", "")
  CALL Species("C_5H_6O_4",                                    "", "MJFLAVCETRHPDB-UHFFFAOYSA-N", "MCM:C5DIALOOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "CTZAXNMAMSERLK-UHFFFAOYSA-N", "MCM:CHOC4OOH", "")
  CALL Species("C_5H_8O_4",                                    "", "HHOOHNHNSGMLRV-UHFFFAOYSA-N", "MCM:CO1M22CO3H", "")
  CALL Species("C_5H_{10}O_3",                                 "", "MEJCTHPUJKQILI-UHFFFAOYSA-N", "MCM:IC4CHOAOOH", "")
  CALL Species("C_5H_6O_4",                                    "", "UTZOYJZISPOIDM-UHFFFAOYSA-N", "MCM:MC3ODBCO3H", "")
  CALL Species("C_6H_8O_4",                                    "", "NCACXMVSAKPDND-UHFFFAOYSA-N", "MCM:C522CO3H", "")
  CALL Species("C_6H_{10}O_4",                                 "", "XHCYBISQWAKVQU-UHFFFAOYSA-N", "MCM:C615OOH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "UGGYJIUZOSPDSO-UHFFFAOYSA-N", "MCM:C6DIALOOH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "RHHSYYPMFOTTRH-UHFFFAOYSA-N", "MCM:CHOC4CO3H", "")
  CALL Species("C_6H_{12}O_3",                                 "", "HEPNQRDDKATRCQ-UHFFFAOYSA-N", "MCM:CO1C6OOH", "")
  CALL Species("C_7H_{10}O_5",                                 "", "DXQJLRIPGQOCBO-UHFFFAOYSA-N", "MCM:C615CO3H", "")
  CALL Species("C_7H_{12}O_3",                                 "", "PQESPNYRBXTFKD-UHFFFAOYSA-N", "MCM:C729OOH", "")
  CALL Species("C_8H_{12}O_4",                                 "", "MJXLYBJUYDBXAC-UHFFFAOYSA-N", "MCM:C729CO3H", "")
  CALL Species("C_8H_{14}O_4",                                 "", "YJCLQCSGAKCWEJ-UHFFFAOYSA-N", "MCM:C810OOH", "")
  CALL Species("C_8H_{14}O_3",                                 "", "XWYULULFGKWPMD-UHFFFAOYSA-N", "MCM:C822OOH", "")
  CALL Species("C_8H_{14}O_3",                                 "", "QPQAMUJVCCDVQB-UHFFFAOYSA-N", "MCM:C830OOH", "")
  CALL Species("C_8H_{14}O_4",                                 "", "IGXVNESWPQHYEP-UHFFFAOYSA-N", "MCM:C831OOH", "")
  CALL Species("C_8H_{14}O_3",                                 "", "TYPKFPBYWMWINI-UHFFFAOYSA-N", "MCM:C89OOH", "")
  CALL Species("C_9H_{14}O_4",                                 "", "DRDXOQWGQPPQFG-UHFFFAOYSA-N", "MCM:C822CO3H", "")
  CALL Species("C_9H_{14}O_4",                                 "", "YIGLTTCQERBPNX-UHFFFAOYSA-N", "MCM:C830CO3H", "")
  CALL Species("C_9H_{14}O_4",                                 "", "QJRQWHYQDGSSTJ-UHFFFAOYSA-N", "MCM:C89CO3H", "")
  CALL Species("C_{12}H_{20}O_3",                              "", "KEVCPNVRIXMZTI-UHFFFAOYSA-N", "MCM:C126OOH", "")
  CALL Species("C_{13}H_{20}O_4",                              "", "OZAZBNTYEOZXRC-UHFFFAOYSA-N", "MCM:C126CO3H", "")
  CALL Species("C_{13}H_{22}O_3",                              "", "MAMVWZJYPZZMSJ-UHFFFAOYSA-N", "MCM:C136OOH", "")
  CALL Species("C_{14}H_{22}O_4",                              "", "JKRMUSIUVDAEPW-UHFFFAOYSA-N", "MCM:C136CO3H", "")
  CALL Species("C_3H_4O_5",                                    "", "DIZMXPJJIVKAJT-UHFFFAOYSA-N", "MCM:HCOCOHCO3H", "")
  CALL Species("C_3H_6O_4",                                    "", "GWWSSJVARZQKFN-UHFFFAOYSA-N", "MCM:HOCHOCOOH", "")
  CALL Species("C_3H_6O_4",                                    "", "UFAXCNDNIIDPQE-UHFFFAOYSA-N", "MCM:OCCOHCOOH", "")
  CALL Species("C_4H_8O_4",                                    "", "JJIHBDXOHFOUDV-UHFFFAOYSA-N", "MCM:C4OCCOHOOH", "")
  CALL Species("C_4H_8O_4",                                    "", "XTEYRSCVGRLIOG-UHFFFAOYSA-N", "MCM:COCCOHCOOH", "")
  CALL Species("C_4H_6O_5",                                    "", "AEAZUHMXWJBPCW-UHFFFAOYSA-N", "MCM:COHM2CO3H", "")
  CALL Species("C_4H_8O_5",                                    "", "AWITZWJPGUOCMB-UHFFFAOYSA-N", "MCM:HMACROOH", "")
  CALL Species("C_4H_8O_4",                                    "", "HWQXXVVDRNBYQB-UHFFFAOYSA-N", "MCM:MACROHOOH", "")
  CALL Species("C_4H_8O_4",                                    "", "MOSGWPXPTZVSII-UHFFFAOYSA-N", "MCM:MACROOH", "")
  CALL Species("C_4H_6O_5",                                    "", "MNYHDFHIZDEDLP-UHFFFAOYSA-N", "MCM:MALDIALOOH", "")
  CALL Species("C_5H_{10}O_5",                                 "", "BTJWWKUWEVGVGA-UHFFFAOYSA-N", "MCM:ALC4DOLOOH", "")
  CALL Species("C_5H_8O_5",                                    "", "NJMKFLHNNBFYIH-UHFFFAOYSA-N", "MCM:C4MALOHOOH", "")
  CALL Species("C_5H_{10}O_5",                                 "", "VJGRRPOTERGJKF-UHFFFAOYSA-N", "MCM:C57OOH", "")
  CALL Species("C_5H_{10}O_5",                                 "", "MTCQYLXAFMKVMT-UHFFFAOYSA-N", "MCM:C58OOH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "LTZYUHWRVSINRM-UHFFFAOYSA-N", "MCM:CHOC4OHOOH", "")
  CALL Species("C_5H_{10}O_6",                                 "", "COFCBVJXHDFSHS-UHFFFAOYSA-N", "MCM:HIEB1OOH", "")
  CALL Species("C_5H_{10}O_6",                                 "", "PKIUCWJPHHVKPZ-UHFFFAOYSA-N", "MCM:HIEB2OOH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "LRYOXTYJCQGVIC-UHFFFAOYSA-N", "MCM:HOC4CHOOOH", "")
  CALL Species("C_5H_{10}O_5",                                 "", "MCOKLECUTPXNOW-UHFFFAOYSA-N", "MCM:IEB2OOH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "DOEHHISPDHWTCR-UHFFFAOYSA-N", "MCM:C1H4C5CO3H", "")
  CALL Species("C_6H_{12}O_5",                                 "", "OGBKQGQRAMSXTF-UHFFFAOYSA-N", "MCM:C623OOH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "ONTGLCVTLGTTQS-UHFFFAOYSA-N", "MCM:C67OOH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "ITDKDNQPTMKBIM-UHFFFAOYSA-N", "MCM:C68OOH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "KJWILEAANBSBHN-UHFFFAOYSA-N", "MCM:CO1H63OOH", "")
  CALL Species("C_7H_{14}O_5",                                 "", "WJYXJQPIZCIBJO-UHFFFAOYSA-N", "MCM:C728OOH", "")
  CALL Species("C_7H_{14}O_5",                                 "", "YXKHJQXHKYJCPI-UHFFFAOYSA-N", "MCM:C730OOH", "")
  CALL Species("C_8H_{14}O_4",                                 "", "FGEKWAMEWUMQIL-UHFFFAOYSA-N", "MCM:C824OOH", "")
  CALL Species("C_8H_{14}O_5",                                 "", "MATCTRFTDTXPPO-UHFFFAOYSA-N", "MCM:C826OOH", "")
  CALL Species("C_{12}H_{20}O_4",                              "", "CZSOPCQUOGQYQQ-UHFFFAOYSA-N", "MCM:C127OOH", "")
  CALL Species("C_{13}H_{22}O_4",                              "", "GNGYVQPBVCXXTM-UHFFFAOYSA-N", "MCM:C1311OOH", "")
  CALL Species("C_{13}H_{22}O_5",                              "", "ALNYAIDMJTUDIF-UHFFFAOYSA-N", "MCM:C1313OOH", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Ketones (\chem{RCOR})")
  ! saturated:
  CALL Species("CH_3COCH_3",                            "67-64-1", "CSCPPACGZOOCGX-UHFFFAOYSA-N", "propanone", "acetone")
  CALL Species("CH_3COCH_3",                          "3881-06-9", "CSCPPACGZOOCGX-LBPDFUHNSA-N", "propanone-2-13C", "acetone-2-13C")
  CALL Species("CH_3COCH_2OH",                         "116-09-6", "XLSMFKSTNGKWQX-UHFFFAOYSA-N", "1-hydroxypropanone", "hydroxyacetone")
  CALL Species("C_2H_5COCH_3",                          "78-93-3", "ZWEHNKRNPOVVGH-UHFFFAOYSA-N", "butanone", "methyl ethyl ketone; MEK")
  CALL Species("C_2H_5COCH_3",                       "24313-50-6", "ZWEHNKRNPOVVGH-PDWRLMEDSA-N", "butanone-1,1,1,3,3-d5", "methyl ethyl ketone-d5; MEK-d5")
  CALL Species("C_3H_7COCH_3",                         "107-87-9", "XNLICIUVMPYHGG-UHFFFAOYSA-N", "2-pentanone", "")
  CALL Species("C_2H_5COC_2H_5",                        "96-22-0", "FDPIMTJIUBPUKL-UHFFFAOYSA-N", "3-pentanone", "diethyl ketone")
  CALL Species("C_5H_8O",                              "765-43-5", "HVCFCNAITDHQFX-UHFFFAOYSA-N", "1-cyclopropyl-ethanone", "cyclopropyl methyl ketone")
  CALL Species("C_5H_{10}O",                           "563-80-4", "SYBYTAAJFKOIEJ-UHFFFAOYSA-N", "3-methyl-2-butanone", "isopropyl methyl ketone")
  CALL Species("C_5H_8O",                              "120-92-3", "BGTOWKSIORTVQH-UHFFFAOYSA-N", "cyclopentanone", "")
  CALL Species("C_6H_{12}O",                           "591-78-6", "QQZOPKMRPOGIEB-UHFFFAOYSA-N", "2-hexanone", "")
  CALL Species("C_6H_{12}O",                          "4840-82-8", "QQZOPKMRPOGIEB-ZTIZGVCASA-N", "2-hexanone-1,1,1,3,3-d5", "")
  CALL Species("C_6H_{12}O",                           "589-38-8", "PFCHFHIRKBAQGU-UHFFFAOYSA-N", "3-hexanone", "")
  CALL Species("C_6H_{12}O",                           "565-61-7", "UIHCLUNTQKBZGK-UHFFFAOYSA-N", "3-methyl-2-pentanone", "")
  CALL Species("(CH_3)_2CHCH_2COCH_3",                 "108-10-1", "NTIZESTWPVYFNL-UHFFFAOYSA-N", "4-methyl-2-pentanone", "methyl isobutyl ketone; MIBK")
  CALL Species("C_6H_{12}O",                           "565-69-5", "HYTRYEXINDDXJK-UHFFFAOYSA-N", "2-methyl-3-pentanone", "")
  CALL Species("C_6H_{12}O",                            "75-97-8", "PJGSXYOJTGTZAV-UHFFFAOYSA-N", "3,3-dimethyl-2-butanone", "{\it tert}-butyl methyl ketone")
  CALL Species("C_6H_{10}O",                           "108-94-1", "JHIVVAPYMSGYDF-UHFFFAOYSA-N", "cyclohexanone", "")
  CALL Species("C_7H_{14}O",                           "110-43-0", "CATSNJVOTSVZJV-UHFFFAOYSA-N", "2-heptanone", "")
  CALL Species("C_7H_{14}O",                           "106-35-4", "NGAZZOYFWWSOGK-UHFFFAOYSA-N", "3-heptanone", "")
  CALL Species("C_7H_{14}O",                           "123-19-3", "HCFAJYNVAYBARA-UHFFFAOYSA-N", "4-heptanone", "")
  CALL Species("C_7H_{14}O",                          "2550-21-2", "GYWYASONLSQZBB-UHFFFAOYSA-N", "3-methyl-2-hexanone", "")
  CALL Species("C_7H_{14}O",                           "105-42-0", "XUPXMIAWKPTZLZ-UHFFFAOYSA-N", "4-methyl-2-hexanone", "")
  CALL Species("C_7H_{14}O",                           "110-12-3", "FFWSICBKRCICMR-UHFFFAOYSA-N", "5-methyl-2-hexanone", "")
  CALL Species("C_7H_{14}O",                          "7379-12-6", "HIGGFWFRAWSMBR-UHFFFAOYSA-N", "2-methyl-3-hexanone", "")
  CALL Species("C_7H_{14}O",                         "17042-16-9", "ULPMRIXXHGUZFA-UHFFFAOYSA-N", "4-methyl-3-hexanone", "")
  CALL Species("C_7H_{14}O",                           "623-56-3", "DXVYLFHTJZWTRF-UHFFFAOYSA-N", "5-methyl-3-hexanone", "")
  CALL Species("C_7H_{14}O",                          "6137-03-7", "GSNKRSKIWFBWEG-UHFFFAOYSA-N", "3-ethyl-2-pentanone", "")
  CALL Species("C_7H_{14}O",                         "20669-04-9", "QSHJLBQLQVSEFV-UHFFFAOYSA-N", "3,3-dimethyl-2-pentanone", "")
  CALL Species("C_7H_{14}O",                           "565-78-6", "QXHRQZNDMYRDPA-UHFFFAOYSA-N", "3,4-dimethyl-2-pentanone", "")
  CALL Species("C_7H_{14}O",                           "590-50-1", "AZASWMGVGQEVCS-UHFFFAOYSA-N", "4,4-dimethyl-2-pentanone", "")
  CALL Species("C_7H_{14}O",                           "564-04-5", "VLNUTKMHYLQCQB-UHFFFAOYSA-N", "2,2-dimethyl-3-pentanone", "")
  CALL Species("C_7H_{14}O",                           "565-80-0", "HXVNBWAKAOHACI-UHFFFAOYSA-N", "2,4-dimethyl-3-pentanone", "diisopropyl ketone")
  CALL Species("C_7H_{12}O",                           "502-42-1", "CGZZMOTZOONQIA-UHFFFAOYSA-N", "cycloheptanone", "")
  CALL Species("C_7H_{12}O",                           "583-60-8", "LFSAPCRASZRSKS-UHFFFAOYSA-N", "2-methylcyclohexanone", "")
  CALL Species("C_7H_{12}O",                           "591-24-2", "UJBOOUHRTQVGRU-UHFFFAOYSA-N", "3-methylcyclohexanone", "")
  CALL Species("C_7H_{12}O",                           "589-92-4", "VGVHNLRUAMRIEW-UHFFFAOYSA-N", "4-methylcyclohexanone", "")
  CALL Species("C_7H_{10}O",                          "1121-37-5", "BIPUHAHGLJKIPK-UHFFFAOYSA-N", "dicyclopropylmethanone", "dicyclopropyl ketone")
  CALL Species("C_6H_{13}COCH_3",                      "111-13-7", "ZPVFWPFBNIEHGJ-UHFFFAOYSA-N", "2-octanone", "")
  CALL Species("C_8H_{16}O",                           "106-68-3", "RHLVCLIPMVJYKS-UHFFFAOYSA-N", "3-octanone", "")
  CALL Species("C_8H_{16}O",                           "589-63-9", "YWXLSHOWXZUMSR-UHFFFAOYSA-N", "4-octanone", "")
  CALL Species("C_8H_{16}O",                           "541-85-5", "PSBKJPTZCVYXSD-UHFFFAOYSA-N", "5-methyl-3-heptanone", "")
  CALL Species("C_8H_{16}O",                           "624-42-0", "CCCIYAQYQZQDIZ-UHFFFAOYSA-N", "6-methyl-3-heptanone", "")
  CALL Species("C_6H_{11}COCH_3",                      "823-76-7", "RIFKADJTWUGDOV-UHFFFAOYSA-N", "cyclohexyl methyl ketone", "")
  CALL Species("C_9H_{18}O",                           "821-55-6", "VKCYHJWLYTUGCC-UHFFFAOYSA-N", "2-nonanone", "")
  CALL Species("C_9H_{18}O",                           "925-78-0", "IYTXKIXETAELAV-UHFFFAOYSA-N", "3-nonanone", "")
  CALL Species("C_9H_{18}O",                           "502-56-7", "WSGCRAOTEDLMFQ-UHFFFAOYSA-N", "5-nonanone", "dibutyl ketone")
  CALL Species("C_9H_{18}O",                           "108-83-8", "PTTPXKJBFFKCEK-UHFFFAOYSA-N", "2,6-dimethyl-4-heptanone", "diisobutyl ketone")
  CALL Species("C_9H_{18}O",                           "815-24-7", "UIQGEWJEWJMQSL-UHFFFAOYSA-N", "2,2,4,4-tetramethyl-3-pentanone", "di-({\it tert}-butyl) ketone")
  CALL Species("C_8H_{17}COCH_3",                      "693-54-9", "ZAJNGDIORYACQU-UHFFFAOYSA-N", "2-decanone", "")
  CALL Species("C_9H_{19}COCH_3",                      "112-12-9", "KYWIYKKSMDLRDC-UHFFFAOYSA-N", "2-undecanone", "")
  CALL Species("C_{11}H_{22}O",                        "927-49-1", "ZPQAKYPOZRXKFA-UHFFFAOYSA-N", "6-undecanone", "")
  CALL Species("C_{12}H_{24}O",                       "6175-49-1", "LSKONYYRONEBKA-UHFFFAOYSA-N", "2-dodecanone", "")
  CALL Species("C_{12}H_{24}O",                        "123-18-2", "GFWVDQCGGDBTBS-UHFFFAOYSA-N", "2,6,8-trimethyl-4-nonanone", "")
  CALL Species("C_{13}H_{26}O",                        "593-08-8", "CYIFVRUOHKNECG-UHFFFAOYSA-N", "2-tridecanone", "")
  CALL Species("C_{14}H_{28}O",                       "2345-27-9", "POQLVOYRGNFGRM-UHFFFAOYSA-N", "2-tetradecanone", "")
  CALL Species("C_{15}H_{30}O",                       "2345-28-0", "CJPNOLIZCWDHJK-UHFFFAOYSA-N", "2-pentadecanone", "")
  CALL Species("C_{16}H_{32}O",                      "18787-63-8", "XCXKZBWAKKPFCJ-UHFFFAOYSA-N", "2-hexadecanone", "")
  CALL Species("C_{17}H_{34}O",                       "2922-51-2", "TVTCXPXLRKTHAU-UHFFFAOYSA-N", "2-heptadecanone", "")
  CALL Species("C_{10}H_{18}O",                         "89-80-5", "NFLGAXVYCFJBMK-UHFFFAOYSA-N", "menthone", "")
  CALL Species("C_{10}H_{14}O",                        "700-58-3", "IYKFYARMMIESOX-UHFFFAOYSA-N", "tricyclo[3.3.1.1(3,7)]decanone", "2-adamantanone")
  CALL Species("C_{17}H_{26}O",                      "32388-55-9", "YBUIAJZFOGJGLJ-SWRJLBSHSA-N", "methyl cedryl ketone", "")
  ! unsaturated and aromatic:
  CALL Species("C_4H_6O",                               "78-94-4", "FUSUHKVFWTUUBE-UHFFFAOYSA-N", "3-buten-2-one", "methyl vinyl ketone; MVK")
  CALL Species("C_5H_8O",                              "814-78-8", "ZGHFDIIVVIFNPS-UHFFFAOYSA-N", "methyl isopropenyl ketone", "")
  CALL Species("C_6H_{10}O",                           "141-79-7", "SHOJXDKTYKFBRD-UHFFFAOYSA-N", "4-methyl-3-penten-2-one", "mesityl oxide")
  CALL Species("C_6H_5COCH_3",                          "98-86-2", "KWOLFJPFCHCOCG-UHFFFAOYSA-N", "1-phenylethanone", "acetophenone")
  CALL Species("C_6D_5COCH_3",                       "28077-64-7", "KWOLFJPFCHCOCG-VIQYUKPQSA-N", "1-phenylethanone-d5", "acetophenone-d5")
  CALL Species("C_7H_{14}O_2",                         "107-70-0", "KOKPBCHLPVDQTK-UHFFFAOYSA-N", "4-methoxy-4-methyl-2-pentanone", "")
  CALL Species("C_8H_8O_2",                            "118-93-4", "JECYUBVRTQDVAT-UHFFFAOYSA-N", "2′-hydroxyacetophenone", "")
  CALL Species("C_8H_{14}O",                           "110-93-0", "UHEPJGULSIKKTP-UHFFFAOYSA-N", "6-methyl-5-hepten-2-one", "")
  CALL Species("C_9H_{10}O",                            "93-55-0", "KRIOVPPHQSLHCZ-UHFFFAOYSA-N", "phenyl ethyl ketone", "propiophenone")
  CALL Species("C_9H_{10}O",                           "577-16-2", "YXWWHNCQZBVZPV-UHFFFAOYSA-N", "2′-methylacetophenone", "")
  CALL Species("C_9H_{10}O",                           "585-74-0", "FSPSELPMWGWDRY-UHFFFAOYSA-N", "3′-methylacetophenone", "")
  CALL Species("C_9H_{10}O",                           "122-00-9", "GNKZMNRKLCTJAY-UHFFFAOYSA-N", "4'-methylacetophenone", "")
  CALL Species("C_9H_{10}O_2",                         "579-74-8", "DWPLEOPKBWNPQV-UHFFFAOYSA-N", "2′-methoxyacetophenone", "2-methoxyphenyl methyl ketone")
  CALL Species("C_9H_{10}O_2",                         "586-37-8", "BAYUSCHCCGXLAY-UHFFFAOYSA-N", "3′-methoxyacetophenone", "3-methoxyphenyl methyl ketone")
  CALL Species("C_9H_{10}O_2",                         "100-06-1", "NTPLXRHDUXRPNE-UHFFFAOYSA-N", "4′-methoxyacetophenone", "4-methoxyphenyl methyl ketone")
  CALL Species("C_{10}H_{14}O",                       "6485-40-1", "ULDHMXUKGWMISQ-SECBINFHSA-N", "2-methyl-5-(1-methylethenyl)-2-cyclohexen-1-one", "carvone") ! other CAS: 99-49-0
  CALL Species("C_{13}H_{10}O",                        "119-61-9", "RWCCWEUUXYIKHB-UHFFFAOYSA-N", "benzophenone", "diphenyl ketone")
  CALL Species("C_{13}H_{10}O_3",                      "131-56-6", "ZXDDPOHVAMWLBH-UHFFFAOYSA-N", "2,4-dihydroxybenzophenone", "")
  CALL Species("C_9H_{14}O",                            "78-59-1", "HJOVHMDZYOCNQW-UHFFFAOYSA-N", "3,5,5-trimethyl-2-cyclohexen-1-one", "isophorone")
  CALL Species("C_7H_{10}O",                           "497-38-1", "KPMKEVXVVHNIEY-UHFFFAOYSA-N", "bicyclo[2.2.1]heptan-2-one", "norcamphor; 2-norbornanone")
  CALL Species("C_{10}H_{16}O",                       "1125-12-8", "USMNOWBWPHYOEA-UHFFFAOYSA-N", "4-methyl-1-(1-methylethyl)-bicyclo[3.1.0]hexan-3-one", "thujone")
  CALL Species("C_{10}H_{12}O",                        "611-70-1", "BSMGLVDZZMBWQB-UHFFFAOYSA-N", "isopropyl phenyl ketone", "")
  CALL Species("C_{10}H_{14}O",                         "99-49-0", "ULDHMXUKGWMISQ-UHFFFAOYSA-N", "carvone", "")
  CALL Species("C_{10}H_{16}O",                      "76231-76-0", "USMNOWBWPHYOEA-VWHDNNRLSA-N", "thujone", "")
  CALL Species("C_{11}H_{16}O",                        "488-10-8", "XMLSXPIVAXONDL-PLNGDYQASA-N", "{\it cis}-jasmone", "")
  CALL Species("C_{13}H_{20}O",                      "24720-09-0", "CRIGTVCBMUKRSL-FNORWQNLSA-N", "$\alpha$-damascone", "")
  CALL Species("C_{13}H_{18}O",                      "23726-93-4", "POIARNZEYGURDG-FNORWQNLSA-N", "($E$)-$\beta$-damascenone", "")
  CALL Species("C_{13}H_8O",                           "486-25-9", "YLQWCDOCJODRMT-UHFFFAOYSA-N", "9H-fluoren-9-one", "")
  CALL Species("C_{14}H_{10}O",                         "90-44-8", "RJGDLRCDCYRQOQ-UHFFFAOYSA-N", "anthrone", "")
  CALL Species("C_{14}H_{22}O",                      "33704-61-9", "MIZGSAALSYARKU-UHFFFAOYSA-N", "1,2,3,5,6,7-hexahydro-1,1,2,3,3-pentamethyl-4H-inden-4-one", "")
  CALL Species("C_{14}H_{12}O_2",                      "119-53-9", "ISAOCJYIOMOJEB-UHFFFAOYSA-N", "benzoin", "")
  CALL Species("C_{14}H_{22}O",                      "93302-56-8", "VPKMGDRERYMTJX-XEHSLEBBSA-N", "methyl-alpha-ionone", "")
  CALL Species("C_{16}H_{26}O",                      "54464-57-2", "FVUGZKDGWGKCFE-UHFFFAOYSA-N", "iso-e super", "")
  CALL Species("C_{16}H_{16}O",                        "954-16-5", "HPAFOABSQZMTHE-UHFFFAOYSA-N", "2,4,6-trimethylbenzophenone", "")
  CALL Species("C_{16}H_{26}O",                      "68155-66-8", "NOMWSTMYQKABST-UHFFFAOYSA-N", "1-(1,2,3,5,6,7,8,8a-octahydro-2,3,8,8-tetramethyl-2-naphthyl)ethan-1-one", "")
  CALL Species("C_{17}H_{30}O",                        "542-46-1", "ZKVZSBSZTMPBQR-UPHRSURJSA-N", "civetone", "")
  CALL Species("C_{17}H_{24}O",                      "15323-35-0", "VDBHOHJWUDKDRW-UHFFFAOYSA-N", "1-(2,3-dihydro-1,1,2,3,3,6-hexamethyl-1H-inden-5-yl)ethanone", "")
  CALL Species("C_{17}H_{24}O",                      "13171-00-1", "IKTHMQYJOWTSJO-UHFFFAOYSA-N", "celestolide", "")
  CALL Species("C_{17}H_{10}O",                         "82-05-3", "HUKPVYBUJRAUAG-UHFFFAOYSA-N", "7H-benz[$de$]anthracen-7-one", "benzanthrone")
  CALL Species("C_{17}H_{10}O",                        "479-79-8", "RNICURKFVSAHLQ-UHFFFAOYSA-N", "11H-benzo[$a$]fluoren-11-one", "") ! other CAS: 116232-62-3, 76723-60-9
  CALL Species("C_{17}H_{10}O",                       "3074-03-1", "MLMNDNOSVOKYMT-UHFFFAOYSA-N", "11H-benzo[$b$]fluoren-11-one", "")
  CALL Species("C_{18}H_{10}O_2",                     "2498-66-0", "LHMRXAIRPKSGDE-UHFFFAOYSA-N", "benz[$a$]anthracene-7,12-dione", "")
  CALL Species("C_{18}H_{26}O",                      "68140-48-7", "IMRYETFJNLKUHK-UHFFFAOYSA-N", "1-[2,3-dihydro-1,1,2,6-tetramethyl-3-(1-methylethyl)-1H-inden-5-yl]ethanone", "")
  CALL Species("C_{18}H_{26}O",                      "21145-77-7", "DNRJTBAOUJJKDY-UHFFFAOYSA-N", "tonalide", "") ! other CAS: 1506-02-1
  ! >= 2 keto groups:
  CALL Species("CH_3COCOCH_3",                         "431-03-8", "QSJXEFYPDANLFS-UHFFFAOYSA-N", "2,3-butanedione", "diacetyl; dimethylglyoxal")
  CALL Species("C_5H_8O_2",                            "600-14-6", "TZMFJUDUGYTVRY-UHFFFAOYSA-N", "2,3-pentanedione", "")
  CALL Species("C_5H_8O_2",                            "123-54-6", "YRKCREAYFQTBPV-UHFFFAOYSA-N", "2,4-pentanedione", "acetylacetone")
  CALL Species("C_{10}H_6O_2",                         "524-42-5", "KETQAJRQOHHATG-UHFFFAOYSA-N", "1,2-naphthalenedione", "")
  CALL Species("C_{10}H_6O_2",                         "130-15-4", "FRASJONUBLZVQX-UHFFFAOYSA-N", "1,4-naphthalenedione", "1,4-naphthoquinone")
  CALL Species("C_{11}H_8O_2",                          "58-27-5", "MJVAVZPDRWSRRC-UHFFFAOYSA-N", "menadione", "")
  CALL Species("C_{14}H_{20}O_2",                      "719-22-2", "RDQSIADLBQFVMY-UHFFFAOYSA-N", "2,6-di-{\it tert}-butyl-$p$-benzoquinone", "")
  CALL Species("C_{14}H_8O_2",                          "84-11-7", "YYVYAPXYZVYDHN-UHFFFAOYSA-N", "9,10-phenanthrenedione", "")
  CALL Species("C_{14}H_8O_2",                          "84-65-1", "RZVHIXYEVGDQDX-UHFFFAOYSA-N", "9,10-anthracenedione", "")
  CALL Species("C_{14}H_8O_3",                         "605-32-3", "GCDBEYOJCZLKMC-UHFFFAOYSA-N", "2-hydroxy-9,10-anthracenedione", "")
  CALL Species("C_{14}H_8O_4",                          "81-64-1", "GUEIZVNYDFNHJU-UHFFFAOYSA-N", "1,4-dihydroxy-9,10-anthracenedione", "")
  CALL Species("C_{14}H_{10}O_2",                      "134-81-6", "WURBFLDFSFBTLW-UHFFFAOYSA-N", "benzil", "")
  CALL Species("C_{15}H_{12}O_2",                      "120-46-7", "NZZIMKJIVMHWJC-UHFFFAOYSA-N", "dibenzoylmethane", "")
  CALL Species("C_{15}H_{10}O_2",                       "84-54-8", "NJWGQARXZDRHCD-UHFFFAOYSA-N", "2-methyl-9,10-anthracenedione", "")
  CALL Species("C_{16}H_{12}O_2",                       "84-51-5", "SJEBAWHUJDUKQK-UHFFFAOYSA-N", "2-ethyl-9,10-anthracenedione", "")
  CALL Species("C_{23}H_{16}O_3",                       "82-66-6", "JYGLAHSAISAEAL-UHFFFAOYSA-N", "diphenadione", "")
    ! ketones from MCM:
  CALL Species("C_3H_4O_4",                                    "", "BXASKOSTAOGNPV-UHFFFAOYSA-N", "MCM:CH3COCO3H", "")
  CALL Species("C_3H_6O_3",                                    "", "LSIGHSKIPNHVEN-UHFFFAOYSA-N", "MCM:HYPERACET", "")
  CALL Species("C_4H_6O_4",                                    "", "JYWVDPVJSGSFMT-UHFFFAOYSA-N", "MCM:BIACETOOH", "")
  CALL Species("C_4H_6O_4",                                    "", "SJOPHMGAZWPERQ-UHFFFAOYSA-N", "MCM:CO2C3CO3H", "")
  CALL Species("C_4H_8O_3",                                    "", "VEPDQRPXKSSVPR-UHFFFAOYSA-N", "MCM:MEKAOOH", "")
  CALL Species("C_4H_8O_3",                                    "", "ISENHDUDKGYVAU-UHFFFAOYSA-N", "MCM:MEKBOOH", "")
  CALL Species("C_4H_8O_3",                                    "", "PSPWSCBXMKHAFW-UHFFFAOYSA-N", "MCM:MEKCOOH", "")
  CALL Species("C_4H_6O_3",                                    "", "PRFKYXWFQJOCGY-UHFFFAOYSA-N", "MCM:MVKOOH", "")
  CALL Species("C_5H_8O_4",                                    "", "FAYQBJFLFGEPOW-UHFFFAOYSA-N", "MCM:C41CO3H", "")
  CALL Species("C_5H_6O_4",                                    "", "FKGAAIITVQJZMS-UHFFFAOYSA-N", "MCM:C5CO14OOH", "")
  CALL Species("C_5H_6O_3",                                    "", "MVDYEFQVZNBPPH-UHFFFAOYSA-N", "MCM:C5CO234", "")
  CALL Species("C_5H_6O_5",                                    "", "XNGUZQHZGCMOHY-UHFFFAOYSA-N", "MCM:C5CO234OOH", "")
  CALL Species("C_5H_8O_4",                                    "", "BRUGMCFWMGXRAP-UHFFFAOYSA-N", "MCM:C5CO23OOH", "")
  CALL Species("C_5H_6O_5",                                    "", "BRGQKBIYWSYIOL-UHFFFAOYSA-N", "MCM:CO23C4CO3H", "")
  CALL Species("C_5H_8O_4",                                    "", "QASRSSCEZIIDDC-UHFFFAOYSA-N", "MCM:CO23C54OOH", "")
  CALL Species("C_5H_8O_4",                                    "", "AULGAHCEUUOLTF-UHFFFAOYSA-N", "MCM:CO24C53OOH", "")
  CALL Species("C_5H_8O_4",                                    "", "COVHHGSUFOHLBW-UHFFFAOYSA-N", "MCM:CO2C4CO3H", "")
  CALL Species("C_5H_8O_4",                                    "", "HMOQHHYQXPCOJD-UHFFFAOYSA-N", "MCM:CO3C4CO3H", "")
  CALL Species("C_5H_{10}O_3",                                 "", "GVIFMISHBUBLQF-UHFFFAOYSA-N", "MCM:DIEKAOOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "PNIUWVGTVWVIRO-UHFFFAOYSA-N", "MCM:DIEKBOOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "UWVYZAHMDOAIIN-UHFFFAOYSA-N", "MCM:MIPKAOOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "OWEKNVKRJSTQCB-UHFFFAOYSA-N", "MCM:MIPKBOOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "RFXMBEJILJIJHO-UHFFFAOYSA-N", "MCM:MPRKAOOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "BOKMIGXVNYNOOE-UHFFFAOYSA-N", "MCM:MPRKBOOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "NFPIZNOAHNBIPJ-UHFFFAOYSA-N", "MCM:PE2ONE1OOH", "")
  CALL Species("C_6H_8O_5",                                    "", "FOQTZZGGVXWRDJ-UHFFFAOYSA-N", "MCM:C23C54CO3H", "")
  CALL Species("C_6H_{10}O_4",                                 "", "NQEGAXBMSWAEEJ-UHFFFAOYSA-N", "MCM:C3COCCO3H", "")
  CALL Species("C_6H_8O_2",                                    "", "OTSKZNVDZOOHRX-UHFFFAOYSA-N", "MCM:C4DBDIKET", "")
  CALL Species("C_6H_{10}O_4",                                 "", "JNOOBNCRVAKFDO-UHFFFAOYSA-N", "MCM:C51CO3H", "")
  CALL Species("C_6H_8O_4",                                    "", "DSQMAYIDJPXZOY-UHFFFAOYSA-N", "MCM:C5CODBCO3H", "")
  CALL Species("C_6H_8O_4",                                    "", "NVTHKHSWEWVMOZ-UHFFFAOYSA-N", "MCM:C5DBCOCO3H", "")
  CALL Species("C_6H_{10}O_4",                                 "", "YTLUYDMGHZZGIK-UHFFFAOYSA-N", "MCM:C611OOH", "")
  CALL Species("C_6H_8O_4",                                    "", "CLPCTPYJHZPKKC-UHFFFAOYSA-N", "MCM:C619OOH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "BKNNHKNGGIDPKB-UHFFFAOYSA-N", "MCM:C627OOH", "")
  CALL Species("C_6H_8O_5",                                    "", "NRJLAAIGKSDANL-UHFFFAOYSA-N", "MCM:C62OOH", "")
  CALL Species("C_6H_8O_5",                                    "", "KXHHBNOARBHHNQ-UHFFFAOYSA-N", "MCM:C6CO134OOH", "")
  CALL Species("C_6H_{10}O_2",                                 "", "KVFQMAZOBTXCAZ-UHFFFAOYSA-N", "MCM:C6CO34", "")
  CALL Species("C_6H_{10}O_4",                                 "", "GOPKJUXJYMJAQC-UHFFFAOYSA-N", "MCM:C6CO34OOH", "")
  CALL Species("C_6H_6O_5",                                    "", "JAWAIWJCDBIXQU-UHFFFAOYSA-N", "MCM:C6CYTONOOH", "")
  CALL Species("C_6H_8O_4",                                    "", "JRENYXREJDINFO-UHFFFAOYSA-N", "MCM:C6DCRBBOOH", "")
  CALL Species("C_6H_8O_3",                                    "", "FYLKBSXPBXZFJV-UHFFFAOYSA-N", "MCM:CO234C6", "")
  CALL Species("C_6H_8O_5",                                    "", "RXZSKHWIVPRCNE-UHFFFAOYSA-N", "MCM:CO234C6OOH", "")
  CALL Species("C_6H_8O_3",                                    "", "POASQEFMMPWSLD-UHFFFAOYSA-N", "MCM:CO235C6", "")
  CALL Species("C_6H_8O_5",                                    "", "OFEOQWJKJVUBKA-UHFFFAOYSA-N", "MCM:CO235C6OOH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "QKJDOFMXAZMTSN-UHFFFAOYSA-N", "MCM:CO23C65OOH", "")
  CALL Species("C_6H_{10}O_2",                                 "", "MWVFCEVNXHTDNF-UHFFFAOYSA-N", "MCM:CO23C6", "")
  CALL Species("C_6H_{10}O_4",                                 "", "BCTNDVKHSPOTQH-UHFFFAOYSA-N", "MCM:CO24C63OOH", "")
  CALL Species("C_6H_{10}O_2",                                 "", "NDOGLIPWGGRQCO-UHFFFAOYSA-N", "MCM:CO24C6", "")
  CALL Species("C_6H_{10}O_4",                                 "", "SLWLMHWJPVVUSV-UHFFFAOYSA-N", "MCM:CO24C6OOH", "")
  CALL Species("C_6H_{10}O_2",                                 "", "GSOHKPVFCOWKPU-UHFFFAOYSA-N", "MCM:CO24M3C5", "")
  CALL Species("C_6H_{10}O_2",                                 "", "OJVAMHKKJGICOG-UHFFFAOYSA-N", "MCM:CO25C6", "")
  CALL Species("C_6H_{10}O_4",                                 "", "JXALTZPWVLRWQW-UHFFFAOYSA-N", "MCM:CO25C6OOH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "CNZIZHTUCKWXKF-UHFFFAOYSA-N", "MCM:CO2C54CO3H", "")
  CALL Species("C_6H_{10}O_4",                                 "", "NINAOMADVJMMOS-UHFFFAOYSA-N", "MCM:CO2M33CO3H", "")
  CALL Species("C_6H_{10}O_4",                                 "", "VXADHVSNPNTBLA-UHFFFAOYSA-N", "MCM:CO3C54CO3H", "")
  CALL Species("C_6H_{10}O_4",                                 "", "NEAYDZAAQZUWAX-UHFFFAOYSA-N", "MCM:CO3C5CO3H", "")
  CALL Species("C_6H_8O_4",                                    "", "PCNCBCRTMCKUHD-UHFFFAOYSA-N", "MCM:CY6COCOOOH", "")
  CALL Species("C_6H_8O_4",                                    "", "WZMMYHNHRXLTAA-UHFFFAOYSA-N", "MCM:CY6DIONOOH", "")
  CALL Species("C_6H_8O_2",                                    "", "HJSLFCCWAKVHIW-UHFFFAOYSA-N", "MCM:CYC613DION", "")
  CALL Species("C_6H_8O_2",                                    "", "OILAIQUEIWYQPH-UHFFFAOYSA-N", "MCM:CYC6DIONE", "")
  CALL Species("C_6H_{10}O_3",                                 "", "YLRYKXGGQIKRPE-UHFFFAOYSA-N", "MCM:CYHXONAOOH", "")
  CALL Species("C_6H_6O_6",                                    "", "LOIDTPBEOYVNRI-UHFFFAOYSA-N", "MCM:ECO3CO3H", "")
  CALL Species("C_6H_{12}O_3",                                 "", "UUAGRKYHKVSONO-UHFFFAOYSA-N", "MCM:EIPKAOOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "GSVDUVPILFIVQC-UHFFFAOYSA-N", "MCM:EIPKBOOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "BPODHSIMBCVMGL-UHFFFAOYSA-N", "MCM:HEX2ONAOOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "XXENGSKOCXRCDY-UHFFFAOYSA-N", "MCM:HEX2ONBOOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "UIIPZQPVNWMUDI-UHFFFAOYSA-N", "MCM:HEX2ONCOOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "ZKBVMKAKXKAOJC-UHFFFAOYSA-N", "MCM:HEX3ONAOOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "CFZVDJFTDOASSJ-UHFFFAOYSA-N", "MCM:HEX3ONBOOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "RCWXBCITQMXQRJ-UHFFFAOYSA-N", "MCM:HEX3ONCOOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "GTTLVNDFVPDHBU-UHFFFAOYSA-N", "MCM:HEX3ONDOOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "UYZBTKCJJDJNOE-UHFFFAOYSA-N", "MCM:M2BKAOOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "HXTZBJMRLHBTDT-UHFFFAOYSA-N", "MCM:M2BKBOOH", "")
  CALL Species("C_6H_{10}O_2",                                 "", "JENYBWHRLYZSSZ-UHFFFAOYSA-N", "MCM:MIBK3CO", "")
  CALL Species("C_6H_{10}O_4",                                 "", "REESTLWXXTVHDK-UHFFFAOYSA-N", "MCM:MIBK3COOOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "TVLYPTZVJFAYSU-UHFFFAOYSA-N", "MCM:MIBKAOOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "NTAUGUQNNYBRPR-UHFFFAOYSA-N", "MCM:MIBKBOOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "FSDVGLADHCPPED-UHFFFAOYSA-N", "MCM:MTBKOOH", "")
  CALL Species("C_7H_8O_6",                                    "", "MCULIRAVDBAPRO-UHFFFAOYSA-N", "MCM:C235C6CO3H", "")
  CALL Species("C_7H_{14}O_3",                                 "", "WMYMHSKADBIKCY-UHFFFAOYSA-N", "MCM:C710OOH", "")
  CALL Species("C_7H_{12}O_4",                                 "", "BVAIBJLRLSWCNT-UHFFFAOYSA-N", "MCM:C713OOH", "")
  CALL Species("C_7H_{12}O_4",                                 "", "FQLJODGQMVISEK-UHFFFAOYSA-N", "MCM:C714OOH", "")
  CALL Species("C_7H_{10}O_5",                                 "", "IFAJGMRYFBSYLP-UHFFFAOYSA-N", "MCM:C715OOH", "")
  CALL Species("C_7H_8O_3",                                    "", "ZXGNNLLPQXKXPQ-UHFFFAOYSA-N", "MCM:C7236CO", "")
  CALL Species("C_7H_{10}O_4",                                 "", "DRGOPDAWFSWANX-UHFFFAOYSA-N", "MCM:C726CO5OOH", "")
  CALL Species("C_7H_{10}O_3",                                 "", "XDJAUXXWCBLXGT-UHFFFAOYSA-N", "MCM:C727CO", "")
  CALL Species("C_7H_{12}O_4",                                 "", "IMCLYNFQESJQRO-UHFFFAOYSA-N", "MCM:C727OOH", "")
  CALL Species("C_7H_{10}O_5",                                 "", "QFXLTANEYKLVBH-UHFFFAOYSA-N", "MCM:C73OOH", "")
  CALL Species("C_7H_{10}O_5",                                 "", "ZLQFKUKGFNRBMM-UHFFFAOYSA-N", "MCM:C74OOH", "")
  CALL Species("C_7H_{14}O_3",                                 "", "YNKFHYFADYHYPZ-UHFFFAOYSA-N", "MCM:C75OOH", "")
  CALL Species("C_7H_{10}O_4",                                 "", "KFHCVBUEDOHUEG-UHFFFAOYSA-N", "MCM:C7ADCCO3H", "")
  CALL Species("C_7H_{10}O_2",                                 "", "SGWFXGZCQBUONH-UHFFFAOYSA-N", "MCM:C7BDICARB", "")
  CALL Species("C_7H_{10}O_4",                                 "", "VBFDFNFUMYAFQG-UHFFFAOYSA-N", "MCM:C7DCCO3H", "")
  CALL Species("C_7H_{10}O_4",                                 "", "ITRURPAVBUVUCD-UHFFFAOYSA-N", "MCM:C7DDCCO3H", "")
  CALL Species("C_7H_{10}O_2",                                 "", "YTVSAHBJOWKZHT-UHFFFAOYSA-N", "MCM:C7EDICARB", "")
  CALL Species("C_7H_{10}O_5",                                 "", "VNKKTGYQOPWEKF-UHFFFAOYSA-N", "MCM:C821OOH", "")
  CALL Species("C_7H_{10}O_3",                                 "", "PLNIROICNOROEN-UHFFFAOYSA-N", "MCM:CO235C7", "")
  CALL Species("C_7H_{10}O_3",                                 "", "SOSHREJUCDQQIR-UHFFFAOYSA-N", "MCM:CO245C7", "")
  CALL Species("C_7H_{10}O_5",                                 "", "NZAOVWOYFADSGM-UHFFFAOYSA-N", "MCM:CO25C6CO3H", "")
  CALL Species("C_7H_{12}O_4",                                 "", "NWTMJMDPQCUZCE-UHFFFAOYSA-N", "MCM:CO25C73OOH", "")
  CALL Species("C_7H_{12}O_4",                                 "", "VDRXHYKLZSMLBZ-UHFFFAOYSA-N", "MCM:CO25C74OOH", "")
  CALL Species("C_7H_{12}O_2",                                 "", "HGRGPAAXHOTBAM-UHFFFAOYSA-N", "MCM:CO25C7", "")
  CALL Species("C_7H_{14}O_3",                                 "", "TVVPPVSOZMPRIM-UHFFFAOYSA-N", "MCM:CO3C75OOH", "")
  CALL Species("C_7H_{10}O_4",                                 "", "JFPQYVJBAXIOAT-UHFFFAOYSA-N", "MCM:IC7DCCO3H", "")
  CALL Species("C_7H_{10}O_3",                                 "", "FMABCBYNHKBBEH-UHFFFAOYSA-N", "MCM:M3CO245C6", "")
  CALL Species("C_7H_{12}O_2",                                 "", "NLLILAUVOOREKR-UHFFFAOYSA-N", "MCM:M3CO25C6", "")
  CALL Species("C_8H_{10}O_5",                                 "", "QHSWUEYXPZRIAH-UHFFFAOYSA-N", "MCM:C5DBECO3H", "")
  CALL Species("C_8H_{10}O_5",                                 "", "YXJCVNQUVAHMKT-UHFFFAOYSA-N", "MCM:C5EDBCO3H", "")
  CALL Species("C_8H_{12}O_5",                                 "", "FJTLPXSURBYIDW-UHFFFAOYSA-N", "MCM:C727CO3H", "")
  CALL Species("C_8H_{12}O_4",                                 "", "GOMWBXPFFZKXIF-UHFFFAOYSA-N", "MCM:C7CO2M5OOH", "")
  CALL Species("C_8H_{12}O_4",                                 "", "BVDWDBGMQXZSJX-UHFFFAOYSA-N", "MCM:C7M2CO5OOH", "")
  CALL Species("C_8H_{10}O_3",                                 "", "WFGYJTNUNWJHJN-UHFFFAOYSA-N", "MCM:C7M3CO", "")
  CALL Species("C_8H_{12}O_4",                                 "", "NMESNSUDSMNPLZ-UHFFFAOYSA-N", "MCM:C7ODLBCO3H", "")
  CALL Species("C_8H_{12}O_2",                                 "", "FXXXYZSMIIVJDG-UHFFFAOYSA-N", "MCM:C816CO", "")
  CALL Species("C_8H_{14}O_3",                                 "", "GWYFGLCLLKWWIQ-UHFFFAOYSA-N", "MCM:C816OOH", "")
  CALL Species("C_8H_{14}O_4",                                 "", "WQMXPSRLZZCZTO-UHFFFAOYSA-N", "MCM:C817OOH", "")
  CALL Species("C_8H_{10}O_3",                                 "", "IOHUPZZTKNTURH-UHFFFAOYSA-N", "MCM:C8236CO", "")
  CALL Species("C_8H_{12}O_4",                                 "", "JGDMXPACXAYITG-UHFFFAOYSA-N", "MCM:C826CO3OOH", "")
  CALL Species("C_8H_{14}O_4",                                 "", "MYZHYUNBFYQPIX-UHFFFAOYSA-N", "MCM:C827OOH", "")
  CALL Species("C_8H_{12}O_5",                                 "", "IPIPKWBUGMDLTO-UHFFFAOYSA-N", "MCM:C828OOH", "")
  CALL Species("C_8H_{12}O_5",                                 "", "MTDHYROWVQXAOV-UHFFFAOYSA-N", "MCM:C83OOH", "")
  CALL Species("C_8H_{14}O_4",                                 "", "HQUQRVBRCVTTSE-UHFFFAOYSA-N", "MCM:C84OOH", "")
  CALL Species("C_8H_{14}O_3",                                 "", "MGHQHHXZPXRHRQ-UHFFFAOYSA-N", "MCM:C85OOH", "")
  CALL Species("C_8H_{10}O_3",                                 "", "KYJUTSWXKQCOOZ-UHFFFAOYSA-N", "MCM:C88CO", "")
  CALL Species("C_8H_{12}O_4",                                 "", "LSPOECQNSUKNPK-UHFFFAOYSA-N", "MCM:C88OOH", "")
  CALL Species("C_8H_{12}O",                                   "", "BYOKRVKHSHTAOM-UHFFFAOYSA-N", "MCM:C8BCCO", "")
  CALL Species("C_8H_{12}O_3",                                 "", "YUHNMHTZZXLCQZ-UHFFFAOYSA-N", "MCM:CO346C8", "")
  CALL Species("C_8H_{14}O_2",                                 "", "CVZGUJMLZZTPKH-UHFFFAOYSA-N", "MCM:CO36C8", "")
  CALL Species("C_8H_{16}O_3",                                 "", "RPACFUNWVVDXHA-UHFFFAOYSA-N", "MCM:CO3C85OOH", "")
  CALL Species("C_8H_8O_2",                                    "", "SENUUPBBLQWHMF-UHFFFAOYSA-N", "MCM:MXYQONE", "")
  CALL Species("C_8H_8O_2",                                    "", "AIACLXROWHONEE-UHFFFAOYSA-N", "MCM:OXYQONE", "")
  CALL Species("C_8H_8O_2",                                    "", "IGRSQEOIAAGSGS-UHFFFAOYSA-N", "MCM:PEBQONE", "")
  CALL Species("C_8H_8O_2",                                    "", "MYKLQMNSFPAPLZ-UHFFFAOYSA-N", "MCM:PXYQONE", "")
  CALL Species("C_9H_{14}O_4",                                 "", "IXOBJDNIWKDFSA-UHFFFAOYSA-N", "MCM:C816CO3H", "")
  CALL Species("C_9H_{14}O_5",                                 "", "NVOUBBWYYUIRJW-UHFFFAOYSA-N", "MCM:C817CO3H", "")
  CALL Species("C_9H_{14}O_5",                                 "", "DZJUGPICXPPSEN-UHFFFAOYSA-N", "MCM:C827CO3H", "")
  CALL Species("C_9H_{12}O_6",                                 "", "XCCOWFUPSKLDCA-UHFFFAOYSA-N", "MCM:C828CO3H", "")
  CALL Species("C_9H_{14}O_4",                                 "", "DAYKQISVSRMABH-UHFFFAOYSA-N", "MCM:C85CO3H", "")
  CALL Species("C_9H_{12}O_5",                                 "", "KOEDWBONHQRVNE-UHFFFAOYSA-N", "MCM:C88CO3H", "")
  CALL Species("C_9H_{14}O_4",                                 "", "LMHVWEUXUKAUEE-UHFFFAOYSA-N", "MCM:C8M2CO6OOH", "")
  CALL Species("C_9H_{12}O_3",                                 "", "VNKKGGMTAOGJCS-UHFFFAOYSA-N", "MCM:C8M3CO", "")
  CALL Species("C_9H_{14}O_4",                                 "", "USPARXGTYFQAAO-UHFFFAOYSA-N", "MCM:C917OOH", "")
  CALL Species("C_9H_{18}O_3",                                 "", "LTHHCRGMHVQJHW-UHFFFAOYSA-N", "MCM:C91OOH", "")
  CALL Species("C_9H_{16}O_3",                                 "", "ZSCOJLRNNZZYOH-UHFFFAOYSA-N", "MCM:C923OOH", "")
  CALL Species("C_9H_{16}O_4",                                 "", "NVMDAOHKHIQTQS-UHFFFAOYSA-N", "MCM:C928OOH", "")
  CALL Species("C_9H_{16}O_4",                                 "", "ZEMHBLLRPWJDGD-UHFFFAOYSA-N", "MCM:C94OOH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "OKPNQXXXVOPUSS-UHFFFAOYSA-N", "MCM:C95OOH", "")
  CALL Species("C_9H_{16}O_3",                                 "", "UOSOYGOFRNBVGT-UHFFFAOYSA-N", "MCM:C96OOH", "")
  CALL Species("C_9H_{12}O_2",                                 "", "MRLMBLCAGJIZHM-UHFFFAOYSA-N", "MCM:C9DC", "")
  CALL Species("C_9H_{10}O_3",                                 "", "CNGBBXSWKSNDLL-UHFFFAOYSA-N", "MCM:C9DCCO", "")
  CALL Species("C_9H_{12}O_4",                                 "", "LVYPIBNFNZWAIN-UHFFFAOYSA-N", "MCM:C9DCOOH", "")
  CALL Species("C_9H_{14}O_3",                                 "", "UAWDJSGDSYOWDW-UHFFFAOYSA-N", "MCM:CO356C9", "")
  CALL Species("C_9H_{16}O_2",                                 "", "NTESOQURXNMSRF-UHFFFAOYSA-N", "MCM:CO36C9", "")
  CALL Species("C_9H_{10}O_2",                                 "", "XLTSBDOZTUSCMX-UHFFFAOYSA-N", "MCM:IPRBQONE", "")
  CALL Species("C_9H_{14}O",                                   "", "HOBBEYSRFFJETF-UHFFFAOYSA-N", "MCM:LIMKET", "")
  CALL Species("C_9H_{10}O_2",                                 "", "FBVFNJVAQSWLOP-UHFFFAOYSA-N", "MCM:METLQONE", "")
  CALL Species("C_9H_{14}O_3",                                 "", "AGOGLNHOAIUZEO-UHFFFAOYSA-N", "MCM:NOPINAOOH", "")
  CALL Species("C_9H_{12}O_2",                                 "", "AQDJLLQBBRXMBZ-UHFFFAOYSA-N", "MCM:NOPINBCO", "")
  CALL Species("C_9H_{14}O_3",                                 "", "ICIAOEDGJQUKIV-UHFFFAOYSA-N", "MCM:NOPINBOOH", "")
  CALL Species("C_9H_{14}O_3",                                 "", "YDBPVAMAGOPNKC-UHFFFAOYSA-N", "MCM:NOPINCOOH", "")
  CALL Species("C_9H_{12}O_2",                                 "", "QEZVNLZLSDOZRT-UHFFFAOYSA-N", "MCM:NOPINDCO", "")
  CALL Species("C_9H_{14}O_3",                                 "", "MUTHAEIFCDWSQQ-UHFFFAOYSA-N", "MCM:NOPINDOOH", "")
  CALL Species("C_9H_{14}O",                                   "", "XZFDKWMYCUEKSS-UHFFFAOYSA-N", "MCM:NOPINONE", "")
  CALL Species("C_9H_{10}O_2",                                 "", "CMBANAGVBLPIPT-UHFFFAOYSA-N", "MCM:OETLQONE", "")
  CALL Species("C_9H_{10}O_2",                                 "", "JUIQOKRNPGGIPV-UHFFFAOYSA-N", "MCM:PETLQONE", "")
  CALL Species("C_9H_{10}O_2",                                 "", "NBHAZVWKRHTWRW-UHFFFAOYSA-N", "MCM:PPRBQONE", "")
  CALL Species("C_9H_{10}O_2",                                 "", "QIXDHVDGPXBRRD-UHFFFAOYSA-N", "MCM:TM124QONE", "")
  CALL Species("C_{10}H_{16}O_2",                              "", "HCTKOTXYXFZDNL-UHFFFAOYSA-N", "MCM:C1011CO", "")
  CALL Species("C_{10}H_{18}O_3",                              "", "SNMQVMGNQAVOQP-UHFFFAOYSA-N", "MCM:C1011OOH", "")
  CALL Species("C_{10}H_{20}O_3",                              "", "SFSDJTUNVGVIHA-UHFFFAOYSA-N", "MCM:C101OOH", "")
  CALL Species("C_{10}H_{18}O_4",                              "", "NAKDXHBWPFMZPE-UHFFFAOYSA-N", "MCM:C104OOH", "")
  CALL Species("C_{10}H_{16}O_5",                              "", "SMUSKFONQIUMCM-UHFFFAOYSA-N", "MCM:C105OOH", "")
  CALL Species("C_{10}H_{16}O_4",                              "", "JSHGCXGTVADTAK-UHFFFAOYSA-N", "MCM:C923CO3H", "")
  CALL Species("C_{10}H_{16}O_5",                              "", "LCMHUZKVXFPENZ-UHFFFAOYSA-N", "MCM:C928CO3H", "")
  CALL Species("C_{10}H_{16}O_4",                              "", "ACPAJMUJFSDSAX-UHFFFAOYSA-N", "MCM:C9M2CO6OOH", "")
  CALL Species("C_{10}H_{14}O_3",                              "", "DYAPBQRZCBYWEA-UHFFFAOYSA-N", "MCM:C9M3CO", "")
  CALL Species("C_{10}H_{16}O_3",                              "", "OKXFSYMNZFSQOL-UHFFFAOYSA-N", "MCM:CO356C10", "")
  CALL Species("C_{10}H_{18}O_2",                              "", "WYDYPTCJLWEFAY-UHFFFAOYSA-N", "MCM:CO36C10", "")
  CALL Species("C_{10}H_{20}O",                                "", "XJLDYKIEURAVBW-UHFFFAOYSA-N", "MCM:DEC3ONE", "")
  CALL Species("C_{10}H_{16}O_4",                              "", "VFMYUIAYOIIENK-UHFFFAOYSA-N", "MCM:PERPINONIC", "")
  CALL Species("C_{11}H_{18}O_4",                              "", "MDKPIQZZHUIYJQ-UHFFFAOYSA-N", "MCM:C1011CO3H", "")
  CALL Species("C_{11}H_{22}O_3",                              "", "AMIJEMPORNLGBW-UHFFFAOYSA-N", "MCM:C111OOH", "")
  CALL Species("C_{11}H_{20}O_4",                              "", "YQNGKLISCMALPY-UHFFFAOYSA-N", "MCM:C114OOH", "")
  CALL Species("C_{11}H_{18}O_5",                              "", "VPBZYPGTLQIJKG-UHFFFAOYSA-N", "MCM:C115OOH", "")
  CALL Species("C_{11}H_{18}O_3",                              "", "YADKKYWEPSBFCL-UHFFFAOYSA-N", "MCM:CO356C11", "")
  CALL Species("C_{11}H_{20}O_2",                              "", "KWHQQLQVDJMZTG-UHFFFAOYSA-N", "MCM:CO36C11", "")
  CALL Species("C_{11}H_{22}O",                                "", "YNMZZHPSYMOGCI-UHFFFAOYSA-N", "MCM:UDEC3ONE", "")
  CALL Species("C_{12}H_{24}O_3",                              "", "NWJGDRGQGOZUDJ-UHFFFAOYSA-N", "MCM:C121OOH", "")
  CALL Species("C_{12}H_{22}O_4",                              "", "DCPRRPHGFZBUMW-UHFFFAOYSA-N", "MCM:C124OOH", "")
  CALL Species("C_{12}H_{20}O_5",                              "", "FTFXETPWXOMYNK-UHFFFAOYSA-N", "MCM:C125OOH", "")
  CALL Species("C_{12}H_{20}O_3",                              "", "FQYVJQVZHPFSKC-UHFFFAOYSA-N", "MCM:CO356C12", "")
  CALL Species("C_{12}H_{22}O_2",                              "", "RUTQSYKISNASLK-UHFFFAOYSA-N", "MCM:CO36C12", "")
  CALL Species("C_{12}H_{24}O",                                "", "PERIHWAPLOBAJM-UHFFFAOYSA-N", "MCM:DDEC3ONE", "")
  CALL Species("C_{13}H_{22}O_4",                              "", "HPZPYFVEAXAGHY-UHFFFAOYSA-N", "MCM:C131OOH", "")
  CALL Species("C_{14}H_{22}O",                                "", "MBZBBVTYLUNZPJ-UHFFFAOYSA-N", "MCM:BCKET", "")
  CALL Species("C_{14}H_{22}O_5",                              "", "LLSRBTGTUZNRQT-UHFFFAOYSA-N", "MCM:C131CO3H", "")
  CALL Species("C_{14}H_{24}O_3",                              "", "HKWHMEKBBPOBSQ-UHFFFAOYSA-N", "MCM:C141OOH", "")
  CALL Species("C_{15}H_{24}O_4",                              "", "ADRMPTUALHKSOU-UHFFFAOYSA-N", "MCM:C141CO3H", "")
  CALL Species("C_6H_6O_3",                                    "", "NGHVTTBOPYXCFB-UHFFFAOYSA-N", "MCM:C619CO", "")
  CALL Species("C_6H_6O_3",                                    "", "GKSCYYCYSPXQFY-UHFFFAOYSA-N", "MCM:CY6TRION", "")
  CALL Species("C_8H_8O_3",                                    "", "BXVCLOREWFXMNM-UHFFFAOYSA-N", "MCM:PHCOMEOOH", "")
  CALL Species("C_9H_{10}O_3",                                 "", "NJDRUBYBGCJLGU-UHFFFAOYSA-N", "MCM:MPHCOMEOOH", "")
  CALL Species("C_9H_8O_4",                                    "", "XMJMKOZGMVOAIC-UHFFFAOYSA-N", "MCM:PHCOCOCOOH", "")
  CALL Species("C_9H_8O_2",                                    "", "BVQVLAIMHVDZEL-UHFFFAOYSA-N", "MCM:PHCOCOME", "")
  CALL Species("C_9H_{10}O_3",                                 "", "RYQHVKZCKUBIFL-UHFFFAOYSA-N", "MCM:PHCOETOOH", "")
  CALL Species("C_{10}H_{12}O",                                "", "BKIHFZLJJUNKMZ-UHFFFAOYSA-N", "MCM:DMPHCOME", "")
  CALL Species("C_{10}H_{12}O_3",                              "", "SEDWELDPOZYWLL-UHFFFAOYSA-N", "MCM:DMPHCOMOOH", "")
  CALL Species("C_{11}H_{14}O",                                "", "CZNQKVDFQIGBEW-UHFFFAOYSA-N", "MCM:EMPHCOME", "")
  CALL Species("C_{11}H_{14}O_3",                              "", "OJNDCDHAKIGFTB-UHFFFAOYSA-N", "MCM:EMPHCOMOOH", "")
  CALL Species("C_4H_6O_3",                                    "", "NXIVQSJQXMAXJR-UHFFFAOYSA-N", "MCM:BIACETOH", "")
  CALL Species("C_4H_6O_5",                                    "", "WNDCXKJVOGKFRM-UHFFFAOYSA-N", "MCM:CO2H3CO3H", "")
  CALL Species("C_4H_6O_6",                                    "", "RXVYYLAFXUAZCR-UHFFFAOYSA-N", "MCM:H13CO2CO3H", "")
  CALL Species("C_4H_6O_4",                                    "", "GJCZUCLKDGABDS-UHFFFAOYSA-N", "MCM:H14CO23C4", "")
  CALL Species("C_4H_6O_5",                                    "", "FQXHRCVOGVCTSR-UHFFFAOYSA-N", "MCM:H1C23C4OOH", "")
  CALL Species("C_4H_8O_4",                                    "", "XTELZFGIGJBUGR-UHFFFAOYSA-N", "MCM:HMVKAOOH", "")
  CALL Species("C_4H_8O_4",                                    "", "NGLQGLCPXKPDMS-UHFFFAOYSA-N", "MCM:HMVKBOOH", "")
  CALL Species("C_4H_8O_3",                                    "", "SEYLPRWNVFCVRQ-UHFFFAOYSA-N", "MCM:HO12CO3C4", "")
  CALL Species("C_4H_8O_3",                                    "", "XBJODPUPYBBDEM-UHFFFAOYSA-N", "MCM:HO14CO2C4", "")
  CALL Species("C_4H_8O_4",                                    "", "GQXQZBROPOTXJQ-UHFFFAOYSA-N", "MCM:HOCO3C4OOH", "")
  CALL Species("C_4H_8O_2",                                    "", "LVSQXDHWDCMMRJ-UHFFFAOYSA-N", "MCM:MEKAOH", "")
  CALL Species("C_4H_8O_2",                                    "", "GFAZHVHNLUBROE-UHFFFAOYSA-N", "MCM:MEKCOH", "")
  CALL Species("C_4H_8O_4",                                    "", "UQPHVQVXLPRNCX-UHFFFAOYSA-N", "MCM:MVKOHAOH", "")
  CALL Species("C_4H_8O_5",                                    "", "MOVXURHCNUWOQO-UHFFFAOYSA-N", "MCM:MVKOHAOOH", "")
  CALL Species("C_4H_8O_5",                                    "", "WSMWIMXDHMPHIN-UHFFFAOYSA-N", "MCM:MVKOHBOOH", "")
  CALL Species("C_4H_6O_2",                                    "", "LHBQGXZUVXFJRH-UHFFFAOYSA-N", "MCM:MVKOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "NQMZUMSFJBKHAU-UHFFFAOYSA-N", "MCM:C517OH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "OAFCGFSFCVGKSH-UHFFFAOYSA-N", "MCM:C517OOH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "RDSFTYTWLRHJGZ-UHFFFAOYSA-N", "MCM:C519OOH", "")
  CALL Species("C_5H_{10}O_2",                                 "", "WOVLKKLXYZJMSN-UHFFFAOYSA-N", "MCM:C51OH2CO", "")
  CALL Species("C_5H_{10}O_3",                                 "", "NJUQBPDKVORANW-UHFFFAOYSA-N", "MCM:C51OH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "IZBMMNKBXWVQLB-UHFFFAOYSA-N", "MCM:C51OOH", "")
  CALL Species("C_5H_8O_4",                                    "", "UOGQGBWLBZOULD-UHFFFAOYSA-N", "MCM:C521OH", "")
  CALL Species("C_5H_8O_5",                                    "", "INSNYERMNWOCIO-UHFFFAOYSA-N", "MCM:C521OOH", "")
  CALL Species("C_5H_8O_3",                                    "", "JVFYSLPFISVGRS-UHFFFAOYSA-N", "MCM:C524CO", "")
  CALL Species("C_5H_{10}O_6",                                 "", "KYXSOONCNSTMJK-UHFFFAOYSA-N", "MCM:C525OOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "PLVCXNLHUHTLCR-UHFFFAOYSA-N", "MCM:C53OH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "VCGFNBZCPLQTJR-UHFFFAOYSA-N", "MCM:C53OOH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "IEXMEMALQGQSKE-UHFFFAOYSA-N", "MCM:C55OOH", "")
  CALL Species("C_5H_{10}O_5",                                 "", "JWNCLQLBUYGJMN-UHFFFAOYSA-N", "MCM:C59OOH", "")
  CALL Species("C_5H_8O_3",                                    "", "SLKPFOVYRYRNBA-UHFFFAOYSA-N", "MCM:C5CO243OH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "DSLVXKSJLFFUGE-UHFFFAOYSA-N", "MCM:C5OHCO4OOH", "")
  CALL Species("C_5H_{10}O_2",                                 "", "JSHPTIGHEWEXRW-UHFFFAOYSA-N", "MCM:CO2C5OH", "")
  CALL Species("C_5H_8O_5",                                    "", "LZACDMRRAOGIBF-UHFFFAOYSA-N", "MCM:CO2H3MCO3H", "")
  CALL Species("C_5H_8O_5",                                    "", "LAOPLCWFGWEKGR-UHFFFAOYSA-N", "MCM:CO3H4CO3H", "")
  CALL Species("C_5H_{10}O_2",                                 "", "QMXCHEVUAIPIRM-UHFFFAOYSA-N", "MCM:DIEKAOH", "")
  CALL Species("C_5H_8O_5",                                    "", "NYZNEBYYPQANOB-UHFFFAOYSA-N", "MCM:H3C2C4CO3H", "")
  CALL Species("C_5H_6O_6",                                    "", "UQNPRLJBDVQJQC-UHFFFAOYSA-N", "MCM:HC23C4CO3H", "")
  CALL Species("C_5H_8O_2",                                    "", "LSMLKPXBSFFBNR-UHFFFAOYSA-N", "MCM:HCOC5", "")
  CALL Species("C_5H_8O_5",                                    "", "VFNFGUWSZFHBRW-UHFFFAOYSA-N", "MCM:HMVKBCO3H", "")
  CALL Species("C_5H_{10}O_3",                                 "", "CGPIPFJOMJUWGK-UHFFFAOYSA-N", "MCM:HO13CO4C5", "")
  CALL Species("C_5H_{10}O_3",                                 "", "XLDKDOKPMNKVMP-UHFFFAOYSA-N", "MCM:HO14CO2C5", "")
  CALL Species("C_5H_{10}O_3",                                 "", "YMIRTILUDYUWGY-UHFFFAOYSA-N", "MCM:HO14CO3C5", "")
  CALL Species("C_5H_8O_3",                                    "", "VBUBLWCIADYRMB-UHFFFAOYSA-N", "MCM:HO1CO24C5", "")
  CALL Species("C_5H_8O_3",                                    "", "LKLKMCUVBQDYFU-UHFFFAOYSA-N", "MCM:HO1CO34C5", "")
  CALL Species("C_5H_{10}O_2",                                 "", "TYXULUBCBKMSSK-UHFFFAOYSA-N", "MCM:HO1CO3C5", "")
  CALL Species("C_5H_{10}O_2",                                 "", "PCYZZYAEGNVNMH-UHFFFAOYSA-N", "MCM:HO2CO4C5", "")
  CALL Species("C_5H_{10}O_4",                                 "", "ZLPJDXTYGYNOQK-UHFFFAOYSA-N", "MCM:HOCO3C5OOH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "MYGJVQXASPJPCS-UHFFFAOYSA-N", "MCM:HOCO4C5OOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "FEIUXLCYYBVUGD-UHFFFAOYSA-N", "MCM:MBOACO", "")
  CALL Species("C_5H_{10}O_2",                                 "", "NBEGXSQMVJTIAR-UHFFFAOYSA-N", "MCM:ME3CO2BUOL", "")
  CALL Species("C_5H_{10}O_2",                                 "", "BNDRWEVUODOUDW-UHFFFAOYSA-N", "MCM:MIPKAOH", "")
  CALL Species("C_5H_{10}O_2",                                 "", "VVSRECWZBBJOTG-UHFFFAOYSA-N", "MCM:MIPKBOH", "")
  CALL Species("C_5H_{10}O_2",                                 "", "HDKKRASBPHFULQ-UHFFFAOYSA-N", "MCM:MPRKAOH", "")
  CALL Species("C_6H_6O_4",                                    "", "OGRXSIBGKCRBOL-UHFFFAOYSA-N", "MCM:BZOBIPEROH", "")
  CALL Species("C_6H_{12}O_2",                                 "", "FDJJNIXWMAWMBP-UHFFFAOYSA-N", "MCM:C4COMEOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "NCEURKGQTPCVEC-UHFFFAOYSA-N", "MCM:C4COMOH3OH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "JZGTXIGKACACHL-UHFFFAOYSA-N", "MCM:C4COMOHOOH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "BJXKNHIKUQDPQL-UHFFFAOYSA-N", "MCM:C4MOHOCO3H", "")
  CALL Species("C_6H_{10}O_5",                                 "", "HTNOTPIWPYZOLN-UHFFFAOYSA-N", "MCM:C517CO3H", "")
  CALL Species("C_6H_{10}O_5",                                 "", "OBKQTPZEBCGGJT-UHFFFAOYSA-N", "MCM:C519CO3H", "")
  CALL Species("C_6H_8O_6",                                    "", "BPYCYKKOURHNJX-UHFFFAOYSA-N", "MCM:C5COHOCO3H", "")
  CALL Species("C_6H_{10}O_5",                                 "", "GCQMJNYHSPBFDA-UHFFFAOYSA-N", "MCM:C5O5OHCO3H", "")
  CALL Species("C_6H_{12}O_3",                                 "", "YFEDHAOQJOZUIL-UHFFFAOYSA-N", "MCM:C610OH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "VBEFRWQFSDXQDV-UHFFFAOYSA-N", "MCM:C610OOH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "PYRMISJRKRVNPG-UHFFFAOYSA-N", "MCM:C611OH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "VCLSUNMNDIKRMY-UHFFFAOYSA-N", "MCM:C612OH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "OANIXDZJWBEQCD-UHFFFAOYSA-N", "MCM:C612OOH", "")
  CALL Species("C_6H_{12}O_5",                                 "", "MBLADDFAKVGAFO-UHFFFAOYSA-N", "MCM:C613OOH", "")
  CALL Species("C_6H_8O_4",                                    "", "DFWHGSVPVJPYJZ-UHFFFAOYSA-N", "MCM:C614CO", "")
  CALL Species("C_6H_{10}O_4",                                 "", "WLPHOQAEOJPJID-UHFFFAOYSA-N", "MCM:C614OH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "AZSLPKKGLPCKMI-UHFFFAOYSA-N", "MCM:C614OOH", "")
  CALL Species("C_6H_8O_3",                                    "", "WEYIBABPZCYASX-UHFFFAOYSA-N", "MCM:C619OH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "MGEAAKFVOQZVFH-UHFFFAOYSA-N", "MCM:C61CO", "")
  CALL Species("C_6H_{12}O_3",                                 "", "JOVKXRBMZIPRBZ-UHFFFAOYSA-N", "MCM:C61OH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "ZUQHNOSGBJLTRH-UHFFFAOYSA-N", "MCM:C61OOH", "")
  CALL Species("C_6H_{10}O_6",                                 "", "OMSKVAAPFLCUBC-UHFFFAOYSA-N", "MCM:C621OOH", "")
  CALL Species("C_6H_{10}O_2",                                 "", "SAJQFIMRYKQCMA-UHFFFAOYSA-N", "MCM:C624CO", "")
  CALL Species("C_6H_{12}O_4",                                 "", "UHZLCNNVPSWLBR-UHFFFAOYSA-N", "MCM:C625OH", "")
  CALL Species("C_6H_{12}O_5",                                 "", "RLEQEEKQTRHVBK-UHFFFAOYSA-N", "MCM:C625OOH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "YXADPHVQSSNJLB-UHFFFAOYSA-N", "MCM:C627OH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "LTOZVYXSZHIEKZ-UHFFFAOYSA-N", "MCM:C630OH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "FPSNWVZQINIJKM-UHFFFAOYSA-N", "MCM:C630OOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "YEHSOHVSOJHCML-UHFFFAOYSA-N", "MCM:C63OH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "RYUSCYCDUHZJRL-UHFFFAOYSA-N", "MCM:C63OOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "DWHFPVYPGLVOER-UHFFFAOYSA-N", "MCM:C64OH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "WKWKMZOJIHCBFC-UHFFFAOYSA-N", "MCM:C64OOH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "PQXOVUFKUOIXAY-UHFFFAOYSA-N", "MCM:C66CO", "")
  CALL Species("C_6H_{12}O_3",                                 "", "YKCRSYHGTATDJD-UHFFFAOYSA-N", "MCM:C66OH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "ZIMKSIUZUDHWGV-UHFFFAOYSA-N", "MCM:C66OOH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "BGZGHPIMJSIBNB-UHFFFAOYSA-N", "MCM:C67CO3H", "")
  CALL Species("C_6H_{12}O_3",                                 "", "UAFCVHAXODPLTF-UHFFFAOYSA-N", "MCM:C69OH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "IBTXITPBBCYNLS-UHFFFAOYSA-N", "MCM:C69OOH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "ZFXUWHPHUNDYBV-UHFFFAOYSA-N", "MCM:C6CO23HO5", "")
  CALL Species("C_6H_{10}O_3",                                 "", "YZRCADLLOAXRLD-UHFFFAOYSA-N", "MCM:C6CO243OH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "LBGWMVXVPMPEID-UHFFFAOYSA-N", "MCM:C6CO34HO1", "")
  CALL Species("C_6H_{12}O_3",                                 "", "AEYYIRCLAOJEMX-UHFFFAOYSA-N", "MCM:C6CO3HO14", "")
  CALL Species("C_6H_{12}O_3",                                 "", "AQJAHIKLRWQRAX-UHFFFAOYSA-N", "MCM:C6CO3HO25", "")
  CALL Species("C_6H_{12}O_2",                                 "", "SKCYVGUCBRYGTE-UHFFFAOYSA-N", "MCM:C6CO3HO4", "")
  CALL Species("C_6H_{12}O_4",                                 "", "VJHRZIUXRMRGDC-UHFFFAOYSA-N", "MCM:C6CO3OHOOH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "DEHZHOWOLMWYLS-UHFFFAOYSA-N", "MCM:C6HOCOOOH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "JUTFYFRULBWUNH-UHFFFAOYSA-N", "MCM:CO24M3C5OH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "MXGSILCFPNVDMX-UHFFFAOYSA-N", "MCM:CO25C6OH", "")
  CALL Species("C_6H_{12}O_2",                                 "", "UHSBCAJZDUQTHH-UHFFFAOYSA-N", "MCM:CO2HO3C6", "")
  CALL Species("C_6H_{12}O_2",                                 "", "ODWYTDVNWFBCLV-UHFFFAOYSA-N", "MCM:CO2HO4C6", "")
  CALL Species("C_6H_{12}O_4",                                 "", "DONIGPAOCGSEHJ-UHFFFAOYSA-N", "MCM:CO2HOC6OOH", "")
  CALL Species("C_6H_{12}O_2",                                 "", "YJCMJVMCCJIPRV-UHFFFAOYSA-N", "MCM:CO2M3C5OH", "")
  CALL Species("C_6H_{12}O_2",                                 "", "NZBRXFKHZBOFBW-UHFFFAOYSA-N", "MCM:CO2MC5OH", "")
  CALL Species("C_6H_8O_3",                                    "", "ACRUCDPQQCIOID-UHFFFAOYSA-N", "MCM:CY6DIONOH", "")
  CALL Species("C_6H_{10}O_2",                                 "", "ODZTXUXIYGJLMC-UHFFFAOYSA-N", "MCM:CYHXOLACO", "")
  CALL Species("C_6H_{10}O_2",                                 "", "TWEVQGUWCLBRMJ-UHFFFAOYSA-N", "MCM:CYHXONAOH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "RQDWELNLPMBYMA-UHFFFAOYSA-N", "MCM:DMK2OH", "")
  CALL Species("C_6H_8O_4",                                    "", "OKDWWGVTKQPUTM-UHFFFAOYSA-N", "MCM:DMKCOOH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "WAHPQVAXIMVNLL-UHFFFAOYSA-N", "MCM:DMKOHOOH", "")
  CALL Species("C_6H_{12}O_2",                                 "", "SYAVYWAMVZKGTF-UHFFFAOYSA-N", "MCM:EIPKAOH", "")
  CALL Species("C_6H_{12}O_2",                                 "", "JYEMLYGXHDIVRA-UHFFFAOYSA-N", "MCM:EIPKBOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "NBNMOYHUQSLZCQ-UHFFFAOYSA-N", "MCM:H13M3CO4C5", "")
  CALL Species("C_6H_{10}O_4",                                 "", "XLUGOLRJPRATMY-UHFFFAOYSA-N", "MCM:H3C25C6OH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "XIJKANBCMKWKRM-UHFFFAOYSA-N", "MCM:H3C25C6OOH", "")
  CALL Species("C_6H_{12}O_2",                                 "", "XHYXKWWFNRBCGE-UHFFFAOYSA-N", "MCM:HEX3ONAOH", "")
  CALL Species("C_6H_{12}O_2",                                 "", "ZWBUSAWJHMPOEJ-UHFFFAOYSA-N", "MCM:HEX3ONCOH", "")
  CALL Species("C_6H_{12}O_2",                                 "", "ITHSWIXXHGKFJW-UHFFFAOYSA-N", "MCM:HEX3ONDOH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "NUDMYLMVZRZHHU-UHFFFAOYSA-N", "MCM:HO1CO24C6", "")
  CALL Species("C_6H_{12}O_2",                                 "", "APQMHEQLBDXGMP-UHFFFAOYSA-N", "MCM:HO1CO4C6", "")
  CALL Species("C_6H_{12}O_2",                                 "", "ZSDLLTJVENEIDW-UHFFFAOYSA-N", "MCM:HO2CO5C6", "")
  CALL Species("C_6H_{12}O_2",                                 "", "ZXZUCILLTLHIBZ-UHFFFAOYSA-N", "MCM:M2BKAOH", "")
  CALL Species("C_6H_{12}O_2",                                 "", "KHCUSEDRQWYNDS-UHFFFAOYSA-N", "MCM:M2BKBOH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "VGLNQNYMXFMHPV-UHFFFAOYSA-N", "MCM:MBKCOOHOOH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "NARMPYMUEZMSEV-UHFFFAOYSA-N", "MCM:MIBKAOH3CO", "")
  CALL Species("C_6H_{12}O_2",                                 "", "IGPIDYBTABPKQT-UHFFFAOYSA-N", "MCM:MIBKBOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "UZSRJTNVDUYOLE-UHFFFAOYSA-N", "MCM:MIBKHO14", "")
  CALL Species("C_6H_{12}O_4",                                 "", "FGHCAVJLMYVGOV-UHFFFAOYSA-N", "MCM:MIBKHO4OOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "OSPIOFUQWMUWOI-UHFFFAOYSA-N", "MCM:MIBKOH34", "")
  CALL Species("C_6H_{12}O_4",                                 "", "WIXFQYMFSUQGGT-UHFFFAOYSA-N", "MCM:MIBKOHAOOH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "NCKNZQNPSVRYAP-UHFFFAOYSA-N", "MCM:MIBKOHBOOH", "")
  CALL Species("C_6H_{12}O_2",                                 "", "DYAWMXSWDGPGOI-UHFFFAOYSA-N", "MCM:MTBKOH", "")
  CALL Species("C_6H_4O_4",                                    "", "LTWQMQDFZHUKCN-UHFFFAOYSA-N", "MCM:PBZQCO", "")
  CALL Species("C_6H_6O_4",                                    "", "KGAUAKJVBKFEJK-UHFFFAOYSA-N", "MCM:PBZQOH", "")
  CALL Species("C_6H_6O_5",                                    "", "GWWKDNIKIVTFLV-UHFFFAOYSA-N", "MCM:PBZQOOH", "")
  CALL Species("C_7H_{10}O_6",                                 "", "LNQZNTUCDYUQNY-UHFFFAOYSA-N", "MCM:C61CO3H", "")
  CALL Species("C_7H_{10}O_6",                                 "", "OTMFEIRKQNFPTO-UHFFFAOYSA-N", "MCM:C62CO3H", "")
  CALL Species("C_7H_{14}O_4",                                 "", "CHMPWQJFRSDBMP-UHFFFAOYSA-N", "MCM:C711OOH", "")
  CALL Species("C_7H_{14}O_3",                                 "", "XKQDJNZCXUTPAB-UHFFFAOYSA-N", "MCM:C712OH", "")
  CALL Species("C_7H_{14}O_4",                                 "", "ISDXEEIPAJYQKS-UHFFFAOYSA-N", "MCM:C712OOH", "")
  CALL Species("C_7H_{12}O_3",                                 "", "KKMPMNHIJNYFMY-UHFFFAOYSA-N", "MCM:C713OH", "")
  CALL Species("C_7H_{12}O_3",                                 "", "BDJIDAHXOAZMLT-UHFFFAOYSA-N", "MCM:C714OH", "")
  CALL Species("C_7H_{12}O_4",                                 "", "SNLGDBZBHBGDGF-UHFFFAOYSA-N", "MCM:C719OH", "")
  CALL Species("C_7H_{12}O_5",                                 "", "RIQZESWBZQGTOF-UHFFFAOYSA-N", "MCM:C719OOH", "")
  CALL Species("C_7H_{14}O_4",                                 "", "NBQALOYIKVIATH-UHFFFAOYSA-N", "MCM:C71OOH", "")
  CALL Species("C_7H_{10}O_3",                                 "", "KXKCGTAVIRFZLI-UHFFFAOYSA-N", "MCM:C726CO3OH", "")
  CALL Species("C_7H_{14}O_3",                                 "", "NSVUCXUEJIZYCX-UHFFFAOYSA-N", "MCM:C72OH", "")
  CALL Species("C_7H_{14}O_4",                                 "", "HOCRXLJRXJDOAI-UHFFFAOYSA-N", "MCM:C72OOH", "")
  CALL Species("C_7H_{14}O_4",                                 "", "RNKPJWWQPAQZAU-UHFFFAOYSA-N", "MCM:C76OOH", "")
  CALL Species("C_7H_{12}O_3",                                 "", "LRWMVVHMPWURQP-UHFFFAOYSA-N", "MCM:C77CO", "")
  CALL Species("C_7H_{14}O_3",                                 "", "BWFQHNYIFKCYIN-UHFFFAOYSA-N", "MCM:C77OH", "")
  CALL Species("C_7H_{14}O_4",                                 "", "VLZZBCPZVYCEGU-UHFFFAOYSA-N", "MCM:C77OOH", "")
  CALL Species("C_7H_{12}O_3",                                 "", "XZDSUXRHWGXGLP-UHFFFAOYSA-N", "MCM:C78CO", "")
  CALL Species("C_7H_{14}O_3",                                 "", "RHPFPSWJNNLJIH-UHFFFAOYSA-N", "MCM:C78OH", "")
  CALL Species("C_7H_{14}O_4",                                 "", "CHSMMMZNDOATLZ-UHFFFAOYSA-N", "MCM:C78OOH", "")
  CALL Species("C_7H_{12}O_5",                                 "", "PAASZTKJDWJXEO-UHFFFAOYSA-N", "MCM:C79OOH", "")
  CALL Species("C_7H_{12}O_4",                                 "", "IQLAANIUQOXZGT-UHFFFAOYSA-N", "MCM:C7BDCOH", "")
  CALL Species("C_7H_{12}O_5",                                 "", "YSGKLAZJXASOJQ-UHFFFAOYSA-N", "MCM:C7BDCOOH", "")
  CALL Species("C_7H_{10}O_4",                                 "", "ZZJOBVIMQADRSG-UHFFFAOYSA-N", "MCM:C7EDCCO", "")
  CALL Species("C_7H_{12}O_4",                                 "", "ILHMXLYATWSMQV-UHFFFAOYSA-N", "MCM:C7EDCOH", "")
  CALL Species("C_7H_{12}O_5",                                 "", "GFYCWCYSZRNKIC-UHFFFAOYSA-N", "MCM:C7EDCOOH", "")
  CALL Species("C_7H_{12}O_3",                                 "", "SUAOXERUOLNODJ-UHFFFAOYSA-N", "MCM:CO25C73OH", "")
  CALL Species("C_7H_{12}O_3",                                 "", "WVDVIEDFZYMNAI-UHFFFAOYSA-N", "MCM:CO25C74OH", "")
  CALL Species("C_7H_{14}O_2",                                 "", "GNNBSAPGFGNCCT-UHFFFAOYSA-N", "MCM:H2M2CO5C6", "")
  CALL Species("C_7H_{14}O_2",                                 "", "JXXNGUYHZPNXHL-UHFFFAOYSA-N", "MCM:H2M3CO5C6", "")
  CALL Species("C_7H_{14}O_2",                                 "", "ZIVAMUMQDXZFCR-UHFFFAOYSA-N", "MCM:H2M4CO5C6", "")
  CALL Species("C_7H_{10}O_6",                                 "", "ARDZNQBCJILRHG-UHFFFAOYSA-N", "MCM:H3C25CCO3H", "")
  CALL Species("C_7H_{10}O_2",                                 "", "WVMVYBISUAVCBD-UHFFFAOYSA-N", "MCM:HCC7CO", "")
  CALL Species("C_7H_{12}O_3",                                 "", "LULWLYVLBHVZLB-UHFFFAOYSA-N", "MCM:HO2CO35C7", "")
  CALL Species("C_7H_{14}O_2",                                 "", "DWMNUVZLRBCGOL-UHFFFAOYSA-N", "MCM:HO2CO5C7", "")
  CALL Species("C_7H_{14}O_2",                                 "", "MQRALIJAASQPNT-UHFFFAOYSA-N", "MCM:HO3CO6C7", "")
  CALL Species("C_7H_6O_4",                                    "", "QCLUMLMJJNMFFX-UHFFFAOYSA-N", "MCM:PTLQCO", "")
  CALL Species("C_7H_8O_4",                                    "", "FWCZASAULMADEU-UHFFFAOYSA-N", "MCM:PTLQOH", "")
  CALL Species("C_7H_8O_5",                                    "", "ULCYCDVSDDOIBS-UHFFFAOYSA-N", "MCM:PTLQOOH", "")
  CALL Species("C_7H_8O_4",                                    "", "BTRXIMYXFXXDNL-UHFFFAOYSA-N", "MCM:TLCOBIPEOH", "")
  CALL Species("C_7H_8O_4",                                    "", "OGIZZGFCOPEREO-UHFFFAOYSA-N", "MCM:TLOBIPEROH", "")
  CALL Species("C_8H_{10}O_6",                                 "", "UAMJQIWVNIVSAA-UHFFFAOYSA-N", "MCM:C6MOCOCO3H", "")
  CALL Species("C_8H_{12}O_3",                                 "", "VIGCXZSBZQABDL-UHFFFAOYSA-N", "MCM:C7CO2M5OH", "")
  CALL Species("C_8H_{10}O_6",                                 "", "PYNUJNZYCLWJPS-UHFFFAOYSA-N", "MCM:C7CO2OCO3H", "")
  CALL Species("C_8H_{12}O_3",                                 "", "IWFVDGADKJRMAI-UHFFFAOYSA-N", "MCM:C7M2CO5OH", "")
  CALL Species("C_8H_{14}O_3",                                 "", "HXKXVESDDNIBIQ-UHFFFAOYSA-N", "MCM:C817OH", "")
  CALL Species("C_8H_{12}O_4",                                 "", "CRQAGKNTHDNGEW-UHFFFAOYSA-N", "MCM:C818CO", "")
  CALL Species("C_8H_{14}O_4",                                 "", "GYBDPSWJEGVQPC-UHFFFAOYSA-N", "MCM:C818OH", "")
  CALL Species("C_8H_{14}O_5",                                 "", "DLMBPALZQHFDOH-UHFFFAOYSA-N", "MCM:C818OOH", "")
  CALL Species("C_8H_{14}O_5",                                 "", "JRXMSSDNDAZISI-UHFFFAOYSA-N", "MCM:C819OOH", "")
  CALL Species("C_8H_{16}O_4",                                 "", "SFFBCYBCCUXGLX-UHFFFAOYSA-N", "MCM:C81OOH", "")
  CALL Species("C_8H_{12}O_6",                                 "", "MPCACGHSMGOFNV-UHFFFAOYSA-N", "MCM:C820OOH", "")
  CALL Species("C_8H_{12}O_3",                                 "", "GGSZUEZFMJTLBX-UHFFFAOYSA-N", "MCM:C826CO3OH", "")
  CALL Species("C_8H_{14}O_3",                                 "", "VYHQZXLAUHJBAB-UHFFFAOYSA-N", "MCM:C827OH", "")
  CALL Species("C_8H_{14}O_4",                                 "", "ZFRREEXTYBESJX-UHFFFAOYSA-N", "MCM:C829OH", "")
  CALL Species("C_8H_{14}O_5",                                 "", "PFFZQAOPEABNHW-UHFFFAOYSA-N", "MCM:C829OOH", "")
  CALL Species("C_8H_{16}O_4",                                 "", "AQECYOJQUIJAFS-UHFFFAOYSA-N", "MCM:C82OOH", "")
  CALL Species("C_8H_{14}O_3",                                 "", "HXKSWDLNZPICFQ-UHFFFAOYSA-N", "MCM:C84OH", "")
  CALL Species("C_8H_{12}O_3",                                 "", "DCTATDQRTQNXHW-UHFFFAOYSA-N", "MCM:C88OH", "")
  CALL Species("C_8H_{10}O_4",                                 "", "YDVGTQTYIBCIQC-UHFFFAOYSA-N", "MCM:EBZOBPEROH", "")
  CALL Species("C_8H_{16}O_3",                                 "", "GQDVWZAGWHUXKX-UHFFFAOYSA-N", "MCM:HO34CO6C8", "")
  CALL Species("C_8H_{14}O_3",                                 "", "LLYUJOAMIUBKSA-UHFFFAOYSA-N", "MCM:HO3CO46C8", "")
  CALL Species("C_8H_{16}O_2",                                 "", "VJDZJBNTVLGPEM-UHFFFAOYSA-N", "MCM:HO3CO6C8", "")
  CALL Species("C_8H_{10}O_4",                                 "", "BQJOHFWATKFGAQ-UHFFFAOYSA-N", "MCM:MXYOBPEROH", "")
  CALL Species("C_8H_{10}O_4",                                 "", "DNSXSYZLXRHYTM-UHFFFAOYSA-N", "MCM:MXYQOH", "")
  CALL Species("C_8H_{10}O_5",                                 "", "UAWVAAITGKOKNW-UHFFFAOYSA-N", "MCM:MXYQOOH", "")
  CALL Species("C_8H_8O_4",                                    "", "QSCBEAAIZXLBQA-UHFFFAOYSA-N", "MCM:OXYQCO", "")
  CALL Species("C_8H_{10}O_4",                                 "", "LJMMKZLQFVIDLQ-UHFFFAOYSA-N", "MCM:OXYQOH", "")
  CALL Species("C_8H_{10}O_5",                                 "", "NOWQTCYBLCRAEJ-UHFFFAOYSA-N", "MCM:OXYQOOH", "")
  CALL Species("C_8H_{10}O_4",                                 "", "RORXCXPFLRGHQV-UHFFFAOYSA-N", "MCM:PXYQOH", "")
  CALL Species("C_8H_{10}O_5",                                 "", "FYDHRLZPZZTRLJ-UHFFFAOYSA-N", "MCM:PXYQOOH", "")
  CALL Species("C_9H_{12}O_6",                                 "", "KMUWHHLOUXRZEC-UHFFFAOYSA-N", "MCM:C6EO2HCO3H", "")
  CALL Species("C_9H_{12}O_6",                                 "", "HVGFJXMJLWNDQT-UHFFFAOYSA-N", "MCM:C7MJPCO3H", "")
  CALL Species("C_9H_{12}O_6",                                 "", "UXMGGYROBLVMKA-UHFFFAOYSA-N", "MCM:C7MOCOCO3H", "")
  CALL Species("C_9H_{14}O_3",                                 "", "AUYUFIFJEJTOMG-UHFFFAOYSA-N", "MCM:C8M2CO6OH", "")
  CALL Species("C_9H_{14}O_3",                                 "", "DVAQWOBQFULNSX-UHFFFAOYSA-N", "MCM:C917OH", "")
  CALL Species("C_9H_{16}O_4",                                 "", "CXKZGZZGALDRTB-UHFFFAOYSA-N", "MCM:C920OOH", "")
  CALL Species("C_9H_{16}O_5",                                 "", "MIFQELLQXZRDIQ-UHFFFAOYSA-N", "MCM:C921OOH", "")
  CALL Species("C_9H_{16}O_6",                                 "", "IOTRHTNOHNKLJT-UHFFFAOYSA-N", "MCM:C922OOH", "")
  CALL Species("C_9H_{16}O_2",                                 "", "VZYSLJNPEGILSI-UHFFFAOYSA-N", "MCM:C923OH", "")
  CALL Species("C_9H_{14}O_3",                                 "", "IEIWFPIZSBHOOD-UHFFFAOYSA-N", "MCM:C924CO", "")
  CALL Species("C_9H_{16}O_3",                                 "", "OXSRPARPRXNCEH-UHFFFAOYSA-N", "MCM:C924OH", "")
  CALL Species("C_9H_{16}O_4",                                 "", "BVODBOGLMHSHGK-UHFFFAOYSA-N", "MCM:C924OOH", "")
  CALL Species("C_9H_{16}O_6",                                 "", "BGVLBDREANINMN-UHFFFAOYSA-N", "MCM:C925OOH", "")
  CALL Species("C_9H_{16}O_3",                                 "", "JILOVKBSEAXPMT-UHFFFAOYSA-N", "MCM:C927OH", "")
  CALL Species("C_9H_{16}O_4",                                 "", "UVVCFFCMOUIWEJ-UHFFFAOYSA-N", "MCM:C927OOH", "")
  CALL Species("C_9H_{16}O_3",                                 "", "QGCNXROUFNWXOO-UHFFFAOYSA-N", "MCM:C928OH", "")
  CALL Species("C_9H_{14}O_4",                                 "", "BWILLNTUIBCKIM-UHFFFAOYSA-N", "MCM:C929CO", "")
  CALL Species("C_9H_{16}O_4",                                 "", "PMWQIXWFMSCXRH-UHFFFAOYSA-N", "MCM:C929OH", "")
  CALL Species("C_9H_{16}O_5",                                 "", "IKUORCWWMOEDGL-UHFFFAOYSA-N", "MCM:C929OOH", "")
  CALL Species("C_9H_{18}O_4",                                 "", "MOTPUWPPGZYDTM-UHFFFAOYSA-N", "MCM:C92OOH", "")
  CALL Species("C_9H_{16}O_3",                                 "", "NCWLHDSPZIENNF-UHFFFAOYSA-N", "MCM:C93CO", "")
  CALL Species("C_9H_{18}O_3",                                 "", "XOZZRVPGLURXSY-UHFFFAOYSA-N", "MCM:C93OH", "")
  CALL Species("C_9H_{18}O_4",                                 "", "DSWUVSMVGXSNEV-UHFFFAOYSA-N", "MCM:C93OOH", "")
  CALL Species("C_9H_{16}O_3",                                 "", "RCWXDPYULCSTKX-UHFFFAOYSA-N", "MCM:C94OH", "")
  CALL Species("C_9H_{16}O_2",                                 "", "RRFQMYCDTHUGSU-UHFFFAOYSA-N", "MCM:C96OH", "")
  CALL Species("C_9H_{16}O_3",                                 "", "QWYKJUJQBVIHQW-UHFFFAOYSA-N", "MCM:C97OH", "")
  CALL Species("C_9H_{16}O_4",                                 "", "AXRJNDGSGFVBSX-UHFFFAOYSA-N", "MCM:C97OOH", "")
  CALL Species("C_9H_{16}O_4",                                 "", "BWKWRXKCGJAJAO-UHFFFAOYSA-N", "MCM:C98OH", "")
  CALL Species("C_9H_{16}O_5",                                 "", "MEUGYCLQCHCPCY-UHFFFAOYSA-N", "MCM:C98OOH", "")
  CALL Species("C_9H_{12}O_3",                                 "", "DIFRJFLUSYQBJM-UHFFFAOYSA-N", "MCM:C9DCOH", "")
  CALL Species("C_9H_{18}O_2",                                 "", "BPRAKHMXALGYKO-UHFFFAOYSA-N", "MCM:HO4CO7C9", "")
  CALL Species("C_9H_{12}O_4",                                 "", "WRQNGVASMLSOGA-UHFFFAOYSA-N", "MCM:IPBZOBPROH", "")
  CALL Species("C_9H_{10}O_4",                                 "", "UYSSVGYDANWBFJ-UHFFFAOYSA-N", "MCM:IPRBQCO", "")
  CALL Species("C_9H_{12}O_4",                                 "", "BQKYQBQGKYWJSQ-UHFFFAOYSA-N", "MCM:IPRBQOH", "")
  CALL Species("C_9H_{12}O_5",                                 "", "CCPBRYDTGCPJNX-UHFFFAOYSA-N", "MCM:IPRBQOOH", "")
  CALL Species("C_9H_{16}O_3",                                 "", "RDFFVUMNEKCADL-UHFFFAOYSA-N", "MCM:LMKAOH", "")
  CALL Species("C_9H_{16}O_4",                                 "", "ZYSXBTHWQDNLBX-UHFFFAOYSA-N", "MCM:LMKAOOH", "")
  CALL Species("C_9H_{14}O_3",                                 "", "QJYRJOZGIVJKQK-UHFFFAOYSA-N", "MCM:LMKBCO", "")
  CALL Species("C_9H_{16}O_4",                                 "", "PMLHMEIQMSUJKR-UHFFFAOYSA-N", "MCM:LMKBOOH", "")
  CALL Species("C_9H_{12}O_4",                                 "", "ZSTYNOBMCJBFRA-UHFFFAOYSA-N", "MCM:METLOBPROH", "")
  CALL Species("C_9H_{12}O_4",                                 "", "YIUSRNITCRMQJW-UHFFFAOYSA-N", "MCM:METLQOH", "")
  CALL Species("C_9H_{12}O_5",                                 "", "JBNOKZJMGKNXDA-UHFFFAOYSA-N", "MCM:METLQOOH", "")
  CALL Species("C_9H_{14}O_2",                                 "", "CVWGOJDPALGYKB-UHFFFAOYSA-N", "MCM:NOPINAOH", "")
  CALL Species("C_9H_{14}O_2",                                 "", "FPBMXOVOMRHDTB-UHFFFAOYSA-N", "MCM:NOPINBOH", "")
  CALL Species("C_9H_{14}O_2",                                 "", "OXRZVROAQQDWAA-UHFFFAOYSA-N", "MCM:NOPINCOH", "")
  CALL Species("C_9H_{14}O_2",                                 "", "MLOBETUCFXAOOL-UHFFFAOYSA-N", "MCM:NOPINDOH", "")
  CALL Species("C_9H_{10}O_4",                                 "", "WOEKXHYIEUECKR-UHFFFAOYSA-N", "MCM:OETLQCO", "")
  CALL Species("C_9H_{12}O_4",                                 "", "DZHQWTGZQAKVBN-UHFFFAOYSA-N", "MCM:OETLQOH", "")
  CALL Species("C_9H_{12}O_5",                                 "", "KKWSBZSFLIKUTL-UHFFFAOYSA-N", "MCM:OETLQOOH", "")
  CALL Species("C_9H_{12}O_4",                                 "", "JGNLEXNHGWFGHL-UHFFFAOYSA-N", "MCM:PBZOBPEROH", "")
  CALL Species("C_9H_{12}O_4",                                 "", "KZOAJFYPECZGSC-UHFFFAOYSA-N", "MCM:PETLQOH", "")
  CALL Species("C_9H_{12}O_5",                                 "", "BJRXYYOXPIRVAX-UHFFFAOYSA-N", "MCM:PETLQOOH", "")
  CALL Species("C_9H_{12}O_4",                                 "", "WQSRSICCLTUNRX-UHFFFAOYSA-N", "MCM:TM123OBPOH", "")
  CALL Species("C_9H_{12}O_4",                                 "", "GVKLCWOTVYVOTB-UHFFFAOYSA-N", "MCM:TM124QOH", "")
  CALL Species("C_9H_{12}O_5",                                 "", "SLEWHRYFRSXVOQ-UHFFFAOYSA-N", "MCM:TM124QOOH", "")
  CALL Species("C_9H_{12}O_4",                                 "", "MWFXWFJYOSXFGA-UHFFFAOYSA-N", "MCM:TM135OBPOH", "")
  CALL Species("C_{10}H_{16}O_2",                              "", "VZRRCQOUNSHSGB-UHFFFAOYSA-N", "MCM:APINBCO", "")
  CALL Species("C_{10}H_{18}O_2",                              "", "FSFKNSSBELMLGT-UHFFFAOYSA-N", "MCM:C1011OH", "")
  CALL Species("C_{10}H_{20}O_4",                              "", "QHLMVJFPSLBXKZ-UHFFFAOYSA-N", "MCM:C102OOH", "")
  CALL Species("C_{10}H_{18}O_3",                              "", "QIIAZEBRWZSMOM-UHFFFAOYSA-N", "MCM:C103CO", "")
  CALL Species("C_{10}H_{20}O_3",                              "", "SXEGIXJYQPFAQK-UHFFFAOYSA-N", "MCM:C103OH", "")
  CALL Species("C_{10}H_{20}O_4",                              "", "DOFXWCNTKGZWNU-UHFFFAOYSA-N", "MCM:C103OOH", "")
  CALL Species("C_{10}H_{18}O_3",                              "", "FHITZQAABUJLCH-UHFFFAOYSA-N", "MCM:C104OH", "")
  CALL Species("C_{10}H_{16}O_5",                              "", "OFVMBWQGLFFRLM-UHFFFAOYSA-N", "MCM:C920CO3H", "")
  CALL Species("C_{10}H_{16}O_3",                              "", "SLPXKTFAPIGCFF-UHFFFAOYSA-N", "MCM:C9M2CO6OH", "")
  CALL Species("C_{10}H_{14}O_4",                              "", "ZCTKJUASOMJGQC-UHFFFAOYSA-N", "MCM:DMEOBPROH", "")
  CALL Species("C_{10}H_{20}O_2",                              "", "GDOWVFFRHQHNBZ-UHFFFAOYSA-N", "MCM:HO5CO8C10", "")
  CALL Species("C_{10}H_{16}O_2",                              "", "JEQLRDRDFLXSHY-UHFFFAOYSA-N", "MCM:LIMBCO", "")
  CALL Species("C_{11}H_{22}O_4",                              "", "IFXOTQISYXBGBQ-UHFFFAOYSA-N", "MCM:C112OOH", "")
  CALL Species("C_{11}H_{20}O_3",                              "", "VJVDGYIYRBYKOQ-UHFFFAOYSA-N", "MCM:C113CO", "")
  CALL Species("C_{11}H_{22}O_3",                              "", "IBMABDXBRFZEGO-UHFFFAOYSA-N", "MCM:C113OH", "")
  CALL Species("C_{11}H_{22}O_4",                              "", "PBWNXGFXFCLKSE-UHFFFAOYSA-N", "MCM:C113OOH", "")
  CALL Species("C_{11}H_{20}O_3",                              "", "UGVQLPIEIICAJP-UHFFFAOYSA-N", "MCM:C114OH", "")
  CALL Species("C_{11}H_{16}O_4",                              "", "QNKVJNYVFVQVHN-UHFFFAOYSA-N", "MCM:DETLOBPROH", "")
  CALL Species("C_{11}H_{22}O_2",                              "", "HEIFQPPUFKWMSE-UHFFFAOYSA-N", "MCM:HO6CO9C11", "")
  CALL Species("C_{12}H_{24}O_4",                              "", "PWTYXUGOJRDLPW-UHFFFAOYSA-N", "MCM:C122OOH", "")
  CALL Species("C_{12}H_{22}O_3",                              "", "IVLDSNPYDYHHKW-UHFFFAOYSA-N", "MCM:C123CO", "")
  CALL Species("C_{12}H_{24}O_3",                              "", "FTSJAGYKUWYWCN-UHFFFAOYSA-N", "MCM:C123OH", "")
  CALL Species("C_{12}H_{24}O_4",                              "", "YANZHJXFRFYAEY-UHFFFAOYSA-N", "MCM:C123OOH", "")
  CALL Species("C_{12}H_{22}O_3",                              "", "ABVYFIKKAQHKSN-UHFFFAOYSA-N", "MCM:C124OH", "")
  CALL Species("C_{12}H_{24}O_2",                              "", "AJDNADCNQXGLCC-UHFFFAOYSA-N", "MCM:HO7CO10C12", "")
  CALL Species("C_{13}H_{22}O_3",                              "", "AONUZHGLRYBQFR-UHFFFAOYSA-N", "MCM:C131OH", "")
  CALL Species("C_{13}H_{22}O_4",                              "", "FIXMRMMYQTVSPU-UHFFFAOYSA-N", "MCM:C132OH", "")
  CALL Species("C_{13}H_{22}O_5",                              "", "VUANIPKMYHABIK-UHFFFAOYSA-N", "MCM:C132OOH", "")
  CALL Species("C_{13}H_{20}O_5",                              "", "NSPNCPQPMIIWRH-UHFFFAOYSA-N", "MCM:C133CO", "")
  CALL Species("C_{13}H_{22}O_5",                              "", "PJPMMZIOBGUFRQ-UHFFFAOYSA-N", "MCM:C133OH", "")
  CALL Species("C_{13}H_{22}O_6",                              "", "TWNMHUJEMNHLMV-UHFFFAOYSA-N", "MCM:C133OOH", "")
  CALL Species("C_{13}H_{18}O_6",                              "", "XVPKZNMZKWIEGC-UHFFFAOYSA-N", "MCM:C134CO", "")
  CALL Species("C_{13}H_{20}O_6",                              "", "CKMYGJKMMJWXII-UHFFFAOYSA-N", "MCM:C134OH", "")
  CALL Species("C_{13}H_{20}O_7",                              "", "RYVAPXIZXOQMSU-UHFFFAOYSA-N", "MCM:C134OOH", "")
  CALL Species("C_{13}H_{18}O_8",                              "", "VAECXYXKQRKKGE-UHFFFAOYSA-N", "MCM:C135OOH", "")
  CALL Species("C_{14}H_{24}O_3",                              "", "ARSAOHGCYSGONM-UHFFFAOYSA-N", "MCM:BCKAOH", "")
  CALL Species("C_{14}H_{24}O_4",                              "", "OXLVMLPVWDEZIL-UHFFFAOYSA-N", "MCM:BCKAOOH", "")
  CALL Species("C_{14}H_{22}O_3",                              "", "IWYBCVOMQFEOMA-UHFFFAOYSA-N", "MCM:BCKBCO", "")
  CALL Species("C_{14}H_{24}O_4",                              "", "SISQWITWYDFXLW-UHFFFAOYSA-N", "MCM:BCKBOOH", "")
  CALL Species("C_{14}H_{24}O_2",                              "", "OMNNANPOKVZFEY-UHFFFAOYSA-N", "MCM:C141OH", "")
  CALL Species("C_{14}H_{24}O_3",                              "", "TZIWCZUVZZOIAP-UHFFFAOYSA-N", "MCM:C142OH", "")
  CALL Species("C_{14}H_{24}O_4",                              "", "OJVOLXXWTGQGEJ-UHFFFAOYSA-N", "MCM:C142OOH", "")
  CALL Species("C_{14}H_{22}O_4",                              "", "IPHIBGAHOYIGTO-UHFFFAOYSA-N", "MCM:C143CO", "")
  CALL Species("C_{14}H_{24}O_4",                              "", "PUMKKTACSBJGTI-UHFFFAOYSA-N", "MCM:C143OH", "")
  CALL Species("C_{14}H_{24}O_5",                              "", "FMXAUTXFPHKMFA-UHFFFAOYSA-N", "MCM:C143OOH", "")
  CALL Species("C_{14}H_{24}O_6",                              "", "CADDZNHIARUDOF-UHFFFAOYSA-N", "MCM:C145OH", "")
  CALL Species("C_{14}H_{24}O_7",                              "", "CEBXMSFHIDLJKH-UHFFFAOYSA-N", "MCM:C145OOH", "")
  CALL Species("C_{15}H_{24}O_2",                              "", "VLNGGDKMXDHPMK-UHFFFAOYSA-N", "MCM:BCBCO", "")
  CALL Species("C_8H_8O_2",                                    "", "ZWVHTXAYIKBMEE-UHFFFAOYSA-N", "MCM:PHCOMEOH", "")
  CALL Species("C_9H_{10}O_2",                                 "", "CEJINNSYZFLSCS-UHFFFAOYSA-N", "MCM:MPHCOMEOH", "")
  CALL Species("C_9H_{10}O_2",                                 "", "WLVPRARCUSRDNI-UHFFFAOYSA-N", "MCM:PHCOETOH", "")
  CALL Species("C_{10}H_{12}O_2",                              "", "DBYMZNOFJHQFFD-UHFFFAOYSA-N", "MCM:DMPHCOMOH", "")
  CALL Species("C_{11}H_{14}O_2",                              "", "MHJXTDDYZBYNEF-UHFFFAOYSA-N", "MCM:EMPHCOMOH", "")
  CALL Species("C_3H_4O_4",                                    "", "RQBGWFCHIWSUOK-UHFFFAOYSA-N", "MCM:ALCOCH2OOH", "")
  CALL Species("C_3H_2O_3",                                    "", "ICQNCHSXWNQIHC-UHFFFAOYSA-N", "MCM:C33CO", "")
  CALL Species("C_4H_4O_5",                                    "", "GUGBJTDAGNLAJO-UHFFFAOYSA-N", "MCM:C312COCO3H", "")
  CALL Species("C_4H_6O_4",                                    "", "XUKQQLOKOLCMAI-UHFFFAOYSA-N", "MCM:C413COOOH", "")
  CALL Species("C_4H_6O_4",                                    "", "NOBUEMWCPQFCCN-UHFFFAOYSA-N", "MCM:C4CO2OOH", "")
  CALL Species("C_4H_4O_3",                                    "", "OWIMZZMOFXSCLT-UHFFFAOYSA-N", "MCM:C4CODIAL", "")
  CALL Species("C_4H_4O_3",                                    "", "LVZOZHOAAHWEOQ-UHFFFAOYSA-N", "MCM:CO23C3CHO", "")
  CALL Species("C_4H_6O_2",                                    "", "PKQIDSVLSKFZQC-UHFFFAOYSA-N", "MCM:CO2C3CHO", "")
  CALL Species("C_4H_2O_4",                                    "", "OURVMEXCDVXLPR-UHFFFAOYSA-N", "MCM:CO2C4DIAL", "")
  CALL Species("C_4H_6O_2",                                    "", "RWHQMRRVZJSKGX-UHFFFAOYSA-N", "MCM:EGLYOX", "")
  CALL Species("C_4H_4O_2",                                    "", "SDQVYUNYDAWYIK-UHFFFAOYSA-N", "MCM:VGLYOX", "")
  CALL Species("C_5H_4O_5",                                    "", "AXIVSVGHKYPZBE-UHFFFAOYSA-N", "MCM:C4CO2DCO3H", "")
  CALL Species("C_5H_8O_4",                                    "", "RKGLOUQDKXPRQL-UHFFFAOYSA-N", "MCM:C4MCO2OOH", "")
  CALL Species("C_5H_8O_4",                                    "", "NBNOBTZXOPWFTP-UHFFFAOYSA-N", "MCM:C511OOH", "")
  CALL Species("C_5H_6O_5",                                    "", "BXDCDQOWFJMSCY-UHFFFAOYSA-N", "MCM:C5124COOOH", "")
  CALL Species("C_5H_8O_4",                                    "", "UGOJGQBYBBLKDX-UHFFFAOYSA-N", "MCM:C512OOH", "")
  CALL Species("C_5H_4O_4",                                    "", "OONLUXUUWPUJOU-UHFFFAOYSA-N", "MCM:C515CO", "")
  CALL Species("C_5H_6O_5",                                    "", "KCJJHAINVUWKSI-UHFFFAOYSA-N", "MCM:C515OOH", "")
  CALL Species("C_5H_4O_4",                                    "", "KDLDWLXQVYMIBX-UHFFFAOYSA-N", "MCM:C54CO", "")
  CALL Species("C_5H_6O_3",                                    "", "IPHREVVCPNNUEQ-UHFFFAOYSA-N", "MCM:C5CO23CHO", "")
  CALL Species("C_5H_8O_4",                                    "", "QJMOTDLHJFXUFN-UHFFFAOYSA-N", "MCM:C5COCHOOOH", "")
  CALL Species("C_5H_4O_3",                                    "", "SFSCHQJUYKUKJM-UHFFFAOYSA-N", "MCM:C5DIALCO", "")
  CALL Species("C_5H_6O_2",                                    "", "GBLMMVFQENXAFZ-UHFFFAOYSA-N", "MCM:C5DICARB", "")
  CALL Species("C_5H_6O_5",                                    "", "KFSMXRNQWYDJRM-UHFFFAOYSA-N", "MCM:CHOC3COOOH", "")
  CALL Species("C_5H_6O_3",                                    "", "VUSMENNBCBGZQI-UHFFFAOYSA-N", "MCM:CO12C4CHO", "")
  CALL Species("C_5H_6O_3",                                    "", "SSMCSEDSANTCBB-UHFFFAOYSA-N", "MCM:CO13C4CHO", "")
  CALL Species("C_5H_6O_3",                                    "", "QGGCMCHYJIPQCE-UHFFFAOYSA-N", "MCM:CO23C4CHO", "")
  CALL Species("C_5H_6O_3",                                    "", "XHTYNUBGBUBFOA-UHFFFAOYSA-N", "MCM:CO24C4CHO", "")
  CALL Species("C_5H_8O_2",                                    "", "KNTLTMLEQPLVDA-UHFFFAOYSA-N", "MCM:CO2C43CHO", "")
  CALL Species("C_5H_8O_2",                                    "", "KEHNRUNQZGRQHU-UHFFFAOYSA-N", "MCM:CO2C4CHO", "")
  CALL Species("C_5H_8O_2",                                    "", "ZNNXJRURXWWGLN-UHFFFAOYSA-N", "MCM:CO3C4CHO", "")
  CALL Species("C_5H_8O_2",                                    "", "FTDZDHIBENIBKZ-UHFFFAOYSA-N", "MCM:IPRGLYOX", "")
  CALL Species("C_5H_8O_2",                                    "", "GDTHVMAIBQVUMV-UHFFFAOYSA-N", "MCM:PGLYOX", "")
  CALL Species("C_6H_8O_3",                                    "", "CBIOWOAHGWGTSI-UHFFFAOYSA-N", "MCM:C23C54CHO", "")
  CALL Species("C_6H_{10}O_2",                                 "", "PVLKSJIUOKAUMV-UHFFFAOYSA-N", "MCM:C3COCCHO", "")
  CALL Species("C_6H_8O_3",                                    "", "WETHXHAOPNKQLZ-UHFFFAOYSA-N", "MCM:C45IC5CHO", "")
  CALL Species("C_6H_{10}O_4",                                 "", "CMRDMVGIWXQQFX-UHFFFAOYSA-N", "MCM:C4ECO2OOH", "")
  CALL Species("C_6H_8O_3",                                    "", "SLKPDJWOVVFRFW-UHFFFAOYSA-N", "MCM:C511CHO", "")
  CALL Species("C_6H_8O_5",                                    "", "PKVJBWZRBQBAMT-UHFFFAOYSA-N", "MCM:C511CO3H", "")
  CALL Species("C_6H_8O_5",                                    "", "OPLKNVAQLNTMAR-UHFFFAOYSA-N", "MCM:C512CO3H", "")
  CALL Species("C_6H_6O_4",                                    "", "JHVNJBQABGTRPH-UHFFFAOYSA-N", "MCM:C515CHO", "")
  CALL Species("C_6H_6O_6",                                    "", "XPLLKUAMAYNXAT-UHFFFAOYSA-N", "MCM:C515CO3H", "")
  CALL Species("C_6H_6O_5",                                    "", "DHHXAZIZMIZDQV-UHFFFAOYSA-N", "MCM:C5CO2DCO3H", "")
  CALL Species("C_6H_6O_5",                                    "", "GJOWGOOSLUXRQO-UHFFFAOYSA-N", "MCM:C5DCO2CO3H", "")
  CALL Species("C_6H_8O_2",                                    "", "PFLPZDBOPDYOJU-UHFFFAOYSA-N", "MCM:C5MDICARB", "")
  CALL Species("C_6H_6O_6",                                    "", "GGHLQXKHXYQGCE-UHFFFAOYSA-N", "MCM:C5TRONCO3H", "")
  CALL Species("C_6H_6O_3",                                    "", "RFTMILUWMDIPHH-UHFFFAOYSA-N", "MCM:C6125CO", "")
  CALL Species("C_6H_8O_5",                                    "", "HGYTXVUJMUREIP-UHFFFAOYSA-N", "MCM:C6135COOOH", "")
  CALL Species("C_6H_8O_5",                                    "", "DSJLQKNTJKMXRQ-UHFFFAOYSA-N", "MCM:C6145COOOH", "")
  CALL Species("C_6H_8O_4",                                    "", "MOFLPBJDWVVKPL-UHFFFAOYSA-N", "MCM:C615CO2OOH", "")
  CALL Species("C_6H_8O_3",                                    "", "BRRWJVYIMRBLKB-UHFFFAOYSA-N", "MCM:C615CO", "")
  CALL Species("C_6H_8O_5",                                    "", "RZNSBZRPHHAABM-UHFFFAOYSA-N", "MCM:C616OOH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "VPKXUGZDYIOAPR-UHFFFAOYSA-N", "MCM:C617OOH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "AMQFKFSLRYYJMS-UHFFFAOYSA-N", "MCM:C618OOH", "")
  CALL Species("C_6H_8O_5",                                    "", "UWWRYJPKFSIPFS-UHFFFAOYSA-N", "MCM:C620OOH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "DHTMLHKAZOWLFT-UHFFFAOYSA-N", "MCM:C626OOH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "LHKMAYQHAYSMQO-UHFFFAOYSA-N", "MCM:C65OOH", "")
  CALL Species("C_6H_8O_3",                                    "", "VIOZHXSHLWOADQ-UHFFFAOYSA-N", "MCM:C6CO134", "")
  CALL Species("C_6H_4O_4",                                    "", "OGFBTMOHTIWBJU-UHFFFAOYSA-N", "MCM:C6CO4DB", "")
  CALL Species("C_6H_8O_5",                                    "", "WRLXIVUJPLXKIG-UHFFFAOYSA-N", "MCM:C6COALCO3H", "")
  CALL Species("C_6H_{10}O_4",                                 "", "DQAWORJSQGOPLV-UHFFFAOYSA-N", "MCM:C6COCHOOOH", "")
  CALL Species("C_6H_8O_3",                                    "", "PAXFMQWEAJBIJB-UHFFFAOYSA-N", "MCM:C6CODIAL", "")
  CALL Species("C_6H_8O_2",                                    "", "GVKYFODEMNCLGS-UHFFFAOYSA-N", "MCM:C6DICARB", "")
  CALL Species("C_6H_6O_4",                                    "", "GLSSGXNNMIEKBR-UHFFFAOYSA-N", "MCM:CO123C5CHO", "")
  CALL Species("C_6H_6O_4",                                    "", "GPIFCHFPYCMSRY-UHFFFAOYSA-N", "MCM:CO235C5CHO", "")
  CALL Species("C_6H_8O_3",                                    "", "OQYGUGGUHGKPLC-UHFFFAOYSA-N", "MCM:CO24M3CHO", "")
  CALL Species("C_6H_8O_3",                                    "", "YEJDUVJLIVLCFZ-UHFFFAOYSA-N", "MCM:CO2C4GLYOX", "")
  CALL Species("C_6H_{10}O_2",                                 "", "IDAHIBMEKOEBRG-UHFFFAOYSA-N", "MCM:CO2C54CHO", "")
  CALL Species("C_6H_{10}O_2",                                 "", "HOOWKSPRVCRTJK-UHFFFAOYSA-N", "MCM:CO2M33CHO", "")
  CALL Species("C_6H_{10}O_2",                                 "", "GQHWESFTVGVFIA-UHFFFAOYSA-N", "MCM:CO2M3C4CHO", "")
  CALL Species("C_6H_8O_3",                                    "", "KXLMWINKDVAJKB-UHFFFAOYSA-N", "MCM:CO35C5CHO", "")
  CALL Species("C_6H_{10}O_2",                                 "", "HFHZCESKBNQESK-UHFFFAOYSA-N", "MCM:CO3C54CHO", "")
  CALL Species("C_6H_{10}O_2",                                 "", "KKOFYQBBUSZDKJ-UHFFFAOYSA-N", "MCM:CO3C5CHO", "")
  CALL Species("C_6H_8O_3",                                    "", "WXTRRXXKCNKUIA-UHFFFAOYSA-N", "MCM:CO45C5CHO", "")
  CALL Species("C_6H_6O_4",                                    "", "RGMUISDTMIPFQK-UHFFFAOYSA-N", "MCM:ECO4", "")
  CALL Species("C_6H_8O_2",                                    "", "PEBMIJGMJUFNQE-UHFFFAOYSA-N", "MCM:MC5DICARB", "")
  CALL Species("C_7H_8O_5",                                    "", "OAFSDIWBIPMWQP-UHFFFAOYSA-N", "MCM:C4DBM2CO3H", "")
  CALL Species("C_7H_{10}O_3",                                 "", "NFXFJRVNKKYMOX-UHFFFAOYSA-N", "MCM:C617CHO", "")
  CALL Species("C_7H_{10}O_5",                                 "", "GEJLCFNTSCJUHB-UHFFFAOYSA-N", "MCM:C617CO3H", "")
  CALL Species("C_7H_{10}O_5",                                 "", "BCVYZYXNMFZMHQ-UHFFFAOYSA-N", "MCM:C618CO3H", "")
  CALL Species("C_7H_{10}O_3",                                 "", "HBIBDTOPQGUCOQ-UHFFFAOYSA-N", "MCM:C626CHO", "")
  CALL Species("C_7H_{10}O_5",                                 "", "XXGYBOYKTQFTKB-UHFFFAOYSA-N", "MCM:C626CO3H", "")
  CALL Species("C_7H_{10}O_4",                                 "", "BPEPZMFTCHTDHQ-UHFFFAOYSA-N", "MCM:C6M5CO2OOH", "")
  CALL Species("C_7H_{10}O_4",                                 "", "MXRQCKDWMJYUGS-UHFFFAOYSA-N", "MCM:C715CO2OOH", "")
  CALL Species("C_7H_{10}O_5",                                 "", "PDOSKCUHLZEIAA-UHFFFAOYSA-N", "MCM:C716OOH", "")
  CALL Species("C_7H_{10}O_5",                                 "", "PIJZITIWWOWVNF-UHFFFAOYSA-N", "MCM:C717OOH", "")
  CALL Species("C_7H_{12}O_4",                                 "", "IUOFUPZPJIEECL-UHFFFAOYSA-N", "MCM:C718OOH", "")
  CALL Species("C_7H_{12}O_4",                                 "", "GGMJSZWHPXYNJN-UHFFFAOYSA-N", "MCM:C731OOH", "")
  CALL Species("C_7H_{10}O_5",                                 "", "FKXMDVIITUJJPH-UHFFFAOYSA-N", "MCM:C735OOH", "")
  CALL Species("C_7H_8O_3",                                    "", "XIQVOMVDKSOEBZ-UHFFFAOYSA-N", "MCM:C736CO", "")
  CALL Species("C_7H_{10}O_4",                                 "", "FLZNSMFAMOSJOT-UHFFFAOYSA-N", "MCM:C736OOH", "")
  CALL Species("C_7H_{10}O_2",                                 "", "JOIPXRIQHWMPFU-UHFFFAOYSA-N", "MCM:C7ADICARB", "")
  CALL Species("C_7H_8O_3",                                    "", "YGMUVGSOGISXBZ-UHFFFAOYSA-N", "MCM:C7CO2DBAL", "")
  CALL Species("C_7H_8O_5",                                    "", "GSEZJGCNNFTSDI-UHFFFAOYSA-N", "MCM:C7CO2DCO3H", "")
  CALL Species("C_7H_6O_4",                                    "", "RYPSCZRULGRFHH-UHFFFAOYSA-N", "MCM:C7CO4DB", "")
  CALL Species("C_7H_{10}O_2",                                 "", "SRVPSCZJUZZLCY-UHFFFAOYSA-N", "MCM:C7DDICARB", "")
  CALL Species("C_7H_{10}O_2",                                 "", "GLRVVKVYRCSDLF-UHFFFAOYSA-N", "MCM:C7DICARB", "")
  CALL Species("C_7H_8O_4",                                    "", "DKBJFPXQZAZJBK-UHFFFAOYSA-N", "MCM:CO235C6CHO", "")
  CALL Species("C_7H_{10}O_3",                                 "", "ZTMYUKBUAYLBJN-UHFFFAOYSA-N", "MCM:CO25C6CHO", "")
  CALL Species("C_7H_{10}O_2",                                 "", "XQQMMKHYZYQABV-UHFFFAOYSA-N", "MCM:IC7DICARB", "")
  CALL Species("C_7H_{10}O_4",                                 "", "HBFXSTFRJMUUSF-UHFFFAOYSA-N", "MCM:MC6CO2OOH", "")
  CALL Species("C_8H_{10}O_5",                                 "", "MRKOFGXLZVJMLJ-UHFFFAOYSA-N", "MCM:C4DBMECO3H", "")
  CALL Species("C_8H_8O_4",                                    "", "XSTUBABFSAZOJC-UHFFFAOYSA-N", "MCM:C6CO4M2DB", "")
  CALL Species("C_8H_8O_4",                                    "", "OAQBTICOMUFUNE-UHFFFAOYSA-N", "MCM:C6ETCO4DB", "")
  CALL Species("C_8H_{12}O_5",                                 "", "TVOCFMMHLDKVAZ-UHFFFAOYSA-N", "MCM:C718CO3H", "")
  CALL Species("C_8H_{12}O_5",                                 "", "NTZMYPVPFVPPPF-UHFFFAOYSA-N", "MCM:C731CO3H", "")
  CALL Species("C_8H_{12}O_4",                                 "", "SYXNCLOQMRSNCQ-UHFFFAOYSA-N", "MCM:C7M6CO2OOH", "")
  CALL Species("C_8H_{10}O_3",                                 "", "PDQBXAJOXSGWHJ-UHFFFAOYSA-N", "MCM:C7MCO2DBAL", "")
  CALL Species("C_8H_{12}O_4",                                 "", "IVJZKSXAPVTQOQ-UHFFFAOYSA-N", "MCM:C7MCO2OOH", "")
  CALL Species("C_8H_{12}O_4",                                 "", "BDUHFTDKOJODDX-UHFFFAOYSA-N", "MCM:C815CO2OOH", "")
  CALL Species("C_8H_{12}O_3",                                 "", "WSCYQCAMZDXSLH-UHFFFAOYSA-N", "MCM:C817CO", "")
  CALL Species("C_8H_{12}O_2",                                 "", "MHDCXCKIQVLVLU-UHFFFAOYSA-N", "MCM:C830CO", "")
  CALL Species("C_8H_{12}O_3",                                 "", "ZUSGUVUFJOBUST-UHFFFAOYSA-N", "MCM:C831CO", "")
  CALL Species("C_8H_{14}O_4",                                 "", "DHFNMJQLYJIWHS-UHFFFAOYSA-N", "MCM:C86OOH", "")
  CALL Species("C_8H_{10}O_4",                                 "", "ZGSYEZWRFTZRGE-UHFFFAOYSA-N", "MCM:C87CO", "")
  CALL Species("C_8H_{12}O_5",                                 "", "ZOZQVMIZJFBYCH-UHFFFAOYSA-N", "MCM:C87OOH", "")
  CALL Species("C_8H_{10}O_3",                                 "", "WFUAEUZWLUGFPU-UHFFFAOYSA-N", "MCM:C8CO2DBAL", "")
  CALL Species("C_8H_{10}O_5",                                 "", "AFZRTBPBEHNFPV-UHFFFAOYSA-N", "MCM:C8CO2DCO3H", "")
  CALL Species("C_8H_8O_4",                                    "", "XMBXSFIUGNBAMW-UHFFFAOYSA-N", "MCM:C8CO4DB", "")
  CALL Species("C_8H_{10}O_5",                                 "", "LSJULIYFTAQYGE-UHFFFAOYSA-N", "MCM:C8DCO2CO3H", "")
  CALL Species("C_8H_8O_4",                                    "", "UUVQUVHMBIAKFN-UHFFFAOYSA-N", "MCM:CO4DBC8", "")
  CALL Species("C_8H_{12}O_4",                                 "", "XEFYLFAPQOHXQI-UHFFFAOYSA-N", "MCM:MC7CO2OOH", "")
  CALL Species("C_9H_{10}O_4",                                 "", "KWNUZJFJQSMOTR-UHFFFAOYSA-N", "MCM:C6CO4MEDB", "")
  CALL Species("C_9H_{10}O_4",                                 "", "PHOIWTKCTUWXFI-UHFFFAOYSA-N", "MCM:C6IPRCO4DB", "")
  CALL Species("C_9H_{10}O_4",                                 "", "ZAOQHXFQWRKJIE-UHFFFAOYSA-N", "MCM:C6PRCO4DB", "")
  CALL Species("C_9H_{10}O_4",                                 "", "ZLFZFHOVUMHREF-UHFFFAOYSA-N", "MCM:C7CO4EDB", "")
  CALL Species("C_9H_{10}O_4",                                 "", "AWHQTSPUFRSDOO-UHFFFAOYSA-N", "MCM:C7ECO4DB", "")
  CALL Species("C_9H_{14}O_3",                                 "", "WRGXNTJIWBXAOX-UHFFFAOYSA-N", "MCM:C827CHO", "")
  CALL Species("C_9H_{12}O_4",                                 "", "YZUHPZZEPOWVFM-UHFFFAOYSA-N", "MCM:C828CHO", "")
  CALL Species("C_9H_{12}O_6",                                 "", "AJHPLULEOIFIGV-UHFFFAOYSA-N", "MCM:C87CO3H", "")
  CALL Species("C_9H_{12}O_3",                                 "", "NXCXUGLKKPDWEC-UHFFFAOYSA-N", "MCM:C88CHO", "")
  CALL Species("C_9H_{10}O_4",                                 "", "RDBINALOBQAVHT-UHFFFAOYSA-N", "MCM:C914CO", "")
  CALL Species("C_9H_{12}O_5",                                 "", "ZERYOMDBUHTQAV-UHFFFAOYSA-N", "MCM:C914OOH", "")
  CALL Species("C_9H_{14}O_4",                                 "", "IHUGTHJLMVJZBZ-UHFFFAOYSA-N", "MCM:C915OOH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "FURCCROLNKVFJP-UHFFFAOYSA-N", "MCM:C916OOH", "")
  CALL Species("C_9H_{14}O_4",                                 "", "RVRGXPTVZZTPQO-UHFFFAOYSA-N", "MCM:C918OOH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "HGDKSMVLGYFVMB-UHFFFAOYSA-N", "MCM:C919OOH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "YGLJXDYJGXWZRQ-UHFFFAOYSA-N", "MCM:C926OOH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "UQPOFUKCKANZIY-UHFFFAOYSA-N", "MCM:C930OOH", "")
  CALL Species("C_9H_{10}O_4",                                 "", "KCIAXTRCVKQJBW-UHFFFAOYSA-N", "MCM:CO4DBC9", "")
  CALL Species("C_9H_{12}O_4",                                 "", "JZUAMOXWRDMCCH-UHFFFAOYSA-N", "MCM:LMLKACO", "")
  CALL Species("C_9H_{14}O_5",                                 "", "VTXNRXDJNZTEJV-UHFFFAOYSA-N", "MCM:LMLKAOOH", "")
  CALL Species("C_9H_{12}O_4",                                 "", "UMGHNOQMPCKCSD-UHFFFAOYSA-N", "MCM:LMLKBCO", "")
  CALL Species("C_9H_{14}O_5",                                 "", "YVDVZIZUYUAKKJ-UHFFFAOYSA-N", "MCM:LMLKBOOH", "")
  CALL Species("C_9H_{14}O_3",                                 "", "CWEQHJLFCKMWEE-UHFFFAOYSA-N", "MCM:LMLKET", "")
  CALL Species("C_9H_{14}O_2",                                 "", "WPNOYHDEEVPEJH-UHFFFAOYSA-N", "MCM:NORLIMAL", "")
  CALL Species("C_9H_{14}O_2",                                 "", "CRBGTXDFEDFKSU-UHFFFAOYSA-N", "MCM:NORPINAL", "")
  CALL Species("C_{10}H_{16}O_4",                              "", "PQCOZMUQYGSZBV-UHFFFAOYSA-N", "MCM:C1010OOH", "")
  CALL Species("C_{10}H_{16}O_3",                              "", "ZTXNVHZVZBDWRG-UHFFFAOYSA-N", "MCM:C1012CO", "")
  CALL Species("C_{10}H_{18}O_4",                              "", "NQAGNHJKHGEOIP-UHFFFAOYSA-N", "MCM:C1012OOH", "")
  CALL Species("C_{10}H_{16}O_5",                              "", "QENSMPURVPBEBQ-UHFFFAOYSA-N", "MCM:C106OOH", "")
  CALL Species("C_{10}H_{16}O_4",                              "", "JYIMELQJWXRSLA-UHFFFAOYSA-N", "MCM:C107OOH", "")
  CALL Species("C_{10}H_{16}O_5",                              "", "AIXQDYAMZUITKH-UHFFFAOYSA-N", "MCM:C108OOH", "")
  CALL Species("C_{10}H_{14}O_3",                              "", "GPWOGFGLGKGEOT-UHFFFAOYSA-N", "MCM:C109CO", "")
  CALL Species("C_{10}H_{16}O_4",                              "", "ODTCGEGBAGAWAJ-UHFFFAOYSA-N", "MCM:C109OOH", "")
  CALL Species("C_{10}H_{14}O_3",                              "", "OXUPJDFJWCMZMO-UHFFFAOYSA-N", "MCM:LIMALACO", "")
  CALL Species("C_{10}H_{16}O_4",                              "", "DVORBFWFCUOYLW-UHFFFAOYSA-N", "MCM:LIMALAOOH", "")
  CALL Species("C_{10}H_{14}O_3",                              "", "OFCYPGCYWHDSNB-UHFFFAOYSA-N", "MCM:LIMALBCO", "")
  CALL Species("C_{10}H_{16}O_4",                              "", "DCWLKDSHBWMMQU-UHFFFAOYSA-N", "MCM:LIMALBOOH", "")
  CALL Species("C_{10}H_{16}O_2",                              "", "OGCGCISRMFSLTC-UHFFFAOYSA-N", "MCM:LIMAL", "")
  CALL Species("C_{10}H_{16}O_2",                              "", "GCHDWVBHKDJOKU-UHFFFAOYSA-N", "MCM:PINAL", "")
  CALL Species("C_{10}H_{16}O_4",                              "", "IKYOAMQPSDRUHL-UHFFFAOYSA-N", "MCM:PINALOOH", "")
  CALL Species("C_{11}H_{16}O_3",                              "", "XGVKCXDPIYHMTH-UHFFFAOYSA-N", "MCM:C116CO", "")
  CALL Species("C_{11}H_{18}O_4",                              "", "GLQKGYMROFUHSE-UHFFFAOYSA-N", "MCM:C116OOH", "")
  CALL Species("C_{12}H_{18}O_3",                              "", "LCDZSXGLJAAQMI-UHFFFAOYSA-N", "MCM:C116CHO", "")
  CALL Species("C_{12}H_{18}O_5",                              "", "DYOBAPPYFCSOAS-UHFFFAOYSA-N", "MCM:C116CO3H", "")
  CALL Species("C_{12}H_{20}O_4",                              "", "KZRFFJDVJJJBDC-UHFFFAOYSA-N", "MCM:C1210OOH", "")
  CALL Species("C_{12}H_{16}O_4",                              "", "RGVZWLUABXGGBI-UHFFFAOYSA-N", "MCM:C129CO", "")
  CALL Species("C_{12}H_{18}O_5",                              "", "OVAMHHASBHCMIZ-UHFFFAOYSA-N", "MCM:C129OOH", "")
  CALL Species("C_{13}H_{20}O_5",                              "", "BXGKTBFSJURUHS-UHFFFAOYSA-N", "MCM:C1210CO3H", "")
  CALL Species("C_{13}H_{20}O_3",                              "", "FDPPFMUIMWYEIX-UHFFFAOYSA-N", "MCM:C131CO", "")
  CALL Species("C_{14}H_{20}O_4",                              "", "ARRZIWJBJSIOKE-UHFFFAOYSA-N", "MCM:BCLKACO", "")
  CALL Species("C_{14}H_{22}O_5",                              "", "CKBNEDHFWOQWQD-UHFFFAOYSA-N", "MCM:BCLKAOOH", "")
  CALL Species("C_{14}H_{20}O_4",                              "", "NZHCLRCYZNBCMK-UHFFFAOYSA-N", "MCM:BCLKBCO", "")
  CALL Species("C_{14}H_{22}O_5",                              "", "CNYMBWKCKYTSRZ-UHFFFAOYSA-N", "MCM:BCLKBOOH", "")
  CALL Species("C_{14}H_{20}O_4",                              "", "RVZWWCAHYGTHHX-UHFFFAOYSA-N", "MCM:BCLKCCO", "")
  CALL Species("C_{14}H_{22}O_5",                              "", "OVPKMRRDRUAGIL-UHFFFAOYSA-N", "MCM:BCLKCOOH", "")
  CALL Species("C_{14}H_{22}O_3",                              "", "UGDRYISIDBCIAI-UHFFFAOYSA-N", "MCM:BCLKET", "")
  CALL Species("C_{14}H_{22}O_2",                              "", "LUPUYUXNGLIEMW-UHFFFAOYSA-N", "MCM:C141CO", "")
  CALL Species("C_{14}H_{20}O_4",                              "", "ODTZTSYYOIXMEF-UHFFFAOYSA-N", "MCM:C146CO", "")
  CALL Species("C_{14}H_{22}O_5",                              "", "SPXXDEIFDYFSMG-UHFFFAOYSA-N", "MCM:C146OOH", "")
  CALL Species("C_{15}H_{22}O_3",                              "", "ULXUQSOGQJAAAV-UHFFFAOYSA-N", "MCM:BCALACO", "")
  CALL Species("C_{15}H_{24}O_4",                              "", "CATQWYAHYHBBFL-UHFFFAOYSA-N", "MCM:BCALAOOH", "")
  CALL Species("C_{15}H_{22}O_3",                              "", "DVBAGSYXLVWTHM-UHFFFAOYSA-N", "MCM:BCALBCO", "")
  CALL Species("C_{15}H_{24}O_4",                              "", "BXDTVEFUTYSWEH-UHFFFAOYSA-N", "MCM:BCALBOOH", "")
  CALL Species("C_{15}H_{24}O_2",                              "", "PJJCDGMFEIUQRZ-UHFFFAOYSA-N", "MCM:BCAL", "")
  CALL Species("C_{15}H_{22}O_3",                              "", "RMGPJKLPPRXORI-UHFFFAOYSA-N", "MCM:BCALCCO", "")
  CALL Species("C_{15}H_{24}O_4",                              "", "WQOUANLPJNTVBV-UHFFFAOYSA-N", "MCM:BCALCOOH", "")
  CALL Species("C_8H_6O_2",                                    "", "OJUGVDODNPJEEC-UHFFFAOYSA-N", "MCM:PHGLYOX", "")
  CALL Species("C_9H_8O_2",                                    "", "ILRFLXHICGHRIN-UHFFFAOYSA-N", "MCM:MPHGLYOX", "")
  CALL Species("C_{10}H_{10}O_2",                              "", "COKLAMNWIKZGIN-UHFFFAOYSA-N", "MCM:DMPHGLYOX", "")
  CALL Species("C_{11}H_{12}O_2",                              "", "LRURTQBJFCJOFP-UHFFFAOYSA-N", "MCM:EMPHGLYOX", "")
  CALL Species("C_3H_4O_3",                                    "", "JLPAWRLRMTZCSF-UHFFFAOYSA-N", "MCM:HOCH2COCHO", "")
  CALL Species("C_4H_6O_3",                                    "", "AWMGPEHDHJUMEN-UHFFFAOYSA-N", "MCM:CCOCOCOH", "")
  CALL Species("C_4H_6O_3",                                    "", "KATSYVMPKZTIRW-UHFFFAOYSA-N", "MCM:CO13C4OH", "")
  CALL Species("C_4H_6O_3",                                    "", "GTYVZUSWKGYETP-UHFFFAOYSA-N", "MCM:CO2H3CHO", "")
  CALL Species("C_4H_6O_4",                                    "", "RAPYKXPBVBNJGC-UHFFFAOYSA-N", "MCM:H13CO2CHO", "")
  CALL Species("C_4H_4O_4",                                    "", "XSEYITBSKRUSNL-UHFFFAOYSA-N", "MCM:H1CO23CHO", "")
  CALL Species("C_4H_6O_3",                                    "", "CUSSNCHZLYDUPJ-UHFFFAOYSA-N", "MCM:HO1CO3CHO", "")
  CALL Species("C_4H_4O_4",                                    "", "JWBVDBDNUQOADG-UHFFFAOYSA-N", "MCM:HOCOC4DIAL", "")
  CALL Species("C_5H_8O_3",                                    "", "YKNUMNAVGAMCFT-UHFFFAOYSA-N", "MCM:C4MCO2OH", "")
  CALL Species("C_5H_8O_3",                                    "", "ZSJOVUGBPPFONP-UHFFFAOYSA-N", "MCM:C512OH", "")
  CALL Species("C_5H_6O_4",                                    "", "UFXCMYYHMGNRSQ-UHFFFAOYSA-N", "MCM:C5134CO2OH", "")
  CALL Species("C_5H_6O_4",                                    "", "GBFAZHBQOBSKET-UHFFFAOYSA-N", "MCM:C513CO", "")
  CALL Species("C_5H_8O_4",                                    "", "VLANIYKAIBIISW-UHFFFAOYSA-N", "MCM:C513OH", "")
  CALL Species("C_5H_8O_5",                                    "", "FJRVRXOZJCMNIT-UHFFFAOYSA-N", "MCM:C513OOH", "")
  CALL Species("C_5H_8O_4",                                    "", "JLVNMCMGUWUGCO-UHFFFAOYSA-N", "MCM:C514CO23OH", "")
  CALL Species("C_5H_8O_4",                                    "", "BXHDMMHJHUOSNS-UHFFFAOYSA-N", "MCM:C520OH", "")
  CALL Species("C_5H_8O_5",                                    "", "BCWSZSNCYSGOQJ-UHFFFAOYSA-N", "MCM:C520OOH", "")
  CALL Species("C_5H_6O_4",                                    "", "UUUQEQKLOQQXBA-UHFFFAOYSA-N", "MCM:C5CO3OH", "")
  CALL Species("C_5H_8O_5",                                    "", "SXCXUVJXNGWYKY-UHFFFAOYSA-N", "MCM:C5DICAROOH", "")
  CALL Species("C_5H_8O_3",                                    "", "GHHQRUFNLVFZPQ-UHFFFAOYSA-N", "MCM:CO3H4CHO", "")
  CALL Species("C_5H_6O_4",                                    "", "VBLXHDGHHJBCMK-UHFFFAOYSA-N", "MCM:H1C23C4CHO", "")
  CALL Species("C_5H_8O_3",                                    "", "XYGGNMZYIGPNCC-UHFFFAOYSA-N", "MCM:HMVKBCHO", "")
  CALL Species("C_5H_8O_3",                                    "", "NSUJYXKDWVMPFQ-UHFFFAOYSA-N", "MCM:HO2CO4CHO", "")
  CALL Species("C_5H_8O_3",                                    "", "ROKGRLJNHSUEOC-UHFFFAOYSA-N", "MCM:HOIPRGLYOX", "")
  CALL Species("C_5H_8O_5",                                    "", "KGZJHQJCDPZOFG-UHFFFAOYSA-N", "MCM:IEC2OOH", "")
  CALL Species("C_5H_8O_3",                                    "", "OCAJTNNCRSVJNU-UHFFFAOYSA-N", "MCM:MBOCOCO", "")
  CALL Species("C_6H_{10}O_3",                                 "", "NMZPCKZSNUOBSC-UHFFFAOYSA-N", "MCM:C43OHCOCHO", "")
  CALL Species("C_6H_{10}O_3",                                 "", "JGHPNZFTVGSLKF-UHFFFAOYSA-N", "MCM:C517CHO", "")
  CALL Species("C_6H_{10}O_3",                                 "", "PVAHQKAVPKMXGV-UHFFFAOYSA-N", "MCM:C519CHO", "")
  CALL Species("C_6H_6O_6",                                    "", "DETDOHNNOYOALO-UHFFFAOYSA-N", "MCM:C5COOHCO3H", "")
  CALL Species("C_6H_{10}O_3",                                 "", "YVTXQDAOLHXGGS-UHFFFAOYSA-N", "MCM:C5MO132OH", "")
  CALL Species("C_6H_8O_4",                                    "", "ZUSWTYLGBZLTMD-UHFFFAOYSA-N", "MCM:C6134CO2OH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "FTHGMBBEYTXWQT-UHFFFAOYSA-N", "MCM:C614CO23OH", "")
  CALL Species("C_6H_8O_3",                                    "", "AEWAJJBQFQHEAB-UHFFFAOYSA-N", "MCM:C615CO2OH", "")
  CALL Species("C_6H_8O_4",                                    "", "VJOSQGBCXQMKDE-UHFFFAOYSA-N", "MCM:C616OH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "GSQCYSIEWGMUPC-UHFFFAOYSA-N", "MCM:C617OH", "")
  CALL Species("C_6H_8O_4",                                    "", "SMLWDBHBBFROKP-UHFFFAOYSA-N", "MCM:C620OH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "XRZKKMNIWXFNJQ-UHFFFAOYSA-N", "MCM:C628OH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "BGLCDSUTRBZUMD-UHFFFAOYSA-N", "MCM:C628OOH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "MIXLBBAGKJLCSO-UHFFFAOYSA-N", "MCM:C629OH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "PARGUUHBZPMBOS-UHFFFAOYSA-N", "MCM:C629OOH", "")
  CALL Species("C_6H_8O_3",                                    "", "AUMGTZPDEFNUMP-UHFFFAOYSA-N", "MCM:C630CO", "")
  CALL Species("C_6H_{10}O_3",                                 "", "ZWRMEYSYDMGCIE-UHFFFAOYSA-N", "MCM:C65OH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "BZFZPOBVGBFEBG-UHFFFAOYSA-N", "MCM:C67CHO", "")
  CALL Species("C_6H_{10}O_4",                                 "", "ZFPKEBYORKZJID-UHFFFAOYSA-N", "MCM:C6CO2M2OH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "LMEOLDNWXYVMAA-UHFFFAOYSA-N", "MCM:C6CO2OHOOH", "")
  CALL Species("C_6H_8O_4",                                    "", "WBLLHOUAAGAGCV-UHFFFAOYSA-N", "MCM:C6CO3MOH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "HJNIDOQIVMJXHS-UHFFFAOYSA-N", "MCM:C6COHOCHO", "")
  CALL Species("C_6H_{10}O_5",                                 "", "HIILQMKIFXFEHC-UHFFFAOYSA-N", "MCM:C6DICAROOH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "GIDOBZXUXDNRDY-UHFFFAOYSA-N", "MCM:C6O132OH", "")
  CALL Species("C_6H_8O_6",                                    "", "CIXJOARRJVNUNZ-UHFFFAOYSA-N", "MCM:C6TONOHOOH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "MIIVJRGFVJVTGE-UHFFFAOYSA-N", "MCM:H2M3CO4CHO", "")
  CALL Species("C_6H_8O_4",                                    "", "CLPDOVMSDUESLY-UHFFFAOYSA-N", "MCM:H3C25C5CHO", "")
  CALL Species("C_6H_{10}O_3",                                 "", "UTYMQWKKMYQTHJ-UHFFFAOYSA-N", "MCM:HM22COCHO", "")
  CALL Species("C_6H_{10}O_4",                                 "", "LGFUKTGFFSPNKZ-UHFFFAOYSA-N", "MCM:MC52CO2OH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "XFHCIPDXRRXAPB-UHFFFAOYSA-N", "MCM:MC5COOHOOH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "OYDFBAKICJVUQJ-UHFFFAOYSA-N", "MCM:MIBKHO4CHO", "")
  CALL Species("C_7H_{10}O_4",                                 "", "BDFRSVKDVGXKLG-UHFFFAOYSA-N", "MCM:C61CHO", "")
  CALL Species("C_7H_{10}O_4",                                 "", "QCTBGQUOJXZEIH-UHFFFAOYSA-N", "MCM:C62CHO", "")
  CALL Species("C_7H_8O_6",                                    "", "BNWBZUAKPODUMK-UHFFFAOYSA-N", "MCM:C6COOHCO3H", "")
  CALL Species("C_7H_{10}O_3",                                 "", "YAAUSHWRBSRFSJ-UHFFFAOYSA-N", "MCM:C6M5CO2OH", "")
  CALL Species("C_7H_{10}O_3",                                 "", "KBBGTWJMXYAKIO-UHFFFAOYSA-N", "MCM:C715CO2OH", "")
  CALL Species("C_7H_{10}O_4",                                 "", "QFGSQPDLKNLDDS-UHFFFAOYSA-N", "MCM:C716OH", "")
  CALL Species("C_7H_{10}O_4",                                 "", "SEENFYUXRUQNHQ-UHFFFAOYSA-N", "MCM:C717OH", "")
  CALL Species("C_7H_{12}O_3",                                 "", "ALYGFZDZOCWXMF-UHFFFAOYSA-N", "MCM:C718OH", "")
  CALL Species("C_7H_{12}O_3",                                 "", "KHEBQEUBDOAJRG-UHFFFAOYSA-N", "MCM:C731OH", "")
  CALL Species("C_7H_{10}O_4",                                 "", "IRSIMXCKMCQRGW-UHFFFAOYSA-N", "MCM:C733CO", "")
  CALL Species("C_7H_{12}O_4",                                 "", "WRHSSXYTISGZIA-UHFFFAOYSA-N", "MCM:C733OH", "")
  CALL Species("C_7H_{12}O_5",                                 "", "OWEUBRYSVXGVIC-UHFFFAOYSA-N", "MCM:C733OOH", "")
  CALL Species("C_7H_{10}O_4",                                 "", "PHOAHMZZKPNPAM-UHFFFAOYSA-N", "MCM:C735OH", "")
  CALL Species("C_7H_{10}O_3",                                 "", "QWXLVLIZAXRZBR-UHFFFAOYSA-N", "MCM:C736OH", "")
  CALL Species("C_7H_{12}O_4",                                 "", "SVQINHOHTZDOCH-UHFFFAOYSA-N", "MCM:C7ADCOH", "")
  CALL Species("C_7H_{12}O_5",                                 "", "COQUITHNSRNVBJ-UHFFFAOYSA-N", "MCM:C7ADCOOH", "")
  CALL Species("C_7H_{10}O_6",                                 "", "YAEIVOJGWQGWPS-UHFFFAOYSA-N", "MCM:C7CO3OHOOH", "")
  CALL Species("C_7H_{12}O_4",                                 "", "VIPBNCDHLKDPDM-UHFFFAOYSA-N", "MCM:C7DCOH", "")
  CALL Species("C_7H_{12}O_5",                                 "", "AVRZSWZUKXCJRK-UHFFFAOYSA-N", "MCM:C7DCOOH", "")
  CALL Species("C_7H_{12}O_4",                                 "", "HYCGRSOMYJQICU-UHFFFAOYSA-N", "MCM:C7DDCOH", "")
  CALL Species("C_7H_{12}O_5",                                 "", "VSEOFSAKNXJKSZ-UHFFFAOYSA-N", "MCM:C7DDCOOH", "")
  CALL Species("C_7H_{12}O_4",                                 "", "DEDFQTYPJDABTI-UHFFFAOYSA-N", "MCM:IC7DCOH", "")
  CALL Species("C_7H_{12}O_5",                                 "", "ZVFWDEWSMKAAIM-UHFFFAOYSA-N", "MCM:IC7DCOOH", "")
  CALL Species("C_7H_{10}O_3",                                 "", "BNJUZVCUVNAPOM-UHFFFAOYSA-N", "MCM:MC6CO2OH", "")
  CALL Species("C_8H_{10}O_6",                                 "", "BRIBGOXGRALVGI-UHFFFAOYSA-N", "MCM:C5M2OHCO3H", "")
  CALL Species("C_8H_{12}O_3",                                 "", "OEJDBFGEAKTZGI-UHFFFAOYSA-N", "MCM:C7M15CO2OH", "")
  CALL Species("C_8H_{12}O_3",                                 "", "WXVOVBAGQNBDEE-UHFFFAOYSA-N", "MCM:C7M6CO2OH", "")
  CALL Species("C_8H_{12}O_6",                                 "", "UOUKTGYLQYQUAO-UHFFFAOYSA-N", "MCM:C7MO3OHOOH", "")
  CALL Species("C_8H_{10}O_6",                                 "", "AOHPTSIBUCEZIY-UHFFFAOYSA-N", "MCM:C7OHO2CO3H", "")
  CALL Species("C_8H_{12}O_3",                                 "", "IPCSMQASRIZASM-UHFFFAOYSA-N", "MCM:C815CO2OH", "")
  CALL Species("C_8H_{12}O_3",                                 "", "BQDQKENHFIFOJZ-UHFFFAOYSA-N", "MCM:C824CO", "")
  CALL Species("C_8H_{12}O_6",                                 "", "LBBMWNQOJHPRLQ-UHFFFAOYSA-N", "MCM:C83COOHOOH", "")
  CALL Species("C_8H_{12}O_4",                                 "", "DFOLUFIWIMPWQH-UHFFFAOYSA-N", "MCM:C87OH", "")
  CALL Species("C_8H_{12}O_3",                                 "", "BSMWBPAKWVVEKL-UHFFFAOYSA-N", "MCM:MC7CO2OH", "")
  CALL Species("C_9H_{12}O_6",                                 "", "IZKMPGKTEBWFPK-UHFFFAOYSA-N", "MCM:C5MEJCO3H", "")
  CALL Species("C_9H_{12}O_6",                                 "", "MBOWYQZNYXRQRC-UHFFFAOYSA-N", "MCM:C8O2OHCO3H", "")
  CALL Species("C_9H_{12}O_6",                                 "", "CJLPRBWVJFSMFP-UHFFFAOYSA-N", "MCM:C8OHO2CO3H", "")
  CALL Species("C_9H_{12}O_4",                                 "", "LFDXAZOJHXEEGS-UHFFFAOYSA-N", "MCM:C914OH", "")
  CALL Species("C_9H_{14}O_3",                                 "", "FAAQZCMVHKUUSB-UHFFFAOYSA-N", "MCM:C915OH", "")
  CALL Species("C_9H_{14}O_4",                                 "", "MXDMOPCDPNEDIT-UHFFFAOYSA-N", "MCM:C916OH", "")
  CALL Species("C_9H_{14}O_3",                                 "", "NFJZVEFTUBPOLY-UHFFFAOYSA-N", "MCM:C918OH", "")
  CALL Species("C_9H_{14}O_4",                                 "", "ULXJXURNPDAXQL-UHFFFAOYSA-N", "MCM:C919OH", "")
  CALL Species("C_9H_{14}O_4",                                 "", "DBPGQZSZWKEZHT-UHFFFAOYSA-N", "MCM:C926OH", "")
  CALL Species("C_9H_{14}O_4",                                 "", "LNIVXHBNNGOOPM-UHFFFAOYSA-N", "MCM:C930OH", "")
  CALL Species("C_9H_{14}O_4",                                 "", "XDMVMFNHBSPJDJ-UHFFFAOYSA-N", "MCM:LMLKAOH", "")
  CALL Species("C_9H_{14}O_4",                                 "", "JSAHIYWZESQWRA-UHFFFAOYSA-N", "MCM:LMLKBOH", "")
  CALL Species("C_9H_{16}O_5",                                 "", "CUMGKBRSRBORIJ-UHFFFAOYSA-N", "MCM:NORLIMOOH", "")
  CALL Species("C_{10}H_{16}O_3",                              "", "NQWGZMQLCPYJCB-UHFFFAOYSA-N", "MCM:C1010OH", "")
  CALL Species("C_{10}H_{18}O_3",                              "", "ZSBQFHQMMVQAKS-UHFFFAOYSA-N", "MCM:C1012OH", "")
  CALL Species("C_{10}H_{16}O_4",                              "", "LWIDJJPVYZTGLI-UHFFFAOYSA-N", "MCM:C106OH", "")
  CALL Species("C_{10}H_{16}O_3",                              "", "DUMRJPIWTJFZCT-UHFFFAOYSA-N", "MCM:C107OH", "")
  CALL Species("C_{10}H_{16}O_4",                              "", "ZDBVABFEYYIFLR-UHFFFAOYSA-N", "MCM:C108OH", "")
  CALL Species("C_{10}H_{16}O_3",                              "", "PUENXTSKKXKPMK-UHFFFAOYSA-N", "MCM:C109OH", "")
  CALL Species("C_{10}H_{16}O_3",                              "", "VNYSFQVUCPNMDZ-UHFFFAOYSA-N", "MCM:LIMALAOH", "")
  CALL Species("C_{10}H_{16}O_3",                              "", "SMXSHIIAZMJGAE-UHFFFAOYSA-N", "MCM:LIMALBOH", "")
  CALL Species("C_{10}H_{18}O_4",                              "", "ISUQULRVPDSRIP-UHFFFAOYSA-N", "MCM:LIMALOH", "")
  CALL Species("C_{10}H_{18}O_5",                              "", "OQLBLHLZVAPRGN-UHFFFAOYSA-N", "MCM:LIMALOOH", "")
  CALL Species("C_{10}H_{16}O_3",                              "", "NGFXWFWSYUSJMH-UHFFFAOYSA-N", "MCM:PINALOH", "")
  CALL Species("C_{11}H_{18}O_3",                              "", "JMGXHRKVWURTTA-UHFFFAOYSA-N", "MCM:C116OH", "")
  CALL Species("C_{11}H_{18}O_4",                              "", "XYBUENVNXDXQKB-UHFFFAOYSA-N", "MCM:C117OH", "")
  CALL Species("C_{11}H_{18}O_5",                              "", "NUIUIELMTMWFAB-UHFFFAOYSA-N", "MCM:C117OOH", "")
  CALL Species("C_{11}H_{16}O_5",                              "", "QFBAMJSHDLKQRE-UHFFFAOYSA-N", "MCM:C118CO", "")
  CALL Species("C_{11}H_{18}O_5",                              "", "GQNPBTACCNTXDZ-UHFFFAOYSA-N", "MCM:C118OH", "")
  CALL Species("C_{11}H_{18}O_6",                              "", "DZELKNYJHUANPY-UHFFFAOYSA-N", "MCM:C118OOH", "")
  CALL Species("C_{12}H_{20}O_3",                              "", "RDCPBXUNBHOYIW-UHFFFAOYSA-N", "MCM:C1210OH", "")
  CALL Species("C_{12}H_{20}O_4",                              "", "LBONAAUBRWCFLP-UHFFFAOYSA-N", "MCM:C1214OH", "")
  CALL Species("C_{12}H_{20}O_5",                              "", "RDWLNWMMPUJYPV-UHFFFAOYSA-N", "MCM:C1214OOH", "")
  CALL Species("C_{12}H_{18}O_5",                              "", "MFAMPSJMWGSBSP-UHFFFAOYSA-N", "MCM:C1215CO", "")
  CALL Species("C_{12}H_{20}O_5",                              "", "ICTVPTJMPURCGE-UHFFFAOYSA-N", "MCM:C1215OH", "")
  CALL Species("C_{12}H_{20}O_6",                              "", "PWIVJRAMQXZVCR-UHFFFAOYSA-N", "MCM:C1215OOH", "")
  CALL Species("C_{12}H_{18}O_4",                              "", "UGSIBTFJEGNWKQ-UHFFFAOYSA-N", "MCM:C128CO", "")
  CALL Species("C_{12}H_{20}O_4",                              "", "CFXREDIWINSYMD-UHFFFAOYSA-N", "MCM:C128OH", "")
  CALL Species("C_{12}H_{20}O_5",                              "", "UJLXMZVGYRRHNB-UHFFFAOYSA-N", "MCM:C128OOH", "")
  CALL Species("C_{12}H_{18}O_4",                              "", "LZLVFWPHHNPPPO-UHFFFAOYSA-N", "MCM:C129OH", "")
  CALL Species("C_{13}H_{20}O_4",                              "", "OXGUTNXNCKSGQJ-UHFFFAOYSA-N", "MCM:C1312CO", "")
  CALL Species("C_{13}H_{22}O_4",                              "", "IOOAKPNHTHIGJB-UHFFFAOYSA-N", "MCM:C1312OH", "")
  CALL Species("C_{13}H_{22}O_5",                              "", "IMCVJRKOQRMDFG-UHFFFAOYSA-N", "MCM:C1312OOH", "")
  CALL Species("C_{14}H_{22}O_4",                              "", "OIHLYIMTKSRRQE-UHFFFAOYSA-N", "MCM:BCLKAOH", "")
  CALL Species("C_{14}H_{22}O_4",                              "", "LFMWXXSOGPOZAN-UHFFFAOYSA-N", "MCM:BCLKBOH", "")
  CALL Species("C_{14}H_{22}O_4",                              "", "WUCWLUFQNHHRAC-UHFFFAOYSA-N", "MCM:BCLKCOH", "")
  CALL Species("C_{14}H_{24}O_5",                              "", "GLUIAMUUMIPXJY-UHFFFAOYSA-N", "MCM:C144OOH", "")
  CALL Species("C_{14}H_{22}O_4",                              "", "MRXVSIMJFRVVPA-UHFFFAOYSA-N", "MCM:C146OH", "")
  CALL Species("C_{15}H_{24}O_3",                              "", "SOIICZPFVOBBSJ-UHFFFAOYSA-N", "MCM:BCALAOH", "")
  CALL Species("C_{15}H_{24}O_3",                              "", "WYNIZHBHTSXQQC-UHFFFAOYSA-N", "MCM:BCALBOH", "")
  CALL Species("C_{15}H_{24}O_3",                              "", "NQPQIVDLCLYNNZ-UHFFFAOYSA-N", "MCM:BCALCOH", "")
  CALL Species("C_{15}H_{26}O_4",                              "", "MPEWUZJEETZOEB-UHFFFAOYSA-N", "MCM:BCALOH", "")
  CALL Species("C_{15}H_{26}O_5",                              "", "HSCKVTFOUFPNNB-UHFFFAOYSA-N", "MCM:BCALOOH", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Carboxylic acids (\chem{RCOOH}) and peroxy carboxylic acids (\chem{RCOOOH})")
  ! saturated:
  CALL Species("HCOOH",                                 "64-18-6", "BDAGIHXWWSANSR-UHFFFAOYSA-N", "methanoic acid", "formic acid")
  CALL Species("CH_3COOH",                              "64-19-7", "QTBSBXVTEAMEQO-UHFFFAOYSA-N", "ethanoic acid", "acetic acid")
  CALL Species("C_4H_6O_3",                            "108-24-7", "WFDIJRYMOXRFFG-UHFFFAOYSA-N", "acetic anhydride", "")
  CALL Species("C_2H_5COOH",                            "79-09-4", "XBDQKXXYIPTUBI-UHFFFAOYSA-N", "propanoic acid", "propionic acid")
  CALL Species("C_3H_6O_3",                            "625-45-6", "RMIODHQZRUFFFF-UHFFFAOYSA-N", "methoxyacetic acid", "")
  CALL Species("C_3H_7COOH",                           "107-92-6", "FERIUCNNQQJTOY-UHFFFAOYSA-N", "butanoic acid", "butyric acid")
  CALL Species("(CH_3)_2CHCOOH",                        "79-31-2", "KQNPFQTWMSNSAP-UHFFFAOYSA-N", "2-methylpropanoic acid", "isobutyric acid")
  CALL Species("C_4H_9COOH",                           "109-52-4", "NQPDZGIKBAWPEJ-UHFFFAOYSA-N", "pentanoic acid", "valeric acid")
  CALL Species("C_5H_{10}O_2",                         "116-53-0", "WLAMNBDJUVNPJU-UHFFFAOYSA-N", "2-methylbutanoic acid", "")
  CALL Species("(CH_3)_2CHCH_2COOH",                   "503-74-2", "GWYFCOCPABKNJV-UHFFFAOYSA-N", "3-methylbutanoic acid", "isovaleric acid")
  CALL Species("(CH_3)_3CCOOH",                         "75-98-9", "IUGYQRQAERSCNH-UHFFFAOYSA-N", "2,2-dimethylpropanoic acid", "pivalic acid")
  CALL Species("C_5H_{11}COOH",                        "142-62-1", "FUZZWVXGSFPDMH-UHFFFAOYSA-N", "hexanoic acid", "caproic acid")
  CALL Species("C_6H_{12}O_2",                          "97-61-0", "OVBFMEVBMNZIBR-UHFFFAOYSA-N", "2-methylpentanoic acid", "")
  CALL Species("C_6H_{12}O_2",                          "88-09-5", "OXQGTIUCKGYOAA-UHFFFAOYSA-N", "2-ethylbutanoic acid", "")
  CALL Species("C_7H_{14}O_2",                         "111-14-8", "MNWFXJYAOYHMED-UHFFFAOYSA-N", "heptanoic acid", "")
  CALL Species("C_7H_{14}O_2",                       "95823-36-2", "HMMSZUQCCUWXRA-UHFFFAOYSA-N", "4,4-dimethylpentanoic acid", "")
  CALL Species("C_7H_{14}O_2",                       "19889-37-3", "LHJPKLWGGMAUAN-UHFFFAOYSA-N", "2-ethyl-2-methylbutanoic acid", "")
  CALL Species("C_8H_{16}O_2",                         "124-07-2", "WWZKQHOCKIZLMA-UHFFFAOYSA-N", "octanoic acid", "caprylic acid")
  CALL Species("C_8H_{16}O_2",                         "149-57-5", "OBETXYAYXDNJHR-UHFFFAOYSA-N", "2-ethylhexanoic acid", "")
  CALL Species("C_8H_{10}O_5",                         "145-73-3", "GXEKYRXVRROBEV-UHFFFAOYSA-N", "endothal", "")
  CALL Species("C_9H_{18}O_2",                         "112-05-0", "FBUKVWPVBMHYJY-UHFFFAOYSA-N", "nonanoic acid", "pelargic acid")
  CALL Species("C_{10}H_{20}O_2",                      "334-48-5", "GHVNFZFCNZKVNT-UHFFFAOYSA-N", "decanoic acid", "")
  CALL Species("C_{10}H_{20}O_2",                  "1135681-77-4", "WRPPDRMFGQJMAR-UHFFFAOYSA-N", "3,3,5,5-tetramethylhexanoic acid", "")
  CALL Species("C_{10}H_{16}O_3",                      "473-72-3", "SIZDUQQDBXJXLQ-UHFFFAOYSA-N", "pinonic acid", "")
  CALL Species("C_{11}H_{22}O_2",                      "112-37-8", "ZDPHROOEEOARMN-UHFFFAOYSA-N", "undecanoic acid", "")
  CALL Species("C_{12}H_{24}O_2",                      "143-07-7", "POULHZVOKOAJMA-UHFFFAOYSA-N", "dodecanoic acid", "")
  CALL Species("C_{13}H_{26}O_2",                      "638-53-9", "SZHOJFHSIKHZHA-UHFFFAOYSA-N", "tridecanoic acid", "")
  CALL Species("C_{14}H_{28}O_2",                      "544-63-8", "TUNFSRHWOTWDNC-UHFFFAOYSA-N", "tetradecanoic acid", "")
  CALL Species("C_{15}H_{30}O_2",                     "1002-84-2", "WQEPLUUGTLDZJY-UHFFFAOYSA-N", "pentadecanoic acid", "")
  CALL Species("C_{16}H_{32}O_2",                       "57-10-3", "IPCSVZSSVZVIGE-UHFFFAOYSA-N", "hexadecanoic acid", "palmitic acid")
  CALL Species("C_{17}H_{34}O_2",                      "506-12-7", "KEMQGTRYUADPNZ-UHFFFAOYSA-N", "heptadecanoic acid", "margaric acid")
  CALL Species("C_{18}H_{36}O_2",                       "57-11-4", "QIQXTHQIDYTFRH-UHFFFAOYSA-N", "octadecanoic acid", "stearic acid")
  CALL Species("C_{19}H_{38}O_2",                      "646-30-0", "ISYWECDDZWTKFF-UHFFFAOYSA-N", "nonadecanoic acid", "")
  CALL Species("C_{20}H_{40}O_2",                      "506-30-9", "VKOBVWXKNCXXDE-UHFFFAOYSA-N", "eicosanoic acid", "arachidic acid")
  CALL Species("C_{21}H_{42}O_2",                     "2363-71-5", "CKDDRHZIAZRDBW-UHFFFAOYSA-N", "heneicosanoic acid", "")
  CALL Species("C_{22}H_{44}O_2",                      "112-85-6", "UKMSUNONTOPOIO-UHFFFAOYSA-N", "docosanoic acid", "behenic acid")
  CALL Species("C_{23}H_{46}O_2",                     "2433-96-7", "XEZVDURJDFGERA-UHFFFAOYSA-N", "tricosanoic acid", "")
  CALL Species("C_{24}H_{48}O_2",                      "557-59-5", "QZZGJDVWLFXDLK-UHFFFAOYSA-N", "tetracosanoic acid", "lignoceric acid")
  ! unsaturated and aromatic:
  CALL Species("C_3H_4O_2",                             "79-10-7", "NIXOWILDQLNWCW-UHFFFAOYSA-N", "propenoic acid", "acrylic acid")
  CALL Species("C_4H_6O_2",                           "3724-65-0", "LDHQCZJRKDOVOX-UHFFFAOYSA-N", "2-butenoic acid", "crotonic acid")
  CALL Species("C_4H_6O_2",                            "107-93-7", "LDHQCZJRKDOVOX-NSCUHMNNSA-N", "($E$)-2-butenoic acid", "{\it trans}-crotonic acid")
  CALL Species("C_4H_6O_2",                            "503-64-0", "LDHQCZJRKDOVOX-IHWYPQMZSA-N", "($Z$)-2-butenoic acid", "isocrotonic acid")
  CALL Species("C_4H_6O_2",                             "79-41-4", "CERQOIWHTDAKMF-UHFFFAOYSA-N", "2-methyl-2-propenoic acid", "methacrylic acid")
  CALL Species("C_4H_6O_3",                                    "", "CSEUSVYSDPXJAP-UHFFFAOYSA-N", "methacrylic acid epoxide", "")
  CALL Species("C_6H_5COOH",                            "65-85-0", "WPYMKLBDIGXBTP-UHFFFAOYSA-N", "benzenecarboxylic acid", "benzoic acid")
  CALL Species("C_6H_8O_2",                            "110-44-1", "WSWCOQWTEOXDQX-MQQKCMAXSA-N", "sorbic acid", "")
  CALL Species("C_6H_8O_6",                             "89-65-6", "CIWBSHSKHKDKBQ-DUZGATOHSA-N", "$D$(-)-isoascorbic acid", "erythorbic acid")
  CALL Species("C_7H_{10}O_5",                         "138-59-0", "JXOHGGNKMLTUBP-UHFFFAOYSA-N", "shikimic acid", "")
  CALL Species("C_7H_6O_3",                             "99-96-7", "FJKROLUGYXJWQN-UHFFFAOYSA-N", "4-hydroxybenzoic acid", "")
  CALL Species("C_7H_6O_5",                            "149-91-7", "LNTHITQWFMADLM-UHFFFAOYSA-N", "3,4,5-trihydroxybenzoic acid", "gallic acid")
  CALL Species("C_8H_8O_2",                            "118-90-1", "ZWLPBLYKEWSWPD-UHFFFAOYSA-N", "2-methylbenzoic acid", "$o$-toluic acid")
  CALL Species("C_8H_8O_2",                             "99-04-7", "GPSDUZXPYCFOSQ-UHFFFAOYSA-N", "3-methylbenzoic acid", "$m$-toluic acid")
  CALL Species("C_8H_8O_2",                             "99-94-5", "LPNBBFKOUUSUDB-UHFFFAOYSA-N", "4-methylbenzoic acid", "$p$-toluic acid")
  CALL Species("C_7H_6O_3",                             "69-72-7", "YGSDEFSMJLZEOE-UHFFFAOYSA-N", "2-hydroxy-benzoic acid", "salicylic acid")
  CALL Species("C_8H_8O_2",                            "103-82-2", "WLJVXDMOQOGPHL-UHFFFAOYSA-N", "benzeneethanoic acid", "phenylacetic acid")
  CALL Species("C_8H_4O_3",                             "85-44-9", "LGRFSURHDFAFJT-UHFFFAOYSA-N", "phthalic anhydride", "")
  CALL Species("C_8H_6O_4",                             "88-99-3", "XNGIFLGASWRNHJ-UHFFFAOYSA-N", "1,2-benzenedicarboxylic acid", "phthalic acid")
  CALL Species("C_8H_6O_4",                            "100-21-0", "KKEYFWRCBNTPAC-UHFFFAOYSA-N", "terephthalic acid", "")
  CALL Species("C_8H_6O_4",                            "121-91-5", "QQVIHTHCMHWDBS-UHFFFAOYSA-N", "isophthalic acid", "")
  CALL Species("C_8H_8O_4",                            "520-45-6", "PGRHXDWITVMQBC-UHFFFAOYSA-N", "dehydroacetic acid", "")
  CALL Species("C_8H_8O_3",                            "579-75-9", "ILUJQPXNXACGAN-UHFFFAOYSA-N", "2-methoxybenzoic acid", "")
  CALL Species("C_8H_8O_3",                            "100-09-4", "ZEYHEAKUIGZSGI-UHFFFAOYSA-N", "4-methoxybenzoic acid", "")
  CALL Species("C_9H_8O_4",                            "331-39-5", "QAIPRVGONGVQAS-DUXPYHPUSA-N", "caffeic acid", "")
  CALL Species("C_9H_6O_3",                          "19438-61-0", "ZOXBWJMCXHTKNU-UHFFFAOYSA-N", "4-methylphthalic anhydride", "")
  CALL Species("C_9H_8O_2",                            "621-82-9", "WBYWAXJHAXSJNI-UHFFFAOYSA-N", "3-phenyl-2-propenoic acid", "cinnamic acid")
  CALL Species("C_9H_8O_2",                            "140-10-3", "WBYWAXJHAXSJNI-VOTSOKGWSA-N", "{\it trans}-cinnamic acid", "")
  CALL Species("C_9H_8O_3",                            "501-98-4", "NGSWKAQJJWESNS-ZZXKWVIFSA-N", "$p$-coumaric acid", "")
  CALL Species("C_9H_{10}O_4",                          "93-07-2", "DAUAQNGYDSHRET-UHFFFAOYSA-N", "3,4-dimethoxybenzoic acid", "veratric acid")
  CALL Species("C_{10}H_{12}O_5",                      "118-41-2", "SJSOFNCYXJUNBT-UHFFFAOYSA-N", "3,4,5-trimethoxybenzoic acid", "eudesmic acid")
  CALL Species("C_{12}H_{10}O_2",                       "86-87-3", "PRPINYUDVPFIRX-UHFFFAOYSA-N", "1-naphthaleneacetic acid", "")
  CALL Species("C_{12}H_{10}O_3",                      "120-23-0", "RZCJYMOBWVJQGV-UHFFFAOYSA-N", "2-naphthoxyacetic acid", "")
  CALL Species("C_{11}H_{14}O_2",                       "98-73-7", "KDVYCTOWXSLNNI-UHFFFAOYSA-N", "$p$-{\it tert}-butylbenzoic acid", "")
  CALL Species("C_{11}H_{12}O_5",                   "104273-73-6", "DFFWZNDCNBOKDI-UHFFFAOYSA-N", "trinexapac", "")
  CALL Species("C_{14}H_{10}O_3",                       "93-97-0", "CHIHQLCVLOXUJW-UHFFFAOYSA-N", "benzoic acid, anhydride", "")
  CALL Species("C_{10}H_6O_8",                          "89-05-4", "CYIDZMCFTVVTJO-UHFFFAOYSA-N", "pyromellitic acid", "")
  CALL Species("C_{10}H_2O_6",                          "89-32-7", "ANSXAPJVJOKRDJ-UHFFFAOYSA-N", "pyromellitic dianhydride", "")
  CALL Species("C_{14}H_{14}O_3",                    "22204-53-1", "CMWTZPSULFXXJA-VIFPVBQESA-N", "naproxen", "")
  CALL Species("C_{18}H_{32}O_2",                       "60-33-3", "OYHQOLUKZRVURQ-HZJYTTRNSA-N", "($Z$,$Z$)-9,12-octadecadienoic acid", "linoleic acid")
  CALL Species("C_{18}H_{16}O_8",                      "537-15-5", "DOUMFZQKYFQNTF-WUTVXBCWSA-N", "rosmarinic acid", "")
  CALL Species("C_{20}H_{30}O_2",                      "514-10-3", "RSWGJHLUYNHPMX-ONCXSQPRSA-N", "abietic acid", "")
  ! dicarboxylic acids:
  CALL Species("HOOCCOOH",                             "144-62-7", "MUBZPKHOEPUJKR-UHFFFAOYSA-N", "ethanedioic acid", "oxalic acid")
  CALL Species("HOOCCH_2COOH",                         "141-82-2", "OFOBLEOULBTSOW-UHFFFAOYSA-N", "propanedioic acid", "malonic acid")
  CALL Species("HOOC(CH_2)_2COOH",                     "110-15-6", "KDYFGRWQOYBRFD-UHFFFAOYSA-N", "butanedioic acid", "succinic acid")
  CALL Species("HOOC(CH_2)_3COOH",                     "110-94-1", "JFCQEDHGNNZCLN-UHFFFAOYSA-N", "pentanedioic acid", "glutaric acid")
  CALL Species("C_5H_8O_5",                            "638-18-6", "ZQHYXNSQOIDNTL-UHFFFAOYSA-N", "3-hydroxy glutaric acid", "")
  CALL Species("HOOC(CH_2)_4COOH",                     "124-04-9", "WNLRTRBMVRJNCN-UHFFFAOYSA-N", "hexanedioic acid", "adipic acid")
  CALL Species("C_6H_8O_6",                             "99-14-9", "KQTIIICEAUMSDG-UHFFFAOYSA-N", "tricarballylic acid", "")
  CALL Species("C_7H_{12}O_4",                         "111-16-0", "WLJVNTCWHIRURA-UHFFFAOYSA-N", "heptanedioic acid", "pimelic acid")
  CALL Species("C_7H_{10}O_5",                      "149474-71-5", "TZPGYCKKEMNHRS-UHFFFAOYSA-N", "3-acetyl pentanedioic acid", "")
  CALL Species("C_8H_{14}O_4",                         "505-48-6", "TYFQFVWCELRYAO-UHFFFAOYSA-N", "octanedioic acid", "suberic acid")
  CALL Species("C_8H_{12}O_6",                       "77370-41-3", "VMWJGTKDJFMTFZ-UHFFFAOYSA-N", "3-methyl-1,2,3-butanetricarboxylic acid", "")
  CALL Species("C_9H_{16}O_4",                         "123-99-9", "BDJRBEYXGGNYIS-UHFFFAOYSA-N", "nonanedioic acid", "azelaic acid")
  CALL Species("C_{10}H_{18}O_4",                      "111-20-6", "CXMXRPHRNRROMY-UHFFFAOYSA-N", "decanedioic acid", "sebacic acid")
  CALL Species("C_{12}H_{22}O_4",                      "693-23-2", "TVIDDXQYHWJXFK-UHFFFAOYSA-N", "dodecanedioic acid", "")
  CALL Species("HOOC(CH)_2COOH",                       "110-16-7", "VZCYOOQTPOCHFL-UPHRSURJSA-N", "{\it cis}-butenedioic acid", "maleic acid")
  CALL Species("C_5H_6O_4",                            "498-23-7", "HNEGQIOMVPPMNR-IHWYPQMZSA-N", "citraconic acid", "")
  CALL Species("C_5H_6O_4",                             "97-65-4", "LVHBHZANLOWSRM-UHFFFAOYSA-N", "itaconic acid", "")
  CALL Species("C_8H_{12}O_4",                         "619-82-9", "PXGZQGDTEZPERC-UHFFFAOYSA-N", "1,4-cyclohexanedicarboxylic acid", "")
  CALL Species("C_9H_{14}O_4",                         "473-73-4", "LEVONNIFUFSRKZ-UHFFFAOYSA-N", "pinic acid", "")
  ! peroxyacids:
  CALL Species("HCOOOH",                               "107-32-4", "SCKXCAADGDQQCS-UHFFFAOYSA-N", "methanoic peroxyacid", "peroxyformic acid")
  CALL Species("CH_3COOOH",                             "79-21-0", "KFSLWBXXFJQRDL-UHFFFAOYSA-N", "ethanoic peroxyacid", "peroxyacetic acid")
  ! acids from MCM:
  CALL Species("C_6H_{12}O_2",                                 "", "XFOASZQZPWEJAA-UHFFFAOYSA-N", "MCM:M2C43CO2H", "")
  CALL Species("C_6H_{12}O_2",                                 "", "VUAXHMVRKOTJKP-UHFFFAOYSA-N", "MCM:M33C3CO2H", "")
  CALL Species("C_7H_{12}O_4",                                 "", "ZLURETBBUKLNQX-UHFFFAOYSA-N", "MCM:C721OOH", "")
  CALL Species("C_8H_{12}O_5",                                 "", "HYEGRGIGVZWWFM-UHFFFAOYSA-N", "MCM:C721CO3H", "")
  CALL Species("C_8H_{14}O_4",                                 "", "RYDSGVCXHLRTGT-UHFFFAOYSA-N", "MCM:C811OOH", "")
  CALL Species("C_8H_{14}O_4",                                 "", "KIIRSRLFRXTUMU-UHFFFAOYSA-N", "MCM:C823OOH", "")
  CALL Species("C_8H_{12}O_4",                                 "", "KLGKVMMWRDYKJM-UHFFFAOYSA-N", "MCM:NORPINIC", "")
  CALL Species("C_9H_{14}O_5",                                 "", "ZVOGBTSHONFRIW-UHFFFAOYSA-N", "MCM:C811CO3H", "")
  CALL Species("C_9H_{14}O_5",                                 "", "IESQFENJEQJSTC-UHFFFAOYSA-N", "MCM:C823CO3H", "")
  CALL Species("C_9H_{14}O_4",                                 "", "JUCWCJYNDRRZKR-UHFFFAOYSA-N", "MCM:LIMONIC", "")
  CALL Species("C_{13}H_{22}O_4",                              "", "RDYPCLKIKFUKHY-UHFFFAOYSA-N", "MCM:C137OOH", "")
  CALL Species("C_{14}H_{22}O_4",                              "", "IYTMEDMFNUDFPT-UHFFFAOYSA-N", "MCM:C137CO2H", "")
  CALL Species("C_{14}H_{22}O_5",                              "", "WIBVBPZGNYNCKK-UHFFFAOYSA-N", "MCM:C137CO3H", "")
  CALL Species("C_9H_{10}O_2",                                 "", "RIZUCYSQUWMQLX-UHFFFAOYSA-N", "MCM:TM123BCO2H", "")
  CALL Species("C_9H_{10}O_2",                                 "", "OPVAJFQBSDUNQA-UHFFFAOYSA-N", "MCM:TM124BCO2H", "")
  CALL Species("C_9H_{10}O_2",                                 "", "UMVOQQDNEYOJOK-UHFFFAOYSA-N", "MCM:TMBCO2H", "")
  CALL Species("C_3H_6O_4",                                    "", "RBNPOMFGQQGHHO-UHFFFAOYSA-N", "MCM:C2OHOCO2H", "")
  CALL Species("C_3H_6O_3",                                    "", "ALRHLSYJTWAHJZ-UHFFFAOYSA-N", "MCM:HOC2H4CO2H", "")
  CALL Species("C_4H_6O_3",                                    "", "RMQJECWPWQIIPW-UHFFFAOYSA-N", "MCM:HC3CO2H", "")
  CALL Species("C_4H_6O_3",                                    "", "AAMTXHVZOHPPQR-UHFFFAOYSA-N", "MCM:HMACO2H", "")
  CALL Species("C_4H_8O_3",                                    "", "WHBMMWSBFZVSSR-UHFFFAOYSA-N", "MCM:HO2C3CO2H", "")
  CALL Species("C_4H_8O_3",                                    "", "SJZRECIVHVDYJC-UHFFFAOYSA-N", "MCM:HOC3H6CO2H", "")
  CALL Species("C_4H_8O_3",                                    "", "DBXBTMSZEOQQDU-UHFFFAOYSA-N", "MCM:HOIPRCO2H", "")
  CALL Species("C_4H_8O_3",                                    "", "BWLBGMIXKSTLSX-UHFFFAOYSA-N", "MCM:IPRHOCO2H", "")
  CALL Species("C_5H_8O_3",                                    "", "NJMYQRVWBCSLEU-UHFFFAOYSA-N", "MCM:C46CO2H", "")
  CALL Species("C_5H_8O_3",                                    "", "BERUOTKXCOOJJM-UHFFFAOYSA-N", "MCM:HC4ACO2H", "")
  CALL Species("C_5H_8O_3",                                    "", "NCQCQZXQBYAHBZ-UHFFFAOYSA-N", "MCM:HC4CCO2H", "")
  CALL Species("C_5H_{10}O_3",                                 "", "RDFQSFOGKVZWKF-UHFFFAOYSA-N", "MCM:HM22CO2H", "")
  CALL Species("C_5H_{10}O_3",                                 "", "VEXDRERIMPLZLU-UHFFFAOYSA-N", "MCM:HO2C43CO2H", "")
  CALL Species("C_5H_{10}O_3",                                 "", "FMHKPLXYWVCLME-UHFFFAOYSA-N", "MCM:HO2C4CO2H", "")
  CALL Species("C_5H_{10}O_3",                                 "", "JYTYEGKJKIXWOJ-UHFFFAOYSA-N", "MCM:HOBUT2CO2H", "")
  CALL Species("C_6H_{10}O_3",                                 "", "COPVWTGWHHSRKU-UHFFFAOYSA-N", "MCM:C518CO2H", "")
  CALL Species("C_6H_{12}O_3",                                 "", "PQJUMPXLDAZULJ-UHFFFAOYSA-N", "MCM:H2M2C4CO2H", "")
  CALL Species("C_6H_{12}O_3",                                 "", "BNTHHYUWQILHSA-UHFFFAOYSA-N", "MCM:H2M3C4CO2H", "")
  CALL Species("C_6H_{12}O_3",                                 "", "WLKUATLKPJGHKX-UHFFFAOYSA-N", "MCM:HM22C3CO2H", "")
  CALL Species("C_6H_{12}O_3",                                 "", "QSMSJHRQUBQMKD-UHFFFAOYSA-N", "MCM:HM2C43CO2H", "")
  CALL Species("C_6H_{12}O_3",                                 "", "NGRPJOQCQYDOJX-UHFFFAOYSA-N", "MCM:HM33C3CO2H", "")
  CALL Species("C_6H_{12}O_3",                                 "", "IOXPMKQXROCYFL-UHFFFAOYSA-N", "MCM:HO2C54CO2H", "")
  CALL Species("C_6H_{12}O_3",                                 "", "ABIKNKURIGPIRJ-UHFFFAOYSA-N", "MCM:HO3C5CO2H", "")
  CALL Species("C_7H_{12}O_3",                                 "", "PHVXTLWKDLVJPT-UHFFFAOYSA-N", "MCM:C622CO2H", "")
  CALL Species("C_7H_{12}O_3",                                 "", "IOGKLHPXZHQWIC-UHFFFAOYSA-N", "MCM:C624CO2H", "")
  CALL Species("C_7H_{14}O_3",                                 "", "BHWOTLCMMUBASI-UHFFFAOYSA-N", "MCM:H3M3C5CO2H", "")
  CALL Species("C_8H_{14}O_3",                                 "", "UVNHICQQCWWSMK-UHFFFAOYSA-N", "MCM:C811OH", "")
  CALL Species("C_8H_{14}O_4",                                 "", "VFUDATRQVLCLSF-UHFFFAOYSA-N", "MCM:C812OH", "")
  CALL Species("C_8H_{14}O_5",                                 "", "TXANCYYWZQZPGK-UHFFFAOYSA-N", "MCM:C812OOH", "")
  CALL Species("C_8H_{14}O_3",                                 "", "HTVBHPNLYICQSI-UHFFFAOYSA-N", "MCM:C823OH", "")
  CALL Species("C_8H_{14}O_4",                                 "", "GZVYHBJSBFFFII-UHFFFAOYSA-N", "MCM:C825OH", "")
  CALL Species("C_8H_{14}O_5",                                 "", "XYVJCWMMBMHSFM-UHFFFAOYSA-N", "MCM:C825OOH", "")
  CALL Species("C_{13}H_{22}O_3",                              "", "GRQGZSRLPJOOBF-UHFFFAOYSA-N", "MCM:C137OH", "")
  CALL Species("C_{13}H_{22}O_4",                              "", "RQGWJIBQKQBFEC-UHFFFAOYSA-N", "MCM:C139OH", "")
  CALL Species("C_{13}H_{22}O_5",                              "", "MQHLLQIJDGKDCT-UHFFFAOYSA-N", "MCM:C139OOH", "")
  CALL Species("C_4H_6O_5",                                    "", "RIVXGQNQRYOWAN-UHFFFAOYSA-N", "MCM:C44OOH", "")
  CALL Species("C_4H_4O_3",                                    "", "ZOIRMVZWDRLJPI-UHFFFAOYSA-N", "MCM:MALDALCO2H", "")
  CALL Species("C_4H_6O_3",                                    "", "VOKUMXABRRXHAR-UHFFFAOYSA-N", "MCM:PRPAL2CO2H", "")
  CALL Species("C_5H_6O_3",                                    "", "VXAWORVMCLXEKH-UHFFFAOYSA-N", "MCM:C3MCOCO2H", "")
  CALL Species("C_5H_8O_3",                                    "", "SUMZWDXUXLTFFX-UHFFFAOYSA-N", "MCM:CO1M22CO2H", "")
  CALL Species("C_5H_6O_3",                                    "", "QPFVAKLVWFPTSX-UHFFFAOYSA-N", "MCM:MC3ODBCO2H", "")
  CALL Species("C_6H_8O_3",                                    "", "XIZBOUDCWHEQCF-UHFFFAOYSA-N", "MCM:C3EODBCO2H", "")
  CALL Species("C_6H_8O_3",                                    "", "HLFIROOBCUFLFC-UHFFFAOYSA-N", "MCM:C3M2COCO2H", "")
  CALL Species("C_6H_8O_3",                                    "", "NSMTVDIDRFOALU-UHFFFAOYSA-N", "MCM:C522CO2H", "")
  CALL Species("C_6H_8O_3",                                    "", "HNXWTHGWCGDLKB-UHFFFAOYSA-N", "MCM:RGDCO2H", "")
  CALL Species("C_7H_{10}O_4",                                 "", "BRDQYLNZHVUNQF-UHFFFAOYSA-N", "MCM:C615CO2H", "")
  CALL Species("C_7H_{12}O_5",                                 "", "XJSSOLXEKAZFKL-UHFFFAOYSA-N", "MCM:C722OOH", "")
  CALL Species("C_7H_{10}O_3",                                 "", "HLXOYOZOZOGAJW-UHFFFAOYSA-N", "MCM:IP3ODBCO2H", "")
  CALL Species("C_7H_{10}O_3",                                 "", "SHOOYBDWMZRRKO-UHFFFAOYSA-N", "MCM:PC3ODBCO2H", "")
  CALL Species("C_8H_{12}O_3",                                 "", "MQTHUXQLMJSVGN-UHFFFAOYSA-N", "MCM:C721CHO", "")
  CALL Species("C_8H_{12}O_3",                                 "", "NHNJUKWYLMFOQP-UHFFFAOYSA-N", "MCM:C729CO2H", "")
  CALL Species("C_8H_{12}O_3",                                 "", "KUOQBJGRIDTXEZ-UHFFFAOYSA-N", "MCM:C823CO", "")
  CALL Species("C_9H_{14}O_3",                                 "", "ADULCIYKWVJSFE-UHFFFAOYSA-N", "MCM:C822CO2H", "")
  CALL Species("C_9H_{14}O_3",                                 "", "NWNQYPOMHUZLML-UHFFFAOYSA-N", "MCM:C830CO2H", "")
  CALL Species("C_9H_{14}O_3",                                 "", "RFGXRYLTPBBFIG-UHFFFAOYSA-N", "MCM:C89CO2H", "")
  CALL Species("C_{13}H_{20}O_3",                              "", "FRZAETYQWIEERW-UHFFFAOYSA-N", "MCM:C126CO2H", "")
  CALL Species("C_{13}H_{20}O_3",                              "", "FQUQDHNCRLRZNB-UHFFFAOYSA-N", "MCM:C137CO", "")
  CALL Species("C_{14}H_{22}O_3",                              "", "MUFZWTUKICSBHZ-UHFFFAOYSA-N", "MCM:C136CO2H", "")
  CALL Species("C_4H_6O_4",                                    "", "DDNPKLYNPFAIEE-UHFFFAOYSA-N", "MCM:COHM2CO2H", "")
  CALL Species("C_6H_{10}O_4",                                 "", "NIVGPRREFKECGM-UHFFFAOYSA-N", "MCM:C1H4C5CO2H", "")
  CALL Species("C_4H_6O_3",                                    "", "TYEYBOSBBBHJIV-UHFFFAOYSA-N", "MCM:ECOCO2H", "")
  CALL Species("C_5H_8O_3",                                    "", "GCXJINGJZAOJHR-UHFFFAOYSA-N", "MCM:C41CO2H", "")
  CALL Species("C_5H_6O_3",                                    "", "XGTKSWVCNVUVHG-UHFFFAOYSA-N", "MCM:C5CO14OH", "")
  CALL Species("C_5H_8O_3",                                    "", "JOOXCMJARBKPKM-UHFFFAOYSA-N", "MCM:CO2C4CO2H", "")
  CALL Species("C_5H_8O_3",                                    "", "FHSUFDYFOHSYHI-UHFFFAOYSA-N", "MCM:CO3C4CO2H", "")
  CALL Species("C_5H_8O_3",                                    "", "QHKABHOOEWYVLI-UHFFFAOYSA-N", "MCM:IPGLYOXOH", "")
  CALL Species("C_5H_8O_3",                                    "", "KDVFRMMRZOCFLS-UHFFFAOYSA-N", "MCM:PGLYOXOH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "NFIWUVRBASXMGK-UHFFFAOYSA-N", "MCM:C51CO2H", "")
  CALL Species("C_6H_8O_3",                                    "", "YUICRCSHEOZHST-UHFFFAOYSA-N", "MCM:C5CODBCO2H", "")
  CALL Species("C_6H_8O_3",                                    "", "LRRPKULXSOVRRZ-UHFFFAOYSA-N", "MCM:C5DBCOCO2H", "")
  CALL Species("C_6H_8O_3",                                    "", "VUZJJUOKMOGBDH-UHFFFAOYSA-N", "MCM:C6DCRBBOH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "UZTJTTKEYGHTNM-UHFFFAOYSA-N", "MCM:CO2C54CO2H", "")
  CALL Species("C_6H_{10}O_3",                                 "", "CBZZWRQRPKSEQI-UHFFFAOYSA-N", "MCM:CO2M33CO2H", "")
  CALL Species("C_6H_{10}O_3",                                 "", "CLJBDOUIEHLLEN-UHFFFAOYSA-N", "MCM:CO3C5CO2H", "")
  CALL Species("C_7H_{12}O_5",                                 "", "ASSSOSHEUJWVOL-UHFFFAOYSA-N", "MCM:C732OOH", "")
  CALL Species("C_7H_{10}O_3",                                 "", "GJIWJSZYGQVZNC-UHFFFAOYSA-N", "MCM:C7ADCCO2H", "")
  CALL Species("C_7H_{10}O_3",                                 "", "OAGXHOMWGVMDLP-UHFFFAOYSA-N", "MCM:C7CDCCO2H", "")
  CALL Species("C_7H_{10}O_3",                                 "", "LUXOYEZTEWTGMD-UHFFFAOYSA-N", "MCM:C7DCCO2H", "")
  CALL Species("C_7H_{10}O_3",                                 "", "MDDQAVHYGMRQBB-UHFFFAOYSA-N", "MCM:C7DDCCO2H", "")
  CALL Species("C_7H_{10}O_4",                                 "", "WUQQVFQGWWMQLD-UHFFFAOYSA-N", "MCM:CO25C6CO2H", "")
  CALL Species("C_7H_{10}O_3",                                 "", "TZKNMZPMBZQSMV-UHFFFAOYSA-N", "MCM:IC7DCCO2H", "")
  CALL Species("C_8H_{12}O_6",                                 "", "IARLEAITRRLVEZ-UHFFFAOYSA-N", "MCM:C732CO3H", "")
  CALL Species("C_8H_{12}O_5",                                 "", "POLIOAJLCGOOTJ-UHFFFAOYSA-N", "MCM:KLIMONIC", "")
  CALL Species("C_9H_{14}O_4",                                 "", "UOMMGFNPUVSKTE-UHFFFAOYSA-N", "MCM:C827CO2H", "")
  CALL Species("C_9H_{12}O_4",                                 "", "LOFCXTWTTIFNLN-UHFFFAOYSA-N", "MCM:C88CO2H", "")
  CALL Species("C_9H_{14}O_4",                                 "", "RRLLKTDJWPIWHS-UHFFFAOYSA-N", "MCM:KLIMONONIC", "")
  CALL Species("C_{10}H_{16}O_4",                              "", "SANKOOWGNYBKOU-UHFFFAOYSA-N", "MCM:C928CO2H", "")
  CALL Species("C_{10}H_{16}O_3",                              "", "NJOIWWRMLFSDTM-UHFFFAOYSA-N", "MCM:LIMONONIC", "")
  CALL Species("C_{11}H_{18}O_3",                              "", "IUOPXZCJHJMHNP-UHFFFAOYSA-N", "MCM:C1011CO2H", "")
  CALL Species("C_{12}H_{20}O_5",                              "", "PTZJPBFCCQIPAZ-UHFFFAOYSA-N", "MCM:C1211OOH", "")
  CALL Species("C_{13}H_{20}O_5",                              "", "QUEDJGCZCJPFDA-UHFFFAOYSA-N", "MCM:C1211CO2H", "")
  CALL Species("C_{13}H_{20}O_6",                              "", "DVDQWECWXWLOQZ-UHFFFAOYSA-N", "MCM:C1211CO3H", "")
  CALL Species("C_{14}H_{22}O_4",                              "", "QDUXHZZYSCMHPK-UHFFFAOYSA-N", "MCM:C131CO2H", "")
  CALL Species("C_{14}H_{20}O_5",                              "", "MMRJODLWTRWNRH-UHFFFAOYSA-N", "MCM:C147CO", "")
  CALL Species("C_{14}H_{22}O_6",                              "", "MOUQEZQXHGSYER-UHFFFAOYSA-N", "MCM:C147OOH", "")
  CALL Species("C_{15}H_{24}O_3",                              "", "VGNFSHPHIKPNED-UHFFFAOYSA-N", "MCM:C141CO2H", "")
  CALL Species("C_3H_4O_4",                                    "", "HHDDCCUIIUWNGJ-UHFFFAOYSA-N", "MCM:HOCH2COCO2H", "")
  CALL Species("C_5H_8O_4",                                    "", "IQZHXDCQZNRIRW-UHFFFAOYSA-N", "MCM:H3C2C4CO2H", "")
  CALL Species("C_5H_8O_4",                                    "", "ISMQTZHDOHOVBT-UHFFFAOYSA-N", "MCM:HMVKBCO2H", "")
  CALL Species("C_6H_{10}O_4",                                 "", "APPVMSIJNZFGHP-UHFFFAOYSA-N", "MCM:C517CO2H", "")
  CALL Species("C_6H_{10}O_4",                                 "", "ULHNCRHSCMPZGW-UHFFFAOYSA-N", "MCM:C519CO2H", "")
  CALL Species("C_7H_{12}O_4",                                 "", "PKQMWBHBXJKPID-UHFFFAOYSA-N", "MCM:C732OH", "")
  CALL Species("C_7H_{10}O_5",                                 "", "CPJVKYXUMUTCGW-UHFFFAOYSA-N", "MCM:C734CO", "")
  CALL Species("C_7H_{12}O_5",                                 "", "RDRUHSRUZHRAFB-UHFFFAOYSA-N", "MCM:C734OH", "")
  CALL Species("C_7H_{12}O_6",                                 "", "ABEMFUWOTMEDPB-UHFFFAOYSA-N", "MCM:C734OOH", "")
  CALL Species("C_7H_{10}O_5",                                 "", "FMZPEVNVWACALO-UHFFFAOYSA-N", "MCM:H3C25CCO2H", "")
  CALL Species("C_8H_{14}O_5",                                 "", "UHMOZKUUXAVADP-UHFFFAOYSA-N", "MCM:C813OH", "")
  CALL Species("C_8H_{14}O_6",                                 "", "FNGYAHKKJFMZCA-UHFFFAOYSA-N", "MCM:C813OOH", "")
  CALL Species("C_8H_{12}O_4",                                 "", "PGSQVIJDIYGHPF-UHFFFAOYSA-N", "MCM:C825CO", "")
  CALL Species("C_{10}H_{16}O_4",                              "", "MZHKOIVHCBFXJV-UHFFFAOYSA-N", "MCM:HOPINONIC", "")
  CALL Species("C_{12}H_{20}O_4",                              "", "JNTJQAWMLAKHDW-UHFFFAOYSA-N", "MCM:C1211OH", "")
  CALL Species("C_{12}H_{20}O_5",                              "", "OINCQTVJDDDYTJ-UHFFFAOYSA-N", "MCM:C1212OH", "")
  CALL Species("C_{12}H_{20}O_6",                              "", "KZNQDAPBLSOIIR-UHFFFAOYSA-N", "MCM:C1212OOH", "")
  CALL Species("C_{12}H_{18}O_6",                              "", "IKFUCIVDZJUENZ-UHFFFAOYSA-N", "MCM:C1213CO", "")
  CALL Species("C_{12}H_{20}O_6",                              "", "DPGLQQDGNMDAAZ-UHFFFAOYSA-N", "MCM:C1213OH", "")
  CALL Species("C_{12}H_{20}O_7",                              "", "ZUGVXCQEOPRNFT-UHFFFAOYSA-N", "MCM:C1213OOH", "")
  CALL Species("C_{13}H_{20}O_5",                              "", "NHYXXXJOGAKHPV-UHFFFAOYSA-N", "MCM:C1310CO", "")
  CALL Species("C_{13}H_{22}O_5",                              "", "PXEJSKZGVMTSCR-UHFFFAOYSA-N", "MCM:C1310OH", "")
  CALL Species("C_{13}H_{22}O_6",                              "", "HXUMLFGRLLCICH-UHFFFAOYSA-N", "MCM:C1310OOH", "")
  CALL Species("C_{14}H_{22}O_5",                              "", "IPYASLZXSQHCRG-UHFFFAOYSA-N", "MCM:C147OH", "")
  CALL Species("C_{15}H_{26}O_5",                              "", "RRTATIWWWPYIIC-UHFFFAOYSA-N", "MCM:C151OH", "")
  CALL Species("C_{15}H_{26}O_6",                              "", "FDHNLWUJMDRLQD-UHFFFAOYSA-N", "MCM:C151OOH", "")
  CALL Species("C_4H_4O_4",                                    "", "YEZSWHPLZBZVLH-UHFFFAOYSA-N", "MCM:CO13C3CO2H", "")
  CALL Species("C_6H_8O_4",                                    "", "CGRSCFNOTZUXCX-UHFFFAOYSA-N", "MCM:C512CO2H", "")
  CALL Species("C_6H_8O_4",                                    "", "TWNJAPRLIZAXHA-UHFFFAOYSA-N", "MCM:C6COALCO2H", "")
  CALL Species("C_7H_{10}O_4",                                 "", "ZXMACHDUAJJJBQ-UHFFFAOYSA-N", "MCM:C617CO2H", "")
  CALL Species("C_7H_{10}O_4",                                 "", "MXMJWVQRGMMLRI-UHFFFAOYSA-N", "MCM:C618CO2H", "")
  CALL Species("C_7H_{10}O_4",                                 "", "JCTYMUINHXVDEP-UHFFFAOYSA-N", "MCM:C626CO2H", "")
  CALL Species("C_7H_{10}O_4",                                 "", "ATXVRLJHWCSFDB-UHFFFAOYSA-N", "MCM:C732CO", "")
  CALL Species("C_8H_{12}O_4",                                 "", "OBYKYHDGLQUNDK-UHFFFAOYSA-N", "MCM:C718CO2H", "")
  CALL Species("C_8H_{12}O_4",                                 "", "URWVDUFOGQNELZ-UHFFFAOYSA-N", "MCM:C731CO2H", "")
  CALL Species("C_9H_{12}O_5",                                 "", "GZPOIVSRXTZTQZ-UHFFFAOYSA-N", "MCM:C87CO2H", "")
  CALL Species("C_{12}H_{18}O_4",                              "", "HWWVGCXKFWUOCT-UHFFFAOYSA-N", "MCM:C116CO2H", "")
  CALL Species("C_{12}H_{18}O_4",                              "", "VVPUYBXPESADLR-UHFFFAOYSA-N", "MCM:C1211CO", "")
  CALL Species("C_{13}H_{20}O_4",                              "", "MERIWEKKXYNSHZ-UHFFFAOYSA-N", "MCM:C1210CO2H", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Esters (\chem{RCOOR})")
  ! saturated:
  ! carbonates:
  CALL Species("C_3H_4O_3",                             "96-49-1", "KMTRUDSVKNLOMY-UHFFFAOYSA-N", "1,3-dioxolan-2-one", "ethylene carbonate")
  CALL Species("C_3H_6O_3",                            "616-38-6", "IEJIGPNLZYLLBP-UHFFFAOYSA-N", "carbonic acid, dimethyl ester", "dimethyl carbonate")
  CALL Species("C_4H_6O_5",                           "4525-33-1", "GZDFHIJNHHMENY-UHFFFAOYSA-N", "dimethyl dicarbonate", "")
  CALL Species("C_5H_{10}O_3",                         "105-58-8", "OIFBSDVPJOWBCH-UHFFFAOYSA-N", "carbonic acid, diethyl ester", "diethyl carbonate")
  ! methanoate:
  CALL Species("HCOOCH_3",                             "107-31-3", "TZIHFWKZFHZASV-UHFFFAOYSA-N", "methyl methanoate", "methyl formate")
  CALL Species("HCOOC_2H_5",                           "109-94-4", "WBJINCZRORDGAQ-UHFFFAOYSA-N", "ethyl methanoate", "ethyl formate")
  CALL Species("HCOOC_3H_7",                           "110-74-7", "KFNNIILCVOLYIR-UHFFFAOYSA-N", "propyl methanoate", "propyl formate")
  CALL Species("HCOOC_3H_7",                           "625-55-8", "RMOUBSOVHSONPZ-UHFFFAOYSA-N", "isopropyl methanoate", "isopropyl formate")
  CALL Species("HCOOC_4H_9",                           "592-84-7", "NMJJFJNHVMGPGM-UHFFFAOYSA-N", "butyl methanoate", "butyl formate")
  CALL Species("C_5H_{10}O_2",                         "589-40-2", "OAEQYDZVVPONKW-UHFFFAOYSA-N", "(1-methylpropyl)-methanoate", "{\it sec}-butyl formate")
  CALL Species("HCOOC_4H_9",                           "542-55-2", "AVMSWPWPYJVYKY-UHFFFAOYSA-N", "(2-methylpropyl)-methanoate", "isobutyl formate")
  CALL Species("HCOOC_4H_9",                           "762-75-4", "RUPAXCPQAAOIPB-UHFFFAOYSA-N", "(1,1-dimethylethyl)-methanoate", "{\it tert}-butyl formate; TBF")
  CALL Species("C_6H_{12}O_2",                         "638-49-3", "DIQMPQMYFZXDAX-UHFFFAOYSA-N", "pentyl methanoate", "pentyl formate")
  CALL Species("C_6H_{12}O_2",                       "58368-67-5", "YMJOAYHERILFIM-UHFFFAOYSA-N", "3-pentyl methanoate", "3-pentyl formate")
  CALL Species("C_6H_{12}O_2",                         "757-88-0", "SVZIJXUUTNJSEJ-UHFFFAOYSA-N", "1,1-dimethylpropyl methanoate", "1,1-dimethylpropyl formate")
  CALL Species("C_6H_{12}O_2",                       "66794-46-5", "XEQSPMCNFJQDHU-UHFFFAOYSA-N", "1,2-dimethylpropyl methanoate", "1,2-dimethylpropyl formate")
  CALL Species("C_6H_{12}O_2",                       "58368-66-4", "JNRQSKFTIYCDIP-UHFFFAOYSA-N", "1-methylbutyl methanoate", "1-methylbutyl formate")
  CALL Species("C_6H_{12}O_2",                       "23361-67-3", "DGMIPKNXUDSQGI-UHFFFAOYSA-N", "2,2-dimethylpropyl methanoate", "2,2-dimethylpropyl formate")
  CALL Species("C_6H_{12}O_2",                       "35073-27-9", "DWORILFBIRYUDC-UHFFFAOYSA-N", "2-methylbutyl methanoate", "2-methylbutyl formate")
  CALL Species("HCOOC_5H_{11}",                        "110-45-2", "XKYICAQFSCFURC-UHFFFAOYSA-N", "3-methylbutyl methanoate", "isoamyl formate")
  CALL Species("C_7H_{14}O_2",                         "629-33-4", "OUGPMNMLWKSBRI-UHFFFAOYSA-N", "hexyl methanoate", "hexyl formate")
  CALL Species("C_8H_{16}O_2",                         "112-23-2", "XEAMDSXSXYAICO-UHFFFAOYSA-N", "heptyl methanoate", "heptyl formate")
  CALL Species("C_9H_{18}O_2",                         "112-32-3", "AVBRYQRTMPHARE-UHFFFAOYSA-N", "octyl methanoate", "octyl formate")
  CALL Species("C_{10}H_{20}O_2",                     "5451-92-3", "MOGJOLMSQWGXPA-UHFFFAOYSA-N", "nonyl methanoate", "nonyl formate")
  CALL Species("C_{11}H_{22}O_2",                     "5451-52-5", "BCLJZFLDSCTULJ-UHFFFAOYSA-N", "decyl methanoate", "decyl formate")
  CALL Species("C_{12}H_{24}O_2",                     "5454-24-0", "OASFNORBKVGDRW-UHFFFAOYSA-N", "undecyl methanoate", "undecyl formate")
  CALL Species("C_{13}H_{26}O_2",                    "28303-42-6", "WPSGFSPBRBRLIQ-UHFFFAOYSA-N", "dodecyl methanoate", "dodecyl formate")
  CALL Species("C_3H_4O_2",                            "692-45-5", "GFJVXXWOPWLRNU-UHFFFAOYSA-N", "vinyl methanoate", "vinyl formate")
  CALL Species("C_8H_8O_2",                            "104-57-4", "UYWQUFXKFGHYNT-UHFFFAOYSA-N", "benzyl methanoate", "benzyl formate")
  ! ethanoate:
  CALL Species("CH_3COOCH_3",                           "79-20-9", "KXKVLQRXCPHEJC-UHFFFAOYSA-N", "methyl ethanoate", "methyl acetate")
  CALL Species("CH_3COOC_2H_5",                        "141-78-6", "XEKOWRVHYACXOJ-UHFFFAOYSA-N", "ethyl ethanoate", "ethyl acetate")
  CALL Species("CH_3COOC_2H_5",                       "3424-59-7", "XEKOWRVHYACXOJ-AZXPZELESA-N", "ethyl ethanoate-1-13C", "ethyl acetate-1-13C")
  CALL Species("CH_3COOC_3H_7",                        "109-60-4", "YKYONYBAUNKHLG-UHFFFAOYSA-N", "propyl ethanoate", "propyl acetate")
  CALL Species("C_4H_8O_3",                           "6290-49-9", "QRMHDGWGLNLHMN-UHFFFAOYSA-N", "methyl methoxyacetate", "")
  CALL Species("C_5H_8O_2",                            "591-87-7", "FWZUNOYOVVKUNF-UHFFFAOYSA-N", "2-propenyl ethanoate", "")
  CALL Species("CH_3COOC_3H_7",                        "108-21-4", "JMMWKPVZQRWMSS-UHFFFAOYSA-N", "2-propyl ethanoate", "isopropyl acetate")
  CALL Species("C_5H_{10}O_3",                         "110-49-6", "XLLIQLLCWZCATF-UHFFFAOYSA-N", "2-methoxyethyl ethanoate", "methyl cellosolve acetate")
  CALL Species("C_5H_{10}O_4",                       "26446-35-5", "KMZHZAAOEWVPSE-UHFFFAOYSA-N", "glycerol monoacetate", "acetin")
  CALL Species("C_5H_8O_2",                            "108-22-5", "HETCEOQFVDFGSY-UHFFFAOYSA-N", "1-propen-2-ol, acetate", "isopropenyl acetate")
  CALL Species("CH_3COOC_4H_9",                        "123-86-4", "DKPFZGUDAPQIHT-UHFFFAOYSA-N", "butyl ethanoate", "butyl acetate")
  CALL Species("C_6H_{12}O_2",                         "105-46-4", "DCKVNWZUADLDEH-UHFFFAOYSA-N", "2-butyl ethanoate", "{\it sec}-butyl acetate")
  CALL Species("C_6H_{12}O_2",                         "540-88-5", "WMOVHXAZOJBABW-UHFFFAOYSA-N", "1,1-dimethylethyl ethanoate", "{\it tert}-butyl acetate")
  CALL Species("CH_3COOC_4H_9",                        "110-19-0", "GJRQTCIYDGXPES-UHFFFAOYSA-N", "2-methylpropyl ethanoate", "isobutyl acetate")
  CALL Species("CH_3COOC_5H_{11}",                     "628-63-7", "PGMYKACGEOXYJE-UHFFFAOYSA-N", "pentyl ethanoate", "amyl acetate")
  CALL Species("C_7H_{14}O_2",                         "624-41-9", "XHIUFYZDQBSEMF-UHFFFAOYSA-N", "2-methylbutyl ethanoate", "2-methylbutyl acetate")
  CALL Species("C_7H_{14}O_2",                        "5343-96-4", "ZLSXRPTWWRGMTJ-UHFFFAOYSA-N", "1,2-dimethylpropyl ethanoate", "1,2-dimethylpropyl acetate")
  CALL Species("C_7H_{14}O_2",                         "620-11-1", "PBKYSIMDORTIEU-UHFFFAOYSA-N", "1-ethylpropyl ethanoate", "1-ethylpropyl acetate")
  CALL Species("C_7H_{14}O_2",                         "625-16-1", "JCCIFDCPHCKATH-UHFFFAOYSA-N", "1,1-dimethylpropyl ethanoate", "{\it tert}-pentyl acetate")
  CALL Species("C_7H_{14}O_2",                         "926-41-0", "QLNYTCSELYEEPV-UHFFFAOYSA-N", "2,2-dimethylpropyl ethanoate", "neopentyl acetate")
  CALL Species("C_7H_{12}O_4",                         "623-84-7", "MLHOXUWWKVQEJB-UHFFFAOYSA-N", "1,2-propanediol, diacetate", "")
  CALL Species("C_7H_{14}O_2",                         "626-38-0", "GQKZRWSUJHVIPE-UHFFFAOYSA-N", "2-pentanol, acetate", "")
  CALL Species("CH_3COOC_5H_{11}",                     "123-92-2", "MLFHJEHSLIIPHL-UHFFFAOYSA-N", "3-methylbutyl ethanoate", "isoamyl acetate")
  CALL Species("CH_3COOC_6H_{13}",                     "142-92-7", "AOGQPLXWSUTHQB-UHFFFAOYSA-N", "hexyl ethanoate", "hexyl acetate")
  CALL Species("C_8H_{16}O_2",                         "108-84-9", "CPIVYSAVIPTCCX-UHFFFAOYSA-N", "4-methyl-2-pentyl ethanoate", "4-methyl-2-pentyl acetate")
  CALL Species("C_8H_{14}O_2",                         "622-45-7", "YYLLIJHXUHJATK-UHFFFAOYSA-N", "cyclohexyl ethanoate", "cyclohexyl acetate")
  CALL Species("C_8H_{16}O_4",                         "112-15-2", "FPZWZCWUIYYYBU-UHFFFAOYSA-N", "ethanol, 2-(2-ethoxyethoxy)-, acetate", "diethylene glycol monoethyl ether acetate")
  CALL Species("C_8H_8O_2",                            "122-79-2", "IPBVNPXQWQGGJP-UHFFFAOYSA-N", "phenyl ethanoate", "phenyl acetate")
  CALL Species("C_9H_{18}O_2",                         "112-06-1", "ZCZSIDMEHXZRLG-UHFFFAOYSA-N", "heptyl ethanoate", "heptyl acetate")
  CALL Species("C_9H_{10}O_2",                         "140-11-4", "QUKGYYKBILRGFE-UHFFFAOYSA-N", "phenylmethyl ethanoate", "benzyl acetate")
  CALL Species("CH_3COOC_8H_{17}",                     "112-14-1", "YLYBTZIQSIBWLI-UHFFFAOYSA-N", "octyl ethanoate", "octyl acetate")
  CALL Species("C_{10}H_{20}O_2",                      "103-09-3", "WOYWLLHHWAMFCB-UHFFFAOYSA-N", "2-ethylhexyl ethanoate", "2-ethylhexyl acetate")
  CALL Species("C_{11}H_{22}O_2",                      "143-13-5", "GJQIMXVRFNLMTB-UHFFFAOYSA-N", "nonyl ethanoate", "nonyl acetate")
  CALL Species("C_{12}H_{24}O_2",                      "112-17-4", "NUPSHWCALHZGOV-UHFFFAOYSA-N", "decyl ethanoate", "decyl acetate")
  CALL Species("C_{13}H_{26}O_2",                     "1731-81-3", "CKQGCFFDQIFZFA-UHFFFAOYSA-N", "undecyl ethanoate", "undecyl acetate")
  CALL Species("C_{10}H_{18}O_6",                      "111-21-7", "OVOUKWFJRHALDD-UHFFFAOYSA-N", "ethanol, 2,2'-[1,2-ethanediylbis(oxy)]bis-, diacetate", "triethylene glycol, diacetate")
  CALL Species("C_6H_{12}O_3",                         "108-65-6", "LLHKCFNBLRBOGN-UHFFFAOYSA-N", "1-methoxy-2-propyl ethanoate", "1-methoxy-2-propyl acetate")
  CALL Species("C_6H_{12}O_3",                         "111-15-9", "SVONRAPFKPVNKG-UHFFFAOYSA-N", "2-ethoxyethyl ethanoate", "2-ethoxyethyl acetate")
  CALL Species("C_8H_{16}O_3",                         "112-07-2", "NQBXSWAWVZHKBZ-UHFFFAOYSA-N", "2-butoxyethyl ethanoate", "butyl cellosolve acetate")
  CALL Species("C_{10}H_{20}O_4",                      "124-17-4", "VXQBJTKSVGFQOL-UHFFFAOYSA-N", "2-(2-butoxyethoxy)-ethanol, ethanoate", "")
  CALL Species("C_6H_{10}O_4",                         "111-55-7", "JTXMVXSTHSMVQF-UHFFFAOYSA-N", "1,2-ethanediol, diethanoate", "")
  CALL Species("C_{12}H_{20}O_2",                      "105-87-3", "HIGQPQRQIQDZMP-DHZHZOJOSA-N", "geranyl acetate", "")
  CALL Species("C_{12}H_{20}O_2",                      "115-95-7", "UWKAYLJWKGQEPM-UHFFFAOYSA-N", "linalyl acetate", "")
  CALL Species("C_{12}H_{22}O_2",                    "16409-45-3", "XHXUANMFYXWVNG-UHFFFAOYSA-N", "cyclohexanol, 5-methyl-2-(1-methylethyl)-, acetate", "menthyl acetate")
  ! propanoate:
  CALL Species("C_2H_5COOCH_3",                        "554-12-1", "RJUFJBKOKNCXHH-UHFFFAOYSA-N", "methyl propanoate", "methyl propionate")
  CALL Species("C_4H_8O_3",                            "547-64-8", "LPEKGGXMPWTOCB-UHFFFAOYSA-N", "methyl 2-hydroxypropanoate", "methyl lactate")
  CALL Species("C_2H_5COOC_2H_5",                      "105-37-3", "FKRCODPIKNYEAC-UHFFFAOYSA-N", "ethyl propanoate", "ethyl propionate")
  CALL Species("C_2H_5COOC_3H_7",                      "106-36-5", "MCSINKKTEDDPNK-UHFFFAOYSA-N", "propyl propanoate", "propyl propionate")
  CALL Species("C_2H_5COOC_3H_7",                      "637-78-5", "IJMWOMHMDSDKGK-UHFFFAOYSA-N", "isopropyl propanoate", "isopropyl propionate")
  CALL Species("C_7H_{14}O_2",                         "590-01-2", "BTMVHUNTONAYDX-UHFFFAOYSA-N", "butyl propanoate", "butyl propionate")
  CALL Species("C_7H_{14}O_2",                         "540-42-1", "FZXRXKLUIMKDEL-UHFFFAOYSA-N", "(2-methylpropyl)-propanoate", "isobutyl propionate")
  CALL Species("C_7H_{14}O_2",                       "20487-40-5", "JAELLLITIZHOGQ-UHFFFAOYSA-N", "{\it tert}-butyl propanoate", "")
  CALL Species("C_2H_5COOC_5H_{11}",                   "624-54-4", "TWSRVQVEYJNFKQ-UHFFFAOYSA-N", "pentyl propanoate", "amyl propionate")
  CALL Species("C_5H_{10}O_3",                          "97-64-3", "LZCLXQDLBQLTDK-UHFFFAOYSA-N", "propanoic acid, 2-hydroxy-, ethyl ester", "ethyl lactate")
  CALL Species("C_7H_{14}O_3",                         "763-69-9", "BHXIWUJLHYHGSJ-UHFFFAOYSA-N", "ethyl 3-ethoxypropanoate", "ethyl 3-ethoxypropionate")
  CALL Species("C_{11}H_{14}O_2",                      "122-70-3", "HVGZQCSMLUDISR-UHFFFAOYSA-N", "propanoic acid, 2-phenylethyl ester", "")
  ! butanoate:
  CALL Species("C_3H_7COOCH_3",                        "623-42-7", "UUIQMZJEGPQKFD-UHFFFAOYSA-N", "methyl butanoate", "methyl butyrate")
  CALL Species("C_3H_7COOC_2H_5",                      "105-54-4", "OBNCKNCVKJNDBV-UHFFFAOYSA-N", "ethyl butanoate", "ethyl butyrate")
  CALL Species("C_3H_7COOC_3H_7",                      "105-66-8", "HUAZGNHGCJGYNP-UHFFFAOYSA-N", "propyl butanoate", "propyl butyrate")
  CALL Species("C_7H_{14}O_2",                         "638-11-9", "FFOPEPMHKILNIT-UHFFFAOYSA-N", "2-propyl butanoate", "isopropyl butyrate")
  CALL Species("C_8H_{16}O_2",                         "109-21-7", "XUPYJHCZDLZNFP-UHFFFAOYSA-N", "butyl butanoate", "butyl butyrate")
  CALL Species("C_8H_{16}O_2",                         "539-90-2", "RGFNRWTWDWVHDD-UHFFFAOYSA-N", "2-methylpropyl butanoate", "2-methylpropyl butyrate")
  CALL Species("C_8H_{16}O_2",                          "97-85-8", "RXGUIWHIADMCFC-UHFFFAOYSA-N", "2-methylpropyl 2-methylpropanoate", "isobutyl isobutyrate")
  CALL Species("C_9H_{18}O_2",                         "540-18-1", "CFNJLPHOBMVMNS-UHFFFAOYSA-N", "pentyl butanoate", "pentyl butyrate")
  CALL Species("C_9H_{18}O_2",                         "106-27-4", "PQLMXFQTAMDXIZ-UHFFFAOYSA-N", "3-methylbutyl butanoate", "isopentyl butyrate")
  CALL Species("C_5H_{10}O_2",                         "547-63-7", "BHIWKHZACMWKOJ-UHFFFAOYSA-N", "methyl 2-methylpropanoate", "methyl isobutyrate")
  CALL Species("C_6H_{12}O_2",                          "97-62-1", "WDAXFOBOLVPGLV-UHFFFAOYSA-N", "ethyl 2-methylpropanoate", "ethyl isobutyrate")
  CALL Species("C_{10}H_{20}O_2",                     "2639-63-6", "XAPCMTMQBXLDBB-UHFFFAOYSA-N", "hexyl butanoate", "hexyl butyrate")
  CALL Species("C_{10}H_{18}O_2",                     "1551-44-6", "VZHUBBUZNIULNM-UHFFFAOYSA-N", "cyclohexyl butanoate", "cyclohexyl butyrate")
  CALL Species("C_5H_8O_3",                            "105-45-3", "WRQNANDWMGAFTP-UHFFFAOYSA-N", "3-oxobutanoic acid, methyl ester", "methylacetoacetate")
  CALL Species("C_6H_{10}O_3",                         "141-97-9", "XYIBRDXRRQCHLP-UHFFFAOYSA-N", "3-oxobutanoic acid, ethyl ester", "ethylacetoacetate")
  ! pentanoate:
  CALL Species("C_4H_9COOCH_3",                        "624-24-8", "HNBDRPTVWVGKBR-UHFFFAOYSA-N", "methyl pentanoate", "methyl valerate")
  CALL Species("C_4H_9COOC_2H_5",                      "539-82-2", "ICMAFTSLXCXHRK-UHFFFAOYSA-N", "ethyl pentanoate", "ethyl valerate")
  CALL Species("C_6H_{12}O_2",                         "598-98-1", "CNMFHDIDIMZHKY-UHFFFAOYSA-N", "methyl 2,2-dimethylpropanoate", "methyl pivalate")
  CALL Species("C_7H_{14}O_2",                        "7452-79-1", "HCRBXQFHJMCTLF-UHFFFAOYSA-N", "ethyl 2-methylbutanoate", "")
  CALL Species("C_7H_{14}O_2",                         "108-64-5", "PPXUHEORWJQRHJ-UHFFFAOYSA-N", "ethyl 3-methylbutanoate", "ethyl isovalerate")
  CALL Species("C_7H_{14}O_2",                        "3938-95-2", "HHEIMYAXCOIQCJ-UHFFFAOYSA-N", "ethyl 2,2-dimethylpropanoate", "ethyl pivalate")
  CALL Species("C_9H_{18}O_2",                         "591-68-4", "OKJADYKTJJGKDX-UHFFFAOYSA-N", "butyl pentanoate", "butyl valerate")
  CALL Species("C_{10}H_{20}O_2",                      "659-70-1", "XINCECQTMHSORG-UHFFFAOYSA-N", "3-methylbutyl 3-methylbutanoate", "isoamyl isovalerate")
  ! hexanoate:
  CALL Species("C_5H_{11}COOCH_3",                     "106-70-7", "NUKZAGXMHTUAFE-UHFFFAOYSA-N", "methyl hexanoate", "methyl caproate")
  CALL Species("C_5H_{11}COOC_2H_5",                   "123-66-0", "SHZIWNPUGXLXDT-UHFFFAOYSA-N", "ethyl hexanoate", "ethyl caproate")
  CALL Species("C_{18}H_{34}O_6",                       "95-08-9", "JEYLQCXBYFQJRO-UHFFFAOYSA-N", "2-ethylbutanoic acid, 1,2-ethanediylbis(oxy-2,1-ethanediyl) ester", "")
  ! heptanoate:
  CALL Species("C_8H_{16}O_2",                         "106-73-0", "XNCNNDVCAUWAIT-UHFFFAOYSA-N", "methyl heptanoate", "")
  CALL Species("C_6H_{13}COOC_2H_5",                   "106-30-9", "TVQGDYNRXLTQAP-UHFFFAOYSA-N", "ethyl heptanoate", "")
  ! octanoate:
  CALL Species("C_6H_{13}COOCH_3",                     "111-11-5", "JGHZJRVDZXSNKQ-UHFFFAOYSA-N", "methyl octanoate", "")
  CALL Species("C_7H_{15}COOC_2H_5",                   "106-32-1", "YYZUSRORWSJGET-UHFFFAOYSA-N", "ethyl octanoate", "")
  CALL Species("C_{22}H_{44}O_2",                      "646-13-9", "ORFWYUFLWUWSFM-UHFFFAOYSA-N", "octadecanoic acid, 2-methylpropyl ester", "isobutyl stearate")
  CALL Species("C_{22}H_{44}O_2",                      "123-95-5", "ULBTUVJTXULMLP-UHFFFAOYSA-N", "octadecanoic acid, butyl ester", "")
  ! nonanoate:
  CALL Species("C_{10}H_{20}O_2",                     "1731-84-6", "IJXHLVMUNBOGRR-UHFFFAOYSA-N", "methyl nonanoate", "")
  CALL Species("C_8H_{17}COOC_2H_5",                   "123-29-5", "BYEVBITUADOIGY-UHFFFAOYSA-N", "ethyl nonanoate", "")
  CALL Species("C_{25}H_{48}O_4",                      "103-24-2", "ZDWGXBPVPXVXMQ-UHFFFAOYSA-N", "nonanedioic acid, bis(2-ethylhexyl) ester", "di-2-ethylhexyl azelate")
  ! decanoate:
  CALL Species("C_{11}H_{22}O_2",                      "110-42-9", "YRHYCMZPEVDGFQ-UHFFFAOYSA-N", "methyl decanoate", "methyl caprate")
  CALL Species("C_9H_{19}COOC_2H_5",                   "110-38-3", "RGXWDWUGBIJHDO-UHFFFAOYSA-N", "ethyl decanoate", "")
  CALL Species("C_{14}H_{26}O_4",                      "110-40-7", "ONKUXPIBXRRIDU-UHFFFAOYSA-N", "decanedioic acid, diethyl ester", "diethyl sebacate")
  ! dodecanoate:
  CALL Species("C_{13}H_{26}O_2",                      "111-82-0", "UQDUPQYQJKYHQI-UHFFFAOYSA-N", "methyl dodecanoate", "methyl laurate")
  CALL Species("C_{14}H_{28}O_2",                      "106-33-2", "MMXKVMNBHPAILY-UHFFFAOYSA-N", "ethyl dodecanoate", "ethyl laurate")
  CALL Species("C_{15}H_{30}O_2",                     "3681-78-5", "FTBUKOLPOATXGV-UHFFFAOYSA-N", "propyl dodecanoate", "propyl laurate")
  CALL Species("C_{15}H_{30}O_3",                      "142-55-2", "BHIZVZJETFVJMJ-UHFFFAOYSA-N", "2-hydroxypropyl dodecanoate", "")
  CALL Species("C_{15}H_{30}O_3",                   "107328-11-0", "SVWZGNLBKFWCMV-UHFFFAOYSA-N", "dodecanoic acid, 2-hydroxy-1-methylethyl ester", "")
  CALL Species("C_{16}H_{32}O_2",                      "106-18-3", "NDKYEUQMPZIGFN-UHFFFAOYSA-N", "butyl dodecanoate", "butyl laurate")
  CALL Species("C_{20}H_{40}O_2",                    "20292-08-4", "LWLRMRFJCCMNML-UHFFFAOYSA-N", "2-ethylhexyl dodecanoate", "2-ethylhexyl laurate")
  ! >= tetradecanoate:
  CALL Species("C_{15}H_{30}O_2",                      "124-10-7", "ZAZKJZBWRNNLDS-UHFFFAOYSA-N", "methyl tetradecanoate", "methyl myristate")
  CALL Species("C_{17}H_{34}O_2",                      "112-39-0", "FLIACVVOZYBSBS-UHFFFAOYSA-N", "methyl hexadecanoate", "methyl palmitate")
  CALL Species("C_{19}H_{38}O_2",                      "142-91-6", "XUGNVMKQXJXZCD-UHFFFAOYSA-N", "isopropyl palmitate", "")
  CALL Species("C_{22}H_{38}O_7",                      "137-66-6", "QAQJMLQRFWZOBN-LAUBAEHRSA-N", "ascorbic palmitate", "")
  CALL Species("C_{19}H_{38}O_2",                      "112-61-8", "HPEUJPJOZXNMSJ-UHFFFAOYSA-N", "methyl octadecanoate", "methyl stearate")
  CALL Species("C_{21}H_{42}O_2",                     "1120-28-1", "QGBRLVONZXHAKJ-UHFFFAOYSA-N", "methyl eicosanoate", "methyl arachidate")
  CALL Species("C_{23}H_{46}O_2",                      "929-77-1", "QSQLTHHMFHEFIY-UHFFFAOYSA-N", "methyl docosanoate", "methyl behenate")
  ! cyclic:
  CALL Species("C_5H_8O_2",                           "2868-37-3", "PKAHQJNJPDVTDP-UHFFFAOYSA-N", "cyclopropanecarboxylic acid, methyl ester", "")
  CALL Species("C_6H_{11}COOCH_3",                    "4630-82-4", "ZQWPRMPSCMSAJU-UHFFFAOYSA-N", "cyclohexanecarboxylic acid, methyl ester", "")
  CALL Species("C_{19}H_{32}O_2",                      "301-00-8", "DVWSXZIHSUZZKJ-YSTUJMKBSA-N", "($Z$,$Z$,$Z$)-9,12,15-octadecatrienoic acid, methyl ester", "methyl linolenate")
  CALL Species("C_{19}H_{34}O_2",                      "112-63-0", "WTTJVINHCBCLGX-NQLNTKRDSA-N", "($Z$,$Z$)-9,12-octadecadienoic acid, methyl ester", "methyl linolate")
  CALL Species("C_{19}H_{36}O_2",                      "112-62-9", "QYDYPVFESGNLHU-KHPPLWFESA-N", "($Z$)-9-octadecenoic acid, methyl ester", "methyl oleate")
  CALL Species("C_{23}H_{44}O_2",                     "1120-34-9", "ZYNDJIBBPLNPOW-KHPPLWFESA-N", "($Z$)-13-docosenoic acid, methyl ester", "methyl erucate")
  CALL Species("C_9H_{16}O_2",                      "105119-22-0", "WNVCMFHPRIBNCW-SFYZADRCSA-N", "{\it trans}-quercus lactone", "{\it trans}-whisky lactone")
  CALL Species("C_{10}H_{18}O_2",                      "706-14-9", "IFYYFLINQYPWGJ-UHFFFAOYSA-N", "$\gamma$-decalactone", "")
  CALL Species("C_{12}H_{22}O_2",                     "2305-05-7", "WGPCZPLRVAWXPW-UHFFFAOYSA-N", "$\gamma$-dodecalactone", "")
  CALL Species("C_{15}H_{28}O_2",                      "106-02-5", "FKUPPRZPSYCDRS-UHFFFAOYSA-N", "oxacyclohexadecan-2-one", "pentadecalactone")
  CALL Species("C_{12}H_{22}O_2",                       "88-41-5", "FINOAUDUYKVGDS-UHFFFAOYSA-N", "2-{\it tert}-butylcyclohexyl acetate", "")
  CALL Species("C_{13}H_{22}O_3",                    "24851-98-7", "KVWWIYGFBYDJQC-UHFFFAOYSA-N", "hedione", "")
  CALL Species("C_{15}H_{26}O_4",                      "105-95-3", "XRHCAGNSDHCHFJ-UHFFFAOYSA-N", "ethylene brassylate", "")
  CALL Species("C_{15}H_{26}O_3",                   "158474-72-7", "IPDFPNNPBMREIF-UHFFFAOYSA-N", "propyl 3-oxo-2-pentylcyclopentaneacetate", "")
  ! other saturated:
  CALL Species("C_8H_{14}O_3",                         "607-97-6", "OKANYBNORCUPKZ-UHFFFAOYSA-N", "2-ethyl-3-oxo-butanoic acid, ethyl ester", "")
  CALL Species("C_7H_{14}O_3",                         "138-22-7", "MRABAEUHTLLEML-UHFFFAOYSA-N", "2-hydroxypropanoic acid, butyl ester", "")
  ! unsaturated:
  CALL Species("C_4H_6O_2",                             "96-33-3", "BAPJBEWLBFYGME-UHFFFAOYSA-N", "methyl propenoate", "methyl acrylate")
  CALL Species("C_5H_8O_2",                            "140-88-5", "JIGUQPWFLRLWPJ-UHFFFAOYSA-N", "ethyl propenoate", "ethyl acrylate")
  CALL Species("C_7H_{12}O_2",                         "141-32-2", "CQEYYJKEWSMYFG-UHFFFAOYSA-N", "2-propenoic acid, butyl ester", "butyl acrylate")
  CALL Species("C_7H_{12}O_2",                         "106-63-8", "CFVWNXQPGQOHRJ-UHFFFAOYSA-N", "2-propenoic acid, 2-methylpropyl ester", "isobutyl acrylate")
  CALL Species("C_{11}H_{20}O_2",                      "103-11-7", "GOXQRTZXKQZDDN-UHFFFAOYSA-N", "2-propenoic acid, 2-ethylhexyl ester", "2-ethylhexyl acrylate")
  CALL Species("C_5H_8O_3",                            "818-61-1", "OMIGHNLMNHATMP-UHFFFAOYSA-N", "2-propenoic acid, 2-hydroxyethyl ester", "2-hydroxyethyl acrylate")
  CALL Species("C_6H_{10}O_2",                          "97-63-2", "SUPCQIBBMFXVTL-UHFFFAOYSA-N", "2-methyl-2-propenoic acid, ethyl ester", "")
  CALL Species("C_7H_{10}O_2",                          "96-05-9", "FBCQUCJYYPMKRO-UHFFFAOYSA-N", "2-methyl-2-propenoic acid, 2-propenyl ester", "allyl methacrylate")
  CALL Species("C_7H_{10}O_3",                         "106-91-2", "VOZRXNHHFUQHIL-UHFFFAOYSA-N", "2-methyl-2-propenoic acid, oxiranylmethyl ester", "glycidyl methacrylate")
  CALL Species("C_7H_{10}O_3",                        "1118-84-9", "AXLMPTNTPOWPLT-UHFFFAOYSA-N", "allyl acetoacetate", "")
  CALL Species("C_7H_{12}O_2",                        "2210-28-8", "NHARPDSAXCBDDR-UHFFFAOYSA-N", "2-methyl-2-propenoic acid, propyl ester", "propyl methacrylate")
  CALL Species("C_8H_{14}O_2",                          "97-88-1", "SOGAXMICEFXMKE-UHFFFAOYSA-N", "2-methyl-2-propenoic acid, butyl ester", "butyl methacrylate")
  CALL Species("C_8H_{14}O_2",                          "97-86-9", "RUMACXVDVNRZJZ-UHFFFAOYSA-N", "2-methyl-2-propenoic acid, 2-methylpropyl ester", "")
  CALL Species("C_{14}H_{22}O_6",                      "109-16-0", "HWSSEYVMGDIFMH-UHFFFAOYSA-N", "2-methyl-2-propenoic acid, 1,2-ethanediylbis(oxy-2,1-ethanediyl) ester", "")
  CALL Species("C_5H_8O_2",                             "80-62-6", "VVQNEPGJFQJSBK-UHFFFAOYSA-N", "methyl methacrylate", "")
  CALL Species("C_8H_{14}O_2",                        "3681-82-1", "NPFVOOAXDOBMCE-SNAWJCMRSA-N", "($E$)-3-hexenyl ethanoate", "")
  CALL Species("C_8H_{14}O_2",                        "3681-71-8", "NPFVOOAXDOBMCE-PLNGDYQASA-N", "($Z$)-3-hexenyl ethanoate", "")
  CALL Species("CH_3COOCHCH_2",                        "108-05-4", "XTXRWKRVRITETP-UHFFFAOYSA-N", "ethenyl ethanoate", "vinyl acetate")
  CALL Species("C_{14}H_{24}O_2",                    "54364-62-4", "LLRZUAWETKPZJO-SCFJQAPRSA-N", "(7$E$,9$Z$)-dodecadienyl acetate", "")
  CALL Species("C_{16}H_{28}O_2",                    "30507-70-1", "ZZGJZGSVLNSDPG-FDTUMDBZSA-N", "(9$Z$,12$E$)-9,12-tetradecadienyl acetate", "")
  CALL Species("C_{16}H_{28}O_2",                    "28645-51-4", "QILMAYXCYBTEDM-IWQZZHSRSA-N", "isoambrettolide", "")
  CALL Species("C_{17}H_{26}O_2",                      "117-98-6", "UAVFEMBKDRODDE-UHFFFAOYSA-N", "vetyveryl acetate", "")
  CALL Species("C_{17}H_{30}O_2",                    "41096-46-2", "FYQGBXGJFWXIPP-UEVLXMDPSA-N", "hydroprene", "")
  CALL Species("C_{18}H_{26}O_2",                    "54406-48-3", "YUGWDVYLFSETPE-UHFFFAOYSA-N", "empenthrin", "")
  CALL Species("C_{18}H_{26}O_3",                     "5466-77-3", "YBGZDTIWKVFICR-JLHYYAGUSA-N", "3-(4-methoxyphenyl)-2-propenoic acid, 2-ethylhexyl ester", "octinoxate")
  CALL Species("C_{18}H_{28}O_2",                    "42588-37-4", "FZRBKIRIBLNOAM-WHVZTFIZSA-N", "kinoprene", "")
  CALL Species("C_{19}H_{26}O_3",                      "584-79-2", "ZCVAOQKBXKSDMS-UHFFFAOYSA-N", "allethrin", "")
  CALL Species("C_{19}H_{34}O_3",                    "65733-16-6", "NFGXHKASABOEEW-GYMWBFJFSA-N", "S-methoprene", "")
  ! aromatic:
  CALL Species("C_6H_5COOCH_3",                         "93-58-3", "QPJVMBTYPHYUOC-UHFFFAOYSA-N", "methyl benzoate", "")
  CALL Species("C_6H_5COOC_2H_5",                       "93-89-0", "MTZQAGJQAFMTAQ-UHFFFAOYSA-N", "ethyl benzoate", "")
  CALL Species("C_8H_8O_3",                            "119-36-8", "OSWPMRLSEDHDFF-UHFFFAOYSA-N", "2-hydroxybenzoic acid methyl ester", "methyl salicylate")
  CALL Species("C_9H_{10}O_2",                          "99-75-2", "QSSJZLPUHJDYKF-UHFFFAOYSA-N", "benzoic acid, 4-methyl-, methyl ester", "")
  CALL Species("C_9H_8O_4",                             "50-78-2", "BSYNRYMUTXBXSQ-UHFFFAOYSA-N", "acetylsalicylic acid ", "aspirin")
  CALL Species("C_{10}H_{10}O_4",                      "120-61-6", "WOZVHXUHUFLZGK-UHFFFAOYSA-N", "1,4-benzenedicarboxylic acid, dimethyl ester", "")
  CALL Species("C_{11}H_{14}O_2",                      "136-60-7", "XSIFPSYPOVKYCO-UHFFFAOYSA-N", "butyl benzoate", "")
  CALL Species("C_{13}H_{10}O_3",                      "102-09-0", "ROORDVPLFPIABK-UHFFFAOYSA-N", "diphenyl carbonate", "")
  CALL Species("C_{14}H_{12}O_2",                      "120-51-4", "SESFRYSPDFLNCH-UHFFFAOYSA-N", "benzyl benzoate", "")
  CALL Species("C_{11}H_{12}O_2",                      "103-36-6", "KBEBGUQPQBELIU-CMDGGOBGSA-N", "{\it trans}-ethylcinnamate", "")
  CALL Species("C_{20}H_{22}O_5",                    "27138-31-4", "CGLQOIMEUPORRI-UHFFFAOYSA-N", "dipropylene glycol dibenzoate", "")
  ! aromatic phthalates:
  CALL Species("C_{10}H_{10}O_4",                      "131-11-3", "NIQCNGHVCWTJSM-UHFFFAOYSA-N", "dimethyl phthalate", "")
  CALL Species("C_{10}H_{16}O_4",                       "94-60-0", "LNGAGQAGYITKCW-UHFFFAOYSA-N", "1,4-cyclohexanedicarboxylic acid, dimethyl ester", "dimethyl hexahydroterephthalate")
  CALL Species("C_{10}H_{10}O_4",                     "1459-93-4", "VNGOYPQMJFJDLV-UHFFFAOYSA-N", "1,3-benzenedicarboxylic acid, dimethyl ester", "dimethyl isophthalate")
  CALL Species("C_{12}H_{14}O_4",                       "84-66-2", "FLKPEMZONWLCSK-UHFFFAOYSA-N", "diethyl phthalate", "")
  CALL Species("C_{14}H_{18}O_4",                      "131-16-8", "MQHNKCZKNAJROC-UHFFFAOYSA-N", "dipropyl phthalate", "")
  CALL Species("C_{14}H_{14}O_4",                      "131-17-9", "QUDWYFHPNIMBFC-UHFFFAOYSA-N", "diallyl phthalate", "")
  CALL Species("C_{14}H_{18}O_5",                      "117-82-8", "HSUIVCLOAAJSRE-UHFFFAOYSA-N", "bis(2-methoxyethyl) phthalate", "")
  CALL Species("C_{16}H_{22}O_4",                       "84-74-2", "DOIRQSBPFJWKBE-UHFFFAOYSA-N", "dibutyl phthalate", "")
  CALL Species("C_{16}H_{22}O_4",                       "84-69-5", "MGWAVDBGNNKXQV-UHFFFAOYSA-N", "diisobutyl phthalate", "")
  CALL Species("C_{18}H_{24}O_4",                       "84-64-0", "BHKLONWXRPJNAE-UHFFFAOYSA-N", "1,2-benzenedicarboxylic acid, butyl cyclohexyl ester", "butyl cyclohexyl phthalate")
  CALL Species("C_{18}H_{24}O_6",                       "85-70-1", "GOJCZVPJCKEBQV-UHFFFAOYSA-N", "butyl glycolyl butyl phthalate", "")
  CALL Species("C_{18}H_{26}O_4",                      "131-18-0", "IPKKHRVROFYTEK-UHFFFAOYSA-N", "diamyl phthalate", "")
  CALL Species("C_{19}H_{20}O_4",                       "85-68-7", "IRIAEXORFWYRCZ-UHFFFAOYSA-N", "butyl benzyl phthalate", "")
  CALL Species("C_{20}H_{30}O_4",                       "84-75-3", "KCXZNSGUUQJJTR-UHFFFAOYSA-N", "dihexyl phthalate", "")
  CALL Species("C_{20}H_{30}O_4",                       "85-69-8", "AVOLBYOSCILFLL-UHFFFAOYSA-N", "butyl 2-ethylhexyl phthalate", "")
  CALL Species("C_{20}H_{14}O_4",                     "1539-04-4", "HPGJOUYGWKFYQW-UHFFFAOYSA-N", "diphenyl terephthalate", "")
  CALL Species("C_{20}H_{26}O_4",                       "84-61-7", "VOWAEIGWURALJQ-UHFFFAOYSA-N", "dicyclohexyl phthalate", "")
  CALL Species("C_{20}H_{30}O_6",                      "117-83-9", "CMCJNODIWQEOAI-UHFFFAOYSA-N", "bis(2-butoxyethyl) phthalate", "")
  CALL Species("C_{22}H_{34}O_4",                     "3648-21-3", "JQCXWCOOWVGKMT-UHFFFAOYSA-N", "diheptyl phthalate", "") ! other CAS: 68515-44-6
  CALL Species("C_{24}H_{38}O_4",                      "117-84-0", "MQIUGAXCHLFZKX-UHFFFAOYSA-N", "dioctyl phthalate", "")
  CALL Species("C_{24}H_{38}O_4",                    "27554-26-3", "IJFPVINAQGWBRJ-UHFFFAOYSA-N", "diisooctyl phthalate", "")
  CALL Species("C_{24}H_{38}O_4",                    "25724-58-7", "OMQBXAQAHHFSST-UHFFFAOYSA-N", "decyl hexyl phthalate", "")
  CALL Species("C_{24}H_{38}O_4",                      "117-81-7", "BJQHLKABXJIVAM-UHFFFAOYSA-N", "bis(2-ethylhexyl)-phthalate", "DEHP")
  CALL Species("C_{24}H_{38}O_4",                     "6422-86-2", "RWPICVVBGZBXNA-UHFFFAOYSA-N", "bis(2-ethylhexyl) terephthalate", "")
  CALL Species("C_{26}H_{42}O_4",                       "84-76-4", "DROMNWUQASBTFM-UHFFFAOYSA-N", "dinonyl phthalate", "") ! other CAS: 68515-45-7
  CALL Species("C_{26}H_{42}O_4",                    "28553-12-0", "HBGGXOJOCNVPFY-UHFFFAOYSA-N", "diisononyl phthalate", "")
  CALL Species("C_{28}H_{46}O_4",                       "84-77-5", "PGIBJVOPLXHHGS-UHFFFAOYSA-N", "didecyl phthalate", "")
  CALL Species("C_{28}H_{46}O_4",                    "26761-40-0", "ZVFDTKUVRCTHQE-UHFFFAOYSA-N", "diisodecyl phthalate", "")
  CALL Species("C_{30}H_{50}O_4",                     "3648-20-2", "QQVHEQUEHCEAKS-UHFFFAOYSA-N", "diundecyl phthalate", "")
  CALL Species("C_{34}H_{58}O_4",                      "119-06-2", "YCZJVRCZIPDYHH-UHFFFAOYSA-N", "ditridecyl phthalate", "")
  ! diester:
  CALL Species("C_4H_6O_4",                            "553-90-2", "LOMVENUNSWAXEN-UHFFFAOYSA-N", "ethanedioic acid, dimethyl ester", "dimethyl oxalate")
  CALL Species("C_6H_{10}O_4",                          "95-92-1", "WYACBZDAHNBPPB-UHFFFAOYSA-N", "ethanedioic acid, diethyl ester", "")
  CALL Species("C_5H_8O_4",                            "108-59-8", "BEPAFCGSDWSTEL-UHFFFAOYSA-N", "propanedioic acid, dimethyl ester", "dimethyl malonate")
  CALL Species("C_7H_{12}O_4",                         "105-53-3", "IYXGSMUGOJNHAZ-UHFFFAOYSA-N", "propanedioic acid, diethyl ester", "diethyl malonate")
  CALL Species("C_6H_{10}O_4",                         "106-65-0", "MUXOBHXGJLMRAB-UHFFFAOYSA-N", "butanedioic acid, dimethyl ester", "dimethyl succinate")
  CALL Species("C_8H_{14}O_4",                         "123-25-1", "DKMROQRQHGEIOW-UHFFFAOYSA-N", "diethyl succinate", "")
  CALL Species("C_6H_8O_4",                            "624-48-6", "LDCRTTXIJACKKU-ARJAWSKDSA-N", "($Z$)-2-butenedioic acid dimethyl ester", "dimethyl maleate")
  CALL Species("C_6H_8O_4",                            "624-49-7", "LDCRTTXIJACKKU-ONEGZZNKSA-N", "($E$)-2-butenedioic acid dimethyl ester", "dimethyl fumarate")
  CALL Species("C_7H_{12}O_4",                        "1119-40-0", "XTDYIOOONNVFMA-UHFFFAOYSA-N", "pentanedioic acid, dimethyl ester", "")
  CALL Species("C_9H_{16}O_4",                         "818-38-2", "OUWSNHWQZPEFEX-UHFFFAOYSA-N", "diethyl glutarate", "")
  CALL Species("C_{11}H_{20}O_4",                     "2050-20-6", "LKKOGZVQGQUVHF-UHFFFAOYSA-N", "diethyl pimelate", "")
  CALL Species("C_{12}H_{14}O_4",                      "636-53-3", "JLVWYWVLMFVCDI-UHFFFAOYSA-N", "1,3-benzenedicarboxylic acid, diethyl ester", "")
  CALL Species("C_{12}H_{20}O_4",                      "105-76-0", "JBSLOWBPDRZSMB-FPLPWBNLSA-N", "dibutyl maleate", "")
  CALL Species("C_{12}H_{22}O_4",                      "106-79-6", "ALOUNLDAKADEEB-UHFFFAOYSA-N", "dimethyl sebacate", "decanedioic acid, dimethyl ester")
  CALL Species("C_{14}H_{12}O_4",                      "840-65-3", "GYUVMLBYMPKZAZ-UHFFFAOYSA-N", "2,6-naphthalenedicarboxylic acid, dimethyl ester", "")
  !CALL Species("C_{22}H_{42}O_4",                     "123-79-5", "NEHDRDVHPTWWFG-UHFFFAOYSA-N", "hexanedioic acid, dioctyl ester", "di-n-octyl adipate")
  CALL Species("C_{22}H_{42}O_4",                     "1330-86-5", "CJFLBOQMPJCWLR-UHFFFAOYSA-N", "diisooctyl adipate", "")
  CALL Species("C_{22}H_{42}O_4",                      "103-23-1", "SAOKZLXYCUGLFA-UHFFFAOYSA-N", "di-(2-ethylhexyl)-adipate", "")
  ! peroxy ester:
  CALL Species("C_{11}H_{14}O_3",                      "614-45-9", "GJBRNHKUVLOCEB-UHFFFAOYSA-N", "peroxybenzoic acid, {\it tert}-butyl ester", "")
  CALL Species("C_{14}H_{28}O_3",                    "26748-41-4", "NMOALOSNPWTWRH-UHFFFAOYSA-N", "neodecaneperoxoic acid, 1,1-dimethylethyl ester", "")
  CALL Species("C_{16}H_{24}O_3",                   "130097-36-8", "WFAUFYAGXAXBEG-UHFFFAOYSA-N", "neoheptaneperoxoic acid, 1-methyl-1-phenylethyl ester", "")
  ! other esters:
  CALL Species("C_6H_{10}O_3",                       "25584-83-2", "AYEFIAVHMUFQPZ-UHFFFAOYSA-N", "hydroxypropyl acrylate", "")
  CALL Species("C_6H_{10}O_3",                         "868-77-9", "WOBHKFSMXKNTIM-UHFFFAOYSA-N", "2-hydroxyethyl methacrylate", "")
  CALL Species("C_6H_{10}O_3",                         "999-61-1", "GWZMWHWAWHPNHN-UHFFFAOYSA-N", "2-hydroxypropyl acrylate", "")
  CALL Species("C_8H_{14}O_4",                         "627-93-0", "UDSFAEKRVUSQDD-UHFFFAOYSA-N", "hexanedioic acid, dimethyl ester", "dimethyl adipate")
  CALL Species("C_8H_8O_3",                             "99-76-3", "LXCFILQKKLGQFO-UHFFFAOYSA-N", "methyl 4-hydroxybenzoate", "methylparaben")
  CALL Species("C_8H_{12}O_4",                         "623-91-6", "IEPRKVQEAMIZSS-AATRIKPKSA-N", "diethyl fumarate", "")
  CALL Species("C_{10}H_{18}O_4",                      "141-28-6", "VIZORQUEIQEFRT-UHFFFAOYSA-N", "diethyl adipate", "")
  CALL Species("C_{10}H_{12}O_3",                       "94-13-3", "QELSKZZBTMNZEB-UHFFFAOYSA-N", "propyl 4-hydroxybenzoate", "propylparaben")
  CALL Species("C_{10}H_{14}O_5",                     "4074-88-8", "LEJBBGNFPAFPKQ-UHFFFAOYSA-N", "diethylene glycol diacrylate", "")
  CALL Species("C_{11}H_{16}O_4",                     "2223-82-7", "MXFQRSUWYYSPOC-UHFFFAOYSA-N", "(2,2-dimethyl-3-prop-2-enoyloxypropyl) prop-2-enoate", "2,2-dimethyltrimethylene acrylate")
  CALL Species("C_{13}H_{20}O_3",                     "1211-29-6", "GEWDNTWNSAZUDX-NNOMMRTBSA-N", "methyl jasmonate", "")
  CALL Species("C_{14}H_{18}O_4",                      "104-28-9", "CMDKPGRTAQVGFQ-RMKNXTFCSA-N", "cinoxate", "")
  CALL Species("C_{15}H_{20}O_6",                    "15625-89-5", "DAKWPKUUDNSNPN-UHFFFAOYSA-N", "trimethylolpropane triacrylate", "")
  CALL Species("C_{16}H_{14}O_2",                      "103-41-3", "NGHOLYJTSCBCGC-VAWYXSNFSA-N", "benzyl cinnamate", "")
  CALL Species("C_{16}H_{30}O_4",                     "6846-50-0", "OMVSWZDEEGIJJI-UHFFFAOYSA-N", "2,2,4-trimethyl-1,3-pentanediol diisobutyrate", "")
  CALL Species("C_{17}H_{32}O_4",                     "2917-73-9", "RISLXYINQFKFRL-UHFFFAOYSA-N", "nonanedioic acid, dibutyl ester", "dibutyl azelate")
  CALL Species("C_{17}H_{34}O_2",                      "110-27-0", "AXISYYRBXTVTFY-UHFFFAOYSA-N", "isopropyl myristate", "")
  CALL Species("C_{18}H_{34}O_4",                      "109-43-3", "PYGXAGIECVVIOZ-UHFFFAOYSA-N", "decanedioic acid, dibutyl ester", "")
  CALL Species("C_{18}H_{18}O_5",                      "120-55-8", "NXQMCAOPTPLPRL-UHFFFAOYSA-N", "diethylene glycol dibenzoate", "")
  CALL Species("C_{19}H_{36}O_3",                      "141-24-2", "XKGDWZQXVZSXAO-RAXLEYEMSA-N", "12-hydroxy-9-octadecenoic acid, methyl ester", "ricinoleic acid, methyl ester")
  CALL Species("C_{19}H_{26}O_2",                       "70-38-2", "FHNKBSDJERHDHZ-UHFFFAOYSA-N", "chrysanthemumic acid 2,4-dimethylbenzyl ester", "dimethrin")
  CALL Species("C_{20}H_{34}O_8",                       "77-90-7", "QZCLKYGREBVARF-UHFFFAOYSA-N", "tributyl acetylcitrate", "acetyl tributyl citrate")
  CALL Species("C_{22}H_{42}O_8",                      "141-17-3", "SCABKEBYDRTODC-UHFFFAOYSA-N", "hexanedioic acid, bis[2-(2-butoxyethoxy)ethyl] ester", "bis(2-(2-butoxyethoxy)ethyl) adipate")
  CALL Species("C_{26}H_{42}O_4",                      "119-07-3", "LVAGMBHLXLZJKZ-UHFFFAOYSA-N", "1,2-benzenedicarboxylic acid, decyl octyl ester", "")
  CALL Species("C_{26}H_{42}O_4",                     "1330-96-7", "UXLBXTIBYMXRNX-UHFFFAOYSA-N", "phthalic acid, isodecyl octyl ester", "")
  CALL Species("C_{26}H_{48}O_4",                   "166412-78-8", "HORIEOQXBKUKGQ-UHFFFAOYSA-N", "diisononyl hexahydrophthalate", "")
  CALL Species("C_{26}H_{50}O_4",                      "122-62-3", "VJHINFRRDQUWOJ-UHFFFAOYSA-N", "decanedioic acid, bis(2-ethylhexyl) ester", "bis(2-ethylhexyl) sebacate")
  CALL Species("C_{27}H_{50}O_6",                      "538-23-8", "VLPFTAMPNXLGLX-UHFFFAOYSA-N", "glycerol tricaprylate", "tricaprylin")
  CALL Species("C_{33}H_{54}O_6",                     "3319-31-1", "KRADHMIOFJQKEZ-UHFFFAOYSA-N", "tris(2-ethylhexyl) trimellitate", "")
  ! esters from MCM:
  CALL Species("C_2H_4O_4",                                    "", "RHCLSHTZJFAXOU-UHFFFAOYSA-N", "MCM:CHOOCH2OOH", "")
  CALL Species("C_2H_2O_3",                                    "", "VGGRCVDNFAQIKO-UHFFFAOYSA-N", "MCM:CHOOCHO", "")
  CALL Species("C_3H_4O_5",                                    "", "NYMJKVCDMUPZFI-UHFFFAOYSA-N", "MCM:CHOOMCO3H", "")
  CALL Species("C_3H_6O_4",                                    "", "DJECRZORXDSOEF-UHFFFAOYSA-N", "MCM:ETHFORMOOH", "")
  CALL Species("C_3H_4O_5",                                    "", "TUWVUCWHUYJQAO-UHFFFAOYSA-N", "MCM:MECOFOROOH", "")
  CALL Species("C_3H_4O_3",                                    "", "ORWKVZNEPHTCQE-UHFFFAOYSA-N", "MCM:METACETHO", "")
  CALL Species("C_3H_6O_4",                                    "", "AIBRVUZTVKELQP-UHFFFAOYSA-N", "MCM:METACETO2H", "")
  CALL Species("C_3H_4O_4",                                    "", "IIHOUWJOSQHHHH-UHFFFAOYSA-N", "MCM:MMCF", "")
  CALL Species("C_3H_4O_6",                                    "", "LQGPNTLGGGGYKM-UHFFFAOYSA-N", "MCM:MMCFOOH", "")
  CALL Species("C_3H_6O_4",                                    "", "OQIIXPHPHGKJRP-UHFFFAOYSA-N", "MCM:MOCOCH2OOH", "")
  CALL Species("C_4H_8O_4",                                    "", "JNHNRXKAKFYQGM-UHFFFAOYSA-N", "MCM:ACEC2H4OOH", "")
  CALL Species("C_4H_6O_5",                                    "", "UCKKJLQTXHILBL-UHFFFAOYSA-N", "MCM:ACETMECO3H", "")
  CALL Species("C_4H_6O_5",                                    "", "CKNWOSHDVKEZAO-UHFFFAOYSA-N", "MCM:COO2C3CO3H", "")
  CALL Species("C_4H_8O_4",                                    "", "KATJWBCCJXQUDM-UHFFFAOYSA-N", "MCM:COO2C4OOH", "")
  CALL Species("C_4H_8O_4",                                    "", "ZDXGLRBKCRTGQP-UHFFFAOYSA-N", "MCM:EOCOCH2OOH", "")
  CALL Species("C_4H_8O_4",                                    "", "FITNUWWXWGTMFO-UHFFFAOYSA-N", "MCM:ETHACETOOH", "")
  CALL Species("C_4H_8O_4",                                    "", "DHNDYBGDIQCRQB-UHFFFAOYSA-N", "MCM:IPRFORMOOH", "")
  CALL Species("C_4H_2O_3",                            "108-31-6", "FPYJFEHAWHCUMM-UHFFFAOYSA-N", "maleic anhydride", "MCM:MALANHY")
  CALL Species("C_4H_6O_5",                                    "", "ZCPJTWSQAFRVMG-UHFFFAOYSA-N", "MCM:MECOACEOOH", "")
  CALL Species("C_5H_8O_5",                                    "", "MRPVYADEHOUTJI-UHFFFAOYSA-N", "MCM:ACCOETOOH", "")
  CALL Species("C_5H_6O_6",                                    "", "OSZJUWKUAMKWAM-UHFFFAOYSA-N", "MCM:ACCOMECO3H", "")
  CALL Species("C_5H_8O_5",                                    "", "LEMDJBHCHPODAJ-UHFFFAOYSA-N", "MCM:ACETC2CO3H", "")
  CALL Species("C_5H_8O_3",                                    "", "KLUDQUOLAFVLOL-UHFFFAOYSA-N", "MCM:ACETCOC2H5", "")
  CALL Species("C_5H_{10}O_4",                                 "", "HTFWPRQCJNWCFL-UHFFFAOYSA-N", "MCM:BUFORMOOH", "")
  CALL Species("C_5H_8O_5",                                    "", "XFYUESMQFXHHIL-UHFFFAOYSA-N", "MCM:COO2C4CO3H", "")
  CALL Species("C_5H_8O_5",                                    "", "IEXBKLGSFAHQBW-UHFFFAOYSA-N", "MCM:IPRACBCO3H", "")
  CALL Species("C_5H_{10}O_4",                                 "", "HJBGHGSQOFAQDB-UHFFFAOYSA-N", "MCM:IPRACBOOH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "UMMAXRSQAGVERV-UHFFFAOYSA-N", "MCM:IPRACOOH", "")
  CALL Species("C_5H_4O_3",                                    "", "AYKYXWQEBUNJCN-UHFFFAOYSA-N", "MCM:MMALANHY", "")
  CALL Species("C_5H_{10}O_4",                                 "", "OJPAWJIXAIXDOD-UHFFFAOYSA-N", "MCM:MTBEAALOOH", "")
  CALL Species("C_5H_8O_5",                                    "", "GVHAIIVWIJOUAR-UHFFFAOYSA-N", "MCM:MTBEALCO3H", "")
  CALL Species("C_5H_{10}O_4",                                 "", "BAISPQOWBPLMQI-UHFFFAOYSA-N", "MCM:NPRACAOOH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "KUTULJFZWKOEBJ-UHFFFAOYSA-N", "MCM:NPRACBOOH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "GYDFEBDSBUDYEN-UHFFFAOYSA-N", "MCM:NPRACCOOH", "")
  CALL Species("C_5H_8O_3",                                    "", "JKMPLJUAEDQBOK-UHFFFAOYSA-N", "MCM:PRCOFORM", "")
  CALL Species("C_5H_8O_5",                                    "", "VVRRHCGIFBEYON-UHFFFAOYSA-N", "MCM:PRCOFOROOH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "JALZRJAYIBRIKJ-UHFFFAOYSA-N", "MCM:PRCOOMOOH", "")
  CALL Species("C_5H_6O_2",                                    "", "BGLUXFNVVSVEET-UHFFFAOYSA-N", "MCM:TLFUONE", "")
  CALL Species("C_6H_{10}O_5",                                 "", "HNMHVARGKGGINS-UHFFFAOYSA-N", "MCM:ACCOPROOH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "BVQHHUQLZPXYAQ-UHFFFAOYSA-N", "MCM:ACETCOC3H7", "")
  CALL Species("C_6H_{12}O_4",                                 "", "DQCMTBDKSIDVDF-UHFFFAOYSA-N", "MCM:BOCOCH2OOH", "")
  CALL Species("C_6H_6O_3",                                    "", "MFGALGYVFGDXIX-UHFFFAOYSA-N", "MCM:DMMALANHY", "")
  CALL Species("C_6H_8O_2",                                    "", "GOUILHYTHSOMQJ-UHFFFAOYSA-N", "MCM:EBFUONE", "")
  CALL Species("C_6H_6O_3",                                    "", "AXGOOCLYBPQWNG-UHFFFAOYSA-N", "MCM:EMALANHY", "")
  CALL Species("C_6H_{12}O_4",                                 "", "GZZAVFPVGHHZEL-UHFFFAOYSA-N", "MCM:MCOOTBOOH", "")
  CALL Species("C_6H_8O_2",                                    "", "SAXRUMLUKZBSTO-UHFFFAOYSA-N", "MCM:MXYFUONE", "")
  CALL Species("C_6H_{12}O_4",                                 "", "VUJYCIVMLKMZJB-UHFFFAOYSA-N", "MCM:NBUACAOOH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "ZDNJENJHBNLWSZ-UHFFFAOYSA-N", "MCM:NBUACBOOH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "CCXFYGGLIMLVCI-UHFFFAOYSA-N", "MCM:NBUACCOOH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "NIESGQMUWIIUQP-UHFFFAOYSA-N", "MCM:PRCOOMCO3H", "")
  CALL Species("C_6H_{12}O_4",                                 "", "XEFYSNUZBIEBQZ-UHFFFAOYSA-N", "MCM:SBUACAOOH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "SHPWQWOSZRJPCY-UHFFFAOYSA-N", "MCM:SBUACBOOH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "CTLNPRHGCLAPPZ-UHFFFAOYSA-N", "MCM:TBUACCO3H", "")
  CALL Species("C_6H_8O_2",                                    "", "GLCOTRLUPUIAFI-UHFFFAOYSA-N", "MCM:TMB1FUONE", "")
  CALL Species("C_7H_{10}O_2",                                 "", "LWFKYUIZWLTJCA-UHFFFAOYSA-N", "MCM:IPBFUONE", "")
  CALL Species("C_7H_8O_3",                                    "", "QSWLSAYLEATCSH-UHFFFAOYSA-N", "MCM:IPMALANHY", "")
  CALL Species("C_7H_{10}O_2",                                 "", "UVERQUDHQJYTIT-UHFFFAOYSA-N", "MCM:MEBFUONE", "")
  CALL Species("C_7H_{10}O_2",                                 "", "CLCLZCBNFSGUOD-UHFFFAOYSA-N", "MCM:PBFUONE", "")
  CALL Species("C_7H_8O_3",                                    "", "LPFJFXRQANKTRA-UHFFFAOYSA-N", "MCM:PMALANHY", "")
  CALL Species("C_7H_{10}O_2",                                 "", "OSFZDFIHIYXIEL-UHFFFAOYSA-N", "MCM:TMB2FUONE", "")
  CALL Species("C_{10}H_{18}O_4",                              "", "BQYOGPDLQGPEQM-UHFFFAOYSA-N", "MCM:C1013OOH", "")
  CALL Species("C_{11}H_{18}O_5",                              "", "ZWRGNVXICSEPNH-UHFFFAOYSA-N", "MCM:C1013CO3H", "")
  CALL Species("C_5H_6O_2",                                    "", "VGHBEMPMIVEGJP-UHFFFAOYSA-N", "MCM:PXYFUONE", "")
  CALL Species("C_6H_8O_2",                                    "", "QHQDWCHELGHSTO-UHFFFAOYSA-N", "MCM:OXYFUONE", "")
  CALL Species("C_2H_4O_3",                                    "", "APUQIHKCZFWODD-UHFFFAOYSA-N", "MCM:CHOOCH2OH", "")
  CALL Species("C_3H_6O_3",                                    "", "JRUKHAIVAGVYRP-UHFFFAOYSA-N", "MCM:ETHFORMOH", "")
  CALL Species("C_3H_6O_3",                                    "", "UKQJDWBNQNAJHB-UHFFFAOYSA-N", "MCM:ETOHOCHO", "")
  CALL Species("C_3H_6O_3",                                    "", "JYVNDCLJHKQUHE-UHFFFAOYSA-N", "MCM:METACETOH", "")
  CALL Species("C_3H_4O_5",                                    "", "XDYLYGBFMHJJOW-UHFFFAOYSA-N", "MCM:MMCFOH", "")
  CALL Species("C_3H_6O_3",                                    "", "GSJFXBNYJCXDGI-UHFFFAOYSA-N", "MCM:MOXCOCH2OH", "")
  CALL Species("C_4H_8O_5",                                    "", "QQYYYRFGUJQKRH-UHFFFAOYSA-N", "MCM:ACEETOHOOH", "")
  CALL Species("C_4H_6O_4",                                    "", "SGMJBNSHAZVGMC-UHFFFAOYSA-N", "MCM:BZFUOH", "")
  CALL Species("C_4H_6O_5",                                    "", "LZJWYIKGTDPLAQ-UHFFFAOYSA-N", "MCM:BZFUOOH", "")
  CALL Species("C_4H_8O_3",                                    "", "ZANNOFHADGWOLI-UHFFFAOYSA-N", "MCM:EOX2COMEOH", "")
  CALL Species("C_4H_8O_3",                                    "", "HXDLWJWIAHWIKI-UHFFFAOYSA-N", "MCM:ETACETOH", "")
  CALL Species("C_4H_8O_3",                                    "", "MLAFRLBDNVSLPE-UHFFFAOYSA-N", "MCM:ETHACETOH", "")
  CALL Species("C_4H_8O_5",                                    "", "QWYGOSGRTMPWCI-UHFFFAOYSA-N", "MCM:HOACETEOOH", "")
  CALL Species("C_4H_8O_3",                                    "", "SVJMGLNDGCVDIT-UHFFFAOYSA-N", "MCM:IPRFORMOH", "")
  CALL Species("C_4H_4O_5",                                    "", "BOGVTNYNTGOONP-UHFFFAOYSA-N", "MCM:MALANHY2OH", "")
  CALL Species("C_4H_4O_6",                                    "", "UPSCPEYLZZUIND-UHFFFAOYSA-N", "MCM:MALANHYOOH", "")
  CALL Species("C_4H_8O_3",                                    "", "CRMUFGZQBODVGS-UHFFFAOYSA-N", "MCM:PROL2FORM", "")
  CALL Species("C_5H_{10}O_3",                                 "", "KIZCCPGSHHAFRX-UHFFFAOYSA-N", "MCM:BUOHFORM", "")
  CALL Species("C_5H_{10}O_3",                                 "", "YJNKLTDJZSXVHQ-UHFFFAOYSA-N", "MCM:IPRACBOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "MYOAZFOMGTTYOD-UHFFFAOYSA-N", "MCM:IPRACOH", "")
  CALL Species("C_5H_6O_5",                                    "", "DGWWNMWYNUWPBC-UHFFFAOYSA-N", "MCM:MMALNHY2OH", "")
  CALL Species("C_5H_6O_6",                                    "", "JNCHKPBVXWMPAJ-UHFFFAOYSA-N", "MCM:MMALNHYOOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "GQMJADBSHHFADW-UHFFFAOYSA-N", "MCM:MTBEACHOHO", "")
  CALL Species("C_5H_{10}O_3",                                 "", "PPPFYBPQAPISCT-UHFFFAOYSA-N", "MCM:NPRACAOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "VEGXEWGKYMMJKP-UHFFFAOYSA-N", "MCM:NPRACBOH", "")
  CALL Species("C_5H_{10}O_3",                                 "", "DOUBAFNWVFAWEC-UHFFFAOYSA-N", "MCM:NPRACCOH", "")
  CALL Species("C_5H_8O_4",                                    "", "OHTGZAWPVDWARE-UHFFFAOYSA-N", "MCM:PXYFUOH", "")
  CALL Species("C_5H_8O_5",                                    "", "IEFDVDGOIVKGRZ-UHFFFAOYSA-N", "MCM:PXYFUOOH", "")
  CALL Species("C_5H_8O_4",                                    "", "JYHWQRJRDKSSIF-UHFFFAOYSA-N", "MCM:TLFUOH", "")
  CALL Species("C_5H_8O_5",                                    "", "GNXADMJANDYCAE-UHFFFAOYSA-N", "MCM:TLFUOOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "VFGRALUHHHDIQI-UHFFFAOYSA-N", "MCM:BOX2COMOH", "")
  CALL Species("C_6H_8O_5",                                    "", "KIOPGGAGDHZEGB-UHFFFAOYSA-N", "MCM:DMMALYOH", "")
  CALL Species("C_6H_8O_6",                                    "", "VRKLPKITGMCVQP-UHFFFAOYSA-N", "MCM:DMMALYOOH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "AQSJHOPJZAKOAP-UHFFFAOYSA-N", "MCM:EBFUOH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "KIZOVCBYCQGADC-UHFFFAOYSA-N", "MCM:EBFUOOH", "")
  CALL Species("C_6H_8O_5",                                    "", "GPJQUVLKFIAXFZ-UHFFFAOYSA-N", "MCM:EMALNHY2OH", "")
  CALL Species("C_6H_8O_6",                                    "", "LSSHMOINJMVLHQ-UHFFFAOYSA-N", "MCM:EMALNHYOOH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "WCMYKJKBZGIGND-UHFFFAOYSA-N", "MCM:MXYFUOH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "WCIMCWKIICZLLM-UHFFFAOYSA-N", "MCM:MXYFUOOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "KLUHZXMBIDAHSJ-UHFFFAOYSA-N", "MCM:NBUACAOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "BZLQSYFOTWOIDC-UHFFFAOYSA-N", "MCM:NBUACBOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "BMNRJWUOBYYCRX-UHFFFAOYSA-N", "MCM:NBUACCOH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "GZKXVWYDQQSODI-UHFFFAOYSA-N", "MCM:OXYFUOH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "DHHFHIZBMAEZJT-UHFFFAOYSA-N", "MCM:OXYFUOOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "GIOCILWWMFZESP-UHFFFAOYSA-N", "MCM:PRCOOETOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "WAXXMBOGFVJIHX-UHFFFAOYSA-N", "MCM:SBUACAOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "BCWWODMTUXMSAB-UHFFFAOYSA-N", "MCM:SBUACBOH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "WINGEFIITRDOLJ-UHFFFAOYSA-N", "MCM:TBOCOCH2OH", "")
  CALL Species("C_6H_{12}O_3",                                 "", "GWOKHTMCBUZSOP-UHFFFAOYSA-N", "MCM:TBUACOH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "AUTABLFJFWRKGZ-UHFFFAOYSA-N", "MCM:TMB1FUOH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "NFODFJMABSYJFC-UHFFFAOYSA-N", "MCM:TMB1FUOOH", "")
  CALL Species("C_7H_{14}O_5",                                 "", "PDOIGHDIMNJPNL-UHFFFAOYSA-N", "MCM:BOXCOOLOOH", "")
  CALL Species("C_7H_{12}O_4",                                 "", "ZIPHTXAVHSYMQR-UHFFFAOYSA-N", "MCM:IPBFUOH", "")
  CALL Species("C_7H_{12}O_5",                                 "", "HVQWCSPFZPPNOE-UHFFFAOYSA-N", "MCM:IPBFUOOH", "")
  CALL Species("C_7H_{10}O_5",                                 "", "RBIJYTIBBLMIRU-UHFFFAOYSA-N", "MCM:IPMLNHY2OH", "")
  CALL Species("C_7H_{10}O_6",                                 "", "LGGLTRMSNOIPLU-UHFFFAOYSA-N", "MCM:IPMLNHYOOH", "")
  CALL Species("C_7H_{12}O_4",                                 "", "SYHJYGJIPQZFIO-UHFFFAOYSA-N", "MCM:MEBFUOH", "")
  CALL Species("C_7H_{12}O_5",                                 "", "VRHMFTJWRSNJLN-UHFFFAOYSA-N", "MCM:MEBFUOOH", "")
  CALL Species("C_7H_{12}O_4",                                 "", "DMMDFVMQHRHVSU-UHFFFAOYSA-N", "MCM:PBFUOH", "")
  CALL Species("C_7H_{12}O_5",                                 "", "KAMUACRVJWJEFX-UHFFFAOYSA-N", "MCM:PBFUOOH", "")
  CALL Species("C_7H_{10}O_5",                                 "", "PVDNZCIFUUKKBS-UHFFFAOYSA-N", "MCM:PMALNHY2OH", "")
  CALL Species("C_7H_{10}O_6",                                 "", "VKSMRBCXNQEUFP-UHFFFAOYSA-N", "MCM:PMALNHYOOH", "")
  CALL Species("C_7H_{14}O_3",                                 "", "QXXGFEMAULVMST-UHFFFAOYSA-N", "MCM:PRCOOPROL", "")
  CALL Species("C_7H_{12}O_4",                                 "", "YEQCQVSGZKCWAA-UHFFFAOYSA-N", "MCM:TMB2FUOH", "")
  CALL Species("C_7H_{12}O_5",                                 "", "MINPHQZFUPUEBD-UHFFFAOYSA-N", "MCM:TMB2FUOOH", "")
  CALL Species("C_{10}H_{18}O_3",                              "", "OYGVMQZTRRRABW-UHFFFAOYSA-N", "MCM:C1013OH", "")
  CALL Species("C_3H_4O_5",                                    "", "DRQRCDXSDDWNEU-UHFFFAOYSA-N", "MCM:ALCOMOXOOH", "")
  CALL Species("C_3H_2O_5",                                    "", "DOMHJQJKOPZSOM-UHFFFAOYSA-N", "MCM:DFC", "")
  CALL Species("C_3H_4O_3",                                    "", "MFRMAQFRVTUENW-UHFFFAOYSA-N", "MCM:MOXY2CHO", "")
  CALL Species("C_3H_4O_3",                                    "", "KFKXSMSQHIOMSO-UHFFFAOYSA-N", "MCM:MOXYCOCHO", "")
  CALL Species("C_4H_6O_3",                                    "", "GUPGZURVZDIQPM-UHFFFAOYSA-N", "MCM:ACETETCHO", "")
  CALL Species("C_4H_4O_4",                                    "", "ZFWWLKBCHQUFEJ-UHFFFAOYSA-N", "MCM:CO14O3CHO", "")
  CALL Species("C_4H_6O_3",                                    "", "AGADEVQOWQDDFX-UHFFFAOYSA-N", "MCM:COO2C3CHO", "")
  CALL Species("C_4H_6O_3",                                    "", "DBPFRRFGLYGEJI-UHFFFAOYSA-N", "MCM:EOCOCHO", "")
  CALL Species("C_5H_6O_4",                                    "", "GRUDDVNYUQQBCU-UHFFFAOYSA-N", "MCM:ACCOMECHO", "")
  CALL Species("C_5H_8O_3",                                    "", "PRSPLAWXBFRHKV-UHFFFAOYSA-N", "MCM:ACEC2CHO", "")
  CALL Species("C_5H_8O_3",                                    "", "DLZVZNAPRCRXEG-UHFFFAOYSA-N", "MCM:COO2C4CHO", "")
  CALL Species("C_5H_8O_3",                                    "", "FXPPNKAYSGWCQG-UHFFFAOYSA-N", "MCM:IPRACBCHO", "")
  CALL Species("C_5H_8O_3",                                    "", "MDWWHOIDIKOPHH-UHFFFAOYSA-N", "MCM:MTBEACHO13", "")
  CALL Species("C_6H_{10}O_5",                                 "", "MZUJTBRWESAMQA-UHFFFAOYSA-N", "MCM:BOXCOALOOH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "NRYDRJHYTRBBEA-UHFFFAOYSA-N", "MCM:BOXCOCHO", "")
  CALL Species("C_6H_8O_4",                                    "", "FVLOJCOPOCRBFF-UHFFFAOYSA-N", "MCM:C14O3ECHO", "")
  CALL Species("C_6H_{10}O_3",                                 "", "YEBNSRBQIIWQQC-UHFFFAOYSA-N", "MCM:PRCOOMCHO", "")
  CALL Species("C_6H_{10}O_3",                                 "", "WDPZTIFGRQKSEN-UHFFFAOYSA-N", "MCM:TBOCOCHO", "")
  CALL Species("C_6H_{10}O_3",                                 "", "BYYPPVMBDPUHHR-UHFFFAOYSA-N", "MCM:TBUACCO", "")
  CALL Species("C_7H_{10}O_4",                                 "", "NPPWDFRKAMDHMM-UHFFFAOYSA-N", "MCM:C14O3IPCHO", "")
  CALL Species("C_7H_{10}O_4",                                 "", "GNODSIBUYMXKGR-UHFFFAOYSA-N", "MCM:C14O3PCHO", "")
  CALL Species("C_{10}H_{18}O_5",                              "", "REYZAYCUYYNPLJ-UHFFFAOYSA-N", "MCM:C1014OOH", "")
  CALL Species("C_{15}H_{24}O_3",                              "", "FEPYZLRQFOQWRK-UHFFFAOYSA-N", "MCM:BCALBOC", "")
  CALL Species("C_3H_6O_4",                                    "", "CBNAABWGSNQFTA-UHFFFAOYSA-N", "MCM:CHOOMCO2H", "")
  CALL Species("C_{10}H_{18}O_4",                              "", "IJPJFXOOJHUBTK-UHFFFAOYSA-N", "MCM:C1014OH", "")
  CALL Species("C_{15}H_{26}O_5",                              "", "XZXFZFZFZSKVDT-UHFFFAOYSA-N", "MCM:C152OH", "")
  CALL Species("C_{15}H_{26}O_6",                              "", "PKQOWOPWICZSPN-UHFFFAOYSA-N", "MCM:C152OOH", "")
  CALL Species("C_4H_4O_4",                                    "", "RULCSXTUAHNYHJ-UHFFFAOYSA-N", "MCM:C23O3CHO", "")
  CALL Species("C_4H_6O_5",                                    "", "BJTKGFWDIAZUGI-UHFFFAOYSA-N", "MCM:MCOCOMOOOH", "")
  CALL Species("C_4H_6O_3",                                    "", "CWKLZLBVOJRSOM-UHFFFAOYSA-N", "MCM:MCOCOMOX", "")
  CALL Species("C_4H_6O_5",                                    "", "LIFMXPSJWIBAGH-UHFFFAOYSA-N", "MCM:PRNFORMOOH", "")
  CALL Species("C_4H_6O_3",                                    "", "WNVAJGCMEDTLIE-UHFFFAOYSA-N", "MCM:PRONFORM", "")
  CALL Species("C_5H_6O_6",                                    "", "WKJHUBLAYZIBAJ-UHFFFAOYSA-N", "MCM:ACCOCOMOOH", "")
  CALL Species("C_5H_6O_4",                                    "", "OBTVATRSWTWNGS-UHFFFAOYSA-N", "MCM:ACECOCOCH3", "")
  CALL Species("C_5H_8O_3",                                    "", "DBERHVIZRVGDFO-UHFFFAOYSA-N", "MCM:ACEPROPONE", "")
  CALL Species("C_5H_8O_5",                                    "", "FOWIMBLOPIOWII-UHFFFAOYSA-N", "MCM:ACPRONEOOH", "")
  CALL Species("C_5H_6O_6",                                    "", "SPYAIQASSDQEIX-UHFFFAOYSA-N", "MCM:C23O3CCO3H", "")
  CALL Species("C_5H_8O_5",                                    "", "RVGUYYZMHBERKG-UHFFFAOYSA-N", "MCM:C23O3MCOOH", "")
  CALL Species("C_5H_8O_5",                                    "", "GJTAOZDAMDVHTP-UHFFFAOYSA-N", "MCM:PRNOCOMOOH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "BDHJVYCZNHBETI-UHFFFAOYSA-N", "MCM:ACBUOAOOH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "RISONYRIGLPILV-UHFFFAOYSA-N", "MCM:ACBUONBOOH", "")
  CALL Species("C_6H_8O_4",                                    "", "WZGYRWNUCYXCFD-UHFFFAOYSA-N", "MCM:ACCOCOC2H5", "")
  CALL Species("C_6H_8O_6",                                    "", "RKPSGJVLERUIIG-UHFFFAOYSA-N", "MCM:ACCOCOEOOH", "")
  CALL Species("C_6H_8O_4",                                    "", "RYLHMXPGTMYTTF-UHFFFAOYSA-N", "MCM:ACCOPRONE", "")
  CALL Species("C_6H_{10}O_3",                                 "", "LHGWJCBYBIICPP-UHFFFAOYSA-N", "MCM:ACEBUTBONE", "")
  CALL Species("C_6H_{10}O_3",                                 "", "NWCYECXHIYEBJE-UHFFFAOYSA-N", "MCM:ACEBUTONE", "")
  CALL Species("C_6H_8O_6",                                    "", "NKXSSYAJYGYQPN-UHFFFAOYSA-N", "MCM:ACOMCOMOOH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "NLYPTRYHUBHKKI-UHFFFAOYSA-N", "MCM:C23O3ECOOH", "")
  CALL Species("C_6H_8O_4",                                    "", "AIDIMUPPUNPUTI-UHFFFAOYSA-N", "MCM:C23O3EHO", "")
  CALL Species("C_6H_8O_6",                                    "", "FIPISRZHVPVVNA-UHFFFAOYSA-N", "MCM:C23O3MCO3H", "")
  CALL Species("C_6H_6O_7",                                    "", "LLKUMVBDCSCXDR-UHFFFAOYSA-N", "MCM:C6O4KETOOH", "")
  CALL Species("C_6H_6O_5",                                    "", "GAPVBMQBWAUBHC-UHFFFAOYSA-N", "MCM:C6OTKETCO", "")
  CALL Species("C_6H_8O_6",                                    "", "USMDUVUWRKQXQG-UHFFFAOYSA-N", "MCM:C6OTKETOOH", "")
  CALL Species("C_6H_8O_4",                                    "", "XAHWCAXZWOTSBC-UHFFFAOYSA-N", "MCM:C6OTRIKET", "")
  CALL Species("C_6H_8O_6",                                    "", "MVKXXKVCXUKLOS-UHFFFAOYSA-N", "MCM:CO356OCOOH", "")
  CALL Species("C_6H_{10}O_5",                                 "", "DTGNYNGASXJUMP-UHFFFAOYSA-N", "MCM:SBUACCOOH", "")
  CALL Species("C_6H_{10}O_3",                                 "", "ZKPTYCJWRHHBOW-UHFFFAOYSA-N", "MCM:SBUACEONE", "")
  CALL Species("C_7H_{12}O_5",                                 "", "YCDXONIQHOVSSF-UHFFFAOYSA-N", "MCM:BOX2COMOOH", "")
  CALL Species("C_7H_{12}O_3",                                 "", "ZAZUOXBHFXAWMD-UHFFFAOYSA-N", "MCM:BOXCOCOME", "")
  CALL Species("C_7H_{10}O_6",                                 "", "FWVKIPPVGIEQRB-UHFFFAOYSA-N", "MCM:C23O3ECO3H", "")
  CALL Species("C_7H_{10}O_6",                                 "", "DJTBKMNUYULRIV-UHFFFAOYSA-N", "MCM:MC6OTKTOOH", "")
  CALL Species("C_7H_{10}O_4",                                 "", "FJYCWZCSXDGURV-UHFFFAOYSA-N", "MCM:MC6OTRIKET", "")
  CALL Species("C_7H_{12}O_3",                                 "", "AIJLJYUDTAJRDN-UHFFFAOYSA-N", "MCM:PRCOOPRONE", "")
  CALL Species("C_7H_{12}O_5",                                 "", "RGEURRORAZXRIV-UHFFFAOYSA-N", "MCM:PRNOCOPOOH", "")
  CALL Species("C_{10}H_{16}O_3",                              "", "DPRGOCIPALHGLN-UHFFFAOYSA-N", "MCM:C1013CO", "")
  CALL Species("C_4H_4O_4",                                    "", "MFFNSLKCUZEAFI-UHFFFAOYSA-N", "MCM:BZFUCO", "")
  CALL Species("C_4H_2O_5",                                    "", "DBWAPECJPVXLJZ-UHFFFAOYSA-N", "MCM:MALNHYOHCO", "")
  CALL Species("C_5H_8O_4",                                    "", "LBOLMEABDOYQGX-UHFFFAOYSA-N", "MCM:ACPRONEOH", "")
  CALL Species("C_5H_8O_4",                                    "", "WHFDSTOTVFOUIH-UHFFFAOYSA-N", "MCM:C23O3MOH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "UGJVDYQVVBKWGL-UHFFFAOYSA-N", "MCM:ACBUONAOH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "GYQPXJWBTJDNGC-UHFFFAOYSA-N", "MCM:ACBUONBOH", "")
  CALL Species("C_6H_{10}O_4",                                 "", "BVDDSYZMRAYYIJ-UHFFFAOYSA-N", "MCM:C23O3EOH", "")
  CALL Species("C_6H_8O_5",                                    "", "NOUUUBJIFPBZMZ-UHFFFAOYSA-N", "MCM:C6OTKETOH", "")
  CALL Species("C_6H_8O_4",                                    "", "QFGGPABGKQEYQI-UHFFFAOYSA-N", "MCM:EBFUCO", "")
  CALL Species("C_6H_{10}O_4",                                 "", "SUDJMSTXSVVFNQ-UHFFFAOYSA-N", "MCM:SBUACCOH", "")
  CALL Species("C_7H_{10}O_4",                                 "", "NHSFIZKDULUUEP-UHFFFAOYSA-N", "MCM:IPBFUCO", "")
  CALL Species("C_7H_{10}O_5",                                 "", "GQVIXHCRVNFYGI-UHFFFAOYSA-N", "MCM:MC6OTKETOH", "")
  CALL Species("C_7H_{10}O_4",                                 "", "PKKIVTPUHZVJRM-UHFFFAOYSA-N", "MCM:PBFUCO", "")
  CALL Species("C_5H_6O_4",                                    "", "JQZIMULKJBYVEJ-UHFFFAOYSA-N", "MCM:C23O3CCHO", "")
  CALL Species("C_6H_8O_4",                                    "", "KNKMVXKCHLTCEX-UHFFFAOYSA-N", "MCM:C23O3MCHO", "")
  CALL Species("C_6H_8O_4",                                    "", "MFFVRQLCBYBQSQ-UHFFFAOYSA-N", "MCM:C5OCO3M", "")
  CALL Species("C_7H_{10}O_4",                                 "", "RCPFVGJRKWBYKE-UHFFFAOYSA-N", "MCM:C23O3ECHO", "")
  CALL Species("C_{10}H_{16}O_4",                              "", "MPJVRMHYAPYGFM-UHFFFAOYSA-N", "MCM:C1014CO", "")
  CALL Species("C_{14}H_{22}O_4",                              "", "YETGIZICMMLUOK-UHFFFAOYSA-N", "MCM:BCLKBOC", "")
  CALL Species("C_{14}H_{20}O_5",                              "", "WKRISEJACIFSSJ-UHFFFAOYSA-N", "MCM:C148CO", "")
  CALL Species("C_{14}H_{22}O_6",                              "", "QNGFZXSUHFZFIE-UHFFFAOYSA-N", "MCM:C148OOH", "")
  CALL Species("C_{14}H_{22}O_5",                              "", "LXDPLKVHKWYPKP-UHFFFAOYSA-N", "MCM:C148OH", "")
  CALL Species("C_5H_8O_4",                                    "", "RFEXARYJXBYPLD-UHFFFAOYSA-N", "MCM:ACETC2CO2H", "")
  CALL Species("C_5H_6O_5",                                    "", "HSSKPURIBYAXCO-UHFFFAOYSA-N", "MCM:C24O3CCO2H", "")
  CALL Species("C_5H_8O_4",                                    "", "WTLNOANVTIKPEE-UHFFFAOYSA-N", "MCM:IPRACBCO2H", "")
  CALL Species("C_5H_8O_4",                                    "", "BXXOFUQIACXFIW-UHFFFAOYSA-N", "MCM:MTBEALCO2H", "")
  CALL Species("C_{11}H_{18}O_4",                              "", "QWGCLZMSSSTRLX-UHFFFAOYSA-N", "MCM:C1013CO2H", "")
  CALL Species("C_4H_4O_5",                                    "", "OPFWUCMVKDVVTF-UHFFFAOYSA-N", "MCM:CO14O3CO2H", "")
  CALL Species("C_5H_6O_5",                                    "", "WRKCXTWVOWDGBE-UHFFFAOYSA-N", "MCM:C23O3CCO2H", "")
  CALL Species("C_6H_8O_5",                                    "", "OGSBZWITKXVLIV-UHFFFAOYSA-N", "MCM:C23O3MCO2H", "")
  CALL Species("C_7H_{10}O_5",                                 "", "OEONINCNUUTEPU-UHFFFAOYSA-N", "MCM:C23O3ECO2H", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Ethers (\chem{ROR})")
  ! saturated:
  ! C2:
  CALL Species("CH_3OCH_3",                            "115-10-6", "LCGLNKUTAGEVQW-UHFFFAOYSA-N", "dimethyl ether", "")
  ! C3:
  CALL Species("C_2H_5OCH_3",                          "540-67-0", "XOBKSJJDNFUZPF-UHFFFAOYSA-N", "ethyl methyl ether", "")
  ! C4:
  CALL Species("C_2H_5OC_2H_5",                         "60-29-7", "RTZKZFJDLAIYFH-UHFFFAOYSA-N", "diethyl ether", "")
  CALL Species("C_2D_5OC_2D_5",                       "2679-89-2", "RTZKZFJDLAIYFH-MWUKXHIBSA-N", "diethyl ether-d10", "")
  CALL Species("CH_3OC_3H_7",                          "557-17-5", "VNKYTQGIUYNRMY-UHFFFAOYSA-N", "methyl propyl ether", "")
  CALL Species("CH_3OC_3H_7",                          "598-53-8", "RMGHERXMTMUMMV-UHFFFAOYSA-N", "methyl 2-propyl ether", "methyl isopropyl ether")
  CALL Species("C_4H_6O",                              "109-93-3", "QYKIQEUNHZKYBP-UHFFFAOYSA-N", "divinyl ether", "")
  ! C5:
  CALL Species("C_5H_{12}O",                           "628-28-4", "CXBDYQVECUFKRK-UHFFFAOYSA-N", "methyl butyl ether", "")
  CALL Species("C_5H_{12}O",                          "6795-87-5", "FVNIMHIOIXPIQT-UHFFFAOYSA-N", "2-methoxybutane", "methyl {\it sec}-butyl ether")
  CALL Species("C_5H_{12}O",                           "625-44-5", "ZYVYEJXMYBUCMN-UHFFFAOYSA-N", "methyl isobutyl ether", "")
  CALL Species("CH_3OC(CH_3)_3",                      "1634-04-4", "BZLVMXJERCGZMT-UHFFFAOYSA-N", "methyl {\it tert}-butyl ether", "MTBE")
  CALL Species("C_2H_5OC_3H_7",                        "628-32-0", "NVJUHMXYKCUMQA-UHFFFAOYSA-N", "ethyl propyl ether", "")
  CALL Species("C_5H_{12}O",                           "625-54-7", "XSJVWZAETSBXKU-UHFFFAOYSA-N", "ethyl isopropyl ether", "")
  CALL Species("C_5H_{12}O_2",                          "77-76-9", "HEWZVZIVELJPQZ-UHFFFAOYSA-N", "2,2-dimethoxypropane", "")
  ! C6:
  CALL Species("C_6H_{14}O",                         "62016-49-3", "JPUDLQKLSRSRGN-UHFFFAOYSA-N", "methyl 1,2-dimethylpropyl ether", "")
  CALL Species("C_6H_{14}O",                         "36839-67-5", "CQRFEDVNTJTKFU-UHFFFAOYSA-N", "methyl 1-ethylpropyl ether", "")
  CALL Species("C_6H_{14}O",                          "6795-88-6", "XSAJCGUYMQTAHL-UHFFFAOYSA-N", "methyl 1-methylbutyl ether", "")
  CALL Species("C_6H_{14}O",                          "1118-00-9", "JILHZKWLEAKYRC-UHFFFAOYSA-N", "methyl 2,2-dimethylpropyl ether", "")
  CALL Species("C_6H_{14}O",                         "62016-48-2", "XGLHRCWEOMNVKS-UHFFFAOYSA-N", "methyl 2-methylbutyl ether", "")
  CALL Species("C_6H_{14}O",                           "626-91-5", "ZQAYBCWERYRAMF-UHFFFAOYSA-N", "methyl 3-methylbutyl ether", "")
  CALL Species("C_6H_{14}O",                           "628-80-8", "DBUJFULDVAZULB-UHFFFAOYSA-N", "methyl pentyl ether", "")
  CALL Species("C_6H_{14}O",                           "994-05-8", "HVZJRWJGKQPSFL-UHFFFAOYSA-N", "2-methoxy-2-methylbutane", "{\it tert}-amyl methyl ether")
  CALL Species("C_6H_{14}O",                           "628-81-9", "PZHIWRCQKBBTOW-UHFFFAOYSA-N", "1-ethoxy-butane", "ethyl butyl ether")
  CALL Species("C_6H_{14}O",                           "627-02-1", "RQUBQBFVDOLUKC-UHFFFAOYSA-N", "ethyl isobutyl ether", "")
  CALL Species("C_6H_{14}O",                          "2679-87-0", "VSCUCHUDCLERMY-UHFFFAOYSA-N", "ethyl {\it sec}-butyl ether", "")
  CALL Species("C_2H_5OC(CH_3)_3",                     "637-92-3", "NUMQCACRALPSHD-UHFFFAOYSA-N", "ethyl {\it tert}-butyl ether", "ETBE")
  CALL Species("C_3H_7OC_3H_7",                        "111-43-3", "POLCUAVZOMRGSN-UHFFFAOYSA-N", "dipropyl ether", "")
  CALL Species("C_3H_7OC_3H_7",                        "108-20-3", "ZAFNJMIOTHYJRJ-UHFFFAOYSA-N", "diisopropyl ether", "")
  CALL Species("C_6H_{14}O ",                          "627-08-7", "JIEJJGMNDWIGBJ-UHFFFAOYSA-N", "propyl isopropyl ether", "")
  ! C7:
  CALL Species("C_7H_{16}O",                           "919-94-8", "KFRVYYGHSPLXSZ-UHFFFAOYSA-N", "2-ethoxy-2-methylbutane", "{\it tert}-amyl ethyl ether")
  CALL Species("C_7H_{16}O",                         "17348-59-3", "HNFSPSWQNZVCTB-UHFFFAOYSA-N", "{\it tert}-butyl isopropyl ether", "")
  CALL Species("C_7H_{16}O_3",                       "34590-94-8", "WGYZMNBUZFHYRX-UHFFFAOYSA-N", "dipropylene glycol monomethyl ether", "")
  ! C8:
  CALL Species("C_4H_9OC_4H_9",                        "142-96-1", "DURPTKYDGMDSBL-UHFFFAOYSA-N", "dibutyl ether", "")
  CALL Species("C_8H_{18}O",                           "628-55-7", "SZNYYWIUQFZLLT-UHFFFAOYSA-N", "diisobutyl ether", "")
  CALL Species("C_8H_{18}O",                          "6863-58-7", "HHBZZTKMMLDNDN-UHFFFAOYSA-N", "di-{\it sec}-butyl ether", "")
  CALL Species("C_8H_{18}O",                          "6163-66-2", "AQEFLFZSWDEAIP-UHFFFAOYSA-N", "di-{\it tert}-butyl ether", "")
  ! >= C9:
  CALL Species("C_9H_{20}O",                         "62108-41-2", "IKZVAPMTXDXWMX-UHFFFAOYSA-N", "2-methoxy-2,4,4-trimethylpentane", "methyl {\it tert}-octyl ether")
  CALL Species("C_9H_{20}O",                         "76589-16-7", "KJRACWZCOVHBQU-UHFFFAOYSA-N", "2-methoxy-2-methylheptane", "")
  CALL Species("C_{10}H_{22}O",                     "187103-12-4", "JGPJRBWLBUGAQN-UHFFFAOYSA-N", "2-ethoxy-2,4,4-trimethylpentane", "ethyl tert-octyl ether")
  CALL Species("C_{10}H_{22}O",                        "693-65-2", "AOPDRZXCEAKHHW-UHFFFAOYSA-N", "1,1'-oxybispentane", "dipentyl ether")
  CALL Species("C_{10}H_{22}O",                        "544-01-4", "AQZGPSLYZOOYQP-UHFFFAOYSA-N", "1,1'-oxybis(3-methylbutane)", "diisopentyl ether")
  CALL Species("C_{12}H_{26}O",                        "112-58-3", "BPIUIOXAFBGMNB-UHFFFAOYSA-N", "1,1'-oxybishexane", "dihexyl ether")
  CALL Species("C_{12}H_{26}O",                      "22810-10-2", "HCHHIPCZJSRFRT-UHFFFAOYSA-N", "1-ethoxy-3,7-dimethyloctane", "")
  CALL Species("C_{14}H_{30}O",                        "629-64-1", "UJEGHEMJVNQWOJ-UHFFFAOYSA-N", "diheptyl ether", "")
  CALL Species("C_{16}H_{34}O",                        "629-82-3", "NKJOXAZJBOMXID-UHFFFAOYSA-N", "dioctyl ether", "")
  ! cyclic:
  CALL Species("C_7H_{14}O",                           "931-56-6", "GHDIHPNJQVDFBL-UHFFFAOYSA-N", "methoxycyclohexane", "")
  CALL Species("C_{16}H_{28}O",                      "19870-74-7", "HRGPYCVTDOECMG-WALBABNVSA-N", "methyl cedryl ether", "")
  ! more than one O:
  CALL Species("CH_3OCH_2OCH_3",                       "109-87-5", "NKDDWNXOKDWJAK-UHFFFAOYSA-N", "dimethoxymethane", "methylal")
  CALL Species("HC(OCH_3)_3",                          "149-73-5", "PYOKUURKVVELLB-UHFFFAOYSA-N", "trimethoxymethane", "")
  CALL Species("C_5H_{12}O_2",                         "462-95-3", "KLKFAASOGCDTDT-UHFFFAOYSA-N", "diethoxymethane", "")
  CALL Species("(C_2H_5O)_2CHCH_3",                    "105-57-7", "DHKHKXVYLBGOIT-UHFFFAOYSA-N", "1,1-diethoxyethane", "acetal")
  CALL Species("C_2H_5OC_2H_4OC_2H_5",                 "629-14-1", "LZDKZFUFMNSQCJ-UHFFFAOYSA-N", "1,2-diethoxyethane", "")
  CALL Species("CH_3C(OCH_3)_3",                      "1445-45-0", "HDPNBNXLBDFELL-UHFFFAOYSA-N", "1,1,1-trimethoxyethane", "")
  CALL Species("C_4H_{10}O_2",                         "110-71-4", "XTHFKEDIFFGKHM-UHFFFAOYSA-N", "1,2-dimethoxyethane", "")
  CALL Species("C_5H_{12}O_2",                        "2807-30-9", "YEYKMVJDLWJFOA-UHFFFAOYSA-N", "3-oxa-1-hexanol", "2-propoxyethanol")
  CALL Species("C_6H_{14}O_2",                         "111-76-2", "POAOYUHQDCAZBD-UHFFFAOYSA-N", "3-oxa-1-heptanol", "2-butoxyethanol; butyl cellosolve")
  CALL Species("C_4H_{10}O_2",                         "107-98-2", "ARXJGSRGQADJSQ-UHFFFAOYSA-N", "1-methoxy-2-propanol", "")
  CALL Species("C_5H_{12}O_2",                         "109-59-1", "HCGFUIQPSOCUHI-UHFFFAOYSA-N", "4-methyl-3-oxa-1-pentanol", "2-isopropoxyethanol")
  CALL Species("C_{10}H_{22}O_2",                      "112-48-1", "GDXHBFHOEYVPED-UHFFFAOYSA-N", "1,2-dibutoxyethane", "")
  CALL Species("C_8H_{18}O_3",                         "112-34-5", "OAYXUHPQHDHDDZ-UHFFFAOYSA-N", "3,6-dioxa-1-decanol", "butyl carbitol")
  CALL Species("C_{12}H_{26}O_3",                      "112-73-2", "KZVBBTZJMSWGTK-UHFFFAOYSA-N", "diethylene glycol dibutyl ether", "")
  ! unsaturated:
  CALL Species("C_3H_6O",                              "107-25-5", "XJRBAMWJDBPFIM-UHFFFAOYSA-N", "methyl vinyl ether", "")
  CALL Species("C_4H_8O",                              "109-92-2", "FJKIXWOMBXYWOQ-UHFFFAOYSA-N", "ethyl vinyl ether", "")
  CALL Species("C_5H_{10}O",                           "764-47-6", "OVGRCEFMXPHEBL-UHFFFAOYSA-N", "propyl vinyl ether", "")
  CALL Species("C_6H_{10}O_2",                         "106-92-3", "LSWYGACWGAICNM-UHFFFAOYSA-N", "allyl glycidyl ether", "")
  CALL Species("C_6H_{12}O",                           "111-34-2", "UZKWTJUDCOPSNM-UHFFFAOYSA-N", "butyl vinyl ether", "")
  CALL Species("C_6H_{12}O",                           "109-53-5", "OZCMOJQQLBXBKI-UHFFFAOYSA-N", "isobutyl vinyl ether", "")
  ! aromatic:
  CALL Species("C_8H_{10}O",                           "538-86-3", "GQKZBCPTCWJTAS-UHFFFAOYSA-N", "benzyl methyl ether", "")
  CALL Species("C_9H_{12}O",                           "539-30-0", "AXPZDYVDTMMLNB-UHFFFAOYSA-N", "benzyl ethyl ether", "")
  CALL Species("C_6H_5OCH_3",                          "100-66-3", "RDOXTESZEPMUJZ-UHFFFAOYSA-N", "methoxybenzene", "anisole")
  CALL Species("C_8H_{10}O",                           "103-73-1", "DLRJIFUOBPOJNS-UHFFFAOYSA-N", "ethoxybenzene", "phenetole")
  CALL Species("C_8H_{10}O_2",                          "91-16-7", "ABDKAPXRBAPSQN-UHFFFAOYSA-N", "1,2-dimethoxybenzene", "")
  CALL Species("C_8H_{10}O_2",                         "122-99-6", "QCDWFXQBSFUVSP-UHFFFAOYSA-N", "2-phenoxyethanol", "")
  CALL Species("C_9H_{12}O_2",                         "622-08-2", "CUZKCNWZBXLAJX-UHFFFAOYSA-N", "2-(phenylmethoxy)-ethanol", "")
  CALL Species("C_9H_{12}O_3",                         "634-36-6", "CRUILBNAQILVHZ-UHFFFAOYSA-N", "1,2,3-trimethoxybenzene", "")
  CALL Species("C_{10}H_{12}O",                        "104-46-1", "RUVINXPYWBROJD-UHFFFAOYSA-N", "1-methoxy-4-(1-propenyl)-benzene", "anethole") ! other CAS: 4180-23-8
  CALL Species("C_{10}H_{12}O_2",                       "97-53-0", "RRAFCDWBNXTKKO-UHFFFAOYSA-N", "2-methoxy-4-(2-propenyl)-phenol", "eugenol")
  CALL Species("C_{11}H_{14}O_2",                       "93-15-2", "ZYEMGPIYFIJGTP-UHFFFAOYSA-N", "1,2-dimethoxy-4-(2-propenyl)-benzene", "")
  CALL Species("C_{12}H_{10}O",                        "101-84-8", "USIUVYZYUHIAEV-UHFFFAOYSA-N", "diphenyl ether", "")
  CALL Species("C_9H_{10}O_2",                         "122-60-1", "FQYUMYWMJTYZTK-UHFFFAOYSA-N", "(phenoxymethyl)-oxirane", "")
  CALL Species("C_{24}H_{34}O",                     "119345-02-7", "XSAHYEQPUFJGKW-UHFFFAOYSA-N", "1-dodecyl-4-phenoxybenzene", "")
  CALL Species("C_{28}H_{42}O",                      "61702-88-3", "AJDONJVWDSZZQF-UHFFFAOYSA-N", "2,2,4-trimethyl-4-(4-(4-(2,4,4-trimethylpentan-2-yl)phenoxy)phenyl)pentane", "") ! other CAS: 101-58-6
  CALL Species("C_{25}H_{28}O_3",                    "80844-07-1", "YREQHYQNNWYQCJ-UHFFFAOYSA-N", "etofenprox", "")
  ! peroxy ether:
  CALL Species("C_{12}H_{26}O_4",                     "2167-23-9", "HQOVXPHOJANJBR-UHFFFAOYSA-N", "di-{\it tert}-butyl {\it sec}-butylidene diperoxide", "")
  CALL Species("C_{13}H_{20}O_2",                     "3457-61-2", "BIISIZOQPWZPPS-UHFFFAOYSA-N", "peroxide, 1,1-dimethylethyl 1-methyl-1-phenylethyl", "")
  CALL Species("C_{16}H_{34}O_4",                       "78-63-7", "DMWVYCCGCQPJEA-UHFFFAOYSA-N", "di-{\it tert}-butyl 1,1,4,4-tetramethyltetramethylene diperoxide", "")
  CALL Species("C_{20}H_{34}O_4",                     "2781-00-2", "GWQOYRSARAWVTC-UHFFFAOYSA-N", "1,4-bis(1-{\it tert}-butylperoxy-1-methyl-ethyl)benzene", "")
  ! ethers from MCM:
  CALL Species("C_2H_6O_3",                                    "", "CDXAGPPPWKCPRI-UHFFFAOYSA-N", "MCM:CH3OCH2OOH", "")
  CALL Species("C_2H_4O_3",                                    "", "NOEKFNMWGHEGBN-UHFFFAOYSA-N", "MCM:ETHOXOOH", "")
  CALL Species("C_3H_8O_4",                                    "", "SHLFZQKLTPTTSZ-UHFFFAOYSA-N", "MCM:DMMAOOH", "")
  CALL Species("C_3H_8O_4",                                    "", "HODDSHZVZWXLHR-UHFFFAOYSA-N", "MCM:DMMBOOH", "")
  CALL Species("C_3H_8O_3",                                    "", "NZBMKAZNWPZJND-UHFFFAOYSA-N", "MCM:ETOMEOOH", "")
  CALL Species("C_3H_6O_4",                                    "", "PWWCDEMRYRBWOJ-UHFFFAOYSA-N", "MCM:MEMOXYCO3H", "")
  CALL Species("C_4H_{10}O_3",                                 "", "CXWWPQGYBJCHJL-UHFFFAOYSA-N", "MCM:DIETETOOH", "")
  CALL Species("C_4H_{10}O_3",                                 "", "NYVGIYPKZJSYSP-UHFFFAOYSA-N", "MCM:ETOC2OOH", "")
  CALL Species("C_4H_8O_4",                                    "", "ULYBJOMJQUHMOL-UHFFFAOYSA-N", "MCM:ETOMECO3H", "")
  CALL Species("C_4H_{10}O_3",                                 "", "PSBSELFEZHDPTA-UHFFFAOYSA-N", "MCM:IPRMETOOH", "")
  CALL Species("C_5H_{12}O_3",                                 "", "DCHWALHTYQHXTG-UHFFFAOYSA-N", "MCM:BOXMOOH", "")
  CALL Species("C_5H_{12}O_3",                                 "", "ZZXLAMSYBUVKJS-UHFFFAOYSA-N", "MCM:EIPEOOH", "")
  CALL Species("C_5H_{12}O_3",                                 "", "VGMUBGNCFVOQDX-UHFFFAOYSA-N", "MCM:IPROC21OOH", "")
  CALL Species("C_5H_{12}O_3",                                 "", "BGZKVDIFKPQYHL-UHFFFAOYSA-N", "MCM:MTBEAOOH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "NGYXBYFCPPVOQA-UHFFFAOYSA-N", "MCM:MTBEBCO3H", "")
  CALL Species("C_5H_{12}O_3",                                 "", "FPTBRWWJTBOWHN-UHFFFAOYSA-N", "MCM:MTBEBOOH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "BMXLBBYCODRNIJ-UHFFFAOYSA-N", "MCM:BOXMCO3H", "")
  CALL Species("C_6H_{14}O_3",                                 "", "PFVPTWUDXVSVAC-UHFFFAOYSA-N", "MCM:DIIPRETOOH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "UAVAAIVEDVDYSQ-UHFFFAOYSA-N", "MCM:ETBEACO3H", "")
  CALL Species("C_6H_{14}O_3",                                 "", "ITRUGVUCIYLYDK-UHFFFAOYSA-N", "MCM:ETBEAOOH", "")
  CALL Species("C_6H_{14}O_3",                                 "", "VOZHKKMLSSYDIG-UHFFFAOYSA-N", "MCM:ETBEBOOH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "AELSWALKLVZVPO-UHFFFAOYSA-N", "MCM:ETBECCO3H", "")
  CALL Species("C_6H_{14}O_3",                                 "", "WJMPPRSRRBMJID-UHFFFAOYSA-N", "MCM:ETBECOOH", "")
  CALL Species("C_6H_{14}O_3",                                 "", "JCIUPJDJCBRYHD-UHFFFAOYSA-N", "MCM:IPROMC2OOH", "")
  CALL Species("C_6H_{12}O_4",                                 "", "SFJQIQZNOBLTBZ-UHFFFAOYSA-N", "MCM:IPROMCCO3H", "")
  CALL Species("C_{15}H_{24}O_3",                              "", "DXZCAIRIBLPJLG-UHFFFAOYSA-N", "MCM:BCSOZ", "")
  CALL Species("C_2H_6O_2",                                    "", "VHWYCFISAQVCCP-UHFFFAOYSA-N", "MCM:CH3OCH2OH", "")
  CALL Species("C_3H_8O_3",                                    "", "YANOHILBFNXRFM-UHFFFAOYSA-N", "MCM:DMMAOH", "")
  CALL Species("C_3H_8O_3",                                    "", "IIGJYLXJNYBXEO-UHFFFAOYSA-N", "MCM:DMMBOH", "")
  CALL Species("C_3H_8O_2",                                    "", "RRLWYLINGKISHN-UHFFFAOYSA-N", "MCM:ETOMEOH", "")
  CALL Species("C_3H_8O_3",                                    "", "CSCSROFYRUZJJH-UHFFFAOYSA-N", "MCM:MO2EOLA2OH", "")
  CALL Species("C_3H_8O_4",                                    "", "PFSIOEPXGRVTPO-UHFFFAOYSA-N", "MCM:MO2EOLAOOH", "")
  CALL Species("C_3H_8O_3",                                    "", "ONSWVOSXVUHESJ-UHFFFAOYSA-N", "MCM:MO2EOLB2OH", "")
  CALL Species("C_3H_8O_4",                                    "", "VULCDXDIAONDTM-UHFFFAOYSA-N", "MCM:MO2EOLBOOH", "")
  CALL Species("C_4H_{10}O_2",                                 "", "CAFAOQIVXSSFSY-UHFFFAOYSA-N", "MCM:DIETETOH", "")
  CALL Species("C_4H_{10}O_3",                                 "", "WWVBRUMYFUDEJQ-UHFFFAOYSA-N", "MCM:EOX2ETA2OH", "")
  CALL Species("C_4H_{10}O_3",                                 "", "RZYMXMZJVMXDRP-UHFFFAOYSA-N", "MCM:EOX2ETB2OH", "")
  CALL Species("C_4H_{10}O_4",                                 "", "OSAXOMWTXDBXFG-UHFFFAOYSA-N", "MCM:EOX2OLAOOH", "")
  CALL Species("C_4H_{10}O_4",                                 "", "IWDPPKCJRWMAAT-UHFFFAOYSA-N", "MCM:EOX2OLBOOH", "")
  CALL Species("C_4H_{10}O_3",                                 "", "OAPWFUZUONIVKV-UHFFFAOYSA-N", "MCM:H2C3OCOH", "")
  CALL Species("C_4H_{10}O_4",                                 "", "ZZOFHQOVQLUGOX-UHFFFAOYSA-N", "MCM:H2C3OCOOH", "")
  CALL Species("C_4H_{10}O_2",                                 "", "BFSUQRCCKXZXEX-UHFFFAOYSA-N", "MCM:IPRMEETOH", "")
  CALL Species("C_4H_{10}O_4",                                 "", "BIISZUJLPYTCKS-UHFFFAOYSA-N", "MCM:PR2OHMOOOH", "")
  CALL Species("C_4H_{10}O_3",                                 "", "OEYNWAWWSZUGDU-UHFFFAOYSA-N", "MCM:PROH2MOX", "")
  CALL Species("C_5H_{12}O_2",                                 "", "CRHLZRRTZDFDAJ-UHFFFAOYSA-N", "MCM:BOXMOH", "")
  CALL Species("C_5H_{12}O_2",                                 "", "MGMXTRZFWPWZFH-UHFFFAOYSA-N", "MCM:EIPEOH", "")
  CALL Species("C_5H_{10}O_4",                                 "", "YTIKZIBPNMOYQX-UHFFFAOYSA-N", "MCM:HIEPOXB", "")
  CALL Species("C_5H_8O_5",                                    "", "WBOLSVSGSKMCPN-UHFFFAOYSA-N", "MCM:IEACO3H", "")
  CALL Species("C_5H_8O_5",                                    "", "CEWHGLGNUIHIJU-UHFFFAOYSA-N", "MCM:IECCO3H", "")
  CALL Species("C_5H_{10}O_3",                                 "", "CIDUHKBATDRWPE-UHFFFAOYSA-N", "MCM:IEPOXA", "")
  CALL Species("C_5H_{10}O_3",                                 "", "REKLCZSNEUFIBP-UHFFFAOYSA-N", "MCM:IEPOXC", "")
  CALL Species("C_5H_{12}O_2",                                 "", "PWLPTLCMMFJZIU-UHFFFAOYSA-N", "MCM:IPROC21OH", "")
  CALL Species("C_5H_{12}O_2",                                 "", "NHNNIMVRQJIZFW-UHFFFAOYSA-N", "MCM:MTBEAOH", "")
  CALL Species("C_5H_{12}O_2",                                 "", "VMPUAIZSESMILD-UHFFFAOYSA-N", "MCM:MTBEBOH", "")
  CALL Species("C_6H_{14}O_3",                                 "", "WQHNZXURJISVCT-UHFFFAOYSA-N", "MCM:BOX2E2OH", "")
  CALL Species("C_6H_{14}O_4",                                 "", "XRZMTAXNHOGTME-UHFFFAOYSA-N", "MCM:BOX2OHAOOH", "")
  CALL Species("C_6H_{14}O_4",                                 "", "LCMMTUJPIXBSDS-UHFFFAOYSA-N", "MCM:BOX2OHBOOH", "")
  CALL Species("C_6H_{14}O_3",                                 "", "QLKNFVRAYYHHDF-UHFFFAOYSA-N", "MCM:BOXOHETOH", "")
  CALL Species("C_6H_{14}O_2",                                 "", "FRKCZFUUIQCQPC-UHFFFAOYSA-N", "MCM:DIIPRETOH", "")
  CALL Species("C_6H_{14}O_2",                                 "", "WOSZILCYMCIWFB-UHFFFAOYSA-N", "MCM:ETBEAOH", "")
  CALL Species("C_6H_{14}O_2",                                 "", "DUNYNUFVLYAWTI-UHFFFAOYSA-N", "MCM:ETBEBOH", "")
  CALL Species("C_6H_{14}O_2",                                 "", "BDLXTDLGTWNUFM-UHFFFAOYSA-N", "MCM:ETBECOH", "")
  CALL Species("C_6H_{14}O_2",                                 "", "ZFEKANLLFQEKED-UHFFFAOYSA-N", "MCM:IPROMC2OH", "")
  CALL Species("C_7H_{16}O_2",                        "5131-66-8", "RWNUSVWFHDHRCJ-UHFFFAOYSA-N", "1-butoxy-2-propanol", "MCM:BOX2PROL")
  CALL Species("C_7H_{16}O_3",                                 "", "ITIQJVOJGXCBQB-UHFFFAOYSA-N", "MCM:BOXOHPROL", "")
  CALL Species("C_7H_{16}O_4",                                 "", "JGCUFCZSCOGTSN-UHFFFAOYSA-N", "MCM:BOXPOLAOOH", "")
  CALL Species("C_7H_{16}O_4",                                 "", "AJCGQJRNRRPZGZ-UHFFFAOYSA-N", "MCM:BOXPOLBOOH", "")
  CALL Species("C_7H_{16}O_3",                                 "", "XKPKIGSYCLTAJO-UHFFFAOYSA-N", "MCM:BOXPR2OH", "")
  CALL Species("C_{15}H_{26}O_5",                              "", "JQRHKAPCWHNLMS-UHFFFAOYSA-N", "MCM:BCSOZOH", "")
  CALL Species("C_{15}H_{26}O_6",                              "", "VCKYKGHDZKOMMJ-UHFFFAOYSA-N", "MCM:BCSOZOOH", "")
  CALL Species("C_3H_6O_2",                                    "", "YSEFYOVWKJXNCH-UHFFFAOYSA-N", "MCM:MEMOXYCHO", "")
  CALL Species("C_4H_8O_2",                                    "", "IAHZBRPNDIVNNR-UHFFFAOYSA-N", "MCM:EOX2ETCHO", "")
  CALL Species("C_4H_4O_3",                                    "", "IRJHVNZVWOCVLV-UHFFFAOYSA-N", "MCM:EPXC4DIAL", "")
  CALL Species("C_4H_4O_5",                                    "", "OKYIQGCZHSMTGV-UHFFFAOYSA-N", "MCM:EPXDLCO3H", "")
  CALL Species("C_5H_6O_3",                                    "", "SYCYSIWUSJFZTN-UHFFFAOYSA-N", "MCM:EPXMC4DIAL", "")
  CALL Species("C_5H_6O_5",                                    "", "LLYFBEXLHOFXFI-UHFFFAOYSA-N", "MCM:EPXMDLCO3H", "")
  CALL Species("C_5H_{10}O_2",                                 "", "HSCUZOQCNBPBST-UHFFFAOYSA-N", "MCM:MTBEBCHO", "")
  CALL Species("C_6H_{12}O_2",                                 "", "RPLPGIHCAYAYKX-UHFFFAOYSA-N", "MCM:BOX2ECHO", "")
  CALL Species("C_6H_6O_5",                                    "", "LTCNUSVDDXPSIF-UHFFFAOYSA-N", "MCM:BZEMUCCO3H", "")
  CALL Species("C_6H_6O_3",                                    "", "NQHJMOLWTXQPLS-UHFFFAOYSA-N", "MCM:BZEPOXMUC", "")
  CALL Species("C_6H_8O_3",                                    "", "QTKFOEFUZGORGI-UHFFFAOYSA-N", "MCM:EPXM2C4DAL", "")
  CALL Species("C_6H_8O_5",                                    "", "CFBRPHMFNHCZFV-UHFFFAOYSA-N", "MCM:EPXM2DCO3H", "")
  CALL Species("C_6H_{12}O_2",                                 "", "HHIYJPQUMUPZIZ-UHFFFAOYSA-N", "MCM:ETBEACHO", "")
  CALL Species("C_6H_{12}O_2",                                 "", "SMUYZOMGNHBYHU-UHFFFAOYSA-N", "MCM:ETBECCHO", "")
  CALL Species("C_6H_{12}O_2",                                 "", "AAPDVSFWBQNMLJ-UHFFFAOYSA-N", "MCM:IPROMCCHO", "")
  CALL Species("C_7H_{10}O_3",                                 "", "MOJUYSNCEBTQMW-UHFFFAOYSA-N", "MCM:EPXMEC4DAL", "")
  CALL Species("C_7H_{10}O_5",                                 "", "IBELTJZUJIPVRI-UHFFFAOYSA-N", "MCM:EPXMEDCO3H", "")
  CALL Species("C_8H_{10}O_3",                                 "", "QLTFECLMPRWGQQ-UHFFFAOYSA-N", "MCM:OXYEPOXMUC", "")
  CALL Species("C_8H_{10}O_5",                                 "", "PBSKNYKKTPMDFL-UHFFFAOYSA-N", "MCM:OXYMUCCO3H", "")
  CALL Species("C_9H_{12}O_5",                                 "", "UVJSWMTUJCHEOZ-UHFFFAOYSA-N", "MCM:OETLMUCO3H", "")
  CALL Species("C_9H_{12}O_3",                                 "", "NORSPZZKBNMHEY-UHFFFAOYSA-N", "MCM:OETLPOXMUC", "")
  CALL Species("C_5H_8O_3",                                    "", "SXTKPSXTIDLSQV-UHFFFAOYSA-N", "MCM:IEACHO", "")
  CALL Species("C_5H_8O_3",                                    "", "FUNIHXJAXNKVQS-UHFFFAOYSA-N", "MCM:IECCHO", "")
  CALL Species("C_6H_8O_5",                                    "", "HYRDKGAOZPVDDL-UHFFFAOYSA-N", "MCM:BZEMUCOH", "")
  CALL Species("C_6H_8O_6",                                    "", "YPGXWAYLBNMEJL-UHFFFAOYSA-N", "MCM:BZEMUCOOH", "")
  CALL Species("C_8H_{12}O_5",                                 "", "SKTLSFGOHJFJLD-UHFFFAOYSA-N", "MCM:OXYMUCOH", "")
  CALL Species("C_8H_{12}O_6",                                 "", "ZBCSCRLAPOVYAB-UHFFFAOYSA-N", "MCM:OXYMUCOOH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "BKCWOZADWPGCRT-UHFFFAOYSA-N", "MCM:OETLMUCOH", "")
  CALL Species("C_9H_{14}O_6",                                 "", "OMPONQJDZISYFQ-UHFFFAOYSA-N", "MCM:OETLMUCOOH", "")
  CALL Species("C_4H_8O_4",                                    "", "WKDIYCKWLNCFHJ-UHFFFAOYSA-N", "MCM:PRONEMOOOH", "")
  CALL Species("C_4H_8O_2",                                    "", "CUZLJOLBIRPEFB-UHFFFAOYSA-N", "MCM:PRONEMOX", "")
  CALL Species("C_6H_8O_5",                                    "", "ABDSVLWFPNKKBV-UHFFFAOYSA-N", "MCM:EPXKTMCO3H", "")
  CALL Species("C_6H_8O_5",                                    "", "CAGQARJHZCESJI-UHFFFAOYSA-N", "MCM:EPXMKTCO3H", "")
  CALL Species("C_7H_{14}O_4",                                 "", "YHFJWWLEGMYFFZ-UHFFFAOYSA-N", "MCM:BOXPROBOOH", "")
  CALL Species("C_7H_{14}O_2",                                 "", "UYNCDYOFUJEUQN-UHFFFAOYSA-N", "MCM:BOXPRONE", "")
  CALL Species("C_7H_{14}O_4",                                 "", "YUTHBWBUPOPMPG-UHFFFAOYSA-N", "MCM:BOXPROOOH", "")
  CALL Species("C_7H_{10}O_5",                                 "", "PGCJNBUAAIEYIA-UHFFFAOYSA-N", "MCM:EPXEKTCO3H", "")
  CALL Species("C_7H_8O_5",                                    "", "RISGILSSNRWCFR-UHFFFAOYSA-N", "MCM:TLEMUCCO3H", "")
  CALL Species("C_8H_{10}O_5",                                 "", "GTEVSPXWKYUSQE-UHFFFAOYSA-N", "MCM:EBZMUCCO3H", "")
  CALL Species("C_8H_{10}O_5",                                 "", "ROKAXOITEMHIEK-UHFFFAOYSA-N", "MCM:MXYMUCCO3H", "")
  CALL Species("C_8H_{10}O_5",                                 "", "AYUPQGRQZSTAMO-UHFFFAOYSA-N", "MCM:PXYMUCCO3H", "")
  CALL Species("C_9H_{12}O_5",                                 "", "SKJBPGYNTSYQCQ-UHFFFAOYSA-N", "MCM:IPBZMUCO3H", "")
  CALL Species("C_9H_{12}O_5",                                 "", "CXRRYJPLLOUVIM-UHFFFAOYSA-N", "MCM:METLMUCO3H", "")
  CALL Species("C_9H_{12}O_5",                                 "", "ZGGPGQBINYKWGU-UHFFFAOYSA-N", "MCM:PBZMUCCO3H", "")
  CALL Species("C_9H_{12}O_5",                                 "", "KWWJCAVGGMMMRV-UHFFFAOYSA-N", "MCM:PETLMUCO3H", "")
  CALL Species("C_9H_{12}O_3",                                 "", "XYHNNFZVTWOCNN-UHFFFAOYSA-N", "MCM:TM123OXMUC", "")
  CALL Species("C_9H_{12}O_5",                                 "", "FTWAFKXUKWKGAJ-UHFFFAOYSA-N", "MCM:TM124MUO3H", "")
  CALL Species("C_9H_{12}O_5",                                 "", "JNFPQQOSCDIIIK-UHFFFAOYSA-N", "MCM:TM135MUO3H", "")
  CALL Species("C_{10}H_{14}O_5",                              "", "JWBQDZWURHDWCA-UHFFFAOYSA-N", "MCM:DMEBMUO3H", "")
  CALL Species("C_{11}H_{16}O_5",                              "", "QEEAHFHJZRPBGG-UHFFFAOYSA-N", "MCM:DETLMUO3H", "")
  CALL Species("C_{14}H_{22}O_4",                              "", "JTZKRSQXPBULRM-UHFFFAOYSA-N", "MCM:BCKSOZ", "")
  CALL Species("C_4H_8O_3",                                    "", "RETWRLMOZHRKHX-UHFFFAOYSA-N", "MCM:PRONEMOXOH", "")
  CALL Species("C_7H_{14}O_3",                                 "", "ANPLAICHKIQBPV-UHFFFAOYSA-N", "MCM:BOXPRONBOH", "")
  CALL Species("C_7H_{14}O_3",                                 "", "NQJNHGUTJISDHM-UHFFFAOYSA-N", "MCM:BOXPRONOH", "")
  CALL Species("C_9H_{12}O_5",                                 "", "ZVWGUKJQFRZUIG-UHFFFAOYSA-N", "MCM:TM123MUCCO", "")
  CALL Species("C_9H_{14}O_5",                                 "", "JUUFKUYJQYFRGA-UHFFFAOYSA-N", "MCM:TM123MUCOH", "")
  CALL Species("C_9H_{14}O_6",                                 "", "LLVDHGQYZCXGKH-UHFFFAOYSA-N", "MCM:TM123MUOOH", "")
  CALL Species("C_6H_8O_3",                                    "", "QIJMAVIYYVSLPO-UHFFFAOYSA-N", "MCM:EPXALMKT", "")
  CALL Species("C_6H_8O_3",                                    "", "MWBGBWUOLRJCDU-UHFFFAOYSA-N", "MCM:EPXMALKT", "")
  CALL Species("C_7H_{10}O_3",                                 "", "HQSBJPSKMKUTIT-UHFFFAOYSA-N", "MCM:EPXEALKT", "")
  CALL Species("C_8H_{10}O_3",                                 "", "UTCVUMOGNJSPDS-UHFFFAOYSA-N", "MCM:EBZPOXMUC", "")
  CALL Species("C_8H_{10}O_3",                                 "", "VLMKIGFKBHEGBU-UHFFFAOYSA-N", "MCM:MXYEPOXMUC", "")
  CALL Species("C_8H_{10}O_3",                                 "", "LGKLIBFTBUCZPW-UHFFFAOYSA-N", "MCM:PXYEPOXMUC", "")
  CALL Species("C_9H_{12}O_3",                                 "", "NUCYHGVQTCVYLO-UHFFFAOYSA-N", "MCM:IPBZPOXMUC", "")
  CALL Species("C_9H_{12}O_3",                                 "", "SWHMTTAIKVELRE-UHFFFAOYSA-N", "MCM:METLPOXMUC", "")
  CALL Species("C_9H_{12}O_3",                                 "", "AHKQGNMGHQLOPP-UHFFFAOYSA-N", "MCM:PBZPOXMUC", "")
  CALL Species("C_9H_{12}O_3",                                 "", "UTBNMXYREJYVNQ-UHFFFAOYSA-N", "MCM:PETLPOXMUC", "")
  CALL Species("C_9H_{12}O_3",                                 "", "HFTMUUPKJAPOHL-UHFFFAOYSA-N", "MCM:TM124OXMUC", "")
  CALL Species("C_9H_{12}O_3",                                 "", "JFCWARPMSNNZLY-UHFFFAOYSA-N", "MCM:TM135OXMUC", "")
  CALL Species("C_{10}H_{14}O_3",                              "", "VGOICOHBXPGRQT-UHFFFAOYSA-N", "MCM:DMEBPOXMUC", "")
  CALL Species("C_{11}H_{16}O_3",                              "", "VPGNNECRJBEQEI-UHFFFAOYSA-N", "MCM:DETLPOXMUC", "")
  CALL Species("C_6H_6O_5",                                    "", "KQXCODNTIAPVKL-UHFFFAOYSA-N", "MCM:BZEMUCCO", "")
  CALL Species("C_7H_8O_5",                                    "", "AJWOUZMMNQLKHW-UHFFFAOYSA-N", "MCM:TLEMUCCO", "")
  CALL Species("C_7H_{10}O_5",                                 "", "WPTVRQIFDQMAMS-UHFFFAOYSA-N", "MCM:TLEMUCOH", "")
  CALL Species("C_7H_{10}O_6",                                 "", "DVPKQTIPCHKXBD-UHFFFAOYSA-N", "MCM:TLEMUCOOH", "")
  CALL Species("C_8H_{10}O_5",                                 "", "MYHWFJZUBUHIGJ-UHFFFAOYSA-N", "MCM:EBZMUCCO", "")
  CALL Species("C_8H_{12}O_5",                                 "", "VKTUBYIQTACODT-UHFFFAOYSA-N", "MCM:EBZMUCOH", "")
  CALL Species("C_8H_{12}O_6",                                 "", "KWCWGZCSJYJCAS-UHFFFAOYSA-N", "MCM:EBZMUCOOH", "")
  CALL Species("C_8H_{10}O_5",                                 "", "KJOIECUJZBFUMR-UHFFFAOYSA-N", "MCM:MXYMUCCO", "")
  CALL Species("C_8H_{12}O_5",                                 "", "UZLLCOIUEBQPHF-UHFFFAOYSA-N", "MCM:MXYMUCOH", "")
  CALL Species("C_8H_{12}O_6",                                 "", "ZYWDOJNDZBPDDO-UHFFFAOYSA-N", "MCM:MXYMUCOOH", "")
  CALL Species("C_8H_{10}O_5",                                 "", "RJHLIQPRSGXWHC-UHFFFAOYSA-N", "MCM:OXYMUCCO", "")
  CALL Species("C_8H_{10}O_5",                                 "", "UICQALDYFNTPHO-UHFFFAOYSA-N", "MCM:PXYMUCCO", "")
  CALL Species("C_8H_{12}O_5",                                 "", "MIBKDAKBVPRIGB-UHFFFAOYSA-N", "MCM:PXYMUCOH", "")
  CALL Species("C_8H_{12}O_6",                                 "", "GSJPWDRXFYTPMH-UHFFFAOYSA-N", "MCM:PXYMUCOOH", "")
  CALL Species("C_9H_{12}O_5",                                 "", "XOMPDNKHXMMXOK-UHFFFAOYSA-N", "MCM:IPBZMUCCO", "")
  CALL Species("C_9H_{14}O_5",                                 "", "AKVRVUIGBHJRNQ-UHFFFAOYSA-N", "MCM:IPBZMUCOH", "")
  CALL Species("C_9H_{14}O_6",                                 "", "LSOCHQKLDSXALN-UHFFFAOYSA-N", "MCM:IPBZMUCOOH", "")
  CALL Species("C_9H_{12}O_5",                                 "", "ZRMCSOCJQIXUEX-UHFFFAOYSA-N", "MCM:METLMUCCO", "")
  CALL Species("C_9H_{14}O_5",                                 "", "ARJLZFCPXRPKBS-UHFFFAOYSA-N", "MCM:METLMUCOH", "")
  CALL Species("C_9H_{14}O_6",                                 "", "UXABECSPQFCGIJ-UHFFFAOYSA-N", "MCM:METLMUCOOH", "")
  CALL Species("C_9H_{12}O_5",                                 "", "JSKKVRLLVVSVCZ-UHFFFAOYSA-N", "MCM:OETLMUCCO", "")
  CALL Species("C_9H_{12}O_5",                                 "", "FCRYPGHUONBQCX-UHFFFAOYSA-N", "MCM:PBZMUCCO", "")
  CALL Species("C_9H_{14}O_5",                                 "", "VYXNVIKZUYYSSA-UHFFFAOYSA-N", "MCM:PBZMUCOH", "")
  CALL Species("C_9H_{14}O_6",                                 "", "FCKGMMNUJIAHAV-UHFFFAOYSA-N", "MCM:PBZMUCOOH", "")
  CALL Species("C_9H_{12}O_5",                                 "", "RORTYMWKJCCJQU-UHFFFAOYSA-N", "MCM:PETLMUCCO", "")
  CALL Species("C_9H_{14}O_5",                                 "", "ZLOWXLKYIJYQIH-UHFFFAOYSA-N", "MCM:PETLMUCOH", "")
  CALL Species("C_9H_{14}O_6",                                 "", "RSNULGXAYPPPGL-UHFFFAOYSA-N", "MCM:PETLMUCOOH", "")
  CALL Species("C_9H_{12}O_5",                                 "", "DQECTOJLTKBSTJ-UHFFFAOYSA-N", "MCM:TM124MUCCO", "")
  CALL Species("C_9H_{14}O_5",                                 "", "KPYCPCVRGFCQPX-UHFFFAOYSA-N", "MCM:TM124MUCOH", "")
  CALL Species("C_9H_{14}O_6",                                 "", "DQFPDDPNMNSRMG-UHFFFAOYSA-N", "MCM:TM124MUOOH", "")
  CALL Species("C_9H_{14}O_5",                                 "", "CZQCDAJVGDGQTR-UHFFFAOYSA-N", "MCM:TM135MUOH", "")
  CALL Species("C_9H_{14}O_6",                                 "", "FIFCCDBMOKNVLN-UHFFFAOYSA-N", "MCM:TM135MUOOH", "")
  CALL Species("C_{10}H_{16}O_5",                              "", "MURMIEBRJYKLPQ-UHFFFAOYSA-N", "MCM:DMEBMUOH", "")
  CALL Species("C_{10}H_{16}O_6",                              "", "QHUZCFCKANZAOE-UHFFFAOYSA-N", "MCM:DMEBMUOOH", "")
  CALL Species("C_{11}H_{18}O_5",                              "", "BLCXZNPOSIAMRZ-UHFFFAOYSA-N", "MCM:DETLMUOH", "")
  CALL Species("C_{11}H_{18}O_6",                              "", "ZGRWZBBDFVAIEE-UHFFFAOYSA-N", "MCM:DETLMUOOH", "")
  CALL Species("C_4H_8O_3",                                    "", "YZGQDNOIGFBYKF-UHFFFAOYSA-N", "MCM:ETOMECO2H", "")
  CALL Species("C_5H_{10}O_3",                                 "", "BKBZFJRHYSCZQA-UHFFFAOYSA-N", "MCM:MTBEBCO2H", "")
  CALL Species("C_6H_{12}O_3",                                 "", "AJQOASGWDCBKCJ-UHFFFAOYSA-N", "MCM:BOXMCO2H", "")
  CALL Species("C_4H_4O_4",                                    "", "BIFCJMYQXQSHFE-UHFFFAOYSA-N", "MCM:EPXDLCO2H", "")
  CALL Species("C_5H_6O_4",                                    "", "HNJRQDPCPFMZIJ-UHFFFAOYSA-N", "MCM:EPXMDLCO2H", "")
  CALL Species("C_6H_6O_4",                                    "", "YYDFWEKDUXDLMC-UHFFFAOYSA-N", "MCM:BZEMUCCO2H", "")
  CALL Species("C_6H_8O_4",                                    "", "TVFMSHFDVSCCSQ-UHFFFAOYSA-N", "MCM:EPXM2DCO2H", "")
  CALL Species("C_7H_{10}O_4",                                 "", "CTYOPZSWGBYACX-UHFFFAOYSA-N", "MCM:EPXMEDCO2H", "")
  CALL Species("C_8H_{10}O_4",                                 "", "CDVXJWVOWNWWPU-UHFFFAOYSA-N", "MCM:OXYMUCCO2H", "")
  CALL Species("C_9H_{12}O_4",                                 "", "UVQOKFRFLASUOJ-UHFFFAOYSA-N", "MCM:OETLMUCO2H", "")
  CALL Species("C_6H_8O_4",                                    "", "BXVGKWFSHJVVPB-UHFFFAOYSA-N", "MCM:EPXKTMCO2H", "")
  CALL Species("C_6H_8O_4",                                    "", "FTLSPZDBESPPRR-UHFFFAOYSA-N", "MCM:EPXMKTCO2H", "")
  CALL Species("C_7H_{10}O_4",                                 "", "XBMXXNQRKRSRRJ-UHFFFAOYSA-N", "MCM:EPXEKTCO2H", "")
  CALL Species("C_7H_8O_4",                                    "", "YZQFSRDEODCLRC-UHFFFAOYSA-N", "MCM:TLEMUCCO2H", "")
  CALL Species("C_8H_{10}O_4",                                 "", "BSMJHSATRRVHTI-UHFFFAOYSA-N", "MCM:EBZMUCCO2H", "")
  CALL Species("C_8H_{10}O_4",                                 "", "BMJHPUYAWBLOER-UHFFFAOYSA-N", "MCM:MXYMUCCO2H", "")
  CALL Species("C_8H_{10}O_4",                                 "", "FMBYVAMRNZYNGO-UHFFFAOYSA-N", "MCM:PXYMUCCO2H", "")
  CALL Species("C_9H_{12}O_4",                                 "", "LXYULRJMRUJREP-UHFFFAOYSA-N", "MCM:IPBZMUCO2H", "")
  CALL Species("C_9H_{12}O_4",                                 "", "TZAADSLPRMCSBM-UHFFFAOYSA-N", "MCM:METLMUCO2H", "")
  CALL Species("C_9H_{12}O_4",                                 "", "MSVJIRWCUUEXSY-UHFFFAOYSA-N", "MCM:PBZMUCCO2H", "")
  CALL Species("C_9H_{12}O_4",                                 "", "YZGPSCFZKHZHEQ-UHFFFAOYSA-N", "MCM:PETLMUCO2H", "")
  CALL Species("C_9H_{12}O_4",                                 "", "VDBMALGYQLXWML-UHFFFAOYSA-N", "MCM:TM124MUO2H", "")
  CALL Species("C_9H_{12}O_4",                                 "", "RLMHTEATULORAU-UHFFFAOYSA-N", "MCM:TM135MUO2H", "")
  CALL Species("C_{10}H_{14}O_4",                              "", "GTDCDBIZJZUTJP-UHFFFAOYSA-N", "MCM:DMEBMUO2H", "")
  CALL Species("C_{11}H_{16}O_4",                              "", "FTAMRHAZJZCWNK-UHFFFAOYSA-N", "MCM:DETLMUO2H", "")
  CALL Species("C_3H_6O_5",                                    "", "ISFGCMQHKVEPTP-UHFFFAOYSA-N", "MCM:DMCOOH", "")
  CALL Species("C_3H_6O_3",                                    "", "VKWJMTLAAJULGF-UHFFFAOYSA-N", "MCM:MMF", "")
  CALL Species("C_3H_6O_5",                                    "", "AAAZTRZXKHTMLL-UHFFFAOYSA-N", "MCM:MMFOOH", "")
  CALL Species("C_3H_6O_4",                                    "", "YGODNDLTUXBUQH-UHFFFAOYSA-N", "MCM:DMCOH", "")
  CALL Species("C_3H_6O_4",                                    "", "GDDJZBRTXIGGHV-UHFFFAOYSA-N", "MCM:MMFOH", "")
  CALL Species("C_4H_6O_5",                                    "", "LEKXYOUPWKVTGM-UHFFFAOYSA-N", "GOLIG1", "") ! SMILES: O=CC(O)-O-C(O)C=O
  CALL Species("C_4H_8O_6",                                    "", "GWSRJJGBNXJVPO-UHFFFAOYSA-N", "GOLIG2", "") ! SMILES: O=CC(O)-O-C(O)C(O)O
  CALL Species("C_4H_{10}O_7",                                 "", "ADECTKUVLJSMDK-UHFFFAOYSA-N", "GOLIG3", "") ! SMILES: OC(O)C(O)-O-C(O)C(O)O
  CALL Species("C_6H_{10}O_6",                                 "", "UCPBJDKPOQEXSQ-UHFFFAOYSA-N", "MGLYFB", "") ! SMILES: CC(=O)C(=O)-O-C(O)(C)C(O)(O)
  CALL Species("C_6H_{10}O_5",                                 "", "QMSLRDMQAWWKIZ-UHFFFAOYSA-N", "MGLYOXDA", "") ! SMILES: CC(=O)C(O)-O-C(O)C(=O)C
  CALL Species("C_6H_{12}O_6",                                 "", "IUHMFBWAUOLAFM-UHFFFAOYSA-N", "MGLYOXDB", "") ! SMILES: CC(=O)C(O)-O-C(O)(C)C(O)(O)
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Heterocycles with oxygen")
  CALL Species("C_3H_4O_2",                             "57-57-8", "VEZXCJBBBCKRPI-UHFFFAOYSA-N", "$\beta$-propiolactone", "")
  CALL Species("C_4H_8O_2",                            "497-26-7", "HTWIZMNMTWYQRN-UHFFFAOYSA-N", "2-methyl-1,3-dioxolane", "")
  CALL Species("C_5H_8O_3",                                    "", "QGQUFAZKBSHWQB-UHFFFAOYSA-N", "DIEPOXO3", "") ! SMILES: CC1(C(=O)CO)CO1
  CALL Species("C_5H_8O_3",                                    "", "DNQIATHBSKHPLN-UHFFFAOYSA-N", "IEPOXO4", "") ! SMILES: CC1(CO)OC1C=O
  CALL Species("C_5H_{10}O_2",                        "2916-31-6", "SIJBDWPVNAYVGY-UHFFFAOYSA-N", "2,2-dimethyl-1,3-dioxolane", "")
  CALL Species("C_5H_{10}O_3",                                 "", "UNAIOYXCADSMDR-UHFFFAOYSA-N", "METHFDIOL", "") ! SMILES: C1C(O)-O-C(O)C1(C)
  CALL Species("C_6H_{10}O_2",                        "4454-05-1", "XCYWUZHUTJDTGS-UHFFFAOYSA-N", "3,4-dihydro-2-methoxy-2H-pyran", "")
  CALL Species("C_6H_6O_3",                            "118-71-8", "XPCTZQVDEJYUGT-UHFFFAOYSA-N", "maltol", "")
  CALL Species("C_5H_6O",                              "534-22-5", "VQKFNUFAXTZWDK-UHFFFAOYSA-N", "2-methylfuran", "")
  CALL Species("C_5H_6O",                              "930-27-8", "KJRRQXYWFQKJIP-UHFFFAOYSA-N", "3-methylfuran", "")
  CALL Species("C_5H_6O_2",                             "98-00-0", "XPFVYQJUAUNWIW-UHFFFAOYSA-N", "2-furanmethanol", "furfuryl alcohol")
  CALL Species("C_6H_{12}O_2",                         "100-72-1", "ROTONRWJLXYJBD-UHFFFAOYSA-N", "tetrahydropyran-2-methanol", "")
  CALL Species("C_2H_4O",                               "75-21-8", "IAYPIBMASNFSPL-UHFFFAOYSA-N", "oxirane", "ethylene oxide")
  CALL Species("C_3H_6O",                               "75-56-9", "GOOHAUXETOMSMM-UHFFFAOYSA-N", "1,2-epoxypropane", "1,2-propylene oxide") ! other CAS: 16033-71-9
  CALL Species("C_8H_8O",                               "96-09-3", "AWMVMTVKBNGEAK-UHFFFAOYSA-N", "phenyloxirane", "styrene oxide")
  CALL Species("C_4H_4O",                              "110-00-9", "YLQBMQCUIZJEEH-UHFFFAOYSA-N", "oxacyclopentadiene", "furan; furfuran")
  CALL Species("C_{12}H_8O",                           "132-64-9", "TXCDCPKCNAJMEE-UHFFFAOYSA-N", "dibenzofuran", "2,2'-biphenylene oxide")
  CALL Species("C_5H_4O_2",                             "98-01-1", "HYBBIBNJHNGZAN-UHFFFAOYSA-N", "2-furancarboxaldehyde", "furfural; 2-furanaldehyde")
  CALL Species("C_6H_6O_2",                            "620-02-0", "OUDFNZMQXZILJD-UHFFFAOYSA-N", "5-methylfurfural", "")
  CALL Species("C_4H_8O",                              "558-30-5", "GELKGHVAFRCJNA-UHFFFAOYSA-N", "2,2-dimethyloxirane", "isobutylene oxide")
  CALL Species("C_4H_8O",                              "109-99-9", "WYURNTSHIVDZCO-UHFFFAOYSA-N", "tetrahydrofuran", "THF")
  CALL Species("C_4D_8O",                             "1693-74-9", "WYURNTSHIVDZCO-SVYQBANQSA-N", "tetrahydrofuran-d8", "THF-d8")
  CALL Species("CH_3C_4H_7O",                           "96-47-9", "JWUJQDFVADABEY-UHFFFAOYSA-N", "2-methyltetrahydrofuran", "")
  CALL Species("(CH_3)_2C_4H_6O",                     "1003-38-9", "OXMIDRBAFOEOQT-UHFFFAOYSA-N", "2,5-dimethyltetrahydrofuran", "")
  CALL Species("C_5H_{10}O",                           "142-68-7", "DHXVGJBLRPWPCS-UHFFFAOYSA-N", "tetrahydropyran", "THP")
  CALL Species("C_6H_{12}O",                         "26093-63-0", "UJQZTMFRMLEYQN-UHFFFAOYSA-N", "3-methyltetrahydropyran", "")
  CALL Species("C_5H_8O",                              "110-87-2", "BUDQDWGNQVEFAC-UHFFFAOYSA-N", "3,4-dihydro-2H-pyran", "")
  CALL Species("C_3H_6O_2",                            "646-06-0", "WNXJIVFYUVYPPR-UHFFFAOYSA-N", "1,3-dioxolane", "")
  CALL Species("C_4H_8O_2",                            "505-22-6", "VDFVNEFVBPFDSB-UHFFFAOYSA-N", "1,3-dioxane", "")
  CALL Species("C_4H_8O_2",                            "123-91-1", "RYHBNJHYFVUHQT-UHFFFAOYSA-N", "1,4-dioxane", "dioxane")
  CALL Species("C_4D_8O_2",                          "17647-74-4", "RYHBNJHYFVUHQT-SVYQBANQSA-N", "1,4-dioxane-d8", "dioxane-d8")
  CALL Species("C_3H_6O_3",                            "110-88-3", "BGJSXRVXTHVRSN-UHFFFAOYSA-N", "trioxane", "")
  CALL Species("C_4H_6O_3",                            "108-32-7", "RUOJZAUFBMNUDX-UHFFFAOYSA-N", "4-methyl-1,3-dioxolan-2-one", "propylene carbonate")
  CALL Species("C_{10}H_{18}O",                        "470-82-6", "WEEGYLXZBRQIMU-UHFFFAOYSA-N", "1,3,3-trimethyl-2-oxabicyclo[2.2.2]octane", "eucalyptol; 1,8-cineole")
  CALL Species("C_{10}H_{16}O",                       "1195-92-2", "CCEFMUBVSUDRLG-UHFFFAOYSA-N", "limonene oxide", "") ! trans limonene oxide = 4959-35-7
  CALL Species("C_{12}H_8O_2",                         "262-12-4", "NFBOHOGPQUYFRF-UHFFFAOYSA-N", "dibenzo[$b,e$][1,4]dioxin", "dibenzo-$p$-dioxin")
  CALL Species("C_8H_6O_3",                            "120-57-0", "SATCULPHIDQDRE-UHFFFAOYSA-N", "piperonal", "")
  CALL Species("C_6H_{12}O_3",                         "123-63-7", "SQYNKIJPMDEDEG-UHFFFAOYSA-N", "paraldehyde", "")
  CALL Species("C_8H_6O",                              "271-89-6", "IANQTJSKSUMEQM-UHFFFAOYSA-N", "benzofuran", "")
  CALL Species("C_9H_{16}O_2",                         "104-61-0", "OALYTRUKMRCXNH-UHFFFAOYSA-N", "$\gamma$-nonalactone", "")
  CALL Species("C_{13}H_{10}O",                         "92-83-1", "GJCOSYZMQJWQCA-UHFFFAOYSA-N", "xanthene", "")
  CALL Species("C_{16}H_{28}O",                       "3738-00-9", "YPZUZOLGGMJZJO-UHFFFAOYSA-N", "1,5,5,9-tetramethyl-13-oxatricyclo(8.3.0.0(4,9))tridecane", "ambroxan")
  CALL Species("C_{18}H_{26}O",                       "1222-05-5", "ONKNPOPIGWHAQC-UHFFFAOYSA-N", "1,3,4,6,7,8-hexahydro-4,6,6,7,8,8-hexamethylcyclopenta[$g$]-2-benzopyran", "")
  CALL Species("C_{18}H_{26}O_2",                    "87818-31-3", "QMTNOLKHSWIQBE-FGTMMUONSA-N", "cinmethylin", "")
  CALL Species("C_{32}H_{46}O_7",                    "51596-11-3", "VOZIAWLUULBIPN-LRBNAKOISA-N", "milbemycin A4", "milbemectin A4")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Oxidized terpenoids")

  CALL Species("C_{10}H_{18}O",                        "464-45-9", "DTGKSKDOIYIVQL-QXFUBDJGSA-N", "(1S-endo)-1,7,7-trimethyl-bicyclo[2.2.1]heptan-2-ol", "1S-endo-(-)-borneol")
  CALL Species("C_{10}H_{18}O",                       "2217-02-9", "IAIHUHQCLTYTSF-OYNCUSHFSA-N", "(1R)-1,3,3-trimethylbicyclo[2.2.1]heptan-2-ol", "endo-(+)-fenchyl alcohol")
  CALL Species("C_{10}H_{18}O",                         "98-55-5", "WUOACPNHFRMFPN-UHFFFAOYSA-N", "2-(4-methyl-3-cyclohexen-1-yl)-2-propanol", "$\alpha$-terpineol")
  CALL Species("C_{10}H_{18}O",                      "72402-00-7", "ZRVPDCMGGOSDKG-UHFFFAOYSA-N", "1,2-dimethyl-3-(1-methylethenyl)-cyclopentanol", "plinol") ! other CAS: 11039-70-6, 4028-60-8
  CALL Species("C_{10}H_{18}O",                        "470-67-7", "RFFOTVCVTJUTAD-UHFFFAOYSA-N", "1-methyl-4-(1-methylethyl)-7-oxabicyclo[2.2.1]heptane", "1,4-cineole")
  CALL Species("C_{10}H_{16}O",                         "76-22-2", "DSSYKIVIOFKYAU-UHFFFAOYSA-N", "1,7,7-trimethyl-bicyclo[2.2.1]heptan-2-one", "camphor")
  CALL Species("C_{10}H_{16}O",                       "1686-14-2", "NQFUSWIGRKFAHK-UHFFFAOYSA-N", "2,7,7-trimethyl-3-oxatricyclo[4.1.1.0(2,4)]octane", "(-)-$\alpha$-pinene oxide")
  CALL Species("C_{10}H_{16}O",                         "89-82-7", "NZGWDASTMWDZIW-UHFFFAOYSA-N", "5-methyl-2-(1-methylethylidene)-cyclohexanone", "pulegone")
  CALL Species("C_{12}H_{22}O_2",                     "7070-15-7", "IWWCSDGEIDYEJV-JBLDHEPKSA-N", "exo-2-[(1,7,7-trimethylbicyclo[2.2.1]hept-2-yl)-oxy]ethanol", "arbanol")
  CALL Species("C_{12}H_{20}O_2",                     "5655-61-8", "KGEKLUUHTZCSIP-SCVCMEIPSA-N", "bornyl acetate", "")
  CALL Species("C_{13}H_{20}O",                         "79-77-6", "PSQYTAPXSHCGMF-BQYQJAHWSA-N", "$\beta$-ionone", "") ! isomer mix = 14901-07-6
  CALL Species("C_{15}H_{26}O",                       "7212-44-4", "FQTLCLSUCSAZDY-UHFFFAOYSA-N", "nerolidol", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Miscellaneous")
  CALL Species("OHCCOOH",                              "298-12-4", "HHLFWLYXYJOTON-UHFFFAOYSA-N", "oxoethanoic acid", "glyoxylic acid")
  CALL Species("HOCH_2COOH",                            "79-14-1", "AEMRFAOFKBGASW-UHFFFAOYSA-N", "hydroxyethanoic acid", "glycolic acid")
  CALL Species("HOCH_2CHO",                            "141-46-8", "WGCNASOHLSPBMP-UHFFFAOYSA-N", "2-hydroxyethanal", "hydroxyacetaldehyde; glycolaldehyde")
  CALL Species("CH_3COCHO",                             "78-98-8", "AIJULSRZWUXGPQ-UHFFFAOYSA-N", "propanonal", "methylglyoxal; pyruvaldehyde")
  CALL Species("CH_3COCOOH",                           "127-17-3", "LCTONWCANYUPML-UHFFFAOYSA-N", "2-oxopropanoic acid", "pyruvic acid")
  CALL Species("OHCCH_2COOH",                          "926-61-4", "OAKURXIZZOAYBC-UHFFFAOYSA-N", "3-oxopropanoic acid", "")
  CALL Species("CH_3CHOHCOOH",                          "50-21-5", "JVTAAEKCZFNVCJ-UHFFFAOYSA-N", "2-hydroxypropanoic acid", "lactic acid")
  CALL Species("C_3H_4O_2",                            "765-34-4", "IWYRWIUNAVNFPE-UHFFFAOYSA-N", "glycidaldehyde", "")
  CALL Species("C_3H_6O",                              "503-30-0", "AHHWIHXENZJRFG-UHFFFAOYSA-N", "trimethylene oxide", "1,3-epoxypropane; 1,3-propylene oxide; oxetane")
  CALL Species("C_3H_6O_3",                            "367-47-5", "MNQZXJOMYWMBOU-UHFFFAOYSA-N", "2,3-dihydroxypropanal", "glyceraldehyde")
  CALL Species("C_3H_6O_3",                             "96-26-4", "RXKJFZQQPQGTFL-UHFFFAOYSA-N", "dihydroxyacetone", "")
  CALL Species("C_3H_8O_2",                            "109-86-4", "XNWFRZJHXBZDAG-UHFFFAOYSA-N", "2-methoxyethanol", "methyl cellosolve")
  CALL Species("OHC(CH_2)_2COOH",                      "692-29-5", "UIUJIQZEACWQSV-UHFFFAOYSA-N", "4-oxobutanoic acid", "")
  CALL Species("HOOCCHOHCHOHCOOH",                      "87-69-4", "FEWJPZIEWOKRBE-JCYAYHJZSA-N", "2,3-dihydroxybutanedioic acid", "tartaric acid")
  CALL Species("HO(CH_2)_2O(CH_2)_2OH",                "111-46-6", "MTHSVFCYNBDYFN-UHFFFAOYSA-N", "3-oxapentane-1,5-diol", "diethylene glycol")
  CALL Species("HOOCCH_2CHOHCOOH",                    "6915-15-7", "BJEPYKJPYRNKOW-UHFFFAOYSA-N", "hydroxybutanedioic acid", "malic acid")
  CALL Species("C_4H_{10}O_2",                         "110-80-5", "ZNQVEEAIQZEUHB-UHFFFAOYSA-N", "2-ethoxyethanol", "")
  CALL Species("C_4H_{10}O_2",                        "1589-47-5", "YTTFFPATQICAQN-UHFFFAOYSA-N", "2-methoxy-1-propanol", "")
  CALL Species("C_4H_{10}O_2",                         "534-15-6", "SPEUIVXLLWOEMJ-UHFFFAOYSA-N", "1,1-dimethoxyethane", "")
  CALL Species("C_4H_4O_2",                            "674-82-8", "WASQWSOJHCZDFK-UHFFFAOYSA-N", "4-methylene-2-oxetanone", "acetyl ketene")
  CALL Species("C_4H_4O_2",                            "497-23-4", "VIHAEDVKXSOUAT-UHFFFAOYSA-N", "2(5H)-furanone", "")
  CALL Species("C_4H_6O_2",                           "1464-53-5", "ZFIVKAOQEXOYFY-UHFFFAOYSA-N", "2,2'-bioxirane", "")
  CALL Species("C_4H_6O_2",                             "96-48-0", "YEJRWHAVMIAJKC-UHFFFAOYSA-N", "$\gamma$-butyrolactone", "")
  CALL Species("C_4H_8O",                              "106-88-7", "RBACIKXCRWGCBB-UHFFFAOYSA-N", "ethyloxirane", "1,2-epoxybutane")
  CALL Species("C_5H_{10}O_3",                                 "", "FLVAIUDBQNOKHB-UHFFFAOYSA-N", "2,3-epoxy-2-methyl-1,4-butanediol", "IEPOX")
  CALL Species("C_5H_8O_4",                                    "", "RRXXDUUNVWXRFN-UHFFFAOYSA-N", "MEDICO1CO", "") ! SMILES: C(=O)C(=O)C(O)(C)C(O)
  CALL Species("C_5H_8O_4",                                    "", "GXKSWJOBVXKANI-UHFFFAOYSA-N", "MEDICO4CO", "") ! SMILES: C(=O)C(C)(O)C(=O)C(O)
  CALL Species("C_5H_{10}O_4",                                 "", "LCGBCDAYKOJPSO-UHFFFAOYSA-N", "METRICO", "") ! SMILES: C(O)C(C)(O)C(=O)C(O)
  CALL Species("C_7H_8O_3",                                    "", "YHZQHUOBDYTWMQ-UHFFFAOYSA-N", "2,3-epoxy-6-oxo-heptenal", "MCM:TLEPOXMUC")
  CALL Species("C_4H_8O_2",                            "513-86-0", "ROWKJAVDOGWPAT-UHFFFAOYSA-N", "3-hydroxy-2-butanone", "acetoin")
  CALL Species("C_4H_8O_2",                            "764-48-7", "VUIWJRYTWUGOOF-UHFFFAOYSA-N", "2-(vinyloxy)ethanol", "ethylene glycol monovinyl ether")
  CALL Species("C_4H_8O",                             "2167-39-7", "FZIIBDOXPQOKBP-UHFFFAOYSA-N", "2-methyloxetane", "")
  CALL Species("OHC(CH_2)_3COOH",                     "5746-02-1", "VBKPPDYGFUZOAJ-UHFFFAOYSA-N", "5-oxopentanoic acid", "")
  CALL Species("HOOC(CH_2)_2COCOOH",                   "328-50-7", "KPGXRSRHYNQIFN-UHFFFAOYSA-N", "2-oxopentanedioic acid", "$\alpha$-keto glutaric acid")
  CALL Species("C_5H_{10}O_2",                          "97-99-4", "BSYVTEYKTMYBMK-UHFFFAOYSA-N", "tetrahydro-2-furanmethanol", "tetrahydrofurfuryl alcohol")
  CALL Species("C_5H_{10}O_5",                          "58-86-6", "PYMYPHUHKUWMLA-VPENINKCSA-N", "xylose", "")
  CALL Species("C_5H_{12}O_3",                         "111-77-3", "SBASXUCJHJRPEV-UHFFFAOYSA-N", "2-(2-methoxyethoxy)ethanol", "diethylene glycol monomethyl ether")
  CALL Species("HO(CH_2CH_2O)_3H",                     "112-27-6", "ZIBGPFATKBEMQZ-UHFFFAOYSA-N", "3,6-dioxaoctane-1,8-diol", "triethylene glycol")
  CALL Species("C_6H_{10}O_2",                         "502-44-3", "PAPBSGBWRJIAAV-UHFFFAOYSA-N", "2-oxepanone", "caprolactone")
  CALL Species("C_6H_{10}O_3",                        "2238-07-5", "GYZLOYUZLJXAJU-UHFFFAOYSA-N", "glycidyl ether", "diglycidyl ether")
  CALL Species("C_6H_{12}O_2",                         "123-42-2", "SWXVUIWOUIDPGS-UHFFFAOYSA-N", "4-hydroxy-4-methyl-2-pentanone", "diacetone alcohol")
  CALL Species("C_6H_{14}O_2",                        "1569-01-3", "FENFUOGYJVOCRY-UHFFFAOYSA-N", "1-propoxy-2-propanol", "")
  CALL Species("C_6H_{14}O_3",                         "111-90-0", "XXJWXESWEXIICW-UHFFFAOYSA-N", "2-(2-ethoxyethoxy)ethanol", "diethylene glycol monoethyl ether")
  CALL Species("C_6H_{14}O_3",                         "111-96-6", "SBZXBUIDTXKZTM-UHFFFAOYSA-N", "2,5,8-trioxanonane", "diglyme")
  CALL Species("C_6H_{14}O_3",                       "25265-71-8", "SZXQTJUDPRGNJN-UHFFFAOYSA-N", "oxydipropanol", "dipropylene glycol")
  CALL Species("C_6H_4O_2",                            "106-51-4", "AZQWKYJCGOJGHM-UHFFFAOYSA-N", "$p$-benzoquinone", "1,4-benzoquinone")
  CALL Species("C_6H_6O_3",                             "67-47-0", "NOEGNKMFWQHSLB-UHFFFAOYSA-N", "5-hydroxymethylfurfural", "5-hydroxymethyl-2-furfuraldehyde")
  CALL Species("C_6H_6O_4",                            "501-30-4", "BEJNERDRQOWKJM-UHFFFAOYSA-N", "5-hydroxy-2-(hydroxymethyl)-4H-pyran-4-one", "kojic acid")
  CALL Species("C_6H_8O_7",                             "77-92-9", "KRKNYBCHXYNGOX-UHFFFAOYSA-N", "2-hydroxy-1,2,3-propanetricarboxylic acid", "citric acid")
  CALL Species("C_7H_{14}O_2",                        "2426-08-6", "YSUQLAYJZDEMOT-UHFFFAOYSA-N", "(butoxymethyl)oxirane", "n-butyl glycidyl ether")
  CALL Species("C_7H_{16}O_2",                       "57018-52-7", "GQCZPFJGIXHZMB-UHFFFAOYSA-N", "1-(1,1-dimethylethoxy)-2-propanol", "propylene glycol mono-t-butyl ether")
  CALL Species("C_7H_{16}O_4",                         "112-35-6", "JLGLQAWTXXGVEM-UHFFFAOYSA-N", "2-[2-(2-methoxyethoxy)ethoxy]ethanol", "triethylene glycol monomethyl ether")
  CALL Species("C_7H_6O_2",                            "553-97-9", "VTWDKFNVVLAELH-UHFFFAOYSA-N", "2-methyl-$p$-benzoquinone", "")
  CALL Species("C_7H_6O_4",                            "149-29-1", "ZRWPUFFVAOMMNM-UHFFFAOYSA-N", "patulin", "")
  CALL Species("C_7H_8O_2",                            "150-19-6", "ASHGTJPOSUFTGB-UHFFFAOYSA-N", "1-hydroxy-3-methoxybenzene", "3-methoxyphenol")
  CALL Species("C_7H_8O_2",                            "150-76-5", "NWVVVBRKAWDGAB-UHFFFAOYSA-N", "mequinol", "4-methoxyphenol")
  CALL Species("C_7H_8O_2",                             "90-05-1", "LHGVFZTZFXWLCP-UHFFFAOYSA-N", "1-hydroxy-2-methoxybenzene", "guaiacol; 2-methoxyphenol")
  CALL Species("C_8H_{10}O_2",                         "150-78-7", "OHBQPCCCRFSCAX-UHFFFAOYSA-N", "1,4-dimethoxybenzene", "hydroquinone dimethyl ether")
  CALL Species("C_8H_{10}O_2",                          "93-51-6", "PETRWTHZSKVLRE-UHFFFAOYSA-N", "4-methyl-2-methoxyphenol", "")
  CALL Species("C_8H_{10}O_2",                         "501-94-0", "YCCILVSKPBXVIP-UHFFFAOYSA-N", "tyrosol", "4-hydroxybenzeneethanol")
  CALL Species("C_8H_{10}O_3",                          "91-10-1", "KLIDCXVFHGNTTM-UHFFFAOYSA-N", "1,3-dimethoxy-2-hydroxybenzene", "2,6-dimethoxyphenol")
  CALL Species("C_8H_{10}O_3",                          "85-42-7", "MUTGBJKUEZFXGO-UHFFFAOYSA-N", "hexahydro-1,3-isobenzofurandione", "hexahydrophthalic anhydride")
  CALL Species("C_8H_{10}O_3",                         "498-00-0", "ZENOXNGFMSCLLL-UHFFFAOYSA-N", "vanillyl alcohol", "")
  CALL Species("C_8H_{10}O",                           "104-93-8", "CHLICZRVGGXEOD-UHFFFAOYSA-N", "1-methoxy-4-methylbenzene", "")
  CALL Species("C_8H_{14}O_4",                         "828-00-2", "PHMNXPYGVPEQSJ-UHFFFAOYSA-N", "dimethoxane", "")
  CALL Species("C_8H_{14}O_4",                                 "", "MESHXYGXEHAFFE-UHFFFAOYSA-N", "RO5R5", "") ! SMILES: CC(C)(OC=O)C(CO)CC=O
  CALL Species("C_8H_{16}O_2",                                 "", "KZPPEBIAPHLFQD-UHFFFAOYSA-N", "5-hydroxy-2-octanone", "C82CO5OH") ! SMILES: CCCC(O)CCC(=O)C
  CALL Species("C_8H_{16}O_4",                         "108-62-3", "GKKDCARASOJPNG-UHFFFAOYSA-N", "metaldehyde", "")
  CALL Species("C_8H_{18}O_3",                         "112-36-7", "RRQYJINTUHWNHW-UHFFFAOYSA-N", "diethyl carbitol", "diethylene glycol diethyl ether")
  CALL Species("C_8H_{18}O_4",                         "112-50-5", "WFSMVVDJSNMRAR-UHFFFAOYSA-N", "2-[2-(2-ethoxyethoxy)ethoxy]ethanol", "triethylene glycol monoethyl ether")
  CALL Species("C_8H_{18}O_5",                         "112-60-7", "UWHCKJMYHZGTIT-UHFFFAOYSA-N", "tetraethylene glycol", "")
  CALL Species("C_8H_8O_3",                            "121-33-5", "MWOOGOJBHIARFG-UHFFFAOYSA-N", "vanillin", "")
  CALL Species("C_9H_{10}O_3",                         "120-47-8", "NUVBSKCKDOMJSU-UHFFFAOYSA-N", "ethylparaben", "")
  CALL Species("C_9H_{10}O_3",                         "121-32-4", "CBOQJANXLMLOSS-UHFFFAOYSA-N", "ethyl vanillin", "")
  CALL Species("C_9H_{12}O_2",                         "770-35-4", "IBLKWZIFZMJLFL-UHFFFAOYSA-N", "1-phenoxypropan-2-ol", "propylene glycol phenyl ether")
  CALL Species("C_9H_{14}O_6",                         "102-76-1", "URAYPUMNDPQOKB-UHFFFAOYSA-N", "triacetin", "")
  CALL Species("C_9H_{20}O_4",                       "24800-44-0", "LCZVSXRMYJUNFX-UHFFFAOYSA-N", "tripropylene glycol", "")
  CALL Species("C_9H_6O_2",                             "91-64-5", "ZYGHJZDHTFUPRJ-UHFFFAOYSA-N", "coumarin", "")
  CALL Species("C_9H_{12}O_2",                        "2785-89-9", "CHWNEIVBYREQRF-UHFFFAOYSA-N", "4-ethylguaiacol", "4-ethyl-2-methoxyphenol")
  CALL Species("C_9H_{12}O_3",                        "6638-05-7", "ZFBNNSOJNZBLLS-UHFFFAOYSA-N", "4-methylsyringol", "")
  CALL Species("C_9H_{10}O_3",                         "498-02-2", "DFYRUELUNQRZTB-UHFFFAOYSA-N", "acetovanillone", "apocynin")
  CALL Species("C_9H_{10}O_4",                         "134-96-3", "KCDXJAYRVLXPFO-UHFFFAOYSA-N", "syringaldehyde", "")
  CALL Species("C_{10}H_{10}O_2",                      "120-58-1", "VHVOLFRBFDOUSH-UHFFFAOYSA-N", "5-(1-propenyl)-1,3-benzodioxole", "isosafrole")
  CALL Species("C_{10}H_{10}O_2",                       "94-59-7", "ZMQAAUBTXCXRIC-UHFFFAOYSA-N", "safrole", "")
  CALL Species("C_{10}H_{12}O_2",                       "94-58-6", "MYEIDJPOUKASEC-UHFFFAOYSA-N", "5-propyl-1,3-benzodioxole", "dihydrosafrole")
  CALL Species("C_{10}H_{12}O_2",                       "97-54-1", "BJIOGJUNALELMI-UHFFFAOYSA-N", "2-methoxy-4-(1-propenyl)phenol", "isoeugenol")
  CALL Species("C_{10}H_{12}O_2",                    "26447-14-3", "CUFXMPWHOWYNSO-UHFFFAOYSA-N", "$p$-cresyl glycidyl ether", "")
  CALL Species("C_{10}H_{12}O_2",                     "5471-51-2", "NJGBTKGETPDVIK-UHFFFAOYSA-N", "4-(4-hydroxyphenyl)-2-butanone", "raspberry ketone")
  CALL Species("C_{10}H_{14}O_4",                       "93-14-1", "HSRJKNPTNIJEKV-UHFFFAOYSA-N", "guaifenesin", "")
  CALL Species("C_{10}H_{18}O_7",                              "", "YORZXZYHORHTRW-UHFFFAOYSA-N", "RO5R4O2H", "") ! SMILES: CC(C)(OC=O)C(CO)CC(OO)C(=O)CO
  CALL Species("C_{10}H_{20}O",                       "2216-51-5", "NOOLISFMXDJSKH-AEJSXWLSSA-N", "levomenthol", "$L$-menthol")
  CALL Species("C_{10}H_{22}O_3",                      "112-59-4", "GZMAAYIALGURDQ-UHFFFAOYSA-N", "diethylene glycol hexyl ether", "")
  CALL Species("C_{10}H_{22}O_4",                      "143-22-6", "COBPKKZHLDDMTB-UHFFFAOYSA-N", "2-[2-(2-butoxyethoxy)ethoxy]ethanol", "triethylene glycol monobutyl ether")
  CALL Species("C_{10}H_{14}O_2",                     "2785-87-7", "PXIKRTCSSLJURC-UHFFFAOYSA-N", "4-propylguaiacol", "2-methoxy-4-propylphenol")
  CALL Species("C_{10}H_{14}O_3",                    "13184-86-6", "KOCVACNWDMSLBM-UHFFFAOYSA-N", "vanillyl ethyl ether", "4-(ethoxymethyl)-2-methoxyphenol")
  CALL Species("C_{10}H_{14}O_3",                    "14059-92-8", "PJWDIHUFLXQRFF-UHFFFAOYSA-N", "4-ethylsyringol", "")
  CALL Species("C_{10}H_{12}O_3",                     "2503-46-0", "LFVCJQWZGDLHSD-UHFFFAOYSA-N", "guaiacylacetone", "1-(4-hydroxy-3-methoxyphenyl)-2-propanone")
  CALL Species("C_{10}H_{12}O_4",                     "2478-38-8", "OJOBTAOGJIWAGB-UHFFFAOYSA-N", "acetosyringone", "")
  CALL Species("C_{10}H_{10}O_3",                      "458-36-6", "UCZOBXQKIUMJDZ-UHFFFAOYSA-N", "coniferylaldehyde", "")
  CALL Species("C_{11}H_{12}O_3",                      "607-91-0", "BNWJOHGLIBDBOB-UHFFFAOYSA-N", "4-methoxy-6-(2-propenyl)-1,3-benzodioxole", "myristicin")
  CALL Species("C_{11}H_{14}O_3",                       "94-26-8", "QFOHBWFCKVYLES-UHFFFAOYSA-N", "butylparaben", "")
  CALL Species("C_{11}H_{16}O_2",                    "25013-16-5", "MRBKEAMVRSLQPH-UHFFFAOYSA-N", "2-{\it tert}-butyl-4-methoxyphenol", "butylated hydroxyanisole")
  CALL Species("C_{11}H_8O_3",                          "92-70-6", "ALKYHXVLJMQRLQ-UHFFFAOYSA-N", "3-hydroxy-2-naphthalenecarboxylic acid", "")
  CALL Species("C_{11}H_{16}O_3",                     "6766-82-1", "YHEWWEXPVKCVFY-UHFFFAOYSA-N", "4-propylsyringol", "")
  CALL Species("C_{11}H_{14}O_3",                      "122-48-5", "OJYLAHXKWMRDGS-UHFFFAOYSA-N", "zingerone", "vanillylacetone")
  CALL Species("C_{11}H_{14}O_3",                     "5438-54-0", "IYIVYBWBUQPWAU-UHFFFAOYSA-N", "allylsyringol", "")
  CALL Species("C_{11}H_{14}O_3",                     "6635-22-9", "YFHOHYAUMDHSBX-UHFFFAOYSA-N", "4-propenylsyringol", "")
  CALL Species("C_{11}H_{14}O_4",                     "5650-43-1", "CXCPJZXJNRBTGF-UHFFFAOYSA-N", "propionylsyringol", "")
  CALL Species("C_{11}H_{12}O_4",                     "4206-58-0", "CDICDSOGTRCHMG-ONEGZZNKSA-N", "sinapylaldehyde", "")
  CALL Species("C_{12}H_{16}O_7",                      "497-76-7", "BJRNKVDFDLYUGJ-RMPHRYRLSA-N", "arbutin", "")
  CALL Species("C_{12}H_{18}O_4",                      "532-34-3", "OKIJSNGRQAOIGZ-UHFFFAOYSA-N", "butopyronoxyl", "indalone")
  CALL Species("C_{12}H_{18}O_5",                     "2358-84-1", "XFCMNSHQOZQILR-UHFFFAOYSA-N", "diethylene glycol bis(methacrylate)", "")
  CALL Species("C_{12}H_{18}O_7",                    "18467-77-1", "FWCBATIDXGJRMF-FLNNQWSLSA-N", "dikegulac", "")
  CALL Species("C_{12}H_{18}O",                       "2078-54-8", "OLBCVFGFOZPWHH-UHFFFAOYSA-N", "propofol", "")
  CALL Species("C_{12}H_{24}O_{11}",                   "585-86-4", "VQHSOMBJVWLPSR-JVCRWLNRSA-N", "lactitol", "")
  CALL Species("C_{12}H_{24}O_{11}",                   "585-88-6", "VQHSOMBJVWLPSR-WUJBLJFYSA-N", "maltitol", "")
  CALL Species("C_{12}H_6O_3",                          "81-84-5", "GRSMWKLPSNHDHA-UHFFFAOYSA-N", "naphthalic anhydride", "")
  CALL Species("C_{12}H_8O_4",                         "298-81-7", "QXKHYNVANLEOEG-UHFFFAOYSA-N", "methoxsalen", "8-methoxypsoralen")
  CALL Species("C_{12}H_{16}O_4",                   "112468-41-4", "NULBEPOZYDYWOV-UHFFFAOYSA-N", "syringylacetone", "")
  CALL Species("C_{12}H_{16}O_4",                    "69271-91-6", "QFHXMVPZYYMTCS-UHFFFAOYSA-N", "butyrylsyringol", "")
  CALL Species("C_{13}H_{12}O_2",                      "620-92-8", "PXKLMJQFEQBVLD-UHFFFAOYSA-N", "bisphenol F", "")
  CALL Species("C_{13}H_{18}O_2",                    "15687-27-1", "HEFNNWSXXWATRW-UHFFFAOYSA-N", "ibuprofen", "")
  CALL Species("C_{13}H_{16}O_5",                    "95266-40-3", "RVKCCVTVZORVGD-QXMHVHEDSA-N", "trinexapac-ethyl", "")
  CALL Species("C_{14}H_{10}O_4",                       "94-36-0", "OMPJBNCRMGITSC-UHFFFAOYSA-N", "benzoyl peroxide", "")
  CALL Species("C_{14}H_{12}O_3",                      "131-57-7", "DXGLGDHPHMLXJC-UHFFFAOYSA-N", "oxybenzone", "2-hydroxy-4-methoxybenzophenone")
  CALL Species("C_{14}H_{12}O_3",                      "501-36-0", "LUKBXSAWLPMMSZ-OWOJBTEDSA-N", "resveratrol", "")
  CALL Species("C_{14}H_{14}O_3",                       "83-26-1", "RZKYEQDPDZUERB-UHFFFAOYSA-N", "pindone", "")
  CALL Species("C_{14}H_{14}O",                        "103-50-4", "MHDVGSVTJDSBDK-UHFFFAOYSA-N", "1,1'-[oxybis(methylene)]bisbenzene", "dibenzyl ether")
  CALL Species("C_{14}H_{28}O_6",                    "55794-20-2", "HARQWLDROVMFJE-UHFFFAOYSA-N", "butanoic acid, 3,3-bis((1,1-dimethylethyl)dioxy)-, ethyl ester", "")
  CALL Species("C_{14}H_8O_3",                         "129-43-1", "BTLXPCBPYBNQNR-UHFFFAOYSA-N", "1-hydroxy-9,10-anthracenedione", "1-hydroxyanthraquinone")
  CALL Species("C_{14}H_8O_4",                         "117-10-2", "QBPFLULOKWLNNW-UHFFFAOYSA-N", "danthron", "1,8-dihydroxyanthraquinone")
  CALL Species("C_{15}H_{16}O_2",                       "80-05-7", "IISBACLAFKSPIT-UHFFFAOYSA-N", "bisphenol A", "")
  CALL Species("C_{15}H_{20}O_3",                    "73030-71-4", "FBMORZZOJSDNRQ-GLQYFDAESA-N", "atractylenolide III", "")
  CALL Species("C_{15}H_{20}O_6",                    "51481-10-8", "LINOMUASTDIRTM-WHNKEALZSA-N", "deoxynivalenol", "")
  CALL Species("C_{15}H_{20}O_7",                    "23282-20-4", "UKOTXHQERFPCBU-UHFFFAOYSA-N", "nivalenol", "")
  CALL Species("C_{15}H_{26}O_6",                       "60-01-5", "UYXTWWCETRIEDR-UHFFFAOYSA-N", "tributyrin", "")
  CALL Species("C_{16}H_{12}O_6",                      "520-34-3", "MBNGWHIJMBWFHU-UHFFFAOYSA-N", "diosmetin", "")
  CALL Species("C_{16}H_{16}O_5",                      "517-89-5", "NEZONWMXZKDMKF-SNVBAGLBSA-N", "shikonin", "")
  CALL Species("C_{16}H_{18}O_2",                       "77-40-7", "HTVITOHKHWFJKO-UHFFFAOYSA-N", "2,2-bis(4-hydroxyphenyl)butane", "bisphenol B")
  CALL Species("C_{16}H_{32}O_6",                    "67567-23-1", "NICWAKGKDIAMOD-UHFFFAOYSA-N", "ethyl 3,3-bis({\it tert}-amylperoxy)butyrate", "")
  CALL Species("C_{17}H_{12}O_6",                     "1162-65-8", "OQIQSTLJSLGHID-UHFFFAOYSA-N", "aflatoxin B1", "")
  CALL Species("C_{17}H_{12}O_7",                     "1165-39-5", "XWIYFDMXXLINPU-UHFFFAOYSA-N", "aflatoxin G1", "")
  CALL Species("C_{17}H_{14}O_6",                     "7220-81-7", "WWSYXEZEXMQWHT-UHFFFAOYSA-N", "aflatoxin B2", "")
  CALL Species("C_{17}H_{14}O_7",                     "7241-98-7", "WPCVRWVBBXIRMA-UHFFFAOYSA-N", "aflatoxin G2", "")
  CALL Species("C_{17}H_{20}O_2",                       "79-97-0", "YMTYZTXUZLQUSF-UHFFFAOYSA-N", "bisphenol C", "")
  CALL Species("C_{17}H_{20}O_6",                    "56299-00-4", "GSPFUBNBRPVALJ-VIEAGMIOSA-N", "PR-toxin", "")
  CALL Species("C_{17}H_{22}O_8",                    "23255-69-8", "XGCUCFKWVIWWNW-UHFFFAOYSA-N", "fusarenon X", "")
  CALL Species("C_{18}H_{14}O_3",                    "87205-99-0", "HARGZZNYNSYSGJ-JTQLQIEISA-N", "dihydrotanshinone I", "")
  CALL Species("C_{18}H_{20}O_2",                       "56-53-1", "RGLYKWWBQGJZGM-ISLYRVAYSA-N", "diethylstilbestrol", "")
  CALL Species("C_{18}H_{32}O_{16}",                  "9008-22-4", "DBTMGCOVALSLOR-AWHOAIGYSA-N", "laminarin", "")
  CALL Species("C_{18}H_{22}O_2",                       "53-16-7", "DNXHEGUUPJUMQT-UHFFFAOYSA-N", "estrone", "")
  CALL Species("C_{18}H_{24}O_2",                       "50-28-2", "VOXZDWNPVJITMN-AWDGRILASA-N", "estradiol", "")
  CALL Species("C_{18}H_{24}O_3",                       "50-27-1", "PROQIPRRNZUXQM-PVGHXWSTSA-N", "estriol", "")
  CALL Species("C_{18}H_{26}O_2",                      "434-22-0", "NPAGDVCDWIYMMC-SVXFNXITSA-N", "nandrolone", "")
  CALL Species("C_{18}H_{20}O_4",                    "63837-33-2", "ZDOOQPFIGYHZFV-UHFFFAOYSA-N", "diofenolan", "")
  CALL Species("C_{19}H_{12}O_6",                       "66-76-2", "DOBMPNYZJYQDGZ-UHFFFAOYSA-N", "dicumarol", "")
  CALL Species("C_{19}H_{16}O_3",                     "5836-29-3", "ULSLJYXHZDTLQK-UHFFFAOYSA-N", "coumatetralyl", "")
  CALL Species("C_{19}H_{16}O_4",                       "81-81-2", "PJVWKTKQMONHTI-UHFFFAOYSA-N", "warfarin", "")
  CALL Species("C_{19}H_{18}O_3",                      "568-72-9", "HYXITZLLTYIPOF-UHFFFAOYSA-N", "tanshinone II", "")
  CALL Species("C_{19}H_{22}O_6",                       "77-06-5", "IXORZMNAPKEEDV-UHFFFAOYSA-N", "gibberellic acid", "")
  CALL Species("C_{19}H_{24}O_3",                    "23031-36-9", "SMKRKQBMYOFFMU-UHFFFAOYSA-N", "prallethrin", "")
  CALL Species("C_{19}H_{24}O_3",                      "968-93-4", "BPEWUONYVDABNZ-LHXSAFEUSA-N", "testolactone", "")
  CALL Species("C_{19}H_{26}O_2",                       "63-05-8", "AEMFNILZOJDQLW-JRCHKSGSSA-N", "androstenedione", "")
  CALL Species("C_{19}H_{26}O_7",                     "2270-40-8", "AUGQEEXBDZWUJY-UHFFFAOYSA-N", "diacetoxyscirpenol", "")
  CALL Species("C_{19}H_{28}O_2",                       "58-22-0", "MUMGGOZAMZWBJJ-JZJKZLICSA-N", "testosterone", "")
  CALL Species("C_{19}H_{28}O",                      "18339-16-7", "HFVMLYAGWXSTQI-QYXZOKGRSA-N", "5$\alpha$-androst-16-en-4-one", "androstenone")
  CALL Species("C_{19}H_{30}O_3",                       "53-39-4", "QSLJIVKCVHQPLV-WPMSWULFSA-N", "oxandrolone", "")
  CALL Species("C_{19}H_{30}O_5",                       "51-03-6", "FIPWRIJSWJWJAI-UHFFFAOYSA-N", "piperonyl butoxide", "")
  CALL Species("C_{19}H_{34}O_3",                    "40596-69-8", "NFGXHKASABOEEW-LDRANXPESA-N", "methoprene", "")
  CALL Species("C_{20}H_{12}O_5",                     "2321-07-5", "GNBHRKFJIUUOQI-UHFFFAOYSA-N", "fluorescein", "")
  CALL Species("C_{20}H_{14}O_4",                       "77-09-8", "KJFMBFZCATUALV-UHFFFAOYSA-N", "phenolphthalein", "")
  CALL Species("C_{20}H_{22}O_3",                    "70356-09-1", "XNEFYCZVKIDDMS-UHFFFAOYSA-N", "avobenzone", "")
  CALL Species("C_{20}H_{24}O_2",                       "57-63-6", "BFPYWIDHMRZLRN-UHFFFAOYSA-N", "ethinyl estradiol", "")
  CALL Species("C_{20}H_{26}O_2",                       "68-23-5", "ICTXHFFSOAJUMG-CEVCPLMDSA-N", "norethynodrel", "")
  CALL Species("C_{20}H_{26}O",                         "68-22-4", "VIKNJXKGJWUCNN-BROHZWGRSA-N", "norethindrone", "")
  CALL Species("C_{20}H_{28}O_2",                       "72-63-9", "XWALNWXLMVGSFR-NSDIEPNESA-N", "methandrostenolone", "")
  CALL Species("C_{20}H_{28}O_3",                    "25402-06-6", "FMTFEIJHMMQUJI-FPLPWBNLSA-N", "cinerin I", "")
  CALL Species("C_{20}H_{30}O_2",                       "58-18-4", "GCKMFJBGXUYNAG-NSDIEPNESA-N", "17-methyltestosterone", "")
  CALL Species("C_{20}H_{32}O_2",                       "58-19-5", "IKXILDNPCZPPRV-UHFFFAOYSA-N", "drostanolone", "dromostanolone")
  CALL Species("C_{21}H_{20}O_6",                      "458-37-7", "VFLDPWHFBUODDF-FCXRPNKRSA-N", "curcumin", "")
  CALL Species("C_{21}H_{24}O_4",                     "1675-54-3", "LCFVJGUPQDGYKZ-UHFFFAOYSA-N", "bisphenol A diglycidyl ether", "")
  CALL Species("C_{21}H_{26}O_2",                       "72-33-3", "IMSSROKUHAOUJS-ALAWOQLPSA-N", "mestranol", "")
  CALL Species("C_{21}H_{26}O_5",                       "53-03-2", "XOFYZVNMUHMLCC-NUBBXXJJSA-N", "prednisone", "")
  CALL Species("C_{21}H_{28}O_2",                     "6533-00-2", "WWYNJERNGUHSAO-ZUHHCLADSA-N", "norgestrel", "")
  CALL Species("C_{21}H_{28}O_2",                      "797-63-7", "WWYNJERNGUHSAO-XUDSTZEESA-N", "levonorgestrel", "")
  CALL Species("C_{21}H_{28}O_3",                      "121-21-1", "ROVGZAWFACYCSP-CMDGGOBGSA-N", "pyrethrin I", "")
  CALL Species("C_{21}H_{28}O_5",                      "121-20-0", "SHCRDCOTRILILT-WOBDGSLYSA-N", "cinerin II", "")
  CALL Species("C_{21}H_{28}O_5",                       "50-24-8", "OIGNJSKKLXVSLS-UHFFFAOYSA-N", "prednisolone", "")
  CALL Species("C_{21}H_{30}O_2",                     "1972-08-3", "CYQFCXCEBYINGO-UHFFFAOYSA-N", "dronabinol", "delta 9-tetrahydrocannabinol")
  CALL Species("C_{21}H_{30}O_2",                       "57-83-0", "RJKFOVLPORLFTN-UHFFFAOYSA-N", "progesterone", "")
  CALL Species("C_{21}H_{30}O_5",                       "50-23-7", "JYGXADMDTFJGBT-NDNUHCHRSA-N", "hydrocortisone", "")
  CALL Species("C_{21}H_{32}O_2",                    "17021-26-0", "IVFYLRMMHVYGJH-UHFFFAOYSA-N", "calusterone", "")
  CALL Species("C_{21}H_{32}O_3",                      "434-07-1", "ICMWWNHDUZJFDW-RCXBLOTCSA-N", "oxymetholone", "")
  CALL Species("C_{22}H_{26}O_3",                    "10453-86-8", "VEMKTZHHVJILDY-UHFFFAOYSA-N", "resmethrin", "")
  CALL Species("C_{22}H_{28}O_5",                      "121-29-9", "VJFUPGQZSXIULQ-VKTMSVCMSA-N", "pyrethrin II", "")
  CALL Species("C_{22}H_{30}O_5",                       "83-43-2", "VHRSUDSXCMQTMA-UHFFFAOYSA-N", "methylprednisolone", "")
  CALL Species("C_{22}H_{32}O_3",                      "520-85-4", "FRQMUZJSZHZSGN-HBNHAYAOSA-N", "medroxyprogesterone", "")
  CALL Species("C_{23}H_{24}O_5",                     "5221-53-4", "VNKCZJKGJAEOCW-WXUKJITCSA-N", "dimethirimol", "")
  CALL Species("C_{23}H_{22}O_6",                       "83-79-4", "JUVIOZPCNVVQFO-UHFFFAOYSA-N", "rotenone", "")
  CALL Species("C_{23}H_{26}O_3",                    "26002-80-2", "SBNFWQZLDJGRLK-UHFFFAOYSA-N", "phenothrin", "")
  CALL Species("C_{23}H_{30}O_4",                   "283594-90-1", "GOLXNESZZPUPJE-UHFFFAOYSA-N", "spiromesifen", "")
  CALL Species("C_{23}H_{34}O_5",                     "1672-46-4", "SHIBSTMRCDJXLN-KCZCNTNESA-N", "digoxigenin", "")
  CALL Species("C_{24}H_{28}O_4",                     "1393-63-1", "ZVKOASAVGLETCT-LRRSNBNMSA-N", "annatto", "")
  CALL Species("C_{24}H_{32}O_4",                    "57960-19-7", "QDRXWCAVUNHOGA-UHFFFAOYSA-N", "acequinocyl", "")
  CALL Species("C_{24}H_{34}O_9",                    "21259-20-1", "SSHHYBPAMCLKRH-UHFFFAOYSA-N", "T-2 mycotoxin", "")
  CALL Species("C_{25}H_{22}O_{10}",                 "84604-20-6", "SEBFKMXJBCUCAI-VGHNRKBZSA-N", "milk thistle extract", "")
  CALL Species("C_{25}H_{38}O_5",                    "79902-63-9", "RYMZZMVNJRMUDD-UHFFFAOYSA-N", "simvastatin", "")
  CALL Species("C_{27}H_{44}O_3",                    "32222-06-3", "GMRQFYUYWCNGIN-NKMMMXOESA-N", "calcitriol", "1,25-dihydroxycholecalciferol")
  CALL Species("C_{27}H_{44}O_3",                   "131918-61-1", "BPKAHTKRCLCHEA-UBFJEZKGSA-N", "paricalcitol", "")
  CALL Species("C_{27}H_{44}O",                         "67-97-0", "QYSXJUFSXHHAJI-SMGPGMQOSA-N", "cholecalciferol", "")
  CALL Species("C_{27}H_{46}O",                         "57-88-5", "HVYWMOMLDIMFJA-VUDDDUNTSA-N", "cholesterol", "")
  CALL Species("C_{28}H_{44}O",                         "57-87-4", "DNVPQKQSNYMLRS-CVGROQQCSA-N", "ergosterol", "")
  CALL Species("C_{28}H_{46}O",                         "67-96-9", "DTSXXSAWQHPLEF-GFVAUXBKSA-N", "dihydrotachysterol", "")
  CALL Species("C_{29}H_{32}O_{13}",                 "33419-42-0", "VJJPUSNTGOMMGY-KWGSHVRASA-N", "etoposide", "")
  CALL Species("C_{29}H_{48}O",                         "83-48-7", "HCXVJBMSMIARIN-BASBAMEESA-N", "stigmasterol", "")
  CALL Species("C_{30}H_{16}O_9",                    "55954-61-5", "YXBUQQDFTYOHQI-UHFFFAOYSA-N", "pseudohypericin", "")
  CALL Species("C_{30}H_{30}O_8",                      "303-45-7", "QBKSWRVVCFFDOT-UHFFFAOYSA-N", "gossypol", "")
  CALL Species("C_{30}H_{48}O_4",                     "4373-41-5", "MDZKJHQSJHYOHJ-LLICELPBSA-N", "maslinic acid", "")
  CALL Species("C_{31}H_{44}O_7",                    "51596-10-2", "ZLBGSRMUSVULIE-GSMJGMFJSA-N", "milbemectin", "")
  CALL Species("C_{31}H_{24}O_3",                    "56073-07-5", "FVQITOLOYMWVFU-UHFFFAOYSA-N", "difenacoum", "")
  CALL Species("C_{33}H_{60}O_{10}",                 "26571-11-9", "FBWNMEQMRUMQSO-UHFFFAOYSA-N", "nonoxynol 9", "")
  CALL Species("C_{35}H_{44}O_{16}",                 "11141-17-6", "FTNJWQUOZFUQQJ-GWTPYEAISA-N", "azadirachtin", "")
  CALL Species("C_{36}H_{62}O_{11}",                 "17090-79-8", "GAOZTHIDHYLHMS-LXKLZWMJSA-N", "monensin", "")
  CALL Species("C_{36}H_{64}O_4",                    "50933-33-0", "BXJHOKLLMOYSRQ-QOXWLJPHSA-N", "gossyplure", "")
  CALL Species("C_{40}H_{56}O_3",                      "465-42-9", "VYIRVAXUEZSDNC-RDJLEWNRSA-N", "capsanthin", "")
  CALL Species("C_{40}H_{70}O_{33}",                "870721-81-6", "RAUODYOTTYNEJP-RQESCVSBSA-N", "heptamaloxyloglucan", "")
  CALL Species("C_{41}H_{64}O_{13}",                    "71-63-6", "WDJUZGPOPHTGOT-UCKSZOHFSA-N", "digitoxin", "")
  CALL Species("C_{41}H_{64}O_{14}",                 "20830-75-5", "LTMHDMANZUZIPE-PUGKRICDSA-N", "digoxin", "")
  CALL Species("C_{43}H_{56}O_8",                     "8003-34-7", "VXSIXFKKSNGRRO-YWUDCVDHSA-N", "pyrethrum", "")
  CALL Species("C_{48}H_{28}O_{30}",                 "65995-63-3", "SKNLUADAGHCXKF-UHFFFAOYSA-N", "punicalagin", "")
  CALL Species("C_{48}H_{72}O_{14}",                 "71751-41-2", "RRZXIRBKKLTSOM-IGNCFFBFSA-N", "abamectin", "")
  CALL Species("C_{48}H_{84}O_{18}",                 "80418-24-2", "ZBXDHDDTAIOMHK-UWGJVBKGSA-N", "notoginsenoside R1", "")
  CALL Species("C_{57}H_{104}O_6",                     "122-32-7", "PHYFQTYBJUILEZ-IUPFWZBJSA-N", "triolein", "")
  CALL Species("C_{57}H_{110}O_6",                     "555-43-1", "DCXXMTOCNZCJGO-UHFFFAOYSA-N", "tristearin", "")
  !***************************************************************************
  CALL SpeciesGroup(1,"Organic species with nitrogen (\chem{N})")
  !***************************************************************************
  CALL SpeciesGroup(2,"Amines (\chem{C}, \chem{H}, \chem{N})")
  ! C, H, N:
  ! aliphatic:
  CALL Species("CH_2N_2",                              "420-04-2", "XZMCDFZZKTWFGF-UHFFFAOYSA-N", "cyanamide", "")
  CALL Species("CH_6N_2",                               "60-34-4", "HDZGCSFEDULWCS-UHFFFAOYSA-N", "methylhydrazine", "")
  CALL Species("CH_3NH_2",                              "74-89-5", "BAVYZALUXZFZLV-UHFFFAOYSA-N", "methanamine", "methylamine")
  CALL Species("C_2H_5NH_2",                            "75-04-7", "QUSNBJAOOMFDIB-UHFFFAOYSA-N", "ethanamine", "ethylamine")
  CALL Species("C_2H_3N_3",                            "288-88-0", "NSPMIYGKQJPBQR-UHFFFAOYSA-N", "1H-1,2,4-triazole", "")
  CALL Species("C_2H_4N_4",                            "461-58-5", "QGBSISYHAICWAH-UHFFFAOYSA-N", "dicyandiamide", "cyanoguanidine")
  CALL Species("C_2H_5N",                              "151-56-4", "NOWKCMXCCJGMRR-UHFFFAOYSA-N", "ethylenimine", "")
  CALL Species("C_2H_8N_2",                            "540-73-8", "DIIIISSCIXVANO-UHFFFAOYSA-N", "1,2-dimethylhydrazine", "")
  CALL Species("C_2H_8N_2",                             "57-14-7", "RHUYHJGZWVXEHW-UHFFFAOYSA-N", "1,1-dimethylhydrazine", "")
  CALL Species("C_3H_7NH_2",                           "107-10-8", "WGYKZJWCGVVSQN-UHFFFAOYSA-N", "1-propanamine", "propylamine")
  CALL Species("C_3H_9N",                               "75-31-0", "JJWLVOIRVHMVIS-UHFFFAOYSA-N", "2-propanamine", "isopropylamine")
  CALL Species("C_3H_2N_2",                            "109-77-3", "CUONGYYJJVDODC-UHFFFAOYSA-N", "propanedinitrile", "malononitrile")
  CALL Species("C_3H_7N",                               "75-55-8", "OZDGMOYKSFPLSE-UHFFFAOYSA-N", "2-methylaziridine", "")
  CALL Species("C_3H_{10}N_2",                          "78-90-0", "AOHJOMMDDJHIJH-UHFFFAOYSA-N", "1,2-diaminopropane", "")
  CALL Species("C_4H_9NH_2",                           "109-73-9", "HQABUPZFAYXKJW-UHFFFAOYSA-N", "1-butanamine", "butylamine")
  CALL Species("C_4H_{11}N",                         "13952-84-6", "BHRZNVHARXXAHW-UHFFFAOYSA-N", "2-butanamine", "{\it sec}-butylamine")
  CALL Species("C_4H_{11}N",                            "78-81-9", "KDSNLYIMUZNERS-UHFFFAOYSA-N", "2-methyl-1-propanamine", "isobutylamine")
  CALL Species("C_4H_{11}N",                            "75-64-9", "YBRBMKDOPFTVDT-UHFFFAOYSA-N", "2-methyl-2-propanamine", "{\it tert}-butylamine")
  CALL Species("C_4H_{11}N",                           "598-56-1", "DAZXVJBJRMWXJP-UHFFFAOYSA-N", "dimethylethylamine", "")
  CALL Species("C_4H_{11}N_3",                         "462-25-9", "BWMDMTSNSXYYSP-UHFFFAOYSA-N", "2-propylguanidine", "")
  CALL Species("C_4H_{12}N_2",                        "1615-80-1", "YCBOYOYVDOUXLH-UHFFFAOYSA-N", "1,2-diethylhydrazine", "")
  CALL Species("C_4H_{13}N_3",                         "111-40-0", "RPNUMPOLZDHAAY-UHFFFAOYSA-N", "N-(2-aminoethyl)-1,2-ethanediamine", "diethylenetriamine")
  CALL Species("C_4H_4N_2",                            "110-61-2", "IAHFWCOBPZCAEA-UHFFFAOYSA-N", "butanedinitrile", "")
  CALL Species("C_4H_6N_2",                            "693-98-1", "LXBGSDVWAMZHDD-UHFFFAOYSA-N", "2-methyl-1H-imidazole", "")
  CALL Species("C_4H_6N_2",                            "822-36-6", "XLSZMDLNRCVEIJ-UHFFFAOYSA-N", "4-methyl-1H-imidazole", "")
  CALL Species("C_4H_{12}N_2",                        "6291-84-5", "QHJABUZHRJTCAR-UHFFFAOYSA-N", "N-methyl-1,3-propanediamine", "3-(methylamino)propylamine")
  CALL Species("C_5H_{11}NH_2",                        "110-58-7", "DPBLXKKOBLCELK-UHFFFAOYSA-N", "1-pentanamine", "1-pentylamine")
  CALL Species("C_5H_{13}N",                           "107-85-7", "BMFVGAAISNGQNM-UHFFFAOYSA-N", "3-methyl-1-butanamine", "")
  CALL Species("C_5H_{13}N",                           "598-74-3", "JOZZAIIGWFLONA-UHFFFAOYSA-N", "1,2-dimethylpropylamine", "")
  CALL Species("C_5H_{13}N",                           "926-63-6", "ZUHZZVMEUAUWHY-UHFFFAOYSA-N", "dimethylpropylamine", "")
  CALL Species("C_5H_{13}N",                           "996-35-0", "VMOWKUTXPNPTEN-UHFFFAOYSA-N", "dimethylisopropylamine", "")
  CALL Species("C_5H_{13}N",                         "20193-20-8", "XCVNDBIXFPGMIW-UHFFFAOYSA-N", "ethylpropylamine", "")
  CALL Species("C_5H_{13}N",                         "19961-27-4", "RIVIDPPYRINTTH-UHFFFAOYSA-N", "ethylisopropylamine", "")
  CALL Species("C_5H_{13}N",                           "616-39-7", "GNVRJGIVDSQCOP-UHFFFAOYSA-N", "methyldiethylamine", "")
  CALL Species("C_5H_{12}N_2",                         "109-07-9", "JOMNTHCQHJPVAZ-UHFFFAOYSA-N", "2-methylpiperazine", "")
  CALL Species("C_6H_{13}NH_2",                        "111-26-2", "BMVXCPBXGZKUPN-UHFFFAOYSA-N", "1-hexanamine", "hexylamine")
  CALL Species("C_6H_{15}N",                          "5329-79-3", "WGBBUURBHXLGFM-UHFFFAOYSA-N", "2-hexanamine", "")
  CALL Species("C_6H_{15}N",                         "16751-58-9", "HQLZFBUAULNEGP-UHFFFAOYSA-N", "3-hexanamine", "")
  CALL Species("C_6H_{15}N",                         "13364-16-4", "WNDXRJBYZOSNQO-UHFFFAOYSA-N", "1-amino-2-methylpentane", "")
  CALL Species("C_6H_{15}N",                         "42245-37-4", "JLAUIBFZZUVOBB-UHFFFAOYSA-N", "1-amino-3-methylpentane", "")
  CALL Species("C_6H_{15}N",                          "5344-20-7", "QVIAMKXOQGCYCV-UHFFFAOYSA-N", "1-amino-4-methylpentane", "")
  CALL Species("C_6H_{15}N",                         "35399-81-6", "ZFAGOADKDXXTSV-UHFFFAOYSA-N", "2-amino-3-methylpentane", "")
  CALL Species("C_6H_{15}N",                           "108-09-8", "UNBMPKNTYKDYCG-UHFFFAOYSA-N", "2-amino-4-methylpentane", "")
  CALL Species("C_6H_{15}N",                         "54287-41-1", "JYNQKCFJPQEXSL-UHFFFAOYSA-N", "3-amino-2-methylpentane", "")
  CALL Species("C_6H_{15}N",                           "617-79-8", "MGWAGIQQTULHGU-UHFFFAOYSA-N", "1-amino-2-ethylbutane", "")
  CALL Species("C_6H_{15}N",                         "41781-17-3", "PZVPOYBHOPRJNP-UHFFFAOYSA-N", "1-amino-2,2-dimethylbutane", "")
  CALL Species("C_6H_{15}N",                         "66553-05-7", "GBMSZXWHMSSBGP-UHFFFAOYSA-N", "1-amino-2,3-dimethylbutane", "")
  CALL Species("C_6H_{15}N",                         "15673-00-4", "GPWHFPWZAPOYNO-UHFFFAOYSA-N", "1-amino-3,3-dimethylbutane", "")
  CALL Species("C_6H_{15}N",                          "3850-30-4", "DXSUORGKJZADET-UHFFFAOYSA-N", "2-amino-3,3-dimethylbutane", "")
  CALL Species("C_6H_{15}N",                           "927-62-8", "DJEQZVQFEPKLOY-UHFFFAOYSA-N", "dimethylbutylamine", "")
  CALL Species("C_6H_{15}N",                           "921-04-0", "USSPHSVODLAWSA-UHFFFAOYSA-N", "dimethyl-{\it sec}-butylamine", "")
  CALL Species("C_6H_{15}N",                           "918-02-5", "OXQMIXBVXHWDPX-UHFFFAOYSA-N", "dimethyl-{\it tert}-butylamine", "")
  CALL Species("C_6H_{15}N",                         "13360-63-9", "QHCCDDQKNUYGNC-UHFFFAOYSA-N", "ethylbutylamine", "")
  CALL Species("C_6H_{15}N",                         "13205-60-2", "FNLUJDLKYOWMMF-UHFFFAOYSA-N", "ethylisobutylamine", "")
  CALL Species("C_6H_{15}N",                         "21035-44-9", "KFYKZKISJBGVMR-UHFFFAOYSA-N", "ethyl-{\it sec}-butylamine", "")
  CALL Species("C_6H_{15}N",                          "2978-64-5", "BUJFTKPQXSIZFX-UHFFFAOYSA-N", "methyl-1,1-dimethylpropylamine", "")
  CALL Species("C_6H_{15}N",                         "34317-39-0", "LJLWVVCWBURGCC-UHFFFAOYSA-N", "methyl-1,2-dimethylpropylamine", "")
  CALL Species("C_6H_{15}N",                         "26153-91-3", "UQGXHNDRCRTZAC-UHFFFAOYSA-N", "methyl-2,2-dimethylpropylamine", "")
  CALL Species("C_6H_{15}N",                         "51932-19-5", "IPBXLJFBVNLKFE-UHFFFAOYSA-N", "methyl-1-methylbutylamine", "")
  CALL Species("C_6H_{15}N",                          "4458-32-6", "SMBYUOXUISCLCF-UHFFFAOYSA-N", "methylethylpropylamine", "")
  CALL Species("C_6H_{15}N",                         "39198-07-7", "UTLDDSNRFHWERZ-UHFFFAOYSA-N", "methylethylisopropylamine", "")
  CALL Species("C_6H_{15}N",                         "25419-06-1", "UOIWOHLIGKIYFE-UHFFFAOYSA-N", "methylpentylamine", "")
  CALL Species("C_6H_{15}N",                         "21968-17-2", "VLSTXUUYLIALPB-UHFFFAOYSA-N", "propylisopropylamine", "")
  CALL Species("C_6H_{16}N_2",                         "124-09-4", "NAQMVNRVTILPCV-UHFFFAOYSA-N", "1,6-hexanediamine", "hexamethylenediamine")
  CALL Species("C_7H_{14}N_2",                         "693-13-0", "BDNKZNFMNDZQMI-UHFFFAOYSA-N", "N,N'-methanetetraylbis-2-propanamine", "1,3-diisopropylcarbodiimide")
  CALL Species("C_7H_{17}N",                           "105-41-9", "YAHRDLICUYEDAU-UHFFFAOYSA-N", "4-methyl-2-hexanamine", "")
  CALL Species("C_7H_{17}N",                           "111-68-2", "WJYIASZWHGOTOU-UHFFFAOYSA-N", "1-heptanamine", "1-heptylamine")
  CALL Species("C_7H_{17}N",                          "6006-15-1", "ULWOJODHECIZAU-UHFFFAOYSA-N", "diethylisopropylamine", "")
  CALL Species("C_7H_{17}N",                          "4458-31-5", "PQZTVWVYCLIIJY-UHFFFAOYSA-N", "diethylpropylamine", "")
  CALL Species("C_7H_{17}N",                         "57757-60-5", "CUHMMDPUXJFCNB-UHFFFAOYSA-N", "dimethyl-1,1-dimethylpropylamine", "")
  CALL Species("C_7H_{17}N",                         "66225-38-5", "FWBCYOHCOBOARU-UHFFFAOYSA-N", "dimethyl-1,2-dimethylpropylamine", "")
  CALL Species("C_7H_{17}N",                         "10076-31-0", "FUIRUFXAVIHAQB-UHFFFAOYSA-N", "dimethyl-2,2-dimethylpropylamine", "")
  CALL Species("C_7H_{17}N",                         "66225-39-6", "BHMZPPHMQJHCHQ-UHFFFAOYSA-N", "dimethyl-2-methylbutylamine", "")
  CALL Species("C_7H_{17}N",                          "2315-43-7", "KOOQJINBDNZUTB-UHFFFAOYSA-N", "dimethyl-3-methylbutylamine", "")
  CALL Species("C_7H_{17}N",                         "26153-88-8", "IDFANOPDMXWIOP-UHFFFAOYSA-N", "dimethylpentylamine", "")
  CALL Species("C_7H_{17}N",                         "57303-85-2", "LSTZYJQJHGEVKH-UHFFFAOYSA-N", "dimethyl-2-pentylamine", "")
  CALL Species("C_7H_{17}N",                         "18636-94-7", "SUEKSPIQGIMSQM-UHFFFAOYSA-N", "dimethyl-3-pentylamine", "")
  CALL Species("C_7H_{17}N",                         "17839-26-8", "ICVFPLUSMYSIFO-UHFFFAOYSA-N", "ethylpentylamine", "")
  CALL Species("C_7H_{17}N",                         "10342-97-9", "ISRXMEYARGEVIU-UHFFFAOYSA-N", "methyldiisopropylamine", "")
  CALL Species("C_7H_{17}N",                          "3405-42-3", "UVBMZKBIZUWTLV-UHFFFAOYSA-N", "methyldipropylamine", "")
  CALL Species("C_7H_{17}N",                         "66225-40-9", "WOLFCKKMHUVEPN-UHFFFAOYSA-N", "methylethylbutylamine", "")
  CALL Species("C_7H_{17}N",                         "60247-14-5", "QQWXQKMVASVXCI-UHFFFAOYSA-N", "methylethylisobutylamine", "")
  CALL Species("C_7H_{17}N",                         "66225-41-0", "HAFZSBASGRZPLA-UHFFFAOYSA-N", "methylethyl-{\it sec}-butylamine", "")
  CALL Species("C_7H_{17}N",                         "52841-28-8", "BWWLCYLGZIWOHK-UHFFFAOYSA-N", "methylethyl-{\it tert}-butylamine", "")
  CALL Species("C_7H_{17}N",                         "35161-70-7", "XJINZNWPEQMMBV-UHFFFAOYSA-N", "methylhexylamine", "")
  CALL Species("C_7H_{17}N",                         "66225-42-1", "OYQDUCLFZSKBCZ-UHFFFAOYSA-N", "methylpropylisopropylamine", "")
  CALL Species("C_8H{19}N",                            "111-86-4", "IOQPZZOEVPZRBK-UHFFFAOYSA-N", "1-octanamine", "octylamine")
  CALL Species("C_8H_{19}N",                           "104-75-6", "LTHNHFOGQMKPOV-UHFFFAOYSA-N", "2-ethyl-1-hexanamine", "2-ethylhexylamine")
  CALL Species("C_8H_{19}N",                          "4444-68-2", "ORSUTASIQKBEFU-UHFFFAOYSA-N", "diethylbutylamine", "")
  CALL Species("C_8H_{19}N",                          "4385-04-0", "QMHNQZGXPNCMCO-UHFFFAOYSA-N", "dimethylhexylamine", "")
  CALL Species("C_8H_{19}N",                         "20352-67-4", "WSTNFGAKGUERTC-UHFFFAOYSA-N", "ethylhexylamine", "")
  CALL Species("C_8H_{19}N",                         "36343-05-2", "LTGYRKOQQQWWAF-UHFFFAOYSA-N", "methylheptylamine", "")
  CALL Species("C_8H_{12}N_4",                          "78-67-1", "OZAIFHULBGXAKX-UHFFFAOYSA-N", "2,2'-azobis-(2-methylpropanenitrile)", "")
  CALL Species("C_9H_{21}N",                          "2162-91-6", "YZULHOOBWDXEOT-UHFFFAOYSA-N", "diethylpentylamine", "")
  CALL Species("C_9H_{21}N",                          "5277-11-2", "LSICDRUYCNGRIF-UHFFFAOYSA-N", "dimethylheptylamine", "")
  CALL Species("C_9H_{21}N",                         "66793-76-8", "IUZZLNVABCISOI-UHFFFAOYSA-N", "ethylheptylamine", "")
  CALL Species("C_9H_{21}N",                          "2439-54-5", "SEGJNMCIMOLEDM-UHFFFAOYSA-N", "methyloctylamine", "")
  CALL Species("C_9H_{21}N",                           "112-20-9", "FJDUDHYHRVPMJZ-UHFFFAOYSA-N", "nonylamine", "")
  CALL Species("C_{10}H_{23}N",                      "44979-90-0", "XHDKYWMKOLURNK-UHFFFAOYSA-N", "diethylhexylamine", "")
  CALL Species("C_{10}H_{23}N",                       "7378-99-6", "UQKAOOAFEFCDGT-UHFFFAOYSA-N", "dimethyloctylamine", "")
  CALL Species("C_{10}H_{23}N",                       "2050-92-2", "JACMPVXHEARCBO-UHFFFAOYSA-N", "dipentylamine", "")
  CALL Species("C_{10}H_{23}N",                       "4088-36-2", "SDQCOADWEMMSGK-UHFFFAOYSA-N", "ethyloctylamine", "")
  CALL Species("C_{10}H_{23}N",                      "39093-27-1", "OZIXTIPURXIEMB-UHFFFAOYSA-N", "methylnonylamine", "")
  CALL Species("C_{11}H_{25}N",                      "26981-81-7", "YUCNJMBRLIZNMO-UHFFFAOYSA-N", "diethylheptylamine", "")
  CALL Species("C_{11}H_{25}N",                      "17373-27-2", "AMAADDMFZSZCNT-UHFFFAOYSA-N", "dimethylnonylamine", "")
  CALL Species("C_{12}H_{27}N",                       "4088-37-3", "BVUGARXRRGZONH-UHFFFAOYSA-N", "diethyloctylamine", "")
  CALL Species("C_{12}H_{27}N",                        "143-16-8", "PXSXRABJBXYMFT-UHFFFAOYSA-N", "dihexylamine", "")
  CALL Species("C_{12}H_{27}N",                       "1120-24-7", "YWWNNLPSZSEZNZ-UHFFFAOYSA-N", "dimethyldecylamine", "")
  CALL Species("C_{12}H_{27}N",                      "66553-53-5", "JCJFBKCQLFMABE-UHFFFAOYSA-N", "methylundecylamine", "")
  CALL Species("C_{13}H_{29}N",                      "45124-35-4", "IBTOMDSHMLGUHA-UHFFFAOYSA-N", "diethylnonylamine", "")
  CALL Species("C_{13}H_{29}N",                      "17373-28-3", "MMWFTWUMBYZIRZ-UHFFFAOYSA-N", "dimethylundecylamine", "")
  CALL Species("C_{13}H_{29}N",                      "59570-04-6", "LKVBHKWFYHKTSM-UHFFFAOYSA-N", "ethylundecylamine", "")
  CALL Species("C_{13}H_{29}N",                       "7311-30-0", "OMEMQVZNTDHENJ-UHFFFAOYSA-N", "methyldodecylamine", "")
  CALL Species("C_{14}H_{31}N",                       "6308-94-7", "UFFQZCPLBHYOFV-UHFFFAOYSA-N", "diethyldecylamine", "")
  CALL Species("C_{14}H_{31}N",                       "2470-68-0", "NJWMENBYMFZACG-UHFFFAOYSA-N", "diheptylamine", "")
  CALL Species("C_{14}H_{31}N",                      "35902-57-9", "LWIPGCTWFZCIKX-UHFFFAOYSA-N", "ethyldodecylamine", "")
  CALL Species("C_{14}H_{31}N",                      "45165-81-9", "XMRPIOZXPHTSCE-UHFFFAOYSA-N", "methyltridecylamine", "")
  CALL Species("C_{15}H_{33}N",                        "621-77-2", "OOHAUGDGCWURIT-UHFFFAOYSA-N", "tripentylamine", "")
  CALL Species("C_{13}H_{29}N",                       "2869-34-3", "ABVVEAHYODGCLZ-UHFFFAOYSA-N", "1-tridecanamine", "")
  CALL Species("(CH_3)_2NH",                           "124-40-3", "ROSDSFDQCJNGOL-UHFFFAOYSA-N", "dimethylamine", "")
  CALL Species("(C_2H_5)_2NH",                         "109-89-7", "HPNMFZURTQLUMO-UHFFFAOYSA-N", "diethylamine", "")
  CALL Species("(C_3H_7)_2NH",                         "142-84-7", "WEHWNAOGRSTTBQ-UHFFFAOYSA-N", "dipropylamine", "")
  CALL Species("C_4H_{11}N",                           "627-35-0", "GVWISOJSERXQBM-UHFFFAOYSA-N", "N-methylpropanamine", "")
  CALL Species("C_4H_{11}N",                          "4747-21-1", "XHFGWHUWQXTGAT-UHFFFAOYSA-N", "N-methyl-2-propanamine", "")
  CALL Species("C_6H_{15}N",                           "108-18-9", "UAOMVDZJSHZZME-UHFFFAOYSA-N", "N-(1-methylethyl)-2-propanamine", "diisopropylamine")
  CALL Species("C_9H_{21}N",                           "102-69-2", "YFTHZRPMJXBUME-UHFFFAOYSA-N", "N,N-dipropyl-1-propanamine", "tripropylamine")
  CALL Species("C_5H_{13}N",                           "110-68-9", "QCOGKXLOEWLIDC-UHFFFAOYSA-N", "N-methyl-1-butanamine", "N-methylbutylamine")
  CALL Species("(C_4H_9)_2NH",                         "111-92-2", "JQVDAXLFBXTEQA-UHFFFAOYSA-N", "dibutylamine", "")
  CALL Species("C_8H_{19}N",                           "110-96-3", "NJBCRXCAPCODGX-UHFFFAOYSA-N", "diisobutylamine", "")
  CALL Species("C_8H_{19}N",                           "626-23-3", "OBYVIBDTOCAXSN-UHFFFAOYSA-N", "bis-(1-methylpropyl)-amine", "di-{\it sec}-butylamine")
  CALL Species("C_8H_{23}N_5",                         "112-57-2", "FAGUFWYHJQFNRV-UHFFFAOYSA-N", "tetraethylenepentamine", "")
  CALL Species("C_9H_{15}N",                           "102-70-5", "VPYJNCGUESNPMV-UHFFFAOYSA-N", "N,N-di-2-propenyl-2-propen-1-amine", "triallylamine")
  CALL Species("(CH_3)_3N",                             "75-50-3", "GETQZCLCWQTVFV-UHFFFAOYSA-N", "trimethylamine", "")
  CALL Species("(C_2H_5)_3N",                          "121-44-8", "ZMANZCXQSJIPKH-UHFFFAOYSA-N", "triethylamine", "")
  CALL Species("C_{12}H_{27}N",                        "102-82-9", "IMFACGCPASFAPR-UHFFFAOYSA-N", "tributylamine", "")
  CALL Species("C_{14}H_{31}N",                        "112-18-5", "YWFWDNVOPHGWMX-UHFFFAOYSA-N", "N,N-dimethyl-1-dodecanamine", "")
  CALL Species("H_2NCH_2CH_2NH_2",                     "107-15-3", "PIICEJLVQHRZGT-UHFFFAOYSA-N", "ethylenediamine", "")
  CALL Species("C_3H_7N",                              "107-11-9", "VVJKKWFAADXIJK-UHFFFAOYSA-N", "2-propen-1-amine", "allylamine")
  CALL Species("C_6H_{11}N",                           "124-02-7", "DYUWTXWIYMHBQS-UHFFFAOYSA-N", "di-2-propenylamine", "diallylamine")
  CALL Species("(CH_2)_6NH",                           "111-49-9", "ZSIQJIWKELUFRJ-UHFFFAOYSA-N", "hexamethyleneimine", "")
  CALL Species("C_6H_{13}N",                           "108-91-8", "PAFZNILMFXTMIY-UHFFFAOYSA-N", "cyclohexanamine", "cyclohexylamine")
  CALL Species("C_7H_{15}N",                          "6850-35-7", "JYDYHSHPBDZRPU-UHFFFAOYSA-N", "3-methylcyclohexylamine", "")
  CALL Species("C_8H_{17}N",                          "5459-93-8", "AGVKXDPPPSLISR-UHFFFAOYSA-N", "N-ethylcyclohexanamine", "N-ethylcyclohexylamine")
  CALL Species("C_8H_{17}N",                            "98-94-2", "SVYKKECYCPFKGB-UHFFFAOYSA-N", "N,N-dimethylcyclohexylamine", "")
  CALL Species("C_6H_{12}N_4",                         "100-97-0", "VKYKSIONXSXAKP-UHFFFAOYSA-N", "hexamethylenetetramine", "")
  CALL Species("C_{10}H_{23}N",                       "2016-57-1", "MHZGKXUYDGKKIU-UHFFFAOYSA-N", "1-decanamine", "")
  CALL Species("C_{12}H_{23}N",                        "101-83-7", "XBPCUCUWBYBCDP-UHFFFAOYSA-N", "N-cyclohexylcyclohexanamine", "dicyclohexylamine")
  CALL Species("C_{12}H_{27}N",                        "124-22-1", "JRBPAEWTRLWTQC-UHFFFAOYSA-N", "1-dodecanamine", "")
  CALL Species("C_{18}H_{39}N",                        "124-30-1", "REYJJPSVUYRZGE-UHFFFAOYSA-N", "1-octadecanamine", "")
  CALL Species("C_{24}H_{51}N",                       "1116-76-3", "XTAZYLNFDRKIHJ-UHFFFAOYSA-N", "N,N-dioctyl-1-octanamine", "tri-N-octylamine")
  ! aromatic:
  CALL Species("C_6H_7N",                               "62-53-3", "PAYRUJLWNCNPSJ-UHFFFAOYSA-N", "aminobenzene", "aniline")
  CALL Species("C_7H_9N",                              "100-46-9", "WGQKYBSKWIADBV-UHFFFAOYSA-N", "benzylamine", "") ! other CAS: 857483-23-9, 858831-93-3, 1647116-31-1
  CALL Species("C_7H_9N",                               "95-53-4", "RNVCVTLRINQCPJ-UHFFFAOYSA-N", "2-methylbenzenamine", "2-methylaniline; $o$-toluidine")
  CALL Species("C_7H_9N",                              "108-44-1", "JJYPMNFTHPTTDI-UHFFFAOYSA-N", "3-methylbenzenamine", "3-methylaniline; $m$-toluidine")
  CALL Species("C_7H_9N",                              "106-49-0", "RZXMPPFPUUCRFN-UHFFFAOYSA-N", "4-methylbenzenamine", "4-methylaniline; $p$-toluidine")
  CALL Species("C_8H_{11}N",                           "578-54-1", "MLPVBIWIRCKMJV-UHFFFAOYSA-N", "2-ethylaniline", "$o$-ethylaniline")
  CALL Species("C_8H_{11}N",                           "589-16-2", "HRXZRAXKKNUKRF-UHFFFAOYSA-N", "4-ethylaniline", "$p$-ethylaniline")
  CALL Species("C_8H_{11}N",                            "95-68-1", "CZZZABOKJQXEBO-UHFFFAOYSA-N", "2,4-dimethylbenzenamine", "2,4-dimethylaniline; 2,4-xylidine")
  CALL Species("C_8H_{11}N",                            "95-64-7", "DOLQYFPDPKPQSS-UHFFFAOYSA-N", "3,4-dimethylbenzenamine", "3,4-dimethylaniline; 3,4-xylidine")
  CALL Species("C_8H_{11}N",                            "95-78-3", "VOWZNBNDMFLQGM-UHFFFAOYSA-N", "2,5-dimethylbenzenamine", "2,5-dimethylaniline; 2,5-xylidine")
  CALL Species("C_8H_{11}N",                            "87-62-7", "UFFBMTHBGFGIHF-UHFFFAOYSA-N", "2,6-dimethylbenzenamine", "2,6-dimethylaniline; 2,6-xylidine")
  CALL Species("C_9H_{13}N",                           "137-17-7", "BMIPMKQAAJKBKP-UHFFFAOYSA-N", "2,4,5-trimethylbenzenamine", "2,4,5-trimethylaniline")
  CALL Species("C_9H_{13}N",                           "643-28-7", "YKOLZVXSPGIIBJ-UHFFFAOYSA-N", "2-(1-methylethyl)-benzenamine", "2-isopropylaniline")
  CALL Species("C_{10}H_{15}N",                        "579-66-8", "FOYHNROGBXVLLX-UHFFFAOYSA-N", "2,6-diethylbenzenamine", "2,6-diethylaniline")
  CALL Species("C_6H_8N_2",                             "95-54-5", "GEYOCULIXLDCMW-UHFFFAOYSA-N", "1,2-benzenediamine", "$o$-phenylenediamine")
  CALL Species("C_6H_8N_2",                            "108-45-2", "WZCQRUWWHSTZEM-UHFFFAOYSA-N", "1,3-benzenediamine", "$m$-phenylenediamine")
  CALL Species("C_6H_8N_2",                            "106-50-3", "CBCKQZAAMUWICA-UHFFFAOYSA-N", "1,4-benzenediamine", "$p$-phenylenediamine")
  CALL Species("C_7H_{10}N_2",                         "823-40-5", "RLYCRLGLCUXUPO-UHFFFAOYSA-N", "2-methyl-1,3-benzenediamine", "")
  CALL Species("C_7H_{10}N_2",                          "95-70-5", "OBCSAIDCZQSFQH-UHFFFAOYSA-N", "2-methyl-1,4-benzenediamine", "")
  CALL Species("C_7H_{10}N_2",                        "2687-25-4", "AXNUJYHFQHQZBE-UHFFFAOYSA-N", "3-methyl-1,2-benzenediamine", "2,3-diaminotoluene")
  CALL Species("C_7H_{10}N_2",                          "95-80-7", "VOZKAJLKRJDJLL-UHFFFAOYSA-N", "4-methyl-1,3-benzenediamine", "toluene-2,4-diamine")
  CALL Species("C_7H_{10}N_2",                         "108-71-4", "LVNDUJYMLJDECN-UHFFFAOYSA-N", "3,5-diaminotoluene", "")
  CALL Species("C_6H_8N_2",                            "100-63-0", "HKOOXMFOFWEVGF-UHFFFAOYSA-N", "phenylhydrazine", "")
  CALL Species("C_7H_9N",                              "100-61-8", "AFBPFSWMIHJQDM-UHFFFAOYSA-N", "(methylamino)-benzene", "N-methylaniline")
  CALL Species("C_8H_{11}N",                           "103-69-5", "OJGMBLNIHDZDGS-UHFFFAOYSA-N", "(ethylamino)-benzene", "N-ethylaniline")
  CALL Species("C_8H_{11}N",                           "121-69-7", "JLTDJTHDQAWBAV-UHFFFAOYSA-N", "(dimethylamino)-benzene", "N,N-dimethylaniline")
  CALL Species("C_8H_{11}N",                            "64-04-0", "BHHGXPLMPWCGHP-UHFFFAOYSA-N", "benzeneethanamine", "2-phenylethylamine")
  CALL Species("C_8H_{11}N",                            "87-59-2", "VVAKEQGKZNKUSU-UHFFFAOYSA-N", "2,3-dimethylbenzenamine", "2,3-xylidine")
  CALL Species("C_8H_{11}N",                           "108-69-0", "MKARNSWMMBGSHX-UHFFFAOYSA-N", "3,5-dimethylbenzenamine", "")
  CALL Species("C_8H_{11}N",                          "1300-73-8", "CDULGHZNHURECF-UHFFFAOYSA-N", "dimethylaniline", "xylidine")
  CALL Species("C_8H_{12}N_2",                          "51-71-8", "RMUCZJUITONUFY-UHFFFAOYSA-N", "phenelzine", "")
  CALL Species("C_8H_{12}N_2",                          "99-98-9", "BZORFPDSXLZWJF-UHFFFAOYSA-N", "N,N-dimethyl-1,4-benzenediamine", "")
  CALL Species("C_9H_{13}N",                            "88-05-1", "KWVPRPSXBZNOHS-UHFFFAOYSA-N", "2,4,6-trimethylbenzenamine", "2,4,6-trimethylaniline")
  CALL Species("C_9H_{13}N",                           "102-27-2", "GUYMMHOQXYZMJQ-UHFFFAOYSA-N", "N-ethyl-3-methylbenzenamine", "")
  CALL Species("C_9H_{13}N",                           "768-52-5", "FRCFWPVMFJMNDP-UHFFFAOYSA-N", "N-(1-methylethyl)benzenamine", "")
  CALL Species("C_9H_{13}N",                         "24549-06-2", "JJVKJJNCIILLRP-UHFFFAOYSA-N", "2-ethyl-6-methylbenzenamine", "")
  CALL Species("C_9H_{13}N",                           "103-83-3", "XXBDWLFCJWSEKW-UHFFFAOYSA-N", "N,N-dimethylbenzylamine", "")
  CALL Species("C_9H_{13}N",                            "99-97-8", "GYVGXEWAOAAJEU-UHFFFAOYSA-N", "N,N,4-trimethylbenzenamine", "")
  CALL Species("C_{10}H_{24}N_2",                     "4062-60-6", "KGHYGBGIWLNFAV-UHFFFAOYSA-N", "N,N'-di-{\it tert}-butylethylenediamine", "")
  CALL Species("C_{10}H_{15}N",                         "91-66-7", "GGSUCNLOZRCGPQ-UHFFFAOYSA-N", "(diethylamino)-benzene", "N,N-diethylaniline")
  CALL Species("C_{10}H_9N",                           "134-32-7", "RUFPHBVGCFYCNW-UHFFFAOYSA-N", "1-naphthylamine", "")
  CALL Species("C_{10}H_9N",                            "91-59-8", "JBIJLHTVPXGSAM-UHFFFAOYSA-N", "2-naphthylamine", "")
  CALL Species("C_{10}H_{10}N_2",                     "2243-62-1", "KQSABULTKYLFEV-UHFFFAOYSA-N", "1,5-naphthalenediamine", "")
  CALL Species("C_{10}H_{15}N",                        "122-09-8", "DHHVAGZRUROJKS-UHFFFAOYSA-N", "phentermine", "")
  CALL Species("C_{10}H_{16}N_2",                       "93-05-0", "QNGVNLMMEQUVQK-UHFFFAOYSA-N", "N,N-diethyl-1,4-benzenediamine", "")
  CALL Species("C_{11}H_{18}N_2",                     "2095-01-4", "RQEOBXYYEPMCPJ-UHFFFAOYSA-N", "3,5-diethyltoluene-2,6-diamine", "")
  CALL Species("C_{11}H_{18}N_2",                     "2095-02-5", "PISLZQACAJMAIO-UHFFFAOYSA-N", "2,4-diethyl-6-methylbenzene-1,3-diamine", "")
  CALL Species("C_{12}H_{11}N",                        "122-39-4", "DMBHHRLKUKUOEG-UHFFFAOYSA-N", "diphenylamine", "")
  CALL Species("C_{12}H_{12}N_2",                       "92-87-5", "HFACYLZERDEVSX-UHFFFAOYSA-N", "benzidine", "")
  CALL Species("C_{12}H_{12}N_2",                      "530-50-7", "YHYKLKNNBYLTQY-UHFFFAOYSA-N", "1,1-diphenylhydrazine", "")
  CALL Species("C_{12}H_{12}N_2",                      "122-66-7", "YBQZXXMEJHZYMB-UHFFFAOYSA-N", "1,2-diphenylhydrazine", "N,N'-bianiline")
  CALL Species("C_{12}H_{11}N_3",                       "60-09-3", "QPQKUYVSJWQSDY-UHFFFAOYSA-N", "4-(phenylazo)-benzenamine", "4-aminoazobenzene")
  CALL Species("C_{12}H_{10}N_2",                      "103-33-3", "DMLAVOWQYNRWNQ-UHFFFAOYSA-N", "azobenzene", "")
  CALL Species("C_{12}H_{11}N",                         "90-41-5", "TWBPWBPGNQWFSJ-UHFFFAOYSA-N", "2-aminobiphenyl", "")
  CALL Species("C_{12}H_{11}N",                         "92-67-1", "DMVOXQPQNTYEKQ-UHFFFAOYSA-N", "4-aminobiphenyl", "")
  CALL Species("C_{12}H_{12}N_2",                      "101-54-2", "ATGUVEKSASEFFO-UHFFFAOYSA-N", "N-phenyl-1,4-benzenediamine", "$p$-aminodiphenylamine")
  CALL Species("C_{13}H_{11}N",                        "153-78-6", "CFRFHWQYWJMEJN-UHFFFAOYSA-N", "2-fluorenamine", "")
  CALL Species("C_{13}H_{14}N_2",                      "101-77-9", "YBRVSVVVWCFQMG-UHFFFAOYSA-N", "4,4'-methylenebisbenzenamine", "")
  CALL Species("C_{14}H_{11}N",                        "613-13-8", "YCSBALJAGZKWFF-UHFFFAOYSA-N", "2-anthracenamine", "")
  CALL Species("C_{14}H_{16}N_2",                      "119-93-7", "NUIURNJTPRWVAP-UHFFFAOYSA-N", "3,3'-dimethylbenzidine", "")
  CALL Species("C_{14}H_{15}N_3",                       "60-11-7", "JCYPECIVGRXBMO-UHFFFAOYSA-N", "N,N-dimethyl-4-(phenylazo)-benzenamine", "")
  CALL Species("C_{14}H_{15}N_3",                       "97-56-3", "PFRYFZZSECNQOL-UHFFFAOYSA-N", "$o$-aminoazotoluene", "")
  CALL Species("C_{15}H_{17}N",                         "92-59-1", "HSZCJVZRHXPCIA-UHFFFAOYSA-N", "N-ethyl-N-phenylbenzenemethanamine", "")
  CALL Species("C_{15}H_{18}N_2",                      "101-72-4", "OUBMGJOQLXMSNT-UHFFFAOYSA-N", "N-(1-methylethyl)-N'-phenyl-1,4-benzenediamine", "4-(iso-propylamino)diphenylamine")
  CALL Species("C_{15}H_{18}N_2",                     "1807-55-2", "ZMVMYBGDGJLCHV-UHFFFAOYSA-N", "4,4'-methylene-bis-(N-methylaniline)", "")
  CALL Species("C_{16}H_{13}N_3",                       "85-84-7", "KLCDQSGLLRINHY-UHFFFAOYSA-N", "C.I. Food Yellow 10", "")
  CALL Species("C_{16}H_{20}N_2",                    "54827-17-7", "UAIUNKRWKOVEES-UHFFFAOYSA-N", "3,3',5,5'-tetramethylbenzidine", "")
  CALL Species("C_{16}H_{13}N",                         "90-30-2", "XQVWYOYUZDUNRW-UHFFFAOYSA-N", "N-phenyl-1-naphthalenamine", "")
  CALL Species("C_{17}H_{15}N_3",                      "131-79-3", "BWLVSYUUKOQICP-FMQUCBEESA-N", "yellow OB", "")
  CALL Species("C_{17}H_{21}N_3",                      "492-80-8", "JPIYZTWMUGTEHX-UHFFFAOYSA-N", "auramine", "")
  CALL Species("C_{17}H_{21}N",                        "156-08-1", "YXKTVDFXDRQTKV-UHFFFAOYSA-N", "benzphetamine", "")
  CALL Species("C_{17}H_{22}N_2",                      "101-61-1", "JNRLEMMIVRBKJE-UHFFFAOYSA-N", "4,4'-methylenebis(N,N-dimethylbenzenamine)", "bis(p-dimethylamino)phenylmethane")
  CALL Species("C_{17}H_{25}N",                         "77-10-1", "JTJMJGYZQZDUJJ-UHFFFAOYSA-N", "phencyclidine", "")
  CALL Species("C_{18}H_{16}N_2",                       "74-31-7", "UTGQNNCQYDRXCH-UHFFFAOYSA-N", "N,N'-diphenyl-1,4-benzenediamine", "")
  CALL Species("C_{18}H_{24}N_2",                      "793-24-8", "ZZMVLMVFYMGSMY-UHFFFAOYSA-N", "N-(1,3-dimethylbutyl)-N'-phenyl-1,4-phenylenediamine", "")
  CALL Species("C_{19}H_{23}N_3",                    "33089-61-1", "QXAITBQSYVNQDR-UHFFFAOYSA-N", "amitraz", "")
  CALL Species("C_{22}H_{40}N_2",                      "139-60-6", "JUHXTONDLXIGGK-UHFFFAOYSA-N", "N,N'-bis(1-ethyl-3-methylpentyl)-1,4-benzenediamine", "")
  CALL Species("C_{23}H_{26}N_2",                      "129-73-7", "WZKXBGJNNCGHIC-UHFFFAOYSA-N", "$p$,$p$'-benzylidenebis(N,N-dimethylaniline)", "leucomalachite green")
  CALL Species("C_{24}H_{29}N",                      "51772-35-1", "SNWVRVDHQRBBFG-UHFFFAOYSA-N", "N-phenyl-N-(2,4,4-trimethyl-2-pentanyl)-1-naphthalenamine", "")
  CALL Species("C_{24}H_{51}N",                       "1860-26-0", "BZUDVELGTZDOIG-UHFFFAOYSA-N", "tris(2-ethylhexyl)amine", "")
  CALL Species("C_{25}H_{31}N_3",                      "603-48-5", "OAZWDJGLIYNYMU-UHFFFAOYSA-N", "4,4',4''-methylidyne-tris(N,N-dimethylbenzenamine)", "Leucocrystal violet")
  CALL Species("C_{30}H_{47}N",                      "68608-79-7", "FSPSHPMYFQHGQD-UHFFFAOYSA-N", "N-phenylbenzenamide", "")
  CALL Species("C_{31}H_{43}N_3",                    "68814-02-8", "HOGMPEULJBVHLZ-UHFFFAOYSA-N", "4,4',4''-methanetriyltris(N,N-diethylaniline)", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Heterocycles with nitrogen (\chem{C}, \chem{H}, \chem{N})")
  ! C2:
  CALL Species("C_2H_4N_4",                             "61-82-5", "KLSJWNVTNUYHDU-UHFFFAOYSA-N", "amitrole", "")
  ! C3:
  CALL Species("C_3H_7N",                              "503-29-7", "HONIICLYMWZJFZ-UHFFFAOYSA-N", "azetidine", "")
  CALL Species("C_3H_4N_2",                            "288-32-4", "RAXXELZNTBOGNW-UHFFFAOYSA-N", "imidazole", "")
  CALL Species("C_3H_6N_6",                            "108-78-1", "JDSHMPZPIAZGSV-UHFFFAOYSA-N", "1,3,5-triazine-2,4,6-triamine", "")
  ! C4:
  CALL Species("C_4H_8NH",                             "123-75-1", "RWRDLPDLKQPQOW-UHFFFAOYSA-N", "pyrrolidine", "")
  CALL Species("C_4H_7N",                             "5724-81-2", "ZVJHJDDKYZXRJI-UHFFFAOYSA-N", "1-pyrroline", "")
  CALL Species("C_4H_7N",                              "109-96-6", "JVQIKJMSUIMUDI-UHFFFAOYSA-N", "3-pyrroline", "")
  CALL Species("C_4H_{10}N_2",                         "110-85-0", "GLUUGHFHXGJENI-UHFFFAOYSA-N", "1,4-diazacyclohexane", "piperazine")
  CALL Species("C_4H_5N",                              "109-97-7", "KAESVJOAVNADME-UHFFFAOYSA-N", "pyrrole", "1H-pyrrole")
  CALL Species("C_4H_6N_2",                            "616-47-7", "MCTWTZJPVLRJOU-UHFFFAOYSA-N", "1-methyl-1H-imidazole", "")
  CALL Species("C_4H_4N_2",                            "289-95-2", "CZPWVGJYEJSRLH-UHFFFAOYSA-N", "1,3-diazine", "")
  ! C5:
  CALL Species("C_4H_8NCH_3",                          "120-94-5", "AVFZOVWCLRSYKC-UHFFFAOYSA-N", "N-methylpyrrolidine", "")
  CALL Species("C_5H_{10}NH",                          "110-89-4", "NQRYJNQNLNOLGT-UHFFFAOYSA-N", "piperidine", "")
  CALL Species("C_5H_8N_2",                           "1072-62-4", "PQAMFDRRWURCFQ-UHFFFAOYSA-N", "2-ethylimidazole", "")
  CALL Species("C_5H_8N_2",                           "1739-84-0", "GIWQSPITLQVMSG-UHFFFAOYSA-N", "1,2-dimethylimidazole", "")
  CALL Species("C_5H_{12}N_2",                         "109-01-3", "PVOAHINGSUIXLS-UHFFFAOYSA-N", "N-methylpiperazine", "1-methylpiperazine")
  CALL Species("C_5H_7N",                               "96-54-8", "OXHNLMTVIGZXSG-UHFFFAOYSA-N", "1-methyl-1H-pyrrole", "")
  CALL Species("C_5H_5N",                              "110-86-1", "JUJWROOIHBZHMG-UHFFFAOYSA-N", "pyridine", "")
  CALL Species("C_5D_5N",                             "7291-22-7", "JUJWROOIHBZHMG-RALIUCGRSA-N", "pyridine-d5", "")
  CALL Species("C_5H_6N_2",                            "504-24-5", "NUKYPUAOHBNCPY-UHFFFAOYSA-N", "4-aminopyridine", "")
  CALL Species("C_5H_6N_2",                            "504-29-0", "ICSNLGPSRYBMBD-UHFFFAOYSA-N", "2-aminopyridine", "")
  CALL Species("C_4N_2H_3CH_3",                        "109-08-0", "CAWHJQAVHZEVTJ-UHFFFAOYSA-N", "2-methylpyrazine", "")
  CALL Species("C_5H_5N_5",                             "73-24-5", "GFFGJBXGBJISGV-UHFFFAOYSA-N", "adenine", "")
  ! C6:
  CALL Species("C_6H_7N_5",                            "700-00-5", "WRXCXOUDSPTXNX-UHFFFAOYSA-N", "9-methyladenine", "")
  CALL Species("C_5H_{10}NCH_3",                       "626-67-5", "PAMIQIKDUOTOBW-UHFFFAOYSA-N", "N-methylpiperidine", "")
  CALL Species("C_6H_{12}N_2",                         "280-57-9", "IMNIMPAHZVJRPE-UHFFFAOYSA-N", "triethylenediamine", "")
  CALL Species("C_6H_{10}N_2",                         "931-36-2", "ULKLGIFJWFIQFF-UHFFFAOYSA-N", "2-ethyl-4-methylimidazole", "")
  CALL Species("C_6H_8N_2",                           "3731-52-0", "HDOUGSFASVGDCS-UHFFFAOYSA-N", "3-(aminomethyl)pyridine", "")
  CALL Species("C_6H_{14}N_2",                         "106-58-1", "RXYPXQSKLGGKOL-UHFFFAOYSA-N", "N,N'-dimethylpiperazine", "1,4-dimethylpiperazine")
  CALL Species("C_6H_{10}N_6",                       "66215-27-8", "LVQDKIWDGQRHTE-UHFFFAOYSA-N", "cyromazine", "")
  CALL Species("C_6H_5N_3",                             "95-14-7", "QRUDEWIWKLJBPS-UHFFFAOYSA-N", "1H-benzotriazole", "1,2,3-benzotriazole")
  CALL Species("C_6H_{15}N_3",                         "140-31-8", "IMUDHTPIFIBORV-UHFFFAOYSA-N", "1-piperazineethanamine", "N-(2-aminoethyl)piperazine")
  CALL Species("C_5H_4NCH_3",                          "109-06-8", "BSKHPKMHTQYZBB-UHFFFAOYSA-N", "2-methylpyridine", "2-picoline; $\alpha$-picoline")
  CALL Species("C_5H_4NCH_3",                          "108-99-6", "ITQTTZVARXURQS-UHFFFAOYSA-N", "3-methylpyridine", "3-picoline; $\beta$-picoline")
  CALL Species("C_5H_4NCH_3",                          "108-89-4", "FKNQCJSGGFJEIZ-UHFFFAOYSA-N", "4-methylpyridine", "")
  CALL Species("C_6H_4N_2",                            "100-54-9", "GZPHSAQLYPIAIN-UHFFFAOYSA-N", "3-cyanopyridine", "")
  CALL Species("C_6H_4N_2",                            "100-48-1", "GPHQHTOMRSGBNZ-UHFFFAOYSA-N", "4-cyanopyridine", "")
  CALL Species("C_4N_2H_3(C_2H_5)",                  "13925-00-3", "KVFIJIWMDBAGDP-UHFFFAOYSA-N", "2-ethylpyrazine", "")
  CALL Species("C_6H_8N_2",                            "123-32-0", "LCZUOKDVTBMCMX-UHFFFAOYSA-N", "2,5-dimethylpyrazine", "")
  CALL Species("C_6H_8N_2",                            "108-50-9", "HJFZAYHYIWGLNL-UHFFFAOYSA-N", "2,6-dimethylpyrazine", "3,5-dimethylpyrazine")
  ! C7:
  CALL Species("C_7H_{15}N",                           "766-09-6", "HTLZVHNRZJPSMI-UHFFFAOYSA-N", "N-ethylpiperidine", "1-ethylpiperidine")
  CALL Species("C_7H_6N_2",                             "51-17-2", "HYZJCKYKOHLVJF-UHFFFAOYSA-N", "1H-benzimidazole", "")
  CALL Species("C_7H_7N",                              "100-69-6", "KGIGUEBEKRSTEW-UHFFFAOYSA-N", "2-ethenylpyridine", "2-vinylpyridine")
  CALL Species("C_7H_7N",                              "100-43-6", "KFDVPJUYSDEJTH-UHFFFAOYSA-N", "4-ethenylpyridine", "4-vinylpyridine")
  CALL Species("C_5H_4NC_2H_5",                        "100-71-0", "NRGGMCIBEHEAIL-UHFFFAOYSA-N", "2-ethylpyridine", "")
  CALL Species("C_5H_4NC_2H_5",                        "536-78-7", "MFEIKQPHQINPRI-UHFFFAOYSA-N", "3-ethylpyridine", "")
  CALL Species("C_5H_4NC_2H_5",                        "536-75-4", "VJXRKZJMGVSXPX-UHFFFAOYSA-N", "4-ethylpyridine", "")
  CALL Species("C_5H_3N(CH_3)_2",                      "583-61-9", "HPYNZHMRTTWQTB-UHFFFAOYSA-N", "2,3-dimethylpyridine", "2,3-lutidine")
  CALL Species("C_5H_3N(CH_3)_2",                      "108-47-4", "JYYNAJVZFGKDEQ-UHFFFAOYSA-N", "2,4-dimethylpyridine", "2,4-lutidine")
  CALL Species("C_5H_3N(CH_3)_2",                      "589-93-5", "XWKFPIODWVPXLX-UHFFFAOYSA-N", "2,5-dimethylpyridine", "2,5-lutidine")
  CALL Species("C_5H_3N(CH_3)_2",                      "108-48-5", "OISVCGZHLKNMSJ-UHFFFAOYSA-N", "2,6-dimethylpyridine", "2,6-lutidine")
  CALL Species("C_5H_3N(CH_3)_2",                      "583-58-4", "NURQLCJSMXZBPC-UHFFFAOYSA-N", "3,4-dimethylpyridine", "3,4-lutidine")
  CALL Species("C_5H_3N(CH_3)_2",                      "591-22-0", "HWWYDZCSSYKIAD-UHFFFAOYSA-N", "3,5-dimethylpyridine", "3,5-lutidine")
  ! C8:
  CALL Species("C_8H_{11}N",                           "104-90-5", "NTSLROIKFLNUIJ-UHFFFAOYSA-N", "5-ethyl-2-methylpyridine", "")
  CALL Species("C_5H_2N(CH_3)_3",                      "108-75-8", "BWZVCCNYKMEVEX-UHFFFAOYSA-N", "2,4,6-trimethylpyridine", "2,4,6-collidine")
  CALL Species("C_8H_7N",                              "120-72-9", "SIKJAQJRHWYJAI-UHFFFAOYSA-N", "indole", "")
  CALL Species("C_8H_{12}N_2",                       "13925-07-0", "JZBCTZLGKSYRSF-UHFFFAOYSA-N", "2-ethyl-3,5-dimethylpyrazine", "")
  CALL Species("C_4N_2H_3C_4H_9",                    "29460-92-2", "YAIMUUJMEBJXAA-UHFFFAOYSA-N", "2-isobutylpyrazine", "")
  CALL Species("C_8H_{12}N_2",                       "29460-93-3", "NFFQZEXYZVZKNN-UHFFFAOYSA-N", "2-(1-methylpropyl)-pyrazine", "")
  CALL Species("C_8H_9N",                              "140-76-1", "VJOWMORERYNYON-UHFFFAOYSA-N", "5-ethenyl-2-methylpyridine", "")
  ! C9:
  CALL Species("C_9H_{13}N",                          "3978-81-2", "YSHMQTRICHYLGF-UHFFFAOYSA-N", "4-(1,1-dimethylethyl)-pyridine", "4-{\it tert}-butylpyridine")
  CALL Species("C_9H_{12}N_2",                         "494-97-3", "MYKUKUCHPMASKF-UHFFFAOYSA-N", "nornicotine", "")
  CALL Species("C_9H_9N_5",                             "91-76-9", "GZVHEAJQGPRDLQ-UHFFFAOYSA-N", "2,4-diamino-6-phenyl-1,3,5-triazine", "")
  CALL Species("C_9H_9N",                              "603-76-9", "BLRHMMGNCXNXJL-UHFFFAOYSA-N", "N-methylindole", "")
  CALL Species("C_9H_9N",                               "83-34-1", "ZFRKQXVRDFCRJG-UHFFFAOYSA-N", "3-methylindole", "skatole")
  CALL Species("C_9H_{14}N_2",                       "18138-04-0", "PSINWXIDJYEXLO-UHFFFAOYSA-N", "2,3-diethyl-5-methylpyrazine", "")
  CALL Species("C_9H_7N",                               "91-22-5", "SMWDFEZZVXVKRB-UHFFFAOYSA-N", "benzo[$b$]pyridine", "quinoline")
  CALL Species("C_9H_7N",                              "119-65-3", "AWJUIBRHMBBTKR-UHFFFAOYSA-N", "benzo[$c$]pyridine", "isoquinoline")
  ! C10:
  CALL Species("C_{10}H_{14}N_2",                       "54-11-5", "SNICXCGAKADSCV-SNVBAGLBSA-N", "nicotine", "")
  CALL Species("C_{10}H_8N_2",                         "366-18-7", "ROFVEXUMMXZLPA-UHFFFAOYSA-N", "2,2'-bipyridine", "")
  CALL Species("C_{10}H_8N_2",                         "553-26-4", "MWVTWFVJZLCBMC-UHFFFAOYSA-N", "4,4'-bipyridine", "")
  CALL Species("C_{10}H_9N",                            "91-63-4", "SMUQFGGVLNAIOZ-UHFFFAOYSA-N", "2-methylquinoline", "")
  CALL Species("C_{10}H_9N",                           "491-35-0", "MUDSDYNRBDKLGK-UHFFFAOYSA-N", "4-methylquinoline", "")
  ! C11:
  CALL Species("C_{11}H_{11}N_5",                    "77500-04-0", "DVCCCQNKIYNAKB-UHFFFAOYSA-N", "MEIQX", "2-amino-3,8-dimethylimidazo[4,5-f]quinoxaline")
  CALL Species("C_{11}H_{11}N_5",                       "94-78-0", "QPFYXYFORQJZEC-UHFFFAOYSA-N", "3-(phenylazo)-2,6-pyridinediamine", "phenazopyridine")
  CALL Species("C_{11}H_9N_3",                       "26148-68-5", "FJTNLJLPLJDTRM-UHFFFAOYSA-N", "2-amino-9H-pyrido[2,3-b]indole", "")
  ! C12:
  CALL Species("C_{12}H_9N",                            "86-74-8", "UJOBWOGCFQCDNV-UHFFFAOYSA-N", "carbazole", "")
  CALL Species("C_{12}H_8N_2",                          "66-71-7", "DGEZNRSVGBDHLK-UHFFFAOYSA-N", "$o$-phenanthroline", "")
  CALL Species("C_{12}H_{11}N_5",                     "1214-39-7", "NWBJYWHLCVSVIJ-UHFFFAOYSA-N", "benzyladenine", "")
  CALL Species("C_{12}H_{12}N_2",                     "2764-72-9", "SYJFEGQWDCRVNX-UHFFFAOYSA-N", "diquat", "")
  CALL Species("C_{12}H_{12}N_4",                    "77094-11-2", "GMGWMIJIGUYNAY-UHFFFAOYSA-N", "MEIQ", "2-amino-3,4-dimethylimidazo[4,5-f]quinoxaline")
  CALL Species("C_{12}H_{13}N_3",                    "53112-28-0", "ZLIBICFPKPWGIZ-UHFFFAOYSA-N", "pyrimethanil", "")
  CALL Species("C_{12}H_{14}N_2",                     "4685-14-7", "INFDPOAKFNIJBF-UHFFFAOYSA-N", "paraquat", "")
  CALL Species("C_{12}H_{16}N_2",                       "61-50-7", "DMULVCHRPCFFGV-UHFFFAOYSA-N", "N,N-dimethyltryptamine", "")
  ! C13:
  CALL Species("C_{13}H_9N",                            "85-02-9", "HCAUQPZEWLULFJ-UHFFFAOYSA-N", "benzo[$f$]quinoline", "")
  CALL Species("C_{13}H_{21}N",                        "585-48-8", "UWKQJZCTQGMHKD-UHFFFAOYSA-N", "2,6-bis-(1,1-dimethylethyl)-pyridine", "2,6-di-{\it tert}-butylpyridine")
  CALL Species("C_{13}H_{12}N_4",                   "105650-23-5", "UQVKZNNCIHJZLS-UHFFFAOYSA-N", "PHIP", "2-amino-1-methyl-6-phenylimidazo[4,5-b]pyridine")
  CALL Species("C_{13}H_{13}N_3",                      "102-06-7", "OWRCNXZUPFZXOS-UHFFFAOYSA-N", "N,N'-diphenylguanidine", "")
  CALL Species("C_{13}H_9N",                           "260-94-6", "DZBUGLKDJFMEHC-UHFFFAOYSA-N", "acridine", "")
  CALL Species("C_{13}H_9N",                           "229-87-8", "RDOWQLZANAYVLL-UHFFFAOYSA-N", "phenanthridine", "")
  ! C14:
  CALL Species("C_{14}H_{13}N_3",                   "110235-47-7", "CIFWZNRJIBNXRE-UHFFFAOYSA-N", "mepanipyrim", "")
  CALL Species("C_{14}H_{15}N_3",                   "121552-61-2", "HAORKNGNJCEJBX-UHFFFAOYSA-N", "cyprodinil", "")
  CALL Species("C_{14}H_{16}N_4",                    "99011-02-6", "DOUYETYNHWVLEO-UHFFFAOYSA-N", "imiquimod", "")
  ! >= C15:
  CALL Species("C_{15}H_{25}N_5",                   "865318-97-4", "GGKQIOFASHYUJZ-UHFFFAOYSA-N", "ametoctradin", "")
  CALL Species("C_{15}H_{18}N_4",                    "89269-64-7", "GOWLARCWZRESHU-UHFFFAOYSA-N", "ferimzone", "")
  CALL Species("C_{17}H_{11}N",                        "225-51-4", "OAPPEBNXKAKQGS-UHFFFAOYSA-N", "benz[$c$]acridine", "")
  CALL Species("C_{18}H_{27}N",                      "10594-03-3", "FNUATPIDZQSFPD-UHFFFAOYSA-N", "6-pentyl-1,2,3,4,7,8,9,10-octahydrophenanthridine", "")
  CALL Species("C_{18}H_{21}N",                        "519-74-4", "RWTNXJXZVGHMGI-UHFFFAOYSA-N", "2-diphenylmethylpiperidine", "desoxypipradrol")
  CALL Species("C_{19}H_{31}N ",                     "67306-00-7", "MGNFYQILYYYUBS-UHFFFAOYSA-N", "fenpropidin", "")
  CALL Species("C_{19}H_{38}N_4",                    "91273-04-0", "AVBBHCMDRGQBNW-UHFFFAOYSA-N", "N,N-bis(2-ethylhexyl)-1H-1,2,4-triazole-1-methanamine", "")
  CALL Species("C_{20}H_{13}N",                        "194-59-2", "STJXCDGCXVZHDU-UHFFFAOYSA-N", "7H-dibenzo[$c,g$]carbazole", "")
  CALL Species("C_{20}H_{14}N_4",                      "101-60-0", "RKCAIXNGYQCCAL-YYOYBPFYSA-N", "porphyrin", "porphin")
  CALL Species("C_{21}H_{13}N",                        "224-42-0", "ANUCHZVCBDOPOX-UHFFFAOYSA-N", "dibenz[$a,j$]acridine", "")
  CALL Species("C_{21}H_{13}N",                        "226-36-8", "JNCSIWAONQTVCF-UHFFFAOYSA-N", "dibenz[$a,h$]acridine", "")
  CALL Species("C_{21}H_{39}N_3",                     "6281-14-7", "ZLLRUEJANKJPQE-UHFFFAOYSA-N", "1,3,5-tricyclohexylhexahydro-1,3,5-triazine", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Nitriles (\chem{C}, \chem{H}, \chem{N})")
  CALL Species("CN",                                  "2074-87-5", "JEVCWSUVFOYBFI-UHFFFAOYSA-N", "cyano radical", "")
  CALL Species("HCN",                                   "74-90-8", "LELOWRISYMNNSU-UHFFFAOYSA-N", "hydrogen cyanide", "hydrocyanic acid")
  CALL Species("CH_3CN",                                "75-05-8", "WEVYAHXRMPXWCK-UHFFFAOYSA-N", "ethanenitrile", "acetonitrile")
  CALL Species("C_2H_5CN",                             "107-12-0", "FVSKHRXBFJPNKK-UHFFFAOYSA-N", "propanenitrile", "propionitrile")
  CALL Species("C_3H_7CN",                             "109-74-0", "KVNRLNFWIYMESJ-UHFFFAOYSA-N", "butanenitrile", "butyronitrile")
  CALL Species("C_4H_7N",                               "78-82-0", "LRDFRRGEGBBSRN-UHFFFAOYSA-N", "2-methylpropanenitrile", "isobutyronitrile")
  CALL Species("C_4H_9CN",                             "110-59-8", "RFFFKMOABOFIDF-UHFFFAOYSA-N", "pentanenitrile", "butyl cyanide; valeronitrile")
  CALL Species("C_6H_{11}N",                           "628-73-9", "AILKHAQXUAOOFU-UHFFFAOYSA-N", "hexanenitrile", "")
  CALL Species("C_7H_{13}N",                           "629-08-3", "SDAXRHHPNYTELL-UHFFFAOYSA-N", "heptanenitrile", "")
  CALL Species("C_8H_{15}N",                           "124-12-9", "YSIMAPNUZAVQER-UHFFFAOYSA-N", "octanenitrile", "")
  CALL Species("C_9H_{17}N",                          "2243-27-8", "PLZZPPHAMDJOSR-UHFFFAOYSA-N", "nonanenitrile", "")
  CALL Species("C_{10}H_{19}N",                       "1975-78-6", "HBZDPWBWBJMYRY-UHFFFAOYSA-N", "decanenitrile", "")
  CALL Species("C_{11}H_{21}N",                       "2244-07-7", "SZKKNEOUHLFYNA-UHFFFAOYSA-N", "undecanenitrile", "")
  CALL Species("C_7H_{11}N",                           "766-05-2", "VBWIZSYFQSOUFQ-UHFFFAOYSA-N", "cyclohexanecarbonitrile", "")
  CALL Species("C_2N_2",                               "460-19-5", "JMANVNJQNLATNU-UHFFFAOYSA-N", "ethanedinitrile", "cyanogen")
  CALL Species("C_5H_6N_2",                            "544-13-8", "ZTOMUSMDRMJOTH-UHFFFAOYSA-N", "pentanedinitrile", "")
  CALL Species("C_6H_8N_2",                            "111-69-3", "BTGRAWJCKBQKAO-UHFFFAOYSA-N", "hexanedinitrile", "adiponitrile")
  CALL Species("C_6H_8N_2",                           "4553-62-2", "FPPLREPCQJZDAQ-UHFFFAOYSA-N", "2-methylpentanedinitrile", "")
  CALL Species("C_8H_{12}N_2",                        "3333-52-6", "ZVQXQPNJHRNGID-UHFFFAOYSA-N", "tetramethylbutanedinitrile", "")
  CALL Species("C_3H_3N",                              "107-13-1", "NLHHRLWOUZZQLW-UHFFFAOYSA-N", "2-propenenitrile", "acrylonitrile")
  CALL Species("C_4H_5N",                             "4786-20-3", "NKKMVIVFRUYPLQ-NSCUHMNNSA-N", "2-butenenitrile", "crotononitrile")
  CALL Species("C_4H_5N",                              "109-75-1", "SJNALLRHIVGIBI-UHFFFAOYSA-N", "3-butenenitrile", "vinylacetonitrile")
  CALL Species("C_4H_5N",                              "126-98-7", "GYCMBHHDWRMZGG-UHFFFAOYSA-N", "2-methyl-2-propene nitrile", "methacrylonitrile")
  CALL Species("C_6H_5CN",                             "100-47-0", "JFDZBHWFFUWGJE-UHFFFAOYSA-N", "benzenenitrile", "benzonitrile")
  CALL Species("C_6H_4N_2",                            "100-70-9", "FFNVQNRYTPFDDP-UHFFFAOYSA-N", "2-pyridinecarbonitrile", "")
  CALL Species("C_6H_9N_3",                            "111-94-4", "SBAJRGRUGUQKAF-UHFFFAOYSA-N", "3,3'-iminobispropanenitrile", "")
  CALL Species("C_8H_7N",                              "529-19-1", "NWPNXBQSRGKSJB-UHFFFAOYSA-N", "2-methylbenzonitrile", "$o$-tolunitrile")
  CALL Species("C_8H_7N",                              "620-22-4", "BOHCMQZJWOGWTA-UHFFFAOYSA-N", "3-methylbenzonitrile", "$m$-tolunitrile")
  CALL Species("C_8H_7N",                              "140-29-4", "SUSQOBVLVYHIEX-UHFFFAOYSA-N", "benzeneacetonitrile", "phenylacetonitrile")
  CALL Species("C_8H_4N_2",                             "91-15-6", "XQZYPMVTSDWCCE-UHFFFAOYSA-N", "1,2-benzenedicarbonitrile", "")
  CALL Species("C_{10}H_{15}N",                       "5146-66-7", "HLCSDJLATUNSSI-JXMROGBWSA-N", "3,7-dimethyl-2,6-octadienenitrile", "geranyl nitrile")
  CALL Species("C_{10}H_{16}N_4",                    "13472-08-7", "AVTLBBWTUPQRAY-UHFFFAOYSA-N", "2,2'-azobis(2-methylbutyronitrile)", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Amines, amides, amino acids (\chem{C}, \chem{H}, \chem{O}, \chem{N})")
  ! C, H, O, N:
  ! C1:
  CALL Species("CH_3NO",                                "75-12-7", "ZHNUHDYFZUAESO-UHFFFAOYSA-N", "formamide", "")
  CALL Species("CH_3ONO",                              "624-91-9", "BLLFVUPNHCTMSV-UHFFFAOYSA-N", "methyl nitrite", "")
  CALL Species("CH_4N_2O",                              "57-13-6", "XSQUKJJJFZCRTK-UHFFFAOYSA-N", "urea", "")
  ! C2:
  CALL Species("HOC_2H_4NH_2",                         "141-43-5", "HZAXFHJVJLSVMW-UHFFFAOYSA-N", "ethanolamine", "")
  CALL Species("C_2H_4N_4O_2",                         "123-77-3", "XOZUGNYVDXMRKW-UHFFFAOYSA-N", "1,1'-azodiformamide", "")
  CALL Species("C_2H_5ONO",                            "109-95-5", "QQZWEECEMNQSTG-UHFFFAOYSA-N", "ethyl nitrite", "")
  CALL Species("C_2H_5NO_2",                           "598-55-0", "GTCAXTIRRLKXRU-UHFFFAOYSA-N", "carbamic acid, methyl ester", "")
  CALL Species("C_2H_5NO",                             "107-29-9", "FZENGILVLUJGJX-UHFFFAOYSA-N", "acetaldoxime", "acetaldehyde oxime")
  CALL Species("C_2H_5NO",                              "60-35-5", "DLFVBJFMPXGRIB-UHFFFAOYSA-N", "ethanamide", "acetamide")
  CALL Species("C_2H_5NO",                             "123-39-7", "ATHHXGZTWNVVOU-UHFFFAOYSA-N", "N-methylmethanamide", "N-methylformamide")
  CALL Species("C_2H_6N_2O",                            "62-75-9", "UMFJAHHVKNCGLG-UHFFFAOYSA-N", "N-nitrosodimethylamine", "")
  CALL Species("C_2H_5N_3O_2",                         "684-93-5", "ZRKWMRDKSOPRRS-UHFFFAOYSA-N", "methylnitrosourea", "")
  CALL Species("C_2H_4N_2O_2",                         "471-46-5", "YIKSCQDJHCMVMK-UHFFFAOYSA-N", "ethanediamide", "")
  ! C3:
  CALL Species("C_3H_6N_2O",                         "15216-10-1", "SNKTZBNDUVWOAZ-UHFFFAOYSA-N", "nitrosoazetidine", "")
  CALL Species("C_3H_6N_2O_2",                        "7417-67-6", "FFLFWNRFMZRFKU-UHFFFAOYSA-N", "methylnitrosoacetamide", "")
  CALL Species("C_3H_5N_3O",                         "38434-77-4", "LMIMSGCBKHFTDY-UHFFFAOYSA-N", "ethylnitrosocyanamide", "")
  CALL Species("C_3H_5NO",                              "79-06-1", "HRPVXLWXLXDGHG-UHFFFAOYSA-N", "2-propenamide", "acrylamide")
  CALL Species("C_3H_6N_2O",                          "4549-40-0", "AWZVYNHQGTZJIH-UHFFFAOYSA-N", "methylvinylnitrosamine", "N-nitrosomethylvinylamine")
  CALL Species("C_3H_7NO_2",                            "51-79-6", "JOYRKODLDBILNP-UHFFFAOYSA-N", "urethane", "")
  CALL Species("C_3H_7NO",                              "79-05-0", "QLNJFJADRCOGBJ-UHFFFAOYSA-N", "propanamide", "propionamide")
  CALL Species("C_3H_7NO",                              "68-12-2", "ZMXDDKWLCZADIW-UHFFFAOYSA-N", "N,N-dimethylmethanamide", "N,N-dimethylformamide")
  CALL Species("C_3H_7NO",                              "79-16-3", "OHLUUHNLEMFGTQ-UHFFFAOYSA-N", "N-methylacetamide", "")
  CALL Species("C_3H_8N_2O",                         "10595-95-6", "RTDCJKARQCRONF-UHFFFAOYSA-N", "N-methyl-N-nitrosoethanamine", "N-nitrosomethylethylamine")
  CALL Species("C_3H_9NO",                             "109-85-3", "ASUDFOJKTJLAIK-UHFFFAOYSA-N", "2-methoxyethanamine", "2-methoxyethylamine")
  CALL Species("C_3H_9NO",                             "109-83-1", "OPKOKAMJFNKNAS-UHFFFAOYSA-N", "2-(methylamino)ethanol", "")
  CALL Species("C_3H_9NO",                              "78-96-6", "HXKKHQJGJAFBHI-UHFFFAOYSA-N", "1-amino-2-propanol", "")
  CALL Species("C_3H_9NO",                            "6168-72-5", "BKMMTJMQCTUHRP-UHFFFAOYSA-N", "2-amino-1-propanol", "alaninol")
  ! C4:
  CALL Species("C_4H_8N_2O_3",                         "615-53-2", "CAUBWLYZCDDYEF-UHFFFAOYSA-N", "N-nitroso-N-methylurethane", "N-nitroso-N-methylurethane")
  CALL Species("C_4H_8N_4O_2",                         "140-79-4", "WNSYEWGYAFFSSQ-UHFFFAOYSA-N", "dinitrosopiperazine", "")
  CALL Species("C_4H_{10}N_2O_2",                    "16676-91-8", "UJVHVMNGOZXSOZ-UHFFFAOYSA-N", "2-amino-3(methylamino)propionic acid", "3-(methylamino)-(DL)-alanine")
  CALL Species("C_4H_{10}N_2O_3",                     "1116-54-7", "YFCDLVPYFMHRQZ-UHFFFAOYSA-N", "N-nitrosodiethanolamine", "")
  CALL Species("C_4H_{10}N_2O",                         "55-18-5", "WBNQDOYYEUMPFS-UHFFFAOYSA-N", "N-nitrosodiethylamine", "")
  CALL Species("C_4H_{11}NO_2",                        "111-42-2", "ZBCBWPMODOFKDW-UHFFFAOYSA-N", "diethanolamine", "")
  CALL Species("C_4H_{11}NO_2",                        "929-06-6", "GIAFURWZWWWBQT-UHFFFAOYSA-N", "diglycolamine", "")
  CALL Species("C_4H_{11}NO",                         "5332-73-0", "FAXDZWQIWUSWJH-UHFFFAOYSA-N", "3-methoxy-1-propanamine", "3-methoxypropylamine")
  CALL Species("C_4H_{11}NO",                          "124-68-5", "CBTVGIZVANVGBH-UHFFFAOYSA-N", "2-amino-2-methyl-1-propanol", "")
  CALL Species("C_4H_{11}NO",                          "108-01-0", "UEEJHVSXFDXPFK-UHFFFAOYSA-N", "N,N-dimethylaminoethanol", "dimethylethanolamine")
  CALL Species("C_4H_{12}N_2O",                        "111-41-1", "LHIJANUOQQMGNT-UHFFFAOYSA-N", "2-[(2-aminoethyl)amino]ethanol", "")
  CALL Species("C_4H_{13}NO",                           "75-59-2", "WGTYBPLFGIVFAS-UHFFFAOYSA-M", "tetramethylammonium hydroxide", "")
  CALL Species("C_4H_7NO",                              "79-39-0", "FQPSGWSUVKBHSU-UHFFFAOYSA-N", "methacrylamide", "")
  CALL Species("C_4H_7NO",                              "75-86-5", "MWFMGBPGAXYFAR-UHFFFAOYSA-N", "acetone cyanohydrin", "")
  CALL Species("C_4H_9NO_2",                          "1746-77-6", "OVPLZYJGTGDFNB-UHFFFAOYSA-N", "carbamic acid, 1-methylethyl ester", "")
  CALL Species("C_4H_9NO_2",                           "627-12-3", "YNTOKMNHRPSGFU-UHFFFAOYSA-N", "propylcarbamate", "")
  CALL Species("C_4H_9NO",                             "541-35-5", "DNSISZSEWVHGLH-UHFFFAOYSA-N", "butanamide", "")
  CALL Species("C_4H_9NO",                             "127-19-5", "FXHOOIRPVKKKFG-UHFFFAOYSA-N", "N,N-dimethylacetamide", "")
  CALL Species("C_4H_9NO",                              "96-29-7", "WHIVNJATOVLWBW-UHFFFAOYSA-N", "2-butanone, oxime", "")
  ! C5:
  CALL Species("C_5H_{10}N_2O_3",                      "614-95-9", "RAUQLNDTFONODT-UHFFFAOYSA-N", "nitrosoethylurethane", "N-ethyl-N-nitrosourethane")
  CALL Species("C_5H_{11}NO",                          "617-84-5", "SUAKHGWARZSWIH-UHFFFAOYSA-N", "N,N-diethylmethanamide", "N,N-diethylformamide")
  CALL Species("C_5H_{11}N_3O_2",                      "869-01-2", "LSWOCDLIYSKTRX-UHFFFAOYSA-N", "N-nitroso-N-butylurea", "")
  CALL Species("C_5H_{13}NO_2",                        "105-59-9", "CRVGTESFCCXCTH-UHFFFAOYSA-N", "N-methyldiethanolamine", "")
  CALL Species("C_5H_{12}N_2O",                       "7068-83-9", "PKTSCJXWLVREKX-UHFFFAOYSA-N", "methylbutylnitrosamine", "")
  CALL Species("C_5H_{13}NO",                          "109-56-8", "RILLZYSZSDGYGV-UHFFFAOYSA-N", "2-(isopropylamino)ethanol", "")
  CALL Species("C_5H_{13}NO_2",                        "623-57-4", "QCMHUGYTOGXZIW-UHFFFAOYSA-N", "3-(dimethylamino)-1,2-propanediol", "")
  CALL Species("C_5H_{13}NO_2",                        "115-70-8", "IOAOAKDONABGPZ-UHFFFAOYSA-N", "2-amino-2-ethyl-1,3-propanediol", "")
  ! C6:
  CALL Species("C_6H_{11}NO_3",                       "1596-84-5", "NOQGZXFMHARMLW-UHFFFAOYSA-N", "daminozide", "")
  CALL Species("C_6H_{14}N_2O",                      "13256-07-0", "KSFCDINBDBFFSI-UHFFFAOYSA-N", "methylpentylnitrosamine", "")
  CALL Species("C_6H_{14}N_2O",                       "4549-44-4", "ZGMCNGHHUQZNIH-UHFFFAOYSA-N", "ethylbutylnitrosamine", "")
  CALL Species("C_6H_{12}N_2O",                        "932-83-2", "UZMVSVHUTOAPTD-UHFFFAOYSA-N", "nitrosohexamethyleneimine", "")
  CALL Species("C_6H_{12}N_2O_2",                     "1456-28-6", "DPYMAXOKJUBANR-UHFFFAOYSA-N", "2,6-dimethylnitrosomorpholine", "")
  CALL Species("C_6H_{12}N_4O_2",                    "55380-34-2", "JIWAGFGPBKDFQN-UHFFFAOYSA-N", "2,6-dimethyldinitrosopiperazine", "")
  CALL Species("C_6H_{11}NO",                         "2210-25-5", "QNILTEGFHQSKFF-UHFFFAOYSA-N", "N-(1-methylethyl)-2-propenamide", "N-isopropylacrylamide")
  CALL Species("C_6H_{13}NO",                          "628-02-4", "ALBYIUDWACNRRB-UHFFFAOYSA-N", "hexanamide", "")
  CALL Species("C_6H_{13}NO",                         "1119-49-9", "GYLDXXLJMRTVSS-UHFFFAOYSA-N", "N-butylacetamide", "")
  CALL Species("C_6H_{14}N_2O",                        "601-77-4", "AUIKJTGFPFLMFP-UHFFFAOYSA-N", "N-(1-methylethyl)-N-nitroso-2-propanamine", "nitrosodiisopropylamine")
  CALL Species("C_6H_{14}N_2O",                        "621-64-7", "YLKFDHTUAUWZPQ-UHFFFAOYSA-N", "N-nitrosodipropylamine", "N,N-dipropylnitrosamine")
  CALL Species("C_6H_{14}N_2O",                        "103-76-4", "WFCSWCVEJLETKA-UHFFFAOYSA-N", "N-(2-hydroxyethyl)piperazine", "")
  CALL Species("C_6H_{15}NO_2",                        "139-87-7", "AKNUHUCEWALCOI-UHFFFAOYSA-N", "N-ethyldiethanolamine", "")
  CALL Species("C_6H_{15}NO_3",                        "102-71-6", "GSEJCLTVZPLZKY-UHFFFAOYSA-N", "triethanolamine", "")
  CALL Species("C_6H_7NO",                              "95-55-6", "CDAWCLOXVUBKRW-UHFFFAOYSA-N", "$o$-aminophenol", "2-aminophenol")
  CALL Species("C_6H_7NO",                             "123-30-8", "PLIKAWJENQZMHA-UHFFFAOYSA-N", "4-aminophenol", "")
  CALL Species("C_6H_7NO",                             "591-27-5", "CWLKGDAVCFYWJK-UHFFFAOYSA-N", "3-aminophenol", "")
  CALL Species("C_6H_{13}NO_2",                        "622-40-2", "KKFDCBRMNNSAAW-UHFFFAOYSA-N", "4-(2-hydroxyethyl)morpholine", "")
  CALL Species("C_6H_{13}NO",                          "106-52-5", "BAUWRHPMUVYFOD-UHFFFAOYSA-N", "4-hydroxy-N-methylpiperidine", "")
  CALL Species("C_6H_{15}NO_2",                       "1704-62-7", "YSAANLSYLSUVHB-UHFFFAOYSA-N", "dimethylaminoethoxyethanol", "")
  CALL Species("C_6H_{15}NO",                          "100-37-8", "BFSVOASYOCHEOV-UHFFFAOYSA-N", "2-(diethylamino)ethanol", "")
  CALL Species("C_6H_{15}NO_2",                        "111-95-5", "IBZKBSXREAQDTO-UHFFFAOYSA-N", "bis(2-methoxyethyl)amine", "")
  CALL Species("C_6H_{11}NO",                          "100-64-1", "VEZUQRBDRNJBJY-UHFFFAOYSA-N", "cyclohexanone oxime", "")
  ! C7:
  CALL Species("C_7H_{10}N_2O",                       "5307-02-8", "HGUYBLVGLMAUFF-UHFFFAOYSA-N", "$p$-diaminoanisole", "2-methoxy-1,4-benzenediamine")
  CALL Species("C_7H_{10}N_2O",                        "615-05-4", "BAHPQISAXRFLCL-UHFFFAOYSA-N", "4-methoxy-1,3-benzenediamine", "")
  CALL Species("C_7H_{10}N_4O_3",                    "57966-95-7", "XERJKGMBORTKEO-UHFFFAOYSA-N", "2-cyano-N-[(ethylamino)carbonyl]-2-(methoxyimino)acetamide", "cymoxanil")
  CALL Species("C_7H_{11}NO",                         "3173-53-3", "KQWGXHWJMSMDJJ-UHFFFAOYSA-N", "isocyanatocyclohexane", "")
  CALL Species("C_7H_{14}N_2O_3",                     "3081-61-6", "DATAGRPVKZEWHA-YFKPBYRVSA-N", "$L$-theanine", "")
  CALL Species("C_7H_5N_5O_8",                         "479-45-8", "AGUIVNYEYSCPNI-UHFFFAOYSA-N", "tetryl", "")
  CALL Species("C_7H_7NO_2",                           "118-92-3", "RWZYAGGXGHYGMB-UHFFFAOYSA-N", "anthranilic acid", "")
  CALL Species("C_7H_7NO_2",                            "99-05-8", "XFDUHJPVQKIXHO-UHFFFAOYSA-N", "3-aminobenzoic acid", "")
  CALL Species("C_7H_7NO_2",                           "150-13-0", "ALYNCZNDIQEVRV-UHFFFAOYSA-N", "4-aminobenzoic acid", "")
  CALL Species("C_7H_7NO_2",                            "65-45-2", "SKZKKFZAGNVIMN-UHFFFAOYSA-N", "salicylamide", "")
  CALL Species("C_7H_7NO_3",                            "89-57-6", "KBOPZPXVLCULAV-UHFFFAOYSA-N", "mesalamine", "")
  CALL Species("C_7H_7NO",                             "103-70-8", "DYDNPESBYVVLBO-UHFFFAOYSA-N", "N-phenylformamide", "")
  CALL Species("C_7H_7NO",                              "55-21-0", "KXDAEFPNCMNJSK-UHFFFAOYSA-N", "benzamide", "")
  CALL Species("C_7H_8N_2O",                            "88-68-6", "PXBFMLJZNCDSMP-UHFFFAOYSA-N", "anthranilamide", "2-aminobenzamide")
  CALL Species("C_7H_8N_2O",                           "614-00-6", "MAXCWSIJKVASQC-UHFFFAOYSA-N", "N-methyl-N-nitrosobenzenamine", "")
  CALL Species("C_7H_9NO",                              "90-04-0", "VMPITZXILSNTON-UHFFFAOYSA-N", "2-methoxy-benzenamine", "2-methoxyaniline")
  CALL Species("C_7H_9NO",                             "536-90-3", "NCBZRJODKRCREW-UHFFFAOYSA-N", "3-methoxy-benzenamine", "3-methoxyaniline")
  CALL Species("C_7H_9NO",                             "104-94-9", "BHAAPTBBJKJZER-UHFFFAOYSA-N", "4-methoxy-benzenamine", "4-methoxyaniline")
  CALL Species("C_7H_9NO",                             "150-75-4", "ZFIQGRISGKSVAG-UHFFFAOYSA-N", "4-(methylamino)phenol", "N-methyl-4-aminophenol")
  CALL Species("C_7H_{17}NO_2",                        "621-56-7", "LTACQVCHVAUOKN-UHFFFAOYSA-N", "3-(diethylamino)-1,2-propanediol", "")
  CALL Species("C_7H_{15}NO",                         "3040-44-6", "KZTWONRVIPPDKH-UHFFFAOYSA-N", "1-(2-hydroxyethyl)piperidine", "2-piperidinoethanol")
  CALL Species("C_7H_{15}NO",                         "1484-84-0", "PTHDBHDZSMGHKF-UHFFFAOYSA-N", "2-piperidineethanol", "")
  CALL Species("C_7H_{15}NO_2",                         "78-28-4", "SLWGJZPKHAXZQL-UHFFFAOYSA-N", "emylcamate", "")
  ! C8:
  CALL Species("C_8H_{10}N_2O",                       "1007-36-9", "SQBHGDSDVWCPHN-UHFFFAOYSA-N", "defenuron", "")
  CALL Species("C_8H_{10}N_2O",                        "937-40-6", "NGXUJKBJBFLCAR-UHFFFAOYSA-N", "N-methyl-N-nitrosobenzenemethanamine", "")
  CALL Species("C_8H_{11}NO_3",                         "51-41-2", "SFLSHLFXELFNJZ-QMMMGPOBSA-N", "norepinephrine", "")
  CALL Species("C_8H_{11}NO_2",                       "2735-04-8", "GEQNZVKIDIPGCO-UHFFFAOYSA-N", "2,4-dimethoxyaniline", "")
  CALL Species("C_8H_{11}NO",                          "102-50-1", "CDGNLUSBENXDGG-UHFFFAOYSA-N", "4-methoxy-2-methylbenzenamine", "$m$-cresidine")
  CALL Species("C_8H_{11}NO",                          "120-71-8", "WXWCDTXEKCVRRO-UHFFFAOYSA-N", "$p$-cresidine", "")
  CALL Species("C_8H_{18}N_2O",                        "924-16-3", "YGJHZCLPZAZIHH-UHFFFAOYSA-N", "N-nitrosodi-N-butylamine", "")
  CALL Species("C_8H_{19}NO",                           "96-80-0", "ZYWUVGFIXPNBDL-UHFFFAOYSA-N", "(diisopropylamino)-ethanol", "")
  CALL Species("C_8H_8N_2O_2",                          "88-96-0", "NAYYNDKKHOIIOD-UHFFFAOYSA-N", "phthalamide", "")
  CALL Species("C_8H_9NO_2",                           "103-90-2", "RZVAJINKPMORJF-UHFFFAOYSA-N", "acetaminophen", "")
  CALL Species("C_8H_9NO_2",                           "134-20-3", "VAMXMNNIEUEQDV-UHFFFAOYSA-N", "methyl anthranilate", "")
  CALL Species("C_8H_9NO",                              "99-92-3", "GPRYKVSEZCQIHD-UHFFFAOYSA-N", "4'-aminoacetophenone", "4-acetylaniline")
  CALL Species("C_8H_9NO",                             "103-84-4", "FZERHIULMFGESH-UHFFFAOYSA-N", "N-phenylacetamide", "acetanilide")
  CALL Species("C_8H_{15}NO",                          "120-29-6", "CYHOMWAPJJPNMW-UHFFFAOYSA-N", "tropine", "")
  CALL Species("C_8H_{20}N_2O",                       "3033-62-3", "GTEXIOINCJRBIO-UHFFFAOYSA-N", "bis[2-(N,N-dimethylamino)ethyl] ether", "")
  ! C9:
  CALL Species("C_9H_{11}NO_2",                       "1129-41-5", "VOEYXMAFNDNNED-UHFFFAOYSA-N", "methylcarbamic acid, 3-methylphenyl ester", "metolcarb")
  CALL Species("C_9H_{11}NO_2",                        "101-99-5", "LBKPGNUOUPTQKA-UHFFFAOYSA-N", "phenylcarbamic acid, ethyl ester", "")
  CALL Species("C_9H_{11}NO_2",                         "87-25-2", "TWLLPUMZVVGILS-UHFFFAOYSA-N", "ethyl anthranilate", "")
  CALL Species("C_9H_{11}NO_2",                         "94-09-7", "BLFLLBZGZJTVJG-UHFFFAOYSA-N", "benzocaine", "")
  CALL Species("C_9H_{11}NO",                           "70-69-9", "FSWXOANXOQPCFF-UHFFFAOYSA-N", "1-(4-aminophenyl)-1-propanone", "4-aminopropiophenone")
  CALL Species("C_9H_{12}N_2O_2",                      "150-69-6", "GGLIEWRLXDLBBF-UHFFFAOYSA-N", "(4-ethoxyphenyl)urea", "dulcin")
  CALL Species("C_9H_{12}N_2O",                        "101-42-8", "XXOYNJXVWVNOOJ-UHFFFAOYSA-N", "fenuron", "")
  CALL Species("C_9H_{13}NO_3",                         "51-43-4", "UCTWMZQNUQWSLP-UHFFFAOYSA-N", "epinephrine", "")
  CALL Species("C_9H_{18}N_2O_4",                       "57-53-4", "NPPQSCRMBWNHMW-UHFFFAOYSA-N", "meprobamate", "")
  CALL Species("C_9H_{20}N_2O_2",                    "24579-73-5", "WZZLDXDUQPOXNW-UHFFFAOYSA-N", "propamocarb", "")
  ! C10:
  CALL Species("C_{10}H_{12}N_2O_2",                  "2828-42-4", "LATYTXGNKNKTDS-UHFFFAOYSA-N", "proximpham", "")
  CALL Species("C_{10}H_{13}NO_4",                    "6988-21-2", "SDKQRNRRDYRQKY-UHFFFAOYSA-N", "dioxacarb", "")
  CALL Species("C_{10}H_{13}NO_2",                    "2655-14-3", "CVQODEWAPZVVBU-UHFFFAOYSA-N", "phenol, 3,5-dimethyl-, methylcarbamate", "3,5-xylyl methylcarbamate")
  CALL Species("C_{10}H_{13}NO_2",                     "122-42-9", "VXPLXMJHHKHSOA-UHFFFAOYSA-N", "phenylcarbamic acid, 1-methylethyl ester", "")
  CALL Species("C_{10}H_{13}NO_2",                    "2425-10-7", "WCJYTPVNMWIZCG-UHFFFAOYSA-N", "xylylcarb", "")
  CALL Species("C_{10}H_{13}NO_2",                      "62-44-2", "CPJSUEIXXCENMM-UHFFFAOYSA-N", "phenacetin", "")
  CALL Species("C_{10}H_{15}NO",                       "299-42-3", "KWGRBVOPPLSCSI-PSASIEDQSA-N", "ephedrine", "")
  ! C11:
  CALL Species("C_{11}H_{15}NO_2",                      "64-00-6", "GYKXQTKSWLAUIT-UHFFFAOYSA-N", "$m$-cumenyl methylcarbamate", "3-isopropylphenyl methyl carbamate")
  CALL Species("C_{11}H_{15}NO_3",                     "114-26-1", "ISRUGXGCCGIOQO-UHFFFAOYSA-N", "propoxur", "")
  CALL Species("C_{11}H_{15}NO_5",                     "532-03-6", "GNXFOGHNGIVQEH-UHFFFAOYSA-N", "methocarbamol", "")
  CALL Species("C_{11}H_{16}N_2O_2",                  "2032-59-9", "IMIDOCRTMDIQIJ-UHFFFAOYSA-N", "aminocarb", "")
  CALL Species("C_{11}H_{16}N_2O",                   "34123-57-4", "DOULWWSSZVEPIN-UHFFFAOYSA-N", "monodesmethylisoproturon", "")
  CALL Species("C_{11}H_{22}N_2O",                    "2163-69-1", "DQZCVNGCTZLGAQ-UHFFFAOYSA-N", "cycluron", "")
  CALL Species("C_{11}H_{23}NO",                    "105726-67-8", "GELCOLZWXWHMIB-UHFFFAOYSA-N", "methylneodecanamide", "")
  CALL Species("C_{11}H_{15}NO_2",                    "2631-40-5", "QBSJMKIUCUGGNG-UHFFFAOYSA-N", "isoprocarb", "")
  ! C12:
  CALL Species("C_{12}H_{10}N_2O",                    "1689-82-3", "BEYOBVMPDRKTNR-UHFFFAOYSA-N", "4-(phenylazo)phenol", "4-hydroxyazobenzene")
  CALL Species("C_{12}H_{11}NO",                        "86-86-2", "XFNJVKMNNVCYEK-UHFFFAOYSA-N", "1-naphthaleneacetamide", "")
  CALL Species("C_{12}H_{16}N_2O_3",                 "16118-49-3", "AMRQXHFXNZFDCH-UHFFFAOYSA-N", "carbetamide", "")
  CALL Species("C_{12}H_{10}N_2O",                      "86-30-6", "UBUCNCOMADRQHX-UHFFFAOYSA-N", "N-nitrosodiphenylamine", "N,N-Diphenylnitrosamine")
  CALL Species("C_{12}H_{11}NO_2",                      "63-25-2", "CVXBEEMKQHEXEN-UHFFFAOYSA-N", "carbaryl", "")
  CALL Species("C_{12}H_{12}N_2O",                     "101-80-4", "HLBLWEWZXPIGSM-UHFFFAOYSA-N", "4,4'-oxybisbenzenamine", "bis(4-aminophenyl) ether")
  CALL Species("C_{12}H_{15}NO_3",                    "1563-66-2", "DUEPRVBVGDRKAG-UHFFFAOYSA-N", "carbofuran", "")
  CALL Species("C_{12}H_{16}NO_2",                    "2631-37-0", "DTAPQAJKAFRNJB-UHFFFAOYSA-N", "phenol, 3-methyl-5-(1-methylethyl)-, methylcarbamate", "promecarb")
  CALL Species("C_{12}H_{17}NO_2",                    "3766-81-2", "DIRFUJHNVNOBMY-UHFFFAOYSA-N", "fenobucarb", "")
  CALL Species("C_{12}H_{17}NO",                       "134-62-3", "MMOXZBCLCQITDF-UHFFFAOYSA-N", "diethyltoluamide", "DEET")
  CALL Species("C_{12}H_{18}N_2O",                   "34123-59-6", "PUIYMUZLKQOUOZ-UHFFFAOYSA-N", "N,N-dimethyl-N'-[4-(1-methylethyl)phenyl]-urea", "isoproturon")
  CALL Species("C_{12}H_{24}N_2O_3",                  "6425-39-4", "ZMSQJSMSLXVTKN-UHFFFAOYSA-N", "dimorpholinodiethyl ether", "")
  CALL Species("C_{12}H_{24}N_2O_4",                    "78-44-4", "OFZCIYFFPZCNJE-UHFFFAOYSA-N", "carisoprodol", "")
  CALL Species("C_{12}H_9N_3O_3",                     "1435-60-5", "NRJPVIOTANUINF-YPKPFQOOSA-N", "4-hydroxy-4'-nitroazobenzene", "")
  CALL Species("C_{12}H_9N_3O_4",                      "961-68-2", "RHTVQEPJVKUMPI-UHFFFAOYSA-N", "2,4-dinitro-N-phenyl-benzenamine", "")
  CALL Species("C_{12}H_9N_3O_5",                      "119-15-3", "BCPQALWAROJVLE-UHFFFAOYSA-N", "N-(2,4-dinitrophenyl)-N-(4-hydroxyphenyl)amine", "")
  CALL Species("C_{12}H_{10}N_4O_2",                   "730-40-5", "UNBOSJFEZZJZLR-UHFFFAOYSA-N", "disperse orange 3", "")
  ! C13:
  CALL Species("C_{13}H_{19}NO",                        "90-84-6", "XXEPPPIWZFICOJ-UHFFFAOYSA-N", "diethylpropion", "")
  CALL Species("C_{13}H_{21}NO_3",                   "18559-94-9", "NDAUXUAQIAJITI-UHFFFAOYSA-N", "salbutamol", "albuterol")
  ! C14:
  CALL Species("C_{14}H_{12}N_4O_2",                  "2475-45-8", "JSFUMBWFPQSADC-UHFFFAOYSA-N", "disperse blue 1", "")
  CALL Species("C_{14}H_{16}N_2O_2",                   "119-90-4", "JRBJSXQPQWSCCF-UHFFFAOYSA-N", "3,3'-dimethoxybenzidine", "")
  CALL Species("C_{14}H_{18}N_2O_5",                 "22839-47-0", "IAOZJIPTCAWIRG-QWRGUYRKSA-N", "aspartame", "")
  CALL Species("C_{14}H_{18}N_2O_7",                   "973-21-7", "HDWLUGYOLUHEMN-UHFFFAOYSA-N", "dinobuton", "dessin")
  CALL Species("C_{14}H_{20}N_2O",                    "1982-49-6", "JXVIIQLNUPXOII-UHFFFAOYSA-N", "N-(2-methylcyclohexyl)-N'-phenylurea", "siduron")
  CALL Species("C_{14}H_{21}N_3O_4",                 "33629-47-9", "SPNQRCTZKIBOAX-UHFFFAOYSA-N", "butralin", "")
  CALL Species("C_{14}H_{31}NO",                      "1643-20-5", "SYELZBGXAIXKHU-UHFFFAOYSA-N", "lauramine oxide", "")
  CALL Species("C_{14}H_9NO_2",                        "117-79-3", "XOGPDSATLSAZEK-UHFFFAOYSA-N", "2-aminoanthraquinone", "")
  CALL Species("C_{14}H_9NO_2",                         "82-45-1", "KHUFHLFHOQVFGB-UHFFFAOYSA-N", "1-amino-9,10-anthracenedione", "")
  CALL Species("C_{14}H_9NO_3",                        "116-85-8", "AQXYVFBSOOBBQV-UHFFFAOYSA-N", "1-amino-4-hydroxy-9,10-anthracenedione", "")
  CALL Species("C_{14}H_{10}N_2O_2",                   "128-95-0", "FBMQNRKSAWNXBT-UHFFFAOYSA-N", "1,4-diamino-9,10-anthracenedione", "")
  CALL Species("C_{14}H_{14}N_4O_2",                  "2491-74-9", "LSFRFLVWCKLQTO-UHFFFAOYSA-N", "N,N-dimethyl-4-[(4-nitrophenyl)azo]-benzenamine", "")
  CALL Species("C_{14}H_{21}N_3O_3",                  "4849-32-5", "OWNAXTAAAQTBSP-UHFFFAOYSA-N", "karbutilate", "")
  CALL Species("C_{14}H_{21}NO_4",                   "87130-20-9", "LNJNFVJKDJYTEU-UHFFFAOYSA-N", "diethofencarb", "")
  ! C15:
  CALL Species("C_{15}H_{11}NO_2",                      "82-28-0", "ZLCUIOWQYBYEBG-UHFFFAOYSA-N", "1-amino-2-methyl-9,10-anthracenedione", "1-amino-2-methylanthraquinone")
  CALL Species("C_{15}H_{13}NO",                        "53-96-3", "CZIHNRWJTSTCEX-UHFFFAOYSA-N", "N-2-fluorenylacetamide", "2-acetylaminofluorene")
  CALL Species("C_{15}H_{23}NO",                     "35256-85-0", "RJKCKKDSSSRYCB-UHFFFAOYSA-N", "tebutam", "")
  CALL Species("C_{15}H_{23}N_3O_4",                 "33820-53-0", "NEKOXWSIMFDGMA-UHFFFAOYSA-N", "isopropalin", "")
  CALL Species("C_{15}H_{33}N_3O_2",                  "2439-10-3", "YIKWKLYQRFRGPM-UHFFFAOYSA-N", "dodine", "doguadine")
  CALL Species("C_{15}H_{21}NO_4",                   "57837-19-1", "ZQEIXNIJLIKNTD-UHFFFAOYSA-N", "metalaxyl", "")
  CALL Species("C_{15}H_{21}NO_4",                   "70630-17-0", "ZQEIXNIJLIKNTD-GFCCVEGCSA-N", "metalaxyl-m", "")
  CALL Species("C_{15}H_{25}NO_3",                   "37350-58-6", "IUBSYMUCCVWXPE-UHFFFAOYSA-N", "metoprolol", "")
  CALL Species("C_{15}H_{11}NO_2",                      "82-38-2", "SVTDYSXXLJYUTM-UHFFFAOYSA-N", "1-(methylamino)anthraquinone", "")
  CALL Species("C_{15}H_{15}N_3O_2",                  "2832-40-8", "PXOZAFXVEWKXED-UHFFFAOYSA-N", "4'-[(2-hydroxy-5-methylphenyl)azo]acetanilide", "")
  ! C16:
  CALL Species("C_{16}H_{16}N_2O_4",                 "13684-56-5", "WZJZMXBKUWKXTQ-UHFFFAOYSA-N", "desmedipham", "")
  CALL Species("C_{16}H_{16}N_2O_4",                 "13684-63-4", "IDOWTHOLJBTAFI-UHFFFAOYSA-N", "phenmedipham", "betanal")
  CALL Species("C_{16}H_{17}NO",                       "957-51-7", "QAHFOPIILNICLA-UHFFFAOYSA-N", "fenam", "")
  CALL Species("C_{16}H_{18}N_2O_3",                 "14214-32-5", "AMVYOVYGIJXTQB-UHFFFAOYSA-N", "difenoxuron", "")
  CALL Species("C_{16}H_{25}NO_2",                    "2655-19-8", "SLZWBCGZQRRUNG-UHFFFAOYSA-N", "butacarb", "")
  CALL Species("C_{16}H_{28}N_2O_4",                "196618-13-0", "VSZGPKBBMSAYNT-RRFJBIMHSA-N", "oseltamivir", "")
  CALL Species("C_{16}H_{33}NO_3",                     "120-40-1", "AOMUHOFOVNGZAN-UHFFFAOYSA-N", "N,N-bis(2-hydroxyethyl)dodecanamide", "")
  CALL Species("C_{16}H_{14}N_2O_2",                  "2475-44-7", "QOSTVEDABRQTSU-UHFFFAOYSA-N", "1,4-bis(methylamino)-9,10-anthracenedione", "")
  CALL Species("C_{16}H_{16}N_2O_3",                "133408-50-1", "HIIRDDUVRXCDBN-SDXDJHTJSA-N", "metominostrobin", "")
  ! C17:
  CALL Species("C_{17}H_{14}N_2O_2",                  "1229-55-6", "ALLOLPOYFRLCCX-VHEBQXMUSA-N", "1-[(2-methoxyphenyl)azo]-2-naphthol", "")
  CALL Species("C_{17}H_{19}NO_4",                   "57646-30-7", "CIEXPHRYOLIQQD-UHFFFAOYSA-N", "furalaxyl", "")
  CALL Species("C_{17}H_{19}NO_4",                   "72490-01-8", "HJUFTIJOISQSKQ-UHFFFAOYSA-N", "fenoxycarb", "") ! other CAS: 79127-80-3
  CALL Species("C_{17}H_{20}N_2O_3",                "149877-41-8", "VHLKTXFWDRXILV-UHFFFAOYSA-N", "bifenazate", "")
  CALL Species("C_{17}H_{21}NO_2",                   "15299-99-7", "WXZVAROIGSFCFJ-UHFFFAOYSA-N", "napropamide", "")
  CALL Species("C_{17}H_{21}NO_2",                   "41643-35-0", "WXZVAROIGSFCFJ-CYBMUJFWSA-N", "napropamide-M", "") ! enantiomer mix = 15299-99-7
  CALL Species("C_{17}H_{27}NO_2",                   "21245-02-3", "WYWZRNAHINYAEF-UHFFFAOYSA-N", "padimate O", "")
  CALL Species("C_{17}H_{27}NO_4",                   "42200-33-9", "VWPOSFSPZNDTMJ-UHFFFAOYSA-N", "nadolol", "")
  CALL Species("C_{17}H_{29}NO",                        "88-27-7", "VMZVBRIIHDRYGK-UHFFFAOYSA-N", "2,6-di-{\it tert}-butyl-4-(dimethylaminomethyl)phenol", "")
  CALL Species("C_{17}H_{19}NO_2",                   "55814-41-0", "BCTQJXQXJVLSIG-UHFFFAOYSA-N", "mepronil", "")
  CALL Species("C_{17}H_{20}N_2O",                   "42609-52-9", "NNYRZQHKCHEXSD-UHFFFAOYSA-N", "daimuron", "")
  ! C18:
  CALL Species("C_{18}H_{13}NO_3",                     "132-66-1", "JXTHEWSKYLZVJC-UHFFFAOYSA-N", "naptalam", "")
  CALL Species("C_{18}H_{16}N_2O_3",                  "6358-53-8", "ULLJIFAIUSUJBP-ZZEZOPTASA-N", "citrus red 2", "")
  CALL Species("C_{18}H_{19}NO_4",                  "143390-89-0", "ZOTBXTZVPHCKPN-UHFFFAOYSA-N", "kresoxim-methyl", "")
  CALL Species("C_{18}H_{24}N_2O_6",                 "39300-45-3", "RNDNTTTUTQAXII-GORDUTHDSA-N", "dinocap", "")
  CALL Species("C_{18}H_{25}N_5O_5",                "248593-16-0", "JHIPUJPTQJYEQK-ZLHHXESBSA-N", "orysastrobin", "")
  CALL Species("C_{18}H_{35}NO_2",                  "118134-30-8", "PUYXTUJWRLOUCW-UHFFFAOYSA-N", "spiroxamine", "")
  CALL Species("C_{18}H_{27}NO_3",                     "404-86-4", "YKPUWZUDDOIDPM-SOFGYWHQSA-N", "capsaicin", "")
  ! C19:
  CALL Species("C_{19}H_{23}NO_3",                  "173662-97-0", "PDPWCKVFIFAQIQ-UHFFFAOYSA-N", "mandestrobin", "")
  CALL Species("C_{19}H_{22}N_2O_3",                "149961-52-4", "WXUZAHCNPWONDH-DYTRJAOYSA-N", "dimoxystrobin", "")
  CALL Species("C_{19}H_{17}NO_2",                   "52570-16-8", "LVKTWOXHRYGDMM-UHFFFAOYSA-N", "($RS$)-$\alpha$-2-naphthoxypropionanilide", "naproanilide")
  CALL Species("C_{19}H_{20}N_2O_2",                    "50-33-9", "VYMDGNCVAMGZFE-UHFFFAOYSA-N", "phenylbutazone", "")
  CALL Species("C_{19}H_{22}N_2O_4",                 "57375-63-0", "PWEOEHNGYFXZLI-UHFFFAOYSA-N", "phenisopham", "")
  CALL Species("C_{19}H_{24}N_2O_4",                 "73573-87-2", "BPZSYCZIITTYBL-ORAYPTAESA-N", "formoterol", "")
  ! C20:
  CALL Species("C_{20}H_{23}NO_3",                   "98243-83-5", "CJPQIRJHIZUAQP-MRXNPFEDSA-N", "benalaxyl-m", "")
  CALL Species("C_{20}H_{23}NO_3",                   "71626-11-4", "CJPQIRJHIZUAQP-UHFFFAOYSA-N", "benalaxyl", "")
  CALL Species("C_{20}H_{27}NO_3",                   "87820-88-0", "SOTLWPHEAQOHHC-UHFFFAOYSA-N", "tralkoxydim", "")
  CALL Species("C_{20}H_{30}N_2O_5",                "165450-17-9", "HLIAVLHNDJUHFG-HOTGVXAUSA-N", "neotame", "")
  ! C22:
  CALL Species("C_{22}H_{25}NO_6",                      "64-86-8", "IAKHMKGGTNLKSZ-MRXNPFEDSA-N", "colchicine", "")
  CALL Species("C_{22}H_{28}N_2O_2",                "112410-23-8", "QYPNKSZPJQQLRK-UHFFFAOYSA-N", "tebufenozide", "")
  CALL Species("C_{22}H_{28}N_2O_3",                "161050-58-4", "QCAWEPFNJXQPAN-UHFFFAOYSA-N", "methoxyfenozide", "")
  CALL Species("C_{22}H_{29}NO_2",                     "469-62-5", "XLMALTXPSGQGBX-PGRDOPGGSA-N", "propoxyphene", "")
  CALL Species("C_{22}H_{43}NO",                       "112-84-5", "UAUDZVJPLUQNMU-KTKRTIGZSA-N", "($Z$)-13-docosenamide", "erucamide")
  ! >= C24:
  CALL Species("C_{24}H_{33}NO_4",                  "138164-12-2", "ZOGDSYNXUXQGHF-XIEYBQDHSA-N", "butroxydim", "")
  CALL Species("C_{27}H_{26}N_2O_2",                 "13080-86-9", "KMKWGXGSGPYISJ-UHFFFAOYSA-N", "2,2-bis[4-(4-aminophenoxy)phenyl]propane", "")
  CALL Species("C_{28}H_{22}N_2O_2",                   "128-80-3", "TVRGPOFMYCMNRB-UHFFFAOYSA-N", "1,4-bis[(4-methylphenyl)amino]-9,10-anthracenedione", "D\&C Green No. 6")
  CALL Species("C_{29}H_{35}NO_2",                   "84371-65-3", "VKHAHZOOUSRJNA-GCNJZUOMSA-N", "mifepristone", "")
  CALL Species("C_{34}H_{34}N_2O_3",                 "70516-41-5", "HUSIBQLZEMMTCQ-UHFFFAOYSA-N", "2'-anilino-6'-[ethyl(3-methylbutyl)amino]-3'-methylspiro[isobenzofuran-1(3H),9'-[9H]xanthene]-3-one", "")
  CALL Species("C_{42}H_{69}NO_{10}",               "935545-74-7", "GOENIMGKWNZVDA-OAMCMWGQSA-N", "spinetoram", "")
  CALL Species("C_{42}H_{71}NO_9",                  "168316-95-8", "RQOIAWYOVOXMST-UHFFFAOYSA-N", "spinosad", "")
  CALL Species("C_{56}H_{81}NO_{15}",               "155569-91-8", "GCKZANITAMOIAR-UHFFFAOYSA-N", "emamectin benzoate", "") ! other CAS: 119791-41-2
  ! amino acids:
  CALL Species("C_5H_9NO_4",                           "617-65-2", "WHUUTDBJXJRKMK-UHFFFAOYSA-N", "glutamic acid", "")
  CALL Species("C_4H_8N_2O_3",                          "70-47-3", "DCXYFEDJOCDNAF-UWTATZPHSA-N", "asparagine", "")
  CALL Species("C_3H_7NO_3",                           "302-84-1", "MTCFGRXMJLQNBG-UHFFFAOYSA-N", "serine", "")
  CALL Species("C_5H_{10}N_2O_3",                       "56-85-9", "ZDXPYRJPNDTMRX-GSVOUGTGSA-N", "glutamine", "")
  CALL Species("C_2H_5NO_2",                            "56-40-6", "DHMQDGOQFOQNFH-UHFFFAOYSA-N", "glycine", "")
  CALL Species("C_6H_{14}N_4O_2",                       "74-79-3", "ODKSFYDXXFIFQN-SCSAIBSYSA-N", "arginine", "")
  CALL Species("C_3H_7NO_2",                           "302-72-7", "QNAYBMKLOCPYGJ-UHFFFAOYSA-N", "alanine", "")
  CALL Species("C_6H_{13}NO_2",                        "328-39-2", "ROHFNLRQFUQHCH-UHFFFAOYSA-N", "leucine", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Heterocycles with oxygen and nitrogen (\chem{C}, \chem{H}, \chem{O}, \chem{N})")
  CALL Species("C_3H_3N_3O_3",                         "108-80-5", "ZFSLODLOARCGLH-UHFFFAOYSA-N", "cyanuric acid", "")
  CALL Species("C_3H_3NO",                             "288-14-2", "CTAPFRYPJLPFDF-UHFFFAOYSA-N", "isoxazole", "")
  CALL Species("C_3H_5NO_2",                          "5694-00-8", "FMAZQSYXRGRESX-UHFFFAOYSA-N", "glycidamide", "")
  CALL Species("C_3H_6N_6O_6",                         "121-82-4", "XTFIVUDBNACUBN-UHFFFAOYSA-N", "cyclonite", "")
  CALL Species("C_4H_5NO_2",                         "10004-44-1", "KGVPNLBXJKTABS-UHFFFAOYSA-N", "5-methyl-3-(2H)-isoxazolone", "hymexazol")
  CALL Species("C_4H_5N_3O",                         "10071-13-3", "MMZLICVOTDAZOX-UHFFFAOYSA-N", "3-amino-1H-pyridazin-6-one", "maleic hydrazide")
  CALL Species("C_4H_4N_2O_2",                         "123-33-1", "BGRDGMRNKXEXQD-UHFFFAOYSA-N", "maleic hydrazide", "")
  CALL Species("C_4H_6N_4O_3",                          "97-59-6", "POJWUDADGALRAB-UHFFFAOYSA-N", "allantoin", "")
  CALL Species("C_4H_7NO",                             "616-45-5", "HNJBEVLQSNELDL-UHFFFAOYSA-N", "2-pyrrolidinone", "")
  CALL Species("C_4H_8N_2O_2",                          "59-89-2", "ZKXDGKXYMTYWTB-UHFFFAOYSA-N", "4-nitrosomorpholine", "")
  CALL Species("C_4H_8N_2O",                           "930-55-2", "WNYADZVDBIBLJJ-UHFFFAOYSA-N", "N-nitrosopyrrolidine", "")
  CALL Species("C_4H_8N_8O_8",                        "2691-41-0", "UZGLIIJVICEWHF-UHFFFAOYSA-N", "cyclotetramethylenetetranitramine", "")
  CALL Species("C_4H_9NO",                             "110-91-8", "YNAVUWVOSKDBBP-UHFFFAOYSA-N", "1-oxa-4-azacyclohexane", "morpholine")
  CALL Species("C_4H_9NO",                            "1072-52-2", "VYONOYYDEFODAJ-UHFFFAOYSA-N", "1-aziridineethanol", "")
  CALL Species("C_4N_2H_3(C_2H_5)OCH_3",             "25680-58-4", "DPCILIMHENXHQX-UHFFFAOYSA-N", "2-ethyl-3-methoxypyrazine", "")
  CALL Species("C_4N_2H_3(C_4H_9)OCH_3",             "24683-00-9", "UXFSPRAGHGMRSQ-UHFFFAOYSA-N", "2-isobutyl-3-methoxypyrazine", "")
  CALL Species("C_5H_{10}N_2O",                        "100-75-4", "UWSDONTXWQOZFN-UHFFFAOYSA-N", "N-nitrosopiperidine", "")
  CALL Species("C_5H_{11}NO_2",                        "592-35-8", "SKKTUOZKZKCGTB-UHFFFAOYSA-N", "butyl carbamate", "")
  CALL Species("C_5H_{11}NO",                          "109-02-4", "SJRJJKPEHAURKC-UHFFFAOYSA-N", "4-methyl-1-oxa-4-azacyclohexane", "N-methylmorpholine; 4-methylmorpholine")
  CALL Species("C_5H_4N_4O",                           "315-30-0", "OFCNXPDARWKPPY-UHFFFAOYSA-N", "allopurinol", "")
  CALL Species("C_5H_4NOCH_3",                         "620-08-6", "XQABVLBGNWBWIV-UHFFFAOYSA-N", "4-methoxypyridine", "")
  CALL Species("C_5H_9NO",                             "872-50-4", "SECXISVLQFMRJM-UHFFFAOYSA-N", "N-methyl-2-pyrrolidone", "")
  CALL Species("C_5H_8N_2O_2",                          "77-71-4", "YIROYDNZEPTFOL-UHFFFAOYSA-N", "5,5-dimethyl-2,4-imidazolidinedione", "")
  CALL Species("C_5H_6N_2O_2",                         "615-77-0", "XBCXJKGHPABGSD-UHFFFAOYSA-N", "1-methyluracil", "")
  CALL Species("C_6H_{11}NO",                          "105-60-2", "JBKVHLHDHHXQEQ-UHFFFAOYSA-N", "2-azacycloheptanone", "caprolactam")
  CALL Species("C_6H_{11}NO",                         "4030-18-6", "LNWWQYYLZVZXKS-UHFFFAOYSA-N", "N-acetylpyrrolidine", "")
  CALL Species("C_6H_{13}NO_5",                       "3416-24-8", "MSWZFWKMSRAUBD-IVMDWMLBSA-N", "glucosamine", "")
  CALL Species("C_6H_{13}NO",                          "100-74-3", "HVCNXQOWACZAFN-UHFFFAOYSA-N", "N-ethylmorpholine", "")
  CALL Species("C_6H_5NO",                             "500-22-1", "QJZUKDFHGGYHMC-UHFFFAOYSA-N", "3-formylpyridine", "")
  CALL Species("C_6H_5NO",                             "872-85-5", "BGUWFUQJCDRPTL-UHFFFAOYSA-N", "4-formylpyridine", "")
  CALL Species("C_6H_6N_2O",                            "98-92-0", "DFPAKSUCGFBDDF-UHFFFAOYSA-N", "niacinamide", "")
  CALL Species("C_6H_9N_3O_3",                         "443-48-1", "VAOCPAMSLUNLGC-UHFFFAOYSA-N", "metronidazole", "")
  CALL Species("C_6H_5NO_2",                            "59-67-6", "PVNIIMVLHYAWGP-UHFFFAOYSA-N", "nicotinic acid", "")
  CALL Species("C_6H_5NO_2",                            "55-22-1", "TWBYWOBDOCUKOW-UHFFFAOYSA-N", "isonicotinic acid", "")
  CALL Species("C_6H_8N_2O_2",                        "4160-72-9", "GKMIDMKPBOUSBQ-UHFFFAOYSA-N", "1-methylthymine", "")
  CALL Species("C_7H_{12}N_2O_4",                     "6440-58-0", "WSDISUOETYTPRL-UHFFFAOYSA-N", "glydant", "1,3-dimethylol-5,5-dimethylhydantoin")
  CALL Species("C_7H_{13}NO",                         "1619-34-7", "IVLICPVPXWEGCA-UHFFFAOYSA-N", "3-quinuclidinol", "")
  CALL Species("C_7H_{14}N_4O_3",                   "165252-70-0", "YKBZOVFACRVRJN-UHFFFAOYSA-N", "dinotefuran", "")
  CALL Species("C_7H_5N_3O",                            "90-16-4", "DMSSTTLDFWKBSX-UHFFFAOYSA-N", "1,2,3-benzotriazin-4(1H)-one", "")
  CALL Species("C_7H_7NO",                            "1122-54-9", "WMQUKDQWMMOHSA-UHFFFAOYSA-N", "4-acetylpyridine", "")
  CALL Species("C_7H_7NO",                             "350-03-8", "WEGYGNROSJDEIW-UHFFFAOYSA-N", "3-acetylpyridine", "")
  CALL Species("C_7H_8N_4O_2",                          "58-55-9", "ZFXYFBGIUFBOJW-UHFFFAOYSA-N", "theophylline", "")
  CALL Species("C_7H_8N_4O_2",                          "83-67-0", "YAPQBXQYLJRXSA-UHFFFAOYSA-N", "theobromine", "")
  CALL Species("C_7H_9NO",                             "103-74-2", "BXGYBSJAZFGIPX-UHFFFAOYSA-N", "2-pyridineethanol", "")
  CALL Species("C_8H_{10}N_4O_2",                       "58-08-2", "RYYVLZVUVIJVGH-UHFFFAOYSA-N", "caffeine", "")
  CALL Species("C_8H_{11}N_5O_3",                    "59277-89-3", "MKUXAQIIEYXACX-UHFFFAOYSA-N", "acyclovir", "")
  CALL Species("C_8H_{12}N_2O",                      "25773-40-4", "NTOPKICPEQUPPH-UHFFFAOYSA-N", "2-methoxy-3-(1-methylethyl)-pyrazine", "")
  CALL Species("C_8H_{15}N_5O",                        "673-04-1", "HKAMKLBXTLTVCN-UHFFFAOYSA-N", "simeton", "")
  CALL Species("C_8H_{17}NO",                        "10315-98-7", "QKVSMSABRNCNRS-UHFFFAOYSA-N", "N-isobutylmorpholine", "")
  CALL Species("C_8H_5NO_2",                            "85-41-6", "XKJCHHZQLQNZHY-UHFFFAOYSA-N", "phthalimide", "")
  CALL Species("C_8H_7N_3O_5",                          "67-45-8", "PLHJDBGFXBMTGZ-UHFFFAOYSA-N", "furazolidone", "")
  CALL Species("C_8H_9NO_2",                            "85-40-5", "CIFFBTOJCKSRJY-UHFFFAOYSA-N", "1,2,3,6-tetrahydrophthalimide", "")
  CALL Species("C_9H_{11}N_3O",                      "16543-55-8", "XKABJYQDMJTNGQ-UHFFFAOYSA-N", "N-nitrosonornicotine", "")
  CALL Species("C_9H_{13}N_5O_4",                    "82410-32-0", "IRSCQMHQWWYFCW-UHFFFAOYSA-N", "9-[(1,3-dihydroxy-2-propoxy)methyl]guanine", "ganciclovir")
  CALL Species("C_9H_{14}N_2O",                      "24168-70-5", "QMQDJVIJVPEQHE-UHFFFAOYSA-N", "2-{\it sec}-butyl-3-methoxypyrazine", "")
  CALL Species("C_9H_{17}N_5O",                       "1610-17-9", "PXWUKZGIHQRDHL-UHFFFAOYSA-N", "atraton", "")
  CALL Species("C_9H_{18}NO_2",                       "2226-96-2", "UZFMOKQJFYMBGY-UHFFFAOYSA-N", "4-hydroxy-2,2,6,6-tetramethyl-1-piperidinyloxy", "")
  CALL Species("C_9H_7NO",                             "148-24-3", "MCJGNVYPOGVAJF-UHFFFAOYSA-N", "8-hydroxyquinoline", "")
  CALL Species("C_9H_9N_3O_2",                       "10605-21-7", "TWFZGCMQGLPBSX-UHFFFAOYSA-N", "carbendazim", "")
  CALL Species("C_{10}H_{10}N_4O",                   "41394-05-2", "VHCNQEUWZYOAEV-UHFFFAOYSA-N", "metamitron", "")
  CALL Species("C_{10}H_{11}N_5O",                  "123312-89-0", "QHMTXANCGGJZRX-UHFFFAOYSA-N", "pymetrozin", "")
  CALL Species("C_{10}H_{11}NO_2",                     "102-01-2", "DYRDKSSFIWVSNM-UHFFFAOYSA-N", "3-oxo-N-phenylbutanamide", "acetoacetanilide")
  CALL Species("C_{10}H_{12}N_2O_4",                  "3056-17-5", "XNKLLVCARDGLGL-JGVFFNPUSA-N", "2,3'-didehydro-3'-deoxythymidine (stavudine)", "stavudine")
  CALL Species("C_{10}H_{12}N_2O",                     "486-56-6", "UIKROCXWUNQSPJ-UHFFFAOYSA-N", "cotinine", "")
  CALL Species("C_{10}H_{13}N_3O_2",                 "64091-91-4", "FLAQQSHRLBFIEZ-UHFFFAOYSA-N", "4-(N-nitroso-N-methylamino)-1-(3-pyridyl)-1-butanone", "")
  CALL Species("C_{10}H_{15}N_5O_3",                 "39809-25-1", "JNTOCHDNEULJHD-UHFFFAOYSA-N", "9-(4-hydroxy-3-hydroxymethylbut-1-yl)guanine", "penciclovir")
  CALL Species("C_{10}H_{15}NO",                     "64285-06-9", "SGNXVBOIDPPRJJ-UHFFFAOYSA-N", "anatoxin A", "")
  CALL Species("C_{10}H_{16}N_4O_3",                   "644-64-4", "RDBIYWSVMRVKSG-UHFFFAOYSA-N", "dimetilan", "")
  CALL Species("C_{10}H_{17}N_3O_2",                   "119-38-0", "RNNBHZYEKNHLKT-UHFFFAOYSA-N", "isolan", "")
  CALL Species("C_{10}H_{19}N_5O_2",                "129909-90-6", "ORFPWVRKFLOQHK-UHFFFAOYSA-N", "amicarbazone", "")
  CALL Species("C_{10}H_{19}N_5O",                    "1610-18-0", "ISEUFVQQFVOBCY-UHFFFAOYSA-N", "prometone", "prometon")
  CALL Species("C_{10}H_{19}N_5O",                   "26259-45-0", "ZJMZZNVGNSWOOM-UHFFFAOYSA-N", "secbumeton", "")
  CALL Species("C_{10}H_{19}N_5O",                   "33693-04-8", "BCQMBFHBDZVHKU-UHFFFAOYSA-N", "terbumeton", "")
  CALL Species("C_{10}H_9N_5O",                        "525-79-1", "QANMHLXAZMSUEX-UHFFFAOYSA-N", "kinetin", "")
  CALL Species("C_{10}H_{17}N_3O_2",                 "55861-78-4", "JLLJHQLUZAKJFH-UHFFFAOYSA-N", "isouron", "")
  CALL Species("C_{11}H_8N_2O",                       "3878-19-1", "UYJUZNLFJAWNEZ-UHFFFAOYSA-N", "fuberidazole", "")
  CALL Species("C_{11}H_{10}N_4O_4",                  "6804-07-5", "OVGGLBAWFMIPPY-WUXMJOGZSA-N", "carbadox", "")
  CALL Species("C_{11}H_{13}NO_4",                   "22781-23-3", "XEGGRYVFLWGFHI-UHFFFAOYSA-N", "bendiocarb", "")
  CALL Species("C_{11}H_{15}NO_2",                    "2655-15-4", "NYOKZHDTNBDPOB-UHFFFAOYSA-N", "2,3,5-trimethylphenol, methylcarbamate", "2,3,5-trimethacarb")
  CALL Species("C_{11}H_{16}N_2O_3",                    "77-26-9", "UZVHFVZFNXBMQJ-UHFFFAOYSA-N", "butalbital", "")
  CALL Species("C_{11}H_{16}N_4O_4",                 "24584-09-6", "BMKDZUISNHGIBY-ZETCQYMHSA-N", "dexrazoxane", "")
  CALL Species("C_{11}H_{18}N_2O_3",                    "76-74-4", "WEXRUCMBJFQVBZ-UHFFFAOYSA-N", "pentobarbital", "")
  CALL Species("C_{11}H_{18}N_4O_2",                 "23103-98-2", "YFGYUFNIOHWBOB-UHFFFAOYSA-N", "pirimor", "pirimicarb")
  CALL Species("C_{11}H_{19}N_3O",                   "23947-60-6", "BBXXLROWFHWFQY-UHFFFAOYSA-N", "ethirimol", "")
  CALL Species("C_{11}H_{11}NO",                     "57369-32-1", "XRJLAOUDSILTFT-UHFFFAOYSA-N", "pyroquilon", "")
  CALL Species("C_{12}H_{11}NO_2",                   "24691-80-3", "JFSPBVWPKOEZCB-UHFFFAOYSA-N", "fenfuram", "")
  CALL Species("C_{12}H_{12}N_2O_3",                 "74782-23-3", "WFVUIONFJOAYPK-UHFFFAOYSA-N", "oxabetrinil", "")
  CALL Species("C_{12}H_{12}N_2O_3",                    "50-06-6", "DDBREPKUVSBGFI-UHFFFAOYSA-N", "phenobarbital", "")
  CALL Species("C_{12}H_{13}N_3O_2",                    "68-76-8", "PXSOHRWMIRDKMP-UHFFFAOYSA-N", "triaziquone", "")
  CALL Species("C_{12}H_{15}N_3O_3",                   "101-37-1", "BJELTSYBAHKXRW-UHFFFAOYSA-N", "triallyl cyanurate", "")
  CALL Species("C_{12}H_{15}N_5O_3",                "142217-69-4", "QDGZDCVAUDNJFG-FXQIFTODSA-N", "entecavir", "")
  CALL Species("C_{12}H_{15}NO_3",                    "1665-48-1", "IMWZZHHPURKASS-UHFFFAOYSA-N", "metaxalone", "")
  CALL Species("C_{12}H_{17}NO",                       "634-03-7", "MFOCDFTXLCYLKU-UHFFFAOYSA-N", "phendimetrazine", "")
  CALL Species("C_{12}H_{20}N_4O_2",                 "51235-04-2", "CAWXEEYDBZRFPE-UHFFFAOYSA-N", "hexazinone", "")
  CALL Species("C_{12}H_{23}NO_3",                  "119515-38-7", "QLHULAHOXSSASE-UHFFFAOYSA-N", "picaridin", "")
  CALL Species("C_{13}H_{18}N_2O_2",                  "2164-08-1", "ZTMKADLOSYKWCA-UHFFFAOYSA-N", "lenacil", "")
  CALL Species("C_{13}H_{12}N_4O_3",                 "53558-25-1", "CLKZWXHKFXZIMA-UHFFFAOYSA-N", "pyrinuron", "pyriminil")
  CALL Species("C_{13}H_{16}N_2O_2",                    "73-31-4", "DRLFMBDRBRZALE-UHFFFAOYSA-N", "melatonin", "")
  CALL Species("C_{13}H_9NO",                          "257-07-8", "NPUACKRELIJTFM-UHFFFAOYSA-N", "dibenz[$b,f$]][1,4]oxazepine", "")
  CALL Species("C_{14}H_{18}N_2O_4",                 "77732-09-3", "UWVQIROCRJWDKL-UHFFFAOYSA-N", "oxadixyl", "")
  CALL Species("C_{14}H_{18}N_4O_3",                 "17804-35-2", "RIOXQFHNBCKOKP-UHFFFAOYSA-N", "benomyl", "")
  CALL Species("C_{14}H_{18}N_4O_3",                   "738-70-5", "IEDVJHCEMCRBQM-UHFFFAOYSA-N", "trimethoprim", "")
  CALL Species("C_{14}H_{19}NO",                        "91-53-2", "DECIPOUIJURFOJ-UHFFFAOYSA-N", "ethoxyquin", "")
  CALL Species("C_{14}H_{19}N_5O_4",                "104227-87-4", "GGXKWVWZWMLJEH-UHFFFAOYSA-N", "famciclovir", "")
  CALL Species("C_{14}H_{21}NO_3",                   "60568-05-0", "QTDRLOKFLJJHTG-UHFFFAOYSA-N", "furmecyclox", "")
  CALL Species("C_{15}H_{12}N_2O_2",                 "28721-07-5", "CTRLABGOLIVAIY-UHFFFAOYSA-N", "oxcarbazepine", "")
  CALL Species("C_{15}H_{12}N_2O_2",                    "57-41-0", "CXOFVDLJLONNDW-UHFFFAOYSA-N", "phenytoin", "")
  CALL Species("C_{15}H_{12}N_2O",                     "298-46-4", "FFGPTBGBLSHEPO-UHFFFAOYSA-N", "carbamazepine", "")
  CALL Species("C_{15}H_{12}N_2O",                      "51-52-5", "KNAHARQHSZJURB-UHFFFAOYSA-N", "propylthiouracil", "")
  CALL Species("C_{15}H_{16}N_2O_2",                 "12771-68-5", "HUTDUHSNJYTCAR-UHFFFAOYSA-N", "ancymidol", "")
  CALL Species("C_{15}H_{19}N_3O_3",                 "81335-77-5", "XVOKUMIPKHGGTN-UHFFFAOYSA-N", "imazethapyr", "")
  CALL Species("C_{15}H_{19}N_3O_4",                "114311-32-9", "NUPJIGQFXCQJBK-UHFFFAOYSA-N", "imazamox", "")
  CALL Species("C_{15}H_{23}NO_4",                      "66-81-9", "YPHMISFOHDHNIV-FSZOTQKASA-N", "cycloheximide", "")
  CALL Species("C_{15}H_{24}N_2O_2",                 "16837-52-8", "XVPBINOPNYFXID-LHDUFFHYSA-N", "oxymatrine", "")
  CALL Species("C_{15}H_{25}N_3O",                   "76608-88-3", "CNFMJLVJDNGPHR-UKTHLTGXSA-N", "triapenthenol", "")
  CALL Species("C_{16}H_{13}N_3O_3",                 "31431-39-7", "OPXLLQIJSORQAM-UHFFFAOYSA-N", "mebendazole", "")
  CALL Species("C_{16}H_{20}N_2O_3",                 "81405-85-8", "FFCCBBNQPIMUJI-UHFFFAOYSA-N", "imazamethabenz-methyl", "")
  CALL Species("C_{17}H_{18}N_2O_6",                 "21829-25-4", "HYIMSNHJOBLJNT-UHFFFAOYSA-N", "nifedipine", "")
  CALL Species("C_{17}H_{19}NO_4",                      "76-41-5", "UQCNKQCJZOAFTQ-UHFFFAOYSA-N", "oxymorphone", "")
  CALL Species("C_{17}H_{21}NO_2",                     "427-00-9", "LNNWVNGFPYWNQE-UHFFFAOYSA-N", "desomorphine", "")
  CALL Species("C_{17}H_{21}NO_4",                      "50-36-2", "ZPUCINDJVBIVPJ-PFSRBDOWSA-N", "cocaine", "")
  CALL Species("C_{17}H_{25}NO_2",                     "113-48-4", "WLLGXSLBOPFWQV-UHFFFAOYSA-N", "N-(2-ethylhexyl)-5-norbornene-2,3-dicarboximide", "")
  CALL Species("C_{17}H_{17}N_3O_3",                 "81335-37-7", "CABMTIJINOIHOD-UHFFFAOYSA-N", "imazaquin", "")
  CALL Species("C_{17}H_{19}N_3O_6",                "147411-69-6", "USSIUIGPBLPCDF-KEBDBYFISA-N", "($E$)-pyriminobac-methyl", "")
  CALL Species("C_{17}H_{19}N_3O_6",                "147411-70-9", "USSIUIGPBLPCDF-JMIUGGIZSA-N", "($Z$)-pyriminobac-methyl", "")
  CALL Species("C_{17}H_{22}N_2O_4",                 "72963-72-5", "VPRAQYXPZIFIOH-UHFFFAOYSA-N", "imiprothrin", "")
  CALL Species("C_{18}H_{28}N_2O_3",                "140923-17-7", "NWUWYYSKZYIQAE-LBAUFKAWSA-N", "iprovalicarb", "")
  CALL Species("C_{18}H_{11}NO_2",                    "8003-22-3", "IZMJMCDDWKSTTK-UHFFFAOYSA-N", "quinophthalone", "")
  CALL Species("C_{18}H_{23}N_3O_4",                "101903-30-4", "WBTYBAGIHOISOQ-UHFFFAOYSA-N", "pefurazoate", "")
  CALL Species("C_{19}H_{25}NO_4",                    "7696-12-0", "CXBMCYHAMVGWJQ-UHFFFAOYSA-N", "tetramethrin", "")
  CALL Species("C_{19}H_{25}NO_4",                   "82558-50-7", "PMHURSZHKKJGBM-UHFFFAOYSA-N", "isoxaben", "")
  CALL Species("C_{19}H_{27}N_5O_4",                 "81403-80-7", "WNMJYKCGWZFFKR-UHFFFAOYSA-N", "alfuzosin", "")
  CALL Species("C_{19}H_{39}NO",                     "24602-86-6", "YTOPFCCWCSOHFV-UHFFFAOYSA-N", "2,6-dimethyl-4-tridecylmorpholine", "tridemorph")
  CALL Species("C_{20}H_{19}NO_3",                   "95737-68-1", "NHDHVHZZCFYRSB-UHFFFAOYSA-N", "pyriproxyfen", "")
  CALL Species("C_{20}H_{21}NO_4",                      "58-74-2", "XQYZDYMELSJDRZ-UHFFFAOYSA-N", "papaverine", "")
  CALL Species("C_{20}H_{22}N_2O",                  "120928-09-8", "DMYHGDXADUDKCQ-UHFFFAOYSA-N", "fenazaquin", "")
  CALL Species("C_{20}H_{23}N_3O_2",                 "55179-31-2", "VGPIBGGRCVEHQZ-UHFFFAOYSA-N", "bitertanol", "")
  CALL Species("C_{20}H_{23}N_3O_2",                 "70585-36-3", "VGPIBGGRCVEHQZ-OALUTQOASA-N", "bitertanol diastereoisomer a", "")
  CALL Species("C_{20}H_{23}N_3O_2",                 "70585-38-5", "VGPIBGGRCVEHQZ-RBUKOAKNSA-N", "bitertanol diastereoisomer b", "")
  CALL Species("C_{20}H_{23}NO_4",                   "16590-41-3", "DQCKKXVULJGBQN-UHFFFAOYSA-N", "naltrexone", "")
  CALL Species("C_{20}H_{25}N_3O",                      "50-37-3", "VAYOSLLFUXYJDT-UHFFFAOYSA-N", "$D$-lysergic acid N,N-diethylamide", "LSD")
  CALL Species("C_{20}H_{26}N_2O",                      "83-74-9", "HSIBGVUMFOSJPD-UHFFFAOYSA-N", "ibogaine", "")
  CALL Species("C_{20}H_{33}NO",                     "67564-91-4", "RYAUSSKQMZRMAI-ALOPSCKCSA-N", "fenpropimorph", "")
  CALL Species("C_{21}H_{22}N_2O_2",                    "57-24-9", "QMGVPVSNSZLJIA-UHFFFAOYSA-N", "strychnine", "")
  CALL Species("C_{21}H_{25}NO_3",                   "55096-26-9", "WJBLNOPPDWQMCH-UHFFFAOYSA-N", "nalmefene", "")
  CALL Species("C_{21}H_{25}NO",                        "86-13-5", "GIJXKZJWITVLHI-UHFFFAOYSA-N", "benztropine", "")
  CALL Species("C_{21}H_{32}N_2O",                   "10418-03-8", "LKAJKIOFIWVMDJ-KIWJEFSTSA-N", "stanozolol", "")
  CALL Species("C_{21}H_{27}NO_5",                  "203313-25-1", "CLSVJBIHYWPGQY-GGYDESQDSA-N", "spirotetramat", "")
  CALL Species("C_{21}H_{23}NO_5",                     "561-27-3", "GVGLGOZIDCSQPN-PVHGPHFFSA-N", "diacetylmorphine", "heroin")
  CALL Species("C_{21}H_{23}N_3O_4",                "915410-70-7", "DWTVBEZBWMDXIY-UHFFFAOYSA-N", "pyrametostrobin", "")
  CALL Species("C_{22}H_{17}N_3O_5",                "131860-33-8", "WFDXOXNFNRHQEC-GHRIWEEISA-N", "azoxystrobin", "")
  CALL Species("C_{22}H_{18}N_2O_4",                "131807-57-3", "PCCSBWNGDMYFCW-UHFFFAOYSA-N", "famoxadone", "")
  CALL Species("C_{22}H_{19}N_3O_4",                "171596-29-5", "WOXKDUGGOYFFRN-IIBYNOLFSA-N", "tadalafil", "")
  CALL Species("C_{22}H_{19}NO_4",                     "603-50-9", "KHOITXIGCFIULA-UHFFFAOYSA-N", "bisacodyl", "")
  CALL Species("C_{22}H_{28}N_2O",                     "437-38-7", "PJMPHNIQZUBGLI-UHFFFAOYSA-N", "fentanyl", "")
  CALL Species("C_{23}H_{23}NO",                      "1420-06-0", "ZJMLMBICUVVJDX-UHFFFAOYSA-N", "4-(triphenylmethyl)morpholine", "trifenmorph")
  CALL Species("C_{23}H_{26}N_2O_4",                   "357-57-3", "RRKTZKIUPZVBMF-UHFFFAOYSA-N", "brucine", "")
  CALL Species("C_{23}H_{31}NO_7",                  "128794-94-5", "RTGDFNSFWBGLEC-SYZQJQIISA-N", "mycophenolate mofetil", "")
  CALL Species("C_{23}H_{32}N_2O_4",                "243973-20-8", "MGOHCFMYLBAPRN-UHFFFAOYSA-N", "pinoxaden", "")
  CALL Species("C_{24}H_{27}N_3O_4",                "134098-61-6", "YYJNOYZRYGDPNH-MFKUBSTISA-N", "fenpyroximate", "")
  CALL Species("C_{24}H_{29}N_5O_3",                "137862-53-4", "ACWBQPMHZXGDFX-QFIPXVFZSA-N", "valsartan", "")
  CALL Species("C_{24}H_{29}NO_3",                  "120014-06-4", "ADEBPBSSDYVVLD-UHFFFAOYSA-N", "donepezil", "")
  CALL Species("C_{24}H_{30}N_2O_3",                "143807-66-3", "HPNSNYBUADCFDR-UHFFFAOYSA-N", "chromafenozide", "")
  CALL Species("C_{24}H_{31}N_3O_2",                "560121-52-0", "APJLTUBHYCOZJI-VZCXRCSSSA-N", "cyenopyrafen", "")
  CALL Species("C_{29}H_{20}N_2O_2",                  "5242-49-9", "SOTPOQQKAUOHRO-BQYQJAHWSA-N", "2-[4-[4-(2-benzoxazolyl)styryl]phenyl]-5-methylbenzoxazole", "")
  CALL Species("C_{30}H_{29}N_3O",                   "70321-86-7", "OLFNXLXEGXRUOI-UHFFFAOYSA-N", "2-(2H-benzotriazol-2-yl)-4,6-bis(1-methyl-1-phenylethyl)phenol", "")
  CALL Species("C_{31}H_{38}N_2O_{11}",             "517875-34-2", "QGTOTYJSCYHYFK-RBODFLQRSA-N", "fenpicoxamid", "")
  CALL Species("C_{33}H_{25}N_3O_3",                   "991-42-4", "DNTHHIVFNQZZRD-UHFFFAOYSA-N", "norbormide", "")
  CALL Species("C_{33}H_{40}N_2O_9",                    "50-55-5", "QEVHRUUCFGRFIF-UHFFFAOYSA-N", "reserpine", "")
  CALL Species("C_{33}H_{30}N_4O_2",                "144701-48-4", "RMMXLENWKUUMAY-UHFFFAOYSA-N", "telmisartan", "")
  CALL Species("C_{36}H_{53}N_7O_6",                "402957-28-2", "BBAWEDCPNXPBQM-GDEBMMAJSA-N", "telaprevir", "")
  CALL Species("C_{37}H_{48}N_4O_5",                "192725-17-0", "KJHKTHWMRKYKJE-WRHCQWCJSA-N", "lopinavir", "")
  CALL Species("C_{38}H_{52}N_6O_7",                "198904-31-3", "AXRYRYVKAWYZBR-GASGPIRDSA-N", "atazanavir", "")
  CALL Species("C_{46}H_{77}NO_{17}",                 "1401-69-0", "WBPYTXDJUQJLPQ-SNQVITFCSA-N", "tylosin", "")
  CALL Species("C_{47}H_{75}NO_{17}",                 "1400-61-9", "VQOXZBDYSJBXMA-QFHUWGMOSA-N", "nystatin", "")
  CALL Species("C_{48}H_{69}N_3O_6",                 "27676-62-6", "VNQNXQYZMPJLQX-UHFFFAOYSA-N", "1,3,5-tris(3,5-di-{\it tert}-butyl-4-hydroxybenzyl)-1,3,5-triazinane-2,4,6-trione", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Nitrates (\chem{RONO_2})")
  CALL Species("CH_5N_3O_4",                           "124-47-0", "AYTGUZPQPXGYFS-UHFFFAOYSA-N", "urea nitrate", "")
  CALL Species("CH_3ONO_2",                            "598-58-3", "LRMHVVPPGGOAJQ-UHFFFAOYSA-N", "methyl nitrate", "")
  CALL Species("C_2H_5ONO_2",                          "625-58-1", "IDNUEBSJWINEMI-UHFFFAOYSA-N", "ethyl nitrate", "")
  CALL Species("C_3H_7ONO_2",                          "627-13-4", "JNTOKFNBDFMTIV-UHFFFAOYSA-N", "1-propyl nitrate", "")
  CALL Species("C_3H_7ONO_2",                         "1712-64-7", "GAPFWGOSHOCNBM-UHFFFAOYSA-N", "2-propyl nitrate", "isopropyl nitrate")
  CALL Species("C_4H_9ONO_2",                          "928-45-0", "QQHZPQUHCAKSOL-UHFFFAOYSA-N", "1-butyl nitrate", "")
  CALL Species("C_4H_9ONO_2",                          "924-52-7", "DYONNFFVDNILGI-UHFFFAOYSA-N", "2-butyl nitrate", "")
  CALL Species("C_4H_9ONO_2",                          "543-29-3", "LNNXFUZKZLXPOF-UHFFFAOYSA-N", "isobutyl nitrate", "")
  CALL Species("C_5H_{11}ONO_2",                      "1002-16-0", "HSNWZBCBUUSSQD-UHFFFAOYSA-N", "1-pentyl nitrate", "amyl nitrate")
  CALL Species("C_5H_{11}ONO_2",                     "21981-48-6", "RWRBSYOTDDOXKC-UHFFFAOYSA-N", "2-pentyl nitrate", "MCM:PEBNO3")
  CALL Species("C_5H_{13}ONO_2",                     "82944-59-0", "WQZKKVJFBZPJSU-UHFFFAOYSA-N", "3-pentyl nitrate", "")
  CALL Species("C_5H_{11}ONO_2",                       "543-87-3", "NTHGIYFSMNNHSC-UHFFFAOYSA-N", "3-methyl-1-butanol nitrate", "isoamyl nitrate")
  CALL Species("C_5H_8N_4O_{12}",                       "78-11-5", "TZRXHJWUDPFEEY-UHFFFAOYSA-N", "pentaerythritol tetranitrate", "")
  CALL Species("C_6H_{13}ONO_2",                     "20633-11-8", "AGDYNDJUZRMYRG-UHFFFAOYSA-N", "1-hexyl nitrate", "")
  CALL Species("HOC_2H_4ONO_2",                      "16051-48-2", "HTKIMWYSDZQQBP-UHFFFAOYSA-N", "2-nitrooxyethanol", "")
  CALL Species("C_3H_7O_4N",                         "20266-65-3", "OMDFJTSKBNDDRM-UHFFFAOYSA-N", "1-nitrooxy-2-propanol", "")
  CALL Species("C_3H_7O_4N",                         "20266-74-4", "HGCMMKIIGJXXMW-UHFFFAOYSA-N", "2-nitrooxy-1-propanol", "")
  CALL Species("C_4H_9O_4N",                        "147794-11-4", "KNUQGVIXAYDSOX-UHFFFAOYSA-N", "1-nitrooxy-2-butanol", "")
  CALL Species("C_4H_9O_4N",                        "147794-12-5", "YXMNEYKMHSBVTD-UHFFFAOYSA-N", "2-nitrooxy-1-butanol", "")
  CALL Species("C_4H_9O_4N",                        "147794-10-3", "CGFCSKMZZXPWEY-UHFFFAOYSA-N", "2-nitrooxy-3-butanol", "")
  CALL Species("C_4H_9O_4N",                                   "", "FOHXKGXDMSQVTH-UHFFFAOYSA-N", "3-nitrooxy-1-butanol", "")
  CALL Species("C_4H_9O_4N",                         "22911-39-3", "FBOGSWRRYABFKU-UHFFFAOYSA-N", "4-nitrooxy-1-butanol", "")
  CALL Species("C_4H_9O_4N",                        "141299-18-5", "WUKDMTQXKGFHBU-UHFFFAOYSA-N", "4-nitrooxy-2-butanol", "3-hydroxy-1-nitrooxy-butane")
  CALL Species("C_5H_{11}O_4N",                                "", "SVIFKHUNGQLAEN-UHFFFAOYSA-N", "4-nitrooxy-1-pentanol", "")
  CALL Species("C_5H_{11}O_4N",                                "", "DVOSRLSVEOCANS-UHFFFAOYSA-N", "5-nitrooxy-1-pentanol", "")
  CALL Species("C_5H_{11}O_4N",                                "", "RIQPKERROQFFJK-UHFFFAOYSA-N", "5-nitrooxy-2-pentanol", "MCM:HO2C5NO3")
  CALL Species("CH_3COCH_2ONO_2",                     "6745-71-7", "ISWXYJQANHQYSR-UHFFFAOYSA-N", "1-nitrooxy-2-propanone", "nitrooxyacetone")
  CALL Species("C_5H_7NO_5",                                   "", "XXORYENTKCFFSM-UHFFFAOYSA-N", "ISOP1N6CO", "") ! SMILES: O=C/C(=C\CO)CON(=O)=O
  CALL Species("C_5H_7NO_5",                                   "", "QJXLMYNMVOSKND-UHFFFAOYSA-N", "C52COCONO2", "") ! SMILES: CC(=O)CCC(=O)ON(=O)=O
  CALL Species("C_5H_7NO_5",                                   "", "WBVYILWVTRBTDJ-UHFFFAOYSA-N", "ISOP1N23O4CO", "") ! SMILES: CC1(CON(=O)=O)OC1C=O
  CALL Species("C_6H_{11}NO_5",                                "", "COKJDYWOSQONMU-UHFFFAOYSA-N", "ROO6R7ONO2", "") ! SMILES: CC(=O)CCC(CO)ON(=O)=O
  CALL Species("C_7H_{15}NO_3",                                "", "HHXLSUKHLTZWKR-UHFFFAOYSA-N", "2-heptyl nitrate", "C7H15ONO2") ! SMILES: CC(ON(=O)(=O))CCCCC
  CALL Species("C_7H_{15}NO_4",                                "", "MZJHMUSTUUJGSJ-UHFFFAOYSA-N", "C7OHONO2", "") ! SMILES: CCC(ON(=O)(=O))CCC(O)C
  CALL Species("C_7H_{11}NO_5",                                "", "DJCVHBPUJHAWIU-UHFFFAOYSA-N", "C622CONO2", "") ! SMILES: C=C(C)C(CO)CC(=O)ON(=O)=O
  CALL Species("C_8H_{17}NO_3",                                "", "QCOKASLKYUXYJH-UHFFFAOYSA-N", "2-octyl nitrate", "C8H17ONO2") ! SMILES: CC(ON(=O)(=O))CCCCCC
  CALL Species("C_8H_{17}NO_4",                                "", "UEQWMROPIQKFIM-UHFFFAOYSA-N", "C8OHONO2", "") ! SMILES: CCCC(ON(=O)(=O))CCC(O)C
  CALL Species("C_8H_{13}NO_5",                                "", "YMYILPYDFNRVBZ-UHFFFAOYSA-N", "C824ONO2", "") ! SMILES: C=C(C)C(CCO)C(C=O)ON(=O)=O
  CALL Species("C_8H_{13}NO_6",                                "", "XLUWXWLWQTYVOF-UHFFFAOYSA-N", "C819ONO2", "") ! SMILES: CC(=O)CCC(=O)C(C)(CO)ON(=O)=O
  CALL Species("C_9H_{15}NO_5",                                "", "PNFDSIMYWKOAKE-UHFFFAOYSA-N", "C92ONO2", "") ! SMILES: C=C(C)C(CO)CC(ON(=O)=O)C(C)=O
  CALL Species("C_9H_{15}NO_6",                                "", "MUTWZANDXPDWFF-UHFFFAOYSA-N", "NORLIMONO2", "") ! SMILES: CC(=O)CCC(C=O)C(C)(CO)ON(=O)=O
  CALL Species("C_9H_{13}NO_5",                                "", "RVFLVBTUEWAABB-UHFFFAOYSA-N", "C822CONO2", "") ! SMILES: C=C(C)C(CC=O)CCC(=O)ON(=O)=O
  CALL Species("C_9H_{13}NO_6",                                "", "LHFXXFGIVAQFTO-UHFFFAOYSA-N", "C817CONO2", "") ! SMILES: CC(=O)CCC(CC(=O)ON(=O)=O)C(C)=O
  CALL Species("C_{10}H_{15}NO_5",                             "", "FNVMVXZLPDKVAS-UHFFFAOYSA-N", "C9CONO2", "") ! SMILES: C=C(C)C(CCC(C)=O)CC(=O)(ON(=O)(=O))
  CALL Species("C_{10}H_{15}NO_5",                             "", "DEKWHDOXDJNWRD-UHFFFAOYSA-N", "LIMONO2", "") ! SMILES: C=C(C)C(CC=O)CCC(=O)CON(=O)=O
  ! dinitrates and trinitrates:
  CALL Species("O_3NCH_2CH_2ONO_2",                    "628-96-6", "UQXKXGWGFRWILX-UHFFFAOYSA-N", "1,2-ethanediol dinitrate", "1,2-ethane dinitrate")
  CALL Species("C_3H_6(ONO_2)_2",                     "6423-43-4", "PSXCGTLGGVDWFU-UHFFFAOYSA-N", "1,2-propanediol dinitrate", "1,2-propane dinitrate")
  CALL Species("C_3H_6N_2O_6",                        "3457-90-7", "KOSAMXZBGUIISK-UHFFFAOYSA-N", "1,3-propanediol dinitrate", "")
  CALL Species("C_3H_5N_3O_9",                          "55-63-0", "SNIOPGDIGTZGOP-UHFFFAOYSA-N", "1,2,3-propanetriol trinitrate", "nitroglycerin")
  CALL Species("C_4H_8N_2O_6",                       "20820-41-1", "CTISQZXTUUHJNC-UHFFFAOYSA-N", "1,2-butanediol dinitrate", "")
  CALL Species("C_4H_8N_2O_6",                        "6423-44-5", "DGFBULNMARLFTH-UHFFFAOYSA-N", "1,3-butanediol dinitrate", "")
  CALL Species("C_4H_8N_2O_6",                        "3457-91-8", "QELUAJBXJAWSRC-UHFFFAOYSA-N", "1,4-butanediol dinitrate", "")
  CALL Species("C_4H_8N_2O_6",                        "6423-45-6", "RVDDYBGRQLZMSB-UHFFFAOYSA-N", "2,3-butanediol dinitrate", "")
  CALL Species("C_5H_{10}N_2O_6",                    "89365-05-9", "MZWHVGLDAWPHHW-UHFFFAOYSA-N", "1,2-pentanediol dinitrate", "")
  CALL Species("C_5H_{10}N_2O_6",                    "25385-63-1", "IUTIKUKYGRINOD-UHFFFAOYSA-N", "1,4-pentanediol dinitrate", "")
  CALL Species("C_5H_{10}N_2O_6",                     "3457-92-9", "MIYIEPHJPVBSEV-UHFFFAOYSA-N", "1,5-pentanediol dinitrate", "")
  CALL Species("C_5H_{10}N_2O_6",                   "208252-05-5", "",                            "(2R,4S)-2,4-pentanediol dinitrate", "{\it cis}-2,4-pentanediol dinitrate")
  CALL Species("C_5H_{10}N_2O_6",                   "208252-04-4", "",                            "(2R,4R)-2,4-pentanediol dinitrate", "{\it trans}-2,4-pentanediol dinitrate")
  CALL Species("C_6H_{12}N_2O_6",                   "110539-07-6", "UJKJGCZXZPXTGS-UHFFFAOYSA-N", "1,2-hexanediol dinitrate", "")
  CALL Species("C_6H_{12}N_2O_6",                   "206443-83-6", "PGDWEAOSOZNKFU-UHFFFAOYSA-N", "1,5-hexanediol dinitrate", "")
  CALL Species("C_6H_{12}N_2O_6",                     "3457-93-0", "GCVAYIWGKFFWEU-UHFFFAOYSA-N", "1,6-hexanediol dinitrate", "")
  CALL Species("C_6H_{12}N_2O_6",                    "99115-63-6", "ISSLCMSTXXUOEU-UHFFFAOYSA-N", "2,5-hexanediol dinitrate", "")
  CALL Species("C_6H_{10}N_2O_6",                    "32342-28-2", "",                            "(1R,2S)-1,2-cyclohexanediol dinitrate", "{\it cis}-1,2-cyclohexanediol dinitrate")
  CALL Species("C_6H_{10}N_2O_6",                    "32342-29-3", "",                            "(1R,2R)-1,2-cyclohexanediol dinitrate", "{\it trans}-1,2-cyclohexanediol dinitrate")
  CALL Species("C_6H_{10}N_2O_6",                   "170994-36-2", "",                            "(1R,3S)-1,3-cyclohexanediol dinitrate", "{\it cis}-1,3-cyclohexanediol dinitrate")
  CALL Species("C_6H_{10}N_2O_6",                   "170994-41-9", "",                            "(1R,3R)-1,3-cyclohexanediol dinitrate", "{\it trans}-1,3-cyclohexanediol dinitrate")
  CALL Species("C_7H_{14}N_2O_6",                     "3457-94-1", "KIERETFMVSIXIJ-UHFFFAOYSA-N", "1,7-heptanediol dinitrate", "")
  CALL Species("C_7H_{12}N_2O_6",                   "208252-06-6", "",                            "(1R,2R)-1,2-cycloheptanediol dinitrate", "{\it trans}-1,2-cycloheptanediol dinitrate")
  CALL Species("C_8H_{16}N_2O_6",                   "121222-48-8", "RLNKMZWMCSTIEM-UHFFFAOYSA-N", "1,2-octanediol dinitrate", "")
  CALL Species("C_8H_{16}N_2O_6",                     "3457-95-2", "BVIHOKFRPSZJEI-UHFFFAOYSA-N", "1,8-octanediol dinitrate", "")
  CALL Species("C_{10}H_{20}N_2O_6",                 "60123-40-2", "KKNGVXOJDIUADE-UHFFFAOYSA-N", "1,2-decanediol dinitrate", "")
  CALL Species("C_{10}H_{20}N_2O_6",                  "3457-97-4", "RHUZOYMELURURD-UHFFFAOYSA-N", "1,10-decanediol dinitrate", "")
  CALL Species("C_4H_8N_2O_7",                         "693-21-0", "LYAGTVMJGHTIDH-UHFFFAOYSA-N", "diethylene glycol dinitrate", "")
  CALL Species("C_5H_8N_2O_6",                                 "", "FQSKJUXVWIFNKX-UHFFFAOYSA-N", "ISOP1N2ONO2", "") ! SMILES: C=CC(C)(CON(=O)=O)ON(=O)=O
  CALL Species("C_5H_8N_2O_7",                                 "", "UAFKAIYDADHRMX-UHFFFAOYSA-N", "EPXISOPNONO2", "") ! SMILES: CC(CON(=O)=O)(ON(=O)=O)C1CO1
  CALL Species("C_5H_8N_2O_7",                                 "", "FSZUIXLMCRZJMS-UHFFFAOYSA-N", "ISOP1N23O4ONO2", "") ! SMILES: CC1(CON(=O)=O)OC1CON(=O)=O
  CALL Species("C_5H_8N_2O_7",                                 "", "WRGRJJOKHABEJT-UHFFFAOYSA-N", "ISOP1N5ONO2", "") ! SMILES: O=N(=O)OC/C(=C\CO)CON(=O)=O
  CALL Species("C_{10}H_{18}N_2O_8",                           "", "UNHNJOVXNACOSQ-UHFFFAOYSA-N", "LIMAB15ONO22", "") ! SMILES: CC(CO)(ON(=O)=O)C1CCC(C)(ON(=O)=O)C(O)C1
  ! peroxy nitrates:
  CALL Species("CH_3COOONO_2",                        "2278-22-0", "VGQXTTSVLMQFHM-UHFFFAOYSA-N", "peroxyacetyl nitrate", "PAN")
  CALL Species("C_2H_5COOONO_2",                      "5796-89-4", "TXINBPKSWKFMNB-UHFFFAOYSA-N", "peroxypropionyl nitrate", "PPN")
  CALL Species("C_3H_7COOONO_2",                     "27746-48-1", "HZUMMZVMNQSPFF-UHFFFAOYSA-N", "nitro butaneperoxoate", "PnBN")
  CALL Species("CH_2C(CH_3)COOONO_2",                "88181-75-3", "LLZWPQFQEBKRLX-UHFFFAOYSA-N", "peroxy-2-propenoyl nitrate", "peroxymethacryloyl nitrate; MPAN")
  CALL Species("C_3H_7COOONO_2",                     "65424-60-4", "BDNFHGUXBRZLRQ-UHFFFAOYSA-N", "peroxy-isobutyryl nitrate", "PiBN")
  CALL Species("C_5H_3NO_6",                                   "", "NMNZVYLGOWCHOD-UHFFFAOYSA-N", "furoyl peroxynitrate", "fur-PAN") ! C1=COC(=C1)C(=O)OO[N](=O)=O InChI=1S/C5H3NO6/c7-5(11-12-6(8)9)4-2-1-3-10-4/h1-3H
  CALL Species("CH_3NO_4",                                     "", "LCFGXMPUQSXLCQ-UHFFFAOYSA-N", "MCM:CH3O2NO2", "")
  CALL Species("C_2H_5NO_5",                                   "", "IYCQDYYQELIXMU-UHFFFAOYSA-N", "MCM:ETHO2HNO3", "")
  CALL Species("C_2H_3NO_6",                                   "", "VLCOOHYMNNDETJ-UHFFFAOYSA-N", "MCM:NO3CH2CO3H", "")
  CALL Species("C_2H_2N_2O_8",                                 "", "CSNUPQYBRZIENU-UHFFFAOYSA-N", "MCM:NO3CH2PAN", "")
  CALL Species("C_3H_3NO_5",                        "157258-66-7", "SFKRQQJZRXQGLC-UHFFFAOYSA-N", "MCM:ACRPAN", "")
  CALL Species("C_3H_7NO_5",                                   "", "FROUFTMXOYPVCX-UHFFFAOYSA-N", "MCM:PR1O2HNO3", "")
  CALL Species("C_3H_7NO_5",                                   "", "UHIVGIHVGRYUPL-UHFFFAOYSA-N", "MCM:PR2O2HNO3", "")
  CALL Species("C_3H_5NO_6",                                   "", "FMRKJWQAPYMRJD-UHFFFAOYSA-N", "MCM:PRNO3CO3H", "")
  CALL Species("C_3H_4N_2O_8",                                 "", "ZLJYZUGEJFPONP-UHFFFAOYSA-N", "MCM:PRNO3PAN", "")
  CALL Species("C_4H_9NO_5",                                   "", "MFTWJXGCASCYHP-UHFFFAOYSA-N", "MCM:BU1ENO3OOH", "")
  CALL Species("C_4H_5NO_5",                                   "", "YVSVRLGWMDVSPN-UHFFFAOYSA-N", "MCM:C3DBPAN", "")
  CALL Species("C_4H_9NO_5",                                   "", "CGBCWMNQUOIZIB-UHFFFAOYSA-N", "MCM:C42NO33OOH", "")
  CALL Species("C_4H_9NO_5",                                   "", "BQDGWSCZHPCGGT-UHFFFAOYSA-N", "MCM:C43NO34OOH", "")
  CALL Species("C_4H_4N_2O_8",                                 "", "SHASFEZZYJNCIM-UHFFFAOYSA-N", "MCM:C4PAN9", "")
  CALL Species("C_4H_9NO_5",                                   "", "DTPMQEMACNJTBJ-UHFFFAOYSA-N", "MCM:MPRANO3OOH", "")
  CALL Species("C_4H_9NO_5",                                   "", "RCQWCNCMOFDCKD-UHFFFAOYSA-N", "MCM:MPRBNO3OOH", "")
  CALL Species("C_4H_6N_2O_8",                                 "", "RAYRXKOIFAVDJK-UHFFFAOYSA-N", "MCM:MPRBNO3PAN", "")
  CALL Species("C_4H_7NO_6",                                   "", "HDNCPEGDTKOFQI-UHFFFAOYSA-N", "MCM:MPRNO3CO3H", "")
  CALL Species("C_4H_7NO_5",                                   "", "ZIAMBHNGSMXNCM-UHFFFAOYSA-N", "MCM:NBUTDAOOH", "")
  CALL Species("C_4H_6N_2O_6",                                 "", "GCQJZAFTGYZGDG-UHFFFAOYSA-N", "MCM:NBUTDBNO3", "")
  CALL Species("C_4H_7NO_5",                                   "", "SSNXKZNERJCDKO-UHFFFAOYSA-N", "MCM:NBUTDBOOH", "")
  CALL Species("C_4H_5NO_6",                                   "", "ZXNVOGOFZJSFAE-UHFFFAOYSA-N", "MCM:NC3CO3H", "")
  CALL Species("C_4H_9NO_3",                           "926-05-6", "AZAKMLHUDVIDFN-UHFFFAOYSA-N", "MCM:TC4H9NO3", "")
  CALL Species("C_5H_9NO_5",                                   "", "OLNWAQLSTCXURT-UHFFFAOYSA-N", "MCM:C3ME3PAN", "")
  CALL Species("C_5H_9NO_6",                                   "", "UNTXQXBLSROXLI-UHFFFAOYSA-N", "MCM:C3MNO3CO3H", "")
  CALL Species("C_5H_8N_2O_8",                                 "", "ULZBZSVLLIBTGC-UHFFFAOYSA-N", "MCM:C3MNO3PAN", "")
  CALL Species("C_5H_9NO_6",                                   "", "KYFRJEXZVMOAMN-UHFFFAOYSA-N", "MCM:C43NO3CO3H", "")
  CALL Species("C_5H_8N_2O_8",                                 "", "WPFKHDFVFPXDFZ-UHFFFAOYSA-N", "MCM:C43NO3PAN", "")
  CALL Species("C_5H_{11}NO_5",                                "", "BOAAVKBCMOPNIQ-UHFFFAOYSA-N", "MCM:C4M2NO3OOH", "")
  CALL Species("C_5H_{11}NO_5",                                "", "LRRBWUKXCGVENZ-UHFFFAOYSA-N", "MCM:C4MNO31OOH", "")
  CALL Species("C_5H_{11}NO_5",                                "", "HXNZNKRZULBVRO-UHFFFAOYSA-N", "MCM:C4MNO32OOH", "")
  CALL Species("C_5H_{11}NO_5",                                "", "HCKTYRRBEFAEAC-UHFFFAOYSA-N", "MCM:C4NO32MOOH", "")
  CALL Species("C_5H_9NO_6",                                   "", "JPUVYTOGMBAUBQ-UHFFFAOYSA-N", "MCM:C4NO3CO3H", "")
  CALL Species("C_5H_{11}NO_5",                                "", "MWSZSODDMCBKIX-UHFFFAOYSA-N", "MCM:C4NO3M1OOH", "")
  CALL Species("C_5H_{11}NO_5",                                "", "QXHHHGYKLMZJBA-UHFFFAOYSA-N", "MCM:C4NO3M2OOH", "")
  CALL Species("C_5H_8N_2O_8",                                 "", "GGZFCBHNGNBLNW-UHFFFAOYSA-N", "MCM:C4NO3PAN", "")
  CALL Species("C_5H_{11}NO_5",                                "", "YPLQKABNDDXYJG-UHFFFAOYSA-N", "MCM:C51NO32OOH", "")
  CALL Species("C_5H_{11}NO_5",                                "", "DCAGTGOPLFTTTQ-UHFFFAOYSA-N", "MCM:C52NO31OOH", "")
  CALL Species("C_5H_{11}NO_5",                                "", "FQGADFCNSPWVJM-UHFFFAOYSA-N", "MCM:C52NO33OOH", "")
  CALL Species("C_5H_{11}NO_5",                                "", "STNFTACLRXEYDM-UHFFFAOYSA-N", "MCM:C53NO32OOH", "")
  CALL Species("C_5H_6N_2O_8",                                 "", "JYUXMKSXMIBDSN-UHFFFAOYSA-N", "MCM:C5PAN18", "")
  CALL Species("C_5H_9NO_5",                                   "", "LGNOWHJLTKBAFC-UHFFFAOYSA-N", "MCM:C5PAN4", "")
  CALL Species("C_5H_9NO_5",                                   "", "VQRIYOXAZWZCOV-UHFFFAOYSA-N", "MCM:C5PAN6", "")
  CALL Species("C_5H_{11}NO_3",                                "", "QSSWLPYLLKHHLB-UHFFFAOYSA-N", "MCM:IPEANO3", "")
  CALL Species("C_5H_{11}NO_3",                                "", "OTVLXFGCWJFXJU-UHFFFAOYSA-N", "MCM:IPEBNO3", "")
  CALL Species("C_5H_{11}NO_3",                                "", "UENFRVTUGZKXNH-UHFFFAOYSA-N", "MCM:IPECNO3", "")
  CALL Species("C_5H_7NO_6",                                   "", "SLKHOLOIOGBFQB-UHFFFAOYSA-N", "MCM:NC4CO3H", "")
  CALL Species("C_5H_{11}NO_3",                                "", "YJGBGWFCCCQXIM-UHFFFAOYSA-N", "MCM:NEOPNO3", "")
  CALL Species("C_5H_8N_2O_6",                                 "", "JGJBVRGABXKDRR-UHFFFAOYSA-N", "MCM:NISOPNO3", "")
  CALL Species("C_5H_9NO_5",                                   "", "IRFXVIPRKCCSLU-UHFFFAOYSA-N", "MCM:NISOPOOH", "")
  CALL Species("C_5H_9NO_5",                                   "", "UUYNBJNAIGBJAO-UHFFFAOYSA-N", "MCM:PPEN", "")
  CALL Species("C_6H_{13}NO_5",                                "", "KKTQLMUUOWWZBO-UHFFFAOYSA-N", "MCM:C42MNO3OOH", "")
  CALL Species("C_6H_{13}NO_5",                                "", "PJRWVMFJYMZPCW-UHFFFAOYSA-N", "MCM:C62NO33OOH", "")
  CALL Species("C_6H_{13}NO_5",                                "", "RLIDOIUHNSZVBK-UHFFFAOYSA-N", "MCM:C63NO32OOH", "")
  CALL Species("C_6H_{13}NO_5",                                "", "ZVRSMGCQHDCSBC-UHFFFAOYSA-N", "MCM:C65NO36OOH", "")
  CALL Species("C_6H_{11}NO_6",                                "", "RASQUKWKKRVIQJ-UHFFFAOYSA-N", "MCM:C65NO3CO3H", "")
  CALL Species("C_6H_{10}N_2O_8",                              "", "PNNYZMYYAAWSJH-UHFFFAOYSA-N", "MCM:C65NO3PAN", "")
  CALL Species("C_6H_{13}NO_5",                                "", "YPVNGNKLOFDKIF-UHFFFAOYSA-N", "MCM:C66NO35OOH", "")
  CALL Species("C_6H_{11}NO_5",                                "", "INNSDEOCGQAUTR-UHFFFAOYSA-N", "MCM:C6PAN10", "")
  CALL Species("C_6H_{11}NO_5",                                "", "GWYKJYISUPTXQD-UHFFFAOYSA-N", "MCM:C6PAN15", "")
  CALL Species("C_6H_{11}NO_5",                                "", "CUMLTNLXWSMQLS-UHFFFAOYSA-N", "MCM:C6PAN17", "")
  CALL Species("C_6H_{11}NO_5",                                "", "FPWBUKNMBKRQCI-UHFFFAOYSA-N", "MCM:C6PAN18", "")
  CALL Species("C_6H_{11}NO_5",                                "", "BRVOFEXYGAZHOW-UHFFFAOYSA-N", "MCM:C6PAN3", "")
  CALL Species("C_6H_{11}NO_3",                                "", "HLYOOCIMLHNMOG-UHFFFAOYSA-N", "MCM:CHEXNO3", "")
  CALL Species("C_6H_{13}NO_3",                                "", "JLGBOJJVQLNMGV-UHFFFAOYSA-N", "MCM:HEXBNO3", "")
  CALL Species("C_6H_{13}NO_3",                                "", "OJOZCOXRANAOPV-UHFFFAOYSA-N", "MCM:HEXCNO3", "")
  CALL Species("C_6H_{13}NO_3",                                "", "YUZOHIKIQPTSGJ-UHFFFAOYSA-N", "MCM:M22C43NO3", "")
  CALL Species("C_6H_{13}NO_3",                                "", "JOWPFKXUGRKJFA-UHFFFAOYSA-N", "MCM:M22C4NO3", "")
  CALL Species("C_6H_{13}NO_3",                                "", "PMABLOPIFVACLQ-UHFFFAOYSA-N", "MCM:M23C43NO3", "")
  CALL Species("C_6H_{13}NO_3",                                "", "HHRFTVZVEUIWEO-UHFFFAOYSA-N", "MCM:M23C4NO3", "")
  CALL Species("C_6H_{13}NO_3",                                "", "CETUNBADQVCCRS-UHFFFAOYSA-N", "MCM:M2PEANO3", "")
  CALL Species("C_6H_{13}NO_3",                                "", "AYWPQHSAUPNICG-UHFFFAOYSA-N", "MCM:M2PEBNO3", "")
  CALL Species("C_6H_{13}NO_3",                                "", "ZLERWGPAWILHHJ-UHFFFAOYSA-N", "MCM:M2PECNO3", "")
  CALL Species("C_6H_{13}NO_3",                                "", "LFIPHSRTIDHJFT-UHFFFAOYSA-N", "MCM:M2PEDNO3", "")
  CALL Species("C_6H_{13}NO_3",                                "", "DENSRMRCUIZNMK-UHFFFAOYSA-N", "MCM:M33C4NO3", "")
  CALL Species("C_6H_{13}NO_3",                                "", "HRVRJIMBNILCGN-UHFFFAOYSA-N", "MCM:M3PEANO3", "")
  CALL Species("C_6H_{13}NO_3",                                "", "IEHIEWLLOPOHIZ-UHFFFAOYSA-N", "MCM:M3PEBNO3", "")
  CALL Species("C_6H_{13}NO_3",                                "", "GFKONLNVJRACGN-UHFFFAOYSA-N", "MCM:M3PECNO3", "")
  CALL Species("C_6H_{11}NO_5",                                "", "ITHMZXMFOKWUTQ-UHFFFAOYSA-N", "MCM:PHXN", "")
  CALL Species("C_7H_{15}NO_3",                                "", "RFSNRWONUSBTJN-UHFFFAOYSA-N", "MCM:HEPTNO3", "")
  CALL Species("C_7H_{15}NO_3",                                "", "YZGBSAOHUZMZSL-UHFFFAOYSA-N", "MCM:M2HEXANO3", "")
  CALL Species("C_7H_{15}NO_3",                                "", "CORAYZMLWQTCOE-UHFFFAOYSA-N", "MCM:M2HEXBNO3", "")
  CALL Species("C_7H_{15}NO_3",                                "", "ZIINVFVZOGZDQJ-UHFFFAOYSA-N", "MCM:M3HEXANO3", "")
  CALL Species("C_7H_{15}NO_3",                                "", "ADYNXMRWYWNUDX-UHFFFAOYSA-N", "MCM:M3HEXBNO3", "")
  CALL Species("C_7H_{13}NO_5",                                "", "XCJVDRQLXVBBEC-UHFFFAOYSA-N", "MCM:PHPTN", "")
  CALL Species("C_8H_{13}NO_3",                                "", "OSNOKXIMBITXOJ-UHFFFAOYSA-N", "MCM:C8BCNO3", "")
  CALL Species("C_8H_{17}NO_3",                                "", "KDICWCURMCOQTP-UHFFFAOYSA-N", "MCM:OCTNO3", "")
  CALL Species("C_9H_{19}NO_3",                                "", "YXUXBXICIGKFLH-UHFFFAOYSA-N", "MCM:NONNO3", "")
  CALL Species("C_{10}H_{21}NO_3",                             "", "VRGDYCVXNCXQKR-UHFFFAOYSA-N", "MCM:DECNO3", "")
  CALL Species("C_{10}H_{17}NO_5",                             "", "IKGBGFRUISEOBM-UHFFFAOYSA-N", "MCM:NAPINAOOH", "")
  CALL Species("C_{10}H_{17}NO_5",                             "", "RUHGEBIMHPPSGQ-UHFFFAOYSA-N", "MCM:NAPINBOOH", "")
  CALL Species("C_{10}H_{17}NO_5",                             "", "VJCKFKSOBAWLFB-UHFFFAOYSA-N", "MCM:NBPINAOOH", "")
  CALL Species("C_{10}H_{17}NO_5",                             "", "AWECGRRBHMAEGP-UHFFFAOYSA-N", "MCM:NBPINBOOH", "")
  CALL Species("C_{10}H_{15}NO_6",                             "", "TWWSFGIBZQJYPL-UHFFFAOYSA-N", "MCM:NC91CO3H", "")
  CALL Species("C_{10}H_{14}N_2O_8",                           "", "LCMKIEFNSOTVJY-UHFFFAOYSA-N", "MCM:NC91PAN", "")
  CALL Species("C_{10}H_{17}NO_5",                             "", "HSXYRSLOXSIYGP-UHFFFAOYSA-N", "MCM:NLIMOOH", "")
  CALL Species("C_{11}H_{23}NO_3",                             "", "KGEKWWIIMVPQKA-UHFFFAOYSA-N", "MCM:UDECNO3", "")
  CALL Species("C_{12}H_{25}NO_3",                             "", "BVAQOKLOUKLOJD-UHFFFAOYSA-N", "MCM:DDECNO3", "")
  CALL Species("C_{15}H_{25}NO_5",                             "", "BEFFZATUSVKAHP-UHFFFAOYSA-N", "MCM:NBCOOH", "")
  CALL Species("C_7H_7NO_3",                                   "", "WOIVNLSVAKYSKX-UHFFFAOYSA-N", "MCM:C6H5CH2NO3", "")
  CALL Species("C_7H_5NO_5",                                   "", "ONDCXZPWEKXYJE-UHFFFAOYSA-N", "MCM:PBZN", "")
  CALL Species("C_8H_9NO_3",                                   "", "REJUUAZLLYKBCW-UHFFFAOYSA-N", "MCM:C6H5C2NO3", "")
  CALL Species("C_8H_7NO_5",                                   "", "KKRIGRPRRNIVGL-UHFFFAOYSA-N", "MCM:C6H5CH2PAN", "")
  CALL Species("C_8H_9NO_3",                                   "", "OIEIRDPNAPXLBC-UHFFFAOYSA-N", "MCM:MXYLNO3", "")
  CALL Species("C_8H_7NO_5",                                   "", "HJGURQOGPAEQPE-UHFFFAOYSA-N", "MCM:MXYLPAN", "")
  CALL Species("C_8H_9NO_5",                                   "", "LKBHTMHNYICJQS-UHFFFAOYSA-N", "MCM:NSTYRENOOH", "")
  CALL Species("C_8H_9NO_3",                                   "", "WKRHODAGVPDBTG-UHFFFAOYSA-N", "MCM:OXYLNO3", "")
  CALL Species("C_8H_7NO_5",                                   "", "TXXXSKKGTSQWKQ-UHFFFAOYSA-N", "MCM:OXYLPAN", "")
  CALL Species("C_8H_9NO_3",                                   "", "SHZSDLPSACZYMH-UHFFFAOYSA-N", "MCM:PXYLNO3", "")
  CALL Species("C_8H_7NO_5",                                   "", "ZGFUWGCXWYZODV-UHFFFAOYSA-N", "MCM:PXYLPAN", "")
  CALL Species("C_9H_{11}NO_3",                                "", "LYNFROWVHACAMO-UHFFFAOYSA-N", "MCM:ETOLNO3", "")
  CALL Species("C_9H_{11}NO_3",                                "", "IFDAHDBLKMQHMM-UHFFFAOYSA-N", "MCM:PHC3NO3", "")
  CALL Species("C_9H_{11}NO_3",                                "", "YCCLKZYFPBVILU-UHFFFAOYSA-N", "MCM:PHIC3NO3", "")
  CALL Species("C_9H_{11}NO_3",                                "", "GDXSVUXDQKEMHM-UHFFFAOYSA-N", "MCM:TM123BNO3", "")
  CALL Species("C_9H_9NO_5",                                   "", "VMALQUVLPIFIEM-UHFFFAOYSA-N", "MCM:TM123BPAN", "")
  CALL Species("C_9H_{11}NO_3",                                "", "VLXWVVSZKXVGNM-UHFFFAOYSA-N", "MCM:TM124BNO3", "")
  CALL Species("C_9H_9NO_5",                                   "", "SSMPYZHCWJGIJH-UHFFFAOYSA-N", "MCM:TM124BPAN", "")
  CALL Species("C_9H_{11}NO_3",                                "", "CHDZRXQPKVNDRJ-UHFFFAOYSA-N", "MCM:TMBNO3", "")
  CALL Species("C_9H_9NO_5",                                   "", "QURBGWBLEPWFEF-UHFFFAOYSA-N", "MCM:TMBPAN", "")
  CALL Species("C_{10}H_{13}NO_3",                             "", "CZPIEIXHJQQIGZ-UHFFFAOYSA-N", "MCM:DM35EBNO3", "")
  CALL Species("C_{10}H_{11}NO_5",                             "", "DPRDWMVUHNTRQI-UHFFFAOYSA-N", "MCM:EMPHPAN", "")
  CALL Species("C_{11}H_{15}NO_3",                             "", "XLLLSPDIGUQFTC-UHFFFAOYSA-N", "MCM:DE35TNO3", "")
  CALL Species("C_2H_3NO_6",                                   "", "KMDWHTZYVJXWAH-UHFFFAOYSA-N", "MCM:PHAN", "")
  CALL Species("C_3H_5NO_7",                                   "", "RQFAVCUBLOXLKI-UHFFFAOYSA-N", "MCM:A2PAN", "")
  CALL Species("C_3H_5NO_6",                                   "", "ZDHDBTZUUXEIQG-UHFFFAOYSA-N", "MCM:C3PAN1", "")
  CALL Species("C_3H_7NO_4",                        "100502-66-7", "PTMLFFXFTRSBJW-UHFFFAOYSA-N", "MCM:HO1C3NO3", "")
  CALL Species("C_3H_5NO_6",                                   "", "VGQGEUUGHUVAJJ-UHFFFAOYSA-N", "MCM:IPROPOLPAN", "")
  CALL Species("C_4H_7NO_4",                                   "", "JVISETCEJQRRQM-UHFFFAOYSA-N", "MCM:BUTDANO3", "")
  CALL Species("C_4H_7NO_4",                                   "", "MVWTUBMDIKBQOB-UHFFFAOYSA-N", "MCM:BUTDBNO3", "")
  CALL Species("C_4H_7NO_6",                                   "", "ZAYFYZJEFHQRME-UHFFFAOYSA-N", "MCM:C4PAN1", "")
  CALL Species("C_4H_7NO_7",                                   "", "KFPIYXZEKUAVMP-UHFFFAOYSA-N", "MCM:C4PAN2", "")
  CALL Species("C_4H_7NO_6",                                   "", "PIBQEVCOTXTKAE-UHFFFAOYSA-N", "MCM:C4PAN3", "")
  CALL Species("C_4H_7NO_6",                                   "", "NXHSYZWHYBOEBH-UHFFFAOYSA-N", "MCM:C4PAN4", "")
  CALL Species("C_4H_7NO_6",                                   "", "BZGCEXLIVXRKKX-UHFFFAOYSA-N", "MCM:C4PAN5", "")
  CALL Species("C_4H_5NO_6",                                   "", "YKQOPDHVAHDODD-UHFFFAOYSA-N", "MCM:C4PAN7", "")
  CALL Species("C_4H_5NO_6",                                   "", "QZCDOTFCQHJUGH-UHFFFAOYSA-N", "MCM:C4PAN8", "")
  CALL Species("C_4H_9NO_5",                                   "", "XYUWBXDQANMXIL-UHFFFAOYSA-N", "MCM:C56NO3", "")
  CALL Species("C_4H_5NO_6",                                   "", "XPXMBKPWAOWOBP-UHFFFAOYSA-N", "MCM:HMPAN", "")
  CALL Species("C_4H_9NO_5",                                   "", "IOYLZWMTXYBAPI-UHFFFAOYSA-N", "MCM:HO13C4NO3", "")
  CALL Species("C_4H_7NO_6",                                   "", "JGMDPJZYJFPSOE-UHFFFAOYSA-N", "MCM:HO3C3PAN", "")
  CALL Species("C_4H_9NO_4",                                   "", "ILMPSGIQBJFTRG-UHFFFAOYSA-N", "MCM:IBUTOLBNO3", "")
  CALL Species("C_4H_9NO_4",                                   "", "OARBGYKXUANYLR-UHFFFAOYSA-N", "MCM:IBUTOLCNO3", "")
  CALL Species("C_4H_6N_2O_9",                                 "", "HHPQSZXZUAZGAQ-UHFFFAOYSA-N", "MCM:MACRNBPAN", "")
  CALL Species("C_4H_7NO_7",                                   "", "VCAMMMQHKHRNFT-UHFFFAOYSA-N", "MCM:MACRNCO3H", "")
  CALL Species("C_4H_6N_2O_9",                                 "", "YVNHVFURNFQJQM-UHFFFAOYSA-N", "MCM:MACRNPAN", "")
  CALL Species("C_4H_7NO_4",                                   "", "ZANUSWCYRLKDAN-UHFFFAOYSA-N", "MCM:NBUTDAOH", "")
  CALL Species("C_4H_9NO_4",                                   "", "SPXXYWSDFUWLEP-UHFFFAOYSA-N", "MCM:TBUTOLNO3", "")
  CALL Species("C_5H_9NO_6",                                   "", "NSGPYXAFGPXNMV-UHFFFAOYSA-N", "MCM:C3M3OH2PAN", "")
  CALL Species("C_5H_7NO_6",                                   "", "WDMYRNZGJSFESH-UHFFFAOYSA-N", "MCM:C46PAN", "")
  CALL Species("C_5H_9NO_7",                                   "", "INCMEOVDRBCULR-UHFFFAOYSA-N", "MCM:C4OH2CPAN", "")
  CALL Species("C_5H_9NO_6",                                   "", "GAAOIPAQIRGTBI-UHFFFAOYSA-N", "MCM:C4OHPAN", "")
  CALL Species("C_5H_9NO_5",                                   "", "IRMGDWDXIVCKPE-UHFFFAOYSA-N", "MCM:C524NO3", "")
  CALL Species("C_5H_{11}NO_4",                                "", "NBCYUKLMSBOGIC-UHFFFAOYSA-N", "MCM:C52NO3", "")
  CALL Species("C_5H_{11}NO_5",                                "", "OXSZOTYJVSIXIT-UHFFFAOYSA-N", "MCM:C54NO3", "")
  CALL Species("C_5H_9NO_8",                                   "", "BPRAPKBGVRGZEK-UHFFFAOYSA-N", "MCM:C57NO3CO3H", "")
  CALL Species("C_5H_8N_2O_{10}",                              "", "QHZGCLHPLOENFL-UHFFFAOYSA-N", "MCM:C57NO3PAN", "")
  CALL Species("C_5H_9NO_8",                                   "", "NFQMHUFQLLKQQS-UHFFFAOYSA-N", "MCM:C58NO3CO3H", "")
  CALL Species("C_5H_8N_2O_{10}",                              "", "VIMTZSSJOGPGII-UHFFFAOYSA-N", "MCM:C58NO3PAN", "")
  CALL Species("C_5H_9NO_7",                                   "", "YRCMXGZRWLAVBH-UHFFFAOYSA-N", "MCM:C5PAN10", "")
  CALL Species("C_5H_9NO_6",                                   "", "ISSHIIFFUOEHLT-UHFFFAOYSA-N", "MCM:C5PAN11", "")
  CALL Species("C_5H_9NO_7",                                   "", "VRGURBHINCFLEI-UHFFFAOYSA-N", "MCM:C5PAN12", "")
  CALL Species("C_5H_9NO_7",                                   "", "GDYHQRWKNLRCBR-UHFFFAOYSA-N", "MCM:C5PAN13", "")
  CALL Species("C_5H_9NO_6",                                   "", "KAYGTWBSLGDXRV-UHFFFAOYSA-N", "MCM:C5PAN14", "")
  CALL Species("C_5H_9NO_6",                                   "", "DDVUSZKFLFMJLG-UHFFFAOYSA-N", "MCM:C5PAN15", "")
  CALL Species("C_5H_7NO_6",                                   "", "WOHMTHRFBCBPEN-UHFFFAOYSA-N", "MCM:C5PAN17", "")
  CALL Species("C_5H_7NO_6",                                   "", "SXXBKHKGTQGHOF-UHFFFAOYSA-N", "MCM:C5PAN19", "")
  CALL Species("C_5H_9NO_6",                                   "", "BXAPPDLQCWAVAO-UHFFFAOYSA-N", "MCM:C5PAN1", "")
  CALL Species("C_5H_9NO_6",                                   "", "GANOVMQBFKDCJS-UHFFFAOYSA-N", "MCM:C5PAN3", "")
  CALL Species("C_5H_9NO_6",                                   "", "SNADUXVVPGYKTN-UHFFFAOYSA-N", "MCM:C5PAN5", "")
  CALL Species("C_5H_9NO_6",                                   "", "AVADEHRLOLZNNT-UHFFFAOYSA-N", "MCM:C5PAN8", "")
  CALL Species("C_5H_{11}NO_4",                                "", "KRCHLCXLZXFPMO-UHFFFAOYSA-N", "MCM:H2M3C4NO3", "")
  CALL Species("C_5H_{11}NO_4",                                "", "GJZIQWDDCYOUOR-UHFFFAOYSA-N", "MCM:HM22C3NO3", "")
  CALL Species("C_5H_{11}NO_4",                                "", "OLLXHFMQLUGEEM-UHFFFAOYSA-N", "MCM:HM2C43NO3", "")
  CALL Species("C_5H_{11}NO_4",                                "", "LCUGTHWCLWAZIN-UHFFFAOYSA-N", "MCM:HM33C3NO3", "")
  CALL Species("C_5H_{11}NO_5",                                "", "HRXGEJHIRPMYKD-UHFFFAOYSA-N", "MCM:HO13C5NO3", "")
  CALL Species("C_5H_{11}NO_5",                                "", "GDKLMXJUXHUPHP-UHFFFAOYSA-N", "MCM:HO24C5NO3", "")
  CALL Species("C_5H_{11}NO_4",                                "", "BDOVYOMPHMUDCA-UHFFFAOYSA-N", "MCM:HO2C54NO3", "")
  CALL Species("C_5H_{11}NO_4",                                "", "JLEBTGVTEJACMC-UHFFFAOYSA-N", "MCM:HO2M2C4NO3", "")
  CALL Species("C_5H_{11}NO_4",                                "", "VSJMKXXNGJNYTF-UHFFFAOYSA-N", "MCM:HO3C5NO3", "")
  CALL Species("C_5H_9NO_8",                                   "", "NANYPKIUOGNJFQ-UHFFFAOYSA-N", "MCM:INAHCO3H", "")
  CALL Species("C_5H_8N_2O_{10}",                              "", "ISRNXRSYOJKFEQ-UHFFFAOYSA-N", "MCM:INAHPAN", "")
  CALL Species("C_5H_9NO_9",                                   "", "ZYWYMWCOZFANRV-UHFFFAOYSA-N", "MCM:INAHPCO3H", "")
  CALL Species("C_5H_8N_2O_{11}",                              "", "ZXVKOORKVQYPPA-UHFFFAOYSA-N", "MCM:INAHPPAN", "")
  CALL Species("C_5H_8N_2O_{10}",                              "", "HKZKRKUNOXITIG-UHFFFAOYSA-N", "MCM:INANCO3H", "")
  CALL Species("C_5H_{10}N_2O_8",                              "", "VVVQXVVEBQASSQ-UHFFFAOYSA-N", "MCM:INANO3", "")
  CALL Species("C_5H_7N_3O_{12}",                              "", "PTXQCTHWIXKTGJ-UHFFFAOYSA-N", "MCM:INANPAN", "")
  CALL Species("C_5H_{11}NO_6",                                "", "KOIOFJRFFFLZDV-UHFFFAOYSA-N", "MCM:INAOH", "")
  CALL Species("C_5H_{11}NO_7",                                "", "JIJPRURLLPNXAY-UHFFFAOYSA-N", "MCM:INAOOH", "")
  CALL Species("C_5H_9NO_9",                                   "", "UQVCIXBCKWUTPD-UHFFFAOYSA-N", "MCM:INB1HPCO3H", "")
  CALL Species("C_5H_8N_2O_{11}",                              "", "MXDOILKWVFGWCS-UHFFFAOYSA-N", "MCM:INB1HPPAN", "")
  CALL Species("C_5H_8N_2O_{10}",                              "", "IWCHPIGOQCZQCT-UHFFFAOYSA-N", "MCM:INB1NACO3H", "")
  CALL Species("C_5H_7N_3O_{12}",                              "", "NYNQOGZUUKXJDZ-UHFFFAOYSA-N", "MCM:INB1NAPAN", "")
  CALL Species("C_5H_8N_2O_{10}",                              "", "KPBUCMMBNBINOH-UHFFFAOYSA-N", "MCM:INB1NBCO3H", "")
  CALL Species("C_5H_7N_3O_{12}",                              "", "JLOMTJWSNAHKLK-UHFFFAOYSA-N", "MCM:INB1NBPAN", "")
  CALL Species("C_5H_{10}N_2O_8",                              "", "JJLGQNDDHJZYHP-UHFFFAOYSA-N", "MCM:INB1NO3", "")
  CALL Species("C_5H_{11}NO_6",                                "", "CMMIHJKAMSCCNX-UHFFFAOYSA-N", "MCM:INB1OH", "")
  CALL Species("C_5H_{11}NO_7",                                "", "QLIFJAFKYUVUMD-UHFFFAOYSA-N", "MCM:INB1OOH", "")
  CALL Species("C_5H_{11}NO_7",                                "", "KNHLEPZPJYMJKZ-UHFFFAOYSA-N", "MCM:INB2OOH", "")
  CALL Species("C_5H_8N_2O_{10}",                              "", "OBBLXUYPBVASEH-UHFFFAOYSA-N", "MCM:INCNCO3H", "")
  CALL Species("C_5H_{10}N_2O_8",                              "", "YANCNBBJBURWKL-UHFFFAOYSA-N", "MCM:INCNO3", "")
  CALL Species("C_5H_7N_3O_{12}",                              "", "MINPDRSMAYIHHB-UHFFFAOYSA-N", "MCM:INCNPAN", "")
  CALL Species("C_5H_{11}NO_6",                                "", "CLYWBEAWDTVRBE-UHFFFAOYSA-N", "MCM:INCOH", "")
  CALL Species("C_5H_{11}NO_7",                                "", "LZOWWVNBCGLMRZ-UHFFFAOYSA-N", "MCM:INCOOH", "")
  CALL Species("C_5H_9NO_8",                                   "", "QUYOEJZVAFIYRP-UHFFFAOYSA-N", "MCM:INDHCO3H", "")
  CALL Species("C_5H_8N_2O_{10}",                              "", "WUJYJPIJVADQSX-UHFFFAOYSA-N", "MCM:INDHPAN", "")
  CALL Species("C_5H_9NO_9",                                   "", "XHWXDLVQLOECHL-UHFFFAOYSA-N", "MCM:INDHPCO3H", "")
  CALL Species("C_5H_8N_2O_{11}",                              "", "MNXVJBKQIAYCJC-UHFFFAOYSA-N", "MCM:INDHPPAN", "")
  CALL Species("C_5H_{11}NO_6",                                "", "HMAKIHHEIIANEM-UHFFFAOYSA-N", "MCM:INDOH", "")
  CALL Species("C_5H_{11}NO_7",                                "", "MAYHMSJZSHQSOH-UHFFFAOYSA-N", "MCM:INDOOH", "")
  CALL Species("C_5H_9NO_4",                                   "", "ISDFXKLTMWDHJL-UHFFFAOYSA-N", "MCM:ISOPANO3", "")
  CALL Species("C_5H_9NO_4",                                   "", "CIXVZPWFWFMUOH-UHFFFAOYSA-N", "MCM:ISOPBNO3", "")
  CALL Species("C_5H_9NO_4",                                   "", "IDJHVSOIJITEQU-UHFFFAOYSA-N", "MCM:ISOPCNO3", "")
  CALL Species("C_5H_9NO_4",                                   "", "PYTOMGVARIWPAT-UHFFFAOYSA-N", "MCM:ISOPDNO3", "")
  CALL Species("C_5H_{11}NO_4",                                "", "AGMVBVSNQJHQAX-UHFFFAOYSA-N", "MCM:M2BU2OLNO3", "")
  CALL Species("C_5H_{11}NO_4",                                "", "RQINUHXDGZGUIX-UHFFFAOYSA-N", "MCM:M2BUOL2NO3", "")
  CALL Species("C_5H_{11}NO_4",                                "", "HJMHUJWXCZNPJL-UHFFFAOYSA-N", "MCM:M3BU2OLNO3", "")
  CALL Species("C_5H_{11}NO_5",                                "", "SMFXIBRHTKUPLI-UHFFFAOYSA-N", "MCM:MBOANO3", "")
  CALL Species("C_5H_{11}NO_5",                                "", "DYXSFFPMZZUQFC-UHFFFAOYSA-N", "MCM:MBOBNO3", "")
  CALL Species("C_5H_{11}NO_4",                                "", "SBGDAJKUHMJEMT-UHFFFAOYSA-N", "MCM:ME2BUOLNO3", "")
  CALL Species("C_5H_{11}NO_4",                                "", "YWTWLHUMBHVVJS-UHFFFAOYSA-N", "MCM:ME3BUOLNO3", "")
  CALL Species("C_5H_9NO_7",                                   "", "PAYOKBFCVVFNAB-UHFFFAOYSA-N", "MCM:NC4OHCO3H", "")
  CALL Species("C_5H_8N_2O_9",                                 "", "JNBJPCXCCYUJJV-UHFFFAOYSA-N", "MCM:NC4OHCPAN", "")
  CALL Species("C_5H_{10}N_2O_9",                              "", "NSZZUBPWOYNZJW-UHFFFAOYSA-N", "MCM:NC524NO3", "")
  CALL Species("C_5H_{11}NO_7",                                "", "ZZGZKDXCMVGDSK-UHFFFAOYSA-N", "MCM:NC524OH", "")
  CALL Species("C_5H_{11}NO_8",                                "", "IVLBZHUBMDVGDP-UHFFFAOYSA-N", "MCM:NC524OOH", "")
  CALL Species("C_5H_{11}NO_6",                                "", "JWWMAKNKCQEFPZ-UHFFFAOYSA-N", "MCM:NMBOAOOH", "")
  CALL Species("C_5H_{11}NO_6",                                "", "ZVOCBPRLBFFNTM-UHFFFAOYSA-N", "MCM:NMBOBOOH", "")
  CALL Species("C_5H_{11}NO_4",                                "", "VVLSYMLJBKJHQI-UHFFFAOYSA-N", "MCM:PE1ENEANO3", "")
  CALL Species("C_5H_{11}NO_4",                                "", "ZAYLQHAUCSLXCO-UHFFFAOYSA-N", "MCM:PE1ENEBNO3", "")
  CALL Species("C_5H_{11}NO_4",                                "", "SNSYWPGYFWSVQC-UHFFFAOYSA-N", "MCM:PE2ENEANO3", "")
  CALL Species("C_5H_{11}NO_4",                                "", "FSHUAFNCTFMDNJ-UHFFFAOYSA-N", "MCM:PE2ENEBNO3", "")
  CALL Species("C_5H_{11}NO_4",                                "", "XLOGQOLFBMMDEL-UHFFFAOYSA-N", "MCM:PROL11MNO3", "")
  CALL Species("C_5H_9NO_6",                                   "", "KGIFHBAEXLJKCU-UHFFFAOYSA-N", "MCM:PROL1MPAN", "")
  CALL Species("C_6H_7NO_6",                                   "", "RBJPNJQUNYROJW-UHFFFAOYSA-N", "MCM:BZBIPERNO3", "")
  CALL Species("C_6H_{13}NO_4",                                "", "KWGTWPROGIVFDL-UHFFFAOYSA-N", "MCM:C4ME2OHNO3", "")
  CALL Species("C_6H_9NO_6",                                   "", "SMGAVNKPBYTONP-UHFFFAOYSA-N", "MCM:C518PAN", "")
  CALL Species("C_6H_{11}NO_4",                                "", "UCAFMJJTBHTENA-UHFFFAOYSA-N", "MCM:C622NO3", "")
  CALL Species("C_6H_{11}NO_4",                                "", "NPRBCQJDIIBUMY-UHFFFAOYSA-N", "MCM:C624NO3", "")
  CALL Species("C_6H_{13}NO_4",                                "", "SZVWRLIYDKPIRS-UHFFFAOYSA-N", "MCM:C64OH5NO3", "")
  CALL Species("C_6H_{13}NO_4",                                "", "ZZPOFDLVCBSRPC-UHFFFAOYSA-N", "MCM:C65OH4NO3", "")
  CALL Species("C_6H_{13}NO_4",                                "", "FGVUHVRIXFWTDQ-UHFFFAOYSA-N", "MCM:C66NO35OH", "")
  CALL Species("C_6H_{13}NO_4",                                "", "WNZDORMXFCCFFY-UHFFFAOYSA-N", "MCM:C6OH5NO3", "")
  CALL Species("C_6H_{11}NO_6",                                "", "MMZOJEONIKVQSZ-UHFFFAOYSA-N", "MCM:C6PAN11", "")
  CALL Species("C_6H_{11}NO_6",                                "", "UVFJOYDNGFNBNV-UHFFFAOYSA-N", "MCM:C6PAN13", "")
  CALL Species("C_6H_{11}NO_6",                                "", "OFISHWPBQMHPEB-UHFFFAOYSA-N", "MCM:C6PAN14", "")
  CALL Species("C_6H_{11}NO_6",                                "", "ZTVAXWHNQKLGPW-UHFFFAOYSA-N", "MCM:C6PAN19", "")
  CALL Species("C_6H_{11}NO_6",                                "", "INIWXNWKFGUMOJ-UHFFFAOYSA-N", "MCM:C6PAN1", "")
  CALL Species("C_6H_{11}NO_6",                                "", "HBAXDJNNSYEITK-UHFFFAOYSA-N", "MCM:C6PAN20", "")
  CALL Species("C_6H_{11}NO_6",                                "", "PXGXPDBWNDXFTJ-UHFFFAOYSA-N", "MCM:C6PAN21", "")
  CALL Species("C_6H_{11}NO_6",                                "", "YIFZDOMLXOKIHL-UHFFFAOYSA-N", "MCM:C6PAN22", "")
  CALL Species("C_6H_{11}NO_6",                                "", "OODNPGDDVDMKSB-UHFFFAOYSA-N", "MCM:C6PAN23", "")
  CALL Species("C_6H_{11}NO_6",                                "", "ZWWUNLHZACGCEV-UHFFFAOYSA-N", "MCM:C6PAN4", "")
  CALL Species("C_6H_{11}NO_6",                                "", "UOLRWSMDFSCHJH-UHFFFAOYSA-N", "MCM:C6PAN8", "")
  CALL Species("C_6H_{11}NO_4",                                "", "GVSCAKVHXWAQQJ-UHFFFAOYSA-N", "MCM:CYHXOLANO3", "")
  CALL Species("C_6H_{13}NO_5",                                "", "ZRQJKBICSYYJDC-UHFFFAOYSA-N", "MCM:H13M3C5NO3", "")
  CALL Species("C_6H_{13}NO_4",                                "", "YQESDLQNDPCVPT-UHFFFAOYSA-N", "MCM:H1MC5NO3", "")
  CALL Species("C_6H_{13}NO_4",                                "", "OQOPDPWKHFKGPO-UHFFFAOYSA-N", "MCM:H2MC5NO3", "")
  CALL Species("C_6H_{13}NO_4",                                "", "KFPWVQORUDKTSF-UHFFFAOYSA-N", "MCM:H3M3C5NO3", "")
  CALL Species("C_6H_{13}NO_4",                                "", "VALJBIPPHZOXME-UHFFFAOYSA-N", "MCM:HM22C4NO3", "")
  CALL Species("C_6H_{13}NO_4",                                "", "VYFSYIPQAVXJDI-UHFFFAOYSA-N", "MCM:HM23C4NO3", "")
  CALL Species("C_6H_{13}NO_4",                                "", "HUEIARDLQBFSAX-UHFFFAOYSA-N", "MCM:HM33C4NO3", "")
  CALL Species("C_6H_{13}NO_4",                                "", "ITCMBKDCMKPMFX-UHFFFAOYSA-N", "MCM:HO1C6NO3", "")
  CALL Species("C_6H_{13}NO_4",                                "", "HKEZUADWXQLOIL-UHFFFAOYSA-N", "MCM:HO1MC5NO3", "")
  CALL Species("C_6H_{13}NO_4",                                "", "PAWWQEMXBSLVJC-UHFFFAOYSA-N", "MCM:HO2C6NO3", "")
  CALL Species("C_6H_{13}NO_4",                                "", "VYPLOHRVLBPDEY-UHFFFAOYSA-N", "MCM:HO2M2C5NO3", "")
  CALL Species("C_6H_{13}NO_4",                                "", "HYCOTEZKEHIINQ-UHFFFAOYSA-N", "MCM:HO2MC5NO3", "")
  CALL Species("C_6H_{13}NO_4",                                "", "DLCRGHMYOOYQLW-UHFFFAOYSA-N", "MCM:HO3C6NO3", "")
  CALL Species("C_6H_5N_3O_{12}",                              "", "CXGWHQULUCBXTG-UHFFFAOYSA-N", "MCM:NDNPHENOOH", "")
  CALL Species("C_6H_6N_2O_{11}",                              "", "SEZKEDQZGXKDIE-UHFFFAOYSA-N", "MCM:NNCATECOOH", "")
  CALL Species("C_6H_7NO_7",                                   "", "RMHIEAGXGVAIOV-UHFFFAOYSA-N", "MCM:NPHENOH", "")
  CALL Species("C_6H_7NO_8",                                   "", "AMKSGTJAMDYUMG-UHFFFAOYSA-N", "MCM:NPHENOOH", "")
  CALL Species("C_7H_{11}NO_6",                                "", "YZCFVDKMDQZEJM-UHFFFAOYSA-N", "MCM:C622PAN", "")
  CALL Species("C_7H_{11}NO_6",                                "", "GVGFFJILZZBAQC-UHFFFAOYSA-N", "MCM:C624PAN", "")
  CALL Species("C_7H_{11}NO_4",                                "", "VRUSDNGIDALYSE-UHFFFAOYSA-N", "MCM:C720NO3", "")
  CALL Species("C_7H_{13}NO_6",                                "", "HWQKKNUBUUPBAM-UHFFFAOYSA-N", "MCM:C7PAN1", "")
  CALL Species("C_7H_{13}NO_6",                                "", "LEPHBBVBSNQAKS-UHFFFAOYSA-N", "MCM:C7PAN2", "")
  CALL Species("C_7H_{15}NO_4",                                "", "CAEJQFHLOCBFOE-UHFFFAOYSA-N", "MCM:H2M2C65NO3", "")
  CALL Species("C_7H_{15}NO_4",                                "", "GNQNFXQMHRYYMA-UHFFFAOYSA-N", "MCM:H2M4C65NO3", "")
  CALL Species("C_7H_{15}NO_4",                                "", "JWGQNCMAKPVWAL-UHFFFAOYSA-N", "MCM:H2M5C65NO3", "")
  CALL Species("C_7H_{15}NO_4",                                "", "HWCRTHIHVAVLJX-UHFFFAOYSA-N", "MCM:H3M3C6NO3", "")
  CALL Species("C_7H_{15}NO_4",                                "", "FLDFLEPRHCTYNR-UHFFFAOYSA-N", "MCM:HO3C76NO3", "")
  CALL Species("C_7H_8N_2O_{11}",                              "", "QARZHCGSBCBWKI-UHFFFAOYSA-N", "MCM:MNNCATCOOH", "")
  CALL Species("C_7H_9NO_7",                                   "", "DNQMNVUMZARJRH-UHFFFAOYSA-N", "MCM:NCRESOH", "")
  CALL Species("C_7H_9NO_8",                                   "", "CBYLLFTZDVJCIH-UHFFFAOYSA-N", "MCM:NCRESOOH", "")
  CALL Species("C_7H_7N_3O_{12}",                              "", "LRLYMWFOIDHJSH-UHFFFAOYSA-N", "MCM:NDNCRESOOH", "")
  CALL Species("C_7H_9NO_6",                                   "", "BBVQCFWACDVWPL-UHFFFAOYSA-N", "MCM:TLBIPERNO3", "")
  CALL Species("C_8H_{15}NO_6",                                "", "JRARROZAJMGAPP-UHFFFAOYSA-N", "MCM:C8PAN1", "")
  CALL Species("C_8H_{11}NO_6",                                "", "GVVNXZZYBMLIBS-UHFFFAOYSA-N", "MCM:EBZBPERNO3", "")
  CALL Species("C_8H_{10}N_2O_{11}",                           "", "SUCBZAKUUVBJMN-UHFFFAOYSA-N", "MCM:ENNCATCOOH", "")
  CALL Species("C_8H_{17}NO_4",                                "", "WNECKFLKAYSMQF-UHFFFAOYSA-N", "MCM:HO3C86NO3", "")
  CALL Species("C_8H_{10}N_2O_{11}",                           "", "SNJAPGUPFXVMJK-UHFFFAOYSA-N", "MCM:MXNNCATOOH", "")
  CALL Species("C_8H_{11}NO_6",                                "", "QFCKPTYGEYQVET-UHFFFAOYSA-N", "MCM:MXYBIPENO3", "")
  CALL Species("C_8H_9N_3O_{12}",                              "", "XZASAIPCUIAVQG-UHFFFAOYSA-N", "MCM:NDNEBZLOOH", "")
  CALL Species("C_8H_9N_3O_{12}",                              "", "AAXPRTISYYDKLT-UHFFFAOYSA-N", "MCM:NDNMXYLOOH", "")
  CALL Species("C_8H_9N_3O_{12}",                              "", "SEMBUWBKKSEFTL-UHFFFAOYSA-N", "MCM:NDNOXYLOOH", "")
  CALL Species("C_8H_9N_3O_{12}",                              "", "YXONGYHUJBRFJO-UHFFFAOYSA-N", "MCM:NDNPXYLOOH", "")
  CALL Species("C_8H_{11}NO_7",                                "", "GKEIZOZEJLMBMT-UHFFFAOYSA-N", "MCM:NEBNZOLOH", "")
  CALL Species("C_8H_{11}NO_8",                                "", "DZFBJORWJYJOSR-UHFFFAOYSA-N", "MCM:NEBNZOLOOH", "")
  CALL Species("C_8H_{11}NO_7",                                "", "SFEIDLQIDXHPNL-UHFFFAOYSA-N", "MCM:NMXYOLOH", "")
  CALL Species("C_8H_{11}NO_8",                                "", "ZTRVHLWFIBLUPT-UHFFFAOYSA-N", "MCM:NMXYOLOOH", "")
  CALL Species("C_8H_{11}NO_7",                                "", "UBRWKHDWLQNZCI-UHFFFAOYSA-N", "MCM:NOXYOLOH", "")
  CALL Species("C_8H_{11}NO_8",                                "", "UHJNRWMXKFNWGQ-UHFFFAOYSA-N", "MCM:NOXYOLOOH", "")
  CALL Species("C_8H_{11}NO_7",                                "", "DMOUSJNYQFTOEV-UHFFFAOYSA-N", "MCM:NPXYOLOH", "")
  CALL Species("C_8H_{11}NO_8",                                "", "XNVGITFEYRRJBZ-UHFFFAOYSA-N", "MCM:NPXYOLOOH", "")
  CALL Species("C_8H_{10}N_2O_{11}",                           "", "GYUDSTHOULZHIU-UHFFFAOYSA-N", "MCM:OXNNCATOOH", "")
  CALL Species("C_8H_{11}NO_6",                                "", "QQSYVIZUYAXYBZ-UHFFFAOYSA-N", "MCM:OXYBIPENO3", "")
  CALL Species("C_8H_{10}N_2O_{11}",                           "", "PMLOPKQQVRFHBT-UHFFFAOYSA-N", "MCM:PXNNCATOOH", "")
  CALL Species("C_8H_{11}NO_6",                                "", "DHJHBQWQGWNRCX-UHFFFAOYSA-N", "MCM:PXYBIPENO3", "")
  CALL Species("C_9H_{17}NO_6",                                "", "XXDOIRAKHRPRCF-UHFFFAOYSA-N", "MCM:C9PAN1", "")
  CALL Species("C_9H_{19}NO_4",                                "", "IBTJZAOBCFQTAU-UHFFFAOYSA-N", "MCM:HO3C96NO3", "")
  CALL Species("C_9H_{13}NO_6",                                "", "VLTHUNWGYHRCPS-UHFFFAOYSA-N", "MCM:IPBZBPRNO3", "")
  CALL Species("C_9H_{12}N_2O_{11}",                           "", "QCQAPHFHFZPQHQ-UHFFFAOYSA-N", "MCM:IPNNCATOOH", "")
  CALL Species("C_9H_{13}NO_6",                                "", "VQZACYIDHNLNBQ-UHFFFAOYSA-N", "MCM:METLBIPNO3", "")
  CALL Species("C_9H_{12}N_2O_{11}",                           "", "ILPSXJBNDMCAEH-UHFFFAOYSA-N", "MCM:MTNNCATOOH", "")
  CALL Species("C_9H_{11}N_3O_{12}",                           "", "HEZPBZRCUHPAHJ-UHFFFAOYSA-N", "MCM:NDNIPBLOOH", "")
  CALL Species("C_9H_{11}N_3O_{12}",                           "", "GVOHWVMFRUNQQB-UHFFFAOYSA-N", "MCM:NDNMETLOOH", "")
  CALL Species("C_9H_{11}N_3O_{12}",                           "", "DLTVFZDBANLVJD-UHFFFAOYSA-N", "MCM:NDNOETLOOH", "")
  CALL Species("C_9H_{11}N_3O_{12}",                           "", "JBFCRLZRPRJLGG-UHFFFAOYSA-N", "MCM:NDNPBZLOOH", "")
  CALL Species("C_9H_{11}N_3O_{12}",                           "", "HIQGLTMZAYSJOX-UHFFFAOYSA-N", "MCM:NDNPETLOOH", "")
  CALL Species("C_9H_{11}N_3O_{12}",                           "", "UGLBOUATFGGZQD-UHFFFAOYSA-N", "MCM:NDNT123OOH", "")
  CALL Species("C_9H_{11}N_3O_{12}",                           "", "JFYUNHMAYFMSCF-UHFFFAOYSA-N", "MCM:NDNT124OOH", "")
  CALL Species("C_9H_{13}NO_7",                                "", "IAYADIRZIVDBNK-UHFFFAOYSA-N", "MCM:NIPBNZOLOH", "")
  CALL Species("C_9H_{13}NO_8",                                "", "UJRLUYDBOZESDX-UHFFFAOYSA-N", "MCM:NIPBZOLOOH", "")
  CALL Species("C_9H_{13}NO_7",                                "", "JUUDFVZFULKTPV-UHFFFAOYSA-N", "MCM:NMETOLOH", "")
  CALL Species("C_9H_{13}NO_8",                                "", "CCCPQNSHVNXETJ-UHFFFAOYSA-N", "MCM:NMETOLOOH", "")
  CALL Species("C_9H_{13}NO_7",                                "", "RRTVORKFFCXYCH-UHFFFAOYSA-N", "MCM:NOETOLOH", "")
  CALL Species("C_9H_{13}NO_8",                                "", "CGRZNCSNBNPNBR-UHFFFAOYSA-N", "MCM:NOETOLOOH", "")
  CALL Species("C_9H_{13}NO_7",                                "", "PSGNFPQCVGIEDF-UHFFFAOYSA-N", "MCM:NPBNZOLOH", "")
  CALL Species("C_9H_{13}NO_8",                                "", "SNFHXCYEBCZPSO-UHFFFAOYSA-N", "MCM:NPBNZOLOOH", "")
  CALL Species("C_9H_{13}NO_7",                                "", "ILVSHXKPUYWBMX-UHFFFAOYSA-N", "MCM:NPETOLOH", "")
  CALL Species("C_9H_{13}NO_8",                                "", "BKSLAYLCQMXSES-UHFFFAOYSA-N", "MCM:NPETOLOOH", "")
  CALL Species("C_9H_{13}NO_8",                                "", "BTWUGVBELCWISQ-UHFFFAOYSA-N", "MCM:NTM123LOOH", "")
  CALL Species("C_9H_{13}NO_7",                                "", "DXGFZPSQDYIOOU-UHFFFAOYSA-N", "MCM:NTM123OLOH", "")
  CALL Species("C_9H_{13}NO_8",                                "", "JDMNMHRINWGUFB-UHFFFAOYSA-N", "MCM:NTM124LOOH", "")
  CALL Species("C_9H_{13}NO_7",                                "", "RASHBSOBGFFOLP-UHFFFAOYSA-N", "MCM:NTM124OLOH", "")
  CALL Species("C_9H_{13}NO_8",                                "", "AUGMYCIDZMPUKI-UHFFFAOYSA-N", "MCM:NTM135LOOH", "")
  CALL Species("C_9H_{13}NO_7",                                "", "RAAKFDOLNJCXRG-UHFFFAOYSA-N", "MCM:NTM135OLOH", "")
  CALL Species("C_9H_{13}NO_6",                                "", "FQBYEMVYEQQXMF-UHFFFAOYSA-N", "MCM:OETLBIPNO3", "")
  CALL Species("C_9H_{12}N_2O_{11}",                           "", "QVFSJCAPJVJPQJ-UHFFFAOYSA-N", "MCM:OTNNCATOOH", "")
  CALL Species("C_9H_{13}NO_6",                                "", "XWXMYNZXCPDSAQ-UHFFFAOYSA-N", "MCM:PBZBPERNO3", "")
  CALL Species("C_9H_{13}NO_6",                                "", "SOIPWHDZHXSJJN-UHFFFAOYSA-N", "MCM:PETLBIPNO3", "")
  CALL Species("C_9H_{12}N_2O_{11}",                           "", "VZHROOMDTDJZFR-UHFFFAOYSA-N", "MCM:PNNCATCOOH", "")
  CALL Species("C_9H_{12}N_2O_{11}",                           "", "DSPOEZNZVSBNSI-UHFFFAOYSA-N", "MCM:PTNNCATOOH", "")
  CALL Species("C_9H_{13}NO_9",                                "", "QTWHDHJHZAXHRP-UHFFFAOYSA-N", "MCM:T123NCTOOH", "")
  CALL Species("C_9H_{12}N_2O_{11}",                           "", "DNVSLQYQQCSWFT-UHFFFAOYSA-N", "MCM:T123NNCOOH", "")
  CALL Species("C_9H_{13}NO_9",                                "", "TXEYLBVXEVQRAA-UHFFFAOYSA-N", "MCM:T124NCTOOH", "")
  CALL Species("C_9H_{12}N_2O_{11}",                           "", "ATYVZJLHFNLHKS-UHFFFAOYSA-N", "MCM:T124NNCOOH", "")
  CALL Species("C_9H_{13}NO_6",                                "", "RQYOJMSOWIZBID-UHFFFAOYSA-N", "MCM:TM123BPNO3", "")
  CALL Species("C_9H_{13}NO_6",                                "", "KUPKWWJOBRXSMW-UHFFFAOYSA-N", "MCM:TM124BPNO3", "")
  CALL Species("C_9H_{13}NO_6",                                "", "VYQMVFHQGVVJTP-UHFFFAOYSA-N", "MCM:TM135BPNO3", "")
  CALL Species("C_{10}H_{17}NO_4",                             "", "YITBNQHGOKCTCE-UHFFFAOYSA-N", "MCM:APINANO3", "")
  CALL Species("C_{10}H_{17}NO_4",                             "", "AFJBVFUZFQZZRG-UHFFFAOYSA-N", "MCM:APINBNO3", "")
  CALL Species("C_{10}H_{17}NO_4",                             "", "IHZLDCJLQGWYSD-UHFFFAOYSA-N", "MCM:APINCNO3", "")
  CALL Species("C_{10}H_{17}NO_4",                             "", "LIKORQJDVXFFRR-UHFFFAOYSA-N", "MCM:BPINANO3", "")
  CALL Species("C_{10}H_{17}NO_4",                             "", "PIICVRQEXAAMFO-UHFFFAOYSA-N", "MCM:BPINBNO3", "")
  CALL Species("C_{10}H_{17}NO_4",                             "", "IVSZOCZGKNSHLW-UHFFFAOYSA-N", "MCM:BPINCNO3", "")
  CALL Species("C_{10}H_{19}NO_6",                             "", "PVQDOCOVRZVCAL-UHFFFAOYSA-N", "MCM:C10PAN1", "")
  CALL Species("C_{10}H_{15}NO_6",                             "", "GIWCNEZLGDWGNI-UHFFFAOYSA-N", "MCM:C918PAN", "")
  CALL Species("C_{10}H_{15}NO_6",                             "", "LWYZBKFMLLCHQU-UHFFFAOYSA-N", "MCM:DMEBIPNO3", "")
  CALL Species("C_{10}H_{21}NO_4",                             "", "LRSOGLAVONYUMB-UHFFFAOYSA-N", "MCM:HO3C106NO3", "")
  CALL Species("C_{10}H_{17}NO_4",                             "", "LZDKYYHMAURBIK-UHFFFAOYSA-N", "MCM:LIMANO3", "")
  CALL Species("C_{10}H_{17}NO_4",                             "", "WIRXFNWUDJEZSG-UHFFFAOYSA-N", "MCM:LIMBNO3", "")
  CALL Species("C_{10}H_{17}NO_4",                             "", "ZELLOEPLERCREX-UHFFFAOYSA-N", "MCM:LIMCNO3", "")
  CALL Species("C_{10}H_{15}NO_7",                             "", "XEZYIWLRTFENIO-UHFFFAOYSA-N", "MCM:NDMEPHOLOH", "")
  CALL Species("C_{10}H_{15}NO_8",                             "", "CUWVWZNCHDONFF-UHFFFAOYSA-N", "MCM:NDMEPLOOH", "")
  CALL Species("C_{11}H_{17}NO_6",                             "", "HUKONYHKUULPQM-UHFFFAOYSA-N", "MCM:DETLBIPNO3", "")
  CALL Species("C_{11}H_{23}NO_4",                             "", "ZSRHOFPLHKNYIP-UHFFFAOYSA-N", "MCM:HO3C116NO3", "")
  CALL Species("C_{11}H_{17}NO_7",                             "", "SFAWLNJVZQJDAW-UHFFFAOYSA-N", "MCM:NDEMPHOLOH", "")
  CALL Species("C_{11}H_{17}NO_8",                             "", "KQXYYOILDOZYPH-UHFFFAOYSA-N", "MCM:NDEMPLOOH", "")
  CALL Species("C_{12}H_{25}NO_4",                             "", "NBVYOETUGSGBPE-UHFFFAOYSA-N", "MCM:HO3C126NO3", "")
  CALL Species("C_{15}H_{25}NO_4",                             "", "DXJZLWPVAKABFC-UHFFFAOYSA-N", "MCM:BCANO3", "")
  CALL Species("C_{15}H_{25}NO_4",                             "", "GGDFBGYMWXIISJ-UHFFFAOYSA-N", "MCM:BCBNO3", "")
  CALL Species("C_{15}H_{25}NO_4",                             "", "AJHNYRDSMBHFTD-UHFFFAOYSA-N", "MCM:BCCNO3", "")
  CALL Species("C_2HNO_6",                                     "", "WWNQODGUPABTBB-UHFFFAOYSA-N", "MCM:GLYPAN", "")
  CALL Species("C_2H_3NO_4",                                   "", "ABUBKAMLUVOXSP-UHFFFAOYSA-N", "MCM:NO3CH2CHO", "")
  CALL Species("C_3H_3NO_6",                                   "", "JCWPFKBPQPIIIA-UHFFFAOYSA-N", "MCM:C3PAN2", "")
  CALL Species("C_3H_5NO_4",                                   "", "IIFXHQMWCHWMFS-UHFFFAOYSA-N", "MCM:CHOPRNO3", "")
  CALL Species("C_4H_7NO_4",                                   "", "JTMAFXAEAWTSPM-UHFFFAOYSA-N", "MCM:BUTAL2NO3", "")
  CALL Species("C_4H_7NO_4",                                   "", "YGQOBBYMNDZOFN-UHFFFAOYSA-N", "MCM:BUTALNO3", "")
  CALL Species("C_4H_7NO_6",                                   "", "OPYZRKWHBYWVFV-UHFFFAOYSA-N", "MCM:C4CONO3OOH", "")
  CALL Species("C_4H_7NO_6",                                   "", "LPNJZRUFCUXQKM-UHFFFAOYSA-N", "MCM:C4NO3COOOH", "")
  CALL Species("C_4H_5NO_6",                                   "", "RNHARUXEGCOCNU-UHFFFAOYSA-N", "MCM:CHOC2PAN", "")
  CALL Species("C_4H_5NO_5",                                   "", "UXLHNUQVUCPVFF-UHFFFAOYSA-N", "MCM:CONM2CHO", "")
  CALL Species("C_4H_5NO_7",                                   "", "KDPSYMPAERQTCV-UHFFFAOYSA-N", "MCM:CONM2CO3H", "")
  CALL Species("C_4H_4N_2O_9",                                 "", "PRUSZGKEVUCSBM-UHFFFAOYSA-N", "MCM:CONM2PAN", "")
  CALL Species("C_4H_7NO_4",                                   "", "BBEYVRKCPRBWGK-UHFFFAOYSA-N", "MCM:IBUALANO3", "")
  CALL Species("C_4H_5NO_6",                                   "", "ANKFJODHGNELSW-UHFFFAOYSA-N", "MCM:IBUDIALPAN", "")
  CALL Species("C_4H_3NO_6",                                   "", "JVYQADFHFGBZRY-UHFFFAOYSA-N", "MCM:MALDIALPAN", "")
  CALL Species("C_4H_7NO_4",                                   "", "ZFRSNPXDNOXJAQ-UHFFFAOYSA-N", "MCM:MPRBNO3CHO", "")
  CALL Species("C_4H_5NO_4",                                   "", "IRCIUAVPVOGPAB-UHFFFAOYSA-N", "MCM:NC3CHO", "")
  CALL Species("C_5H_9NO_4",                                   "", "FSONNVXXYIKSKL-UHFFFAOYSA-N", "MCM:C3M3CHONO3", "")
  CALL Species("C_5H_5NO_6",                                   "", "ABUKHSDGKGHQTM-UHFFFAOYSA-N", "MCM:C3MCODBPAN", "")
  CALL Species("C_5H_9NO_4",                                   "", "IIGIHDSEOSVDKL-UHFFFAOYSA-N", "MCM:C3MNO3CHO", "")
  CALL Species("C_5H_9NO_4",                                   "", "IRTPVYBICUATCF-UHFFFAOYSA-N", "MCM:C4CHOBNO3", "")
  CALL Species("C_5H_9NO_4",                                   "", "WGTULXFSTHCAFV-UHFFFAOYSA-N", "MCM:C4NO3CHO", "")
  CALL Species("C_5H_7NO_5",                                   "", "CEGGSEAULUFJEH-UHFFFAOYSA-N", "MCM:C514NO3", "")
  CALL Species("C_5H_9NO_4",                                   "", "OIPNOUWHATXHHM-UHFFFAOYSA-N", "MCM:C52NO31CO", "")
  CALL Species("C_5H_7NO_6",                                   "", "JNBWLVYXFYINSW-UHFFFAOYSA-N", "MCM:CO1M22PAN", "")
  CALL Species("C_5H_5NO_6",                                   "", "AENWJVOXGCGHKR-UHFFFAOYSA-N", "MCM:MC3CODBPAN", "")
  CALL Species("C_5H_7NO_4",                                   "", "FHQODWHGFJXLCS-UHFFFAOYSA-N", "MCM:NC4CHO", "")
  CALL Species("C_6H_7NO_6",                                   "", "BLDVCFQHUDZGIY-UHFFFAOYSA-N", "MCM:C522PAN", "")
  CALL Species("C_6H_{11}NO_4",                                "", "AWOTULOVYXZRSX-UHFFFAOYSA-N", "MCM:C65NO36CHO", "")
  CALL Species("C_6H_9NO_6",                                   "", "LKJHECXHBJSEQK-UHFFFAOYSA-N", "MCM:CHOC4PAN", "")
  CALL Species("C_6H_{11}NO_4",                                "", "BSOUFWBHIRXMPX-UHFFFAOYSA-N", "MCM:CO1C6NO3", "")
  CALL Species("C_7H_9NO_7",                                   "", "QXDOZRAOCVERTF-UHFFFAOYSA-N", "MCM:C615PAN", "")
  CALL Species("C_7H_{11}NO_4",                                "", "IVMGVFSKIJXXOW-UHFFFAOYSA-N", "MCM:C729NO3", "")
  CALL Species("C_8H_{11}NO_6",                                "", "UQYKHYVNMQUYAN-UHFFFAOYSA-N", "MCM:C729PAN", "")
  CALL Species("C_8H_{13}NO_5",                                "", "CHBSGOHAWLCLGZ-UHFFFAOYSA-N", "MCM:C810NO3", "")
  CALL Species("C_8H_{13}NO_4",                                "", "JWAJNALJDDSHOA-UHFFFAOYSA-N", "MCM:C822NO3", "")
  CALL Species("C_8H_{13}NO_4",                                "", "FXBNRUYCAFEPCZ-UHFFFAOYSA-N", "MCM:C830NO3", "")
  CALL Species("C_8H_{13}NO_5",                                "", "OVEOYJGENOZMLJ-UHFFFAOYSA-N", "MCM:C831NO3", "")
  CALL Species("C_8H_{13}NO_4",                                "", "JVGNHSSLWSUTIF-UHFFFAOYSA-N", "MCM:C89NO3", "")
  CALL Species("C_8H_{13}NO_7",                                "", "LIEZCAAIBRALIF-UHFFFAOYSA-N", "MCM:NC826OOH", "")
  CALL Species("C_9H_{13}NO_6",                                "", "AEOJBADXVLGVNE-UHFFFAOYSA-N", "MCM:C822PAN", "")
  CALL Species("C_9H_{13}NO_6",                                "", "NSAJKAFLWKCEMC-UHFFFAOYSA-N", "MCM:C830PAN", "")
  CALL Species("C_9H_{13}NO_6",                                "", "MAIUDZACSGCMBD-UHFFFAOYSA-N", "MCM:C89PAN", "")
  CALL Species("C_{10}H_{15}NO_4",                             "", "AVQCYYCZMFYNOE-UHFFFAOYSA-N", "MCM:NC91CHO", "")
  CALL Species("C_{12}H_{19}NO_4",                             "", "LCRFSVKVQOPOBC-UHFFFAOYSA-N", "MCM:C126NO3", "")
  CALL Species("C_{13}H_{19}NO_6",                             "", "TVUSLNPYCCAOKX-UHFFFAOYSA-N", "MCM:C126PAN", "")
  CALL Species("C_{13}H_{21}NO_4",                             "", "FAQBCXSJSFDOSN-UHFFFAOYSA-N", "MCM:C136NO3", "")
  CALL Species("C_{13}H_{21}NO_7",                             "", "AIWUKZRYSUWIQN-UHFFFAOYSA-N", "MCM:NC1313OOH", "")
  CALL Species("C_{14}H_{21}NO_6",                             "", "HVYQJFOLIDVKQK-UHFFFAOYSA-N", "MCM:C136PAN", "")
  CALL Species("C_3H_5NO_5",                                   "", "CQVFGUOZVNZZAM-UHFFFAOYSA-N", "MCM:C42AOH", "")
  CALL Species("C_3H_3NO_7",                                   "", "GGQYFRDCXKQQIA-UHFFFAOYSA-N", "MCM:HCOCOHPAN", "")
  CALL Species("C_4H_7NO_6",                                   "", "SGYMVPWBXCYWGY-UHFFFAOYSA-N", "MCM:C41NO3", "")
  CALL Species("C_4H_7NO_6",                                   "", "OIAGCOLJHCONRV-UHFFFAOYSA-N", "MCM:C42OH", "")
  CALL Species("C_4H_7NO_7",                                   "", "KUKJNXCETLVUQA-UHFFFAOYSA-N", "MCM:C42OOH", "")
  CALL Species("C_4H_7NO_5",                                   "", "NEHFOAWUQWHZTI-UHFFFAOYSA-N", "MCM:C4OCCOHNO3", "")
  CALL Species("C_4H_7NO_5",                                   "", "HMCDHWALZVCNKO-UHFFFAOYSA-N", "MCM:COCCOHNO3", "")
  CALL Species("C_4H_5NO_7",                                   "", "UXUAXYPQPQSXRR-UHFFFAOYSA-N", "MCM:COHM2PAN", "")
  CALL Species("C_4H_7NO_5",                                   "", "DLZDCWJOERFHAS-UHFFFAOYSA-N", "MCM:MACRNB", "")
  CALL Species("C_4H_7NO_5",                                   "", "ALINXPRBVMXOGH-UHFFFAOYSA-N", "MCM:MACRNO3", "")
  CALL Species("C_5H_7NO_6",                                   "", "UOGOTHSJBRKVRR-UHFFFAOYSA-N", "MCM:C4M2ALOHNO3", "")
  CALL Species("C_5H_9NO_6",                                   "", "ZCOZZYNQPKXJEJ-UHFFFAOYSA-N", "MCM:C510OH", "")
  CALL Species("C_5H_9NO_7",                                   "", "GGBWELVRYZRQMP-UHFFFAOYSA-N", "MCM:C510OOH", "")
  CALL Species("C_5H_9NO_6",                                   "", "KHAJTJUOMDNQGW-UHFFFAOYSA-N", "MCM:C57NO3", "")
  CALL Species("C_5H_9NO_6",                                   "", "UZUIBKBJXOKLGO-UHFFFAOYSA-N", "MCM:C58NO3", "")
  CALL Species("C_5H_8N_2O_9",                                 "", "WXHJTLSKHGDJHS-UHFFFAOYSA-N", "MCM:DNC524CO", "")
  CALL Species("C_5H_9NO_7",                                   "", "UHQMTOKWUFCERL-UHFFFAOYSA-N", "MCM:HNC524CO", "")
  CALL Species("C_5H_9NO_8",                                   "", "QASKBBWYYSGCPQ-UHFFFAOYSA-N", "MCM:HPNC524CO", "")
  CALL Species("C_5H_9NO_6",                                   "", "GJDCYRXOLZPRRL-UHFFFAOYSA-N", "MCM:INAHCHO", "")
  CALL Species("C_5H_9NO_7",                                   "", "WOIAKHQAPCBXNP-UHFFFAOYSA-N", "MCM:INAHPCHO", "")
  CALL Species("C_5H_8N_2O_8",                                 "", "AOWQROMNZIXQRL-UHFFFAOYSA-N", "MCM:INANCHO", "")
  CALL Species("C_5H_9NO_7",                                   "", "PPUPDRIHXISUBS-UHFFFAOYSA-N", "MCM:INB1HPCHO", "")
  CALL Species("C_5H_8N_2O_8",                                 "", "FPDRKJBIBBHIBW-UHFFFAOYSA-N", "MCM:INB1NACHO", "")
  CALL Species("C_5H_8N_2O_8",                                 "", "DIJODJNNHWDXRN-UHFFFAOYSA-N", "MCM:INB1NBCHO", "")
  CALL Species("C_5H_8N_2O_8",                                 "", "QYWHAZCEYLWYSZ-UHFFFAOYSA-N", "MCM:INCNCHO", "")
  CALL Species("C_5H_9NO_6",                                   "", "LXJORHVMGZGMRK-UHFFFAOYSA-N", "MCM:INDHCHO", "")
  CALL Species("C_5H_9NO_7",                                   "", "NKPKKBYOCYFEQQ-UHFFFAOYSA-N", "MCM:INDHPCHO", "")
  CALL Species("C_5H_7NO_8",                                   "", "WWJYGONDDIBFHK-UHFFFAOYSA-N", "MCM:MMALNACO3H", "")
  CALL Species("C_5H_6N_2O_{10}",                              "", "JNHZFZYBXYVXPX-UHFFFAOYSA-N", "MCM:MMALNAPAN", "")
  CALL Species("C_5H_7NO_8",                                   "", "ASZGKTQVJXPLHM-UHFFFAOYSA-N", "MCM:MMALNBCO3H", "")
  CALL Species("C_5H_6N_2O_{10}",                              "", "QNDDWZGVLTYMIK-UHFFFAOYSA-N", "MCM:MMALNBPAN", "")
  CALL Species("C_5H_9NO_5",                                   "", "ZWLCCOCHHFERQT-UHFFFAOYSA-N", "MCM:NMBOBCO", "")
  CALL Species("C_6H_9NO_7",                                   "", "XKXYOEFJYDLELH-UHFFFAOYSA-N", "MCM:C1H4C5PAN", "")
  CALL Species("C_6H_{11}NO_6",                                "", "UTKYBRKMQKBGOV-UHFFFAOYSA-N", "MCM:C623NO3", "")
  CALL Species("C_6H_{11}NO_5",                                "", "MTROCBFSNOSSFX-UHFFFAOYSA-N", "MCM:C67NO3", "")
  CALL Species("C_6H_{11}NO_5",                                "", "DFCRZDMTYDUCKH-UHFFFAOYSA-N", "MCM:C68NO3", "")
  CALL Species("C_6H_{11}NO_5",                                "", "QWPYKWKOPQKYFO-UHFFFAOYSA-N", "MCM:CO1H63NO3", "")
  CALL Species("C_6H_{11}NO_6",                                "", "LIALSLHETRNECK-UHFFFAOYSA-N", "MCM:NC623OH", "")
  CALL Species("C_6H_{11}NO_7",                                "", "LUCHFEJMQFYPEG-UHFFFAOYSA-N", "MCM:NC623OOH", "")
  CALL Species("C_7H_{13}NO_6",                                "", "CDBHNFBQKSTCON-UHFFFAOYSA-N", "MCM:C728NO3", "")
  CALL Species("C_7H_{13}NO_6",                                "", "RVWIAVBELSCRER-UHFFFAOYSA-N", "MCM:C730NO3", "")
  CALL Species("C_7H_{13}NO_6",                                "", "RMGZPHZIJLBXBA-UHFFFAOYSA-N", "MCM:NC728OH", "")
  CALL Species("C_7H_{13}NO_7",                                "", "NMPAVXKBWDGSMK-UHFFFAOYSA-N", "MCM:NC728OOH", "")
  CALL Species("C_7H_{13}NO_6",                                "", "ZPFRZNAADTZJOS-UHFFFAOYSA-N", "MCM:NC730OH", "")
  CALL Species("C_7H_{13}NO_7",                                "", "LJYXGYOAXKBDIY-UHFFFAOYSA-N", "MCM:NC730OOH", "")
  CALL Species("C_8H_{13}NO_6",                                "", "HYUHCBAMOFNBPC-UHFFFAOYSA-N", "MCM:C826NO3", "")
  CALL Species("C_8H_{13}NO_6",                                "", "UXZRCYKYQRKKOI-UHFFFAOYSA-N", "MCM:NC826OH", "")
  CALL Species("C_{12}H_{19}NO_5",                             "", "RZJFRKYKWJUXLW-UHFFFAOYSA-N", "MCM:C127NO3", "")
  CALL Species("C_{13}H_{21}NO_5",                             "", "HIIZKUNLVYOKLJ-UHFFFAOYSA-N", "MCM:C1311NO3", "")
  CALL Species("C_{13}H_{21}NO_6",                             "", "SXMORIYKGFOKAB-UHFFFAOYSA-N", "MCM:C1313NO3", "")
  CALL Species("C_{13}H_{21}NO_6",                             "", "RAUQQNONBFDDJB-UHFFFAOYSA-N", "MCM:NC1313OH", "")
  CALL Species("C_3H_5NO_6",                                   "", "JXBSDKIVLYRFTK-UHFFFAOYSA-N", "MCM:C3NO3COOOH", "")
  CALL Species("C_3H_3NO_6",                                   "", "BNYSUSDQQIWDSZ-UHFFFAOYSA-N", "MCM:CH3COPAN", "")
  CALL Species("C_4H_7NO_4",                                   "", "KFRUNLQRYXGRRS-UHFFFAOYSA-N", "MCM:BUTONENO3", "")
  CALL Species("C_4H_5NO_5",                                   "", "RSORCZUYAJEINP-UHFFFAOYSA-N", "MCM:CO23C4NO3", "")
  CALL Species("C_4H_5NO_6",                                   "", "VXGDXYQGFKIYIJ-UHFFFAOYSA-N", "MCM:CO2C3PAN", "")
  CALL Species("C_4H_5NO_7",                                   "", "PLOOSZKRSUJKKF-UHFFFAOYSA-N", "MCM:CO2N3CO3H", "")
  CALL Species("C_4H_4N_2O_9",                                 "", "RXUYCSQEPKEYNB-UHFFFAOYSA-N", "MCM:CO2N3PAN", "")
  CALL Species("C_4H_7NO_4",                        "138779-12-1", "OVISQPFXRZOZFI-UHFFFAOYSA-N", "2-oxobutyl nitrate", "")
  CALL Species("C_4H_7NO_6",                                   "", "AEHFBVVGPAXMNG-UHFFFAOYSA-N", "MCM:COC4NO3OOH", "")
  CALL Species("C_4H_7NO_4",                                   "", "SQFMCLNZTZWUJY-UHFFFAOYSA-N", "MCM:MEKANO3", "")
  CALL Species("C_4H_5NO_4",                                   "", "YTEQPVIDLVIVHW-UHFFFAOYSA-N", "MCM:NMVK", "")
  CALL Species("C_5H_9NO_4",                                   "", "DIMIHFOKCQZMGY-UHFFFAOYSA-N", "MCM:C4M2NO3ONE", "")
  CALL Species("C_5H_9NO_4",                                   "", "SAHLZHFOPIFWKH-UHFFFAOYSA-N", "MCM:C4M3NO3ONE", "")
  CALL Species("C_5H_9NO_6",                                   "", "AOCDDMVLVLTZNT-UHFFFAOYSA-N", "MCM:C4MCNO3OOH", "")
  CALL Species("C_5H_7NO_5",                                   "", "KIDZPQUIRFWTHY-UHFFFAOYSA-N", "MCM:C51NO324CO", "")
  CALL Species("C_5H_9NO_4",                                   "", "TTZVKGPORDTQGG-UHFFFAOYSA-N", "MCM:C51NO32CO", "")
  CALL Species("C_5H_9NO_4",                                   "", "VOCQUTRGVIAJPG-UHFFFAOYSA-N", "MCM:C52NO33CO", "")
  CALL Species("C_5H_7NO_5",                                   "", "QYJXVKSLJJQONK-UHFFFAOYSA-N", "MCM:C53NO324CO", "")
  CALL Species("C_5H_9NO_4",                                   "", "LVGYWFUEHRGRSS-UHFFFAOYSA-N", "MCM:C53NO32CO", "")
  CALL Species("C_5H_9NO_6",                                   "", "ZDTBCADGPXXLKE-UHFFFAOYSA-N", "MCM:C5CONO3OOH", "")
  CALL Species("C_5H_5NO_6",                                   "", "BZPLAICQOBBIOR-UHFFFAOYSA-N", "MCM:C5COO2NO2", "")
  CALL Species("C_5H_9NO_6",                                   "", "CASCWEIOXSCVGJ-UHFFFAOYSA-N", "MCM:C5NO3O4OOH", "")
  CALL Species("C_5H_9NO_6",                                   "", "FLHIUJMTRYIPLY-UHFFFAOYSA-N", "MCM:C5NO3OAOOH", "")
  CALL Species("C_5H_9NO_6",                                   "", "LOZNRBAEPWQBFW-UHFFFAOYSA-N", "MCM:C5ONO34OOH", "")
  CALL Species("C_5H_7NO_6",                                   "", "IZEBTYZDWKBBFQ-UHFFFAOYSA-N", "MCM:C5PAN16", "")
  CALL Species("C_5H_7NO_6",                                   "", "XMORPXKVSPHXDA-UHFFFAOYSA-N", "MCM:C5PAN2", "")
  CALL Species("C_5H_7NO_6",                                   "", "VONRVKPFQAOOPX-UHFFFAOYSA-N", "MCM:C5PAN7", "")
  CALL Species("C_5H_5NO_7",                                   "", "HJZPHZLLKGPYCO-UHFFFAOYSA-N", "MCM:C5PAN9", "")
  CALL Species("C_5H_9NO_4",                                   "", "WKTFKAAKLZJYGP-UHFFFAOYSA-N", "MCM:DIEKBNO3", "")
  CALL Species("C_5H_6N_2O_{10}",                              "", "OKQCOTADQFFQES-UHFFFAOYSA-N", "MCM:INANCOCO3H", "")
  CALL Species("C_5H_5N_3O_{12}",                              "", "OHHXWPYTKFCNFA-UHFFFAOYSA-N", "MCM:INANCOPAN", "")
  CALL Species("C_5H_9NO_4",                                   "", "PNKPSVUHDDVFRC-UHFFFAOYSA-N", "MCM:MIPKBNO3", "")
  CALL Species("C_5H_9NO_4",                                   "", "GMHWHQCXUOCSQQ-UHFFFAOYSA-N", "MCM:MPRKNO3", "")
  CALL Species("C_6H_9NO_6",                                   "", "QAGGXZNGFPYUOO-UHFFFAOYSA-N", "MCM:C3COCPAN", "")
  CALL Species("C_6H_7NO_6",                                   "", "OBEVNDXPXGMJPT-UHFFFAOYSA-N", "MCM:C4MCODBPAN", "")
  CALL Species("C_6H_9NO_5",                                   "", "ABDNFJAHWTWRFH-UHFFFAOYSA-N", "MCM:C62NO335CO", "")
  CALL Species("C_6H_{11}NO_4",                                "", "OBVLXHVNJQTRAU-UHFFFAOYSA-N", "MCM:C62NO33CO", "")
  CALL Species("C_6H_{11}NO_4",                                "", "QSKMGJAWRGUYGL-UHFFFAOYSA-N", "MCM:C63NO32CO", "")
  CALL Species("C_6H_9NO_5",                                   "", "CHGSCHAFWLLPDM-UHFFFAOYSA-N", "MCM:C64NO335CO", "")
  CALL Species("C_6H_{11}NO_4",                                "", "NNYDFJRPKYRBEY-UHFFFAOYSA-N", "MCM:C66NO35CO", "")
  CALL Species("C_6H_7NO_7",                                   "", "OIZONYBQGYIWDV-UHFFFAOYSA-N", "MCM:C6CO134PAN", "")
  CALL Species("C_6H_{11}NO_6",                                "", "JAMSWIXAFBYGGR-UHFFFAOYSA-N", "MCM:C6CONO3OOH", "")
  CALL Species("C_6H_7NO_6",                                   "", "QTAROYORLXCHIO-UHFFFAOYSA-N", "MCM:C6DCARBPAN", "")
  CALL Species("C_6H_9NO_5",                                   "", "ZLHGCQIPJTVAEP-UHFFFAOYSA-N", "MCM:C6NO324CO", "")
  CALL Species("C_6H_{11}NO_6",                                "", "HQTZSBFILMVSJZ-UHFFFAOYSA-N", "MCM:C6NO3COOOH", "")
  CALL Species("C_6H_9NO_6",                                   "", "LSIJQLAUAXHOJI-UHFFFAOYSA-N", "MCM:C6PAN12", "")
  CALL Species("C_6H_9NO_6",                                   "", "JDUNWYCOIXFXGK-UHFFFAOYSA-N", "MCM:C6PAN16", "")
  CALL Species("C_6H_9NO_6",                                   "", "BSRDZZSTWMTZHH-UHFFFAOYSA-N", "MCM:C6PAN2", "")
  CALL Species("C_6H_9NO_6",                                   "", "AYORGEAGFIRFHW-UHFFFAOYSA-N", "MCM:C6PAN5", "")
  CALL Species("C_6H_9NO_6",                                   "", "STXATUQYWORJHN-UHFFFAOYSA-N", "MCM:C6PAN6", "")
  CALL Species("C_6H_7NO_7",                                   "", "FEUNFERSGUKVTQ-UHFFFAOYSA-N", "MCM:C6PAN7", "")
  CALL Species("C_6H_{11}NO_6",                                "", "BPHZXGKZHPQLPB-UHFFFAOYSA-N", "MCM:CONO3C6OOH", "")
  CALL Species("C_6H_9NO_4",                                   "", "VLCBNIODXGFPBM-UHFFFAOYSA-N", "MCM:CYHXONANO3", "")
  CALL Species("C_6H_5NO_8",                                   "", "GLZSBLKUFXIYNN-UHFFFAOYSA-N", "MCM:ECO3PAN", "")
  CALL Species("C_6H_{11}NO_4",                                "", "QPLZWEVCYZLYTL-UHFFFAOYSA-N", "MCM:EIPKBNO3", "")
  CALL Species("C_6H_{11}NO_4",                                "", "OZSNHPHDGAJMHG-UHFFFAOYSA-N", "MCM:HEX2ONANO3", "")
  CALL Species("C_6H_{11}NO_4",                                "", "JYVFNKYOALTNTA-UHFFFAOYSA-N", "MCM:HEX2ONBNO3", "")
  CALL Species("C_6H_{11}NO_4",                                "", "KTYGBCIWLHIEKP-UHFFFAOYSA-N", "MCM:HEX3ONANO3", "")
  CALL Species("C_6H_{11}NO_4",                                "", "PRTKDFINSKNZSU-UHFFFAOYSA-N", "MCM:HEX3ONDNO3", "")
  CALL Species("C_6H_{11}NO_4",                                "", "NYAAKBDXIQFYDD-UHFFFAOYSA-N", "MCM:M2BKANO3", "")
  CALL Species("C_6H_7NO_6",                                   "", "WHFHKQLMYAOVSY-UHFFFAOYSA-N", "MCM:MC4CODBPAN", "")
  CALL Species("C_6H_{11}NO_4",                                "", "PKJJSCYTBLHEMI-UHFFFAOYSA-N", "MCM:MIBKANO3", "")
  CALL Species("C_6H_{11}NO_4",                                "", "OYSBYTCKEAIMOW-UHFFFAOYSA-N", "MCM:MTBKNO3", "")
  CALL Species("C_6H_5NO_7",                                   "", "NXGPVQDYDGLWQQ-UHFFFAOYSA-N", "MCM:NBZQOOH", "")
  CALL Species("C_7H_9NO_7",                                   "", "FQVTWXYWQQSSRX-UHFFFAOYSA-N", "MCM:C627PAN", "")
  CALL Species("C_7H_9NO_6",                                   "", "OFKNKTFVVCSQBW-UHFFFAOYSA-N", "MCM:C7ADCPAN", "")
  CALL Species("C_7H_9NO_6",                                   "", "ZGBUEZLBCHGZPP-UHFFFAOYSA-N", "MCM:C7DCPAN", "")
  CALL Species("C_7H_9NO_6",                                   "", "WCHGYJHJZMHIAR-UHFFFAOYSA-N", "MCM:C7DDCPAN", "")
  CALL Species("C_7H_7NO_8",                                   "", "XMNXEIJPBOBVEP-UHFFFAOYSA-N", "MCM:C7PAN3", "")
  CALL Species("C_7H_9NO_6",                                   "", "NZLPHNCQTJFNGH-UHFFFAOYSA-N", "MCM:IC7DCPAN", "")
  CALL Species("C_7H_7NO_6",                                   "", "CDHAVACRDDMHGZ-UHFFFAOYSA-N", "MCM:NC71CO", "")
  CALL Species("C_7H_9NO_7",                                   "", "CIJBWMVCSXGACJ-UHFFFAOYSA-N", "MCM:NC71OOH", "")
  CALL Species("C_7H_7NO_8",                                   "", "RYHDNZNGTUJGAD-UHFFFAOYSA-N", "MCM:NC72OOH", "")
  CALL Species("C_7H_7NO_7",                                   "", "WIZJVJIMHIROQF-UHFFFAOYSA-N", "MCM:NPTLQOOH", "")
  CALL Species("C_8H_9NO_7",                                   "", "OYOZKJOWUFGKRQ-UHFFFAOYSA-N", "MCM:C5DBEPAN", "")
  CALL Species("C_8H_9NO_7",                                   "", "KNPMBKOCKIUXFV-UHFFFAOYSA-N", "MCM:C5EDBPAN", "")
  CALL Species("C_8H_{11}NO_7",                                "", "WBGZRMCIPBEEGH-UHFFFAOYSA-N", "MCM:C727PAN", "")
  CALL Species("C_8H_{11}NO_6",                                "", "QVHZZCVTEAGIPK-UHFFFAOYSA-N", "MCM:C7CODBPAN", "")
  CALL Species("C_8H_{13}NO_5",                                "", "ZONZWPKPCIWKRN-UHFFFAOYSA-N", "MCM:C817NO3", "")
  CALL Species("C_8H_9NO_7",                                   "", "MDRQMTCFLWPCNU-UHFFFAOYSA-N", "MCM:NMXYQOOH", "")
  CALL Species("C_8H_9NO_7",                                   "", "SFYCEAPRSKIFCS-UHFFFAOYSA-N", "MCM:NOXYQOOH", "")
  CALL Species("C_8H_9NO_7",                                   "", "HZFUWULOZSGVHH-UHFFFAOYSA-N", "MCM:NPEBQOOH", "")
  CALL Species("C_8H_9NO_7",                                   "", "LMPSHWMXIGVDOI-UHFFFAOYSA-N", "MCM:NPXYQOOH", "")
  CALL Species("C_9H_{13}NO_6",                                "", "JXXOSNUALVJMQL-UHFFFAOYSA-N", "MCM:C816PAN", "")
  CALL Species("C_9H_{13}NO_7",                                "", "AGDIJRUETYYZKI-UHFFFAOYSA-N", "MCM:C817PAN", "")
  CALL Species("C_9H_{13}NO_7",                                "", "QMDLSNSKOZAILD-UHFFFAOYSA-N", "MCM:C827PAN", "")
  CALL Species("C_9H_{11}NO_8",                                "", "LCFHBKSAGMTDCX-UHFFFAOYSA-N", "MCM:C828PAN", "")
  CALL Species("C_9H_{11}NO_7",                                "", "KLGKHKJTYFCRCN-UHFFFAOYSA-N", "MCM:C88PAN", "")
  CALL Species("C_9H_{13}NO_5",                                "", "KVWUPUMJPKRVQS-UHFFFAOYSA-N", "MCM:C917NO3", "")
  CALL Species("C_9H_{15}NO_4",                                "", "JEWJQSHMOKPQHI-UHFFFAOYSA-N", "MCM:C923NO3", "")
  CALL Species("C_9H_{15}NO_5",                                "", "HCHMIBGRYQMLLZ-UHFFFAOYSA-N", "MCM:C928NO3", "")
  CALL Species("C_9H_{15}NO_4",                                "", "MMTOEOITRCIZDG-UHFFFAOYSA-N", "MCM:C96NO3", "")
  CALL Species("C_9H_{11}NO_5",                                "", "TURIVFFHDFEFMU-UHFFFAOYSA-N", "MCM:C9DCNO3", "")
  CALL Species("C_9H_{13}NO_6",                                "", "VXDDXZNSSANCGN-UHFFFAOYSA-N", "MCM:C9PAN2", "")
  CALL Species("C_9H_{11}NO_7",                                "", "KGZDDJNUZZWZIJ-UHFFFAOYSA-N", "MCM:NIPRBQOOH", "")
  CALL Species("C_9H_{15}NO_6",                                "", "ZOTFHCKOVMRARL-UHFFFAOYSA-N", "MCM:NLMKAOOH", "")
  CALL Species("C_9H_{11}NO_7",                                "", "FSVFAZAPMWXECG-UHFFFAOYSA-N", "MCM:NMETLQOOH", "")
  CALL Species("C_9H_{11}NO_7",                                "", "GHGZYHLEQYQZHR-UHFFFAOYSA-N", "MCM:NOETLQOOH", "")
  CALL Species("C_9H_{13}NO_4",                                "", "JEOFIBHUYPQAJL-UHFFFAOYSA-N", "MCM:NOPINANO3", "")
  CALL Species("C_9H_{13}NO_4",                                "", "QGVNTXJLRAHBMW-UHFFFAOYSA-N", "MCM:NOPINBNO3", "")
  CALL Species("C_9H_{13}NO_4",                                "", "CLMUTFVSCXCGKI-UHFFFAOYSA-N", "MCM:NOPINCNO3", "")
  CALL Species("C_9H_{11}NO_7",                                "", "IMJMJNZTMWKOFO-UHFFFAOYSA-N", "MCM:NPETLQOOH", "")
  CALL Species("C_9H_{11}NO_7",                                "", "MMAZKJNYHBQGDL-UHFFFAOYSA-N", "MCM:NPPRBQOOH", "")
  CALL Species("C_9H_{11}NO_7",                                "", "ZMXXAPJGXNLYQU-UHFFFAOYSA-N", "MCM:NTM124QOOH", "")
  CALL Species("C_{10}H_{17}NO_4",                             "", "KLQCUGBDZZARMU-UHFFFAOYSA-N", "MCM:C1011NO3", "")
  CALL Species("C_{10}H_{15}NO_6",                             "", "XGWGKMXMXMPDAK-UHFFFAOYSA-N", "MCM:C10PAN2", "")
  CALL Species("C_{10}H_{15}NO_6",                             "", "ZVYIPTRRRFMSSU-UHFFFAOYSA-N", "MCM:C923PAN", "")
  CALL Species("C_{10}H_{15}NO_7",                             "", "MIISGCVMOACDOR-UHFFFAOYSA-N", "MCM:C928PAN", "")
  CALL Species("C_{10}H_{15}NO_4",                             "", "BCIULZBFFUODJR-UHFFFAOYSA-N", "MCM:NC101CO", "")
  CALL Species("C_{10}H_{15}NO_6",                             "", "AQVSRLWLNUXIOL-UHFFFAOYSA-N", "MCM:NC101OOH", "")
  CALL Species("C_{10}H_{15}NO_7",                             "", "ZNOCONLMTXMLFU-UHFFFAOYSA-N", "MCM:NC102OOH", "")
  CALL Species("C_{11}H_{17}NO_6",                             "", "PRDFPHJPDVZXHH-UHFFFAOYSA-N", "MCM:C1011PAN", "")
  CALL Species("C_{13}H_{21}NO_5",                             "", "HFRDBZMTZNPZMV-UHFFFAOYSA-N", "MCM:C131NO3", "")
  CALL Species("C_{14}H_{21}NO_7",                             "", "YHMAABVAIQVHRP-UHFFFAOYSA-N", "MCM:C131PAN", "")
  CALL Species("C_{14}H_{23}NO_4",                             "", "JPXQKVXALDMDKF-UHFFFAOYSA-N", "MCM:C141NO3", "")
  CALL Species("C_{14}H_{23}NO_6",                             "", "IMGMBHKHHVQTQK-UHFFFAOYSA-N", "MCM:NBCKOOH", "")
  CALL Species("C_{15}H_{23}NO_6",                             "", "PGNAVHCNSKAOHZ-UHFFFAOYSA-N", "MCM:C141PAN", "")
  CALL Species("C_4H_5NO_8",                                   "", "JOAGXXIRMSWAAZ-UHFFFAOYSA-N", "MCM:C4PAN10", "")
  CALL Species("C_4H_5NO_7",                                   "", "QRAIBPAZBCTEOB-UHFFFAOYSA-N", "MCM:C4PAN6", "")
  CALL Species("C_4H_7NO_5",                                   "", "HZFDSYFRCPAXEA-UHFFFAOYSA-N", "MCM:CO3C4NO3OH", "")
  CALL Species("C_4H_7NO_5",                                   "", "UFDPWCOBIOWDKA-UHFFFAOYSA-N", "MCM:HMVKANO3", "")
  CALL Species("C_4H_7NO_6",                                   "", "NCLLRWDJHCXXDR-UHFFFAOYSA-N", "MCM:HMVKNO3", "")
  CALL Species("C_4H_5NO_6",                                   "", "HPGBXXMJBUDRLY-UHFFFAOYSA-N", "MCM:HNBIACET", "")
  CALL Species("C_4H_7NO_6",                                   "", "KPZZNRQGHDTQNJ-UHFFFAOYSA-N", "MCM:HNMVKOH", "")
  CALL Species("C_4H_7NO_7",                                   "", "CTKAFSDGNPEZFZ-UHFFFAOYSA-N", "MCM:HNMVKOOH", "")
  CALL Species("C_4H_7NO_5",                                   "", "SCJOQBCZNBCTOQ-UHFFFAOYSA-N", "MCM:MVKNO3", "")
  CALL Species("C_4H_7NO_6",                                   "", "DMSMZXCVKMBQGA-UHFFFAOYSA-N", "MCM:MVKOHANO3", "")
  CALL Species("C_5H_9NO_5",                                   "", "VRAOJUVNSLNISW-UHFFFAOYSA-N", "MCM:C4MCONO3OH", "")
  CALL Species("C_5H_9NO_5",                                   "", "DLLRFAPEEPVGBN-UHFFFAOYSA-N", "MCM:C517NO3", "")
  CALL Species("C_5H_9NO_5",                                   "", "OMTWKQRBUUWUQB-UHFFFAOYSA-N", "MCM:C51NO3", "")
  CALL Species("C_5H_9NO_5",                                   "", "GEKIHWHZVPPEBT-UHFFFAOYSA-N", "MCM:C5NO3CO4OH", "")
  CALL Species("C_5H_7NO_7",                                   "", "KOFNDQLSBTYFBZ-UHFFFAOYSA-N", "MCM:CO2OH3MPAN", "")
  CALL Species("C_5H_7NO_7",                                   "", "KVXLZPRANFUVNJ-UHFFFAOYSA-N", "MCM:CO3H4PAN", "")
  CALL Species("C_5H_5NO_8",                                   "", "ZNIKVNWPMOTSJK-UHFFFAOYSA-N", "MCM:H1C23C4PAN", "")
  CALL Species("C_5H_7NO_7",                                   "", "UDPFGHOTTZCMEQ-UHFFFAOYSA-N", "MCM:H3C2C4PAN", "")
  CALL Species("C_5H_7NO_7",                                   "", "VVIGYLNPENSKGX-UHFFFAOYSA-N", "MCM:HMVKBPAN", "")
  CALL Species("C_5H_8N_2O_8",                                 "", "IESHUNGXVKLDSS-UHFFFAOYSA-N", "MCM:INANCO", "")
  CALL Species("C_5H_9NO_6",                                   "", "ASROEBVFUUHBLI-UHFFFAOYSA-N", "MCM:INB1CO", "")
  CALL Species("C_5H_9NO_6",                                   "", "HXWBEXHMXOXGKY-UHFFFAOYSA-N", "MCM:INCCO", "")
  CALL Species("C_6H_7NO_8",                                   "", "KHOZKKAVCHSHMN-UHFFFAOYSA-N", "MCM:C4COMOHPAN", "")
  CALL Species("C_6H_9NO_7",                                   "", "LVRCIQOIMYLYHB-UHFFFAOYSA-N", "MCM:C4MOHOPAN", "")
  CALL Species("C_6H_9NO_7",                                   "", "BRUHFHISVNPPNB-UHFFFAOYSA-N", "MCM:C517PAN", "")
  CALL Species("C_6H_9NO_7",                                   "", "QFUOUYQJXYVTOO-UHFFFAOYSA-N", "MCM:C519PAN", "")
  CALL Species("C_6H_9NO_7",                                   "", "VCYALMFEDUSLKE-UHFFFAOYSA-N", "MCM:C5O45OHPAN", "")
  CALL Species("C_6H_{11}NO_5",                                "", "KOUUCDJHLKRSEQ-UHFFFAOYSA-N", "MCM:C610NO3", "")
  CALL Species("C_6H_9NO_6",                                   "", "QPXWFZAVXNBNFW-UHFFFAOYSA-N", "MCM:C614NO3", "")
  CALL Species("C_6H_{11}NO_5",                                "", "SYDWDJOZRNVKSU-UHFFFAOYSA-N", "MCM:C61NO3", "")
  CALL Species("C_6H_{11}NO_5",                                "", "RHTCZESJRHZQFA-UHFFFAOYSA-N", "MCM:C63NO3", "")
  CALL Species("C_6H_{11}NO_5",                                "", "IYYWGRLOLOMZAZ-UHFFFAOYSA-N", "MCM:C64NO3", "")
  CALL Species("C_6H_{11}NO_5",                                "", "OLTUAQAPCXKDOJ-UHFFFAOYSA-N", "MCM:C6CONO34OH", "")
  CALL Species("C_6H_{11}NO_5",                                "", "WWUUDQUMERSNEM-UHFFFAOYSA-N", "MCM:C6NO3CO4OH", "")
  CALL Species("C_6H_{11}NO_5",                                "", "IZBQRALUURUKJG-UHFFFAOYSA-N", "MCM:C6NO3CO5OH", "")
  CALL Species("C_6H_9NO_7",                                   "", "NJFFIXOWLQJQIZ-UHFFFAOYSA-N", "MCM:C6PAN9", "")
  CALL Species("C_6H_{11}NO_5",                                "", "YCHDRSSXCQCYSO-UHFFFAOYSA-N", "MCM:MIBKAOHNO3", "")
  CALL Species("C_7H_9NO_8",                                   "", "FWCVDVIZOKXUGJ-UHFFFAOYSA-N", "MCM:C61CPAN", "")
  CALL Species("C_7H_9NO_8",                                   "", "LHOMRHYSYFWOPZ-UHFFFAOYSA-N", "MCM:C62CPAN", "")
  CALL Species("C_7H_{13}NO_5",                                "", "RPPGGLBPWJPSSV-UHFFFAOYSA-N", "MCM:C712NO3", "")
  CALL Species("C_7H_{11}NO_6",                                "", "IXGSPJJVHFTXPK-UHFFFAOYSA-N", "MCM:C719NO3", "")
  CALL Species("C_7H_{13}NO_5",                                "", "MWIGFUUCEWIWBB-UHFFFAOYSA-N", "MCM:C72NO3", "")
  CALL Species("C_7H_{13}NO_5",                                "", "KZCKPUAVFWISTB-UHFFFAOYSA-N", "MCM:C77NO3", "")
  CALL Species("C_7H_9NO_8",                                   "", "GUUOCMMOMFUPSI-UHFFFAOYSA-N", "MCM:H3C25C6PAN", "")
  CALL Species("C_8H_9NO_8",                                   "", "PDGRPGJWHYLYKK-UHFFFAOYSA-N", "MCM:C6MOHCOPAN", "")
  CALL Species("C_8H_9NO_8",                                   "", "MICPXOSCTLYWLE-UHFFFAOYSA-N", "MCM:C7CO2OHPAN", "")
  CALL Species("C_8H_{13}NO_6",                                "", "CDMYSTRAPZWSHD-UHFFFAOYSA-N", "MCM:C829NO3", "")
  CALL Species("C_8H_{15}NO_5",                                "", "HUKHBWOLGKLCRF-UHFFFAOYSA-N", "MCM:C82NO3", "")
  CALL Species("C_9H_{11}NO_8",                                "", "YGQCKHJQVYPZEV-UHFFFAOYSA-N", "MCM:C6EO2OHPAN", "")
  CALL Species("C_9H_{11}NO_8",                                "", "XJDHGIQDEMCTOX-UHFFFAOYSA-N", "MCM:C7MJPPAN", "")
  CALL Species("C_9H_{11}NO_8",                                "", "SORJSZWEJQTGJA-UHFFFAOYSA-N", "MCM:C7MOHCOPAN", "")
  CALL Species("C_9H_{15}NO_5",                                "", "RKOGSSYMWVPWJW-UHFFFAOYSA-N", "MCM:C927NO3", "")
  CALL Species("C_9H_{17}NO_5",                                "", "OEEOAFFOONSSLD-UHFFFAOYSA-N", "MCM:C93NO3", "")
  CALL Species("C_9H_{15}NO_6",                                "", "PQCIVFJRBSNULM-UHFFFAOYSA-N", "MCM:C98NO3", "")
  CALL Species("C_9H_{15}NO_5",                                "", "OCTKYXAOGSBZDD-UHFFFAOYSA-N", "MCM:LMKANO3", "")
  CALL Species("C_9H_{15}NO_5",                                "", "MVWAQQXSBOJSHS-UHFFFAOYSA-N", "MCM:LMKBNO3", "")
  CALL Species("C_{10}H_{19}NO_5",                             "", "NTZGBOUJHFKYIP-UHFFFAOYSA-N", "MCM:C103NO3", "")
  CALL Species("C_{10}H_{15}NO_7",                             "", "JPFVAVYDZBUWDH-UHFFFAOYSA-N", "MCM:C920PAN", "")
  CALL Species("C_{11}H_{21}NO_5",                             "", "XIMKJTRLGBXFHP-UHFFFAOYSA-N", "MCM:C113NO3", "")
  CALL Species("C_{12}H_{23}NO_5",                             "", "MVMJPHZGIRWSLB-UHFFFAOYSA-N", "MCM:C123NO3", "")
  CALL Species("C_{13}H_{21}NO_7",                             "", "ODJJFBKGWDLDSF-UHFFFAOYSA-N", "MCM:C133NO3", "")
  CALL Species("C_{14}H_{23}NO_5",                             "", "YULYIMMPXZFYHU-UHFFFAOYSA-N", "MCM:BCKANO3", "")
  CALL Species("C_{14}H_{23}NO_5",                             "", "VHSFPWDPGIOBJY-UHFFFAOYSA-N", "MCM:BCKBNO3", "")
  CALL Species("C_{14}H_{23}NO_5",                             "", "AZGAJOUTVTYWPD-UHFFFAOYSA-N", "MCM:C142NO3", "")
  CALL Species("C_{14}H_{23}NO_6",                             "", "NNHMCYUMVAAOGX-UHFFFAOYSA-N", "MCM:C143NO3", "")
  CALL Species("C_3H_3NO_5",                                   "", "OPXOPVKLHYDIMX-UHFFFAOYSA-N", "MCM:NMGLYOX", "")
  CALL Species("C_4H_3NO_7",                                   "", "RRBQALAYWSLXKO-UHFFFAOYSA-N", "MCM:C312COPAN", "")
  CALL Species("C_4H_5NO_5",                                   "", "RXLGSPGPVVHCFU-UHFFFAOYSA-N", "MCM:C4CONO3CO", "")
  CALL Species("C_4H_3NO_6",                                   "", "VZDGBDWMYVQZKU-UHFFFAOYSA-N", "MCM:NCO23CHO", "")
  CALL Species("C_5H_3NO_7",                                   "", "BTFAHSUMXINULH-UHFFFAOYSA-N", "MCM:C4CO2DBPAN", "")
  CALL Species("C_5H_7NO_5",                                   "", "FQGALMJVSZOCNN-UHFFFAOYSA-N", "MCM:C4M22CONO3", "")
  CALL Species("C_5H_7NO_5",                                   "", "OAONLGDTCNNTSO-UHFFFAOYSA-N", "MCM:C512NO3", "")
  CALL Species("C_5H_5NO_7",                                   "", "MEMYBUIJIAQZMF-UHFFFAOYSA-N", "MCM:CHOC3COPAN", "")
  CALL Species("C_5H_6N_2O_8",                                 "", "ALBXLNVOOLMBSV-UHFFFAOYSA-N", "MCM:INANCOCHO", "")
  CALL Species("C_6H_7NO_7",                                   "", "ADLPISUKTIUEBV-UHFFFAOYSA-N", "MCM:C511PAN", "")
  CALL Species("C_6H_5NO_8",                                   "", "DHCQJQARWDKNBK-UHFFFAOYSA-N", "MCM:C5124COPAN", "")
  CALL Species("C_6H_7NO_7",                                   "", "OLSCQSOEJBIJSE-UHFFFAOYSA-N", "MCM:C512PAN", "")
  CALL Species("C_6H_5NO_8",                                   "", "QIUJGMPSWRWOQB-UHFFFAOYSA-N", "MCM:C515PAN", "")
  CALL Species("C_6H_5NO_7",                                   "", "LCULQJDFGZNNJC-UHFFFAOYSA-N", "MCM:C5CO2DBPAN", "")
  CALL Species("C_6H_5NO_7",                                   "", "QMYFWHUFARKUQS-UHFFFAOYSA-N", "MCM:C5DBCO2PAN", "")
  CALL Species("C_6H_9NO_5",                                   "", "OCVBVQJPDKPHST-UHFFFAOYSA-N", "MCM:C626NO3", "")
  CALL Species("C_6H_7NO_7",                                   "", "YGLXJUWEIHLTAS-UHFFFAOYSA-N", "MCM:C6COCHOPAN", "")
  CALL Species("C_7H_7NO_7",                                   "", "LSIPSXVETZQMNK-UHFFFAOYSA-N", "MCM:C4DBM2PAN", "")
  CALL Species("C_7H_9NO_7",                                   "", "PZZHENCPCXAAMZ-UHFFFAOYSA-N", "MCM:C617PAN", "")
  CALL Species("C_7H_9NO_7",                                   "", "TUYXNXVHKYYCNT-UHFFFAOYSA-N", "MCM:C618PAN", "")
  CALL Species("C_7H_9NO_7",                                   "", "JTOZIQOFBRFINW-UHFFFAOYSA-N", "MCM:C626PAN", "")
  CALL Species("C_7H_9NO_6",                                   "", "YBNFZHXBUOLIKA-UHFFFAOYSA-N", "MCM:C717NO3", "")
  CALL Species("C_7H_{11}NO_5",                                "", "WMUFPKACUPNARK-UHFFFAOYSA-N", "MCM:C718NO3", "")
  CALL Species("C_7H_{11}NO_5",                                "", "WJSMASANGLXUCK-UHFFFAOYSA-N", "MCM:C731NO3", "")
  CALL Species("C_7H_7NO_7",                                   "", "FJSOTAPJABBDID-UHFFFAOYSA-N", "MCM:C7CO2DBPAN", "")
  CALL Species("C_7H_7NO_9",                                   "", "RCADYDMYBSYTRH-UHFFFAOYSA-N", "MCM:NC61CO3H", "")
  CALL Species("C_7H_6N_2O_{11}",                              "", "LNWBMFOWVOYBBT-UHFFFAOYSA-N", "MCM:NC6PAN1", "")
  CALL Species("C_8H_9NO_7",                                   "", "JDCDQJWNYIFIQT-UHFFFAOYSA-N", "MCM:C4DBMEPAN", "")
  CALL Species("C_8H_{11}NO_7",                                "", "IZJQRCOOVGKQNV-UHFFFAOYSA-N", "MCM:C718PAN", "")
  CALL Species("C_8H_{11}NO_7",                                "", "BCDBCVPVNWUZKV-UHFFFAOYSA-N", "MCM:C731PAN", "")
  CALL Species("C_8H_9NO_7",                                   "", "IPAWZRQBQOFPJA-UHFFFAOYSA-N", "MCM:C8CO2DBPAN", "")
  CALL Species("C_8H_9NO_7",                                   "", "ZDCAPCKMKJMPHV-UHFFFAOYSA-N", "MCM:C8DBCO2PAN", "")
  CALL Species("C_9H_{11}NO_8",                                "", "KZPDHKTVZKNFJF-UHFFFAOYSA-N", "MCM:C87PAN", "")
  CALL Species("C_9H_{13}NO_5",                                "", "CFOCUCLSGKHNEG-UHFFFAOYSA-N", "MCM:C915NO3", "")
  CALL Species("C_9H_{13}NO_6",                                "", "VOXWFVUGDBUBCW-UHFFFAOYSA-N", "MCM:C916NO3", "")
  CALL Species("C_9H_{13}NO_5",                                "", "AMOFMDWAGAYJCS-UHFFFAOYSA-N", "MCM:C918NO3", "")
  CALL Species("C_9H_{13}NO_6",                                "", "DCZHYWWXGROBKB-UHFFFAOYSA-N", "MCM:C919NO3", "")
  CALL Species("C_9H_{13}NO_6",                                "", "DDSHYSHFPVKRPI-UHFFFAOYSA-N", "MCM:C930NO3", "")
  CALL Species("C_{10}H_{15}NO_5",                             "", "GVXIDPZAUKXFPR-UHFFFAOYSA-N", "MCM:C1010NO3", "")
  CALL Species("C_{10}H_{17}NO_5",                             "", "XKIZNDJDYSDNBL-UHFFFAOYSA-N", "MCM:C1012NO3", "")
  CALL Species("C_{10}H_{15}NO_6",                             "", "DSEFLMWIAZKUSJ-UHFFFAOYSA-N", "MCM:C106NO3", "")
  CALL Species("C_{10}H_{15}NO_6",                             "", "MVSLQVXNIWKHQS-UHFFFAOYSA-N", "MCM:C108NO3", "")
  CALL Species("C_{10}H_{17}NO_7",                             "", "NTLQQKIUYMIZPN-UHFFFAOYSA-N", "MCM:NLIMALOOH", "")
  CALL Species("C_{10}H_{15}NO_5",                             "", "FNTMCASBPWQXGD-UHFFFAOYSA-N", "MCM:PINALNO3", "")
  CALL Species("C_{11}H_{17}NO_5",                             "", "AWSJYNTZBDMUKU-UHFFFAOYSA-N", "MCM:C116NO3", "")
  CALL Species("C_{12}H_{17}NO_7",                             "", "IHJLCJPVFFZKMO-UHFFFAOYSA-N", "MCM:C116PAN", "")
  CALL Species("C_{12}H_{19}NO_5",                             "", "IJHSXQWUSMROBJ-UHFFFAOYSA-N", "MCM:C1210NO3", "")
  CALL Species("C_{13}H_{19}NO_7",                             "", "PWZZEUPXUFAYML-UHFFFAOYSA-N", "MCM:C1210PAN", "")
  CALL Species("C_{15}H_{25}NO_7",                             "", "TUYLAULXHOKFAG-UHFFFAOYSA-N", "MCM:NBCALOOH", "")
  CALL Species("C_4H_5NO_6",                                   "", "VCTGKRYJUDQYFK-UHFFFAOYSA-N", "MCM:H3NCO2CHO", "")
  CALL Species("C_4H_5NO_6",                                   "", "VDDXFUNTPSDTJP-UHFFFAOYSA-N", "MCM:HMVKNGLYOX", "")
  CALL Species("C_5H_7NO_6",                                   "", "NAZCWKUVRXJZPX-UHFFFAOYSA-N", "MCM:INB1GLYOX", "")
  CALL Species("C_5H_7NO_6",                                   "", "DGCXGPGOOTUCCO-UHFFFAOYSA-N", "MCM:INCGLYOX", "")
  CALL Species("C_6H_5NO_8",                                   "", "MLSWEUDOYVEBFO-UHFFFAOYSA-N", "MCM:C5CO2OHPAN", "")
  CALL Species("C_7H_7NO_8",                                   "", "GITGCCFTTYJWBT-UHFFFAOYSA-N", "MCM:C6CO2OHPAN", "")
  CALL Species("C_8H_9NO_8",                                   "", "MJYXUWCXTAYXTN-UHFFFAOYSA-N", "MCM:C5M2OHOPAN", "")
  CALL Species("C_8H_9NO_8",                                   "", "WKRTWLVIFYOGIB-UHFFFAOYSA-N", "MCM:C7OHCO2PAN", "")
  CALL Species("C_9H_{11}NO_8",                                "", "KTOTUVLJTIGMKC-UHFFFAOYSA-N", "MCM:C5MEJPAN", "")
  CALL Species("C_9H_{11}NO_8",                                "", "UEMVFSSSLHUULZ-UHFFFAOYSA-N", "MCM:C8CO2OHPAN", "")
  CALL Species("C_9H_{11}NO_8",                                "", "GUTUSTBTBKWXHL-UHFFFAOYSA-N", "MCM:C8OHCO2PAN", "")
  CALL Species("C_{10}H_{17}NO_6",                             "", "FDBNDHKWTKEDSC-UHFFFAOYSA-N", "MCM:LIMALNO3", "")
  CALL Species("C_{10}H_{17}NO_6",                             "", "NNOYMJCUDKQQLW-UHFFFAOYSA-N", "MCM:NLIMALOH", "")
  CALL Species("C_{11}H_{17}NO_6",                             "", "VRGCJRKSASUXOA-UHFFFAOYSA-N", "MCM:C117NO3", "")
  CALL Species("C_{11}H_{17}NO_7",                             "", "RHEKUCZWBZRVBN-UHFFFAOYSA-N", "MCM:C118NO3", "")
  CALL Species("C_{12}H_{19}NO_7",                             "", "HIFZNEWSKKHKQF-UHFFFAOYSA-N", "MCM:C1215NO3", "")
  CALL Species("C_{12}H_{19}NO_6",                             "", "XVMHBNQZULJJQH-UHFFFAOYSA-N", "MCM:C128NO3", "")
  CALL Species("C_{13}H_{21}NO_6",                             "", "LCWPKTPKZZROTF-UHFFFAOYSA-N", "MCM:C1312NO3", "")
  CALL Species("C_{15}H_{25}NO_6",                             "", "KJHBLMRUHUAACD-UHFFFAOYSA-N", "MCM:BCALNO3", "")
  CALL Species("C_{15}H_{25}NO_6",                             "", "YUOPYTDNMDNILJ-UHFFFAOYSA-N", "MCM:NBCALOH", "")
  CALL Species("C_2H_3NO_5",                                   "", "VHOVOPOBBVJMEP-UHFFFAOYSA-N", "MCM:NO3CH2CO2H", "")
  CALL Species("C_3H_5NO_5",                                   "", "DVOPCQDTEPKYKW-UHFFFAOYSA-N", "MCM:PRNO3CO2H", "")
  CALL Species("C_4H_7NO_5",                                   "", "MSBKTJKUKRRTOZ-UHFFFAOYSA-N", "MCM:MPRNO3CO2H", "")
  CALL Species("C_4H_5NO_5",                                   "", "IYBHCPMQXJSPPT-UHFFFAOYSA-N", "MCM:NC3CO2H", "")
  CALL Species("C_5H_9NO_5",                                   "", "CCKPHGGNTPAJHG-UHFFFAOYSA-N", "MCM:C3MNO3CO2H", "")
  CALL Species("C_5H_9NO_5",                                   "", "PAJLSJNKCRUQAA-UHFFFAOYSA-N", "MCM:C43NO3CO2H", "")
  CALL Species("C_5H_9NO_5",                                   "", "LHOHBJOUNVJDSQ-UHFFFAOYSA-N", "MCM:C4NO3CO2H", "")
  CALL Species("C_5H_7NO_5",                                   "", "ZKTRJZJYWMAEEY-UHFFFAOYSA-N", "MCM:NC4CO2H", "")
  CALL Species("C_6H_{11}NO_5",                                "", "JEDGAPUZBUOYQF-UHFFFAOYSA-N", "MCM:C65NO3CO2H", "")
  CALL Species("C_8H_{11}NO_7",                                "", "OMUZIQDFMVGNTC-UHFFFAOYSA-N", "MCM:C721PAN", "")
  CALL Species("C_8H_{13}NO_5",                                "", "WYXUZPCNBFFDRO-UHFFFAOYSA-N", "MCM:C811NO3", "")
  CALL Species("C_8H_{13}NO_5",                                "", "QAJOXMTYFQHFPC-UHFFFAOYSA-N", "MCM:C823NO3", "")
  CALL Species("C_9H_{13}NO_7",                                "", "BOYXRVFXWMBNIT-UHFFFAOYSA-N", "MCM:C811PAN", "")
  CALL Species("C_9H_{13}NO_7",                                "", "GRHNTSHNCZZYLE-UHFFFAOYSA-N", "MCM:C823PAN", "")
  CALL Species("C_{13}H_{21}NO_5",                             "", "CQGOUXQAADNZJX-UHFFFAOYSA-N", "MCM:C137NO3", "")
  CALL Species("C_{14}H_{21}NO_7",                             "", "KJDMPVAHMJBAAO-UHFFFAOYSA-N", "MCM:C137PAN", "")
  CALL Species("C_4H_7NO_6",                                   "", "WQOIEXBGIWRREH-UHFFFAOYSA-N", "MCM:MACRNBCO2H", "")
  CALL Species("C_4H_7NO_6",                                   "", "GPIMHHNLFCYMAO-UHFFFAOYSA-N", "MCM:MACRNCO2H", "")
  CALL Species("C_5H_9NO_7",                                   "", "VNPCVOYEGJTTSH-UHFFFAOYSA-N", "MCM:C57NO3CO2H", "")
  CALL Species("C_5H_9NO_7",                                   "", "QOEJXJLKIMUJIJ-UHFFFAOYSA-N", "MCM:C58NO3CO2H", "")
  CALL Species("C_5H_9NO_7",                                   "", "KDGUUBLLRPAORR-UHFFFAOYSA-N", "MCM:INAHCO2H", "")
  CALL Species("C_5H_9NO_8",                                   "", "CHIYBIMSWIBBRD-UHFFFAOYSA-N", "MCM:INAHPCO2H", "")
  CALL Species("C_5H_8N_2O_9",                                 "", "QEPBKMYMBRWSSY-UHFFFAOYSA-N", "MCM:INANCO2H", "")
  CALL Species("C_5H_9NO_8",                                   "", "XWZUZABUMCCUCE-UHFFFAOYSA-N", "MCM:INB1HPCO2H", "")
  CALL Species("C_5H_8N_2O_9",                                 "", "FONBWUWRCYYCPB-UHFFFAOYSA-N", "MCM:INB1NACO2H", "")
  CALL Species("C_5H_8N_2O_9",                                 "", "YGORAHXKOXOIMV-UHFFFAOYSA-N", "MCM:INB1NBCO2H", "")
  CALL Species("C_5H_8N_2O_9",                                 "", "LXTUVHDUSAOJOD-UHFFFAOYSA-N", "MCM:INCNCO2H", "")
  CALL Species("C_{13}H_{21}NO_7",                             "", "UFNZJLDHNJULFY-UHFFFAOYSA-N", "MCM:C139NO3", "")
  CALL Species("C_4H_5NO_6",                                   "", "LTGNKSWVEYSUMY-UHFFFAOYSA-N", "MCM:CONM2CO2H", "")
  CALL Species("C_5H_7NO_7",                                   "", "REHIBMNEJUPYOU-UHFFFAOYSA-N", "MCM:MMALNACO2H", "")
  CALL Species("C_5H_7NO_7",                                   "", "RAMGTVAYJVLLRG-UHFFFAOYSA-N", "MCM:MMALNBCO2H", "")
  CALL Species("C_5H_6N_2O_9",                                 "", "XEVMEFHPLZUFAK-UHFFFAOYSA-N", "MCM:INANCOCO2H", "")
  CALL Species("C_7H_{11}NO_6",                                "", "SXYZBZXZNRYERD-UHFFFAOYSA-N", "MCM:C732NO3", "")
  CALL Species("C_8H_{11}NO_8",                                "", "WNUKRJOERGBBRT-UHFFFAOYSA-N", "MCM:C732PAN", "")
  CALL Species("C_{12}H_{19}NO_6",                             "", "DQVGSELEPUBBSS-UHFFFAOYSA-N", "MCM:C1211NO3", "")
  CALL Species("C_{13}H_{19}NO_8",                             "", "IROGWMVYBREROI-UHFFFAOYSA-N", "MCM:C1211PAN", "")
  CALL Species("C_8H_{13}NO_7",                                "", "GYZVFQOTJWTWBE-UHFFFAOYSA-N", "MCM:C813NO3", "")
  CALL Species("C_{12}H_{19}NO_8",                             "", "NTNWJSADTNTFHH-UHFFFAOYSA-N", "MCM:C1212NO3", "")
  CALL Species("C_{12}H_{19}NO_8",                             "", "FRAJAIDKFGOMCR-UHFFFAOYSA-N", "MCM:C1213NO3", "")
  CALL Species("C_{13}H_{21}NO_7",                             "", "YNRLOWIUKIUEHR-UHFFFAOYSA-N", "MCM:C1310NO3", "")
  CALL Species("C_{15}H_{25}NO_7",                             "", "BHPYCCPMXIBJRO-UHFFFAOYSA-N", "MCM:C151NO3", "")
  CALL Species("C_2H_3NO_5",                                   "", "OAQKKIQKQLJUJT-UHFFFAOYSA-N", "MCM:CHOOCH2NO3", "")
  CALL Species("C_3H_3NO_7",                                   "", "XZTQWYQUABOKPG-UHFFFAOYSA-N", "MCM:CHOOMPAN", "")
  CALL Species("C_3H_5NO_5",                                   "", "DOXRBPLNMOIQGP-UHFFFAOYSA-N", "MCM:ETHFORMNO3", "")
  CALL Species("C_3H_5NO_5",                                   "", "HRSAJZJMUADCNM-UHFFFAOYSA-N", "MCM:METACETNO3", "")
  CALL Species("C_3H_3NO_7",                                   "", "IFMSYUVMGRDEMV-UHFFFAOYSA-N", "MCM:MMCFNO3", "")
  CALL Species("C_4H_7NO_5",                                   "", "QOXSFWPQUXLCPI-UHFFFAOYSA-N", "MCM:ACEC2H4NO3", "")
  CALL Species("C_4H_5NO_7",                                   "", "QMSGSFOWAUFKPL-UHFFFAOYSA-N", "MCM:ACETMEPAN", "")
  CALL Species("C_4H_5NO_7",                                   "", "YQMLKDOXYNVTNM-UHFFFAOYSA-N", "MCM:COO2C3PAN", "")
  CALL Species("C_4H_7NO_5",                                   "", "KORDSAOMZNGUAL-UHFFFAOYSA-N", "MCM:ETACETNO3", "")
  CALL Species("C_4H_7NO_5",                                   "", "DZSICYNSKFSRMV-UHFFFAOYSA-N", "MCM:IPRFORMNO3", "")
  CALL Species("C_4H_5NO_7",                                   "", "TXWHLSUKLWEERR-UHFFFAOYSA-N", "MCM:NBZFUOOH", "")
  CALL Species("C_5H_5NO_8",                                   "", "ZQOVYYMCAFZVPL-UHFFFAOYSA-N", "MCM:ACCOMEPAN", "")
  CALL Species("C_5H_7NO_7",                                   "", "VMVPPLVFXDAZML-UHFFFAOYSA-N", "MCM:ACETC2PAN", "")
  CALL Species("C_5H_7NO_7",                                   "", "GBAXNZCSCJEGAE-UHFFFAOYSA-N", "MCM:COO2C4PAN", "")
  CALL Species("C_5H_9NO_5",                                   "", "ZEVCPCUWRCOHCL-UHFFFAOYSA-N", "MCM:IPRACBNO3", "")
  CALL Species("C_5H_7NO_7",                                   "", "UJCZQTHKWBYHPW-UHFFFAOYSA-N", "MCM:IPRACBPAN", "")
  CALL Species("C_5H_9NO_5",                                   "", "JAQKAPGOMZJVBI-UHFFFAOYSA-N", "MCM:IPRACNO3", "")
  CALL Species("C_5H_9NO_5",                                   "", "NIMOOIUOXYQIIM-UHFFFAOYSA-N", "MCM:MTBEAALNO3", "")
  CALL Species("C_5H_7NO_7",                                   "", "WKTKTTPKCZOXAQ-UHFFFAOYSA-N", "MCM:MTBEAALPAN", "")
  CALL Species("C_5H_9NO_5",                                   "", "FPQZKAXTSOKJRB-UHFFFAOYSA-N", "MCM:NPRACANO3", "")
  CALL Species("C_5H_9NO_5",                                   "", "HTKMYQNNJMTEJQ-UHFFFAOYSA-N", "MCM:NPRACBNO3", "")
  CALL Species("C_5H_9NO_5",                                   "", "PSJLJILPBUYXRJ-UHFFFAOYSA-N", "MCM:NPRACCNO3", "")
  CALL Species("C_5H_7NO_7",                                   "", "ARRSNVVNYKGUFK-UHFFFAOYSA-N", "MCM:NPXYFUOOH", "")
  CALL Species("C_5H_7NO_7",                                   "", "OAXQEILKHMZHNV-UHFFFAOYSA-N", "MCM:NTLFUOOH", "")
  CALL Species("C_6H_{11}NO_5",                                "", "BCWPDUXSEGSVPT-UHFFFAOYSA-N", "MCM:MCOOTBNO3", "")
  CALL Species("C_6H_{11}NO_5",                                "", "GNPRUWODPALWHJ-UHFFFAOYSA-N", "MCM:NBUACANO3", "")
  CALL Species("C_6H_{11}NO_5",                                "", "UFOXFUDLAIUTHC-UHFFFAOYSA-N", "MCM:NBUACBNO3", "")
  CALL Species("C_6H_{11}NO_5",                                "", "PYXJHVBEHHCABK-UHFFFAOYSA-N", "MCM:NBUACCNO3", "")
  CALL Species("C_6H_7NO_8",                                   "", "RZSLUQVSVWAZFJ-UHFFFAOYSA-N", "MCM:NDMMALYOOH", "")
  CALL Species("C_6H_9NO_7",                                   "", "CZSCMTWZZQSKMX-UHFFFAOYSA-N", "MCM:NEBFUOOH", "")
  CALL Species("C_6H_9NO_7",                                   "", "URYSYTGABCBDCM-UHFFFAOYSA-N", "MCM:NMXYFUOOH", "")
  CALL Species("C_6H_9NO_7",                                   "", "CPPAIMSHWPDUBG-UHFFFAOYSA-N", "MCM:NOXYFUOOH", "")
  CALL Species("C_6H_9NO_7",                                   "", "WRMOSOIHGVCRMO-UHFFFAOYSA-N", "MCM:NTMB1FUOOH", "")
  CALL Species("C_6H_9NO_7",                                   "", "NSDDFRROKVQLRJ-UHFFFAOYSA-N", "MCM:PRCOOMPAN", "")
  CALL Species("C_6H_{11}NO_5",                                "", "FWAMAOJLVLFAOA-UHFFFAOYSA-N", "MCM:SBUACANO3", "")
  CALL Species("C_6H_{11}NO_5",                                "", "FVMFJLQQOIKLHP-UHFFFAOYSA-N", "MCM:SBUACBNO3", "")
  CALL Species("C_6H_9NO_7",                                   "", "WTCUHXTZKDYOTB-UHFFFAOYSA-N", "MCM:TBUACPAN", "")
  CALL Species("C_7H_{11}NO_7",                                "", "MJSJHBOZYWKETN-UHFFFAOYSA-N", "MCM:NIPBFUOOH", "")
  CALL Species("C_7H_{11}NO_7",                                "", "LKMBYBKEMVMSAU-UHFFFAOYSA-N", "MCM:NMEBFUOOH", "")
  CALL Species("C_7H_{11}NO_7",                                "", "DGKCEZHRNXTHAZ-UHFFFAOYSA-N", "MCM:NPBFUOOH", "")
  CALL Species("C_7H_{11}NO_7",                                "", "IRLXCLTZUJZGMJ-UHFFFAOYSA-N", "MCM:NTMB2FUOOH", "")
  CALL Species("C_{10}H_{17}NO_5",                             "", "TWKDJCLTDHUPBZ-UHFFFAOYSA-N", "MCM:C1013NO3", "")
  CALL Species("C_{11}H_{17}NO_7",                             "", "QVQKWVAIHQGLJY-UHFFFAOYSA-N", "MCM:C1013PAN", "")
  CALL Species("C_{10}H_{17}NO_6",                             "", "OQZDTLQTCPZSBL-UHFFFAOYSA-N", "MCM:C1014NO3", "")
  CALL Species("C_{15}H_{25}NO_7",                             "", "VUWXYQDBWQFZDF-UHFFFAOYSA-N", "MCM:C152NO3", "")
  CALL Species("C_4H_3NO_6",                                   "", "LXADUFLCRLLGHU-UHFFFAOYSA-N", "MCM:NBZFUONE", "")
  CALL Species("C_5H_5NO_8",                                   "", "BCRDXKGRUUAZSU-UHFFFAOYSA-N", "MCM:C23O3CPAN", "")
  CALL Species("C_6H_9NO_6",                                   "", "PDSBHIWVTCGSAN-UHFFFAOYSA-N", "MCM:ACBUONANO3", "")
  CALL Species("C_6H_7NO_8",                                   "", "MJYWDTPJLIMPPV-UHFFFAOYSA-N", "MCM:C23O3MCPAN", "")
  CALL Species("C_6H_7NO_6",                                   "", "LZGWKDALCOZNIJ-UHFFFAOYSA-N", "MCM:NEBFUONE", "")
  CALL Species("C_7H_9NO_8",                                   "", "SOMJWLUSPZTMJM-UHFFFAOYSA-N", "MCM:C23O3ECPAN", "")
  CALL Species("C_7H_9NO_6",                                   "", "OEYHEYBMUYSQHE-UHFFFAOYSA-N", "MCM:NIPBFUONE", "")
  CALL Species("C_7H_9NO_6",                                   "", "WTNUMEKWPLLNDJ-UHFFFAOYSA-N", "MCM:NPBFUONE", "")
  CALL Species("C_2H_5NO_4",                                   "", "VHAYDBAVTGOZLW-UHFFFAOYSA-N", "MCM:CH3OCH2NO3", "")
  CALL Species("C_3H_7NO_5",                                   "", "ZKGVKOFAVUIIEY-UHFFFAOYSA-N", "MCM:DMMANO3", "")
  CALL Species("C_3H_7NO_5",                                   "", "ADJOFNYUECRONI-UHFFFAOYSA-N", "MCM:DMMBNO3", "")
  CALL Species("C_3H_7NO_3",                                   "", "ZMUPMMOTQUHRHQ-UHFFFAOYSA-N", "MCM:ETOMENO3", "")
  CALL Species("C_3H_5NO_6",                                   "", "JBHGAGLULWBGCW-UHFFFAOYSA-N", "MCM:MEMOXYPAN", "")
  CALL Species("C_4H_9NO_4",                                   "", "CAMRTYJYAQGENN-UHFFFAOYSA-N", "MCM:DIETETNO3", "")
  CALL Species("C_4H_9NO_4",                                   "", "GDNQXPDYGNUKII-UHFFFAOYSA-N", "MCM:ETOC2NO3", "")
  CALL Species("C_4H_7NO_6",                                   "", "CCYZAMBLJVQHMC-UHFFFAOYSA-N", "MCM:ETOMEPAN", "")
  CALL Species("C_5H_{11}NO_4",                                "", "YQGFTPNVRZNXAP-UHFFFAOYSA-N", "MCM:IPROC21NO3", "")
  CALL Species("C_5H_{11}NO_4",                                "", "NXHUJDLNRCHXIE-UHFFFAOYSA-N", "MCM:MTBEANO3", "")
  CALL Species("C_5H_{11}NO_4",                                "", "HWZCMFJGZXOLOM-UHFFFAOYSA-N", "MCM:MTBEBNO3", "")
  CALL Species("C_5H_9NO_6",                                   "", "PQCDPGWYIKXNFH-UHFFFAOYSA-N", "MCM:MTBEBPAN", "")
  CALL Species("C_6H_{11}NO_6",                                "", "CPESEOWHBGRFRT-UHFFFAOYSA-N", "MCM:BOXMPAN", "")
  CALL Species("C_6H_{13}NO_4",                                "", "RYKOUPKPIPKSCM-UHFFFAOYSA-N", "MCM:DIIPRETNO3", "")
  CALL Species("C_6H_{13}NO_4",                                "", "WGHLTNOFPVNDHK-UHFFFAOYSA-N", "MCM:ETBEANO3", "")
  CALL Species("C_6H_{11}NO_6",                                "", "ZFJPSAIQYMAFAB-UHFFFAOYSA-N", "MCM:ETBEAPAN", "")
  CALL Species("C_6H_{13}NO_4",                                "", "BIDDBAQEKWTVIC-UHFFFAOYSA-N", "MCM:ETBEBNO3", "")
  CALL Species("C_6H_{13}NO_4",                                "", "MGOGEALWXAMIEO-UHFFFAOYSA-N", "MCM:ETBECNO3", "")
  CALL Species("C_6H_{11}NO_6",                                "", "JICVHVAZXJWJCM-UHFFFAOYSA-N", "MCM:ETBECPAN", "")
  CALL Species("C_6H_{13}NO_4",                                "", "XJQWDDGHWZGMDK-UHFFFAOYSA-N", "MCM:IPROMC2NO3", "")
  CALL Species("C_6H_{11}NO_6",                                "", "SFYHCJJQTXJNNN-UHFFFAOYSA-N", "MCM:IPROMCPAN", "")
  CALL Species("C_3H_7NO_5",                                   "", "FJVLHDUAJQRZPG-UHFFFAOYSA-N", "MCM:MO2EOLANO3", "")
  CALL Species("C_3H_7NO_5",                                   "", "QPXHYMSZINTGRC-UHFFFAOYSA-N", "MCM:MO2EOLBNO3", "")
  CALL Species("C_4H_9NO_5",                                   "", "UVLBVBPSPHSYND-UHFFFAOYSA-N", "MCM:EOX2OLANO3", "")
  CALL Species("C_4H_9NO_5",                                   "", "DLOZDRFLLIHYBD-UHFFFAOYSA-N", "MCM:EOX2OLBNO3", "")
  CALL Species("C_4H_9NO_5",                                   "", "VJBSYBJCULHXOZ-UHFFFAOYSA-N", "MCM:H2C3OCNO3", "")
  CALL Species("C_4H_9NO_5",                                   "", "YOBYTHKQRGEURN-UHFFFAOYSA-N", "MCM:PR2OHMONO3", "")
  CALL Species("C_5H_7NO_7",                                   "", "NFJKKZSQGXUZSI-UHFFFAOYSA-N", "MCM:IEAPAN", "")
  CALL Species("C_5H_7NO_7",                                   "", "UMKIAKGFUATPHT-UHFFFAOYSA-N", "MCM:IECPAN", "")
  CALL Species("C_6H_{13}NO_5",                                "", "NBMGDPHUZJHVTA-UHFFFAOYSA-N", "MCM:BOXEOHANO3", "")
  CALL Species("C_6H_{13}NO_5",                                "", "YIIBDLSBOAJITL-UHFFFAOYSA-N", "MCM:BOXEOHBNO3", "")
  CALL Species("C_7H_{15}NO_5",                                "", "QDPFCROPCZKQGS-UHFFFAOYSA-N", "MCM:BOXPOLANO3", "")
  CALL Species("C_7H_{15}NO_5",                                "", "UNWFJVCQCSAFHA-UHFFFAOYSA-N", "MCM:BOXPOLBNO3", "")
  CALL Species("C_{15}H_{25}NO_7",                             "", "HGUJSQZJFHDFKY-UHFFFAOYSA-N", "MCM:BCSOZNO3", "")
  CALL Species("C_4H_3NO_7",                                   "", "VASNFLPKLUXZKF-UHFFFAOYSA-N", "MCM:EPXDLPAN", "")
  CALL Species("C_5H_5NO_7",                                   "", "KZASOTCYMWAOFE-UHFFFAOYSA-N", "MCM:EPXMDLPAN", "")
  CALL Species("C_6H_5NO_7",                                   "", "RFFCGVCNFANFMQ-UHFFFAOYSA-N", "MCM:BZEMUCPAN", "")
  CALL Species("C_6H_7NO_7",                                   "", "KYBHJMDXHJGYTK-UHFFFAOYSA-N", "MCM:EPXM2DLPAN", "")
  CALL Species("C_7H_9NO_7",                                   "", "XUUSESMFYTYXHU-UHFFFAOYSA-N", "MCM:EPXMEDLPAN", "")
  CALL Species("C_8H_9NO_7",                                   "", "VIRSPYOLFRMIKR-UHFFFAOYSA-N", "MCM:OXYMUCPAN", "")
  CALL Species("C_9H_{11}NO_7",                                "", "PYAKBDFKVOPHPS-UHFFFAOYSA-N", "MCM:OETLMUCPAN", "")
  CALL Species("C_6H_7NO_7",                                   "", "UGJSRYKPNYMSMJ-UHFFFAOYSA-N", "MCM:BZEMUCNO3", "")
  CALL Species("C_8H_{11}NO_7",                                "", "PMXJWIAVDRSVOJ-UHFFFAOYSA-N", "MCM:OXYMUCNO3", "")
  CALL Species("C_9H_{13}NO_7",                                "", "KUQUSIKCNRTUGT-UHFFFAOYSA-N", "MCM:OETLMUCNO3", "")
  CALL Species("C_6H_7NO_7",                                   "", "DBXZZDVADSLSOS-UHFFFAOYSA-N", "MCM:EPXKTMPAN", "")
  CALL Species("C_6H_7NO_7",                                   "", "UTKXQAWEMGMHOV-UHFFFAOYSA-N", "MCM:EPXMKTPAN", "")
  CALL Species("C_7H_{13}NO_5",                                "", "FWQXAWHLSYKKGA-UHFFFAOYSA-N", "MCM:BOXPROANO3", "")
  CALL Species("C_7H_9NO_7",                                   "", "ZMAMXAXJQIDKIS-UHFFFAOYSA-N", "MCM:EPXEKTPAN", "")
  CALL Species("C_7H_7NO_7",                                   "", "AUCFXVOKAUAJMN-UHFFFAOYSA-N", "MCM:TLEMUCPAN", "")
  CALL Species("C_8H_9NO_7",                                   "", "IJCUBYYAOSTAFY-UHFFFAOYSA-N", "MCM:EBZMUCPAN", "")
  CALL Species("C_8H_9NO_7",                                   "", "FZEVZMXGRRYMCK-UHFFFAOYSA-N", "MCM:MXYMUCPAN", "")
  CALL Species("C_8H_9NO_7",                                   "", "JEHOQLADMHJYMP-UHFFFAOYSA-N", "MCM:PXYMUCPAN", "")
  CALL Species("C_9H_{11}NO_7",                                "", "JAYKUYZEOSXCLM-UHFFFAOYSA-N", "MCM:IPBZMUCPAN", "")
  CALL Species("C_9H_{11}NO_7",                                "", "ZBESLNAUTTYMKY-UHFFFAOYSA-N", "MCM:METLMUCPAN", "")
  CALL Species("C_9H_{11}NO_7",                                "", "BNKFVONIJIEPHK-UHFFFAOYSA-N", "MCM:PBZMUCPAN", "")
  CALL Species("C_9H_{11}NO_7",                                "", "ZDDBRQJYUBFFIJ-UHFFFAOYSA-N", "MCM:PETLMUCPAN", "")
  CALL Species("C_9H_{11}NO_7",                                "", "TXJRVENADFPPLH-UHFFFAOYSA-N", "MCM:TM124MUPAN", "")
  CALL Species("C_9H_{11}NO_7",                                "", "APWJKVMWYCHHEO-UHFFFAOYSA-N", "MCM:TM135MUPAN", "")
  CALL Species("C_{10}H_{13}NO_7",                             "", "OJIPANRKTQADST-UHFFFAOYSA-N", "MCM:DMEBMUPAN", "")
  CALL Species("C_{11}H_{15}NO_7",                             "", "WGGPAZKPQBFIGY-UHFFFAOYSA-N", "MCM:DETLMUPAN", "")
  CALL Species("C_9H_{13}NO_7",                                "", "CNKHLWZBBXJTJR-UHFFFAOYSA-N", "MCM:TM123MUNO3", "")
  CALL Species("C_7H_9NO_7",                                   "", "KLAQKLFQZHHGRD-UHFFFAOYSA-N", "MCM:TLEMUCNO3", "")
  CALL Species("C_8H_{11}NO_7",                                "", "ZZJNOSBMORNYPY-UHFFFAOYSA-N", "MCM:EBZMUCNO3", "")
  CALL Species("C_8H_{11}NO_7",                                "", "YZJRXCREGHPQLR-UHFFFAOYSA-N", "MCM:MXYMUCNO3", "")
  CALL Species("C_8H_{11}NO_7",                                "", "GICKPQRZPMBHKH-UHFFFAOYSA-N", "MCM:PXYMUCNO3", "")
  CALL Species("C_9H_{13}NO_7",                                "", "UHFJHZLCDDVXBJ-UHFFFAOYSA-N", "MCM:IPBZMUCNO3", "")
  CALL Species("C_9H_{13}NO_7",                                "", "WWNOXWKNSLKIMY-UHFFFAOYSA-N", "MCM:METLMUCNO3", "")
  CALL Species("C_9H_{13}NO_7",                                "", "QLDJRFHCOXXYTH-UHFFFAOYSA-N", "MCM:PBZMUCNO3", "")
  CALL Species("C_9H_{13}NO_7",                                "", "PIVPLJYVWKBKLM-UHFFFAOYSA-N", "MCM:PETLMUCNO3", "")
  CALL Species("C_9H_{13}NO_7",                                "", "NIRISDMZVMYRLC-UHFFFAOYSA-N", "MCM:TM124MUNO3", "")
  CALL Species("C_9H_{13}NO_7",                                "", "WYJKGTDXAIWZNI-UHFFFAOYSA-N", "MCM:TM135MUNO3", "")
  CALL Species("C_{10}H_{15}NO_7",                             "", "KKQBKVQGVIRRAG-UHFFFAOYSA-N", "MCM:DMEBMUNO3", "")
  CALL Species("C_{11}H_{17}NO_7",                             "", "YPPYKYPDSBIFHG-UHFFFAOYSA-N", "MCM:DETLMUNO3", "")
  CALL Species("C_3H_5NO_6",                                   "", "XNSIPMZBSRCICM-UHFFFAOYSA-N", "MCM:DMCNO3", "")
  CALL Species("C_3H_5NO_6",                                   "", "KROMLYNKDFILIM-UHFFFAOYSA-N", "MCM:MMFNO3", "")
  CALL Species("C_5H_9NO_6",                                   "", "LLNBZHMZWYQLAS-UHFFFAOYSA-N", "ISOP1N5OOH", "") ! SMILES: O=N(=O)OC/C(=C/CO)COO
  CALL Species("C_5H_9NO_8",                                   "", "ZSADVAXZKGJKSG-UHFFFAOYSA-N", "C520ONO2", "") ! SMILES: O=C(CO)C(CO)(COO)ON(=O)=O
  CALL Species("C_6H_{11}NO_6",                                "", "CRECMHNOVVIMCR-UHFFFAOYSA-N", "ROO6R6ONO2", "") ! SMILES: CC(=O)CCC(CON(=O)=O)OO
  CALL Species("C_6H_{11}NO_7",                                "", "XYBUKFJLXVIRLY-UHFFFAOYSA-N", "C624ONO2", "") ! SMILES: C=C(COO)C(CO)(CO)ON(=O)=O
  CALL Species("C_7H_{11}NO_7",                                "", "LNWOWXJMUJHMJJ-UHFFFAOYSA-N", "ROO6R5ONO2", "") ! SMILES: CC(=O)CCC(CC(=O)ON(=O)=O)OO
  CALL Species("C_{10}H_{17}NO_6",                             "", "JOFCABYMPNXMIS-UHFFFAOYSA-N", "ROO6R1ONO2", "") ! SMILES: CC1(CO)OOC2CC1CCC2(C)ON(=O)=O
  CALL Species("C_{10}H_{19}NO_7",                             "", "JBKULLYZCMWERK-UHFFFAOYSA-N", "LIMAB15ONO2OOH", "") ! SMILES: CC(CO)(OO)C1CCC(C)(ON(=O)=O)C(O)C1
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Nitriles with oxygen (\chem{C}, \chem{H}, \chem{O}, \chem{N})")
  CALL Species("HNCO",                                  "75-13-8", "OWIKHYCFFJSOEH-UHFFFAOYSA-N", "isocyanic acid", "")
  CALL Species("CH_3NCO",                              "624-83-9", "HAMGRBXTJNITHG-UHFFFAOYSA-N", "methyl isocyanate", "")
  CALL Species("C_2H_3NO",                             "107-16-4", "LTYRAPJYLUPLCI-UHFFFAOYSA-N", "hydroxyacetonitrile", "glycolonitrile")
  CALL Species("C_3H_5NO",                              "78-97-7", "WOFDVDFSGLBFAC-UHFFFAOYSA-N", "2-hydroxypropanenitrile", "")
  CALL Species("C_3H_5NO",                             "109-78-4", "WSGYTJNNHPZFKR-UHFFFAOYSA-N", "3-hydroxypropanenitrile", "ethylene cyanohydrin")
  CALL Species("C_4H_5NO_2",                           "105-34-0", "ANGDWNBGPBMQHW-UHFFFAOYSA-N", "methyl cyanoacetate", "")
  CALL Species("C_5H_7NO_2",                           "105-56-6", "ZIUSEGSNTOUIPT-UHFFFAOYSA-N", "cyanoethanoic acid, ethyl ester", "ethyl cyanoacetate")
  CALL Species("C_7H_5NO",                             "611-20-1", "CHZCERSEMVWNHL-UHFFFAOYSA-N", "2-hydroxybenzoic acid nitrile", "2-cyanophenol")
  CALL Species("C_7H_5NO",                             "873-62-1", "SGHBRHKBCLLVCI-UHFFFAOYSA-N", "3-hydroxybenzoic acid nitrile", "3-cyanophenol")
  CALL Species("C_7H_5NO",                             "767-00-0", "CVNOWLNNPYYEOH-UHFFFAOYSA-N", "4-hydroxybenzoic acid nitrile", "4-cyanophenol")
  CALL Species("C_7H_5NO",                             "103-71-9", "DGTNSSLYPYDJGL-UHFFFAOYSA-N", "phenyl isocyanate", "")
  CALL Species("C_9H_{21}NO_3",                        "122-20-3", "SLINHMUFWFWBMU-UHFFFAOYSA-N", "1,1',1''-nitrilotris-2-propanol", "triisopropanolamine")
  CALL Species("C_{10}H_7N_3O",                      "78370-21-5", "PYKLUAIDKVVEOS-JLHYYAGUSA-N", "cyometrinil", "")
  CALL Species("C_{22}H_{23}NO_3",                   "39515-41-8", "XQUXKZZNEFRCAW-UHFFFAOYSA-N", "fenpropathrin", "") ! other CAS: 64257-84-7
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Nitro compounds (\chem{RNO_2})")
  ! aliphatic:
  CALL Species("CH_3NO_2",                              "75-52-5", "LYGJENNIWJXYER-UHFFFAOYSA-N", "nitromethane", "")
  CALL Species("CH_3NO_2",                           "32480-00-5", "LYGJENNIWJXYER-OUBTZVSYSA-N", "nitromethane-13C", "")
  CALL Species("C_2H_5NO_2",                            "79-24-3", "MCSAJNNLRCFZED-UHFFFAOYSA-N", "nitroethane", "")
  CALL Species("C_3H_7NO_2",                           "108-03-2", "JSZOAYXJRCEYSX-UHFFFAOYSA-N", "1-nitropropane", "")
  CALL Species("CH_3CH(NO_2)CH_3",                      "79-46-9", "FGLBSLMDCBOPQK-UHFFFAOYSA-N", "2-nitropropane", "")
  CALL Species("C_4H_9NO_2",                           "627-05-4", "NALZTFARIYUCBY-UHFFFAOYSA-N", "1-nitrobutane", "")
  CALL Species("C_4H_9ONO",                            "540-80-7", "IOGXOCVLYRDXLW-UHFFFAOYSA-N", "{\it tert}-butylnitrite", "")
  CALL Species("C_5H_{11}NO_2",                        "628-05-7", "BVALZCVRLDMXOQ-UHFFFAOYSA-N", "1-nitropentane", "")
  CALL Species("C_5H_9N_3O_9",                        "3032-55-1", "IPPYBNCEPZCLNI-UHFFFAOYSA-N", "tris(hydroxymethyl)ethane trinitrate", "")
  CALL Species("C_6H_{13}NO_2",                        "646-14-0", "FEYJIFXFOHFGCC-UHFFFAOYSA-N", "1-nitrohexane", "")
  CALL Species("C_6H_{11}NO_2",                       "1122-60-7", "NJNQUTDUIPVROZ-UHFFFAOYSA-N", "nitrocyclohexane", "")
  CALL Species("C_2H_5NO_3",                           "625-48-9", "KIPMDPDAFINLIV-UHFFFAOYSA-N", "2-nitroethanol", "")
  CALL Species("C_3H_7NO_3",                          "3156-73-8", "PFNCKQIYLAVYJF-UHFFFAOYSA-N", "1-nitro-2-propanol", "")
  CALL Species("C_3H_7NO_3",                          "2902-96-7", "PCNWBUOSTLGPMI-UHFFFAOYSA-N", "2-nitro-1-propanol", "")
  CALL Species("C_4H_9NO_3",                          "3156-74-9", "FMEFHKJRIGHSLB-UHFFFAOYSA-N", "1-nitro-2-butanol", "")
  CALL Species("C_4H_9NO_3",                           "609-31-4", "MHIHRIPETCJEMQ-UHFFFAOYSA-N", "2-nitro-1-butanol", "")
  CALL Species("C_4H_9NO_3",                          "6270-16-2", "OJVOGABFNZDOOZ-UHFFFAOYSA-N", "3-nitro-2-butanol", "")
  CALL Species("CH_4N_4O_2",                           "556-88-7", "IDCPFAYURAQKDZ-UHFFFAOYSA-N", "nitroguanidine", "")
  CALL Species("CN_4O_8",                              "509-14-8", "NYTOUQBROMCLBJ-UHFFFAOYSA-N", "tetranitromethane", "")
  CALL Species("C_2H_5N_5O_3",                          "70-25-7", "VZUNGTLZRAYYDE-UHFFFAOYSA-N", "N-methyl-N'-nitro-N-nitrosoguanidine", "")
  CALL Species("C_4H_9NO_5",                           "126-11-4", "OLQJQHSAWMFDJE-UHFFFAOYSA-N", "2-(hydroxymethyl)-2-nitro-1,3-propanediol", "")
  CALL Species("C_4H_3NO_5",                                   "", "ODFCIXRXNCCVGH-UHFFFAOYSA-N", "MCM:NC4DCO2H", "")
  CALL Species("C_5H_5NO_5",                                   "", "LXEJKNKBIPISQO-UHFFFAOYSA-N", "MCM:NC4MDCO2H", "")
  CALL Species("C_6H_7NO_5",                                   "", "GVSLZEQKUBDLJE-UHFFFAOYSA-N", "MCM:NC4EDCO2H", "")
  CALL Species("C_6H_6N_2O_{10}",                              "", "AKJGOKAIBHUAAE-UHFFFAOYSA-N", "MCM:DNPHENOOH", "")
  CALL Species("C_6H_7NO_9",                                   "", "UERHMILVGXASDJ-UHFFFAOYSA-N", "MCM:NCATECOOH", "")
  CALL Species("C_6H_7NO_5",                                   "", "VRGZGZRSZACGSZ-UHFFFAOYSA-N", "MCM:NC5MDCO2H", "")
  CALL Species("C_7H_9NO_5",                                   "", "VOBLVGUNJWYRHD-UHFFFAOYSA-N", "MCM:NC4IPDCO2H", "")
  CALL Species("C_7H_9NO_5",                                   "", "MSTRQTMGALLSFO-UHFFFAOYSA-N", "MCM:NC4PDCO2H", "")
  CALL Species("C_7H_8N_2O_{10}",                              "", "FHWDGQSKNURQKZ-UHFFFAOYSA-N", "MCM:DNCRESOOH", "")
  CALL Species("C_7H_9NO_9",                                   "", "OCHAAZKMUDWIOE-UHFFFAOYSA-N", "MCM:MNCATECOOH", "")
  CALL Species("C_7H_9NO_8",                                   "", "MEXRHSIIQRTIBR-UHFFFAOYSA-N", "MCM:TL4ONO2OOH", "")
  CALL Species("C_8H_{10}N_2O_{10}",                           "", "FNCRENCSRXRIPJ-UHFFFAOYSA-N", "MCM:DNEBNZLOOH", "")
  CALL Species("C_8H_{10}N_2O_{10}",                           "", "KSAVSEUSLRIGAB-UHFFFAOYSA-N", "MCM:DNMXYOLOOH", "")
  CALL Species("C_8H_{10}N_2O_{10}",                           "", "PBZRLTNSSYSOGL-UHFFFAOYSA-N", "MCM:DNOXYOLOOH", "")
  CALL Species("C_8H_{10}N_2O_{10}",                           "", "AYHFAKIJCOHXBG-UHFFFAOYSA-N", "MCM:DNPXYOLOOH", "")
  CALL Species("C_8H_{11}NO_9",                                "", "QJBJEZDKVLRDGQ-UHFFFAOYSA-N", "MCM:ENCATECOOH", "")
  CALL Species("C_8H_{11}NO_9",                                "", "QYJOURKZCRGVGB-UHFFFAOYSA-N", "MCM:MXNCATCOOH", "")
  CALL Species("C_8H_{11}NO_8",                                "", "RRWTXIWIABDEOZ-UHFFFAOYSA-N", "MCM:MXOHNO2OOH", "")
  CALL Species("C_8H_{11}NO_9",                                "", "GWPIPVPJLOZVKC-UHFFFAOYSA-N", "MCM:OXNCATCOOH", "")
  CALL Species("C_8H_{11}NO_9",                                "", "RMUFWTRDQQMPCD-UHFFFAOYSA-N", "MCM:PXNCATCOOH", "")
  CALL Species("C_8H_{11}NO_8",                                "", "LGMZUZOZZXWRQX-UHFFFAOYSA-N", "MCM:TM124NOOH", "")
  CALL Species("C_9H_{12}N_2O_{10}",                           "", "YKBFACRSJWRBHR-UHFFFAOYSA-N", "MCM:DNIPBZLOOH", "")
  CALL Species("C_9H_{12}N_2O_{10}",                           "", "LAMZNFDUTXWEFA-UHFFFAOYSA-N", "MCM:DNMETOLOOH", "")
  CALL Species("C_9H_{12}N_2O_{10}",                           "", "KMVIJWGCTFHDMC-UHFFFAOYSA-N", "MCM:DNOETOLOOH", "")
  CALL Species("C_9H_{12}N_2O_{10}",                           "", "YOHJWFCUZUQBMA-UHFFFAOYSA-N", "MCM:DNPBNZLOOH", "")
  CALL Species("C_9H_{12}N_2O_{10}",                           "", "IOZSITWJVFMIBH-UHFFFAOYSA-N", "MCM:DNPETOLOOH", "")
  CALL Species("C_9H_{12}N_2O_{10}",                           "", "GBPJMJLXQYJDMC-UHFFFAOYSA-N", "MCM:DNT123LOOH", "")
  CALL Species("C_9H_{12}N_2O_{10}",                           "", "INCIWBVFUFSSNB-UHFFFAOYSA-N", "MCM:DNT124LOOH", "")
  CALL Species("C_9H_{13}NO_8",                                "", "FZXCIHOWKZKJRK-UHFFFAOYSA-N", "MCM:EMPONO2OOH", "")
  CALL Species("C_9H_{13}NO_9",                                "", "NYRKVILWQFQUNU-UHFFFAOYSA-N", "MCM:IPNCATCOOH", "")
  CALL Species("C_9H_{13}NO_9",                                "", "MCLPDHQAEZEULP-UHFFFAOYSA-N", "MCM:MTNCATCOOH", "")
  CALL Species("C_9H_{13}NO_9",                                "", "MVMRAYXEOTUUSG-UHFFFAOYSA-N", "MCM:OTNCATCOOH", "")
  CALL Species("C_9H_{13}NO_9",                                "", "AWPUGBPLGLTYEJ-UHFFFAOYSA-N", "MCM:PNCATECOOH", "")
  CALL Species("C_9H_{13}NO_9",                                "", "IJEUJFHQTRRKRP-UHFFFAOYSA-N", "MCM:PTNCATCOOH", "")
  ! aromatic:
  CALL Species("C_6H_5NO_2",                            "98-95-3", "LQNUZADURLCDLV-UHFFFAOYSA-N", "nitrobenzene", "")
  CALL Species("C_6D_5NO_2",                          "4165-60-0", "LQNUZADURLCDLV-RALIUCGRSA-N", "nitrobenzene-d5", "")
  CALL Species("C_6H_4(NO_2)CH_3",                      "88-72-2", "PLAZTCDQAHEYBI-UHFFFAOYSA-N", "2-nitrotoluene", "")
  CALL Species("C_6H_4(NO_2)CH_3",                      "99-08-1", "QZYHIOPPLUPUJF-UHFFFAOYSA-N", "3-nitrotoluene", "")
  CALL Species("C_6H_4(NO_2)CH_3",                      "99-99-0", "ZPTVNYMJQHSSEA-UHFFFAOYSA-N", "4-nitrotoluene", "")
  CALL Species("C_6H_4N_2O_4",                         "528-29-0", "IZUKQUVSCNEFMJ-UHFFFAOYSA-N", "1,2-dinitrobenzene", "")
  CALL Species("C_6H_4N_2O_4",                          "99-65-0", "WDCYWAQPCXBPJA-UHFFFAOYSA-N", "1,3-dinitrobenzene", "")
  CALL Species("C_6H_4N_2O_4",                         "100-25-4", "FYFDQJRXFWGIBS-UHFFFAOYSA-N", "1,4-dinitrobenzene", "")
  CALL Species("C_6H_3N_3O_6",                          "99-35-4", "UATJOMSPNYCXIX-UHFFFAOYSA-N", "1,3,5-trinitrobenzene", "")
  CALL Species("C_6H_3N_3O_7",                          "88-89-1", "OXNIZHLAWKMVMX-UHFFFAOYSA-N", "2,4,6-trinitrophenol", "picric acid")
  CALL Species("C_6H_6N_2O_2",                          "88-74-4", "DPJCXCZTLWNFOH-UHFFFAOYSA-N", "2-nitrobenzenamine", "2-nitroaniline")
  CALL Species("C_6H_6N_2O_2",                          "99-09-2", "XJCVRTZCHMZPBD-UHFFFAOYSA-N", "3-nitrobenzenamine", "3-nitroaniline")
  CALL Species("C_6H_6N_2O_2",                         "100-01-6", "TYMLOMAKGOJONV-UHFFFAOYSA-N", "4-nitrobenzenamine", "4-nitroaniline")
  CALL Species("C_6H_5N_3O_4",                          "97-02-9", "LXQOQPGNCGEELI-UHFFFAOYSA-N", "2,4-dinitrobenzenamine", "")
  CALL Species("C_7H_8N_2O_2",                         "570-24-1", "FCMRHMPITHLLLA-UHFFFAOYSA-N", "2-methyl-6-nitroaniline", "")
  CALL Species("C_7H_6N_2O_4",                         "602-01-7", "DYSXLQBUUOPLBB-UHFFFAOYSA-N", "1-methyl-2,3-dinitrobenzene", "2,3-dinitrotoluene; 2,3-DNT")
  CALL Species("C_7H_6N_2O_4",                         "121-14-2", "RMBFBMJGBANMMK-UHFFFAOYSA-N", "1-methyl-2,4-dinitrobenzene", "2,4-dinitrotoluene; 2,4-DNT")
  CALL Species("C_7H_6N_2O_4",                         "619-15-8", "KZBOXYKTSUUBTO-UHFFFAOYSA-N", "2-methyl-1,4-dinitrobenzene", "2,5-dinitrotoluene; 2,5-DNT")
  CALL Species("C_7H_6N_2O_4",                         "606-20-2", "XTRDKALNCIHHNI-UHFFFAOYSA-N", "2-methyl-1,3-dinitrobenzene", "2,6-dinitrotoluene; 2,6-DNT")
  CALL Species("C_7H_6N_2O_4",                         "610-39-9", "INYDMNPNDHRJQJ-UHFFFAOYSA-N", "4-methyl-1,2-dinitrobenzene", "3,4-dinitrotoluene; 3,4-DNT")
  CALL Species("C_7H_5N_3O_6",                         "118-96-7", "SPSSULHKWOKEEL-UHFFFAOYSA-N", "1-methyl-2,4,6-trinitrobenzene", "2,4,6-trinitrotoluene; TNT")
  CALL Species("C_{12}H_{15}N_3O_6",                    "81-15-2", "XMWRWTSZNLOZFN-UHFFFAOYSA-N", "2,4,6-trinitro-1,3-dimethyl-5-{\it tert}-butylbenzene", "musk xylene")
  CALL Species("HOC_6H_4(NO_2)",                        "88-75-5", "IQUPABOKLQSFBK-UHFFFAOYSA-N", "2-nitrophenol", "")
  CALL Species("HOC_6H_4(NO_2)",                       "554-84-7", "RTZZCYNQPHTPPL-UHFFFAOYSA-N", "3-nitrophenol", "")
  CALL Species("HOC_6H_4(NO_2)",                       "100-02-7", "BTJIUGUIPKRLHP-UHFFFAOYSA-N", "4-nitrophenol", "")
  CALL Species("C_7H_7NO_3",                           "100-17-4", "BNUHAJGCKIQFGE-UHFFFAOYSA-N", "4-nitroanisole", "")
  CALL Species("C_7H_7NO_3",                          "4920-77-8", "QIORDSKCCHRSSD-UHFFFAOYSA-N", "3-methyl-2-nitrophenol", "")
  CALL Species("C_7H_7NO_3",                           "119-33-5", "SYDNSSSQVSOXTN-UHFFFAOYSA-N", "4-methyl-2-nitrophenol", "")
  CALL Species("C_7H_7NO_3",                           "700-38-9", "NQXUSSVLFOBRSE-UHFFFAOYSA-N", "5-methyl-2-nitrophenol", "")
  CALL Species("C_7H_7NO_3",                         "13073-29-5", "AQDKZPFDOWHRDZ-UHFFFAOYSA-N", "6-methyl-2-nitrophenol", "")
  CALL Species("C_7H_7NO_3",                          "2581-34-2", "PIIZYNQECPTVEO-UHFFFAOYSA-N", "3-methyl-4-nitrophenol", "")
  CALL Species("C_7H_7NO_4",                          "1568-70-3", "YBUGOACXDPDUIR-UHFFFAOYSA-N", "4-methoxy-2-nitrophenol", "")
  CALL Species("C_7H_7N_3O_4",                       "19406-51-0", "KQRJATLINVYHEZ-UHFFFAOYSA-N", "4-amino-2,6-dinitrotoluene", "")
  CALL Species("C_7H_5NO_4",                          "3011-34-5", "YTHJCZRFJGXPTL-UHFFFAOYSA-N", "4-hydroxy-3-nitro-benzaldehyde", "")
  CALL Species("C_6H_4N_2O_5",                          "51-28-5", "UFBJCMHMOXMLKC-UHFFFAOYSA-N", "2,4-dinitrophenol", "") ! other CAS: 1326-82-5
  CALL Species("C_6H_4N_2O_5",                         "329-71-5", "UWEZBKLLMKVIPI-UHFFFAOYSA-N", "2,5-dinitrophenol", "")
  CALL Species("C_6H_5N_3O_5",                          "96-91-3", "QXYMVUZOGFVPGH-UHFFFAOYSA-N", "picramic acid", "4,6-dinitro-2-aminophenol")
  CALL Species("C_6H_6N_2O_3",                         "119-34-6", "WHODQVWERNSQEO-UHFFFAOYSA-N", "4-amino-2-nitrophenol", "")
  CALL Species("C_6H_6N_2O_3",                         "121-88-0", "DOPJTDJKZNWLRB-UHFFFAOYSA-N", "2-amino-5-nitrophenol", "")
  CALL Species("C_6H_6N_2O_3",                          "99-57-0", "VLZVIIYRNMWPSN-UHFFFAOYSA-N", "2-amino-4-nitrophenol", "")
  CALL Species("C_6H_7N_3O_2",                          "99-56-9", "RAUWPNXIALNKQM-UHFFFAOYSA-N", "4-nitro-$o$-phenylenediamine", "4-nitro-1,2-diaminobenzene")
  CALL Species("C_6H_7N_3O_2",                        "5131-58-8", "DPIZKMGPXNXSGL-UHFFFAOYSA-N", "4-nitrobenzene-1,3-diamine", "")
  CALL Species("C_6H_7N_3O_2",                        "5307-14-2", "HVHNMNGARPCGGD-UHFFFAOYSA-N", "2-nitro-1,4-benzenediamine", "")
  CALL Species("C_7H_6N_2O_5",                         "609-93-8", "HOYRZHJJAHRMLL-UHFFFAOYSA-N", "4-methyl-2,6-dinitrophenol", "2,6-dinitro-$p$-cresol")
  CALL Species("C_7H_6N_2O_5",                         "534-52-1", "ZXVONLUNISGICL-UHFFFAOYSA-N", "2-methyl-4,6-dinitrophenol", "6-methyl-2,4-dinitrophenol; 4,6-dinitro-$o$-cresol; DNOC")
  CALL Species("C_7H_5N_3O_2",                          "94-52-0", "XPAZGLFMMUODDK-UHFFFAOYSA-N", "5-nitrobenzimidazole", "")
  CALL Species("C_7H_5NO_4",                           "121-92-6", "AFPHTEQTJZKQAQ-UHFFFAOYSA-N", "3-nitrobenzoic acid", "")
  CALL Species("C_7H_5NO_4",                            "62-23-7", "OTLNPYWUJOZPPA-UHFFFAOYSA-N", "4-nitrobenzoic acid", "")
  CALL Species("C_7H_4N_2O_6",                          "99-34-3", "VYWYYJYRVSBHJQ-UHFFFAOYSA-N", "3,5-dinitrobenzoic acid", "")
  CALL Species("C_7H_3N_3O_8",                         "129-66-8", "KAQBNBSMMVTKRN-UHFFFAOYSA-N", "2,4,6-trinitrobenzoic acid", "")
  CALL Species("C_7H_6N_2O_4",                       "25321-14-6", "MOSFIJXAXDLOML-UHFFFAOYSA-N", "dinitrotoluene", "")
  CALL Species("C_7H_6N_2O_4",                         "618-85-9", "RUIFULUFLANOCI-UHFFFAOYSA-N", "1-methyl-3,5-dinitrobenzene", "")
  CALL Species("C_7H_7NO_3",                            "91-23-6", "CFBYEGUGFPZCNF-UHFFFAOYSA-N", "1-methoxy-2-nitrobenzene", "")
  CALL Species("C_7H_8N_2O_2",                          "99-55-8", "DSBIJCMXAIKKKI-UHFFFAOYSA-N", "2-methyl-5-nitrobenzenamine", "5-nitro-$o$-toluidine")
  CALL Species("C_7H_8N_2O_3",                          "99-59-2", "NIPDVSLAMPAWTP-UHFFFAOYSA-N", "2-methoxy-5-nitrobenzenamine", "5-nitro-$o$-anisidine")
  CALL Species("C_8H_7NO_2",                           "102-96-5", "PIAOLBVUVDXHHL-UHFFFAOYSA-N", "(2-nitroethenyl)benzene", "")
  CALL Species("C_8H_9NO_2",                            "83-41-0", "FVHAWXWFPBPFOS-UHFFFAOYSA-N", "1,2-dimethyl-3-nitrobenzene", "")
  CALL Species("C_8H_9NO_2",                            "99-51-4", "HFZKOYWDLDYELC-UHFFFAOYSA-N", "1,2-dimethyl-4-nitrobenzene", "")
  CALL Species("C_8H_9NO_2",                            "89-58-7", "BSFHJMGROOFSRA-UHFFFAOYSA-N", "1,4-dimethyl-2-nitrobenzene", "")
  CALL Species("C_8H_9NO_2",                            "89-87-2", "BBUPBICWUURTNP-UHFFFAOYSA-N", "2,4-dimethyl-1-nitrobenzene", "")
  CALL Species("C_8H_9NO_3",                           "119-10-8", "LGNMURXRPLMVJI-UHFFFAOYSA-N", "4-methyl-2-nitroanisole", "")
  CALL Species("C_8H_7NO_4",                          "1975-50-4", "YPQAFWHSMWWPLX-UHFFFAOYSA-N", "2-methyl-3-nitrobenzoic acid", "")
  CALL Species("C_8H_7NO_4",                          "3113-71-1", "XDTTUTIFWDAMIX-UHFFFAOYSA-N", "3-methyl-4-nitrobenzoic acid", "")
  CALL Species("C_8H_6N_2O_6",                       "28169-46-2", "CDVNZMKTJIBBBV-UHFFFAOYSA-N", "2-methyl-3,5-dinitrobenzoic acid", "")
  CALL Species("C_9H_{11}NO_2",                       "1817-47-6", "JXMYUMNAEKRMIP-UHFFFAOYSA-N", "1-(1-methylethyl)-4-nitrobenzene", "")
  CALL Species("C_9H_6N_2O_3",                        "4008-48-4", "RJIWZDNTCBHXAL-UHFFFAOYSA-N", "5-nitro-8-hydroxyquinoline", "nitroxoline")
  CALL Species("C_{10}H_{12}N_2O_5",                    "88-85-7", "OWZPCEFYPSAJFR-UHFFFAOYSA-N", "2-(1-methylpropyl)-4,6-dinitrophenol", "dinoseb")
  CALL Species("C_{10}H_7NO_2",                         "86-57-7", "RJKGJBPXVHTNJL-UHFFFAOYSA-N", "1-nitronaphthalene", "")
  CALL Species("C_{10}H_7NO_2",                        "581-89-5", "ZJYJZEAJZXVAMF-UHFFFAOYSA-N", "2-nitronaphthalene", "")
  CALL Species("C_{10}H_6N_2O_4",                      "606-37-1", "ULALSFRIGPMWRS-UHFFFAOYSA-N", "1,3-dinitronaphthalene", "")
  CALL Species("C_{10}H_6N_2O_4",                      "605-71-0", "ZUTCJXFCHHDFJS-UHFFFAOYSA-N", "1,5-dinitronaphthalene", "")
  CALL Species("C_{10}H_6N_2O_4",                      "602-38-0", "AVCSMMMOCOTIHF-UHFFFAOYSA-N", "1,8-dinitronaphthalene", "")
  CALL Species("C_{10}H_{12}N_2O_5",                  "1420-07-1", "IIPZYDQGBIWLBU-UHFFFAOYSA-N", "dinoterb", "")
  CALL Species("C_{10}H_{13}NO_3",                    "3555-18-8", "GCDCKEORRIGZKI-UHFFFAOYSA-N", "4-(1-methylpropyl)-2-nitrophenol", "4-{\it sec}-butyl-2-nitrophenol")
  CALL Species("C_{11}H_9NO_2",                        "880-93-3", "FRLVKAJKOYFHKQ-UHFFFAOYSA-N", "1-methyl-4-nitronaphthalene", "")
  CALL Species("C_{11}H_9NO_2",                      "91137-27-8", "AOSRALZFDNFXFZ-UHFFFAOYSA-N", "1-methyl-5-nitronaphthalene", "")
  CALL Species("C_{11}H_9NO_2",                     "105752-67-8", "SOXBGESONWKYDX-UHFFFAOYSA-N", "1-methyl-6-nitronaphthalene", "")
  CALL Species("C_{11}H_9NO_2",                        "881-03-8", "IZNWACYOILBFEG-UHFFFAOYSA-N", "2-methyl-1-nitronaphthalene", "")
  CALL Species("C_{11}H_9NO_2",                      "13615-38-8", "HKMFJWNUOWBRGF-UHFFFAOYSA-N", "3-methyl-1-nitronaphthalene", "")
  CALL Species("C_{12}H_7NO_3",                       "5410-97-9", "UVFAHDAUVZRVCC-UHFFFAOYSA-N", "3-nitrodibenzofuran", "")
  CALL Species("C_{12}H_{16}N_2O_5",                    "83-66-9", "SUAUILGSCPYJCS-UHFFFAOYSA-N", "musk ambrette (artificial)", "")
  CALL Species("C_{12}H_8N_2O_5",                      "101-63-3", "MWAGUKZCDDRDCS-UHFFFAOYSA-N", "bis($p$-nitrophenyl) ether", "")
  CALL Species("C_{12}H_9N_3O_2",                     "2491-52-3", "TZTDJBMGPQLSLI-UHFFFAOYSA-N", "4-nitroazobenzene", "")
  CALL Species("C_{12}H_{10}N_2O_2",                   "836-30-6", "XXYMSQQCBUKFHE-UHFFFAOYSA-N", "4-nitro-N-phenylbenzenamine", "")
  CALL Species("C_{12}H_{14}N_2O_5",                   "131-89-5", "QJYHUJAGJUHXJN-UHFFFAOYSA-N", "2-cyclohexyl-4,6-dinitrophenol", "")
  CALL Species("C_{12}H_{14}N_2O_6",                  "2813-95-8", "RDJTWDKSYLLHRW-UHFFFAOYSA-N", "dinoseb acetate", "")
  CALL Species("C_{12}H_5N_7O_{12}",                   "131-73-7", "CBCIHIVRDWLAME-UHFFFAOYSA-N", "dipicrylamine", "2,2',4,4',6,6'-hexanitrodiphenylamine")
  CALL Species("C_{12}H_9NO_2",                        "602-87-9", "CUARLQDWYSRQDF-UHFFFAOYSA-N", "1,2-dihydro-5-nitroacenaphthylene", "5-nitroacenaphthene")
  CALL Species("C_{12}H_9NO_2",                         "86-00-0", "YOJKKXRJMXIKSR-UHFFFAOYSA-N", "2-nitro-1,1'-biphenyl", "")
  CALL Species("C_{12}H_9NO_2",                       "2113-58-8", "FYRPEHRWMVMHQM-UHFFFAOYSA-N", "3-nitro-1,1'-biphenyl", "")
  CALL Species("C_{12}H_9NO_2",                         "92-93-3", "BAJQRLZAPXASRD-UHFFFAOYSA-N", "4-nitro-1,1'-biphenyl", "")
  CALL Species("C_{13}H_9NO_2",                        "607-57-8", "XFOHWECQTFIEIX-UHFFFAOYSA-N", "2-nitro-9H-fluorene", "")
  CALL Species("C_{13}H_{18}N_2O_4",                   "145-39-1", "MINYPECWDZURGR-UHFFFAOYSA-N", "5-{\it tert}-butyl-4,6-dinitro-1,2,3-trimethylbenzene", "")
  CALL Species("C_{13}H_{19}N_3O_4",                 "40487-42-1", "CHIFOSRWCNZCFN-UHFFFAOYSA-N", "penoxaline", "pendimethalin")
  CALL Species("C_{14}H_{20}N_2O_5",                  "4097-33-0", "NYGISSDEOKKXOE-UHFFFAOYSA-N", "2,6-dinitro-4-octylphenol", "")
  CALL Species("C_{14}H_{18}N_2O_5",                    "81-14-1", "WXCMHFPAUCOJIG-UHFFFAOYSA-N", "musk ketone", "")
  CALL Species("C_{14}H_{18}N_2O_4",                   "116-66-5", "UHWURQRPEIFIAK-UHFFFAOYSA-N", "moskene", "")
  CALL Species("C_{14}H_{17}NO_6",                   "10552-74-6", "VJAWBEFMCIINFU-UHFFFAOYSA-N", "nitrothal-isopropyl", "")
  CALL Species("C_{14}H_{12}N_2O_2",                    "86-20-4", "WONHLSYSHMRRGO-UHFFFAOYSA-N", "9-ethyl-3-nitrocarbazole", "")
  CALL Species("C_{14}H_9NO_2",                       "3586-69-4", "NZWGQBVOKHEKLD-UHFFFAOYSA-N", "2-nitroanthracene", "")
  CALL Species("C_{14}H_9NO_2",                        "602-60-8", "LSIKFJXEYJIZNB-UHFFFAOYSA-N", "9-nitroanthracene", "")
  CALL Species("C_{14}H_9NO_2",                      "17024-19-0", "CPRHWWUDRYJODK-UHFFFAOYSA-N", "3-nitrophenanthrene", "")
  CALL Species("C_{14}H_9NO_2",                        "954-46-1", "QTTCNQHPKFAYEZ-UHFFFAOYSA-N", "9-nitrophenanthrene", "")
  CALL Species("C_{15}H_{18}N_2O_6",                   "485-31-4", "ZRDUSMYWDRPZRM-UHFFFAOYSA-N", "binapacryl", "")
  CALL Species("C_{16}H_9NO_2",                      "13177-29-2", "VBCBFNMZBHKVQN-UHFFFAOYSA-N", "2-nitrofluoranthene", "")
  CALL Species("C_{16}H_9NO_2",                        "892-21-7", "PIHGQKMEAMSUNA-UHFFFAOYSA-N", "3-nitrofluoranthene", "")
  CALL Species("C_{16}H_{10}N_4O_5",                  "3468-63-1", "HBHZKFOUIUMKHV-UHFFFAOYSA-N", "1-[(2,4-dinitrophenyl)azo]-2-naphthol", "C.I. pigment orange 5")
  CALL Species("C_{16}H_8N_2O_4",                   "105735-71-5", "WAAHHGKGQYVTNS-UHFFFAOYSA-N", "3,7-dinitrofluoranthene", "")
  CALL Species("C_{16}H_8N_2O_4",                    "75321-20-9", "UJIPQBOHUQDIAA-UHFFFAOYSA-N", "1,3-dinitropyrene", "")
  CALL Species("C_{16}H_8N_2O_4",                    "42397-64-8", "GUXACCKTQWVTLG-UHFFFAOYSA-N", "1,6-dinitropyrene", "")
  CALL Species("C_{16}H_8N_2O_4",                    "42397-65-9", "BLYXNIHKOMELAP-UHFFFAOYSA-N", "1,8-dinitropyrene", "")
  CALL Species("C_{16}H_9NO_2",                       "5522-43-0", "ALRLPDGCPYIVHP-UHFFFAOYSA-N", "1-nitropyrene", "")
  CALL Species("C_{16}H_9NO_2",                        "789-07-1", "MAZCGYFIOOIVHE-UHFFFAOYSA-N", "2-nitropyrene", "")
  CALL Species("C_{16}H_9NO_2",                      "57835-92-4", "UISKIUIWPSPSAV-UHFFFAOYSA-N", "4-nitropyrene", "")
  CALL Species("C_{16}H_{18}N_4O_2",                  "3025-52-3", "LVQIWDUSUJTZJF-ISLYRVAYSA-N", "N,N-diethyl-4-[(4-nitrophenyl)azo]aniline", "")
  CALL Species("C_{16}H_{18}N_4O_3",                  "2872-52-8", "FOQABOMYTOFLPZ-ZCXUNETKSA-N", "disperse red 1", "")
  CALL Species("C_{17}H_{13}N_3O_3",                  "2425-85-6", "ZLFVRXUOSPRRKQ-UHFFFAOYSA-N", "1-[(4-methyl-2-nitrophenyl)azo]-2-naphthalenol", "C.I. Pigment Red 3")
  CALL Species("C_{17}H_{11}NO_5",                   "65208-34-6", "DMPUGHYNLCGVPX-UHFFFAOYSA-N", "phenyl 1-hydroxy-4-nitro-2-naphthoate", "")
  CALL Species("C_{18}H_{11}NO_2",                   "20268-51-3", "KOPVBVBUIYTJBG-UHFFFAOYSA-N", "7-nitrobenz[$a$]anthracene", "")
  CALL Species("C_{18}H_{11}NO_2",                    "7496-02-8", "UAWLTQJFZUYROA-UHFFFAOYSA-N", "6-nitrochrysene", "")
  CALL Species("C_{18}H_{24}N_2O_6",                   "131-72-6", "NIOPZPCMRQGZCE-WEVVVXLNSA-N", "meptyldinocap", "") ! isomer mix = 6119-92-2
  CALL Species("C_{20}H_{11}NO_2",                   "70021-99-7", "ICKISBPYFVBVQG-UHFFFAOYSA-N", "1-nitrobenzo[$a$]pyrene", "")
  CALL Species("C_{20}H_{11}NO_2",                   "70021-98-6", "CQIJHYPCKYZMIV-UHFFFAOYSA-N", "3-nitrobenzo[$a$]pyrene", "")
  CALL Species("C_{20}H_{11}NO_2",                   "63041-90-7", "NMMAFYSZGOFZCM-UHFFFAOYSA-N", "6-nitrobenzo[$a$]pyrene", "")
  CALL Species("C_6H_5NO_4",                                   "", "QKMAXSKQIHICGE-UHFFFAOYSA-N", "MCM:NPHEN1OOH", "")
  CALL Species("C_6H_5NO_4",                                   "", "XJNPNXSISMKQEX-UHFFFAOYSA-N", "MCM:NCATECHOL", "")
  CALL Species("C_7H_7NO_4",                                   "", "PUXUHFKGXFXGAC-UHFFFAOYSA-N", "MCM:NCRES1OOH", "")
  CALL Species("C_7H_7NO_4",                                   "", "DPKDSDOOIONLAG-UHFFFAOYSA-N", "MCM:MNCATECH", "")
  CALL Species("C_8H_9NO_4",                                   "", "WELXCAQABDDCET-UHFFFAOYSA-N", "MCM:NEBNZ1OOH", "")
  CALL Species("C_8H_9NO_4",                                   "", "WHVCHTZZXIECJY-UHFFFAOYSA-N", "MCM:NMXYOL1OOH", "")
  CALL Species("C_8H_9NO_4",                                   "", "WTJTXPQAQHBAPV-UHFFFAOYSA-N", "MCM:NOXYOL1OOH", "")
  CALL Species("C_8H_9NO_4",                                   "", "MKCNKDVBVLGUFV-UHFFFAOYSA-N", "MCM:NPXYOL1OOH", "")
  CALL Species("C_8H_9NO_3",                                   "", "KGDIYDUZVHFMHQ-UHFFFAOYSA-N", "MCM:DM124OHNO2", "")
  CALL Species("C_8H_9NO_3",                                   "", "YXNYMZXPUWOUJT-UHFFFAOYSA-N", "MCM:DMPHOHNO2", "")
  CALL Species("C_8H_8N_2O_5",                                 "", "SYWMIOFIFBKHTK-UHFFFAOYSA-N", "MCM:DNEBNZOL", "")
  CALL Species("C_8H_8N_2O_5",                                 "", "MHXAYMPVZDCVJX-UHFFFAOYSA-N", "MCM:DNMXYOL", "")
  CALL Species("C_8H_8N_2O_5",                                 "", "JCTQXQRGEDMCSI-UHFFFAOYSA-N", "MCM:DNOXYOL", "")
  CALL Species("C_8H_8N_2O_5",                                 "", "RROXWBBJMPCPHD-UHFFFAOYSA-N", "MCM:DNPXYOL", "")
  CALL Species("C_8H_9NO_3",                                   "", "RSETVJMGQZKFCF-UHFFFAOYSA-N", "MCM:EBNZOHNO2", "")
  CALL Species("C_8H_9NO_4",                                   "", "CGDBCFCRALNPBR-UHFFFAOYSA-N", "MCM:ENCATECH", "")
  CALL Species("C_8H_9NO_4",                                   "", "DNJZITGVNONOFW-UHFFFAOYSA-N", "MCM:MXNCATECH", "")
  CALL Species("C_8H_9NO_3",                                   "", "KJRCHILWKQLEBC-UHFFFAOYSA-N", "MCM:MXY1OHNO2", "")
  CALL Species("C_8H_9NO_4",                                   "", "BBBHATWMUPCFKG-UHFFFAOYSA-N", "MCM:OXNCATECH", "")
  CALL Species("C_8H_9NO_3",                                   "", "KXWOAPZXQJGYPU-UHFFFAOYSA-N", "MCM:OXY1OHNO2", "")
  CALL Species("C_8H_9NO_4",                                   "", "MAHUOFZDOBGDSU-UHFFFAOYSA-N", "MCM:PXNCATECH", "")
  CALL Species("C_8H_9NO_3",                                   "", "VIQHHRZADKSPIM-UHFFFAOYSA-N", "MCM:PXY1OHNO2", "")
  CALL Species("C_9H_{11}NO_4",                                "", "IXEXRYHUTBEDFT-UHFFFAOYSA-N", "MCM:NIPBNZ1OOH", "")
  CALL Species("C_9H_{11}NO_4",                                "", "OTBVEKBEKVMFBF-UHFFFAOYSA-N", "MCM:NMETOL1OOH", "")
  CALL Species("C_9H_{11}NO_4",                                "", "ZYYASKSQLLYACE-UHFFFAOYSA-N", "MCM:NOETOL1OOH", "")
  CALL Species("C_9H_{11}NO_4",                                "", "CVKCTJQZVRPANA-UHFFFAOYSA-N", "MCM:NPBNZ1OOH", "")
  CALL Species("C_9H_{11}NO_4",                                "", "PGRGKKYGDUKVIK-UHFFFAOYSA-N", "MCM:NPETOL1OOH", "")
  CALL Species("C_9H_{11}NO_4",                                "", "QUQYRHULCDEZAA-UHFFFAOYSA-N", "MCM:NT123L1OOH", "")
  CALL Species("C_9H_{11}NO_4",                                "", "DLQITMPSPXAOFG-UHFFFAOYSA-N", "MCM:NT124L1OOH", "")
  CALL Species("C_9H_{10}N_2O_5",                    "29385-11-3", "HBYHYLBZPLCIEE-UHFFFAOYSA-N", "MCM:DNIPBNZOL", "")
  CALL Species("C_9H_{10}N_2O_5",                              "", "NLLBSWNZAUXBGJ-UHFFFAOYSA-N", "MCM:DNMETOL", "")
  CALL Species("C_9H_{10}N_2O_5",                              "", "AGMONASRVOHMRW-UHFFFAOYSA-N", "MCM:DNOETOL", "")
  CALL Species("C_9H_{10}N_2O_5",                              "", "FCIYPWNHZQHVEQ-UHFFFAOYSA-N", "MCM:DNPBNZOL", "")
  CALL Species("C_9H_{10}N_2O_5",                              "", "DZWALJMRDDSGJQ-UHFFFAOYSA-N", "MCM:DNPETOL", "")
  CALL Species("C_9H_{10}N_2O_5",                              "", "KTYJIHPJHVSEQH-UHFFFAOYSA-N", "MCM:DNT123BOL", "")
  CALL Species("C_9H_{10}N_2O_5",                              "", "HCZZKJMHIZGVJN-UHFFFAOYSA-N", "MCM:DNT124BOL", "")
  CALL Species("C_9H_{11}NO_3",                                "", "YSPMBFYZTNTHGD-UHFFFAOYSA-N", "MCM:HOEMPHNO2", "")
  CALL Species("C_9H_{11}NO_3",                                "", "RRFSVDKJKYCCEK-UHFFFAOYSA-N", "MCM:IPBNZOHNO2", "")
  CALL Species("C_9H_{11}NO_4",                                "", "MCPZUPZVJHDAPC-UHFFFAOYSA-N", "MCM:IPNCATECH", "")
  CALL Species("C_9H_{11}NO_3",                                "", "QDGFKFKXXCXYOS-UHFFFAOYSA-N", "MCM:MET1OHNO2", "")
  CALL Species("C_9H_{11}NO_4",                                "", "CJFAOEWQLMJWBQ-UHFFFAOYSA-N", "MCM:MTNCATECH", "")
  CALL Species("C_9H_{11}NO_3",                                "", "MEMHWZGPMLWUCR-UHFFFAOYSA-N", "MCM:OET1OHNO2", "")
  CALL Species("C_9H_{11}NO_4",                                "", "JWEUFFGZFSJOFB-UHFFFAOYSA-N", "MCM:OTNCATECH", "")
  CALL Species("C_9H_{11}NO_3",                                "", "SNIBPLNVBYUZKZ-UHFFFAOYSA-N", "MCM:PBNZOHNO2", "")
  CALL Species("C_9H_{11}NO_3",                                "", "DXOFURYTWRHSOS-UHFFFAOYSA-N", "MCM:PET1OHNO2", "")
  CALL Species("C_9H_{11}NO_4",                                "", "UWDKGIAMIWLDOZ-UHFFFAOYSA-N", "MCM:PNCATECH", "")
  CALL Species("C_9H_{11}NO_4",                                "", "POSTUROPYARVPT-UHFFFAOYSA-N", "MCM:PTNCATECH", "")
  CALL Species("C_9H_{11}NO_4",                                "", "NRIGBHNXEXAXSY-UHFFFAOYSA-N", "MCM:T123NCATEC", "")
  CALL Species("C_9H_{11}NO_4",                                "", "XNILZTOSMSXQOP-UHFFFAOYSA-N", "MCM:T124NCATEC", "")
  CALL Species("C_9H_{11}NO_3",                                "", "GRYTUHBAWMNFLK-UHFFFAOYSA-N", "MCM:TM123OHNO2", "")
  CALL Species("C_9H_{11}NO_3",                                "", "NKOCMNXLDSIDQC-UHFFFAOYSA-N", "MCM:TM124OHNO2", "")
  !***************************************************************************
  CALL SpeciesGroup(1,"Organic species with fluorine (\chem{F})")
  !***************************************************************************
  CALL SpeciesGroup(2,"Organic fluorine")
  ! C, H, F:
  ! saturated:
  CALL Species("CH_3F",                                "593-53-3", "NBVXSUQYWXRMNV-UHFFFAOYSA-N", "fluoromethane", "")
  CALL Species("CH_2F_2",                               "75-10-5", "RWRIWBAIICGTTQ-UHFFFAOYSA-N", "difluoromethane", "R32")
  CALL Species("CHF_3",                                 "75-46-7", "XPDWGBQVDMORPB-UHFFFAOYSA-N", "trifluoromethane", "R23")
  CALL Species("CF_4",                                  "75-73-0", "TXEYQDLBPFQVAA-UHFFFAOYSA-N", "tetrafluoromethane", "carbontetrafluoride")
  CALL Species("C_2H_5F",                              "353-36-6", "UHCBBWUQDAVSMS-UHFFFAOYSA-N", "fluoroethane", "")
  CALL Species("C_2H_4F_2",                             "75-37-6", "NPNPZTNLOVBDOC-UHFFFAOYSA-N", "1,1-difluoroethane", "R152a")
  CALL Species("C_2H_4F_2",                            "624-72-6", "AHFMSNDOYCFEPH-UHFFFAOYSA-N", "1,2-difluoroethane", "")
  CALL Species("C_2H_2F_4",                            "811-97-2", "LVGUZGTVOIAKKC-UHFFFAOYSA-N", "1,1,1,2-tetrafluoroethane", "R134a")
  CALL Species("C_2H_2F_4",                            "359-35-3", "WXGNWUVNYMJENI-UHFFFAOYSA-N", "1,1,2,2-tetrafluoroethane", "")
  CALL Species("C_2HF_5",                              "354-33-6", "GTLACDSXYULKMZ-UHFFFAOYSA-N", "pentafluoroethane", "R125")
  CALL Species("C_2F_6",                                "76-16-4", "WMIYKQLTONQJES-UHFFFAOYSA-N", "hexafluoroethane", "")
  CALL Species("C_3H_7F",                              "460-13-9", "JRHNUZCXXOTJCA-UHFFFAOYSA-N", "1-fluoropropane", "")
  CALL Species("C_3H_7F",                              "420-26-8", "PRNZBCYBKGCOFI-UHFFFAOYSA-N", "2-fluoropropane", "")
  CALL Species("C_3H_3F_5",                           "1814-88-6", "FDOPVENYMZRARC-UHFFFAOYSA-N", "1,1,1,2,2-pentafluoropropane", "")
  CALL Species("C_3H_2F_6",                            "690-39-1", "NSGXIBWMJZWTPY-UHFFFAOYSA-N", "1,1,1,3,3,3-hexafluoropropane", "")
  CALL Species("C_3HF_7",                              "431-89-0", "YFMFNYKEUDLDTL-UHFFFAOYSA-N", "1,1,1,2,3,3,3-heptafluoropropane", "R227")
  CALL Species("C_3F_8",                                "76-19-7", "QYSGYZVSCZSLHT-UHFFFAOYSA-N", "octafluoropropane", "R218")
  CALL Species("C_4F_{10}",                            "355-25-9", "KAVGMUDTWQVPDF-UHFFFAOYSA-N", "decafluorobutane", "")
  CALL Species("C_4F_8",                               "115-25-3", "BCCOBQSFUDVTJQ-UHFFFAOYSA-N", "octafluorocyclobutane", "")
  CALL Species("C_5H_2F_{10}",                      "138495-42-8", "RIQRGMUSBYGDBL-UHFFFAOYSA-N", "1,1,1,2,2,3,4,5,5,5-decafluoropentane", "")
  CALL Species("C_5F_{12}",                            "678-26-2", "NJCBUSHGCBERSK-UHFFFAOYSA-N", "dodecafluoropentane", "")
  CALL Species("C_6F_{14}",                            "355-42-0", "ZJIJAJXFLBMLCK-UHFFFAOYSA-N", "tetradecafluorohexane", "perflexane")
  CALL Species("C_6H_{11}F",                           "372-46-3", "GOBGVVAHHOUMDK-UHFFFAOYSA-N", "fluorocyclohexane", "")
  CALL Species("C_7H_{15}F",                           "661-11-0", "BITLXSQYFZTQGC-UHFFFAOYSA-N", "1-fluoroheptane", "")
  CALL Species("C_7F_{16}",                            "335-57-9", "LGUZHRODIJCVOC-UHFFFAOYSA-N", "hexadecafluoroheptane", "")
  CALL Species("C_8H_{17}F",                           "463-11-6", "DHIVLKMGKIZOHF-UHFFFAOYSA-N", "1-fluorooctane", "")
  CALL Species("C_8F_{18}",                            "307-34-6", "YVBBRRALBYAZBM-UHFFFAOYSA-N", "perfluorooctane", "")
  CALL Species("C_9F_{20}",                            "375-96-2", "UVWPNDVAQBNQBG-UHFFFAOYSA-N", "eicosafluorononane", "")
  CALL Species("C_{11}F_{24}",                         "307-49-3", "VCIVYCHKSHULON-UHFFFAOYSA-N", "perfluoroundecane", "")
  CALL Species("C_{12}H_5F_{21}",                   "154478-87-2", "HUPGRQWHZOWFPQ-UHFFFAOYSA-N", "1,1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9,\+10,10-henicosafluorododecane", "perfluorodecyl ethane")
  CALL Species("C_{14}H_{17}F_{13}",                "133331-77-8", "WRYIIOKOQSICTB-UHFFFAOYSA-N", "1,1,1,2,2,3,3,4,4,5,5,6,6-tridecafluorotetradecane", "")
  CALL Species("C_{20}H_{29}F_{13}",                "154628-00-9", "BREOHRVZEZMFOB-UHFFFAOYSA-N", "1,1,1,2,2,3,3,4,4,5,5,6,6-tridecafluoroicosane", "")
  CALL Species("C_{22}H_{33}F_{13}",                "133310-71-1", "ZKYMFADZZFTYJH-UHFFFAOYSA-N", "1,1,1,2,2,3,3,4,4,5,5,6,6-tridecafluorodocosane", "")
  CALL Species("C_{24}H_{33}F_{17}",                "117146-18-6", "FTECWCULPOFDJS-UHFFFAOYSA-N", "1,1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8-heptadecafluorotetracosane", "")
  CALL Species("C_{26}H_{33}F_{21}",                           "", "LZENXKBSJNMIKY-UHFFFAOYSA-N", "1,1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9,\+10,10-henicosafluorohexacosane", "")
  CALL Species("C_{26}H_{29}F_{25}",                 "93454-73-0", "OUASUHCMZXPMCH-UHFFFAOYSA-N", "1,1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9,\+10,10,11,11,12,12-pentacosafluorohexacosane", "")
  CALL Species("C_{28}H_{33}F_{25}",                 "93454-74-1", "CZCMNCOQMXNFTL-UHFFFAOYSA-N", "1,1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9,\+10,10,11,11,12,12-pentacosafluorooctacosane", "")
  ! unsaturated:
  CALL Species("C_2H_3F",                               "75-02-5", "XUCNUKMRBVNAPB-UHFFFAOYSA-N", "fluoroethene", "vinyl fluoride")
  CALL Species("C_2H_2F_2",                             "75-38-7", "BQCIDUSAKPWEOX-UHFFFAOYSA-N", "1,1-difluoroethene", "")
  CALL Species("C_2HF_3",                              "359-11-5", "MIZLGWKEZAPEFJ-UHFFFAOYSA-N", "trifluoroethene", "")
  CALL Species("C_2F_4",                               "116-14-3", "BFKJFAAPBSQJPD-UHFFFAOYSA-N", "tetrafluoroethene", "")
  CALL Species("C_3H_3F_3",                            "677-21-4", "FDMFUZHCIRHGRG-UHFFFAOYSA-N", "3,3,3-trifluoropropene", "")
  CALL Species("C_3F_6",                               "116-15-4", "HCDGVLDPFQMKDK-UHFFFAOYSA-N", "hexafluoropropene", "")
  CALL Species("C_4F_8",                               "382-21-8", "DAFIBNSJXIGBQB-UHFFFAOYSA-N", "1,1,3,3,3-pentafluoro-2-(trifluoromethyl)-1-propene", "perfluoroisobutylene")
  CALL Species("C_6H_3F_9",                          "19430-93-4", "GVEUEBXMTMZVSD-UHFFFAOYSA-N", "(perfluorobutyl)ethene", "4:2 FTO")
  CALL Species("C_6F_{12}",                           "3709-71-5", "SAPOZTRFWJZUFT-OWOJBTEDSA-N", "($E$)-perfluoro(4-methyl-2-pentene)", "")
  CALL Species("C_8H_3F_{13}",                       "25291-17-2", "FYQFWFHDPNXORA-UHFFFAOYSA-N", "(perfluorohexyl)ethene", "6:2 FTO")
  CALL Species("C_{10}H_3F_{17}",                    "21652-58-4", "NKAMGQZDVMQEJL-UHFFFAOYSA-N", "(perfluorooctyl)ethene", "8:2 FTO")
  CALL Species("C_{12}H_3F_{21}",                    "30389-25-4", "UCHSAVGOZUCXHC-UHFFFAOYSA-N", "(perfluorodecyl)ethene", "10:2 FTO")
  CALL Species("C_{28}H_{31}F_{25}",                           "", "LBVDHZOJOFOXJM-FOCLMDBBSA-N", "1,1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9,\+10,10,11,11,12,12-pentacosafluorooctacos-13-ene", "")
  ! aromatic:
  CALL Species("C_6H_5F",                              "462-06-6", "PYLWMHQQBFSUBP-UHFFFAOYSA-N", "fluorobenzene", "")
  CALL Species("C_6H_4F_2",                            "367-11-3", "GOYDNIKZWGIXJT-UHFFFAOYSA-N", "1,2-difluorobenzene", "$o$-difluorobenzene")
  CALL Species("C_6H_4F_2",                            "372-18-9", "UEMGWPRHOOEKTA-UHFFFAOYSA-N", "1,3-difluorobenzene", "$m$-difluorobenzene")
  CALL Species("C_6H_4F_2",                            "540-36-3", "QUGUFLJIAFISSW-UHFFFAOYSA-N", "1,4-difluorobenzene", "$p$-difluorobenzene")
  CALL Species("C_6H_2F_4",                           "2367-82-0", "UHHYOKRQTQBKSB-UHFFFAOYSA-N", "1,2,3,5-tetrafluorobenzene", "")
  CALL Species("C_6H_2F_4",                            "327-54-8", "SDXUIOOHCIQXRP-UHFFFAOYSA-N", "1,2,4,5-tetrafluorobenzene", "")
  CALL Species("C_6HF_5",                              "363-72-4", "WACNXHCZHTVBJM-UHFFFAOYSA-N", "pentafluorobenzene", "")
  CALL Species("C_6F_6",                               "392-56-3", "ZQBFAOFFOQMSGJ-UHFFFAOYSA-N", "hexafluorobenzene", "")
  CALL Species("C_6H_5CF_3",                            "98-08-8", "GETTZEONDQJALK-UHFFFAOYSA-N", "(trifluoromethyl)-benzene", "$\alpha$,$\alpha$,$\alpha$-trifluorotoluene; benzotrifluoride")
  CALL Species("C_{10}F_{10}",                         "434-90-2", "ONUFSRWQCKNVSL-UHFFFAOYSA-N", "decafluorobiphenyl", "")
  ! C, H, O, F:
  ! aliphatic:
  CALL Species("COF_2",                                "353-50-4", "IYRWEQXVUNLMAY-UHFFFAOYSA-N", "carbonyl fluoride", "")
  CALL Species("FCHO",                                "1493-02-3", "NHGVZTMBVDFPHJ-UHFFFAOYSA-N", "formyl fluoride", "")
  CALL Species("C_2H_5FO",                             "371-62-0", "GGDYAKVUZMZKRV-UHFFFAOYSA-N", "2-fluoroethanol", "")
  CALL Species("C_2H_4F_2O",                           "359-13-7", "VOGSDFLJZPNWHY-UHFFFAOYSA-N", "2,2-difluoroethanol", "")
  CALL Species("CF_3CH_2OH",                            "75-89-8", "RHQDFWAXVIIEBN-UHFFFAOYSA-N", "2,2,2-trifluoroethanol", "")
  CALL Species("CF_3CHOHCH_3",                         "374-01-6", "GILIYJDBJZWGBG-UHFFFAOYSA-N", "1,1,1-trifluoro-2-propanol", "")
  CALL Species("CHF_2CF_2CH_2OH",                       "76-37-9", "NBUKAOOFKZFCGD-UHFFFAOYSA-N", "2,2,3,3-tetrafluoro-1-propanol", "")
  CALL Species("CF_3CF_2CH_2OH",                       "422-05-9", "PSQZJKGXDGNDFP-UHFFFAOYSA-N", "2,2,3,3,3-pentafluoro-1-propanol", "")
  CALL Species("CF_3CHOHCF_3",                         "920-66-1", "BYEAHWXPCBROCE-UHFFFAOYSA-N", "1,1,1,3,3,3-hexafluoro-2-propanol", "")
  CALL Species("CF_3COF",                              "354-34-7", "DCEPGADSNJKOJK-UHFFFAOYSA-N", "trifluoroacetylfluoride", "")
  CALL Species("CH_2FCOCH_3",                          "430-51-3", "MSWVMWGCNZQPIA-UHFFFAOYSA-N", "1-fluoro-2-propanone", "fluoroacetone")
  CALL Species("CF_3COCH_3",                           "421-50-1", "FHUDAMLDXFJHJE-UHFFFAOYSA-N", "1,1,1-trifluoro-2-propanone", "1,1,1-trifluoroacetone")
  CALL Species("C_3H_6F_2O",                           "461-57-4", "CRGZRXUKXVTRNO-UHFFFAOYSA-N", "1,1-difluoro-2-methoxyethane", "")
  CALL Species("CH_2FCOOH",                            "144-49-0", "QEWYKACRFQMRMB-UHFFFAOYSA-N", "fluoroethanoic acid", "fluoroacetic acid")
  CALL Species("CHF_2COOH",                            "381-73-7", "PBWZKZYHONABLN-UHFFFAOYSA-N", "difluoroethanoic acid", "difluoroacetic acid")
  CALL Species("CF_3COOH",                              "76-05-1", "DTQVDTLACAAQTR-UHFFFAOYSA-N", "trifluoroethanoic acid", "trifluoroacetic acid")
  CALL Species("C_3HF_5O_2",                           "422-64-0", "LRMSQVBRUNSOJL-UHFFFAOYSA-N", "perfluoropropanoic acid", "")
  CALL Species("C_4HF_7O_2",                           "375-22-4", "YPJUNDFVDDCYIH-UHFFFAOYSA-N", "perfluorobutanoic acid", "")
  CALL Species("C_5HF_9O_2",                          "2706-90-3", "CXZGQIAOTKWCDB-UHFFFAOYSA-N", "perfluoropentanoic acid", "")
  CALL Species("C_6HF_{11}O_2",                        "307-24-4", "PXUULQAPEKKVAH-UHFFFAOYSA-N", "perfluorohexanoic acid", "")
  CALL Species("C_7HF_{13}O_2",                        "375-85-9", "ZWBAMYVPMDSJGQ-UHFFFAOYSA-N", "perfluoroheptanoic acid", "")
  CALL Species("C_8HF_{15}O_2",                        "335-67-1", "SNGREZUHAYWORS-UHFFFAOYSA-N", "pentadecafluorooctanoic acid", "perfluorooctanoic acid; PFOA")
  CALL Species("C_9HF_{17}O_2",                        "375-95-1", "UZUFPBIDKMEQEQ-UHFFFAOYSA-N", "perfluorononanoic acid", "")
  CALL Species("C_{10}H_3F_{17}O_2",                 "27854-31-5", "XTBXSCIWOVSSGB-UHFFFAOYSA-N", "2H,2H-perfluorodecanoic acid", "8:2 FTCA")
  CALL Species("C_{10}HF_{19}O_2",                     "335-76-2", "PCIUEQPBYFRTEM-UHFFFAOYSA-N", "perfluorodecanoic acid", "")
  CALL Species("C_{11}HF_{21}O_2",                    "2058-94-8", "SIDINRCMMRKXGQ-UHFFFAOYSA-N", "perfluoroundecanoic acid", "")
  CALL Species("C_{12}HF_{23}O_2",                     "307-55-1", "CXGONMQFMIYUJR-UHFFFAOYSA-N", "perfluorododecanoic acid", "")
  CALL Species("C_{14}HF_{27}O_2",                     "376-06-7", "RUDINRUXCKIXAJ-UHFFFAOYSA-N", "perfluorotetradecanoic acid", "")
  CALL Species("C_3F_6O",                              "684-16-2", "VBZWSGALLODQNC-UHFFFAOYSA-N", "1,1,1,3,3,3-hexafluoro-2-propanone", "")
  CALL Species("C_3H_2F_6O",                         "57041-67-5", "DPYMFVXJLLWWEU-UHFFFAOYSA-N", "desflurane", "")
  CALL Species("C_4H_3F_7O",                         "28523-86-6", "DFEYYRMXOJXZRJ-UHFFFAOYSA-N", "sevoflurane", "")
  CALL Species("C_4H_7F_3O",                           "461-24-5", "ZKNHDJMXIUOHLX-UHFFFAOYSA-N", "ethyl 2,2,2-trifluoroethyl ether", "")
  CALL Species("C_4H_4F_6O",                         "13171-18-1", "VNXYDFNVQBICRO-UHFFFAOYSA-N", "iso-indoklon", "1,1,1,3,3,3-hexafluoro-2-methoxypropane")
  CALL Species("C_4H_4F_6O",                           "333-36-8", "KGPPDNUWZNWPSI-UHFFFAOYSA-N", "di(2,2,2-trifluoroethyl) ether", "flurothyl")
  CALL Species("CF_3CH_2OCHCH_2",                      "406-90-6", "DLEGDLSLRSOURQ-UHFFFAOYSA-N", "(2,2,2-trifluoroethoxy)-ethene", "fluoroxene; fluroxene")
  CALL Species("HCOOCH_2CF_3",                       "32042-38-9", "CAFROQYMUICGNO-UHFFFAOYSA-N", "2,2,2-trifluoroethyl methanoate", "")
  CALL Species("CH_3COOCH_2CF_3",                      "406-95-1", "ZOWSJJBOQDKOHI-UHFFFAOYSA-N", "2,2,2-trifluoroethyl ethanoate", "")
  CALL Species("CF_3COOCH_3",                          "431-47-0", "VMVNZNXAVJHNDJ-UHFFFAOYSA-N", "trifluoroethanoic acid, methyl ester", "methyl trifluoroacetate")
  CALL Species("CF_3COOC_2H_5",                        "383-63-1", "STSCVKRWJPWALQ-UHFFFAOYSA-N", "trifluoroethanoic acid, ethyl ester", "ethyl trifluoroacetate")
  CALL Species("C_3F_6O",                              "428-59-1", "PGFXOWRDDHCDTE-UHFFFAOYSA-N", "trifluoro(trifluoromethyl)-oxirane", "")
  CALL Species("C_4H_5OF_5",                         "54949-74-5", "JPMHUDBOKDBBLG-UHFFFAOYSA-N", "3,3,4,4,4-pentafluorobutan-1-ol", "")
  CALL Species("C_4H_4F_6O",                          "1515-14-6", "FQDXJYBXPOMIBX-UHFFFAOYSA-N", "1,1,1,3,3,3-hexafluoro-2-methyl-2-propanol", "")
  CALL Species("C_4H_4F_6O",                           "382-31-0", "LVFXLZRISXUAIL-UHFFFAOYSA-N", "2,2,3,4,4,4-hexafluoro-1-butanol", "")
  CALL Species("C_4H_3F_7O",                           "375-01-9", "WXJFKAZDSQLPBX-UHFFFAOYSA-N", "2,2,3,3,4,4,4-heptafluoro-1-butanol", "")
  CALL Species("C_5H_5F_7O",                           "375-14-4", "RBPHBIMHZSTIDT-UHFFFAOYSA-N", "3,3,4,4,5,5,5-heptafluoro-2-pentanol", "")
  CALL Species("C_5H_4F_8O",                           "355-80-6", "JUGSKHLZINSXPQ-UHFFFAOYSA-N", "2,2,3,3,4,4,5,5-octafluoro-1-pentanol", "")
  CALL Species("C_5H_3F_9O",                        "163702-07-6", "OKIYQFLILPKULA-UHFFFAOYSA-N", "1,1,1,2,2,3,3,4,4-nonafluoro-4-methoxybutane", "")
  CALL Species("C_6H_5F_9O",                        "163702-06-5", "SQEGLLMNIBLLNQ-UHFFFAOYSA-N", "1-ethoxy-1,1,2,3,3,3-hexafluoro-2-(trifluoromethyl)propane", "")
  CALL Species("C_6H_5F_9O",                          "2043-47-2", "JCMNMOBHVPONLD-UHFFFAOYSA-N", "1H,1H,2H,2H-perfluorohexan-1-ol", "4:2 FTOH")
  CALL Species("C_6H_3F_{11}O",                        "423-46-1", "QZFZPVVDBGXQTB-UHFFFAOYSA-N", "2,2,3,3,4,4,5,5,6,6,6-undecafluoro-1-hexanol", "")
  CALL Species("C_6H_5F_9O",                        "163702-05-4", "DFUYAWQUODQGFF-UHFFFAOYSA-N", "1-ethoxy-1,1,2,2,3,3,4,4,4-nonafluorobutane", "")
  CALL Species("C_7H_4F_{12}O",                        "335-99-9", "BYKNGMLDSIEFFG-UHFFFAOYSA-N", "2,2,3,3,4,4,5,5,6,6,7,7-dodecafluoro-1-heptanol", "")
  CALL Species("C_7H_3F_{13}O",                        "375-82-6", "STLNAVFVCIRZLL-UHFFFAOYSA-N", "2,2,3,3,4,4,5,5,6,6,7,7,7-tridecafluoro-1-heptanol", "")
  CALL Species("C_8H_5F_{13}O",                        "647-42-7", "GRJRKPMIRMSBNK-UHFFFAOYSA-N", "1H,1H,2H,2H-perfluoro-1-octanol", "6:2 FTOH")
  CALL Species("C_8H_3F_{15}O",                        "307-30-2", "PJDOLCGOTSNFJM-UHFFFAOYSA-N", "2,2,3,3,4,4,5,5,6,6,7,7,8,8,8-pentadecafluoro-1-octanol", "")
  CALL Species("C_9H_5F_{15}O",                     "297730-93-9", "HHBBIOLEJRWIGU-UHFFFAOYSA-N", "3-ethoxyperfluoro(2-methylhexane)", "")
  CALL Species("C_{10}H_3F_{15}O_5",                 "69116-73-0", "JOMJXRTUQWIHQD-UHFFFAOYSA-N", "methyl perfluoro(8-(fluoroformyl)-5-methyl-4,7-dioxanonanoate)", "")
  CALL Species("C_{10}H_9F_9O_2",                     "1799-84-4", "TYNRPOFACABVSI-UHFFFAOYSA-N", "3,3,4,4,5,5,6,6,6-nonafluorohexyl methacrylate", "")
  CALL Species("C_{10}H_5F_{17}O",                     "678-39-7", "JJUBFBTUBACDHW-UHFFFAOYSA-N", "1H,1H,2H,2H-perfluorodecan-1-ol", "8:2 FTOH")
  CALL Species("C_{11}H_3F_{19}O_2",                "957209-18-6", "YRGYOFYTTFLPQM-UHFFFAOYSA-N", "2-methoxyperfluoro(2,5-di(propan-2-yl)oxolane)", "")
  CALL Species("C_{11}H_7F_{13}O_2",                 "17527-29-6", "VPKQPPJQTZJZDB-UHFFFAOYSA-N", "3,3,4,4,5,5,6,6,7,7,8,8,8-tridecafluorooctyl acrylate", "")
  CALL Species("C_{12}H_9F_{13}O_2",                  "2144-53-8", "CDXFIRXEAJABAZ-UHFFFAOYSA-N", "2-(perfluorohexyl)ethyl methacrylate", "")
  CALL Species("C_{12}H_5F_{21}O",                     "865-86-1", "FLXYIZWPNQYPIT-UHFFFAOYSA-N", "1,1,2,2-tetrahydroperfluoro dodecanol", "10:2 FTOH")
  CALL Species("C_{13}H_7F_{17}O_2",                 "27905-45-9", "QUKRIOLKOHUUBM-UHFFFAOYSA-N", "3,3,4,4,5,5,6,6,7,7,8,8,9,9,10,10,10-heptadecafluorodecyl acrylate", "")
  CALL Species("C_{14}H_9F_{17}O_2",                  "1996-88-9", "HBZFBSFGXQBQTB-UHFFFAOYSA-N", "3,3,4,4,5,5,6,6,7,7,8,8,9,9,10,10,10-heptadecafluorodecyl methacrylate", "")
  CALL Species("C_{14}H_5F_{25}O",                   "39239-77-5", "QBBJBWKVSJWYQK-UHFFFAOYSA-N", "3,3,4,4,5,5,6,6,7,7,8,8,9,9,\+10,10,11,11,12,12,13,13,14,14,14-pentacosafluorotetradecan-1-ol", "")
  CALL Species("C_{15}H_7F_{21}O_2",                 "17741-60-5", "FIAHOPQKBBASOY-UHFFFAOYSA-N", "2-(perfluorodecyl)ethyl acrylate", "")
  CALL Species("C_{16}H_5OF_{29}",                   "60699-51-6", "ZDUOTHMDVYXZBS-UHFFFAOYSA-N", "1,1,2,2-tetrahydroperfluoro-1-hexadecanol", "")
  CALL Species("C_{16}H_9F_{21}O_2",                  "2144-54-9", "FQHLOOOXLDQLPF-UHFFFAOYSA-N", "2-(perfluorodecyl)ethyl methacrylate", "")
  CALL Species("C_{17}H_7F_{25}O_2",                 "34395-24-9", "SWTZSHBOMGAQKX-UHFFFAOYSA-N", "3,3,4,4,5,5,6,6,7,7,8,8,9,9,\+10,10,11,11,12,12,13,13,14,14,14-pentacosafluorotetradecyl prop-2-enoate", "")
  CALL Species("C_{17}H_{18}F_4O_2",                "223419-20-3", "AGMMRUPNXPWLGF-AATRIKPKSA-N", "profluthrin", "")
  CALL Species("C_{18}H_9F_{25}O_2",                  "6014-75-1", "LFEGLDRNIDJMKB-UHFFFAOYSA-N", "2-perfluorododecylethyl methacrylate", "")
  CALL Species("C_{18}H_5OF_{33}",                   "65104-67-8", "UYSGWTCETIRUHO-UHFFFAOYSA-N", "1,1,2,2-tetrahydroperfluoro-1-octadecanol", "")
  CALL Species("C_{18}H_{20}F_4O_3",                "240494-70-6", "KVIZNNVXXNFLMU-AATRIKPKSA-N", "metofluthrin", "")
  CALL Species("C_{19}H_7F{29}O_2",                  "34362-49-7", "KLOHTAIHCCMZIL-UHFFFAOYSA-N", "1,1,2,2-tetrahydroperfluorohexadecyl acrylate", "")
  CALL Species("C_{20}H_{29}FO_3",                      "76-43-7", "YLRFCQOZQXIBAB-YXVJBPKESA-N", "fluoxymesterone", "")
  CALL Species("C_{22}H_{29}FO_5",                      "50-02-2", "UREBDLICKHMUKA-GCMAGEFQSA-N", "dexamethasone", "")
  CALL Species("C_{20}H_5OF_{37}",                   "65104-65-6", "FDCQNVKWWMNRQN-UHFFFAOYSA-N", "1,1,2,2-tetrahydroperfluoroeicosyl alcohol", "")
  ! aromatic:
  CALL Species("C_6H_5FO",                             "367-12-4", "HFHFGHLXUCOHLN-UHFFFAOYSA-N", "2-hydroxyfluorobenzene", "$o$-fluorophenol")
  CALL Species("C_6H_5FO",                             "371-41-5", "RHMPLDJJXGPMEX-UHFFFAOYSA-N", "4-hydroxyfluorobenzene", "$p$-fluorophenol")
  CALL Species("C_6H_5FO",                             "372-20-3", "SJTBRFHBXDZMPS-UHFFFAOYSA-N", "3-fluorophenol", "")
  CALL Species("C_6H_4F_2O",                         "28177-48-2", "CKKOVFGIBXCEIJ-UHFFFAOYSA-N", "2,6-difluorophenol", "")
  CALL Species("C_{15}H_{10}F_6O_2",                  "1478-61-1", "ZFVMWEVVKGLCIJ-UHFFFAOYSA-N", "4,4'-(hexafluoroisopropylidene)diphenol", "")
  CALL Species("C_{33}H_{25}F_3O_4",                 "90035-08-8", "KKBGNYHHEIAGOH-UHFFFAOYSA-N", "flocoumafen", "")
  ! C, H, N, F:
  CALL Species("C_4F_7N",                            "42532-60-5", "AASDJASZOZGYMM-UHFFFAOYSA-N", "2,3,3,3-tetrafluoro-2-(trifluoromethyl)propanenitrile", "")
  CALL Species("C_6H_6FN",                             "348-54-9", "FTZQXOJYPFINKJ-UHFFFAOYSA-N", "2-fluoroaniline", "")
  CALL Species("C_6H_6FN",                             "371-40-4", "KRZCOLNOCZKSDF-UHFFFAOYSA-N", "4-fluoroaniline", "")
  CALL Species("C_7H_6F_3N",                            "98-16-8", "VIUDTWATMPPKEL-UHFFFAOYSA-N", "3-(trifluoromethyl)aniline", "")
  CALL Species("C_{12}F_{27}N",                        "311-89-7", "RVZRBWKZFJCCIB-UHFFFAOYSA-N", "perfluorotributylamine", "")
  CALL Species("C_{12}H_{16}F_3N",                     "458-24-2", "DBGIVFWFUFKIQN-UHFFFAOYSA-N", "N-ethyl-1-[3-(trifluoromethyl)phenyl]-2-propanamine", "fenfluramine")
  CALL Species("C_{15}F_{33}N",                        "338-84-1", "AQZYBQIAUSKCCS-UHFFFAOYSA-N", "tris(undecafluoropentyl)amine", "")
  CALL Species("C_{16}H_{20}FN_5",                  "730979-19-8", "YFONKFDEZLYQDH-OPQQBVKSSA-N", "indaziflam A", "")
  CALL Species("C_{16}H_{20}FN_5",                  "730979-32-5", "YFONKFDEZLYQDH-OUJBWJOFSA-N", "indaziflam B", "")
  CALL Species("C_{22}H_{22}F_3N",                  "226256-56-0", "VDHAWDNDOKGFTD-MRXNPFEDSA-N", "cinacalcet", "")
  CALL Species("C_{25}H_{24}F_6N_4",                 "67485-29-4", "IQVNEKKDSLOHHK-FNCQTZNRSA-N", "hydramethylnon", "")
  ! C, H, O, N, F:
  CALL Species("C_2H_4FNO",                            "640-19-7", "FVTWJXMFYOXOKK-UHFFFAOYSA-N", "2-fluoroacetamide", "")
  CALL Species("C_4H_3FN_2O_2",                         "51-21-8", "GHASVSINZRGABV-UHFFFAOYSA-N", "5-fluorouracil", "")
  CALL Species("C_5F_{11}NO",                          "382-28-5", "PQMAKJUXOOVROI-UHFFFAOYSA-N", "perfluoro-N-methylmorpholine", "")
  CALL Species("C_6H_3FN_2O_4",                         "70-34-8", "LOTKRQAVGJMPNV-UHFFFAOYSA-N", "1-fluoro-2,4-dinitrobenzene", "")
  CALL Species("C_6H_4FNO_3",                          "446-36-6", "QQURWFRNETXFTN-UHFFFAOYSA-N", "5-fluoro-2-nitrophenol", "")
  CALL Species("C_7H_4F_3NO_3",                         "88-30-2", "ZEFMBAFMCSYJOO-UHFFFAOYSA-N", "4-nitro-3-(trifluoromethyl)phenol", "")
  CALL Species("C_7H_4F_3NO_2",                         "98-46-4", "WHNAMGUAXHGCHH-UHFFFAOYSA-N", "1-nitro-3-(trifluoromethyl)benzene", "")
  CALL Species("C_8H_4F_3NO",                          "329-01-1", "SXJYSIBLFGQAND-UHFFFAOYSA-N", "1-isocyanato-3-(trifluoromethyl)-benzene", "")
  CALL Species("C_9H_6F_3N_3O",                     "158062-67-0", "RLQJEEJISHYWON-UHFFFAOYSA-N", "flonicamid", "")
  CALL Species("C_{10}H_{11}F_3N_2O_5",                 "70-00-8", "VSQQQLOSPVPRAZ-RRKCRQDMSA-N", "trifluridine", "")
  CALL Species("C_{10}H_7F_7N_2O_2",                   "847-51-8", "STPOJASQXPXVMS-UHFFFAOYSA-N", "N-(4-amino-2-hydroxyphenyl)-2,2,3,3,4,4,4-heptafluorobutanamide", "")
  CALL Species("C_{10}H_{11}F_3N_2O",                 "2164-17-2", "RZILCCPWPBTYDO-UHFFFAOYSA-N", "fluometuron", "")
  CALL Species("C_{11}H_{13}F_3N_4O_4",              "29091-05-2", "OFDYMSKSGFSLLM-UHFFFAOYSA-N", "dinitramine", "")
  CALL Species("C_{12}H_9F_3N_2O_2",                 "75706-12-6", "VHOGYURTWQBHIL-UHFFFAOYSA-N", "5-methyl-N-[4-(trifluoromethyl)phenyl]-4-isoxazolecarboxamide", "leflunomide")
  CALL Species("C_{12}H_6F_2N_2O_2",                "131341-86-1", "MUJOIMFVNIBMKC-UHFFFAOYSA-N", "fludioxonil", "")
  CALL Species("C_{13}H_{12}F_2N_6O",                "86386-73-4", "RFHAOTPXVQNOHP-UHFFFAOYSA-N", "fluconazole", "")
  CALL Species("C_{13}H_{14}F_3N_3O_4",              "55283-68-6", "PTFJIKYUEPWBMS-UHFFFAOYSA-N", "ethalfluralin", "")
  CALL Species("C_{13}H_{16}F_3N_3O_4",               "1861-40-1", "SMDHCQAYESWHAE-UHFFFAOYSA-N", "benfluralin", "benefin")
  CALL Species("C_{13}H_{16}F_3N_3O_4",               "1582-09-8", "ZSDSQXJSNMTJDA-UHFFFAOYSA-N", "trifluralin", "")
  CALL Species("C_{13}H_{17}F_3N_4O_4",              "29091-21-2", "RSVPPPHXAASNOL-UHFFFAOYSA-N", "prodiamine", "")
  CALL Species("C_{13}H_7F_3N_2O_5",                 "15457-05-3", "HHMCAJWVGYGUEF-UHFFFAOYSA-N", "fluorodifen", "")
  CALL Species("C_{14}H_{16}F_3N_3O_4",              "26399-36-0", "ITVQAKZNYJEWKS-UHFFFAOYSA-N", "profluralin", "")
  CALL Species("C_{14}H_{12}FNO_3",                  "42835-25-6", "DPSPPJIUMHPXMA-UHFFFAOYSA-N", "flumequine", "")
  CALL Species("C_{15}H_{12}F_3NO_4",                "69335-91-7", "YUVKUEAFAVKILW-UHFFFAOYSA-N", "fluazifop", "")
  CALL Species("C_{15}H_{12}F_3NO_4",                "83066-88-0", "YUVKUEAFAVKILW-SECBINFHSA-N", "fluazifop-p", "")
  CALL Species("C_{15}H_{16}F_3N_5O_4S",             "94125-34-5", "LTUNNEGNEKBSEH-UHFFFAOYSA-N", "prosulfuron", "")
  CALL Species("C_{15}H_{15}N_2O_2F_3",              "56425-91-3", "VEVZCONIUDBCDC-UHFFFAOYSA-N", "flurprimidol", "")
  CALL Species("C_{16}H_{13}F_2N_3O",                "76674-21-0", "JWUCHKBSVLQQCO-UHFFFAOYSA-N", "flutriafol", "")
  CALL Species("C_{16}H_{12}FN_3O_3",                 "1622-62-4", "PPTYJKAXVCCBDU-UHFFFAOYSA-N", "flunitrazepam", "")
  CALL Species("C_{16}H_{12}FNO_4",                 "122008-78-0", "ROBSGBGTWRRYSK-SNVBAGLBSA-N", "cyhalofop", "")
  CALL Species("C_{16}H_{15}F_3N_2O_3",            "1449021-97-9", "ZYXYTGQFPZEUFX-UHFFFAOYSA-N", "benzpyrimoxan", "")
  CALL Species("C_{16}H_{21}F_3N_2O_3",             "911499-62-2", "RSOBJVBYZCMJOS-CYBMUJFWSA-N", "tolprocarb", "")
  CALL Species("C_{17}H_{16}F_3NO_2",                "66332-96-5", "PTCGDEVVHUXTMP-UHFFFAOYSA-N", "flutolanil", "")
  CALL Species("C_{18}H_{17}F_4NO_2",               "113614-08-7", "FFQPZWRNXKPNPX-UHFFFAOYSA-N", "beflubutamid", "")
  CALL Species("C_{18}H_{12}F_5N_3O",               "907204-31-3", "SXSGXWCSHSVPGB-UHFFFAOYSA-N", "fluxapyroxad", "")
  CALL Species("C_{18}H_{14}F_3NO_2",                "96525-23-4", "NYRMIJKDBAQCHC-UHFFFAOYSA-N", "flurtamone", "")
  CALL Species("C_{18}H_{16}F_3NO_4",               "117428-22-5", "IBSNKSODLGJUMQ-SDNWHVSQSA-N", "picoxystrobin", "")
  CALL Species("C_{18}H_{19}F_2N_3O",               "874967-67-6", "XQJQCBDIXRIYRP-UHFFFAOYSA-N", "sedaxane", "")
  CALL Species("C_{18}H_{24}FN_3O",                 "494793-67-8", "GOFJDXZZHFNFLV-UHFFFAOYSA-N", "penflufen", "")
  CALL Species("C_{19}H_{20}F_3NO_4",                "79241-46-6", "VAIZTNZGPYBOGF-CYBMUJFWSA-N", "fluazifop-p-butyl", "")
  CALL Species("C_{19}H_{12}F_4N_2O_2",             "137641-05-5", "CWKFPEBMTGKLKX-UHFFFAOYSA-N", "picolinafen", "")
  CALL Species("C_{19}H_{11}F_5N_2O_2",              "83164-33-4", "WYEHFWKAOXOVJD-UHFFFAOYSA-N", "diflufenican", "")
  CALL Species("C_{19}H_{20}F_3NO_4",                "69806-50-4", "VAIZTNZGPYBOGF-UHFFFAOYSA-N", "fluazifop-butyl", "")
  CALL Species("C_{19}H_{15}FN_2O_4",               "103361-09-7", "FOUWCSDKDDHKQP-UHFFFAOYSA-N", "flumioxazin", "")
  CALL Species("C_{19}H_{14}F_3NO",                  "59756-60-4", "YWBVHLJPRPCRSD-UHFFFAOYSA-N", "fluridone", "")
  CALL Species("C_{20}H_{20}FNO_4",                 "122008-85-9", "TYIYMOAHACZAMQ-CQSZACIVSA-N", "cyhalofop-butyl", "")
  CALL Species("C_{20}H_{21}FN_6O_5",               "518048-05-0", "CZFFBEXEKNGXKS-UHFFFAOYSA-N", "raltegravir", "")
  CALL Species("C_{20}H_{21}N_2O_5F_3",             "229977-93-9", "MXWAGQASUDSFBG-RVDMUPIBSA-N", "fluacrypyrim", "")
  CALL Species("C_{20}H_{19}F_3N_2O_4",             "141517-21-7", "ONCZDRURRATYFI-UHFFFAOYSA-N", "trifloxystrobin", "")
  CALL Species("C_{20}H_{23}F_2N_3O",               "881685-58-1", "XTDZGXBTXBEZDN-UHFFFAOYSA-N", "isopyrazam", "")
  CALL Species("C_{20}H_{23}F_2N_3O",               "683777-13-1", "XTDZGXBTXBEZDN-HEHGZKQESA-N", "syn-isopyrazam", "")
  CALL Species("C_{20}H_{23}F_2N_3O",               "683777-14-2", "XTDZGXBTXBEZDN-XEZPLFJOSA-N", "anti-isopyrazam", "")
  CALL Species("C_{21}H_{23}F_2NO_2",               "153233-91-1", "IXSZQYVWNJNRAL-UHFFFAOYSA-N", "etoxazole", "")
  CALL Species("C_{22}H_{22}FN_3O_2",                  "548-73-2", "RMEDXOLNCUSCGS-UHFFFAOYSA-N", "droperidol", "")
  CALL Species("C_{23}H_{27}FN_4O_3",               "144598-75-4", "PMXMIIMHBWHSKN-UHFFFAOYSA-N", "paliperidone", "")
  CALL Species("C_{23}H_{27}FN_4O_2",               "106266-06-2", "RAPZEAPATHNIPO-UHFFFAOYSA-N", "risperidone", "")
  CALL Species("C_{24}H_{21}F_2NO_3",               "163222-33-1", "OLNTVTPDXPETLC-XPWALMASSA-N", "ezetimibe", "")
  CALL Species("C_{24}H_{24}F_3NO_4",               "400882-07-7", "AWSZRJQNBMEZOI-UHFFFAOYSA-N", "cyflumetofen", "")
  CALL Species("C_{26}H_{21}F_6NO_5",               "101007-06-1", "YLFSVIMMRPNPFK-WEQBUNFVSA-N", "acrinathrin", "")
  CALL Species("C_{26}H_{34}FNO_5",                 "145599-86-6", "SEERZIQQUAZTOL-ANMDKAQQSA-N", "cerivastatin", "")
  CALL Species("C_{26}H_{23}F_2NO_4",                "70124-77-5", "GBIHOLCMZGAKNG-UHFFFAOYSA-N", "flucythrinate, isomer 1", "")
  ! PFBHA-derivatives:
  CALL Species("H_2C=NOCH_2C_6F_5",                  "86356-73-2", "SRTQFRQWTUMMTC-UHFFFAOYSA-N", "PFBHA-methanal", "")
  CALL Species("CH_3CH=NOCH_2C_6F_5",               "114611-59-5", "AKDRYEADQPNLOH-UHFFFAOYSA-N", "PFBHA-ethanal", "")
  CALL Species("(CH_3)_2C=NOCH_2C_6F_5",            "899828-53-6", "DLIFNTQMBOCKTL-UHFFFAOYSA-N", "PFBHA-propanone", "")
  CALL Species("(C_2H_5)(CH_3)C=NOCH_2C_6F_5",                 "", "LNDQFOSWZJYEIC-UHFFFAOYSA-N", "PFBHA-butanone", "")
  CALL Species("(C_3H_7)(CH_3)C=NOCH_2C_6F_5",                 "", "YKQOTQPNOJSJFA-UHFFFAOYSA-N", "PFBHA-2-pentanone", "")
  CALL Species("C_5H_{11}CH=NOCH_2C_6F_5",                     "", "GPAVMFOMYSGJDM-UHFFFAOYSA-N", "PFBHA-hexanal", "")
  CALL Species("C_7H_{15}CH=NOCH_2C_6F_5",                     "", "RLSQIXNUITTXKZ-UHFFFAOYSA-N", "PFBHA-octanal", "")
  CALL Species("C_9H_{19}CH=NOCH_2C_6F_5",                     "", "NUGDFCWVLOJWOP-UHFFFAOYSA-N", "PFBHA-decanal", "")
  CALL Species("CH_2CHCH=NOCH_2C_6F_5",                        "", "ICDUEGOPUWNJNF-UHFFFAOYSA-N", "PFBHA-propenal", "")
  CALL Species("C_{10}H_6F_5NO",                    "932710-55-9", "ICDUEGOPUWNJNF-HQYXKAPLSA-N", "($E$)-PFBHA-propenal", "")
  CALL Species("CH_3CHCHCH=NOCH_2C_6F_5",           "932710-52-6", "QNPFFCQTVXPCLD-UHFFFAOYSA-N", "PFBHA-crotonaldehyde", "")
  CALL Species("C_6H_5CH=NOCH_2C_6F_5",                        "", "UKSAZCDAGVHMRF-UHFFFAOYSA-N", "PFBHA-benzaldehyde", "")
  CALL Species("C_8H_8=NOCH_2C_6F_5",                          "", "UCASBURGYXMQLW-UHFFFAOYSA-N", "PFBHA-4-methyl-benzaldehyde", "")
  CALL Species("C_{13}H_8=NOCH_2C_6F_5",                       "", "UABDVZDYKDIZFO-UHFFFAOYSA-N", "PFBHA-9-fluorenone", "")
  CALL Species("(HC=NOCH_2C_6F_5)_2",               "618858-54-1", "VNVBOBJSRGZDCW-UHFFFAOYSA-N", "PFBHA-ethanedial", "")
  CALL Species("(CH_2OH)(CH_3)C=NOCH_2C_6F_5",                 "", "BCTASVPUYLXFBW-UHFFFAOYSA-N", "PFBHA-1-hydroxypropanone", "")
  CALL Species("(HOC_3H_6)(CH_3)C=NOCH_2C_6F_5",               "", "VNWPJWLGZUDTBZ-UHFFFAOYSA-N", "PFBHA-3-hydroxy-3-methyl-2-butanone", "")
  !***************************************************************************
  CALL SpeciesGroup(1,"Organic species with chlorine (\chem{Cl})")
  !***************************************************************************
  CALL SpeciesGroup(2,"Chlorocarbons (\chem{C}, \chem{H}, \chem{Cl})")
  ! C, H, Cl:
  ! saturated:
  CALL Species("CH_3Cl",                                "74-87-3", "NEHMKBQYUWJMIP-UHFFFAOYSA-N", "chloromethane", "methyl chloride")
  CALL Species("CH_2Cl_2",                              "75-09-2", "YMWUJEATGCHHMB-UHFFFAOYSA-N", "dichloromethane", "methylene chloride")
  CALL Species("CD_2Cl_2",                            "1665-00-5", "YMWUJEATGCHHMB-DICFDUPASA-N", "dichloromethane-d2", "methylene chloride-d2")
  CALL Species("CHCl_3",                                "67-66-3", "HEDRZPFGACZZDS-UHFFFAOYSA-N", "trichloromethane", "chloroform")
  CALL Species("CCl_4",                                 "56-23-5", "VZGDMQKNWNREIO-UHFFFAOYSA-N", "tetrachloromethane", "carbontetrachloride")
  CALL Species("C_2H_5Cl",                              "75-00-3", "HRYZWHHZPQKTII-UHFFFAOYSA-N", "chloroethane", "")
  CALL Species("CHCl_2CH_3",                            "75-34-3", "SCYULBFZEHDVBN-UHFFFAOYSA-N", "1,1-dichloroethane", "")
  CALL Species("CH_2ClCH_2Cl",                         "107-06-2", "WSLDOOZREJYCGB-UHFFFAOYSA-N", "1,2-dichloroethane", "")
  CALL Species("CD_2ClCD_2Cl",                       "17060-07-0", "WSLDOOZREJYCGB-LNLMKGTHSA-N", "1,2-dichloroethane-d4", "")
  CALL Species("CH_3CCl_3",                             "71-55-6", "UOCLXMDMGBRAIB-UHFFFAOYSA-N", "1,1,1-trichloroethane", "methylchloroform; MCF")
  CALL Species("CHCl_2CH_2Cl",                          "79-00-5", "UBOXGVDOUJQMTN-UHFFFAOYSA-N", "1,1,2-trichloroethane", "")
  CALL Species("CDCl_2CD_2Cl",                      "171086-93-4", "UBOXGVDOUJQMTN-FUDHJZNOSA-N", "1,1,2-trichloroethane-d3", "")
  CALL Species("CCl_3CH_2Cl",                          "630-20-6", "QVLAWKAXOMEXPM-UHFFFAOYSA-N", "1,1,1,2-tetrachloroethane", "")
  CALL Species("CHCl_2CHCl_2",                          "79-34-5", "QPFMBZIOSGYJDE-UHFFFAOYSA-N", "1,1,2,2-tetrachloroethane", "")
  CALL Species("CHCl_2CCl_3",                           "76-01-7", "BNIXVQGCZULYKV-UHFFFAOYSA-N", "pentachloroethane", "")
  CALL Species("C_2Cl_6",                               "67-72-1", "VHHHONWQHHHLTI-UHFFFAOYSA-N", "hexachloroethane", "")
  CALL Species("C_3H_7Cl",                             "540-54-5", "SNMVRZFUUCLYTO-UHFFFAOYSA-N", "1-chloropropane", "")
  CALL Species("C_3H_7Cl",                              "75-29-6", "ULYZAYCEDJDHCC-UHFFFAOYSA-N", "2-chloropropane", "")
  CALL Species("C_3H_6Cl_2",                            "78-99-9", "WIHMGGWNMISDNJ-UHFFFAOYSA-N", "1,1-dichloropropane", "")
  CALL Species("C_3H_6Cl_2",                            "78-87-5", "KNKRKFALVUDBJE-UHFFFAOYSA-N", "1,2-dichloropropane", "") ! other CAS: 26198-63-0
  CALL Species("C_3D_6Cl_2",                         "93952-08-0", "KNKRKFALVUDBJE-LIDOUZCJSA-N", "1,2-dichloropropane-d6", "")
  CALL Species("C_3H_6Cl_2",                           "142-28-9", "YHRUOJUYPBUZOS-UHFFFAOYSA-N", "1,3-dichloropropane", "")
  CALL Species("C_3H_6Cl_2",                           "594-20-7", "ZEOVXNVKXIPWMS-UHFFFAOYSA-N", "2,2-dichloropropane", "")
  CALL Species("C_3H_5Cl_3",                          "7789-89-1", "AVGQTJUPLKNPQP-UHFFFAOYSA-N", "1,1,1-trichloropropane", "")
  CALL Species("C_3H_5Cl_3",                           "598-77-6", "GRSQYISVQKPZCW-UHFFFAOYSA-N", "1,1,2-trichloropropane", "")
  CALL Species("C_3H_5Cl_3",                            "96-18-4", "CFXQEHVMCRXUSD-UHFFFAOYSA-N", "1,2,3-trichloropropane", "")
  CALL Species("C_3H_3Cl_5",                         "16714-68-4", "IYFMQUDCYNWFTL-UHFFFAOYSA-N", "1,1,2,2,3-pentachloropropane", "")
  CALL Species("C_4H_9Cl",                             "513-36-0", "QTBFPMKWQKYFLR-UHFFFAOYSA-N", "1-chloro-2-methylpropane", "")
  CALL Species("C_4H_9Cl",                             "507-20-0", "NBRKLOOSMBRFMH-UHFFFAOYSA-N", "2-chloro-2-methylpropane", "{\it tert}-butyl chloride")
  CALL Species("C_4H_9Cl",                             "109-69-3", "VFWCMGCRMGJXDK-UHFFFAOYSA-N", "1-chlorobutane", "butyl chloride")
  CALL Species("C_4H_9Cl",                              "78-86-4", "BSPCSKHALVHRSR-UHFFFAOYSA-N", "2-chlorobutane", "{\it sec}-butyl chloride")
  CALL Species("C_4H_8Cl_2",                           "541-33-3", "SEQRDAAUNCRFIT-UHFFFAOYSA-N", "1,1-dichlorobutane", "")
  CALL Species("C_4H_8Cl_2",                           "110-56-5", "KJDRSWPQXHESDQ-UHFFFAOYSA-N", "1,4-dichlorobutane", "")
  CALL Species("C_4H_8Cl_2",                          "7581-97-7", "RMISVOPUIFJTEO-UHFFFAOYSA-N", "2,3-dichlorobutane", "")
  CALL Species("C_5H_{11}Cl",                          "543-59-9", "SQCZQTSHSZLZIQ-UHFFFAOYSA-N", "1-chloropentane", "")
  CALL Species("C_5H_{11}Cl",                          "625-29-6", "NFRKUDYZEVQXTE-UHFFFAOYSA-N", "2-chloropentane", "")
  CALL Species("C_5H_{11}Cl",                          "616-20-6", "CXQSCYIVCSCSES-UHFFFAOYSA-N", "3-chloropentane", "")
  CALL Species("C_5H_{10}Cl_2",                       "1674-33-5", "PPLBPDUKNRCHGG-UHFFFAOYSA-N", "1,2-dichloropentane", "")
  CALL Species("C_5H_{10}Cl_2",                        "628-76-2", "LBKDGROORAKTLC-UHFFFAOYSA-N", "1,5-dichloropentane", "")
  CALL Species("C_5H_{10}Cl_2",                        "600-11-3", "HVFJQRZGBBKTPL-UHFFFAOYSA-N", "2,3-dichloropentane", "")
  CALL Species("C_5H_{11}Cl",                          "594-36-5", "CRNIHJHMEQZAAS-UHFFFAOYSA-N", "2-chloro-2-methylbutane", "")
  CALL Species("C_6H_{13}Cl",                          "544-10-5", "MLRVZFYXUZQSRU-UHFFFAOYSA-N", "1-chlorohexane", "")
  CALL Species("C_6H_{13}Cl",                          "638-28-8", "GLCIPJOIEVLTPR-UHFFFAOYSA-N", "2-chlorohexane", "")
  CALL Species("C_6H_{13}Cl",                         "2346-81-8", "BXSMMAVTEURRGG-UHFFFAOYSA-N", "3-chlorohexane", "")
  CALL Species("C_7H_{15}Cl",                          "629-06-1", "DZMDPHNGKBEVRE-UHFFFAOYSA-N", "1-chloroheptane", "")
  CALL Species("C_7H_{15}Cl",                         "1001-89-4", "PTSLUOSUHFGQHV-UHFFFAOYSA-N", "2-chloroheptane", "")
  CALL Species("C_7H_{15}Cl",                          "999-52-0", "DMKNOEJJJSHSML-UHFFFAOYSA-N", "3-chloroheptane", "")
  CALL Species("C_7H_{15}Cl",                          "998-95-8", "MGSGWAXIEMEWCQ-UHFFFAOYSA-N", "4-chloroheptane", "")
  CALL Species("C_8H_{17}Cl",                          "111-85-3", "CNDHHGUSRIZDSL-UHFFFAOYSA-N", "1-chlorooctane", "")
  CALL Species("C_8H_{17}Cl",                          "628-61-5", "HKDCIIMOALDWHF-UHFFFAOYSA-N", "2-chlorooctane", "")
  CALL Species("C_8H_{17}Cl",                          "123-04-6", "WLVCBAMXYMWGLJ-UHFFFAOYSA-N", "3-(chloromethyl)-heptane", "")
  CALL Species("C_8H_{16}Cl_2",                       "2162-99-4", "WXYMNDFVLNUAIA-UHFFFAOYSA-N", "1,8-dichlorooctane", "")
  CALL Species("C_9H_{19}Cl",                         "2473-01-0", "RKAMCQVGHFRILV-UHFFFAOYSA-N", "1-chlorononane", "")
  CALL Species("C_9H_{19}Cl",                         "2216-36-6", "DTWJISBCMBWFNY-UHFFFAOYSA-N", "2-chlorononane", "")
  CALL Species("C_9H_{19}Cl",                        "28123-70-8", "GHLDSOWZIOPMTC-UHFFFAOYSA-N", "5-chlorononane", "")
  CALL Species("C_{10}H_{21}Cl",                      "1002-69-3", "ZTEHOZMYMCEYRM-UHFFFAOYSA-N", "1-chlorodecane", "")
  CALL Species("C_{10}H_{20}Cl_2",                    "2162-98-3", "RBBNTRDPSVZESY-UHFFFAOYSA-N", "1,10-dichlorodecane", "")
  CALL Species("C_{10}H_{18}Cl_4",                  "205646-11-3", "VXBHNYIEBLRXAW-UHFFFAOYSA-N", "1,2,9,10-tetrachlorodecane", "")
  CALL Species("C_{10}H_{17}Cl_5",                  "175801-37-3", "AMKBEJYNNQNKGD-UHFFFAOYSA-N", "pentachlorodecane isomers", "")
  CALL Species("C_{11}H_{23}Cl",                      "2473-03-2", "ZHKKNUKCXPWZOP-UHFFFAOYSA-N", "1-chloroundecane", "")
  CALL Species("C_{11}H_{20}Cl_4",                  "210049-49-3", "VVAAXDBMZVCVPA-UHFFFAOYSA-N", "1,2,10,11-tetrachloroundecane", "")
  CALL Species("C_{11}H_{19}Cl_5",                  "210175-48-7", "BBCUCNDIXLWBNQ-UHFFFAOYSA-N", "pentachloroundecane isomers", "")
  CALL Species("C_{12}H_{25}Cl",                       "112-52-7", "YAYNEUUHHLGGAH-UHFFFAOYSA-N", "1-chlorododecane", "")
  CALL Species("C_{12}H_{24}Cl_2",                    "3922-28-9", "RNXPZVYZVHJVHM-UHFFFAOYSA-N", "1,12-dichlorododecane", "")
  CALL Species("C_{13}H_{27}Cl",                       "822-13-9", "ASZMYJSJEOGSBR-UHFFFAOYSA-N", "1-chlorotridecane", "")
  CALL Species("C_{14}H_{29}Cl",                      "2425-54-9", "RNHWYOLIEJIAMV-UHFFFAOYSA-N", "1-chlorotetradecane", "")
  ! cyclic:
  CALL Species("C_5H_6Cl_4",                         "59808-78-5", "ZFMWDTNZPKDVBU-UHFFFAOYSA-N", "tetrachlorocyclopentane", "")
  CALL Species("C_5H_4Cl_6",                         "68258-91-3", "RPUFWOAXMFQSDJ-UHFFFAOYSA-N", "1,1,2,3,3,4-hexachlorocyclopentane", "")
  CALL Species("C_5H_3Cl_7",                         "68258-90-2", "XCEUTYGYMGYCBG-UHFFFAOYSA-N", "1,1,2,3,3,4,5-heptachlorocyclopentane", "")
  CALL Species("C_5H_2Cl_6",                                   "", "OSRFTCIVMWPVNP-UHFFFAOYSA-N", "1,2,3,3,4,5-hexachlorocyclopentene", "")
  CALL Species("C_5HCl_7",                           "62111-47-1", "AJUXFTIOMRYFRL-UHFFFAOYSA-N", "heptachlorocyclopentene", "")
  CALL Species("C_6H_{11}Cl",                          "542-18-7", "UNFUYWDGSFDHCW-UHFFFAOYSA-N", "chlorocyclohexane", "")
  CALL Species("C_6H_6Cl_6",                           "319-84-6", "JLYXXMFPNIAWKQ-SHFUYGGZSA-N", "$\alpha$-1,2,3,4,5,6-hexachlorocyclohexane", "$\alpha$-lindane; $\alpha$-HCH")
  CALL Species("C_6H_6Cl_6",                           "319-85-7", "JLYXXMFPNIAWKQ-CDRYSYESSA-N", "$\beta$-1,2,3,4,5,6-hexachlorocyclohexane", "$\beta$-lindane; $\beta$-HCH")
  CALL Species("C_6H_6Cl_6",                            "58-89-9", "JLYXXMFPNIAWKQ-GNIYUCBRSA-N", "$\gamma$-1,2,3,4,5,6-hexachlorocyclohexane", "$\gamma$-lindane; lindane; $\gamma$-HCH") ! other CAS: 55963-79-6
  CALL Species("C_6H_6Cl_6",                           "319-86-8", "JLYXXMFPNIAWKQ-GPIVLXJGSA-N", "$\delta$-1,2,3,4,5,6-hexachlorocyclohexane", "$\delta$-lindane; $\delta$-HCH")
  CALL Species("C_{10}H_6Cl_6",                       "3734-48-3", "XCJXQCUJXDUNDN-UHFFFAOYSA-N", "4,5,6,7,8,8-hexachloro-3a,4,7,7a-tetrahydro-4,7-methano-1H-indene", "")
  CALL Species("C_{10}Cl_{10}",                       "2227-17-0", "LWLJUMBEZJHXHV-UHFFFAOYSA-N", "dienochlor", "")
  CALL Species("C_{10}Cl_{12}",                       "2385-85-5", "GVYLCNUFSHDAAW-UHFFFAOYSA-N", "mirex", "dodecachloropentacyclodecane")
  CALL Species("C_{18}H_{12}Cl_{12}",                "13560-89-9", "UGQQAJOWXNCOPY-UHFFFAOYSA-N", "dechlorane plus", "")
  ! toxaphene is a mixture, not a pure substance:
  ! CALL Species("toxaphene", "C_{10}H_{10}Cl_8", "", "8001-35-2")
  ! unsaturated:
  CALL Species("CH_2CHCl",                              "75-01-4", "BZHJMEDXRYGGRV-UHFFFAOYSA-N", "chloroethene", "vinyl chloride")
  CALL Species("CD_2CDCl",                            "6745-35-3", "BZHJMEDXRYGGRV-FUDHJZNOSA-N", "chloroethene-d3", "vinyl chloride-d3")
  CALL Species("CH_2CCl_2",                             "75-35-4", "LGXVIGDEPROXKC-UHFFFAOYSA-N", "1,1-dichloroethene", "")
  CALL Species("C_2H_2Cl_2",                           "540-59-0", "KFUSEUYYWQURPO-UHFFFAOYSA-N", "1,2-dichloroethene", "")
  CALL Species("CHClCHCl",                             "156-59-2", "KFUSEUYYWQURPO-UPHRSURJSA-N", "($Z$)-1,2-dichloroethene", "{\it cis}-1,2-dichloroethene")
  CALL Species("CHClCHCl",                             "156-60-5", "KFUSEUYYWQURPO-OWOJBTEDSA-N", "($E$)-1,2-dichloroethene", "{\it trans}-1,2-dichloroethene")
  CALL Species("C_2HCl_3",                              "79-01-6", "XSTXAVWGXDQKEL-UHFFFAOYSA-N", "trichloroethene", "trichloroethylene")
  CALL Species("C_2Cl_4",                              "127-18-4", "CYTYCFOTNPOANT-UHFFFAOYSA-N", "tetrachloroethene", "tetrachloroethylene")
  CALL Species("C_2Cl_2",                             "7572-29-4", "ZMJOVJSTYLQINE-UHFFFAOYSA-N", "dichloroethyne", "")
  CALL Species("C_3H_5Cl",                             "590-21-6", "OWXJKYNZGFSVRC-UHFFFAOYSA-N", "1-chloro-1-propene", "")
  CALL Species("C_3H_5Cl",                             "557-98-2", "PNLQPWWBHXMFCA-UHFFFAOYSA-N", "2-chloro-1-propene", "")
  CALL Species("C_3H_5Cl",                             "107-05-1", "OSDWBNJEKMUWAV-UHFFFAOYSA-N", "3-chloro-1-propene", "allyl chloride")
  CALL Species("C_3H_4Cl_2",                           "563-58-6", "ZAIDIVBQUMFXEC-UHFFFAOYSA-N", "1,1-dichloropropene", "")
  CALL Species("C_3H_4Cl_2",                           "563-54-2", "PPKPKFIWDXDAGC-UHFFFAOYSA-N", "1,2-dichloropropene", "")
  CALL Species("C_3H_4Cl_2",                           "542-75-6", "UOORRWUZONOOLO-UHFFFAOYSA-N", "1,3-dichloropropene", "")
  CALL Species("C_3H_4Cl_2",                         "10061-01-5", "UOORRWUZONOOLO-UPHRSURJSA-N", "{\it cis}-1,3-dichloropropene", "")
  CALL Species("C_3H_4Cl_2",                         "10061-02-6", "UOORRWUZONOOLO-OWOJBTEDSA-N", "{\it trans}-1,3-dichloropropene", "")
  CALL Species("C_3H_4Cl_2",                            "78-88-6", "FALCMQXTWHPRIH-UHFFFAOYSA-N", "2,3-dichloropropene", "")
  CALL Species("C_3H_3Cl_3",                            "96-19-5", "HIILBTHBHCLUER-UHFFFAOYSA-N", "1,2,3-trichloro-1-propene", "")
  CALL Species("C_3Cl_6",                             "1888-71-7", "VFDYKPARTDCDCU-UHFFFAOYSA-N", "1,1,2,3,3,3-hexachloro-1-propene", "")
  CALL Species("C_4H_7Cl",                             "563-52-0", "VZGLVCFVUREVDP-UHFFFAOYSA-N", "3-chloro-1-butene", "")
  CALL Species("C_4H_7Cl",                             "563-47-3", "OHXAOPZTJOUYKM-UHFFFAOYSA-N", "3-chloro-2-methyl-1-propene", "")
  CALL Species("C_4H_7Cl",                             "591-97-9", "YTKRILODNOEEPX-UHFFFAOYSA-N", "1-chloro-2-butene", "crotyl chloride")
  CALL Species("C_4H_7Cl",                            "4628-21-1", "YTKRILODNOEEPX-IHWYPQMZSA-N", "($Z$)-1-chloro-2-butene", "{\it cis}-1-chloro-2-butene")
  CALL Species("C_4H_7Cl",                            "4894-61-5", "YTKRILODNOEEPX-NSCUHMNNSA-N", "($E$)-1-chloro-2-butene", "{\it trans}-1-chloro-2-butene")
  CALL Species("C_4H_6Cl_2",                           "926-57-8", "WLIADPFXSACYLS-UHFFFAOYSA-N", "1,3-dichloro-2-butene", "")
  CALL Species("C_4H_6Cl_2",                           "764-41-0", "FQDIANVAWVHZIR-UHFFFAOYSA-N", "1,4-dichloro-2-butene", "")
  CALL Species("C_4H_6Cl_2",                          "1476-11-5", "FQDIANVAWVHZIR-UPHRSURJSA-N", "($Z$)-1,4-dichloro-2-butene", "")
  CALL Species("C_4H_6Cl_2",                           "110-57-6", "FQDIANVAWVHZIR-OWOJBTEDSA-N", "($E$)-1,4-dichloro-2-butene", "")
  CALL Species("C_4H_6Cl_2",                           "760-23-6", "XVEASTGLHPVZNA-UHFFFAOYSA-N", "3,4-dichloro-1-butene", "")
  CALL Species("C_4H_7Cl",                             "513-37-1", "KWISWUFGPUHDRY-UHFFFAOYSA-N", "1-chloro-2-methylpropene", "dimethylvinyl chloride")
  CALL Species("C_4H_5Cl",                             "126-99-8", "YACLQRRMGMJLJV-UHFFFAOYSA-N", "2-chloro-1,3-butadiene", "")
  CALL Species("CCl_2CClCClCCl_2",                      "87-68-3", "RWNKSTSCBHKHTB-UHFFFAOYSA-N", "hexachlorobutadiene", "")
  CALL Species("C_5Cl_6",                               "77-47-4", "VUNCWTMEJYMOOR-UHFFFAOYSA-N", "hexachlorocyclopentadiene", "")
  CALL Species("C_{10}H_6Cl_8",                         "57-74-9", "BIWJNBZANLAXMG-UHFFFAOYSA-N", "chlordane", "") ! other CAS: 12789-03-6
  CALL Species("C_{10}H_6Cl_8",                       "5103-71-9", "BIWJNBZANLAXMG-KMMBHOGFSA-N", "{\it cis}-chlordane", "$\alpha$-chlordane")
  CALL Species("C_{10}H_6Cl_8",                       "5103-74-2", "BIWJNBZANLAXMG-OESJLNMISA-N", "{\it trans}-chlordane", "$\beta$-chlordane")
  CALL Species("C_{10}H_6Cl_8",                       "5566-34-7", "JBZJEPYXXVKOKF-UHFFFAOYSA-N", "$\gamma$-chlordane", "")
  CALL Species("C_{10}H_5Cl_9",                       "5103-73-1", "OCHOKXCPKDPNQU-BBXWSCHTSA-N", "{\it cis}-nonachlor", "")
  CALL Species("C_{10}H_5Cl_9",                      "39765-80-5", "OCHOKXCPKDPNQU-FLVMBEMLSA-N", "{\it trans}-nonachlor", "")
  ! aromatic:
  CALL Species("C_6H_5Cl",                             "108-90-7", "MVPPADPHJFYWMZ-UHFFFAOYSA-N", "chlorobenzene", "")
  CALL Species("C_6D_5Cl",                            "3114-55-4", "MVPPADPHJFYWMZ-RALIUCGRSA-N", "chlorobenzene-d5", "")
  CALL Species("C_6H_4Cl_2",                            "95-50-1", "RFFLAFLAYFXFSW-UHFFFAOYSA-N", "1,2-dichlorobenzene", "$o$-dichlorobenzene")
  CALL Species("C_6D_4Cl_2",                          "2199-69-1", "RFFLAFLAYFXFSW-RHQRLBAQSA-N", "1,2-dichlorobenzene-d4", "$o$-dichlorobenzene-d4")
  CALL Species("C_6H_4Cl_2",                           "541-73-1", "ZPQOPVIELGIULI-UHFFFAOYSA-N", "1,3-dichlorobenzene", "$m$-dichlorobenzene")
  CALL Species("C_6H_4Cl_2",                           "106-46-7", "OCJBOOLMMGQPQU-UHFFFAOYSA-N", "1,4-dichlorobenzene", "$p$-dichlorobenzene")
  CALL Species("C_6H_3Cl_3",                            "87-61-6", "RELMFMZEBKVZJC-UHFFFAOYSA-N", "1,2,3-trichlorobenzene", "")
  CALL Species("C_6D_3Cl_3",                          "3907-98-0", "SMNOERSLNYGGOU-UHFFFAOYSA-N", "1,2,3-trichlorobenzene-d3", "")
  CALL Species("C_6H_3Cl_3",                           "120-82-1", "PBKONEOXTCPAFI-UHFFFAOYSA-N", "1,2,4-trichlorobenzene", "")
  CALL Species("C_6D_3Cl_3",                          "2199-72-6", "PBKONEOXTCPAFI-CBYSEHNBSA-N", "1,2,4-trichlorobenzene-d3", "")
  CALL Species("C_6H_3Cl_3",                           "108-70-3", "XKEFYDZQGKAQCN-UHFFFAOYSA-N", "1,3,5-trichlorobenzene", "")
  CALL Species("C_6H_2Cl_4",                           "634-66-2", "GBDZXPJXOMHESU-UHFFFAOYSA-N", "1,2,3,4-tetrachlorobenzene", "")
  CALL Species("C_6H_2Cl_4",                           "634-90-2", "QZYNWJQFTJXIRN-UHFFFAOYSA-N", "1,2,3,5-tetrachlorobenzene", "")
  CALL Species("C_6H_2Cl_4",                            "95-94-3", "JHBKHLUZVFWLAG-UHFFFAOYSA-N", "1,2,4,5-tetrachlorobenzene", "")
  CALL Species("C_6HCl_5",                             "608-93-5", "CEOCDNVZRAIOQZ-UHFFFAOYSA-N", "pentachlorobenzene", "")
  CALL Species("C_6Cl_6",                              "118-74-1", "CKAPSXZOOQJIBF-UHFFFAOYSA-N", "hexachlorobenzene", "")
  CALL Species("C_6H_5CH_2Cl",                         "100-44-7", "KCXMKQUNVWSEMD-UHFFFAOYSA-N", "(chloromethyl)-benzene", "benzylchloride")
  CALL Species("C_7H_7Cl",                              "95-49-8", "IBSQPLPBRSHTTG-UHFFFAOYSA-N", "1-chloro-2-methylbenzene", "$o$-chlorotoluene")
  CALL Species("C_7H_7Cl",                             "108-41-8", "OSOUNOBYRMOXQQ-UHFFFAOYSA-N", "1-chloro-3-methylbenzene", "$m$-chlorotoluene")
  CALL Species("C_7H_7Cl",                             "106-43-4", "NPDACUSDTOMAMK-UHFFFAOYSA-N", "1-chloro-4-methylbenzene", "$p$-chlorotoluene")
  CALL Species("C_7H_6Cl_2",                            "98-87-3", "CAHQGWAXKLQREW-UHFFFAOYSA-N", "(dichloromethyl)-benzene", "")
  CALL Species("C_7H_6Cl_2",                            "95-75-0", "WYUIWKFIFOJVKW-UHFFFAOYSA-N", "1,2-dichloro-4-methylbenzene", "3,4-dichlorotoluene")
  CALL Species("C_7H_6Cl_2",                           "118-69-4", "DMEDNTFWIHCBRK-UHFFFAOYSA-N", "1,3-dichloro-2-methylbenzene", "2,6-dichlorotoluene")
  CALL Species("C_7H_6Cl_2",                            "95-73-8", "FUNUTBJJKQIVSY-UHFFFAOYSA-N", "2,4-dichloro-1-methylbenzene", "2,4-dichlorotoluene")
  CALL Species("C_7H_5Cl_3",                          "2077-46-5", "UZYYBZNZSSNYSA-UHFFFAOYSA-N", "2,3,6-trichloromethylbenzene", "2,3,6-trichlorotoluene")
  CALL Species("C_7H_5Cl_3",                          "6639-30-1", "ZCXHZKNWIYVQNC-UHFFFAOYSA-N", "2,4,5-trichloromethylbenzene", "2,4,5-trichlorotoluene")
  CALL Species("C_7H_5Cl_3",                         "23749-65-7", "RCTKUIOMKBEGTG-UHFFFAOYSA-N", "2,4,6-trichloromethylbenzene", "2,4,6-trichlorotoluene")
  CALL Species("C_7H_3Cl_5",                           "877-11-2", "AVSIMRGRHWKCAY-UHFFFAOYSA-N", "pentachloromethylbenzene", "2,3,4,5,6-pentachlorotoluene")
  CALL Species("C_7H_6Cl_2",                           "611-19-8", "BASMANVIUSSIIM-UHFFFAOYSA-N", "1-chloro-2-(chloromethyl)benzene", "")
  CALL Species("C_7H_6Cl_2",                           "104-83-6", "JQZAEUFPPSRDOP-UHFFFAOYSA-N", "1-chloro-4-(chloromethyl)benzene", "")
  CALL Species("C_7H_5Cl_3",                            "88-66-4", "BXSVYGKOUULJCL-UHFFFAOYSA-N", "chloro(dichloromethyl)benzene", "")
  CALL Species("C_7H_5Cl_3",                            "98-07-7", "XEMRAKSQROQPBR-UHFFFAOYSA-N", "(trichloromethyl)-benzene", "")
  CALL Species("C_7H_4Cl_4",                          "5216-25-1", "LVZPKYYPPLUECL-UHFFFAOYSA-N", "1-chloro-4-(trichloromethyl)benzene", "")
  CALL Species("C_8H_7Cl",                            "2039-85-2", "BOVQCIDBZXNFEJ-UHFFFAOYSA-N", "1-chloro-3-ethenylbenzene", "")
  CALL Species("C_8H_7Cl",                            "1073-67-2", "KTZVZZJJVJQZHV-UHFFFAOYSA-N", "1-chloro-4-ethenylbenzene", "")
  CALL Species("C_8H_8Cl_2",                          "1124-05-6", "UTGSRNVBAFCOEU-UHFFFAOYSA-N", "1,4-dichloro-2,5-dimethylbenzene", "")
  CALL Species("C_8H_4Cl_6",                            "68-36-0", "OTEKOJQFKOIXMU-UHFFFAOYSA-N", "1,4-bis(trichloromethyl)benzene", "")
  CALL Species("C_8H_8Cl_2",                           "612-12-4", "FMGGHNGKHRCJLL-UHFFFAOYSA-N", "$\alpha$,$\alpha$-dichloro-$o$-xylene", "")
  CALL Species("C_8H_7Cl",                            "2039-87-4", "ISRGONDNXBCDBM-UHFFFAOYSA-N", "2-chlorostyrene", "")
  CALL Species("C_8Cl_8",                            "29082-74-4", "RUYUCCQRWINUHE-UHFFFAOYSA-N", "octachlorostyrene", "")
  CALL Species("C_{10}H_5Cl_7",                         "76-44-8", "FRCCEHPWNOQAEU-UHFFFAOYSA-N", "heptachlor", "")
  CALL Species("C_{10}H_{13}Cl",                       "515-40-2", "DNXXUUPUQXSUFH-UHFFFAOYSA-N", "1-chloro-2-methyl-2-phenylpropane", "")
  CALL Species("C_{10}H_8Cl_6",                      "73588-42-8", "DELZPCKTBPIBEE-VIFPVBQESA-N", "1,3-dichloro-5-[(2$S$)-2,4,4,4-tetrachlorobutan-2-yl]benzene", "")
  CALL Species("C_{10}H_5Cl_9",                       "3734-49-4", "OCHOKXCPKDPNQU-UHFFFAOYSA-N", "1,2,3,4,5,6,7,8,8-nonachloro-2,3,3a,4,7,7a-hexahydro-4,7-methano-1H-indene", "nonachlor")
  CALL Species("C_{14}H_{10}Cl_4",                      "72-54-8", "AHJKRLASYNVKDZ-UHFFFAOYSA-N", "1,1-dichloro-2,2-bis-(4-chlorophenyl)-ethane", "p,p'-DDD")
  CALL Species("C_{14}H_{10}Cl_4",                      "53-19-0", "JWBOIMRXGHLCPP-UHFFFAOYSA-N", "mitotane", "o,p'-DDD")
  CALL Species("C_{14}H_8Cl_4",                         "72-55-9", "UCNVFOCBFJOQAL-UHFFFAOYSA-N", "1,1-dichloro-2,2-bis-(4-chlorophenyl)-ethene", "p,p'-DDE")
  CALL Species("C_{14}H_8Cl_4",                       "3424-82-6", "ZDYJWDIWLRZXDB-UHFFFAOYSA-N", "o,p'-DDE", "")
  CALL Species("C_{14}H_9Cl_5",                        "789-02-6", "CVUGPAFCQJIYDT-UHFFFAOYSA-N", "1,1,1-trichloro-2-(2-chlorophenyl)-2-(4-chlorophenyl)ethane", "o,p'-DDT")
  CALL Species("C_{14}H_9Cl_5",                         "50-29-3", "YVGGHNCTFXOJCH-UHFFFAOYSA-N", "1,1,1-trichloro-2,2-bis-(4-chlorophenyl)-ethane", "DDT; p,p'-DDT")
  CALL Species("C_{12}H_8Cl_6",                        "309-00-2", "QBYJBZPUGVGKQQ-SJJAEHHWSA-N", "aldrin", "")
  CALL Species("C_{12}H_8Cl_6",                        "465-73-6", "QBYJBZPUGVGKQQ-DIFDVCDBSA-N", "isodrin", "")
  CALL Species("C_{18}H_{20}Cl_2",                      "72-56-0", "QFMDFTQOJHFVNR-UHFFFAOYSA-N", "1,1'-(2,2-dichloroethylidene)bis[4-ethylbenzene", "perthane")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Polychlorinated naphthalenes (PCNs)")
  ! MonoCNs: 25586-43-0
  CALL Species("C_{10}H_7Cl",                           "90-13-1", "JTPNRXUCIXHOKM-UHFFFAOYSA-N", "1-chloronaphthalene",                  "PCN-1")
  CALL Species("C_{10}H_7Cl",                           "91-58-7", "CGYGETOMCSJHJU-UHFFFAOYSA-N", "2-chloronaphthalene",                  "PCN-2")
  ! DiCNs: 28699-88-9
  CALL Species("C_{10}H_6Cl_2",                       "2050-69-3", "MOXLHAPKZWTHEX-UHFFFAOYSA-N", "1,2-dichloronaphthalene",              "PCN-3")
  CALL Species("C_{10}H_6Cl_2",                       "2198-75-6", "AMCBMCWLCDERHY-UHFFFAOYSA-N", "1,3-dichloronaphthalene",              "PCN-4")
  CALL Species("C_{10}H_6Cl_2",                       "1825-31-6", "JDPKCYMVSKDOGS-UHFFFAOYSA-N", "1,4-dichloronaphthalene",              "PCN-5")
  CALL Species("C_{10}H_6Cl_2",                       "1825-30-5", "ZBQZXTBAGBTUAD-UHFFFAOYSA-N", "1,5-dichloronaphthalene",              "PCN-6")
  !CALL Species("C_{10}H_6Cl_2",                      "2050-72-8", "CEDZMDSZTVUPSP-UHFFFAOYSA-N", "1,6-dichloronaphthalene",              "PCN-7")
  !CALL Species("C_{10}H_6Cl_2",                      "2050-73-9", "VQBIYCQGHAHVPN-UHFFFAOYSA-N", "1,7-dichloronaphthalene",              "PCN-8")
  !CALL Species("C_{10}H_6Cl_2",                      "2050-74-0", "ADRYPAGQXFMVFP-UHFFFAOYSA-N", "1,8-dichloronaphthalene",              "PCN-9")
  !CALL Species("C_{10}H_6Cl_2",                      "2050-75-1", "SKGXUFZRYNGFJS-UHFFFAOYSA-N", "2,3-dichloronaphthalene",              "PCN-10")
  !CALL Species("C_{10}H_6Cl_2",                      "2065-70-5", "YCFUHBHONRJFHI-UHFFFAOYSA-N", "2,6-dichloronaphthalene",              "PCN-11")
  CALL Species("C_{10}H_6Cl_2",                       "2198-77-8", "DWBQZSYTSNYEEJ-UHFFFAOYSA-N", "2,7-dichloronaphthalene",              "PCN-12")
  ! TriCNs: 1321-64-8
  !CALL Species("C_{10}H_5Cl_3",                     "50402-52-3", "QEPTXDCPBXMWJC-UHFFFAOYSA-N", "1,2,3-trichloronaphthalene",           "PCN-13")
  CALL Species("C_{10}H_5Cl_3",                      "50402-51-2", "MRJBOPVWPORBKZ-UHFFFAOYSA-N", "1,2,4-trichloronaphthalene",           "PCN-14")
  CALL Species("C_{10}H_5Cl_3",                      "55720-33-7", "MMHZKSMAFILONG-UHFFFAOYSA-N", "1,2,5-trichloronaphthalene",           "PCN-15")
  CALL Species("C_{10}H_5Cl_3",                      "51570-44-6", "AAUJSCRITBXDJH-UHFFFAOYSA-N", "1,2,6-trichloronaphthalene",           "PCN-16")
  CALL Species("C_{10}H_5Cl_3",                      "55720-34-8", "QYYUVUXSJZJCLQ-UHFFFAOYSA-N", "1,2,7-trichloronaphthalene",           "PCN-17")
  !CALL Species("C_{10}H_5Cl_3",                     "55720-35-9", "YSFLKFZEQBZEPU-UHFFFAOYSA-N", "1,2,8-trichloronaphthalene",           "PCN-18")
  CALL Species("C_{10}H_5Cl_3",                      "51570-43-5", "DZHZYPCCEISGSQ-UHFFFAOYSA-N", "1,3,5-trichloronaphthalene",           "PCN-19")
  !CALL Species("C_{10}H_5Cl_3",                     "55720-36-0", "CJRUXWSXMDPLDQ-UHFFFAOYSA-N", "1,3,6-trichloronaphthalene",           "PCN-20")
  !CALL Species("C_{10}H_5Cl_3",                     "55720-37-1", "CFEUGIGSIREATC-UHFFFAOYSA-N", "1,3,7-trichloronaphthalene",           "PCN-21")
  !CALL Species("C_{10}H_5Cl_3",                     "55720-38-2", "AXNCYYJNXAOOJQ-UHFFFAOYSA-N", "1,3,8-trichloronaphthalene",           "PCN-22")
  CALL Species("C_{10}H_5Cl_3",                       "2437-55-0", "VQSNXLCFFHGXTI-UHFFFAOYSA-N", "1,4,5-trichloronaphthalene",           "PCN-23")
  CALL Species("C_{10}H_5Cl_3",                       "2437-54-9", "RLTTZFDRZKJVKJ-UHFFFAOYSA-N", "1,4,6-trichloronaphthalene",           "PCN-24")
  CALL Species("C_{10}H_5Cl_3",                      "55720-39-3", "FUEZTEBYLIMNHN-UHFFFAOYSA-N", "1,6,7-trichloronaphthalene",           "PCN-25")
  !CALL Species("C_{10}H_5Cl_3",                     "55720-40-6", "ZYTLBFKRYKUCMJ-UHFFFAOYSA-N", "2,3,6-trichloronaphthalene",           "PCN-26")
  ! TetraCNs: 1335-88-2
  CALL Species("C_{10}H_4Cl_4",                      "20020-02-4", "NAQWICRLNQSPPW-UHFFFAOYSA-N", "1,2,3,4-tetrachloronaphthalene",       "PCN-27")
  CALL Species("C_{10}H_4Cl_4",                      "53555-63-8", "HJJKSUVYCQAMBG-UHFFFAOYSA-N", "1,2,3,5-tetrachloronaphthalene",       "PCN-28")
  !CALL Species("C_{10}H_4Cl_4",                    "149864-78-8", "ZVORWIZONZUEEI-UHFFFAOYSA-N", "1,2,3,6-tetrachloronaphthalene",       "PCN-29")
  !CALL Species("C_{10}H_4Cl_4",                     "55720-41-7", "AULMCNDOHMTPGC-UHFFFAOYSA-N", "1,2,3,7-tetrachloronaphthalene",       "PCN-30")
  !CALL Species("C_{10}H_4Cl_4",                    "149864-81-3", "UVMHXYSILPJXPK-UHFFFAOYSA-N", "1,2,3,8-tetrachloronaphthalene",       "PCN-31")
  CALL Species("C_{10}H_4Cl_4",                       "6733-54-6", "BIVDISPRSYAHQQ-UHFFFAOYSA-N", "1,2,4,5-tetrachloronaphthalene",       "PCN-32")
  CALL Species("C_{10}H_4Cl_4",                      "51570-45-7", "GLVVZPZGCNEVEM-UHFFFAOYSA-N", "1,2,4,6-tetrachloronaphthalene",       "PCN-33")
  CALL Species("C_{10}H_4Cl_4",                      "67922-21-8", "PWXOBMRJWBBEED-UHFFFAOYSA-N", "1,2,4,7-tetrachloronaphthalene",       "PCN-34")
  CALL Species("C_{10}H_4Cl_4",                       "6529-87-9", "WCMSFBRREKZZFL-UHFFFAOYSA-N", "1,2,4,8-tetrachloronaphthalene",       "PCN-35")
  CALL Species("C_{10}H_4Cl_4",                      "67922-22-9", "ZPUUGNBIWHBXBM-UHFFFAOYSA-N", "1,2,5,6-tetrachloronaphthalene",       "PCN-36")
  CALL Species("C_{10}H_4Cl_4",                      "67922-23-0", "KQZUIOXHRIVQGR-UHFFFAOYSA-N", "1,2,5,7-tetrachloronaphthalene",       "PCN-37")
  CALL Species("C_{10}H_4Cl_4",                     "149864-80-2", "DLTBLLHAQLBHDR-UHFFFAOYSA-N", "1,2,5,8-tetrachloronaphthalene",       "PCN-38")
  !CALL Species("C_{10}H_4Cl_4",                    "149864-79-9", "AXLIBZIJTKPCHR-UHFFFAOYSA-N", "1,2,6,7-tetrachloronaphthalene",       "PCN-39")
  CALL Species("C_{10}H_4Cl_4",                      "67922-24-1", "OVAYDYKVLHHIDQ-UHFFFAOYSA-N", "1,2,6,8-tetrachloronaphthalene",       "PCN-40")
  !CALL Species("C_{10}H_4Cl_4",                    "149864-82-4", "DARLUNVQYCQWRW-UHFFFAOYSA-N", "1,2,7,8-tetrachloronaphthalene",       "PCN-41")
  CALL Species("C_{10}H_4Cl_4",                      "53555-64-9", "OTTCXKPQKOLSJN-UHFFFAOYSA-N", "1,3,5,7-tetrachloronaphthalene",       "PCN-42")
  CALL Species("C_{10}H_4Cl_4",                      "31604-28-1", "VFTLNRFRXWCJJK-UHFFFAOYSA-N", "1,3,5,8-tetrachloronaphthalene",       "PCN-43")
  !CALL Species("C_{10}H_4Cl_4",                     "55720-42-8", "BULBRUKKSUUPNT-UHFFFAOYSA-N", "1,3,6,7-tetrachloronaphthalene",       "PCN-44")
  CALL Species("C_{10}H_4Cl_4",                     "150224-15-0", "XXWQPOHDPJIYIN-UHFFFAOYSA-N", "1,3,6,8-tetrachloronaphthalene",       "PCN-45")
  CALL Species("C_{10}H_4Cl_4",                       "3432-57-3", "LITCKAVLJAKHOE-UHFFFAOYSA-N", "1,4,5,8-tetrachloronaphthalene",       "PCN-46")
  CALL Species("C_{10}H_4Cl_4",                      "55720-43-9", "VJZRCIYSYVGDMU-UHFFFAOYSA-N", "1,4,6,7-tetrachloronaphthalene",       "PCN-47")
  !CALL Species("C_{10}H_4Cl_4",                     "34588-40-4", "XTTLUUBHRXWFSZ-UHFFFAOYSA-N", "2,3,6,7-tetrachloronaphthalene",       "PCN-48")
  ! PentaCNs: 1321-64-8
  !CALL Species("C_{10}H_3Cl_5",                     "67922-25-2", "JRZKNHITLINYHV-UHFFFAOYSA-N", "1,2,3,4,5-pentachloronaphthalene",     "PCN-49")
  CALL Species("C_{10}H_3Cl_5",                      "67922-26-3", "BAOLNVSMVTYGDA-UHFFFAOYSA-N", "1,2,3,4,6-pentachloronaphthalene",     "PCN-50")
  !CALL Species("C_{10}H_3Cl_5",                    "150224-18-3", "CHADMNIGNFLQOI-UHFFFAOYSA-N", "1,2,3,5,6-pentachloronaphthalene",     "PCN-51")
  CALL Species("C_{10}H_3Cl_5",                      "53555-65-0", "OVSKLQPHXHPXDR-UHFFFAOYSA-N", "1,2,3,5,7-pentachloronaphthalene",     "PCN-52")
  CALL Species("C_{10}H_3Cl_5",                     "150224-24-1", "HVYRFNJXZVEGFK-UHFFFAOYSA-N", "1,2,3,5,8-pentachloronaphthalene",     "PCN-53")
  !CALL Species("C_{10}H_3Cl_5",                    "150224-16-1", "MKCASBPTHPJCDC-UHFFFAOYSA-N", "1,2,3,6,7-pentachloronaphthalene",     "PCN-54")
  !CALL Species("C_{10}H_3Cl_5",                    "150224-23-0", "NMAMEWIQKMFSSI-UHFFFAOYSA-N", "1,2,3,6,8-pentachloronaphthalene",     "PCN-55")
  !CALL Species("C_{10}H_3Cl_5",                    "150205-21-3", "BFRRJXVSQTZQHM-UHFFFAOYSA-N", "1,2,3,7,8-pentachloronaphthalene",     "PCN-56")
  CALL Species("C_{10}H_3Cl_5",                     "150224-20-7", "KPOZENRUJCHWOA-UHFFFAOYSA-N", "1,2,4,5,6-pentachloronaphthalene",     "PCN-57")
  CALL Species("C_{10}H_3Cl_5",                     "150224-19-4", "WYLDWCYZCFRVRH-UHFFFAOYSA-N", "1,2,4,5,7-pentachloronaphthalene",     "PCN-58")
  CALL Species("C_{10}H_3Cl_5",                     "150224-25-2", "FEIKEVSWLMYFFF-UHFFFAOYSA-N", "1,2,4,5,8-pentachloronaphthalene",     "PCN-59")
  CALL Species("C_{10}H_3Cl_5",                     "150224-17-2", "GXQUDLBNLKOIQB-UHFFFAOYSA-N", "1,2,4,6,7-pentachloronaphthalene",     "PCN-60")
  CALL Species("C_{10}H_3Cl_5",                     "150224-22-9", "HGSDQSUMXKHGTH-UHFFFAOYSA-N", "1,2,4,6,8-pentachloronaphthalene",     "PCN-61")
  CALL Species("C_{10}H_3Cl_5",                     "150224-21-8", "LBCOXKFWBDTJTF-UHFFFAOYSA-N", "1,2,4,7,8-pentachloronaphthalene",     "PCN-62")
  ! HexaCNs: 1335-87-1
  CALL Species("C_{10}H_2Cl_6",                      "58877-88-6", "CTLMCQOGOWNFHA-UHFFFAOYSA-N", "1,2,3,4,5,6-hexachloronaphthalene",    "PCN-63")
  CALL Species("C_{10}H_2Cl_6",                      "67922-27-4", "SWRNUKWDDYPZGV-UHFFFAOYSA-N", "1,2,3,4,5,7-hexachloronaphthalene",    "PCN-64")
  CALL Species("C_{10}H_2Cl_6",                     "103426-93-3", "PGCDNPCENWGYMA-UHFFFAOYSA-N", "1,2,3,4,5,8-hexachloronaphthalene",    "PCN-65")
  CALL Species("C_{10}H_2Cl_6",                     "103426-96-6", "ZRNSVEOEIWQEMU-UHFFFAOYSA-N", "1,2,3,4,6,7-hexachloronaphthalene",    "PCN-66")
  CALL Species("C_{10}H_2Cl_6",                     "103426-97-7", "XZLJCGGEQLNWDT-UHFFFAOYSA-N", "1,2,3,5,6,7-hexachloronaphthalene",    "PCN-67")
  CALL Species("C_{10}H_2Cl_6",                     "103426-95-5", "FQELOCOACCYGLL-UHFFFAOYSA-N", "1,2,3,5,6,8-hexachloronaphthalene",    "PCN-68")
  CALL Species("C_{10}H_2Cl_6",                     "103426-94-4", "JPQLLIUTUFJWMH-UHFFFAOYSA-N", "1,2,3,5,7,8-hexachloronaphthalene",    "PCN-69")
  !CALL Species("C_{10}H_2Cl_6",                     "17062-87-2", "WJYZNPLWZGYFIE-UHFFFAOYSA-N", "1,2,3,6,7,8-hexachloronaphthalene",    "PCN-70")
  CALL Species("C_{10}H_2Cl_6",                      "90948-28-0", "JHKLUUFTHIWTKX-UHFFFAOYSA-N", "1,2,4,5,6,8-hexachloronaphthalene",    "PCN-71")
  CALL Species("C_{10}H_2Cl_6",                     "103426-92-2", "SFZREMCYQNYZMZ-UHFFFAOYSA-N", "1,2,4,5,7,8-hexachloronaphthalene",    "PCN-72")
  ! HeptaCNs: 32241-08-0
  CALL Species("C_{10}HCl_7",                        "58863-14-2", "NDZIBNJHNBUHKW-UHFFFAOYSA-N", "1,2,3,4,5,6,7-heptachloronaphthalene", "PCN-73")
  CALL Species("C_{10}HCl_7",                        "58863-15-3", "QYEGXUUXWMKHHS-UHFFFAOYSA-N", "1,2,3,4,5,6,8-heptachloronaphthalene", "PCN-74")
  ! OctaCN:
  CALL Species("C_{10}Cl_8",                          "2234-13-1", "RTNLUFLDZOAXIC-UHFFFAOYSA-N", "octachloronaphthalene",                "PCN-75")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Polychlorinated biphenyls (PCBs)")
  ! Aroclor products C_12H_xCl_(10-x) are identified by a four-digit
  ! numbering code in which the first two digits (12) indicate that the
  ! parent molecule is biphenyl and the last two digits indicate the
  ! percentage of chlorine content by weight. The exception to this
  ! formula is Aroclor 1016, which has an average chlorine percentage of
  ! 41.5 percent.
  ! (www.environmentwriter.org/resources/backissues/chemicals/pcb.htm)
  ! aroclor1016 12674-11-2
  ! aroclor1221 11104-28-2
  ! aroclor1232 11141-16-5
  ! aroclor1242 53469-21-9
  ! aroclor1248 12672-29-6
  ! aroclor1254 11097-69-1
  ! aroclor1260 11096-82-5
  ! aroclor1268 11100-14-4
  ! PCBs with IUPAC (BZ) PCB-number:
  CALL Species("C_{12}H_9Cl",                         "2051-60-7", "LAXBNTIAOJWAOP-UHFFFAOYSA-N", "2-chlorobiphenyl", "PCB-1")
  CALL Species("C_{12}H_9Cl",                         "2051-61-8", "NMWSKOLWZZWHPL-UHFFFAOYSA-N", "3-chlorobiphenyl", "PCB-2")
  CALL Species("C_{12}H_9Cl",                         "2051-62-9", "FPWNLURCHDRMHC-UHFFFAOYSA-N", "4-chlorobiphenyl", "PCB-3")
  CALL Species("C_{12}H_8Cl_2",                      "13029-08-8", "JAYCNKDKIKZTAF-UHFFFAOYSA-N", "2,2'-dichlorobiphenyl", "PCB-4")
  CALL Species("C_{12}H_8Cl_2",                      "16605-91-7", "XOMKZKJEJBZBJJ-UHFFFAOYSA-N", "2,3-dichlorobiphenyl", "PCB-5")
  CALL Species("C_{12}H_8Cl_2",                      "25569-80-6", "ZHBBDTRJIVXKEX-UHFFFAOYSA-N", "2,3'-dichlorobiphenyl", "PCB-6")
  CALL Species("C_{12}H_8Cl_2",                      "33284-50-3", "WEJZHZJJXPXXMU-UHFFFAOYSA-N", "2,4-dichlorobiphenyl", "PCB-7")
  CALL Species("C_{12}H_8Cl_2",                      "34883-43-7", "UFNIBRDIUNVOMX-UHFFFAOYSA-N", "2,4'-dichlorobiphenyl", "PCB-8")
  CALL Species("C_{12}H_8Cl_2",                      "34883-39-1", "KKQWHYGECTYFIA-UHFFFAOYSA-N", "2,5-dichlorobiphenyl", "PCB-9")
  CALL Species("C_{12}H_8Cl_2",                      "33146-45-1", "IYZWUWBAFUBNCH-UHFFFAOYSA-N", "2,6-dichlorobiphenyl", "PCB-10")
  CALL Species("C_{12}H_8Cl_2",                       "2050-67-1", "KTXUOWUHFLBZPW-UHFFFAOYSA-N", "3,3'-dichlorobiphenyl", "PCB-11")
  CALL Species("C_{12}H_8Cl_2",                       "2974-92-7", "ZGHQUYZPMWMLBM-UHFFFAOYSA-N", "3,4-dichlorobiphenyl", "PCB-12")
  CALL Species("C_{12}H_8Cl_2",                       "2974-90-5", "CJDNEKOMKXLSBN-UHFFFAOYSA-N", "3,4'-dichlorobiphenyl", "PCB-13")
  CALL Species("C_{12}H_8Cl_2",                      "34883-41-5", "QHZSDTDMQZPUKC-UHFFFAOYSA-N", "3,5-dichlorobiphenyl", "PCB-14")
  CALL Species("C_{12}H_8Cl_2",                       "2050-68-2", "YTBRNEUEFCNVHC-UHFFFAOYSA-N", "4,4'-dichlorobiphenyl", "PCB-15")
  CALL Species("C_{12}H_7Cl_3",                      "38444-78-9", "XVIZMMSINIOIQP-UHFFFAOYSA-N", "2,2',3-trichlorobiphenyl", "PCB-16")
  CALL Species("C_{12}H_7Cl_3",                      "37680-66-3", "YKKYCYQDUUXNLN-UHFFFAOYSA-N", "2,2',4-trichlorobiphenyl", "PCB-17")
  CALL Species("C_{12}H_7Cl_3",                      "37680-65-2", "DCMURXAZTZQAFB-UHFFFAOYSA-N", "2,2',5-trichlorobiphenyl", "PCB-18")
  CALL Species("C_{12}H_7Cl_3",                      "38444-73-4", "MVXIJRBBCDLNLX-UHFFFAOYSA-N", "2,2',6-trichlorobiphenyl", "PCB-19")
  CALL Species("C_{12}H_7Cl_3",                      "38444-84-7", "SXHLTVKPNQVZGL-UHFFFAOYSA-N", "2,3,3'-trichlorobiphenyl", "PCB-20")
  CALL Species("C_{12}H_7Cl_3",                      "55702-46-0", "IUYHQGMDSZOPDZ-UHFFFAOYSA-N", "2,3,4-trichlorobiphenyl", "PCB-21")
  CALL Species("C_{12}H_7Cl_3",                      "38444-85-8", "ZMHWQAHZKUPENF-UHFFFAOYSA-N", "2,3,4'-trichlorobiphenyl", "PCB-22")
  CALL Species("C_{12}H_7Cl_3",                      "55720-44-0", "GBUCDGDROYMOAN-UHFFFAOYSA-N", "2,3,5-trichlorobiphenyl", "PCB-23")
  CALL Species("C_{12}H_7Cl_3",                      "55702-45-9", "LVROLHVSYNLFBE-UHFFFAOYSA-N", "2,3,6-trichlorobiphenyl", "PCB-24")
  CALL Species("C_{12}H_7Cl_3",                      "55712-37-3", "XBBZAULFUPBZSP-UHFFFAOYSA-N", "2,3',4-trichlorobiphenyl", "PCB-25")
  CALL Species("C_{12}H_7Cl_3",                      "38444-81-4", "ONNCPBRWFSKDMQ-UHFFFAOYSA-N", "2,3',5-trichlorobiphenyl", "PCB-26")
  CALL Species("C_{12}H_7Cl_3",                      "38444-76-7", "VQOFJPFYTCHPTR-UHFFFAOYSA-N", "2,3',6-trichlorobiphenyl", "PCB-27")
  CALL Species("C_{12}H_7Cl_3",                       "7012-37-5", "BZTYNSQSZHARAZ-UHFFFAOYSA-N", "2,4,4'-trichlorobiphenyl", "PCB-28")
  CALL Species("C_{12}H_7Cl_3",                      "15862-07-4", "VGVIKVCCUATMNG-UHFFFAOYSA-N", "2,4,5-trichlorobiphenyl", "PCB-29")
  CALL Species("C_{12}H_7Cl_3",                      "35693-92-6", "MTLMVEWEYZFYTH-UHFFFAOYSA-N", "2,4,6-trichlorobiphenyl", "PCB-30")
  CALL Species("C_{12}H_7Cl_3",                      "16606-02-3", "VAHKBZSAUKPEOV-UHFFFAOYSA-N", "2,4',5-trichlorobiphenyl", "PCB-31")
  CALL Species("C_{12}H_7Cl_3",                      "38444-77-8", "IHIDFKLAWYPTKB-UHFFFAOYSA-N", "2,4',6-trichlorobiphenyl", "PCB-32")
  CALL Species("C_{12}H_7Cl_3",                      "38444-86-9", "RIMXLXBUOQMDHV-UHFFFAOYSA-N", "2,3',4'-trichlorobiphenyl", "PCB-33")
  CALL Species("C_{12}H_7Cl_3",                      "37680-68-5", "GXVMAQACUOSFJF-UHFFFAOYSA-N", "2,3',5'-trichlorobiphenyl", "PCB-34")
  CALL Species("C_{12}H_7Cl_3",                      "37680-69-6", "JHBVPKZLIBDTJR-UHFFFAOYSA-N", "3,3',4-trichlorobiphenyl", "PCB-35")
  CALL Species("C_{12}H_7Cl_3",                      "38444-87-0", "RIBGNAJQTOXRDK-UHFFFAOYSA-N", "3,3',5-trichlorobiphenyl", "PCB-36")
  CALL Species("C_{12}H_7Cl_3",                      "38444-90-5", "YZANRISAORXTHU-UHFFFAOYSA-N", "3,4,4'-trichlorobiphenyl", "PCB-37")
  CALL Species("C_{12}H_7Cl_3",                      "53555-66-1", "BSFZSQRJGZHMMV-UHFFFAOYSA-N", "3,4,5-trichlorobiphenyl", "PCB-38")
  CALL Species("C_{12}H_7Cl_3",                      "38444-88-1", "SYSBNFJJSJLZMM-UHFFFAOYSA-N", "3,4',5-trichlorobiphenyl", "PCB-39")
  CALL Species("C_{12}H_6Cl_4",                      "38444-93-8", "VTLYHLREPCPDKX-UHFFFAOYSA-N", "2,2',3,3'-tetrachlorobiphenyl", "PCB-40")
  CALL Species("C_{12}H_6Cl_4",                      "52663-59-9", "SEWHDNLIHDBVDZ-UHFFFAOYSA-N", "2,2',3,4-tetrachlorobiphenyl", "PCB-41")
  CALL Species("C_{12}H_6Cl_4",                      "36559-22-5", "ALFHIHDQSYXSGP-UHFFFAOYSA-N", "2,2',3,4'-tetrachlorobiphenyl", "PCB-42")
  CALL Species("C_{12}H_6Cl_4",                      "70362-46-8", "NRBNBYFPJCCKTO-UHFFFAOYSA-N", "2,2',3,5-tetrachlorobiphenyl", "PCB-43")
  CALL Species("C_{12}H_6Cl_4",                      "41464-39-5", "ALDJIKXAHSDLLB-UHFFFAOYSA-N", "2,2',3,5'-tetrachlorobiphenyl", "PCB-44")
  CALL Species("C_{12}H_6Cl_4",                      "70362-45-7", "VHGHHZZTMJLTJX-UHFFFAOYSA-N", "2,2',3,6-tetrachlorobiphenyl", "PCB-45")
  CALL Species("C_{12}H_6Cl_4",                      "41464-47-5", "CUGLICQCTXWQNF-UHFFFAOYSA-N", "2,2',3,6'-tetrachlorobiphenyl", "PCB-46")
  CALL Species("C_{12}H_6Cl_4",                       "2437-79-8", "QORAVNMWUNPXAO-UHFFFAOYSA-N", "2,2',4,4'-tetrachlorobiphenyl", "PCB-47")
  CALL Species("C_{12}H_6Cl_4",                      "70362-47-9", "XBTHILIDLBPRPM-UHFFFAOYSA-N", "2,2',4,5-tetrachlorobiphenyl", "PCB-48")
  CALL Species("C_{12}H_6Cl_4",                      "41464-40-8", "ZWPVHELAQPIZHO-UHFFFAOYSA-N", "2,2',4,5'-tetrachlorobiphenyl", "PCB-49")
  CALL Species("C_{12}H_6Cl_4",                      "62796-65-0", "VLLVVZDKBSYMCG-UHFFFAOYSA-N", "2,2',4,6-tetrachlorobiphenyl", "PCB-50")
  CALL Species("C_{12}H_6Cl_4",                      "68194-04-7", "WVHNUGRFECMVLQ-UHFFFAOYSA-N", "2,2',4,6'-tetrachlorobiphenyl", "PCB-51")
  CALL Species("C_{12}H_6Cl_4",                      "35693-99-3", "HCWZEPKLWVAEOV-UHFFFAOYSA-N", "2,2',5,5'-tetrachlorobiphenyl", "PCB-52")
  CALL Species("C_{12}H_6Cl_4",                      "41464-41-9", "SFTUSTXGTCCSHX-UHFFFAOYSA-N", "2,2',5,6'-tetrachlorobiphenyl", "PCB-53")
  CALL Species("C_{12}H_6Cl_4",                      "15968-05-5", "PXAGFNRKXSYIHU-UHFFFAOYSA-N", "2,2',6,6'-tetrachlorobiphenyl", "PCB-54")
  CALL Species("C_{12}H_6Cl_4",                      "74338-24-2", "ZKGSEEWIVLAUNH-UHFFFAOYSA-N", "2,3,3',4-tetrachlorobiphenyl", "PCB-55")
  CALL Species("C_{12}H_6Cl_4",                      "41464-43-1", "UNCGJRRROFURDV-UHFFFAOYSA-N", "2,3,3',4'-tetrachlorobiphenyl", "PCB-56")
  CALL Species("C_{12}H_6Cl_4",                      "70424-67-8", "DHDBTLFALXRTLB-UHFFFAOYSA-N", "2,3,3',5-tetrachlorobiphenyl", "PCB-57")
  CALL Species("C_{12}H_6Cl_4",                      "41464-49-7", "IOPBNBSKOPJKEG-UHFFFAOYSA-N", "2,3,3',5'-tetrachlorobiphenyl", "PCB-58")
  CALL Species("C_{12}H_6Cl_4",                      "74472-33-6", "WZNAMGYIQPAXDH-UHFFFAOYSA-N", "2,3,3',6-tetrachlorobiphenyl", "PCB-59")
  CALL Species("C_{12}H_6Cl_4",                      "33025-41-1", "XLDBTRJKXLKYTC-UHFFFAOYSA-N", "2,3,4,4'-tetrachlorobiphenyl", "PCB-60")
  CALL Species("C_{12}H_6Cl_4",                      "33284-53-6", "HLQDGCWIOSOMDP-UHFFFAOYSA-N", "2,3,4,5-tetrachlorobiphenyl", "PCB-61")
  CALL Species("C_{12}H_6Cl_4",                      "54230-22-7", "HOBRTVXSIVSXIA-UHFFFAOYSA-N", "2,3,4,6-tetrachlorobiphenyl", "PCB-62")
  CALL Species("C_{12}H_6Cl_4",                      "74472-34-7", "CITMYAMXIZQCJD-UHFFFAOYSA-N", "2,3,4',5-tetrachlorobiphenyl", "PCB-63")
  CALL Species("C_{12}H_6Cl_4",                      "52663-58-8", "FXRXQYZZALWWGA-UHFFFAOYSA-N", "2,3,4',6-tetrachlorobiphenyl", "PCB-64")
  CALL Species("C_{12}H_6Cl_4",                      "33284-54-7", "BLAYIQLVUNIICD-UHFFFAOYSA-N", "2,3,5,6-tetrachlorobiphenyl", "PCB-65")
  CALL Species("C_{12}H_6Cl_4",                      "32598-10-0", "RKLLTEAEZIJBAU-UHFFFAOYSA-N", "2,3',4,4'-tetrachlorobiphenyl", "PCB-66")
  CALL Species("C_{12}H_6Cl_4",                      "73575-53-8", "LQEGJNOKOZHBBZ-UHFFFAOYSA-N", "2,3',4,5-tetrachlorobiphenyl", "PCB-67")
  CALL Species("C_{12}H_6Cl_4",                      "73575-52-7", "KTTXLLZIBIDUCR-UHFFFAOYSA-N", "2,3',4,5'-tetrachlorobiphenyl", "PCB-68")
  CALL Species("C_{12}H_6Cl_4",                      "60233-24-1", "CKUBKYSLNCKBOI-UHFFFAOYSA-N", "2,3',4,6-tetrachlorobiphenyl", "PCB-69")
  CALL Species("C_{12}H_6Cl_4",                      "32598-11-1", "KENZYIHFBRWMOD-UHFFFAOYSA-N", "2,3',4',5-tetrachlorobiphenyl", "PCB-70")
  CALL Species("C_{12}H_6Cl_4",                      "41464-46-4", "WYVBETQIUHPLFO-UHFFFAOYSA-N", "2,3',4',6-tetrachlorobiphenyl", "PCB-71")
  CALL Species("C_{12}H_6Cl_4",                      "41464-42-0", "WBTMFEPLVQOWFI-UHFFFAOYSA-N", "2,3',5,5'-tetrachlorobiphenyl", "PCB-72")
  CALL Species("C_{12}H_6Cl_4",                      "74338-23-1", "HDULUCZRGGWTMZ-UHFFFAOYSA-N", "2,3',5',6-tetrachlorobiphenyl", "PCB-73")
  CALL Species("C_{12}H_6Cl_4",                      "32690-93-0", "TULCXSBAPHCWCF-UHFFFAOYSA-N", "2,4,4',5-tetrachlorobiphenyl", "PCB-74")
  CALL Species("C_{12}H_6Cl_4",                      "32598-12-2", "RZFZBHKDGHISSH-UHFFFAOYSA-N", "2,4,4',6-tetrachlorobiphenyl", "PCB-75")
  CALL Species("C_{12}H_6Cl_4",                      "70362-48-0", "QILUYCYPNYWMIL-UHFFFAOYSA-N", "2,3',4',5'-tetrachlorobiphenyl", "PCB-76")
  CALL Species("C_{12}H_6Cl_4",                      "32598-13-3", "UQMGJOKDKOLIDP-UHFFFAOYSA-N", "3,3',4,4'-tetrachlorobiphenyl", "PCB-77")
  CALL Species("C_{12}H_6Cl_4",                      "70362-49-1", "SXFLURRQRFKBNN-UHFFFAOYSA-N", "3,3',4,5-tetrachlorobiphenyl", "PCB-78")
  CALL Species("C_{12}H_6Cl_4",                      "41464-48-6", "QLCTXEMDCZGPCG-UHFFFAOYSA-N", "3,3',4,5'-tetrachlorobiphenyl", "PCB-79")
  CALL Species("C_{12}H_6Cl_4",                      "33284-52-5", "UTMWFJSRHLYRPY-UHFFFAOYSA-N", "3,3',5,5'-tetrachlorobiphenyl", "PCB-80")
  CALL Species("C_{12}H_6Cl_4",                      "70362-50-4", "BHWVLZJTVIYLIV-UHFFFAOYSA-N", "3,4,4',5-tetrachlorobiphenyl", "PCB-81")
  CALL Species("C_{12}H_5Cl_5",                      "52663-62-4", "AUGNBQPSMWGAJE-UHFFFAOYSA-N", "2,2',3,3',4-pentachlorobiphenyl", "PCB-82")
  CALL Species("C_{12}H_5Cl_5",                      "60145-20-2", "SUBRHHYLRGOTHL-UHFFFAOYSA-N", "2,2',3,3',5-pentachlorobiphenyl", "PCB-83")
  CALL Species("C_{12}H_5Cl_5",                      "52663-60-2", "QVWUJLANSDKRAH-UHFFFAOYSA-N", "2,2',3,3',6-pentachlorobiphenyl", "PCB-84")
  CALL Species("C_{12}H_5Cl_5",                      "65510-45-4", "LACXVZHAJMVESG-UHFFFAOYSA-N", "2,2',3,4,4'-pentachlorobiphenyl", "PCB-85")
  CALL Species("C_{12}H_5Cl_5",                      "55312-69-1", "AIURIRUDHVDRFQ-UHFFFAOYSA-N", "2,2',3,4,5-pentachlorobiphenyl", "PCB-86")
  CALL Species("C_{12}H_5Cl_5",                      "38380-02-8", "OPKYDBFRKPQCBS-UHFFFAOYSA-N", "2,2',3,4,5'-pentachlorobiphenyl", "PCB-87")
  CALL Species("C_{12}H_5Cl_5",                      "55215-17-3", "QGDKRLQRLFUJPP-UHFFFAOYSA-N", "2,2',3,4,6-pentachlorobiphenyl", "PCB-88")
  CALL Species("C_{12}H_5Cl_5",                      "73575-57-2", "GLOOIONSKMZYQZ-UHFFFAOYSA-N", "2,2',3,4,6'-pentachlorobiphenyl", "PCB-89")
  CALL Species("C_{12}H_5Cl_5",                      "68194-07-0", "SUOAMBOBSWRMNQ-UHFFFAOYSA-N", "2,2',3,4',5-pentachlorobiphenyl", "PCB-90")
  CALL Species("C_{12}H_5Cl_5",                      "68194-05-8", "CXKIGWXPPVZSQK-UHFFFAOYSA-N", "2,2',3,4',6-pentachlorobiphenyl", "PCB-91")
  CALL Species("C_{12}H_5Cl_5",                      "52663-61-3", "CRCBRZBVCDKPGA-UHFFFAOYSA-N", "2,2',3,5,5'-pentachlorobiphenyl", "PCB-92")
  CALL Species("C_{12}H_5Cl_5",                      "73575-56-1", "BMXRLHMJGHJGLR-UHFFFAOYSA-N", "2,2',3,5,6-pentachlorobiphenyl", "PCB-93")
  CALL Species("C_{12}H_5Cl_5",                      "73575-55-0", "FJUVPPYNSDTRQV-UHFFFAOYSA-N", "2,2',3,5,6'-pentachlorobiphenyl", "PCB-94")
  CALL Species("C_{12}H_5Cl_5",                      "38379-99-6", "GXNNLIMMEXHBKV-UHFFFAOYSA-N", "2,2',3,5',6-pentachlorobiphenyl", "PCB-95")
  CALL Species("C_{12}H_5Cl_5",                      "73575-54-9", "QQFGAXUIQVKBKU-UHFFFAOYSA-N", "2,2',3,6,6'-pentachlorobiphenyl", "PCB-96")
  CALL Species("C_{12}H_5Cl_5",                      "41464-51-1", "JTUSORDQZVOEAZ-UHFFFAOYSA-N", "2,2',3,4',5'-pentachlorobiphenyl", "PCB-97")
  CALL Species("C_{12}H_5Cl_5",                      "60233-25-2", "GOFFZTAPOOICFT-UHFFFAOYSA-N", "2,2',3,4',6'-pentachlorobiphenyl", "PCB-98")
  CALL Species("C_{12}H_5Cl_5",                      "38380-01-7", "LMQJBFRGXHMNOX-UHFFFAOYSA-N", "2,2',4,4',5-pentachlorobiphenyl", "PCB-99")
  CALL Species("C_{12}H_5Cl_5",                      "39485-83-1", "RKUAZJIXKHPFRK-UHFFFAOYSA-N", "2,2',4,4',6-pentachlorobiphenyl", "PCB-100")
  CALL Species("C_{12}H_5Cl_5",                      "37680-73-2", "LAHWLEDBADHJGA-UHFFFAOYSA-N", "2,2',4,5,5'-pentachlorobiphenyl", "PCB-101")
  CALL Species("C_{12}H_5Cl_5",                      "68194-06-9", "BWWVXHRLMPBDCK-UHFFFAOYSA-N", "2,2',4,5,6'-pentachlorobiphenyl", "PCB-102")
  CALL Species("C_{12}H_5Cl_5",                      "60145-21-3", "PQHZWWBJPCNNGI-UHFFFAOYSA-N", "2,2',4,5',6-pentachlorobiphenyl", "PCB-103")
  CALL Species("C_{12}H_5Cl_5",                      "56558-16-8", "MTCPZNVSDFCBBE-UHFFFAOYSA-N", "2,2',4,6,6'-pentachlorobiphenyl", "PCB-104")
  CALL Species("C_{12}H_5Cl_5",                      "32598-14-4", "WIDHRBRBACOVOY-UHFFFAOYSA-N", "2,3,3',4,4'-pentachlorobiphenyl", "PCB-105")
  CALL Species("C_{12}H_5Cl_5",                      "70424-69-0", "BQENMISTWGTJIJ-UHFFFAOYSA-N", "2,3,3',4,5-pentachlorobiphenyl", "PCB-106")
  CALL Species("C_{12}H_5Cl_5",                      "70424-68-9", "PVYBHVJTMRRXLG-UHFFFAOYSA-N", "2,3,3',4',5-pentachlorobiphenyl", "PCB-107")
  CALL Species("C_{12}H_5Cl_5",                      "70362-41-3", "MPCDNZSLJWJDNW-UHFFFAOYSA-N", "2,3,3',4,5'-pentachlorobiphenyl", "PCB-108")
  CALL Species("C_{12}H_5Cl_5",                      "74472-35-8", "XGQBSVVYMVILEL-UHFFFAOYSA-N", "2,3,3',4,6-pentachlorobiphenyl", "PCB-109")
  CALL Species("C_{12}H_5Cl_5",                      "38380-03-9", "ARXHIJMGSIYYRZ-UHFFFAOYSA-N", "2,3,3',4',6-pentachlorobiphenyl", "PCB-110")
  CALL Species("C_{12}H_5Cl_5",                      "39635-32-0", "QMUDLTGWHILKHH-UHFFFAOYSA-N", "2,3,3',5,5'-pentachlorobiphenyl", "PCB-111")
  CALL Species("C_{12}H_5Cl_5",                      "74472-36-9", "NTKSJAPQYKCFPP-UHFFFAOYSA-N", "2,3,3',5,6-pentachlorobiphenyl", "PCB-112")
  CALL Species("C_{12}H_5Cl_5",                      "68194-10-5", "YDGFMDPEJCJZEV-UHFFFAOYSA-N", "2,3,3',5',6-pentachlorobiphenyl", "PCB-113")
  CALL Species("C_{12}H_5Cl_5",                      "74472-37-0", "SXZSFWHOSHAKMN-UHFFFAOYSA-N", "2,3,4,4',5-pentachlorobiphenyl", "PCB-114")
  CALL Species("C_{12}H_5Cl_5",                      "74472-38-1", "IOVARPVVZDOPGQ-UHFFFAOYSA-N", "2,3,4,4',6-pentachlorobiphenyl", "PCB-115")
  CALL Species("C_{12}H_5Cl_5",                      "18259-05-7", "GGMPTLAAIUQMIE-UHFFFAOYSA-N", "2,3,4,5,6-pentachlorobiphenyl", "PCB-116")
  CALL Species("C_{12}H_5Cl_5",                      "68194-11-6", "ZDDZPDTVCZLFFC-UHFFFAOYSA-N", "2,3,4',5,6-pentachlorobiphenyl", "PCB-117")
  CALL Species("C_{12}H_5Cl_5",                      "31508-00-6", "IUTPYMGCWINGEY-UHFFFAOYSA-N", "2,3',4,4',5-pentachlorobiphenyl", "PCB-118")
  CALL Species("C_{12}H_5Cl_5",                      "56558-17-9", "OAEQTHQGPZKTQP-UHFFFAOYSA-N", "2,3',4,4',6-pentachlorobiphenyl", "PCB-119")
  CALL Species("C_{12}H_5Cl_5",                      "68194-12-7", "ZLGYJAIAVPVCNF-UHFFFAOYSA-N", "2,3',4,5,5'-pentachlorobiphenyl", "PCB-120")
  CALL Species("C_{12}H_5Cl_5",                      "56558-18-0", "XBVSGJGMWSKAKL-UHFFFAOYSA-N", "2,3',4,5',6-pentachlorobiphenyl", "PCB-121")
  CALL Species("C_{12}H_5Cl_5",                      "76842-07-4", "GWOWBISZHLPYEK-UHFFFAOYSA-N", "2,3,3',4',5'-pentachlorobiphenyl", "PCB-122")
  CALL Species("C_{12}H_5Cl_5",                      "65510-44-3", "YAHNWSSFXMVPOU-UHFFFAOYSA-N", "2,3',4,4',5'-pentachlorobiphenyl", "PCB-123")
  CALL Species("C_{12}H_5Cl_5",                      "70424-70-3", "PIVBPZFQXKMHBD-UHFFFAOYSA-N", "2,3',4',5,5'-pentachlorobiphenyl", "PCB-124")
  CALL Species("C_{12}H_5Cl_5",                      "74472-39-2", "WAZUWHGJMMZVHH-UHFFFAOYSA-N", "2,3',4',5',6-pentachlorobiphenyl", "PCB-125")
  CALL Species("C_{12}H_5Cl_5",                      "57465-28-8", "REHONNLQRWTIFF-UHFFFAOYSA-N", "3,3',4,4',5-pentachlorobiphenyl", "PCB-126")
  CALL Species("C_{12}H_5Cl_5",                      "39635-33-1", "MXVAYAXIPRGORY-UHFFFAOYSA-N", "3,3',4,5,5'-pentachlorobiphenyl", "PCB-127")
  CALL Species("C_{12}H_4Cl_6",                      "38380-07-3", "BTAGRXWGMYTPBY-UHFFFAOYSA-N", "2,2',3,3',4,4'-hexachlorobiphenyl", "PCB-128")
  CALL Species("C_{12}H_4Cl_6",                      "55215-18-4", "VQQKIXKPMJTUMP-UHFFFAOYSA-N", "2,2',3,3',4,5-hexachlorobiphenyl", "PCB-129")
  CALL Species("C_{12}H_4Cl_6",                      "52663-66-8", "YFSLABAYQDPWPF-UHFFFAOYSA-N", "2,2',3,3',4,5'-hexachlorobiphenyl", "PCB-130")
  CALL Species("C_{12}H_4Cl_6",                      "61798-70-7", "WDLTVNWWEZJMPF-UHFFFAOYSA-N", "2,2',3,3',4,6-hexachlorobiphenyl", "PCB-131")
  CALL Species("C_{12}H_4Cl_6",                      "38380-05-1", "OKBJVIVEFXPEOU-UHFFFAOYSA-N", "2,2',3,3',4,6'-hexachlorobiphenyl", "PCB-132")
  CALL Species("C_{12}H_4Cl_6",                      "35694-04-3", "AJKLKINFZLWHQE-UHFFFAOYSA-N", "2,2',3,3',5,5'-hexachlorobiphenyl", "PCB-133")
  CALL Species("C_{12}H_4Cl_6",                      "52704-70-8", "RVWLHPJFOKUPNM-UHFFFAOYSA-N", "2,2',3,3',5,6-hexachlorobiphenyl", "PCB-134")
  CALL Species("C_{12}H_4Cl_6",                      "52744-13-5", "UUTNFLRSJBQQJM-UHFFFAOYSA-N", "2,2',3,3',5,6'-hexachlorobiphenyl", "PCB-135")
  CALL Species("C_{12}H_4Cl_6",                      "38411-22-2", "FZFUUSROAHKTTF-UHFFFAOYSA-N", "2,2',3,3',6,6'-hexachlorobiphenyl", "PCB-136")
  CALL Species("C_{12}H_4Cl_6",                      "35694-06-5", "CKLLRBPBZLTGDJ-UHFFFAOYSA-N", "2,2',3,4,4',5-hexachlorobiphenyl", "PCB-137")
  CALL Species("C_{12}H_4Cl_6",                      "35065-28-2", "RPUMZMSNLZHIGZ-UHFFFAOYSA-N", "2,2',3,4,4',5'-hexachlorobiphenyl", "PCB-138")
  CALL Species("C_{12}H_4Cl_6",                      "56030-56-9", "SPOPSCCFZQFGDL-UHFFFAOYSA-N", "2,2',3,4,4',6-hexachlorobiphenyl", "PCB-139")
  CALL Species("C_{12}H_4Cl_6",                      "59291-64-4", "XBBRGUHRZBZMPP-UHFFFAOYSA-N", "2,2',3,4,4',6'-hexachlorobiphenyl", "PCB-140")
  CALL Species("C_{12}H_4Cl_6",                      "52712-04-6", "UCLKLGIYGBLTSM-UHFFFAOYSA-N", "2,2',3,4,5,5'-hexachlorobiphenyl", "PCB-141")
  CALL Species("C_{12}H_4Cl_6",                      "41411-61-4", "RUEIBQJFGMERJD-UHFFFAOYSA-N", "2,2',3,4,5,6-hexachlorobiphenyl", "PCB-142")
  CALL Species("C_{12}H_4Cl_6",                      "68194-15-0", "UQPQKLGBEKEZBV-UHFFFAOYSA-N", "2,2',3,4,5,6'-hexachlorobiphenyl", "PCB-143")
  CALL Species("C_{12}H_4Cl_6",                      "68194-14-9", "CXXRQFOKRZJAJA-UHFFFAOYSA-N", "2,2',3,4,5',6-hexachlorobiphenyl", "PCB-144")
  CALL Species("C_{12}H_4Cl_6",                      "74472-40-5", "JZFZCLFEPXCRCA-UHFFFAOYSA-N", "2,2',3,4,6,6'-hexachlorobiphenyl", "PCB-145")
  CALL Species("C_{12}H_4Cl_6",                      "51908-16-8", "BQHCQAQLTCQFJZ-UHFFFAOYSA-N", "2,2',3,4',5,5'-hexachlorobiphenyl", "PCB-146")
  CALL Species("C_{12}H_4Cl_6",                      "68194-13-8", "AQONCPKMJSBHQT-UHFFFAOYSA-N", "2,2',3,4',5,6-hexachlorobiphenyl", "PCB-147")
  CALL Species("C_{12}H_4Cl_6",                      "74472-41-6", "CTVRBEKNQHJPLX-UHFFFAOYSA-N", "2,2',3,4',5,6'-hexachlorobiphenyl", "PCB-148")
  CALL Species("C_{12}H_4Cl_6",                      "38380-04-0", "LKHLFUVHHXCNJH-UHFFFAOYSA-N", "2,2',3,4',5',6-hexachlorobiphenyl", "PCB-149")
  CALL Species("C_{12}H_4Cl_6",                      "68194-08-1", "RPPNJBZNXQNKNM-UHFFFAOYSA-N", "2,2',3,4',6,6'-hexachlorobiphenyl", "PCB-150")
  CALL Species("C_{12}H_4Cl_6",                      "52663-63-5", "UHCLFIWDCYOTOL-UHFFFAOYSA-N", "2,2',3,5,5',6-hexachlorobiphenyl", "PCB-151")
  CALL Species("C_{12}H_4Cl_6",                      "68194-09-2", "CLODVDBWNVQLGO-UHFFFAOYSA-N", "2,2',3,5,6,6'-hexachlorobiphenyl", "PCB-152")
  CALL Species("C_{12}H_4Cl_6",                      "35065-27-1", "MVWHGTYKUMDIHL-UHFFFAOYSA-N", "2,2',4,4',5,5'-hexachlorobiphenyl", "PCB-153")
  CALL Species("C_{12}H_4Cl_6",                      "60145-22-4", "QXZHEJWDLVUFFB-UHFFFAOYSA-N", "2,2',4,4',5,6'-hexachlorobiphenyl", "PCB-154")
  CALL Species("C_{12}H_4Cl_6",                      "33979-03-2", "ICOAEPDGFWLUTI-UHFFFAOYSA-N", "2,2',4,4',6,6'-hexachlorobiphenyl", "PCB-155")
  CALL Species("C_{12}H_4Cl_6",                      "38380-08-4", "LCXMEXLGMKFLQO-UHFFFAOYSA-N", "2,3,3',4,4',5-hexachlorobiphenyl", "PCB-156")
  CALL Species("C_{12}H_4Cl_6",                      "69782-90-7", "YTWXDQVNPCIEOX-UHFFFAOYSA-N", "2,3,3',4,4',5'-hexachlorobiphenyl", "PCB-157")
  CALL Species("C_{12}H_4Cl_6",                      "74472-42-7", "ZQUPQXINXTWCQR-UHFFFAOYSA-N", "2,3,3',4,4',6-hexachlorobiphenyl", "PCB-158")
  CALL Species("C_{12}H_4Cl_6",                      "39635-35-3", "YZKLGRAIIIGOHB-UHFFFAOYSA-N", "2,3,3',4,5,5'-hexachlorobiphenyl", "PCB-159")
  CALL Species("C_{12}H_4Cl_6",                      "41411-62-5", "JHJMZCXLJXRCHK-UHFFFAOYSA-N", "2,3,3',4,5,6-hexachlorobiphenyl", "PCB-160")
  CALL Species("C_{12}H_4Cl_6",                      "74472-43-8", "UNPTZXSJGZTGJJ-UHFFFAOYSA-N", "2,3,3',4,5',6-hexachlorobiphenyl", "PCB-161")
  CALL Species("C_{12}H_4Cl_6",                      "39635-34-2", "AHZUOPSGLWYCNF-UHFFFAOYSA-N", "2,3,3',4',5,5'-hexachlorobiphenyl", "PCB-162")
  CALL Species("C_{12}H_4Cl_6",                      "74472-44-9", "ZAGRQXMWMRUYRB-UHFFFAOYSA-N", "2,3,3',4',5,6-hexachlorobiphenyl", "PCB-163")
  CALL Species("C_{12}H_4Cl_6",                      "74472-45-0", "HAZQOLYHFUUJJN-UHFFFAOYSA-N", "2,3,3',4',5',6-hexachlorobiphenyl", "PCB-164")
  CALL Species("C_{12}H_4Cl_6",                      "74472-46-1", "ZEATXTCWXKQPHO-UHFFFAOYSA-N", "2,3,3',5,5',6-hexachlorobiphenyl", "PCB-165")
  CALL Species("C_{12}H_4Cl_6",                      "41411-63-6", "BTOCFTAWZMMTNB-UHFFFAOYSA-N", "2,3,4,4',5,6-hexachlorobiphenyl", "PCB-166")
  CALL Species("C_{12}H_4Cl_6",                      "52663-72-6", "AZXHAWRMEPZSSV-UHFFFAOYSA-N", "2,3',4,4',5,5'-hexachlorobiphenyl", "PCB-167")
  CALL Species("C_{12}H_4Cl_6",                      "59291-65-5", "PITHIPNORFGJPI-UHFFFAOYSA-N", "2,3',4,4',5',6-hexachlorobiphenyl", "PCB-168")
  CALL Species("C_{12}H_4Cl_6",                      "32774-16-6", "ZHLICBPIXDOFFG-UHFFFAOYSA-N", "3,3',4,4',5,5'-hexachlorobiphenyl", "PCB-169")
  CALL Species("C_{12}H_3Cl_7",                      "35065-30-6", "RMPWIIKNWPVWNG-UHFFFAOYSA-N", "2,2',3,3',4,4',5-heptachlorobiphenyl", "PCB-170")
  CALL Species("C_{12}H_3Cl_7",                      "52663-71-5", "TZMHVHLTPWKZCI-UHFFFAOYSA-N", "2,2',3,3',4,4',6-heptachlorobiphenyl", "PCB-171")
  CALL Species("C_{12}H_3Cl_7",                      "52663-74-8", "HOPMUCXYRNOABF-UHFFFAOYSA-N", "2,2',3,3',4,5,5'-heptachlorobiphenyl", "PCB-172")
  CALL Species("C_{12}H_3Cl_7",                      "68194-16-1", "PAYFWJAKKLILIT-UHFFFAOYSA-N", "2,2',3,3',4,5,6-heptachlorobiphenyl", "PCB-173")
  CALL Species("C_{12}H_3Cl_7",                      "38411-25-5", "ZDLMBNHYTPHDLF-UHFFFAOYSA-N", "2,2',3,3',4,5,6'-heptachlorobiphenyl", "PCB-174")
  CALL Species("C_{12}H_3Cl_7",                      "40186-70-7", "KJBDZJFSYQUNJT-UHFFFAOYSA-N", "2,2',3,3',4,5',6-heptachlorobiphenyl", "PCB-175")
  CALL Species("C_{12}H_3Cl_7",                      "52663-65-7", "HGMYRFJAJNYBRX-UHFFFAOYSA-N", "2,2',3,3',4,6,6'-heptachlorobiphenyl", "PCB-176")
  CALL Species("C_{12}H_3Cl_7",                      "52663-70-4", "CXOYNJAHPUASHN-UHFFFAOYSA-N", "2,2',3,3',4,5',6'-heptachlorobiphenyl", "PCB-177")
  CALL Species("C_{12}H_3Cl_7",                      "52663-67-9", "WCIBKXHMIXUQHK-UHFFFAOYSA-N", "2,2',3,3',5,5',6-heptachlorobiphenyl", "PCB-178")
  CALL Species("C_{12}H_3Cl_7",                      "52663-64-6", "XYHVYEUZLSYHDP-UHFFFAOYSA-N", "2,2',3,3',5,6,6'-heptachlorobiphenyl", "PCB-179")
  CALL Species("C_{12}H_3Cl_7",                      "35065-29-3", "WBHQEUPUMONIKF-UHFFFAOYSA-N", "2,2',3,4,4',5,5'-heptachlorobiphenyl", "PCB-180")
  CALL Species("C_{12}H_3Cl_7",                      "74472-47-2", "DJEUXBQAKBLKPO-UHFFFAOYSA-N", "2,2',3,4,4',5,6-heptachlorobiphenyl", "PCB-181")
  CALL Species("C_{12}H_3Cl_7",                      "60145-23-5", "RXRLRYZUMSYVLS-UHFFFAOYSA-N", "2,2',3,4,4',5,6'-heptachlorobiphenyl", "PCB-182")
  CALL Species("C_{12}H_3Cl_7",                      "52663-69-1", "KQBFUDNJKCZEDQ-UHFFFAOYSA-N", "2,2',3,4,4',5',6-heptachlorobiphenyl", "PCB-183")
  CALL Species("C_{12}H_3Cl_7",                      "74472-48-3", "OBIUJJSQKPGKME-UHFFFAOYSA-N", "2,2',3,4,4',6,6'-heptachlorobiphenyl", "PCB-184")
  CALL Species("C_{12}H_3Cl_7",                      "52712-05-7", "PYZHTHZEHQHHEN-UHFFFAOYSA-N", "2,2',3,4,5,5',6-heptachlorobiphenyl", "PCB-185")
  CALL Species("C_{12}H_3Cl_7",                      "74472-49-4", "FGDPOTMRBQHPJK-UHFFFAOYSA-N", "2,2',3,4,5,6,6'-heptachlorobiphenyl", "PCB-186")
  CALL Species("C_{12}H_3Cl_7",                      "52663-68-0", "UDMZPLROONOSEF-UHFFFAOYSA-N", "2,2',3,4',5,5',6-heptachlorobiphenyl", "PCB-187")
  CALL Species("C_{12}H_3Cl_7",                      "74487-85-7", "MMTJWDQKGUNSDK-UHFFFAOYSA-N", "2,2',3,4',5,6,6'-heptachlorobiphenyl", "PCB-188")
  CALL Species("C_{12}H_3Cl_7",                      "39635-31-9", "XUAWBXBYHDRROL-UHFFFAOYSA-N", "2,3,3',4,4',5,5'-heptachlorobiphenyl", "PCB-189")
  CALL Species("C_{12}H_3Cl_7",                      "41411-64-7", "TYEDCFVCFDKSBK-UHFFFAOYSA-N", "2,3,3',4,4',5,6-heptachlorobiphenyl", "PCB-190")
  CALL Species("C_{12}H_3Cl_7",                      "74472-50-7", "TVFXBXWAXIMLAQ-UHFFFAOYSA-N", "2,3,3',4,4',5',6-heptachlorobiphenyl", "PCB-191")
  CALL Species("C_{12}H_3Cl_7",                      "74472-51-8", "ZUTDUGMNROUBOX-UHFFFAOYSA-N", "2,3,3',4,5,5',6-heptachlorobiphenyl", "PCB-192")
  CALL Species("C_{12}H_3Cl_7",                      "69782-91-8", "SSTJUBQGYXNFFP-UHFFFAOYSA-N", "2,3,3',4',5,5',6-heptachlorobiphenyl", "PCB-193")
  CALL Species("C_{12}H_2Cl_8",                      "35694-08-7", "DTMRKGRREZAYAP-UHFFFAOYSA-N", "2,2',3,3',4,4',5,5'-octachlorobiphenyl", "PCB-194")
  CALL Species("C_{12}H_2Cl_8",                      "52663-78-2", "JAHJITLFJSDRCG-UHFFFAOYSA-N", "2,2',3,3',4,4',5,6-octachlorobiphenyl", "PCB-195")
  CALL Species("C_{12}H_2Cl_8",                      "42740-50-1", "BQFCCUSDZLKBJG-UHFFFAOYSA-N", "2,2',3,3',4,4',5,6'-octachlorobiphenyl", "PCB-196")
  CALL Species("C_{12}H_2Cl_8",                      "33091-17-7", "YPDBBDKYNWRFMF-UHFFFAOYSA-N", "2,2',3,3',4,4',6,6'-octachlorobiphenyl", "PCB-197")
  CALL Species("C_{12}H_2Cl_8",                      "68194-17-2", "PJHBSPRZHUOIAS-UHFFFAOYSA-N", "2,2',3,3',4,5,5',6-octachlorobiphenyl", "PCB-198")
  CALL Species("C_{12}H_2Cl_8",                      "52663-75-9", "HJBYDWKNARZTMJ-UHFFFAOYSA-N", "2,2',3,3',4,5,5',6'-octachlorobiphenyl", "PCB-199")
  CALL Species("C_{12}H_2Cl_8",                      "52663-73-7", "HHXNVASVVVNNDG-UHFFFAOYSA-N", "2,2',3,3',4,5,6,6'-octachlorobiphenyl", "PCB-200")
  CALL Species("C_{12}H_2Cl_8",                      "40186-71-8", "LJQOBQLZTUSEJA-UHFFFAOYSA-N", "2,2',3,3',4,5',6,6'-octachlorobiphenyl", "PCB-201")
  CALL Species("C_{12}H_2Cl_8",                       "2136-99-4", "JPOPEORRMSDUIP-UHFFFAOYSA-N", "2,2',3,3',5,5',6,6'-octachlorobiphenyl", "PCB-202")
  CALL Species("C_{12}H_2Cl_8",                      "52663-76-0", "DCPDZFRGNJDWPP-UHFFFAOYSA-N", "2,2',3,4,4',5,5',6-octachlorobiphenyl", "PCB-203")
  CALL Species("C_{12}H_2Cl_8",                      "74472-52-9", "JDZUWXRNKHXZFE-UHFFFAOYSA-N", "2,2',3,4,4',5,6,6'-octachlorobiphenyl", "PCB-204")
  CALL Species("C_{12}H_2Cl_8",                      "74472-53-0", "VXXBCDUYUQKWCK-UHFFFAOYSA-N", "2,3,3',4,4',5,5',6-octachlorobiphenyl", "PCB-205")
  CALL Species("C_{12}HCl_9",                        "40186-72-9", "JFIMDKGRGPNPRQ-UHFFFAOYSA-N", "2,2',3,3',4,4',5,5',6-nonachlorobiphenyl", "PCB-206")
  CALL Species("C_{12}HCl_9",                        "52663-79-3", "YGDPIDTZOQGPAX-UHFFFAOYSA-N", "2,2',3,3',4,4',5,6,6'-nonachlorobiphenyl", "PCB-207")
  CALL Species("C_{12}HCl_9",                        "52663-77-1", "XIFFTDRFWYFAPO-UHFFFAOYSA-N", "2,2',3,3',4,5,5',6,6'-nonachlorobiphenyl", "PCB-208")
  CALL Species("C_{12}Cl_{10}",                       "2051-24-3", "ONXPZLFXDMAPRO-UHFFFAOYSA-N", "decachlorobiphenyl", "PCB-209")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Oxygenated chlorocarbons (\chem{C}, \chem{H}, \chem{O}, \chem{Cl})")
  ! C, H, O, Cl:
  ! aliphatic:
  CALL Species("CCl_2O",                                "75-44-5", "YGYAWVDWMABLBF-UHFFFAOYSA-N", "phosgene", "")
  CALL Species("CHO_2Cl_3",                                    "", "DUBMHDTUZCIGCY-UHFFFAOYSA-N", "MCM:CCL3OOH", "")
  CALL Species("CH_3O_2Cl",                                    "", "DUNYOPWYTOSXOC-UHFFFAOYSA-N", "MCM:CH2CLOOH", "")
  CALL Species("CH_2O_2Cl_2",                                  "", "TUIYYMKEIGYDJB-UHFFFAOYSA-N", "MCM:CHCL2OOH", "")
  CALL Species("CHOCl_3",                                      "", "GYLIOGDFGLKMOL-UHFFFAOYSA-N", "MCM:CCL3OH", "")
  CALL Species("CH_3OCl",                                      "", "BCUPGIHTCQJCSI-UHFFFAOYSA-N", "MCM:CH2CLOH", "")
  CALL Species("CH_2OCl_2",                                    "", "GJYVZUKSNFSLCL-UHFFFAOYSA-N", "MCM:CHCL2OH", "")
  CALL Species("CHOCl",                                        "", "GFAUNYMRSKVDJL-UHFFFAOYSA-N", "MCM:CHOCL", "")
  CALL Species("CHO_2Cl",                                      "", "AOGYCOYQMAVAFD-UHFFFAOYSA-N", "MCM:CLCO2H", "")
  CALL Species("C_2H_2Cl_2O",                           "79-02-7", "NWQWQKUXRJYXFH-UHFFFAOYSA-N", "dichloroacetaldehyde", "")
  CALL Species("C_2H_2Cl_2O",                           "79-04-9", "VGCXGMAHQTYDJK-UHFFFAOYSA-N", "chloroacetyl chloride", "")
  CALL Species("C_2H_3Cl_3O_2",                        "302-17-0", "RNFNDJAIBTYOQL-UHFFFAOYSA-N", "chloral hydrate", "")
  CALL Species("C_2H_3ClO",                            "107-20-0", "QSKPIOLLBIHNAC-UHFFFAOYSA-N", "chloroacetaldehyde", "")
  CALL Species("C_2H_5ClO",                            "107-07-3", "SZIFAVKTNFCBPC-UHFFFAOYSA-N", "2-chloroethanol", "")
  CALL Species("C_2H_4Cl_2O",                          "598-38-9", "IDJOCJAIQSKSOP-UHFFFAOYSA-N", "2,2-dichloroethanol", "")
  CALL Species("ClC_2D_4OH",                        "117067-62-6", "SZIFAVKTNFCBPC-LNLMKGTHSA-N", "2-chloroethanol-d4", "")
  CALL Species("C_2H_2O_2Cl_4",                                "", "SWNDCAXLXSSRFN-UHFFFAOYSA-N", "MCM:CCL2CL2OOH", "")
  CALL Species("C_2H_3O_2Cl_3",                                "", "RJIYMCLTECJZCN-UHFFFAOYSA-N", "MCM:CCL3CH2OOH", "")
  CALL Species("C_2HO_3Cl_3",                                  "", "DRTNVNKYVPOWCY-UHFFFAOYSA-N", "MCM:CCL3CO3H", "")
  CALL Species("C_2H_3O_2Cl_3",                                "", "CHVIFDTZRLKJET-UHFFFAOYSA-N", "MCM:CH2CL3OOH", "")
  CALL Species("C_2H_5O_2Cl",                                  "", "PIXGNJBYAUTOCI-UHFFFAOYSA-N", "MCM:CH2CLCOOH", "")
  CALL Species("C_2H_4O_2Cl_2",                                "", "YUVMNUKOYYKKNS-UHFFFAOYSA-N", "MCM:CH3CCL2OOH", "")
  CALL Species("C_2H_5O_2Cl",                                  "", "MCYQTMMYWIXRIL-UHFFFAOYSA-N", "MCM:CH3CHCLOOH", "")
  CALL Species("C_2H_2O_3Cl_2",                                "", "VWNSYHZUZUFFDM-UHFFFAOYSA-N", "MCM:CHCL2CO3H", "")
  CALL Species("C_2H_4O_2Cl_2",                                "", "GJBKLBBJOKITMY-UHFFFAOYSA-N", "MCM:CHCL2COOH", "")
  CALL Species("C_2H_3O_2Cl_3",                                "", "AMASFZACKVPSTN-UHFFFAOYSA-N", "MCM:CHCL3OOH", "")
  CALL Species("C_2H_3O_3Cl",                                  "", "JSQPGNJFQUOEPA-UHFFFAOYSA-N", "MCM:CLETO3H", "")
  CALL Species("C_2H_4O_2Cl_2",                                "", "UPXDMONURWDHKL-UHFFFAOYSA-N", "MCM:DICLETO2H", "")
  CALL Species("C_2H_4O_2Cl_2",                                "", "HHQQTVXFKGIICR-UHFFFAOYSA-N", "MCM:C2CL2OH2", "")
  CALL Species("C_2H_4O_3Cl_2",                                "", "NCBKAYIJWMIXBU-UHFFFAOYSA-N", "MCM:C2CL2OHOOH", "")
  CALL Species("C_2H_3O_2Cl_3",                                "", "MYDJEUINZIFHKK-UHFFFAOYSA-N", "MCM:C2CL32OH", "")
  CALL Species("C_2H_3O_3Cl_3",                                "", "KHLAQOPQQHOMKQ-UHFFFAOYSA-N", "MCM:C2CL3HOOOH", "")
  CALL Species("C_2H_3O_3Cl_3",                                "", "CEKNFZNLLTXCLL-UHFFFAOYSA-N", "MCM:C2CL3OHOOH", "")
  CALL Species("C_2H_3OCl_3",                          "115-20-8", "KPWDGTGXUYRARH-UHFFFAOYSA-N", "trichloroethanol", "")
  CALL Species("C_2H_5O_3Cl",                                  "", "MIFGQALJBIDZDW-UHFFFAOYSA-N", "MCM:CCLOHCOOH", "")
  CALL Species("C_2H_3OCl_3",                                  "", "HYCHPIPDVAXCCJ-UHFFFAOYSA-N", "MCM:CH2CL3OH", "")
  CALL Species("C_2H_5O_2Cl",                                  "", "VVKFXODVPQSIHU-UHFFFAOYSA-N", "MCM:CH2OHCCLOH", "")
  CALL Species("C_2H_4OCl_2",                                  "", "DNBABSPIEDTPHK-UHFFFAOYSA-N", "MCM:CH3CCL2OH", "")
  CALL Species("C_2H_5OCl",                                    "", "KJESGYZFVCIMDE-UHFFFAOYSA-N", "MCM:CH3CHCLOH", "")
  CALL Species("C_2H_2OCl_4",                                  "", "LQINPQOSBLVJBS-UHFFFAOYSA-N", "MCM:CHCL2CL2OH", "")
  CALL Species("C_2H_3OCl_3",                                  "", "NIBKDWIGIKUFKL-UHFFFAOYSA-N", "MCM:CHCL3OH", "")
  CALL Species("C_2H_4O_2Cl_2",                                "", "ZOZPDNLNQWCVSW-UHFFFAOYSA-N", "MCM:CL2OHCH2OH", "")
  CALL Species("C_2H_2O_4Cl_2",                                "", "MACSTFWYVGNISK-UHFFFAOYSA-N", "MCM:CL2OHCO3H", "")
  CALL Species("C_2H_4O_3Cl_2",                                "", "WOJAYLCSWSBEJV-UHFFFAOYSA-N", "MCM:CL2OHOOH", "")
  CALL Species("C_2H_3O_4Cl",                                  "", "IODLIDNNOHQSDW-UHFFFAOYSA-N", "MCM:CLOHCO3H", "")
  CALL Species("C_2H_5O_3Cl",                                  "", "WXPZQCJQULOKCY-UHFFFAOYSA-N", "MCM:COHCCLOOH", "")
  CALL Species("C_2H_4OCl_2",                                  "", "QXBDFCZHAAOUBY-UHFFFAOYSA-N", "MCM:DICLETOH", "")
  CALL Species("C_2H_2O_2Cl_4",                                "", "BVBMRJKFECUARX-UHFFFAOYSA-N", "MCM:TCE2OH", "")
  CALL Species("C_2H_2O_3Cl_4",                                "", "YKIXQGIQZGTABG-UHFFFAOYSA-N", "MCM:TCEOHOOH", "")
  CALL Species("C_2H_3O_2Cl",                                  "", "RYRGLFUQPWFXSD-UHFFFAOYSA-N", "MCM:CCLOHCHO", "")
  CALL Species("C_2H_2O_2Cl_2",                                "", "LCOUNIIIIIWZOM-UHFFFAOYSA-N", "MCM:CL2OHCHO", "")
  CALL Species("C_2H_3OCl",                                    "", "WETWJCDKMRHUPV-UHFFFAOYSA-N", "MCM:CH3COCL", "")
  CALL Species("C_2HOCl_3",                                    "", "FBCCMZVIWNDFMO-UHFFFAOYSA-N", "MCM:CHCL2COCL", "")
  CALL Species("C_2H_3O_3Cl",                                  "", "ARKKUYXZQNWAAF-UHFFFAOYSA-N", "MCM:CLCOCH2OOH", "")
  CALL Species("C_2HO_3Cl_3",                                  "", "FXUYMCPKRPYDCU-UHFFFAOYSA-N", "MCM:CLCOCL2OOH", "")
  CALL Species("C_2H_2O_3Cl_2",                                "", "WROSSTYSMNZXTC-UHFFFAOYSA-N", "MCM:CLCOCLOOH", "")
  CALL Species("C_2O_2Cl_2",                                   "", "CTSLXHKWHWQRSH-UHFFFAOYSA-N", "MCM:CLGLYOX", "")
  CALL Species("C_2HO_2Cl_3",                                  "", "WBBOIYQGQVKNEO-UHFFFAOYSA-N", "MCM:CCL2OHCOCL", "")
  CALL Species("C_2H_2O_2Cl_2",                                "", "NABBVDLPPITPRF-UHFFFAOYSA-N", "MCM:CCLOHCOCL", "")
  CALL Species("C_2H_3O_2Cl",                                  "", "LCIMJULVQOQTEZ-UHFFFAOYSA-N", "MCM:CH2OHCOCL", "")
  CALL Species("C_2HO_2Cl",                                    "", "ZNBGBHISQKMEPA-UHFFFAOYSA-N", "MCM:CL1GLYOX", "")
  CALL Species("C_2H_3O_3Cl",                                  "", "SORBEIKZIPTJRS-UHFFFAOYSA-N", "MCM:CLOHCO2H", "")
  CALL Species("C_3H_3Cl_3O",                          "918-00-3", "SMZHKGXSEAGRTI-UHFFFAOYSA-N", "1,1,1-trichloro-2-propanone", "")
  CALL Species("C_3H_4Cl_2O",                          "513-88-2", "CSVFWMMPUJDVKH-UHFFFAOYSA-N", "1,1-dichloro-2-propanone", "1,1-dichloroacetone")
  CALL Species("C_3H_4Cl_2O_2",                        "627-11-2", "SVDDJQGVOFZBNX-UHFFFAOYSA-N", "carbonochloridic acid, 2-chloroethyl ester", "chloroethyl chloroformate")
  CALL Species("C_3H_5ClO_2",                          "541-41-3", "RIFGWPKJUGCATF-UHFFFAOYSA-N", "carbonochloridic acid, ethyl ester", "ethyl chloroformate")
  CALL Species("C_3H_5ClO_2",                          "598-78-7", "GAWAYYRQGQZKCR-UHFFFAOYSA-N", "2-chloropropanoic acid", "")
  CALL Species("C_3H_6Cl_2O",                          "616-23-9", "ZXCYIJGIGSDJQQ-UHFFFAOYSA-N", "2,3-dichloro-1-propanol", "")
  CALL Species("C_3H_6Cl_2O",                           "96-23-1", "DEWLEGDTCGBNGU-UHFFFAOYSA-N", "1,3-dichloro-2-propanol", "")
  CALL Species("C_3H_7ClO_2",                           "96-24-2", "SSZWWUDQMAHNAQ-UHFFFAOYSA-N", "3-chloro-1,2-propanediol", "")
  CALL Species("C_3H_7ClO",                            "127-00-4", "YYTSGNJTASLUOY-UHFFFAOYSA-N", "1-chloro-2-propanol", "")
  CALL Species("C_3H_7ClO",                             "78-89-7", "VZIQXGLTRZLBEX-UHFFFAOYSA-N", "2-chloro-1-propanol", "")
  CALL Species("C_3H_7ClO",                            "627-30-5", "LAMUXTNQCICZQX-UHFFFAOYSA-N", "3-chloro-1-propanol", "")
  CALL Species("CCl_3CHO",                              "75-87-6", "HFFLGKNGCAIQMO-UHFFFAOYSA-N", "trichloroethanal", "trichloroacetaldehyde; chloral")
  CALL Species("CH_2ClCOCH_3",                          "78-95-5", "BULLHNJGPPOUOX-UHFFFAOYSA-N", "chloro-2-propanone", "chloroacetone")
  CALL Species("CH_2ClCOOH",                            "79-11-8", "FOCAUTSVDIKZOP-UHFFFAOYSA-N", "chloroethanoic acid", "chloroacetic acid")
  CALL Species("CHCl_2COOH",                            "79-43-6", "JXTHNDFMNIQAHM-UHFFFAOYSA-N", "dichloroethanoic acid", "dichloroacetic acid")
  CALL Species("CCl_3COOH",                             "76-03-9", "YNJBWRMUSHSURL-UHFFFAOYSA-N", "trichloroethanoic acid", "trichloroacetic acid")
  CALL Species("C_3H_4Cl_2O_2",                         "75-99-0", "NDUPDOJHUQKPAG-UHFFFAOYSA-N", "2,2-dichloro-propanoic acid", "")
  CALL Species("CCl_3COCl",                             "76-02-8", "PVFOMCVHYWHZJE-UHFFFAOYSA-N", "trichloroacetylchloride", "")
  CALL Species("C_3Cl_6O",                             "116-16-5", "DOJXGHGHTWFZHK-UHFFFAOYSA-N", "hexachloroacetone", "")
  CALL Species("C_3H_5ClO",                            "106-89-8", "BRLQWZUYTZBJKN-UHFFFAOYSA-N", "(chloromethyl)-oxirane", "epichlorohydrin") ! other CAS: 13403-37-7
  CALL Species("C_3H_5ClO_2",                           "96-34-4", "QABLOFMHHSOFRJ-UHFFFAOYSA-N", "methyl chloroethanoate", "")
  CALL Species("C_3H_4O_3Cl_2",                                "", "VMJCNPRTZBGZGF-UHFFFAOYSA-N", "MCM:CL12CO3H", "")
  CALL Species("C_3H_6O_2Cl_2",                                "", "WGVINOMLJVZPKZ-UHFFFAOYSA-N", "MCM:CL12PRAOOH", "")
  CALL Species("C_3H_6O_2Cl_2",                                "", "OMIMXWULHYQJIC-UHFFFAOYSA-N", "MCM:CL12PRBOOH", "")
  CALL Species("C_3H_6O_2Cl_2",                                "", "ULSSYZYIMOSSQB-UHFFFAOYSA-N", "MCM:CL12PRCOOH", "")
  CALL Species("C_3H_6OCl_2",                                  "", "QEZDCTNHTRSNMD-UHFFFAOYSA-N", "MCM:CL12PRBOH", "")
  CALL Species("C_3H_6OCl_2",                                  "", "FLTSEOGWHPJWRV-UHFFFAOYSA-N", "MCM:CL12PRCOH", "")
  CALL Species("C_3H_4OCl_2",                                  "", "IZRKUJREXIKAQM-UHFFFAOYSA-N", "MCM:CL12PRCHO", "")
  CALL Species("C_3H_4OCl_2",                                  "", "JEQDSBVHLKBEIZ-UHFFFAOYSA-N", "MCM:CH3CLCOCL", "")
  CALL Species("C_3H_5O_3Cl",                                  "", "HLLYIZVQRAGDFF-UHFFFAOYSA-N", "MCM:CH3COCLOOH", "")
  CALL Species("C_3H_4O_3Cl_2",                                "", "RUPGTCZALLJKQM-UHFFFAOYSA-N", "MCM:CLCOCLMOOH", "")
  CALL Species("C_3H_4O_2Cl_2",                                "", "GKFWNPPZHDYVLI-UHFFFAOYSA-N", "MCM:CL12CO2H", "")
  CALL Species("C_4H_7ClO",                           "4091-39-8", "OIMRLHCSLQUXLL-UHFFFAOYSA-N", "3-chloro-2-butanone", "")
  CALL Species("C_4H_7ClO_2",                          "105-39-5", "VEUUMBGHMNQHGO-UHFFFAOYSA-N", "ethyl chloroethanoate", "")
  CALL Species("C_4H_7ClO_2",                          "108-23-6", "IVRIRQXJSNCSPQ-UHFFFAOYSA-N", "carbonochloridic acid, 1-methylethyl ester", "")
  CALL Species("C_4H_4Cl_2O_3",                        "541-88-8", "PNVPNXKRAUBJGW-UHFFFAOYSA-N", "chloroacetic acid anhydride", "")
  CALL Species("C_4H_9ClO",                           "1320-66-7", "OSASVXMJTNOKOY-UHFFFAOYSA-N", "chlorobutanol", "")
  CALL Species("C_5H_7ClO_3",                         "4755-81-1", "GYQRIAVRKLRQKP-UHFFFAOYSA-N", "methyl 2-chloroacetoacetate", "")
  CALL Species("C_5H_3Cl_3O_2",                     "122551-89-7", "WNQKLIFDPFSPIZ-UHFFFAOYSA-N", "3-chloro-4-(dichloromethyl)-2-(5H)-furanone", "")
  CALL Species("C_5H_3Cl_3O_3",                      "77439-76-0", "WNTRMRXAGJOLCU-UHFFFAOYSA-N", "3-chloro-4-(dichloromethyl)-5-hydroxy-2-(5H)-furanone", "")
  CALL Species("C_6H_9Cl_3O",                       "145556-04-3", "KAUWEMQJFDRELR-UHFFFAOYSA-N", "1,2,4-trichloro-2-methyl-3-pentanone", "")
  CALL Species("C_6H_{12}Cl_2O",                     "39638-32-9", "BULHJTXRZFEUDQ-UHFFFAOYSA-N", "bis(2-chloroisopropyl) ether", "")
  CALL Species("C_8H_9Cl_5O_2",                      "75147-20-5", "JZJILZTVTMMGAR-UHFFFAOYSA-N", "butyl 2,2,3,4,4-pentachloro-3-butenoate", "")
  CALL Species("C_8H_9Cl_3O",                        "52314-67-7", "CHLAOFANYRDCPD-UHFFFAOYSA-N", "3-(2,2-dichlorovinyl)-2,2-dimethylcyclopropane carbonyl chloride", "")
  CALL Species("C_8H_{10}Cl_2O_2",                   "55701-05-8", "LLMLSUSAKZVFOA-UHFFFAOYSA-N", "3-(2,2-dichlorovinyl)-2,2-dimethylcyclopropane carboxylic acid", "")
  CALL Species("C_9H_{14}Cl_4O_2",                   "64667-33-0", "POFHGKISWXYKLB-UHFFFAOYSA-N", "hexanoic acid, 3,3-dimethyl-4,6,6,6-tetrachloro, methyl ester", "")
  CALL Species("C_9H_{12}Cl_2O_2",                   "61898-95-1", "QJOOIMSFFIUFKX-UHFFFAOYSA-N", "methyl 3-(2,2-dichlorovinyl)-2,2-dimethylcyclopropanecarboxylate", "")
  CALL Species("C_{10}H_4Cl_8O",                     "27304-13-8", "VWGNQYSIWFHEQU-UHFFFAOYSA-N", "oxychlordane", "")
  CALL Species("C_{10}Cl_{10}O",                       "143-50-0", "LHHGDZSESBACKH-UHFFFAOYSA-N", "kepone", "")
  ! aliphatic ethers:
  CALL Species("C_2H_4Cl_2O",                          "542-88-1", "HRQGCQVOJVTVLU-UHFFFAOYSA-N", "bis-(chloromethyl) ether", "")
  CALL Species("C_4H_8Cl_2O",                          "111-44-4", "ZNSMNVMLTJELDZ-UHFFFAOYSA-N", "1,5-dichloro-3-oxapentane", "bis-(2-chloroethyl)-ether")
  CALL Species("C_4H_9ClO_2",                           "97-97-2", "CRZJPEIBPQWDGJ-UHFFFAOYSA-N", "2-chloro-1,1-dimethoxyethane", "")
  CALL Species("C_4H_7ClO",                            "110-75-8", "DNJRKFKAFWSXSE-UHFFFAOYSA-N", "(2-chloroethoxy)-ethene", "2-chloroethylvinylether")
  CALL Species("C_5H_{10}Cl_2O_2",                     "111-91-1", "NLXGURFLBLRZRO-UHFFFAOYSA-N", "bis-(2-chloroethoxy)-methane", "")
  CALL Species("C_6H_{12}Cl_2O",                       "108-60-1", "QCFYJCYNJLBDRT-UHFFFAOYSA-N", "bis-(2-chloroisopropyl) ether", "DCIP")
  CALL Species("C_6H_{12}Cl_2O_2",                     "112-26-5", "AGYUOJIYYGGHKV-UHFFFAOYSA-N", "1,2-bis(2-chloroethoxy)ethane", "")
  ! aromatic:
  CALL Species("C_6H_5ClO",                             "95-57-8", "ISPYQTSUDJAMAB-UHFFFAOYSA-N", "2-hydroxychlorobenzene", "$o$-chlorophenol")
  CALL Species("C_6H_5ClO",                            "108-43-0", "HORNXRXVQWOLPJ-UHFFFAOYSA-N", "3-hydroxychlorobenzene", "$m$-chlorophenol")
  CALL Species("C_6H_5ClO",                            "106-48-9", "WXNZTHHGJRFXKQ-UHFFFAOYSA-N", "4-hydroxychlorobenzene", "$p$-chlorophenol")
  CALL Species("C_6H_4Cl_2O",                          "576-24-9", "UMPSXRYVXUPCOS-UHFFFAOYSA-N", "2,3-dichlorophenol", "")
  CALL Species("C_6H_4Cl_2O",                          "120-83-2", "HFZWRUODUSTPEG-UHFFFAOYSA-N", "2,4-dichlorophenol", "")
  CALL Species("C_6H_4Cl_2O",                          "583-78-8", "RANCECPPZPIPNO-UHFFFAOYSA-N", "2,5-dichlorophenol", "")
  CALL Species("C_6H_4Cl_2O",                           "87-65-0", "HOLHYSJJBXSLMV-UHFFFAOYSA-N", "2,6-dichlorophenol", "")
  CALL Species("C_6H_4Cl_2O",                           "95-77-2", "WDNBURPWRNALGP-UHFFFAOYSA-N", "3,4-dichlorophenol", "")
  CALL Species("C_6H_4Cl_2O",                          "591-35-5", "VPOMSPZBQMDLTM-UHFFFAOYSA-N", "3,5-dichlorophenol", "")
  CALL Species("C_6H_3Cl_3O",                        "15950-66-0", "HSQFVBWFPBKHEB-UHFFFAOYSA-N", "2,3,4-trichlorophenol", "")
  CALL Species("C_6H_3Cl_3O",                          "933-78-8", "WWGQHTJIFOQAOC-UHFFFAOYSA-N", "2,3,5-trichlorophenol", "")
  CALL Species("C_6H_3Cl_3O",                           "95-95-4", "LHJGJYXLEPZJPM-UHFFFAOYSA-N", "2,4,5-trichlorophenol", "")
  CALL Species("C_6H_3Cl_3O",                          "933-75-5", "XGCHAIDDPMFRLJ-UHFFFAOYSA-N", "2,3,6-trichlorophenol", "")
  CALL Species("C_6H_3Cl_3O",                           "88-06-2", "LINPIYWFGCPVIE-UHFFFAOYSA-N", "2,4,6-trichlorophenol", "")
  CALL Species("C_6H_3Cl_3O",                          "609-19-8", "GBNHEBQXJVDXSW-UHFFFAOYSA-N", "3,4,5-trichlorophenol", "")
  CALL Species("C_6H_2Cl_4O",                         "4901-51-3", "RULKYXXCCZZKDZ-UHFFFAOYSA-N", "2,3,4,5-tetrachlorophenol", "")
  CALL Species("C_6H_2Cl_4O",                           "58-90-2", "VGVRPFIJEJYOFN-UHFFFAOYSA-N", "2,3,4,6-tetrachlorophenol", "")
  CALL Species("C_6H_2Cl_4O",                          "935-95-5", "KEWNKZNZRIAIAK-UHFFFAOYSA-N", "2,3,5,6-tetrachlorophenol", "")
  CALL Species("C_6HCl_5O",                             "87-86-5", "IZUPBVBPLAPZRR-UHFFFAOYSA-N", "hydroxypentachlorobenzene", "pentachlorophenol")
  CALL Species("C_6H_3Cl_3O_2",                      "56961-20-7", "FUTDYIMYZIMPBJ-UHFFFAOYSA-N", "3,4,5-trichloro-1,2-benzenediol", "3,4,5-trichlorocatechol")
  CALL Species("C_6H_4Cl_2O_2",                       "3428-24-8", "ACCHWUWBKYGKNM-UHFFFAOYSA-N", "4,5-dichloro-1,2-benzenediol", "4,5-dichlorocatechol")
  CALL Species("C_6H_2Cl_4O_2",                       "1198-55-6", "RRBMVWQICIXSEO-UHFFFAOYSA-N", "3,4,5,6-tetrachloro-1,2-benzenediol", "tetrachlorocatechol")
  CALL Species("C_6Cl_4O_2",                           "118-75-2", "UGNWTBMOAKPKBL-UHFFFAOYSA-N", "2,3,5,6-tetrachloro-$p$-benzoquinone", "chloranil")
  CALL Species("C_7H_7ClO",                            "615-74-7", "SMFHPCZZAAMJJO-UHFFFAOYSA-N", "2-chloro-5-methylphenol", "")
  CALL Species("C_7H_7ClO",                           "1570-64-5", "RHPUJHQBPORFGV-UHFFFAOYSA-N", "4-chloro-2-methylphenol", "")
  CALL Species("C_7H_7ClO",                             "59-50-7", "CFKMVGJGLGKFKI-UHFFFAOYSA-N", "4-chloro-3-methylphenol", "")
  CALL Species("C_7H_7ClO",                            "766-51-8", "QGRPVMLBTFGQDQ-UHFFFAOYSA-N", "1-chloro-2-methoxybenzene", "2-chloroanisole")
  CALL Species("C_7H_7ClO",                           "2845-89-8", "YUKILTJWFRTXGB-UHFFFAOYSA-N", "1-chloro-3-methoxybenzene", "3-chloroanisole")
  CALL Species("C_7H_7ClO",                            "623-12-1", "YRGAYAGBVIXNAQ-UHFFFAOYSA-N", "1-chloro-4-methoxybenzene", "4-chloroanisole")
  CALL Species("C_7H_6Cl_2O",                         "1984-59-4", "HFEASCCDHUVYKU-UHFFFAOYSA-N", "1,2-dichloro-3-methoxybenzene", "2,3-dichloroanisole")
  CALL Species("C_7H_6Cl_2O",                          "553-82-2", "CICQUFBZCADHHX-UHFFFAOYSA-N", "1,5-dichloro-2-methoxybenzene", "2,4-dichloroanisole")
  CALL Species("C_7H_6Cl_2O",                         "1984-58-3", "QKMNFFSBZRGHDJ-UHFFFAOYSA-N", "1,4-dichloro-2-methoxybenzene", "2,5-dichloroanisole")
  CALL Species("C_7H_6Cl_2O",                         "1984-65-2", "KZLMCDNAVVJKPX-UHFFFAOYSA-N", "1,3-dichloro-2-methoxybenzene", "2,6-dichloroanisole")
  CALL Species("C_7H_6Cl_2O",                        "36404-30-5", "VISJRVXHPNMYRH-UHFFFAOYSA-N", "1,2-dichloro-4-methoxybenzene", "3,4-dichloroanisole")
  CALL Species("C_7H_6Cl_2O",                        "33719-74-3", "SSNXYMVLSOMJLU-UHFFFAOYSA-N", "1,3-dichloro-5-methoxybenzene", "3,5-dichloroanisole")
  CALL Species("C_7H_5Cl_3O",                        "54135-80-7", "FRQUNVLMWIYOLV-UHFFFAOYSA-N", "1,2,3-trichloro-4-methoxybenzene", "2,3,4-trichloroanisole")
  CALL Species("C_7H_5Cl_3O",                        "54135-81-8", "MKERQGLKSFEKAE-UHFFFAOYSA-N", "1,2,5-trichloro-3-methoxybenzene", "2,3,5-trichloroanisole")
  CALL Species("C_7H_5Cl_3O",                        "50375-10-5", "OTFNCXLUCRUNCH-UHFFFAOYSA-N", "1,2,4-trichloro-3-methoxybenzene", "2,3,6-trichloroanisole")
  CALL Species("C_7H_5Cl_3O",                         "6130-75-2", "SXKBHOQOOGRFJF-UHFFFAOYSA-N", "1,2,4-trichloro-5-methoxybenzene", "2,4,5-trichloroanisole")
  CALL Species("C_7H_5Cl_3O",                           "87-40-1", "WCVOGSZTONGSQY-UHFFFAOYSA-N", "1,3,5-trichloro-2-methoxybenzene", "2,4,6-trichloroanisole")
  CALL Species("C_7H_5Cl_3O",                        "54135-82-9", "GUCFBWGWRCILHN-UHFFFAOYSA-N", "1,2,3-trichloro-5-methoxybenzene", "3,4,5-trichloroanisole")
  CALL Species("C_7H_4Cl_4O",                          "938-86-3", "FUUHMSUPRUNWRQ-UHFFFAOYSA-N", "1,2,3,4-tetrachloro-5-methoxybenzene", "2,3,4,5-tetrachloroanisole")
  CALL Species("C_7H_4Cl_4O",                          "938-22-7", "ITXDBGLYYSJNPK-UHFFFAOYSA-N", "1,2,3,5-tetrachloro-4-methoxybenzene", "2,3,4,6-tetrachloroanisole")
  CALL Species("C_7H_4Cl_4O",                         "6936-40-9", "WMMFIDNWZNCBCT-UHFFFAOYSA-N", "1,2,4,5-tetrachloro-3-methoxybenzene", "2,3,5,6-tetrachloroanisole")
  CALL Species("C_7H_3Cl_5O",                         "1825-21-4", "BBABSCYTNHOKOG-UHFFFAOYSA-N", "pentachloromethoxybenzene", "pentachloroanisole")
  CALL Species("C_7H_6Cl_2O_2",                       "2460-49-3", "HAAFFTHBNFBVKY-UHFFFAOYSA-N", "4,5-dichloro-2-methoxyphenol", "4,5-dichloroguaiacol")
  CALL Species("C_7H_5Cl_3O_2",                      "57057-83-7", "RKEHLKXRUVUBJN-UHFFFAOYSA-N", "3,4,5-trichloro-2-methoxyphenol", "3,4,5-trichloroguaiacol")
  CALL Species("C_7H_5Cl_3O_2",                       "2668-24-8", "NIAJPNQTKGWEOI-UHFFFAOYSA-N", "4,5,6-trichloro-2-methoxyphenol", "4,5,6-trichloroguaiacol")
  CALL Species("C_7H_4Cl_4O_2",                       "2539-17-5", "YZZVKLJKDFFSFL-UHFFFAOYSA-N", "2,3,4,5-tetrachloro-6-methoxyphenol", "tetrachloroguaiacol")
  CALL Species("C_7H_5ClO_2",                          "118-91-2", "IKCLCGXPQILATA-UHFFFAOYSA-N", "2-chlorobenzoic acid", "")
  CALL Species("C_7H_5ClO_2",                          "535-80-8", "LULAYUGMBFYYEX-UHFFFAOYSA-N", "3-chlorobenzoic acid", "")
  CALL Species("C_7H_5ClO_2",                           "74-11-3", "XRHGYUZYPHTUJZ-UHFFFAOYSA-N", "4-chlorobenzoic acid", "")
  CALL Species("C_7H_4Cl_2O_2",                         "51-44-5", "VPHHJAOJUJHJKD-UHFFFAOYSA-N", "3,4-dichlorobenzoic acid", "")
  CALL Species("C_7H_3Cl_3O_2",                         "50-31-7", "XZIDTOHMJBOSOX-UHFFFAOYSA-N", "2,3,6-trichlorobenzoic acid", "")
  CALL Species("C_8H_7Cl_3O_2",                      "16766-29-3", "VKNITLPENCJQOP-UHFFFAOYSA-N", "1,2,3-trichloro-4,5-dimethoxybenzene", "3,4,5-trichloroveratrole")
  CALL Species("C_8H_6Cl_4O_2",                        "944-61-6", "NCYHCGGUQGDEQW-UHFFFAOYSA-N", "1,2,3,4-tetrachloro-5,6-dimethoxybenzene", "tetrachloroveratrole")
  CALL Species("C_8H_9ClO_3",                        "18113-22-9", "WYEMCZZCZXKDNC-UHFFFAOYSA-N", "3-chloro-2,6-dimethoxyphenol", "3-chlorosyringol")
  CALL Species("C_8H_8Cl_2O_3",                      "78782-46-4", "IDKMFABKPPHDBI-UHFFFAOYSA-N", "3,5-dichloro-2,6-dimethoxyphenol", "3,5-dichlorosyringol")
  CALL Species("C_7H_4Cl_2O_3",                        "320-72-9", "CNJGWCQEGROXEE-UHFFFAOYSA-N", "3,5-dichloro-2-hydroxybenzoic acid", "")
  CALL Species("C_7H_4Cl_2O_3",                       "3401-80-7", "FKIKPQHMWFZFEB-UHFFFAOYSA-N", "3,6-dichloro-2-hydroxybenzoic acid", "")
  CALL Species("C_8H_7ClO_3",                         "3438-16-2", "HULDRQRKKXRXBI-UHFFFAOYSA-N", "5-chloro-2-methoxybenzoic acid", "")
  CALL Species("C_8H_7ClO",                            "532-27-4", "IMACFCSSMIZSPP-UHFFFAOYSA-N", "2-chloroacetophenone", "")
  CALL Species("C_8H_3Cl_5O",                         "1203-86-7", "WKJXVVFAALGBOH-UHFFFAOYSA-N", "2,2,2',4',5'-pentachloroacetophenone", "")
  CALL Species("C_8Cl_6O_2",                           "719-32-4", "YJIRZJAZKDWEIJ-UHFFFAOYSA-N", "tetrachloroterephthaloyl chloride", "")
  CALL Species("C_8H_9ClO",                             "88-04-0", "OSDLLIBGSJNGJE-UHFFFAOYSA-N", "chloroxylenol", "")
  CALL Species("C_8Cl_4O_3",                           "117-08-8", "AUHHYELHRWCWEZ-UHFFFAOYSA-N", "4,5,6,7-tetrachloro-1,3-isobenzofurandione", "")
  CALL Species("C_8H_7Cl_3O_3",                       "2539-26-6", "ZZCSBXFJFLSDRR-UHFFFAOYSA-N", "3,4,5-trichloro-2,6-dimethoxyphenol", "trichlorosyringol")
  CALL Species("C_8H_2Cl_4O_2",                      "27355-22-2", "NMWKWBPNKPGATC-UHFFFAOYSA-N", "4,5,6,7-tetrachlorophthalide", "")
  CALL Species("C_8H_6Cl_2O_3",                       "1918-00-9", "IWEDIXLBFLAXBO-UHFFFAOYSA-N", "dicamba", "banvel")
  CALL Species("C_8H_6Cl_2O_3",                         "94-75-7", "OVSKIKFHRZPJSS-UHFFFAOYSA-N", "(2,4-dichlorophenoxy)-ethanoic acid", "(2,4-dichlorophenoxy)-acetic acid; 2,4-D")
  CALL Species("C_8H_5Cl_3O_3",                         "93-76-5", "SMYMJHWAQXWPDB-UHFFFAOYSA-N", "2,4,5-trichlorophenoxyethanoic acid", "2,4,5-T")
  CALL Species("C_8H_8Cl_2O_2",                       "2675-77-6", "PFIADAMVCJPXSF-UHFFFAOYSA-N", "1,4-dichloro-2,5-dimethoxybenzene", "")
  CALL Species("C_8H_5Cl_3O_2",                         "85-34-7", "QZXCCPZJCKEPSA-UHFFFAOYSA-N", "2,3,6-trichlorophenylacetic acid", "")
  CALL Species("C_8H_7ClO_2",                          "100-07-2", "MXMOTZIXVICDSD-UHFFFAOYSA-N", "4-methoxy-benzoyl chloride", "$p$-anisoyl chloride")
  CALL Species("C_8H_7ClO_3",                          "122-88-3", "SODPIMGUZLOIPE-UHFFFAOYSA-N", "(4-chlorophenoxy)acetic acid", "")
  CALL Species("C_9H_4Cl_8O",                          "297-78-9", "LRWHHSXTGZSMSN-UHFFFAOYSA-N", "isobenzan", "")
  CALL Species("C_9H_9ClO_4",                        "76341-69-0", "GRIHRCLOUQZXPD-UHFFFAOYSA-N", "2-chloro-4-hydroxy-3,5-dimethoxybenzaldehyde", "2-chlorosyringaldehyde")
  CALL Species("C_9H_8Cl_2O_4",                      "76330-06-8", "CTFRWEPMHUGVMM-UHFFFAOYSA-N", "2,6-dichloro-4-hydroxy-3,5-dimethoxybenzaldehyde", "2,6-dichlorosyringaldehyde")
  CALL Species("C_9H_8Cl_2O_3",                       "1928-38-7", "HWIGZMADSFQMOI-UHFFFAOYSA-N", "methyl 2,4-dichlorophenoxyethanoate", "")
  CALL Species("C_9H_9ClO_3",                           "94-74-6", "WHKUVVPPKQRRBV-UHFFFAOYSA-N", "(2-methyl-4-chlorophenoxy)acetic acid", "MCPA")
  CALL Species("C_9H_8Cl_2O_3",                        "120-36-5", "MZHCENGPTKEIGP-UHFFFAOYSA-N", "$\alpha$-(2,4-dichlorophenoxy)propionic acid", "dichloroprop")
  CALL Species("C_9H_8Cl_2O_3",                      "15165-67-0", "MZHCENGPTKEIGP-RXMQYKEDSA-N", "($R$)-2-(2,4-dichlorophenoxy)propanoic acid", "dichlorprop-p")
  CALL Species("C_9H_7Cl_3O_3",                         "93-72-1", "ZLSWBLPERHFHIS-UHFFFAOYSA-N", "2-(2,4,5-trichlorophenoxy)propanoic acid", "")
  CALL Species("C_9H_9O_4Cl",                         "6386-63-6", "ZJRUTGDCLVIVRD-UHFFFAOYSA-N", "cloxyfonac", "")
  CALL Species("C_{10}H_7Cl_5O",                     "58138-08-2", "IBZHOAONZVJLOB-UHFFFAOYSA-N", "tridiphane", "")
  CALL Species("C_{10}H_7O_2Cl_5",                   "21757-82-4", "FSGNOVKGEXRRHD-UHFFFAOYSA-N", "plifenat", "")
  CALL Species("C_{10}H_{10}Cl_2O_2",                "14437-17-3", "YJKIALIXRCSISK-UHFFFAOYSA-N", "chlorfenprop-methyl", "")
  CALL Species("C_{10}H_{10}Cl_2O_3",                  "533-23-3", "JSLBZIVMVVHMDJ-UHFFFAOYSA-N", "ethyl 2,4-dichlorophenoxyethanoate", "")
  CALL Species("C_{10}H_{11}ClO_3",                     "93-65-2", "WNTGYJSOUMFZEP-UHFFFAOYSA-N", "mecoprop", "") ! other CAS: 7085-19-0
  CALL Species("C_{10}H_{11}ClO_3",                  "16484-77-8", "WNTGYJSOUMFZEP-SSDOTTSWSA-N", "($R$)-2-(4-chloro-2-methylphenoxy)propanoic acid", "mecoprop-p")
  CALL Species("C_{10}H_6Cl_4O_4",                    "1861-32-1", "NPOJQCVWMSKXDN-UHFFFAOYSA-N", "dacthal", "DCPA") ! other CAS: 65862-98-8
  CALL Species("C_{10}H_{10}Cl_2O_3",                   "94-82-6", "YIVXMZJTEQBPQO-UHFFFAOYSA-N", "4-(2,4-dichlorophenoxy)-butanoic acid", "2,4-DB")
  CALL Species("C_{10}H_4Cl_2O_2",                     "117-80-6", "SVPKNMBRVBMTLB-UHFFFAOYSA-N", "dichlone", "")
  CALL Species("C_{10}H_{11}Cl_3O_2",                 "1861-44-5", "LJWIIRATRWPHBA-UHFFFAOYSA-N", "1-[(2,3,6-trichlorophenyl)methoxy]propan-2-ol", "tritac")
  CALL Species("C_{11}H_{12}Cl_2O_3",                   "94-11-1", "WHOKDONDRZNCBC-UHFFFAOYSA-N", "(2,4-dichlorophenoxy)-acetic acid 1-methylethyl ester", "")
  CALL Species("C_{11}H_{13}ClO_3",                     "94-81-5", "LLWADFLAOKUBDR-UHFFFAOYSA-N", "4-(4-chloro-2-methylphenoxy)butanoic acid", "MCPB")
  CALL Species("C_{12}H_7Cl_3O_2",                    "3380-34-5", "XEFQLINVKFYRCS-UHFFFAOYSA-N", "triclosan", "")
  CALL Species("C_{12}H_{10}Cl_4O_4",                "24261-19-6", "WKYMTJULVAGWJM-UHFFFAOYSA-N", "monobutyl tetrachlorophthalate", "")
  CALL Species("C_{12}H_{14}Cl_2O_3",                   "94-80-4", "UQMRAFJOBWOFNS-UHFFFAOYSA-N", "(2,4-dichlorophenoxy)-acetic acid butyl ester", "")
  CALL Species("C_{12}H_{19}Cl_3O_8",                "56038-13-2", "BAQAVOSOZGMPRM-QBMZZYIRSA-N", "sucralose", "")
  CALL Species("C_{12}H_8Cl_6O",                      "7421-93-4", "HCTWZIFNBBCVGM-UHFFFAOYSA-N", "endrin aldehyde", "")
  CALL Species("C_{13}H_{11}ClO",                      "120-32-1", "NCKMMSIFQUPKCK-UHFFFAOYSA-N", "clorophene", "4-chloro-2-benzylphenol")
  CALL Species("C_{13}H_9ClO",                         "134-85-0", "UGVRJVHOJNYEHR-UHFFFAOYSA-N", "(4-chlorophenyl)phenylmethanone", "4-chlorobenzophenone")
  CALL Species("C_{13}H_{17}ClO",                    "66346-01-8", "ILQGIJDYSLHIOX-UHFFFAOYSA-N", "1-(4-chlorophenyl)-4,4-dimethyl-3-pentanone", "")
  CALL Species("C_{13}H_6Cl_6O_2",                      "70-30-4", "ACGUYXCXAPNIKK-UHFFFAOYSA-N", "hexachlorophene", "")
  CALL Species("C_{13}H_8Cl_2O",                        "85-29-0", "YXMYPHLWXBXNFF-UHFFFAOYSA-N", "2,4'-dichlorobenzophenone", "")
  CALL Species("C_{13}H_{15}ClO",                     "1577-03-3", "LXJZYHPGRKBVGF-UHFFFAOYSA-N", "1-(4-chlorophenyl)-4,4-dimethylpent-1-en-3-one", "")
  CALL Species("C_{13}H_{10}Cl_2O_2",                   "97-23-4", "MDNWOSOZYLHTCG-UHFFFAOYSA-N", "dichlorophen", "")
  CALL Species("C_{13}H_{17}ClO_3",                  "10443-70-6", "XNKARWLGLZGMGX-UHFFFAOYSA-N", "MCPB ethyl ester", "")
  CALL Species("C_{14}H_7ClO_2",                       "131-09-9", "FPKCTSIVDAWGFA-UHFFFAOYSA-N", "2-chloro-9,10-anthracenedione", "")
  CALL Species("C_{14}H_9Cl_5O",                       "115-32-2", "UOAMTSKGCBMZTC-UHFFFAOYSA-N", "dicofol", "")
  CALL Species("C_{14}H_6Cl_4O_4",                     "133-14-2", "WRXCBRHBHGNNQA-UHFFFAOYSA-N", "bis(2,4-dichlorobenzoyl)peroxide", "")
  CALL Species("C_{14}H_{14}Cl_4O_4",                 "6928-67-2", "QJRSPJSHRYYBJL-UHFFFAOYSA-N", "dipropyl tetrachlorophthalate", "")
  CALL Species("C_{14}H_9ClO_3",                        "85-56-3", "YWECCEXWKFHHQJ-UHFFFAOYSA-N", "2-(4-chlorobenzoyl)benzoic acid", "")
  CALL Species("C_{14}H_{17}Cl_3O_4",                 "2545-59-7", "GLDWASBMYWLQGG-UHFFFAOYSA-N", "(2,4,5-trichlorophenoxy)acetic acid butoxyethanol ester", "")
  CALL Species("C_{14}H_{18}Cl_2O_4",                 "1929-73-3", "ZMWGIGHRZQTQRE-UHFFFAOYSA-N", "(2,4-dichlorophenoxy)-acetic acid, 2-butoxyethyl ester", "")
  CALL Species("C_{14}H_7ClO_2",                        "82-44-0", "BOCJQSFSGAZAPQ-UHFFFAOYSA-N", "1-chloro-9,10-anthracenedione", "1-chloroanthraquinone")
  CALL Species("C_{15}H_{11}ClO_3",                   "2536-31-4", "LINPVWIEWJTEEJ-UHFFFAOYSA-N", "chlorflurenol methyl", "")
  CALL Species("C_{15}H_{12}Cl_4O_2",                   "79-95-8", "KYPYTERUKNKOLP-UHFFFAOYSA-N", "4,4'-(1-methylethylidene)bis(2,6-dichlorophenol)", "2,2',6,6'-tetrachlorobisphenol A")
  CALL Species("C_{15}H_{12}Cl_2O_4",                "40843-25-2", "OOLBCHYXZDXLDS-UHFFFAOYSA-N", "diclofop", "")
  CALL Species("C_{16}H_{15}Cl_3O_2",                   "72-43-5", "IAKOZHOLGAGEJT-UHFFFAOYSA-N", "methoxychlor", "")
  CALL Species("C_{16}H_{14}Cl_2O_4",                "51338-27-3", "BACHBFVBHLGWSL-UHFFFAOYSA-N", "diclofop-methyl", "")
  CALL Species("C_{16}H_{14}Cl_2O_3",                  "510-15-6", "RAPBNVDSDCTNRC-UHFFFAOYSA-N", "chlorobenzilate", "")
  CALL Species("C_{16}H_{22}Cl_2O_3",                 "1928-43-4", "QZSFJRIWRPJUOH-UHFFFAOYSA-N", "(2,4-dichlorophenoxy)-acetic acid 2-ethylhexyl ester", "")
  CALL Species("C_{16}H_{22}Cl_2O_3",                "25168-26-7", "BBPLSOGERZQYQC-UHFFFAOYSA-N", "(2,4-dichlorophenoxy)-acetic acid, isooctyl ester", "")
  CALL Species("C_{16}H_{23}ClO_4",                  "23359-62-8", "GWFGUAFFJVHZKY-UHFFFAOYSA-N", "mecoprop-2-butoxyethyl ester", "") ! other CAS: 97659-39-7
  CALL Species("C_{17}H_{16}Cl_2O_3",                 "5836-10-2", "AXGUBXVWZBFQGA-UHFFFAOYSA-N", "chloropropylate", "")
  CALL Species("C_{17}H_{25}ClO_3",                  "29450-45-1", "IDGRPSMONFWWEK-UHFFFAOYSA-N", "MCPA-2-ethylhexyl ester", "")
  CALL Species("C_{18}H_{29}ClO_2",                  "65925-28-2", "FITQCDWGUKECBJ-UHFFFAOYSA-N", "1-(2-(2-chloroethoxy)ethoxy)-4-(1,1,3,3-tetramethylbutyl)benzene", "")
  CALL Species("C_{20}H_{21}ClO_4",                  "49562-28-9", "YMTINGFKWWXKFG-UHFFFAOYSA-N", "fenofibrate", "")
  CALL Species("C_{20}H_{17}O_3Cl",                 "133220-30-1", "PMAAYIYCDXGUAP-UHFFFAOYSA-N", "indanofan", "")
  CALL Species("C_{21}H_{24}Cl_2O_4",               "148477-71-8", "DTDSAWVUFPGDMX-UHFFFAOYSA-N", "spirodiclofen", "")
  CALL Species("C_{21}H_{20}Cl_2O_3",                "52645-53-1", "RLLPVAHGXHCWKJ-UHFFFAOYSA-N", "permethrin", "")
  CALL Species("C_{23}H_{15}ClO_3",                   "3691-35-8", "UDHXJZHVNHGCEC-UHFFFAOYSA-N", "chlorophacinone", "")
  CALL Species("C_{23}H_{29}ClO_4",                    "302-22-7", "QMBJSIBWORFWQT-UHFFFAOYSA-N", "chlormadinone acetate", "")
  CALL Species("C_{24}H_{34}Cl_4O_4",                "34832-88-7", "BELGUQVGZMFEAQ-UHFFFAOYSA-N", "3,4,5,6-tetrachlorophthalic acid bis(2-ethylhexyl) ester", "")
  ! polycyclic:
  CALL Species("C_9H_8Cl_6O_2",                       "2157-19-9", "GTSJHTSVFKEASK-UHFFFAOYSA-N", "endosulfan alcohol", "")
  CALL Species("C_9H_2Cl_6O_3",                        "115-27-5", "FLBJFXNAEMSXGL-UHFFFAOYSA-N", "chlorendic anhydride", "")
  CALL Species("C_9H_4Cl_6O_4",                        "115-28-6", "DJKGDNKYTKCJKD-UHFFFAOYSA-N", "1,4,5,6,7,7-hexachloro-bicyclo[2.2.1]hept-5-ene-2,3-dicarboxylic acid", "")
  CALL Species("C_{10}H_5Cl_7O",                      "1024-57-3", "ZXFXBSWRVIQKOD-UHFFFAOYSA-N", "heptachlorepoxide", "") ! other CAS: 66240-71-9
  CALL Species("C_{12}H_8OCl_6",                        "60-57-1", "DFBKLUNHFCTMDC-NLUYNBKHSA-N", "dieldrin", "")
  CALL Species("C_{12}H_8Cl_6O",                        "72-20-8", "DFBKLUNHFCTMDC-GKRDHZSOSA-N", "endrin", "")
  CALL Species("C_{17}H_{20}Cl_6O_4",                 "1770-80-5", "UJAHPBDUQZFDLA-UHFFFAOYSA-N", "1,4,5,6,7,7-hexachlorobicyclo[2.2.1]hept-5-ene-2,3-dicarboxylic acid, dibutyl ester", "")
  CALL Species("C_{25}H_{36}Cl_6O_4",                 "4827-55-8", "ONIHOIFWLALAQH-UHFFFAOYSA-N", "di-2-ethylhexyl chlorendate", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Polychlorinated diphenyl ethers (PCDEs)")
  ! if the casrn is unknown, the PCDE number with an underscore prefix is used:
  CALL Species("C_{12}H_9ClO",                        "2689-07-8", "IPBRZLMGGXHHMS-UHFFFAOYSA-N", "2-chlorodiphenyl ether", "PCDE-1")
  CALL Species("C_{12}H_9ClO",                        "6452-49-9", "BMURONZFJJPAOK-UHFFFAOYSA-N", "3-chlorodiphenyl ether", "PCDE-2")
  CALL Species("C_{12}H_9ClO",                        "7005-72-3", "PGPNJCAMHOJTEF-UHFFFAOYSA-N", "4-chlorodiphenyl ether", "PCDE-3")
 !CALL Species("C_{12}H_8Cl_2O",                      "7024-98-8", "ZBCGKBWJNUKWNL-UHFFFAOYSA-N", "2,2'-dichlorodiphenyl ether", "PCDE-4")
  CALL Species("C_{12}H_8Cl_2O",                     "68486-28-2", "VSKSUBSGORDMQX-UHFFFAOYSA-N", "2,3-dichlorodiphenyl ether", "PCDE-5")
 !CALL Species("C_{12}H_8Cl_2O",                      "7024-99-9", "NWRJCFQSGUPLID-UHFFFAOYSA-N", "2,3'-dichlorodiphenyl ether", "PCDE-6")
  CALL Species("C_{12}H_8Cl_2O",                     "51892-26-3", "KXIPYLZZJZMMPD-UHFFFAOYSA-N", "2,4-dichlorodiphenyl ether", "PCDE-7")
  CALL Species("C_{12}H_8Cl_2O",                      "6903-65-7", "MWKULKMSBBSGTP-UHFFFAOYSA-N", "2,4'-dichlorodiphenyl ether", "PCDE-8")
  CALL Species("C_{12}H_8Cl_2O",                     "24910-69-8", "VITXVDNQHXYQSK-UHFFFAOYSA-N", "2,5-dichlorodiphenyl ether", "PCDE-9")
  CALL Species("C_{12}H_8Cl_2O",                     "28419-69-4", "IRLZOQDGEAEIPX-UHFFFAOYSA-N", "2,6-dichlorodiphenyl ether", "PCDE-10")
 !CALL Species("C_{12}H_8Cl_2O",                      "6903-62-4", "RQMZSXZFMWCEET-UHFFFAOYSA-N", "3,3'-dichlorodiphenyl ether", "PCDE-11")
  CALL Species("C_{12}H_8Cl_2O",                     "55538-69-7", "QFQLZVAAQOJUFS-UHFFFAOYSA-N", "3,4-dichlorodiphenyl ether", "PCDE-12")
  CALL Species("C_{12}H_8Cl_2O",                      "6842-62-2", "HPRGYUWRGCTBAV-UHFFFAOYSA-N", "3,4'-dichlorodiphenyl ether", "PCDE-13") ! other CAS: 51289-10-2
  CALL Species("C_{12}H_8Cl_2O",                     "24910-68-7", "LEKOWSSKXYFERR-UHFFFAOYSA-N", "3,5-dichlorodiphenyl ether", "PCDE-14")
  CALL Species("C_{12}H_8Cl_2O",                      "2444-89-5", "URUJZHZLCCIILC-UHFFFAOYSA-N", "4,4'-dichlorodiphenyl ether", "PCDE-15")
 !CALL Species("C_{12}H_7Cl_3O",                    "727738-38-7", "XFQQAPMKSRDRNO-UHFFFAOYSA-N", "2,2',3-trichlorodiphenyl ether", "PCDE-16")
  CALL Species("C_{12}H_7Cl_3O",                     "68914-97-6", "YXMNUPKWUMIZAV-UHFFFAOYSA-N", "2,2',4-trichlorodiphenyl ether", "PCDE-17")
 !CALL Species("C_{12}H_7Cl_3O",                    "727738-42-3", "NTJXMYMAAKIBAL-UHFFFAOYSA-N", "2,2',5-trichlorodiphenyl ether", "PCDE-18")
 !CALL Species("C_{12}H_7Cl_3O",                    "727738-44-5", "CQVFBUDIKATBSM-UHFFFAOYSA-N", "2,2',6-trichlorodiphenyl ether", "PCDE-19")
 !CALL Species("C_{12}H_7Cl_3O",                    "727738-39-8", "WRBFXFUXFGLONT-UHFFFAOYSA-N", "2,3,3'-trichlorodiphenyl ether", "PCDE-20")
  CALL Species("C_{12}H_7Cl_3O",                     "85918-32-7", "ANKBTLMYMVMWBS-UHFFFAOYSA-N", "2,3,4-trichlorodiphenyl ether", "PCDE-21")
  CALL Species("C_{12}H_7Cl_3O",                    "157683-71-1", "KOTNFWJIMRGVBR-UHFFFAOYSA-N", "2,3,4'-trichlorodiphenyl ether", "PCDE-22")
  CALL Species("C_{12}H_7Cl_3O",                    "162853-24-9", "MDSPKCWGOVVFJD-UHFFFAOYSA-N", "2,3,5-trichlorodiphenyl ether", "PCDE-23")
  CALL Species("C_{12}H_7Cl_3O",                    "162853-25-0", "RQSRPDGSUDRYLH-UHFFFAOYSA-N", "2,3,6-trichlorodiphenyl ether", "PCDE-24")
  CALL Species("C_{12}H_7Cl_3O",                    "155999-93-2", "BJFOCMRBCBZFJT-UHFFFAOYSA-N", "2,3',4-trichlorodiphenyl ether", "PCDE-25")
 !CALL Species("C_{12}H_7Cl_3O",                    "727738-43-4", "FTGPHPONDJREBV-UHFFFAOYSA-N", "2,3',5-trichlorodiphenyl ether", "PCDE-26")
 !CALL Species("C_{12}H_7Cl_3O",                    "727738-45-6", "CSKOAUQGNHMPBS-UHFFFAOYSA-N", "2,3',6-trichlorodiphenyl ether", "PCDE-27")
  CALL Species("C_{12}H_7Cl_3O",                     "59039-21-3", "PIORTDHJOLELKR-UHFFFAOYSA-N", "2,4,4'-trichlorodiphenyl ether", "PCDE-28")
  CALL Species("C_{12}H_7Cl_3O",                     "52322-80-2", "UWKZWXCTDPYXHU-UHFFFAOYSA-N", "2,4,5-trichlorodiphenyl ether", "PCDE-29")
  CALL Species("C_{12}H_7Cl_3O",                     "63646-52-6", "WXLQUFLKMICSSY-UHFFFAOYSA-N", "2,4,6-trichlorodiphenyl ether", "PCDE-30")
  CALL Species("C_{12}H_7Cl_3O",                     "65075-00-5", "FZBSTAVCYOMFMD-UHFFFAOYSA-N", "2,4',5-trichlorodiphenyl ether", "PCDE-31")
  CALL Species("C_{12}H_7Cl_3O",                    "157683-72-2", "ZMRFCSWFCKICQE-UHFFFAOYSA-N", "2,4',6-trichlorodiphenyl ether", "PCDE-32")
  CALL Species("C_{12}H_7Cl_3O",                     "61328-44-7", "VBNJGYOQZAENPO-UHFFFAOYSA-N", "2,3',4'-trichlorodiphenyl ether", "PCDE-33")
 !CALL Species("C_{12}H_7Cl_3O",                    "727738-40-1", "WNTKDANBGHJDLL-UHFFFAOYSA-N", "2,3',5'-trichlorodiphenyl ether", "PCDE-34")
  CALL Species("C_{12}H_7Cl_3O",                     "66794-60-3", "BUKLLCIUMZELOH-UHFFFAOYSA-N", "3,3',4-trichlorodiphenyl ether", "PCDE-35")
 !CALL Species("C_{12}H_7Cl_3O",                    "727738-41-2", "NIYPWVDRZJTBLM-UHFFFAOYSA-N", "3,3',5-trichlorodiphenyl ether", "PCDE-36")
  CALL Species("C_{12}H_7Cl_3O",                     "63646-51-5", "FTZDEZOARSJHGU-UHFFFAOYSA-N", "3,4,4'-trichlorodiphenyl ether", "PCDE-37")
  CALL Species("C_{12}H_7Cl_3O",                     "63646-53-7", "ZQCCHNBMSHOJAT-UHFFFAOYSA-N", "3,4,5-trichlorodiphenyl ether", "PCDE-38")
  CALL Species("C_{12}H_7Cl_3O",                     "24910-73-4", "CCEZFZGXWFJQEF-UHFFFAOYSA-N", "3,4',5-trichlorodiphenyl ether", "PCDE-39")
 !CALL Species("C_{12}H_6Cl_4O",                    "727738-46-7", "WXEOWWVZWWGBBQ-UHFFFAOYSA-N", "2,2',3,3'-tetrachlorodiphenyl ether", "PCDE-40")
  CALL Species("C_{12}H_6Cl_4O",                    "220002-37-9", "VTTDWGUAWMLHPR-UHFFFAOYSA-N", "2,2',3,4-tetrachlorodiphenyl ether", "PCDE-41")
  CALL Species("C_{12}H_6Cl_4O",                    "147102-63-4", "UYVBJPIPRAJEKW-UHFFFAOYSA-N", "2,2',3,4'-tetrachlorodiphenyl ether", "PCDE-42")
 !CALL Species("C_{12}H_6Cl_4O",                    "727738-50-3", "RTDOXTNDEIQMSD-UHFFFAOYSA-N", "2,2',3,5-tetrachlorodiphenyl ether", "PCDE-43")
 !CALL Species("C_{12}H_6Cl_4O",                    "727738-47-8", "WTZJRRWYCWWOOF-UHFFFAOYSA-N", "2,2',3,5'-tetrachlorodiphenyl ether", "PCDE-44")
 !CALL Species("C_{12}H_6Cl_4O",                    "727738-52-5", "FKCWLBPOXAHCEZ-UHFFFAOYSA-N", "2,2',3,6-tetrachlorodiphenyl ether", "PCDE-45")
 !CALL Species("C_{12}H_6Cl_4O",                    "727738-48-9", "KDCKWINSPKLQNE-UHFFFAOYSA-N", "2,2',3,6'-tetrachlorodiphenyl ether", "PCDE-46")
  CALL Species("C_{12}H_6Cl_4O",                     "28076-73-5", "ZRWRPGGXCSSBAO-UHFFFAOYSA-N", "2,2',4,4'-tetrachlorodiphenyl ether", "PCDE-47")
  CALL Species("C_{12}H_6Cl_4O",                    "162853-26-1", "DRVAMLZVBNYSQE-UHFFFAOYSA-N", "2,2',4,5-tetrachlorodiphenyl ether", "PCDE-48")
  CALL Species("C_{12}H_6Cl_4O",                    "155999-92-1", "XIXPEFLLHQAIOY-UHFFFAOYSA-N", "2,2',4,5'-tetrachlorodiphenyl ether", "PCDE-49")
 !CALL Species("C_{12}H_6Cl_4O",                    "727738-54-7", "KTWRMXMTWFBDRJ-UHFFFAOYSA-N", "2,2',4,6-tetrachlorodiphenyl ether", "PCDE-50")
 !CALL Species("C_{12}H_6Cl_4O",                    "147102-65-6", "DDQTXMCKLUKGCX-UHFFFAOYSA-N", "2,2',4,6'-tetrachlorodiphenyl ether", "PCDE-51")
 !CALL Species("C_{12}H_6Cl_4O",                    "727738-56-9", "YWBKRNRSWKEARX-UHFFFAOYSA-N", "2,2',5,5'-tetrachlorodiphenyl ether", "PCDE-52")
 !CALL Species("C_{12}H_6Cl_4O",                    "727738-57-0", "PTDYNPOYHAVZAN-UHFFFAOYSA-N", "2,2',5,6'-tetrachlorodiphenyl ether", "PCDE-53")
 !CALL Species("C_{12}H_6Cl_4O",                     "91371-42-5", "QGEKQTVINSUSJJ-UHFFFAOYSA-N", "2,2',6,6'-tetrachlorodiphenyl ether", "PCDE-54")
  CALL Species("C_{12}H_6Cl_4O",                    "220002-39-1", "IZISEKJPNSBPRX-UHFFFAOYSA-N", "2,3,3',4-tetrachlorodiphenyl ether", "PCDE-55")
  CALL Species("C_{12}H_6Cl_4O",                    "162853-27-2", "JVFGXWCLGNACOG-UHFFFAOYSA-N", "2,3,3',4'-tetrachlorodiphenyl ether", "PCDE-56")
 !CALL Species("C_{12}H_6Cl_4O",                    "727738-51-4", "WDEDSACPPIUBIM-UHFFFAOYSA-N", "2,3,3',5-tetrachlorodiphenyl ether", "PCDE-57")
 !CALL Species("C_{12}H_6Cl_4O",                    "727738-49-0", "PSENAJOMGBHPFL-UHFFFAOYSA-N", "2,3,3',5'-tetrachlorodiphenyl ether", "PCDE-58")
 !CALL Species("C_{12}H_6Cl_4O",                    "727738-53-6", "XBAUODQBIVRIOY-UHFFFAOYSA-N", "2,3,3',6-tetrachlorodiphenyl ether", "PCDE-59")
  CALL Species("C_{12}H_6Cl_4O",                     "65075-01-6", "ZPOBHUZRVVTGKB-UHFFFAOYSA-N", "2,3,4,4'-tetrachlorodiphenyl ether", "PCDE-60")
  CALL Species("C_{12}H_6Cl_4O",                    "220002-40-4", "QAPPZLVGIXQUJW-UHFFFAOYSA-N", "2,3,4,5-tetrachlorodiphenyl ether", "PCDE-61")
  CALL Species("C_{12}H_6Cl_4O",                     "85918-33-8", "WJZPRRFFZWVZJL-UHFFFAOYSA-N", "2,3,4,6-tetrachlorodiphenyl ether", "PCDE-62")
  CALL Species("C_{12}H_6Cl_4O",                    "220002-41-5", "YHQRYZGDQCSHFO-UHFFFAOYSA-N", "2,3,4',5-tetrachlorodiphenyl ether", "PCDE-63")
  CALL Species("C_{12}H_6Cl_4O",                    "220002-42-6", "HJCNTJIMUWQMSA-UHFFFAOYSA-N", "2,3,4',6-tetrachlorodiphenyl ether", "PCDE-64")
  CALL Species("C_{12}H_6Cl_4O",                     "63646-54-8", "UFGXCDQXRUFDMA-UHFFFAOYSA-N", "2,3,5,6-tetrachlorodiphenyl ether", "PCDE-65")
  CALL Species("C_{12}H_6Cl_4O",                     "61328-46-9", "CAWSVYYOUZEMHA-UHFFFAOYSA-N", "2,3',4,4'-tetrachlorodiphenyl ether", "PCDE-66")
  CALL Species("C_{12}H_6Cl_4O",                    "152833-52-8", "QDQGECFQVPRMEO-UHFFFAOYSA-N", "2,3',4,5-tetrachlorodiphenyl ether", "PCDE-67")
  CALL Species("C_{12}H_6Cl_4O",                    "147102-64-5", "XORVNHCKWFPDLN-UHFFFAOYSA-N", "2,3',4,5'-tetrachlorodiphenyl ether", "PCDE-68")
 !CALL Species("C_{12}H_6Cl_4O",                    "727738-55-8", "AVSRXQFGVHEINV-UHFFFAOYSA-N", "2,3',4,6-tetrachlorodiphenyl ether", "PCDE-69")
  CALL Species("C_{12}H_6Cl_4O",                    "159553-67-0", "MFTPXFIGIJCNJM-UHFFFAOYSA-N", "2,3',4',5-tetrachlorodiphenyl ether", "PCDE-70")
  CALL Species("C_{12}H_6Cl_4O",                    "130892-66-9", "IZZULMOBRTYHMH-UHFFFAOYSA-N", "2,3',4',6-tetrachlorodiphenyl ether", "PCDE-71")
 !CALL Species("C_{12}H_6Cl_4O",                    "727738-58-1", "AHMHHSWRNYLGBD-UHFFFAOYSA-N", "2,3',5,5'-tetrachlorodiphenyl ether", "PCDE-72")
 !CALL Species("C_{12}H_6Cl_4O",                    "727738-59-2", "GMYOHLGVVRWHJK-UHFFFAOYSA-N", "2,3',5',6-tetrachlorodiphenyl ether", "PCDE-73")
  CALL Species("C_{12}H_6Cl_4O",                     "61328-45-8", "KXOJXWNARBQHNX-UHFFFAOYSA-N", "2,4,4',5-tetrachlorodiphenyl ether", "PCDE-74")
  CALL Species("C_{12}H_6Cl_4O",                     "63553-30-0", "YCWRSOTXXJYCKF-UHFFFAOYSA-N", "2,4,4',6-tetrachlorodiphenyl ether", "PCDE-75")
 !CALL Species("C_{12}H_6Cl_4O",                    "727738-60-5", "DLMAJSUWVRZPHT-UHFFFAOYSA-N", "2,3',4',5'-tetrachlorodiphenyl ether", "PCDE-76")
  CALL Species("C_{12}H_6Cl_4O",                     "56348-72-2", "DHLVZXZRIZBPKG-UHFFFAOYSA-N", "3,3',4,4'-tetrachlorodiphenyl ether", "PCDE-77")
 !CALL Species("C_{12}H_6Cl_4O",                    "727738-61-6", "WOLCBXVZYPVADU-UHFFFAOYSA-N", "3,3',4,5-tetrachlorodiphenyl ether", "PCDE-78")
  CALL Species("C_{12}H_6Cl_4O",                    "552884-22-7", "BUPRSRBWDJVKGG-UHFFFAOYSA-N", "3,3',4,5'-tetrachlorodiphenyl ether", "PCDE-79")
 !CALL Species("C_{12}H_6Cl_4O",                     "85918-34-9", "MOQYEWLNKZQUBJ-UHFFFAOYSA-N", "3,3',5,5'-tetrachlorodiphenyl ether", "PCDE-80")
  CALL Species("C_{12}H_6Cl_4O",                     "62615-07-0", "CAUBBRCOHHHCNG-UHFFFAOYSA-N", "3,4,4',5-tetrachlorodiphenyl ether", "PCDE-81")
  CALL Species("C_{12}H_5Cl_5O",                    "160282-10-0", "ABRDPORILGADAP-UHFFFAOYSA-N", "2,2',3,3',4-pentachlorodiphenyl ether", "PCDE-82")
 !CALL Species("C_{12}H_5Cl_5O",                    "727738-65-0", "PFQONVKWGPWBIJ-UHFFFAOYSA-N", "2,2',3,3',5-pentachlorodiphenyl ether", "PCDE-83")
 !CALL Species("C_{12}H_5Cl_5O",                    "727738-71-8", "PGFVVFUTVNUGGF-UHFFFAOYSA-N", "2,2',3,3',6-pentachlorodiphenyl ether", "PCDE-84")
  CALL Species("C_{12}H_5Cl_5O",                     "71585-37-0", "RSBUDFTVQYJNHK-UHFFFAOYSA-N", "2,2',3,4,4'-pentachlorodiphenyl ether", "PCDE-85")
 !CALL Species("C_{12}H_5Cl_5O",                    "727738-62-7", "WKCMCEFIDNESSP-UHFFFAOYSA-N", "2,2',3,4,5-pentachlorodiphenyl ether", "PCDE-86")
  CALL Species("C_{12}H_5Cl_5O",                    "160282-09-7", "SCLCXTQIVCULDR-UHFFFAOYSA-N", "2,2',3,4,5'-pentachlorodiphenyl ether", "PCDE-87")
 !CALL Species("C_{12}H_5Cl_5O",                    "182346-19-6", "VMHYACXCJLMWFU-UHFFFAOYSA-N", "2,2',3,4,6-pentachlorodiphenyl ether", "PCDE-88")
  CALL Species("C_{12}H_5Cl_5O",                     "85918-35-0", "JGXAONSRHOSPTJ-UHFFFAOYSA-N", "2,2',3,4,6'-pentachlorodiphenyl ether", "PCDE-89")
  CALL Species("C_{12}H_5Cl_5O",                    "157683-73-3", "QPEBRZQSEUOWPA-UHFFFAOYSA-N", "2,2',3,4',5-pentachlorodiphenyl ether", "PCDE-90")
  CALL Species("C_{12}H_5Cl_5O",                    "116995-20-1", "BRNKJDZWZYZVNM-UHFFFAOYSA-N", "2,2',3,4',6-pentachlorodiphenyl ether", "PCDE-91")
 !CALL Species("C_{12}H_5Cl_5O",                    "727738-66-1", "HFNDNQXOPGBGAM-UHFFFAOYSA-N", "2,2',3,5,5'-pentachlorodiphenyl ether", "PCDE-92")
 !CALL Species("C_{12}H_5Cl_5O",                    "727738-69-4", "MGQCBCDXLWFWEN-UHFFFAOYSA-N", "2,2',3,5,6-pentachlorodiphenyl ether", "PCDE-93")
 !CALL Species("C_{12}H_5Cl_5O",                    "727738-67-2", "JOWOWBQKVYRISB-UHFFFAOYSA-N", "2,2',3,5,6'-pentachlorodiphenyl ether", "PCDE-94")
 !CALL Species("C_{12}H_5Cl_5O",                    "727738-72-9", "WLRLUWBUJXOZNG-UHFFFAOYSA-N", "2,2',3,5',6-pentachlorodiphenyl ether", "PCDE-95")
 !CALL Species("C_{12}H_5Cl_5O",                    "727738-73-0", "YSQZLJVFAAGJPM-UHFFFAOYSA-N", "2,2',3,6,6'-pentachlorodiphenyl ether", "PCDE-96")
  CALL Species("C_{12}H_5Cl_5O",                    "160282-08-6", "ABYZYVAIOYICJQ-UHFFFAOYSA-N", "2,2',3,4',5'-pentachlorodiphenyl ether", "PCDE-97")
 !CALL Species("C_{12}H_5Cl_5O",                    "727738-75-2", "CMSKHCOLRHTFBZ-UHFFFAOYSA-N", "2,2',3,4',6'-pentachlorodiphenyl ether", "PCDE-98")
  CALL Species("C_{12}H_5Cl_5O",                     "60123-64-0", "AVURWLKSFZBANQ-UHFFFAOYSA-N", "2,2',4,4',5-pentachlorodiphenyl ether", "PCDE-99")
  CALL Species("C_{12}H_5Cl_5O",                    "104294-16-8", "FONWDRSQXQZNBN-UHFFFAOYSA-N", "2,2',4,4',6-pentachlorodiphenyl ether", "PCDE-100")
  CALL Species("C_{12}H_5Cl_5O",                    "131138-21-1", "MKIAKZAWONVRFF-UHFFFAOYSA-N", "2,2',4,5,5'-pentachlorodiphenyl ether", "PCDE-101")
  CALL Species("C_{12}H_5Cl_5O",                    "130892-67-0", "BOJBTUWEQMUWHT-UHFFFAOYSA-N", "2,2',4,5,6'-pentachlorodiphenyl ether", "PCDE-102")
 !CALL Species("C_{12}H_5Cl_5O",                    "727738-76-3", "XNEDCRGDWSPQTR-UHFFFAOYSA-N", "2,2',4,5',6-pentachlorodiphenyl ether", "PCDE-103")
 !CALL Species("C_{12}H_5Cl_5O",                    "727738-77-4", "BTEUFGHCTDYVPC-UHFFFAOYSA-N", "2,2',4,6,6'-pentachlorodiphenyl ether", "PCDE-104")
  CALL Species("C_{12}H_5Cl_5O",                     "85918-31-6", "PKEGIXYNAGGYPN-UHFFFAOYSA-N", "2,3,3',4,4'-pentachlorodiphenyl ether", "PCDE-105")
 !CALL Species("C_{12}H_5Cl_5O",                    "727738-63-8", "DLDSGDIHZMLFNO-UHFFFAOYSA-N", "2,3,3',4,5-pentachlorodiphenyl ether", "PCDE-106")
 !CALL Species("C_{12}H_5Cl_5O",                    "159553-68-1", "ZVFHTYGIYIXYOJ-UHFFFAOYSA-N", "2,3,3',4',5-pentachlorodiphenyl ether", "PCDE-107")
  CALL Species("C_{12}H_5Cl_5O",                    "160282-07-5", "HCTKORQPUZCIAO-UHFFFAOYSA-N", "2,3,3',4,5'-pentachlorodiphenyl ether", "PCDE-108")
  CALL Species("C_{12}H_5Cl_5O",                    "727738-64-9", "SEIXDCGLHKOBEO-UHFFFAOYSA-N", "2,3,3',4,6-pentachlorodiphenyl ether", "PCDE-109")
  CALL Species("C_{12}H_5Cl_5O",                    "159553-69-2", "KASGFDPQAMDPAZ-UHFFFAOYSA-N", "2,3,3',4',6-pentachlorodiphenyl ether", "PCDE-110")
 !CALL Species("C_{12}H_5Cl_5O",                    "727738-68-3", "YJVQHOLZYOYYTR-UHFFFAOYSA-N", "2,3,3',5,5'-pentachlorodiphenyl ether", "PCDE-111")
 !CALL Species("C_{12}H_5Cl_5O",                    "727738-70-7", "NDUYWFKVCDCFGA-UHFFFAOYSA-N", "2,3,3',5,6-pentachlorodiphenyl ether", "PCDE-112")
 !CALL Species("C_{12}H_5Cl_5O",                    "727738-74-1", "XNJNBZDXPSVWSJ-UHFFFAOYSA-N", "2,3,3',5',6-pentachlorodiphenyl ether", "PCDE-113")
  CALL Species("C_{12}H_5Cl_5O",                    "113464-17-8", "WSTVMACYLXUQIM-UHFFFAOYSA-N", "2,3,4,4',5-pentachlorodiphenyl ether", "PCDE-114")
  CALL Species("C_{12}H_5Cl_5O",                    "160282-05-3", "LUNNFBBTEHRYMW-UHFFFAOYSA-N", "2,3,4,4',6-pentachlorodiphenyl ether", "PCDE-115")
  CALL Species("C_{12}H_5Cl_5O",                     "22274-42-6", "JDWOFUWJURZFFF-UHFFFAOYSA-N", "2,3,4,5,6-pentachlorodiphenyl ether", "PCDE-116")
  CALL Species("C_{12}H_5Cl_5O",                     "63646-55-9", "UETIKWAWMZLQQZ-UHFFFAOYSA-N", "2,3,4',5,6-pentachlorodiphenyl ether", "PCDE-117")
  CALL Species("C_{12}H_5Cl_5O",                     "60123-65-1", "NRAVUGAGYOBXIG-UHFFFAOYSA-N", "2,3',4,4',5-pentachlorodiphenyl ether", "PCDE-118")
  CALL Species("C_{12}H_5Cl_5O",                    "157683-74-4", "POOXVKSDWCXIBU-UHFFFAOYSA-N", "2,3',4,4',6-pentachlorodiphenyl ether", "PCDE-119")
  CALL Species("C_{12}H_5Cl_5O",                    "160282-04-2", "XIFRTJZLSYOLIU-UHFFFAOYSA-N", "2,3',4,5,5'-pentachlorodiphenyl ether", "PCDE-120")
 !CALL Species("C_{12}H_5Cl_5O",                     "76621-13-1", "QZWNZTBNKKZIBR-UHFFFAOYSA-N", "2,3',4,5',6-pentachlorodiphenyl ether", "PCDE-121")
 !CALL Species("C_{12}H_5Cl_5O",                    "727738-78-5", "WXFDCSJOOWTDKN-UHFFFAOYSA-N", "2,3,3',4',5'-pentachlorodiphenyl ether", "PCDE-122")
  CALL Species("C_{12}H_5Cl_5O",                    "160282-06-4", "BUAXMSGDEZZOJD-UHFFFAOYSA-N", "2,3',4,4',5'-pentachlorodiphenyl ether", "PCDE-123")
 !CALL Species("C_{12}H_5Cl_5O",                    "727738-79-6", "KUGBGRTUMUNYBS-UHFFFAOYSA-N", "2,3',4',5,5'-pentachlorodiphenyl ether", "PCDE-124")
 !CALL Species("C_{12}H_5Cl_5O",                    "727738-80-9", "NHSFXYSHJBBWMZ-UHFFFAOYSA-N", "2,3',4',5',6-pentachlorodiphenyl ether", "PCDE-125")
  CALL Species("C_{12}H_5Cl_5O",                     "94339-59-0", "WDBLKMRZRGLISL-UHFFFAOYSA-N", "3,3',4,4',5-pentachlorodiphenyl ether", "PCDE-126")
 !CALL Species("C_{12}H_5Cl_5O",                    "220002-43-7", "UNFZSLPZWOFSAL-UHFFFAOYSA-N", "3,3',4,5,5'-pentachlorodiphenyl ether", "PCDE-127")
  CALL Species("C_{12}H_4Cl_6O",                     "71585-39-2", "GSWMXIIUDJOXNF-UHFFFAOYSA-N", "2,2',3,3',4,4'-hexachlorodiphenyl ether", "PCDE-128")
 !CALL Species("C_{12}H_4Cl_6O",                    "159553-70-5", "NLXTWOFWKUBHQN-UHFFFAOYSA-N", "2,2',3,3',4,5-hexachlorodiphenyl ether", "PCDE-129")
  CALL Species("C_{12}H_4Cl_6O",                     "76621-14-2", "JTEWACHFMNTRBB-UHFFFAOYSA-N", "2,2',3,3',4,5'-hexachlorodiphenyl ether", "PCDE-130")
 !CALL Species("C_{12}H_4Cl_6O",                    "727738-84-3", "IUAAWWDTUKIICO-UHFFFAOYSA-N", "2,2',3,3',4,6-hexachlorodiphenyl ether", "PCDE-131")
  CALL Species("C_{12}H_4Cl_6O",                    "124076-66-0", "YDVYLSIVXYLBBX-UHFFFAOYSA-N", "2,2',3,3',4,6'-hexachlorodiphenyl ether", "PCDE-132")
 !CALL Species("C_{12}H_4Cl_6O",                    "727738-87-6", "RJDQOTCZUUQNPX-UHFFFAOYSA-N", "2,2',3,3',5,5'-hexachlorodiphenyl ether", "PCDE-133")
 !CALL Species("C_{12}H_4Cl_6O",                    "727738-90-1", "ZVPXTJYWPPQWHQ-UHFFFAOYSA-N", "2,2',3,3',5,6-hexachlorodiphenyl ether", "PCDE-134")
 !CALL Species("C_{12}H_4Cl_6O",                    "727738-88-7", "HDUBDRLESJYOFX-UHFFFAOYSA-N", "2,2',3,3',5,6'-hexachlorodiphenyl ether", "PCDE-135")
 !CALL Species("C_{12}H_4Cl_6O",                    "117948-40-0", "UESXDYAQZBZDKY-UHFFFAOYSA-N", "2,2',3,3',6,6'-hexachlorodiphenyl ether", "PCDE-136")
  CALL Species("C_{12}H_4Cl_6O",                     "71585-38-1", "LDPJEMNNPDFBCQ-UHFFFAOYSA-N", "2,2',3,4,4',5-hexachlorodiphenyl ether", "PCDE-137")
  CALL Species("C_{12}H_4Cl_6O",                     "71585-36-9", "PHSJJYZIFPWCLZ-UHFFFAOYSA-N", "2,2',3,4,4',5'-hexachlorodiphenyl ether", "PCDE-138")
  CALL Species("C_{12}H_4Cl_6O",                    "106220-83-1", "BIWCEXHDIQZFHI-UHFFFAOYSA-N", "2,2',3,4,4',6-hexachlorodiphenyl ether", "PCDE-139")
  CALL Species("C_{12}H_4Cl_6O",                    "106220-82-0", "XDAOZRZWYVKIAK-UHFFFAOYSA-N", "2,2',3,4,4',6'-hexachlorodiphenyl ether", "PCDE-140")
 !CALL Species("C_{12}H_4Cl_6O",                    "727738-81-0", "CFPVKPARHZZFTP-UHFFFAOYSA-N", "2,2',3,4,5,5'-hexachlorodiphenyl ether", "PCDE-141")
 !CALL Species("C_{12}H_4Cl_6O",                    "727738-82-1", "OULAUCWOZGSVPO-UHFFFAOYSA-N", "2,2',3,4,5,6-hexachlorodiphenyl ether", "PCDE-142")
 !CALL Species("C_{12}H_4Cl_6O",                    "159553-71-6", "YEXFSSRNNAZOCL-UHFFFAOYSA-N", "2,2',3,4,5,6'-hexachlorodiphenyl ether", "PCDE-143")
 !CALL Species("C_{12}H_4Cl_6O",                    "727738-85-4", "GUWQFNUOPQKNMH-UHFFFAOYSA-N", "2,2',3,4,5',6-hexachlorodiphenyl ether", "PCDE-144")
 !CALL Species("C_{12}H_4Cl_6O",                    "165282-28-0", "YPWSQLYCZDWFFL-UHFFFAOYSA-N", "2,2',3,4,6,6'-hexachlorodiphenyl ether", "PCDE-145")
  CALL Species("C_{12}H_4Cl_6O",                    "162853-28-3", "SFTBUGPDOJDENF-UHFFFAOYSA-N", "2,2',3,4',5,5'-hexachlorodiphenyl ether", "PCDE-146")
  CALL Species("C_{12}H_4Cl_6O",                    "116995-18-7", "DIBYIHMYEPMGKC-UHFFFAOYSA-N", "2,2',3,4',5,6-hexachlorodiphenyl ether", "PCDE-147")
 !CALL Species("C_{12}H_4Cl_6O",                    "727738-89-8", "LYODMXFYEXLACS-UHFFFAOYSA-N", "2,2',3,4',5,6'-hexachlorodiphenyl ether", "PCDE-148")
  CALL Species("C_{12}H_4Cl_6O",                     "85918-37-2", "RDKSFIZNZGWNLA-UHFFFAOYSA-N", "2,2',3,4',5',6-hexachlorodiphenyl ether", "PCDE-149")
 !CALL Species("C_{12}H_4Cl_6O",                    "116995-19-8", "PTJHUCWECJWOEQ-UHFFFAOYSA-N", "2,2',3,4',6,6'-hexachlorodiphenyl ether", "PCDE-150")
 !CALL Species("C_{12}H_4Cl_6O",                    "727738-91-2", "YZRAPDULDMXSPA-UHFFFAOYSA-N", "2,2',3,5,5',6-hexachlorodiphenyl ether", "PCDE-151")
 !CALL Species("C_{12}H_4Cl_6O",                    "727738-92-3", "RYUWTVSUXULBMG-UHFFFAOYSA-N", "2,2',3,5,6,6'-hexachlorodiphenyl ether", "PCDE-152")
  CALL Species("C_{12}H_4Cl_6O",                     "71859-30-8", "PECXRRMHOQBOIE-UHFFFAOYSA-N", "2,2',4,4',5,5'-hexachlorodiphenyl ether", "PCDE-153")
  CALL Species("C_{12}H_4Cl_6O",                    "106220-81-9", "XKQUAQGQLZDDBV-UHFFFAOYSA-N", "2,2',4,4',5,6'-hexachlorodiphenyl ether", "PCDE-154")
 !CALL Species("C_{12}H_4Cl_6O",                      "6973-37-1", "UPFQVVDUHJVSON-UHFFFAOYSA-N", "2,2',4,4',6,6'-hexachlorodiphenyl ether", "PCDE-155")
  CALL Species("C_{12}H_4Cl_6O",                    "109828-22-0", "GHTWMHLFFDEYRE-UHFFFAOYSA-N", "2,3,3',4,4',5-hexachlorodiphenyl ether", "PCDE-156")
  CALL Species("C_{12}H_4Cl_6O",                     "94339-60-3", "FEIDIWDEJQVIPO-UHFFFAOYSA-N", "2,3,3',4,4',5'-hexachlorodiphenyl ether", "PCDE-157")
 !CALL Species("C_{12}H_4Cl_6O",                     "85918-36-1", "NRCNIBYWQMNZON-UHFFFAOYSA-N", "2,3,3',4,4',6-hexachlorodiphenyl ether", "PCDE-158")
 !CALL Species("C_{12}H_4Cl_6O",                    "159553-72-7", "CIJJWEVJIHTJNE-UHFFFAOYSA-N", "2,3,3',4,5,5'-hexachlorodiphenyl ether", "PCDE-159")
 !CALL Species("C_{12}H_4Cl_6O",                    "727738-83-2", "ZBIQOLLVDDCFFA-UHFFFAOYSA-N", "2,3,3',4,5,6-hexachlorodiphenyl ether", "PCDE-160")
 !CALL Species("C_{12}H_4Cl_6O",                    "727738-86-5", "JPLMACIRTLXPOQ-UHFFFAOYSA-N", "2,3,3',4,5',6-hexachlorodiphenyl ether", "PCDE-161")
 !CALL Species("C_{12}H_4Cl_6O",                     "89026-26-6", "FBVVCNLKVIABHV-UHFFFAOYSA-N", "2,3,3',4',5,5'-hexachlorodiphenyl ether", "PCDE-162")
  CALL Species("C_{12}H_4Cl_6O",                    "155999-97-6", "YLAYOOJLJKJGCF-UHFFFAOYSA-N", "2,3,3',4',5,6-hexachlorodiphenyl ether", "PCDE-163")
 !CALL Species("C_{12}H_4Cl_6O",                    "727738-93-4", "WYUYGMOEDXDGQA-UHFFFAOYSA-N", "2,3,3',4',5',6-hexachlorodiphenyl ether", "PCDE-164")
 !CALL Species("C_{12}H_4Cl_6O",                    "130480-89-6", "WGESSRWJTGVWDL-UHFFFAOYSA-N", "2,3,3',5,5',6-hexachlorodiphenyl ether", "PCDE-165")
  CALL Species("C_{12}H_4Cl_6O",                     "63646-56-0", "MXDRLBNLTLOWGV-UHFFFAOYSA-N", "2,3,4,4',5,6-hexachlorodiphenyl ether", "PCDE-166")
  CALL Species("C_{12}H_4Cl_6O",                    "131138-20-0", "DYSUSQZPGJXVAL-UHFFFAOYSA-N", "2,3',4,4',5,5'-hexachlorodiphenyl ether", "PCDE-167")
 !CALL Species("C_{12}H_4Cl_6O",                    "727738-94-5", "SJIBNBXSYVZESK-UHFFFAOYSA-N", "2,3',4,4',5',6-hexachlorodiphenyl ether", "PCDE-168")
 !CALL Species("C_{12}H_4Cl_6O",                    "727738-95-6", "GYICBGVARZIPNT-UHFFFAOYSA-N", "3,3',4,4',5,5'-hexachlorodiphenyl ether", "PCDE-169")
  CALL Species("C_{12}H_3Cl_7O",                     "71585-40-5", "BLBURLWSCHSPIS-UHFFFAOYSA-N", "2,2',3,3',4,4',5-heptachlorodiphenyl ether", "PCDE-170")
 !CALL Species("C_{12}H_3Cl_7O",                    "727738-99-0", "KKVXTOYNZOPDNA-UHFFFAOYSA-N", "2,2',3,3',4,4',6-heptachlorodiphenyl ether", "PCDE-171")
 !CALL Species("C_{12}H_3Cl_7O",                     "83992-74-9", "STEQFHAOHOCXAP-UHFFFAOYSA-N", "2,2',3,3',4,5,5'-heptachlorodiphenyl ether", "PCDE-172")
 !CALL Species("C_{12}H_3Cl_7O",                    "727738-96-7", "UIBVUVMDJNWEGY-UHFFFAOYSA-N", "2,2',3,3',4,5,6-heptachlorodiphenyl ether", "PCDE-173")
  CALL Species("C_{12}H_3Cl_7O",                    "159553-73-8", "DQYYZAABYZAOOC-UHFFFAOYSA-N", "2,2',3,3',4,5,6'-heptachlorodiphenyl ether", "PCDE-174")
 !CALL Species("C_{12}H_3Cl_7O",                    "727739-00-6", "POOJWIFLMKEATK-UHFFFAOYSA-N", "2,2',3,3',4,5',6-heptachlorodiphenyl ether", "PCDE-175")
 !CALL Species("C_{12}H_3Cl_7O",                    "727739-01-7", "HSXGZTDEKOCJLP-UHFFFAOYSA-N", "2,2',3,3',4,6,6'-heptachlorodiphenyl ether", "PCDE-176")
  CALL Species("C_{12}H_3Cl_7O",                     "83992-71-6", "RXPQVFMDOJTCKE-UHFFFAOYSA-N", "2,2',3,3',4,5',6'-heptachlorodiphenyl ether", "PCDE-177")
 !CALL Species("C_{12}H_3Cl_7O",                    "727739-03-9", "WURJHLJFXPVIEL-UHFFFAOYSA-N", "2,2',3,3',5,5',6-heptachlorodiphenyl ether", "PCDE-178")
 !CALL Species("C_{12}H_3Cl_7O",                    "727739-04-0", "BLTBVEJZWOYWDS-UHFFFAOYSA-N", "2,2',3,3',5,6,6'-heptachlorodiphenyl ether", "PCDE-179")
  CALL Species("C_{12}H_3Cl_7O",                     "83992-69-2", "YBYYCWMPXZRBNJ-UHFFFAOYSA-N", "2,2',3,4,4',5,5'-heptachlorodiphenyl ether", "PCDE-180")
  CALL Species("C_{12}H_3Cl_7O",                    "157683-75-5", "RPNFRXWGMTYOPA-UHFFFAOYSA-N", "2,2',3,4,4',5,6-heptachlorodiphenyl ether", "PCDE-181")
  CALL Species("C_{12}H_3Cl_7O",                     "88467-63-4", "XWRZWPPSQDHTFU-UHFFFAOYSA-N", "2,2',3,4,4',5,6'-heptachlorodiphenyl ether", "PCDE-182")
 !CALL Species("C_{12}H_3Cl_7O",                    "106220-85-3", "WBKRPARQLGRVHF-UHFFFAOYSA-N", "2,2',3,4,4',5',6-heptachlorodiphenyl ether", "PCDE-183")
  CALL Species("C_{12}H_3Cl_7O",                    "106220-84-2", "WTYSJAGEYAAIFW-UHFFFAOYSA-N", "2,2',3,4,4',6,6'-heptachlorodiphenyl ether", "PCDE-184")
 !CALL Species("C_{12}H_3Cl_7O",                    "727738-97-8", "CFYZMYMEFYKOFG-UHFFFAOYSA-N", "2,2',3,4,5,5',6-heptachlorodiphenyl ether", "PCDE-185")
 !CALL Species("C_{12}H_3Cl_7O",                     "22274-48-2", "WQIBKUPSASYXTE-UHFFFAOYSA-N", "2,2',3,4,5,6,6'-heptachlorodiphenyl ether", "PCDE-186")
  CALL Species("C_{12}H_3Cl_7O",                    "109828-23-1", "BHJCMMGUFRZLKK-UHFFFAOYSA-N", "2,2',3,4',5,5',6-heptachlorodiphenyl ether", "PCDE-187")
 !CALL Species("C_{12}H_3Cl_7O",                    "116995-22-3", "HSLDIWDVXQRGEW-UHFFFAOYSA-N", "2,2',3,4',5,6,6'-heptachlorodiphenyl ether", "PCDE-188")
  CALL Species("C_{12}H_3Cl_7O",                     "83992-72-7", "JUHPMUOWCYBMEY-UHFFFAOYSA-N", "2,3,3',4,4',5,5'-heptachlorodiphenyl ether", "PCDE-189")
  CALL Species("C_{12}H_3Cl_7O",                     "83992-70-5", "QLSBRXLSQINWHM-UHFFFAOYSA-N", "2,3,3',4,4',5,6-heptachlorodiphenyl ether", "PCDE-190")
 !CALL Species("C_{12}H_3Cl_7O",                    "727739-02-8", "YUBXSCDZYOJXNX-UHFFFAOYSA-N", "2,3,3',4,4',5',6-heptachlorodiphenyl ether", "PCDE-191")
 !CALL Species("C_{12}H_3Cl_7O",                    "727738-98-9", "HQXOKVVYOHXWEY-UHFFFAOYSA-N", "2,3,3',4,5,5',6-heptachlorodiphenyl ether", "PCDE-192")
 !CALL Species("C_{12}H_3Cl_7O",                    "727739-05-1", "VCSBVBKQKLHPOG-UHFFFAOYSA-N", "2,3,3',4',5,5',6-heptachlorodiphenyl ether", "PCDE-193")
  CALL Species("C_{12}H_2Cl_8O",                     "57379-40-5", "IXZVOZCULZBCDY-UHFFFAOYSA-N", "2,2',3,3',4,4',5,5'-octachlorodiphenyl ether", "PCDE-194")
  CALL Species("C_{12}H_2Cl_8O",                     "65075-02-7", "YHUPMPHVHYBYJE-UHFFFAOYSA-N", "2,2',3,3',4,4',5,6-octachlorodiphenyl ether", "PCDE-195")
  CALL Species("C_{12}H_2Cl_8O",                     "85918-38-3", "HYNYIQBCBHJTOZ-UHFFFAOYSA-N", "2,2',3,3',4,4',5,6'-octachlorodiphenyl ether", "PCDE-196")
  CALL Species("C_{12}H_2Cl_8O",                    "117948-62-6", "GRWRDFGBFYSOKR-UHFFFAOYSA-N", "2,2',3,3',4,4',6,6'-octachlorodiphenyl ether", "PCDE-197")
 !CALL Species("C_{12}H_2Cl_8O",                    "152833-53-9", "PJSUXNJBOJXQBR-UHFFFAOYSA-N", "2,2',3,3',4,5,5',6-octachlorodiphenyl ether", "PCDE-198")
  CALL Species("C_{12}H_2Cl_8O",                     "83992-76-1", "MRHQOXCYYPFGEE-UHFFFAOYSA-N", "2,2',3,3',4,5,5',6'-octachlorodiphenyl ether", "PCDE-199")
 !CALL Species("C_{12}H_2Cl_8O",                    "727739-06-2", "ZTIKPIULDQZYBS-UHFFFAOYSA-N", "2,2',3,3',4,5,6,6'-octachlorodiphenyl ether", "PCDE-200")
 !CALL Species("C_{12}H_2Cl_8O",                    "116995-21-2", "MHJYPBPVZSIJNU-UHFFFAOYSA-N", "2,2',3,3',4,5',6,6'-octachlorodiphenyl ether", "PCDE-201")
 !CALL Species("C_{12}H_2Cl_8O",                    "727739-08-4", "DIMZRRNMPDXLMZ-UHFFFAOYSA-N", "2,2',3,3',5,5',6,6'-octachlorodiphenyl ether", "PCDE-202")
  CALL Species("C_{12}H_2Cl_8O",                     "83992-75-0", "PKOSPVZTRLMBSK-UHFFFAOYSA-N", "2,2',3,4,4',5,5',6-octachlorodiphenyl ether", "PCDE-203")
 !CALL Species("C_{12}H_2Cl_8O",                    "157683-76-6", "FOTAEIZNNOYBMU-UHFFFAOYSA-N", "2,2',3,4,4',5,6,6'-octachlorodiphenyl ether", "PCDE-204")
 !CALL Species("C_{12}H_2Cl_8O",                    "727739-07-3", "VGJWAHFNNDDRKP-UHFFFAOYSA-N", "2,3,3',4,4',5,5',6-octachlorodiphenyl ether", "PCDE-205")
  CALL Species("C_{12}HCl_9O",                       "83992-73-8", "FPEYJPVHPGDXDD-UHFFFAOYSA-N", "2,2',3,3',4,4',5,5',6-nonachlorodiphenyl ether", "PCDE-206")
 !CALL Species("C_{12}HCl_9O",                      "148934-69-4", "HEGNYKCNUVBAIL-UHFFFAOYSA-N", "2,2',3,3',4,4',5,6,6'-nonachlorodiphenyl ether", "PCDE-207")
 !CALL Species("C_{12}HCl_9O",                      "148934-68-3", "MVPXEXAITKQEMV-UHFFFAOYSA-N", "2,2',3,3',4,5,5',6,6'-nonachlorodiphenyl ether", "PCDE-208")
  CALL Species("C_{12}Cl_{10}O",                     "31710-30-2", "CIPFDHFTBYJKQB-UHFFFAOYSA-N", "decachlorodiphenyl ether", "PCDE-209")
  !---------------------------------------------------------------------------
  ! some casrn from: http://chemicalland21.com/info/DIBENZOFURAN PRODUCTS.htm
  CALL SpeciesGroup(2,"Polychlorinated dibenzofuranes (PCDFs)")
  CALL Species("C_{12}H_7ClO",                       "84761-86-4", "WRSMJZYBNIAAEE-UHFFFAOYSA-N", "1-chlorodibenzofuran", "PCDF-1")
  CALL Species("C_{12}H_7ClO",                       "51230-49-0", "PRKTYWJFCODJOA-UHFFFAOYSA-N", "2-chlorodibenzofuran", "PCDF-2")
  CALL Species("C_{12}H_7ClO",                       "25074-67-3", "BBOZMMAURMEVAR-UHFFFAOYSA-N", "3-chlorodibenzofuran", "PCDF-3")
  CALL Species("C_{12}H_7ClO",                       "74992-96-4", "RHRYBWFAHXCUCR-UHFFFAOYSA-N", "4-chlorodibenzofuran", "PCDF-4")
  CALL Species("C_{12}H_6Cl_2O",                     "64126-85-8", "QVAPRJWSOUITIF-UHFFFAOYSA-N", "1,2-dichlorodibenzofuran", "PCDF-12")
  CALL Species("C_{12}H_6Cl_2O",                     "94538-00-8", "VKIBKEFGJSPRJC-UHFFFAOYSA-N", "1,3-dichlorodibenzofuran", "PCDF-13")
  CALL Species("C_{12}H_6Cl_2O",                     "94538-01-9", "VHQCMZLPHWGUDB-UHFFFAOYSA-N", "1,4-dichlorodibenzofuran", "PCDF-14")
  CALL Species("C_{12}H_6Cl_2O",                     "74992-97-5", "JRSRWACZUFLWKF-UHFFFAOYSA-N", "1,6-dichlorodibenzofuran", "PCDF-16")
  CALL Species("C_{12}H_6Cl_2O",                     "94538-02-0", "XRQNHPGZRFKBJW-UHFFFAOYSA-N", "1,7-dichlorodibenzofuran", "PCDF-17")
  CALL Species("C_{12}H_6Cl_2O",                     "81638-37-1", "UAFNQQBPUWFPGC-UHFFFAOYSA-N", "1,8-dichlorodibenzofuran", "PCDF-18")
  CALL Species("C_{12}H_6Cl_2O",                     "70648-14-5", "AEKSGHBVWJHELW-UHFFFAOYSA-N", "1,9-dichlorodibenzofuran", "PCDF-19")
  CALL Species("C_{12}H_6Cl_2O",                     "64126-86-9", "GETJJZZRPQFSFM-UHFFFAOYSA-N", "2,3-dichlorodibenzofuran", "PCDF-23")
  CALL Species("C_{12}H_6Cl_2O",                     "24478-74-8", "LHTCKMYRNOGUOA-UHFFFAOYSA-N", "2,4-dichlorodibenzofuran", "PCDF-24")
  CALL Species("C_{12}H_6Cl_2O",                     "60390-27-4", "XVLCNKFGFHUQNL-UHFFFAOYSA-N", "2,6-dichlorodibenzofuran", "PCDF-26")
  CALL Species("C_{12}H_6Cl_2O",                     "74992-98-6", "DOZUTNBCPVUMPY-UHFFFAOYSA-N", "2,7-dichlorodibenzofuran", "PCDF-27")
  CALL Species("C_{12}H_6Cl_2O",                      "5409-83-6", "IVVRJIDVYSPKFZ-UHFFFAOYSA-N", "2,8-dichlorodibenzofuran", "PCDF-28")
  CALL Species("C_{12}H_6Cl_2O",                     "94570-83-9", "HYQGGNWDPQZLGX-UHFFFAOYSA-N", "3,4-dichlorodibenzofuran", "PCDF-34")
  CALL Species("C_{12}H_6Cl_2O",                     "74918-40-4", "DMOBGFAKTSXOHG-UHFFFAOYSA-N", "3,6-dichlorodibenzofuran", "PCDF-36")
  CALL Species("C_{12}H_6Cl_2O",                     "58802-21-4", "OLTYAGNRSNFHLK-UHFFFAOYSA-N", "3,7-dichlorodibenzofuran", "PCDF-37")
  CALL Species("C_{12}H_6Cl_2O",                     "64560-13-0", "BZFIZJNKNCJEAD-UHFFFAOYSA-N", "4,6-dichlorodibenzofuran", "PCDF-46")
  CALL Species("C_{12}H_5Cl_3O",                     "83636-47-9", "LADSWAAWPMGHBE-UHFFFAOYSA-N", "1,2,3-trichlorodibenzofuran", "PCDF-123")
  CALL Species("C_{12}H_5Cl_3O",                     "24478-73-7", "RQIWKWHVZLDXEJ-UHFFFAOYSA-N", "1,2,4-trichlorodibenzofuran", "PCDF-124")
  CALL Species("C_{12}H_5Cl_3O",                     "64560-15-2", "CYRZCBUWRUACKJ-UHFFFAOYSA-N", "1,2,6-trichlorodibenzofuran", "PCDF-126")
  CALL Species("C_{12}H_5Cl_3O",                     "83704-37-4", "AFOVQGPNOZDUEP-UHFFFAOYSA-N", "1,2,7-trichlorodibenzofuran", "PCDF-127")
  CALL Species("C_{12}H_5Cl_3O",                     "83704-34-1", "UYIGPSPCOXGBCS-UHFFFAOYSA-N", "1,2,8-trichlorodibenzofuran", "PCDF-128")
  CALL Species("C_{12}H_5Cl_3O",                     "83704-38-5", "ICTXINQOXPMQPM-UHFFFAOYSA-N", "1,2,9-trichlorodibenzofuran", "PCDF-129")
  CALL Species("C_{12}H_5Cl_3O",                     "82911-61-3", "FIPOITUDJFQRRN-UHFFFAOYSA-N", "1,3,4-trichlorodibenzofuran", "PCDF-134")
  CALL Species("C_{12}H_5Cl_3O",                     "83704-39-6", "AGPTXRLVBIRYLV-UHFFFAOYSA-N", "1,3,6-trichlorodibenzofuran", "PCDF-136")
  CALL Species("C_{12}H_5Cl_3O",                     "64560-16-3", "INRBXOGZPDFCLQ-UHFFFAOYSA-N", "1,3,7-trichlorodibenzofuran", "PCDF-137")
  CALL Species("C_{12}H_5Cl_3O",                     "76621-12-0", "PHFSTDOPTZHECA-UHFFFAOYSA-N", "1,3,8-trichlorodibenzofuran", "PCDF-138")
  CALL Species("C_{12}H_5Cl_3O",                     "83704-40-9", "SPPGDLROKBFNNO-UHFFFAOYSA-N", "1,3,9-trichlorodibenzofuran", "PCDF-139")
  CALL Species("C_{12}H_5Cl_3O",                     "82911-60-2", "XDQRWSUJOURTHK-UHFFFAOYSA-N", "1,4,6-trichlorodibenzofuran", "PCDF-146")
  CALL Species("C_{12}H_5Cl_3O",                     "83704-41-0", "XTLMVJUSIANHND-UHFFFAOYSA-N", "1,4,7-trichlorodibenzofuran", "PCDF-147")
  CALL Species("C_{12}H_5Cl_3O",                     "64560-14-1", "PCQCYDNFBUADOK-UHFFFAOYSA-N", "1,4,8-trichlorodibenzofuran", "PCDF-148")
  CALL Species("C_{12}H_5Cl_3O",                     "70648-13-4", "NVYUPBVXKLTYHV-UHFFFAOYSA-N", "1,4,9-trichlorodibenzofuran", "PCDF-149")
  CALL Species("C_{12}H_5Cl_3O",                     "83704-46-5", "HEWLCNCSWBMZHG-UHFFFAOYSA-N", "1,6,7-trichlorodibenzofuran", "PCDF-167") ! = PCDF-349
  CALL Species("C_{12}H_5Cl_3O",                     "82911-59-9", "ZOBVYDQWZXUJNO-UHFFFAOYSA-N", "1,6,8-trichlorodibenzofuran", "PCDF-168") ! = PCDF-249
  CALL Species("C_{12}H_5Cl_3O",                     "58802-18-9", "YNMCXGLWHTVMQO-UHFFFAOYSA-N", "1,7,8-trichlorodibenzofuran", "PCDF-178") ! = PCDF-239
  CALL Species("C_{12}H_5Cl_3O",                     "57117-34-7", "YDLADWPBKZODCJ-UHFFFAOYSA-N", "2,3,4-trichlorodibenzofuran", "PCDF-234")
  CALL Species("C_{12}H_5Cl_3O",                     "57117-33-6", "CPMGJTLNRBIKQM-UHFFFAOYSA-N", "2,3,6-trichlorodibenzofuran", "PCDF-236")
  CALL Species("C_{12}H_5Cl_3O",                     "58802-17-8", "CKXMNTLGGAOERF-UHFFFAOYSA-N", "2,3,7-trichlorodibenzofuran", "PCDF-237")
  CALL Species("C_{12}H_5Cl_3O",                     "57117-32-5", "NUNSNNOYACKRIK-UHFFFAOYSA-N", "2,3,8-trichlorodibenzofuran", "PCDF-238")
  CALL Species("C_{12}H_5Cl_3O",                     "58802-14-5", "GTKURBTWZFQHHY-UHFFFAOYSA-N", "2,4,6-trichlorodibenzofuran", "PCDF-246")
  CALL Species("C_{12}H_5Cl_3O",                     "83704-42-1", "SZBZRVJHPZIGAY-UHFFFAOYSA-N", "2,4,7-trichlorodibenzofuran", "PCDF-247")
  CALL Species("C_{12}H_5Cl_3O",                     "54589-71-8", "WJURXKWTCOMRCE-UHFFFAOYSA-N", "2,4,8-trichlorodibenzofuran", "PCDF-248")
  CALL Species("C_{12}H_5Cl_3O",                     "83704-43-2", "XEUHBCMRIXEBTD-UHFFFAOYSA-N", "3,4,6-trichlorodibenzofuran", "PCDF-346")
  CALL Species("C_{12}H_5Cl_3O",                     "83704-44-3", "RPJJBTSXPNFUTA-UHFFFAOYSA-N", "3,4,7-trichlorodibenzofuran", "PCDF-347")
  CALL Species("C_{12}H_5Cl_3O",                     "83704-45-4", "YTZWNIQGGIJHJX-UHFFFAOYSA-N", "3,4,8-trichlorodibenzofuran", "PCDF-348")
  CALL Species("C_{12}H_4Cl_4O",                     "24478-72-6", "AETAPIFVELRIDN-UHFFFAOYSA-N", "1,2,3,4-tetrachlorodibenzofuran", "PCDF-1234")
  CALL Species("C_{12}H_4Cl_4O",                     "83704-21-6", "BBAXFLIBRPXRBP-UHFFFAOYSA-N", "1,2,3,6-tetrachlorodibenzofuran", "PCDF-1236")
  CALL Species("C_{12}H_4Cl_4O",                     "83704-22-7", "MDNZFYGATDCKRB-UHFFFAOYSA-N", "1,2,3,7-tetrachlorodibenzofuran", "PCDF-1237")
  CALL Species("C_{12}H_4Cl_4O",                     "62615-08-1", "KFQRHGKKSHJMCO-UHFFFAOYSA-N", "1,2,3,8-tetrachlorodibenzofuran", "PCDF-1238")
  CALL Species("C_{12}H_4Cl_4O",                     "83704-23-8", "GNQHLJHTOATTOJ-UHFFFAOYSA-N", "1,2,3,9-tetrachlorodibenzofuran", "PCDF-1239")
  CALL Species("C_{12}H_4Cl_4O",                     "71998-73-7", "NLVWEKPJFUPVMZ-UHFFFAOYSA-N", "1,2,4,6-tetrachlorodibenzofuran", "PCDF-1246")
  CALL Species("C_{12}H_4Cl_4O",                     "83719-40-8", "MFURKMJLDQBHRA-UHFFFAOYSA-N", "1,2,4,7-tetrachlorodibenzofuran", "PCDF-1247")
  CALL Species("C_{12}H_4Cl_4O",                     "64126-87-0", "BFTASCFRRBHFFK-UHFFFAOYSA-N", "1,2,4,8-tetrachlorodibenzofuran", "PCDF-1248")
  CALL Species("C_{12}H_4Cl_4O",                     "83704-24-9", "ZUYYTCCHKQGGOV-UHFFFAOYSA-N", "1,2,4,9-tetrachlorodibenzofuran", "PCDF-1249")
  CALL Species("C_{12}H_4Cl_4O",                     "83704-25-0", "AUEWYHHKDYUYMI-UHFFFAOYSA-N", "1,2,6,7-tetrachlorodibenzofuran", "PCDF-1267")
  CALL Species("C_{12}H_4Cl_4O",                     "83710-07-0", "KCVGVSIBBGIJNZ-UHFFFAOYSA-N", "1,2,6,8-tetrachlorodibenzofuran", "PCDF-1268")
  CALL Species("C_{12}H_4Cl_4O",                     "70648-18-9", "IEPGLLVEBKXASE-UHFFFAOYSA-N", "1,2,6,9-tetrachlorodibenzofuran", "PCDF-1269")
  CALL Species("C_{12}H_4Cl_4O",                     "58802-20-3", "JODWPAQNABOHDG-UHFFFAOYSA-N", "1,2,7,8-tetrachlorodibenzofuran", "PCDF-1278")
  CALL Species("C_{12}H_4Cl_4O",                     "83704-26-1", "PDMFRPIFZAKMLH-UHFFFAOYSA-N", "1,2,7,9-tetrachlorodibenzofuran", "PCDF-1279")
  CALL Species("C_{12}H_4Cl_4O",                     "70648-22-5", "OHYCQUKMNPHFPT-UHFFFAOYSA-N", "1,2,8,9-tetrachlorodibenzofuran", "PCDF-1289")
  CALL Species("C_{12}H_4Cl_4O",                     "83704-27-2", "CBBDONKZEJKFJP-UHFFFAOYSA-N", "1,3,4,6-tetrachlorodibenzofuran", "PCDF-1346")
  CALL Species("C_{12}H_4Cl_4O",                     "70648-16-7", "UMKCVDZTSZNWRH-UHFFFAOYSA-N", "1,3,4,7-tetrachlorodibenzofuran", "PCDF-1347")
  CALL Species("C_{12}H_4Cl_4O",                     "92341-04-3", "XWKRJSMNDOXIHS-UHFFFAOYSA-N", "1,3,4,8-tetrachlorodibenzofuran", "PCDF-1348")
  CALL Species("C_{12}H_4Cl_4O",                     "83704-28-3", "PNCYJHOTZSKZPA-UHFFFAOYSA-N", "1,3,4,9-tetrachlorodibenzofuran", "PCDF-1349")
  CALL Species("C_{12}H_4Cl_4O",                     "57117-36-9", "PITDPGCTIMCYEZ-UHFFFAOYSA-N", "1,3,6,7-tetrachlorodibenzofuran", "PCDF-1367")
  CALL Species("C_{12}H_4Cl_4O",                     "71998-72-6", "BDXKVABZWRZKOS-UHFFFAOYSA-N", "1,3,6,8-tetrachlorodibenzofuran", "PCDF-1368")
  CALL Species("C_{12}H_4Cl_4O",                     "83690-98-6", "NJQQZRLWVLWNGD-UHFFFAOYSA-N", "1,3,6,9-tetrachlorodibenzofuran", "PCDF-1369")
  CALL Species("C_{12}H_4Cl_4O",                     "57117-35-8", "CSXDVUGDFSYXTD-UHFFFAOYSA-N", "1,3,7,8-tetrachlorodibenzofuran", "PCDF-1378")
  CALL Species("C_{12}H_4Cl_4O",                     "64560-17-4", "IEMJMCVYAORWJV-UHFFFAOYSA-N", "1,3,7,9-tetrachlorodibenzofuran", "PCDF-1379")
  CALL Species("C_{12}H_4Cl_4O",                     "66794-59-0", "RBFNYMHNIOKXLA-UHFFFAOYSA-N", "1,4,6,7-tetrachlorodibenzofuran", "PCDF-1467")
  CALL Species("C_{12}H_4Cl_4O",                     "82911-58-8", "VHOBVPNETFRJED-UHFFFAOYSA-N", "1,4,6,8-tetrachlorodibenzofuran", "PCDF-1468")
  CALL Species("C_{12}H_4Cl_4O",                     "70648-19-0", "JAYSBJFHDJOMGZ-UHFFFAOYSA-N", "1,4,6,9-tetrachlorodibenzofuran", "PCDF-1469")
  CALL Species("C_{12}H_4Cl_4O",                     "83704-29-4", "IVRDRRWQABCXLY-UHFFFAOYSA-N", "1,4,7,8-tetrachlorodibenzofuran", "PCDF-1478")
  CALL Species("C_{12}H_4Cl_4O",                     "83704-33-0", "KOJMOXYETDLOPN-UHFFFAOYSA-N", "1,6,7,8-tetrachlorodibenzofuran", "PCDF-1678")
  CALL Species("C_{12}H_4Cl_4O",                     "83704-30-7", "JNVHSHPAAMRSKK-UHFFFAOYSA-N", "2,3,4,6-tetrachlorodibenzofuran", "PCDF-2346")
  CALL Species("C_{12}H_4Cl_4O",                     "83704-31-8", "BROFYOSLFHTGCQ-UHFFFAOYSA-N", "2,3,4,7-tetrachlorodibenzofuran", "PCDF-2347")
  CALL Species("C_{12}H_4Cl_4O",                     "83704-32-9", "IIKXRERKNIJJQY-UHFFFAOYSA-N", "2,3,4,8-tetrachlorodibenzofuran", "PCDF-2348")
  CALL Species("C_{12}H_4Cl_4O",                     "57117-39-2", "MJCNKMDTLRSHNK-UHFFFAOYSA-N", "2,3,6,7-tetrachlorodibenzofuran", "PCDF-2367")
  CALL Species("C_{12}H_4Cl_4O",                     "57117-37-0", "SPMZHWVRMWWTSY-UHFFFAOYSA-N", "2,3,6,8-tetrachlorodibenzofuran", "PCDF-2368")
  CALL Species("C_{12}H_4Cl_4O",                     "51207-31-9", "KSMVNVHUTQZITP-UHFFFAOYSA-N", "2,3,7,8-tetrachlorodibenzofuran", "PCDF-2378")
  CALL Species("C_{12}H_4Cl_4O",                     "57117-38-1", "NSDXKMRVQOSASS-UHFFFAOYSA-N", "2,4,6,7-tetrachlorodibenzofuran", "PCDF-2467")
  CALL Species("C_{12}H_4Cl_4O",                     "58802-19-0", "ZSPJEACWUWSAGS-UHFFFAOYSA-N", "2,4,6,8-tetrachlorodibenzofuran", "PCDF-2468")
  CALL Species("C_{12}H_4Cl_4O",                     "57117-40-5", "LMJLCLBSUNZLNW-UHFFFAOYSA-N", "3,4,6,7-tetrachlorodibenzofuran", "PCDF-3467")
  CALL Species("C_{12}H_3Cl_5O",                     "83704-47-6", "LIQJBAPSLUZUTB-UHFFFAOYSA-N", "1,2,3,4,6-pentachlorodibenzofuran", "PCDF-12346")
  CALL Species("C_{12}H_3Cl_5O",                     "83704-48-7", "DOJZTBGOWIYFAC-UHFFFAOYSA-N", "1,2,3,4,7-pentachlorodibenzofuran", "PCDF-12347")
  CALL Species("C_{12}H_3Cl_5O",                     "67517-48-0", "ZCTNDJSCLPJCRA-UHFFFAOYSA-N", "1,2,3,4,8-pentachlorodibenzofuran", "PCDF-12348")
  CALL Species("C_{12}H_3Cl_5O",                     "83704-49-8", "IQSZNZVZXOFRJS-UHFFFAOYSA-N", "1,2,3,4,9-pentachlorodibenzofuran", "PCDF-12349")
  CALL Species("C_{12}H_3Cl_5O",                     "57117-42-7", "NZUPQBVDIWCPBX-UHFFFAOYSA-N", "1,2,3,6,7-pentachlorodibenzofuran", "PCDF-12367")
  CALL Species("C_{12}H_3Cl_5O",                     "83704-51-2", "VHQJZOFUPXEZQZ-UHFFFAOYSA-N", "1,2,3,6,8-pentachlorodibenzofuran", "PCDF-12368")
  CALL Species("C_{12}H_3Cl_5O",                     "83704-52-3", "IPSFQAKXDJVQOX-UHFFFAOYSA-N", "1,2,3,6,9-pentachlorodibenzofuran", "PCDF-12369")
  CALL Species("C_{12}H_3Cl_5O",                     "57117-41-6", "SBMIVUVRFPGOEB-UHFFFAOYSA-N", "1,2,3,7,8-pentachlorodibenzofuran", "PCDF-12378")
  CALL Species("C_{12}H_3Cl_5O",                     "83704-53-4", "JVUSEQPOWCBYNG-UHFFFAOYSA-N", "1,2,3,7,9-pentachlorodibenzofuran", "PCDF-12379")
  CALL Species("C_{12}H_3Cl_5O",                     "83704-54-5", "NKGLWUJPCTUDEH-UHFFFAOYSA-N", "1,2,3,8,9-pentachlorodibenzofuran", "PCDF-12389")
  CALL Species("C_{12}H_3Cl_5O",                     "83704-50-1", "BQTCODOVGXJHRB-UHFFFAOYSA-N", "1,2,4,6,7-pentachlorodibenzofuran", "PCDF-12467")
  CALL Species("C_{12}H_3Cl_5O",                     "69698-57-3", "JDTUAYPJSSXDNO-UHFFFAOYSA-N", "1,2,4,6,8-pentachlorodibenzofuran", "PCDF-12468")
  CALL Species("C_{12}H_3Cl_5O",                     "70648-24-7", "QDQFGVXGPSYEJU-UHFFFAOYSA-N", "1,2,4,6,9-pentachlorodibenzofuran", "PCDF-12469")
  CALL Species("C_{12}H_3Cl_5O",                     "58802-15-6", "GCFDWHIKRXCUPJ-UHFFFAOYSA-N", "1,2,4,7,8-pentachlorodibenzofuran", "PCDF-12478")
  CALL Species("C_{12}H_3Cl_5O",                     "71998-74-8", "IYGVHNHWORPMFU-UHFFFAOYSA-N", "1,2,4,7,9-pentachlorodibenzofuran", "PCDF-12479")
  CALL Species("C_{12}H_3Cl_5O",                     "70648-23-6", "ZSPAPWGNAGTCCA-UHFFFAOYSA-N", "1,2,4,8,9-pentachlorodibenzofuran", "PCDF-12489")
  CALL Species("C_{12}H_3Cl_5O",                     "69433-00-7", "ZAAVDGLJEOXCMV-UHFFFAOYSA-N", "1,2,6,7,8-pentachlorodibenzofuran", "PCDF-12678") ! = PCDF-23489
  CALL Species("C_{12}H_3Cl_5O",                     "70872-82-1", "QQRRQEVOSMEVQB-UHFFFAOYSA-N", "1,2,6,7,9-pentachlorodibenzofuran", "PCDF-12679")
  CALL Species("C_{12}H_3Cl_5O",                     "83704-36-3", "JVYYDUWJIJMOGW-UHFFFAOYSA-N", "1,3,4,6,7-pentachlorodibenzofuran", "PCDF-13467")
  CALL Species("C_{12}H_3Cl_5O",                     "83704-55-6", "COKIAYPRHYHYCH-UHFFFAOYSA-N", "1,3,4,6,8-pentachlorodibenzofuran", "PCDF-13468")
  CALL Species("C_{12}H_3Cl_5O",                     "70648-15-6", "MYSAFUYQSBEEDR-UHFFFAOYSA-N", "1,3,4,6,9-pentachlorodibenzofuran", "PCDF-13469")
  CALL Species("C_{12}H_3Cl_5O",                     "58802-16-7", "ORSUQGVCWLXKLZ-UHFFFAOYSA-N", "1,3,4,7,8-pentachlorodibenzofuran", "PCDF-13478")
  CALL Species("C_{12}H_3Cl_5O",                     "70648-20-3", "GFXPLABVZOBCCW-UHFFFAOYSA-N", "1,3,4,7,9-pentachlorodibenzofuran", "PCDF-13479")
  CALL Species("C_{12}H_3Cl_5O",                     "70648-21-4", "FRLMQDUYUJIHCZ-UHFFFAOYSA-N", "1,3,6,7,8-pentachlorodibenzofuran", "PCDF-13678")
  CALL Species("C_{12}H_3Cl_5O",                     "83704-35-2", "VANGHZRYKXDPRR-UHFFFAOYSA-N", "1,4,6,7,8-pentachlorodibenzofuran", "PCDF-14678")
  CALL Species("C_{12}H_3Cl_5O",                     "57117-43-8", "SJFBZRQKGOGHEV-UHFFFAOYSA-N", "2,3,4,6,7-pentachlorodibenzofuran", "PCDF-23467")
  CALL Species("C_{12}H_3Cl_5O",                     "67481-22-5", "MKRFORPSRBMAIP-UHFFFAOYSA-N", "2,3,4,6,8-pentachlorodibenzofuran", "PCDF-23468")
  CALL Species("C_{12}H_3Cl_5O",                     "57117-31-4", "OGBQILNBLMPPDP-UHFFFAOYSA-N", "2,3,4,7,8-pentachlorodibenzofuran", "PCDF-23478")
  CALL Species("C_{12}H_2Cl_6O",                     "79060-60-9", "SNWFMKXFMVHBKD-UHFFFAOYSA-N", "1,2,3,4,6,7-hexachlorodibenzofuran", "PCDF-123467")
  CALL Species("C_{12}H_2Cl_6O",                     "69698-60-8", "UCFGNWHERVQWMZ-UHFFFAOYSA-N", "1,2,3,4,6,8-hexachlorodibenzofuran", "PCDF-123468")
  CALL Species("C_{12}H_2Cl_6O",                     "91538-83-9", "KFFUZIROJGXTQH-UHFFFAOYSA-N", "1,2,3,4,6,9-hexachlorodibenzofuran", "PCDF-123469")
  CALL Species("C_{12}H_2Cl_6O",                     "70648-26-9", "LVYBAQIVPKCOEE-UHFFFAOYSA-N", "1,2,3,4,7,8-hexachlorodibenzofuran", "PCDF-123478")
  CALL Species("C_{12}H_2Cl_6O",                     "91538-84-0", "BKIXWRBZCQEZAQ-UHFFFAOYSA-N", "1,2,3,4,7,9-hexachlorodibenzofuran", "PCDF-123479")
  CALL Species("C_{12}H_2Cl_6O",                     "92341-07-6", "VSDUQUBUJRNREH-UHFFFAOYSA-N", "1,2,3,4,8,9-hexachlorodibenzofuran", "PCDF-123489")
  CALL Species("C_{12}H_2Cl_6O",                     "57117-44-9", "JEYJJJXOFWNEHN-UHFFFAOYSA-N", "1,2,3,6,7,8-hexachlorodibenzofuran", "PCDF-123678")
  CALL Species("C_{12}H_2Cl_6O",                     "92341-06-5", "JZVOLXQREJNTTL-UHFFFAOYSA-N", "1,2,3,6,7,9-hexachlorodibenzofuran", "PCDF-123679")
  CALL Species("C_{12}H_2Cl_6O",                     "75198-38-8", "WLGQZUOHEXTWFH-UHFFFAOYSA-N", "1,2,3,6,8,9-hexachlorodibenzofuran", "PCDF-123689")
  CALL Species("C_{12}H_2Cl_6O",                     "72918-21-9", "PYUSJFJVDVSXIU-UHFFFAOYSA-N", "1,2,3,7,8,9-hexachlorodibenzofuran", "PCDF-123789")
  CALL Species("C_{12}H_2Cl_6O",                     "67562-40-7", "CKDDYGBBKXIHRY-UHFFFAOYSA-N", "1,2,4,6,7,8-hexachlorodibenzofuran", "PCDF-124678")
  CALL Species("C_{12}H_2Cl_6O",                     "75627-02-0", "FAHIPPHRJJJQOG-UHFFFAOYSA-N", "1,2,4,6,7,9-hexachlorodibenzofuran", "PCDF-124679")
  CALL Species("C_{12}H_2Cl_6O",                     "69698-59-5", "FGZCXIPKUCJGHW-UHFFFAOYSA-N", "1,2,4,6,8,9-hexachlorodibenzofuran", "PCDF-124689")
  CALL Species("C_{12}H_2Cl_6O",                     "71998-75-9", "OGTZINCGZYXZCR-UHFFFAOYSA-N", "1,3,4,6,7,8-hexachlorodibenzofuran", "PCDF-134678")
  CALL Species("C_{12}H_2Cl_6O",                     "92341-05-4", "TURXXOIFVABBPW-UHFFFAOYSA-N", "1,3,4,6,7,9-hexachlorodibenzofuran", "PCDF-134679")
  CALL Species("C_{12}H_2Cl_6O",                     "60851-34-5", "XTAHLACQOVXINQ-UHFFFAOYSA-N", "2,3,4,6,7,8-hexachlorodibenzofuran", "PCDF-234678")
  CALL Species("C_{12}HCl_7O",                       "67562-39-4", "WDMKCPIVJOGHBF-UHFFFAOYSA-N", "1,2,3,4,6,7,8-heptachlorodibenzofuran", "PCDF-1234678")
  CALL Species("C_{12}HCl_7O",                       "70648-25-8", "JWXWOEUKHXOUSK-UHFFFAOYSA-N", "1,2,3,4,6,7,9-heptachlorodibenzofuran", "PCDF-1234679")
  CALL Species("C_{12}HCl_7O",                       "69698-58-4", "BADFHCOLISGRRW-UHFFFAOYSA-N", "1,2,3,4,6,8,9-heptachlorodibenzofuran", "PCDF-1234689")
  CALL Species("C_{12}HCl_7O",                       "55673-89-7", "VEZCTZWLJYWARH-UHFFFAOYSA-N", "1,2,3,4,7,8,9-heptachlorodibenzofuran", "PCDF-1234789")
  CALL Species("C_{12}Cl_8O",                        "39001-02-0", "RHIROFAGUQOFLU-UHFFFAOYSA-N", "octachlorodibenzofuran", "PCDF-12346789")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Polychlorinated dibenzo-$p$-dioxins (PCDDs)")
  CALL Species("C_{12}H_7ClO_2",                     "39227-53-7", "VGGGRWRBGXENKI-UHFFFAOYSA-N", "1-chlorodibenzo-$p$-dioxin", "PCDD-1")
  CALL Species("C_{12}H_7ClO_2",                     "39227-54-8", "GIUGGRUEPHPVNR-UHFFFAOYSA-N", "2-chlorodibenzo-$p$-dioxin", "PCDD-2")
  CALL Species("C_{12}H_6Cl_2O_2",                   "54536-18-4", "DFGDMWHUCCHXIF-UHFFFAOYSA-N", "1,2-dichlorodibenzo-$p$-dioxin", "PCDD-12")
  CALL Species("C_{12}H_6Cl_2O_2",                   "50585-39-2", "AZYJYMAKTBXNSX-UHFFFAOYSA-N", "1,3-dichlorodibenzo-$p$-dioxin", "PCDD-13")
  CALL Species("C_{12}H_6Cl_2O_2",                   "54536-19-5", "MBMUPQZSDWVPQU-UHFFFAOYSA-N", "1,4-dichlorodibenzo-$p$-dioxin", "PCDD-14")
  CALL Species("C_{12}H_6Cl_2O_2",                   "38178-38-0", "MAWMBEVNJGEDAD-UHFFFAOYSA-N", "1,6-dichlorodibenzo-$p$-dioxin", "PCDD-16")
  CALL Species("C_{12}H_6Cl_2O_2",                   "82291-26-7", "IJUWLAFPPVRYGY-UHFFFAOYSA-N", "1,7-dichlorodibenzo-$p$-dioxin", "PCDD-17")
  CALL Species("C_{12}H_6Cl_2O_2",                   "82291-27-8", "PLZYIHQBHROTFD-UHFFFAOYSA-N", "1,8-dichlorodibenzo-$p$-dioxin", "PCDD-18")
  CALL Species("C_{12}H_6Cl_2O_2",                   "82291-28-9", "JZDVJXBKJDADAY-UHFFFAOYSA-N", "1,9-dichlorodibenzo-$p$-dioxin", "PCDD-19")
  CALL Species("C_{12}H_6Cl_2O_2",                   "29446-15-9", "YCIYTXRUZSDMRZ-UHFFFAOYSA-N", "2,3-dichlorodibenzo-$p$-dioxin", "PCDD-23")
  CALL Species("C_{12}H_6Cl_2O_2",                   "33857-26-0", "NBFMTHWVRBOVPE-UHFFFAOYSA-N", "2,7-dichlorodibenzo-$p$-dioxin", "PCDD-27")
  CALL Species("C_{12}H_6Cl_2O_2",                   "38964-22-6", "WMWJCKBJUQDYLM-UHFFFAOYSA-N", "2,8-dichlorodibenzo-$p$-dioxin", "PCDD-28")
  CALL Species("C_{12}H_5Cl_3O_2",                   "54536-17-3", "SKMFBGZVVNDVFR-UHFFFAOYSA-N", "1,2,3-trichlorodibenzo-$p$-dioxin", "PCDD-123")
  CALL Species("C_{12}H_5Cl_3O_2",                   "39227-58-2", "HRVUKLBFRPWXPJ-UHFFFAOYSA-N", "1,2,4-trichlorodibenzo-$p$-dioxin", "PCDD-124")
  CALL Species("C_{12}H_5Cl_3O_2",                   "69760-96-9", "XQBPVWBIUBCJJO-UHFFFAOYSA-N", "1,2,6-trichlorodibenzo-$p$-dioxin", "PCDD-126")
  CALL Species("C_{12}H_5Cl_3O_2",                   "82291-30-3", "TXJMXDWFPQSYEQ-UHFFFAOYSA-N", "1,2,7-trichlorodibenzo-$p$-dioxin", "PCDD-127")
  CALL Species("C_{12}H_5Cl_3O_2",                   "82291-31-4", "QBEOCKSANJLBAE-UHFFFAOYSA-N", "1,2,8-trichlorodibenzo-$p$-dioxin", "PCDD-128")
  CALL Species("C_{12}H_5Cl_3O_2",                   "82291-32-5", "DQLRDBDQLSIOIX-UHFFFAOYSA-N", "1,2,9-trichlorodibenzo-$p$-dioxin", "PCDD-129")
  CALL Species("C_{12}H_5Cl_3O_2",                   "82291-33-6", "LNPVMVSAUXUGHH-UHFFFAOYSA-N", "1,3,6-trichlorodibenzo-$p$-dioxin", "PCDD-136")
  CALL Species("C_{12}H_5Cl_3O_2",                   "67028-17-5", "RPKWIXFZKMDPMH-UHFFFAOYSA-N", "1,3,7-trichlorodibenzo-$p$-dioxin", "PCDD-137")
  CALL Species("C_{12}H_5Cl_3O_2",                   "82306-61-4", "FJAKCOBYQSEWMT-UHFFFAOYSA-N", "1,3,8-trichlorodibenzo-$p$-dioxin", "PCDD-138")
  CALL Species("C_{12}H_5Cl_3O_2",                   "82306-62-5", "DGDADRUTFAIIQQ-UHFFFAOYSA-N", "1,3,9-trichlorodibenzo-$p$-dioxin", "PCDD-139")
  CALL Species("C_{12}H_5Cl_3O_2",                   "82306-63-6", "UTTYFTWIJLRXKB-UHFFFAOYSA-N", "1,4,6-trichlorodibenzo-$p$-dioxin", "PCDD-146")
  CALL Species("C_{12}H_5Cl_3O_2",                   "82306-64-7", "NBWAQBGJBSYXHV-UHFFFAOYSA-N", "1,4,7-trichlorodibenzo-$p$-dioxin", "PCDD-147")
  CALL Species("C_{12}H_5Cl_3O_2",                   "82306-65-8", "CAPCTZJHYADFNX-UHFFFAOYSA-N", "1,7,8-trichlorodibenzo-$p$-dioxin", "PCDD-178")
  CALL Species("C_{12}H_5Cl_3O_2",                   "33857-28-2", "ZSIZNEVHVVRPFF-UHFFFAOYSA-N", "2,3,7-trichlorodibenzo-$p$-dioxin", "PCDD-237")
  CALL Species("C_{12}H_4Cl_4O_2",                   "30746-58-8", "DJHHDLMTUOLVHY-UHFFFAOYSA-N", "1,2,3,4-tetrachlorodibenzo-$p$-dioxin", "PCDD-1234")
  CALL Species("C_{12}H_4Cl_4O_2",                   "71669-25-5", "XEZBZSVTUSXISZ-UHFFFAOYSA-N", "1,2,3,6-tetrachlorodibenzo-$p$-dioxin", "PCDD-1236")
  CALL Species("C_{12}H_4Cl_4O_2",                   "67028-18-6", "SKGXYFVQZVPEFP-UHFFFAOYSA-N", "1,2,3,7-tetrachlorodibenzo-$p$-dioxin", "PCDD-1237")
  CALL Species("C_{12}H_4Cl_4O_2",                   "53555-02-5", "BXKLTNKYLCZOHF-UHFFFAOYSA-N", "1,2,3,8-tetrachlorodibenzo-$p$-dioxin", "PCDD-1238")
  CALL Species("C_{12}H_4Cl_4O_2",                   "71669-26-6", "CMVHZKSHSHQJHS-UHFFFAOYSA-N", "1,2,3,9-tetrachlorodibenzo-$p$-dioxin", "PCDD-1239")
  CALL Species("C_{12}H_4Cl_4O_2",                   "71669-27-7", "KQNBZUDHTCXCNA-UHFFFAOYSA-N", "1,2,4,6-tetrachlorodibenzo-$p$-dioxin", "PCDD-1246")
  CALL Species("C_{12}H_4Cl_4O_2",                   "71669-28-8", "SMPHQCMJQUBTFZ-UHFFFAOYSA-N", "1,2,4,7-tetrachlorodibenzo-$p$-dioxin", "PCDD-1247")
  CALL Species("C_{12}H_4Cl_4O_2",                   "71669-29-9", "XGIKODBWQSAEFQ-UHFFFAOYSA-N", "1,2,4,8-tetrachlorodibenzo-$p$-dioxin", "PCDD-1248")
  CALL Species("C_{12}H_4Cl_4O_2",                   "71665-99-1", "WDAHVJCSSYOALR-UHFFFAOYSA-N", "1,2,4,9-tetrachlorodibenzo-$p$-dioxin", "PCDD-1249")
  CALL Species("C_{12}H_4Cl_4O_2",                   "40581-90-6", "SAMLAWFHXZIRMP-UHFFFAOYSA-N", "1,2,6,7-tetrachlorodibenzo-$p$-dioxin", "PCDD-1267")
  CALL Species("C_{12}H_4Cl_4O_2",                   "67323-56-2", "YYUFYZDSYHKVDP-UHFFFAOYSA-N", "1,2,6,8-tetrachlorodibenzo-$p$-dioxin", "PCDD-1268")
  CALL Species("C_{12}H_4Cl_4O_2",                   "40581-91-7", "ZKMXKYXNLFLUCD-UHFFFAOYSA-N", "1,2,6,9-tetrachlorodibenzo-$p$-dioxin", "PCDD-1269")
  CALL Species("C_{12}H_4Cl_4O_2",                   "34816-53-0", "YDZCLBKUTXYYKS-UHFFFAOYSA-N", "1,2,7,8-tetrachlorodibenzo-$p$-dioxin", "PCDD-1278")
  CALL Species("C_{12}H_4Cl_4O_2",                   "71669-23-3", "QIKHBBZEUNSCAF-UHFFFAOYSA-N", "1,2,7,9-tetrachlorodibenzo-$p$-dioxin", "PCDD-1279")
  CALL Species("C_{12}H_4Cl_4O_2",                   "62470-54-6", "WELWFAGPAZKSBG-UHFFFAOYSA-N", "1,2,8,9-tetrachlorodibenzo-$p$-dioxin", "PCDD-1289")
  CALL Species("C_{12}H_4Cl_4O_2",                   "33423-92-6", "OTQFXRBLGNEOGH-UHFFFAOYSA-N", "1,3,6,8-tetrachlorodibenzo-$p$-dioxin", "PCDD-1368")
  CALL Species("C_{12}H_4Cl_4O_2",                   "71669-24-4", "QAUIRDIJIUMMEP-UHFFFAOYSA-N", "1,3,6,9-tetrachlorodibenzo-$p$-dioxin", "PCDD-1369")
  CALL Species("C_{12}H_4Cl_4O_2",                   "50585-46-1", "VPTDIAYLYJBYQG-UHFFFAOYSA-N", "1,3,7,8-tetrachlorodibenzo-$p$-dioxin", "PCDD-1378")
  CALL Species("C_{12}H_4Cl_4O_2",                   "62470-53-5", "JMGYHLJVDHUACM-UHFFFAOYSA-N", "1,3,7,9-tetrachlorodibenzo-$p$-dioxin", "PCDD-1379")
  CALL Species("C_{12}H_4Cl_4O_2",                   "40581-93-9", "QTIIAIRUSSSOHT-UHFFFAOYSA-N", "1,4,6,9-tetrachlorodibenzo-$p$-dioxin", "PCDD-1469")
  CALL Species("C_{12}H_4Cl_4O_2",                   "40581-94-0", "FCRXUTCUWCJZJI-UHFFFAOYSA-N", "1,4,7,8-tetrachlorodibenzo-$p$-dioxin", "PCDD-1478")
  CALL Species("C_{12}H_4Cl_4O_2",                    "1746-01-6", "HGUFODBRKLSHSI-UHFFFAOYSA-N", "2,3,7,8-tetrachlorodibenzo-$p$-dioxin", "PCDD-2378; TCDD")
  CALL Species("C_{12}H_3Cl_5O_2",                   "67028-19-7", "LNWDBNKKBLRAMH-UHFFFAOYSA-N", "1,2,3,4,6-pentachlorodibenzo-$p$-dioxin", "PCDD-12346")
  CALL Species("C_{12}H_3Cl_5O_2",                   "39227-61-7", "WRNGAZFESPEMCN-UHFFFAOYSA-N", "1,2,3,4,7-pentachlorodibenzo-$p$-dioxin", "PCDD-12347")
  CALL Species("C_{12}H_3Cl_5O_2",                   "71925-15-0", "RLGWDUHOIIWPGN-UHFFFAOYSA-N", "1,2,3,6,7-pentachlorodibenzo-$p$-dioxin", "PCDD-12367")
  CALL Species("C_{12}H_3Cl_5O_2",                   "71925-16-1", "VKDGHBBUEIIEHL-UHFFFAOYSA-N", "1,2,3,6,8-pentachlorodibenzo-$p$-dioxin", "PCDD-12368")
  CALL Species("C_{12}H_3Cl_5O_2",                   "82291-34-7", "NWKWRHSKKNELND-UHFFFAOYSA-N", "1,2,3,6,9-pentachlorodibenzo-$p$-dioxin", "PCDD-12369") ! PCDD-14678
  CALL Species("C_{12}H_3Cl_5O_2",                   "40321-76-4", "FSPZPQQWDODWAU-UHFFFAOYSA-N", "1,2,3,7,8-pentachlorodibenzo-$p$-dioxin", "PCDD-12378")
  CALL Species("C_{12}H_3Cl_5O_2",                   "71925-17-2", "UAOYHTXYVWEPIB-UHFFFAOYSA-N", "1,2,3,7,9-pentachlorodibenzo-$p$-dioxin", "PCDD-12379") ! PCDD-23468
  CALL Species("C_{12}H_3Cl_5O_2",                   "71925-18-3", "VUMZAVNIADYKFC-UHFFFAOYSA-N", "1,2,3,8,9-pentachlorodibenzo-$p$-dioxin", "PCDD-12389") ! PCDD-23467
  CALL Species("C_{12}H_3Cl_5O_2",                   "82291-35-8", "SEKDDGLKEYEVQK-UHFFFAOYSA-N", "1,2,4,6,7-pentachlorodibenzo-$p$-dioxin", "PCDD-12467")
  CALL Species("C_{12}H_3Cl_5O_2",                   "71998-76-0", "SJJWALZHAWITMS-UHFFFAOYSA-N", "1,2,4,6,8-pentachlorodibenzo-$p$-dioxin", "PCDD-12468")
  CALL Species("C_{12}H_3Cl_5O_2",                   "82291-36-9", "GNQVSAMSAKZLKE-UHFFFAOYSA-N", "1,2,4,6,9-pentachlorodibenzo-$p$-dioxin", "PCDD-12469")
  CALL Species("C_{12}H_3Cl_5O_2",                   "58802-08-7", "QUPLGUUISJOUPJ-UHFFFAOYSA-N", "1,2,4,7,8-pentachlorodibenzo-$p$-dioxin", "PCDD-12478")
  CALL Species("C_{12}H_3Cl_5O_2",                   "82291-37-0", "QLBBXWPVEFJZEC-UHFFFAOYSA-N", "1,2,4,7,9-pentachlorodibenzo-$p$-dioxin", "PCDD-12479") ! PCDD-13468
  CALL Species("C_{12}H_3Cl_5O_2",                   "82291-38-1", "KLLFLRKEOJCTGC-UHFFFAOYSA-N", "1,2,4,8,9-pentachlorodibenzo-$p$-dioxin", "PCDD-12489") ! PCDD-13467
  CALL Species("C_{12}H_2Cl_6O_2",                   "58200-66-1", "NLBQVWJHLWAFGJ-UHFFFAOYSA-N", "1,2,3,4,6,7-hexachlorodibenzo-$p$-dioxin", "PCDD-123467")
  CALL Species("C_{12}H_2Cl_6O_2",                   "58200-67-2", "IMALTUQZEIFHJW-UHFFFAOYSA-N", "1,2,3,4,6,8-hexachlorodibenzo-$p$-dioxin", "PCDD-123468")
  CALL Species("C_{12}H_2Cl_6O_2",                   "58200-68-3", "UDYXCMRDCOVQLG-UHFFFAOYSA-N", "1,2,3,4,6,9-hexachlorodibenzo-$p$-dioxin", "PCDD-123469")
  CALL Species("C_{12}H_2Cl_6O_2",                   "39227-28-6", "WCYYQNSQJHPVMG-UHFFFAOYSA-N", "1,2,3,4,7,8-hexachlorodibenzo-$p$-dioxin", "PCDD-123478")
  CALL Species("C_{12}H_2Cl_6O_2",                   "57653-85-7", "YCLUIPQDHHPDJJ-UHFFFAOYSA-N", "1,2,3,6,7,8-hexachlorodibenzo-$p$-dioxin", "PCDD-123678")
  CALL Species("C_{12}H_2Cl_6O_2",                   "64461-98-9", "BQOHWGKNRKCEFT-UHFFFAOYSA-N", "1,2,3,6,7,9-hexachlorodibenzo-$p$-dioxin", "PCDD-123679") ! PCDD-124678
  CALL Species("C_{12}H_2Cl_6O_2",                   "58200-69-4", "GZRQZUFXVFRKBI-UHFFFAOYSA-N", "1,2,3,6,8,9-hexachlorodibenzo-$p$-dioxin", "PCDD-123689") ! PCDD-134678
  CALL Species("C_{12}H_2Cl_6O_2",                   "19408-74-3", "LGIRBUBHIWTVCK-UHFFFAOYSA-N", "1,2,3,7,8,9-hexachlorodibenzo-$p$-dioxin", "PCDD-123789") ! PCDD-234678
  CALL Species("C_{12}H_2Cl_6O_2",                   "39227-62-8", "BSJDQMWAWFTDGD-UHFFFAOYSA-N", "1,2,4,6,7,9-hexachlorodibenzo-$p$-dioxin", "PCDD-124679")
  CALL Species("C_{12}H_2Cl_6O_2",                   "58802-09-8", "URELDHWUZUWPIU-UHFFFAOYSA-N", "1,2,4,6,8,9-hexachlorodibenzo-$p$-dioxin", "PCDD-124689") ! PCDD-134679
  CALL Species("C_{12}HCl_7O_2",                     "35822-46-9", "WCLNVRQZUKYVAI-UHFFFAOYSA-N", "1,2,3,4,6,7,8-heptachlorodibenzo-$p$-dioxin", "PCDD-1234678")
  CALL Species("C_{12}HCl_7O_2",                     "58200-70-7", "KTJJIBIRZGQFQZ-UHFFFAOYSA-N", "1,2,3,4,6,7,9-heptachlorodibenzo-$p$-dioxin", "PCDD-1234679")
  CALL Species("C_{12}Cl_8O_2",                       "3268-87-9", "FOIBFBMSLDGNHL-UHFFFAOYSA-N", "octachlorodibenzo-$p$-dioxin", "PCDD-12346789")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Chlorocarbons with nitrogen (\chem{C}, \chem{H}, \chem{O}, \chem{N}, \chem{Cl})")
  ! C, H, N, Cl:
  ! aliphatic:
  CALL Species("NCCl",                                 "506-77-4", "QPJDMGCKMHUXFD-UHFFFAOYSA-N", "cyanogen chloride", "")
  CALL Species("CH_3NCl_2",                           "7651-91-4", "DWEYPOWXNBAIDU-UHFFFAOYSA-N", "N,N-dichloromethylamine", "")
  CALL Species("C_2H_2ClN",                            "107-14-2", "RENMDAKOXSCIGH-UHFFFAOYSA-N", "chloroacetonitrile", "")
  CALL Species("C_2HCl_2N",                           "3018-12-0", "STZZWJCGRKXEFF-UHFFFAOYSA-N", "dichloroacetonitrile", "")
  CALL Species("C_2Cl_3N",                             "545-06-2", "DRUIESSIVFYOMK-UHFFFAOYSA-N", "trichloroacetonitrile", "")
  CALL Species("C_4H_{12}ClN",                          "75-57-0", "OKIZCWYLBDKLSU-UHFFFAOYSA-M", "tetramethylammonium chloride", "")
  CALL Species("C_4H_{12}ClN_5",                      "1115-70-4", "OETHQSJEHLVLGH-UHFFFAOYSA-N", "metformin hydrochloride", "")
  CALL Species("C_5H_{11}Cl_2N",                        "51-75-2", "HAWPXGHAZFHHAD-UHFFFAOYSA-N", "bis(2-chloroethyl)methylamine", "")
  CALL Species("C_5H_{13}Cl_2N",                       "999-81-5", "OLQFELZRGYJRAZ-UHFFFAOYSA-N", "chlormequat chloride", "")
  CALL Species("C_6H_{12}Cl_3N",                       "555-77-1", "FDAYLTPAFBGXAB-UHFFFAOYSA-N", "tris(2-chloroethyl)amine", "")
  CALL Species("C_6H_{13}Cl_2N",                       "538-07-8", "UQZPGHOJMQTOHB-UHFFFAOYSA-N", "bis(2-chloroethyl)ethylamine", "ethylbis(2-chloroethyl)amine")
  CALL Species("C_{19}H_{42}ClN",                      "112-02-7", "WOWHHFRSBJGXCM-UHFFFAOYSA-M", "cetrimonium chloride", "trimethylhexadecylammonium chloride")
  CALL Species("C_{38}H_{80}ClN",                      "107-64-2", "REZZEXDLIUJMMS-UHFFFAOYSA-M", "dimethyldioctadecylammonium chloride", "")
  ! aromatic:
  CALL Species("C_6H_6ClN",                             "95-51-2", "AKCRQHGQIJBRMN-UHFFFAOYSA-N", "1-amino-2-chlorobenzene", "$o$-chloroaniline")
  CALL Species("C_6H_6ClN",                            "108-42-9", "PNPCRKVUWYDDST-UHFFFAOYSA-N", "1-amino-3-chlorobenzene", "$m$-chloroaniline")
  CALL Species("C_6H_6ClN",                            "106-47-8", "QSNSCYSYFYORTR-UHFFFAOYSA-N", "1-amino-4-chlorobenzene", "$p$-chloroaniline")
  CALL Species("C_6H_5Cl_2N",                          "608-27-5", "BRPSAOUFIJSKOT-UHFFFAOYSA-N", "2,3-dichlorobenzenamine", "2,3-dichloroaniline")
  CALL Species("C_6H_5Cl_2N",                          "554-00-7", "KQCMTOWTPBNWDB-UHFFFAOYSA-N", "2,4-dichlorobenzenamine", "2,4-dichloroaniline")
  CALL Species("C_6H_5Cl_2N",                           "95-76-1", "SDYWXFYBZPNOFX-UHFFFAOYSA-N", "3,4-dichlorobenzenamine", "3,4-dichloroaniline")
  CALL Species("C_6H_5Cl_2N",                          "626-43-7", "UQRLKWGPEVNVHT-UHFFFAOYSA-N", "3,5-dichlorobenzenamine", "3,5-dichloroaniline")
  CALL Species("C_6H_5Cl_2N",                           "95-82-9", "AVYGCQXNNJPXSS-UHFFFAOYSA-N", "2,5-dichlorobenzenamine", "2,5-dichloroaniline")
  CALL Species("C_6H_4Cl_3N",                          "636-30-6", "GUMCAKKKNKYFEB-UHFFFAOYSA-N", "2,4,5-trichlorobenzenamine", "")
  CALL Species("C_6H_4Cl_3N",                          "634-93-5", "NATVSFWWYVJTAZ-UHFFFAOYSA-N", "2,4,6-trichlorobenzenamine", "")
  CALL Species("C_6H_2Cl_5N",                          "527-20-8", "KHCZSJXTDDHLGJ-UHFFFAOYSA-N", "2,3,4,5,6-pentachloroaniline", "")
  CALL Species("C_6H_3Cl_2CN",                        "1194-65-6", "YOYAIZYFCNQIRF-UHFFFAOYSA-N", "2,6-dichlorobenzenenitrile", "dichlobenil")
  CALL Species("C_6H_5Cl_3N_2",                       "5329-12-4", "MULHANRBCQBHII-UHFFFAOYSA-N", "(2,4,6-trichlorophenyl)hydrazine", "")
  CALL Species("C_7H_4ClN",                            "623-03-0", "GJNGXPDXRVXSEH-UHFFFAOYSA-N", "4-chlorobenzonitrile", "")
  CALL Species("C_7H_8ClN",                             "87-60-5", "ZUVPLKVDZNDZCM-UHFFFAOYSA-N", "3-chloro-2-methylbenzenamine", "")
  CALL Species("C_7H_8ClN",                             "95-74-9", "RQKFYFNZSHWXAW-UHFFFAOYSA-N", "3-chloro-4-methylbenzenamine", "")
  CALL Species("C_7H_8ClN",                             "95-69-2", "CXNVOWPRHWWCQR-UHFFFAOYSA-N", "4-chloro-2-methylbenzenamine", "")
  CALL Species("C_7H_8ClN",                             "95-79-4", "WRZOMWDJOLIVQP-UHFFFAOYSA-N", "5-chloro-2-methylbenzenamine", "")
  CALL Species("C_8Cl_4N_2",                          "1897-45-6", "CRQQGFGUEAVUIL-UHFFFAOYSA-N", "2,4,5,6-tetrachloro-1,3-dicyanobenzene", "chlorothalonil")
  CALL Species("C_{10}H_5ClN_2",                      "2698-41-1", "JJNZXLAFIPKXIG-UHFFFAOYSA-N", "2-chlorobenzalmalononitrile", "")
  CALL Species("C_{10}H_{16}ClN",                       "56-93-9", "KXHPPCXNWTUNSB-UHFFFAOYSA-M", "benzyltrimethylammonium chloride", "")
  CALL Species("C_{10}H_{13}ClN_2",                   "6164-98-3", "STUSTWKEFDQFFZ-UHFFFAOYSA-N", "chlordimeform", "")
  CALL Species("C_{12}H_8Cl_2N_2",                    "1602-00-2", "XHQLXCFUPJSGOE-UHFFFAOYSA-N", "4,4'-dichloroazobenzene", "")
  CALL Species("C_{12}H_6Cl_4N_2",                   "14047-09-7", "SOBGIMQKWDUEPY-UHFFFAOYSA-N", "bis(3,4-dichlorophenyl)diazene", "3,4,3',4'-tetrachloroazobenzene")
  CALL Species("C_{11}H_{12}ClN",                     "2012-81-9", "RBGSZIRWNWQDOK-UHFFFAOYSA-N", "2-($p$-chlorophenyl)-3-methylbutyronitrile", "")
  CALL Species("C_{12}H_{10}Cl_2N_2",                   "91-94-1", "HUWXDEQWWKGHRV-UHFFFAOYSA-N", "3,3'-dichloro-(1,1'-biphenyl)-4,4'-diamine", "3,3'-dichlorobenzidine")
  CALL Species("C_{12}H_9ClN_2",                      "4340-77-6", "NJFDMENHTAYHMA-UHFFFAOYSA-N", "4-chloroazobenzene", "")
  CALL Species("C_{13}H_{12}Cl_2N_2",                  "101-14-4", "IBOFVQJTBBUKMU-UHFFFAOYSA-N", "4,4'-methylenebis(2-chlorobenzenamine)", "")
  CALL Species("C_{14}H_8N_4Cl_2",                   "74115-24-5", "UXADOQPNKNTIHB-UHFFFAOYSA-N", "clofentezine", "")
  CALL Species("C_{17}H_{22}ClN_3",                   "2465-27-2", "KSCQDDRPFHTIRL-UHFFFAOYSA-N", "aniline, 4,4'-(imidocarbonyl)bis-(N,N-dimethyl)-, hydrochloride", "auramine hydrochloride")
  CALL Species("C_{20}H_{24}ClN",                      "549-18-8", "KFYRPLNVJVHZGT-UHFFFAOYSA-N", "amitriptyline hydrochloride", "")
  CALL Species("C_{22}H_{30}Cl_2N_{10}",                "55-56-1", "GHXZTYHSJHQHIJ-UHFFFAOYSA-N", "chlorhexidine", "")
  CALL Species("C_{23}H_{25}ClN_2",                    "569-64-2", "FDZZZRQASAIRJF-UHFFFAOYSA-M", "malachite green", "")
  CALL Species("C_{23}H_{42}ClN",                      "139-08-2", "OCBHHZMJRVXXQK-UHFFFAOYSA-M", "tetradecylbenzyldimethyl ammonium chloride", "")
  CALL Species("C_{27}H_{50}ClN",                      "122-19-0", "SFVFIFLLYFPGHH-UHFFFAOYSA-M", "stearyldimethylbenzylammonium chloride", "benzyldimethylstearylammonium chloride")
  ! heterocycles with N:
  CALL Species("C_3Cl_3N_3",                           "108-77-0", "MGNCLNQXLYJVJD-UHFFFAOYSA-N", "2,4,6-trichloro-1,3,5-triazine", "")
  CALL Species("C_5H_4ClN",                            "109-09-1", "OKDGRDCXVWSXDC-UHFFFAOYSA-N", "2-chloropyridine", "")
  CALL Species("C_5H_4ClN",                            "626-60-8", "PWRBCZZQRRPXAB-UHFFFAOYSA-N", "3-chloropyridine", "")
  CALL Species("C_5HCl_4N",                          "14121-36-9", "FZFNDUTVMQOPCT-UHFFFAOYSA-N", "2,3,4,6-tetrachloropyridine", "")
  CALL Species("C_5HCl_4N",                           "2402-79-1", "FATBKZJZAHWCSL-UHFFFAOYSA-N", "2,3,5,6-tetrachloropyridine", "")
  CALL Species("C_5Cl_5N",                            "2176-62-7", "DNDPLEAVNVOOQZ-UHFFFAOYSA-N", "pentachloropyridine", "")
  CALL Species("C_6H_{10}ClN_5",                      "6190-65-4", "DFWFIQKMSFGDCQ-UHFFFAOYSA-N", "desethylatrazine", "")
  CALL Species("C_6H_3Cl_4N",                         "1929-82-4", "DCUJJWWUNKIJPH-UHFFFAOYSA-N", "2-chloro-6-(trichloromethyl)-pyridine", "")
  CALL Species("C_6H_3Cl_4N",                        "10469-02-0", "OZLPEWBQZDDFCQ-UHFFFAOYSA-N", "2,3,4,5-tetrachloro-6-methylpyridine", "")
  CALL Species("C_6H_3Cl_4N",                        "69045-78-9", "VLJIVLGVKMTBOD-UHFFFAOYSA-N", "2-chloro-5-(trichloromethyl)pyridine", "")
  CALL Species("C_6H_2Cl_5N",                        "69045-83-6", "XVBWGQSXLITICX-UHFFFAOYSA-N", "2,3-dichloro-5-(trichloromethyl)pyridine", "")
  CALL Species("C_6H_2Cl_5N",                         "1817-13-6", "MWFCRQNUHFSUNY-UHFFFAOYSA-N", "2,5-dichloro-6-(trichloromethyl)pyridine", "")
  CALL Species("C_6HCl_6N",                           "1201-30-5", "YWSFDYUMEBEQNZ-UHFFFAOYSA-N", "3,4,5-trichloro-2-(trichloromethyl)pyridine", "")
  CALL Species("C_6Cl_7N",                            "1134-04-9", "YMBFWRZKTZICHS-UHFFFAOYSA-N", "2,3,4,5-tetrachloro-6-(trichloromethyl)pyridine", "")
  CALL Species("C_6Cl_4N_2",                         "17824-83-8", "KFPBGJYBKSQIAI-UHFFFAOYSA-N", "3,4,5,6-tetrachloropyridine-2-carbonitrile", "")
  CALL Species("C_6H_2Cl_3N_3",                      "14143-60-3", "AZYYQVGBVUCIEO-UHFFFAOYSA-N", "4-amino-3,5,6-trichloropyridine-2-carbonitrile", "")
  CALL Species("C_7H_{10}ClN_3",                       "535-89-7", "HJIUPFPIEBPYIE-UHFFFAOYSA-N", "crimidine", "")
  CALL Species("C_7H_{12}ClN_5",                       "122-34-9", "ODCWYMIRDDJXKW-UHFFFAOYSA-N", "simazine", "")
  CALL Species("C_7H_{12}ClN_5",                     "30125-63-4", "LMKQNTMFZLAJDV-UHFFFAOYSA-N", "desethylterbuthylazine", "")
  CALL Species("C_7H_{16}NCl",                       "24307-26-4", "VHOVSQVSAAQANU-UHFFFAOYSA-M", "mepiquat chloride", "")
  CALL Species("C_8H_{14}ClN_5",                      "1912-24-9", "MXWJVTOOROXGIU-UHFFFAOYSA-N", "atrazine", "")
  CALL Species("C_9H_9Cl_2N_3",                       "4205-90-7", "GJSURZIOUXUGAL-UHFFFAOYSA-N", "clonidine", "")
  CALL Species("C_9H_{12}ClN_5",                     "22936-86-3", "OOHIAOSLOGDBCE-UHFFFAOYSA-N", "cyprazine", "")
  CALL Species("C_9H_{16}ClN_5",                       "139-40-2", "WJNRPILHGGKWCK-UHFFFAOYSA-N", "propazine", "")
  CALL Species("C_9H_{16}ClN_5",                      "7286-69-3", "BZRUVKZGXNSXMB-UHFFFAOYSA-N", "sebuthylazine", "")
  CALL Species("C_9H_{16}ClN_5",                      "5915-41-3", "FZXISNSWEXTPMF-UHFFFAOYSA-N", "terbuthylazine", "")
  CALL Species("C_9H_{13}ClN_6",                     "21725-46-2", "MZZBPDKVEFVLFF-UHFFFAOYSA-N", "cyanazine", "") ! other CAS: 11096-88-1
  CALL Species("C_9H_5Cl_3N_4",                        "101-05-3", "IMHBYKMAHXWHRP-UHFFFAOYSA-N", "anilazine", "")
  CALL Species("C_9H_5Cl_2N",                           "86-98-6", "HXEWMTXDBOQQKO-UHFFFAOYSA-N", "4,7-dichloroquinoline", "")
  CALL Species("C_{10}H_6Cl_2N_2",                    "3740-92-9", "NRFQZTCQAYEXEE-UHFFFAOYSA-N", "fenclorim", "")
  CALL Species("C_{10}H_{11}ClN_4",                 "135410-20-7", "WCXDHFDTOYPNIE-UHFFFAOYSA-N", "acetamiprid", "")
  CALL Species("C_{12}H_{13}ClN_4",                     "58-14-0", "WKSAUQYGYAYLPV-UHFFFAOYSA-N", "pyrimethamine", "")
  CALL Species("C_{13}H_{15}Cl_2N_3",                "66246-88-6", "WKBPZYKAUNRMKP-UHFFFAOYSA-N", "penconazole", "")
  CALL Species("C_{15}H_{17}ClN_4",                  "88671-89-0", "HZJKXKUJVSEEFU-UHFFFAOYSA-N", "myclobutanil", "")
  CALL Species("C_{19}H_{18}N_3Cl",                   "1237-53-2", "LVWOBZPDFCTAOU-UHFFFAOYSA-N", "2-chloro-4,6-bis(2,4-dimethylphenyl)-1,3,5-triazine", "")
  CALL Species("C_{19}H_{17}ClN_4",                 "114369-43-6", "RQDJADAKIFFEKQ-UHFFFAOYSA-N", "fenbuconazole", "")
  ! C, H, O, N, Cl:
  ! aliphatic:
  CALL Species("CCl_3NO_2",                             "76-06-2", "LFHISGNCFUNFFM-UHFFFAOYSA-N", "trichloronitromethane", "chloropicrin")
  CALL Species("CHCl_2NO",                            "1794-86-1", "JIRJHEXNDQBKRZ-UHFFFAOYSA-N", "phosgene oxime", "")
  CALL Species("C_2H_3Cl_2NO_2",                       "594-72-9", "OQOGEOLRYAOSKO-UHFFFAOYSA-N", "1,1-dichloro-1-nitroethane", "")
  CALL Species("C_2H_4ClNO",                            "79-07-2", "VXIVSQZSERGHQP-UHFFFAOYSA-N", "2-chloroacetamide", "")
  CALL Species("C_2NO_5Cl_3",                                  "", "LLOMRHYBAIQQGH-UHFFFAOYSA-N", "MCM:CCL3PAN", "")
  CALL Species("C_2H_4NO_5Cl",                                 "", "GVGGZHVOOFUNFY-UHFFFAOYSA-N", "MCM:CCLNO3COOH", "")
  CALL Species("C_2HNO_5Cl_2",                                 "", "BORKLXDOXKHVRO-UHFFFAOYSA-N", "MCM:CHCL2PAN", "")
  CALL Species("C_2H_2NO_5Cl",                                 "", "IEQWXJNETVFAEB-UHFFFAOYSA-N", "MCM:CLETPAN", "")
  CALL Species("C_2H_4NO_5Cl",                                 "", "FBIXZLOOAAICML-UHFFFAOYSA-N", "MCM:CNO3CLOOH", "")
  CALL Species("C_2H_4NO_4Cl",                                 "", "PMLDKFZZOPGPNK-UHFFFAOYSA-N", "MCM:CCLNO3COH", "")
  CALL Species("C_2HNO_6Cl_2",                                 "", "YJZUCQDIDXVPNP-UHFFFAOYSA-N", "MCM:CL2OHPAN", "")
  CALL Species("C_2H_2NO_6Cl",                                 "", "ZNYKQWDXHYNGTP-UHFFFAOYSA-N", "MCM:CLOHPAN", "")
  CALL Species("C_2H_4ClNO_2",                         "598-92-5", "LPIWIOBGUAPNQW-UHFFFAOYSA-N", "1-chloro-1-nitroethane", "")
  CALL Species("C_2H_4NO_4Cl",                                 "", "OUFVLIWCNZOATK-UHFFFAOYSA-N", "MCM:CNO3CLOH", "")
  CALL Species("C_2H_2NO_4Cl",                                 "", "XDEFPIDZLAIVCM-UHFFFAOYSA-N", "MCM:CCLCONO3", "")
  CALL Species("C_2H_2NO_4Cl",                                 "", "ODQFUUDIEKKDDJ-UHFFFAOYSA-N", "MCM:CNO3OCL", "")
  CALL Species("C_3Cl_3N_3O_3",                         "87-90-1", "YRIZYWQGELRKNT-UHFFFAOYSA-N", "symclosene", "trichloroisocyanuric acid")
  CALL Species("C_3H_3NO_5Cl_2",                               "", "MSMFQRZPTBDNBF-UHFFFAOYSA-N", "MCM:CL12PAN", "")
  CALL Species("C_3H_6ClNO_2",                         "600-25-9", "XEKUXTMJEFPWCG-UHFFFAOYSA-N", "1-chloro-1-nitropropane", "")
  CALL Species("C_5H_6Cl_2N_2O_2",                     "118-52-5", "KEQGZUUPPQEDPF-UHFFFAOYSA-N", "1,3-dichloro-5,5-dimethylhydantoin", "")
  CALL Species("C_5H_9Cl_2N_3O_2",                     "154-93-8", "DLGOEMSEDOSKAD-UHFFFAOYSA-N", "carmustine", "")
  CALL Species("C_8H_{12}ClNO",                         "93-71-0", "MDBGGTQNNUOQRC-UHFFFAOYSA-N", "2-chloro-N,N-di-2-propenylacetamide", "")
  CALL Species("C_8H_{11}Cl_2NO",                    "37764-25-3", "YRMLFORXOOIJDR-UHFFFAOYSA-N", "2,2-dichloro-N,N-di-2-propenylacetamide", "dichlormid")
  CALL Species("C_9H_{16}ClN_3O_2",                  "13010-47-4", "GQYIWUVLTXOXAJ-UHFFFAOYSA-N", "lomustine", "")
  CALL Species("C_{10}H_{18}ClN_3O_2",               "13909-09-6", "FVLVBPDQNARYJU-UHFFFAOYSA-N", "semustine", "")
  CALL Species("C_{11}H_{13}Cl_2NO_3",              "121776-33-8", "MCNOFYBITGAAGM-UHFFFAOYSA-N", "furilazole", "")
  CALL Species("C_{12}H_{14}ClNO_2",                 "81777-89-1", "KIEDNEWSYUYDSN-UHFFFAOYSA-N", "dimethazone", "clomazone")
  ! aromatic:
  CALL Species("C_6H_4ClNO_2",                          "88-73-3", "BFCFYVKQTRLZHA-UHFFFAOYSA-N", "2-chloronitrobenzene", "$o$-chloronitrobenzene")
  CALL Species("C_6H_4ClNO_2",                         "121-73-3", "KMAQZIILEGKYQZ-UHFFFAOYSA-N", "3-chloronitrobenzene", "$m$-chloronitrobenzene")
  CALL Species("C_6H_4ClNO_2",                         "100-00-5", "CZGCEKJOLUNIFY-UHFFFAOYSA-N", "4-chloronitrobenzene", "$p$-chloronitrobenzene")
  CALL Species("C_6H_3Cl_2NO_2",                        "99-54-7", "NTBYINQTYWZXLH-UHFFFAOYSA-N", "1,2-dichloro-4-nitrobenzene", "")
  CALL Species("C_6H_3Cl_2NO_2",                        "89-61-2", "RZKKOBGFCAHLCZ-UHFFFAOYSA-N", "1,4-dichloro-2-nitrobenzene", "")
  CALL Species("C_6H_3Cl_2NO_2",                      "3209-22-1", "CMVQZRLQEOAYSW-UHFFFAOYSA-N", "2,3-dichloronitrobenzene", "")
  CALL Species("C_6H_3Cl_2NO_2",                       "611-06-3", "QUIMTLZDMCNYGY-UHFFFAOYSA-N", "2,4-dichloronitrobenzene", "")
  CALL Species("C_6H_3Cl_2NO_2",                       "618-62-2", "RNABGKOKSBUFHW-UHFFFAOYSA-N", "3,5-dichloronitrobenzene", "")
  CALL Species("C_6Cl_5NO_2",                           "82-68-8", "LKPLKUMXSAEKID-UHFFFAOYSA-N", "pentachloronitrobenzene", "quintozene")
  CALL Species("C_6H_4ClNO_3",                          "89-64-5", "NWSIFTLPLKCTSX-UHFFFAOYSA-N", "4-chloro-2-nitrophenol", "")
  CALL Species("C_6H_5ClN_2O_2",                       "121-87-9", "LOCWBQIWHWIRGN-UHFFFAOYSA-N", "2-chloro-4-nitrobenzenamine", "")
  CALL Species("C_6H_5ClN_2O_2",                      "6283-25-6", "KWIXNFOTNVKIGM-UHFFFAOYSA-N", "2-chloro-5-nitrobenzenamine", "")
  CALL Species("C_6H_4ClN_3O_4",                      "5388-62-5", "CLMQUEQFVUMDPC-UHFFFAOYSA-N", "4-chloro-2,6-dinitrobenzenamine", "")
  CALL Species("C_6H_3ClN_2O_4",                        "97-00-7", "VYZAHLCBVHPDDF-UHFFFAOYSA-N", "1-chloro-2,4-dinitrobenzene", "")
  CALL Species("C_6H_3ClN_2O_4",                       "606-21-3", "BPPMIQPXQVIZNJ-UHFFFAOYSA-N", "1-chloro-2,6-dinitrobenzene", "")
  CALL Species("C_6H_2ClN_3O_6",                        "88-88-0", "HJRJRUMKQCMYDL-UHFFFAOYSA-N", "2-chloro-1,3,5-trinitrobenzene", "")
  CALL Species("C_6H_2Cl_3NO_2",                     "17700-09-3", "BGKIECJVXXHLDP-UHFFFAOYSA-N", "2,3,4-trichloronitrobenzene", "")
  CALL Species("C_6HCl_4NO_2",                         "879-39-0", "MTBYTWZDRVOMBR-UHFFFAOYSA-N", "2,3,4,5-tetrachloronitrobenzene", "")
  CALL Species("C_6HCl_4NO_2",                         "117-18-0", "XQTLDIFVVHJORV-UHFFFAOYSA-N", "1,2,4,5-tetrachloronitrobenzene", "tecnazene")
  CALL Species("C_6H_5ClN_2O_2",                        "89-63-4", "PBGKNXWGYQPUJK-UHFFFAOYSA-N", "4-chloro-2-nitrobenzenamine", "")
  CALL Species("C_6H_5ClN_2O_2",                       "635-22-3", "FOHHWGVAOVDVLP-UHFFFAOYSA-N", "4-chloro-3-nitrobenzenamine", "")
  CALL Species("C_6H_4Cl_2N_2O_2",                      "99-30-9", "BIXZHMJUSMUDOQ-UHFFFAOYSA-N", "botran", "")
  CALL Species("C_7H_3Cl_2NO",                       "34893-92-0", "XEFUJGURFLOFAN-UHFFFAOYSA-N", "3,5-dichlorophenyl isocyanate", "")
  CALL Species("C_7H_4ClNO_4",                        "2516-96-3", "QUEKGYQTRJVEQC-UHFFFAOYSA-N", "2-chloro-5-nitrobenzoic acid", "")
  CALL Species("C_7H_4ClNO_4",                          "96-99-1", "DFXQXFGFOLXAPO-UHFFFAOYSA-N", "4-chloro-3-nitrobenzoic acid", "")
  CALL Species("C_7H_4ClNO",                           "104-12-1", "ADAKRBAJFHTIEW-UHFFFAOYSA-N", "1-chloro-4-isocyanatobenzene", "")
  CALL Species("C_7H_3Cl_2NO",                         "102-36-3", "MFUVCHZWGSJKEQ-UHFFFAOYSA-N", "1,2-dichloro-4-isocyanatobenzene", "")
  CALL Species("C_7H_6ClNO_2",                         "121-86-8", "LLYXJBROWQDVMI-UHFFFAOYSA-N", "2-chloro-1-methyl-4-nitrobenzene", "")
  CALL Species("C_7H_5Cl_2NO_3",                     "39549-27-4", "VMBRJHMTAZXHES-UHFFFAOYSA-N", "2,4-dichloro-3-methyl-6-nitrophenol", "")
  CALL Species("C_7H_6ClNO_3",                        "7147-89-9", "JBMGJOKJUYGIJH-UHFFFAOYSA-N", "4-chloro-5-methyl-2-nitrophenol", "4-chloro-6-nitro-$m$-cresol")
  CALL Species("C_7H_5Cl_2NO_2",                       "133-90-4", "HSSBORCLYSCBJR-UHFFFAOYSA-N", "3-amino-2,5-dichlorobenzoic acid", "")
  CALL Species("C_7H_3Cl_4NO_3",                      "2438-88-2", "BGPPUXMKKQMWLV-UHFFFAOYSA-N", "2,3,5,6-tetrachloro-4-nitroanisole", "TCNA")
  CALL Species("C_7H_5Cl_2NO",                        "2008-58-4", "JHSPCUHPSIUQRB-UHFFFAOYSA-N", "2,6-dichlorobenzamide", "")
  CALL Species("C_8H_7Cl_2NO",                        "1918-18-9", "WOZQBERUBLYCEG-UHFFFAOYSA-N", "swep", "")
  CALL Species("C_8H_8ClNO",                           "539-03-7", "GGUOCFNAWIODMF-UHFFFAOYSA-N", "N-(4-chlorophenyl)acetamide", "$p$-chloroacetanilide")
  CALL Species("C_8H_6ClNO_4",                       "51282-49-6", "JGBJHRKCUKTQOE-UHFFFAOYSA-N", "methyl 5-chloro-2-nitrobenzoate", "")
  CALL Species("C_8H_8ClNO_4",                        "6940-53-0", "ORLPGMKKCAEWOW-UHFFFAOYSA-N", "4-chloro-2,5-dimethoxynitrobenzene", "")
  CALL Species("C_9H_7Cl_5N_2O",                     "20856-57-9", "REEFSLKDEDEWAO-UHFFFAOYSA-N", "chloraniformethan", "")
  CALL Species("C_9H_{11}ClN_2O",                      "150-68-5", "BMLIZLVNXIYGCK-UHFFFAOYSA-N", "monuron", "")
  CALL Species("C_9H_{11}ClN_2O_2",                   "1746-81-2", "LKJPSUCKSLORMF-UHFFFAOYSA-N", "monolinuron", "")
  CALL Species("C_9H_{10}Cl_2N_2O",                    "330-54-1", "XMTQQYYKAHVGBJ-UHFFFAOYSA-N", "diuron", "")
  CALL Species("C_9H_{10}Cl_2N_2O_2",                  "330-55-2", "XKJMBINCVNINCA-UHFFFAOYSA-N", "linuron", "")
  CALL Species("C_9H_9Cl_2NO",                         "709-98-8", "LFULEKSKNZEWOE-UHFFFAOYSA-N", "propanil", "")
  CALL Species("C_9H_6Cl_2N_2O_3",                   "20354-26-1", "LRUUNMYPIBZBQH-UHFFFAOYSA-N", "methazole", "")
  CALL Species("C_{10}H_6ClNO_2",                     "2797-51-5", "OBLNWSCLAYSJJR-UHFFFAOYSA-N", "1,4-naphthalenedione, 2-amino-3-chloro-", "quinoclamine")
  CALL Species("C_{10}H_{13}ClN_2O",                 "15545-48-9", "JXCGFZXSOMJFOA-UHFFFAOYSA-N", "chlortoluron", "chlorotoluron")
  CALL Species("C_{10}H_{13}ClN_2O_2",               "19937-59-8", "DSRNRYQBBJQVCW-UHFFFAOYSA-N", "metoxuron", "")
  CALL Species("C_{10}H_{12}ClNO_2",                   "101-21-3", "CWJSHJJYOPWUGX-UHFFFAOYSA-N", "chlorpropham", "")
  CALL Species("C_{10}H_8ClN_3O",                     "1698-60-8", "WYKYKTKDBLFHCY-UHFFFAOYSA-N", "pyrazon", "")
  CALL Species("C_{10}H_{13}Cl_2NO_3",                "2008-39-1", "IUQJDHJVPLLKFL-UHFFFAOYSA-N", "2,4-D dimethylamine", "(2,4-dichlorophenoxy)acetic acid dimethylamine")
  CALL Species("C_{10}H_9Cl_2NO",                     "2759-71-9", "PLQDLOBGKJCDSZ-UHFFFAOYSA-N", "3',4'-dichlorocyclopropanecarboxanilide", "cypromid")
  CALL Species("C_{10}H_6N_5OCl",                    "72459-58-6", "IQGKIPDJXCAMSM-UHFFFAOYSA-N", "triazoxide", "")
  CALL Species("C_{11}H_{10}ClNO_2",                  "1967-16-4", "ULBXWWGWDPVHAO-UHFFFAOYSA-N", "chlorbufam", "")
  CALL Species("C_{11}H_{11}N_2O_2Cl",               "84527-51-5", "VLBZAQJMGULJIU-UHFFFAOYSA-N", "zarilamid", "")
  CALL Species("C_{11}H_{12}Cl_2N_2O_5",                "56-75-7", "WIIZWVCIJKGZOK-IUCAKERBSA-N", "chloramphenicol", "")
  CALL Species("C_{11}H_{14}ClNO_4",                 "51487-69-5", "PITWUHDDNUVBPT-UHFFFAOYSA-N", "cloethocarb", "")
  CALL Species("C_{11}H_{16}ClN_3O_2",               "23422-53-9", "MYPKGPZHHQEODQ-UHFFFAOYSA-N", "formetanate hydrochloride", "")
  CALL Species("C_{11}H_9Cl_2NO_3",                 "113136-77-9", "GLWWLNJJJCTFMZ-UHFFFAOYSA-N", "cyclanilide", "")
  CALL Species("C_{11}H_{14}ClNO",                    "1918-16-7", "MFOUDYKPLGXPGO-UHFFFAOYSA-N", "propachlor", "")
  CALL Species("C_{11}H_9Cl_2NO_2",                    "101-27-9", "MCOQHIWZJUDQIC-UHFFFAOYSA-N", "barban", "")
  CALL Species("C_{12}H_9ClN_2O_3",                  "74070-46-5", "DDBMQDADIHOWIC-UHFFFAOYSA-N", "aclonifen", "")
  CALL Species("C_{12}H_{11}Cl_2NO",                 "23950-58-5", "PHNUZKMIPFFYSO-UHFFFAOYSA-N", "propyzamide", "pronamide")
  CALL Species("C_{12}H_6Cl_3NO_3",                   "1836-77-7", "XQNAUQUKWRBODG-UHFFFAOYSA-N", "2,4,6-trichlorophenyl 4-nitrophenyl ether", "chlornitrofen")
  CALL Species("C_{12}H_7Cl_2NO_3",                   "1836-75-5", "XITQUSLLOSKDTB-UHFFFAOYSA-N", "nitrofen", "")
  CALL Species("C_{12}H_{13}ClN_2O",                  "3766-60-7", "BYYMILHAKOURNM-UHFFFAOYSA-N", "buturon", "")
  CALL Species("C_{13}H_9Cl_3N_2O",                    "101-20-2", "ICUTUKXCWQYESQ-UHFFFAOYSA-N", "triclocarban", "")
  CALL Species("C_{13}H_{18}ClNO_2",                 "50563-36-5", "SCCDDNKJYDZXMM-UHFFFAOYSA-N", "dimethachlor", "")
  CALL Species("C_{13}H_7Cl_4NO_2",                   "1154-59-2", "SJQBHPJLLIJASD-UHFFFAOYSA-N", "3,5-dichloro-N-(3,4-dichlorophenyl)-2-hydroxybenzamide", "")
  CALL Species("C_{13}H_{11}Cl_2NO_2",               "32809-16-8", "QXJKBPAVAHBARF-UHFFFAOYSA-N", "procymidone", "")
  CALL Species("C_{13}H_{18}Cl_2N_2O_2",               "148-82-3", "SGDBTWWWUNNDEQ-LBPRGKRZSA-N", "melphalan", "")
  CALL Species("C_{13}H_8Cl_2N_2O_4",                   "50-65-7", "RJMUSRYZPJIFPJ-UHFFFAOYSA-N", "niclosamide", "")
  CALL Species("C_{13}H_{13}Cl_2N_3O_3",             "36734-19-7", "ONUFESLQCSAYKA-UHFFFAOYSA-N", "iprodione", "")
  CALL Species("C_{14}H_{16}Cl_3NO_2",              "156052-68-5", "SOUGWDPPRBKJEX-UHFFFAOYSA-N", "zoxamide", "")
  CALL Species("C_{14}H_{17}Cl_2NO_2",              "126833-17-8", "VDLGAVXLJYLFDH-UHFFFAOYSA-N", "fenhexamid", "")
  CALL Species("C_{14}H_{19}Cl_2NO_2",                 "305-03-3", "JCKYGMPEJWAADB-UHFFFAOYSA-N", "chlorambucil", "")
  CALL Species("C_{14}H_{20}ClNO_2",                 "34256-82-1", "VTNQPKFIQCLBDU-UHFFFAOYSA-N", "2-chloro-N-(ethoxymethyl)-N-(2-ethyl-6-methylphenyl)acetamide", "acetochlor")
  CALL Species("C_{14}H_{20}ClNO_2",                 "15972-60-8", "XCSGPAVHZFQHGE-UHFFFAOYSA-N", "alachlor", "")
  CALL Species("C_{14}H_9Cl_2NO_3",                  "42576-02-3", "SUSRORUBZHMPCO-UHFFFAOYSA-N", "bifenox", "")
  CALL Species("C_{15}H_{22}ClNO_2",                 "87392-12-9", "WVQBLGZPHOPPFO-LBPRGKRZSA-N", "$S$-metolachlor", "")
  CALL Species("C_{15}H_{22}ClNO_2",                 "51218-45-2", "WVQBLGZPHOPPFO-UHFFFAOYSA-N", "metolachlor", "")
  CALL Species("C_{15}H_{15}Cl_2N_3O_5",              "1420-04-8", "XYCDHXSQODHSLG-UHFFFAOYSA-N", "clonitralid", "")
  CALL Species("C_{15}H_{15}ClN_2O_2",                "1982-47-4", "IVUXTESCPZUGJC-UHFFFAOYSA-N", "chloroxuron", "")
  CALL Species("C_{15}H_{18}ClNO_4",                 "67932-85-8", "FHZMAAGOSXDIBJ-UHFFFAOYSA-N", "CGA 80000", "") ! other CAS: 79555-80-9
  CALL Species("C_{15}H_{14}N_2O_3Cl_2",             "39807-15-3", "DVOODWOZJVJKQR-UHFFFAOYSA-N", "oxadiargyl", "")
  CALL Species("C_{15}H_{17}Cl_2N_3O",               "83657-24-3", "FBOUIAKEJMZPQG-UHFFFAOYSA-N", "diniconazole", "") ! keto tautomer = NSLVZQNHOCYSEE-UHFFFAOYSA-N
  CALL Species("C_{16}H_{22}ClNO_2",                "106700-29-2", "CSWIKHNSBZVWNQ-UHFFFAOYSA-N", "pethoxamid", "")
  CALL Species("C_{16}H_{10}ClN_3O_3",                "2814-77-9", "XLTMWFMRJZDFFD-VHEBQXMUSA-N", "pigment red 4", "")
  CALL Species("C_{17}H_{15}Cl_2N_5O_2",             "13301-61-6", "KHZRTXVUEZJYNE-UHFFFAOYSA-N", "darendoside b", "")
  CALL Species("C_{17}H_{24}NO_2Cl",                 "87310-56-3", "HZDIJTXDRLNTIS-DAXSKMNVSA-N", "butenachlor", "")
  CALL Species("C_{17}H_{26}ClNO_2",                 "23184-66-9", "HKPHPIREJKHECO-UHFFFAOYSA-N", "butachlor", "")
  CALL Species("C_{17}H_{26}ClNO_2",                 "51218-49-6", "YLPGTOIOYRQOHV-UHFFFAOYSA-N", "pretilachlor", "")
  CALL Species("C_{18}H_{19}ClN_2O_2",              "112226-61-6", "CNKHSLKYRMDDNQ-UHFFFAOYSA-N", "halofenozide", "")
  CALL Species("C_{18}H_{18}ClNO_5",                 "29104-30-1", "BZMIHNKNQJJVRO-UHFFFAOYSA-N", "benzoximate", "")
  CALL Species("C_{19}H_{21}ClN_2O",                 "66063-05-6", "OGYFATSSENRIKG-UHFFFAOYSA-N", "pencycuron", "")
  CALL Species("C_{19}H_{27}ClN_2O_5",              "283159-90-0", "DBXFMOWZRXXBRN-LWKPJOBUSA-N", "valifenalate", "")
  CALL Species("C_{20}H_{16}Cl_2N_2O_3",             "71561-11-0", "FKERUJTUOYLBKB-UHFFFAOYSA-N", "pyrazoxyfen", "")
  CALL Species("C_{22}H_{19}Cl_2NO_3",               "67375-30-8", "KAATUXNTWXVJKI-DXCJPMOASA-N", "$\alpha$-cypermethrin", "")
  CALL Species("C_{22}H_{19}Cl_2NO_3",             "1224510-29-5", "KAATUXNTWXVJKI-FLXSOZOKSA-N", "cypermethrin-$\beta$", "") ! similar name as beta-cypermethrin
  CALL Species("C_{22}H_{19}Cl_2NO_3",               "65731-84-2", "KAATUXNTWXVJKI-NSHGMRRFSA-N", "$\beta$-cypermethrin", "") ! similar name as cypermethrin-beta
  CALL Species("C_{22}H_{19}Cl_2NO_3",               "52315-07-8", "KAATUXNTWXVJKI-UHFFFAOYSA-N", "$\delta$-cypermethrin", "cypermethrin; alphamethrin")
  CALL Species("C_{22}H_{19}Cl_2NO_3",               "71697-59-1", "KAATUXNTWXVJKI-GGPKGHCWSA-N", "$\theta$-cypermethrin", "")
  CALL Species("C_{22}H_{19}Cl_2NO_3",             "1315501-18-8", "KAATUXNTWXVJKI-QPIRBTGLSA-N", "$\zeta$-cypermethrin", "")
  CALL Species("C_{22}H_{22}ClNO_6",                  "4865-85-4", "BPZZWRPHVVDAPT-PXAZEXFGSA-N", "ochratoxin C", "")
  CALL Species("C_{22}H_{21}N_2O_5Cl",              "119738-06-6", "BBKDWPHJZANJGB-UHFFFAOYSA-N", "quizalofop-p-tefuryl", "")
  CALL Species("C_{23}H_{22}ClNO_4",                "374726-62-2", "KWLVWJPJKJMCSH-UHFFFAOYSA-N", "mandipropamid", "")
  CALL Species("C_{25}H_{22}ClNO_3",                 "51630-58-1", "NYPJDWWKZLNGGM-UHFFFAOYSA-N", "fenvalerate", "")
  CALL Species("C_{25}H_{22}ClNO_3",                 "66230-04-4", "NYPJDWWKZLNGGM-ZEQRLZLVSA-N", "esfenvalerate", "")
  ! heterocycles with N:
  CALL Species("C_6H_3Cl_2NO_2",                      "1702-17-6", "HUBANNPOLNYSAD-UHFFFAOYSA-N", "clopyralid", "")
  CALL Species("C_6H_3Cl_3N_2O_2",                    "1918-02-1", "NQQVFXUMIDALNH-UHFFFAOYSA-N", "picloram", "")
  CALL Species("C_6H_4Cl_2N_2O_2",                  "150114-71-9", "NIXXQNOQHKNPEJ-UHFFFAOYSA-N", "aminopyralid", "")
  CALL Species("C_6HCl_4NO_2",                       "10469-09-7", "GXFRQLQUKBSYQL-UHFFFAOYSA-N", "3,4,5,6-tetrachloropyridine-2-carboxylic acid", "")
  CALL Species("C_7H_4Cl_3NO_3",                     "55335-06-3", "REEQLXCGVXDJSQ-UHFFFAOYSA-N", "[(3,5,6-trichloro-2-pyridinyl)oxy]-acetic acid", "triclopyr")
  CALL Species("C_7H_7Cl_2NO",                        "2971-90-6", "ZDPIZLCVJAAHHR-UHFFFAOYSA-N", "clopidol", "")
  CALL Species("C_8H_6Cl_3NO_3",                     "60825-26-5", "MNYBZEHWPRTNJY-UHFFFAOYSA-N", "[(3,5,6-trichloro-2-pyridinyl)oxy]-acetic acid, methyl ester", "")
  CALL Species("C_8H_8ClN_3O_2",                    "858956-08-8", "KWAIHLIXESXTJL-UHFFFAOYSA-N", "aminocyclopyrachlor", "")
  CALL Species("C_8H_{11}Cl_2N_3O_2",                   "66-75-1", "IDPUKCWIGUEADI-UHFFFAOYSA-N", "uracil mustard", "")
  CALL Species("C_9H_{10}ClN_5O_2",                 "138261-41-3", "YWTYJOPNNQFBPC-UHFFFAOYSA-N", "imidacloprid", "")
  CALL Species("C_9H_8Cl_3NO_3",                     "60825-27-6", "KXAVVWXJUDQGDA-UHFFFAOYSA-N", "ethyl [(3,5,6-trichloro-2-pyridinyl)oxy]acetate", "")
  CALL Species("C_9H_3Cl_4NO_2",                     "14737-80-5", "OHCSZUQRNNNMRG-UHFFFAOYSA-N", "N-methyl-3,4,5,6-tetrachlorophthalimide", "")
  CALL Species("C_9H_{13}ClN_2O_2",                   "5902-51-2", "NBQCNZYJJMBDKY-UHFFFAOYSA-N", "terbacil", "")
  CALL Species("C_9H_{14}ClN_5O_2",                   "6616-80-4", "YESXTECNXIKUMM-UHFFFAOYSA-N", "eglinazine-ethyl", "")
  CALL Species("C_{10}H_{16}ClN_5O_2",               "68228-18-2", "QQADVTSTCZBBOE-UHFFFAOYSA-N", "proglinazine ethyl ester", "")
  CALL Species("C_{10}H_{14}Cl_6N_4O_2",             "26644-46-2", "RROQIUMZODEXOR-UHFFFAOYSA-N", "triforine", "")
  CALL Species("C_{10}H_8ClN_3O_2",                   "5707-69-7", "OOTHTARUZHONSW-UHFFFAOYSA-N", "drazoxolon", "")
  CALL Species("C_{10}H_7Cl_2N_3O",                  "68475-42-3", "OTBXOEAOVRKTNQ-UHFFFAOYSA-N", "anagrelide", "")
  CALL Species("C_{11}H_6Cl_2N_2",                   "74738-17-3", "FKLFBQCQQYDUAM-UHFFFAOYSA-N", "fenpiclonil", "")
  CALL Species("C_{11}H_{11}Cl_2NO_2",               "98730-04-2", "PFJJMJDEVDLPNE-UHFFFAOYSA-N", "benoxacor", "")
  CALL Species("C_{11}H_{11}ClN_2O_2",               "27512-72-7", "GLPZEHFBLBYFHN-UHFFFAOYSA-N", "ethychlozate", "")
  CALL Species("C_{11}H_8ClNO_2",                    "90717-03-6", "ALZOLUNSQWINIR-UHFFFAOYSA-N", "quinmerac", "")
  CALL Species("C_{12}H_6Cl_3NO_3",                  "27541-88-4", "ZIEWAMOXCOLNSJ-UHFFFAOYSA-N", "quinonamid", "")
  CALL Species("C_{12}H_8N_3O_2Cl_5",               "103112-35-2", "GMBRUAIJEFRHFQ-UHFFFAOYSA-N", "fenchlorazole-ethyl", "")
  CALL Species("C_{12}H_9Cl_2NO_3",                  "50471-44-8", "FSCWZHGZWWDELK-UHFFFAOYSA-N", "vinclozoline", "vinclozolin")
  CALL Species("C_{12}H_{10}ClN_3O",                 "68157-60-8", "GPXLRLUVLMHHIK-UHFFFAOYSA-N", "forchlorfenuron", "")
  CALL Species("C_{12}H_{11}Cl_2N_3O_2",             "60207-31-0", "AKNQMEBLVAMSNZ-UHFFFAOYSA-N", "azaconazole", "")
  CALL Species("C_{12}H_{11}NO_4Cl_2",               "54864-61-8", "FTCOKXNKPOUEFH-UHFFFAOYSA-N", "myclozolin", "")
  CALL Species("C_{13}H_{16}Cl_3NO_4",               "64700-56-7", "IVDRCZNHVGQBHZ-UHFFFAOYSA-N", "triclopyr-butotyl", "")
  CALL Species("C_{13}H_{11}ClN_2O_3",              "129025-54-3", "PIZCXVUFSNPNON-UHFFFAOYSA-N", "clofencet", "")
  CALL Species("C_{13}H_{11}NO_5Cl_2",               "84332-86-5", "IGUYEXXAGBDLLX-UHFFFAOYSA-N", "chlozolinate", "")
  CALL Species("C_{13}H_{18}ClNO",                    "7287-36-7", "KXGYBSNVFXBPNO-UHFFFAOYSA-N", "monalide", "")
  CALL Species("C_{13}H_9Cl_2NO_4",                  "32861-85-1", "DXXVCXKMSWHGTF-UHFFFAOYSA-N", "chlomethoxyfen", "")
  CALL Species("C_{14}H_{16}ClN_3O",                 "67129-08-2", "STEPQTYSZVCJPV-UHFFFAOYSA-N", "metazachlor", "")
  CALL Species("C_{14}H_{15}Cl_2N_3O_2",             "60207-93-4", "DWRKFAJEBUWTQM-UHFFFAOYSA-N", "etaconazole", "")
  CALL Species("C_{14}H_{18}ClN_3O_2",               "55219-65-3", "BAZVSMNPJJMILC-UHFFFAOYSA-N", "triadimenol", "")
  CALL Species("C_{14}H_{17}Cl_2N_3O",               "79983-71-4", "STMIIPIFODONDC-UHFFFAOYSA-N", "hexaconazole", "")
  CALL Species("C_{14}H_{16}ClN_3O_2",               "43121-43-3", "WURBVZBTWMNKQT-UHFFFAOYSA-N", "triadimefon", "")
  CALL Species("C_{14}H_{14}Cl_2N_2O",               "35554-44-0", "PZBPKYOVPCNPJY-UHFFFAOYSA-N", "imazalil", "")
  CALL Species("C_{14}H_{12}Cl_2N_2O",               "88283-41-4", "CKPCAYZTYMHQEX-UHFFFAOYSA-N", "pyrifenox", "")
  CALL Species("C_{14}H_{12}ClNO_2",                 "39985-63-2", "MJQBFSWPMMHVSM-UHFFFAOYSA-N", "chlorphthalim", "")
  CALL Species("C_{14}H_{14}ClNO_3",                 "69581-33-5", "KRZUZYJEQBXUIN-UHFFFAOYSA-N", "cyprofuram", "")
  CALL Species("C_{14}H_{16}ClNO_3",                 "58810-48-3", "OWDLFBLNMPCXSD-UHFFFAOYSA-N", "ofurace", "")
  CALL Species("C_{14}H_{18}ClN_3O_2",               "82200-72-4",                            "", "triadimenol B", "") ! InChIKey = "BAZVSMNPJJMILC-??????????-N"
  CALL Species("C_{14}H_{18}ClN_3O_2",               "89482-17-7",                            "", "triadimenol A", "") ! InChIKey = "BAZVSMNPJJMILC-??????????-N"
  CALL Species("C_{15}H_{17}Cl_2N_3O_2",             "60207-90-1", "STJLVHWMYQXCPB-UHFFFAOYSA-N", "propiconazole", "")
  CALL Species("C_{15}H_{10}ClN_3O_3",                "1622-61-3", "DGBIGWXXNGSACT-UHFFFAOYSA-N", "clonazepam", "")
  CALL Species("C_{15}H_{12}ClN_2O_2",                 "604-75-1", "ADIMAYPTOBDMTL-UHFFFAOYSA-N", "oxazepam", "")
  CALL Species("C_{15}H_{18}Cl_2N_2O_3",             "19666-30-9", "CHNUNORXWHYHNE-UHFFFAOYSA-N", "oxadiazon", "")
  CALL Species("C_{15}H_{18}ClN_3O",                 "94361-06-5", "UFNOUKDBUJZYDE-UHFFFAOYSA-N", "cyproconazole", "")
  CALL Species("C_{15}H_{19}Cl_2N_3O",               "75736-33-3", "URDNHJIVMYZFRT-UHFFFAOYSA-N", "diclobutrazol", "")
  CALL Species("C_{15}H_{20}ClN_3O",                 "76738-62-0", "RMOGWMIKYWRTKW-UHFFFAOYSA-N", "paclobutrazol", "")
  CALL Species("C_{15}H_{16}Cl_3N_3O_2",             "67747-09-5", "TVLSRXXIMLFWEO-UHFFFAOYSA-N", "prochloraz", "")
  CALL Species("C_{15}H_{17}Cl_2N_3O_2",            "112721-87-6",                            "", "{\it cis}-propiconazole", "")   ! InChIKey = "STJLVHWMYQXCPB-??????????-N"
  CALL Species("C_{15}H_{17}Cl_2N_3O_2",            "120523-07-1",                            "", "{\it trans}-propiconazole", "") ! InChIKey = "STJLVHWMYQXCPB-??????????-N"
  CALL Species("C_{15}H_{18}Cl_2N_2O_2",            "115852-48-7", "IUOKJNROJISWRO-UHFFFAOYSA-N", "fenoxanil", "")
  CALL Species("C_{15}H_{18}Cl_3NO",                "104030-54-8", "RXDMAYSSBPYBFW-UHFFFAOYSA-N", "carpropamid", "")
  CALL Species("C_{15}H_{22}ClNO_2",                 "86763-47-5", "KZNDFYDURHAESM-UHFFFAOYSA-N", "propisochlor", "")
  CALL Species("C_{16}H_{18}Cl_2N_2O_4",            "135590-91-9", "OPGCOAPTHCZZIW-UHFFFAOYSA-N", "mefenpyr-diethyl", "")
  CALL Species("C_{16}H_{13}ClN_2O",                   "439-14-5", "AAOVKJBEBIDNHE-UHFFFAOYSA-N", "diazepam", "")
  CALL Species("C_{16}H_{21}Cl_2N_3O_2",             "16506-27-7", "YTKUWDBFDASYHO-UHFFFAOYSA-N", "bendamustine", "")
  CALL Species("C_{16}H_{20}ClN_5O_2",              "158237-07-1", "LLQPHQFNMLZJMP-UHFFFAOYSA-N", "fentrazamide", "")
  CALL Species("C_{16}H_{21}Cl_2NO_2",                "3478-94-2", "BZGLBXYQOMFXAU-UHFFFAOYSA-N", "piperalin", "")
  CALL Species("C_{16}H_{22}ClN_3O",                "107534-96-3", "PXMNMQRDXWABCY-UHFFFAOYSA-N", "tebuconazole", "")
  CALL Species("C_{16}H_{22}ClNO_3",                 "38727-55-8", "WFKSADNZWSKCRZ-UHFFFAOYSA-N", "diethatyl-ethyl", "")
  CALL Species("C_{17}H_{12}Cl_2N_2O",               "60168-88-9", "NHOWDZOIZKMVAI-UHFFFAOYSA-N", "fenarimol", "")
  CALL Species("C_{17}H_{19}ClN_2O",                 "99485-76-4", "VYNOULHXXDFBLU-UHFFFAOYSA-N", "cumyluron", "")
  CALL Species("C_{17}H_{20}N_3OCl",                "131983-72-7", "PPDBOQMNKNNODG-ZROIWOOFSA-N", "triticonazole", "")
  CALL Species("C_{17}H_{20}ClN_3O_2",              "123572-88-3", "NRTLIYOWLVMQBO-UHFFFAOYSA-N", "furametpyr", "")
  CALL Species("C_{17}H_{22}ClN_3O",                "125116-23-6", "XWPZUHJBOLQNMN-UHFFFAOYSA-N", "metconazole", "")
  CALL Species("C_{17}H_{24}ClNO_4",                "149979-41-9", "IOYNQIMAUDJVEI-BMVIKAAMSA-N", "tepraloxydim", "")
  CALL Species("C_{18}H_{22}ClNO_3",                 "99607-70-2", "COYBRKAVBMYYSF-UHFFFAOYSA-N", "cloquintocet-mexyl", "")
  CALL Species("C_{18}H_{16}ClNO_5",                 "71283-80-2", "PQKBPHSEKWERTG-LLVKDONJSA-N", "fenoxaprop-p-ethyl", "")
  CALL Species("C_{18}H_{24}ClN_3O",                "125225-28-7", "QTYCMDBMOLSEAM-UHFFFAOYSA-N", "ipconazole", "")
  CALL Species("C_{18}H_{20}ClNO_5",                "688046-61-9", "NMVCBWZLCXANER-UHFFFAOYSA-N", "pyriofenone", "")
  CALL Species("C_{18}H_{15}ClN_2O_5",              "130561-48-7", "QLMNCUHSDAGQGT-UHFFFAOYSA-N", "cintofen", "sintofen")
  CALL Species("C_{18}H_{12}Cl_2N_2O",              "188425-85-6", "WYEMLYFITZORAB-UHFFFAOYSA-N", "boscalid", "")
  CALL Species("C_{18}H_{24}ClN_3O",                "119168-77-3", "ZZYSLNWGKKDOML-UHFFFAOYSA-N", "tebufenpyrad", "")
  CALL Species("C_{18}H_6Cl_4N_2O",                  "20749-68-2", "UBZVRROHBDDCQY-UHFFFAOYSA-N", "8,9,10,11-tetrachloro-12-phthaloperinone", "")
  CALL Species("C_{18}H_{10}Cl_2N_2O_2",             "84632-65-5", "JNNHVXMCVRYTTN-UHFFFAOYSA-N", "pigment red 254", "")
  CALL Species("C_{18}H_{16}ClNO_5",                 "66441-23-4", "PQKBPHSEKWERTG-UHFFFAOYSA-N", "fenoxaprop-ethyl", "")
  CALL Species("C_{18}H_{17}Cl_2NO_3",               "22212-55-1", "SLCGUGMPSUYJAY-UHFFFAOYSA-N", "benzoylprop-ethyl", "")
  CALL Species("C_{19}H_{17}Cl_2N_3O_3",            "119446-68-3", "BQYJATMQXGBDHF-UHFFFAOYSA-N", "difenoconazole", "")
  CALL Species("C_{19}H_{17}ClN_2O_4",               "76578-14-8", "OSUHJPCHFDQAIT-UHFFFAOYSA-N", "quizalofop ethyl", "")
  CALL Species("C_{19}H_{17}ClN_2O_4",              "100646-51-3", "OSUHJPCHFDQAIT-GFCCVEGCSA-N", "quizalofop-p-ethyl", "") ! isomer mix = 76578-14-8
  CALL Species("C_{19}H_{18}ClN_3O_4",              "175013-18-0", "HZRSNVGNWUDEFX-UHFFFAOYSA-N", "pyraclostrobine", "")
  CALL Species("C_{20}H_{28}ClN_3O_2",              "105779-78-0", "ITKAIUGKVKDENI-UHFFFAOYSA-N", "pyrimidifen", "")
  CALL Species("C_{21}H_8Cl_3NO_3",                   "6373-31-5", "SQAQTWYUQXFOMH-UHFFFAOYSA-N", "ponsol red violet 2rnx", "")
  CALL Species("C_{21}H_{22}NO_4Cl",                "110488-70-5", "QNBTYORWCCMPQP-JXAWBTAJSA-N", "dimethomorph", "")
  CALL Species("C_{22}H_{22}ClN_3O_5",              "111479-05-1", "FROBCXTULYFHEJ-UHFFFAOYSA-N", "propaquizafop", "")
  CALL Species("C_{23}H_{27}Cl_2N_3O_2",            "129722-12-9", "CEUORZQYGODEFX-UHFFFAOYSA-N", "aripiprazole", "")
  CALL Species("C_{26}H_6Cl_8N_2O_4",                "30125-47-4", "ZEHOVWPIGREOPO-UHFFFAOYSA-N", "ag-g-86814", "")
  CALL Species("C_{34}H_{22}Cl_2N_4O_2",              "6358-30-1", "CGLVZFOCZLHKOH-UHFFFAOYSA-N", "8,18-dichloro-5,15-diethyl-5,15-dihydrodiindolo(3,2-b:3',2'-m)triphenodioxazine", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Chlorofluorocarbons (\chem{C}, \chem{H}, \chem{O}, \chem{N}, \chem{F}, \chem{Cl})")
  ! C, H, F, Cl:
  CALL Species("CH_2FCl",                              "593-70-4", "XWCDCDSDNJVCLO-UHFFFAOYSA-N", "chlorofluoromethane", "R31")
  CALL Species("CHF_2Cl",                               "75-45-6", "VOPWNXZWBYDODV-UHFFFAOYSA-N", "chlorodifluoromethane", "R22")
  CALL Species("CHFCl_2",                               "75-43-4", "UMNKXPULIDJLSU-UHFFFAOYSA-N", "dichlorofluoromethane", "R21")
  CALL Species("CF_3Cl",                                "75-72-9", "AFYPFACVUDMOHA-UHFFFAOYSA-N", "chlorotrifluoromethane", "R13")
  CALL Species("CF_2Cl_2",                              "75-71-8", "PXBRQCKWGAHEHS-UHFFFAOYSA-N", "dichlorodifluoromethane", "R12")
  CALL Species("CFCl_3",                                "75-69-4", "CYRMSUTZVYGINF-UHFFFAOYSA-N", "trichlorofluoromethane", "R11")
  CALL Species("C_2Cl_4F_2",                            "76-11-9", "SLGOCMATMKJJCE-UHFFFAOYSA-N", "1,1,1,2-tetrachlorodifluoroethane", "")
  CALL Species("C_2F_2Cl_4",                            "76-12-0", "UGCSPKPEHQEOSR-UHFFFAOYSA-N", "1,1,2,2-tetrachlorodifluoroethane", "R112")
  CALL Species("C_2F_3Cl_3",                           "354-58-5", "BOSAWIQFTJIYIS-UHFFFAOYSA-N", "1,1,1-trichloro-2,2,2-trifluoroethane", "R113a")
  CALL Species("C_2F_3Cl_3",                            "76-13-1", "AJDIZQLSFPQPEY-UHFFFAOYSA-N", "1,1,2-trichloro-1,2,2-trifluoroethane", "R113")
  CALL Species("C_2F_4Cl_2",                           "374-07-2", "BAMUEXIPKSRTBS-UHFFFAOYSA-N", "1,1-dichlorotetrafluoroethane", "R114a")
  CALL Species("C_2F_4Cl_2",                            "76-14-2", "DDMOUSALMHHKOS-UHFFFAOYSA-N", "1,2-dichlorotetrafluoroethane", "R114")
  CALL Species("C_2F_5Cl",                              "76-15-3", "RFCAUADVODFSLZ-UHFFFAOYSA-N", "chloropentafluoroethane", "R115")
  CALL Species("C_2HCl_4F",                            "354-14-3", "LUBCGHUOCJOIJA-UHFFFAOYSA-N", "1,1,2,2-tetrachloro-1-fluoroethane", "")
  CALL Species("C_2HCl_2F_3",                          "812-04-4", "AFTSHZRGGNMLHC-UHFFFAOYSA-N", "1,1-dichloro-1,2,2-trifluoroethane", "")
  CALL Species("C_2HCl_2F_3",                          "354-23-4", "YMRMDGSNYHCUCL-UHFFFAOYSA-N", "1,2-dichloro-1,1,2-trifluoroethane", "")
  CALL Species("C_2HF_3Cl_2",                          "306-83-2", "OHMHBGPWCHTMQE-UHFFFAOYSA-N", "2,2-dichloro-1,1,1-trifluoroethane", "R123")
  CALL Species("C_2HClF_4",                            "354-25-6", "JQZFYIGAYWLRCC-UHFFFAOYSA-N", "1-chloro-1,1,2,2-tetrafluoroethane", "")
  CALL Species("C_2HF_4Cl",                           "2837-89-0", "BOUGCJDAQLKBQH-UHFFFAOYSA-N", "1-chloro-1,2,2,2-tetrafluoroethane", "R124")
  CALL Species("C_2H_2Cl_2F_2",                       "1649-08-7", "SKDFWEPBABSFMG-UHFFFAOYSA-N", "1,2-dichloro-1,1-difluoroethane", "")
  CALL Species("C_2H_2F_3Cl",                           "75-88-7", "CYXIKYKBLDZZNW-UHFFFAOYSA-N", "2-chloro-1,1,1-trifluoroethane", "R133a")
  CALL Species("CH_3CFCl_2",                          "1717-00-6", "FRCHKSNAZZFGCA-UHFFFAOYSA-N", "1,1-dichloro-1-fluoroethane", "R141b")
  CALL Species("CH_3CF_2Cl",                            "75-68-3", "BHNZEZWIUMJCGF-UHFFFAOYSA-N", "1-chloro-1,1-difluoroethane", "R142b")
  CALL Species("C_2H_3ClF_2",                          "338-64-7", "UOVSDUIHNGNMBZ-UHFFFAOYSA-N", "1-chloro-1,2-difluoroethane", "")
  CALL Species("C_2H_2F_3Cl",                          "421-04-5", "HILNUELUDBMBJQ-UHFFFAOYSA-N", "1-chloro-1,1,2-trifluoroethane", "R133b")
  CALL Species("C_2HClF_2",                            "359-10-4", "HTHNTJCVPNKCPZ-UHFFFAOYSA-N", "2-chloro-1,1-difluoroethene", "R1122")
  CALL Species("C_2ClF_3",                              "79-38-9", "UUAGAQFQZIEFAH-UHFFFAOYSA-N", "chlorotrifluoroethene", "")
  CALL Species("CF_3CF_2CHCl_2",                       "422-56-0", "COAUHYBSXMIJDK-UHFFFAOYSA-N", "3,3-dichloro-1,1,1,2,2-pentafluoropropane", "R225ca")
  CALL Species("CClF_2CF_2CHClF",                      "507-55-1", "UJIGKESMIPTWJH-UHFFFAOYSA-N", "1,3-dichloro-1,1,2,2,3-pentafluoro-propane", "R225cb")
  CALL Species("C_6H_4ClF",                            "348-51-6", "ZCJAYDKWZAWMPR-UHFFFAOYSA-N", "1-chloro-2-fluorobenzene", "")
  CALL Species("C_6H_4ClF",                            "625-98-9", "VZHJIJZEOCBKRA-UHFFFAOYSA-N", "1-chloro-3-fluorobenzene", "")
  CALL Species("C_7H_4ClF_3",                           "98-15-7", "YTCGOUNVIAWCMG-UHFFFAOYSA-N", "1-chloro-3-(trifluoromethyl)benzene", "")
  CALL Species("C_7H_4ClF_3",                           "98-56-6", "QULYNCCPRWKEMF-UHFFFAOYSA-N", "1-chloro-4-(trifluoromethyl)benzene", "")
  CALL Species("C_7H_3ClF_4",                        "78068-85-6", "BKHVEYHSOXVAOP-UHFFFAOYSA-N", "3-chloro-4-fluorobenzotrifluoride", "")
  CALL Species("C_7H_3Cl_2F_3",                        "328-84-7", "XILPLWOGHPSJBK-UHFFFAOYSA-N", "3,4-dichlorobenzotrifluoride", "")
  ! C, H, O, F, Cl:
  CALL Species("CF_2ClCOOH",                            "76-04-0", "OAWAZQITIZDJRB-UHFFFAOYSA-N", "chlorodifluoroethanoic acid", "chlorodifluoroacetic acid")
  CALL Species("COFCl",                                "353-49-1", "OXVVNXMNLYYMOL-UHFFFAOYSA-N", "carbonic chloride fluoride", "")
  CALL Species("CF_3COCl",                             "354-32-5", "PNQBEPDZQUOCNY-UHFFFAOYSA-N", "trifluoroacetylchloride", "")
  CALL Species("C_3H_4Cl_2F_2O",                        "76-38-0", "RFKMCNOHBTXSMU-UHFFFAOYSA-N", "2,2-dichloro-1,1-difluoro-1-methoxyethane", "methoxyflurane")
  CALL Species("C_3H_2ClF_5O",                       "33018-78-9", "HPDPVPUXPBZBOJ-UHFFFAOYSA-N", "2-(chlorodifluoromethoxy)-1,1,1-trifluoroethane", "")
  CALL Species("C_3H_2ClF_5O",                       "56885-28-0", "GGEBMRHBGQLKGJ-UHFFFAOYSA-N", "2-(chlorofluoromethoxy)-1,1,1,2-tetrafluoroethane", "")
  CALL Species("C_3H_2ClF_5O",                       "26675-46-7", "PIWKPBJCKXDKJR-UHFFFAOYSA-N", "1-chloro-2,2,2-trifluoroethyl difluoromethyl ether", "forane; isoflurane")
  CALL Species("C_3H_2ClF_5O",                       "13838-16-9", "JPGQOUSTVILISH-UHFFFAOYSA-N", "2-chloro-1,1,2-trifluoroethyl difluoromethyl ether", "enflurane")
  CALL Species("C_{14}H_8ClF_3O_3",                  "63734-62-3", "ONKRUAQFUNKYAX-UHFFFAOYSA-N", "3-[2-chloro-4-(trifluoromethyl)phenoxy]benzoic acid", "")
  CALL Species("C_{15}H_{10}ClF_3O_3",               "50594-77-9", "KNFRYRRZMRETJX-UHFFFAOYSA-N", "3-(2-chloro-4-(trifluoromethyl)phenoxy)phenyl acetate", "")
  CALL Species("C_{15}H_{12}Cl_2F_4O_2",            "118712-89-3", "DDVNRFNDOPPVQJ-HQJQHLMTSA-N", "transfluthrin", "")
  CALL Species("C_{17}H_{14}O_2ClF_7",               "79538-32-2", "ZFHGXWPMULPQSE-GPCIZFCYSA-N", "tefluthrin", "")
  CALL Species("C_{22}H_{28}ClFO_4",                 "25122-41-2", "FCSHDIVRCWTZOX-DVTGEIKXSA-N", "clobetasol", "")
  CALL Species("C_{24}H_{22}ClF_3O_3",              "107713-58-6", "RURQAJURNPMSSK-UHFFFAOYSA-N", "flufenprox", "")
  CALL Species("C_{28}H_{33}Cl_3F_6O_{11}",          "68954-01-8", "KVXVLCBFSKMBAR-UHFFFAOYSA-N", "EINECS 273-236-7", "")
  ! C, H, N, F, Cl:
  CALL Species("C_5Cl_2F_3N",                         "1737-93-5", "PKSORSNCSXBXOT-UHFFFAOYSA-N", "3,5-dichloro-2,4,6-trifluoropyridine", "")
  ! C, H, O, N, F, Cl:
  CALL Species("CClF_2OONO_2",                       "70490-95-8", "HWHZTSGXJYKIPK-UHFFFAOYSA-N", "chlorodifluoronitrooxymethane", "")
  CALL Species("C_7H_3ClF_3NO_2",                      "121-17-5", "TZGFQIXRVUHDLE-UHFFFAOYSA-N", "1-chloro-2-nitro-4-(trifluoromethyl)-benzene", "")
  CALL Species("C_7H_2ClF_3N_2O_4",                    "393-75-9", "HFHAVERNVFNSHL-UHFFFAOYSA-N", "2-chloro-1,3-dinitro-5-(trifluoromethyl)-benzene", "")
  CALL Species("C_7H_5Cl_2FN_2O_3",                  "69377-81-7", "MEFQWPUMEMWTJP-UHFFFAOYSA-N", "fluroxypyr", "")
  CALL Species("C_{10}H_4Cl_2FNO_2",                 "41205-21-4", "IPENDKRRWFURRE-UHFFFAOYSA-N", "fluoroimide", "")
  CALL Species("C_{12}H_9ClF_3N_3O",                 "27314-13-2", "NVGOPFQZYCNLDU-UHFFFAOYSA-N", "norflurazon", "")
  CALL Species("C_{12}H_{13}ClF_3N_3O_4",            "33245-39-5", "MNFMIVVPXOGUMX-UHFFFAOYSA-N", "fluchloralin", "")
  CALL Species("C_{12}H_{10}Cl_2F_3NO",              "61213-25-0", "OQZCSNDVOWYALR-UHFFFAOYSA-N", "flurochloridone", "")
  CALL Species("C_{12}H_{11}ClF_2N_2O_2",           "951659-40-8", "QOIYTRGFOFZNKF-UHFFFAOYSA-N", "flupyradifurone", "")
  CALL Species("C_{12}H_{11}NO_3ClF_3",              "88485-37-4", "UKSLKNUCVPZQCQ-GZTJUZNOSA-N", "fluxofenim", "")
  CALL Species("C_{13}H_4Cl_2F_6N_4O_4",             "79622-59-6", "UZCGKGPEKUCDTF-UHFFFAOYSA-N", "fluazinam", "")
  CALL Species("C_{13}H_7ClF_3NO_4",                 "42874-63-5", "WYTRKEWETULQOA-UHFFFAOYSA-N", "5-(2-chloro-4-(trifluoromethyl)phenoxy)-2-nitrophenol", "")
  CALL Species("C_{13}H_8Cl_2F_2N_2O_3",            "943832-81-3", "XFZUQTKDBCOXPP-UHFFFAOYSA-N", "florpyrauxifen", "")
  CALL Species("C_{13}H_{11}Cl_2F_4N_3O",           "112281-77-3", "LQDARGUHUSPFNL-UHFFFAOYSA-N", "tetraconazole", "")
  CALL Species("C_{14}H_8Cl_3F_3N_2O",              "239110-15-7", "GBOYJIHYACSLGN-UHFFFAOYSA-N", "fluopicolide", "")
  CALL Species("C_{14}H_9ClF_2N_2O_2",               "35367-38-5", "QQQYTWIFVNKMRW-UHFFFAOYSA-N", "diflubenzuron", "difluron")
  CALL Species("C_{14}H_9ClF_3NO_2",                "154598-52-4", "XPOQHMRABVBWPR-ZDUSSCGKSA-N", "efavirenz", "")
  CALL Species("C_{14}H_{11}Cl_2FN_2O_3",           "943831-98-9", "KDHKOPYYWOHESS-UHFFFAOYSA-N", "halauxifen-methyl", "")
  CALL Species("C_{14}H_{19}Cl_2FN_2O_4",           "154486-27-8", "ZKFARSBUEBZZJT-UHFFFAOYSA-N", "fluroxypyr-butometyl", "")
  CALL Species("C_{14}H_6N_2O_2Cl_2F_4",             "83121-18-0", "CJDWRQLODFKPEL-UHFFFAOYSA-N", "teflubenzuron", "")
  CALL Species("C_{15}H_{21}Cl_2FN_2O_3",            "81406-37-3", "OLZQTUCTGLHFTQ-UHFFFAOYSA-N", "fluroxypyr-meptyl", "attain A")
  CALL Species("C_{15}H_8Cl_2FNO",                  "124495-18-7", "WRPIRSINYZBGPK-UHFFFAOYSA-N", "quinoxyfen", "")
  CALL Species("C_{15}H_9ClF_3NO_5",                 "50594-44-0", "PSWSPFSDVZVVDC-UHFFFAOYSA-N", "5-(2-chloro-4-(trifluoromethyl)phenoxy)-2-nitrophenyl acetate", "")
  CALL Species("C_{15}H_{10}ClF_3N_2O_3",            "64628-44-0", "XAIPTRIXGHTTNT-UHFFFAOYSA-N", "triflumuron", "")
  CALL Species("C_{15}H_{11}ClF_3NO_4",              "42874-03-3", "OQMBBFQZGJFLBU-UHFFFAOYSA-N", "oxyfluorfen", "")
  CALL Species("C_{15}H_{13}Cl_2F_3N_2O_4",         "129630-19-9", "APTZNLHMIGJTEW-UHFFFAOYSA-N", "pyraflufen-ethyl", "")
  CALL Species("C_{15}H_{14}Cl_2F_3N_3O_2",         "112839-32-4", "ULCWZQJLFZEXCS-KGLIPLIRSA-N", "{\it cis}-furconazole", "") ! maybe InChIKey = "ULCWZQJLFZEXCS-KGLIPLIRSA-N"
  CALL Species("C_{15}H_{14}Cl_2F_3N_3O_3",         "128639-02-1", "MLKCGVHIFJBRCD-UHFFFAOYSA-N", "carfentrazone ethyl", "")
  CALL Species("C_{15}H_{15}ClF_3N_3O",              "68694-11-1", "HSMVPDGQOIQYSR-UHFFFAOYSA-N", "triflumizole", "") ! other CAS: 99387-89-0
  CALL Species("C_{15}H_{16}N_3OClF_2",             "130339-07-0", "NEKULYKCZPJMMJ-UHFFFAOYSA-N", "diflumetorim", "")
  CALL Species("C_{16}H_8Cl_2FN_5O",                "136426-54-5", "IJJVMEJXYNJXOJ-UHFFFAOYSA-N", "3-(2,4-dichlorophenyl)-6-fluoro-2-(1H-1,2,4-triazol-1-yl)-quinazolin-4(3H)-one", "fluquinconazole")
  CALL Species("C_{16}H_8Cl_2F_6N_2O_3",             "86479-06-3", "RGNPBRKPHBKNKX-UHFFFAOYSA-N", "hexaflumuron", "")
  CALL Species("C_{16}H_{11}ClF_6N_2O",             "658066-35-4", "KVDJTXBXMWJJEF-UHFFFAOYSA-N", "fluopyram", "")
  CALL Species("C_{16}H_{13}ClF_3NO_4",              "72619-32-0", "MFSWTRQUCLNFOM-SECBINFHSA-N", "haloxyfop-p-methyl", "")
  CALL Species("C_{16}H_{12}ClF_4N_3O_4",            "62924-70-3", "PWNAWOCHVWERAR-UHFFFAOYSA-N", "flumetralin", "")
  CALL Species("C_{16}H_{13}ClF_3NO_4",              "69806-40-2", "MFSWTRQUCLNFOM-UHFFFAOYSA-N", "haloxyfop-methyl", "")
  CALL Species("C_{16}H_{16}Cl_3F_2N_3O_2",        "1228284-64-7", "DGOAXBPOVUPPEB-UHFFFAOYSA-N", "pydiflumetofen", "")
  CALL Species("C_{17}H_{12}ClFN_2O",                "63284-71-9", "SAPGTCDSBGMXCD-UHFFFAOYSA-N", "nuarimol", "")
  CALL Species("C_{17}H_{13}ClFNO_4",               "105512-06-9", "JBDHZKLJNAIJNC-LLVKDONJSA-N", "clodinafop-propargyl", "")
  CALL Species("C_{17}H_{15}ClFNO_3",                "52756-25-9", "RBNIGDFIUWJJEV-UHFFFAOYSA-N", "flamprop-methyl", "")
  CALL Species("C_{18}H_{14}Cl_4F_3NO_3",           "179101-81-6", "UBSUICNYBBPFHD-UHFFFAOYSA-N", "pyridalyl", "")
  CALL Species("C_{18}H_{15}ClF_3N_3O_2",          "1417782-03-6", "JERZEQUMJNCPRJ-UHFFFAOYSA-N", "mefentrifluconazole", "")
  CALL Species("C_{18}H_{12}N_3OCl_2F_3",           "581809-46-3", "LDLMOOXUCMHBMZ-UHFFFAOYSA-N", "bixafen", "")
  CALL Species("C_{18}H_{15}Cl_2F_2N_3O",          "1072957-71-1", "CCCGEKHKTPTUHJ-UHFFFAOYSA-N", "benzovindiflupyr", "")
  CALL Species("C_{19}H_{15}ClF_3NO_7",              "77501-63-4", "CONWAEURSVPLRM-UHFFFAOYSA-N", "lactofen", "")
  CALL Species("C_{19}H_{14}N_4O_2ClF_5",           "119126-15-7", "AOQMRUTZEYVDIL-UHFFFAOYSA-N", "flupoxam", "")
  CALL Species("C_{19}H_{19}NO_3ClF",                "63782-90-1", "IKVXBIIHQGXQRQ-CYBMUJFWSA-N", "flamprop-m-isopropyl", "")
  CALL Species("C_{19}H_{19}ClF_3NO_5",              "87237-48-7", "MIJLZGZLQLAQCM-UHFFFAOYSA-N", "haloxyfop-ethoxyethyl", "")
  CALL Species("C_{20}H_9Cl_3F_5N_3O_3",             "71422-67-8", "UISUNVFOGSJSKD-UHFFFAOYSA-N", "chlorfluazuron", "")
  CALL Species("C_{20}H_{10}N_3O_3Cl_2F_5",          "86811-58-7", "YOWNVPAUWYHLQX-UHFFFAOYSA-N", "fluazuron", "")
  CALL Species("C_{21}H_{11}ClF_6N_2O_3",           "101463-69-8", "RYLHNOVXKPXDIP-UHFFFAOYSA-N", "flufenoxuron", "")
  CALL Species("C_{21}H_{16}N_4O_5ClF",             "361377-29-9", "UFEODZBUAFNAEU-NLRVBDNBSA-N", "fluoxastrobin", "")
  CALL Species("C_{21}H_{23}ClFNO_2",                   "52-86-8", "LNEPOXFFQSENCJ-UHFFFAOYSA-N", "haloperidol", "")
  CALL Species("C_{22}H_{17}ClF_3N_3O_7",           "173584-44-6", "VBCVPMMZEGZULK-NRFANRHFSA-N", "indoxacarb", "") ! enantiomer mix = 144171-61-9
  CALL Species("C_{22}H_{18}Cl_2FNO_3",              "68359-37-5", "QQODLKZGRKWIFG-UHFFFAOYSA-N", "cyfluthrin", "")
  CALL Species("C_{22}H_{18}Cl_2FNO_3",              "86560-92-1", "QQODLKZGRKWIFG-QSFXBCCZSA-N", "cyfluthrin I", "")
  CALL Species("C_{22}H_{18}Cl_2FNO_3",              "86560-93-2", "QQODLKZGRKWIFG-XFQXTVEOSA-N", "$\beta$-cyfluthrin (cis)", "")
  CALL Species("C_{22}H_{18}Cl_2FNO_3",              "86560-94-3", "QQODLKZGRKWIFG-BPAFIMBUSA-N", "cyfluthrin II", "")
  CALL Species("C_{22}H_{18}Cl_2FNO_3",              "86560-95-4", "QQODLKZGRKWIFG-QKYXUNIQSA-N", "$\beta$-cyfluthrin (trans)", "")
  CALL Species("C_{23}H_{18}N_2O_4ClF",             "256412-89-2", "ADDQHLREJDZPMT-CQSZACIVSA-N", "metamifop", "")
  CALL Species("C_{23}H_{18}Cl_2F_3N_3O_4",        "1257598-43-8", "YYXSCUSVVALMNW-FOWTUZBSSA-N", "pyriminostrobin", "")
  CALL Species("C_{23}H_{19}NO_3ClF_3",              "68085-85-8", "ZXQYGBMAQZUVMI-BWHPXCRDSA-N", "cyhalothrin", "")
  CALL Species("C_{23}H_{22}ClF_3O_2",               "82657-04-3", "OMFRMAHOUUJSGP-UNOMPAQXSA-N", "bifenthrin", "")
  CALL Species("C_{23}H_{19}ClF_3NO_3",              "91465-08-6", "DFVKOWFGNASVPK-QQDHXZELSA-N", "$\lambda$-cyhalothrin", "")
  CALL Species("C_{23}H_{19}ClF_3NO_3",              "76703-62-3", "ZXQYGBMAQZUVMI-GCMPRSNUSA-N", "$\gamma$-cyhalothrin", "")
  CALL Species("C_{25}H_{20}ClF_2N_3O_3",            "94050-52-9", "PCKNFPQPGUWFHO-UQRQXUALSA-N", "flucycloxuron", "")
  CALL Species("C_{26}H_{22}ClF_3N_2O_3",           "102851-06-9", "INISTDXBRIBGOC-XMMISQBUSA-N", "$\tau$-fluvalinate", "")
  CALL Species("C_{26}H_{22}ClF_3N_2O_3",            "69409-94-5", "INISTDXBRIBGOC-UHFFFAOYSA-N", "fluvalinate", "")
  !***************************************************************************
  CALL SpeciesGroup(1,"Organic species with bromine (\chem{Br})")
  !***************************************************************************
  CALL SpeciesGroup(2,"Bromocarbons (\chem{C}, \chem{H}, \chem{O}, \chem{N}, \chem{Br})")
  ! C, H, Br:
  ! saturated:
  CALL Species("CH_3Br",                                "74-83-9", "GZUXJHMPEANEGY-UHFFFAOYSA-N", "bromomethane", "methyl bromide")
  CALL Species("CH_2Br_2",                              "74-95-3", "FJBFPHVGVWTDIP-UHFFFAOYSA-N", "dibromomethane", "")
  CALL Species("CHBr_3",                                "75-25-2", "DIKBFYAXUHHXCS-UHFFFAOYSA-N", "tribromomethane", "bromoform")
  CALL Species("CBr_4",                                "558-13-4", "HJUGFYREWKUQJT-UHFFFAOYSA-N", "tetrabromomethane", "")
  CALL Species("C_2H_5Br",                              "74-96-4", "RDHPKYGYEGBMSE-UHFFFAOYSA-N", "bromoethane", "")
  CALL Species("C_2H_4Br_2",                           "557-91-5", "APQIUTYORBAGEZ-UHFFFAOYSA-N", "1,1-dibromoethane", "")
  CALL Species("C_2H_4Br_2",                           "106-93-4", "PAAZPARNPHGIKF-UHFFFAOYSA-N", "1,2-dibromoethane", "ethylene dibromide")
  CALL Species("C_2D_4Br_2",                         "22581-63-1", "PAAZPARNPHGIKF-LNLMKGTHSA-N", "1,2-dibromoethane-d4", "ethylene dibromide-d4")
  CALL Species("C_2H_3Br_3",                            "78-74-0", "QUMDOMSJJIFTCA-UHFFFAOYSA-N", "1,1,2-tribromoethane", "")
  CALL Species("C_2H_2Br_4",                            "79-27-6", "QXSZNDIIPUOQMB-UHFFFAOYSA-N", "1,1,2,2-tetrabromoethane", "")
  CALL Species("C_3H_7Br",                             "106-94-5", "CYNYIHKIEHGYOZ-UHFFFAOYSA-N", "1-bromopropane", "")
  CALL Species("C_3H_7Br",                              "75-26-3", "NAMYKGVDVNBCFQ-UHFFFAOYSA-N", "2-bromopropane", "")
  CALL Species("C_3H_6Br_2",                            "78-75-1", "XFNJYAKDBJUJAJ-UHFFFAOYSA-N", "1,2-dibromopropane", "")
  CALL Species("C_3H_6Br_2",                           "109-64-8", "VEFLKXRACNJHOV-UHFFFAOYSA-N", "1,3-dibromopropane", "")
  CALL Species("C_4H_9Br",                             "109-65-9", "MPPPKRYCTPRNTB-UHFFFAOYSA-N", "1-bromobutane", "")
  CALL Species("C_4H_9Br",                              "78-76-2", "UPSXAPQYNGXVBF-UHFFFAOYSA-N", "2-bromobutane", "")
  CALL Species("C_4H_9Br",                              "78-77-3", "HLVFKOKELQSXIQ-UHFFFAOYSA-N", "1-bromo-2-methylpropane", "")
  CALL Species("C_4H_9Br",                             "507-19-7", "RKSOPLXZQNSWAS-UHFFFAOYSA-N", "2-bromo-2-methylpropane", "")
  CALL Species("C_5H_{11}Br",                          "107-82-4", "YXZFFTJAHVMMLF-UHFFFAOYSA-N", "1-bromo-3-methylbutane", "")
  CALL Species("C_4H_8Br_2",                           "110-52-1", "ULTHEAFYOOPTTB-UHFFFAOYSA-N", "1,4-dibromobutane", "")
  CALL Species("C_5H_{11}Br",                          "110-53-2", "YZWKKMVJZFACSU-UHFFFAOYSA-N", "1-bromopentane", "")
  CALL Species("C_5H_{11}Br",                        "10422-35-2", "XKVLZBNEPALHIO-UHFFFAOYSA-N", "1-bromo-2-methylbutane", "")
  CALL Species("C_6H_{13}Br",                          "111-25-1", "MNDIARAMWBIKFW-UHFFFAOYSA-N", "1-bromohexane", "")
  CALL Species("C_6H_{13}Br",                        "51116-73-5", "MDCCBJLCTOTLKM-UHFFFAOYSA-N", "1-bromo-3-methylpentane", "")
  CALL Species("C_6H_{11}Br",                          "108-85-0", "AQNQQHJNRPDOQV-UHFFFAOYSA-N", "bromocyclohexane", "")
  CALL Species("C_7H_{15}Br",                          "629-04-9", "LSXKDWGTSHCFPP-UHFFFAOYSA-N", "1-bromoheptane", "")
  CALL Species("C_8H_{17}Br",                          "111-83-1", "VMKOFRJSULQZRM-UHFFFAOYSA-N", "1-bromooctane", "")
  CALL Species("C_8H_{16}Br_2",                       "4549-32-0", "DKEGCUDAFWNSSO-UHFFFAOYSA-N", "1,8-dibromooctane", "")
  CALL Species("C_8H_{12}Br_4",                       "3322-93-8", "PQRRSJBLKOPVJV-UHFFFAOYSA-N", "1,2-dibromo-4-(1,2-dibromoethyl)cyclohexane", "")
  CALL Species("C_9H_{19}Br",                          "693-58-3", "AYMUQTNXKPEMLM-UHFFFAOYSA-N", "1-bromononane", "")
  CALL Species("C_{10}H_{21}Br",                       "112-29-8", "MYMSJFSOOQERIO-UHFFFAOYSA-N", "1-bromodecane", "")
  CALL Species("C_{12}H_{18}Br_6",                    "3194-55-6", "DEIGXXQKDWULML-UHFFFAOYSA-N", "hexabromocyclododecane", "") ! other CAS: 25637-99-4
  ! unsaturated:
  CALL Species("C_2H_3Br",                             "593-60-2", "INLLPKCGLOXCIV-UHFFFAOYSA-N", "vinyl bromide", "")
  CALL Species("C_2H_2Br_2",                           "540-49-8", "UWTUEMKLYAGTNQ-UHFFFAOYSA-N", "1,2-dibromoethene", "")
  !CALL Species("C_2H_2Br_2",                          "590-12-5", "UWTUEMKLYAGTNQ-OWOJBTEDSA-N", "($E$)-1,2-dibromoethene", "{\it trans}-1,2-dibromoethene")
  CALL Species("C_2H_2Br_2",                           "590-11-4", "UWTUEMKLYAGTNQ-UPHRSURJSA-N", "($Z$)-1,2-dibromoethene", "{\it cis}-1,2-dibromoethene")
  CALL Species("C_3H_5Br",                             "106-95-6", "BHELZAPQIKSEDF-UHFFFAOYSA-N", "3-bromo-1-propene", "allyl bromide")
  CALL Species("C_3H_3Br",                             "106-96-7", "YORCIIVHUBAYBQ-UHFFFAOYSA-N", "3-bromo-1-propyne", "propargyl bromide")
  CALL Species("C_6H_9Br",                            "2044-08-8", "QBUMXSSCYUMVAW-UHFFFAOYSA-N", "1-bromocyclohexene", "")
  CALL Species("C_7H_{11}Br",                        "31053-84-6", "LIUFYLLBQSSKQS-UHFFFAOYSA-N", "1-bromo-4-methylcyclohexene", "")
  ! aromatic:
  CALL Species("C_6H_5Br",                             "108-86-1", "QARVLSVVCXYDNA-UHFFFAOYSA-N", "bromobenzene", "")
  CALL Species("C_6D_5Br",                            "4165-57-5", "QARVLSVVCXYDNA-RALIUCGRSA-N", "bromobenzene-d5", "")
  CALL Species("C_6H_4Br_2",                           "583-53-9", "WQONPSCCEXUXTQ-UHFFFAOYSA-N", "1,2-dibromobenzene", "")
  CALL Species("C_6H_4Br_2",                           "108-36-1", "JSRLURSZEMLAFO-UHFFFAOYSA-N", "1,3-dibromobenzene", "")
  CALL Species("C_6H_4Br_2",                           "106-37-6", "SWJPEBQEEAHIGZ-UHFFFAOYSA-N", "1,4-dibromobenzene", "")
  CALL Species("C_6H_3Br_3",                           "615-54-3", "FWAJPSIPOULHHH-UHFFFAOYSA-N", "1,2,4-tribromobenzene", "")
  CALL Species("C_6H_3Br_3",                           "626-39-1", "YWDUZLFWHVQCHY-UHFFFAOYSA-N", "1,3,5-tribromobenzene", "")
  CALL Species("C_6H_2Br_4",                           "636-28-2", "QCKHVNQHBOGZER-UHFFFAOYSA-N", "1,2,4,5-tetrabromobenzene", "")
  CALL Species("C_6Br_6",                               "87-82-1", "CAYGQBVSOZLICD-UHFFFAOYSA-N", "hexabromobenzene", "")
  CALL Species("C_7H_7Br",                             "100-39-0", "AGEZXYOZHKGVCM-UHFFFAOYSA-N", "(bromomethyl)-benzene", "benzyl bromide")
  CALL Species("C_7H_6Br_2",                           "589-15-1", "YLRBJYMANQKEAW-UHFFFAOYSA-N", "$p$-bromobenzyl bromide", "")
  CALL Species("BrC_6H_4CH_3",                          "95-46-5", "QSSXJPIWXQTSIX-UHFFFAOYSA-N", "1-bromo-2-methylbenzene", "$o$-bromotoluene")
  CALL Species("BrC_6H_4CH_3",                         "591-17-3", "WJIFKOVZNJTSGO-UHFFFAOYSA-N", "1-bromo-3-methylbenzene", "$m$-bromotoluene")
  CALL Species("BrC_6H_4CH_3",                         "106-38-7", "ZBTMRBYMKUEVEU-UHFFFAOYSA-N", "1-bromo-4-methylbenzene", "$p$-bromotoluene")
  CALL Species("C_7H_6Br_2",                          "1611-92-3", "DPKKOVGCHDUSAI-UHFFFAOYSA-N", "3,5-dibromotoluene", "")
  CALL Species("C_7H_3Br_5",                            "87-83-2", "OZHJEQVYCBTHJT-UHFFFAOYSA-N", "pentabromotoluene", "")
  CALL Species("C_8H_9Br",                            "1973-22-4", "HVRUGFJYCAFAAN-UHFFFAOYSA-N", "1-bromo-2-ethylbenzene", "")
  CALL Species("C_8H_9Br",                            "1585-07-5", "URFPRAHGGBYNPW-UHFFFAOYSA-N", "1-bromo-4-ethylbenzene", "")
  CALL Species("C_8H_9Br",                              "89-92-9", "WGVYCXYGPNNUQA-UHFFFAOYSA-N", "1-(bromomethyl)-2-methylbenzene", "$o$-xylyl bromide")
  CALL Species("C_8H_9Br",                             "620-13-3", "FWLWTILKTABGKQ-UHFFFAOYSA-N", "1-(bromomethyl)-3-methylbenzene", "$m$-xylyl bromide")
  CALL Species("C_8H_9Br",                             "104-81-4", "WZRKSPFYXUXINF-UHFFFAOYSA-N", "1-(bromomethyl)-4-methylbenzene", "$p$-xylyl bromide")
  CALL Species("C_8H_9Br",                             "103-63-9", "WMPPDTMATNBGJN-UHFFFAOYSA-N", "(2-bromoethyl)-benzene", "")
  CALL Species("C_8H_7Br",                          "125904-11-2", "SSZOCHFYWWVSAI-UHFFFAOYSA-N", "2-bromostyrene", "")
  CALL Species("C_8H_7Br",                             "103-64-0", "YMOONIIMQBGTDU-VOTSOKGWSA-N", "(2-bromoethenyl)benzene", "")
  CALL Species("C_8H_5Br_5",                            "85-22-3", "FIAXCDIQXHJNIX-UHFFFAOYSA-N", "2,3,4,5,6-pentabromoethylbenzene", "")
  CALL Species("BrC_6H_4C_3H_7",                      "7073-94-1", "LECYCYNAEJDSIL-UHFFFAOYSA-N", "1-bromo-2-(2-propyl)-benzene", "$o$-bromocumene")
  CALL Species("C_{10}H_7Br",                           "90-11-9", "DLKQHBOKULLWDQ-UHFFFAOYSA-N", "1-bromonaphthalene", "")
  CALL Species("C_{10}H_7Br",                          "580-13-2", "APSMUYYLXZULMS-UHFFFAOYSA-N", "2-bromonaphthalene", "")
  CALL Species("C_{10}H_6Br_2",                         "83-53-4", "IBGUDZMIAZLJNY-UHFFFAOYSA-N", "1,4-dibromonaphthalene", "")
  CALL Species("C_{12}Br_{10}",                      "13654-09-6", "AQPHBYQUCKHJLT-UHFFFAOYSA-N", "decabromobiphenyl", "")
  CALL Species("C_{12}H_9Br",                           "92-66-0", "PKJBWOWQJHHAHG-UHFFFAOYSA-N", "4-bromo-1,1'-biphenyl", "")
  CALL Species("C_{12}H_2Br_8",                      "27858-07-7", "NDRKXFLZSRHAJE-UHFFFAOYSA-N", "octabromobiphenyl", "")
  CALL Species("C_{12}H_4Br_6",                      "59080-40-9", "HMBBJSKXDBUNNT-UHFFFAOYSA-N", "2,2',4,4',5,5'-hexabromobiphenyl", "")
  CALL Species("C_{14}H_4Br_{10}",                   "84852-53-9", "BZQKBFHEWDPQHD-UHFFFAOYSA-N", "1,2-bis(pentabromophenyl) ethane", "")
  CALL Species("C_{16}H_6Br_4",                        "128-63-2", "ZKBKRTZIYOKNRG-UHFFFAOYSA-N", "1,3,6,8-tetrabromopyrene", "")
  CALL Species("C_{20}H_{11}Br",                     "21248-00-0", "MJSYSGSEEADMTK-UHFFFAOYSA-N", "6-bromobenzo[$a$]pyrene", "")
  ! C, H, O, Br:
  ! aliphatic:
  CALL Species("CH_2BrOH",                           "50398-29-3", "OEDMOCYNWLHUDP-UHFFFAOYSA-N", "bromomethanol", "")
  CALL Species("CHBr_2OH",                          "166600-78-8", "WICMSCLXMMZMFF-UHFFFAOYSA-N", "dibromomethanol", "")
  CALL Species("CBr_3OH",                             "5405-30-1", "ACRXLLXANWELLX-UHFFFAOYSA-N", "tribromomethanol", "")
  CALL Species("CHBrO",                               "7726-11-6", "AIFARXRIYKCEEV-UHFFFAOYSA-N", "formyl bromide", "")
  CALL Species("CBr_2O",                               "593-95-3", "MOIPGXQKZSZOQX-UHFFFAOYSA-N", "carbonyl bromide", "")
  CALL Species("CH_2BrO_2H",                                   "", "WXVCUAGYVVWQBP-UHFFFAOYSA-N", "bromomethyl peroxide", "")
  CALL Species("CHBr_2O_2H",                                   "", "JBKQCSQVMCXDPZ-UHFFFAOYSA-N", "dibromomethyl peroxide", "")
  CALL Species("CBr_3O_2H",                                    "", "WJYKCADCZNTKBI-UHFFFAOYSA-N", "tribromomethyl peroxide", "")
  CALL Species("CH_2BrCOOH",                            "79-08-3", "KDPAWGWELVVRCH-UHFFFAOYSA-N", "bromoethanoic acid", "bromoacetic acid")
  CALL Species("CHBr_2COOH",                           "631-64-1", "SIEILFNCEFEENQ-UHFFFAOYSA-N", "dibromoethanoic acid", "dibromoacetic acid")
  CALL Species("CBr_3COOH",                             "75-96-7", "QIONYIKHPASLHO-UHFFFAOYSA-N", "tribromoethanoic acid", "tribromoacetic acid")
  CALL Species("C_2H_3O_3Br",                                  "", "RGXORGOPJDPEIK-UHFFFAOYSA-N", "MCM:BRETO3H", "")
  CALL Species("C_2H_4O_2Br_2",                                "", "ZXSZEFURCAPOPD-UHFFFAOYSA-N", "MCM:DIBRETO2H", "")
  CALL Species("C_2H_4OBr_2",                                  "", "IYJYRKYSUOTLAE-UHFFFAOYSA-N", "MCM:DIBRETOH", "")
  CALL Species("C_2H_3OBr",                                    "", "NMPVEAUIHMEAQP-UHFFFAOYSA-N", "MCM:BRETAL", "")
  CALL Species("C_3H_7BrO",                            "627-18-9", "RQFUZUMFPRMVDX-UHFFFAOYSA-N", "3-bromopropanol", "")
  CALL Species("C_3H_6Br_2O",                           "96-13-9", "QWVCIORZLNBIIC-UHFFFAOYSA-N", "2,3-dibromopropanol", "")
  CALL Species("C_3H_5BrO",                            "598-31-2", "VQFAIAKCILWQPZ-UHFFFAOYSA-N", "bromoacetone", "")
  CALL Species("C_3H_5BrO",                           "3132-64-7", "GKIPXFAANLTWBM-UHFFFAOYSA-N", "(bromomethyl)oxirane", "epibromohydrin")
  CALL Species("C_4H_8Br_2O_2",                      "90801-18-6", "OXYNQEOLHRWEPE-QWWZWVQMSA-N", "2,3-dibromobutane-1,4-diol", "")
  CALL Species("C_4H_7BrO_2",                          "105-36-2", "PQJJJMRNHATNKG-UHFFFAOYSA-N", "bromoacetic acid, ethyl ester", "")
  CALL Species("C_4H_7Br_3O",                           "76-08-4", "JUGRTVJQTFZHOM-UHFFFAOYSA-N", "brometone", "1,1,1-tribromo-2-methyl-2-propanol")
  CALL Species("C_5H_{10}Br_2O_2",                    "3296-90-0", "CHUGKEQJSLOLHL-UHFFFAOYSA-N", "2,2-bis(bromomethyl)-1,3-propanediol", "")
  CALL Species("C_5H_9Br_3O",                        "36483-57-5", "HQWKMYFWGMCJSW-UHFFFAOYSA-N", "trisbromoneopentyl alcohol", "")
  ! aromatic:
  CALL Species("HOC_6H_4Br",                            "95-56-7", "VADKRMSMGWJZCF-UHFFFAOYSA-N", "2-bromophenol", "")
  CALL Species("HOC_6H_4Br",                           "591-20-8", "MNOJRWOWILAHAV-UHFFFAOYSA-N", "3-bromophenol", "")
  CALL Species("HOC_6H_4Br",                           "106-41-2", "GZFGOTFRPZRKDS-UHFFFAOYSA-N", "4-bromophenol", "")
  CALL Species("C_6H_4Br_2O",                          "615-58-7", "FAXWFCTVSHEODL-UHFFFAOYSA-N", "2,4-dibromophenol", "")
  CALL Species("C_6H_4Br_2O",                          "608-33-3", "SSIZLKDLDKIHEV-UHFFFAOYSA-N", "2,6-dibromophenol", "")
  CALL Species("C_6H_3Br_3O",                          "118-79-6", "BSWWXRFVMJHFBN-UHFFFAOYSA-N", "2,4,6-tribromophenol", "")
  CALL Species("C_6H_2Br_4O",                        "14400-94-3", "CXPJZISGVIVNEL-UHFFFAOYSA-N", "2,3,4,6-tetrabromophenol", "")
  CALL Species("C_6HBr_5O",                            "608-71-9", "SVHOVVJFOWGYJO-UHFFFAOYSA-N", "pentabromophenol", "")
  CALL Species("C_7H_7BrO",                            "578-57-4", "HTDQSWDEWGSAMN-UHFFFAOYSA-N", "1-bromo-2-methoxybenzene", "2-bromoanisole")
  CALL Species("C_7H_7BrO",                           "2398-37-0", "PLDWAJLZAAHOGG-UHFFFAOYSA-N", "1-bromo-3-methoxybenzene", "3-bromoanisole")
  CALL Species("C_7H_7BrO",                            "104-92-7", "QJPJQTDYNZXKQF-UHFFFAOYSA-N", "1-bromo-4-methoxybenzene", "4-bromoanisole")
  CALL Species("C_7H_6Br_2O",                        "21702-84-1", "XGXUGXPKRBQINS-UHFFFAOYSA-N", "1,5-dibromo-2-methoxybenzene", "2,4-dibromoanisole")
  CALL Species("C_7H_6Br_2O",                        "38603-09-7", "BMZVDHQOAJUZJL-UHFFFAOYSA-N", "1,3-dibromo-2-methoxybenzene", "2,6-dibromoanisole")
  CALL Species("C_7H_5Br_3O",                        "95970-13-1", "NKWYZAWFQZLPSU-UHFFFAOYSA-N", "1,2,3-tribromo-4-methoxybenzene", "2,3,4-tribromoanisole")
  CALL Species("C_7H_5Br_3O",                        "95970-19-7", "XZYCSFIRWAPGJV-UHFFFAOYSA-N", "1,3,4-tribromo-2-methoxybenzene", "2,3,6-tribromoanisole")
  CALL Species("C_7H_5Br_3O",                          "607-99-8", "YXTRCOAFNXQTKL-UHFFFAOYSA-N", "1,3,5-tribromo-2-methoxybenzene", "2,4,6-tribromoanisole")
  CALL Species("C_7H_3Br_5O",                         "1825-26-9", "VEFNQGLEDJPUJF-UHFFFAOYSA-N", "pentabromomethoxybenzene", "pentabromoanisole")
  CALL Species("C_8H_7Br_3O",                        "41424-36-6", "NMPPAMFEYWGNCI-UHFFFAOYSA-N", "1,3,5-tribromo-2-methoxy-4-methylbenzene", "")
  CALL Species("C_8Br_4O_3",                           "632-79-1", "QHWKHLYUUZGSCW-UHFFFAOYSA-N", "4,5,6,7-tetrabromo-1,3-isobenzofurandione", "")
  CALL Species("C_9H_7Br_3O",                         "3278-89-5", "RZLLIOPGUFOWOD-UHFFFAOYSA-N", "allyl 2,4,6-tribromophenyl ether", "")
  CALL Species("C_{10}H_{10}Br_2O_2",                "75150-13-9", "XQTJZNGNEJLXTR-UHFFFAOYSA-N", "2,4-dibromo-6-methylphenyl glycidyl ether", "")
  CALL Species("C_{11}H_9Br_3O_3",                    "7347-19-5", "AMBJXYFIMKHOQE-UHFFFAOYSA-N", "2-(2,4,6-tribromophenoxy)ethyl acrylate", "")
  CALL Species("C_{12}H_4Br_4O_2",                   "50585-41-6", "JZLQUWSWOJPCAK-UHFFFAOYSA-N", "2,3,7,8-tetrabromodibenzo-$p$-dioxin", "")
  CALL Species("C_{12}Br_8O_2",                       "2170-45-8", "XAHTWKGGNHXJRP-UHFFFAOYSA-N", "octabromodibenzo-$p$-dioxin", "")
  CALL Species("C_{13}H_8Br_4O_2",                   "21825-03-6", "WPZJSWWEEJJSIZ-UHFFFAOYSA-N", "4,4'-methylenebis(2,6-dibromophenol)", "")
  CALL Species("C_{14}H_4Br_{10}O_2",                "61262-53-1", "JJEPQBZQAGCZTH-UHFFFAOYSA-N", "1,1'-[1,2-ethanediylbis(oxy)]bis pentabromobenzene", "")
  CALL Species("C_{14}H_8Br_2O_2",                   "35578-47-3", "NYCBYBDDECLFPE-UHFFFAOYSA-N", "4,4'-dibromobenzil", "")
  CALL Species("C_{14}H_8Br_6O_2",                   "37853-59-1", "YATIGPZCMOYEGE-UHFFFAOYSA-N", "1,2-bis(2,4,6-tribromophenoxy)ethane", "BTBPE")
  CALL Species("C_{15}H_{18}Br_4O_2",               "183658-27-7", "HVDXCGSGEQKWGB-UHFFFAOYSA-N", "2-ethylhexyl-2,3,4,5-tetrabromobenzoate", "EHTeBB")
  CALL Species("C_{15}H_{13}Br_3O_2",                 "6386-73-8", "WYBOEVJIVYIEJL-UHFFFAOYSA-N", "tribromobisphenol A", "")
  CALL Species("C_{15}H_{12}Br_4O_2",                   "79-94-7", "VEORPZCZECFIRK-UHFFFAOYSA-N", "4,4'-(1-methylethylidene)bis(2,6-dibromophenol)", "")
  CALL Species("C_{15}H_{12}Br_4O_2",                "94334-64-2", "KIZJVNGSIWXYTL-UHFFFAOYSA-N", "4-[2-[2,6-bis(bromanyl)-4-oxidanyl-phenyl]propan-2-yl]-3,5-bis(bromanyl)phenol", "")
  CALL Species("C_{15}H_{16}Br_4O_7",                "20566-35-2", "OQHHASWHOGRCRC-UHFFFAOYSA-N", "2-(2-hydroxyethoxy)ethyl 2-hydroxypropyl 3,4,5,6-tetrabromophthalate", "")
  CALL Species("C_{17}H_{16}Br_2O_3",                "18181-80-1", "FOANIXZHAMJWOI-UHFFFAOYSA-N", "bromopropylate", "")
  CALL Species("C_{18}O_2Br_{14}",                   "58965-66-5", "YMIUHIAWWDYGGU-UHFFFAOYSA-N", "1,2,4,5-tetrabromo-3,6-bis(pentabromophenoxy)benzene", "")
  CALL Species("C_{19}H_{20}Br_4O_4",                 "4162-45-2", "RVHUMFJSCJBNGS-UHFFFAOYSA-N", "2,2-bis(3,5-dibromo-4-(2-hydroxyethoxy)phenyl)propane", "")
  CALL Species("C_{19}H_{21}BrO_5",                 "220899-03-6", "AMSPWOYQQAWRRM-UHFFFAOYSA-N", "metrafenone", "")
  CALL Species("C_{20}H_8Br_4O_5",                   "15086-94-9", "DBZJJPROPLPMSN-UHFFFAOYSA-N", "solvent red 43", "")
  CALL Species("C_{21}H_{20}Br_8O_2",                "21850-44-2", "LXIZRZRTWSDLKK-UHFFFAOYSA-N", "2,2-bis[4-(2,3-dibromopropoxy)-3,5-dibromophenyl]-propane", "")
  CALL Species("C_{21}H_{20}Br_4O_2",                "25327-89-3", "PWXTUWQHMIFLKL-UHFFFAOYSA-N", "2,2-bis(4-allyloxy-3,5-dibromophenyl)propane", "")
  CALL Species("C_{21}H_{24}Br_4O_4",                "33294-14-3", "UTMWVHZWNKBNKF-UHFFFAOYSA-N", "AC1MJ2TG", "") ! other CAS: 93907-20-1
  CALL Species("C_{22}H_{14}Br_4O_4",                 "1176-74-5", "SQFXATUXPUCFFO-UHFFFAOYSA-N", "tetrabromophenolphthalein, ethyl ester", "")
  CALL Species("C_{22}H_8Br_2O_2",                    "4378-61-4", "HTENFZMEHKCNMD-UHFFFAOYSA-N", "4,10-dibromodibenzo[$def,mno$]\+chrysene-6,12-dione", "")
  CALL Species("C_{24}H_{34}Br_4O_4",                "26040-51-7", "UUEDINPOVKWVAZ-UHFFFAOYSA-N", "bis(2-ethylhexyl)-3,4,5,6-tetrabromophthalate", "TBPH")
  CALL Species("C_{30}H_{23}BrO_4",                  "28772-56-7", "OWNRRUFOJXFKCU-UHFFFAOYSA-N", "bromadiolone", "")
  CALL Species("C_{31}H_{23}BrO_3",                  "56073-10-0", "VEUZZDOCACZPRY-UHFFFAOYSA-N", "brodifacoum", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Polybrominated diphenyl ethers (PBDEs)")

  CALL Species("C_{12}H_9BrO",                       "36563-47-0", "RRWFUWRLNIZICP-UHFFFAOYSA-N", "2-bromodiphenyl ether", "PBDE-1")
  CALL Species("C_{12}H_9BrO",                        "6876-00-2", "AHDAKFFMKLQPTD-UHFFFAOYSA-N", "3-bromodiphenyl ether", "PBDE-2")
  CALL Species("C_{12}H_9BrO",                         "101-55-3", "JDUYPUMQALQRCN-UHFFFAOYSA-N", "4-bromodiphenyl ether", "PBDE-3")
  CALL Species("C_{12}H_8Br_2O",                     "51452-87-0", "JMSKYMHFNWGUJG-UHFFFAOYSA-N", "2,2'-dibromodiphenyl ether", "PBDE-4")
  CALL Species("C_{12}H_8Br_2O",                    "446254-14-4", "JTYRXXKXOULVAP-UHFFFAOYSA-N", "2,3-dibromodiphenyl ether", "PBDE-5")
  CALL Species("C_{12}H_8Br_2O",                    "147217-72-9", "GODQTPRKFHOLPH-UHFFFAOYSA-N", "2,3'-dibromodiphenyl ether", "PBDE-6")
  CALL Species("C_{12}H_8Br_2O",                    "171977-44-9", "JMCIHKKTRDLVCO-UHFFFAOYSA-N", "2,4-dibromodiphenyl ether", "PBDE-7")
  CALL Species("C_{12}H_8Br_2O",                    "147217-71-8", "RJQLQJZMLISKRJ-UHFFFAOYSA-N", "2,4'-dibromodiphenyl ether", "PBDE-8")
  CALL Species("C_{12}H_8Br_2O",                    "337513-66-3", "URDWJMUOJJSXAE-UHFFFAOYSA-N", "2,5-dibromodiphenyl ether", "PBDE-9")
  CALL Species("C_{12}H_8Br_2O",                     "51930-04-2", "MUVDKHMQIZJFTC-UHFFFAOYSA-N", "2,6-dibromodiphenyl ether", "PBDE-10")
  CALL Species("C_{12}H_8Br_2O",                      "6903-63-5", "ALSVFJIXSNRBLE-UHFFFAOYSA-N", "3,3'-dibromodiphenyl ether", "PBDE-11")
  CALL Species("C_{12}H_8Br_2O",                    "189084-59-1", "SUUJFDKVPDCZQZ-UHFFFAOYSA-N", "3,4-dibromodiphenyl ether", "PBDE-12")
  CALL Species("C_{12}H_8Br_2O",                     "83694-71-7", "BGPOVBPKODCMMN-UHFFFAOYSA-N", "3,4'-dibromodiphenyl ether", "PBDE-13")
  CALL Species("C_{12}H_8Br_2O",                     "46438-88-4", "FOXBZJLXVUHYQZ-UHFFFAOYSA-N", "3,5-dibromodiphenyl ether", "PBDE-14")
  CALL Species("C_{12}H_8Br_2O",                      "2050-47-7", "YAWIAFUBXXPJMQ-UHFFFAOYSA-N", "4,4'-dibromodiphenyl ether", "PBDE-15")
  CALL Species("C_{12}H_7Br_3O",                    "147217-74-1", "VRNGWCVCSHJUEJ-UHFFFAOYSA-N", "2,2',3-tribromodiphenyl ether", "PBDE-16")
  CALL Species("C_{12}H_7Br_3O",                    "147217-75-2", "VYBFILXLBMWOLI-UHFFFAOYSA-N", "2,2',4-tribromodiphenyl ether", "PBDE-17")
  CALL Species("C_{12}H_7Br_3O",                    "407606-55-7", "FAZLXBWRNJAGSV-UHFFFAOYSA-N", "2,2',5-tribromodiphenyl ether", "PBDE-18")
  CALL Species("C_{12}H_7Br_3O",                    "147217-73-0", "YDFQHBRKURQGAH-UHFFFAOYSA-N", "2,2',6-tribromodiphenyl ether", "PBDE-19")
  CALL Species("C_{12}H_7Br_3O",                    "147217-76-3", "RQJUBSPXDSGLRB-UHFFFAOYSA-N", "2,3,3'-tribromodiphenyl ether", "PBDE-20")
  CALL Species("C_{12}H_7Br_3O",                    "337513-67-4", "RXWRVYYPLRPDOS-UHFFFAOYSA-N", "2,3,4-tribromodiphenyl ether", "PBDE-21")
  CALL Species("C_{12}H_7Br_3O",                    "446254-15-5", "WZHNIFQVNBINLF-UHFFFAOYSA-N", "2,3,4'-tribromodiphenyl ether", "PBDE-22")
  CALL Species("C_{12}H_7Br_3O",                    "446254-16-6", "XQHLKDAUZRXBGC-UHFFFAOYSA-N", "2,3,5-tribromodiphenyl ether", "PBDE-23")
  CALL Species("C_{12}H_7Br_3O",                    "218304-36-0", "GFLRHBRMAZDOIG-UHFFFAOYSA-N", "2,3,6-tribromodiphenyl ether", "PBDE-24")
  CALL Species("C_{12}H_7Br_3O",                    "147217-77-4", "AURKEOPYVUYTLO-UHFFFAOYSA-N", "2,3',4-tribromodiphenyl ether", "PBDE-25")
  CALL Species("C_{12}H_7Br_3O",                    "337513-75-4", "VUOBKVBAFJQQDB-UHFFFAOYSA-N", "2,3',5-tribromodiphenyl ether", "PBDE-26")
  CALL Species("C_{12}H_7Br_3O",                    "337513-53-8", "JUPZALSVNWJHII-UHFFFAOYSA-N", "2,3',6-tribromodiphenyl ether", "PBDE-27")
  CALL Species("C_{12}H_7Br_3O",                     "41318-75-6", "UPNBETHEXPIWQX-UHFFFAOYSA-N", "2,4,4'-tribromodiphenyl ether", "PBDE-28")
  CALL Species("C_{12}H_7Br_3O",                    "337513-56-1", "LTMKAFUXYKEDLR-UHFFFAOYSA-N", "2,4,5-tribromodiphenyl ether", "PBDE-29")
  CALL Species("C_{12}H_7Br_3O",                    "155999-95-4", "TVZAPPGLBLTACB-UHFFFAOYSA-N", "2,4,6-tribromodiphenyl ether", "PBDE-30")
  CALL Species("C_{12}H_7Br_3O",                     "65075-08-3", "PURZBWMLFRWRMG-UHFFFAOYSA-N", "2,4',5-tribromodiphenyl ether", "PBDE-31")
  CALL Species("C_{12}H_7Br_3O",                    "189084-60-4", "TYDVYKIQSZGUMV-UHFFFAOYSA-N", "2,4',6-tribromodiphenyl ether", "PBDE-32")
  CALL Species("C_{12}H_7Br_3O",                     "49690-94-0", "BUQBQEYUVAKJQK-UHFFFAOYSA-N", "2,3',4'-tribromodiphenyl ether", "PBDE-33")
  CALL Species("C_{12}H_7Br_3O",                    "446254-17-7", "XMNXHCHZIPYCNA-UHFFFAOYSA-N", "2,3',5'-tribromodiphenyl ether", "PBDE-34")
  CALL Species("C_{12}H_7Br_3O",                    "147217-80-9", "CDVYKQPKJYPWRO-UHFFFAOYSA-N", "3,3',4-tribromodiphenyl ether", "PBDE-35")
  CALL Species("C_{12}H_7Br_3O",                    "147217-79-6", "XUKPJLVONRTECE-UHFFFAOYSA-N", "3,3',5-tribromodiphenyl ether", "PBDE-36")
  CALL Species("C_{12}H_7Br_3O",                    "147217-81-0", "YALAYFVVZFORPV-UHFFFAOYSA-N", "3,4,4'-tribromodiphenyl ether", "PBDE-37")
  CALL Species("C_{12}H_7Br_3O",                    "337513-54-9", "DPGVQKLGQZZLMI-UHFFFAOYSA-N", "3,4,5-tribromodiphenyl ether", "PBDE-38")
  CALL Species("C_{12}H_7Br_3O",                    "407606-57-9", "UFFNOPDHJNQYKD-UHFFFAOYSA-N", "3,4',5-tribromodiphenyl ether", "PBDE-39")
  CALL Species("C_{12}H_6Br_4O",                    "337513-77-6", "SXSUUFZWSVMTRL-UHFFFAOYSA-N", "2,2',3,3'-tetrabromodiphenyl ether", "PBDE-40")
  CALL Species("C_{12}H_6Br_4O",                    "337513-68-5", "UAEBSKBXZAIRMX-UHFFFAOYSA-N", "2,2',3,4-tetrabromodiphenyl ether", "PBDE-41")
  CALL Species("C_{12}H_6Br_4O",                    "446254-18-8", "HQDQKPAHIDGGMH-UHFFFAOYSA-N", "2,2',3,4'-tetrabromodiphenyl ether", "PBDE-42")
  CALL Species("C_{12}H_6Br_4O",                    "446254-19-9", "LKMQHSYDVDIECC-UHFFFAOYSA-N", "2,2',3,5-tetrabromodiphenyl ether", "PBDE-43")
  CALL Species("C_{12}H_6Br_4O",                    "446254-20-2", "VBGBGTYMDIVKNK-UHFFFAOYSA-N", "2,2',3,5'-tetrabromodiphenyl ether", "PBDE-44")
  CALL Species("C_{12}H_6Br_4O",                    "446254-21-3", "VTFWUBIOZQCMQS-UHFFFAOYSA-N", "2,2',3,6-tetrabromodiphenyl ether", "PBDE-45")
  CALL Species("C_{12}H_6Br_4O",                    "446254-22-4", "GBUUKJRFSKCMTB-UHFFFAOYSA-N", "2,2',3,6'-tetrabromodiphenyl ether", "PBDE-46")
  CALL Species("C_{12}H_6Br_4O",                      "5436-43-1", "XYBSIYMGXVUVGY-UHFFFAOYSA-N", "2,2',4,4'-tetrabromodiphenyl ether", "PBDE-47")
  CALL Species("C_{12}H_6Br_4O",                    "337513-55-0", "FJGDNHOVDFREMP-UHFFFAOYSA-N", "2,2',4,5-tetrabromodiphenyl ether", "PBDE-48")
  CALL Species("C_{12}H_6Br_4O",                    "243982-82-3", "QWVDUBDYUPHNHY-UHFFFAOYSA-N", "2,2',4,5'-tetrabromodiphenyl ether", "PBDE-49")
  CALL Species("C_{12}H_6Br_4O",                    "446254-23-5", "FXUAKFRJBKFDSY-UHFFFAOYSA-N", "2,2',4,6-tetrabromodiphenyl ether", "PBDE-50")
  CALL Species("C_{12}H_6Br_4O",                    "189084-57-9", "WKBBBTLDLKYGBI-UHFFFAOYSA-N", "2,2',4,6'-tetrabromodiphenyl ether", "PBDE-51")
  CALL Species("C_{12}H_6Br_4O",                    "446254-24-6", "CDTHXJORUCZHMD-UHFFFAOYSA-N", "2,2',5,5'-tetrabromodiphenyl ether", "PBDE-52")
  CALL Species("C_{12}H_6Br_4O",                    "446254-25-7", "SDVQGIMOFXMKHR-UHFFFAOYSA-N", "2,2',5,6'-tetrabromodiphenyl ether", "PBDE-53")
  CALL Species("C_{12}H_6Br_4O",                    "446254-26-8", "WCDCHQGVTZHVSO-UHFFFAOYSA-N", "2,2',6,6'-tetrabromodiphenyl ether", "PBDE-54")
  CALL Species("C_{12}H_6Br_4O",                    "446254-27-9", "VIHUMJGEWQPWOT-UHFFFAOYSA-N", "2,3,3',4-tetrabromodiphenyl ether", "PBDE-55")
  CALL Species("C_{12}H_6Br_4O",                    "446254-28-0", "NFOIVCGFYJIYIB-UHFFFAOYSA-N", "2,3,3',4'-tetrabromodiphenyl ether", "PBDE-56")
  CALL Species("C_{12}H_6Br_4O",                    "337513-82-3", "CSIFWDKYUJLQEB-UHFFFAOYSA-N", "2,3,3',5-tetrabromodiphenyl ether", "PBDE-57")
  CALL Species("C_{12}H_6Br_4O",                    "446254-29-1", "SWOYBZHGPZIRHS-UHFFFAOYSA-N", "2,3,3',5'-tetrabromodiphenyl ether", "PBDE-58")
  CALL Species("C_{12}H_6Br_4O",                    "446254-30-4", "DMAMJZQQOWYEHT-UHFFFAOYSA-N", "2,3,3',6-tetrabromodiphenyl ether", "PBDE-59")
  CALL Species("C_{12}H_6Br_4O",                    "446254-31-5", "ARERIMFZYPFJAV-UHFFFAOYSA-N", "2,3,4,4'-tetrabromodiphenyl ether", "PBDE-60")
  CALL Species("C_{12}H_6Br_4O",                    "446254-32-6", "NDRSXNBQWAOQPP-UHFFFAOYSA-N", "2,3,4,5-tetrabromodiphenyl ether", "PBDE-61")
  CALL Species("C_{12}H_6Br_4O",                    "446254-33-7", "YIQYWYZZLOZVRM-UHFFFAOYSA-N", "2,3,4,6-tetrabromodiphenyl ether", "PBDE-62")
  CALL Species("C_{12}H_6Br_4O",                    "446254-34-8", "HNICYXFGCWPYGC-UHFFFAOYSA-N", "2,3,4',5-tetrabromodiphenyl ether", "PBDE-63")
  CALL Species("C_{12}H_6Br_4O",                    "446254-35-9", "LDCXVFJUWKKBNY-UHFFFAOYSA-N", "2,3,4',6-tetrabromodiphenyl ether", "PBDE-64")
  CALL Species("C_{12}H_6Br_4O",                    "446254-36-0", "HPEUYVBOPJQVPN-UHFFFAOYSA-N", "2,3,5,6-tetrabromodiphenyl ether", "PBDE-65")
  CALL Species("C_{12}H_6Br_4O",                    "189084-61-5", "DHUMTYRHKMCVAG-UHFFFAOYSA-N", "2,3',4,4'-tetrabromodiphenyl ether", "PBDE-66")
  CALL Species("C_{12}H_6Br_4O",                    "446254-37-1", "OARGWSONVLGXQA-UHFFFAOYSA-N", "2,3',4,5-tetrabromodiphenyl ether", "PBDE-67")
  CALL Species("C_{12}H_6Br_4O",                    "446254-38-2", "UFWGRLCUOLLWAO-UHFFFAOYSA-N", "2,3',4,5'-tetrabromodiphenyl ether", "PBDE-68")
  CALL Species("C_{12}H_6Br_4O",                    "327185-09-1", "NHZNRCYNZJADTG-UHFFFAOYSA-N", "2,3',4,6-tetrabromodiphenyl ether", "PBDE-69")
  CALL Species("C_{12}H_6Br_4O",                    "446254-39-3", "GHQMTYWQVJZWAR-UHFFFAOYSA-N", "2,3',4',5-tetrabromodiphenyl ether", "PBDE-70")
  CALL Species("C_{12}H_6Br_4O",                    "189084-62-6", "COPAGYRSCJVION-UHFFFAOYSA-N", "2,3',4',6-tetrabromodiphenyl ether", "PBDE-71")
  CALL Species("C_{12}H_6Br_4O",                    "446254-40-6", "GBBNZKQTOOZGIS-UHFFFAOYSA-N", "2,3',5,5'-tetrabromodiphenyl ether", "PBDE-72")
  CALL Species("C_{12}H_6Br_4O",                    "446254-41-7", "WQFLVWXBCRJAQN-UHFFFAOYSA-N", "2,3',5',6-tetrabromodiphenyl ether", "PBDE-73")
  CALL Species("C_{12}H_6Br_4O",                    "446254-42-8", "LXCFDVVDUVPAGR-UHFFFAOYSA-N", "2,4,4',5-tetrabromodiphenyl ether", "PBDE-74")
  CALL Species("C_{12}H_6Br_4O",                    "189084-63-7", "BWCNKMFFUGBFGB-UHFFFAOYSA-N", "2,4,4',6-tetrabromodiphenyl ether", "PBDE-75")
  CALL Species("C_{12}H_6Br_4O",                    "446254-43-9", "NCSWBJSFVPJPPK-UHFFFAOYSA-N", "2,3',4',5'-tetrabromodiphenyl ether", "PBDE-76")
  CALL Species("C_{12}H_6Br_4O",                     "93703-48-1", "RYGLOWMCGZHYRQ-UHFFFAOYSA-N", "3,3',4,4'-tetrabromodiphenyl ether", "PBDE-77")
  CALL Species("C_{12}H_6Br_4O",                    "446254-45-1", "HWOBLTZZSVXBOJ-UHFFFAOYSA-N", "3,3',4,5-tetrabromodiphenyl ether", "PBDE-78")
  CALL Species("C_{12}H_6Br_4O",                    "446254-48-4", "LELQGHJEUVRPEV-UHFFFAOYSA-N", "3,3',4,5'-tetrabromodiphenyl ether", "PBDE-79")
  CALL Species("C_{12}H_6Br_4O",                    "103173-66-6", "HFIOZJQRZKNPKJ-UHFFFAOYSA-N", "3,3',5,5'-tetrabromodiphenyl ether", "PBDE-80")
  CALL Species("C_{12}H_6Br_4O",                    "446254-50-8", "ULFOIXCXIWHJDS-UHFFFAOYSA-N", "3,4,4',5-tetrabromodiphenyl ether", "PBDE-81")
  CALL Species("C_{12}H_5Br_5O",                    "327185-11-5", "RQMSPGJESCCPQX-UHFFFAOYSA-N", "2,2',3,3',4-pentabromodiphenyl ether", "PBDE-82")
  CALL Species("C_{12}H_5Br_5O",                    "446254-51-9", "XAHYSNUYJLNDBX-UHFFFAOYSA-N", "2,2',3,3',5-pentabromodiphenyl ether", "PBDE-83")
  CALL Species("C_{12}H_5Br_5O",                    "446254-52-0", "PPIZNRAVQHNLJM-UHFFFAOYSA-N", "2,2',3,3',6-pentabromodiphenyl ether", "PBDE-84")
  CALL Species("C_{12}H_5Br_5O",                    "182346-21-0", "DMLQSUZPTTUUDP-UHFFFAOYSA-N", "2,2',3,4,4'-pentabromodiphenyl ether", "PBDE-85")
  CALL Species("C_{12}H_5Br_5O",                    "446254-53-1", "YMVWYUWOUOQCQP-UHFFFAOYSA-N", "2,2',3,4,5-pentabromodiphenyl ether", "PBDE-86")
  CALL Species("C_{12}H_5Br_5O",                    "446254-54-2", "WKYQUGCIKNOXFW-UHFFFAOYSA-N", "2,2',3,4,5'-pentabromodiphenyl ether", "PBDE-87")
  CALL Species("C_{12}H_5Br_5O",                    "446254-55-3", "OPZUHBCVIZNZFB-UHFFFAOYSA-N", "2,2',3,4,6-pentabromodiphenyl ether", "PBDE-88")
  CALL Species("C_{12}H_5Br_5O",                    "446254-56-4", "XGFLJLJXVIMCNR-UHFFFAOYSA-N", "2,2',3,4,6'-pentabromodiphenyl ether", "PBDE-89")
  CALL Species("C_{12}H_5Br_5O",                    "446254-57-5", "BATFXMGTVIESIQ-UHFFFAOYSA-N", "2,2',3,4',5-pentabromodiphenyl ether", "PBDE-90")
  CALL Species("C_{12}H_5Br_5O",                    "446254-58-6", "HWNJTZKDPNZUSO-UHFFFAOYSA-N", "2,2',3,4',6-pentabromodiphenyl ether", "PBDE-91")
  CALL Species("C_{12}H_5Br_5O",                    "446254-59-7", "QWSQOVAGRDRZLM-UHFFFAOYSA-N", "2,2',3,5,5'-pentabromodiphenyl ether", "PBDE-92")
  CALL Species("C_{12}H_5Br_5O",                    "446254-60-0", "BRTPVPJQMWLDNO-UHFFFAOYSA-N", "2,2',3,5,6-pentabromodiphenyl ether", "PBDE-93")
  CALL Species("C_{12}H_5Br_5O",                    "446254-61-1", "JOPASNJHCFYVHD-UHFFFAOYSA-N", "2,2',3,5,6'-pentabromodiphenyl ether", "PBDE-94")
  CALL Species("C_{12}H_5Br_5O",                    "446254-62-2", "BZDYRALIEYVMEP-UHFFFAOYSA-N", "2,2',3,5',6-pentabromodiphenyl ether", "PBDE-95")
  CALL Species("C_{12}H_5Br_5O",                    "446254-63-3", "ZFCJNRDWGBZUED-UHFFFAOYSA-N", "2,2',3,6,6'-pentabromodiphenyl ether", "PBDE-96")
  CALL Species("C_{12}H_5Br_5O",                    "446254-64-4", "MAGYDGJRSCULJL-UHFFFAOYSA-N", "2,2',3,4',5'-pentabromodiphenyl ether", "PBDE-97")
  CALL Species("C_{12}H_5Br_5O",                     "38463-82-0", "OCLWEJVGAUFXQU-UHFFFAOYSA-N", "2,2',3,4',6'-pentabromodiphenyl ether", "PBDE-98")
  CALL Species("C_{12}H_5Br_5O",                     "60348-60-9", "WHPVYXDFIXRKLN-UHFFFAOYSA-N", "2,2',4,4',5-pentabromodiphenyl ether", "PBDE-99")
  CALL Species("C_{12}H_5Br_5O",                    "189084-64-8", "NSKIRYMHNFTRLR-UHFFFAOYSA-N", "2,2',4,4',6-pentabromodiphenyl ether", "PBDE-100")
  CALL Species("C_{12}H_5Br_5O",                    "446254-65-5", "QUZWDWNIWWAQDI-UHFFFAOYSA-N", "2,2',4,5,5'-pentabromodiphenyl ether", "PBDE-101")
  CALL Species("C_{12}H_5Br_5O",                    "446254-66-6", "JHFMCUVMAIQWRI-UHFFFAOYSA-N", "2,2',4,5,6'-pentabromodiphenyl ether", "PBDE-102")
  CALL Species("C_{12}H_5Br_5O",                    "446254-67-7", "RJEMKRNASVHYKR-UHFFFAOYSA-N", "2,2',4,5',6-pentabromodiphenyl ether", "PBDE-103")
  CALL Species("C_{12}H_5Br_5O",                    "446254-68-8", "CRSCWEYUPUKHPI-UHFFFAOYSA-N", "2,2',4,6,6'-pentabromodiphenyl ether", "PBDE-104")
  CALL Species("C_{12}H_5Br_5O",                    "373594-78-6", "LBPWAGZGYNOKAM-UHFFFAOYSA-N", "2,3,3',4,4'-pentabromodiphenyl ether", "PBDE-105")
  CALL Species("C_{12}H_5Br_5O",                    "446254-69-9", "KLQKWMYXEWUAFP-UHFFFAOYSA-N", "2,3,3',4,5-pentabromodiphenyl ether", "PBDE-106")
  CALL Species("C_{12}H_5Br_5O",                    "446254-70-2", "OMGVAMFMRSETEG-UHFFFAOYSA-N", "2,3,3',4',5-pentabromodiphenyl ether", "PBDE-107")
  CALL Species("C_{12}H_5Br_5O",                    "446254-71-3", "VBKPKHVLHGOKOJ-UHFFFAOYSA-N", "2,3,3',4,5'-pentabromodiphenyl ether", "PBDE-108")
  CALL Species("C_{12}H_5Br_5O",                    "446254-72-4", "FXXXWTMLIQLDRP-UHFFFAOYSA-N", "2,3,3',4,6-pentabromodiphenyl ether", "PBDE-109")
  CALL Species("C_{12}H_5Br_5O",                    "446254-73-5", "LESZGJVTZILBTK-UHFFFAOYSA-N", "2,3,3',4',6-pentabromodiphenyl ether", "PBDE-110")
  CALL Species("C_{12}H_5Br_5O",                    "446254-74-6", "PCHDCOXHJBWEPW-UHFFFAOYSA-N", "2,3,3',5,5'-pentabromodiphenyl ether", "PBDE-111")
  CALL Species("C_{12}H_5Br_5O",                    "446254-75-7", "MFBMNSFADPTAKZ-UHFFFAOYSA-N", "2,3,3',5,6-pentabromodiphenyl ether", "PBDE-112")
  CALL Species("C_{12}H_5Br_5O",                    "446254-76-8", "OGZHLJXRGZFVLI-UHFFFAOYSA-N", "2,3,3',5',6-pentabromodiphenyl ether", "PBDE-113")
  CALL Species("C_{12}H_5Br_5O",                    "446254-77-9", "SFNAUTSNWPPDSY-UHFFFAOYSA-N", "2,3,4,4',5-pentabromodiphenyl ether", "PBDE-114")
  CALL Species("C_{12}H_5Br_5O",                    "446254-78-0", "BKTLDVXDOVSTEV-UHFFFAOYSA-N", "2,3,4,4',6-pentabromodiphenyl ether", "PBDE-115")
  CALL Species("C_{12}H_5Br_5O",                    "189084-65-9", "ACRQLFSHISNWRY-UHFFFAOYSA-N", "2,3,4,5,6-pentabromodiphenyl ether", "PBDE-116")
  CALL Species("C_{12}H_5Br_5O",                    "446254-79-1", "SOJBOGWFDBDWEG-UHFFFAOYSA-N", "2,3,4',5,6-pentabromodiphenyl ether", "PBDE-117")
  CALL Species("C_{12}H_5Br_5O",                    "446254-80-4", "VTMFEPLDDHZBGI-UHFFFAOYSA-N", "2,3',4,4',5-pentabromodiphenyl ether", "PBDE-118")
  CALL Species("C_{12}H_5Br_5O",                    "189084-66-0", "KXEOYBYEJCRPGB-UHFFFAOYSA-N", "2,3',4,4',6-pentabromodiphenyl ether", "PBDE-119")
  CALL Species("C_{12}H_5Br_5O",                    "417727-71-0", "AKSBEUHDCRZJAN-UHFFFAOYSA-N", "2,3',4,5,5'-pentabromodiphenyl ether", "PBDE-120")
  CALL Species("C_{12}H_5Br_5O",                    "446254-81-5", "GVGNVZBJVFDAAO-UHFFFAOYSA-N", "2,3',4,5',6-pentabromodiphenyl ether", "PBDE-121")
  CALL Species("C_{12}H_5Br_5O",                    "446254-82-6", "CDNHGSPFIUITTN-UHFFFAOYSA-N", "2,3,3',4',5'-pentabromodiphenyl ether", "PBDE-122")
  CALL Species("C_{12}H_5Br_5O",                    "446254-83-7", "SBKMUEQNZNDYFW-UHFFFAOYSA-N", "2,3',4,4',5'-pentabromodiphenyl ether", "PBDE-123")
  CALL Species("C_{12}H_5Br_5O",                    "446254-84-8", "FGHJTAAHIFEHLT-UHFFFAOYSA-N", "2,3',4',5,5'-pentabromodiphenyl ether", "PBDE-124")
  CALL Species("C_{12}H_5Br_5O",                    "446254-85-9", "SESXKFPOVUVGLR-UHFFFAOYSA-N", "2,3',4',5',6-pentabromodiphenyl ether", "PBDE-125")
  CALL Species("C_{12}H_5Br_5O",                    "366791-32-4", "SJNIIWPIAVQNRK-UHFFFAOYSA-N", "3,3',4,4',5-pentabromodiphenyl ether", "PBDE-126")
  CALL Species("C_{12}H_5Br_5O",                    "446254-86-0", "RATMRXKBPDCKCZ-UHFFFAOYSA-N", "3,3',4,5,5'-pentabromodiphenyl ether", "PBDE-127")
  CALL Species("C_{12}H_4Br_6O",                    "182677-28-7", "WFLVELCLEGVBIH-UHFFFAOYSA-N", "2,2',3,3',4,4'-hexabromodiphenyl ether", "PBDE-128")
  CALL Species("C_{12}H_4Br_6O",                    "446254-87-1", "PRNCVYAUCSGSOE-UHFFFAOYSA-N", "2,2',3,3',4,5-hexabromodiphenyl ether", "PBDE-129")
  CALL Species("C_{12}H_4Br_6O",                    "446254-88-2", "YURCHLXPAGSJHU-UHFFFAOYSA-N", "2,2',3,3',4,5'-hexabromodiphenyl ether", "PBDE-130")
  CALL Species("C_{12}H_4Br_6O",                    "446254-89-3", "MGKVPJFIGGBCBA-UHFFFAOYSA-N", "2,2',3,3',4,6-hexabromodiphenyl ether", "PBDE-131")
  CALL Species("C_{12}H_4Br_6O",                    "446254-90-6", "FFEKBOKDYRZGRV-UHFFFAOYSA-N", "2,2',3,3',4,6'-hexabromodiphenyl ether", "PBDE-132")
  CALL Species("C_{12}H_4Br_6O",                    "446254-91-7", "XTBFPFHQPGZZJX-UHFFFAOYSA-N", "2,2',3,3',5,5'-hexabromodiphenyl ether", "PBDE-133")
  CALL Species("C_{12}H_4Br_6O",                    "446254-92-8", "MIBDGPWSGDWIQR-UHFFFAOYSA-N", "2,2',3,3',5,6-hexabromodiphenyl ether", "PBDE-134")
  CALL Species("C_{12}H_4Br_6O",                    "446254-93-9", "AMGHASDTWACNCS-UHFFFAOYSA-N", "2,2',3,3',5,6'-hexabromodiphenyl ether", "PBDE-135")
  CALL Species("C_{12}H_4Br_6O",                    "446254-94-0", "NTWGDSLWLUPCDW-UHFFFAOYSA-N", "2,2',3,3',6,6'-hexabromodiphenyl ether", "PBDE-136")
  CALL Species("C_{12}H_4Br_6O",                    "446254-95-1", "HSTYYNPYXZYIAG-UHFFFAOYSA-N", "2,2',3,4,4',5-hexabromodiphenyl ether", "PBDE-137")
  CALL Species("C_{12}H_4Br_6O",                    "182677-30-1", "IZFQCEZFGCMHOM-UHFFFAOYSA-N", "2,2',3,4,4',5'-hexabromodiphenyl ether", "PBDE-138")
 !CALL Species("C_{12}H_4Br_6O",                    "446254-96-2", "YESDYWNWEVPOLZ-UHFFFAOYSA-N", "2,2',3,4,4',6-hexabromodiphenyl ether", "PBDE-139") ! not used yet
  CALL Species("C_{12}H_4Br_6O",                    "243982-83-4", "FLRODCDHJZNIGA-UHFFFAOYSA-N", "2,2',3,4,4',6'-hexabromodiphenyl ether", "PBDE-140")
  CALL Species("C_{12}H_4Br_6O",                    "446254-97-3", "XTXIYMGRRUJOIT-UHFFFAOYSA-N", "2,2',3,4,5,5'-hexabromodiphenyl ether", "PBDE-141")
  CALL Species("C_{12}H_4Br_6O",                    "446254-98-4", "LJDGJCNHVGGOFW-UHFFFAOYSA-N", "2,2',3,4,5,6-hexabromodiphenyl ether", "PBDE-142")
  CALL Species("C_{12}H_4Br_6O",                    "446254-99-5", "RQLZDUSZXOOBTM-UHFFFAOYSA-N", "2,2',3,4,5,6'-hexabromodiphenyl ether", "PBDE-143")
  CALL Species("C_{12}H_4Br_6O",                    "446255-00-1", "ZMSJCQOCTPYCQP-UHFFFAOYSA-N", "2,2',3,4,5',6-hexabromodiphenyl ether", "PBDE-144")
  CALL Species("C_{12}H_4Br_6O",                    "446255-01-2", "BTKLHMBWCRVCLC-UHFFFAOYSA-N", "2,2',3,4,6,6'-hexabromodiphenyl ether", "PBDE-145")
  CALL Species("C_{12}H_4Br_6O",                    "446255-02-3", "HGXPYDNHBUCRTR-UHFFFAOYSA-N", "2,2',3,4',5,5'-hexabromodiphenyl ether", "PBDE-146")
  CALL Species("C_{12}H_4Br_6O",                    "116995-33-6", "OWBKWMDBTWHGHS-UHFFFAOYSA-N", "2,2',3,4',5,6-hexabromodiphenyl ether", "PBDE-147")
  CALL Species("C_{12}H_4Br_6O",                    "446255-03-4", "OJMHGSMSQZEBFH-UHFFFAOYSA-N", "2,2',3,4',5,6'-hexabromodiphenyl ether", "PBDE-148")
  CALL Species("C_{12}H_4Br_6O",                    "446255-04-5", "UJOUSZKYGGTPFQ-UHFFFAOYSA-N", "2,2',3,4',5',6-hexabromodiphenyl ether", "PBDE-149")
  CALL Species("C_{12}H_4Br_6O",                    "446255-05-6", "SQNOZOVDXXSLSG-UHFFFAOYSA-N", "2,2',3,4',6,6'-hexabromodiphenyl ether", "PBDE-150")
  CALL Species("C_{12}H_4Br_6O",                    "446255-06-7", "NGOQQUYCSISZMY-UHFFFAOYSA-N", "2,2',3,5,5',6-hexabromodiphenyl ether", "PBDE-151")
  CALL Species("C_{12}H_4Br_6O",                    "446255-07-8", "BYBJJARTBKUIJD-UHFFFAOYSA-N", "2,2',3,5,6,6'-hexabromodiphenyl ether", "PBDE-152")
  CALL Species("C_{12}H_4Br_6O",                     "68631-49-2", "RZXIRSKYBISPGF-UHFFFAOYSA-N", "2,2',4,4',5,5'-hexabromodiphenyl ether", "PBDE-153")
  CALL Species("C_{12}H_4Br_6O",                    "207122-15-4", "VHNPZYZQKWIWOD-UHFFFAOYSA-N", "2,2',4,4',5,6'-hexabromodiphenyl ether", "PBDE-154")
  CALL Species("C_{12}H_4Br_6O",                     "35854-94-5", "HRSCBOSGEKXXSI-UHFFFAOYSA-N", "2,2',4,4',6,6'-hexabromodiphenyl ether", "PBDE-155")
  CALL Species("C_{12}H_4Br_6O",                    "405237-85-6", "JSDPCMJWYRDQEV-UHFFFAOYSA-N", "2,3,3',4,4',5-hexabromodiphenyl ether", "PBDE-156")
  CALL Species("C_{12}H_4Br_6O",                    "446255-08-9", "JUOAMVUIJQJZSZ-UHFFFAOYSA-N", "2,3,3',4,4',5'-hexabromodiphenyl ether", "PBDE-157")
  CALL Species("C_{12}H_4Br_6O",                    "446255-09-0", "KRYHHTVQOOJNHQ-UHFFFAOYSA-N", "2,3,3',4,4',6-hexabromodiphenyl ether", "PBDE-158")
  CALL Species("C_{12}H_4Br_6O",                    "446255-10-3", "IDYFFNCFLRCOPZ-UHFFFAOYSA-N", "2,3,3',4,5,5'-hexabromodiphenyl ether", "PBDE-159")
  CALL Species("C_{12}H_4Br_6O",                    "446255-11-4", "OCVOYHGOXIIONK-UHFFFAOYSA-N", "2,3,3',4,5,6-hexabromodiphenyl ether", "PBDE-160")
  CALL Species("C_{12}H_4Br_6O",                    "446255-12-5", "WEYWRBBPPKSRGU-UHFFFAOYSA-N", "2,3,3',4,5',6-hexabromodiphenyl ether", "PBDE-161")
  CALL Species("C_{12}H_4Br_6O",                    "446255-13-6", "UKPNCLHMNJCGCJ-UHFFFAOYSA-N", "2,3,3',4',5,5'-hexabromodiphenyl ether", "PBDE-162")
  CALL Species("C_{12}H_4Br_6O",                    "446255-14-7", "NUEAHMLXQFHEJN-UHFFFAOYSA-N", "2,3,3',4',5,6-hexabromodiphenyl ether", "PBDE-163")
  CALL Species("C_{12}H_4Br_6O",                    "446255-15-8", "UJVYVXIHTJOJBZ-UHFFFAOYSA-N", "2,3,3',4',5',6-hexabromodiphenyl ether", "PBDE-164")
  CALL Species("C_{12}H_4Br_6O",                    "446255-16-9", "KXERERDGMTWBGZ-UHFFFAOYSA-N", "2,3,3',5,5',6-hexabromodiphenyl ether", "PBDE-165")
  CALL Species("C_{12}H_4Br_6O",                    "189084-58-0", "KVYODBMKQYVNEK-UHFFFAOYSA-N", "2,3,4,4',5,6-hexabromodiphenyl ether", "PBDE-166")
  CALL Species("C_{12}H_4Br_6O",                    "446255-17-0", "NMUPLZRHSXJCJQ-UHFFFAOYSA-N", "2,3',4,4',5,5'-hexabromodiphenyl ether", "PBDE-167")
  CALL Species("C_{12}H_4Br_6O",                     "53551-87-4", "HWZAPXGFMVEGPW-UHFFFAOYSA-N", "2,3',4,4',5',6-hexabromodiphenyl ether", "PBDE-168")
  CALL Species("C_{12}H_4Br_6O",                    "446255-18-1", "JKFBMDHBJYKFKL-UHFFFAOYSA-N", "3,3',4,4',5,5'-hexabromodiphenyl ether", "PBDE-169")
  CALL Species("C_{12}H_3Br_7O",                    "327185-13-7", "DLPNCMQTNWLTHD-UHFFFAOYSA-N", "2,2',3,3',4,4',5-heptabromodiphenyl ether", "PBDE-170")
  CALL Species("C_{12}H_3Br_7O",                    "446255-19-2", "FRMMMROUUPQUMZ-UHFFFAOYSA-N", "2,2',3,3',4,4',6-heptabromodiphenyl ether", "PBDE-171")
  CALL Species("C_{12}H_3Br_7O",                    "407606-59-1", "DSRRSKFMOJQETR-UHFFFAOYSA-N", "2,2',3,3',4,5,5'-heptabromodiphenyl ether", "PBDE-172")
  CALL Species("C_{12}H_3Br_7O",                    "446255-20-5", "NLBLNZDNOSSGPW-UHFFFAOYSA-N", "2,2',3,3',4,5,6-heptabromodiphenyl ether", "PBDE-173")
  CALL Species("C_{12}H_3Br_7O",                    "446255-21-6", "VUUWOHUOYUGBEO-UHFFFAOYSA-N", "2,2',3,3',4,5,6'-heptabromodiphenyl ether", "PBDE-174")
  CALL Species("C_{12}H_3Br_7O",                      "6255-22-7", "YATZWTXATDYQCK-UHFFFAOYSA-N", "2,2',3,3',4,5',6-heptabromodiphenyl ether", "PBDE-175")
  CALL Species("C_{12}H_3Br_7O",                    "407606-61-5", "SWUALKCOTZOSMY-UHFFFAOYSA-N", "2,2',3,3',4,6,6'-heptabromodiphenyl ether", "PBDE-176")
  CALL Species("C_{12}H_3Br_7O",                    "446255-23-8", "ZHUHLPXIJIBQBJ-UHFFFAOYSA-N", "2,2',3,3',4,5',6'-heptabromodiphenyl ether", "PBDE-177")
  CALL Species("C_{12}H_3Br_7O",                    "446255-24-9", "UWUVZUPEEORCRG-UHFFFAOYSA-N", "2,2',3,3',5,5',6-heptabromodiphenyl ether", "PBDE-178")
  CALL Species("C_{12}H_3Br_7O",                    "446255-25-0", "COVXWWKOLMNRQE-UHFFFAOYSA-N", "2,2',3,3',5,6,6'-heptabromodiphenyl ether", "PBDE-179")
  CALL Species("C_{12}H_3Br_7O",                    "446255-26-1", "STMBXVOJNOJRPZ-UHFFFAOYSA-N", "2,2',3,4,4',5,5'-heptabromodiphenyl ether", "PBDE-180")
  CALL Species("C_{12}H_3Br_7O",                    "189084-67-1", "GVNRIAPLVGNZPL-UHFFFAOYSA-N", "2,2',3,4,4',5,6-heptabromodiphenyl ether", "PBDE-181")
  CALL Species("C_{12}H_3Br_7O",                    "442690-45-1", "ZYHDTADADSNMLV-UHFFFAOYSA-N", "2,2',3,4,4',5,6'-heptabromodiphenyl ether", "PBDE-182")
  CALL Species("C_{12}H_3Br_7O",                    "207122-16-5", "ILPSCQCLBHQUEM-UHFFFAOYSA-N", "2,2',3,4,4',5',6-heptabromodiphenyl ether", "PBDE-183")
  CALL Species("C_{12}H_3Br_7O",                    "117948-63-7", "JHDCZVAQPRXHEL-UHFFFAOYSA-N", "2,2',3,4,4',6,6'-heptabromodiphenyl ether", "PBDE-184")
  CALL Species("C_{12}H_3Br_7O",                    "405237-86-7", "YRNMIFAQDSUFTR-UHFFFAOYSA-N", "2,2',3,4,5,5',6-heptabromodiphenyl ether", "PBDE-185")
  CALL Species("C_{12}H_3Br_7O",                    "446255-27-2", "WUFQDCMRKKDNSF-UHFFFAOYSA-N", "2,2',3,4,5,6,6'-heptabromodiphenyl ether", "PBDE-186")
  CALL Species("C_{12}H_3Br_7O",                    "446255-28-3", "RFZPXOBFDARWHV-UHFFFAOYSA-N", "2,2',3,4',5,5',6-heptabromodiphenyl ether", "PBDE-187")
  CALL Species("C_{12}H_3Br_7O",                    "116995-32-5", "YGYDHFDPVGAMTL-UHFFFAOYSA-N", "2,2',3,4',5,6,6'-heptabromodiphenyl ether", "PBDE-188")
  CALL Species("C_{12}H_3Br_7O",                    "259087-35-9", "CQVLRTUESBMMJW-UHFFFAOYSA-N", "2,3,3',4,4',5,5'-heptabromodiphenyl ether", "PBDE-189")
  CALL Species("C_{12}H_3Br_7O",                    "189084-68-2", "OUEYHQIMJGHOQN-UHFFFAOYSA-N", "2,3,3',4,4',5,6-heptabromodiphenyl ether", "PBDE-190")
  CALL Species("C_{12}H_3Br_7O",                    "446255-30-7", "BNBFKFHSIPERIM-UHFFFAOYSA-N", "2,3,3',4,4',5',6-heptabromodiphenyl ether", "PBDE-191")
  CALL Species("C_{12}H_3Br_7O",                    "407578-53-4", "ABLZOLAUBUSUHT-UHFFFAOYSA-N", "2,3,3',4,5,5',6-heptabromodiphenyl ether", "PBDE-192")
  CALL Species("C_{12}H_3Br_7O",                    "446255-34-1", "AUFJSWANTKXCFZ-UHFFFAOYSA-N", "2,3,3',4',5,5',6-heptabromodiphenyl ether", "PBDE-193")
  CALL Species("C_{12}H_2Br_8O",                     "32536-52-0", "ORYGKUIDIMIRNN-UHFFFAOYSA-N", "2,2',3,3',4,4',5,5'-octabromodiphenyl ether", "PBDE-194")
  CALL Species("C_{12}H_2Br_8O",                    "446255-38-5", "GPQLSLKPHQEEOP-UHFFFAOYSA-N", "2,2',3,3',4,4',5,6-octabromodiphenyl ether", "PBDE-195")
  CALL Species("C_{12}H_2Br_8O",                    "446255-39-6", "IEWFKOVTVJNWFF-UHFFFAOYSA-N", "2,2',3,3',4,4',5,6'-octabromodiphenyl ether", "PBDE-196")
  CALL Species("C_{12}H_2Br_8O",                    "117964-21-3", "AAFUUKPTSPVXJH-UHFFFAOYSA-N", "2,2',3,3',4,4',6,6'-octabromodiphenyl ether", "PBDE-197")
  CALL Species("C_{12}H_2Br_8O",                    "446255-42-1", "IBKRHVDFFHQOSC-UHFFFAOYSA-N", "2,2',3,3',4,5,5',6-octabromodiphenyl ether", "PBDE-198")
  CALL Species("C_{12}H_2Br_8O",                    "446255-43-2", "JNSLJYRXDGBNBE-UHFFFAOYSA-N", "2,2',3,3',4,5,5',6'-octabromodiphenyl ether", "PBDE-199")
  CALL Species("C_{12}H_2Br_8O",                    "446255-46-5", "JWMXGEPFVCRXQR-UHFFFAOYSA-N", "2,2',3,3',4,5,6,6'-octabromodiphenyl ether", "PBDE-200")
  CALL Species("C_{12}H_2Br_8O",                    "446255-50-1", "HQWFMMKREWXIGN-UHFFFAOYSA-N", "2,2',3,3',4,5',6,6'-octabromodiphenyl ether", "PBDE-201")
  CALL Species("C_{12}H_2Br_8O",                     "67797-09-5", "AHNZLQAZTWRRDW-UHFFFAOYSA-N", "2,2',3,3',5,5',6,6'-octabromodiphenyl ether", "PBDE-202")
  CALL Species("C_{12}H_2Br_8O",                    "337513-72-1", "RTUZOQFRIPIWPS-UHFFFAOYSA-N", "2,2',3,4,4',5,5',6-octabromodiphenyl ether", "PBDE-203")
  CALL Species("C_{12}H_2Br_8O",                    "446255-54-5", "YZABCBOJTHQTSX-UHFFFAOYSA-N", "2,2',3,4,4',5,6,6'-octabromodiphenyl ether", "PBDE-204")
  CALL Species("C_{12}H_2Br_8O",                    "446255-56-7", "CVMKCYDBEYHNBM-UHFFFAOYSA-N", "2,3,3',4,4',5,5',6-octabromodiphenyl ether", "PBDE-205")
  CALL Species("C_{12}HBr_9O",                       "63387-28-0", "CYRHBNRLQMLULE-UHFFFAOYSA-N", "2,2',3,3',4,4',5,5',6-nonabromodiphenyl ether", "PBDE-206")
  CALL Species("C_{12}HBr_9O",                      "437701-79-6", "IEEVDIAVLGLVOW-UHFFFAOYSA-N", "2,2',3,3',4,4',5,6,6'-nonabromodiphenyl ether", "PBDE-207")
  CALL Species("C_{12}HBr_9O",                      "437701-78-5", "ASGZXYIDLFWXID-UHFFFAOYSA-N", "2,2',3,3',4,5,5',6,6'-nonabromodiphenyl ether", "PBDE-208")
  CALL Species("C_{12}Br_{10}O",                      "1163-19-5", "WHHGLZMJPXIBIX-UHFFFAOYSA-N", "decabromodiphenyl ether", "PBDE-209") ! other CAS: 145538-74-5
  ! C, H, N, Br:
  CALL Species("C_2HBr_2N",                           "3252-43-5", "NDSBDLSWTGLNQA-UHFFFAOYSA-N", "dibromoacetonitrile", "")
  CALL Species("C_2H_2BrN",                            "590-17-0", "REXUYBKPWIPONM-UHFFFAOYSA-N", "bromoacetonitrile", "")
  CALL Species("C_5H_4BrN",                            "109-04-6", "IMRWILPUOVGIMU-UHFFFAOYSA-N", "2-bromopyridine", "")
  CALL Species("C_5H_4BrN",                            "626-55-1", "NYPYPOZNGOXYSU-UHFFFAOYSA-N", "3-bromopyridine", "")
  CALL Species("C_6H_6Br_2N_2",                      "35691-65-7", "DHVLDKHFGIVEIP-UHFFFAOYSA-N", "1,2-dibromo-2,4-dicyanobutane", "")
  CALL Species("C_6H_6BrN",                            "106-40-1", "WDFQBORIUYODSI-UHFFFAOYSA-N", "4-bromobenzenamine", "")
  CALL Species("C_6H_4Br_3N",                          "147-82-0", "GVPODVKBTHCGFU-UHFFFAOYSA-N", "2,4,6-tribromobenzenamine", "")
  CALL Species("C_{10}H_6Br_6N_2",                  "253798-63-9", "BUKWTHPBLJVYMZ-UHFFFAOYSA-N", "N,N'-dimethyl-3,3',4,4',5,5'-hexabromo-2,2'-bipyrrole", "DBP-Br6")
  CALL Species("C_{12}H_{12}N_2Br",                     "85-00-7", "JXEXEPZXXFNEHA-UHFFFAOYSA-M", "1,1'-ethylene 2,2'-dipyridylium dibromide", "diquat dibromide")
  CALL Species("C_{22}H_{19}NO_3Br_4",               "66841-25-6", "YWSCPYYRJXKUDB-KAKFPZCNSA-N", "tralomethrin", "")
  ! C, H, O, N, Br:
  CALL Species("CH_2BrO_2NO_2",                                "", "JTAFUKGUWIYDNI-UHFFFAOYSA-N", "bromomethyl peroxynitrate", "")
  CALL Species("CHBr_2O_2NO_2",                                "", "CRBGJNSRTFAVGH-UHFFFAOYSA-N", "dibromomethyl peroxynitrate", "")
  CALL Species("CBr_3O_2NO_2",                                 "", "BVMOLEXBJGZUAI-UHFFFAOYSA-N", "tribromomethyl peroxynitrate", "")
  CALL Species("C_2H_2NO_5Br",                                 "", "OOFSXJVGALRPLK-UHFFFAOYSA-N", "MCM:BRETPAN", "")
  CALL Species("C_3H_2Br_2N_2O",                     "10222-01-2", "UUIVKBHZENILKB-UHFFFAOYSA-N", "2,2-dibromo-2-cyanoacetamide", "2,2-dibromo-3-nitrilopropionamide")
  CALL Species("C_3H_6BrNO_4",                          "52-51-7", "LVDKZNITIUWNER-UHFFFAOYSA-N", "bronopol", "")
  CALL Species("C_6H_4Br_2N_2O_2",                     "827-94-1", "YMZIFDLWYUSZCC-UHFFFAOYSA-N", "2,6-dibromo-4-nitroaniline", "")
  CALL Species("C_6H_4BrN_3O_4",                      "1817-73-8", "KWMDHCLJYMVBNS-UHFFFAOYSA-N", "2-bromo-4,6-dinitroaniline", "")
  CALL Species("C_6H_4BrNO_2",                         "585-79-5", "FWIROFMBWVMWLB-UHFFFAOYSA-N", "3-bromonitrobenzene", "$m$-bromonitrobenzene")
  CALL Species("C_6H_4BrNO_2",                         "586-78-7", "ZDFBKZUDCQQKAC-UHFFFAOYSA-N", "4-bromonitrobenzene", "$p$-bromonitrobenzene")
  CALL Species("C_7H_3Br_2NO",                        "1689-84-5", "UPMXNNIRAGDFEH-UHFFFAOYSA-N", "3,5-dibromo-4-hydroxy-benzonitrile", "bromoxynil")
  CALL Species("C_8H_7Br_2NO_3",                     "62265-99-0", "RBAJFFLBHVZCDY-UHFFFAOYSA-N", "2,6-dibromo-3-methyl-4-nitroanisole", "")
  CALL Species("C_9H_{13}BrN_2O_2",                    "314-40-9", "CTSLUCNDVMMDHG-UHFFFAOYSA-N", "bromacil", "")
  CALL Species("C_9H_{11}BrN_2O",                     "3060-89-7", "WLFDQEVORAMCIM-UHFFFAOYSA-N", "N'-(4-bromophenyl)-N-methoxy-N-methylurea", "metobromuron")
  CALL Species("C_9H_{11}BrN_2O",                     "3408-97-7", "GSNZNZUNAJCHDO-UHFFFAOYSA-N", "bromuron", "")
  CALL Species("C_{11}H_9Br_2NO_2",                   "3861-41-4", "PGMZYNZXIYOOHJ-UHFFFAOYSA-N", "bromoxynil butyrate", "")
  CALL Species("C_{12}H_{15}Br_6N_3O_3",             "52434-90-9", "NZUPFZNVGSWLQC-UHFFFAOYSA-N", "tris(2,3-dibromopropyl)isocyanurate", "")
  CALL Species("C_{13}H_7Br_2N_3O_6",                "13181-17-4", "VTQWKUZUPOTXEH-UHFFFAOYSA-N", "bromofenoxim", "")
  CALL Species("C_{13}H_8Br_3NO_2",                     "87-10-5", "KVSKGMLNBAPGKH-UHFFFAOYSA-N", "tribromsalan", "")
  CALL Species("C_{14}H_{15}Br_2NO_2",               "56634-95-8", "BHZWBQPHPLFZSV-UHFFFAOYSA-N", "bromoxynil heptanoate", "")
  CALL Species("C_{14}H_7Br_2NO_2",                     "81-49-2", "ZINRVIQBCHAZMM-UHFFFAOYSA-N", "1-amino-2,4-dibromo-9,10-anthracenedione", "1-amino-2,4-dibromoanthraquinone")
  CALL Species("C_{15}H_{17}BrNO_2",                  "1689-99-2", "DQKWXTIYGWPGOO-UHFFFAOYSA-N", "2,6-dibromo-4-cyanophenyl octanoate", "")
  CALL Species("C_{15}H_{22}BrNO",                   "74712-19-9", "WZDDLAZXUYIVMU-UHFFFAOYSA-N", "bromobutide", "")
  CALL Species("C_{16}H_8Br_6N_2O_2",                "92484-07-6", "IJUNKLAVMFKPCX-OWOJBTEDSA-N", "(2$E$)-N,N'-bis(2,4,6-tribromophenyl)-2-butenediamide", "")
  CALL Species("C_{18}H_4Br_8N_2O_4",                "32588-76-4", "DYIZJUDNMOIZQO-UHFFFAOYSA-N", "SAYTEX BT 93", "")
  CALL Species("C_{20}H_{20}Br_4N_2O_4",             "52907-07-0", "WOFYQUJNULCFLN-UHFFFAOYSA-N", "SAYTEX BN 451", "")
  CALL Species("C_{22}H_{19}Br_2NO_3",               "52918-63-5", "OWZREIFADZCYQD-NSHGMRRFSA-N", "deltamethrin", "")
  CALL Species("C_{29}H_{10}N_2O_4Br_8",             "32588-74-2", "RHFKNNBDKARGKL-UHFFFAOYSA-N", "2,2'-(methylenedi-4,1-phenylene)bis(4,5,6,7-tetrabromo-1H-isoindole-1,3(2H)-dione", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Bromine, chlorine and fluorine (\chem{C}, \chem{H}, \chem{N}, \chem{O}, \chem{F}, \chem{Cl}, \chem{Br})")
  ! C, H, F, Br:
  ! aliphatic:
  CALL Species("CF_3Br",                                "75-63-8", "RJCQBQGAPKAMLL-UHFFFAOYSA-N", "bromotrifluoromethane", "")
  CALL Species("CBr_2F_2",                              "75-61-6", "AZSZCFSOHXEJQE-UHFFFAOYSA-N", "dibromodifluoromethane", "")
  CALL Species("C_2HBrF_4",                            "124-72-1", "RZXZIZDRFQFCTA-UHFFFAOYSA-N", "1-bromo-1,2,2,2-tetrafluoroethane", "teflurane")
  CALL Species("C_2Br_2F_4",                           "124-73-2", "KVBKAPANDHPRDG-UHFFFAOYSA-N", "1,2-dibromotetrafluoroethane", "")
  ! aromatic:
  CALL Species("C_6H_4BrF",                           "1072-85-1", "IPWBFGUBXWMIPR-UHFFFAOYSA-N", "1-bromo-2-fluorobenzene", "")
  CALL Species("C_6H_4BrF",                           "1073-06-9", "QDFKKJYEIFBEFC-UHFFFAOYSA-N", "1-bromo-3-fluorobenzene", "")
  CALL Species("C_6H_4BrF",                            "460-00-4", "AITNMTXHTIIIBB-UHFFFAOYSA-N", "1-bromo-4-fluorobenzene", "")
  CALL Species("C_6BrF_5",                             "344-04-7", "XEKTVXADUPBFOA-UHFFFAOYSA-N", "bromopentafluorobenzene", "")
  ! C, H, O, F, Br:
  CALL Species("C_{24}H_{23}BrF_2O_3",              "111872-58-3", "WIFXJBMOTMKRMM-UHFFFAOYSA-N", "halfenprox", "")
  ! C, H, O, N, F, Br:
  CALL Species("C_{14}H_7Br_3F_3N_3O_4",             "63333-35-7", "USMZPYXTVKAYST-UHFFFAOYSA-N", "bromethalin", "")
  CALL Species("C_{26}H_{22}BrF_2NO_4",             "160791-64-0", "BUHNCQOJJZAOMJ-UHFFFAOYSA-N", "brofluthrinate", "")
  ! C, H, Cl, Br:
  ! aliphatic:
  CALL Species("CH_2BrCl",                              "74-97-5", "JPOXNPPZZKNXOV-UHFFFAOYSA-N", "bromochloromethane", "")
  CALL Species("CHCl_2Br",                              "75-27-4", "FMWLUWPQPKEARP-UHFFFAOYSA-N", "bromodichloromethane", "")
  CALL Species("CBrCl_3",                               "75-62-7", "XNNQFQFUQLJSQT-UHFFFAOYSA-N", "bromotrichloromethane", "")
  CALL Species("CHClBr_2",                             "124-48-1", "GATVIKZLVQHOMN-UHFFFAOYSA-N", "dibromochloromethane", "")
  CALL Species("C_2H_4BrCl",                           "107-04-0", "IBYHHJPAARCAIE-UHFFFAOYSA-N", "1-chloro-2-bromoethane", "")
  CALL Species("C_2H_2Br_2Cl_2",                        "75-81-0", "FIYBYNHDEOSJPL-UHFFFAOYSA-N", "1,2-dibromo-1,1-dichloroethane", "")
  CALL Species("C_3H_6BrCl",                           "109-70-6", "MFESCIUQSIBMSM-UHFFFAOYSA-N", "1-bromo-3-chloropropane", "")
  CALL Species("C_3H_5Br_2Cl",                          "96-12-8", "WBEJYOJJBDISQU-UHFFFAOYSA-N", "1,2-dibromo-3-chloropropane", "")
  CALL Species("C_6H_6Br_5Cl",                          "87-84-3", "UZOSVZSBPTTWIG-UHFFFAOYSA-N", "1,2,3,4,5-pentabromo-6-chlorocyclohexane", "")
  CALL Species("C_6H_6Br_4Cl_2",                               "", "GABFTOZBVQIDBB-UHFFFAOYSA-N", "1,2,3,4-tetrabromo-5,6-dichlorocyclohexane", "")
  CALL Species("C_6H_6Br_3Cl_3",                               "", "ZDEQCIGFZLZXBZ-UHFFFAOYSA-N", "1,2,3-tribromo-4,5,6-trichlorocyclohexane", "")
  ! aromatic:
  CALL Species("C_6H_4BrCl",                           "106-39-8", "NHDODQWIKUYWMW-UHFFFAOYSA-N", "1-bromo-4-chlorobenzene", "")
  CALL Species("C_6Br_3Cl_3",                        "13075-01-9", "XAIKFWSNTFEXDC-UHFFFAOYSA-N", "1,2,4-tribromo-3,5,6-trichlorobenzene", "")
  CALL Species("C_7H_6BrCl",                           "611-17-6", "PURSZYWBIQIANP-UHFFFAOYSA-N", "1-(bromomethyl)-2-chlorobenzene", "")
  ! C, H, O, Cl, Br:
  CALL Species("C_7H_6BrClO",                        "60633-25-2", "YJEMGEBDXDPBSP-UHFFFAOYSA-N", "2-bromo-4-chloro-1-methoxybenzene", "2-bromo-4-chloroanisole")
  CALL Species("C_7H_6BrClO",                       "174913-10-1", "NWOYYECMNBWCNK-UHFFFAOYSA-N", "2-bromo-6-chloro-1-methoxybenzene", "2-bromo-6-chloroanisole")
  CALL Species("C_7H_6BrClO",                        "50638-47-6", "FPIQNBOUYZLESW-UHFFFAOYSA-N", "4-bromo-2-chloro-1-methoxybenzene", "4-bromo-2-chloroanisole")
  CALL Species("C_7H_5BrCl_2O",                      "73931-43-8", "LCHJNNXHVDDVAT-UHFFFAOYSA-N", "2-bromo-3,5-dichloro-1-methoxybenzene", "2-bromo-3,5-dichloroanisole")
  CALL Species("C_7H_5BrCl_2O",                      "60633-26-3", "OEYKUHBCPJRXGZ-UHFFFAOYSA-N", "2-bromo-4,6-dichloro-1-methoxybenzene", "2-bromo-4,6-dichloroanisole")
  CALL Species("C_7H_5BrCl_2O",                     "109803-52-3", "BTRCDLZQISXWHZ-UHFFFAOYSA-N", "4-bromo-2,3-dichloro-1-methoxybenzene", "4-bromo-2,3-dichloroanisole")
  CALL Species("C_7H_5BrCl_2O",                      "19240-91-6", "OAYSFAKCFYRCRU-UHFFFAOYSA-N", "4-bromo-2,6-dichloro-1-methoxybenzene", "4-bromo-2,6-dichloroanisole")
  CALL Species("C_7H_5BrCl_2O",                     "174913-20-3", "SVEZPKVUIXAEOX-UHFFFAOYSA-N", "4-bromo-3,5-dichloro-1-methoxybenzene", "4-bromo-3,5-dichloroanisole")
  CALL Species("C_7H_5BrCl_2O",                     "174913-16-7", "OXINPOQPENRUHL-UHFFFAOYSA-N", "3-bromo-2,4-dichloro-1-methoxybenzene", "3-bromo-2,4-dichloroanisole")
  CALL Species("C_7H_5BrCl_2O",                     "174913-18-9", "ANLVZYYGYBXMIX-UHFFFAOYSA-N", "3-bromo-2,6-dichloro-1-methoxybenzene", "3-bromo-2,6-dichloroanisole")
  CALL Species("C_7H_5BrCl_2O",                     "174913-22-5", "SJIIFORJBWPNPM-UHFFFAOYSA-N", "5-bromo-2,4-dichloro-1-methoxybenzene", "5-bromo-2,4-dichloroanisole")
  CALL Species("C_7H_5BrCl_2O",                     "174913-23-6", "PNLDDSLZYVNPQW-UHFFFAOYSA-N", "6-bromo-2,3-dichloro-1-methoxybenzene", "6-bromo-2,3-dichloroanisole")
  CALL Species("C_7H_5BrCl_2O",                     "174913-14-5", "BVSOVPUWHJWGEP-UHFFFAOYSA-N", "6-bromo-2,5-dichloro-1-methoxybenzene", "6-bromo-2,5-dichloroanisole")
  CALL Species("C_7H_4BrCl_3O",                                "", "JIVCPRHFBFVZQL-UHFFFAOYSA-N", "2-bromo-3,4,5-trichloro-1-methoxybenzene", "2-bromo-3,4,5-trichloroanisole")
  CALL Species("C_7H_4BrCl_3O",                     "174913-28-1", "OIHHRLYOSUNYTC-UHFFFAOYSA-N", "3-bromo-2,4,6-trichloro-1-methoxybenzene", "3-bromo-2,4,6-trichloroanisole")
  CALL Species("C_7H_4BrCl_3O",                      "78647-93-5", "ZUXVGCIQOSXRJN-UHFFFAOYSA-N", "3-bromo-2,5,6-trichloro-1-methoxybenzene", "3-bromo-2,5,6-trichloroanisole")
  CALL Species("C_7H_4BrCl_3O",                      "78647-87-7", "VZRSUQWPXPUWCS-UHFFFAOYSA-N", "4-bromo-2,3,6-trichloro-1-methoxybenzene", "4-bromo-2,3,6-trichloroanisole")
  CALL Species("C_7H_4BrCl_3O",                                "", "GVCDCZPXMGMNTR-UHFFFAOYSA-N", "6-bromo-2,3,4-trichloro-1-methoxybenzene", "6-bromo-2,3,4-trichloroanisole")
  CALL Species("C_7H_3BrCl_4O",                     "174913-33-8", "YSVAURKXNCNUNS-UHFFFAOYSA-N", "4-bromo-2,3,5,6-tetrachloro-1-methoxybenzene", "4-bromo-2,3,5,6-tetrachloroanisole")
  CALL Species("C_7H_5Br_2ClO",                                "", "YXYRJAVLYYNRCE-UHFFFAOYSA-N", "2,4-dibromo-3-chloro-1-methoxybenzene", "2,4-dibromo-3-chloroanisole")
  CALL Species("C_7H_5Br_2ClO",                     "174913-38-3", "OTZRCEAVEJYZQS-UHFFFAOYSA-N", "2,4-dibromo-5-chloro-1-methoxybenzene", "2,4-dibromo-5-chloroanisole")
  CALL Species("C_7H_5Br_2ClO",                                "", "AHBGIHFKTBTQJP-UHFFFAOYSA-N", "2,6-dibromo-3-chloro-1-methoxybenzene", "2,6-dibromo-3-chloroanisole")
  CALL Species("C_7H_5Br_2ClO",                     "174913-44-1", "WHRDPXDGAMJNGH-UHFFFAOYSA-N", "2,6-dibromo-4-chloro-1-methoxybenzene", "2,6-dibromo-4-chloroanisole")
  CALL Species("C_7H_4Br_2Cl_2O",                   "174913-52-1", "YZMXEWDTXSTFIJ-UHFFFAOYSA-N", "2,4-dibromo-3,5-dichloro-1-methoxybenzene", "2,4-dibromo-3,5-dichloroanisole")
  CALL Species("C_7H_4Br_2Cl_2O",                              "", "MUUDTVDSRYEMBU-UHFFFAOYSA-N", "2,4-dibromo-5,6-dichloro-1-methoxybenzene", "2,4-dibromo-5,6-dichloroanisole")
  CALL Species("C_7H_4Br_2Cl_2O",                              "", "WDRCOFFEZVLCLX-UHFFFAOYSA-N", "2,3-dibromo-5,6-dichloro-1-methoxybenzene", "2,3-dibromo-5,6-dichloroanisole")
  CALL Species("C_7H_3Br_2Cl_3O",                              "", "RFUGYJHXLQAXOX-UHFFFAOYSA-N", "2,6-dibromo-3,4,5-trichloro-1-methoxybenzene", "2,6-dibromo-3,4,5-trichloroanisole")
  CALL Species("C_7H_4Br_3ClO",                     "174913-78-1", "FWKUBDOTUFVPBX-UHFFFAOYSA-N", "2,4,6-tribromo-3-chloro-1-methoxybenzene", "2,4,6-tribromo-3-chloroanisole")
  CALL Species("C_{12}H_4Br_2Cl_2O_2",               "50585-40-5", "SBSJXPIUKNBZND-UHFFFAOYSA-N", "2,3-dibromo-7,8-dichlorodibenzo-$p$-dioxin", "")
  CALL Species("C_{20}H_4Br_4Cl_4O_5",               "13473-26-2", "ZYIBVBKZZZDFOY-UHFFFAOYSA-N", "2',4',5',7'-tetrabromo-3,4,5,6-tetrachlorofluorescein", "")
  ! C, H, N, Cl, Br:
  CALL Species("C_2HBrClN",                          "83463-62-1", "BMWPPNAUMLRKML-UHFFFAOYSA-N", "bromochloroacetonitrile", "")
  CALL Species("C_{10}H_6Br_3Cl_3N_2",              "400766-93-0", "JSQYFPHYSOZLPT-UHFFFAOYSA-N", "N,N'-dimethyl-3,3',4-tribromo-4,5,5'-trichloro-2,2'-bipyrrole", "DBP-Br3Cl3a")
  CALL Species("C_{10}H_6Br_3Cl_3N_2",              "666856-68-4", "GUWAXPTVCYBOMV-UHFFFAOYSA-N", "N,N'-dimethyl- 3,4,4'-tribromo-3',5,5'-trichloro-2,2'-bipyrrole", "DBP-Br3Cl3b") ! cas from ref2521, not verified
  CALL Species("C_{10}H_6Br_4Cl_2N_2",              "253798-64-0", "FQGSQKFVAAJWFY-UHFFFAOYSA-N", "N,N'-dimethyl-3,3',4,4'-tetrabromo-5,5'-dichloro-2,2'-bipyrrole", "DBP-Br4Cl2")
  CALL Species("C_{10}H_6Br_5ClN_2",                "400767-00-2", "LIYSVIAQQQEQBT-UHFFFAOYSA-N", "N,N'-dimethyl-3,3',4,4',5-pentabromo-5'-chloro-2,2'-bipyrrole", "DBP-Br5Cl")
  ! C, H, O, N, Cl, Br:
  CALL Species("C_5H_6BrClN_2O_2",                   "16079-88-2", "PIEXCQIOSMOEOU-UHFFFAOYSA-N", "1-bromo-3-chloro-5,5-dimethylhydantoin", "")
  CALL Species("C_9H_{10}BrClN_2O_2",                "13360-45-7", "NLYNUTMZTCLNOO-UHFFFAOYSA-N", "3-(4-bromo-3-chlorophenyl)-1-methoxy-1-methylurea", "chlorbromuron")
  CALL Species("C_9H_8BrCl_2NO",                     "68399-95-1", "MRPNWLPYJSSISQ-UHFFFAOYSA-N", "N-(4-bromo-2,6-dichloro-3-methylphenyl)acetamide", "")
  CALL Species("C_{12}H_7NO_2BrCl",                  "34462-96-9", "YDNLKBDXQCHOTH-UHFFFAOYSA-N", "halacrinate", "")
  CALL Species("C_{13}H_{12}BrCl_2N_3O",            "116255-48-2", "HJJVPARKXDDIQD-UHFFFAOYSA-N", "bromuconazole", "")
  CALL Species("C_{13}H_{12}BrCl_2N_3O",            "114544-80-8",                            "", "($\pm$)-($2R^*$,$4S^*$)-bromuconazole", "") ! InChIKey = "HJJVPARKXDDIQD-??????????-N"
  CALL Species("C_{13}H_{12}BrCl_2N_3O",            "114544-81-9",                            "", "($\pm$)-($2R^*$,$4R^*$)-bromuconazole", "") ! InChIKey = "HJJVPARKXDDIQD-??????????-N"
  CALL Species("C_{16}H_6Br_3ClN_2O_2",              "85702-64-3", "UOWUGDPSFZLTMZ-BUHFOSPRSA-N", "5,7-dibromo-2-(5-bromo-7-chloro-1,3-dihydro-3-oxo-2H-indol-2-ylidene)-1,2-dihydro-3H-indol-3-one", "")
  CALL Species("C_{18}H_{14}BrCl_2N_5O_2",          "500008-45-7", "IZCBNTYXTMZSDP-UHFFFAOYSA-N", "chlorantraniliprole", "")
  CALL Species("C_{19}H_{14}BrClN_6O_2",            "736994-63-1", "DVBUIBGJRQBEDP-UHFFFAOYSA-N", "cyantraniliprole", "")
  ! C, H, F, Cl, Br:
  CALL Species("CBr_3F",                               "353-54-8", "IHZAEIHJPNTART-UHFFFAOYSA-N", "tribromofluoromethane", "")
  CALL Species("CBrClF_2",                             "353-59-3", "MEXUFEQDCXZEON-UHFFFAOYSA-N", "bromochlorodifluoromethane", "")
  CALL Species("C_2HBrClF_3",                          "151-67-7", "BCQZXOMGPXTTIC-UHFFFAOYSA-N", "1-bromo-1-chloro-2,2,2-trifluoroethane", "halothane")
  ! C, H, N, F, Cl, Br:
  CALL Species("C_{12}H_5BrClF_3N_2",               "122454-29-9", "XNFIRYXKTXAHAC-UHFFFAOYSA-N", "tralopyril", "")
  ! C, H, O, N, F, Cl, Br:
  CALL Species("C_{15}H_{11}BrClF_3N_2O",           "122453-73-0", "CWFOCCVIPCEQCK-UHFFFAOYSA-N", "chlorfenapyr", "")
  CALL Species("C_{15}H_{12}N_2O_2BrClF_4",         "174514-07-9", "FKLQIONHGSFYJY-UHFFFAOYSA-N", "fluazolate", "")
  !***************************************************************************
  CALL SpeciesGroup(1,"Organic species with iodine (\chem{I})")
  !***************************************************************************
  CALL SpeciesGroup(2,"Iodocarbons (\chem{C}, \chem{H}, \chem{O}, \chem{Cl}, \chem{I})")
  ! C, H, I:
  ! aliphatic:
  CALL Species("CH_3I",                                 "74-88-4", "INQOMBQAUSQDDS-UHFFFAOYSA-N", "iodomethane", "methyl iodide")
  CALL Species("CH_2I_2",                               "75-11-6", "NZZFYRREKKOMAT-UHFFFAOYSA-N", "diiodomethane", "")
  CALL Species("CHI_3",                                 "75-47-8", "OKJPEAGHQZHRQV-UHFFFAOYSA-N", "triiodomethane", "iodoform")
  CALL Species("C_2H_5I",                               "75-03-6", "HVTICUPFWKNHNG-UHFFFAOYSA-N", "iodoethane", "")
  CALL Species("C_2H_2I_2",                            "590-27-2", "CVOGMKGEVNGRSK-OWOJBTEDSA-N", "($E$)-1,2-diiodoethene", "")
  CALL Species("C_3H_7I",                              "107-08-4", "PVWOIHVRPOBWPI-UHFFFAOYSA-N", "1-iodopropane", "propyl iodide")
  CALL Species("C_3H_7I",                               "75-30-9", "FMKOJHQHASLBPH-UHFFFAOYSA-N", "2-iodopropane", "isopropyl iodide")
  CALL Species("C_4H_9I",                              "542-69-8", "KMGBZBJJOKUPIA-UHFFFAOYSA-N", "1-iodobutane", "")
  CALL Species("C_4H_9I",                              "513-48-4", "IQRUSQUYPCHEKN-UHFFFAOYSA-N", "2-iodobutane", "")
  CALL Species("C_5H_{11}I",                           "628-17-1", "BLXSFCHWMBESKV-UHFFFAOYSA-N", "1-iodopentane", "")
  CALL Species("C_6H_{13}I",                           "638-45-9", "ANOOTOPTCJRUPK-UHFFFAOYSA-N", "1-iodohexane", "")
  CALL Species("C_7H_{15}I",                          "4282-40-0", "LMHCYRULPLGEEZ-UHFFFAOYSA-N", "1-iodoheptane", "")
  CALL Species("C_6H_{11}I",                           "626-62-0", "FUCOMWZKWIEKRK-UHFFFAOYSA-N", "iodocyclohexane", "")
  CALL Species("C_3H_5I",                              "556-56-9", "HFEHLDPGIKPNKL-UHFFFAOYSA-N", "3-iodo-1-propene", "")
  CALL Species("C_6H_9I",                            "17497-53-9", "CAROGICRCNKEOD-UHFFFAOYSA-N", "1-iodocyclohexene", "")
  ! aromatic:
  CALL Species("C_6H_5I",                              "591-50-4", "SNHMUERNLJLMHN-UHFFFAOYSA-N", "iodobenzene", "")
  ! C, H, O, I:
  CALL Species("C_2H_3IO_2",                            "64-69-7", "JDNTWHVOXJZDSN-UHFFFAOYSA-N", "iodoacetic acid", "")
  CALL Species("C_6H_5IO",                             "533-58-4", "KQDJTBPASNJQFQ-UHFFFAOYSA-N", "2-iodophenol", "")
  CALL Species("C_6H_5IO",                             "626-02-8", "FXTKWBZFNQHAAO-UHFFFAOYSA-N", "3-iodophenol", "")
  CALL Species("C_6H_5IO",                             "540-38-5", "VSMDINRNYYEDRN-UHFFFAOYSA-N", "4-iodophenol", "")
  CALL Species("C_{20}H_8I_4O_5",                    "16423-68-0", "OALHHIHQOFIMEF-UHFFFAOYSA-N", "erythrosine", "") ! other CAS: 15905-32-5
  ! C, H, O, N, I:
  CALL Species("C_7H_3I_2NO",                         "1689-83-4", "NRXQIUSYPAHGNM-UHFFFAOYSA-N", "4-hydroxy-3,5-diiodo-benzonitrile", "ioxynil")
  CALL Species("C_8H_{12}INO_2",                     "55406-53-6", "WYVVKGNFXHOCQV-UHFFFAOYSA-N", "3-iodo-2-propynyl butylcarbamate", "")
  CALL Species("C_{11}H_9I_3N_2O_4",                   "117-96-4", "YVPYQUNUQOZFHG-UHFFFAOYSA-N", "diatrizoic acid", "")
  CALL Species("C_{11}H_9I_3N_2O_4",                  "2276-90-6", "UXIGWFXRQKWHHA-UHFFFAOYSA-N", "iothalamic acid", "")
  CALL Species("C_{13}H_{10}INO",                    "15310-01-7", "LJOZMWRYMKECFF-UHFFFAOYSA-N", "benodanil", "")
  CALL Species("C_{14}H_{17}IN_2O_2",               "189278-12-4", "FLVBXVXXXMLMOX-UHFFFAOYSA-N", "proquinazid", "")
  CALL Species("C_{17}H_{22}I_3N_3O_8",              "60166-93-0", "XQZXYNRDCRIARQ-LURJTMIESA-N", "iopamidol", "")
  CALL Species("C_{24}H_{21}I_6N_5O_8",              "59017-64-0", "TYYBFXNZMFNZJT-UHFFFAOYSA-N", "ioxaglic acid", "")
  ! C, F, I:
  CALL Species("C_6F_{13}I",                           "355-43-1", "BULLJMKUVKYZDJ-UHFFFAOYSA-N", "perfluorohexyl iodide", "")
  ! C, H, F, I:
  CALL Species("C_5H_4F_7I",                         "68188-12-5", "TZNRRNKRZXHADL-UHFFFAOYSA-N", "1,1,1,2,2,3,3-heptafluoro-5-iodopentane", "")
  CALL Species("C_6H_4FI",                             "352-34-1", "KGNQDBQYEBMPFZ-UHFFFAOYSA-N", "1-fluoro-4-iodobenzene", "")
  CALL Species("C_6H_4F_9I",                          "2043-55-2", "CXHFIVFPHDGZIS-UHFFFAOYSA-N", "5-diethylamiloride", "")
  CALL Species("C_8H_4F_{13}I",                       "2043-57-4", "NVVZEKTVIXIUKW-UHFFFAOYSA-N", "1,1,1,2,2,3,3,4,4,5,5,6,6-tridecafluoro-8-iodooctane", "")
  CALL Species("C_8H_2F_{13}I",                     "150223-14-6", "WDWNMIBWWPZNJK-OWOJBTEDSA-N", "1H,2H-perfluoro-1-iodooct-1-ene", "")
  CALL Species("C_{10}H_4F_{17}I",                    "2043-53-0", "XVKJSLBVVRCOIT-UHFFFAOYSA-N", "1,1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8-heptadecafluoro-10-iododecane", "")
  ! C, H, Cl, I:
  CALL Species("CH_2ClI",                              "593-71-5", "PJGJQVRXEUVAFT-UHFFFAOYSA-N", "chloroiodomethane", "")
  CALL Species("C_6H_4ClI",                            "637-87-6", "GWQSENYKCGJTRI-UHFFFAOYSA-N", "1-chloro-4-iodobenzene", "")
  ! C, H, Br, I:
  CALL Species("CH_2BrI",                              "557-68-6", "TUDWMIUPYRKEFN-UHFFFAOYSA-N", "bromoiodomethane", "")
  CALL Species("C_6H_4BrI",                            "589-87-7", "UCCUXODGPMAHRL-UHFFFAOYSA-N", "1-bromo-4-iodobenzene", "")
  !***************************************************************************
  CALL SpeciesGroup(1,"Organic species with sulfur (\chem{S})")
  !***************************************************************************
  CALL SpeciesGroup(2,"Sulfur (\chem{C}, \chem{H}, \chem{O}, \chem{N}, \chem{Cl}, \chem{S})")
  ! C, H, S:
  ! saturated:
  ! with -SH:
  CALL Species("CH_3SH",                                "74-93-1", "LSDPWZHWYPCBBB-UHFFFAOYSA-N", "methanethiol", "methyl mercaptan")
  CALL Species("C_2H_5SH",                              "75-08-1", "DNJIEGIFACGWOD-UHFFFAOYSA-N", "ethanethiol", "ethyl mercaptan")
  CALL Species("C_2H_6S_2",                            "540-63-6", "VYMPLPIFKRHAAC-UHFFFAOYSA-N", "1,2-ethanedithiol", "")
  CALL Species("C_2H_4S",                              "420-12-2", "VOVUARRWDCVURC-UHFFFAOYSA-N", "thiirane", "ethylene sulfide")
  CALL Species("C_3H_7SH",                             "107-03-9", "SUVIGLJNEAMWEG-UHFFFAOYSA-N", "1-propanethiol", "propyl mercaptan")
  CALL Species("C_3H_8S",                               "75-33-2", "KJRCEJOSASVSRA-UHFFFAOYSA-N", "2-propanethiol", "isopropyl mercaptan")
  CALL Species("C_4H_9SH",                             "109-79-5", "WQAQPCDUOCURKW-UHFFFAOYSA-N", "1-butanethiol", "butyl mercaptan")
  CALL Species("C_4H_{10}S",                           "513-53-1", "LOCHFZBWPCLPAN-UHFFFAOYSA-N", "2-butanethiol", "{\it sec}-butyl mercaptan")
  CALL Species("C_4H_{10}S",                           "513-44-0", "BDFAOUQQXJIZDG-UHFFFAOYSA-N", "2-methyl-1-propanethiol", "isobutyl mercaptan")
  CALL Species("C_4H_{10}S",                            "75-66-1", "WMXCDAVJEZZYLT-UHFFFAOYSA-N", "2-methyl-2-propanethiol", "{\it tert}-butyl mercaptan")
  CALL Species("C_4H_8S_2",                            "505-29-3", "LOZWAPSEEHRYPG-UHFFFAOYSA-N", "1,4-dithiane", "")
  CALL Species("C_5H_{11}SH",                          "110-66-7", "ZRKMQKLGEQPLNS-UHFFFAOYSA-N", "1-pentanethiol", "pentyl mercaptan")
  CALL Species("C_5H_{12}S",                          "1679-08-9", "LSUXMVNABVPWMF-UHFFFAOYSA-N", "2,2-dimethyl-1-propanethiol", "")
  CALL Species("C_5H_{12}S",                          "1878-18-8", "WGQKBCSACFQGQY-UHFFFAOYSA-N", "2-methyl-1-butanethiol", "")
  CALL Species("C_5H_{12}S",                          "1679-09-0", "IQIBYAHJXQVQGB-UHFFFAOYSA-N", "2-methyl-2-butanethiol", "")
  CALL Species("C_5H_{12}S",                          "2084-19-7", "QUSTYFNPKBDELJ-UHFFFAOYSA-N", "2-pentanethiol", "")
  CALL Species("C_5H_{12}S",                           "616-31-9", "WICKAMSPKJXSGN-UHFFFAOYSA-N", "3-pentanethiol", "")
  CALL Species("C_5H_{12}S",                           "541-31-1", "GIJGXNFNUUFEGH-UHFFFAOYSA-N", "3-methyl-1-butanethiol", "")
  CALL Species("C_5H_{12}S",                          "2084-18-6", "BFLXFRNPNMTTAA-UHFFFAOYSA-N", "3-methyl-2-butanethiol", "")
  CALL Species("C_6H_{14}S",                           "111-31-9", "PMBXCGGQNSVESQ-UHFFFAOYSA-N", "1-hexanethiol", "hexyl mercaptan")
  CALL Species("C_6H_{14}S",                          "1679-06-7", "ABNPJVOPTXYSQW-UHFFFAOYSA-N", "2-hexanethiol", "")
  CALL Species("C_7H_{16}S",                          "1639-09-4", "VPIAKHNXCOTPAY-UHFFFAOYSA-N", "1-heptanethiol", "heptyl mercaptan")
  CALL Species("C_7H_{16}S",                           "628-00-2", "DAZNOIJJVKASGS-UHFFFAOYSA-N", "2-heptanethiol", "")
  CALL Species("C_8H_{18}S",                           "111-88-6", "KZCOBXFFBQJQHH-UHFFFAOYSA-N", "1-octanethiol", "octyl mercaptan")
  CALL Species("C_8H_{18}S",                           "141-59-3", "QZLAEIZEPJAELS-UHFFFAOYSA-N", "{\it tert}-octanethiol", "{\it tert}-octyl mercaptan")
  CALL Species("C_8H_{18}S",                          "3001-66-9", "BZXFEMZFRLXGCY-UHFFFAOYSA-N", "2-octanethiol", "")
  CALL Species("C_9H_{20}S",                          "1455-21-6", "ZVEZMVFBMOOHAT-UHFFFAOYSA-N", "1-nonanethiol", "nonyl mercaptan")
  CALL Species("C_9H_{20}S",                         "13281-11-3", "UOMSUBPWUZCGQU-UHFFFAOYSA-N", "2-nonanethiol", "")
  CALL Species("C_{10}H_{22}S",                        "143-10-2", "VTXVGVNLYGSIAR-UHFFFAOYSA-N", "1-decanethiol", "decyl mercaptan")
  CALL Species("C_{10}H_{22}S",                      "13402-60-3", "NWKXKAHGQAWFQP-UHFFFAOYSA-N", "2-decanethiol", "")
  CALL Species("C_{11}H_{24}S",                       "5332-52-5", "CCIDWXHLGNEQSL-UHFFFAOYSA-N", "1-undecanethiol", "undecyl mercaptan")
  CALL Species("C_{11}H_{24}S",                      "62155-02-6", "KRMLVHZORKTOLI-UHFFFAOYSA-N", "2-undecanethiol", "")
  CALL Species("C_{12}H_{26}S",                        "112-55-0", "WNAHIZMDSQCWRP-UHFFFAOYSA-N", "1-dodecanethiol", "dodecyl mercaptan")
  CALL Species("C_{12}H_{26}S",                      "14402-50-7", "UROXMPKAGAWKPP-UHFFFAOYSA-N", "2-dodecanethiol", "")
  CALL Species("C_{12}H_{26}S",                                "", "BGQHJKADBZPYGJ-UHFFFAOYSA-N", "5-propyl-5-nonanethiol", "")
  CALL Species("C_{13}H_{28}S",                      "19484-26-5", "IPBROXKVGHZHJV-UHFFFAOYSA-N", "1-tridecanethiol", "")
  CALL Species("C_{13}H_{28}S",                      "62155-03-7", "ZAPRCDVCNIREGW-UHFFFAOYSA-N", "2-tridecanethiol", "")
  CALL Species("C_{14}H_{30}S",                       "2079-95-0", "GEKDEMKPCKTKEC-UHFFFAOYSA-N", "1-tetradecanethiol", "")
  CALL Species("C_{14}H_{30}S",                      "62155-04-8", "YESAYZVWGLMPDY-UHFFFAOYSA-N", "2-tetradecanethiol", "")
  ! without -SH:
  CALL Species("CH_3SCH_3",                             "75-18-3", "QMMFVYPAHWMCMS-UHFFFAOYSA-N", "dimethyl sulfide", "DMS")
  CALL Species("C_2H_5SCH_3",                          "624-89-5", "WXEHBUMAEPOYKP-UHFFFAOYSA-N", "ethyl methyl sulfide", "")
  CALL Species("C_2H_5SC_2H_5",                        "352-93-2", "LJSQFQKUNVCTIA-UHFFFAOYSA-N", "diethyl sulfide", "")
  CALL Species("C_4H_{10}S",                          "1551-21-9", "ROSSIHMZZJOVOU-UHFFFAOYSA-N", "methyl isopropyl sulfide", "")
  CALL Species("C_4H_{10}S",                          "3877-15-4", "ZOASGOXWEHUTKZ-UHFFFAOYSA-N", "methyl propyl sulfide", "")
  CALL Species("C_4H_8S",                              "110-01-0", "RAOIDOHSFRTOEL-UHFFFAOYSA-N", "tetrahydrothiophene", "")
  CALL Species("C_5H_{12}S",                          "5145-99-3", "NZUQQADVSXWVNW-UHFFFAOYSA-N", "2-methyl-3-thiapentane", "")
  CALL Species("C_5H_{12}S",                          "6163-64-0", "CJFVCTVYZFTORU-UHFFFAOYSA-N", "3,3-dimethyl-2-thiabutane", "")
  CALL Species("C_5H_{12}S",                         "10359-64-5", "IJRCRFQMYAJPPO-UHFFFAOYSA-N", "3-methyl-2-thiapentane", "")
  CALL Species("C_5H_{12}S",                          "5008-69-5", "UYVGFIKOUAFDOZ-UHFFFAOYSA-N", "4-methyl-2-thiapentane", "")
  CALL Species("C_5H_{12}S",                          "4110-50-3", "ZDDDFDQTSXYYSE-UHFFFAOYSA-N", "ethyl propyl sulfide", "")
  CALL Species("C_5H_{12}S",                           "628-29-5", "WCXXISMIJBRDQK-UHFFFAOYSA-N", "methyl butyl sulfide", "")
  CALL Species("C_3H_7SC_3H_7",                        "111-47-7", "ZERULLAPCVRMCO-UHFFFAOYSA-N", "dipropyl sulfide", "")
  CALL Species("(C_3H_7)_2S",                          "625-80-9", "XYWDPYKBIRQXQS-UHFFFAOYSA-N", "di-(2-propyl)-sulfide", "diisopropyl sulfide")
  CALL Species("C_6H_{14}S",                         "14290-92-7", "GZJUDUMQICJSFJ-UHFFFAOYSA-N", "2,2-dimethyl-3-thiapentane", "")
  CALL Species("C_6H_{14}S",                          "5008-73-1", "BDFDQOJJDDORSR-UHFFFAOYSA-N", "2-methyl-3-thiahexane", "")
  CALL Species("C_6H_{14}S",                         "13286-92-5", "SJOHDSPLOVBXOF-UHFFFAOYSA-N", "3,3-dimethyl-2-thiapentane", "")
  CALL Species("C_6H_{14}S",                         "53897-51-1", "AUNQXXJGFDKEMS-UHFFFAOYSA-N", "3,4-dimethyl-2-thiapentane", "")
  CALL Species("C_6H_{14}S",                         "57093-84-2", "ZQOHJVSSVAWXQZ-UHFFFAOYSA-N", "3-ethyl-2-thiapentane", "")
  CALL Species("C_6H_{14}S",                         "13286-91-4", "WGBHWWSSUGCSCP-UHFFFAOYSA-N", "3-methyl-2-thiahexane", "")
  CALL Species("C_6H_{14}S",                          "6079-57-8", "YUFAJXXIXSYTMS-UHFFFAOYSA-N", "4,4-dimethyl-2-thiapentane", "")
  CALL Species("C_6H_{14}S",                         "15013-37-3", "XMOUDDXUCXXDTJ-UHFFFAOYSA-N", "4-methyl-2-thiahexane", "")
  CALL Species("C_6H_{14}S",                          "5008-72-0", "JFNGZXUPUVUYST-UHFFFAOYSA-N", "4-methyl-3-thiahexane", "")
  CALL Species("C_6H_{14}S",                         "13286-90-3", "ABIKQXWLJOURPN-UHFFFAOYSA-N", "5-methyl-2-thiahexane", "")
  CALL Species("C_6H_{14}S",                          "1613-45-2", "OIRKGXWQBSPXLQ-UHFFFAOYSA-N", "5-methyl-3-thiahexane", "")
  CALL Species("C_6H_{14}S",                           "638-46-0", "XJIRSLHMKBUGMR-UHFFFAOYSA-N", "ethyl butyl sulfide", "")
  CALL Species("C_6H_{14}S",                          "1741-83-9", "FOJGPFUFFHWGFQ-UHFFFAOYSA-N", "methyl pentyl sulfide", "")
  CALL Species("C_7H_{16}S",                         "26158-99-6", "SOGIWVXLDPPMMF-UHFFFAOYSA-N", "ethyl pentyl sulfide", "")
  CALL Species("C_7H_{16}S",                         "20291-60-5", "LZRXQHHKXDXOIC-UHFFFAOYSA-N", "methyl hexyl sulfide", "")
  CALL Species("C_7H_{16}S",                          "1613-46-3", "ZBRWJPVULTZZCE-UHFFFAOYSA-N", "propyl butyl sulfide", "")
  CALL Species("C_8H_{18}S",                           "544-40-1", "HTIRHQRTDBPHNZ-UHFFFAOYSA-N", "dibutyl sulfide", "")
  CALL Species("C_8H_{18}S",                          "7309-44-6", "MGVUJBCOCITTRS-UHFFFAOYSA-N", "ethyl hexyl sulfide", "")
  CALL Species("C_8H_{18}S",                         "20291-61-6", "FJDWJOQOEZRIDJ-UHFFFAOYSA-N", "methyl heptyl sulfide", "")
  CALL Species("C_8H_{18}S",                         "42841-80-5", "MJRCCWJSYFOGBX-UHFFFAOYSA-N", "propyl pentyl sulfide", "")
  CALL Species("C_9H_{20}S",                         "24768-42-1", "RNEUXBDXTNIASG-UHFFFAOYSA-N", "butyl pentyl sulfide", "")
  CALL Species("C_9H_{20}S",                         "24768-44-3", "PYPULUCCVXMPFP-UHFFFAOYSA-N", "ethyl heptyl sulfide", "")
  CALL Species("C_9H_{20}S",                          "3698-95-1", "AHCJTMBRROLNHV-UHFFFAOYSA-N", "methyl octyl sulfide", "")
  CALL Species("C_9H_{20}S",                         "24768-43-2", "ABZLKKGJOVPBBL-UHFFFAOYSA-N", "propyl hexyl sulfide", "")
  CALL Species("C_{10}H_{22}S",                      "16967-04-7", "YZUHMAFUXBPUKH-UHFFFAOYSA-N", "butyl hexyl sulfide", "")
  CALL Species("C_{10}H_{22}S",                        "872-10-6", "JOZDADPMWLVEJK-UHFFFAOYSA-N", "dipentyl sulfide", "")
  CALL Species("C_{10}H_{22}S",                       "3698-94-0", "WAITXWGCJQLPGH-UHFFFAOYSA-N", "ethyl octyl sulfide", "")
  CALL Species("C_{10}H_{22}S",                      "59973-07-8", "FCRSULZJMFDBIK-UHFFFAOYSA-N", "methyl nonyl sulfide", "")
  CALL Species("C_{10}H_{22}S",                      "24768-46-5", "PCPVCHFKVPNTBH-UHFFFAOYSA-N", "propyl heptyl sulfide", "")
  CALL Species("C_{11}H_{24}S",                      "40813-84-1", "HYUPOCGXBZUYFY-UHFFFAOYSA-N", "butyl heptyl sulfide", "")
  CALL Species("C_{11}H_{24}S",                      "59973-08-9", "LUAABLIRQSWMGZ-UHFFFAOYSA-N", "ethyl nonyl sulfide", "")
  CALL Species("C_{11}H_{24}S",                      "22438-39-7", "HKGUUZAACYBIID-UHFFFAOYSA-N", "methyl decyl sulfide", "")
  CALL Species("C_{11}H_{24}S",                       "3698-93-9", "GPJXDRJGQAKGLH-UHFFFAOYSA-N", "propyl octyl sulfide", "")
  CALL Species("C_{12}H_{26}S",                      "16900-07-5", "UNIAPWPIAGJFDG-UHFFFAOYSA-N", "butyl octyl sulfide", "")
  CALL Species("C_{12}H_{26}S",                       "6294-31-1", "LHNRHYOMDUJLLM-UHFFFAOYSA-N", "dihexyl sulfide", "")
  CALL Species("C_{12}H_{26}S",                      "19313-61-2", "VSSRSPLEFYQIEK-UHFFFAOYSA-N", "ethyl decyl sulfide", "")
  CALL Species("C_{12}H_{26}S",                       "7289-44-3", "HDOADYQJIBYVGE-UHFFFAOYSA-N", "methyl undecyl sulfide", "")
  CALL Species("C_{12}H_{26}S",                      "62103-66-6", "AQAOPNMAEGBYHI-UHFFFAOYSA-N", "propyl nonyl sulfide", "")
  CALL Species("C_{13}H_{28}S",                      "66577-32-0", "FWRIVMHSSSZAFD-UHFFFAOYSA-N", "butyl nonyl sulfide", "")
  CALL Species("C_{13}H_{28}S",                      "66577-30-8", "OSWITQLVZPPUIR-UHFFFAOYSA-N", "ethyl undecyl sulfide", "")
  CALL Species("C_{13}H_{28}S",                       "3698-89-3", "KJWHJDGMOQJLGF-UHFFFAOYSA-N", "methyl dodecyl sulfide", "")
  CALL Species("C_{13}H_{28}S",                      "66577-31-9", "HPJCKXDELORROO-UHFFFAOYSA-N", "propyl decyl sulfide", "")
  CALL Species("C_{14}H_{30}S",                      "19313-57-6", "CLBLVLKZMJDLOT-UHFFFAOYSA-N", "butyl decyl sulfide", "")
  CALL Species("C_{14}H_{30}S",                        "629-65-2", "LEMIDOZYVQXGLI-UHFFFAOYSA-N", "diheptyl sulfide", "")
  CALL Species("C_{14}H_{30}S",                       "2851-83-4", "QECBTJWQRXCSCU-UHFFFAOYSA-N", "ethyl dodecyl sulfide", "")
  CALL Species("C_{14}H_{30}S",                      "62155-09-3", "QVXOOYOFFITGCV-UHFFFAOYSA-N", "methyl tridecyl sulfide", "")
  CALL Species("C_{14}H_{30}S",                      "66826-84-4", "ITNOZFAANYXVSG-UHFFFAOYSA-N", "propyl undecyl sulfide", "")
  ! more than 1 S:
  CALL Species("CH_3SSCH_3",                           "624-92-0", "WQOXQRCZOLPYPM-UHFFFAOYSA-N", "dimethyl disulfide", "")
  CALL Species("C_2H_5SSC_2H_5",                       "110-81-6", "CETBSQOFQKLHHZ-UHFFFAOYSA-N", "diethyl disulfide", "")
  CALL Species("C_3H_7SSC_3H_7",                       "629-19-6", "ALVPFGSHPUPROW-UHFFFAOYSA-N", "dipropyl disulfide", "")
  CALL Species("CS_2",                                  "75-15-0", "QGJOPFRUJISHPQ-UHFFFAOYSA-N", "carbon disulfide", "")
  CALL Species("C_2H_6S_3",                           "3658-80-8", "YWHLKYXPLRWGSE-UHFFFAOYSA-N", "2,3,4-trithiapentane", "dimethyl trisulfide")
  CALL Species("C_{12}H_{22}S_2",                     "2550-40-5", "ODHAQPXNQDBHSH-UHFFFAOYSA-N", "dicyclohexyldisulfide", "")
  ! unsaturated:
  CALL Species("C_3H_6S",                              "870-23-5", "ULIKDJVNUXNQHS-UHFFFAOYSA-N", "allyl mercaptan", "")
  CALL Species("(C_3H_5)_2S",                          "592-88-1", "UBJVUCKUDDKUJF-UHFFFAOYSA-N", "3,3'-thiobis-1-propene", "diallyl sulfide")
  CALL Species("CH_2CHCH_2SCH_3",                    "10152-76-8", "NVLPQIPTCCLBEU-UHFFFAOYSA-N", "allyl methyl sulfide", "2-propenyl methyl sulfide")
  CALL Species("C_4H_4S",                              "110-02-1", "YTPLMLYBLZKORZ-UHFFFAOYSA-N", "thiophene", "")
  CALL Species("CH_3C_4H_3S",                          "554-14-3", "XQQBUAPQHNYYRS-UHFFFAOYSA-N", "2-methylthiophene", "")
  CALL Species("CH_3C_4H_3S",                          "616-44-4", "QENGPZGAWFQWCZ-UHFFFAOYSA-N", "3-methylthiophene", "")
  CALL Species("C_6H_{12}S_2",                        "2179-59-1", "FCSSPCOFDUKHPV-UHFFFAOYSA-N", "propyl allyl disulfide", "")
  CALL Species("C_6H_{10}S_2",                        "2179-57-9", "PFRGXCVKLLPLIP-UHFFFAOYSA-N", "4,5-dithia-1,7-octadiene", "diallyl disulfide")
  CALL Species("C_6H_8S",                              "632-16-6", "BZYUMXXOAYSFOW-UHFFFAOYSA-N", "2,3-dimethylthiophene", "")
  CALL Species("C_6H_8S",                              "638-00-6", "CPULIKNSOUFMPL-UHFFFAOYSA-N", "2,4-dimethylthiophene", "")
  CALL Species("C_6H_8S",                              "638-02-8", "GWQOOADXMVQEFT-UHFFFAOYSA-N", "2,5-dimethylthiophene", "")
  CALL Species("C_6H_8S",                              "872-55-9", "JCCCMAAJYSNBPR-UHFFFAOYSA-N", "2-ethylthiophene", "")
  CALL Species("C_6H_8S",                              "632-15-5", "GPSFYJDZKSRMKZ-UHFFFAOYSA-N", "3,4-dimethylthiophene", "")
  CALL Species("C_6H_8S",                             "1795-01-3", "SLDBAXYJAIRQMX-UHFFFAOYSA-N", "3-ethylthiophene", "")
  CALL Species("C_7H_{10}S",                          "1795-04-6", "MAVVDCDMBKFUES-UHFFFAOYSA-N", "2,3,4-trimethylthiophene", "")
  CALL Species("C_7H_{10}S",                          "1795-05-7", "QKZJQIHBRCFDGQ-UHFFFAOYSA-N", "2,3,5-trimethylthiophene", "")
  CALL Species("C_7H_{10}S",                          "4095-22-1", "LOXBELRNKUFSRD-UHFFFAOYSA-N", "2-isopropylthiophene", "")
  CALL Species("C_7H_{10}S",                         "53119-51-0", "RBRAJDCWXUJHIY-UHFFFAOYSA-N", "2-methyl-3-ethylthiophene", "")
  CALL Species("C_7H_{10}S",                         "13678-54-1", "KIWVMUQUYOKTKU-UHFFFAOYSA-N", "2-methyl-4-ethylthiophene", "")
  CALL Species("C_7H_{10}S",                         "40323-88-4", "VOIVNYVBGCJFRW-UHFFFAOYSA-N", "2-methyl-5-ethylthiophene", "")
  CALL Species("C_7H_{10}S",                          "1551-27-5", "BTXIJTYYMLCUHI-UHFFFAOYSA-N", "2-propylthiophene", "")
  CALL Species("C_7H_{10}S",                         "29488-27-5", "LJPDBPCGTFTUDE-UHFFFAOYSA-N", "3-isopropylthiophene", "")
  CALL Species("C_7H_{10}S",                         "31805-48-8", "VVSCPYCHXFEKLF-UHFFFAOYSA-N", "3-methyl-2-ethylthiophene", "")
  CALL Species("C_7H_{10}S",                         "66577-03-5", "QMHNYDJBWOPGHE-UHFFFAOYSA-N", "3-methyl-4-ethylthiophene", "")
  CALL Species("C_7H_{10}S",                         "66577-04-6", "NZOYEHPXDWJOCD-UHFFFAOYSA-N", "3-methyl-5-ethylthiophene", "2-ethyl-4-methylthiophene")
  CALL Species("C_7H_{10}S",                          "1518-75-8", "QZNFRMXKQCIPQY-UHFFFAOYSA-N", "3-propylthiophene", "")
  ! aromatic:
  CALL Species("C_6H_5SH",                             "108-98-5", "RMVRSNDYEFQCLF-UHFFFAOYSA-N", "benzenethiol", "thiophenol")
  CALL Species("C_6H_5SCH_3",                          "100-68-5", "HNKJADCVZUBCPG-UHFFFAOYSA-N", "methyl phenyl sulfide", "thioanisole")
  CALL Species("C_7H_8S",                              "137-06-4", "LXUNZSDDXMPKLP-UHFFFAOYSA-N", "2-methylbenzenethiol", "2-thiocresol")
  CALL Species("C_7H_8S",                              "108-40-7", "WRXOZRLZDJAYDR-UHFFFAOYSA-N", "3-methylbenzenethiol", "3-thiocresol")
  CALL Species("C_7H_8S",                              "106-45-6", "WLHCBQAPPJAULW-UHFFFAOYSA-N", "4-methylbenzenethiol", "4-thiocresol")
  CALL Species("C_7H_8S",                              "100-53-8", "UENWRTRMUIOCKN-UHFFFAOYSA-N", "benzenemethanethiol", "")
  CALL Species("C_8H_6S",                               "95-15-8", "FCEHBMOGCRZNNI-UHFFFAOYSA-N", "benzo[$b$]thiophene", "")
  CALL Species("C_{12}H_8S",                           "132-65-0", "IYYZUPMFVPLQIF-UHFFFAOYSA-N", "dibenzothiophene", "")
  CALL Species("C_{12}H_8S_2",                          "92-85-3", "GVIJJXMXTUZIOD-UHFFFAOYSA-N", "thianthrene", "")
  CALL Species("C_{13}H_{10}S",                      "20928-02-3", "VHUXLBLPAMBOJS-UHFFFAOYSA-N", "2-methyldibenzothiophene", "")
  CALL Species("C_{14}H_{12}S",                      "31317-17-6", "KEIKATUAMBEIQN-UHFFFAOYSA-N", "2,3-dimethyldibenzothiophene", "")
  CALL Species("C_{14}H_{12}S",                       "1207-15-4", "RRYWCJRYULRSJM-UHFFFAOYSA-N", "2,8-dimethyldibenzothiophene", "")
  CALL Species("C_{14}H_{14}S",                        "538-74-9", "LUFPJJNWMYZRQE-UHFFFAOYSA-N", "benzyl sulfide", "")
  CALL Species("C_{15}H_{14}S",                                "", "HJSMFSZKADBZIO-UHFFFAOYSA-N", "2,3,7-trimethyldibenzothiophene", "")
  CALL Species("C_{15}H_{14}S",                                "", "QSOGKFSFLARZIE-UHFFFAOYSA-N", "2,3,8-trimethyldibenzothiophene", "")
  CALL Species("C_{15}H_{14}S",                     "216983-03-8", "YDWNRDFOOADVGC-UHFFFAOYSA-N", "2,4,7-trimethyldibenzothiophene", "")
  CALL Species("C_{16}H_{16}S",                     "132034-91-4", "UMQGGSYHJPHWFV-UHFFFAOYSA-N", "4,6-diethyldibenzothiophene", "")
  CALL Species("C_{16}H_{16}S",                     "147792-31-2", "BEQMTJNPMZQPKZ-UHFFFAOYSA-N", "2-butyldibenzothiophene", "")
  ! C, H, O, S:
  CALL Species("OCS",                                  "463-58-1", "JJWKPURADFRFRB-UHFFFAOYSA-N", "carbon oxide sulfide", "carbonyl sulfide")
  CALL Species("CH_3SO_3H",                             "75-75-2", "AFVFQIVMOAPDHO-UHFFFAOYSA-N", "methanesulfonic acid", "MSA")
  CALL Species("CH_4O_4S",                                     "", "BENVNICSYZXTGE-UHFFFAOYSA-N", "MCM:CH3SO2OOH", "")
  CALL Species("CH_4O_3S",                                     "", "IZTYBZWDPIUSAG-UHFFFAOYSA-N", "MCM:CH3SOOOH", "")
  CALL Species("CH_4O_2S",                                     "", "XNEFVTBPCXGIRX-UHFFFAOYSA-N", "MCM:MSIA", "")
  CALL Species("C_2H_6O_4S",                            "77-78-1", "VAYGXNSJCAHWJZ-UHFFFAOYSA-N", "sulfuric acid, dimethyl ester", "dimethyl sulfate")
  CALL Species("CH_3SOCH_3",                            "67-68-5", "IAZDPXIOMUYVGZ-UHFFFAOYSA-N", "dimethylsulfoxide", "DMSO")
  CALL Species("CH_3SO_2CH_3",                          "67-71-0", "HHVIBTZHLRERCL-UHFFFAOYSA-N", "dimethylsulfone", "DMSO2")
  CALL Species("C_2H_4O_2S",                            "68-11-1", "CWERGRDVMFNCDR-UHFFFAOYSA-N", "mercaptoacetic acid", "")
  CALL Species("C_2H_6OS",                              "60-24-2", "DGVVWUTYPXICAM-UHFFFAOYSA-N", "2-mercaptoethanol", "")
  CALL Species("C_2H_6O_3S",                            "66-27-3", "MBABOKRGFJTBAE-UHFFFAOYSA-N", "methanesulfonic acid, methyl ester", "")
  CALL Species("C_2H_6O_2S",                                   "", "LDZPECMXMDESRV-UHFFFAOYSA-N", "MCM:CH3SCH2OOH", "")
  CALL Species("C_2H_6O_4S",                                   "", "GCLQZLAJAMWHPS-UHFFFAOYSA-N", "MCM:DMSO2OOH", "")
  CALL Species("C_2H_6OS",                                     "", "ZSSFPSNLAUYOFG-UHFFFAOYSA-N", "MCM:CH3SCH2OH", "")
  CALL Species("C_2H_6O_3S",                                   "", "ICHBUPLXTAHKLA-UHFFFAOYSA-N", "MCM:DMSO2OH", "")
  CALL Species("C_2H_4OS",                                     "", "LFJRGYNYRORDDM-UHFFFAOYSA-N", "MCM:CH3SCHO", "")
  CALL Species("C_2H_4O_3S",                                   "", "LUGQISNSRPQOGS-UHFFFAOYSA-N", "MCM:CH3SO2CHO", "")
  CALL Species("C_3H_6O_2S",                          "2365-48-2", "MKIJJIMOAABWGF-UHFFFAOYSA-N", "mercaptoacetic acid, methyl ester", "methyl thioglycolate")
  CALL Species("C_3H_6O_3S",                          "1120-71-4", "FSSPGSAQUIYDCN-UHFFFAOYSA-N", "1,3-propane sultone", "")
  CALL Species("C_3H_8O_3S",                            "62-50-0", "PLUBXMRUUVWRLT-UHFFFAOYSA-N", "methanesulfonic acid, ethyl ester", "")
  CALL Species("C_3H_8OS_2",                            "59-52-9", "WQABCVAJNWAXTE-UHFFFAOYSA-N", "2,3-dimercapto-1-propanol", "")
  CALL Species("C_4H_6OS",                            "1115-15-7", "HQSMEHLVLOGBCK-UHFFFAOYSA-N", "divinyl sulfoxide", "vinyl sulfoxide")
  CALL Species("C_4H_6O_2S",                            "77-77-0", "AFOSIXZFDONLBT-UHFFFAOYSA-N", "divinyl sulphone", "")
  CALL Species("C_4H_6O_2S",                            "77-79-2", "MBDNRNMVTZADMQ-UHFFFAOYSA-N", "2,5-dihydrothiophene sulfone", "2,5-dihydrothiophene 1,1-dioxide")
  CALL Species("C_4H_6O_4S",                           "123-93-3", "UVZICZIVKIMRNE-UHFFFAOYSA-N", "thiodiacetic acid", "")
  CALL Species("C_4H_8O_2S",                           "126-33-0", "HXJUTPCZVOIRIF-UHFFFAOYSA-N", "thiophene, tetrahydro-, 1,1-dioxide", "sulfolane")
  CALL Species("C_4H_{10}OS",                          "110-77-0", "LNRIEBFNWGMXKP-UHFFFAOYSA-N", "2-(ethylthio)ethanol", "")
  CALL Species("C_4H_{10}O_2S",                        "111-48-8", "YODZTKMDCQEPHD-UHFFFAOYSA-N", "thiodiglycol", "")
  CALL Species("C_4H_{10}O_3S",                        "926-06-7", "SWWHCQCMVCPLEQ-UHFFFAOYSA-N", "methanesulfonic acid, 1-methylethyl ester", "")
  CALL Species("C_4H_{10}O_4S",                         "64-67-5", "DENRZWYUOJLTMF-UHFFFAOYSA-N", "diethyl sulfate", "")
  CALL Species("C_5H_{10}OS",                         "2432-51-1", "GRLJIIJNZJVMGP-UHFFFAOYSA-N", "S-methyl butanethioate", "")
  CALL Species("C_6H_6O_4S",                            "98-67-9", "FEPBITJSIHRMRT-UHFFFAOYSA-N", "4-hydroxybenzenesulfonic acid", "")
  CALL Species("C_6H_6O_3S",                            "98-11-3", "SRSXLGNVWSONIS-UHFFFAOYSA-N", "benzenesulfonic acid", "")
  CALL Species("C_6H_{10}O_4S_2",                    "55290-64-7", "PHVNLLCAQHGNKU-UHFFFAOYSA-N", "dimethipin", "")
  CALL Species("C_7H_8O_3S",                           "104-15-4", "JOXIMZWYDAKGHI-UHFFFAOYSA-N", "4-methylbenzenesulfonic acid", "")
  CALL Species("C_7H_8O_3S",                           "100-87-8", "NIXKBAZVOQAHGC-UHFFFAOYSA-N", "phenylmethanesulfonic acid", "benzylsulfonic acid")
  CALL Species("C_{12}H_{18}O_4S_2",                 "50512-35-1", "UFHLMYOGRXOCSL-UHFFFAOYSA-N", "isoprothiolane", "")
  CALL Species("C_{12}H_{10}O_4S",                      "80-09-1", "VPWNQTHUCYMVMZ-UHFFFAOYSA-N", "4,4'-sulfonyldiphenol", "bisphenol S")
  CALL Species("C_{12}H_{16}O_4S",                   "68505-69-1", "QGQSRQPXXMTJCM-UHFFFAOYSA-N", "benfuresate", "")
  CALL Species("C_{12}H_{26}O_4S",                     "151-41-7", "MOTZDAYCYVMXPC-UHFFFAOYSA-N", "lauryl sulfate", "dodecyl sulfate")
  CALL Species("C_{13}H_{18}O_5S",                   "26225-79-6", "IRCMYGHHKLLGHV-UHFFFAOYSA-N", "ethofumesate", "")
  CALL Species("C_{18}H_{22}O_2S",                   "57913-35-6", "PKQWGBLKCJIMDP-UHFFFAOYSA-N", "1,1'-sulfonylbis(4-(1-methylethyl)-benzene", "")
  CALL Species("C_{19}H_{26}O_4S",                    "2312-35-8", "ZYHMJXZULPZUED-UHFFFAOYSA-N", "propargite", "")
  CALL Species("C_{23}H_{24}O_4S",                   "58769-20-3", "UGWALRUNBSBTGI-QJLCOAGJSA-N", "kadethrin", "")
  CALL Species("C_{24}H_{32}O_4S",                      "52-01-7", "LXMSZDCAJNLERA-PJKOONHHSA-N", "spironolactone", "")
  CALL Species("C_{28}H_{42}O_2S",                    "3294-03-9", "WQYFETFRIRDUPJ-UHFFFAOYSA-N", "2,2'-thiobis(4-(1,1,3,3-tetramethylbutyl)phenol)", "")
  CALL Species("C_{30}H_{58}O_4S",                     "123-28-4", "GHKOFFNLGXMVNJ-UHFFFAOYSA-N", "propanoic acid, 3,3'-thiobis-, didodecyl ester", "dilauryl thiodipropionate")
  CALL Species("C_{42}H_{82}O_4S",                     "693-36-7", "PWWSSIYVTQUJQQ-UHFFFAOYSA-N", "dioctadecyl 3,3'-thiodipropionate", "")
  ! C, H, N, S:
  CALL Species("CH_3NCS",                              "556-61-6", "LGDSHSYDSCRFAB-UHFFFAOYSA-N", "methyl isothiocyanate", "")
  CALL Species("CH_4N_2S",                              "62-56-6", "UMGDCJDMYOKAJW-UHFFFAOYSA-N", "thiourea", "")
  CALL Species("CH_5N_3S",                              "79-19-6", "BRWIZMBXBAOCCF-UHFFFAOYSA-N", "hydrazinecarbothioamide", "1-amino-2-thiourea")
  CALL Species("C_2H_3NS",                             "556-64-9", "VYHVQEYOFIYNJP-UHFFFAOYSA-N", "thiocyanic acid, methyl ester", "")
  CALL Species("C_2H_5NS",                              "62-55-5", "YUKQRDCYNOVPGJ-UHFFFAOYSA-N", "ethanethioamide", "thioacetamide")
  CALL Species("C_2H_7NS",                              "60-23-1", "UFULAYFCSOUIOV-UHFFFAOYSA-N", "mercaptamine", "cysteamine")
  CALL Species("C_3H_5NS",                             "542-90-5", "WFCLYEAZTHWNEH-UHFFFAOYSA-N", "thiocyanic acid, ethyl ester", "")
  CALL Species("C_3H_6N_2S",                            "96-45-7", "PDQAZBWRQCGBEV-UHFFFAOYSA-N", "2-imidazolidinethione", "ethylene thiourea")
  CALL Species("C_3H_8N_2S",                           "625-53-6", "GMEHFXXZSWDEDB-UHFFFAOYSA-N", "ethylthiourea", "")
  CALL Species("C_4H_5NS",                              "57-06-7", "ZOJBYZNEUISWFT-UHFFFAOYSA-N", "allyl isothiocyanate", "")
  CALL Species("C_5H_{10}N_2S_2",                      "533-74-4", "QAYICIQNSGETAS-UHFFFAOYSA-N", "dazomet", "")
  CALL Species("C_5H_{12}N_2S",                        "105-55-5", "FLVIGYVXZHLUHP-UHFFFAOYSA-N", "N,N'-diethylthiourea", "")
  CALL Species("C_5H_{12}N_2S",                       "2782-91-4", "MNOILHPDHOHILI-UHFFFAOYSA-N", "tetramethylthiourea", "")
  CALL Species("C_6H_{12}N_2S_4",                      "137-26-8", "KUAZQDVKQLNFPE-UHFFFAOYSA-N", "thiram", "")
  CALL Species("C_6H_{12}N_2S_3",                       "97-74-5", "REQPQFUJGGOFQL-UHFFFAOYSA-N", "bis(dimethylthiocarbamyl) sulfide", "bis(dimethylthiocarbamoyl) sulfide")
  CALL Species("C_7H_5NS",                              "95-16-9", "IOJUPLGTWVMSFF-UHFFFAOYSA-N", "benzothiazole", "")
  CALL Species("C_7H_5NS_2",                           "149-30-4", "YXIWHUQXZSMYRE-UHFFFAOYSA-N", "2-mercaptobenzothiazole", "")
  CALL Species("C_7H_6N_2S",                           "136-95-8", "UHGULLIUJBCTEF-UHFFFAOYSA-N", "2-benzothiazolamine", "")
  CALL Species("C_7H_8N_2S",                           "103-85-5", "FULZLIGZKMKICU-UHFFFAOYSA-N", "phenylthiourea", "")
  CALL Species("C_7H_{11}N_7S",                       "4658-28-0", "AFIIBUOYKYSPKB-UHFFFAOYSA-N", "aziprotryn", "")
  CALL Species("C_7H_5NS",                             "103-72-0", "QKFJKGMPGYROCL-UHFFFAOYSA-N", "isothiocyanatobenzene", "")
  CALL Species("C_8H_{15}N_5S",                       "1014-70-6", "MGLWZSOBALDPEK-UHFFFAOYSA-N", "simetryn", "")
  CALL Species("C_8H_{15}N_5S",                       "1014-69-3", "HCRWJJJUKUVORR-UHFFFAOYSA-N", "desmetryn", "")
  CALL Species("C_9H_4N_2S_3",                          "93-75-4", "ILERPRJWJPJZDN-UHFFFAOYSA-N", "thioquinox", "")
  CALL Species("C_9H_6N_2S_3",                       "21564-17-0", "TUBQDCKAWGHZPF-UHFFFAOYSA-N", "thiocyanic acid, (2-benzothiazolylthio)methyl ester", "")
  CALL Species("C_9H_{17}N_5S",                        "834-12-8", "RQVYBGPQFYCBGX-UHFFFAOYSA-N", "ametryn", "")
  CALL Species("C_{10}H_{16}N_6S",                   "51481-61-9", "AQIXAKUUQRKLND-UHFFFAOYSA-N", "cimetidine", "")
  CALL Species("C_{10}H_{19}N_5S",                    "7287-19-6", "AAEVYOVXGOFMJO-UHFFFAOYSA-N", "prometryn", "")
  CALL Species("C_{10}H_{19}N_5S",                     "886-50-0", "IROINLKCQGIITA-UHFFFAOYSA-N", "terbutryn", "")
  CALL Species("C_{10}H_{20}N_2S_4",                    "97-77-8", "AUZONCFQVSMFAP-UHFFFAOYSA-N", "disulfiram", "")
  CALL Species("C_{11}H_{10}N_2S",                      "86-88-4", "PIVQQUNOTICCSA-UHFFFAOYSA-N", "1-naphthalenylthiourea", "")
  CALL Species("C_{12}H_{12}N_2S",                     "139-65-1", "ICNFHJVPAJKPHW-UHFFFAOYSA-N", "4,4'-thiobisbenzenamine", "bis(4-aminophenyl) sulfide")
  CALL Species("C_{11}H_{19}N_5S",                    "4147-51-7", "NPWMZOGDXOFZIN-UHFFFAOYSA-N", "dipropetryn", "")
  CALL Species("C_{11}H_{21}N_5S",                   "22936-75-0", "IKYICRRUVNIHPP-UHFFFAOYSA-N", "dimethametryn", "")
  CALL Species("C_{12}H_9NS",                           "92-84-2", "WJFKNYWRSNBZNX-UHFFFAOYSA-N", "phenothiazine", "")
  CALL Species("C_{12}H_{14}N_2S",                   "61676-87-7", "YUAUPYJCVKNAEC-UHFFFAOYSA-N", "cymiazole", "")
  CALL Species("C_{14}H_8N_2S_4",                      "120-78-5", "AFZSMODLJJCVPP-UHFFFAOYSA-N", "2,2'-dithiobisbenzothiazole", "2,2'-dibenzothiazyl disulfide")
  CALL Species("C_{14}H_{19}N_3S",                      "91-80-5", "HNJJXZKZRAWDPF-UHFFFAOYSA-N", "methapyrilene", "")
  CALL Species("C_{17}H_{20}N_4S",                  "132539-06-1", "KVWDHTXUZHCGIO-UHFFFAOYSA-N", "olanzapine", "")
  CALL Species("C_{18}H_{17}N_3S_4",                  "3741-80-8", "VILGDADBAQFRJE-UHFFFAOYSA-N", "N-(1,1-dimethylethyl)bis(2-benzothiazolesulfen)amide", "")
  CALL Species("C_{21}H_{28}N_2S_2",                 "51308-54-4", "ZZVVDIVWGXTDRQ-UHFFFAOYSA-N", "buthiobate", "")
  ! C, H, O, N, S:
  CALL Species("CH_3NO_6S",                                    "", "IMLJWBRSRKRPAQ-UHFFFAOYSA-N", "MCM:CH3SO4NO2", "")
  CALL Species("CH_3NO_5S",                                    "", "PGTDFOBGUOOGGZ-UHFFFAOYSA-N", "MCM:CH3SOO2NO2", "")
  CALL Species("C_2H_7NO_3S",                          "107-35-7", "XOAAWQZATWQOTB-UHFFFAOYSA-N", "taurine", "")
  CALL Species("C_3H_3N_3O_2S",                        "121-66-4", "MIHADVKEHAFNPG-UHFFFAOYSA-N", "2-amino-5-nitrothiazole", "")
  CALL Species("C_3H_6N_2OS",                          "591-08-2", "IPCRBOOJBPETMF-UHFFFAOYSA-N", "N-(aminothioxomethyl)acetamide", "1-acetyl-2-thiourea")
  CALL Species("C_4H_5NO_4S",                        "33665-90-6", "YGCFIWIQZPHFLU-UHFFFAOYSA-N", "acesulfame", "")
  CALL Species("C_5H_{10}N_2O_2S",                   "16752-77-5", "UHXUZOCRWCRNSJ-UHFFFAOYSA-N", "methomyl", "")
  CALL Species("C_6H_7NO_3S",                          "121-57-3", "HVBSAKJJOYLTQU-UHFFFAOYSA-N", "4-aminobenzenesulfonic acid", "sulfanilic acid")
  CALL Species("C_6H_8N_2O_2S",                         "63-74-1", "FDDDEECHVMSUSB-UHFFFAOYSA-N", "sulfanilamide", "")
  CALL Species("C_6H_8N_4O_3S",                        "139-94-6", "FQSUTLQHSDLLAN-UHFFFAOYSA-N", "nithiazide", "")
  CALL Species("C_7H_9NO_2S",                           "88-19-7", "YCMLQMDWSXFTIF-UHFFFAOYSA-N", "2-methylbenzenesulfonamide", "$o$-toluenesulfonamide")
  CALL Species("C_7H_9NO_2S",                           "70-55-3", "LMYRWZFENFIFIT-UHFFFAOYSA-N", "4-methylbenzenesulfonamide", "$p$-toluenesulfonamide")
  CALL Species("C_7H_{12}N_4O_3S_2",                 "30043-49-3", "KCOCSOWTADCKOL-UHFFFAOYSA-N", "ethidimuron", "")
  CALL Species("C_7H_{13}N_3O_3S",                   "23135-22-0", "KZAUOCCYDRDERY-UITAMQMPSA-N", "oxamyl", "")
  CALL Species("C_7H_{14}N_2O_2S",                     "116-06-3", "QGLZXHRNAYXIBU-WEVVVXLNSA-N", "aldicarb", "")
  CALL Species("C_7H_{14}N_2O_3S",                    "1646-87-3", "BXPMAGSOWXBZHS-UHFFFAOYSA-N", "aldicarb sulfoxide", "")
  CALL Species("C_7H_{14}N_2O_4S",                    "1646-88-4", "YRRKLBAKDXSTNC-UHFFFAOYSA-N", "aldicarb sulfone", "")
  CALL Species("C_7H_{14}N_2O_2S",                   "34681-10-2", "SFNPDDSJBGRXLW-UHFFFAOYSA-N", "butocarboxim", "")
  CALL Species("C_7H_{14}N_2O_4S",                   "34681-23-7", "CTJBHIROCMPUKL-UHFFFAOYSA-N", "butoxycarboxim", "")
  CALL Species("C_7H_5NO_3S",                           "81-07-2", "CVHZOJJKTDOEJC-UHFFFAOYSA-N", "saccharin", "")
  CALL Species("C_8H_6N_2OS_2",                     "135158-54-2", "UELITFHSCLAHKR-UHFFFAOYSA-N", "acibenzolar-S-methyl", "")
  CALL Species("C_8H_6N_2OS_2",                       "2439-01-2", "FBQQHUGEACOBDN-UHFFFAOYSA-N", "quinomethionate", "")
  CALL Species("C_8H_6N_4O_4S",                       "3570-75-0", "DUWYZHLZDVCZIO-UHFFFAOYSA-N", "nifurthiazole", "")
  CALL Species("C_8H_7NO_3S",                         "4083-64-1", "VLJQDHDVZJXNQL-UHFFFAOYSA-N", "4-methylbenzenesulfonyl isocyanate", "")
  CALL Species("C_8H_{13}N_3O_4S",                   "19387-91-8", "HJLSLZFTEKNLFI-UHFFFAOYSA-N", "tinidazole", "")
  CALL Species("C_8H_{14}N_4OS",                     "21087-64-9", "FOXFZRUHNHCZPX-UHFFFAOYSA-N", "metribuzin", "")
  CALL Species("C_9H_7N_3S",                         "41814-78-2", "DQJCHOQLCLEDLL-UHFFFAOYSA-N", "tricyclazole", "")
  CALL Species("C_9H_9N_3OS",                         "1929-88-0", "DTCJYIIKPVRVDD-UHFFFAOYSA-N", "benzthiazuron", "")
  CALL Species("C_9H_8N_4OS",                        "51707-55-2", "HFCYZXMHUIHAQI-UHFFFAOYSA-N", "thidiazuron", "")
  CALL Species("C_9H_9N_3O_2S_2",                       "72-14-0", "JNMRHUJNCSQMMB-UHFFFAOYSA-N", "sulfathiazole", "")
  CALL Species("C_9H_{10}N_4O_2S_2",                   "144-82-1", "VACCAVUAMIDAGB-UHFFFAOYSA-N", "sulfamethizole", "")
  CALL Species("C_9H_{16}N_4OS",                     "64529-56-2", "ADZSGNDOZREKJK-UHFFFAOYSA-N", "ethiozin", "")
  CALL Species("C_9H_{16}N_4OS",                     "34014-18-1", "HBPDKDSFLXWOAE-UHFFFAOYSA-N", "tebuthiuron", "")
  CALL Species("C_9H_{17}NOS",                        "2212-67-1", "DEDOPGXGGQYYMW-UHFFFAOYSA-N", "molinate", "")
  CALL Species("C_9H_{18}N_2O_2S",                   "39196-18-4", "FZSVSABTBYGOQH-UHFFFAOYSA-N", "thiofanox", "")
  CALL Species("C_9H_{15}N_5O_7S_2",                "120923-37-7", "CTTHWASMBLQOFR-UHFFFAOYSA-N", "amidosulfuron", "")
  CALL Species("C_9H_{19}NOS",                         "759-94-4", "GUVLYNGULCJVDO-UHFFFAOYSA-N", "S-ethyl dipropylthiocarbamate", "eptam; EPTC")
  CALL Species("C_{10}H_7N_3S",                        "148-79-8", "WJCNZQLZVWNLKY-UHFFFAOYSA-N", "thiabendazole", "")
  CALL Species("C_{10}H_9NO_2S",                      "1079-33-0", "BOTUVXISJHKZKJ-UHFFFAOYSA-N", "benzo[$b$]thiophene-4-ol, methylcarbamate", "mobam")
  CALL Species("C_{10}H_{11}N_3O_3S",                  "723-46-6", "JLKIGFTWXXRPMT-UHFFFAOYSA-N", "sulfamethoxazole", "")
  CALL Species("C_{10}H_{12}N_2O_3S",                "25057-89-0", "ZOMSMJKLGFBRBS-UHFFFAOYSA-N", "bentazone", "")
  CALL Species("C_{10}H_{16}N_4O_2S",                "55511-98-3", "SWMGXKSQWDSBKV-UHFFFAOYSA-N", "buthidazole", "")
  CALL Species("C_{10}H_{18}N_4O_4S_3",              "59669-26-0", "XDOTVMNBCQVZKG-UHFFFAOYSA-N", "thiodicarb", "")
  CALL Species("C_{10}H_{21}NOS",                     "1114-71-2", "SGEJQUSYQTVSIU-UHFFFAOYSA-N", "pebulate", "")
  CALL Species("C_{10}H_{21}NOS",                     "1929-77-7", "OKUGPJPKMAEJOE-UHFFFAOYSA-N", "vernolate", "")
  CALL Species("C_{10}H_{16}N_2O_3S",                   "58-85-5", "YBJHBAHKTGYVGT-ZKWXMUAHSA-N", "biotin", "")
  CALL Species("C_{10}H_{11}N_3OS",                  "18691-97-9", "RRVIAQKBTUQODI-UHFFFAOYSA-N", "methabenzthiazuron", "")
  CALL Species("C_{11}H_{13}N_3O_3S",                  "127-69-5", "NHUHCSRWZMLRLA-UHFFFAOYSA-N", "sulfisoxazole", "")
  CALL Species("C_{11}H_{15}NO_2S",                  "29973-13-5", "HEZNVIYQEUHLNI-UHFFFAOYSA-N", "ethiofencarb", "")
  CALL Species("C_{11}H_{15}NO_2S",                   "2032-65-7", "YFBPRJGDJKVWAH-UHFFFAOYSA-N", "methiocarb", "")
  CALL Species("C_{11}H_{21}NOS",                     "1134-23-2", "DFCAFRGABIXSDS-UHFFFAOYSA-N", "cycloate", "")
  CALL Species("C_{11}H_{21}N_5OS",                    "841-06-5", "DDUIUBPJPOKOMV-UHFFFAOYSA-N", "methoprotryn", "")
  CALL Species("C_{11}H_{23}NOS",                     "2008-41-5", "BMTAFVWTTFSTOG-UHFFFAOYSA-N", "butylate", "")
  CALL Species("C_{11}H_{19}NOS",                    "26530-20-1", "JPMIIZHYYWMHDT-UHFFFAOYSA-N", "2-octyl-3(2H)-isothiazolone", "")
  CALL Species("C_{12}H_{12}N_2OS",                  "21452-18-6", "UDSJPFPDKCMYBD-UHFFFAOYSA-N", "metsulfovax", "")
  CALL Species("C_{12}H_{13}NO_2S",                   "5234-68-4", "GYSSRZJIHXQEHQ-UHFFFAOYSA-N", "carboxin", "")
  CALL Species("C_{12}H_{13}NO_4S",                   "5259-88-1", "AMEKQAFGQBKLKX-UHFFFAOYSA-N", "oxycarboxin", "")
  CALL Species("C_{12}H_{13}N_5O_6S_2",              "79277-27-3", "AHTPATJNIAFOLR-UHFFFAOYSA-N", "thifensulfuron-methyl", "")
  CALL Species("C_{12}H_{14}N_4O_2S",                   "57-68-1", "ASWVTGNCAZCNNR-UHFFFAOYSA-N", "sulfamethazine", "")
  CALL Species("C_{12}H_{14}N_4O_4S_2",              "23564-05-8", "QGHREAKMXXNCOA-UHFFFAOYSA-N", "thiophanate-methyl", "")
  CALL Species("C_{12}H_{14}N_4O_5S_2",                 "80-51-3", "NBOCQTNZUPTTEI-UHFFFAOYSA-N", "4,4'-oxydi(benzenesulphonohydrazide)", "")
  CALL Species("C_{12}H_{15}N_3O_2S",                "54965-21-8", "HXHWSAZORRCQMX-UHFFFAOYSA-N", "albendazole", "")
  CALL Species("C_{12}H_{18}N_4O_6S",                "19044-88-3", "UNAHYJYOSSSJHH-UHFFFAOYSA-N", "oryzalin", "")
  CALL Species("C_{12}H_{19}N_3O_3S",                "56046-62-9", "XWQURWIJAIIPQP-UHFFFAOYSA-N", "STK366145", "N-(2-ethyl(3-methyl-4-nitrosophenyl)amino)ethyl)-methanesulfonamide")
  CALL Species("C_{12}H_{20}N_4OS",                  "57052-04-7", "MZTLOILRKLUURT-UHFFFAOYSA-N", "isomethiozin", "")
  CALL Species("C_{12}H_{14}N_4O_7S_2",             "317815-83-1", "XSKZXGDFSCCXQX-UHFFFAOYSA-N", "thiencarbazone-methyl", "")
  CALL Species("C_{12}H_6N_2O_4S",                  "109041-38-5", "VMQHOWOVMXIROE-UHFFFAOYSA-N", "2,8-dinitrodibenzothiophene", "")
  CALL Species("C_{12}H_7NO_2S",                      "6639-36-7", "GXLYVLHWXVRVKI-UHFFFAOYSA-N", "2-nitrodibenzothiophene", "")
  CALL Species("C_{13}H_{16}N_{10}O_5S",            "120162-55-2", "MAHPNPYYQAIOJN-UHFFFAOYSA-N", "azimsulfuron", "")
  CALL Species("C_{13}H_{19}NO_2S",                  "62850-32-2", "HMIBKHHNXANVHR-UHFFFAOYSA-N", "fenothiocarb", "")
  CALL Species("C_{13}H_{19}NO_2S",                    "115-31-1", "IXEVGHXRXDBAOB-RUETXSTFSA-N", "isobornyl thiocyanoacetate", "")
  CALL Species("C_{13}H_{19}N_3O_6S",                 "4726-14-1", "UMKANAFDOQQUKE-UHFFFAOYSA-N", "nitralin", "")
  CALL Species("C_{13}H_{24}N_4O_3S",                "41483-43-6", "DSKJPMWIHSOYEA-UHFFFAOYSA-N", "bupirimate", "")
  CALL Species("C_{13}H_{24}N_4O_3S",                "26839-75-8", "BLJRIMJGRPQVNF-JTQLQIEISA-N", "timolol", "")
  CALL Species("C_{13}H_{22}N_4O_3S",               "112143-82-5", "NKNFWVNSBIXGLL-UHFFFAOYSA-N", "triazamate", "")
  CALL Species("C_{14}H_4N_2O_2S_2",                  "3347-22-6", "PYZSVQVRHDXQSL-UHFFFAOYSA-N", "dithianone", "")
  CALL Species("C_{14}H_{15}NO_2S",                  "17796-82-6", "UEZWYKZHXASYJN-UHFFFAOYSA-N", "N-(cyclohexylthio)phthalimide", "")
  CALL Species("C_{14}H_{15}N_5O_6S",                "74223-64-6", "RSMUVYRMZCOLBH-UHFFFAOYSA-N", "metsulfuron-methyl", "")
  CALL Species("C_{14}H_{17}N_5O_7S_2",             "122931-48-0", "MEFOUWRMVYJCQC-UHFFFAOYSA-N", "rimsulfuron", "")
  CALL Species("C_{14}H_{18}N_4O_4S_2",              "23564-06-9", "YFNCATAIYKQPOO-UHFFFAOYSA-N", "thiophanate", "")
  CALL Species("C_{14}H_{21}N_3O_2S",               "103628-46-2", "KQKPFRSPSRPDEB-UHFFFAOYSA-N", "sumatriptan", "")
  CALL Species("C_{14}H_{13}NO_7S",                 "104206-82-8", "KPUREKXXPHOJQT-UHFFFAOYSA-N", "mesotrione", "")
  CALL Species("C_{14}H_{21}NOS",                    "52888-80-9", "NQLVQOSNDJXLKG-UHFFFAOYSA-N", "prosulfocarb", "")
  CALL Species("C_{15}H_{14}N_2O_4S",               "135186-78-6", "RRKHIAYNPVQKEF-UHFFFAOYSA-N", "pyriftalid", "")
  CALL Species("C_{15}H_{18}N_6O_6S",               "111991-09-4", "RTCOGUMHFFWOJV-UHFFFAOYSA-N", "nicosulfuron", "")
  CALL Species("C_{15}H_{16}N_4O_5S",                "74222-97-2", "ZDXMLEQEMNLCQG-UHFFFAOYSA-N", "sulfometuron methyl", "")
  CALL Species("C_{15}H_{17}N_5O_6S",               "101200-48-0", "VLCQZHSMCYCDJL-UHFFFAOYSA-N", "tribenuron-methyl", "")
  CALL Species("C_{15}H_{18}N_6O_6S",                "97780-06-8", "ZINJLDJMHCUBIP-UHFFFAOYSA-N", "ethametsulfuron-methyl", "")
  CALL Species("C_{15}H_{18}N_4O_7S",               "145026-81-9", "JTHMVYBOQLDDIY-UHFFFAOYSA-N", "propoxycarbazone", "")
  CALL Species("C_{15}H_{21}NOS",                    "61432-55-1", "BWUPSGJXXPATLU-UHFFFAOYSA-N", "dimepiperate", "")
  CALL Species("C_{15}H_{23}NOS",                    "85785-20-2", "BXEHUCNTIZGSOJ-UHFFFAOYSA-N", "esprocarb", "")
  CALL Species("C_{16}H_{14}N_2O_3S",               "181695-72-7", "LNPDTQAFDNKSHK-UHFFFAOYSA-N", "valdecoxib", "")
  CALL Species("C_{16}H_{17}N_3O_5S",               "210631-68-8", "BPPVUXSMLBXYGG-UHFFFAOYSA-N", "topramezone", "")
  CALL Species("C_{16}H_{18}N_6O_7S_2",             "141776-32-1", "RBSXHDIPCIWOMG-UHFFFAOYSA-N", "sulfosulfuron", "")
  CALL Species("C_{16}H_{20}N_6O_6S",               "213464-77-8", "UCDPMNSCCRBWIC-UHFFFAOYSA-N", "orthosulfamuron", "")
  CALL Species("C_{16}H_{22}N_4O_3S",               "125306-83-4", "HFEJHAAIJZXXRE-UHFFFAOYSA-N", "cafenstrole", "")
  CALL Species("C_{16}H_{23}N_3OS",                  "69327-76-0", "PRLVTUNWOQKEAI-UHFFFAOYSA-N", "buprofezin", "")
  CALL Species("C_{16}H_{18}N_4O_7S",                "83055-99-6", "XMQFTWRPUQYINF-UHFFFAOYSA-N", "bensulfuron methyl", "")
  CALL Species("C_{16}H_{14}N_2O_2S",                "73250-68-7", "XIGAUIHYSDTJHW-UHFFFAOYSA-N", "mefenacet", "")
  CALL Species("C_{17}H_{25}N_3O_4S_2",              "83130-01-2", "GMAUQNJOSOMMHI-JXAWBTAJSA-N", "alanycarb", "")
  CALL Species("C_{17}H_{17}N_3OS",                 "161326-34-7", "LMVPQMGRYSRMIW-KRWDZBQOSA-N", "fenamidone", "")
  CALL Species("C_{17}H_{19}N_3O_3S",               "119141-88-7", "SUBDBMMJDZJVOS-UHFFFAOYSA-N", "esomeprazole", "")
  CALL Species("C_{17}H_{20}N_6O_7S",               "173159-57-4", "PXDNXJSDGQBLKS-UHFFFAOYSA-N", "foramsulfuron", "")
  CALL Species("C_{17}H_{29}NO_3S",                  "74051-80-2", "NMHGOXYVOKDNHF-UHFFFAOYSA-N", "sethoxydim", "")
  CALL Species("C_{17}H_{21}N_5O_9S_2",             "208465-21-8", "NIFKBBMCXCMCAO-UHFFFAOYSA-N", "mesosulfuron-methyl", "")
  CALL Species("C_{17}H_{27}NO_3S",                 "101205-02-1", "HAHCNFVGRVWFIP-VKAVYKQESA-N", "cycloxydim", "")
  CALL Species("C_{17}H_{21}N_3O_2S",               "473798-59-3", "UTOHZQYBSYOOGC-UHFFFAOYSA-N", "fenpyrazamine", "")
  CALL Species("C_{18}H_{18}N_2O_5S",               "221667-31-8", "OAWUUPVZMNKZRY-UHFFFAOYSA-N", "cyprosulfamide", "")
  CALL Species("C_{18}H_{19}N_3O_3S",               "122320-73-4", "YASAKCUCGLMORW-UHFFFAOYSA-N", "rosiglitazone", "")
  CALL Species("C_{18}H_{21}N_3O_3S",               "117976-89-3", "YREYEVIYCVEVJK-UHFFFAOYSA-N", "rabeprazole", "")
  CALL Species("C_{18}H_{22}N_2O_2S",                "88678-67-5", "VTRWMTJQBQJKQH-UHFFFAOYSA-N", "pyributicarb", "")
  CALL Species("C_{18}H_{26}N_2O_5S",                "65907-30-4", "HAWJXYBZNNRMNO-UHFFFAOYSA-N", "furathiocarb", "")
  CALL Species("C_{18}H_{34}N_2O_6S",                  "154-21-2", "OJMMVQQUTAEWLP-UHFFFAOYSA-N", "lincomycin", "")
  CALL Species("C_{19}H_{20}N_2O_3S",               "111025-46-8", "HYAFETHFCAUJAY-UHFFFAOYSA-N", "pioglitazone", "")
  CALL Species("C_{20}H_{25}NO_3S",                 "875915-78-9", "WMKZDPFZIZQROT-UHFFFAOYSA-N", "isofetamid", "")
  CALL Species("C_{20}H_{28}N_2O_5S",               "106133-20-4", "DRHKJLXJIQTDTD-OAHLLOKOSA-N", "tamsulosin", "")
  CALL Species("C_{20}H_{32}N_2O_3S",                "55285-14-8", "JLQUFIHWVLZVTJ-UHFFFAOYSA-N", "carbosulfan", "")
  CALL Species("C_{22}H_{30}N_2O_2S",                "56030-54-7", "GGCSSNBKKAUURC-UHFFFAOYSA-N", "sufentanil", "")
  CALL Species("C_{22}H_{30}N_6O_4S",               "139755-83-2", "BNRNXUUZRGQAQC-UHFFFAOYSA-N", "sildenafil", "")
  CALL Species("C_{20}H_{30}N_2O_5S",                "82560-54-1", "FYZBOYWSHKHDMT-UHFFFAOYSA-N", "benfuracarb", "")
  CALL Species("C_{22}H_{36}N_2O_5S",               "144494-65-5", "COKMIXFXJJXBQG-NRFANRHFSA-N", "tirofiban", "")
  CALL Species("C_{23}H_{32}N_6O_4S",               "224785-90-4", "SECKRCOLJRRGGV-UHFFFAOYSA-N", "vardenafil", "")
  CALL Species("C_{26}H_{45}NO_7S",                     "81-24-3", "WBWWGRHZICKQGZ-HZAMXZRMSA-N", "taurocholic acid", "")
  CALL Species("C_{34}H_{50}N_4O_9S",               "112362-50-2", "SUYRLXYYZQTJHF-FUODUHIRSA-N", "dalfopristin", "")
  CALL Species("C_{37}H_{37}N_2O_6S_2",               "4680-78-8", "SRRJCDUOSQWHGS-UHFFFAOYSA-O", "C.I. acid green 3", "")
  CALL Species("C_{40}H_{40}N_{12}O_8S_2",           "24231-46-7", "YGUMVDWOQQJBGA-VAWYXSNFSA-N", "tinopal", "")
  CALL Species("C_{53}H_{67}N_9O_{10}S",            "120138-50-3", "WTHRRGMBUAHGNI-UHFFFAOYSA-N", "quinupristin", "")
  ! C, H, F, S:
  CALL Species("C_4H_5F_5S",                         "68140-18-1", "WEILNYJKAUGBAU-UHFFFAOYSA-N", "3,3,4,4,4-pentafluorobutane-1-thiol", "")
  CALL Species("C_6H_5F_9S",                         "68140-20-5", "GQJXVHYUQPXZOL-UHFFFAOYSA-N", "3,3,4,4,5,5,6,6,6-nonafluoro-1-hexanethiol", "")
  CALL Species("C_{10}H_7F_{15}S",                   "68140-21-6", "ROKKEEDUUQVHFZ-UHFFFAOYSA-N", "3,3,4,4,5,5,7,7,8,8,9,9,10,10,10-pentadecafluoro-1-decanethiol", "")
  CALL Species("C_{12}H_{14}F_{12}S_2",             "118400-71-8", "CIZUOSOWGOENRE-UHFFFAOYSA-N", "3,3,4,4,5,5-hexafluoro-1-(3,3,4,4,5,5-hexafluorohexyldisulfanyl)hexane", "")
  ! C, H, O, F, S:
  CALL Species("CH_3FO_2S",                            "558-25-8", "KNWQLFOXPQZGPX-UHFFFAOYSA-N", "methanesulfonyl fluoride", "")
  CALL Species("C_4HF_9O_3S",                          "375-73-5", "JGTNAGYHADQMCM-UHFFFAOYSA-N", "perfluorobutane sulfonic acid", "PFBS")
  CALL Species("C_6H_5F_5OS",                          "774-94-7", "XHJLGVIUMCBMHL-UHFFFAOYSA-N", "4-(pentafluorosulfanyl)phenol", "")
  CALL Species("C_6H_5F_9O_3S",                     "757124-72-4", "TXGIGTRUEITPSC-UHFFFAOYSA-N", "1H,1H,2H,2H-perfluorohexane sulfonic acid", "4:2 FTS")
  CALL Species("C_6HF_{13}O_3S",                       "355-46-4", "QZHDEAJFRJCDMF-UHFFFAOYSA-N", "perfluorohexane sulfonic acid", "PFHxS")
  CALL Species("C_8H_5F_{13}O_3S",                   "27619-97-2", "VIONGDJUYAYOPU-UHFFFAOYSA-N", "1H,1H,2H,2H-perfluorooctane sulfonic acid", "6:2 FTS")
  CALL Species("C_8HF_{17}O_3S",                      "1763-23-1", "YFSUTJLHUFNCNZ-UHFFFAOYSA-N", "perfluorooctane sulfonic acid", "PFOS")
  CALL Species("C_8F_{18}O_2S",                        "307-35-7", "BHFJBHMTEDLICO-UHFFFAOYSA-N", "heptadecafluorooctanesulphonyl fluoride", "perfluorooctylsulfonyl fluoride")
  CALL Species("C_{10}H_5F_{17}O_3S",                "39108-34-4", "ALVYVCQIFHTIRD-UHFFFAOYSA-N", "1H,1H,2H,2H-perfluorodecane sulfonic acid", "8:2 FTS")
  CALL Species("C_{22}H_{27}F_3O_4S",                "90566-53-3", "MGNNYOODZCAHBA-GQKYHHCASA-N", "fluticasone", "")
  ! C, H, N, F, S:
  CALL Species("C_6H_6F_5NS",                         "2993-22-8", "NUFLICUHOXHWER-UHFFFAOYSA-N", "3-aminophenylsulfur pentafluoride", "")
  CALL Species("C_6H_6F_5NS",                         "2993-24-0", "MZGZUHNSMNNSRJ-UHFFFAOYSA-N", "4-aminophenylsulfur pentafluoride", "")
  CALL Species("C_{17}H_{10}F_6N_4S",                "37893-02-0", "IZFZCMFMJKDHJZ-UHFFFAOYSA-N", "flubenzimine", "")
  ! C, H, O, N, F, S:
  CALL Species("C_6H_7F_3N_4OS",                     "25366-23-8", "BBJPZPLAZVZTGR-UHFFFAOYSA-N", "thiazafluron", "")
  CALL Species("C_6H_4F_5NO_2S",                      "2613-27-6", "AGNCKMHGYZKMLN-UHFFFAOYSA-N", "4-nitrophenylsulfur pentafluoride", "")
  CALL Species("C_6H_4F_{11}NO_2S",                  "68298-13-5", "BKKNDZBSSSAGIB-UHFFFAOYSA-N", "undecafluoro-N-methyl-1-pentanesulfonamide", "")
  CALL Species("C_7H_8F_9NO_3S",                     "34454-97-2", "DSRUAYIFDCHEEV-UHFFFAOYSA-N", "1,1,2,2,3,3,4,4,4-nonafluoro-N-(2-hydroxyethyl)-N-methylbutane-1-sulfonamide", "")
  CALL Species("C_7H_4F_{13}NO_2S",                  "68259-15-4", "HPPDPHZGXWMRHN-UHFFFAOYSA-N", "tridecafluoro-N-methyl-1-hexanesulfonamide", "")
  CALL Species("C_8H_2F_{17}NO_2S",                    "754-91-6", "RRRXPPIDPYTNJG-UHFFFAOYSA-N", "perfluorooctane sulfonamide", "PFOSA")
  CALL Species("C_8H_{10}FN_3O_3S",                 "143491-57-0", "XQSPYNMVSIKCOC-NTSWFWBYSA-N", "emtricitabine", "")
  CALL Species("C_8H_{10}F_9NO_3S",                  "34449-89-3", "ZSBOIPHQFKYRMG-UHFFFAOYSA-N", "N-ethyl-1,1,2,2,3,3,4,4,4-nonafluoro-N-(2-hydroxyethyl)butane-1-sulfonamide", "")
  CALL Species("C_8H_4NO_2F_{15}S",                  "68259-14-3", "KDHCALLFPWZTPN-UHFFFAOYSA-N", "1,1,2,2,3,3,4,4,5,5,6,6,7,7,7-pentadecafluoro-N-methylheptane-1-sulphonamide", "")
  CALL Species("C_8H_8NO_3F_{11}S",                  "68555-74-8", "BRBCKWCOTRPYGH-UHFFFAOYSA-N", "1,1,2,2,3,3,4,4,5,5,5-undecafluoro-N-(2-hydroxyethyl)-N-methylpentane-1-sulphonamide", "")
  CALL Species("C_9H_{13}F_9N_2O_2S",                "68555-77-1", "XMRMVBVJGSKMEN-UHFFFAOYSA-N", "N-(3-(dimethylamino)propyl)-nonafluoro-1-butanesulfonamide", "")
  CALL Species("C_9H_4F_{17}NO_2S",                  "31506-32-8", "SRMWNTGHXHOWBT-UHFFFAOYSA-N", "N-methylperfluorooctanesulphonamide", "N-MeFOSA")
  CALL Species("C_9H_{10}NO_3F_{11}S",               "68555-72-6", "GBPAQIZWHVCENQ-UHFFFAOYSA-N", "N-ethyl-1,1,2,2,3,3,4,4,5,5,5-undecafluoro-N-(2-hydroxyethyl)-1-pentanesulfonamide", "")
  CALL Species("C_9H_8NO_3F_{13}S",                  "68555-75-9", "UYIBZOUSVFOPJK-UHFFFAOYSA-N", "1,1,2,2,3,3,4,4,5,5,6,6,6-tridecafluoro-N-(2-hydroxyethyl)-N-methyl-1-hexanesulfonamide", "")
  CALL Species("C_{10}H_{10}F_{13}NO_3S",            "34455-03-3", "SSGYCIQAXNQIBC-UHFFFAOYSA-N", "N-ethyl-1,1,2,2,3,3,4,4,5,5,6,6,6-tridecafluoro-N-(2-hydroxyethyl)hexane-1-sulfonamide", "")
  CALL Species("C_{10}H_{10}F_9NO_4S",               "67584-55-8", "KEMCLRGAIUJRAN-UHFFFAOYSA-N", "2-{methyl[(nonafluorobutyl)sulfonyl]\+amino}ethyl acrylate", "")
  CALL Species("C_{10}H_6F_{17}NO_2S",                "4151-50-2", "CCEKAJIANROZEO-UHFFFAOYSA-N", "N-ethylperfluorooctanesulfonamide", "N-EtFOSA")
  CALL Species("C_{10}H_8NO_3F_{15}S",               "68555-76-0", "UIZUTEDYGNRNSW-UHFFFAOYSA-N", "1,1,2,2,3,3,4,4,5,5,6,6,7,7,7-pentadecafluoro-N-(2-hydroxyethyl)-N-methylheptane-1-sulphonamide", "")
  CALL Species("C_{11}H_8F_{17}NO_3S",               "24448-09-7", "PLGACQRCZCVKGK-UHFFFAOYSA-N", "N-methyl perfluorooctane sulfonamidoethanol", "MeFOSE")
  CALL Species("C_{11}H_{10}NO_3F_{15}S",            "68555-73-7", "HINASMOVWHGCAK-UHFFFAOYSA-N", "N-ethyl-1,1,2,2,3,3,4,4,5,5,6,6,7,7,7-pentadecafluoro-N-(2-hydroxyethyl)-1-heptanesulfonamide", "")
  CALL Species("C_{11}H_{13}F_3N_2O_3S",             "53780-34-0", "OKIBNKKYNPBDRS-UHFFFAOYSA-N", "mefluidide", "")
  CALL Species("C_{11}H_{13}N_2O_2F_{13}S",          "50598-28-2", "INDOGKYZYLAGEM-UHFFFAOYSA-N", "N-(3-(dimethylamino)propyl)-1,1,2,2,3,3,4,4,5,5,6,6,6-tridecafluoro-1-hexanesulfonamide", "")
  CALL Species("C_{11}H_{10}F_{11}NO_4S",            "67584-56-9", "FZWFDJBZTLTRGH-UHFFFAOYSA-N", "2-(methyl((undecafluoropentyl)\+sulfonyl)amino)ethyl prop-2-enoate", "")
  CALL Species("C_{11}H_{12}F_9NO_4S",               "67584-59-2", "BEIWUHUHJDEMQO-UHFFFAOYSA-N", "2-(methyl((nonafluorobutyl)\+sulphonyl)amino)ethyl methacrylate", "")
  CALL Species("C_{12}H_{13}F_{15}N_2O_2S",          "67584-54-7", "RFJQYRXZGNILIY-UHFFFAOYSA-N", "N-(3-(dimethylamino)propyl)\+pentadecafluoro-1-heptanesulfonamide", "")
  CALL Species("C_{12}H_{10}F_{13}NO_4S",            "67584-57-0", "HLKZFXXWGVPYAY-UHFFFAOYSA-N", "acrylic acid 2-[methyl[(tridecafluorohexyl)sulfonyl]\+amino]ethyl ester", "")
  CALL Species("C_{12}H_{10}F_{17}NO_3S",             "1691-99-2", "HUFHNYZNTFSKCT-UHFFFAOYSA-N", "N-ethyl perfluorooctane sulfonamidoethanol", "EtFOSE")
  CALL Species("C_{12}H_9F_2N_5O_2S",                "98967-40-9", "RXCPQSJAVKGONC-UHFFFAOYSA-N", "flumetsulam", "")
  CALL Species("C_{12}H_8F_3N_5O_3S",               "145701-23-1", "QZXATCCPQKOEIH-UHFFFAOYSA-N", "florasulam", "")
  CALL Species("C_{13}H_{10}F_{15}NO_4S",            "68084-62-8", "BEYGZFVOTUDDJK-UHFFFAOYSA-N", "2-(methyl((pentadecafluoroheptyl)\+sulphonyl)amino)ethyl acrylate", "")
  CALL Species("C_{13}H_{12}F_3N_5O_5S",            "104040-78-0", "HWATZEJQIXKWQS-UHFFFAOYSA-N", "flazasulfuron", "")
  CALL Species("C_{13}H_9F_6N_5O_4S",               "142469-14-5", "KVEQCVKVIFQSGC-UHFFFAOYSA-N", "tritosulfuron", "")
  CALL Species("C_{14}H_{13}F_3N_6O_5S",            "422556-08-9", "GLBLPMUBLHYFCW-UHFFFAOYSA-N", "pyroxsulam", "")
  CALL Species("C_{14}H_{10}F_{17}NO_4S",            "25268-77-3", "RTJZWOGSCLVJLD-UHFFFAOYSA-N", "N-methyl perfluorooctane sulfonamidoethylacrylate", "MeFOSEA")
  CALL Species("C_{14}H_{13}F_3N_2O_4S",            "365400-11-9", "CZRVDACSCJKRFL-UHFFFAOYSA-N", "pyrasulfotole", "")
  CALL Species("C_{14}H_{13}F_3N_4O_5S",            "874195-61-6", "GBHVIWKSEHWFDD-UHFFFAOYSA-N", "triafamone", "")
  CALL Species("C_{14}H_{13}F_4N_3O_2S",            "142459-58-3", "IANUJLZYFUDJIH-UHFFFAOYSA-N", "flufenacet", "")
  CALL Species("C_{14}H_{14}F_{17}NO_3S",             "2263-09-4", "AQWROZBAYZBWIH-UHFFFAOYSA-N", "N-butyl-1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,8-heptadecafluoro-N-(2-hydroxyethyl)-1-octanesulfonamide", "")
  CALL Species("C_{14}H_{12}NO_4F_{17}S",             "1869-77-8", "LMUUXHHNCDERBQ-UHFFFAOYSA-N", "ethyl N-ethyl-N-[(heptadecafluorooctyl)\+sulphonyl]glycinate", "")
  CALL Species("C_{14}H_{12}F_3NO_4S_2",             "37924-13-3", "WHTBVLXUSXVMEV-UHFFFAOYSA-N", "perfluidone", "")
  CALL Species("C_{15}H_{12}F_{17}NO_4S",            "14650-24-9", "UZMOXNBUTMPDCX-UHFFFAOYSA-N", "2-(((heptadecafluorooctyl)sulfonyl)\+methylamino)ethyl methacrylate", "")
  CALL Species("C_{15}H_{12}F_{17}NO_4S",              "423-82-5", "ZAZJGBCGMUKZEL-UHFFFAOYSA-N", "2-(N-ethylperfluorooctane-sulfonamido)ethyl acrylate", "")
  CALL Species("C_{15}H_{12}F_3NO_4S",              "141112-29-0", "OYIKARCXOQLFHF-UHFFFAOYSA-N", "isoxaflutole", "")
  CALL Species("C_{15}H_{12}F_4N_4O_7S",             "86209-51-0", "ZTYVMAQSHCZXLF-UHFFFAOYSA-N", "primisulfuron-methyl", "")
  CALL Species("C_{15}H_{16}F_5NO_2S_2",             "97886-45-8", "YUBJPYNSGLJZPQ-UHFFFAOYSA-N", "dithiopyr", "")
  CALL Species("C_{16}H_{14}F_5N_5O_5S",            "219714-96-2", "SYJGKVOENHZYMQ-UHFFFAOYSA-N", "penoxsulam", "")
  CALL Species("C_{16}H_{15}F_2N_3O_4S",            "102625-70-7", "IQPSEEYGBUAQFF-UHFFFAOYSA-N", "pantoprazole", "")
  CALL Species("C_{16}H_{14}F_{17}NO_4S",              "376-14-7", "DBCGADAHIXJHCE-UHFFFAOYSA-N", "2-(N-ethylperfluorooctanesulfamido)ethyl methacrylate", "")
  CALL Species("C_{16}H_{17}F_5N_2O_2S",            "117718-60-2", "YIJZJEYQBAAWRJ-UHFFFAOYSA-N", "thiazopyr", "")
  CALL Species("C_{16}H_{20}F_3N_3OS",              "183675-82-3", "PFFIDZXUXFLSSR-UHFFFAOYSA-N", "penthiopyrad", "")
  CALL Species("C_{17}H_{19}F_3N_6O_6S",            "126535-15-7", "IMEVJVISCHQJRM-UHFFFAOYSA-N", "triflusulfuron-methyl", "")
  CALL Species("C_{17}H_{14}F_3N_3O_2S",            "169590-42-5", "RZEKVGVHFLEQIL-UHFFFAOYSA-N", "celecoxib", "")
  CALL Species("C_{17}H_{16}F_{17}NO_4S",              "383-07-3", "AQQNNAXBGPWALO-UHFFFAOYSA-N", "2-(butyl((heptadecafluorooctyl)\+sulfonyl)amino)ethyl acrylate", "")
  CALL Species("C_{18}H_{17}N_4O_2FS",              "123249-43-4", "HZKBYBNLTLVSPX-JZJYNLBNSA-N", "thidiazimin", "")
  CALL Species("C_{18}H_{24}FN_3O_3S",              "177406-68-7", "USRKFGIXLGKMKU-IAQYHMDHSA-N", "benthiavalicarb isopropyl", "")
  CALL Species("C_{19}H_{14}F_4N_2OS_2",            "958647-10-4", "KGXUEPOHGFWQKF-ZCXUNETKSA-N", "flutianil", "")
  CALL Species("C_{22}H_{28}N_3O_6FS",              "287714-41-4", "BPRHUIZQVSMCRT-YXWZHEERSA-N", "rosuvastatin", "")
  CALL Species("C_{24}H_{22}F_5N_5O_2S",           "1003318-67-9", "IAQLCKZJGNTRDO-UHFFFAOYSA-N", "oxathiapiprolin", "")
  ! C, H, Cl, S:
  CALL Species("CCl_4S",                               "594-42-3", "RYFZYYUIAZYQLC-UHFFFAOYSA-N", "trichloromethanesulfenyl chloride", "")
  CALL Species("C_2HCl_5S",                           "1185-09-7", "LCVOCDOSGJHZFH-UHFFFAOYSA-N", "1,1,2,2-tetrachloroethanesulfenyl chloride", "")
  CALL Species("C_3H_7ClS",                            "542-81-4", "MYFKLQFBFSHBPA-UHFFFAOYSA-N", "2-chloroethyl methyl sulfide", "")
  CALL Species("C_4H_9ClS",                            "693-07-2", "GBNVXYXIRHSYEG-UHFFFAOYSA-N", "2-chloroethyl ethyl sulfide", "")
  CALL Species("(ClCH_2CH_2)_2S",                      "505-60-2", "QKSKPIVNLNLAAV-UHFFFAOYSA-N", "2,2'-dichlorodiethylsulfide", "mustard gas") ! other CAS: 39472-40-7, 68157-62-0, 69020-37-7
  CALL Species("C_6H_{12}Cl_2S_2",                    "3563-36-8", "AMGNHZVUZWILSB-UHFFFAOYSA-N", "1,2-bis(2-chloroethylthio)ethane", "sesquimustard")
  CALL Species("C_6HCl_5S",                            "133-49-3", "LLMLGZUZTFMXSA-UHFFFAOYSA-N", "pentachlorobenzenethiol", "")
  CALL Species("C_8H_9ClS",                           "5535-49-9", "QDXIHHOPZFCEAP-UHFFFAOYSA-N", "chloroethyl phenyl sulfide", "")
  CALL Species("C_{12}H_6Cl_4S",                      "2227-13-6", "QUWSDLYBOVGOCW-UHFFFAOYSA-N", "tetrasul", "")
  ! C, H, O, Cl, S:
  CALL Species("CH_3ClO_2S",                           "124-63-0", "QARBMVPHQWIHKH-UHFFFAOYSA-N", "methanesulfonyl chloride", "")
  CALL Species("C_2Cl_6O_2S",                         "3064-70-8", "YBNLWIZAWPBUKQ-UHFFFAOYSA-N", "bis(trichloromethyl)sulfone", "")
  CALL Species("C_6H_5ClO_2S",                          "98-09-9", "CSKNSYBAZOQPLR-UHFFFAOYSA-N", "benzenesulfonyl chloride", "")
  CALL Species("C_7H_7ClO_2S",                          "98-59-9", "YYROPELSRYBVMQ-UHFFFAOYSA-N", "4-methylbenzenesulfonyl chloride", "")
  CALL Species("C_9H_6Cl_6O_3S",                       "115-29-7", "RDYMFSUJUZBWLH-UHFFFAOYSA-N", "endosulfan", "")
  CALL Species("C_9H_6Cl_6O_3S",                       "959-98-8", "RDYMFSUJUZBWLH-AMHWMVONSA-N", "$\alpha$-endosulfan", "endosulfan I")
  CALL Species("C_9H_6Cl_6O_3S",                     "33213-65-9", "RDYMFSUJUZBWLH-MDBBVBRHSA-N", "$\beta$-endosulfan", "endosulfan II") ! other CAS: 891-86-1, 12640-59-4, 19670-15-6
  CALL Species("C_9H_6Cl_6O_4S",                      "1031-07-8", "AAPVQEMYVNZIOO-UHFFFAOYSA-N", "endosulfan sulfate", "")
  CALL Species("C_{11}H_{13}ClO_2S",                 "25319-90-8", "AZFKQCNGMSSWDS-UHFFFAOYSA-N", "mcpa-thioethyl", "")
  CALL Species("C_{12}H_8Cl_2O_2S",                     "80-07-9", "GPAPPPVRLPGFEQ-UHFFFAOYSA-N", "1,1'-sulfonylbis(4-chlorobenzene)", "")
  CALL Species("C_{12}H_6Cl_4O_2S",                     "97-18-7", "JFIOVJDNOJYLKP-UHFFFAOYSA-N", "2,2'-thiobis-4,6-dichlorophenol", "bithionol")
  CALL Species("C_{12}H_6Cl_4O_2S",                    "116-29-0", "MLGCXEBRWGEOQX-UHFFFAOYSA-N", "1,2,4-trichloro-5-[(4-chlorophenyl)sulfonyl]benzene", "tetradifon")
  CALL Species("C_{12}H_8Cl_2O_3S",                     "80-33-1", "RZXLPPRPEOUENN-UHFFFAOYSA-N", "ovex", "chlorfenson")
  CALL Species("C_{12}H_9ClO_2S",                       "80-00-2", "OFCFYWOKHPOXKF-UHFFFAOYSA-N", "sulphenone", "")
  CALL Species("C_{14}H_{13}ClO_5S",                 "99105-77-8", "PQTBTIFWAXVEPB-UHFFFAOYSA-N", "sulcotrione", "")
  CALL Species("C_{15}H_{23}ClO_4S",                   "140-57-8", "YKFRAOGHWKADFJ-UHFFFAOYSA-N", "aramite", "")
  CALL Species("C_{18}H_{10}Cl_2O_2S_2",              "2379-74-0", "NDDLLTAIKYHPOD-ISLYRVAYSA-N", "6-chloro-2-(6-chloro-4-methyl-3-oxobenzo[$b$]thien-2(3H)-ylidene)-4-methylbenzo[$b$]thiophene-3(2H)-one", "")
  ! C, H, N, Cl, S:
  CALL Species("C_3Cl_4N_2S",                         "5848-93-1", "MARKPJMFLDWCID-UHFFFAOYSA-N", "5-chloro-3-(trichloromethyl)-1,2,4-thiadiazole", "")
  CALL Species("C_4H_3Cl_2N_3S",                     "13705-05-0", "MWPZLWRHHPWTFS-UHFFFAOYSA-N", "2,4-dichloro-6-(methylthio)-1,3,5-triazine", "")
  CALL Species("C_7H_5Cl_2NS",                        "1918-13-4", "KGKGSIUWJCAFPX-UHFFFAOYSA-N", "chlorthiamid", "")
  CALL Species("C_7H_7ClN_2S",                        "5344-82-1", "YZUKKTCDYSIWKJ-UHFFFAOYSA-N", "(2-chlorophenyl)thiourea", "")
  CALL Species("C_8H_{14}ClNS_2",                       "95-06-7", "XJCLWVXTCRQIDI-UHFFFAOYSA-N", "2-chloroallyl-N,N-diethyldithiocarbamate", "")
  CALL Species("C_{10}H_9ClN_4S",                   "111988-49-9", "HOKKPVIRMVDYPB-UHFFFAOYSA-N", "thiacloprid", "")
  CALL Species("C_{10}H_{13}N_2ClS",                 "28217-97-2", "IBZZDPVVVSNQOY-UHFFFAOYSA-N", "chloromethiuron", "")
  CALL Species("C_{17}H_{13}Cl_3N_4S",               "86598-92-7", "AGKSTYPVMZODRV-UHFFFAOYSA-N", "imibenconazole", "")
  ! C, H, O, N, Cl, S:
  CALL Species("C_5H_5Cl_3N_2OS",                     "2593-15-9", "KQTVWCSONPJJPE-UHFFFAOYSA-N", "etridiazole", "")
  CALL Species("C_6H_5ClN_2O_4S",                       "97-09-6", "SPZGXONNVLTQDE-UHFFFAOYSA-N", "4-chloro-3-nitrobenzenesulfonamide", "")
  CALL Species("C_6H_8ClN_5O_2S",                   "210880-92-5", "PGOOBECODWQEAB-UHFFFAOYSA-N", "clothianidin", "")
  CALL Species("C_8H_6NOClS",                        "63755-05-5", "QCPASDYEQAVIJF-UHFFFAOYSA-N", "chlobenthiazone", "")
  CALL Species("C_8H_{19}ClN_2OS",                   "19622-19-6", "NMFAMPYSJHIYMR-UHFFFAOYSA-N", "prothiocarb hydrochloride", "")
  CALL Species("C_8H_5N_2OClS_2",                   "116170-30-0", "GNOOAFGERMHQJE-UHFFFAOYSA-N", "thicyofen", "")
  CALL Species("C_8H_{10}ClN_5O_3S",                "153719-23-4", "NWWZPOKUUAIXIW-DHZHZOJOSA-N", "thiamethoxam", "")
  CALL Species("C_9H_{12}Cl_2N_2O_2S",              "151574-12-8", "SBBHTEMLRJJIGK-UHFFFAOYSA-N", "4-amino-3,5-dichloro-N-ethyl-2-methylbenzenesulfonamide", "")
  CALL Species("C_9H_{11}Cl_2FN_2O_2S_2",             "1085-98-9", "WURGXGVFSMYFCG-UHFFFAOYSA-N", "dichlofluanid", "")
  CALL Species("C_9H_8Cl_3NO_2S",                      "133-06-2", "LDVVMCZRFWMZSG-UHFFFAOYSA-N", "captan", "")
  CALL Species("C_9H_6ClNO_3S",                       "3813-05-6", "HYJSGOXICXYZGS-UHFFFAOYSA-N", "benazolin", "")
  CALL Species("C_9H_4Cl_3NO_2S",                      "133-07-3", "HKIOYBQGHSTUDB-UHFFFAOYSA-N", "folpet", "")
  CALL Species("C_{10}H_9Cl_4NO_2S",                  "2425-06-1", "JHRWWRDRBPCWTF-UHFFFAOYSA-N", "captafol", "difolatan")
  CALL Species("C_{10}H_{17}Cl_2NOS",                 "2303-16-4", "SPANOECCGNXGNR-UITAMQMPSA-N", "diallate", "avadex")
  CALL Species("C_{10}H_{16}Cl_3NOS",                 "2303-17-5", "MWBPRDONLNQCFV-UHFFFAOYSA-N", "triallate", "")
  CALL Species("C_{10}H_{13}Cl_2FN_2O_2S_2",           "731-27-1", "HYVWIQDYBVKITD-UHFFFAOYSA-N", "tolylfluanid", "")
  CALL Species("C_{11}H_{10}ClN_3OS",               "223580-51-6", "VJQYLJSMBWXGDV-UHFFFAOYSA-N", "tiadinil", "")
  CALL Species("C_{11}H_{10}NO_3ClS",                "25059-80-7", "WQRCEBAZAUAUQC-UHFFFAOYSA-N", "benazolin-ethyl", "")
  CALL Species("C_{12}H_{16}ClNOS",                  "34622-58-7", "LLLFASISUZUJEQ-UHFFFAOYSA-N", "orbencarb", "")
  CALL Species("C_{12}H_{16}ClNOS",                  "28249-77-6", "QHTQREMOGMZHJV-UHFFFAOYSA-N", "S-(4-chlorobenzyl) diethylthiocarbamate", "thiobencarb")
  CALL Species("C_{12}H_{11}ClN_2O_5S",                 "54-31-9", "ZZUFCTLCJUWOSV-UHFFFAOYSA-N", "furosemide", "")
  CALL Species("C_{12}H_{12}ClN_5O_4S",              "64902-72-3", "VJYIFXVZLXQVHO-UHFFFAOYSA-N", "chlorsulfuron", "")
  CALL Species("C_{12}H_{15}ClNO_4S_2",               "2310-17-0", "IOUNQDKNJZEDEP-UHFFFAOYSA-N", "phosalone", "")
  CALL Species("C_{12}H_{18}ClNO_2S",                "87674-68-8", "JLYFCTQDENRSOL-UHFFFAOYSA-N", "dimethenamid", "")
  CALL Species("C_{12}H_{18}ClNO_2S",               "163515-14-8", "JLYFCTQDENRSOL-VIFPVBQESA-N", "dimethenamid-p", "")
  CALL Species("C_{13}H_{13}ClN_4O_2S",             "120116-88-3", "YXKMMRDKEKCERS-UHFFFAOYSA-N", "cyazofamid", "")
  CALL Species("C_{14}H_{13}Cl_2N_5O_4S",           "139528-85-1", "VGHPMIFEKOFHHQ-UHFFFAOYSA-N", "metosulam", "")
  CALL Species("C_{14}H_{15}Cl_2N_3OS",             "178928-70-6", "MNHVNIJQQRJYDH-UHFFFAOYSA-N", "prothioconazole", "")
  CALL Species("C_{14}H_{16}ClN_5O_5S",              "82097-50-5", "XOPFESVZMSQIKC-UHFFFAOYSA-N", "triasulfuron", "")
  CALL Species("C_{15}H_{15}ClN_4O_6S",              "90982-32-4", "NSWAMPCUPHPTTC-UHFFFAOYSA-N", "chlorimuron-ethyl", "")
  CALL Species("C_{16}H_{16}ClNO_2S",               "113665-84-2", "GKTWGGQPFAXNFI-HNNXBMFYSA-N", "clopidogrel", "")
  CALL Species("C_{16}H_{18}ClNO_2S",                "96491-05-3", "KDWQYMVPYJGPHS-UHFFFAOYSA-N", "thenylchlor", "")
  CALL Species("C_{17}H_{19}ClN_2S",                    "50-53-3", "ZPEIMTDSQAKGNT-UHFFFAOYSA-N", "chlorpromazine", "")
  CALL Species("C_{17}H_{21}N_2O_2ClS",              "78587-05-0", "XGWIJUOSCAQSSV-XHDPSFHLSA-N", "hexythiazox", "")
  CALL Species("C_{17}H_{26}ClNO_3S",                "99129-21-2", "INNPZTGYZSAJFN-ZTVUPKSFSA-N", "clethodim", "")
  CALL Species("C_{18}H_{33}N_2O_5ClS",              "18323-44-9", "KDLRVYVGXIQJDK-AWPVFWJPSA-N", "clindamycin", "")
  CALL Species("C_{19}H_{23}ClN_2O_2S",              "55512-33-9", "JTZCTMAVMHRNTR-UHFFFAOYSA-N", "pyridate", "")
  CALL Species("C_{19}H_{14}Cl_2N_2O_3S",           "879085-55-9", "BPQMGSKTAYIVFO-UHFFFAOYSA-N", "vismodegib", "")
  CALL Species("C_{19}H_{25}ClN_2OS",                "96489-71-3", "DWFZBUWUXWZWKD-UHFFFAOYSA-N", "pyridaben", "")
  ! C, H, O, F, Cl, S:
  CALL Species("C_{17}H_{16}ClF_3O_6S",             "335104-84-2", "IUQAXCIUEPFPSF-UHFFFAOYSA-N", "tembotrione", "")
  ! C, H, O, N, F, Cl, S:
  CALL Species("C_7H_5ClF_3NO_2S_2",                "318290-98-1", "XSNMWAPKHUGZGQ-UHFFFAOYSA-N", "fluensulfone", "")
  CALL Species("C_{10}H_{10}Cl_2F_2N_2OS",           "33439-45-1", "YFEUKKUPOVGUIW-UHFFFAOYSA-N", "fluothiuron", "")
  CALL Species("C_{11}H_{10}Cl_2F_2N_4O_3S",        "122836-35-5", "OORLZFUTLGXMEF-UHFFFAOYSA-N", "sulfentrazone", "")
  CALL Species("C_{12}H_7NO_2ClF_3S",                "72850-64-7", "MKQSWTQPLLCSOB-UHFFFAOYSA-N", "flurazole", "")
  CALL Species("C_{12}H_4Cl_2F_6N_4OS",             "120068-37-3", "ZOCSXAVNDGMNBV-UHFFFAOYSA-N", "fipronil", "")
  CALL Species("C_{13}H_7Cl_2F_3N_2O_4S",           "106917-52-6", "GNVDAZSPJWCIQZ-UHFFFAOYSA-N", "flusulfamide", "")
  CALL Species("C_{13}H_9Cl_2F_3N_4OS",             "181587-01-9", "FNELVJVBIYMIMC-UHFFFAOYSA-N", "ethiprole", "")
  CALL Species("C_{13}H_{10}Cl_2FN_5O_3S",          "145701-21-9", "QNXAVFXEJCPCJO-UHFFFAOYSA-N", "diclosulam", "")
  CALL Species("C_{15}H_{13}ClFN_5O_5S",            "147150-35-4", "BIKACRYIQSLICJ-UHFFFAOYSA-N", "cloransulam-methyl", "")
  CALL Species("C_{15}H_{15}ClFN_3O_3S_2",          "117337-19-6", "ZCNQYNHDVRPZIH-UHFFFAOYSA-N", "fluthiacet-methyl", "")
  CALL Species("C_{23}H_{18}ClF_2N_3O_3S",          "918504-65-1", "GPXBXXGIAQBQNI-UHFFFAOYSA-N", "vemurafenib", "")
  ! C, H, O, Br, S:
  CALL Species("C_{12}H_6Br_4O_4S",                  "39635-79-5", "JHJUYGMZIWDHMO-UHFFFAOYSA-N", "tetrabromobisphenol S", "")
  CALL Species("C_{18}H_{14}Br_8O_4S",               "42757-55-1", "CWZVMVIHYSYLSI-UHFFFAOYSA-N", "1,3-dibromo-5-[3,5-dibromo-4-(2,3-dibromopropoxy)benzenesulfonyl]-2-(2,3-dibromopropoxy)benzene", "")
  CALL Species("C_{19}H_{10}Br_4O_5S",                 "115-39-9", "UDSAIICHUKSCKT-UHFFFAOYSA-N", "bromphenol blue", "")
  CALL Species("C_{21}H_{14}Br_4O_5S",                  "76-60-8", "FRPHFZCDPYBUAU-UHFFFAOYSA-N", "bromcresol green", "")
  CALL Species("C_{21}H_{16}Br_2O_5S",                 "115-40-2", "ABIUHPWEYMSGSR-UHFFFAOYSA-N", "bromocresol purple", "")
  CALL Species("C_{31}H_{23}BrO_2S",                "104653-34-1", "VSVAQRUUFVBBFS-UHFFFAOYSA-N", "difethialone", "") ! another isomer: JHELOZJAKXYVBE-UHFFFAOYSA-N
  ! C, H, O, N, F, Br, S:
  CALL Species("C_{13}H_{13}BrFN_5O_4S_2",          "348635-87-0", "BREATYVWRHIPIY-UHFFFAOYSA-N", "amisulbrom", "")
  CALL Species("C_{13}H_6Br_2F_6N_2O_2S",           "130000-40-7", "WOSNCVAPUOFXEH-UHFFFAOYSA-N", "thifluzamide", "")
  ! C, H, O, I, S:
  CALL Species("C_8H_8I_2O_2S",                      "20018-09-1", "XOILGBPDXMVFIP-UHFFFAOYSA-N", "amical 48", "diiodomethyl $p$-tolyl sulfone")
  ! C, H, O, N, I, S:
  CALL Species("C_{23}H_{22}F_7IN_2O_4S",           "272451-65-7", "ZGNITFSDLCMLGI-UHFFFAOYSA-N", "flubendiamide", "")
  !***************************************************************************
  CALL SpeciesGroup(1,"Organic species with phosphorus (\chem{P})")
  !***************************************************************************
  CALL SpeciesGroup(2,"Phosphorus (\chem{C}, \chem{H}, \chem{O}, \chem{N}, \chem{Cl}, \chem{Br}, \chem{S}, \chem{P})")
  ! C, H, P:
  CALL Species("C_{28}H_{55}P",                      "13886-99-2", "UNOOEFGBOLKBFW-UHFFFAOYSA-N", "9-icosyl-9-phosphabicyclo[4.2.1]nonane", "")
  CALL Species("C_{18}H_{15}P",                        "603-35-0", "RIOQSEWOXXDEQQ-UHFFFAOYSA-N", "triphenylphosphine", "")
  ! C, H, O, P:
  CALL Species("C_3H_9O_4P",                           "512-56-1", "WVLBCYQITXONBZ-UHFFFAOYSA-N", "phosphoric acid, trimethyl ester", "trimethyl phosphate")
  CALL Species("C_3H_9O_3P",                           "121-45-9", "CYTQBVOFDCPGCX-UHFFFAOYSA-N", "trimethyl phosphite", "")
  CALL Species("C_3H_9O_3P",                           "756-79-6", "VONWDASPFIQPDY-UHFFFAOYSA-N", "dimethyl methylphosphonate", "")
  CALL Species("C_4H_{11}O_3P",                        "762-04-9", "MJUJXFBTEFXVKU-UHFFFAOYSA-N", "diethyl hydrogen phosphite", "")
  CALL Species("C_6H_{15}O_4P",                         "78-40-0", "DQWPFSLDHJDLRL-UHFFFAOYSA-N", "triethylphosphate", "")
  CALL Species("C_6H_{15}O_3P",                         "78-38-6", "AATNZNJRDOVKDD-UHFFFAOYSA-N", "diethyl ethyl phosphonate", "")
  CALL Species("C_7H_{13}O_6P",                       "7786-34-7", "GEPDYQSQVLXLEU-AATRIKPKSA-N", "mevinphos", "")
  CALL Species("C_7H_{17}O_3P",                       "1445-75-6", "WOAFDHWYKSOANX-UHFFFAOYSA-N", "diisopropyl methanephosphonate", "")
  CALL Species("C_8H_{19}O_3P",                       "1809-19-4", "NFJPGAKRJKLOJK-UHFFFAOYSA-N", "dibutyl hydrogen phosphite", "")
  CALL Species("C_8H_{19}O_4P",                        "107-66-4", "JYFHYPJRHGVZDY-UHFFFAOYSA-N", "dibutyl phosphate", "")
  CALL Species("C_8H_{20}O_7P_2",                      "107-49-3", "IDCBOTIENDVCBQ-UHFFFAOYSA-N", "tetraethyl pyrophosphate", "")
  CALL Species("C_9H_{21}O_4P",                        "513-08-6", "RXPQRKFMDQNODS-UHFFFAOYSA-N", "tripropyl phosphate", "")
  CALL Species("C_9H_{15}O_4P",                       "1623-19-4", "XHGIFBQQEGRTPB-UHFFFAOYSA-N", "triallyl phosphate", "")
  CALL Species("C_{12}H_{27}O_4P",                     "126-73-8", "STCOOQWBFONSKY-UHFFFAOYSA-N", "tributylphosphate", "")
  CALL Species("C_{12}H_{30}O_{13}P_4",                "757-58-4", "DAJYZXUXDOSMCG-UHFFFAOYSA-N", "hexaethyl tetraphosphate", "")
  CALL Species("C_{14}H_{19}O_6P",                    "7700-17-6", "XXXSILNSXNPGKG-ZHACJKMWSA-N", "crotoxyphos", "")
  CALL Species("C_{14}H_{23}O_4P",                    "2528-36-1", "YICSVBJRVMLQNS-UHFFFAOYSA-N", "phosphoric acid, dibutyl phenyl ester", "")
  CALL Species("C_{16}H_{35}O_3P",                    "3658-48-8", "HZIUHEQKVCPTAJ-UHFFFAOYSA-N", "bis(2-ethylhexyl) hydrogen phosphite", "")
  CALL Species("C_{16}H_{35}O_4P",                     "298-07-7", "SEGLCEQVOFDUPX-UHFFFAOYSA-N", "bis(2-ethylhexyl)hydrogen phosphate", "bis(2-ethylhexyl) phosphate")
  CALL Species("C_{18}H_{15}O_4P",                     "115-86-6", "XZZNDPSIHUTMOC-UHFFFAOYSA-N", "triphenyl phosphate", "")
  CALL Species("C_{18}H_{39}O_7P",                      "78-51-3", "WTLBZVNBAKMVDP-UHFFFAOYSA-N", "tris(2-butoxyethyl) phosphate", "")
  CALL Species("C_{19}H_{17}O_4P",                      "78-31-9", "OJUZRFGUKHQNJX-UHFFFAOYSA-N", "$p$-cresyl diphenyl phosphate", "")
  CALL Species("C_{18}H_{15}OP",                       "791-28-6", "FIQMHBFVRAXMOP-UHFFFAOYSA-N", "triphenylphosphine oxide", "")
  CALL Species("C_{18}H_{15}O_3P",                     "101-02-0", "HVLLSGMXQDNUAL-UHFFFAOYSA-N", "phosphorous acid, triphenyl ester", "")
  CALL Species("C_{18}H_{39}OP",                      "3084-48-8", "PPDZLUVUQQGIOJ-UHFFFAOYSA-N", "trihexylphosphine oxide", "")
  CALL Species("C_{20}H_{27}O_4P",                     "115-88-8", "YAFOVCNAQTZDQB-UHFFFAOYSA-N", "phosphoric acid, octyl diphenyl ester", "")
  CALL Species("C_{20}H_{43}OP",                     "31160-64-2", "XHRRUIJGMKIISX-UHFFFAOYSA-N", "octyldihexylphosphine oxide", "")
  CALL Species("C_{20}H_{27}O_4P",                    "1241-94-7", "CGSLYBDCEGBZCG-UHFFFAOYSA-N", "octicizer", "")
  CALL Species("C_{21}H_{21}O_4P",                    "1330-78-5", "IUJIYUAKFBGBCG-UHFFFAOYSA-N", "tris(methylphenyl) phosphate", "tricresyl phosphate")
  CALL Species("C_{21}H_{21}O_4P",                   "28108-99-8", "JJXNVYMIYBNZQX-UHFFFAOYSA-N", "phosphoric acid, (1-methylethyl)phenyl diphenyl ester", "")
  CALL Species("C_{21}H_{21}O_4P",                      "78-30-8", "YSMRWXYRXBRSND-UHFFFAOYSA-N", "phosphoric acid, tris(2-methylphenyl) ester", "tri-$o$-cresyl phosphate")
  CALL Species("C_{21}H_{21}O_4P",                     "563-04-2", "RMLPZKRPSQVRAB-UHFFFAOYSA-N", "phosphoric acid, tris(3-methylphenyl) ester", "tri-$m$-cresyl phosphate")
  CALL Species("C_{21}H_{21}O_4P",                      "78-32-0", "BOSMZFBHAYFUBJ-UHFFFAOYSA-N", "phosphoric acid, tris(4-methylphenyl) ester", "tri-$p$-cresyl phosphate")
  CALL Species("C_{22}H_{23}O_4P",                             "", "NIAVXHWAURFNOW-UHFFFAOYSA-N", "(3-{\it tert}-butylphenyl) diphenyl phosphate", "")
  CALL Species("C_{22}H_{23}O_4P",                     "981-40-8", "ULGAVXUJJBOWOD-UHFFFAOYSA-N", "(4-{\it tert}-butylphenyl) diphenyl phosphate", "")
  CALL Species("C_{22}H_{31}O_4P",                   "29761-21-5", "RYUJRXVZSJCHDZ-UHFFFAOYSA-N", "isodecyl diphenyl phosphate", "")
  CALL Species("C_{22}H_{47}OP",                     "31160-66-4", "MKEFGIKZZDCMQC-UHFFFAOYSA-N", "dioctylhexylphosphine oxide", "")
  CALL Species("C_{24}H_{27}O_4P",                    "3862-12-2", "KOWVWXQNQNCRRS-UHFFFAOYSA-N", "tris(2,4-dimethylphenyl)phosphate", "")
  CALL Species("C_{24}H_{27}O_4P",                   "19074-59-0", "MDHAARLWBHZGIP-UHFFFAOYSA-N", "tris(2,5-dimethylphenyl)phosphate", "")
  CALL Species("C_{24}H_{27}O_4P",                     "121-06-2", "QLORRTLBSJTMSN-UHFFFAOYSA-N", "tris(2,6-dimethylphenyl)phosphate", "")
  CALL Species("C_{24}H_{27}O_4P",                    "3862-11-1", "BCTKCHOESSAGCN-UHFFFAOYSA-N", "tris(3,4-dimethylphenyl)phosphate", "")
  CALL Species("C_{24}H_{27}O_4P",                   "25653-16-1", "LLPMAOBOEQFPRE-UHFFFAOYSA-N", "tris(3,5-dimethylphenyl)phosphate", "")
  CALL Species("C_{27}H_{33}O_4P",                   "26967-76-0", "ANVREEJNGJMLOV-UHFFFAOYSA-N", "tris(4-isopropylphenyl) phosphate", "")
  CALL Species("C_{24}H_{51}OP",                        "78-50-2", "ZMBHCYHQLYEYDV-UHFFFAOYSA-N", "trioctylphosphine oxide", "")
  CALL Species("C_{24}H_{51}O_3P",                     "126-63-6", "GOCVCBDBQYEFQD-UHFFFAOYSA-N", "bis(2-ethylhexyl)-2-ethylhexyl phosphonate", "")
  CALL Species("C_{24}H_{51}O_4P",                    "7057-92-3", "JTXUVYOABGUBMX-UHFFFAOYSA-N", "didodecyl hydrogen phosphate", "")
  CALL Species("C_{24}H_{51}O_4P",                      "78-42-2", "GTVWRXDRKAHEAD-UHFFFAOYSA-N", "phosphoric acid, tris(2-ethylhexyl) ester", "trioctyl phosphate")
  CALL Species("C_{26}H_{47}O_3P",                   "25550-98-5", "SXXILWLQSQDLDL-UHFFFAOYSA-N", "diisodecylphenyl phosphite", "")
  CALL Species("C_{27}H_{33}O_4P",                   "64532-97-4", "LMCLPMXCYFSRNG-UHFFFAOYSA-N", "4-nonylphenyl diphenyl phosphate", "")
  CALL Species("C_{30}H_{24}O_8P_2",                 "57583-54-7", "OWICEWMBIBPFAH-UHFFFAOYSA-N", "resorcinol bis(diphenyl phosphate)", "")
  CALL Species("C_{30}H_{24}O_8P_2",                 "51732-57-1", "RECLNCPBBUHRDY-UHFFFAOYSA-N", "hydroquinone bis(diphenyl phosphate)", "")
  CALL Species("C_{30}H_{39}O_4P",                      "78-33-1", "LORSVOJSXMHDHF-UHFFFAOYSA-N", "tris(4-{\it tert}-butylphenyl) phosphate", "")
  CALL Species("C_{42}H_{63}O_3P",                   "31570-04-4", "JKIJEFPNVSHHEI-UHFFFAOYSA-N", "tris-(2,4-di-{\it tert}-butylphenyl) phosphite", "")
  ! C, H, O, N, P:
  CALL Species("C_3H_8NO_5P",                         "1071-83-6", "XDDAORKBJWWYJS-UHFFFAOYSA-N", "glyphosate", "")
  CALL Species("C_3H_{11}N_2O_4P",                   "25954-13-6", "OTSAMNSACVKIOJ-UHFFFAOYSA-N", "krenite", "fosamine-ammonium")
  CALL Species("C_5H_{11}N_2O_2P",                      "77-81-6", "PJVJTCIRVMBVIA-UHFFFAOYSA-N", "tabun", "")
  CALL Species("C_5H_{15}N_2O_4P",                   "77182-82-2", "ZBMRKNMTMPPMMK-UHFFFAOYSA-N", "glufosinate-ammonium", "")
  CALL Species("C_6H_{18}N_3OP",                       "680-31-9", "GNOIPBMMFNIUFM-UHFFFAOYSA-N", "hexamethylphosphoramide", "")
  CALL Species("C_7H_{14}NO_5P",                      "6923-22-4", "KRTSDMXIXPKRQR-AATRIKPKSA-N", "monocrotophos", "")
  CALL Species("C_8H_{16}NO_5P",                       "141-66-2", "VEENJGZXVHKXNB-VOTSOKGWSA-N", "dicrotophos", "")
  CALL Species("C_8H_{24}N_4O_3P_2",                   "152-16-9", "SZKKRCSOSQAJDE-UHFFFAOYSA-N", "octamethyldiphosphoramide", "schradan")
  CALL Species("C_9H_{22}NO_5P",                      "2781-11-5", "CCJKFLLIJCGHMO-UHFFFAOYSA-N", "fyrol 6", "diethyl ((diethanolamino)methyl) phosphonate")
  CALL Species("C_{10}H_{14}NO_6P",                    "311-45-5", "WYMSBXTXOHUIGT-UHFFFAOYSA-N", "diethyl 4-nitrophenyl phosphate", "paraoxon")
  CALL Species("C_8H_{10}NO_6P",                       "950-35-6", "BAFQDKPJKOLXFZ-UHFFFAOYSA-N", "dimethyl 4-nitrophenyl phosphate", "methyl paraoxon")
  CALL Species("C_{18}H_{38}NO_3P",                  "51249-05-9", "NMBXMBCZBXUXAM-UHFFFAOYSA-N", "buminafos", "")
  ! C, H, O, F, P:
  CALL Species("CH_3F_2OP",                            "676-99-3", "PQIOSYKVBBWRRI-UHFFFAOYSA-N", "methylphosphonyldifluoride", "")
  CALL Species("C_4H_{10}FO_2P",                       "107-44-8", "DYAHQFWOVKZOOW-UHFFFAOYSA-N", "sarin", "")
  CALL Species("C_4H_{12}FN_2OP",                      "115-26-4", "PGJBQBDNXAZHBP-UHFFFAOYSA-N", "dimefox", "")
  CALL Species("C_6H_{14}FO_3P",                        "55-91-4", "MUCZHBLJLSDCSD-UHFFFAOYSA-N", "isoflurophate", "diisopropyl fluorophosphate")
  CALL Species("C_7H_{14}FO_2P",                       "329-99-7", "SNTRKUOVAPUGAY-UHFFFAOYSA-N", "cyclohexyl methylphosphonofluoridate", "cyclosarin")
  CALL Species("C_7H_{16}FO_2P",                        "96-64-0", "GRXKLBBBQUKJJZ-UHFFFAOYSA-N", "soman", "")
  ! C, H, O, N, F, P:
  CALL Species("C_6H_{16}FN_2OP",                      "371-86-8", "UOSHUBFBCPGQAY-UHFFFAOYSA-N", "mipafox", "")
  ! C, H, Cl, P:
  CALL Species("C_6H_5Cl_2P",                          "644-97-3", "IMDXZWRLUZPMDH-UHFFFAOYSA-N", "phenylphosphonous dichloride", "")
  CALL Species("C_{19}H_{32}Cl_3P",                    "115-78-6", "IVHVNMLJNASKHW-UHFFFAOYSA-M", "chlorphonium chloride", "")
  ! C, H, O, Cl, P:
  CALL Species("C_2H_4Cl_3O_4P",                       "306-52-5", "YYQRGCZGSFRBAM-UHFFFAOYSA-N", "triclofos", "")
  CALL Species("C_2H_6ClO_3P",                       "16672-87-0", "UDPGUMQDCGORJQ-UHFFFAOYSA-N", "(2-chloroethyl)-phosphonic acid", "ethephon")
  CALL Species("C_4H_8Cl_3O_4P",                        "52-68-6", "NFACJZMKEDPNKN-UHFFFAOYSA-N", "1-hydroxy-2,2,2-trichloroethylphosphonic acid, dimethyl ester", "trichlorfon")
  CALL Species("C_4H_7Cl_2O_4P",                        "62-73-7", "OEBRKCOSUFCWJD-UHFFFAOYSA-N", "dimethyl-2,2-dichlorovinyl phosphate", "dichlorvos")
  CALL Species("C_6H_{12}Cl_3O_4P",                    "115-96-8", "HQUQLFOMPYWACS-UHFFFAOYSA-N", "tris(2-chloroethyl) phosphate", "")
  CALL Species("C_7H_{15}Cl_2N_2O_2P",                  "50-18-0", "CMSMOCZEIVJLDB-UHFFFAOYSA-N", "cyclophosphamide", "")
  CALL Species("C_7H_{15}Cl_2N_2O_2P",                "3778-73-2", "HOMGKSMUEGBAAB-UHFFFAOYSA-N", "ifosfamide", "")
  CALL Species("C_8H_{14}Cl_3O_5P",                    "126-22-7", "BKAQXYNWONVOAX-UHFFFAOYSA-N", "butonate", "")
  CALL Species("C_9H_{12}ClO_4P",                    "23560-59-0", "GBAWQJNHVWMTLU-UHFFFAOYSA-N", "phosphoric acid, 7-chlorobicyclo[3.2.0]hepta-2,6-dien-6-yl dimethyl ester", "heptenophos")
  CALL Species("C_9H_{15}Cl_6O_4P",                     "78-43-3", "JZZBTMVTLBHJHL-UHFFFAOYSA-N", "tris(2,3-dichloropropyl) phosphate", "")
  CALL Species("C_9H_{15}Cl_6O_4P",                  "13674-87-8", "ASLWPAWFJZFCKF-UHFFFAOYSA-N", "tris(1,3-dichloroisopropyl)phosphate", "")
  CALL Species("C_9H_{18}Cl_3O_4P",                   "6145-73-9", "GTRSAMFYSUBAGN-UHFFFAOYSA-N", "tris(2-chloropropyl) phosphate", "")
  CALL Species("C_9H_{18}Cl_3O_4P",                  "13674-84-5", "KVMPUXDNESXNOH-UHFFFAOYSA-N", "tri-(2-chloroisopropyl)phosphate", "")
  CALL Species("C_9H_{18}Cl_3O_4P",                  "76649-15-5", "WDLBXPUJJVVRGX-UHFFFAOYSA-N", "bis(2-chloropropyl) 2-chloro-1-methylethyl phosphate", "")
  CALL Species("C_{10}H_9Cl_4O_4P",                  "22248-79-9", "UBCKGWBNUIFUST-YHYXMXQVSA-N", "tetrachlorvinphos", "") ! other CAS: 961-11-5
  CALL Species("C_{10}H_{10}Cl_3O_4P",                "2274-67-1", "QSGNQELHULIMSJ-UHFFFAOYSA-N", "dimethylvinphos", "")
  CALL Species("C_{12}H_{10}ClO_3P",                  "2524-64-3", "BHIIGRBMZRSDRI-UHFFFAOYSA-N", "diphenyl chlorophosphate", "")
  CALL Species("C_{12}H_{14}Cl_3O_4P",                 "470-90-6", "FSAVDKDHPDSCTO-XYOKQWHBSA-N", "chlorfenvinphos", "clofenvinfos")
  CALL Species("C_{12}H_{19}ClNO_3P",                  "299-86-5", "BOFHKBLZOYVHSI-UHFFFAOYSA-N", "crufomate", "ruelene")
  CALL Species("C_{14}H_{11}O_4Cl_4P",               "36519-00-3", "HEMINMLPKZELPP-UHFFFAOYSA-N", "phosdiphen", "")
  ! C, H, O, N, Cl, P:
  CALL Species("C_{10}H_{19}ClNO_5P",                "13171-21-6", "RGCLLPNLLBQHPF-HJWRWDBZSA-N", "phosphamidon", "")
  ! C, H, Br, P:
  CALL Species("C_9H_{15}Br_6O_4P",                    "126-72-7", "PQYJRMFWJJONBO-UHFFFAOYSA-N", "tris(2,3-dibromo-1-propyl) phosphate", "")
  ! C, H, Cl, Br, P:
  CALL Species("C_4H_7Br_2Cl_2O_4P",                   "300-76-5", "BUYMVQAILCEWRR-UHFFFAOYSA-N", "naled", "")
  ! C, H, O, Cl, Br, P:
  CALL Species("C_9H_{18}Br_2ClO_4P",               "125997-20-8", "GZSKSYDWLZIPOX-UHFFFAOYSA-N", "2-bromo-1,1-dimethylethyl 2-bromoethyl 2-chloroethyl phosphate", "")
  CALL Species("C_{13}H_{10}O_3BrCl_2P",             "21609-90-5", "CVRALZAYCYJELZ-UHFFFAOYSA-N", "leptophos", "")
  ! C, H, S, P:
  CALL Species("C_{12}H_{27}PS_3",                     "150-50-5", "KLAPGAOQRZTCBI-UHFFFAOYSA-N", "tributyl phosphorotrithioite", "")
  CALL Species("C_{40}H_{60}P_2S_5",                 "68400-79-3", "ZZMOHCWFHCHISQ-UHFFFAOYSA-N", "bis(2,6,6-trimethylbicyclo[3.1.1]hept-2-enyl) bis(2,6,6-trimethylbicyclo[3.1.1]hept-2-enyl)thiodiphosphonate", "")
  ! C, H, O, S, P:
  CALL Species("C_6H_{15}O_2PS_3",                     "640-15-3", "OPASCBHCTNRLRM-UHFFFAOYSA-N", "thiometon", "")
  CALL Species("C_6H_{15}O_5PS_2",                   "17040-19-6", "PZIRJMYRYORVIT-UHFFFAOYSA-N", "demeton-S-methyl sulfone", "")
  CALL Species("C_6H_{15}O_4PS_2",                     "301-12-2", "PMCVMORKVPSKHZ-UHFFFAOYSA-N", "oxydemeton-methyl", "")
  CALL Species("C_6H_{15}O_3PS_2",                     "867-27-6", "ZVZQKNVMDKSGGF-UHFFFAOYSA-N", "demeton-O-methyl", "")
  CALL Species("C_6H_{15}O_3PS_2",                     "919-86-8", "WEBQKRLKWNIYKK-UHFFFAOYSA-N", "demeton-S-methyl", "")
  CALL Species("C_7H_{13}O_5PS",                     "62610-77-9", "NTAHCMPOMKHKEU-AATRIKPKSA-N", "methacrifos", "")
  CALL Species("C_7H_{17}O_2PS_3",                     "298-02-2", "BULVZWIRKLYCBC-UHFFFAOYSA-N", "phorate", "")
  CALL Species("C_8H_9O_3PS",                         "3811-49-2", "OUNSASXJZHBGAI-UHFFFAOYSA-N", "salithion", "")
  CALL Species("C_8H_{17}O_5PS",                      "2425-25-4", "ZRCQYAQOWIQUBA-UHFFFAOYSA-N", "acetoxon", "")
  CALL Species("C_8H_{19}O_3PS_2",                     "298-03-3", "DGLIBALSRMUQDD-UHFFFAOYSA-N", "demeton-O", "")
  CALL Species("C_8H_{19}O_3PS_2",                     "126-75-0", "GRPRVIYRYGLIJU-UHFFFAOYSA-N", "demeton-S", "isosystox")
 !CALL Species("C_8H_{19}O_3PS_2",                     "298-03-3", "DGLIBALSRMUQDD-UHFFFAOYSA-N", "demeton", "")! mixture of demeton-O and demeton-S, not used here
  CALL Species("C_8H_{20}O_5P_2S_2",                  "3689-24-5", "XIUROWKZWPIAIB-UHFFFAOYSA-N", "sulfotep", "")
  CALL Species("C_8H_{24}O_{12}P_2S",                "55566-30-8", "YIEDHPBKGZGLIK-UHFFFAOYSA-L", "tetrakis(hydroxymethyl) phosphonium sulfate", "")
  CALL Species("C_8H_{19}O_2PS_2",                   "13194-48-4", "VJYFKVYYMZPMAB-UHFFFAOYSA-N", "ethoprophos", "")
  CALL Species("C_8H_{19}O_2PS_3",                     "298-04-4", "DOFZAZXDOSGAJZ-UHFFFAOYSA-N", "disulfoton", "")
  CALL Species("C_9H_{13}O_6PS",                      "2778-04-3", "YCAGGFXSFQFVQL-UHFFFAOYSA-N", "endothion", "")
  CALL Species("C_9H_{21}O_2PS_3",                   "13071-79-9", "XLNZEKHULJKQBA-UHFFFAOYSA-N", "terbufos", "")
  CALL Species("C_9H_{22}O_4P_2S_4",                   "563-12-2", "RIZMRRKBZQXFOY-UHFFFAOYSA-N", "ethion", "")
  CALL Species("C_{10}H_{15}OPS_2",                    "944-22-9", "KVGLBTYUCJYMND-UHFFFAOYSA-N", "fonofos", "")
  CALL Species("C_{10}H_{15}O_3PS_2",                   "55-38-9", "PNVJTZOFSHSLTO-UHFFFAOYSA-N", "fenthion", "")
  CALL Species("C_{10}H_{19}O_6PS_2",                  "121-75-5", "JXSJBGJIGXNWCI-UHFFFAOYSA-N", "malathion", "")
  CALL Species("C_{10}H_{19}O_7PS",                   "1634-78-2", "WSORODGWGUUOBO-UHFFFAOYSA-N", "malaoxon", "")
  CALL Species("C_{10}H_{23}O_2PS_2",                "95465-99-9", "KXRPCFINVWWFHQ-UHFFFAOYSA-N", "cadusafos", "")
  CALL Species("C_{11}H_{17}O_4PS_2",                  "115-90-2", "XDNBJTQLKCIJBV-UHFFFAOYSA-N", "fensulfothion", "")
  CALL Species("C_{12}H_{17}O_4PS_2",                 "2597-03-7", "XAMUDJHXFNRLCY-UHFFFAOYSA-N", "phenthoate", "")
  CALL Species("C_{12}H_{19}O_2PS_3",                "35400-43-2", "JXHJNEJVUNHLKO-UHFFFAOYSA-N", "sulprofos", "")
  CALL Species("C_{12}H_{27}OPS_3",                     "78-48-8", "ZOKXUAHZSKEQSS-UHFFFAOYSA-N", "S,S,S-tributyl phosphorotrithioate", "DEF")
  CALL Species("C_{12}H_{28}O_5P_2S_2",               "3244-90-4", "IIDFEIDMIKSJSV-UHFFFAOYSA-N", "aspon", "tetrapropyl dithiopyrophosphate")
  CALL Species("C_{13}H_{21}O_3PS",                  "26087-47-8", "FCOAHACKGGIURQ-UHFFFAOYSA-N", "iprobenphos", "")
  CALL Species("C_{13}H_{21}O_4PS",                   "7292-16-2", "PWYIUEFFPNVCMW-UHFFFAOYSA-N", "propaphos", "")
  CALL Species("C_{14}H_{15}O_2PS_2",                "17109-49-8", "AWZOLILCOUMRDG-UHFFFAOYSA-N", "edifenphos", "")
  CALL Species("C_{16}H_{38}O_6P_2S_4",               "8065-48-3", "FAXIJTUDSBIMHY-UHFFFAOYSA-N", "systox", "")
  CALL Species("C_{16}H_{20}O_6P_2S_3",               "3383-96-8", "WWJZWCUNLNYYAU-UHFFFAOYSA-N", "temefos", "temephos")
  ! C, H, O, N, S, P:
  CALL Species("C_2H_8NOPS_2",                       "10265-92-6", "NNKVPIKMPCQWCG-UHFFFAOYSA-N", "methamidophos", "")
  CALL Species("C_4H_{10}NO_3PS",                    "30560-19-1", "YASYVMFAVPKPKE-UHFFFAOYSA-N", "acephate", "")
  CALL Species("C_5H_{12}NO_3PS_2",                     "60-51-5", "MCWXGJITAZMZEV-UHFFFAOYSA-N", "dimethoate", "") ! other CAS: 56833-73-9, 79956-18-6
  CALL Species("C_5H_{12}NO_4PS",                     "1113-02-6", "PZXOQEXFMJCDPG-UHFFFAOYSA-N", "omethoate", "")
  CALL Species("C_6H_{11}N_2O_4PS_3",                  "950-37-8", "MEBQXILRKZHVCX-UHFFFAOYSA-N", "methidathion", "")
  CALL Species("C_6H_{12}NO_3PS_2",                  "21548-32-3", "RHJOIOVESMTJEK-UHFFFAOYSA-N", "fosthietan", "")
  CALL Species("C_6H_{12}NO_4PS_2",                   "2540-82-1", "AIKKULXCBHRFOS-UHFFFAOYSA-N", "formothion", "")
  CALL Species("C_6H_{12}N_5O_2PS_2",                   "78-57-9", "SUYHYHLFUHHVJQ-UHFFFAOYSA-N", "menazon", "")
  CALL Species("C_6H_{14}NO_3PS_2",                    "116-01-8", "DICRHEJCQXFJBY-UHFFFAOYSA-N", "ethoate-methyl", "")
  CALL Species("C_6H_{16}NO_5PS",                    "81591-81-3", "RUCAXVJJQQJZGU-UHFFFAOYSA-M", "glyphosate-trimesium", "")
  CALL Species("C_7H_{14}NO_3PS_2",                    "947-02-4", "ILBONRFSLATCRE-UHFFFAOYSA-N", "phosfolan", "cyolane")
  CALL Species("C_8H_{10}NO_5PS",                      "298-00-0", "RLBIQVVOMOPOHC-UHFFFAOYSA-N", "methylparathion", "parathion-methyl")
  CALL Species("C_8H_{13}N_2O_3PS",                    "297-97-2", "IRVDMKJLOCGUBJ-UHFFFAOYSA-N", "zinophos", "thionazin")
  CALL Species("C_8H_{18}NO_4PS_2",                   "2275-23-2", "LESVOLZBIFDZGS-UHFFFAOYSA-N", "vamidothion", "")
  CALL Species("C_9H_{10}NO_3PS",                     "2636-26-2", "SCKHCCSZFPSHGR-UHFFFAOYSA-N", "cyanophos", "")
  CALL Species("C_9H_{12}NO_5PS",                      "122-14-5", "ZNOLGFHPUIJIMJ-UHFFFAOYSA-N", "fenitrothion", "")
  CALL Species("C_9H_{18}NO_3PS_2",                  "98886-44-3", "DUFVKSUJRWYZQP-UHFFFAOYSA-N", "fosthiazate-1", "")
  CALL Species("C_9H_{20}NO_3PS_2",                   "2275-18-5", "QTXHFDHVLBDJIO-UHFFFAOYSA-N", "prothoate", "trimethoate")
  CALL Species("C_{10}H_{24}NO_3PS",                    "78-53-5", "PJISLFCKHOHLLP-UHFFFAOYSA-N", "amiton", "")
  CALL Species("C_{10}H_{12}N_3O_3PS_2",                "86-50-0", "CJJOSEISRRTUQB-UHFFFAOYSA-N", "azinphos-methyl", "")
  CALL Species("C_{10}H_{14}NO_5PS",                    "56-38-2", "LCCNCVORNKJIRZ-UHFFFAOYSA-N", "parathion", "E 605")
  CALL Species("C_{10}H_{14}NO_5PS",                   "597-88-6", "BGWJTLLALYACOG-UHFFFAOYSA-N", "isoparathion", "")
  CALL Species("C_{10}H_{17}N_2O_4PS",               "38260-54-7", "FGIWFCGDPUIBEZ-UHFFFAOYSA-N", "etrimfos", "")
  CALL Species("C_{10}H_{20}NO_4PS",                 "31218-83-4", "BZNDWPRGXNILMS-VQHVLOKHSA-N", "propetamphos", "")
  CALL Species("C_{10}H_{20}NO_5PS_2",                "2595-54-2", "KLGMSAOQDHLCOS-UHFFFAOYSA-N", "mecarbam", "")
  CALL Species("C_{11}H_{12}NO_4PS_2",                 "732-11-6", "LMNZTLDVJIUSHT-UHFFFAOYSA-N", "phosmet", "")
  CALL Species("C_{11}H_{20}N_3O_3PS",               "29232-93-7", "QHOQHJPRIBSPCY-UHFFFAOYSA-N", "pirimiphos methyl", "")
  CALL Species("C_{11}H_{21}N_4O_2PS",              "140163-89-9", "PPCUNNLZTNMXFO-UHFFFAOYSA-N", "imicyafos", "")
  CALL Species("C_{11}H_{26}NO_2PS",                 "50782-69-9", "JJIUCEJQJXNMHV-UHFFFAOYSA-N", "Agent VX", "")
  CALL Species("C_{12}H_{14}NO_4PS",                  "5131-24-8", "MTBZIGHNGSTDJV-UHFFFAOYSA-N", "ditalimfos", "")
  CALL Species("C_{12}H_{15}N_2O_3PS",               "13593-03-8", "JYQUHIFYBATCCY-UHFFFAOYSA-N", "quinalphos", "")
  CALL Species("C_{12}H_{15}N_2O_3PS",               "14816-18-3", "ATROHALUCMTWTB-UHFFFAOYSA-N", "phoxim", "")
  CALL Species("C_{12}H_{16}N_3O_3PS",               "24017-47-8", "AMFGTOFWMRQMEM-UHFFFAOYSA-N", "triazophos", "")
  CALL Species("C_{12}H_{16}N_3O_3PS_2",              "2642-71-9", "RQVGAIADHNPSME-UHFFFAOYSA-N", "azinphos-ethyl", "")
  CALL Species("C_{12}H_{21}N_2O_3PS",                 "333-41-5", "FHIVAFMUCKRCQO-UHFFFAOYSA-N", "diazinon", "dimpylate")
  CALL Species("C_{13}H_{16}NO_4PS",                 "18854-01-8", "SDMSCIWHRZJSRN-UHFFFAOYSA-N", "isoxathion", "")
  CALL Species("C_{13}H_{21}N_2O_4PS",               "36335-67-8", "OEYOMNZEMCPTKN-UHFFFAOYSA-N", "butamifos", "")
  CALL Species("C_{13}H_{22}NO_3PS",                 "22224-92-6", "ZCJPOPBZHLUFHF-UHFFFAOYSA-N", "fenamiphos", "")
  CALL Species("C_{13}H_{23}N_2O_3PS",               "96182-53-5", "AWYOMXWDGWUJHS-UHFFFAOYSA-N", "tebupirimfos", "")
  CALL Species("C_{13}H_{24}N_3O_3PS",               "23505-41-1", "TZBPRYIIJAJUOY-UHFFFAOYSA-N", "pirimiphos ethyl", "")
  CALL Species("C_{14}H_{24}NO_4PS_3",                 "741-58-2", "RRNIZKPFKNDSRS-UHFFFAOYSA-N", "bensulide", "")
  CALL Species("C_{14}H_{14}NO_4PS",                  "2104-64-5", "AIGRXSNSLVJMEA-UHFFFAOYSA-N", "ethyl $p$-nitrophenyl benzenethiophosphonate", "")
  CALL Species("C_{14}H_{17}N_2O_4PS",                 "119-12-0", "CXJSOEPQXUCJSA-UHFFFAOYSA-N", "pyridaphenthion", "")
  CALL Species("C_{14}H_{28}NO_3PS_2",               "24151-93-7", "UNLYSVIDNRIVFJ-UHFFFAOYSA-N", "piperophos", "")
  CALL Species("C_{15}H_{24}NO_4PS",                 "25311-71-1", "HOQADATXFBOEGG-UHFFFAOYSA-N", "isofenphos", "")
  ! C, H, O, Cl, S, P:
  CALL Species("C_5H_{12}ClO_2PS_2",                 "24934-91-6", "QGTYWWGEWOBMAK-UHFFFAOYSA-N", "chlormephos", "")
  CALL Species("C_6H_{11}Cl_4O_3PS",                 "54593-83-8", "XFDJMIHUAHSGKG-UHFFFAOYSA-N", "chlorethoxyfos", "")
  CALL Species("C_8H_8O_3Cl_3PS",                      "299-84-3", "JHJOOSLFWRRSGU-UHFFFAOYSA-N", "ronnel", "")
  CALL Species("C_9H_{11}Cl_2O_3PS",                 "57018-04-9", "OBZIQQJJIKNWNO-UHFFFAOYSA-N", "tolclofos-methyl", "")
  CALL Species("C_9H_{12}ClO_2PS_3",                   "953-17-3", "OUCCVXVYGFBXSV-UHFFFAOYSA-N", "methyl trithion", "")
  CALL Species("C_{10}H_{12}Cl_3O_2PS",                "327-98-0", "ANIAQSUBRGXWLS-UHFFFAOYSA-N", "trichloronate", "")
  CALL Species("C_{10}H_{13}Cl_2O_3PS",                 "97-17-6", "WGOWCPGHOCIHBW-UHFFFAOYSA-N", "dichlofenthion", "")
  CALL Species("C_{11}H_{15}Cl_2O_2PS_2",            "34643-46-4", "FITIWKDOCAUBQD-UHFFFAOYSA-N", "prothiofos", "")
  CALL Species("C_{11}H_{15}Cl_2O_3PS_2",            "21923-23-9", "JAZJVWLGNLCNDD-UHFFFAOYSA-N", "chlorthiophos", "")
  CALL Species("C_{11}H_{16}ClO_2PS_3",                "786-19-6", "VEDTXTNSFWUXGQ-UHFFFAOYSA-N", "carbophenothion", "")
  CALL Species("C_{14}H_{16}ClO_5PS",                   "56-72-4", "BXNANOICGRISHX-UHFFFAOYSA-N", "coumaphos", "")
  ! C, H, O, N, Cl, S, P:
  CALL Species("C_7H_7NO_3Cl_3PS",                    "5598-13-0", "HRBKVYFZANMGRE-UHFFFAOYSA-N", "methylchlorpyrifos", "")
  CALL Species("C_8H_9NO_5ClPS",                      "2463-84-5", "OTKXWJHPGBRXCR-UHFFFAOYSA-N", "dicapthon", "")
  CALL Species("C_8H_9ClNO_5PS",                       "500-28-7", "NZNRRXXETLSZRO-UHFFFAOYSA-N", "chlorthion", "")
  CALL Species("C_9H_{10}ClN_2O_5PS",                "35575-96-3", "VNKBTWQZTQIWDV-UHFFFAOYSA-N", "azamethiphos", "")
  CALL Species("C_9H_{17}ClN_3O_3PS",                "42509-80-8", "XRHGWAGWAHHFLF-UHFFFAOYSA-N", "isazophos", "") ! other CAS: 65357-77-9
  CALL Species("C_9H_{11}Cl_3NO_3PS",                 "2921-88-2", "SBPBAQFWLVIOKP-UHFFFAOYSA-N", "chlorpyrifos", "")
  CALL Species("C_{12}H_{14}ClN_2O_3PS",             "14816-20-7", "GQKRUMZWUHSLJF-NTCAYCPXSA-N", "chlorphoxim", "")
  CALL Species("C_{14}H_{11}Cl_2N_2O_4PS",            "4104-14-7", "XIBXUAZIZXDFTG-UHFFFAOYSA-N", "phosazetim", "")
  CALL Species("C_{14}H_{17}ClNO_4PS_2",             "10311-84-9", "MUMQYXACQUZOFP-UHFFFAOYSA-N", "dialifor", "")
  CALL Species("C_{14}H_{18}ClN_2O_3PS",             "77458-01-6", "QHGVXILFMXYDRS-UHFFFAOYSA-N", "pyraclofos", "")
  ! C, H, O, Cl, Br, S, P:
  CALL Species("C_8H_8BrCl_2O_3PS",                   "2104-96-3", "NYQDCVLCJXRDSK-UHFFFAOYSA-N", "bromophos", "")
  CALL Species("C_{10}H_{12}BrCl_2O_3PS",             "4824-78-6", "KWGUFOITWDSNQY-UHFFFAOYSA-N", "bromophos-ethyl", "")
  CALL Species("C_{11}H_{15}BrClO_3PS",              "41198-08-7", "QYMMJNLHFKGANY-UHFFFAOYSA-N", "profenofos", "")
  ! C, H, O, Cl, I, S, P:
  CALL Species("C_8H_8Cl_2IO_3PS",                   "18181-70-9", "LFVLUOAHQIVABZ-UHFFFAOYSA-N", "iodofenphos", "")
  !***************************************************************************
  CALL SpeciesGroup(1,"Organic species with other elements")
  !***************************************************************************
  CALL SpeciesGroup(2,"Sodium (\chem{Na})") ! 11
  CALL Species("C_8H_7Cl_2NaO_5S",                     "136-78-7", "KISFEBPWFCGRGN-UHFFFAOYSA-M", "sesone", "2,4-dichlorophenoxyethyl sulfate")
  CALL Species("C_{14}H_{14}IN_5O_6SNa",            "144550-36-7", "JUJFQMPKBJPSFZ-UHFFFAOYSA-M", "iodosulfuron-methyl-sodium", "")
  CALL Species("C_{15}H_{17}N_4O_7NaS",             "181274-15-7", "JRQGDDUXDKCWRF-UHFFFAOYSA-M", "propoxycarbazone-sodium", "")
  CALL Species("C_{20}H_{37}NaO_7S",                   "577-11-7", "APSBXTVYXVQYAB-UHFFFAOYSA-M", "dioctyl sulfosuccinatesodium salt", "bis(2-ethylhexyl) sodium sulfosuccinate")
  CALL Species("C_{18}H_{29}NaO_3S",                 "25155-30-0", "BWUAQXVVJWAXHR-UHFFFAOYSA-M", "dodecylbenzenesulfonic acid sodium salt", "sodium dodecylbenzenesulfonate")
  CALL Species("C_{20}H_{17}NO_8Na_2S_2",             "8004-92-0", "NYMFWSWVZMTZQO-UHFFFAOYSA-L", "D\&C yellow 10", "")
  CALL Species("C_{20}H_{10}Na_2O_5",                  "518-47-8", "NJDNXYGOVLYJHP-UHFFFAOYSA-L", "D\&C yellow 8", "fluorescein sodium")
  CALL Species("C_{22}H_{14}N_6Na_2O_9S_2",           "1064-48-8", "HKBVRFLHNUEVRO-DWTBGCDMSA-L", "D\&C black 1", "amido black 10B")
  CALL Species("C_{28}H_{20}N_2Na_2O_8S_2",           "4403-90-1", "FPAYXBWMYIMERV-UHFFFAOYSA-L", "D\&C green 5", "")
  CALL Species("C_{37}H_{34}N_2Na_2O_9S3",            "5141-20-8", "DGOBMKYRQHEFGQ-UHFFFAOYSA-L", "FD\&C green 2", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Aluminum (\chem{Al})") ! 13
  CALL Species("C_6H_{18}AlO_9P_3",                  "39148-24-8", "ZKZMJOFIHHZSRW-UHFFFAOYSA-K", "fosetyl-aluminum", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Silicon (\chem{Si})") ! 14
  ! H, Si:
  CALL Species("C_4H_{12}Si",                           "75-76-3", "CZDYPVPMEAXLPK-UHFFFAOYSA-N", "tetramethylsilane", "")
  CALL Species("C_8H_{20}Si",                          "631-36-7", "VCZQFJFZMMALHB-UHFFFAOYSA-N", "tetraethylsilane", "")
  ! O, Si:
  CALL Species("(CH_3)_3SiOH",                        "1066-40-6", "AAPLIUHOKVUFCC-UHFFFAOYSA-N", "trimethylsilanol", "TMS")
  CALL Species("Si(OH)_4",                           "10193-36-9", "RMAQACBXLXPBSY-UHFFFAOYSA-N", "silicic acid", "")
  CALL Species("C_2H_8O_2Si",                         "1066-42-8", "XCLIHDJZGPCUBT-UHFFFAOYSA-N", "dimethylsilanediol", "")
  CALL Species("C_4H_{12}O_4Si",                       "681-84-5", "LFQCEHFDDXELDD-UHFFFAOYSA-N", "tetramethyl silicate", "")
  CALL Species("C_5H_{16}O_2Si_2",                   "56428-93-4", "FGOLZCPMTWJPOU-UHFFFAOYSA-N", "pentamethyldisiloxanol", "")
  CALL Species("C_8H_{20}O_4Si",                        "78-10-4", "BOTDANWDWHJENH-UHFFFAOYSA-N", "tetraethyl silicate", "")
  CALL Species("C_{10}H_{20}O_5Si",                   "2530-85-0", "XDLMVUHYZWKMMD-UHFFFAOYSA-N", "trimethoxysilylpropyl methacrylate", "")
  CALL Species("C_6H_{18}OSi_2",                       "107-46-0", "UQEAIHBTYFGYIE-UHFFFAOYSA-N", "hexamethyldisiloxane", "L2")
  CALL Species("C_8H_{24}O_2Si_3",                     "107-51-7", "CXQXSVUQTKDNFP-UHFFFAOYSA-N", "octamethyltrisiloxane", "L3")
  CALL Species("C_{10}H_{30}O_3Si_4",                  "141-62-8", "YFCGDEUVHLPRCZ-UHFFFAOYSA-N", "decamethyltetrasiloxane", "L4")
  CALL Species("C_{12}H_{36}O_4Si_5",                  "141-63-9", "FBZANXDWQAVSTQ-UHFFFAOYSA-N", "dodecamethylpentasiloxane", "L5")
  CALL Species("C_{13}H_{21}NOSSi",                 "175217-20-6", "MXMXHPPIGKYTAR-UHFFFAOYSA-N", "silthiofam", "")
  CALL Species("C_{14}H_{42}O_5Si_6",                  "107-52-8", "ADANNTOYRVPQLJ-UHFFFAOYSA-N", "tetradecamethylhexasiloxane", "L6")
  CALL Species("C_{16}H_{48}O_6Si_7",                  "541-01-5", "NFVSFLUJRHRSJG-UHFFFAOYSA-N", "hexadecamethylheptasiloxane", "L7")
  CALL Species("C_{18}H_{54}O_7Si_8",                  "556-69-4", "VWGDPBQTSZDFMX-UHFFFAOYSA-N", "octadecamethyloctasiloxane", "L8")
  CALL Species("C_6H_{18}O_3Si_3",                     "541-05-9", "HTDJPCNNEPUOOQ-UHFFFAOYSA-N", "hexamethylcyclotrisiloxane", "D3")       ! "O1[Si](O[Si](O[Si]1(C)C)(C)C)(C)C"
  CALL Species("C_8H_{24}O_4Si_4",                     "556-67-2", "HMMGMWAXVFQUOA-UHFFFAOYSA-N", "octamethylcyclotetrasiloxane", "D4")     ! "[Si](O[Si](O[Si](O[Si]1(C)C)(C)C)(C)C)(C)C"
  CALL Species("C_{10}H_{30}O_5Si_5",                  "541-02-6", "XMSXQFUHVRWGNA-UHFFFAOYSA-N", "decamethylcyclopentasiloxane", "D5")     ! "O1[Si](O[Si](O[Si](O[Si](O[Si]1(C)C)(C)C)(C)C)(C)C)(C)C"
  CALL Species("C_{12}H_{36}O_6Si_6",                  "540-97-6", "IUMSDRXLFWAGNT-UHFFFAOYSA-N", "dodecamethylcyclohexasiloxane", "D6")
  CALL Species("C_4H_{14}O_3Si_2",                    "1118-15-6", "PFEAZKFNWPIFCV-UHFFFAOYSA-N", "tetramethyldisiloxane-1,3-diol", "")
  CALL Species("C_6H_{20}O_4Si_3",                    "3663-50-1", "XYBQTTAROZGWOZ-UHFFFAOYSA-N", "hexamethyltrisiloxane-1,5-diol", "")
  CALL Species("C_8H_{26}O_5Si_4",                    "3081-07-0", "VERNMKKMBJGSQB-UHFFFAOYSA-N", "octamethyltetrasiloxane-1,7-diol", "")
  CALL Species("C_5H_{16}O_4Si_3",                  "106916-50-1", "OGNYSZJXUXVWRE-UHFFFAOYSA-N", "pentamethylcyclotrisiloxanol", "D3OH")   ! "O[Si]1(O[Si](O[Si](O1)(C)C)(C)C)C"
  CALL Species("C_7H_{22}O_5Si_4",                    "5290-02-8", "MLGUQSFCJNMKPT-UHFFFAOYSA-N", "heptamethylcyclotetrasiloxanol", "D4OH") ! "O[Si]1(O[Si](O[Si](O[Si](O1)(C)C)(C)C)(C)C)C"
  CALL Species("C_9H_{28}O_6Si_5",                    "5290-04-0", "DKTKKERLEMHMHS-UHFFFAOYSA-N", "nonamethylcyclopentasiloxanol", "D5OH")  ! "O[Si]1(O[Si](O[Si](O[Si](O[Si](O1)(C)C)(C)C)(C)C)(C)C)C"
  CALL Species("C_6H_{16}O_5Si_3",                             "", "TZIXLALHTGGYNZ-UHFFFAOYSA-N", "D3FormEst", "")                          ! "O=CO[Si]1(O[Si](O[Si](O1)(C)C)(C)C)C"
  CALL Species("C_5H_{14}O_6Si_3",                             "", "RMJPVGNUQFKGPC-UHFFFAOYSA-N", "D3OHFormEst", "")                        ! "O=CO[Si]1(O[Si](O)(O[Si](O1)(C)C)C)C"
  CALL Species("C_6H_{18}O_5Si_3",                             "", "JKDUHABXPCJOJB-UHFFFAOYSA-N", "D3Hydroperoxide", "")                    ! "OOC[Si]1(O[Si](O[Si](O1)(C)C)(C)C)C"
  CALL Species("C_6H_{18}O_6Si_3",                             "", "LHGAYMWHCKTFJV-UHFFFAOYSA-N", "D3EtherHydroperoxide", "")               ! "OOCO[Si]1(O[Si](O[Si](O1)(C)C)(C)C)C"
  CALL Species("C_6H_{14}O_7Si_3",                             "", "WKTLUKWDJJZJCN-UHFFFAOYSA-N", "D3FormEst2", "")                         ! "O=CO[Si]1(O[Si](OC=O)(O[Si](O1)(C)C)C)C"
  CALL Species("C_4H_{14}O_5Si_3",                             "", "UGEQNRIYBONGCP-UHFFFAOYSA-N", "D3OH2O1", "")                            ! "O[Si]1(O[Si](O)(O[Si](O1)(C)C)C)C"
  CALL Species("C_8H_{22}O_6Si_4",                             "", "PWVQOGDRWKITJQ-UHFFFAOYSA-N", "D4FormEst", "")                          ! "O=CO[Si]1(O[Si](O[Si](O[Si](O1)(C)C)(C)C)(C)C)C"
  CALL Species("C_7H_{20}O_7Si_4",                             "", "AXBVNNKTZSENOJ-UHFFFAOYSA-N", "D4OHFormEst", "")                        ! "O=CO[Si]1(O[Si](O)(O[Si](O[Si](O1)(C)C)(C)C)C)C"
  CALL Species("C_7H_{22}O_2Si_3",                    "1873-88-7", "QNWOFLWXQGHSRH-UHFFFAOYSA-N", "1,1,1,3,5,5,5-heptamethyltrisiloxane", "")
  CALL Species("C_8H_{24}O_6Si_4",                             "", "IEOAKTNTPADFJB-UHFFFAOYSA-N", "D4Hydroperoxide", "")                    ! "OOC[Si]1(O[Si](O[Si](O[Si](O1)(C)C)(C)C)(C)C)C"
  CALL Species("C_8H_{24}O_7Si_4",                             "", "BKBHFCQPJIYNBR-UHFFFAOYSA-N", "D4EtherHydroperoxide", "")               ! "OOCO[Si]1(O[Si](O[Si](O[Si](O1)(C)C)(C)C)(C)C)C"
  CALL Species("C_8H_{20}O_8Si_4",                             "", "WJOJQLFAIHHNFX-UHFFFAOYSA-N", "D4FormEst2", "")                         ! "O=CO[Si]1(O[Si](OC=O)(O[Si](O[Si](O1)(C)C)(C)C)C)C"
  CALL Species("C_6H_{20}O_6Si_4",                             "", "ONAQLRRZBYFKLA-UHFFFAOYSA-N", "D4OH2", "")                              ! "O[Si]1(O[Si](O)(O[Si](O[Si](O1)(C)C)(C)C)C)C"
  CALL Species("C_{10}H_{28}O_7Si_5",                          "", "IGERVNZEGVJPED-UHFFFAOYSA-N", "D5FormEst", "")                          ! "O=CO[Si]1(O[Si](O[Si](O[Si](O[Si](O1)(C)C)(C)C)(C)C)(C)C)C"
  CALL Species("C_9H_{20}O_5Si",                      "2530-83-8", "BPSIOYPQMFLKFR-UHFFFAOYSA-N", "glycidoxypropyltrimethoxysilane", "")
  CALL Species("C_9H_{26}O_8Si_5",                             "", "KNIXZSAQQHGMJX-UHFFFAOYSA-N", "D5OHFormEst", "")                        ! "O=CO[Si]1(O[Si](O)(O[Si](O[Si](O[Si](O1)(C)C)(C)C)(C)C)C)C"
  CALL Species("C_{10}H_{30}O_7Si_5",                          "", "VGMLSGIAATVHBV-UHFFFAOYSA-N", "D5Hydroperoxide", "")                    ! "OOC[Si]1(O[Si](O[Si](O[Si](O[Si](O1)(C)C)(C)C)(C)C)(C)C)C"
  CALL Species("C_{10}H_{30}O_8Si_5",                          "", "GSAPDUDDTPAQSH-UHFFFAOYSA-N", "D5EtherHydroperoxide", "")               ! "OOCO[Si]1(O[Si](O[Si](O[Si](O[Si](O1)(C)C)(C)C)(C)C)(C)C)C"
  CALL Species("C_{10}H_{26}O_9Si_5",                          "", "QFORSIVEJZTNKQ-UHFFFAOYSA-N", "D5FormEst2", "")                         ! "O=CO[Si]1(O[Si](OC=O)(O[Si](O[Si](O[Si](O1)(C)C)(C)C)(C)C)C)C"
  CALL Species("C_{10}H_{24}O_3Si",                  "17980-47-1", "ALVYUZIFSCKIFP-UHFFFAOYSA-N", "isobutyltriethoxysilane", "")
  CALL Species("C_{10}H_{30}O_3Si_4",                "17928-28-8", "RGMZNZABJYWAEC-UHFFFAOYSA-N", "methyltris(trimethylsiloxy)silane", "")
  CALL Species("C_{11}H_{24}O_6Si",                   "1067-53-4", "WOXXJEVNDJOOLV-UHFFFAOYSA-N", "vinyltris(2-methoxyethoxy)silane", "")
  CALL Species("C_{12}H_{36}O_4Si_5",                 "3555-47-3", "VNRWTCZXQWOWIG-UHFFFAOYSA-N", "tetrakis(trimethylsiloxy)silane", "")
  CALL Species("C_{14}H_{28}O_4Si",                  "10217-34-2", "UDUKMRHNZZLJRB-UHFFFAOYSA-N", "2-(3,4-epoxycyclohexyl)\+ethyltriethoxysilane", "")
  CALL Species("C_8H_{26}O_7Si_5",                             "", "AGIASGIZCMBNMT-UHFFFAOYSA-N", "D5OH2", "")                              ! "O[Si]1(O[Si](O)(O[Si](O[Si](O[Si](O1)(C)C)(C)C)(C)C)C)C"
  ! N, Si:
  CALL Species("C_6H_{19}NSi_2",                       "999-97-3", "FFUAGWLWBBFQJT-UHFFFAOYSA-N", "hexamethyldisilazane", "")
  ! O, N, Si:
  CALL Species("C_6H_{17}NO_6Si_3",                            "", "WGZJMUDCTAXYSF-UHFFFAOYSA-N", "D3Organonitrate", "")      ! "O=[N+]([O-])OC[Si]1(O[Si](O[Si](O1)(C)C)(C)C)C"
  CALL Species("C_6H_{17}NO_7Si_3",                            "", "CJWFNCZCCFYEKP-UHFFFAOYSA-N", "D3EtherOrganonitrate", "") ! "O=[N+]([O-])OCO[Si]1(O[Si](O[Si](O1)(C)C)(C)C)C"
  CALL Species("C_8H_{23}NO_7Si_4",                            "", "CYMCLGYPZZKOOI-UHFFFAOYSA-N", "D4Organonitrate", "")      ! "O=[N+]([O-])OC[Si]1(O[Si](O[Si](O[Si](O1)(C)C)(C)C)(C)C)C"
  CALL Species("C_8H_{23}NO_8Si_4",                            "", "XHFUWVKWVPBCDO-UHFFFAOYSA-N", "D4EtherOrganonitrate", "") ! "O=[N+]([O-])OCO[Si]1(O[Si](O[Si](O[Si](O1)(C)C)(C)C)(C)C)C"
  CALL Species("C_9H_{23}NO_3Si",                      "919-30-2", "WYTZZXDRDKSJID-UHFFFAOYSA-N", "(3-aminopropyl)triethoxysilane", "")
  CALL Species("C_{10}H_{29}NO_8Si_5",                         "", "KWHVDACCYNMOAR-UHFFFAOYSA-N", "D5Organonitrate", "")      ! "O=[N+]([O-])OC[Si]1(O[Si](O[Si](O[Si](O[Si](O1)(C)C)(C)C)(C)C)(C)C)C"
  CALL Species("C_{10}H_{29}NO_9Si_5",                         "", "BQINUVFPZWRRBW-UHFFFAOYSA-N", "D5EtherOrganonitrate", "") ! "O=[N+]([O-])OCO[Si]1(O[Si](O[Si](O[Si](O[Si](O1)(C)C)(C)C)(C)C)(C)C)C"
  ! F, Si:
  CALL Species("C_{25}H_{29}FO_2Si",                "105024-66-6", "HPYNBECUCCGGPA-UHFFFAOYSA-N", "silafluofen", "")
  ! N, F, Si:
  CALL Species("C_{16}H_{15}F_2N_3Si",               "85509-19-9", "FQKUGOMFVDPBIZ-UHFFFAOYSA-N", "flusilazole", "")
  CALL Species("C_{14}H_{20}FN_3OSi",               "149508-90-7", "YABFPHSQTSFWQB-UHFFFAOYSA-N", "simeconazole", "")
  ! Cl, Si:
  CALL Species("CH_4Cl_2Si",                            "75-54-7", "NWKBSEBOBPHMKL-UHFFFAOYSA-N", "dichloromethylsilane", "methyldichlorosilane")
  ! O, Cl, Si:
  CALL Species("C_{11}H_{25}O_6ClSi",                "37894-46-5", "SLZWEMYSYKOWCG-UHFFFAOYSA-N", "etacelasil", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Calcium (\chem{Ca})") ! 20
  CALL Species("C_{20}H_{22}O_{10}Ca",              "127277-53-6", "VQIYODJWCVPJRC-UHFFFAOYSA-N", "prohexadione-calcium", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Zinc (\chem{Zn})") ! 30
  CALL Species("C_4H_6N_2S_4Zn",                     "12122-67-7", "AMHNZOICSMBGDH-UHFFFAOYSA-L", "zineb", "")
  CALL Species("C_6H_{12}N_2S_4Zn",                    "137-30-4", "DUBNHZYBDBBJHD-UHFFFAOYSA-L", "ziram", "")
  CALL Species("C_8H_{12}MnN_4S_8Zn",                 "8018-01-7", "CHNQZRKUZPNOOH-UHFFFAOYSA-J", "mancozeb", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Arsenic (\chem{Sn})") ! 33
  CALL Species("CH_5As",                               "593-52-2", "IDDBICIFODFKQO-UHFFFAOYSA-N", "methylarsine", "")
  CALL Species("C_2H_7As",                             "593-59-9", "OSAGMAAQMRQLLW-UHFFFAOYSA-N", "ethylarsine", "")
  CALL Species("C_4H_{11}As",                          "692-42-2", "JZCIYTSNUPIOMK-UHFFFAOYSA-N", "diethyl arsine", "")
  CALL Species("C_6H_5AsO",                            "637-03-6", "BQVCCPGCDUSGOE-UHFFFAOYSA-N", "phenylarsine oxide", "")
  CALL Species("C_{13}H_{10}AsN",                    "23525-22-6", "BDHNJKLLVSRGDK-UHFFFAOYSA-N", "diphenylarsanylformonitrile", "")
  CALL Species("CH_3AsCl_2",                           "593-89-5", "VXRMBBLRHSRVDK-UHFFFAOYSA-N", "dichloro(methyl)arsane", "")
  CALL Species("C_2H_2AsCl_3",                         "541-25-3", "GIKLTQKNOXNBNY-OWOJBTEDSA-N", "lewisite", "")
  CALL Species("C_{12}H_{10}AsCl",                     "712-48-1", "YHHKGKCOLGRKKB-UHFFFAOYSA-N", "diphenylchloroarsine", "")
  CALL Species("C_2H_2AsClO",                         "3088-37-7", "MVCVAGFCWDFQQX-OWOJBTEDSA-N", "lewisite oxide", "")
  CALL Species("C_6H_5AsCl_2",                         "696-28-6", "UDHDFEGCOJAVRE-UHFFFAOYSA-N", "phenyldichloroarsine", "")
  CALL Species("C_{12}H_9AsClN",                       "578-94-9", "PBNSPNYJYOYWTA-UHFFFAOYSA-N", "adamsite", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Selenium (\chem{Se})") ! 34
  CALL Species("C_2H_6Se",                             "593-79-3", "RVIXKDRPFPUUOO-UHFFFAOYSA-N", "dimethyl selenide", "")
  CALL Species("C_2H_6Se_2",                          "7101-31-7", "VLXBWPOEOIIREY-UHFFFAOYSA-N", "dimethyl diselenide", "")
  CALL Species("C_5H_{11}NO_2Se",                     "1464-42-2", "RJFAYQIBOAGBLC-UHFFFAOYSA-N", "2-amino-4-(methylselenyl)butyric acid", "selenium methionine")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Tin (\chem{Sn})") ! 50
  CALL Species("C_4H_{12}Sn",                          "594-27-4", "VXKWYPOMXBVZSJ-UHFFFAOYSA-N", "tetramethylstannane", "tetramethyltin")
  CALL Species("C_8H_{20}Sn",                          "597-64-8", "RWWNQEOPUOCKGR-UHFFFAOYSA-N", "tetraethylstannane", "tetraethyltin")
  CALL Species("C_{16}H_{36}Sn",                      "1461-25-2", "AFCAKJKUYFLYFK-UHFFFAOYSA-N", "tetrabutylstannane", "tetra-butyl tin")
  CALL Species("C_{18}H_{16}OSn",                       "76-87-9", "BFWMWWXRWVJXSE-UHFFFAOYSA-M", "triphenyltin hydroxide", "")
  CALL Species("C_{24}H_{54}OSn_2",                     "56-35-9", "APQHKWPGGHMYKJ-UHFFFAOYSA-N", "hexabutyldistannoxane", "bis(tributyltin)oxide")
  CALL Species("C_{60}H_{78}OSn_2",                  "13356-08-6", "HOXINJBQVZWYGZ-UHFFFAOYSA-N", "hexakis(2-methyl-2-phenylpropyl)distannoxane", "fenbutatin oxide")
  CALL Species("C_{20}H_{18}O_2Sn",                    "900-95-8", "WDQNIWFZKXZFAY-UHFFFAOYSA-M", "fentin acetate", "")
  CALL Species("C_{20}H_{35}N_3Sn",                  "41083-11-8", "ONHBDDJJTDTLIR-UHFFFAOYSA-N", "1-(tricyclohexylstannyl)1H-1,2,4-triazole", "azocyclotin")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Mercury (\chem{Hg})") ! 80
  ! C, H, Hg:
  CALL Species("C_2H_6Hg",                             "593-74-8", "ATZBPOVXVPIOMR-UHFFFAOYSA-N", "dimethylmercury", "")
  CALL Species("C_4H_{10}Hg",                          "627-44-1", "SPIUPAOJDZNUJH-UHFFFAOYSA-N", "diethylmercury", "")
  CALL Species("C_6H_{14}Hg",                          "628-85-3", "UZTYYBPPVOXULF-UHFFFAOYSA-N", "dipropylmercury", "")
  CALL Species("C_6H_{14}Hg",                         "1071-39-2", "UVUGOJQWNVFTRT-UHFFFAOYSA-N", "diisopropylmercury", "")
  CALL Species("C_8H_{18}Hg",                          "629-35-6", "CCYKQVBIPYDCKS-UHFFFAOYSA-N", "dibutylmercury", "")
  CALL Species("C_{12}H_{10}Hg",                       "587-85-9", "HWMTUNCVVYPZHZ-UHFFFAOYSA-N", "diphenylmercury", "")
  ! C, H, O, Hg:
  CALL Species("CH_3HgOH",                            "1184-57-2", "KRZWEBVPFGCYMY-UHFFFAOYSA-M", "hydroxymethylmercury", "")
  CALL Species("C_8H_8HgO_2",                           "62-38-4", "XEBWQGVWTUSTLN-UHFFFAOYSA-M", "phenyl mercuric ethanoate", "")
  ! C, H, N, Hg:
  CALL Species("C_3H_6N_4Hg",                          "502-39-6", "JVJUWCMBRUMDDQ-UHFFFAOYSA-N", "(3-cyanoguanidino)methylmercury", "methylmercuric dicyanamide")
  ! C, H, Cl, Hg:
  CALL Species("CH_3HgCl",                             "115-09-3", "BABMCXWQNSQAOC-UHFFFAOYSA-M", "chloromethylmercury", "")
  CALL Species("C_2H_5HgCl",                           "107-27-7", "QWUGXIXRFGEYBD-UHFFFAOYSA-M", "chloroethylmercury", "")
  CALL Species("C_3H_7HgCl",                          "2440-40-6", "ZLAYJSKLDWSALK-UHFFFAOYSA-M", "chloropropylmercury", "")
  CALL Species("C_3H_7HgCl",                         "30615-19-1", "YOKZNQICWABELX-UHFFFAOYSA-M", "chloroisopropylmercury", "")
  CALL Species("C_4H_9HgCl",                           "543-63-5", "OKPMTXZRMGMMOO-UHFFFAOYSA-M", "chlorobutylmercury", "")
  CALL Species("C_5H_{11}HgCl",                        "544-15-0", "UHFZINPMKCNPQL-UHFFFAOYSA-M", "chloropentylmercury", "")
  CALL Species("C_6H_5HgCl",                           "100-56-1", "AWGTVRDHKJQFAX-UHFFFAOYSA-M", "chlorophenylmercury", "")
  ! C, H, O, Cl, Hg:
  CALL Species("CH_3OC_2H_4HgCl",                      "123-88-6", "VJTAZCKMHINUKO-UHFFFAOYSA-M", "2-methoxyethylmercury chloride", "aretan")
  ! C, H, Br, Hg:
  CALL Species("CH_3HgBr",                             "506-83-2", "ZDHHIJSLJCLMPX-UHFFFAOYSA-M", "bromomethylmercury", "")
  CALL Species("C_2H_5HgBr",                           "107-26-6", "UREACWLAXSOUKG-UHFFFAOYSA-M", "bromoethylmercury", "")
  CALL Species("C_6H_5HgBr",                          "1192-89-8", "PUPHNPSAIJQNEE-UHFFFAOYSA-M", "bromophenylmercury", "")
  ! C, H, I, Hg:
  CALL Species("CH_3HgI",                              "143-36-2", "JVDIOYBHEYUIBM-UHFFFAOYSA-M", "iodomethylmercury", "")
  CALL Species("C_2H_5HgI",                           "2440-42-8", "BYIGJUQTUPMNMF-UHFFFAOYSA-M", "iodoethylmercury", "")
  CALL Species("C_6H_5HgI",                            "823-04-1", "BISBXZWWFIOZSX-UHFFFAOYSA-M", "iodophenylmercury", "")
  !---------------------------------------------------------------------------
  CALL SpeciesGroup(2,"Lead (\chem{Pb})") ! 82
  CALL Species("C_4H_{12}Pb",                           "75-74-1", "XOOGZRUBTYCLHG-UHFFFAOYSA-N", "tetramethyl lead", "")
  CALL Species("C_5H_{14}Pb",                         "1762-26-1", "KHQJREYATBQBHY-UHFFFAOYSA-N", "ethyltrimethylplumbane", "")
  CALL Species("C_6H_{16}Pb",                         "1762-27-2", "OLOAJSHVLXNSQV-UHFFFAOYSA-N", "diethyldimethylplumbane", "diethyldimethyl lead")
  CALL Species("C_7H_{18}Pb",                         "1762-28-3", "KGFRUGHBHNUHOS-UHFFFAOYSA-N", "triethylmethylplumbane", "methyltriethyl lead")
  CALL Species("C_8H_{20}Pb",                           "78-00-2", "MRMOZBOQVYRSEM-UHFFFAOYSA-N", "tetraethyl lead", "")
  CALL Species("C_3H_9ClPb",                          "1520-78-1", "HPQRSQFZILKRDH-UHFFFAOYSA-M", "trimethyl lead chloride", "")
  CALL Species("C_6H_{15}ClPb",                       "1067-14-7", "UQWYUMLFPUILRT-UHFFFAOYSA-M", "triethyl lead chloride", "")
  !---------------------------------------------------------------------------

  ! check which dummy CAS have already been used:
  ! gr _cas henry.f90 | sort

  ! Typesetting of names:
  ! see also: https://www.acdlabs.com/iupac/nomenclature/93/r93_74.htm
  ! - cis, trans, sec, and tert are written in italics with \it:
  !   {\it cis} {\it trans}
  ! - o, m, and p (only in trivial name) are in math mode: $o$-, $m$-, $p$-
  ! - ($Z$) and ($E$) in round parentheses
  ! - ($R$) and ($S$) in round parentheses
  ! - enantiomers $D$ and $L$
  ! - element names are uppercase and upright, e.g.: N-methylaniline
  ! CALL Species("formula", "casrn", "inchikey", "iupacname", "trivialname")
  ! CALL Species("formula", "casrn", "inchikey", "iupacname", "triv1; triv2")

  ! - show structure:
  !   grep -iE "^ *!|SpeciesGroup" henry.f90 > ~/tmp/speciesgroups.f90

  ! - convert formula to LaTeX syntax:
  !   - first, all numbers in brackets:
  !     (query-replace-regexp "\\([chonfis]\\|br\\|cl\\)\\([0-9]+\\)" (quote (replace-eval-replacement concat (replace-quote (capitalize (match-string 1))) "_{\\2}")))
  !   - then, remove brackets for 1-digit numbers:
  !     (query-replace-regexp "{\\([0-9]\\)}" "\\1")

  ! CALL Species("",                                             "", "",                            "", "")

  !---------------------------------------------------------------------------
  CALL Finalize
  !---------------------------------------------------------------------------

END PROGRAM Henry
!*****************************************************************************
!                                  end of file
!*****************************************************************************
