!*****************************************************************************
!                   Time-stamp: <2023-09-26 13:24:54 sander>
!      Author: Rolf Sander, Max-Planck Institute Mainz, Germany
!*****************************************************************************

! This program is free software: you can redistribute it and/or modify
! it under the terms of the GNU General Public License as published by
! the Free Software Foundation, either version 3 of the License, or (at
! your option) any later version. This program is distributed in the
! hope that it will be useful, but WITHOUT ANY WARRANTY; without even
! the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
! PURPOSE. See the GNU General Public License for more details. You
! should have received a copy of the GNU General Public License along
! with this program. If not, see: https://www.gnu.org/licenses/

!*****************************************************************************

MODULE henry_ref4000

  USE henry_mem
  USE henry_util
  IMPLICIT NONE

CONTAINS

  !---------------------------------------------------------------------------

  SUBROUTINE ref3506 ! Hscp [M/atm]
    IMPLICIT NONE

    ref = "3506"

    ! Tab. 2:
    CALL CalcH("fenpropidin",  "67306-00-7", &
      (/ 283.15, 283.25, 288.05, 288.15, 293.15, 293.15, 293.15, 293.15, 293.15 /), &
      (/ 20.8, 18.7, 15.7, 20.2, 8.8, 8.1, 11.8, 10.0, 11.1 /) )
    CALL CalcH("pyrimethanil", "53112-28-0", &
      (/ 278.25, 283.15, 283.25, 287.95, 288.15, 292.95, 293.15, 293.15, 293.15, 293.15 /), &
      (/ 131., 34.2, 46.0, 19.8, 24.0, 7.3, 8.3, 8.3, 8.5, 7.7 /) )

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp_, HLC)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp_, HLC
      chem=chem_ ; casrn=casrn_ ! make global for Output
      type = "M"
      ndata = SIZE(HLC)
      ALLOCATE(Harray(ndata))
      Harray = Hscp_TO_HscpSI * 1E4 * HLC
      CALL HTdep(temp_, Harray, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
      DEALLOCATE(Harray)
      IF (casrn_=="67306-00-7") THEN
        CALL SettypeX("tomlin98")
        CALL MakeNoteOtherTempK(293)
      CALL Output(Hscp_TO_HscpSI * 0.01E3)
      ENDIF
      IF (casrn_=="53112-28-0") THEN
        CALL SettypeX("tomlin98")
        CALL MakeNoteOtherTempK(293)
      CALL Output(Hscp_TO_HscpSI * 28E3)
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3506

  !---------------------------------------------------------------------------

  SUBROUTINE ref3507 ! Hscp [M/atm]
    IMPLICIT NONE

    ref = "3507"
    type = "M"

    ! Tabs. 2, 3, 4:
    CALL CalcH("methylbenzene",          "108-88-3", 0.15,  -38.4E3) ! toluene
    CALL CalcH("ethyl ethanoate",        "141-78-6", 5.94,  -43.4E3) ! ethyl acetate
    CALL CalcH("ethyl trifluoroacetate", "383-63-1", 0.072, -41E3)   ! ethyl trifluoroacetate
    CALL CalcH("hexanal",                 "66-25-1", 2.31,  -43.0E3)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, H298, Delta_H_sol)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: H298, Delta_H_sol
      chem=chem_ ; casrn=casrn_ ! make global for Output
      Hominus = Hscp_TO_HscpSI * H298
      CALL Output(Hominus, -Delta_H_sol/Rgas)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3507

  !---------------------------------------------------------------------------

  SUBROUTINE ref3508 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3508"
    type = "M"
    ndata = 4
    ALLOCATE(temp(ndata), Harray(ndata))
    temp = (/ 4., 11., 18., 25. /) + CtoK
    CALL CalcH("dimethyl sulfide",    "75-18-3", (/ 2.79E-2, 4.12E-2, 6.08E-2, 8.61E-2 /)) ! DMS
    CALL CalcH("dimethyl disulfide", "624-92-0", (/ 1.70E-2, 2.60E-2, 4.15E-2, 5.90E-2 /)) ! DMDS
    CALL CalcH("2-methylpropanal",    "78-84-2", (/ 1.96E-3, 3.85E-3, 6.91E-3, 1.27E-2 /)) ! 2-MP
    CALL CalcH("3-methylbutanal",    "590-86-3", (/ 2.25E-3, 4.52E-3, 8.20E-2, 1.43E-2 /)) ! 3-MB
    CALL CalcH("hexanal",             "66-25-1", (/ 1.54E-3, 3.46E-3, 6.80E-3, 1.41E-2 /)) ! HEX
    DEALLOCATE(temp, Harray)

CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, Kaw)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: Kaw
      chem=chem_ ; casrn=casrn_ ! make global for Output
      Harray = Hvcc_TO_HscpSI(Kaw,temp)
      CALL HTdep(temp, Harray, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3508

  !---------------------------------------------------------------------------

  SUBROUTINE ref3512 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3512"
    type = "Q"

    ! Table 3:
    CALL CalcH(  1, "1,3-Butadiene",                   "106-99-0",     3.45)
    CALL CalcH(  2, "1,4-Pentadiene",                  "591-93-5",     3.50)
    CALL CalcH(  3, "1,5-Hexadiene",                   "592-42-7",     3.76)
    CALL CalcH(  4, "1-Butene",                        "106-98-9",     4.21)
    CALL CalcH(  5, "1-Heptene",                       "592-76-7",     4.84)
    CALL CalcH(  6, "1-Heptyne",                       "628-71-7",     3.74)
    CALL CalcH(  7, "1-Hexene",                        "592-41-6",     4.62)
    CALL CalcH(  8, "1-Hexyne",                        "693-02-7",     3.59)
    CALL CalcH(  9, "1-Octene",                        "111-66-0",     5.28)
    CALL CalcH( 10, "1-Pentene",                       "109-67-1",     4.32)
    CALL CalcH( 11, "1-Pentyne",                       "627-19-0",     3.46)
    CALL CalcH( 12, "2,2,3,3,4-Pentamethylpentane",    "16747-44-7",   5.68)
    CALL CalcH( 13, "2,2,3,3-Tetramethylhexane",       "13475-81-5",   5.58)
    CALL CalcH( 14, "2,2,3,3-Tetramethylpentane",      "7154-79-2",    5.24)
    CALL CalcH( 15, "2,2,3,4,4-Pentamethylpentane",    "16747-45-8",   5.65)
    CALL CalcH( 16, "2,2,3,4-Tetramethylhexane",       "52897-08-2",   5.69)
    CALL CalcH( 17, "2,2,3,4-Tetramethylpentane",      "1186-53-4",    5.52)
    CALL CalcH( 18, "2,2,3,5-Tetramethylhexane",       "52897-09-3",   5.69)
    CALL CalcH( 19, "2,2,3-Trimethylbutane",           "464-06-2",     4.88)
    CALL CalcH( 20, "2,2,3-Trimethylheptane",          "52896-92-1",   5.69)
    CALL CalcH( 21, "2,2,3-Trimethylhexane",           "16747-25-4",   5.50)
    CALL CalcH( 22, "2,2,3-Trimethylpentane",          "564-02-3",     5.18)
    CALL CalcH( 23, "2,2,4,4-Tetramethylhexane",       "51750-65-3",   5.74)
    CALL CalcH( 24, "2,2,4,4-Tetramethylpentane",      "1070-87-7",    5.55)
    CALL CalcH( 25, "2,2,4,5-Tetramethylhexane",       "16747-42-5",   5.76)
    CALL CalcH( 26, "2,2,4-Trimethylheptane",          "14720-74-2",   5.74)
    CALL CalcH( 27, "2,2,4-Trimethylhexane",           "16747-26-5",   5.56)
    CALL CalcH( 28, "2,2,4-Trimethylpentane",          "540-84-1",     4.78)
    CALL CalcH( 29, "2,2,5,5-Tetramethylhexane",       "1071-81-4",    5.61)
    CALL CalcH( 30, "2,2,5-Trimethylheptane",          "20291-95-6",   5.61)
    CALL CalcH( 31, "2,2,5-Trimethylhexane",           "3522-94-9",    5.47)
    CALL CalcH( 32, "2,2,6-Trimethylheptane",          "1190-83-6",    5.68)
    CALL CalcH( 33, "2,2-Dimethyl-3-ethylpentane",     "16747-32-3",   5.54)
    CALL CalcH( 34, "2,2-Dimethylbutane",              "75-83-2",      4.65)
    CALL CalcH( 35, "2,2-Dimethylheptane",             "1071-26-7",    5.50)
    CALL CalcH( 36, "2,2-Dimethylhexane",              "590-73-8",     4.53)
    CALL CalcH( 37, "2,2-Dimethylpentane",             "590-35-2",     4.88)
    CALL CalcH( 38, "2,3,3,4-Tetramethylhexane",       "52897-10-6",   5.69)
    CALL CalcH( 39, "2,3,3,4-Tetramethylpentane",      "16747-38-9",   5.50)
    CALL CalcH( 40, "2,3,3,5-Tetramethylhexane",       "52897-11-7",   5.72)
    CALL CalcH( 41, "2,3,3-Trimethyl-1-butene",        "594-56-9",     4.90)
    CALL CalcH( 42, "2,3,3-Trimethylheptane",          "52896-93-2",   5.72)
    CALL CalcH( 43, "2,3,3-Trimethylhexane",           "16747-28-7",   5.17)
    CALL CalcH( 44, "2,3,3-Trimethylpentane",          "560-21-4",     5.29)
    CALL CalcH( 45, "2,3,4,4-Tetramethylhexane",       "52897-12-8",   5.62)
    CALL CalcH( 46, "2,3,4,5-Tetramethylhexane",       "52897-15-1",   5.68)
    CALL CalcH( 47, "2,3,4-Trimethylheptane",          "52896-95-4",   5.64)
    CALL CalcH( 48, "2,3,4-Trimethylhexane",           "921-47-1",     5.54)
    CALL CalcH( 49, "2,3,4-Trimethylpentane",          "565-75-3",     5.30)
    CALL CalcH( 50, "2,3,5-Trimethylheptane",          "20278-85-7",   5.68)
    CALL CalcH( 51, "2,3,5-Trimethylhexane",           "1069-53-0",    5.25)
    CALL CalcH( 52, "2,3,6-Trimethylheptane",          "4032-93-3",    5.65)
    CALL CalcH( 53, "2,3-Dimethyl-1,3-butadiene",      "513-81-5",     3.91)
    CALL CalcH( 54, "2,3-Dimethyl-1-butene",           "563-78-0",     4.20)
    CALL CalcH( 55, "2,3-Dimethyl-2-butene",           "563-79-1",     4.41)
    CALL CalcH( 56, "2,3-Dimethylbutane",              "79-29-8",      4.35)
    CALL CalcH( 57, "2,3-Dimethylheptane",             "3074-71-3",    5.13)
    CALL CalcH( 58, "2,3-Dimethyloctane",              "7146-60-3",    5.68)
    CALL CalcH( 59, "2,4,4-Trimethyl-1-pentene",       "107-39-1",     5.03)
    CALL CalcH( 60, "2,4,4-Trimethyl-2-pentene",       "107-40-4",     4.23)
    CALL CalcH( 61, "2,4,4-Trimethylheptane",          "4032-92-2",    5.73)
    CALL CalcH( 62, "2,4,4-Trimethylhexane",           "16747-30-1",   5.60)
    CALL CalcH( 63, "2,4,5-Trimethylheptane",          "20278-84-6",   5.72)
    CALL CalcH( 64, "2,4,6-Trimethylheptane",          "2613-61-8",    5.70)
    CALL CalcH( 65, "2,4-Dimethyl-3-ethylpentane",     "1068-87-7",    5.33)
    CALL CalcH( 66, "2,4-Dimethyl-3-isopropylpentane", "13475-79-1",   5.70)
    CALL CalcH( 67, "2,4-Dimethylheptane",             "2213-23-2",    5.54)
    CALL CalcH( 68, "2,4-Dimethyloctane",              "4032-94-4",    5.72)
    CALL CalcH( 69, "2,4-Dimethylpentane",             "108-08-7",     4.89)
    CALL CalcH( 70, "2,5,5-Trimethylheptane",          "1189-99-7",    5.72)
    CALL CalcH( 71, "2,5-Dimethylhexane",              "592-13-2",     5.14)
    CALL CalcH( 72, "2,5-Dimethyloctane",              "15869-89-3",   5.70)
    CALL CalcH( 73, "2,6-Dimethylheptane",             "1072-05-5",    5.52)
    CALL CalcH( 74, "2,6-Dimethyloctane",              "2051-30-1",    5.70)
    CALL CalcH( 75, "2,7-Dimethyloctane",              "1072-16-8",    5.69)
    CALL CalcH( 76, "2-Ethyl-1-butene",                "760-21-4",     4.65)
    CALL CalcH( 77, "2-Ethyl-1-pentene",               "3404-71-5",    4.92)
    CALL CalcH( 78, "2-Hexyne",                        "764-35-2",     3.70)
    CALL CalcH( 79, "2-Methyl-1-butene",               "563-46-2",     4.56)
    CALL CalcH( 80, "2-Methyl-1-hexene",               "6094-02-6",    4.83)
    CALL CalcH( 81, "2-Methyl-1-pentene",              "763-29-1",     4.24)
    CALL CalcH( 82, "2-Methyl-2,3-butadiene",          "598-25-4",     3.65)
    CALL CalcH( 83, "2-Methyl-2-butene",               "513-35-9",     3.73)
    CALL CalcH( 84, "2-Methyl-2-pentene",              "625-27-4",     3.93)
    CALL CalcH( 85, "2-Methylheptane",                 "592-27-8",     5.28)
    CALL CalcH( 86, "2-Methylhexane",                  "591-76-4",     4.97)
    CALL CalcH( 87, "2-Methylnonane",                  "871-83-0",     5.73)
    CALL CalcH( 88, "2-Methyloctane",                  "3221-61-2",    5.27)
    CALL CalcH( 89, "2-Methylpentane",                 "107-83-5",     4.64)
    CALL CalcH( 90, "2-Pentyne",                       "627-21-4",     3.59)
    CALL CalcH( 91, "3,3,4,4-Tetramethylhexane",       "5171-84-6",    5.58)
    CALL CalcH( 92, "3,3,4-Trimethylheptane",          "20278-87-9",   5.68)
    CALL CalcH( 93, "3,3,4-Trimethylhexane",           "16747-31-2",   5.56)
    CALL CalcH( 94, "3,3,5-Trimethylheptane",          "7154-80-5",    5.72)
    CALL CalcH( 95, "3,3-Diethyl-2-methylpentane",     "52897-16-2",   5.68)
    CALL CalcH( 96, "3,3-Diethylhexane",               "17302-02-2",   5.75)
    CALL CalcH( 97, "3,3-Diethylpentane",              "1067-20-5",    5.33)
    CALL CalcH( 98, "3,3-Dimethyl-1-butene",           "558-37-2",     4.35)
    CALL CalcH( 99, "3,3-Dimethylhexane",              "563-16-6",     5.35)
    CALL CalcH(100, "3,3-Dimethyloctane",              "4110-44-5",    5.73)
    CALL CalcH(101, "3,3-Dimethylpentane",             "562-49-2",     4.76)
    CALL CalcH(102, "3,4,4-Trimethylheptane",          "20278-88-0",   5.70)
    CALL CalcH(103, "3,4,5-Trimethylheptane",          "20278-89-1",   5.66)
    CALL CalcH(104, "3,4-Diethylhexane",               "19398-77-7",   5.74)
    CALL CalcH(105, "3,4-Dimethylheptane",             "922-28-1",     5.46)
    CALL CalcH(106, "3,4-Dimethyloctane",              "15869-92-8",   5.68)
    CALL CalcH(107, "3,5-Dimethylheptane",             "926-82-9",     5.47)
    CALL CalcH(108, "3,5-Dimethyloctane",              "15869-93-9",   5.69)
    CALL CalcH(109, "3,6-Dimethyloctane",              "15869-94-0",   5.69)
    CALL CalcH(110, "3-Ethyl-1-pentene",               "4038-04-4",    4.42)
    CALL CalcH(111, "3-Ethyl-2,2,3-trimethylpentane",  "52897-17-3",   5.63)
    CALL CalcH(112, "3-Ethyl-2,2,4-trimethylpentane",  "52897-18-4",   5.64)
    CALL CalcH(113, "3-Ethyl-2,3,4-trimethylpentane",  "52897-19-5",   5.70)
    CALL CalcH(114, "3-Ethyl-2,3-dimethylpentane",     "16747-33-4",   5.54)
    CALL CalcH(115, "3-Ethyl-2,2-dimethylhexane",      "20291-91-2",   5.73)
    CALL CalcH(116, "3-Ethyl-2,3-dimethylhexane",      "52897-00-4",   5.71)
    CALL CalcH(117, "3-Ethyl-2,4-dimethylhexane",      "7220-26-0",    5.74)
    CALL CalcH(118, "3-Ethyl-2,5-dimethylhexane",      "52897-04-8",   5.76)
    CALL CalcH(119, "3-Ethyl-2-methylheptane",         "14676-29-0",   5.70)
    CALL CalcH(120, "3-Ethyl-2-methylhexane",          "16789-46-1",   5.22)
    CALL CalcH(121, "3-Ethyl-2-methylpentane",         "609-26-7",     5.19)
    CALL CalcH(122, "3-Ethyl-3,4-dimethylhexane",      "52897-06-0",   5.64)
    CALL CalcH(123, "3-Ethyl-3-methylheptane",         "17302-01-1",   5.73)
    CALL CalcH(124, "3-Ethyl-3-methylhexane",          "3074-76-8",    5.62)
    CALL CalcH(125, "3-Ethyl-4-methylheptane",         "52896-91-0",   5.72)
    CALL CalcH(126, "3-Ethyl-4-methylhexane",          "3074-77-9",    5.38)
    CALL CalcH(127, "3-Ethyl-5-methylheptane",         "52896-90-9",   5.76)
    CALL CalcH(128, "3-Ethylheptane",                  "15869-80-4",   5.53)
    CALL CalcH(129, "3-Ethylpentane",                  "617-78-7",     5.08)
    CALL CalcH(130, "3-Hexyne",                        "928-49-4",     3.68)
    CALL CalcH(131, "3-Isopropyl-2-methylhexane",      "62016-13-1",   5.66)
    CALL CalcH(132, "3-Methyl-1-butene",               "563-45-1",     4.02)
    CALL CalcH(133, "3-Methyl-1-hexene",               "3404-61-3",    3.71)
    CALL CalcH(134, "3-Methyl-1-pentene",              "760-20-3",     4.18)
    CALL CalcH(135, "3-Methyl-3-ethylpentane",         "1067-08-9",    5.32)
    CALL CalcH(136, "3-Methylheptane",                 "589-81-1",     5.31)
    CALL CalcH(137, "3-Methylhexane",                  "589-34-4",     5.00)
    CALL CalcH(138, "3-Methylnonane",                  "5911-04-6",    5.70)
    CALL CalcH(139, "3-Methyloctane",                  "2216-33-3",    5.55)
    CALL CalcH(140, "3-Methylpentane",                 "96-14-0",      4.87)
    CALL CalcH(141, "4,4-Dimethylheptane",             "1068-19-5",    5.57)
    CALL CalcH(142, "4,4-Dimethyloctane",              "15869-95-1",   5.73)
    CALL CalcH(143, "4,5-Dimethyloctane",              "15869-96-2",   5.72)
    CALL CalcH(144, "4-Ethyl-2,2-dimethylhexane",      "52896-99-8",   5.72)
    CALL CalcH(145, "4-Ethyl-2,3-dimethylhexane",      "52897-01-5",   5.67)
    CALL CalcH(146, "4-Ethyl-2,4-dimethylhexane",      "52897-03-7",   5.71)
    CALL CalcH(147, "4-Ethyl-2-methylheptane",         "52896-88-5",   5.71)
    CALL CalcH(148, "4-Ethyl-2-methylhexane",          "3074-75-7",    5.52)
    CALL CalcH(149, "4-Ethyl-3,3-dimethylhexane",      "52897-05-9",   5.73)
    CALL CalcH(150, "4-Ethyl-3-methylheptane",         "52896-89-6",   5.66)
    CALL CalcH(151, "4-Ethyl-4-methylheptane",         "17302-04-4",   5.75)
    CALL CalcH(152, "4-Ethylheptane",                  "2216-32-2",    5.58)
    CALL CalcH(153, "4-Ethyloctane",                   "15869-86-0",   5.76)
    CALL CalcH(154, "4-Isopropylheptane",              "52896-87-4",   5.66)
    CALL CalcH(155, "4-Methyl-1-hexene",               "3769-23-1",    4.43)
    CALL CalcH(156, "4-Methyl-1-pentene",              "691-37-2",     4.01)
    CALL CalcH(157, "4-Methyl-cis-2-pentene",          "691-38-3",     4.68)
    CALL CalcH(158, "4-Methylheptane",                 "589-53-7",     5.23)
    CALL CalcH(159, "4-Methylnonane",                  "17301-94-9",   5.74)
    CALL CalcH(160, "4-Methyloctane",                  "2216-34-4",    5.52)
    CALL CalcH(161, "4-Methyl-trans-2-pentene",        "674-76-0",     4.40)
    CALL CalcH(162, "4-Propylheptane",                 "3178-29-8",    5.73)
    CALL CalcH(163, "5-Ethyl-2-methylheptane",         "13475-78-0",   5.64)
    CALL CalcH(164, "5-Methylnonane",                  "15869-85-9",   5.73)
    CALL CalcH(165, "cis-2-Butene",                    "590-18-1",     4.07)
    CALL CalcH(166, "cis-2-Heptene",                   "6443-92-1",    3.79)
    CALL CalcH(167, "cis-2-Hexene",                    "7688-21-3",    4.24)
    CALL CalcH(168, "cis-2-Pentene",                   "627-20-3",     4.32)
    CALL CalcH(169, "cis-3-Heptene",                   "7642-10-6",    4.67)
    CALL CalcH(170, "cis-3-Hexene",                    "7642-09-3",    4.50)
    CALL CalcH(171, "Ethane",                          "74-84-0",      4.48)
    CALL CalcH(172, "Isobutane",                       "75-28-5",      4.70)
    CALL CalcH(173, "Isobutene",                       "115-11-7",     3.89)
    CALL CalcH(174, "Isopentane",                      "78-78-4",      4.67)
    CALL CalcH(175, "Isoprene",                        "78-79-5",      3.60)
    CALL CalcH(176, "n-Butane",                        "106-97-8",     4.75)
    CALL CalcH(177, "Neopentane",                      "463-82-1",     4.72)
    CALL CalcH(178, "n-Heptane",                       "142-82-5",     5.12)
    CALL CalcH(179, "n-Hexane",                        "110-54-3",     4.93)
    CALL CalcH(180, "n-Nonane",                        "111-84-2",     5.59)
    CALL CalcH(181, "n-Pentane",                       "109-66-0",     4.86)
    CALL CalcH(182, "Propane",                         "74-98-6",      4.68)
    CALL CalcH(183, "trans-2-Butene",                  "624-64-6",     3.86)
    CALL CalcH(184, "trans-2-Heptene",                 "14686-13-6",   4.51)
    CALL CalcH(185, "trans-2-Hexene",                  "4050-45-7",    4.27)
    CALL CalcH(186, "trans-2-Pentene",                 "646-04-8",     3.69)
    CALL CalcH(187, "trans-3-Heptene",                 "14686-14-7",   4.61)
    CALL CalcH(188, "trans-3-Hexene",                  "13269-52-8",   4.32)
    CALL CalcH(189, "trans-3-Methyl-2-pentene",        "616-12-6",     4.29)

  CONTAINS

    SUBROUTINE CalcH (number, chem_, casrn_, Hvcc)
      IMPLICIT NONE
      INTEGER,          INTENT(IN) :: number
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: Hvcc
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL MakeNote(TRIM(ref), &
        TRIM(citet())//" use different descriptors for the QSPR models. They "// &
        "conclude that their ``COSA'' method and the artificial neural "// &
        "network (ANN) are best. However, as COSA is not ideal for hydrocarbons "// &
        "with low solubility, only results obtained with ANN are shown here.")
      CALL Output(Hvcc_TIMES_HscpSI_atT0/EXP(Hvcc))
    END SUBROUTINE CalcH

  END SUBROUTINE ref3512

  !---------------------------------------------------------------------------

  SUBROUTINE ref3517 ! Hvpx [GPa]
    IMPLICIT NONE

    ref = "3517"
    type = "Q"
    ndata = 3
    ALLOCATE(temp(ndata), Harray(ndata))
    temp = (/ 275., 295., 315. /)
    ! (only Ne was measured in water, but data from Fig. 2 are not available)
    ! Tabs. S2-S4:
    CALL CalcH("helium",   "7440-59-7", (/ 16.570, 17.427, 16.562 /), "thiswork" ) ! He
    CALL CalcH("helium",   "7440-59-7", (/ 16.331, 17.506, 16.836 /), "Warr"     ) ! He
    CALL CalcH("neon",     "7440-01-9", (/ 14.923, 16.795, 16.668 /), "thiswork" ) ! Ne
    CALL CalcH("neon",     "7440-01-9", (/ 10.187, 12.142, 12.595 /), "Warr"     ) ! Ne
    CALL CalcH("argon",    "7440-37-1", (/  5.004,  7.085,  8.526 /), "Warr"     ) ! Ar
    CALL CalcH("argon",    "7440-37-1", (/  3.784,  5.303,  6.214 /), "Vrabec"   ) ! Ar
    CALL CalcH("krypton",  "7439-90-9", (/  2.349,  3.580,  4.866 /), "Warr"     ) ! Kr
    CALL CalcH("krypton",  "7439-90-9", (/  2.346,  3.527,  4.419 /), "Vrabec"   ) ! Kr
    CALL CalcH("xenon",    "7440-63-3", (/  0.412,  0.733,  1.172 /), "Warr"     ) ! Xe
    CALL CalcH("xenon",    "7440-63-3", (/  1.135,  1.839,  2.610 /), "Vrabec"   ) ! Xe
    CALL CalcH("radon",   "10043-92-2", (/  0.379,  0.687,  1.138 /), "Mick"     ) ! Rn
    DEALLOCATE(temp, Harray)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, Hi, xref_)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: Hi
      CHARACTER(LEN=*),   INTENT(IN) :: xref_
      chem=chem_ ; casrn=casrn_ ! make global for Output
      Harray = cH2O / (Hi*1E9)
      CALL HTdep(temp, Harray, Hominus, mindHR)
      IF (xref_ == "Warr") THEN
        CALL MakeNote(TRIM(ref)//"-"//xref_, "Calculated employing molecular " // &
          "force field models for the solutes from \citet{3550}.")
      ENDIF
      IF (xref_ == "Vrabec") THEN
        CALL MakeNote(TRIM(ref)//"-"//xref_, "Calculated employing molecular " // &
          "force field models for the solutes from \citet{3551}.")
      ENDIF
      IF (xref_ == "Mick") THEN
        CALL MakeNote(TRIM(ref)//"-"//xref_, "Calculated employing molecular " // &
          "force field models for the solutes from \citet{3552}.")
      ENDIF
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3517

  !---------------------------------------------------------------------------

  SUBROUTINE ref3518 ! Hvpx [kPa]
    IMPLICIT NONE

    ref = "3518"

    ! Table 6.1: Toluene and 1-butanol

    ! cp tdep from ref3006

    ! "toluene", "108-88-3",
    ! 298.15, 3.35E4,   3.69E4  [192] Sanemasa 1982
    ! 308.15, 5.46E4,   5.44E4  [201] Sanemasa 1981
    !
    ! "1-butanol",       "71-36-3",
    ! 298.15, 69.4,     49.4    [202] Buttery
    ! 308.15, 105.,     115     [203] Butler
    ! 313.15, 140.,     123
    ! 323.15, 261.,     274     [204] Hovorka

    ! pdf page 104: Table 6.5: anisole etc.

    ! anisole
    ! 280.95 8.35E2
    ! 293.16 1.48E3 ! 1.48E3 [206] Dewulf
    ! 308.15 2.95E3
    ! 322.90 5.72E3
    !
    ! 1,2-difluorobenzene
    ! 298.19 4.35E4
    ! 308.12 6.65E4
    ! 317.90 9.33E4
    ! 322.89 1.05E5
    !
    ! 4-bromotoluene
    ! 293.10 1.18E4
    ! 298.16 1.71E4
    ! 308.10 2.76E4
    ! 318.03 4.03E4
    ! 322.91 5.28E4
    !
    ! 1,2,3-trichlorobenzene
    ! 280.95 3.36E3
    ! 293.06 6.84E3
    ! 308.14 1.44E4
    ! 322.99 2.81E4
    !
    ! 2,4-dichlorotoluene
    ! 280.94 7.14E3
    ! 293.09 1.68E4
    ! 308.15 3.25E4
    ! 323.10 7.54E4

    ! pdf page 188-207: data table C2 (eqn 101 is on pdf page 44)
    ! page 182 = pdf-page 188:
    ! 1-alkenes:
    CALL CalcH("1-pentene",                         "109-67-1", &
               100,  1.867E+06,         0.,          0.,         0.,  0, "S",  1.8667E+06, "S")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("1-hexene",                          "592-41-6", &
               100,  2.190E+06,         0.,          0.,         0.,  0, "S",  2.1902E+06, "S")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("1-heptene",                         "592-76-7", &
               100,  2.828E+06,         0.,          0.,         0.,  0, "S",  2.8281E+06, "S")  ! 298.15, 298.15,  "<10%",
    CALL CalcH("1-octene",                          "111-66-0", &
               100,  3.555E+06,         0.,          0.,         0.,  0, "S",  3.5550E+06, "S")  ! 298.15, 298.15,  "<10%",
    CALL CalcH("1-nonene",                          "124-11-8", &
               100,  4.854E+06,         0.,          0.,         0.,  0, "S",  4.8537E+06, "S")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("1-decene",                          "872-05-9", &
               100,  6.459E+06,         0.,          0.,         0.,  0, "P",  6.4586E+06, "P")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("1-undecene",                        "821-95-4", &
               100,  8.669E+06, -1.003E+04,  -3.126E+01,         0.,  0, "P"                  )  ! 298.15, 298.15,  "<25%",
    CALL CalcH("1-dodecene",                        "112-41-4", &
               100,  1.108E+07, -5.042E+04,  -2.173E+02,  2.501E-01,  1, "P"                  )  ! 298.15, 298.15,  "<25%",
    CALL CalcH("1-tridecene",                      "2437-56-1", &
               100,  1.513E+07, -1.203E+04,  -3.844E+01,         0.,  0, "P"                  )  ! 298.15, 298.15,  "<50%",
    CALL CalcH("1-tetradecene",                    "1120-36-1", &
               100,  1.827E+07,         0.,          0.,         0.,  0, "P"                  )  ! 298.15, 298.15,  "<50%",
    CALL CalcH("1-pentadecene",                   "13360-61-7", &
               100,  2.493E+07,         0.,          0.,         0.,  0, "S",  2.4928E+07, "S")  ! 298.15, 298.15,  "<50%",
    ! 2,3,4-alkenes:
    CALL CalcH("cis-2-pentene",                     "627-20-3", &
               100,  1.262E+06,         0.,          0.,         0.,  0, "E",  1.2624E+06, "D")  ! 298.15, 298.15,  "<5%",
    CALL CalcH("trans-2-pentene",                   "646-04-8", &
               100,  1.293E+06,         0.,          0.,         0.,  0, "E",  1.2932E+06, "D")  ! 298.15, 298.15,  "<5%",
    CALL CalcH("cis-2-heptene",                    "6443-92-1", &
               100,  2.341E+06,         0.,          0.,         0.,  0, "E",  2.3412E+06, "D")  ! 298.15, 298.15,  "<10%",
    CALL CalcH("trans-2-heptene",                 "14686-13-6", &
               100,  2.292E+06,         0.,          0.,         0.,  0, "E",  2.2918E+06, "D")  ! 298.15, 298.15,  "<10%",
    ! acetates:
    CALL CalcH("methyl acetate",                     "79-20-9", &
               101,  4.579E+02, -2.509E+04,  -6.446E+01,         0.,  0, "E",  6.7019E+02, "S")  ! 293.15, 343.25,  "<10%",
    CALL CalcH("ethyl acetate",                     "141-78-6", &
               101,  2.419E+02, -1.524E+04,  -3.230E+01,         0.,  0, "E",  8.4064E+02, "S")  ! 288.15, 353.15,  "<10%",
    CALL CalcH("n-propyl acetate",                  "109-60-4", &
               101,  2.912E+02, -1.778E+04,  -3.939E+01,         0.,  0, "E",  1.2278E+03, "S")  ! 298,    343.25,  "<10%",
    CALL CalcH("n-butyl acetate",                   "123-86-4", &
               101,  2.499E+02, -1.619E+04,  -3.305E+01,         0.,  0, "E",  1.4682E+03, "S")  ! 273.15, 363.65,  "<10%",
    CALL CalcH("isobutyl acetate",                  "110-19-0", &
               101,  8.284E+01, -8.464E+03,  -8.209E+00,         0.,  0, "E",  2.1658E+03, "S")  ! 273.15, 323.15,  "<25%",
    CALL CalcH("isopentyl acetate",                 "123-92-2", &
               101,  3.240E+02, -1.974E+04,  -4.386E+01,         0.,  0, "E",  2.7926E+03, "S")  ! 273.15, 363.85,  "<10%",
    ! page 183 = pdf-page 189:
    CALL CalcH("isopropyl acetate",                 "108-21-4", &
               101, -6.279E+01, -2.050E+03,   1.353E+01,         0.,  0, "E",  1.6585E+03, "S")  ! 273.15, 310.2,   "<25%",
    CALL CalcH("sec-butyl acetate",                 "105-46-4", &
               101,  2.287E+02, -1.494E+04,  -2.998E+01,         0.,  0, "E",  2.4256E+03, "S")  ! 273.15, 363.65,  "<10%",
    CALL CalcH("vinyl acetate",                     "108-05-4", &
               101,  1.816E+02, -1.163E+04,  -2.364E+01,         0.,  0, "E",  2.7687E+03, "S")  ! 273.35, 333.15,  "<3%",
    CALL CalcH("n-pentyl acetate",                  "628-63-7", &
               101,  4.256E+02, -2.473E+04,  -5.879E+01,         0.,  0, "E",  2.2402E+03, "S")  ! 273.15, 364.5,   "<25%",
    CALL CalcH("n-hexyl acetate",                   "142-92-7", &
               101,  4.425E+02, -2.542E+04,  -6.131E+01,         0.,  0, "E",  2.9135E+03, "S")  ! 273.15, 363.35,  "<25%",
    CALL CalcH("n-heptyl acetate",                  "112-06-1", &
               101,  3.811E+03,  1.191E+02,          0.,         0.,  0, "P",  3.8109E+03, "P")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("n-octyl acetate",                   "112-14-1", &
               101,  5.471E+03, -1.036E+04,  -3.420E+01,         0.,  0, "P",  5.4713E+03, "P")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("tert-butyl acetate",                "540-88-5", &
               101,  2.451E+02, -1.574E+04,  -3.227E+01,         0.,  0, "E",  4.7259E+03, "S")  ! 273.15, 353.65,  "<10%",
    CALL CalcH("cyclohexyl acetate",                "622-45-7", &
               101,  2.617E+02, -1.729E+04,  -3.465E+01,         0.,  0, "E",  5.0496E+02, "S")  ! 273.15, 363.45,  "<10%",
    CALL CalcH("phenyl acetate",                    "122-79-2", &
               101,  2.503E+02, -1.713E+04,  -3.319E+01,         0.,  0, "E",  4.2332E+01, "S")  ! 273.15, 363.95,  "<10%",
    ! aldehydes:
    CALL CalcH("acetaldehyde",                       "75-07-0", &
               101,  2.462E+01, -5.570E+03,          0.,         0.,  0, "E",  3.7813E+02, "S")  ! 273,    318.15,  "<10%",
    CALL CalcH("propanal",                          "123-38-6", &
               101,  2.821E+02, -1.701E+04,  -3.844E+01,         0.,  0, "E",  4.3628E+02, "S")  ! 278.15, 318.15,  "<5%",
    CALL CalcH("butanal",                           "123-72-8", &
               101,  2.664E+01, -6.032E+03,          0.,         0.,  0, "E",  6.0597E+02, "S")  ! 278.15, 318.15,  "<10%",
    CALL CalcH("2-methylpropanal",                   "78-84-2", &
               100,  1.093E+03,         0.,          0.,         0.,  0, "S",  1.0934E+03, "S")  ! 298.15, 298.15,  "<5%",
    CALL CalcH("pentanal",                          "110-62-3", &
               101,  2.851E+01, -6.494E+03,          0.,         0.,  0, "E",  8.3509E+02, "S")  ! 278.15, 318.15,  "<10%",
    CALL CalcH("heptanal",                          "111-71-7", &
               101,  3.256E+01, -7.502E+03,          0.,         0.,  0, "E",  1.6308E+03, "S")  ! 283.15, 318.15,  "<25%",
    CALL CalcH("hexanal",                            "66-25-1", &
               101,  2.848E+01, -6.372E+03,          0.,         0.,  0, "E",  1.2229E+03, "S")  ! 283.15, 318.15,  "<25%",
    CALL CalcH("octanal",                           "124-13-0", &
               101,  3.253E+01, -7.342E+03,          0.,         0.,  0, "E",  2.7045E+03, "S")  ! 283.15, 318.15,  "<10%",
    CALL CalcH("nonanal",                           "124-19-6", &
               101,  3.145E+01, -6.837E+03,          0.,         0.,  0, "E",  4.9990E+03, "S")  ! 283.15, 318.15,  "<25%",
    CALL CalcH("2-ethylhexanal",                    "123-05-7", &
               101,  2.662E+01, -5.414E+03,          0.,         0.,  0, "E",  4.7357E+03, "S")  ! 273.15, 363.15,  "<25%",
    CALL CalcH("2-ethyl-2-hexenal",                 "645-62-5", &
               101,  2.647E+01, -5.815E+03,          0.,         0.,  0, "E",  1.0617E+03, "S")  ! 273.15, 363.15,  "<50%",
    CALL CalcH("decanal",                           "112-31-2", &
               101,  3.897E+01, -8.912E+03,          0.,         0.,  0, "E",  8.7393E+03, "S")  ! 298.15, 318.15,  "<50%",
    CALL CalcH("2-methyl-butyraldehyde",             "96-17-3", &
               101,  2.641E+01, -5.574E+03,          0.,         0.,  0, "P",  2.2328E+03, "P")  ! 273.15, 363.15,  "<25%",
    CALL CalcH("acrolein",                          "107-02-8", &
               101,  2.385E+01, -5.131E+03,          0.,         0.,  0, "E",  7.6218E+02, "E")  ! 273,    298,     "<100%",
    ! page 184 = pdf-page 190:
    CALL CalcH("trans-crotonaldehyde",              "123-73-9", &
               101,  1.928E+02, -1.287E+04,  -2.545E+01,         0.,  0, "EP", 1.0285E+02, "S")  ! 273.15, 363.15,  "<25%",
    CALL CalcH("methacrolein",                       "78-85-3", &
               101,  2.256E+01, -4.605E+03,          0.,         0.,  0, "E",  1.2347E+03, "S")  ! 273.15, 333.15,  "<10%",
    CALL CalcH("p-tolualdehyde",                    "104-87-0", &
               101,  2.876E+01, -7.189E+03,          0.,         0.,  0, "E",  1.0440E+02, "S")  ! 278,    298,     "<25%",
    CALL CalcH("benzaldehyde",                      "100-52-7", &
               101,  5.844E+02, -3.018E+04,  -8.394E+01,         0.,  0, "E",  1.4523E+02, "S")  ! 273.08, 323.15,  "<10%",
    CALL CalcH("m-tolualdehyde",                    "620-23-5", &
               101,  2.911E+01, -7.166E+03,          0.,         0.,  0, "E",  1.6029E+02, "S")  ! 278,    298,     "<25%",
    ! aliphatic ethers:
    CALL CalcH("diethyl ether",                      "60-29-7", &
               101,  2.854E+02, -1.727E+04,  -3.841E+01,         0.,  0, "E",  5.7579E+03, "S")  ! 273.35, 333.25,  "<5%",
    CALL CalcH("diisopropyl ether",                 "108-20-3", &
               101, -1.401E+01, -4.538E+03,   6.784E+00,         0.,  0, "E",  1.2343E+04, "S")  ! 273.15, 334.15,  "<10%",
    CALL CalcH("di-n-butyl ether",                  "142-96-1", &
               101,  3.269E+01, -6.711E+03,          0.,         0.,  0, "E",  2.6272E+04, "S")  ! 273.15, 363.65,  "<25%",
    CALL CalcH("methyl tert-butyl ether",          "1634-04-4", &
               101,  1.038E+02, -9.286E+03,  -1.129E+01,         0.,  0, "E",  4.1832E+03, "S")  ! 273.15, 343.15,  "<10%",
    CALL CalcH("methyl isobutyl ether",             "625-44-5", &
               101,  3.182E+01, -6.669E+03,          0.,         0.,  0, "E",  1.2742E+04, "S")  ! 273.15, 298.15,  "<5%",
    CALL CalcH("di-n-hexyl ether",                  "112-58-3", &
               101,  8.915E+04, -7.488E+03,  -2.548E+01,         0.,  0, "P",  8.9153E+04, "P")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("methyl n-butyl ether",              "628-28-4", &
               101,  7.536E+02, -3.836E+04,  -1.081E+02,         0.,  0, "E",  1.0077E+04, "S")  ! 273.15, 298.15,  "<25%",
    CALL CalcH("di-n-pentyl ether",                 "693-65-2", &
               101,  5.151E+04, -5.574E+03,          0.,         0.,  0, "P",  5.1508E+04, "P")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("methyl tert-pentyl ether",          "994-05-8", &
               101,  2.294E+02, -1.545E+04,  -2.965E+01,         0.,  0, "E",  5.6708E+03, "S")  ! 273.15, 352.35,  "<10%",
    CALL CalcH("tert-butyl ethyl ether",            "637-92-3", &
               101,  3.193E+01, -6.851E+03,          0.,         0.,  0, "E",  7.7653E+03, "S")  ! 278.15, 318.15,  "<25%",
    CALL CalcH("ethyl tert-pentyl ether",           "919-94-8", &
               101,  3.404E+01, -7.373E+03,          0.,         0.,  0, "E",  1.1017E+04, "S")  ! 288.15, 318.15,  "<25%",
    CALL CalcH("di-n-propyl ether",                 "111-43-3", &
               101,  4.575E+02, -2.541E+04,  -6.361E+01,         0.,  0, "E",  1.8369E+04, "S")  ! 273.15, 333.25,  "<5%",
    CALL CalcH("n-butyl ethyl ether",               "628-81-9", &
               101,  2.585E+02, -1.600E+04,  -3.439E+01,         0.,  0, "E",  7.2904E+03, "S")  ! 273.15, 363.85,  "<25%",
    ! alkylcyclohexanes:
    CALL CalcH("methylcyclohexane",                 "108-87-2", &
               101,  4.101E+02, -2.147E+04,  -5.678E+01,         0.,  0, "E",  2.1421E+06, "S")  ! 293.15, 443.95,  "<25%",
    CALL CalcH("ethylcyclohexane",                 "1678-91-7", &
               101,  1.847E+03, -6.190E+04,  -3.046E+02,  3.719E-01,  1, "E",  2.7811E+06, "S")  ! 273.15, 443.95,  "<10%",
    CALL CalcH("cis-1,2-dimethylcyclohexane",      "2207-01-4", &
               101,  3.907E+02, -2.062E+04,  -5.388E+01,         0.,  0, "E",  2.0121E+06, "S")  ! 273.15, 313.15,  "<25%",
    CALL CalcH("trans-1,2-dimethylcyclohexane",    "6876-23-9", &
               101,  1.558E+03, -5.359E+04,  -2.548E+02,  2.970E-01,  1, "E",  3.4285E+06, "S")  ! 273.15, 443.25,  "<5%",
    ! page 185 = pdf-page 191:
    ! alkylcyclopentanes:
    CALL CalcH("methylcyclopentane",                 "96-37-7", &
               100,  2.005E+06,  7.831E+03,   2.727E+01,  6.321E-06,  2, "S",  2.0047E+06, "S")  ! 298.15, 298.15,  "<3%",
    ! alkynes:
    CALL CalcH("ethylacetylene",                    "107-00-6", &
               100,  9.853E+04,         0.,          0.,         0.,  0, "S",  9.8530E+04, "S")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("1-pentyne",                         "627-19-0", &
               100,  1.436E+05,         0.,          0.,         0.,  0, "S",  1.4360E+05, "S")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("1-hexyne",                          "693-02-7", &
               100,  2.093E+05,         0.,          0.,         0.,  0, "S",  2.0930E+05, "S")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("1-octyne",                          "629-05-0", &
               100,  4.445E+05,         0.,          0.,         0.,  0, "S",  4.4450E+05, "S")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("1-nonyne",                         "3452-09-3", &
               100,  6.477E+05,         0.,          0.,         0.,  0, "S",  6.4770E+05, "S")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("1-heptyne",                         "628-71-7", &
               100,  3.050E+05,         0.,          0.,         0.,  0, "S",  3.0500E+05, "S")  ! 298.15, 298.15,  "<25%",
    ! aromatic alcohols:
    CALL CalcH("2,3-xylenol",                       "526-75-0", &
               101,  2.249E+01, -6.077E+03,          0.,         0.,  0, "S"                  )  ! 349.05, 371.65,  "<25%",
    CALL CalcH("2,4-xylenol",                       "105-67-9", &
               101,  2.115E+01, -5.479E+03,          0.,         0.,  0, "E",  1.5966E+01, "S")  ! 349.05, 371.65,  "<25%",
    CALL CalcH("2,5-xylenol",                        "95-87-4", &
               101,  4.433E+02, -2.706E+04,  -6.154E+01,         0.,  0, "E",  7.0922E+00, "S")  ! 278,    371.65,  "<10%",
    CALL CalcH("2,6-xylenol",                       "576-26-1", &
               101,  1.892E+02, -1.364E+04,  -2.455E+01,         0.,  0, "E",  3.4833E+01, "S")  ! 278,    371.65,  "<10%",
    CALL CalcH("3,4-xylenol",                        "95-65-8", &
               101,  2.495E+01, -7.210E+03,          0.,         0.,  0, "E",  2.1606E+00, "S")  ! 293.15, 371.65,  "<25%",
    CALL CalcH("3,5-xylenol",                       "108-68-9", &
               101,  1.997E+02, -1.533E+04,  -2.580E+01,         0.,  0, "E",  3.5323E+00, "S")  ! 293.15, 371.65,  "<25%",
    CALL CalcH("phenol",                            "108-95-2", &
               101,  5.283E+02, -3.105E+04,  -7.428E+01,         0.,  0, "E",  2.5082E+00, "S")  ! 278,    373.15,  "<25%",
    CALL CalcH("o-cresol",                           "95-48-7", &
               101,  3.028E+02, -1.968E+04,  -4.119E+01,         0.,  0, "E",  8.2552E+00, "S")  ! 293.15, 371.65,  "<5%",
    CALL CalcH("m-cresol",                          "108-39-4", &
               101,  7.124E+01, -8.354E+03,  -7.310E+00,         0.,  0, "E",  4.8030E+00, "S")  ! 298.15, 371.65,  "<25%",
    CALL CalcH("p-cresol",                          "106-44-5", &
               101,  1.167E+02, -1.074E+04,  -1.390E+01,         0.,  0, "E",  4.3850E+00, "S")  ! 298,    371.65,  "<3%",
    CALL CalcH("2-phenylethanol",                    "60-12-8", &
               101,  2.325E+02, -1.641E+04,  -3.094E+01,         0.,  0, "P",  3.1574E+00, "P")  ! 283.15, 373.15,  "<10%",
    ! aromatic amines:
    CALL CalcH("o-toluidine",                        "95-53-4", &
               101,  2.612E+01, -7.057E+03,          0.,         0.,  0, "E",  1.1565E+01, "S")  ! 298.15, 323.15,  "<25%",
    CALL CalcH("m-toluidine",                       "108-44-1", &
               101,  2.644E+01, -7.202E+03,          0.,         0.,  0, "E",  9.8018E+00, "S")  ! 298.15, 323.15,  "<25%",
    CALL CalcH("p-toluidine",                       "106-49-0", &
               101,  1.765E+01, -4.381E+03,          0.,         0.,  0, "P",  1.9157E+01, "S")  ! 293.15, 313.15,  "<5%",
    ! page 186 = pdf-page 192:
    CALL CalcH("N,N-diethylaniline",                 "91-66-7", &
               101,  2.913E+01, -6.682E+03,          0.,         0.,  0, "E",  8.3101E+02, "S")  ! 273.15, 333.15,  "<25%",
    CALL CalcH("pyridine",                          "110-86-1", &
               101,  1.987E+01, -4.699E+03,          0.,         0.,  0, "E",  6.1142E+01, "S")  ! 298,    374.15,  "<25%",
    CALL CalcH("aniline",                            "62-53-3", &
               101,  2.395E+02, -1.634E+04,  -3.199E+01,         0.,  0, "E",  1.1727E+01, "S")  ! 298,    374.15,  "<10%",
    CALL CalcH("2-methylpyridine",                  "109-06-8", &
               101,  2.533E+01, -6.348E+03,          0.,         0.,  0, "E",  5.6453E+01, "S")  ! 298,    313.15,  "<5%",
    CALL CalcH("o-ethylaniline",                    "578-54-1", &
               101,  2.240E+02, -1.605E+04,  -2.933E+01,         0.,  0, "P",  2.2524E+01, "P")  ! 283.15, 363.15,  "<10%",
    CALL CalcH("dibenzopyrrole",                     "86-74-8", &
               101,  1.387E+01, -4.292E+03,          0.,         0.,  0, "E",  5.9192E-01, "S")  ! 278.15, 308.15,  "<50%",
    CALL CalcH("2,6-dimethyl-pyridine",             "108-48-5", &
               101,  2.870E+01, -7.334E+03,          0.,         0.,  0, "E",  6.0121E+01, "S")  ! 298,    313.15,  "<25%",
    CALL CalcH("3-methylpyridine",                  "108-99-6", &
               101,  2.516E+01, -6.371E+03,          0.,         0.,  0, "E",  4.4215E+01, "S")  ! 298,    313.15,  "<25%",
    CALL CalcH("4-methylpyridine",                  "108-89-4", &
               101,  2.527E+01, -6.484E+03,          0.,         0.,  0, "E",  3.3943E+01, "S")  ! 298,    313.15,  "<5%",
    CALL CalcH("2,4-dimethylaniline",                "95-68-1", &
               101,  3.271E+02, -2.102E+04,  -4.451E+01,         0.,  0, "P",  2.0392E+01, "P")  ! 273.15, 363.15,  "<25%",
    CALL CalcH("N-ethylaniline",                    "103-69-5", &
               101,  2.866E+02, -1.928E+04,  -3.821E+01,         0.,  0, "E",  6.5222E+01, "S")  ! 273.15, 363.15,  "<10%",
    ! aromatic chlorides:
    CALL CalcH("benzyl chloride",                   "100-44-7", &
               101,  2.274E+01, -4.447E+03,          0.,         0.,  0, "E",  2.5098E+03, "S")  ! 277.65, 303.15,  "<25%",
    CALL CalcH("monochlorobenzene",                 "108-90-7", &
               101,  3.469E+02, -1.808E+04,  -4.849E+01,         0.,  0, "E",  2.1129E+04, "S")  ! 274.15, 353.16,  "<10%",
    CALL CalcH("o-dichlorobenzene",                  "95-50-1", &
               101,  8.041E+02, -3.979E+04,  -1.161E+02,         0.,  0, "E",  1.0267E+04, "S")  ! 278.15, 318.15,  "<10%",
    CALL CalcH("m-dichlorobenzene",                 "541-73-1", &
               101,  2.467E+01, -4.435E+03,          0.,         0.,  0, "E",  1.7908E+04, "S")  ! 278.15, 318.15,  "<10%",
    CALL CalcH("p-dichlorobenzene",                 "106-46-7", &
               101,  5.888E+02, -3.085E+04,  -8.350E+01,         0.,  0, "E",  1.3908E+04, "S")  ! 278.15, 346.55,  "<10%",
    CALL CalcH("hexachlorobenzene",                 "118-74-1", &
               101,  3.108E+01, -6.570E+03,          0.,         0.,  0, "E",  8.4566E+03, "S")  ! 278.15, 318.15,  "<25%",
    CALL CalcH("o-chlorotoluene",                    "95-49-8", &
               101,  2.530E+01, -4.528E+03,          0.,         0.,  0, "E",  2.4693E+04, "S")  ! 274.15, 318.15,  "<25%",
    CALL CalcH("p-chlorotoluene",                   "106-43-4", &
               101,  2.621E+01, -4.854E+03,          0.,         0.,  0, "E",  2.0445E+04, "S")  ! 278.15, 318.15,  "<25%",
    CALL CalcH("2,4-dichlorotoluene",                "95-73-8", &
               101,  2.660E+01, -4.973E+03,          0.,         0.,  0, "E",  2.0337E+04, "S")  ! 278.15, 323.1,   "<10%",
    CALL CalcH("1,2,4-trichlorobenzene",            "120-82-1", &
               101,  3.085E+01, -6.372E+03,          0.,         0.,  0, "E",  1.3112E+04, "S")  ! 278.15, 318.15,  "<25%",
    CALL CalcH("1,2,3-trichlorobenzene",             "87-61-6", &
               101,  2.446E+01, -4.581E+03,          0.,         0.,  0, "E",  8.9256E+03, "S")  ! 277.15, 323.15,  "<25%",
    ! aromatic esters:
    CALL CalcH("benzyl acetate",                    "140-11-4", &
               101,  2.710E+01, -6.836E+03,          0.,         0.,  0, "E",  6.5097E+01, "S")  ! 273.15, 322.95,  "<25%",
    ! page 187 = pdf-page 193:
    CALL CalcH("di-n-propyl phthalate",             "131-16-8", &
               100,  2.556E+00,         0.,          0.,         0.,  0, "S",  2.5555E+00, "S")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("di-n-hexyl phthalate",               "84-75-3", &
               100,  6.687E+00,         0.,          0.,         0.,  0, "S",  6.6872E+00, "S")  ! 298.15, 298.15,  "<50%",
    CALL CalcH("methyl benzoate",                    "93-58-3", &
               101,  2.728E+02, -1.738E+04,  -3.673E+01,         0.,  0, "E",  1.8175E+02, "S")  ! 273.15, 363.65,  "<25%",
    CALL CalcH("ethyl benzoate",                     "93-89-0", &
               101,  1.896E+02, -1.354E+04,  -2.429E+01,         0.,  0, "E",  3.3110E+02, "S")  ! 273.15, 363.45,  "<25%",
    CALL CalcH("benzyl formate",                    "104-57-4", &
               101,  2.700E+01, -7.056E+03,          0.,         0.,  0, "E",  2.8015E+01, "S")  ! 273.15, 322.85,  "<25%",
    CALL CalcH("diethyl phthalate",                  "84-66-2", &
               101,  3.994E+02, -2.656E+04,  -5.448E+01,         0.,  0, "E",  8.9347E-01, "S")  ! 278.15, 374.15,  "<10%",
    CALL CalcH("di-n-butyl phthalate",               "84-74-2", &
               101,  9.165E+02, -5.076E+04,  -1.307E+02,         0.,  0, "E",  5.2831E+00, "S")  ! 283.15, 323.15,  "<25%",
    CALL CalcH("dimethyl phthalate",                "131-11-3", &
               101,  2.739E+02, -1.899E+04,  -3.688E+01,         0.,  0, "EP", 1.1053E+00, "P")  ! 283.15, 374.15,  "<10%",
    ! C, H, Br compounds:
    CALL CalcH("bromomethane",                       "74-83-9", &
               101,  2.316E+01, -3.762E+03,          0.,         0.,  0, "E",  3.7821E+04, "S")  ! 273.15, 305.15,  "<25%",
    CALL CalcH("bromoethane",                        "74-96-4", &
               101,  2.373E+01, -3.906E+03,          0.,         0.,  0, "E",  4.1374E+04, "S")  ! 273.15, 303.15,  "<3%",
    CALL CalcH("1,1,2,2-tetrabromoethane",           "79-27-6", &
               101,  1.252E+01, -1.268E+03,          0.,         0.,  0, "E",  3.8812E+03, "S")  ! 298.15, 323.15,  "<50%",
    CALL CalcH("1-bromopropane",                    "106-94-5", &
               101,  2.624E+01, -4.582E+03,          0.,         0.,  0, "E",  5.2435E+04, "S")  ! 273.15, 303.15,  "<5%",
    CALL CalcH("2-bromopropane",                     "75-26-3", &
               101,  3.871E+02, -2.007E+04,  -5.419E+01,         0.,  0, "E",  6.1437E+04, "S")  ! 273.15, 303.15,  "<5%",
    CALL CalcH("1-bromobutane",                     "109-65-9", &
               100,  7.447E+04,         0.,          0.,         0.,  0, "S",  7.4474E+04, "S")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("1-bromoheptane",                    "629-04-9", &
               100,  2.483E+05,         0.,          0.,         0.,  0, "S",  2.4825E+05, "S")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("1,2-dibromoethane",                 "106-93-4", &
               101,  2.246E+01, -4.218E+03,          0.,         0.,  0, "E",  4.0636E+03, "S")  ! 273.15, 363.75,  "<10%",
    CALL CalcH("m-dibromobenzene",                  "108-36-1", &
               100,  4.750E+03,         0.,          0.,         0.,  0, "E",  4.7500E+03, "D")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("bromobenzene",                      "108-86-1", &
               101,  4.526E+02, -2.357E+04,  -6.390E+01,         0.,  0, "E",  1.2337E+04, "S")  ! 283.15, 343.15,  "<10%",
    CALL CalcH("tribromomethane",                    "75-25-2", &
               101,  2.420E+01, -4.824E+03,          0.,         0.,  0, "E",  3.0319E+03, "S")  ! 283,    313.15,  "<10%",
    CALL CalcH("dibromomethane",                     "74-95-3", &
               101,  1.239E+02, -8.812E+03,  -1.508E+01,         0.,  0, "E",  4.8569E+03, "S")  ! 273.15, 363.25,  "<5%",
    CALL CalcH("2-bromobutane",                      "78-76-2", &
               100,  6.319E+04,         0.,          0.,         0.,  0, "E",  6.3195E+04, "D")  ! 298.15, 298.15,  "<10%",
    CALL CalcH("p-bromotoluene",                    "106-38-7", &
               101,  2.572E+01, -4.794E+03,          0.,         0.,  0, "E",  1.5321E+04, "S")  ! 293.1,  322.91,  "<25%",
    ! C, H, F compounds:
    CALL CalcH("methyl fluoride",                   "593-53-3", &
               101,  1.396E+02, -7.560E+03,  -1.805E+01,         0.,  0, "E",  9.5359E+04, "S")  ! 273.21, 353.4,   "<3%",
    ! page 188 = pdf-page 194:
    CALL CalcH("perfluoro-n-hexane",                "355-42-0", &
               100,  5.966E+09,  1.272E+03,          0.,         0.,  0, "E",  5.9661E+09, "D")  ! 298.15, 298.15,  "<100%",
    CALL CalcH("perfluoro-n-octane",                "307-34-6", &
               100,  6.880E+10, -2.945E+04,  -9.548E+01,         0.,  0, "E",  6.8797E+10, "D")  ! 298.15, 298.15,  "<100%",
    CALL CalcH("1,2-difluorobenzene",               "367-11-3", &
               101,  2.309E+01, -3.710E+03,          0.,         0.,  0, "E",  4.2141E+04, "S")  ! 298.15, 322.89,  "<25%",
    CALL CalcH("fluorobenzene",                     "462-06-6", &
               101,  1.836E+02, -1.116E+04,  -2.381E+01,         0.,  0, "E",  3.7174E+04, "S")  ! 273.15, 353.15,  "<10%",
    CALL CalcH("hexafluorobenzene",                 "392-56-3", &
               101,  3.285E+02, -1.839E+04,  -4.470E+01,         0.,  0, "E",  1.8597E+05, "S")  ! 281.46, 339.76,  "<3%",
    CALL CalcH("perfluoro-n-pentane",               "678-26-2", &
               100,  1.211E+09,  5.338E+02,          0.,         0.,  0, "E",  1.2114E+09, "D")  ! 298.15, 298.15,  "<100%",
    CALL CalcH("perfluoro-n-heptane",               "335-57-9", &
               100,  1.828E+10, -4.822E+03,          0.,         0.,  0, "E",  1.8284E+10, "D")  ! 298.15, 298.15,  "<100%",
    ! C, H, I compounds:
    CALL CalcH("methyl iodide",                      "74-88-4", &
               101,  3.635E+02, -1.838E+04,  -5.118E+01,         0.,  0, "E",  2.9424E+04, "S")  ! 273.15, 322.91,  "<5%",
    CALL CalcH("ethyl iodide",                       "75-03-6", &
               101,  3.349E+02, -1.753E+04,  -4.661E+01,         0.,  0, "E",  3.8621E+04, "S")  ! 273.15, 303.15,  "<3%",
    CALL CalcH("n-propyl iodide",                   "107-08-4", &
               101,  2.604E+01, -4.523E+03,          0.,         0.,  0, "E",  5.2297E+04, "S")  ! 273.15, 303.15,  "<5%",
    CALL CalcH("isopropyl iodide",                   "75-30-9", &
               101,  2.673E+01, -4.664E+03,          0.,         0.,  0, "E",  6.5461E+04, "S")  ! 273.15, 303.15,  "<5%",
    CALL CalcH("iodobenzene",                       "591-50-4", &
               101,  1.569E+00,  2.185E+03,          0.,         0.,  0, "E",  7.3212E+03, "S")  ! 298,    303.15,  "<25%",
    CALL CalcH("n-hexyl iodide",                    "638-45-9", &
               100,  1.635E+05,         0.,          0.,         0.,  0, "S",  1.6347E+05, "S")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("n-butyl iodide",                    "542-69-8", &
               100,  8.057E+04,         0.,          0.,         0.,  0, "S",  8.0572E+04, "S")  ! 298.15, 298.15,  "<25%",
    ! C, H, multihalogen compounds:
    CALL CalcH("dichlorodifluoro-methane",           "75-71-8", &
               101,  2.848E+02, -1.478E+04,  -3.876E+01,         0.,  0, "E",  1.8669E+06, "S")  ! 274,    313.9,   "<25%",
    CALL CalcH("trichlorofluoro-methane",            "75-69-4", &
               101,  2.471E+01, -3.438E+03,          0.,         0.,  0, "E",  5.2851E+05, "S")  ! 274,    313.9,   "<10%",
    CALL CalcH("1,1,2-trichlorotrifluoro-ethane",    "76-13-1", &
               101,  2.823E+01, -4.110E+03,          0.,         0.,  0, "E",  1.8719E+06, "S")  ! 275.15, 303.15,  "<25%",
    ! C, H, NO2 compounds:
    CALL CalcH("nitromethane",                       "75-52-5", &
               101,  2.448E+02, -1.397E+04,  -3.386E+01,         0.,  0, "E",  1.4923E+02, "S")  ! 273.15, 363.15,  "<10%",
    CALL CalcH("nitroethane",                        "79-24-3", &
               101,  1.804E+02, -1.174E+04,  -2.377E+01,         0.,  0, "E",  2.5784E+02, "S")  ! 273.15, 363.25,  "<5%",
    CALL CalcH("1-nitropropane",                    "108-03-2", &
               101,  2.032E+02, -1.313E+04,  -2.688E+01,         0.,  0, "E",  4.1378E+02, "S")  ! 273.15, 363.65,  "<10%",
    CALL CalcH("2-nitropropane",                     "79-46-9", &
               101,  1.697E+02, -1.138E+04,  -2.195E+01,         0.,  0, "E",  6.5009E+02, "S")  ! 292.85, 363.35,  "<3%",
    ! page 189 = pdf-page 195:
    CALL CalcH("1-nitrobutane",                     "627-05-4", &
               100,  7.245E+02,         0.,          0.,         0.,  0, "S",  7.2451E+02, "S")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("o-nitrotoluene",                     "88-72-2", &
               101,  2.493E+01, -5.806E+03,          0.,         0.,  0, "E",  2.3476E+02, "S")  ! 283.15, 313.15,  "<10%",
    CALL CalcH("p-nitrotoluene",                     "99-99-0", &
               101,  2.804E+01, -7.129E+03,          0.,         0.,  0, "E",  6.2304E+01, "S")  ! 281,    338,     "<10%",
    CALL CalcH("m-nitrotoluene",                     "99-08-1", &
               101,  2.866E+01, -6.962E+03,          0.,         0.,  0, "E",  2.0274E+02, "S")  ! 283.15, 313.15,  "<25%",
    CALL CalcH("nitrobenzene",                       "98-95-3", &
               101,  2.229E+02, -1.487E+04,  -2.953E+01,         0.,  0, "E",  1.1835E+02, "S")  ! 283.15, 372.85,  "<5%",
    CALL CalcH("2,4-dinitrotoluene",                "121-14-2", &
               101,  2.605E+02, -1.830E+04,  -3.489E+01,         0.,  0, "E",  1.4084E+00, "S")  ! 277.15, 334.95,  "<5%",
    CALL CalcH("2,6-dinitrotoluene",                "606-20-2", &
               101,  2.793E+01, -7.598E+03,          0.,         0.,  0, "E",  1.1571E+01, "S")  ! 278,    314,     "<5%",
    CALL CalcH("3,4-dinitrotoluene",                "610-39-9", &
               101,  3.267E+01, -9.586E+03,          0.,         0.,  0, "E",  1.6741E+00, "S")  ! 278.2,  313.2,   "<5%",
    CALL CalcH("2,4,6-trinitrotoluene",             "118-96-7", &
               101,  1.351E+02, -1.270E+04,  -1.661E+01,         0.,  0, "E",  1.2083E-01, "S")  ! 275.55, 334.15,  "<10%",
    ! C1/C2 aliphatic chlorides:
    CALL CalcH("carbon tetrachloride",               "56-23-5", &
               101,  3.746E+02, -1.968E+04,  -5.206E+01,         0.,  0, "E",  1.6765E+05, "S")  ! 273.15, 366.15,  "<5%",
    CALL CalcH("methyl chloride",                    "74-87-3", &
               101,  1.775E+02, -9.830E+03,  -2.345E+01,         0.,  0, "E",  5.2937E+04, "S")  ! 273.15, 366.15,  "<10%",
    CALL CalcH("ethyl chloride",                     "75-00-3", &
               101,  1.936E+02, -1.056E+04,  -2.581E+01,         0.,  0, "E",  6.6090E+04, "S")  ! 281.15, 366.15,  "<25%",
    CALL CalcH("vinyl chloride",                     "75-01-4", &
               101,  3.305E+01, -3.487E+03,  -1.681E+00,         0.,  0, "E",  1.3062E+05, "S")  ! 273.35, 366.15,  "<25%",
    CALL CalcH("dichloromethane",                    "75-09-2", &
               101,  1.263E+02, -7.983E+03,  -1.578E+01,         0.,  0, "E",  1.4546E+04, "S")  ! 273.15, 366.15,  "<10%",
    CALL CalcH("chloroform",                         "67-66-3", &
               101,  2.287E+02, -1.327E+04,  -3.056E+01,         0.,  0, "E",  2.2108E+04, "S")  ! 273.15, 366.15,  "<3%",
    CALL CalcH("1,1-dichloroethane",                 "75-34-3", &
               101,  2.587E+02, -1.445E+04,  -3.508E+01,         0.,  0, "E",  3.2146E+04, "S")  ! 273.15, 366.15,  "<5%",
    CALL CalcH("1,2-dichloroethane",                "107-06-2", &
               101,  2.782E+02, -1.560E+04,  -3.810E+01,         0.,  0, "E",  6.6406E+03, "S")  ! 273.15, 366.15,  "<10%",
    CALL CalcH("1,1,2-trichloroethane",              "79-00-5", &
               101,  1.159E+02, -8.493E+03,  -1.384E+01,         0.,  0, "E",  4.9329E+03, "S")  ! 273.15, 363.95,  "<5%",
    CALL CalcH("hexachloroethane",                   "67-72-1", &
               101,  3.026E+01, -6.068E+03,          0.,         0.,  0, "E",  2.0130E+04, "S")  ! 283.15, 303.15,  "<25%",
    CALL CalcH("1,1,1-trichloroethane",              "71-55-6", &
               101,  7.812E+01, -6.437E+03,  -7.910E+00,         0.,  0, "E",  9.4916E+04, "S")  ! 273.15, 348.15,  "<10%",
    CALL CalcH("1,1,1,2-tetrachloroethane",         "630-20-6", &
               101,  2.594E+01, -4.909E+03,          0.,         0.,  0, "E",  1.3082E+04, "S")  ! 273.15, 323.15,  "<10%",
    CALL CalcH("1,1,2,2-tetrachloroethane",          "79-34-5", &
               101,  2.448E+01, -4.984E+03,          0.,         0.,  0, "E",  2.3451E+03, "S")  ! 283.15, 328.65,  "<10%",
    CALL CalcH("trichloroethylene",                  "79-01-6", &
               101,  2.642E+02, -1.488E+04,  -3.570E+01,         0.,  0, "E",  5.4319E+04, "S")  ! 274.22, 366.15,  "<5%",
    ! page 190 = pdf-page 196:
    CALL CalcH("tetrachloroethylene",               "127-18-4", &
               101,  3.077E+02, -1.717E+04,  -4.188E+01,         0.,  0, "E",  9.8707E+04, "S")  ! 273.15, 364.95,  "<10%",
    CALL CalcH("cis-1,2-dichloroethylene",          "156-59-2", &
               101,  2.228E+02, -1.290E+04,  -2.975E+01,         0.,  0, "E",  2.3105E+04, "S")  ! 281.15, 366.15,  "<10%",
    CALL CalcH("trans-1,2-dichloroethylene",        "156-60-5", &
               101,  2.410E+01, -3.940E+03,          0.,         0.,  0, "E",  5.3633E+04, "S")  ! 283.15, 323.15,  "<10%",
    CALL CalcH("pentachloroethane",                  "76-01-7", &
               101,  3.521E+01, -7.667E+03,          0.,         0.,  0, "E",  1.3266E+04, "S")  ! 293.15, 298,     "<25%",
    CALL CalcH("1,1-dichloroethylene",               "75-35-4", &
               101,  3.060E+02, -1.643E+04,  -4.191E+01,         0.,  0, "E",  1.7069E+05, "S")  ! 275.15, 363.65,  "<10%",
    ! C3 & higher aliphatic chlorides:
    CALL CalcH("1,4-dichlorobutane",                "110-56-5", &
               101,  2.001E+01, -3.599E+03,          0.,         0.,  0, "E",  2.7961E+03, "S")  ! 286.65, 298.05,  "<10%",
    CALL CalcH("1,5-dichloropentane",               "628-76-2", &
               101,  2.216E+01, -4.045E+03,          0.,         0.,  0, "E",  5.3788E+03, "S")  ! 274.15, 295.85,  "<50%",
    CALL CalcH("1,2-dichloropropane",                "78-87-5", &
               101,  2.546E+02, -1.434E+04,  -3.454E+01,         0.,  0, "E",  1.6001E+04, "S")  ! 273.15, 353.35,  "<10%",
    CALL CalcH("isopropyl chloride",                 "75-29-6", &
               101,  2.621E+01, -4.393E+03,          0.,         0.,  0, "E",  9.6129E+04, "S")  ! 273.15, 303.15,  "<10%",
    CALL CalcH("1,2,3-trichloropropane",             "96-18-4", &
               101,  1.366E+01, -1.857E+03,          0.,         0.,  0, "E",  1.6851E+03, "S")  ! 275.05, 318.15,  "<25%",
    CALL CalcH("hexachloro-1,3-butadiene",           "87-68-3", &
               101,  2.978E+01, -5.451E+03,          0.,         0.,  0, "E",  9.8863E+04, "S")  ! 275.15, 298.15,  "<50%",
    CALL CalcH("propyl chloride",                   "540-54-5", &
               101,  2.625E+01, -4.489E+03,          0.,         0.,  0, "E",  7.2709E+04, "S")  ! 273.15, 310.15,  "<10%",
    CALL CalcH("n-butyl chloride",                  "109-69-3", &
               101,  2.644E+01, -4.455E+03,          0.,         0.,  0, "E",  9.8517E+04, "S")  ! 273.35, 310.15,  "<10%",
    CALL CalcH("sec-butyl chloride",                 "78-86-4", &
               101,  2.691E+01, -4.502E+03,          0.,         0.,  0, "E",  1.3477E+05, "S")  ! 275.05, 298.05,  "<10%",
    CALL CalcH("1-chloropentane",                   "543-59-9", &
               101,  4.078E+02, -2.100E+04,  -5.714E+01,         0.,  0, "E",  1.2764E+05, "S")  ! 275.65, 310.15,  "<5%",
    CALL CalcH("1,3-dichloropropane",               "142-28-9", &
               101,  1.898E+01, -3.134E+03,          0.,         0.,  0, "E",  4.7562E+03, "S")  ! 275.05, 303.15,  "<25%",
    ! cycloaliphatic alcohols:
    CALL CalcH("cyclohexanol",                      "108-93-0", &
               101,  2.835E+01, -7.692E+03,          0.,         0.,  0, "E",  1.2847E+01, "S")  ! 298,    333.15,  "<25%",
    ! cycloalkanes:
    CALL CalcH("cyclopentane",                      "287-92-3", &
               101,  2.262E+02, -1.246E+04,  -2.995E+01,         0.,  0, "E",  9.8100E+05, "S")  ! 298,    426.25,  "<10%",
    CALL CalcH("cyclohexane",                       "110-82-7", &
               101, -1.486E+03,  3.489E+04,   2.667E+02, -4.584E-01,  1, "E",  9.4972E+05, "S")  ! 274.25, 424.15,  "<25%",
    CALL CalcH("cycloheptane",                      "291-64-5", &
               100,  8.267E+05, -3.094E+04,  -7.154E+01,         0.,  0, "P",  8.2667E+05, "P")  ! 298.15, 298.15,  "<10%",
    ! page 191 = pdf-page 197:
    CALL CalcH("cyclooctane",                       "292-64-8", &
               101,  1.860E+02, -1.172E+04,  -2.337E+01,         0.,  0, "E",  7.6345E+05, "S")  ! 273.15, 313.15,  "<10%",
    ! cycloalkenes:
    CALL CalcH("cyclopentene",                      "142-29-0", &
               101,  2.369E+01, -3.321E+03,          0.,         0.,  0, "E",  2.8372E+05, "S")  ! 283.15, 298.15,  "<50%",
    CALL CalcH("cyclohexene",                       "110-83-8", &
               101,  2.423E+01, -3.602E+03,          0.,         0.,  0, "E",  1.8948E+05, "S")  ! 278.26, 318.36,  "<25%",
    CALL CalcH("cycloheptene",                      "628-92-2", &
               100,  2.705E+05, -8.903E+03,          0.,  1.478E+06, -2, "E",  2.7050E+05, "D")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("cyclooctene",                       "931-88-4", &
               101,  1.856E+02, -1.135E+04,  -2.370E+01,         0.,  0, "E",  2.6909E+05, "S")  ! 273.15, 313.15,  "<5%",
    ! dialkenes:
    CALL CalcH("1,4-cyclohexadiene",                "628-41-1", &
               101,  2.424E+01,   -4047.17,          0.,         0.,  0, "E",  4.3001E+04, "S")  ! 278.26, 318.36,  "<25%",
    ! dimethylalkanes:
    CALL CalcH("2,2-dimethylbutane",                 "75-83-2", &
               100,  9.781E+06,         0.,          0.,         0.,  0, "S",  9.7812E+06, "S")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("2,3-dimethylbutane",                 "79-29-8", &
               101,  3.898E+02, -2.037E+04,  -5.366E+01,         0.,  0, "E",  7.1887E+06, "S")  ! 273.15, 422.65,  "<25%",
    CALL CalcH("2,4-dimethylpentane",               "108-08-7", &
               100,  1.937E+07,         0.,          0.,         0.,  0, "S",  1.9367E+07, "S")  ! 298.15, 298.15,  "<50%",
    CALL CalcH("3,3-dimethylpentane",               "562-49-2", &
               101, -1.836E+03,  4.668E+04,   3.255E+02, -5.334E-01,  1, "E",  1.0373E+07, "S")  ! 298.15, 423.64,  "<25%",
    CALL CalcH("2,2-dimethylhexane",                "590-73-8", &
               101,  6.831E+02, -3.353E+04,  -9.718E+01,         0.,  0, "E",  2.2630E+07, "S")  ! 273.15, 313.15,  "<5%",
    CALL CalcH("2,5-dimethylhexane",                "592-13-2", &
               101,  8.022E+02, -3.833E+04,  -1.153E+02,         0.,  0, "E",  2.2712E+07, "S")  ! 273.15, 313.15,  "<3%",
    ! diphenyl/polyaromatics:
    CALL CalcH("biphenyl",                           "92-52-4", &
               101,  6.933E+02, -3.653E+04,  -9.889E+01,         0.,  0, "E",  1.5402E+03, "S")  ! 293.15, 337.65,  "<10%",
    ! epoxides:
    CALL CalcH("1,4-dioxane",                       "123-91-1", &
               101,  1.235E+02, -1.032E+04,  -1.500E+01,         0.,  0, "E",  2.9564E+01, "S")  ! 273.35, 374.15,  "<25%",
    CALL CalcH("tetrahydropyran",                   "142-68-7", &
               101,  2.170E+02, -1.432E+04,  -2.851E+01,         0.,  0, "E",  6.7975E+02, "S")  ! 273.35, 328.15,  "<5%",
    CALL CalcH("tetrahydrofuran",                   "109-99-9", &
               101,  2.492E+01, -5.663E+03,          0.,         0.,  0, "E",  3.7471E+02, "S")  ! 273.35, 328.15,  "<10%",
    CALL CalcH("dibenzofuran",                      "132-64-9", &
               101,  2.617E+01, -5.831E+03,          0.,         0.,  0, "E",  7.4383E+02, "S")  ! 278.15, 318.15,  "<25%",
    ! formates:
    CALL CalcH("methyl formate",                    "107-31-3", &
               101,  1.995E+01, -3.806E+03,          0.,         0.,  0, "E",  1.3237E+03, "D")  ! 285.65, 333.25,  "<10%",
    CALL CalcH("ethyl formate",                     "109-94-4", &
               101,  2.076E+01, -3.974E+03,          0.,         0.,  0, "E",  1.6926E+03, "S")  ! 293.15, 333.25,  "<25%",
    CALL CalcH("n-propyl formate",                  "110-74-7", &
               101,  2.356E+01, -4.788E+03,          0.,         0.,  0, "E",  1.8180E+03, "S")  ! 272.15, 333.15,  "<10%",
    ! page 192 = pdf-page 198:
    CALL CalcH("n-butyl formate",                   "592-84-7", &
               101,  2.007E+01, -3.697E+03,          0.,         0.,  0, "E",  2.1415E+03, "P")  ! 273.15, 363.75,  "<25%",
    CALL CalcH("isobutyl formate",                  "542-55-2", &
               101,  2.564E+01, -5.223E+03,          0.,         0.,  0, "E",  3.3686E+03, "S")  ! 273.15, 333.55,  "<10%",
    CALL CalcH("n-pentyl formate",                  "638-49-3", &
               101,  2.720E+02, -1.682E+04,  -3.642E+01,         0.,  0, "E",  3.0716E+03, "S")  ! 273.15, 363.65,  "<25%",
    CALL CalcH("tert-butyl formate",                "762-75-4", &
               101,  2.041E+01, -3.621E+03,          0.,         0.,  0, "E",  3.8948E+03, "S")  ! 278.15, 308.15,  "<5%",
    ! ketones:
    CALL CalcH("acetone",                            "67-64-1", &
               101,  1.327E+02, -1.016E+04,  -1.639E+01,         0.,  0, "E",  1.9892E+02, "S")  ! 273,    373.15,  "<10%",
    CALL CalcH("methyl ethyl ketone",                "78-93-3", &
               101,  2.253E+02, -1.451E+04,  -3.000E+01,         0.,  0, "E",  3.0126E+02, "S")  ! 273,    373.15,  "<10%",
    CALL CalcH("3-pentanone",                        "96-22-0", &
               101,  2.558E+02, -1.603E+04,  -3.433E+01,         0.,  0, "E",  5.8894E+02, "S")  ! 273.15, 310.2,   "<10%",
    CALL CalcH("methyl isobutyl ketone",            "108-10-1", &
               101,  2.306E+02, -1.522E+04,  -3.034E+01,         0.,  0, "E",  8.1503E+02, "S")  ! 273.15, 363.55,  "<10%",
    CALL CalcH("3-heptanone",                       "106-35-4", &
               101,  2.236E+02, -1.559E+04,  -2.890E+01,         0.,  0, "E",  7.8587E+02, "S")  ! 273.15, 363.25,  "<10%",
    CALL CalcH("4-heptanone",                       "123-19-3", &
               101,  2.766E+02, -1.775E+04,  -3.687E+01,         0.,  0, "E",  1.1765E+03, "S")  ! 273.15, 363.35,  "<10%",
    CALL CalcH("3-hexanone",                        "589-38-8", &
               101,  3.027E+01, -7.070E+03,          0.,         0.,  0, "E",  7.0108E+02, "S")  ! 293.15, 310.2,   "<10%",
    CALL CalcH("2-pentanone",                       "107-87-9", &
               101,  2.179E+02, -1.450E+04,  -2.865E+01,         0.,  0, "E",  4.0969E+02, "S")  ! 273.15, 363.65,  "<10%",
    CALL CalcH("methyl isopropyl ketone",           "563-80-4", &
               101,  2.737E+02, -1.686E+04,  -3.702E+01,         0.,  0, "P",  5.1455E+02, "P")  ! 265.15, 453.15,  "<10%",
    CALL CalcH("2-hexanone",                        "591-78-6", &
               101,  1.536E+02, -1.206E+04,  -1.876E+01,         0.,  0, "E",  5.4080E+02, "S")  ! 273.15, 312.75,  "<10%",
    CALL CalcH("2-heptanone",                       "110-43-0", &
               101,  2.878E+02, -1.827E+04,  -3.858E+01,         0.,  0, "E",  8.0639E+02, "S")  ! 273.15, 363.65,  "<5%",
    CALL CalcH("5-methyl-2-hexanone",               "110-12-3", &
               101,  3.052E+02, -1.904E+04,  -4.117E+01,         0.,  0, "E",  8.8878E+02, "S")  ! 273.15, 362.85,  "<5%",
    CALL CalcH("3,3-dimethyl-2-butanone",            "75-97-8", &
               101,  1.694E+02, -1.217E+04,  -2.131E+01,         0.,  0, "E",  1.2372E+03, "S")  ! 273.15, 363.35,  "<5%",
    CALL CalcH("diisobutyl ketone",                 "108-83-8", &
               101,  3.516E+02, -2.165E+04,  -4.750E+01,         0.,  0, "E",  4.3295E+03, "S")  ! 273.15, 363.85,  "<10%",
    CALL CalcH("diisopropyl ketone",                "565-80-0", &
               101,  1.603E+02, -1.197E+04,  -1.972E+01,         0.,  0, "E",  2.4265E+03, "S")  ! 273.15, 363.45,  "<10%",
    CALL CalcH("5-nonanone",                        "502-56-7", &
               101,  2.901E+02, -1.938E+04,  -3.824E+01,         0.,  0, "E",  1.3894E+03, "S")  ! 273.15, 363.75,  "<25%",
    CALL CalcH("2-nonanone",                        "821-55-6", &
               101,  3.154E+02, -2.001E+04,  -4.227E+01,         0.,  0, "E",  1.5963E+03, "S")  ! 273.15, 364.35,  "<25%",
    CALL CalcH("acetylacetone",                     "123-54-6", &
               101,  1.685E+01, -3.780E+03,          0.,         0.,  0, "E",  6.4870E+01, "S")  ! 303.15, 373.15,  "<25%",
    CALL CalcH("cyclopentanone",                    "120-92-3", &
               101,  2.341E+01, -5.848E+03,          0.,         0.,  0, "E",  4.4727E+01, "S")  ! 293.2,  353.15,  "<100%",
    ! page 193 = pdf-page 199:
    CALL CalcH("cyclohexanone",                     "108-94-1", &
               101,  2.494E+01, -6.365E+03,          0.,         0.,  0, "E",  3.6312E+01, "S")  ! 298,    353.15,  "<10%",
    CALL CalcH("2-octanone",                        "111-13-7", &
               101,  3.274E+02, -2.068E+04,  -4.406E+01,         0.,  0, "E",  1.0319E+03, "S")  ! 273.15, 364.15,  "<25%",
    CALL CalcH("acetophenone",                       "98-86-2", &
               101,  4.071E+02, -2.464E+04,  -5.626E+01,         0.,  0, "E",  5.0333E+01, "S")  ! 278,    318.15,  "<25%",
    CALL CalcH("3-octanone",                        "106-68-3", &
               101,  1.965E+02, -1.427E+04,  -2.483E+01,         0.,  0, "E",  1.3047E+03, "S")  ! 273.15, 364.15,  "<10%",
    CALL CalcH("3-nonanone",                        "925-78-0", &
               101,  2.908E+01, -6.587E+03,          0.,         0.,  0, "E",  1.0827E+03, "S")  ! 273.15, 353.35,  "<25%",
    CALL CalcH("2,6,8-trimethyl-4-nonanone",        "123-18-2", &
               101,  5.209E+03, -4.277E+03,  -9.471E+00,         0.,  0, "P",  5.2094E+03, "P")  ! 298.15, 298.15,  "<25%",
    ! mercaptans:
    CALL CalcH("isopropyl mercaptan",                "75-33-2", &
               101,  2.295E+01, -3.670E+03,          0.,         0.,  0, "E",  4.1922E+04, "S")  ! 288.15, 318.15,  "<10%",
    CALL CalcH("n-butyl mercaptan",                 "109-79-5", &
               101,  4.008E+02, -2.101E+04,  -5.613E+01,         0.,  0, "E",  3.7378E+04, "S")  ! 288.15, 333.15,  "<10%",
    ! methylalkanes:
    CALL CalcH("2-methylpentane",                   "107-83-5", &
               101,  4.013E+02, -2.096E+04,  -5.529E+01,         0.,  0, "E",  8.9514E+06, "S")  ! 273.15, 422.65,  "<25%",
    CALL CalcH("3-methylpentane",                    "96-14-0", &
               100,  9.326E+06,  1.259E+04,   5.099E+01, -4.402E-02,  1, "S",  9.3257E+06, "S")  ! 298.15, 298.15,  "<3%",
    CALL CalcH("2-methylhexane",                    "591-76-4", &
               100,  1.920E+07,         0.,          0.,         0.,  0, "E",  1.9204E+07, "D")  ! 298.15, 298.15,  "<10%",
    CALL CalcH("3-methylhexane",                    "589-34-4", &
               100,  1.716E+07,         0.,          0.,         0.,  0, "E",  1.7157E+07, "S")  ! 298.15, 298.15,  "<10%",
    CALL CalcH("3-methylheptane",                   "589-81-1", &
               100,  2.094E+07,         0.,          0.,         0.,  0, "E",  2.0935E+07, "S")  ! 298.15, 298.15,  "<10%",
    CALL CalcH("4-methylheptane",                   "589-53-7", &
               100,  2.066E+07,         0.,          0.,         0.,  0, "E",  2.0657E+07, "D")  ! 298.15, 298.15,  "<5%",
    ! methylalkenes:
    CALL CalcH("2-methyl-2-butene",                 "513-35-9", &
               101,  1.486E+02, -8.954E+03,  -1.839E+01,         0.,  0, "E",  9.6127E+05, "S")  ! 273.16, 322.88,  "<3%",
    ! n-alcohols:
    CALL CalcH("methanol",                           "67-56-1", &
               101,  1.040E+02, -9.075E+03,  -1.233E+01,         0.,  0, "E",  2.8376E+01, "S")  ! 273,    373.15,  "<10%",
    CALL CalcH("ethanol",                            "64-17-5", &
               101,  1.745E+02, -1.301E+04,  -2.237E+01,         0.,  0, "E",  3.1518E+01, "S")  ! 273,    373.15,  "<10%",
    CALL CalcH("1-propanol",                         "71-23-8", &
               101,  2.216E+02, -1.567E+04,  -2.904E+01,         0.,  0, "E",  3.9058E+01, "S")  ! 273,    373.15,  "<10%",
    CALL CalcH("1-butanol",                          "71-36-3", &
               101,  3.189E+02, -2.039E+04,  -4.329E+01,         0.,  0, "E",  4.7161E+01, "S")  ! 273,    373.15,  "<10%",
    CALL CalcH("1-pentanol",                         "71-41-0", &
               101,  2.743E+02, -1.835E+04,  -3.661E+01,         0.,  0, "E",  6.3569E+01, "S")  ! 273.35, 371.15,  "<10%",
    CALL CalcH("1-hexanol",                         "111-27-3", &
               101,  2.987E+02, -1.978E+04,  -4.003E+01,         0.,  0, "E",  7.6111E+01, "S")  ! 298,    363.15,  "<25%",
    ! page 194 = pdf-page 200:
    CALL CalcH("1-heptanol",                        "111-70-6", &
               101,  3.399E+02, -2.193E+04,  -4.593E+01,         0.,  0, "EP", 1.0435E+02, "S")  ! 273,    375,     "<10%",
    CALL CalcH("1-octanol",                         "111-87-5", &
               101,  4.751E+02, -2.858E+04,  -6.567E+01,         0.,  0, "EP", 1.5895E+02, "S")  ! 288.15, 363.45,  "<10%",
    CALL CalcH("1-nonanol",                         "143-08-8", &
               101,  4.823E+02, -2.911E+04,  -6.658E+01,         0.,  0, "E",  2.0923E+02, "S")  ! 288.15, 363.15,  "<25%",
    CALL CalcH("1-decanol",                         "112-30-1", &
               101,  5.385E+02, -3.155E+04,  -7.495E+01,         0.,  0, "E",  3.0679E+02, "S")  ! 288.15, 363.15,  "<10%",
    CALL CalcH("1-undecanol",                       "112-42-5", &
               100,  4.030E+02,         0.,          0.,         0.,  0, "P",  4.0302E+02, "P")  ! 298.15, 298.15,  "<50%",
    CALL CalcH("1-dodecanol",                       "112-53-8", &
               100,  4.668E+02,         0.,          0.,         0.,  0, "P",  4.6684E+02, "P")  ! 298.15, 298.15,  "<50%",
    ! n-aliphatic primary amines:
    CALL CalcH("ethylamine",                         "75-04-7", &
               100,  6.618E+01,  1.162E+04,   2.974E+01,         0.,  0, "S",  6.6181E+01, "S")  ! 298.15, 298.15,  "<10%",
    CALL CalcH("n-propylamine",                     "107-10-8", &
               100,  8.255E+01,  1.063E+04,   2.880E+01,         0.,  0, "S",  8.2550E+01, "S")  ! 298.15, 298.15,  "<10%",
    CALL CalcH("n-butylamine",                      "109-73-9", &
               100,  1.030E+02,  1.277E+04,   3.289E+01,         0.,  0, "S",  1.0298E+02, "S")  ! 298.15, 298.15,  "<10%",
    CALL CalcH("n-pentylamine",                     "110-58-7", &
               100,  1.284E+02,  8.883E+03,   2.125E+01,         0.,  0, "S",  1.2844E+02, "S")  ! 298.15, 298.15,  "<10%",
    CALL CalcH("n-hexylamine",                      "111-26-2", &
               100,  1.602E+02,         0.,          0.,         0.,  0, "S",  1.6021E+02, "S")  ! 298.15, 298.15,  "<10%",
    CALL CalcH("n-heptylamine",                     "111-68-2", &
               100,  1.999E+02,         0.,          0.,         0.,  0, "S",  1.9986E+02, "S")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("n-octylamine",                      "111-86-4", &
               100,  2.493E+02,         0.,          0.,         0.,  0, "S",  2.4929E+02, "S")  ! 298.15, 298.15,  "<25%",
    ! n-alkanes:
    CALL CalcH("n-pentane",                         "109-66-0", &
               101, -1.239E+03,  2.706E+04,   2.257E+02, -4.087E-01,  1, "E",  6.5485E+06, "S")  ! 273.15, 422.65,  "<25%",
    CALL CalcH("n-hexane",                          "110-54-3", &
               101,  4.056E+02, -2.109E+04,  -5.595E+01,         0.,  0, "E",  9.0978E+06, "S")  ! 273.15, 443.15,  "<10%",
    CALL CalcH("n-heptane",                         "142-82-5", &
               101,  4.213E+02, -2.189E+04,  -5.819E+01,         0.,  0, "E",  1.2916E+07, "S")  ! 273.15, 443.75,  "<25%",
    CALL CalcH("n-octane",                          "111-65-9", &
               101,  5.052E+02, -2.622E+04,  -7.032E+01,         0.,  0, "E",  1.6258E+07, "S")  ! 273.15, 456.15,  "<25%",
    CALL CalcH("n-nonane",                          "111-84-2", &
               101,  7.275E+02, -3.721E+04,  -1.028E+02,         0.,  0, "E",  3.0731E+07, "S")  ! 287.95, 409.75,  "<50%",
    CALL CalcH("n-decane",                          "124-18-5", &
               100,  4.165E+07,         0.,          0.,         0.,  0, "S",  4.1648E+07, "S")  ! 298.15, 298.15,  "<50%",
    ! n-alkylbenzenes:
    CALL CalcH("benzene",                            "71-43-2", &
               101,  6.091E+02, -2.377E+04,  -9.619E+01,  9.737E-02,  1, "E",  3.1114E+04, "S")  ! 273.15, 527.15,  "<3%",
    CALL CalcH("toluene",                           "108-88-3", &
               101,  6.605E+02, -2.545E+04,  -1.049E+02,  1.101E-01,  1, "E",  3.3054E+04, "S")  ! 261.7,  473.55,  "<5%",
    CALL CalcH("ethylbenzene",                      "100-41-4", &
               101,  3.619E+02, -1.988E+04,  -4.994E+01,         0.,  0, "E",  4.2613E+04, "S")  ! 273.15, 373.15,  "<5%",
    CALL CalcH("n-propylbenzene",                   "103-65-1", &
               101,  2.463E+02, -1.450E+04,  -3.276E+01,         0.,  0, "E",  5.9359E+04, "S")  ! 273.65, 328.15,  "<10%",
    ! page 195 = pdf-page 201:
    CALL CalcH("n-butylbenzene",                    "104-51-8", &
               101,  4.132E+02, -2.226E+04,  -5.744E+01,         0.,  0, "E",  7.8555E+04, "S")  ! 273.65, 373.15,  "<5%",
    CALL CalcH("n-heptylbenzene",                  "1078-71-3", &
               100,  2.079E+05, -1.779E+04,  -5.443E+01,         0.,  0, "P",  2.0792E+05, "P")  ! 298.15, 298.15,  "<10%",
    CALL CalcH("n-decylbenzene",                    "104-72-3", &
               100,  7.285E+05,         0.,          0.,         0.,  0, "S",  7.2846E+05, "S")  ! 298.15, 298.15,  "<10%",
    CALL CalcH("n-pentylbenzene",                   "538-68-1", &
               101,  4.005E+02, -2.240E+04,  -5.510E+01,         0.,  0, "E",  1.0026E+05, "S")  ! 280.15, 318.15,  "<10%",
    CALL CalcH("n-hexylbenzene",                   "1077-16-3", &
               101,  3.675E+02, -2.136E+04,  -4.985E+01,         0.,  0, "E",  1.3470E+05, "S")  ! 278.15, 318.15,  "<10%",
    CALL CalcH("n-octylbenzene",                   "2189-60-8", &
               100,  3.005E+05,  5.570E+03,          0.,         0.,  0, "P",  3.0050E+05, "P")  ! 298.15, 298.15,  "<10%",
    CALL CalcH("n-nonylbenzene",                   "1081-77-2", &
               100,  3.866E+05, -4.112E+04,  -2.080E+02,  2.466E-01,  1, "P",  3.8660E+05, "P")  ! 298.15, 298.15,  "<25%",
    ! naphthalenes:
    CALL CalcH("naphthalene",                        "91-20-3", &
               101,  3.592E+02, -2.027E+04,  -4.973E+01,         0.,  0, "E",  2.5234E+03, "S")  ! 275.05, 346.55,  "<3%",
    CALL CalcH("1-methyl-naphthalene",               "90-12-0", &
               101,  2.722E+01, -5.758E+03,          0.,         0.,  0, "E",  2.7244E+03, "S")  ! 277.25, 304.85,  "<10%",
    CALL CalcH("2-methyl-naphthalene",               "91-57-6", &
               101,  2.506E+01, -5.113E+03,          0.,         0.,  0, "E",  2.7289E+03, "S")  ! 277.25, 304.15,  "<25%",
    CALL CalcH("1-ethyl-naphthalene",              "1127-76-0", &
               101,  2.778E+01, -5.919E+03,          0.,         0.,  0, "E",  2.7757E+03, "S")  ! 281.55, 304.85,  "<10%",
    ! nitriles:
    CALL CalcH("acetonitrile",                       "75-05-8", &
               101, -2.253E+00, -3.365E+03,   3.205E+00,         0.,  0, "E",  1.1210E+02, "S")  ! 273,    353.15,  "<5%",
    CALL CalcH("propionitrile",                     "107-12-0", &
               101,  9.456E+01, -7.820E+03,  -1.110E+01,         0.,  0, "E",  1.6565E+02, "S")  ! 278.15, 298.15,  "<25%",
    CALL CalcH("acrylonitrile",                     "107-13-1", &
               101,  1.926E+01, -3.863E+03,          0.,         0.,  0, "E",  5.4807E+02, "S")  ! 273.15, 343.15,  "<3%",
    CALL CalcH("methacrylonitrile",                 "126-98-7", &
               101,  2.729E+02, -1.537E+04,  -3.756E+01,         0.,  0, "P",  1.5916E+03, "P")  ! 273.15, 353.15,  "<10%",
    CALL CalcH("n-butyronitrile",                   "109-74-0", &
               101,  1.333E+02, -1.001E+04,  -1.650E+01,         0.,  0, "E",  2.9080E+02, "S")  ! 273.15, 313.15,  "<3%",
    CALL CalcH("valeronitrile",                     "110-59-8", &
               101,  2.630E+01, -6.092E+03,          0.,         0.,  0, "E",  3.5359E+02, "S")  ! 273.15, 303.15,  "<25%",
    CALL CalcH("isobutyronitrile",                   "78-82-0", &
               101,  2.382E+01, -5.246E+03,          0.,         0.,  0, "E",  5.0410E+02, "S")  ! 273.15, 303.15,  "<25%",
    CALL CalcH("benzonitrile",                      "100-47-0", &
               101,  3.097E+02, -1.834E+04,  -4.267E+01,         0.,  0, "E",  1.6637E+02, "S")  ! 273.15, 363.15,  "<25%",
    CALL CalcH("vinylacetonitrile",                 "109-75-1", &
               101,  2.209E+01, -4.916E+03,          0.,         0.,  0, "E",  2.7177E+02, "S")  ! 273.15, 303.15,  "<5%",
    CALL CalcH("phenylacetonitrile",                "140-29-4", &
               101,  2.665E+01, -6.778E+03,          0.,         0.,  0, "E",  5.0250E+01, "S")  ! 273.15, 303.15,  "<100%",
    ! nitroamines:
    CALL CalcH("o-nitroaniline",                     "88-74-4", &
               101,  2.256E+01, -6.802E+03,          0.,         0.,  0, "E",  7.7514E-01, "S")  ! 298.15, 313.25,  "<50%",
    ! page 196 = pdf-page 202:
    CALL CalcH("p-nitroaniline",                    "100-01-6", &
               101,  2.228E+01, -8.150E+03,          0.,         0.,  0, "E",  6.3686E-03, "S")  ! 293.15, 313.25,  "<10%",
    ! organic salts:
    CALL CalcH("diethyl carbonate",                 "105-58-8", &
               101,  3.022E+02, -1.859E+04,  -4.099E+01,         0.,  0, "E",  5.3166E+02, "S")  ! 273.15, 363.45,  "<5%",
    CALL CalcH("dimethyl carbonate",                "616-38-6", &
               101,  2.193E+02, -1.373E+04,  -2.937E+01,         0.,  0, "E",  3.5902E+02, "S")  ! 273.35, 333.15,  "<3%",
    ! other aliphatic alcohols:
    CALL CalcH("isopropanol",                        "67-63-0", &
               101,  2.321E+02, -1.607E+04,  -3.060E+01,         0.,  0, "E",  4.8094E+01, "S")  ! 273,    373.15,  "<10%",
    CALL CalcH("2-methyl-1-propanol",                "78-83-1", &
               101,  3.840E+02, -2.321E+04,  -5.299E+01,         0.,  0, "E",  6.7391E+01, "S")  ! 273.35, 374.15,  "<10%",
    CALL CalcH("2-butanol",                          "78-92-2", &
               101,  2.945E+02, -1.939E+04,  -3.956E+01,         0.,  0, "E",  5.9124E+01, "S")  ! 273,    371.15,  "<5%",
    CALL CalcH("2-methyl-2-propanol",                "75-65-0", &
               101,  2.378E+02, -1.682E+04,  -3.109E+01,         0.,  0, "E",  6.8926E+01, "S")  ! 273,    371.15,  "<10%",
    CALL CalcH("2-pentanol",                       "6032-29-7", &
               101,  3.240E+02, -2.093E+04,  -4.377E+01,         0.,  0, "E",  7.9763E+01, "S")  ! 273.35, 371.15,  "<5%",
    CALL CalcH("2-methyl-2-butanol",                 "75-85-4", &
               101,  2.800E+02, -1.898E+04,  -3.719E+01,         0.,  0, "E",  8.1409E+01, "S")  ! 273.35, 371.45,  "<5%",
    CALL CalcH("2-methyl-1-butanol",                "137-32-6", &
               101,  3.671E+02, -2.288E+04,  -5.021E+01,         0.,  0, "E",  7.2382E+01, "S")  ! 273.35, 363.15,  "<5%",
    CALL CalcH("2,2-dimethyl-1-propanol",            "75-84-3", &
               101,  3.360E+02, -2.136E+04,  -4.548E+01,         0.,  0, "P",  1.7596E+02, "P")  ! 285.15, 363.15,  "<25%",
    CALL CalcH("2-hexanol",                         "626-93-7", &
               101,  3.326E+02, -2.148E+04,  -4.486E+01,         0.,  0, "EP", 1.4331E+02, "S")  ! 273.15, 363.35,  "<25%",
    CALL CalcH("3-hexanol",                         "623-37-0", &
               101,  3.385E+02, -2.197E+04,  -4.560E+01,         0.,  0, "EP", 1.5227E+02, "S")  ! 273.15, 363.45,  "<5%",
    CALL CalcH("3-methyl-1-pentanol",               "589-35-5", &
               101,  4.097E+02, -2.540E+04,  -5.620E+01,         0.,  0, "P",  7.6763E+01, "P")  ! 273.15, 363.65,  "<10%",
    CALL CalcH("3-methyl-3-pentanol",                "77-74-7", &
               101,  1.770E+02, -1.383E+04,  -2.197E+01,         0.,  0, "P",  2.1990E+02, "P")  ! 282.95, 363.55,  "<10%",
    CALL CalcH("3-pentanol",                        "584-02-1", &
               101,  3.054E+02, -1.989E+04,  -4.110E+01,         0.,  0, "E",  9.2713E+01, "S")  ! 273.35, 371.15,  "<5%",
    CALL CalcH("3-methyl-1-butanol",                "123-51-3", &
               101,  4.590E+02, -2.701E+04,  -6.392E+01,         0.,  0, "E",  6.9731E+01, "S")  ! 273.35, 374.15,  "<10%",
    CALL CalcH("3-methyl-2-butanol",                "598-75-4", &
               101,  3.121E+02, -2.017E+04,  -4.208E+01,         0.,  0, "E",  1.1070E+02, "S")  ! 273.35, 371.15,  "<3%",
    CALL CalcH("2-heptanol",                        "543-49-7", &
               101,  3.782E+02, -2.400E+04,  -5.135E+01,         0.,  0, "P",  1.7236E+02, "P")  ! 273.15, 363.35,  "<5%",
    CALL CalcH("3-heptanol",                        "589-82-2", &
               101,  3.659E+02, -2.376E+04,  -4.933E+01,         0.,  0, "E",  1.7651E+02, "S")  ! 273.15, 363.35,  "<10%",
    CALL CalcH("4-methyl-2-pentanol",               "108-11-2", &
               101,  2.657E+02, -1.777E+04,  -3.520E+01,         0.,  0, "EP", 2.4972E+02, "S")  ! 273.15, 363.35,  "<10%",
    CALL CalcH("2-octanol",                         "123-96-6", &
               101,  5.167E+02, -3.094E+04,  -7.154E+01,         0.,  0, "P",  2.1047E+02, "P")  ! 273.15, 363.65,  "<10%",
    CALL CalcH("2-nonanol",                         "628-99-9", &
               101,  5.400E+02, -3.288E+04,  -7.444E+01,         0.,  0, "P",  2.7628E+02, "P")  ! 273.15, 363.65,  "<25%",
    ! page 197 = pdf-page 203:
    CALL CalcH("2,6-dimethyl-4-heptanol",           "108-82-7", &
               101,  2.593E+02, -1.915E+04,  -3.313E+01,         0.,  0, "P",  5.3226E+02, "P")  ! 273.15, 363.65,  "<25%",
    ! other aliphatic amines:
    CALL CalcH("dimethylamine",                     "124-40-3", &
               100,  8.902E+01,         0.,          0.,         0.,  0, "S",  8.9023E+01, "S")  ! 298.15, 298.15,  "<50%",
    CALL CalcH("triethylamine",                     "121-44-8", &
               100,  5.961E+02,         0.,          0.,         0.,  0, "S",  5.9606E+02, "S")  ! 298.15, 298.15,  "<5%",
    CALL CalcH("di-n-propylamine",                  "142-84-7", &
               100,  2.576E+02,         0.,          0.,         0.,  0, "S",  2.5765E+02, "S")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("diethylamine",                      "109-89-7", &
               100,  1.514E+02, -1.652E+04,  -4.148E+01,         0.,  0, "S",  1.5144E+02, "S")  ! 298.15, 298.15,  "<25%",
    CALL CalcH("di-n-butylamine",                   "111-92-2", &
               100,  4.383E+02,         0.,          0.,         0.,  0, "S",  4.3834E+02, "S")  ! 298.15, 298.15,  "<50%",
    CALL CalcH("tri-n-butylamine",                  "102-82-9", &
               101,  3.425E+01, -8.719E+03,          0.,         0.,  0, "E",  1.4890E+02, "S")  ! 273.15, 353.15,  "<25%",
    ! other alkanes:
    CALL CalcH("2,3,4-trimethylpentane",            "565-75-3", &
               100,  1.701E+07, -1.047E+04,  -3.146E+01,         0.,  0, "P",  1.7009E+07, "S")  ! 298.15, 298.15,  "<5%",
    CALL CalcH("2,2,5-trimethylhexane",            "3522-94-9", &
               101,  3.797E+01, -6.194E+03,          0.,         0.,  0, "E",  2.9316E+07, "D")  ! 273.15, 298.15,  "<10%",
    ! other alkylbenzenes:
    CALL CalcH("o-xylene",                           "95-47-6", &
               101,  3.284E+02, -1.815E+04,  -4.516E+01,         0.,  0, "E",  2.7047E+04, "S")  ! 273.15, 353.15,  "<10%",
    CALL CalcH("m-xylene",                          "108-38-3", &
               101,  4.853E+02, -2.121E+04,  -7.387E+01,  5.787E-02,  1, "E",  3.8659E+04, "S")  ! 273.15, 543.75,  "<10%",
    CALL CalcH("p-xylene",                          "106-42-3", &
               101,  8.256E+02, -3.283E+04,  -1.302E+02,  1.238E-01,  1, "E",  3.8088E+04, "S")  ! 273.15, 568.05,  "<10%",
    CALL CalcH("cumene",                             "98-82-8", &
               101,  2.709E+01, -4.790E+03,          0.,         0.,  0, "E",  6.1375E+04, "S")  ! 288.15, 318.15,  "<10%",
    CALL CalcH("1,2,3-trimethylbenzene",            "526-73-8", &
               101,  2.534E+01, -4.560E+03,          0.,         0.,  0, "E",  2.3000E+04, "S")  ! 288.15, 318.15,  "<25%",
    CALL CalcH("1,2,4-trimethylbenzene",             "95-63-6", &
               101,  2.614E+01, -4.676E+03,          0.,         0.,  0, "E",  3.4730E+04, "S")  ! 288.15, 318.15,  "<5%",
    CALL CalcH("mesitylene",                        "108-67-8", &
               101,  2.301E+02, -1.351E+04,  -3.055E+01,         0.,  0, "E",  4.6913E+04, "S")  ! 283.15, 373.15,  "<25%",
    CALL CalcH("p-diethylbenzene",                  "105-05-5", &
               101,  2.841E+04,  2.844E+03,          0.,         0.,  0, "E",  2.8409E+04, "D")  ! 293.15, 293.15,  "<100%",
    CALL CalcH("1,2,3,4-tetramethylbenzene",        "488-23-3", &
               101,  2.968E+01, -5.913E+03,          0.,         0.,  0, "E",  1.8900E+04, "S")  ! 288.15, 308.15,  "<100%",
    CALL CalcH("1,2,4,5-tetramethylbenzene",         "95-93-2", &
               100,  4.079E+04,         0.,          0.,         0.,  0, "E",  4.0791E+04, "D")  ! 298.15, 298.15,  "<10%",
    ! page 198 = pdf-page 204:
    ! other amines, imines:
    CALL CalcH("cyclohexylamine",                   "108-91-8", &
               101,  2.024E+02, -1.528E+04,  -2.596E+01,         0.,  0, "E",  2.4813E+01, "S")  ! 298.15, 372.82,  "<25%",
    CALL CalcH("triallylamine",                     "102-70-5", &
               101,  4.010E+02, -2.393E+04,  -5.502E+01,         0.,  0, "E",  1.3674E+03, "S")  ! 273.15, 363.15,  "<25%",
    ! other condensed rings:
    CALL CalcH("fluoranthene",                      "206-44-0", &
               101,  2.755E+01, -6.910E+03,          0.,         0.,  0, "E",  7.9065E+01, "S")  ! 281.25, 333.15,  "<10%",
    CALL CalcH("fluorene",                           "86-73-7", &
               101,  2.665E+01, -6.145E+03,          0.,         0.,  0, "E",  4.1834E+02, "S")  ! 279.75, 333.15,  "<10%",
    CALL CalcH("anthracene",                        "120-12-7", &
               101,  3.683E+02, -2.185E+04,  -5.088E+01,         0.,  0, "E",  1.7268E+02, "S")  ! 273.45, 347.85,  "<5%",
    CALL CalcH("phenanthrene",                       "85-01-8", &
               101,  4.223E+02, -2.462E+04,  -5.873E+01,         0.,  0, "E",  1.6014E+02, "S")  ! 277.15, 346.55,  "<5%",
    CALL CalcH("chrysene",                          "218-01-9", &
               101,  4.104E+01, -1.184E+04,          0.,         0.,  0, "E",  3.7646E+00, "S")  ! 279.65, 302.15,  "<25%",
    CALL CalcH("pyrene",                            "129-00-0", &
               101,  3.078E+02, -1.937E+04,  -4.193E+01,         0.,  0, "E",  4.9163E+01, "S")  ! 277.85, 347.85,  "<5%",
    CALL CalcH("acenaphthene",                       "83-32-9", &
               101,  2.469E+01, -5.381E+03,          0.,         0.,  0, "E",  7.7124E+02, "S")  ! 277.25, 347.85,  "<25%",
    CALL CalcH("acenaphthalene",                    "208-96-8", &
               101,  2.748E+01, -6.263E+03,          0.,         0.,  0, "E",  6.4945E+02, "S")  ! 277.25, 304.15,  "<25%",
    CALL CalcH("benzo[a]pyrene",                     "50-32-8", &
               101,  3.108E+01, -8.515E+03,          0.,         0.,  0, "E",  1.2412E+01, "S")  ! 281.15, 308.15,  "<10%",
    CALL CalcH("benzo[b]fluoranthene",              "205-99-2", &
               101,  1.949E+01, -5.422E+03,          0.,         0.,  0, "E",  3.6764E+00, "S")  ! 283.15, 328.15,  "<10%",
    ! other ethers/diethers:
    CALL CalcH("methylal",                          "109-87-5", &
               101,  2.270E+01, -4.738E+03,          0.,         0.,  0, "E",  9.0067E+02, "S")  ! 273.35, 333.15,  "<25%",
    CALL CalcH("1,2-dimethoxyethane",               "110-71-4", &
               101,  2.511E+02, -1.722E+04,  -3.328E+01,         0.,  0, "E",  3.9438E+01, "S")  ! 273.35, 372.5,   "<3%",
    CALL CalcH("diethylene glycol dibutyl ether",   "112-73-2", &
               101,  4.602E+01, -1.280E+04,          0.,         0.,  0, "P",  2.2058E+01, "P")  ! 273.15, 293.15,  "<100%",
    CALL CalcH("anisole",                           "100-66-3", &
               101,  2.158E+01, -4.179E+03,          0.,         0.,  0, "E",  1.9271E+03, "S")  ! 275.15, 363.85,  "<5%",
    CALL CalcH("diphenyl ether",                    "101-84-8", &
               100,  2.903E+03,         0.,          0.,         0.,  0, "S",  2.9031E+03, "S")  ! 298.15, 298.15,  "<50%",
    ! other inorganics:
    CALL CalcH("hydrogen peroxide",                "7722-84-1", &
               101,  2.142E+01, -7.235E+03,          0.,         0.,  0, "E",  5.7967E-02, "S")  ! 278.15, 297.15,  "<25%",
    ! other monoaromatics:
    CALL CalcH("styrene",                           "100-42-5", &
               101,  2.190E+01, -3.653E+03,          0.,         0.,  0, "E",  1.5542E+04, "S")  ! 293.32, 313.02,  "<5%",
    ! page 199 = pdf-page 205:
    ! other polyfunctional C, H, O:
    CALL CalcH("2-butoxyethanol",                   "111-76-2", &
               101,  2.922E+01, -8.275E+03,          0.,         0.,  0, "E",  4.3241E+00, "S")  ! 293.15, 303.15,  "<25%",
    CALL CalcH("vanillin",                          "121-33-5", &
               101, -3.909E+02,  1.144E+04,   6.127E+01,         0.,  0, "E",  3.1102E-02, "S")  ! 293.15, 318.15,  "<5%",
    ! other polyfunctional organics:
    CALL CalcH("malathion",                         "121-75-5", &
               101,  2.879E+01, -8.274E+03,          0.,         0.,  0, "E",  2.8250E+00, "S")  ! 283.15, 303.15,  "<25%",
    ! other saturated aliphatic esters:
    CALL CalcH("ethyl isovalerate",                 "108-64-5", &
               101,  2.443E+02, -1.600E+04,  -3.201E+01,         0.,  0, "E",  3.8440E+03, "S")  ! 273.15, 363.65,  "<10%",
    CALL CalcH("diethyl malonate",                  "105-53-3", &
               101,  2.731E+02, -1.846E+04,  -3.660E+01,         0.,  0, "P",  1.4010E+01, "P")  ! 273.15, 363.75,  "<10%",
    CALL CalcH("ethyl trimethyl acetate",          "3938-95-2", &
               101,  2.619E+02, -1.693E+04,  -3.441E+01,         0.,  0, "E",  8.7394E+03, "S")  ! 273.15, 363.55,  "<25%",
    ! peroxides:
    CALL CalcH("methyl hydroperoxide",             "3031-73-0", &
               101,  2.049E+01, -5.244E+03,          0.,         0.,  0, "E",  1.8200E+01, "S")  ! 277.15, 297.15,  "<3%",
    CALL CalcH("ethyl hydroperoxide",              "3031-74-1", &
               101,  2.288E+01, -5.985E+03,          0.,         0.,  0, "E",  1.6528E+01, "S")  ! 278.15, 297.15,  "<10%",
    ! polyfunctional amides/amines:
    CALL CalcH("N,N-dimethylformamide",              "68-12-2", &
               101,  2.051E+02, -1.570E+04,  -2.692E+01,         0.,  0, "E",  3.6460E-01, "S")  ! 289.4,  372,     "<25%",
    CALL CalcH("N-methylformamide",                 "123-39-7", &
               101,  2.081E+01, -7.161E+03,          0.,         0.,  0, "E",  4.0404E-02, "S")  ! 333.15, 372.65,  "<5%",
    CALL CalcH("N,N-dimethylacetamide",             "127-19-5", &
               101,  2.480E+01, -8.014E+03,          0.,         0.,  0, "E",  1.2519E-01, "P")  ! 343.15, 372.28,  "<10%",
    ! polyfunctional C, H, N, halide, (o):
    CALL CalcH("o-chloroaniline",                    "95-51-2", &
               101,  1.336E+03, -5.042E+04,  -2.173E+02,  2.501E-01,  1, "P",  2.6749E+01, "P")  ! 298.15, 431.15,  "<25%",
    CALL CalcH("p-chloronitrobenzene",              "100-00-5", &
               101,  2.414E+01, -5.868E+03,          0.,         0.,  0, "E",  8.6341E+01, "S")  ! 283.15, 313.15,  "<10%",
    ! polyfunctional C, H, O, halide:
    CALL CalcH("di(2-chloroethyl)ether",            "111-44-4", &
               101,  2.107E+02, -1.440E+04,  -2.767E+01,         0.,  0, "E",  1.1134E+02, "S")  ! 273.15, 364.85,  "<5%",
    CALL CalcH("ethylchloroacetate",                "105-39-5", &
               101,  2.546E+02, -1.630E+04,  -3.415E+01,         0.,  0, "E",  2.1209E+02, "S")  ! 273.15, 333.25,  "<10%",
    ! page 200 = pdf-page 206:
    CALL CalcH("methyl chloroacetate",               "96-34-4", &
               101,  1.755E+02, -1.239E+04,  -2.268E+01,         0.,  0, "E",  1.1352E+02, "S")  ! 273.15, 323.35,  "<5%",
    ! polyfunctional C, H, O, N:
    CALL CalcH("N-methyl-2-pyrrolidone",            "872-50-4", &
               101,  4.600E+02, -3.011E+04,  -6.372E+01,         0.,  0, "E",  1.6824E-02, "S")  ! 298.8,  372.85,  "<5%",
    ! polyfunctional C, H, O, S:
    CALL CalcH("dimethyl sulfoxide",                 "67-68-5", &
               101,  2.115E+01, -7.789E+03,          0.,         0.,  0, "E",  6.9311E-03, "S")  ! 298.8,  337.9,   "<25%",
    ! polyfunctional esters:
    CALL CalcH("methyl salicylate",                 "119-36-8", &
               101,  4.635E+02, -2.852E+04,  -6.394E+01,         0.,  0, "E",  3.3064E+01, "S")  ! 273.15, 363.65,  "<25%",
    CALL CalcH("2-ethoxyethyl acetate",             "111-15-9", &
               100,  1.850E+01,         0.,          0.,         0.,  0, "E",  1.8500E+01, "E")  ! 303.15, 303.15,  "<10%",
    CALL CalcH("ethyl acetoacetate",                "141-97-9", &
               101,  2.610E+01, -7.239E+03,          0.,         0.,  0, "EP", 6.1986E+00, "S")  ! 273.15, 303.15,  "<25%",
    CALL CalcH("2-butoxyethyl acetate",             "112-07-2", &
               100,  3.062E+01,         0.,          0.,         0.,  0, "E",  3.0618E+01, "E")  ! 298.15, 298.15,  "<25%",
    ! propionates and butyrates:
    CALL CalcH("methyl propionate",                 "554-12-1", &
               101,  2.605E+01, -5.719E+03,          0.,         0.,  0, "E",  9.6262E+02, "S")  ! 273.15, 310.2,   "<5%",
    CALL CalcH("ethyl propionate",                  "105-37-3", &
               101,  2.963E+02, -1.786E+04,  -4.021E+01,         0.,  0, "E",  1.4323E+03, "S")  ! 273.15, 353.45,  "<25%",
    CALL CalcH("n-propyl propionate",               "106-36-5", &
               101,  2.299E+02, -1.536E+04,  -2.996E+01,         0.,  0, "E",  2.0359E+03, "S")  ! 273.15, 363.65,  "<10%",
    CALL CalcH("n-butyl propionate",                "590-01-2", &
               101,  2.785E+02, -1.795E+04,  -3.694E+01,         0.,  0, "E",  2.4355E+03, "S")  ! 273.15, 363.55,  "<10%",
    CALL CalcH("n-propyl n-butyrate",               "105-66-8", &
               101,  2.587E+02, -1.660E+04,  -3.422E+01,         0.,  0, "E",  3.0510E+03, "S")  ! 273.15, 363.65,  "<25%",
    CALL CalcH("methyl n-butyrate",                 "623-42-7", &
               101,  1.665E+02, -1.196E+04,  -2.091E+01,         0.,  0, "E",  1.3329E+03, "S")  ! 273.15, 363.65,  "<25%",
    CALL CalcH("ethyl n-butyrate",                  "105-54-4", &
               101,  1.949E+02, -1.355E+04,  -2.488E+01,         0.,  0, "E",  2.1767E+03, "S")  ! 273.15, 363.65,  "<25%",
    CALL CalcH("isobutyl isobutyrate",               "97-85-8", &
               101,  2.887E+02, -1.832E+04,  -3.832E+01,         0.,  0, "E",  7.9493E+03, "S")  ! 273.15, 363.45,  "<25%",
    CALL CalcH("n-butyl n-butyrate",                "109-21-7", &
               101,  3.706E+02, -2.260E+04,  -5.032E+01,         0.,  0, "E",  3.4019E+03, "S")  ! 273.15, 363.45,  "<10%",
    CALL CalcH("methyl isobutyrate",                "547-63-7", &
               101,  2.024E+02, -1.347E+04,  -2.625E+01,         0.,  0, "E",  2.0910E+03, "S")  ! 273.15, 353.45,  "<10%",
    CALL CalcH("ethyl isobutyrate",                  "97-62-1", &
               101,  1.871E+02, -1.308E+04,  -2.370E+01,         0.,  0, "P",  3.5044E+03, "P")  ! 273.15, 363.75,  "<10%",
    CALL CalcH("n-pentyl n-butyrate",               "540-18-1", &
               101,  2.955E+01, -6.393E+03,          0.,         0.,  0, "E",  3.3166E+03, "S")  ! 273.15, 363.65,  "<25%",
    ! page 201 = pdf-page 207:
    CALL CalcH("isopentyl butyrate",                "106-27-4", &
               101,  3.076E+02, -1.973E+04,  -4.087E+01,         0.,  0, "E",  5.2634E+03, "S")  ! 282.55, 363.85,  "<100%",
    ! sulfides/thiophenes:
    CALL CalcH("dimethyl sulfide",                   "75-18-3", &
               101,  2.083E+02, -1.184E+04,  -2.797E+01,         0.,  0, "E",  1.0548E+04, "S")  ! 272.35, 353.45,  "<10%",
    CALL CalcH("dimethyl disulfide",                "624-92-0", &
               101,  2.120E+01, -3.695E+03,          0.,         0.,  0, "E",  6.6448E+03, "S")  ! 273.2,  343.15,  "<10%",
    CALL CalcH("2-methylthiophene",                 "554-14-3", &
               101,  2.390E+01, -4.287E+03,          0.,         0.,  0, "E",  1.3663E+04, "S")  ! 298.15, 343.15,  "<10%",
    CALL CalcH("diisopropyl sulfide",               "625-80-9", &
               101,  2.628E+01, -4.889E+03,          0.,         0.,  0, "E",  1.9586E+04, "S")  ! 298.15, 343.15,  "<10%",
    ! unsaturated aliphatic esters:
    CALL CalcH("ethyl acrylate",                    "140-88-5", &
               101,  2.199E+01, -4.393E+03,          0.,         0.,  0, "P",  1.4139E+03, "P")  ! 298.6,  360.2,   "<10%",
    CALL CalcH("methyl methacrylate",                "80-62-6", &
               101,  2.853E+02, -1.690E+04,  -3.880E+01,         0.,  0, "E",  1.8891E+03, "S")  ! 273.35, 333.15,  "<3%",

    ! see also appendix D with experimental details

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, Eq, A, B, C, D, E, datatype, data_C5, datatype_C5)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN)           :: chem_, casrn_
      INTEGER,          INTENT(IN)           :: Eq ! https://de.wikipedia.org/wiki/DIPPR-Gleichungen
      REAL,             INTENT(IN)           :: A, B, C, D
      INTEGER,          INTENT(IN)           :: E
      CHARACTER(LEN=*), INTENT(IN)           :: datatype
      REAL,             INTENT(IN), OPTIONAL :: data_C5     ! from Tab. C.5
      CHARACTER(LEN=*), INTENT(IN), OPTIONAL :: datatype_C5 ! from Tab. C.5
      REAL :: Hvpx_kPa
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! errors in the table:
      ! p. 182: The CASRN of 1-undecene    is   821-95-4 not 872-05-9
      ! p. 182: The CASRN of 1-dodecene    is   112-41-4 not 872-05-9
      ! p. 182: The CASRN of 1-tridecene   is  2437-56-1 not 592-41-6
      ! p. 182: The CASRN of 1-tetradecene is  1120-36-1 not 872-05-9

      SELECT CASE(casrn_)
      CASE ("112-06-1") ! p. 183
        type = "W"
        CALL MakeNote(TRIM(ref)//"-"//casrn_, &
          "The value listed as $A$ for $n$-heptyl acetate is probably not $A$ but " // &
          "the Henry's law volatility constant $\Hvpx$ at 298\,K.")
        CALL Output(DUMMY)
      CASE ("112-14-1") ! p. 183
        type = "W"
        CALL MakeNote(TRIM(ref)//"-"//casrn_, &
          "The value listed as $A$ for $n$-octyl acetate is probably not $A$ but " // &
          "the Henry's law volatility constant $\Hvpx$ at 298\,K.")
        CALL Output(DUMMY)
      CASE ("693-65-2") ! p. 184
        type = "W"
        CALL MakeNote(TRIM(ref)//"-"//casrn_, &
          "The value listed as $A$ for di-$n$-pentyl ether is probably not $A$ but " // &
          "the Henry's law volatility constant $\Hvpx$ at 298\,K.")
        CALL Output(DUMMY)
      CASE ("112-58-3") ! p. 184
        type = "W"
        CALL MakeNote(TRIM(ref)//"-"//casrn_, &
          "The value listed as $A$ for di-$n$-hexyl ether is probably not $A$ but " // &
          "the Henry's law volatility constant $\Hvpx$ at 298\,K.")
        CALL Output(DUMMY)
      CASE ("591-50-4") ! p. 188
        type = "W"
        CALL MakeNote(TRIM(ref)//"-"//casrn_, &
          "The value listed as $A$ for iodobenzene is probably not $A$ but the " // &
          "Henry's law volatility constant $\Hvpx$ at 298\,K. " // &
          "For the value of $B$, a minus sign is probably missing.")
        CALL Output(DUMMY)
      CASE ("123-18-2") ! p. 193
        type = "W"
        CALL MakeNote(TRIM(ref)//"-"//casrn_, &
          "The value listed as $A$ for 2,6,8-trimethyl-4-nonanone is probably not $A$ but " // &
          "the Henry's law volatility constant $\Hvpx$ at 298\,K.")
        CALL Output(DUMMY)
      CASE ("105-05-5") ! p. 197
        type = "W"
        CALL MakeNote(TRIM(ref)//"-"//casrn_, &
          "The value listed as $A$ for diethylbenzene is probably not $A$ but the " // &
          "Henry's law volatility constant $\Hvpx$ at 298\,K.")
        CALL Output(DUMMY)
      CASE DEFAULT
        type = "L"
        IF (Eq==100) THEN
          IF ((B==0.).AND.(C==0.).AND.(D==0.).AND.(E==0)) THEN
            Hvpx_kPa  = A ! parameters B, C, D, E are not used
            Hominus = cH2O/(1E3*Hvpx_kPa)
            IF (PRESENT(data_C5)) THEN
              CALL consistency_check(Hominus, cH2O/(1E3*data_C5), &
                "Values in Tables~C2 and C5")
            ENDIF
            CALL Output(Hominus)
          ELSE
            IF (PRESENT(data_C5)) THEN
              CALL Output(cH2O/(1E3*data_C5))
            ENDIF
          ENDIF
        ELSEIF (Eq==101) THEN
          ! The temperature dependence is:
          ! ln(KH) = A + B/T + C*ln(T) + D*T^E
          Hvpx_kPa = EXP(A + B/T0 + C*LOG(T0) + D*T0**E) ! Tab. 4.3
          Hominus    = cH2O/(1E3*Hvpx_kPa)
          ! The analytical derivative is:
          ! dln(KH)/d(1/T) = B -C*T -E*D*T^(E+1)
          mindHR = - B + C*T0 + E*D*T0**(E+1) ! the minus sign comes from Hvpx->Hscp
          IF ((C/=0.).OR.(D/=0.).OR.(E/=0)) THEN
            CALL MakeNote("moreTdep")
          ENDIF
          IF (PRESENT(data_C5)) THEN
            CALL consistency_check(Hominus, cH2O/(1E3*data_C5), &
              "Values at 298\,K in Tables~C2 and C5")
          ENDIF
          CALL Output(Hominus, mindHR)
        ELSE
          STOP "ERROR: unknown DIPPR equation number in ref3518"
        ENDIF
      END SELECT

    END SUBROUTINE CalcH

  END SUBROUTINE ref3518

  !---------------------------------------------------------------------------

  ! ref3519 NOTUSED: Only description of database but no data

  !---------------------------------------------------------------------------

  SUBROUTINE ref3521 ! HscpSI [mol/(m3*Pa)]
    IMPLICIT NONE

    ref = "3521"
    type = "Q"

    !                                                     Tab.3       Tab.3     Tab.3     Tab.4
    !                                                     Hscp         lower     upper     exptl
    !                                                                 bound     bound     in H2O
    CALL CalcH("trans-2-heptenal",          "18829-55-5", 1.304)    ! 1.243,    1.368,    0.05
    CALL CalcH("nonanol",                     "143-08-8", 0.535)    ! 0.508,    0.564,    0.11
    CALL CalcH("2-phenylethanol",              "60-12-8", 0.002)    ! 0.002,    0.002,    >37
    CALL CalcH("3-methylbutan-1-ol",          "123-51-3", 0.124)    ! 0.118,    0.130,    0.46
    CALL CalcH("4-allyl-2-methoxyphenol",      "97-53-0", 0.354)    ! 0.320,    0.392,    5.1
    CALL CalcH("α-damascone",               "24720-09-0", 21.494)   ! 20.402,   22.644,   None
    CALL CalcH("2-methoxyphenol",              "90-05-1", 0.555)    ! 0.518,    0.594,    7.7
    CALL CalcH("4-methylacetophenone",        "122-00-9", 4.323)    ! 4.039,    4.628,    1.1
    CALL CalcH("γ-dodecalactone",            "2305-05-7", 37.078)   ! 35.055,   39.219,   none
    CALL CalcH("γ-decalactone",               "706-14-9", 53.404)   ! 50.287,   56.714,   None
    CALL CalcH("t-whiskylactone",          "105119-22-0", 35.479)   ! 33.274,   37.829,   None
    CALL CalcH("ethylbutanoate",              "105-54-4", 3.413)    ! 3.265,    3.567,    0.029
    CALL CalcH("ethylhexanoate",              "123-66-0", 2.954)    ! 2.748,    3.175,    0.014
    CALL CalcH("t-ethylcinnamate",            "103-36-6", 1.390)    ! 1.293,    1.494,    0.162
    CALL CalcH("1,1-diethoxyethane",          "105-57-7", 0.115)    ! 0.106,    0.125,    0.1
    CALL CalcH("beta-ionone",                  "79-77-6", 0.015)    ! 0.014,    0.016,    1.2
    CALL CalcH("3-methylbutylacetate",        "123-92-2", 6.967)    ! 6.587,    7.369,    0.026
    CALL CalcH("vanillin",                    "121-33-5", 1775.807) ! 1619.376, 1947.349, 4700

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, HscpSI)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN)           :: chem_, casrn_
      REAL,             INTENT(IN)           :: HscpSI
      chem=chem_ ; casrn=casrn_ ! make global for Output
      Hominus = HscpSI
      CALL MakeNote(TRIM(ref), &
        "Calculated for an aqueous solution containing 60\,\% ethanol by volume as the solvent.")
      CALL Output(Hominus)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3521

  !---------------------------------------------------------------------------

  SUBROUTINE ref3523 ! Hvpx [kPa]
    IMPLICIT NONE

    ref = "3523"
    type = "V"

    ! Tab. 6:
    CALL CalcH("dimethyl carbonate",  "616-38-6", 43.5430, -37.6982, -20.9153)
    CALL CalcH("diethyl carbonate",   "105-58-8", 66.7681, -60.4491, -39.8940)
    CALL CalcH("vinyl acetate",       "108-05-4", 56.3337, -48.4032, -32.8603)
    CALL CalcH("methyl methacrylate",  "80-62-6", 61.7442, -54.2469, -36.3226)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, A, B, C)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: A, B, C
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! T-dep with 3 parameters:
      ! ln(H) = A + B/tau + C ln(tau) with tau = T/T0 and T0 = 298.15 K
      Hominus = cH2O / (EXP(A+B) * 1E3)
      ! analytical derivative:
      ! d ln(H) / d (1/T)
      mindHR = - ( B*T0 - C*T0 )
      CALL MakeNote("moreTdep")
      CALL Output(Hominus, mindHR)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3523

  !---------------------------------------------------------------------------

  SUBROUTINE ref3524 ! Hvcc [1]
    IMPLICIT NONE
    ref = "3524"
    type = "M"
    chem = "N-nitrosodimethylamine" ; casrn = "62-75-9" ! (CH3)2NNO
    CALL MakeNoteOtherTempK(293)
    CALL Output(Hvcc_TO_HscpSI(1E-4,293.))
  END SUBROUTINE ref3524

  !---------------------------------------------------------------------------

  SUBROUTINE ref3525 ! Hscp [M/atm]
    IMPLICIT NONE
    ref = "3525"
    type = "M"

    chem = "isocyanic acid" ; casrn = "75-13-8"
    ! Heff data for pure water at pH = 3 from Tab. 1:
    CALL HTdep( (/ 279., 286.5, 291., 296.5, 310., 298. /), &
      (/ 68., 42., 33., 26., 12., 26. /) * Hscp_TO_HscpSI, Hominus, mindHR)
    ! use intrinsic H from abstract, not Heff from Tdep regression:
    Hominus = 20. * Hscp_TO_HscpSI
    CALL MakeNote(TRIM(ref), "The value at $T^{\ominus}$ is the intrinsic " // &
      "Henry's law constant, but the temperature dependence refers to the " // &
      "effective Henry's law constant at pH = 3.0.")
    CALL Output(Hominus, mindHR, r2)

    chem = "methyl isocyanate" ; casrn = "624-83-9"
    ! average of 1.3 and 1.4 from Tab. 1:
    CALL Output(1.35*Hscp_TO_HscpSI)

  END SUBROUTINE ref3525

  !---------------------------------------------------------------------------

  SUBROUTINE ref3526 ! Hvpx [kPa]
    IMPLICIT NONE

    ref = "3526"
    type = "M"
    ndata = 6
    ALLOCATE(temp(ndata), Harray(ndata))
    temp = (/ 278.2, 283.2, 288.2, 293.2, 298.2, 303.2 /)

    ! Tab. 2:
    CALL CalcHTdep("propanone", "67-64-1", (/ 5.6E1, 7.5E1, 1.0E2, 1.3E2, 1.7E2, 2.1E2 /)) ! acetone
    CALL CalcHTdep("methanol",  "67-56-1", (/ 7.5, 8.7, 13., 20., 26., 32. /)) ! CH3OH
    CALL CalcHTdep("ethanol",   "64-17-5", (/ 6.5, 10., 13., 20., 26., 33. /)) ! C2H5OH

    ! Tab. 3:
    CALL CalcH("2-fluoroethanol",          "371-62-0", 1.2E2) ! C2H5FO
    CALL CalcH("1-fluoro-2-propanone",     "430-51-3", 1.2E2) ! CH2FCOCH3 fluoroacetone
    CALL CalcH("2,2-difluoroethanol",      "359-13-7", 6E1  ) ! C2H4F2O
    CALL CalcH("1,1-dichloro-2-propanone", "513-88-2", 1.7E2) ! C3H4Cl2O 1,1-dichloroacetone
    CALL CalcH("2,2-dichloroethanol",      "598-38-9", 1.5E2) ! C2H4Cl2O
    CALL CalcH("3-pentanone",               "96-22-0", 2.6E2) ! C2H5COC2H5
    CALL CalcH("1,2-dimethoxyethane",      "110-71-4", 2.6E1) ! C4H{10}O2
    CALL CalcH("cyclopentanone",           "120-92-3", 2.0E1) ! C5H8O
    CALL CalcH("1,3-dioxane",              "505-22-6", 3.0E1) ! C4H8O2

    DEALLOCATE(temp, Harray)

  CONTAINS

    SUBROUTINE CalcHTdep (chem_, casrn_, KH)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: KH
      chem=chem_ ; casrn=casrn_ ! make global for Output
      Harray = cH2O / (KH*1E3)
      CALL HTdep(temp, Harray, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcHTdep

    SUBROUTINE CalcH (chem_, casrn_, KH)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: KH
      chem=chem_ ; casrn=casrn_ ! make global for Output
      Hominus = cH2O / (KH*1E3)
      CALL Output(Hominus)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3526

  !---------------------------------------------------------------------------

  SUBROUTINE ref3527 ! Hvpx [kPa]
    IMPLICIT NONE

    ref = "3527"
    type = "V"

    ! Tab. 5:
    CALL CalcH("2-propanol",          "67-63-0", 61.0445, -69.8548, -58.7948, 12.6022) ! isopropanol
    CALL CalcH("2-butanol",           "78-92-2", 70.4890, -81.3780, -70.9743, 14.9529) ! sec-butanol
    CALL CalcH("2-methyl-1-propanol", "78-83-1", 68.0466, -78.7577, -69.5854, 14.9952) ! isobutanol
    CALL CalcH("2-methyl-2-propanol", "75-65-0", 72.5737, -82.3846, -70.4649, 14.0467) ! tert-butanol

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, A, B, C, D)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: A, B, C, D
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! The temperature dependence is:
      ! ln(KH) = A + B/tau + C*ln(tau) + D*tau with tau = T/T0 and T0 = 298.15 K
      ! At T=T0, this reduces to:
      Hominus = cH2O / (EXP(A+B+D) * 1E3)
      ! The analytical derivative is:
      ! dln(KH)/d(1/T) = B*T0 - C*T - D*T^2/T0
      ! At T=T0, this reduces to:
      mindHR = -(B-C-D)*T0 ! the minus sign comes from Hvpx->Hscp
      CALL MakeNote("moreTdep")
      CALL Output(Hominus, mindHR)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3527

  !---------------------------------------------------------------------------

  SUBROUTINE ref3528 ! Hsxp [atm], Bunsen coefficient alpha [1], Hsxp [1/atm]
    IMPLICIT NONE
    REAL :: A, B, C
    ref = "3528"

    ! recommended value:
    type = "L"
    chem = "ethyne" ; casrn = "74-86-2" ! C2H2 acetylene
    ! p. 1702: ln x1 = -282.531 + 13707.8/(T/K) +  40.255 ln(T/K)
    A = -282.531
    B = 13707.8
    C = 40.255
    Hominus = EXP((A+B/T0+C*LOG(T0))) * Hsxp_TO_HscpSI
    mindHR = (B-C*T0)
    CALL Output(Hominus, mindHR)
    ! data from ref190 is reproduced as:
    ! A = -156.603 ; B = 8165.06 ; C = 21.4152

    ! data from Schoen on p. 1704:
    CALL SettypeX("schoen23")
    chem = "ethyne" ; casrn = "74-86-2" ! C2H2 acetylene
    CALL HTdep( &
      (/ 290.65, 290.65, 290.65, 300.65, 303.15, 304.15, 309.15, 309.65, 310.15, &
      310.15, 310.65, 310.65, 311.15, 312.15, 312.65, 313.15, 314.65, 316.15 /), &
      (/ 1.095, 1.099, 1.098, 0.891, 0.834, 0.860, 0.753, 0.752, 0.742, 0.746, &
      0.740, 0.732, 0.723, 0.718, 0.713, 0.711, 0.699, 0.673 /) &
      * alpha_TO_HscpSI, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)

    ! data from Vitovec on p. 1705:
    CALL SettypeX("vitovec68")
    chem = "ethyne" ; casrn = "74-86-2" ! C2H2 acetylene
    Hominus = 0.00081 * Hsxp_TO_HscpSI
    CALL MakeNoteOtherTempK(293.15)
    CALL Output(Hominus)

  END SUBROUTINE ref3528

  !---------------------------------------------------------------------------

  SUBROUTINE ref3529 ! Hvpx [Pa]
    IMPLICIT NONE
    REAL :: a0, a1, a2

    ref = "3529"
    type = "M"
    chem = "carbon monoxide" ; casrn = "630-08-0" ! CO

    ! Tab. 1:
    ndata = 14
    ALLOCATE(temp(ndata), Harray(ndata))
    temp = (/ 278.147, 283.152, 283.159, 288.167, 293.150, 298.141, &
      298.145, 303.144, 303.158, 308.147, 308.152, 313.139, 318.162, 323.102 /)
    Harray = cH2O / ((/ 3.91005E9, 4.37462E9, 4.37589E9, 4.84162E9, &
      5.28282E9, 5.70337E9, 5.70826E9, 6.11881E9, 6.11497E9, 6.48260E9, &
      6.48389E9, 6.82935E9, 7.13124E9, 7.39010E9 /))
    CALL HTdep_ABC(temp, Harray, Hominus, mindHR)
    DEALLOCATE(temp, Harray)
    CALL Output(Hominus, mindHR, r2)

    ! Eqn. (20) and Tab. 2 yield the same results:
    ! ln H21 = a0 + a1/T + a2/T^2
    a0 = 13.895293
    a1 = 6.4047270E3
    a2 = -1.1477271E6
    ! The analytical derivative is:
    ! dln(H21)/d(1/T) = a1 + 2.*a2/T
    ! At T=T0, this reduces to:
    Hominus = cH2O / EXP(a0+a1/T0+a2/(T0*T0))
    mindHR = -(a1+2.*a2/T0) ! the minus sign comes from Hvpx->Hscp
    ! PRINT *, Hominus, mindHR

  END SUBROUTINE ref3529

  !---------------------------------------------------------------------------

  SUBROUTINE ref3530
    IMPLICIT NONE
    ref = "3530"
    type = "?"
    chem = "hydrogen atom" ; casrn = "12385-13-6"
    CALL MakeNote(TRIM(ref), TRIM(citet()) // &
      " say that the free energy of solvation $\Delta G^\mathrm{H}_\mathrm{solv}$ " // &
      "(and therefore Henry's law constant) of atomic hydrogen is " // &
      "approximated well by that of molecular hydrogen. However, they " // &
      "apparently do not give a value for $\Delta G^\mathrm{H}_\mathrm{solv}$.")
    CALL Output(DUMMY)
  END SUBROUTINE ref3530

  !---------------------------------------------------------------------------

  SUBROUTINE ref3531 ! Hscp [M/atm]
    IMPLICIT NONE
    ref = "3531"
    type = "M"
    chem = "peroxyacetyl nitrate" ; casrn = "2278-22-0" ! CH3COOONO2 PAN
    mindHR = 4780.
    Hominus = 10.**(-6.6)*EXP(mindHR/T0)*Hscp_TO_HscpSI
    CALL Output(Hominus, mindHR)
  END SUBROUTINE ref3531

  !---------------------------------------------------------------------------

  SUBROUTINE ref3532 ! Hscp [M/atm]
    IMPLICIT NONE
    ref = "3532"
    type = "M"

    chem = "2,4-pentanedione" ; casrn = "123-54-6" ! C5H8O2 acetylacetone
    CALL MakeNoteOtherTempK(297)
    CALL Output(155.*Hscp_TO_HscpSI)

    chem = "propanone" ; casrn = "67-64-1" ! CH3COCH3 acetone
    CALL MakeNoteOtherTempK(297)
    CALL Output(27.*Hscp_TO_HscpSI)

  END SUBROUTINE ref3532

  !---------------------------------------------------------------------------

  SUBROUTINE ref3533 ! Hscp [M/atm]
    IMPLICIT NONE
    ref = "3533"
    chem = "hydrogen cyanide" ; casrn = "74-90-8" ! HCN hydrocyanic acid

    type = "M"
    mindHR = 8205.7
    Hominus = EXP(mindHR/T0-25.323)*Hscp_TO_HscpSI
    CALL Output(Hominus, mindHR)

    CALL SettypeX("kotlik74")
    mindHR = 2931.
    Hominus = EXP(mindHR/T0-7.7897)*Hscp_TO_HscpSI
    CALL Output(Hominus, mindHR)

  END SUBROUTINE ref3533

  !---------------------------------------------------------------------------

  SUBROUTINE ref3534 ! Hvpc [MPa*m3/kmol]
    IMPLICIT NONE
    ref = "3534"
    type = "M"
    chem = "hydrogen sulfide" ; casrn = "7783-06-4" ! H2S
    ! Tab. 1:
    CALL HTdep( (/ 25., 40. /) + CtoK, &
      HvpcSI_TIMES_HscpSI / (1E3 * (/ 1.083, 1.400 /)) , Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3534

  !---------------------------------------------------------------------------

  SUBROUTINE ref3535 ! Hvpc [atm*L/mol)]
    IMPLICIT NONE
    ref = "3535"
    type = "M"
    chem = "chlorine (molecular)" ; casrn = "7782-50-5" ! Cl2
    ! Tab. III:
    CALL HTdep( (/ 10., 20., 30. /) + CtoK, &
       Hscp_TO_HscpSI / (/ 9.17, 13.57, 19.16 /), Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3535

  !---------------------------------------------------------------------------

  SUBROUTINE ref3536 ! Hvpb [kPa*kg/mol]
    IMPLICIT NONE
    ref = "3536"
    chem = "sulfur dioxide" ; casrn = "7446-09-5" ! SO2
    CALL SettypeX("rodriguez2001")
    ! p. 1341, after eqn (10):
    CALL Output(rhoH2O/(82.9*1E3))
  END SUBROUTINE ref3536

  !---------------------------------------------------------------------------

  SUBROUTINE ref3539 ! Hvpc [kPa*m3/mol]
    IMPLICIT NONE

    ref = "3539"

    !          Name                                      formula            CASRN        M  t/ºC  KH_kPa        Ref.     note         CRC-Ref.
    CALL CalcH("Acenaphthene",                           "C12H10",      "83-32-9", 154.207, 25,   0.0143,     "2989"            ) ! 112
    CALL CalcH("Acenaphthylene",                         "C12H8",      "208-96-8", 152.192, 20,   0.010,      "2989"            ) ! 112
    CALL CalcH("Acetophenone",                           "C8H8O",       "98-86-2", 120.149, 20,   0.00091,    "1525"            ) ! 124
    CALL CalcH("Aniline",                                "C6H7N",       "62-53-3",  93.127, 25,   0.00019,    "1195"            ) !  91
    CALL CalcH("Anisole",                                "C7H8O",      "100-66-3", 108.138, 20,   0.035,      "2926"            ) !  94
    CALL CalcH("Anthracene",                             "C14H10",     "120-12-7", 178.229, 25,   0.0050,     "2989"            ) ! 112
    CALL CalcH("Benz[a]anthracene",                      "C18H12",      "56-55-3", 228.288, 25,   0.00063,    "2989"            ) ! 112
    CALL CalcH("Benzo[ghi]perylene",                     "C22H12",     "191-24-2", 276.330, 25,   0.000042,   "2989"            ) ! 112
    CALL CalcH("Benzo[a]pyrene",                         "C20H12",      "50-32-8", 252.309, 25,   0.000077,   "2989"            ) ! 112
    CALL CalcH("Benzo[e]pyrene",                         "C20H12",     "192-97-2", 252.309, 17,   0.000030,   "3088"            ) ! 104
    CALL CalcH("Biphenyl",                               "C12H10",      "92-52-4", 154.207, 25,   0.028,      "2338"            ) !  22
    CALL CalcH("Bis(2-chloroethyl)_ether",               "C4H8Cl2O",   "111-44-4", 143.012, 20,   0.0029,     "3088"            ) ! 104
    CALL CalcH("Bromobenzene",                           "C6H5Br",     "108-86-1", 157.008, 25,   0.21,        "479"            ) !   5
    CALL CalcH("1-Bromobutane",                          "C4H9Br",     "109-65-9", 137.018, 25,   2.17,        "637"            ) ! 103
    CALL CalcH("Bromochloromethane",                     "CH2BrCl",     "74-97-5", 129.384, 25,   0.15,       "2891"            ) ! 102
    CALL CalcH("Bromoethane",                            "C2H5Br",      "74-96-4", 108.965, 25,   0.77,       "2493"            ) ! 110
    CALL CalcH("Bromomethane",                           "CH3Br",       "74-83-9",  94.939, 20,   0.59,       "1945"            ) ! 121
    CALL CalcH("1-Bromopropane",                         "C3H7Br",     "106-94-5", 122.992, 25,   0.91,       "2493"            ) ! 110
    CALL CalcH("2-Bromopropane",                         "C3H7Br",      "75-26-3", 122.992, 20,   1.19,       "2493"            ) ! 110
    CALL CalcH("1,3-Butadiene",                          "C4H6",       "106-99-0",  54.091, 25,   7.69,        "479"            ) !   5
    CALL CalcH("Butane",                                 "C4H10",      "106-97-8",  58.122, 25,   100.,        "479"            ) !   5
    CALL CalcH("1-Butene",                               "C4H8",       "106-98-9",  56.107, 25,   25.6,       "2611"            ) ! 113
    CALL CalcH("Butylbenzene",                           "C10H14",     "104-51-8", 134.218, 25,   1.30,        "479"            ) !   5
    CALL CalcH("sec-Butylbenzene",                       "C10H14",     "135-98-8", 134.218, 25,   1.89,        "633"            ) !  11 (other CAS: 36383-15-0)
    CALL CalcH("tert-Butylbenzene",                      "C10H14",      "98-06-6", 134.218, 25,   1.20,        "479"            ) !   5
    CALL CalcH("1-Butyne",                               "C4H6",       "107-00-6",  54.091, 25,   1.92,        "479"            ) !   5
    CALL CalcH("2-Chlorobiphenyl",                       "C12H9Cl",   "2051-60-7", 188.652, 25,   0.033,      "2564"            ) ! 107
    CALL CalcH("1-Chlorobutane",                         "C4H9Cl",     "109-69-3",  92.567, 25,   1.45,       "2938"            ) !  98
    CALL CalcH("Chlorodifluoromethane",                  "CHClF2",      "75-45-6",  86.469, 25,   2.94,       "2626"            ) ! 122
    CALL CalcH("Chloroethane",                           "C2H5Cl",      "75-00-3",  64.514, 25,   1.20,       "1855"            ) ! 125
    CALL CalcH("Chloroethene",                           "C2H3Cl",      "75-01-4",  62.498, 25,   2.63,       "1855"            ) ! 125
    CALL CalcH("Chloromethane",                          "CH3Cl",       "74-87-3",  50.488, 25,   0.91,        "479"            ) !   5
    CALL CalcH("1-Chloro-2-methylpropene",               "C4H7Cl",     "513-37-1",  90.552, 25,   0.192,      "3088"            ) ! 104
    CALL CalcH("1-Chloronaphthalene",                    "C10H7Cl",     "90-13-1", 162.616, 25,   0.0357,     "2339"            ) !  28
    CALL CalcH("2-Chloronaphthalene",                    "C10H7Cl",     "91-58-7", 162.616, 25,   0.0333,     "2339"            ) !  28
    CALL CalcH("Chloropentafluoroethane",                "C2ClF5",      "76-15-3", 154.466, 25,   294.,           ""            ) !  18 (probably from ref190)
    CALL CalcH("1-Chloropentane",                        "C5H11Cl",    "543-59-9", 106.594, 25,   2.38,       "2493"            ) ! 110
    CALL CalcH("1-Chloropropane",                        "C3H7Cl",     "540-54-5",  78.541, 25,   1.45,       "2493"            ) ! 110
    CALL CalcH("3-Chloropropene",                        "C3H5Cl",     "107-05-1",  76.525, 25,   1.10,        "479"            ) !   5
    CALL CalcH("Chlorotrifluoromethane",                 "CClF3",       "75-72-9", 104.459, 25,   107.,           ""            ) !  18 (probably from ref190)
    CALL CalcH("Chrysene",                               "C18H12",     "218-01-9", 228.288, 25,   0.000370,   "2989"            ) ! 112
    CALL CalcH("Cycloheptane",                           "C7H14",      "291-64-5",  98.186, 25,   12.2,       "2612"            ) ! 114 (CRC ref should be 113)
    CALL CalcH("1,3,5-Cycloheptatriene",                 "C7H8",       "544-25-2",  92.139, 25,   0.476,       "635"            ) !  13
    CALL CalcH("Cycloheptene",                           "C7H12",      "628-92-2",  96.170, 25,   3.85,       "2611"            ) ! 113
    CALL CalcH("1,4-Cyclohexadiene",                     "C6H8",       "628-41-1",  80.128, 25,   0.909,      "2611"            ) ! 113
    CALL CalcH("Cyclohexane",                            "C6H12",      "110-82-7",  84.159, 25,   17.86,       "479"            ) !   5
    CALL CalcH("Cyclohexene",                            "C6H10",      "110-83-8",  82.143, 25,   4.00,        "622"            ) ! 116
    CALL CalcH("Cyclooctane",                            "C8H16",      "292-64-8", 112.213, 25,   14.08,      "2936"            ) !  97
    CALL CalcH("Cyclopentane",                           "C5H10",      "287-92-3",  70.133, 25,   18.52,       "479"            ) !   5
    CALL CalcH("Cyclopentene",                           "C5H8",       "142-29-0",  68.118, 25,   4.35,       "2923"            ) !  93
    CALL CalcH("Decachlorobiphenyl",                     "C12Cl10",   "2051-24-3", 498.658, 25,   0.0208,     "2302"            ) !   7
    CALL CalcH("trans-Decahydronaphthalene",             "C10H18",     "493-02-7", 138.250, 25,   3.70,        "635"            ) !  13
    CALL CalcH("Decane",                                 "C10H22",     "124-18-5", 142.282,  0,   714.,        "479"            ) !   5
    CALL CalcH("Dibenzofuran",                           "C12H8O",     "132-64-9", 168.191, 25,   0.0213,     "3088"            ) ! 104
    CALL CalcH("1,2-Dibromoethane",                      "C2H4Br2",    "106-93-4", 187.861, 50,   0.833,      "2891", "wrongref") ! 102
    CALL CalcH("Dibromomethane",                         "CH2Br2",      "74-95-3", 173.835, 20,   0.086,       "635"            ) !  13
    CALL CalcH("Dibutyl_ether",                          "C8H18O",     "142-96-1", 130.228,  0,   0.454,      "2457"            ) ! 109
    CALL CalcH("Dibutyl_ether",                          "C8H18O",     "142-96-1", 130.228, 20,   0.48,        "635"            ) !  13
    CALL CalcH("m-Dichlorobenzene",                      "C6H4Cl2",    "541-73-1", 147.002, 25,   0.357,       "479"            ) !   5
    CALL CalcH("p-Dichlorobenzene",                      "C6H4Cl2",    "106-46-7", 147.002, 25,   0.222,      "1943"            ) ! 101
    CALL CalcH("2,5-Dichlorobiphenyl",                   "C12H8Cl2", "34883-39-1", 223.098, 25,   0.040,      "2475"            ) ! 100
    CALL CalcH("Dichlorodifluoromethane",                "CCl2F2",      "75-71-8", 120.914, 20,   33.3,       "2828"            ) ! 126
    CALL CalcH("1,1-Dichloroethane",                     "C2H4Cl2",     "75-34-3",  98.959, 25,   0.588,       "479"            ) !   5
    CALL CalcH("1,2-Dichloroethane",                     "C2H4Cl2",    "107-06-2",  98.959, 25,   0.11,        "479"            ) !   5
    CALL CalcH("1,1-Dichloroethene",                     "C2H2Cl2",     "75-35-4",  96.943, 25,   2.70,       "1855"            ) ! 125
    CALL CalcH("cis-1,2-Dichloroethene",                 "C2H2Cl2",    "156-59-2",  96.943, 25,   0.38,       "1855"            ) ! 125
    CALL CalcH("trans-1,2-Dichloroethene",               "C2H2Cl2",    "156-60-5",  96.943, 25,   1.00,       "1855"            ) ! 125
    CALL CalcH("Dichloromethane",                        "CH2Cl2",      "75-09-2",  84.933, 25,   0.278,      "1945"            ) ! 121
    CALL CalcH("1,2-Dichloropropane",                    "C3H6Cl2",     "78-87-5", 112.986, 25,   0.294,      "1525"            ) ! 124
    CALL CalcH("cis-1,3-Dichloropropene",                "C3H4Cl2",  "10061-01-5", 110.970, 20,   0.238,       "479"            ) !   5
    CALL CalcH("trans-1,3-Dichloropropene",              "C3H4Cl2",  "10061-02-6", 110.970, 20,   0.179,       "479"            ) !   5
    CALL CalcH("2,3-Dichloropropene",                    "C3H4Cl2",     "78-88-6", 110.970, 25,   0.357,       "479"            ) !   5
    CALL CalcH("1,2-Dichloro-1,1,2,2-tetrafluoroethane", "C2Cl2F4",     "76-14-2", 170.921, 25,   111.,       "2013"            ) ! 118
    CALL CalcH("Diethyl_ether",                          "C4H10O",      "60-29-7",  74.121, 25,   0.0909,     "2891"            ) ! 102
    CALL CalcH("Diethyl_ether",                          "C4H10O",      "60-29-7",  74.121, 82,   0.088,       "635"            ) !  13
    CALL CalcH("Diiodomethane",                          "CH2I2",       "75-11-6", 267.836, 30,   0.043,       "583"            ) ! 127
    CALL CalcH("Diisopropyl_ether",                      "C6H14O",     "108-20-3", 102.174, 20,   0.256,      "2646"            ) !  92
    CALL CalcH("2,2-Dimethylbutane",                     "C6H14",       "75-83-2",  86.175, 25,   172.,        "479"            ) !   5
    CALL CalcH("2,3-Dimethylbutane",                     "C6H14",       "79-29-8",  86.175, 25,   130.,        "479"            ) !   5
    CALL CalcH("cis-1,2-Dimethylcyclohexane",            "C8H16",     "2207-01-4", 112.213, 25,   36.,         "479"            ) !   5
    CALL CalcH("trans-1,2-Dimethylcyclohexane",          "C8H16",     "6876-23-9", 112.213, 30,   58.8,       "2936"            ) !  97
    CALL CalcH("1,5-Dimethylnaphthalene",                "C12H12",     "571-61-9", 156.223, 25,   0.0357,     "2339"            ) !  28
    CALL CalcH("2,2-Dimethylpentane",                    "C7H16",      "590-35-2", 100.202, 25,   323.,        "479"            ) !   5
    CALL CalcH("2,3-Dimethylpentane",                    "C7H16",      "565-59-3", 100.202, 25,   175.,        "479"            ) !   5
    CALL CalcH("2,4-Dimethylpentane",                    "C7H16",      "108-08-7", 100.202, 25,   303.,        "479"            ) !   5
    CALL CalcH("3,3-Dimethylpentane",                    "C7H16",      "562-49-2", 100.202, 25,   185.,        "479"            ) !   5
    CALL CalcH("1,2-Diphenylethane",                     "C14H14",     "103-29-7", 182.261, 25,   0.0169,     "2611"            ) ! 113
    CALL CalcH("Diphenyl_ether",                         "C12H10O",    "101-84-8", 170.206, 25,   0.0286,     "3088"            ) ! 104
    CALL CalcH("Diphenylmethane",                        "C13H12",     "101-81-5", 168.234, 25,   0.0132,     "3088"            ) ! 104
    CALL CalcH("Dipropyl_ether",                         "C6H14O",     "111-43-3", 102.174, 25,   0.333,      "2457"            ) ! 109
    CALL CalcH("Dodecane",                               "C12H26",     "112-40-3", 170.334, 25,   769.,        "479"            ) !   5
    CALL CalcH("Epichlorohydrin",                        "C3H5ClO",    "106-89-8",  92.524, 20,   0.003,       "635"            ) !  13 (other CAS: 13403-37-7)
    CALL CalcH("Ethane",                                 "C2H6",        "74-84-0",  30.069, 25,   50.0,        "479"            ) !   5
    CALL CalcH("Ethylbenzene",                           "C8H10",      "100-41-4", 106.165, 25,   0.769,       "479"            ) !   5
    CALL CalcH("Ethylene",                               "C2H4",        "74-85-1",  28.053, 25,   21.7,        "479"            ) !   5
    CALL CalcH("1-Ethylnaphthalene",                     "C12H12",    "1127-76-0", 156.223, 25,   0.0385,      "479"            ) !   5
    CALL CalcH("2-Ethylnaphthalene",                     "C12H12",     "939-27-5", 156.223, 25,   0.083,       "479"            ) !   5
    CALL CalcH("2-Ethyltoluene",                         "C9H12",      "611-14-3", 120.191, 25,   0.435,       "479"            ) !   5
    CALL CalcH("4-Ethyltoluene",                         "C9H12",      "622-96-8", 120.191, 25,   0.50,        "479"            ) !   5
    CALL CalcH("Fluoranthene",                           "C16H10",     "206-44-0", 202.250, 25,   0.00133,    "2989"            ) ! 112
    CALL CalcH("9H-Fluorene",                            "C13H10",      "86-73-7", 166.218, 25,   0.00909,    "2989"            ) ! 112
    CALL CalcH("Fluorobenzene",                          "C6H5F",      "462-06-6",  96.102, 19,   0.625,       "479"            ) !   5
    CALL CalcH("Furan",                                  "C4H4O",      "110-00-9",  68.074, 25,   0.556,      "3088"            ) ! 104
    CALL CalcH("2,2',3,3',4,4',6-Heptachlorobiphenyl",   "C12H3Cl7", "52663-71-5", 395.323, 25,   0.00526,    "2612"            ) ! 114
    CALL CalcH("Heptane",                                "C7H16",      "142-82-5", 100.202, 50,   181.,       "3088"            ) ! 104
    CALL CalcH("1-Heptanol",                             "C7H16O",     "111-70-6", 116.201, 25,   0.00263,     "583", "wrongref") ! 127
    CALL CalcH("2-Heptanone",                            "C7H14O",     "110-43-0", 114.185, 25,   0.0161,     "2837"            ) ! 106
    CALL CalcH("1-Heptene",                              "C7H14",      "592-76-7",  98.186, 25,   43.5,       "3088"            ) ! 104
    CALL CalcH("trans-2-Heptene",                        "C7H14",    "14686-13-6",  98.186, 25,   41.7,        "479"            ) !   5
    CALL CalcH("1-Heptyne",                              "C7H12",      "628-71-7",  96.170, 25,   7.69,       "2611"            ) ! 113
    CALL CalcH("Hexachlorobenzene",                      "C6Cl6",      "118-74-1", 284.782, 25,   0.0667,     "2948"            ) !  67
    CALL CalcH("2,2',3,3',4,4'-Hexachlorobiphenyl",      "C12H4Cl6", "38380-07-3", 360.878, 25,   0.0357,     "2340"            ) !  31
    CALL CalcH("2,2',4,4',6,6'-Hexachlorobiphenyl",      "C12H4Cl6", "33979-03-2", 360.878, 25,   0.0909,     "2540"            ) ! 111
    CALL CalcH("Hexachloroethane",                       "C2Cl6",       "67-72-1", 236.739, 25,   0.40,        "747"            ) ! 123
    CALL CalcH("Hexane",                                 "C6H14",      "110-54-3",  86.175, 25,   169.,        "479"            ) !   5
    CALL CalcH("1-Hexene",                               "C6H12",      "592-41-6",  84.159, 25,   41.7,        "479"            ) !   5
    CALL CalcH("1-Hexyne",                               "C6H10",      "693-02-7",  82.143, 25,   4.17,       "2611"            ) ! 113
    CALL CalcH("Iodobenzene",                            "C6H5I",      "591-50-4", 204.008, 25,   0.083,      "3088"            ) ! 104
    CALL CalcH("1-Iodobutane",                           "C4H9I",      "542-69-8", 184.018, 17,   1.85,       "2612"            ) ! 114
    CALL CalcH("Iodoethane",                             "C2H5I",       "75-03-6", 155.965, 25,   0.67,       "1943"            ) ! 101
    CALL CalcH("Iodomethane",                            "CH3I",        "74-88-4", 141.939, 20,   0.50,       "1945"            ) ! 121
    CALL CalcH("1-Iodopropane",                          "C3H7I",      "107-08-4", 169.992, 20,   0.909,      "2493"            ) ! 110
    CALL CalcH("Isobutane",                              "C4H10",       "75-28-5",  58.122, 25,   110.,       "1945"            ) ! 121
    CALL CalcH("Isobutene",                              "C4H8",       "115-11-7",  56.107, 25,   17.9,           ""            ) !  18 (probably from ref190)
    CALL CalcH("Isobutylbenzene",                        "C10H14",     "538-93-2", 134.218, 25,   3.33,        "479"            ) !   5
    CALL CalcH("Isopentane",                             "C5H12",       "78-78-4",  72.149, 25,   139.,        "479"            ) !   5
    CALL CalcH("Isopropylbenzene",                       "C9H12",       "98-82-8", 120.191, 25,   0.833,      "1525"            ) ! 124
    CALL CalcH("1-Isopropyl-4-methylbenzene",            "C10H14",      "99-87-6", 134.218, 25,   0.77,        "479"            ) !   5
    CALL CalcH("Methane",                                "CH4",         "74-82-8",  16.043, 25,   71.4,       "2828"            ) ! 126
    CALL CalcH("2-Methyl-1,3-butadiene",                 "C5H8",        "78-79-5",  68.118, 25,   7.769,       "479"            ) !   5
    CALL CalcH("3-Methyl-1-butene",                      "C5H10",      "563-45-1",  70.133, 25,   55.5,        "479"            ) !   5
    CALL CalcH("Methylcyclohexane",                      "C7H14",      "108-87-2",  98.186, 26,   40.0,        "479"            ) !   5
    CALL CalcH("Methylcyclopentane",                     "C6H12",       "96-37-7",  84.159, 25,   37.0,        "479"            ) !   5
    CALL CalcH("3-Methylheptane",                        "C8H18",      "589-81-1", 114.229, 25,   370.,        "479"            ) !   5
    CALL CalcH("2-Methylhexane",                         "C7H16",      "591-76-4", 100.202, 25,   345.,        "479"            ) !   5
    CALL CalcH("3-Methylhexane",                         "C7H16",      "589-34-4", 100.202, 25,   249.,        "635"            ) !  13 (other CAS: 78918-91-9)
    CALL CalcH("1-Methylnaphthalene",                    "C11H10",      "90-12-0", 142.197, 25,   0.0455,     "1943"            ) ! 101
    CALL CalcH("2-Methylnaphthalene",                    "C11H10",      "91-57-6", 142.197, 25,   0.0556,     "1943"            ) ! 101
    CALL CalcH("4-Methyloctane",                         "C9H20",     "2216-34-4", 128.255, 25,   1000.,       "479"            ) !   5
    CALL CalcH("Methyloxirane",                          "C3H6O",       "75-56-9",  58.079, 20,   0.0087,      "635"            ) !  13
    CALL CalcH("2-Methylpentane",                        "C6H14",      "107-83-5",  86.175, 25,   169.,        "479"            ) !   5
    CALL CalcH("3-Methylpentane",                        "C6H14",       "96-14-0",  86.175, 25,   172.,        "479"            ) !   5
    CALL CalcH("2-Methyl-1-pentene",                     "C6H12",      "763-29-1",  84.159, 25,   27.8,        "479"            ) !   5
    CALL CalcH("4-Methyl-1-pentene",                     "C6H12",      "691-37-2",  84.159, 25,   62.5,        "479"            ) !   5
    CALL CalcH("2-Methyl-1-propanol",                    "C4H10O",      "78-83-1",  74.121, 25,   0.0010,     "1945"            ) ! 121
    CALL CalcH("2-Methyltetrahydrofuran",                "C5H10O",      "96-47-9",  86.132, 19,   0.67,        "635"            ) !  13
    CALL CalcH("Naphthacene",                            "C18H12",      "92-24-0", 228.288, 25,   0.0000027,  "2611"            ) ! 113
    CALL CalcH("Naphthalene",                            "C10H8",       "91-20-3", 128.171, 25,   0.0454,     "2989"            ) ! 112
    CALL CalcH("Neopentane",                             "C5H12",      "463-82-1",  72.149, 25,   370.,        "479"            ) !   5
    CALL CalcH("Nonane",                                 "C9H20",      "111-84-2", 128.255, 25,   454.,       "2935"            ) ! 120
    CALL CalcH("2,2',3,3',5,5',6,6'-Octachlorobiphenyl", "C12H2Cl8",  "2136-99-4", 429.768, 25,   0.0018,     "1194"            ) !  95
    CALL CalcH("Octane",                                 "C8H18",      "111-65-9", 114.229, 25,   303.,        "479"            ) !   5
    CALL CalcH("1-Octene",                               "C8H16",      "111-66-0", 112.213, 25,   100.,        "479"            ) !   5
    CALL CalcH("1-Octyne",                               "C8H14",      "629-05-0", 110.197, 25,   7.69,       "2611"            ) ! 113
    CALL CalcH("Pentachlorobenzene",                     "C6HCl5",     "608-93-5", 250.337, 25,   0.0714,     "2948"            ) !  67
    CALL CalcH("2,2',4,5,5'-Pentachlorobiphenyl",        "C12H5Cl5", "37680-73-2", 326.433, 25,   0.0244,     "2540"            ) ! 111
    CALL CalcH("Pentachloroethane",                      "C2HCl5",      "76-01-7", 202.294, 25,   0.222,       "479"            ) !   5
    CALL CalcH("1,4-Pentadiene",                         "C5H8",       "591-93-5",  68.118, 25,   12.,         "479"            ) !   5
    CALL CalcH("Pentane",                                "C5H12",      "109-66-0",  72.149, 25,   125.,        "479"            ) !   5
    CALL CalcH("2-Pentanone",                            "C5H10O",     "107-87-9",  86.132, 25,   0.00625,    "2524"            ) ! 105
    CALL CalcH("1-Pentene",                              "C5H10",      "109-67-1",  70.133, 25,   40.0,        "479"            ) !   5
    CALL CalcH("cis-2-Pentene",                          "C5H10",      "627-20-3",  70.133, 25,   22.7,        "479"            ) !   5
    CALL CalcH("Pentylbenzene",                          "C11H16",     "538-68-1", 148.245, 25,   0.588,       "479"            ) !   5
    CALL CalcH("Pentylcyclopentane",                     "C10H20",    "3741-00-2", 140.266, 25,   185.,        "479"            ) !   5
    CALL CalcH("1-Pentyne",                              "C5H8",       "627-19-0",  68.118, 25,   2.5,         "479"            ) !   5
    CALL CalcH("Perylene",                               "C20H12",     "198-55-0", 252.309, 25,   0.000435,   "2966"            ) ! 119
    CALL CalcH("Phenanthrene",                           "C14H10",      "85-01-8", 178.229, 25,   0.00435,    "2989"            ) ! 112
    CALL CalcH("Propane",                                "C3H8",        "74-98-6",  44.096, 25,   66.7,       "1945"            ) ! 121
    CALL CalcH("Propene",                                "C3H6",       "115-07-1",  42.080, 25,   21.3,        "479"            ) !   5
    CALL CalcH("Propylbenzene",                          "C9H12",      "103-65-1", 120.191, 25,   0.714,       "479"            ) !   5
    CALL CalcH("Propylcyclopentane",                     "C8H16",     "2040-96-2", 112.213, 25,   90.9,        "479"            ) !   5
    CALL CalcH("Propyne",                                "C3H4",        "74-99-7",  40.064, 25,   1.11,        "479"            ) !   5
    CALL CalcH("Pyrene",                                 "C16H10",     "129-00-0", 202.250, 25,   0.00133,    "2989"            ) ! 112
    CALL CalcH("trans-Stilbene",                         "C14H12",     "103-30-0", 180.245, 25,   0.0714,     "3088"            ) ! 104
    CALL CalcH("Styrene",                                "C8H8",       "100-42-5", 104.150, 25,   0.294,      "2938"            ) !  98
    CALL CalcH("Styrene",                                "C8H8",       "100-42-5", 104.150, 50,   0.3,         "635"            ) !  13
    CALL CalcH("1,2,3,4-Tetrachlorobenzene",             "C6H2Cl4",    "634-66-2", 215.892, 25,   0.286,      "2935"            ) ! 120
    CALL CalcH("1,2,3,5-Tetrachlorobenzene",             "C6H2Cl4",    "634-90-2", 215.892, 25,   0.159,      "2339"            ) !  28
    CALL CalcH("1,2,4,5-Tetrachlorobenzene",             "C6H2Cl4",     "95-94-3", 215.892, 25,   0.0556,     "3009"            ) ! 115
    CALL CalcH("1,1,1,2-Tetrachloroethane",              "C2H2Cl4",    "630-20-6", 167.849, 25,   0.238,      "1855"            ) ! 125
    CALL CalcH("1,1,2,2-Tetrachloroethane",              "C2H2Cl4",     "79-34-5", 167.849, 25,   0.0417,     "1855"            ) ! 125
    CALL CalcH("Tetrachloroethene",                      "C2Cl4",      "127-18-4", 165.833, 20,   1.61,       "1855"            ) ! 125
    CALL CalcH("Tetrachloromethane",                     "CCl4",        "56-23-5", 153.823, 25,   2.94,       "2626"            ) ! 122
    CALL CalcH("1,2,4,5-Tetramethylbenzene",             "C10H14",      "95-93-2", 134.218, 25,   2.64,        "479"            ) !   5
    CALL CalcH("Toluene",                                "C7H8",       "108-88-3",  92.139, 25,   0.667,      "1525"            ) ! 124
    CALL CalcH("Tribromomethane",                        "CHBr3",       "75-25-2", 252.731, 25,   0.0588,     "1945"            ) ! 121
    CALL CalcH("1,2,3-Trichlorobenzene",                 "C6H3Cl3",     "87-61-6", 181.447, 25,   0.159,      "2926"            ) !  94
    CALL CalcH("1,2,4-Trichlorobenzene",                 "C6H3Cl3",    "120-82-1", 181.447, 25,   0.172,      "3014"            ) ! 108
    CALL CalcH("1,3,5-Trichlorobenzene",                 "C6H3Cl3",    "108-70-3", 181.447, 25,   0.556,      "1139", "wrongref") ! 117
    CALL CalcH("2,4,5-Trichlorobiphenyl",                "C12H7Cl3", "15862-07-4", 257.543, 25,   0.0303,     "2540"            ) ! 111
    CALL CalcH("2,4,6-Trichlorobiphenyl",                "C12H7Cl3", "35693-92-6", 257.543, 25,   0.0667,     "1140"            ) !  99
    CALL CalcH("1,1,1-Trichloroethane",                  "C2H3Cl3",     "71-55-6", 133.404, 25,   1.67,       "1855"            ) ! 125
    CALL CalcH("1,1,2-Trichloroethane",                  "C2H3Cl3",     "79-00-5", 133.404, 25,   0.0909,     "1855"            ) ! 125
    CALL CalcH("Trichloroethene",                        "C2HCl3",      "79-01-6", 131.388, 25,   0.909,      "1855"            ) ! 125
    CALL CalcH("Trichlorofluoromethane",                 "CCl3F",       "75-69-4", 137.368, 20,   9.09,       "2828"            ) ! 126
    CALL CalcH("Trichloromethane",                       "CHCl3",       "67-66-3", 119.378, 25,   0.40,       "1945"            ) ! 121
    CALL CalcH("1,2,3-Trichloropropane",                 "C3H5Cl3",     "96-18-4", 147.431, 25,   0.0277,     "1525"            ) ! 124
    CALL CalcH("1,1,2-Trichloro-1,2,2-trifluoroethane",  "C2Cl3F3",     "76-13-1", 187.375, 25,   34.5,       "1107"            ) !  96
    CALL CalcH("1,2,3-Trimethylbenzene",                 "C9H12",      "526-73-8", 120.191, 25,   0.323,       "479"            ) !   5
    CALL CalcH("1,2,4-Trimethylbenzene",                 "C9H12",       "95-63-6", 120.191, 25,   0.588,       "479"            ) !   5
    CALL CalcH("1,3,5-Trimethylbenzene",                 "C9H12",      "108-67-8", 120.191, 25,   0.588,       "479"            ) !   5
    CALL CalcH("1,1,3-Trimethylcyclohexane",             "C9H18",     "3073-66-3", 126.239, 25,   105.,       "2611"            ) ! 113
    CALL CalcH("1,1,3-Trimethylcyclopentane",            "C8H16",     "4516-69-2", 112.213, 25,   159.,        "479"            ) !   5
    CALL CalcH("2,2,5-Trimethylhexane",                  "C9H20",     "3522-94-9", 128.255, 25,   244.,       "2611"            ) ! 113
    CALL CalcH("2,2,4-Trimethylpentane",                 "C8H18",      "540-84-1", 114.229, 25,   333.,        "479"            ) !   5
    CALL CalcH("2,3,4-Trimethylpentane",                 "C8H18",      "565-75-3", 114.229, 25,   189.,        "479"            ) !   5
    CALL CalcH("Triphenylene",                           "C18H12",     "217-59-4", 228.288, 25,   0.00001,     "634"            ) !  12
    CALL CalcH("o-Xylene",                               "C8H10",       "95-47-6", 106.165, 25,   0.417,      "1943"            ) ! 101
    CALL CalcH("m-Xylene",                               "C8H10",      "108-38-3", 106.165, 25,   0.714,      "1525"            ) ! 124
    CALL CalcH("p-Xylene",                               "C8H10",      "106-42-3", 106.165, 25,   0.526,      "1943"            ) ! 101

  CONTAINS

    SUBROUTINE CalcH (chem_, formula, casrn_, M, temp, Hvpc, xref_, note)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN)           :: chem_, formula, casrn_
      REAL,             INTENT(IN)           :: M
      INTEGER,          INTENT(IN)           :: temp
      REAL,             INTENT(IN)           :: Hvpc
      CHARACTER(LEN=*), INTENT(IN)           :: xref_
      CHARACTER(LEN=*), INTENT(IN), OPTIONAL :: note
      ! formula and M are currently not used here
      chem=chem_ ; casrn=casrn_ ! make global for Output
      Hominus = HvpcSI_TIMES_HscpSI/(Hvpc*1.E3)
      ! All data are taken from other publications. Show only entries if
      ! they have a note:
      IF (PRESENT(note)) THEN
        IF (temp/=25) THEN
          CALL MakeNoteOtherTempK(temp)
        ENDIF
        IF (note=="wrongref") THEN
          type = "?"
          CALL MakeNote(TRIM(ref)//"-"//casrn_, &
            TRIM(citet())//" refers to \citet{" // xref_ // &
            "} as the source, but this value cannot be found there.")
        ELSE
          IF (xref_=="") THEN
            type = "C"
            ! crc ref 18 is Gevantman from an older version of the CRC Handbook
          ELSE
            type = "X"
            CALL SettypeX(xref_)
          ENDIF
        ENDIF
        CALL Output(Hominus)
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3539

  !---------------------------------------------------------------------------

  SUBROUTINE ref3540 ! Hvpb [atm*kg/mol]
    IMPLICIT NONE

    ref = "3540"
    type = "T"

    ! Tab. 2:
    CALL CalcH("ammonia",          "7664-41-7", 160.559,  -8621.06, -25.6767, 0.035388) ! NH3
    CALL CalcH("carbon dioxide",    "124-38-9", 1082.37,  -34417.2, -182.28,  0.25159)  ! CO2
    CALL CalcH("hydrogen sulfide", "7783-06-4", -403.658, 7056.07,  74.6926,  -0.14529) ! H2S
    CALL CalcH("sulfur dioxide",   "7446-09-5", 1135.63,  -32143.3, -198.14,  0.33836)  ! SO2
    CALL CalcH("hydrogen cyanide",   "74-90-8", 7605.09,  -212343., -1319.22, 2.08347)  ! HCN

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, D1, D2, D3, D4)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL,               INTENT(IN) :: D1, D2, D3, D4
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! ln(Hvpb) = D1 + D2/T + D3*ln(T) + D4*T
      Hominus = EXP(D1 + D2/T0 + D3*LOG(T0) + D4*T0)
      Hominus = (1./Hominus)*Hsbp_TO_HscpSI
      ! analytical derivative: (see also gnuplot/ref3540.gnu)
      mindHR = - D2 + D3*T0 + D4*T0*T0 ! d(lnH)/d(1/T) = -delta H/R
      CALL MakeNote("moreTdep")
      CALL Output(Hominus, mindHR)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3540

  !---------------------------------------------------------------------------

  SUBROUTINE ref3541 ! Hsbp [mol/(100 g*atm)]
    IMPLICIT NONE
    ref = "3541"
    type = "M"
    chem = "chlorine (molecular)" ; casrn = "7782-50-5" ! Cl2
    ! p. 743:
    ! factor 1E3 because "H*10^3" is shown in the paper
    ! factor 0.1 converts "per 100 g" to "per kg"
    ! total factor = 0.01
    CALL MakeNote(TRIM(ref), &
      "The same experimental data were also published by \citet{3928}.")
    CALL HTdep( (/ 10., 15., 20., 25. /) + CtoK, &
      0.01 * (/ 11.6, 9.35, 7.75, 6.23 /) * Hsbp_TO_HscpSI, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3541

  !---------------------------------------------------------------------------

  SUBROUTINE ref3544 ! special definition (DeltaG for Hvpx [atm])
    IMPLICIT NONE
    ref = "3544"
    type = "E"
    chem = "hydrogen atom" ; casrn = "12385-13-6"
    ! p. 692 and Tab. 1: 7.12, 7.07 and 7.10
    Hominus = Hvpx_TIMES_HscpSI / EXP(7.1*kcal/(Rgas*T0))
    CALL MakeNote(TRIM(ref), TRIM(citet()) // &
      " assumes that the free energy of solvation of atomic hydrogen is " // &
      "equal to that of He because of a similar van der Waals radius.")
    CALL Output(Hominus)
  END SUBROUTINE ref3544

  !---------------------------------------------------------------------------

  SUBROUTINE ref3545 ! Hscc [1] = Ostwald coefficient and Hsxp [atm]
    IMPLICIT NONE
    ref = "3545"
    type = "M"
    ! Ostwald coefficients from Tab. II, fitting parameters from Tab. III:
    CALL CalcH("radon", "10043-92-2", &
      (/ 5., 10., 15., 20., 25., 30., 35., 40., 45., 50., 55., 60. /) + CtoK, &
      (/ 0.394, 0.372, 0.285, 0.272, 0.232, 0.202, 0.183, 0.161, &
      0.145, 0.130, 0.118, 0.100 /), -2.00, 0.95, -3.96, -0.91)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, ostwald, A, B, C, D)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, ostwald
      REAL,               INTENT(IN)  :: A, B, C, D
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep_ABC(temp, Hscc_TO_HscpSI(ostwald, temp), Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
      ! compare to fitting parameters:
      ! A+B/T'+Cln(T')+DT'
      ! PRINT *, "Tab. II:  ", Hominus, mindHR
      ! Hominus = EXP( A + B/(T0/100.) + C*LOG(T0/100.) + D*T0/100. ) * Hsxp_TO_HscpSI
      ! mindHR = 100.*B - C*T0 - D*T0**2/100.
      ! PRINT *, "Tab. III: ", Hominus, mindHR
      ! -> Hominus looks ok but mindHR is different, maybe because T-dep is not the same
      !    for Hscc and Hsxp?
    END SUBROUTINE CalcH

  END SUBROUTINE ref3545

  !---------------------------------------------------------------------------

  SUBROUTINE ref3546 ! Bunsen coefficient alpha [1]
    IMPLICIT NONE
    ref = "3546"
    type = "M"
    chem = "radon" ; casrn = "10043-92-2" ! Rn
    CALL HTdep( (/ 18., 9., 0. /) + CtoK, &
      (/ 0.285, 0.39, 0.52 /) * alpha_TO_HscpSI, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3546

  !---------------------------------------------------------------------------

  SUBROUTINE ref3547 ! Hvpx [atm]
    IMPLICIT NONE
    ref = "3547"
    type = "M"
    chem = "argon" ; casrn = "7440-37-1" ! Ar
    ! high-temperature data is not used:
    ! 323.16, 323.15, 338.25, 353.15, 364.26, 365.50, 398.85, 399.03, 495.80, 506.55, 540.42, 561.12
    ! 5.203, 5.216, 5.359, 5.359, 5.342, 5.392, 5.302, 5.274, 3.812, 3.644, 2.541, 1.870
    CALL HTdep( (/ 298.18, 298.25, 298.16, 298.26, 303.19, 303.24, 303.25, 308.16, &
      308.15, 308.14, 313.22, 313.14, 313.20 /), 1.E-4 * Hvpx_TIMES_HscpSI / (/ &
      3.974, 3.968, 3.968, 3.972, 4.317, 4.293, 4.332, 4.613, 4.587, 4.576, 4.831, &
      4.890, 4.917 /), Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3547

  !---------------------------------------------------------------------------

  SUBROUTINE ref3548 ! Bunsen coefficient alpha [1]
    IMPLICIT NONE
    ref = "3548"
    type = "M"
    chem = "helium" ; casrn = "7440-59-7" ! He
    CALL HTdep( (/ 30., 30., 30., 30., 25., 25., 10., 10., 2., 2. /) + CtoK, &
      (/ 0.00836, 0.00821, 0.00803, 0.00810, 0.00859, 0.00862, &
      0.00888, 0.00903, 0.00936, 0.00938 /) * alpha_TO_HscpSI, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3548

  !---------------------------------------------------------------------------

  SUBROUTINE ref3549 ! HvpcSI [Pa*m3/mol]
    IMPLICIT NONE

    ref = "3549"
    type = "?"
    !          name                                     CAS-RN  <>    Hvpc       T/C    pH
    CALL CalcH("abamectine",                      "71751-41-2", "<",  2.7E-3,    "20",  "")
    CALL CalcH("acequinocyl",                     "57960-19-7", "",   9.7E-2,    "T?",  "")
    CALL CalcH("acetamiprid",                    "135410-20-7", "<",  5.3E-8,    "25",  "")
    CALL CalcH("Acibenzolar-S-methyl",           "135158-54-2", "",   1.3E-2,    "T?",  "")
    CALL CalcH("acide acetique",                     "64-19-7", "",   9.3E-2,    "T?",  "")
    CALL CalcH("acide benzoique",                    "65-85-0", "",   6.714E-4,  "20",  "")
    CALL CalcH("acide gibbérellique",                "77-06-5", "",   7.5E-7,    "25",  "")
    !CALL CalcH("acide gras en C7-C18",           "67701-09-1", "",   0.33,      "20",  "") ! mix of several species
    CALL CalcH("acide pelargonique",                "112-05-0", "",   0.33,      "20",  "")
    CALL CalcH("acide 1-naphthylacetique",           "86-87-3", "",   3.03E-3,   "20",  "7")
    CALL CalcH("acide 1-naphthylacetique",           "86-87-3", "",   1.26E-4,   "20",  "10.8")
    CALL CalcH("aclonifen",                       "74070-46-5", "",   3.03E-3,   "20",  "")
    CALL CalcH("acrinathrine",                   "101007-06-1", "",   1.08E-2,   "20",  "")
    CALL CalcH("Alpha-cypermethrine",             "67375-30-8", "",   0.053,     "20",  "")
    CALL CalcH("amétoctradine",                  "865318-97-4", "",   4.13E-7,   "20",  "")
    CALL CalcH("amidosulfuron",                  "120923-37-7", "",   1.56E-6,   "20",  "7")
    CALL CalcH("amidosulfuron",                  "120923-37-7", "",   5.22E-4,   "20",  "4")
    CALL CalcH("amidosulfuron",                  "120923-37-7", "",   6.76E-6,   "20",  "9")
    CALL CalcH("aminopyralide",                  "150114-71-9", "",   9.61E-12,  "T?",  "7")
    CALL CalcH("amisulbrom",                     "348635-87-0", "",   2.8E-5,    "T?",  "")
    !CALL CalcH("azadirachtin",                   "11141-17-6", "",   10-14E-19, "T?",  "") ! value is unclear
    CALL CalcH("azimsulfuron",                   "120162-55-2", "",   5E-10,     "T?",  "7")
    CALL CalcH("azimsulfuron",                   "120162-55-2", "",   8E-9,      "T?",  "5")
    CALL CalcH("azimsulfuron",                   "120162-55-2", "",   9E-11,     "T?",  "9")
    CALL CalcH("azoxystrobine",                  "131860-33-8", "",   7.4E-9,    "T?",  "")
    CALL CalcH("beflubutamid",                   "113614-08-7", "",   1.1E-4,    "T?",  "")
    CALL CalcH("benalaxyl",                       "71626-11-4", "",   6.5E-3,    "20",  "")
    CALL CalcH("benalaxyl-M",                     "98243-83-5", "",   2.33E-4,   "20",  "")
    CALL CalcH("benfluraline",                     "1861-40-1", "",   9.5,       "20",  "")
    CALL CalcH("benoxacor",                       "98730-04-2", "",   7.7E-3,    "20",  "")
    CALL CalcH("bensulfuron-methyle",             "83055-99-6", "",   4E-13,     "25",  "9")
    CALL CalcH("bensulfuron-methyle",             "83055-99-6", "",   2E-11,     "25",  "7")
    CALL CalcH("bensulfuron-methyle",             "83055-99-6", "",   6E-10,     "25",  "5")
    CALL CalcH("bentazone",                       "25057-89-0", "",   7.2E-5,    "20",  "")
    CALL CalcH("bentazone",                       "25057-89-0", "",   2.108E-6,  "20",  "")
    CALL CalcH("benthiavalicarb-isopropyl",      "177406-68-7", "<",  8.72E-3,   "20",  "")
    CALL CalcH("benzovindiflupyr",              "1072957-71-1", "",   1.3E-6,    "T?",  "")
    CALL CalcH("betacyflutrine",                  "68359-37-5", "",   9.3E-2,    "20",  "") ! isomer II
    CALL CalcH("betacyflutrine",                  "68359-37-5", "",   0.6,       "20",  "") ! isomer IV
    CALL CalcH("bifenazate",                     "149877-41-8", "<",  1.01E-3,   "20",  "")
    CALL CalcH("bifenox",                         "42576-02-3", ">",  1.62E-4,   "20",  "")
    CALL CalcH("bixafen",                        "581809-46-3", "",   3.89E-5,   "T?",  "")
    CALL CalcH("boscalid",                       "188425-85-6", "",   5.18E-5,   "T?",  "")
    CALL CalcH("bromadiolone",                    "28772-56-7", "",   8.99E-7,   "T?",  "")
    CALL CalcH("bromoxynil butyrate",              "3861-41-4", "",   0.018,     "20",  "")
    CALL CalcH("bromoxynil heptanoate",           "56634-95-8", "",   0.19,      "20",  "")
    CALL CalcH("bromoxynil octanoate",             "1689-99-2", ">",  1.9E-1,    "20",  "")
    CALL CalcH("bromoxynil phenol",                "1689-84-5", "",   1.0E-6,    "T?",  "9")
    CALL CalcH("bromoxynil phenol",                "1689-84-5", "",   4.6E-4,    "T?",  "4")
    CALL CalcH("bromoxynil phenol",                "1689-84-5", "",   8.7E-7,    "T?",  "7")
    CALL CalcH("bromuconazole",                  "116255-48-2", "",   8.8E-6,    "20",  "")
    CALL CalcH("bupirimate",                      "41483-43-6", "",   1.35,      "20",  "")
    CALL CalcH("captane",                           "133-06-2", "",   3E-4,      "20",  "5")
    CALL CalcH("captane",                           "133-06-2", "",   2E-4,      "20",  "7")
    CALL CalcH("carbetamide",                     "16118-49-3", "",   1.93E-8,   "20",  "")
    CALL CalcH("carboxine",                        "5234-68-4", "",   3.2E-5,    "T?",  "")
    CALL CalcH("carfentrazone ethyle",           "128639-02-1", "",   1.56E-4,   "20",  "")
    CALL CalcH("chlorantraniliprole",            "500008-45-7", "",   3.2E-9,    "T?",  "")
    CALL CalcH("chloridazone",                     "1698-60-8", "",   5.3E-10,   "20",  "")
    CALL CalcH("chlormequat chloride",              "999-81-5", "<",  3.16E-10,  "T?",  "")
    CALL CalcH("chlorpyriphos-ethyl",              "2921-88-2", "",   0.478,     "T?",  "")
    CALL CalcH("chlorpyriphos-methyl",             "5598-13-0", "",   0.235,     "T?",  "")
    CALL CalcH("chlorsulfuron",                   "64902-72-3", "",   3.4E-11,   "T?",  "")
    CALL CalcH("chromafénozide",                 "143807-66-3", "",   1.973E-6,  "20",  "7")
    CALL CalcH("chromafénozide",                 "143807-66-3", "",   1.610E-6,  "20",  "4")
    CALL CalcH("chromafénozide",                 "143807-66-3", "",   1.773E-6,  "20",  "9")
    CALL CalcH("clethodim",                       "99129-21-2", "",   1.4E-7,    "20",  "")
    CALL CalcH("clodinafop-propargyl",           "105512-06-9", "",   2.8E-4,    "T?",  "")
    CALL CalcH("clomazone",                       "81777-89-1", "",   4.18E-3,   "T?",  "")
    CALL CalcH("clopyralid",                       "1702-17-6", "",   3.3E-10,   "20",  "")
    CALL CalcH("clopyralid",                       "1702-17-6", "",   2.2E-11,   "20",  "5")
    CALL CalcH("clopyralid",                       "1702-17-6", "",   1.6E-11,   "20",  "9")
    CALL CalcH("clopyralid",                       "1702-17-6", "",   1.8E-11,   "20",  "7")
    CALL CalcH("cloquintocet-mexyl",              "99607-70-2", "",   3.02E-3,   "25",  "")
    CALL CalcH("clothianidine",                  "210880-92-5", "",   2.9E-11,   "20",  "")
    CALL CalcH("cyantraniliprole",               "736994-63-1", "",   1.7E-13,   "T?",  "")
    CALL CalcH("cyazofamid",                     "120116-88-3", "<",  4.03E-2,   "T?",  "")
    CALL CalcH("cycloxidime",                    "101205-02-1", "",   6.141E-5,  "20",  "")
    CALL CalcH("cyflumetofene",                  "400882-07-7", "<",  9.4E-2,    "T?",  "")
    CALL CalcH("cyhalofop butyl",                "122008-85-9", "",   9.51E-4,   "T?",  "")
    CALL CalcH("cymoxanil",                       "57966-95-7", "",   3.8E-5,    "T?",  "7")
    CALL CalcH("cymoxanil",                       "57966-95-7", "",   3.3E-5,    "T?",  "5")
    CALL CalcH("cypermethrine",                   "52315-07-8", "",   0.024,     "20",  "")
    CALL CalcH("cyproconazole",                   "94361-06-5", "",   5E-5,      "T?",  "")
    !CALL CalcH("cyprodinil",                    "121552-61-2", "",   6.6-7.2E-3","25",  "") ! value is unclear
    CALL CalcH("cyprosulfamide",                 "221667-31-8", "<<", 1.43E-7,   "T?",  "9")
    CALL CalcH("cyprosulfamide",                 "221667-31-8", "<<", 1.1E-3,    "T?",  "4")
    CALL CalcH("cyprosulfamide",                 "221667-31-8", "<<", 3.44E-6,   "T?",  "7")
    CALL CalcH("daminozide",                       "1596-84-5", "",   1.43E-2,   "25",  "")
    CALL CalcH("dazomet",                           "533-74-4", "",   5.10E-5,   "20",  "")
    CALL CalcH("deltamethrine",                   "52918-63-5", "",   0.031,     "25",  "")
    CALL CalcH("dicamba",                          "1918-00-9", "",   1E-4,      "25",  "")
    CALL CalcH("dichlorprop-P",                   "15165-67-0", "",   5.6E-5,    "T?",  "")
    CALL CalcH("diclofop methyl",                 "51338-27-3", "",   0.0105,    "20",  "") ! CAS is not 257-141-8
    CALL CalcH("difenoconazole",                 "119446-68-3", "",   9E-7,      "25",  "")
    CALL CalcH("diflubenzuron",                   "35367-38-5", "",   5.8E-4,    "25",  "")
    CALL CalcH("diflufenican",                    "83164-33-4", ">",  1.18E-2,   "T?",  "")
    CALL CalcH("dimethachlore",                   "50563-36-5", "",   1.7E-4,    "25",  "")
    CALL CalcH("Dimethenamid-P",                 "163515-14-8", "",   4.8E-4,    "25",  "")
    CALL CalcH("dimethomorphe",                  "110488-70-5", "",   5.4E-6,    "T?",  "") ! E
    CALL CalcH("dimethomorphe",                  "110488-70-5", "",   2.5E-5,    "T?",  "") ! Z
    CALL CalcH("dimoxystrobine",                 "149961-52-4", "",   4.554E-8,  "T?",  "")
    !CALL CalcH("disodium phosphanate",           "13708-85-5", "<",  1.90,      "T?",  "") ! inorganic salt, other CAS: 130184-07-5, 16926-95-7
    CALL CalcH("dithianon",                        "3347-22-6", "",   1.347E-7,  "20",  "")
    CALL CalcH("doguadine",                        "2439-10-3", "",   9.1E-6,    "T?",  "")
    CALL CalcH("emamectine benzoate",            "155569-91-8", "",   4.0E-2,    "T?",  "9")
    CALL CalcH("emamectine benzoate",            "155569-91-8", "",   1.7E-4,    "T?",  "7")
    CALL CalcH("emamectine benzoate",            "155569-91-8", "",   1.3E-5,    "T?",  "5")
    CALL CalcH("esfenvalerate",                   "66230-04-4", "",   4.92E-4,   "T?",  "")
    CALL CalcH("ethephon",                        "16672-87-0", "<",  1.45E-7,   "T?",  "")
    CALL CalcH("ethofumesate",                    "26225-79-6", "",   3.72E-3,   "25",  "")
    CALL CalcH("etofenprox",                      "80844-07-1", "",   0.0136,    "25",  "")
    CALL CalcH("etoxazole",                      "153233-91-1", "",   3.6E-2,    "25",  "")
    CALL CalcH("famoxadone",                     "131807-57-3", "",   4.6E-3,    "T?",  "")
    CALL CalcH("fenazaquin",                     "120928-09-8", "",   5.71E-2,   "T?",  "")
    CALL CalcH("fenbuconazole",                  "114369-43-6", "",   3.01E-5,   "T?",  "")
    CALL CalcH("fenhexamid",                     "126833-17-8", "",   5E-6,      "20",  "7")
    CALL CalcH("fenhexamid",                     "126833-17-8", "",   3E-7,      "20",  "9")
    CALL CalcH("fenhexamid",                     "126833-17-8", "",   9E-6,      "20",  "5")
    CALL CalcH("fenoxaprop-P-ethyl",              "71283-80-2", "",   2.739E-4,  "20",  "")
    CALL CalcH("fenoxycarbe",                     "72490-01-8", "",   3.3E-5,    "T?",  "")
    CALL CalcH("fenpicoxamide",                  "517875-34-2", "",   2.38E-3,   "25",  "")
    CALL CalcH("fenpropidine",                    "67306-00-7", "",   10.7,      "25",  "")
    CALL CalcH("fenpropimorphe",                  "67564-91-4", "",   2.74,      "T?",  "")
    CALL CalcH("fenpyrazamine",                  "473798-59-3", "",   1.62E-4,   "20",  "")
    CALL CalcH("flazasulfuron",                  "104040-78-0", "<",  2.58E-6,   "25",  "")
    CALL CalcH("flonicamid",                     "158062-67-0", "",   4.2E-8,    "T?",  "")
    CALL CalcH("florasulam",                     "145701-23-1", "",   4.35E-7,   "20",  "7")
    CALL CalcH("florasulam",                     "145701-23-1", "",   2.94E-8,   "20",  "9")
    CALL CalcH("florasulam",                     "145701-23-1", "",   3.29E-5,   "20",  "5")
    CALL CalcH("fluazifop P butyl",               "79241-46-6", "",   4.9E-2,    "20",  "")
    CALL CalcH("fluazinam",                       "79622-59-6", "",   25.9,      "20",  "")
    CALL CalcH("fludioxonyl",                    "131341-86-1", "",   5.4E-5,    "T?",  "")
    CALL CalcH("flufenacet",                     "142459-58-3", "",   9E-4,      "20",  "")
    CALL CalcH("flumioxazine",                   "103361-09-7", "",   6.36E-2,   "20",  "")
    CALL CalcH("fluopicolide",                   "239110-15-7", "",   4.15E-5,   "T?",  "")
    CALL CalcH("fluopyram",                      "658066-35-4", "",   2.98E-5,   "T?",  "")
    CALL CalcH("fluorure de sulfuryle",            "2699-79-8", "",   1.6E5,     "20",  "")
    CALL CalcH("fluoxastrobine",                 "361377-29-9", "",   1E-7,      "20",  "")
    CALL CalcH("fluquinconazole",                "136426-54-5", "",   2.09E-6,   "T?",  "")
    CALL CalcH("flurochloridone",                 "61213-25-0", "",   2.40E-3,   "25",  "")
    CALL CalcH("fluroxypyr",                      "69377-81-7", "",   1.69E-10,  "20",  "")
    CALL CalcH("fluroxypyr methylheptyl ester",   "81406-37-3", "",   0.027,     "20",  "")
    CALL CalcH("flutolanil",                      "66332-96-5", "",   1.65E-5,   "T?",  "")
    CALL CalcH("flutriafol",                      "76674-21-0", "",   1.27E-6,   "20",  "")
    CALL CalcH("fluxapyroxad",                   "907204-31-3", "",   3.028E-7,  "20",  "")
    CALL CalcH("folpel",                            "133-07-3", "",   8E-3,      "25",  "")
    CALL CalcH("foramsulfuron",                  "173159-57-4", "",   5.8E-12,   "20",  "")
    CALL CalcH("forchlorfenuron",                 "68157-60-8", "",   2.922E-7,  "T?",  "")
    CALL CalcH("formetanate",                     "23422-53-9", "",   5E-10,     "T?",  "")
    CALL CalcH("fosetyl-al",                      "39148-24-8", "<",  3.2E-10,   "20",  "")
    CALL CalcH("fosthiazate",                     "98886-44-3", "",   1.76E-5,   "T?",  "")
    CALL CalcH("gamma cyhalothrine",              "76703-62-3", "",   0.0221,    "T?",  "")
    !CALL CalcH("gibberelline A4 A7",              "8030-53-3", "",   2E-5?0.065","T?",  "") ! value is unclear
    CALL CalcH("glyphosate",                       "1071-83-6", "",   2.1E-7,    "25",  "")
    CALL CalcH("halauxifen-methyl",              "943831-98-9", "",   1.23E-6,   "T?",  "5")
    CALL CalcH("halauxifen-methyl",              "943831-98-9", "",   1.22E-6,   "T?",  "7")
    CALL CalcH("halauxifen-methyl",              "943831-98-9", "",   1.20E-6,   "T?",  "9")
    CALL CalcH("halauxifen-methyl",              "943831-98-9", "",   1.11E-6,   "T?",  "")
    CALL CalcH("haloxyfop-R ester methylique",    "72619-32-0", "",   1.2E-3,    "T?",  "")
    CALL CalcH("heptamaloxyloglucan",            "870721-81-6", "",   0.24E-13,  "20",  "")
    CALL CalcH("hexythiazox",                     "78587-05-0", "",   1.19E-2,   "20",  "")
    !CALL CalcH("huile de colza",                  "8002-13-9", "",   1.49,      "T?",  "") ! rapeseed oil is not a pure substance?
    !CALL CalcH("huile de girofle",                "8000-34-8", "",   0.2,       "T?",  "") ! clove oil is not a pure substance?
    CALL CalcH("huile essentielle d'orange",       "8028-48-6", "",   1580.,     "T?",  "") ! D-limonene
    CALL CalcH("hydrazide maleique",                "123-33-1", "",   4.05E-8,   "20",  "")
    CALL CalcH("hydroxy-8-quinoleine",              "148-24-3", "",   1.83E-3,   "T?",  "")
    CALL CalcH("hymexazol",                       "10004-44-1", "",   1.4E-4,    "T?",  "")
    CALL CalcH("imazalil",                        "35554-44-0", "",   0.000108,  "T?",  "")
    CALL CalcH("imazamox",                       "114311-32-9", "",   4.9E-12,   "T?",  "")
    CALL CalcH("imazaquine",                      "81335-37-7", "",   3.74E-12,  "20",  "")
    CALL CalcH("imidaclopride",                  "138261-41-3", "",   1.7E-10,   "20",  "")
    CALL CalcH("indoxacarbe",                    "173584-44-6", "<",  6E-5,      "25",  "")
    CALL CalcH("iodosulfuron",                   "144550-36-7", "",   2.29E-11,  "20",  "")
    CALL CalcH("ipconazole",                     "125225-28-7", "",   3E-5,      "T?",  "")
    CALL CalcH("iprovalicarbe",                  "140923-17-7", "",   1.4E-6,    "20",  "")
    CALL CalcH("isopyrazam",                     "881685-58-1", "",   1.9E-4,    "T?",  "") ! syn
    CALL CalcH("isopyrazam",                     "881685-58-1", "",   3.7E-5,    "T?",  "") ! anti
    CALL CalcH("isoxaben",                        "82558-50-7", "",   1.96E-4,   "20",  "")
    CALL CalcH("isoxaflutole",                   "141112-29-0", "",   1.87E-5,   "20",  "")
    CALL CalcH("kresoxim methyl",                "143390-89-0", "",   3.6E-4,    "T?",  "")
    CALL CalcH("lambda cyhalothrine",             "91465-08-6", "",   0.02,      "20",  "")
    CALL CalcH("laminarine",                       "9008-22-4", "<",  3.45E-7,   "23",  "")
    CALL CalcH("lenacile",                         "2164-08-1", "",   1.3E-7,    "T?",  "")
    CALL CalcH("mancozebe",                        "8018-01-7", "",   6.17E-2,   "20",  "") ! other CAS: 8065-67-5
    CALL CalcH("mandestrobine",                  "173662-97-0", "",   6.66E-7,   "20",  "")
    CALL CalcH("mandipropamide",                 "374726-62-2", "<",  9.2E-5,    "25",  "")
    CALL CalcH("MCPA",                               "94-74-6", "",   5.5E-5,    "25",  "")
    CALL CalcH("MCPB",                               "94-81-5", "",   3E-5,      "T?",  "")
    CALL CalcH("mecoprop-P",                      "16484-77-8", "",   5.7E-5,    "20",  "")
    CALL CalcH("mefenpyr diethyl",               "135590-91-9", "",   2.61E-4,   "25",  "")
    CALL CalcH("mefenpyr diethyl",               "135590-91-9", "",   1.18E-4,   "20",  "")
    CALL CalcH("méfentrifluconazole",           "1417782-03-6", "",   1.6E-3,    "20",  "")
    CALL CalcH("mepanipyrim",                    "110235-47-7", "",   1.67E-3,   "T?",  "")
    CALL CalcH("mepiquat-chlorure",               "24307-26-4", "",   3E-12,     "T?",  "")
    CALL CalcH("meptyldinocap",                     "131-72-6", "",   1.16E-2,   "T?",  "")
    CALL CalcH("mesosulfuron methyl",            "208465-21-8", "",   1.145E-13, "20",  "9")
    CALL CalcH("mesosulfuron methyl",            "208465-21-8", "",   3.649E-12, "20",  "7")
    CALL CalcH("mesosulfuron methyl",            "208465-21-8", "",   2.434E-10, "20",  "5")
    CALL CalcH("mesotrione",                     "104206-82-8", "<",  5.1E-7,    "20",  "")
    CALL CalcH("metalaxyl-M",                     "70630-17-0", "",   3.5E-5,    "25",  "")
    CALL CalcH("metaldehyde",                       "108-62-3", "",   3.5,       "20",  "")
    CALL CalcH("metamitrone",                     "41394-05-2", "",   8.95E-8,   "T?",  "")
    CALL CalcH("metazachlore",                    "67129-08-2", "",   5.9E-5,    "20",  "")
    CALL CalcH("metconazole",                    "125116-23-6", "",   2.21E-7,   "20",  "")
    CALL CalcH("methoxyfenozide",                "161050-58-4", "<",  1.64E-4,   "20",  "")
    CALL CalcH("zineb",                           "12122-67-7", "<",  0.0054,    "20",  "") ! listed as metiram 9006-42-2
    CALL CalcH("metobromuron",                     "3060-89-7", "",   1.14E-4,   "20",  "")
    CALL CalcH("metosulam",                      "139528-85-1", "",   8E-13,     "20",  "")
    CALL CalcH("metrafenone",                    "220899-03-6", "",   0.132,     "20",  "")
    CALL CalcH("metribuzine",                     "21087-64-9", "",   2.0E-5,    "T?",  "")
    CALL CalcH("metsulfuron methyle",             "74223-64-6", "",   4.8E-9,    "T?",  "")
    CALL CalcH("milbemectine",                    "51596-10-2", "",   1.55E-3,   "T?",  "") ! MILBEMYCYNE A4
    CALL CalcH("milbemectine",                    "51596-10-2", "",   2.56E-3,   "T?",  "") ! MILBEMYCYNE A3
    CALL CalcH("myclobutanil",                    "88671-89-0", "",   4.33E-4,   "20",  "")
    CALL CalcH("napropamide",                     "15299-99-7", "",   8.1E-5,    "T?",  "")
    CALL CalcH("nicosulfuron",                   "111991-09-4", "",   1.48E-11,  "20",  "")
    CALL CalcH("oryzalin",                        "19044-88-3", "",   2.74E-8,   "20",  "5")
    CALL CalcH("oryzalin",                        "19044-88-3", "",   4.07E-9,   "20",  "10")
    CALL CalcH("oryzalin",                        "19044-88-3", "",   3.37E-8,   "20",  "7")
    CALL CalcH("oxamyl",                          "23135-22-0", "",   4.89E-8,   "25",  "")
    CALL CalcH("oxathiapiprolin",               "1003318-67-9", "",   3.521E-3,  "20",  "")
    CALL CalcH("oxyfluorfene",                    "42874-03-3", "",   2.382E-2,  "25",  "")
    CALL CalcH("paclobutrazol",                   "76738-62-0", "",   2.39E-5,   "T?",  "")
    CALL CalcH("penconazole",                     "66246-88-6", "",   6.6E-4,    "20",  "")
    CALL CalcH("pencycuron",                      "66063-05-6", "",   5E-7,      "20",  "")
    CALL CalcH("pendimethaline",                  "40487-42-1", "",   1.27,      "25",  "")
    CALL CalcH("penoxsulame",                    "219714-96-2", "",   8.25E-15,  "T?",  "9")
    CALL CalcH("penoxsulame",                    "219714-96-2", "",   2.13E-12,  "T?",  "5")
    CALL CalcH("penoxsulame",                    "219714-96-2", "",   2.95E-14,  "T?",  "7")
    CALL CalcH("penthiopyrad",                   "183675-82-3", "",   1.40E-4,   "20",  "5")
    CALL CalcH("penthiopyrad",                   "183675-82-3", "",   7.66E-3,   "20",  "7")
    CALL CalcH("penthiopyrad",                   "183675-82-3", "",   4.15E-4,   "20",  "4")
    CALL CalcH("penthiopyrad",                   "183675-82-3", "",   6.36E-3,   "20",  "10")
    CALL CalcH("pethoxamid",                     "106700-29-2", "",   1.18E-3,   "20",  "")
    CALL CalcH("phenmediphame",                   "13684-63-4", "",   5E-8,      "20",  "")
    CALL CalcH("phosmet",                           "732-11-6", "",   1.36E-3,   "25",  "")
    CALL CalcH("piclorame",                        "1918-02-1", "",   4.7E-5,    "T?",  "")
    CALL CalcH("picolinafen",                    "137641-05-5", "",   1.6E-3,    "20",  "")
    CALL CalcH("pinoxaden",                      "243973-20-8", "",   9.2E-7,    "25",  "")
    CALL CalcH("prochloraz",                      "67747-09-5", "",   1.64E-3,   "T?",  "")
    CALL CalcH("prohexadione-calcium",           "127277-53-6", "",   1.92E-5,   "T?",  "")
    CALL CalcH("propamocarbe",                    "24579-73-5", "",   8.5E-9,    "T?",  "")
    CALL CalcH("propaquizafop",                  "111479-05-1", "",   9.2E-8,    "20",  "")
    CALL CalcH("propoxycarbazone",               "181274-15-7", "<",  1E-10,     "20",  "")
    CALL CalcH("propyzamide",                     "23950-58-5", "",   7.52E-9,   "20",  "")
    CALL CalcH("proquinazid",                    "189278-12-4", "",   3E-2,      "25",  "")
    CALL CalcH("prosulfocarbe",                   "52888-80-9", "",   1.52E-2,   "20",  "")
    CALL CalcH("prosulfuron",                     "94125-34-5", "<",  3E-4,      "25",  "")
    CALL CalcH("prothioconazole",                "178928-70-6", "<",  3E-5,      "20",  "")
    CALL CalcH("pyraclostrobine",                "175013-18-0", "",   5.307E-6,  "20",  "")
    CALL CalcH("pyraflufen ethyl",               "129630-19-9", "",   2.2E-5,    "20",  "")
    CALL CalcH("pyrethrines",                      "8003-34-7", "",   7.19E-5,   "25",  "") ! pyrethrin 2
    CALL CalcH("pyrethrines",                      "8003-34-7", "",   7.83E-2,   "25",  "") ! pyrethrin 1
    CALL CalcH("pyridate",                        "55512-33-9", "",   1.21E-4,   "20",  "")
    CALL CalcH("pyrimethanil",                    "53112-28-0", "",   0.0036,    "T?",  "")
    CALL CalcH("pyrimicarbe",                     "23103-98-2", "",   2.9E-5,    "20",  "5.2")
    CALL CalcH("pyrimicarbe",                     "23103-98-2", "",   3.3E-5,    "20",  "7.4")
    CALL CalcH("pyrimicarbe",                     "23103-98-2", "",   3.3E-5,    "20",  "9.3")
    CALL CalcH("pyrimicarbe",                     "23103-98-2", "",   3.6E-5,    "20",  "")
    CALL CalcH("pyrimiphos-methyl",               "29232-93-7", "",   0.06,      "T?",  "")
    CALL CalcH("pyriofénone",                    "688046-61-9", "",   1.9E-4,    "T?",  "")
    CALL CalcH("pyriproxyfen",                    "95737-68-1", "<",  7.37E-2,   "23",  "")
    CALL CalcH("pyroxsulame",                    "422556-08-9", "",   6.94E-7,   "T?",  "")
    CALL CalcH("Quinmerac",                       "90717-03-6", "",   1E-10,     "T?",  "")
    CALL CalcH("quinoclamine",                     "2797-51-5", "",   3.05E-5,   "T?",  "")
    CALL CalcH("quizalofop-P-ethyl",              "76578-14-8", "",   6.7E-5,    "20",  "") ! other CAS: 100646-51-3
    CALL CalcH("rimsulfuron",                    "122931-48-0", "",   1.1E-7,    "25",  "9")
    CALL CalcH("rimsulfuron",                    "122931-48-0", "",   4.6E-6,    "25",  "5")
    CALL CalcH("rimsulfuron",                    "122931-48-0", "",   8.3E-8,    "25",  "7")
    CALL CalcH("sedaxane",                       "874967-67-6", "",   4.0E-6,    "25",  "")
    CALL CalcH("silthiofam",                     "175217-20-6", "",   0.54,      "T?",  "")
    CALL CalcH("sintofen",                       "130561-48-7", "",   7.28E-8,   "20",  "4")
    CALL CalcH("sintofen",                       "130561-48-7", "",   1.84E-10,  "20",  "9")
    CALL CalcH("sintofen",                       "130561-48-7", "",   1.64E-10,  "20",  "7")
    CALL CalcH("s-metolachlore",                  "87392-12-9", "",   2.2E-3,    "25",  "")
    CALL CalcH("soufre",                           "7704-34-9", "",   0.05,      "20",  "")
    CALL CalcH("spinetoram",                     "935545-74-7", "",   9.8E-3,    "T?",  "5")  ! XDE-175-L
    CALL CalcH("spinetoram",                     "935545-74-7", "",   9.4E-5,    "T?",  "5")  ! XDE-175-J
    CALL CalcH("spinetoram",                     "935545-74-7", "",   6.3E-3,    "T?",  "10") ! XDE-175-J
    CALL CalcH("spinetoram",                     "935545-74-7", "",   5.0E-4,    "T?",  "")   ! XDE-175-L
    CALL CalcH("spinetoram",                     "935545-74-7", "",   3.5E-3,    "T?",  "7")  ! XDE-175-J
    CALL CalcH("spinetoram",                     "935545-74-7", "",   3.4E-4,    "T?",  "7")  ! XDE-175-L
    CALL CalcH("spinetoram",                     "935545-74-7", "",   2.3E-2,    "T?",  "10") ! XDE-175-L
    CALL CalcH("spinetoram",                     "935545-74-7", "",   4.0E-3,    "T?",  "")   ! XDE-175-J
    CALL CalcH("spinosad",                       "168316-95-8", "",   1.89E-7,   "T?",  "")   ! SPINOSAD A
    CALL CalcH("spinosad",                       "168316-95-8", "",   2.32E-5,   "T?",  "")   ! SPINOSAD D
    CALL CalcH("spirodiclofen",                  "148477-71-8", "<",  2E-3,      "20",  "")
    CALL CalcH("spirotetramate",                 "203313-25-1", "",   6.99E-8,   "T?",  "7")
    CALL CalcH("spirotetramate",                 "203313-25-1", "",   6.24E-8,   "T?",  "4")
    CALL CalcH("spirotetramate",                 "203313-25-1", "",   1.09E-7,   "T?",  "9")
    CALL CalcH("spiroxamine",                    "118134-30-8", "",   2.5E-3,    "20",  "") ! DIASTEREOISOMERE A
    CALL CalcH("spiroxamine",                    "118134-30-8", "",   5E-3,      "20",  "") ! DIASTEREOISOMERE B
    CALL CalcH("sulcotrione",                     "99105-77-8", "",   6E-7,      "20",  "4.8")
    CALL CalcH("Sulfosulfuron",                  "141776-32-1", "",   2.97E-8,   "20",  "9")
    CALL CalcH("Sulfosulfuron",                  "141776-32-1", "",   8.15E-7,   "20",  "5")
    CALL CalcH("Sulfosulfuron",                  "141776-32-1", "",   8.83E-9,   "20",  "7")
    CALL CalcH("tau-fluvalinate",                "102851-06-9", "",   1.2E-4,    "25",  "")
    CALL CalcH("tebuconazole",                   "107534-96-3", "",   1E-5,      "20",  "")
    CALL CalcH("tebufenozide",                   "112410-23-8", "<",  6.6E-5,    "25",  "")
    CALL CalcH("tebufenpyrad",                   "119168-77-3", "<",  1.1E-3,    "T?",  "")
    CALL CalcH("tefluthrine",                     "79538-32-2", "",   2E2,       "20",  "")
    CALL CalcH("tembotrione",                    "335104-84-2", "",   1.71E-10,  "20",  "")
    CALL CalcH("terbuthylazine",                   "5915-41-3", "",   2.3E-3,    "T?",  "")
    CALL CalcH("tetraconazole",                  "112281-77-3", "",   3.6E-4,    "T?",  "")
    CALL CalcH("thiabendazole",                     "148-79-8", "",   3.7E-6,    "T?",  "")
    CALL CalcH("thiacloprid",                    "111988-49-9", "",   4.8E-10,   "T?",  "")
    CALL CalcH("thiamethoxam",                   "153719-23-4", "",   4.7E-10,   "25",  "")
    CALL CalcH("thiencarbazone-methyl",          "317815-83-1", "",   4.77E-13,  "T?",  "3.9")
    CALL CalcH("thiencarbazone-methyl",          "317815-83-1", "",   2.00E-13,  "T?",  "4")
    CALL CalcH("thiencarbazone-methyl",          "317815-83-1", "",   7.88E-14,  "T?",  "7")
    CALL CalcH("thiencarbazone-methyl",          "317815-83-1", "",   8.24E-14,  "T?",  "9")
    CALL CalcH("thifensulfuron methyle",          "79277-27-3", "",   9.6E-15,   "T?",  "7")
    CALL CalcH("thifensulfuron methyle",          "79277-27-3", "",   2.8E-13,   "T?",  "5")
    CALL CalcH("thiophanate-methyl",              "23564-05-8", "",   1.67E-4,   "T?",  "")
    CALL CalcH("Tolclophos methyle",              "57018-04-9", "",   0.37,      "20",  "")
    CALL CalcH("triadimenol",                     "55219-65-3", "",   2.2E-6,    "T?",  "")
    CALL CalcH("tri-allate",                       "2303-17-5", "",   0.89,      "20",  "")
    CALL CalcH("tribenuron-methyl",              "101200-48-0", "",   9.5E-10,   "T?",  "5")
    CALL CalcH("tribenuron-methyl",              "101200-48-0", "<",  1.33E-10,  "T?",  "9")
    CALL CalcH("triclopyr",                       "55335-06-3", "",   8.8E-5,    "20",  "")
    CALL CalcH("triclopyr",                       "55335-06-3", "",   4.7E-6,    "20",  "5")
    CALL CalcH("triclopyr",                       "55335-06-3", "",   4.4E-6,    "20",  "7")
    CALL CalcH("triclopyr butoxy ethyl ester",    "64700-56-7", "",   2.9E-3,    "20",  "")
    CALL CalcH("trifloxystrobine",               "141517-21-7", "",   2.3E-3,    "25",  "")
    CALL CalcH("triflusulfuron-methyle",         "126535-15-7", "<",  2.4E-3,    "T?",  "4")
    CALL CalcH("trinexapac-ethyl",                "95266-40-3", "",   5.4E-4,    "25",  "")
    CALL CalcH("triticonazole",                  "131983-72-7", "",   3.0E-5,    "T?",  "")
    CALL CalcH("tritosulfuron",                  "142469-14-5", "<",  1E-4,      "20",  "")
    CALL CalcH("valifenalate",                   "283159-90-0", "",   1.6E-6,    "20",  "5.4")
    CALL CalcH("zeta-cypermethrine",              "52315-07-8", "",   2E-8,      "T?",  "")
    CALL CalcH("zirame",                            "137-30-4", "",   5.7E-3,    "25",  "")
    CALL CalcH("zoxamide",                       "156052-68-5", "<",  6.59E-3,   "T?",  "")
    CALL CalcH("1-decanol",                         "112-30-1", "",   22.52,     "T?",  "")
    CALL CalcH("1-naphthylacetamide",                "86-86-2", "",   1.02E-8,   "20",  "")
    CALL CalcH("1,4-dimethylnaphthalene",           "571-58-4", "",   76.7,      "25",  "")
    CALL CalcH("2,4-D",                              "94-75-7", "",   4.0E-6,    "20",  "")
    CALL CalcH("2,4-D ethylhexyl ester",           "1928-43-4", "",   1.82,      "T?",  "")
    CALL CalcH("2,4-DB",                             "94-82-6", "",   4.6E-6,    "T?",  "")
    CALL CalcH("6-benzyladenine",                  "1214-39-7", "",   8.9,       "T?",  "")

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, limit_, Hvpc, temp_, pH)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      CHARACTER(LEN=*), INTENT(IN) :: limit_
      REAL,             INTENT(IN) :: Hvpc
      CHARACTER(LEN=*), INTENT(IN) :: temp_, pH
      INTEGER :: temp_int ! temperature in Celsius as integer
      chem=chem_ ; casrn=casrn_ ! make global for Output
      IF (temp_=="T?") THEN
        CALL MakeNote("unknownT")
      ELSE
        READ(temp_,*) temp_int
        CALL MakeNoteOtherTempC(temp_int)
      ENDIF
      IF (pH/="") THEN
        CALL MakeNote(TRIM(ref)//"-pH"//pH, "At pH = "//pH//".")
      ENDIF
      Hominus = HvpcSI_TIMES_HscpSI/Hvpc
      CALL MakeNote(TRIM(ref), "Data taken from the AGRITOX database file agritox-20210608.zip.")
      IF (limit_=="<") THEN
        CALL Output(Hominus, limit=">") ! lower limit of Hvpc is upper limit of Hscp
      ELSE IF (limit_==">") THEN
        CALL Output(Hominus, limit="<") ! upper limit of Hvpc is lower limit of Hscp
      ELSE
        CALL Output(Hominus)
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3549

  !---------------------------------------------------------------------------

  SUBROUTINE ref3550 ! Hvpx [MPa]
    IMPLICIT NONE

    ref = "3550"
    type = "Q"
    ! data at the optimized scaling factors shown in Tab. 6 from
    ! supplement (only at 293 K):
    CALL CalcH("helium",   "7440-59-7", 14291.3) ! from Tab. S1
    CALL CalcH("neon",     "7440-01-9", 15329.0) ! from Tab. S5
    CALL CalcH("argon",    "7440-37-1",  4721.1) ! from Tab. S9
    CALL CalcH("krypton",  "7439-90-9",  2199.2) ! from Tab. S13
    CALL CalcH("xenon",    "7440-63-3",   672.6) ! from Tab. S17

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, Ki)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL,               INTENT(IN) :: Ki
      chem=chem_ ; casrn=casrn_ ! make global for Output
      Hominus =  cH2O / (Ki*1E6)
      CALL MakeNoteOtherTempK(293)
      CALL Output(Hominus)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3550

  !---------------------------------------------------------------------------

  ! ref3551 NOTUSED: The paper only presents molecular models but no
  !                  Henry constants. It is unclear how ref3517 used it
  !                  to calculate Henry constants.

  !---------------------------------------------------------------------------

  ! ref3552 NOTUSED: The paper only presents a molecular model for Rn
  !                  but no Henry constants. It is unclear how ref3517
  !                  used it to calculate Henry constants.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3554 ! Hvpc [Pa*m3/mol]
    IMPLICIT NONE

    ref = "3554"
    type = "M"
    ! Results on p. 625:
    CALL CalcH("linalool",  "78-70-6", 2.62)
    CALL CalcH("limonene", "138-86-3", 3892.)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, Hpc)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL, INTENT(IN)             :: Hpc
      chem=chem_ ; casrn=casrn_ ! make global for Output
      Hominus = 1./HPc
      CALL Output(Hominus)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3554

  !---------------------------------------------------------------------------

  SUBROUTINE ref3555 ! Hscp [M/atm]
    IMPLICIT NONE
    ref = "3555"
    type = "M"

    chem = "methyl salicylate" ; casrn = "119-36-8" ! C8H8O3
    Hominus = 33.5*Hscp_TO_HscpSI
    CALL Output(Hominus)

    chem = "methyl jasmonate" ; casrn = "1211-29-6" ! C{13}H{20}O3
    Hominus = 5018.*Hscp_TO_HscpSI
    CALL Output(Hominus)

  END SUBROUTINE ref3555

  !---------------------------------------------------------------------------

  SUBROUTINE ref3556 ! Hvpx [MPa]
    IMPLICIT NONE

    ref = "3556"
    type = "M"
    chem = "ethanethiol" ; casrn = "75-08-1" ! C2H5SH ethyl mercaptan
    ! Tab. 4, only data up to 323 K is used here:
    ndata = 4
    ALLOCATE(temp(ndata), Harray(ndata))
    temp = (/ 298.15, 310.9, 313.15, 323.28 /)
    Harray = 1E-6 * cH2O / (/ 22.9, 33.9, 36.1, 47.2 /)
    CALL HTdep(temp, Harray, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
    DEALLOCATE(temp, Harray)

  END SUBROUTINE ref3556

  !---------------------------------------------------------------------------

  SUBROUTINE ref3562 ! special definition (DeltaG for Hscp [M/atm])
    IMPLICIT NONE
    ref = "3562"
    type = "T"
    chem = "hydrogen atom" ; casrn = "12385-13-6" ! H
    ! suppl. page 14: Delta_solv G = 20 kJ/mol
    Hominus = EXP(-20E3/(Rgas*T0)) * Hscp_TO_HscpSI
    CALL Output(Hominus)
  END SUBROUTINE ref3562

  !---------------------------------------------------------------------------

  SUBROUTINE ref3563 ! Hvcc [1]
    IMPLICIT NONE
    ref = "3563"
    type = "T"

    chem = "methanediol" ; casrn = "463-57-0" ! HOCH2OH
    ! H2 for formaldehyde from Tab. 4:
    Hominus = Hvcc_TO_HscpSI(1E-7, 300.)
    CALL MakeNote_inconsistent()
    CALL Output(Hominus)
    ! H2 for formaldehyde from Tab. 6 (probably at 300 K, see caption of Tab. 4):
    Hominus = Hvcc_TO_HscpSI(5.625E-7, 300.)
    CALL MakeNote_inconsistent()
    CALL Output(Hominus)

    chem = "1,1-ethanediol" ; casrn = "4433-56-1" ! C3CH2(OH)2
    ! H2 for acetaldehyde from Tab. 5:
    Hominus = Hvcc_TO_HscpSI(1E-5, 300.)
    CALL MakeNote_inconsistent()
    CALL Output(Hominus)
    ! H2 for acetaldehyde from Tab. 6 (probably at 300 K, see caption of Tab. 5):
    Hominus = Hvcc_TO_HscpSI(2.8E-5, 300.)
    CALL MakeNote_inconsistent()
    CALL Output(Hominus)

  CONTAINS

    SUBROUTINE MakeNote_inconsistent()
      IMPLICIT NONE
      CALL MakeNote(TRIM(ref), "Although " // TRIM(citet()) // &
        " writes that his ``Table~6 summarizes numerical calculations for formaldehyde" // &
        " and acetaldehyde assuming the values given in Tables~4 and 5'', different" // &
        " values for the Henry's law constants are shown in these tables.")
    END SUBROUTINE MakeNote_Inconsistent

  END SUBROUTINE ref3563

  !---------------------------------------------------------------------------

  SUBROUTINE ref3564 ! Hscp [M/atm]
    IMPLICIT NONE
    ref = "3564"
    type = "M"

    CALL CalcH("trichlorofluoromethane",     "75-69-4", 0.0061, 4900.) ! CFCl3 R11
    CALL CalcH("2-methyl-1,3-butadiene",     "78-79-5", 0.012,  4400.) ! C5H8 isoprene
    CALL CalcH("1,1,1,2-tetrafluoroethane", "811-97-2", 0.015,  3100.) ! C2H2F4 R134a
    CALL CalcH("chlorodifluoromethane",      "75-45-6", 0.029,  3100.) ! CHF2Cl R22
    CALL CalcH("iodomethane",                "74-88-4", 0.14,   3900.) ! CH3I methyl iodide
    CALL CalcH("iodoethane",                 "75-03-6", 0.14,   4800.) ! C2H5I
    CALL CalcH("dichloromethane",            "75-09-2", 0.36,   3700.) ! CH2Cl2 methylene chloride
    CALL CalcH("chloroiodomethane",         "593-71-5", 0.84,   6200.) ! CH2ClI
    CALL CalcH("dibromomethane",             "74-95-3", 0.90,   4400.) ! CH2Br2
    CALL CalcH("tribromomethane",            "75-25-2", 1.0,    6200.) ! CHBr3 bromoform

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, H298, Tdep)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: H298, Tdep

      chem=chem_ ; casrn=casrn_ ! make global for Output
      Hominus = H298*Hscp_TO_HscpSI
      mindHR = Tdep
      CALL MakeNote("seawater")
      CALL Output(Hominus, mindHR)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3564

  !---------------------------------------------------------------------------

  SUBROUTINE ref3565 ! Bunsen coefficient alpha [1]
    IMPLICIT NONE
    ref = "3565"
    type = "M"
    chem = "nitrogen" ; casrn = "7727-37-9" ! N2
    ! measured data from Tab. 3:
    CALL HTdep_ABC( (/ 0.58, 2.77, 4.95, 6.87, 9.15, 11.08, 14.88, 20.18, 24.37, 30.71 /) &
      + CtoK, &
      (/ 23.32, 22.17, 20.98, 20.09, 19.13, 18.37, 17.09, 15.57, 14.58, 13.32 /) &
      * 1E-3 * alpha_TO_HscpSI, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
    ! compare Bunsen coefficients (Tab. 4) to solubility data (Tab. 5):
    CALL check_conversion( &
      (/ 0., 1., 2., 3., 4., 5., 6., 7., 8., 9., 10., 11., 12., 13., 14., &
      15., 16., 17., 18., 19., 20., 21., 22., 23., 24., 25., 26., 27., 28., 29., 30., &
      31., 32., 33., 34. /) + CtoK, &
      (/ 18.39, 17.92, 17.48, 17.05, 16.64, 16.25, 15.88, 15.51, 15.17, 14.84, 14.52, &
      14.21, 13.91, 13.63, 13.36, 13.10, 12.84, 12.60, 12.37, 12.14, 11.92, 11.71, 11.51, &
      11.31, 11.12, 10.94, 10.76, 10.59, 10.41, 10.25, 10.09, 9.94, 9.79, 9.65, 9.51 /), &
      (/ 23.69, 23.10, 22.54, 22.00, 21.48, 20.99, 20.52, 20.06, 19.63, 19.22, &
      18.82, 18.44, 18.07, 17.72, 17.38, 17.06, 16.75, 16.45, 16.17, 15.89, &
      15.63, 15.37, 15.13, 14.90, 14.67, 14.46, 14.25, 14.05, 13.85, 13.67, &
      13.49, 13.32, 13.16, 13.00, 12.85 /) )

  CONTAINS

    SUBROUTINE check_conversion(temp, solubility, alpha)
      IMPLICIT NONE
      REAL, DIMENSION(:), INTENT(IN) :: temp, solubility, alpha
      INTEGER :: i
      REAL :: alpha2 ! alpha calculated from solubility
      DO i=1,SIZE(temp)
        alpha2 = solubility(i) / ( 0.7808 * (1.-psat(temp(i))/p0) )
        CALL consistency_check(alpha(i), alpha2, "The data listed in Tables~4 and 5")
        ! PRINT *, temp(i), alpha(i), alpha2
      ENDDO
    END SUBROUTINE check_conversion

  END SUBROUTINE ref3565

  !---------------------------------------------------------------------------

  SUBROUTINE ref3566 ! solubility related to Bunsen coefficient alpha [1]
    IMPLICIT NONE
    ref = "3566"
    type = "M"
    CALL CalcH("oxygen", "7782-44-7", &
      (/ 0.74, 1.80, 4.90, 7.45, 9.95, 15.12, 20.00, 24.95, 29.80, 34.80/) + CtoK, &
      (/ 10.021, 9.713, 8.910, 8.382, 7.887, 7.031, 6.360, 5.794, 5.312, 4.862 /) )

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, solubility)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, solubility
      REAL, DIMENSION(SIZE(temp)) :: alpha ! alpha calculated from solubility
      chem=chem_ ; casrn=casrn_ ! make global for Output
      alpha = solubility / ( 0.2094 * (1.-psat(temp)/p0) )
      CALL HTdep_ABC(temp, 1E-3*alpha*alpha_TO_HscpSI, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3566

  !---------------------------------------------------------------------------

  SUBROUTINE ref3567 ! Bunsen coefficient alpha [1]
    IMPLICIT NONE
    ref = "3567"
    type = "M"
    chem = "argon" ; casrn = "7440-37-1" ! Ar
    CALL HTdep_ABC( (/ 1.05, 5.00, 10.00, 15.00, 20.07, 25.00, 30.08, 35.00 /) + CtoK, &
      (/ 52.045, 46.812, 41.655, 37.478, 34.030, 31.258, 27.874, 26.912 /) &
      * 1E-3 * alpha_TO_HscpSI, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3567

  !---------------------------------------------------------------------------

  SUBROUTINE ref3568 ! Bunsen coefficient alpha [1]
    IMPLICIT NONE
    ref = "3568"
    type = "M"
    chem = "carbon dioxide" ; casrn = "124-38-9" ! CO2
    CALL HTdep_ABC( (/ 1.04, 5.00, 10.00, 15.00, 20.00, 25.00, 32.00, 35.00 /) + CtoK, &
      (/ 1647.1, 1419.9, 1192.6, 1010.0, 865.2, 751.9, 633.6, 590.5 /) &
      * 1E-3 * alpha_TO_HscpSI, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3568

  !---------------------------------------------------------------------------

  SUBROUTINE ref3569 ! Hscc [1] = Ostwald coefficient
    IMPLICIT NONE
    ref = "3569"
    type = "M"
    ! Tab. 1:

    CALL CalcH("methane",                 "74-82-8", & ! CH4
      (/ 288.15, 293.15, 298.15, 303.15 /), (/ 4.095, 3.740, 3.456, 3.227 /))
    CALL CalcH("dichlorodifluoromethane", "75-71-8", & ! CF2Cl2 R12
      (/ 288.19, 298.14, 308.12, 318.25 /), (/ 9.828, 7.110, 5.613, 4.397 /))
    CALL CalcH("chlorotrifluoromethane",  "75-72-9", & ! CF3Cl R13
      (/ 288.22, 298.06, 308.02, 318.02 /), (/ 2.812, 2.253, 1.871, 1.683 /))
    CALL CalcH("tetrafluoromethane",      "75-73-0", & ! CF4
      (/ 288.19, 298.17, 308.15, 318.20 /), (/ 0.6399, 0.5083, 0.4592, 0.4391 /))

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, ostwald)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, ostwald
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep_ABC(temp, Hscc_TO_HscpSI(1E-2*ostwald, temp), Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3569

  !---------------------------------------------------------------------------

  SUBROUTINE ref3570 ! Hscc [1] = Ostwald coefficient L
    IMPLICIT NONE
    ref = "3570"
    type = "Q"
    ! Tab. 5:
    CALL CalcH("HCFC-22",     "75-45-6",  -66.9256, 109.8625,  27.3778) ! CHF2Cl R22
    CALL CalcH("HCFC-141b", "1717-00-6",  -85.6439, 138.0940,  35.6875) ! CH3CFCl2 R141b
    CALL CalcH("HCFC-142b",   "75-68-3",  -73.3682, 118.3104,  29.8797) ! CH3CF2Cl R142b
    CALL CalcH("HFC-134a",   "811-97-2",  -67.1680, 109.1227,  27.0984) ! C2H2F4 R134a
    CALL CalcH("HFC-125",    "354-33-6",  -51.8823,  84.5045,  19.3067) ! C2HF5 R125
    CALL CalcH("HFC-23",      "75-46-7",   30.0046, -31.6631, -18.8072) ! CHF3 R23
    CALL CalcH("PFC-14",      "75-73-0", -113.8218, 162.6686,  49.4215) ! CF4
    CALL CalcH("PFC-116",     "76-16-4", -102.0437, 147.9210,  41.9999) ! C2F6
    CALL CalcH("CFC-12",      "75-71-8", -101.3445, 156.4709,  42.2833) ! CF2Cl2 R12

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, a1, a2, a3)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: a1, a2, a3
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! L = exp( a1 + a2*(100./T) + a3*ln(T/100) )
      Hominus    = Hscc_TO_HscpSI( EXP(a1 + a2*(100./T0) + a3*LOG(T0/100.)), T0)
      ! analytical derivative: (see also gnuplot/ref2279.gnu)
      ! mindHR = 100*a2 - a3*T0
      mindHR = 100.*a2 - a3*T0 + T0 ! see ref958, eqn (34) why T0 is added
      CALL MakeNote("moreTdep")
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3570

  !---------------------------------------------------------------------------

  SUBROUTINE ref3571 ! Hsbp [mol/(kg*atm)]
    IMPLICIT NONE
    ref = "3571"
    type = "M"
    chem = "tetrachloromethane" ; casrn = "56-23-5" ! CCl4 carbontetrachloride
    CALL HTdep_ABC( (/ 1.18, 1.28, 2.68, 10.69, 19.95, 30.65, 30.70, 30.70, &
      32.68, 39.97, 39.97, 40.02 /) + CtoK, &
      (/ 0.12727, 0.12647, 0.10921, 0.06233, 0.03847, 0.02471, 0.02493, &
      0.02503, 0.02267, 0.01774, 0.01791, 0.01715 /) * Hsbp_TO_HscpSI, &
      Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3571

  !---------------------------------------------------------------------------

  SUBROUTINE ref3572 ! Hvpc [Pa*m3/mol]
    IMPLICIT NONE

    ref = "3572"
    type = "M"
    ! Tabs. 3 and 4:
    CALL CalcH("trichloromethane",      "67-66-3", 327.  ) ! CHCl3 chloroform
    CALL CalcH("bromodichloromethane",  "75-27-4", 172.3 ) ! CHCl2Br
    CALL CalcH("dibromochloromethane", "124-48-1", 91.3  ) ! CHClBr2
    CALL CalcH("tribromomethane",       "75-25-2", 51.0  ) ! CHBr3 bromoform

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, H)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL, INTENT(IN)             :: H
      chem=chem_ ; casrn=casrn_ ! make global for Output
      Hominus = 1./H
      CALL Output(Hominus)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3572

  !---------------------------------------------------------------------------

  SUBROUTINE ref3573 ! Hscp [M/atm]
    IMPLICIT NONE
    ref = "3573"
    type = "M"
    chem = "bromomethane" ; casrn = "74-83-9" ! CH3Br methyl bromide
    CALL HTdep_ABC( (/ 1.1, 5.0, 10.0, 13.4, 18.0, 19.4, 21.2, 25.0, 30.5 /) + CtoK, &
      (/ 0.493, 0.387, 0.303, 0.266, 0.230, 0.221, 0.205, 0.170, 0.152 /) * &
      Hscp_TO_HscpSI, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3573

  !---------------------------------------------------------------------------

  SUBROUTINE ref3575 ! Hscp [mol m-3 atm-1]
    IMPLICIT NONE

    ref = "3575"

    ! Tabs. 2 and S4:                                                                     LogKH EPI      SPARC
    CALL CalcH("cis-2-methyl-1,3,4-trihydroxy-1-butene",   "FVYHKVMWBMOFGS-RQOWECAXSA-N", 13.9, 1.28E7,  3.76E11)
    CALL CalcH("3-methyl-2,3,4-trihydroxy-1-butene",       "YXPJRFSYSVRDQI-UHFFFAOYSA-N", 13.6, 1.51E7,  3.18E7)
    CALL CalcH("trans-2-methyl-1,3,4-trihydroxy-1-butene", "FVYHKVMWBMOFGS-DUXPYHPUSA-N", 13.8, 1.28E7,  3.76E11)
    CALL CalcH("2-methylthreitol",                         "HGVJFBSSLICXEM-WHFBIAKZSA-N", 13.3, 2.44E9,  3.36E13)
    CALL CalcH("2-methylerythritol",                       "HGVJFBSSLICXEM-CRCLSJGQSA-N", 13.5, 2.44E9,  3.36E13)
    CALL CalcH("levoglucosan",                             "TWNIBLMWSKIRAT-VFUOTHLCSA-N", 15.7, 7.04E12, 4.35E15)
    CALL CalcH("meso-erythritol",                          "UNXHWFMMPAWVPI-ZXZARUISSA-N", 14.4, 3.25E9,  4.12E13)
    CALL CalcH("fructose",                                 "BJHIKXHVCXFQLS-UYFOZJQFSA-N", 14.1, 1.03E14, 1.62E19)
    CALL CalcH("mannose",                                  "GZCGUPFRVQAUEE-KVTDHHQDSA-N", 14.1, 1.70E10, 1.37E18)
    CALL CalcH("glucose",                                  "GZCGUPFRVQAUEE-SLPGGIOYSA-N", 13.9, 1.03E14, 1.81E20)
    CALL CalcH("xylitol",                                  "HEBKCHPVOIAQTA-SCDXWVJYSA-N", 13.7, 6.67E10, 7.35E16)
    CALL CalcH("arabinitol",                               "HEBKCHPVOIAQTA-QWWZWVQMSA-N", 15.0, 6.67E10, 7.35E16)
    CALL CalcH("mannitol",                                 "FBPFZTCFMRRESA-KVTDHHQDSA-N", 14.5, 1.38E12, 1.04E20)

  CONTAINS

    SUBROUTINE CalcH (chem_, inchikey_, LogKH, epi, sparc)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, inchikey_
      REAL,             INTENT(IN) :: LogKH, epi, sparc
      chem=chem_ ; inchikey=inchikey_ ! make global for Output

      type = "M"
      CALL MakeNote(TRIM(ref)//"-LogKH", "Calculated based on atmospheric measurements.")
      CALL Output(10.**(LogKH)/atm)

      type = "Q"
      CALL MakeNote(TRIM(ref)//"-epi", "Calculated using EPI.")
      CALL Output(epi/atm)

      type = "Q"
      CALL MakeNote(TRIM(ref)//"-sparc", "Calculated using SPARC.")
      CALL Output(sparc/atm)

    END SUBROUTINE CalcH

  END SUBROUTINE ref3575

  !---------------------------------------------------------------------------

  SUBROUTINE ref3578 ! Hscp [M/atm]
    IMPLICIT NONE

    ref = "3578"

    ALLOCATE(temp(2))
    temp = (/ 5., 25. /) + CtoK
    ! Tab. 2.2:
    CALL CalcH("ISOPBOOH", "QTGGFXPTAQEODO-UHFFFAOYSA-N", (/ 1.3E6, 1.18E5 /), 1E5,   1.3E6)
    CALL CalcH("ISOPDOOH", "HYHMYONEYJINON-UHFFFAOYSA-N", (/ 7.3E4, 1.20E4 /), 2.6E6, 1.3E6)
    CALL CalcH("H2O2",     "MHAJPDPJQMAIIY-UHFFFAOYSA-N", (/ 4.97E5, 1.19E5 /))
    DEALLOCATE(temp)

  CONTAINS

    SUBROUTINE CalcH (chem_, inchikey_, KH, EA, GROMHE)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, inchikey_
      REAL, DIMENSION(:), INTENT(IN) :: KH
      REAL, OPTIONAL,     INTENT(IN) :: EA, GROMHE
      chem=chem_ ; inchikey=inchikey_ ! make global for Output
      type = "M"
      CALL HTdep(temp, KH*Hscp_TO_HscpSI, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
      IF (PRESENT(EA)) THEN
        type = "Q"
        CALL MakeNote(TRIM(ref)//"-EA", "Calculated using EVAPORATION and AIOMFAC.")
        CALL Output(EA*Hscp_TO_HscpSI)
      ENDIF
      IF (PRESENT(GROMHE)) THEN
        type = "Q"
        CALL MakeNote(TRIM(ref)//"-GROMHE", "Calculated using the GROMHE model.")
        CALL Output(GROMHE*Hscp_TO_HscpSI)
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3578

  !---------------------------------------------------------------------------

  SUBROUTINE ref3579 ! Hvpx [atm]
    IMPLICIT NONE
    REAL          :: yaws, nngc, temp_c
    INTEGER       :: i

    ref = "3579"
    ! data file was generated with util/ref3579/ref3579.py
    OPEN (10, FILE="input/ref3579.dat", STATUS="OLD")
    DO
      READ (10,*,END=999) i, yaws, nngc, temp_c, inchikey, chem
      ! Yaws:
      type = "X"
      CALL SettypeX("yaws2003handbook")
      Hominus = Hvpx_TIMES_HscpSI / (10.**yaws)
      CALL MakeNoteOtherTempC(temp_c)
      CALL Output(Hominus)
      ! NN-GC (Neural Network-Group Contribution):
      type = "Q"
      Hominus = Hvpx_TIMES_HscpSI / (10.**nngc)
      CALL MakeNote(TRIM(ref)//"-NN-GC", &
        "Calculated using a combination of a group contribution method and neural networks.")
      CALL Output(Hominus)
    ENDDO
999 CONTINUE
    CLOSE(10)

  END SUBROUTINE ref3579

  !---------------------------------------------------------------------------

  SUBROUTINE ref3580 ! Hscp [M/atm]
    IMPLICIT NONE

    ref = "3580"
    type = "Q"
    chem = "IEPOX" ; inchikey = "FLVAIUDBQNOKHB-UHFFFAOYSA-N"
    CALL MakeNote(TRIM(ref), "Calculated using the HENRYWIN program and" // &
        " calibration to 1,3-propanediol.")
    CALL Output(Hscp_TO_HscpSI * 1.3E8)

  END SUBROUTINE ref3580

  !---------------------------------------------------------------------------

  SUBROUTINE ref3581 ! Hscp [M/atm]
    IMPLICIT NONE

    ref = "3581"
    type = "?"
    ! Tab. 1:
    CALL CalcH("IEPOX", "FLVAIUDBQNOKHB-UHFFFAOYSA-N", 3E7)
    CALL CalcH("GLYX",  "LEQAOMBKQFMDFZ-UHFFFAOYSA-N", 2.7E7) ! OHCCHO glyoxal

  CONTAINS

    SUBROUTINE CalcH (chem_, inchikey_, H)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, inchikey_
      REAL,             INTENT(IN) :: H
      chem=chem_ ; inchikey=inchikey_ ! make global for Output
      CALL MakeNote(TRIM(ref), TRIM(citet())// &
        " say that the Henry's law constant was updated based on advances in" // &
        " the literature since \citet{3098} but do not provide further details.")
      CALL Output(Hscp_TO_HscpSI * H)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3581

  !---------------------------------------------------------------------------

  SUBROUTINE ref3582 ! Hscp [M/atm]
    IMPLICIT NONE

    ref = "3582"
    type = "E"
    chem = "IEPOX" ; inchikey = "FLVAIUDBQNOKHB-UHFFFAOYSA-N"
    CALL MakeNote(TRIM(ref), "The value was chosen for a model study because" // &
        " it gave the best agreement with measurements.")
    CALL Output(Hscp_TO_HscpSI * 1.9E7)

  END SUBROUTINE ref3582

  !---------------------------------------------------------------------------

  SUBROUTINE ref3583 ! Hscp [M/atm]
    IMPLICIT NONE

    ref = "3583"
    type = "Q"
    CALL CalcH("IEPOX", "FLVAIUDBQNOKHB-UHFFFAOYSA-N", 2.7E6)
    CALL CalcH("MAE",   "CSEUSVYSDPXJAP-UHFFFAOYSA-N", 1.2E5) ! methacrylic acid epoxide

  CONTAINS

    SUBROUTINE CalcH (chem_, inchikey_, H)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, inchikey_
      REAL,             INTENT(IN) :: H
      chem=chem_ ; inchikey=inchikey_ ! make global for Output
      CALL MakeNote(TRIM(ref), "Calculated using HENRYWIN 3.2" // &
        " (bond contribution method).")
      CALL Output(Hscp_TO_HscpSI * H)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3583

  !---------------------------------------------------------------------------

  SUBROUTINE ref3584 ! Hvpx [Pa]
    IMPLICIT NONE

    ref = "3584"
    ! Tab. 3.3-1:
    CALL CalcH("N-methyl-diethanolamine",      "105-59-9", &
      (/ 40., 45., 50., 55., 60., 65., 70. /),  (/ 29.7, 32.3, 35.0, 41.4, 51.0, 66.5, 85.9 /))           ! MDEA
    CALL CalcH("1,4-diazacyclohexane",         "110-85-0", &
      (/ 40., 45., 50., 55., 60., 65., 70. /),  (/ 46.6, 60.2, 73.5, 109., 159., 246., 361. /))           ! PZ
    CALL CalcH("diglycolamine",                "929-06-6", &
      (/ 40., 45., 50., 55., 60., 65. /),       (/ 19.1, 19.8, 21.0, 25.1, 32.5, 47.5 /))                 ! DGA
    CALL CalcH("C4H12N2",                     "6291-84-5", &
      (/ 40., 45.1, 50., 55., 60., 65.2 /),     (/ 147., 178., 291., 382., 567., 855. /))                 ! MAPA
    CALL CalcH("ethylenediamine",              "107-15-3", &
      (/ 40., 45., 50., 55., 60., 65. /),       (/ 100., 116., 179., 234., 342., 459. /))                 ! EDA
    CALL CalcH("ethanolamine",                 "141-43-5", &
      (/ 40., 60., 80., 100. /),                (/ 160.0, 432., 954., 3481. /))                           ! MEA
    CALL CalcH("C3H10N2",                       "78-90-0", &
      (/ 40., 45., 50.2, 55., 60.1, 65. /),     (/ 118., 226., 295., 397., 561., 750. /))                 ! DAP
    CALL CalcH("N-methylpiperazine",           "109-01-3", &
      (/ 40., 45., 50., 55., 60., 65. /),       (/ 143., 266., 405., 628., 1010., 1566. /))               ! 1MPZ
    CALL CalcH("2-amino-2-methyl-1-propanol",  "124-68-5", &
      (/ 40., 45., 50., 55., 60., 65. /),       (/ 316., 458., 727., 1082., 1547., 2315. /))              ! AMP
    CALL CalcH("C6H16N2",                      "124-09-4", &
      (/ 40., 45., 50., 55., 60., 65., 70. /),  (/ 148., 154., 158., 204., 295., 450., 518. /))           ! HMDA
    CALL CalcH("N,N'-dimethylpiperazine",      "106-58-1", &
      (/ 40., 45., 50., 55., 60., 65., 70. /),  (/ 1510., 2470., 4540., 7160., 11700., 18800., 29500. /)) ! 14DMPZ
    CALL CalcH("dimethylaminoethoxyethanol",  "1704-62-7", &
      (/ 40., 45., 50., 55., 60., 65. /),       (/ 60.0, 73.2, 95.8, 174., 243., 440. /))                 ! DMAEE
    CALL CalcH("C4H11NO",                      "108-01-0", &
      (/ 40., 45., 50., 55., 60., 65., 70. /),  (/ 2110., 3100., 4700., 6860., 9540., 13700., 19200. /))  ! DMEA
    CALL CalcH("1-oxa-4-azacyclohexane",       "110-91-8", &
      (/ 40., 45., 50., 55., 60., 65., 70. /),  (/ 1020., 1530., 2220., 3100., 4620., 6380., 8930. /))    ! MORPH
    CALL CalcH("C12H24N2O3",                  "6425-39-4", &
      (/ 40., 45., 50., 55., 60., 65., 70. /),  (/ 50.7, 52.5, 56.5, 74.0, 78.5, 92.7, 97.8 /))           ! DMORPH
    CALL CalcH("C6H14N2O",                     "103-76-4", &
      (/ 40., 45., 50., 55., 60., 65., 70. /),  (/ 38.0, 72.1, 146., 169., 197., 217., 255. /))           ! HEP
    CALL CalcH("C3H9NO",                      "6168-72-5", &
      (/ 40., 45., 50., 55., 60., 65. /),       (/ 172., 213., 285., 424., 645., 1065. /))                ! DL-ALA
    CALL CalcH("C4H11NO2",                     "111-42-2", &
      (/ 40.1, 45., 50., 55., 60., 65. /),      (/ 19.9, 22.3, 24.1, 25.4, 26.9, 27.2 /))                 ! DEA
    CALL CalcH("C5H12N2",                      "109-07-9", &
      (/ 40., 45., 50., 55., 60., 65., 70. /),  (/ 48.2, 75.0, 127., 170., 265., 431., 630. /))           ! 2MPZ

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, H)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, H
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp+CtoK, cH2O/H, Hominus, mindHR)
      IF (chem_=="ethanolamine") THEN
        type = "?"
        CALL MakeNote(TRIM(ref)//"MEA", TRIM(citet()) // &
          " refer to \citet{3654} as the source, but this value cannot be found there.")
      ELSE
        type = "M"
      ENDIF
      CALL MakeNote("highTextrapol")
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3584

  !---------------------------------------------------------------------------

  SUBROUTINE ref3586 ! Hvpc [atm*m3/mol]
    IMPLICIT NONE
    REAL             :: logKH_Exp_Est, logKH_Eq1
    INTEGER          :: i
    CHARACTER(LEN=4) :: info

    ref = "3586"
    ! data file was generated with util/ref3586.py
    OPEN (10, FILE="input/ref3586.dat", STATUS="OLD")
    DO
      ! number name exptl. eqn(2) deviation Duchowicz deviation
      READ (10,*,END=999) i, logKH_Exp_Est, logKH_Eq1, info, inchikey, casrn, chem
      !---
      ! data in 1st column (from HENRYWIN):
      IF (TRIM(info)/="V") THEN
        ! Tab. 1S (entries 1...530):
        type = "?"
        CALL MakeNote(TRIM(ref)//"_exp", "Experimental value, extracted from "// &
          "HENRYWIN.") ! p. 116
        CALL MakeNote("whichref")
      ELSE
        ! Tab. 2S (entries >=531):
        type = "V"
        CALL MakeNote(TRIM(ref)//"_V", "Estimation based on the quotient between "// &
          "vapor pressure and water solubility, extracted from HENRYWIN.") ! p. 119
      ENDIF
      Hominus = 1. / (atm*(10.**logKH_Exp_Est))
      CALL Output(Hominus)
      !---
      ! data in 2nd column (predicted value from Eqn. 1):
      type = "Q"
      IF (TRIM(info)=="val")  CALL MakeNote(TRIM(ref)//"_val", "Value from the "// &
          "validation set for checking whether the model is satisfactory "// &
          "for compounds that are absent from the training set.") ! p. 117
      IF (TRIM(info)=="test") CALL MakeNote(TRIM(ref)//"_test", "Value from the "// &
          "test set for true external validation.") ! p. 117
      Hominus = 1. / (atm*(10.**logKH_Eq1))
      CALL Output(Hominus)
      !---
    ENDDO
999 CONTINUE
    CLOSE(10)

  END SUBROUTINE ref3586

  !---------------------------------------------------------------------------

  SUBROUTINE ref3587 ! special definition
    IMPLICIT NONE

    ref = "3587"
    type = "V"
    ! Tab. 1:
    ! the first four species are mixtures (not used here)
    CALL CalcH("tetrachloroethene", "127-18-4", 19.9,   400., MC*2.+MCl*4.)
    ! 2,4,5,4',5'-pentachlorobiphenyl does not exist
    CALL CalcH("p,p'-DDE",           "72-55-9", 7.4E-6, 0.04, MC*12.+MH*5.+MCl*5.)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, pvap, solub, M)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: pvap, solub, M
      chem=chem_ ; casrn=casrn_ ! make global for Output
      Hominus = (1E-6*solub*rhoH2O/M) / (pvap*mmHg)
      CALL Output(Hominus)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3587

  !---------------------------------------------------------------------------

  ! ref3588 NOTUSED: The data are the same as in ref0628.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3589 ! Hvcc [1]
    IMPLICIT NONE
    REAL :: A, B

    ref = "3589"
    type = "L"
    chem = "ozone" ; casrn = "10028-15-6" ; inchikey = "CBENFWSGALASAD-UHFFFAOYSA-N"
    A = 6.5987
    B = -1752.
    Hominus = Hvcc_TIMES_HscpSI_atT0 / (10.**(A+B/T0))
    mindHR = -(B*LOG(10.) + T0) ! see ref958, eqn (34) why T0 is added
    CALL Output(Hominus, mindHR)

  END SUBROUTINE ref3589

  !---------------------------------------------------------------------------

  SUBROUTINE ref3590 ! Hvpx [atm]
    IMPLICIT NONE
    REAL :: A0, B0

    ref = "3590"
    type = "M"
    chem = "ozone" ; casrn = "10028-15-6" ; inchikey = "CBENFWSGALASAD-UHFFFAOYSA-N"
    A0 =-1150.6
    B0 = 7.6340
    Hominus = Hvpx_TIMES_HscpSI / (10.**(B0+A0/T0))
    mindHR = -A0*LOG(10.)
    CALL Output(Hominus, mindHR)

  END SUBROUTINE ref3590

  !---------------------------------------------------------------------------

  SUBROUTINE ref3591 ! Hscc [1]
    IMPLICIT NONE
    REAL :: temp

    ref = "3591"
    type = "M"
    chem = "ozone" ; casrn = "10028-15-6" ; inchikey = "CBENFWSGALASAD-UHFFFAOYSA-N"
    temp = 20.+CtoK
    CALL MakeNoteOtherTempK(temp)
    ! use value from Tab. III at mu = 0.1
    CALL Output(Hscc_TO_HscpSI(0.41,temp))

  END SUBROUTINE ref3591

  !---------------------------------------------------------------------------

  SUBROUTINE ref3592 ! Hscc [1]
    IMPLICIT NONE

    ref = "3592"
    type = "M"
    chem = "ozone" ; casrn = "10028-15-6" ; inchikey = "CBENFWSGALASAD-UHFFFAOYSA-N"
    ALLOCATE(temp(5), Harray(5))
    temp = (/ 5., 10., 15., 20., 25. /) + CtoK
    Harray = Hscc_TO_HscpSI((/ 0.45, 0.41, 0.37, 0.34, 0.30 /), temp)
    CALL HTdep(temp, Harray, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
    DEALLOCATE(temp, Harray)

  END SUBROUTINE ref3592

  !---------------------------------------------------------------------------

  SUBROUTINE ref3593 ! Hvpc [mmHg/M]
    IMPLICIT NONE

    ref = "3593"
    type = "M"
    chem = "ozone" ; casrn = "10028-15-6" ; inchikey = "CBENFWSGALASAD-UHFFFAOYSA-N"
    ALLOCATE(temp(4), Harray(4))
    temp = (/ 15.2, 20., 25., 29.4 /) + CtoK
    ! values in 0.01 M HClO4 from Tab. I:
    Harray = 1./(1E4 * dm3 * mmHg * (/ 5.4, 6.8, 7.6, 8.4 /))
    CALL HTdep(temp, Harray, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
    DEALLOCATE(temp, Harray)

  END SUBROUTINE ref3593

  !---------------------------------------------------------------------------

  SUBROUTINE ref3594 ! special definition
    IMPLICIT NONE

    ref = "3594"
    type = "M"
    chem = "ozone" ; casrn = "10028-15-6" ; inchikey = "CBENFWSGALASAD-UHFFFAOYSA-N"
    Hominus = ((57.6E-6/(MO*3.)) / dm3) / (0.06*765*mmHg) ! (n/V) / p
    CALL Output(Hominus)

  END SUBROUTINE ref3594

  !---------------------------------------------------------------------------

  SUBROUTINE ref3595 ! Hscc [1]
    IMPLICIT NONE

    ref = "3595"
    type = "M"
    chem = "ozone" ; casrn = "10028-15-6" ; inchikey = "CBENFWSGALASAD-UHFFFAOYSA-N"
    ALLOCATE(temp(2), Harray(2))
    temp = (/ 0., 20. /) + CtoK
    Harray = Hscc_TO_HscpSI((/ 0.44, 0.23 /), temp)
    CALL HTdep(temp, Harray, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
    DEALLOCATE(temp, Harray)

  END SUBROUTINE ref3595

  !---------------------------------------------------------------------------

  SUBROUTINE ref3596 ! Hscc [1]
    IMPLICIT NONE

    ref = "3596"
    type = "M"
    chem = "ozone" ; casrn = "10028-15-6" ; inchikey = "CBENFWSGALASAD-UHFFFAOYSA-N"
    ! T    | c_aq | c_g  | Hscc
    !---------------------------
    ! 0.   | 39.4 | 61.5 | 0.641
    ! 6.   | 34.3 | 61.  | 0.562
    ! 11.8 | 29.9 | 59.6 | 0.500
    ! 13.  | 28.  | 58.1 | 0.482
    ! 15.  | 25.9 | 56.8 | 0.456
    ! 19.  | 21.  | 55.2 | 0.381
    ! 27.  | 13.9 | 51.4 | 0.270
    ! 32.  | 7.7  | 39.5 | 0.195
    ! 40.  | 4.2  | 37.6 | 0.112
    ! 47.  | 2.4  | 31.2 | 0.077
    ! 55.  | 0.6  | 19.2 | 0.031
    ! 60.  | 0.   | 12.3 | 0.     don't use zero value at 60 C
    ALLOCATE(temp(2), Harray(2))
    ! temp = (/ 0., 6., 11.8, 13., 15., 19., 27., 32., 40., 47., 55. /) + CtoK
    ! Harray = Hscc_TO_HscpSI( &
    !   (/ 0.641, 0.562, 0.500, 0.482, 0.456, 0.381, 0.270, &
    !   0.195, 0.112, 0.077, 0.031 /), temp)
    ! use only data next to 25 C:
    temp = (/ 19., 27. /) + CtoK
    Harray = Hscc_TO_HscpSI( (/ 0.381, 0.270 /), temp)
    CALL HTdep(temp, Harray, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
    DEALLOCATE(temp, Harray)

  END SUBROUTINE ref3596

  !---------------------------------------------------------------------------

  SUBROUTINE ref3597 ! special definition
    IMPLICIT NONE

    ref = "3597"
    type = "M"
    chem = "ozone" ; casrn = "10028-15-6" ; inchikey = "CBENFWSGALASAD-UHFFFAOYSA-N"
    Hominus = ((0.0189E-3/(MO*3.)) / dm3) / (0.0329*741.5*mmHg) ! (n/V) / p
    CALL MakeNoteOtherTempC(18.2)
    CALL Output(Hominus)

  END SUBROUTINE ref3597

  !---------------------------------------------------------------------------

  ! ref3598 NOTUSED: In the recommended fit, log(H) is proportional to
  !                  T, not 1/T.

  !---------------------------------------------------------------------------

  ! ref3599 NOTUSED: Data from Fig. 7 are not available, and in the
  !                  recommended fit, log(H) is proportional to T (in
  !                  °C), not 1/T

  !---------------------------------------------------------------------------

  SUBROUTINE ref3600 ! Hsxp [1/atm]
    IMPLICIT NONE

    ref = "3600"
    type = "M"
    chem = "oxygen" ; casrn = "7782-44-7" ! O2
    Hominus = 2.26E-5 * Hsxp_TO_HscpSI
    CALL MakeNoteOtherTempC(20)
    CALL Output(Hominus)

  END SUBROUTINE ref3600

  !---------------------------------------------------------------------------

  SUBROUTINE ref3601 ! Hsxp [1/atm]
    IMPLICIT NONE

    ref = "3601"
    type = "M"
    CALL CalcH("argon",               "7440-37-1", "XKRFYHLGVUSROY-UHFFFAOYSA-N", &
      (/ 278.15, 283.15, 288.15, 293.15, 298.15, 303.15 /), &
      (/ 0.3185, 0.3333, 0.2988, 0.2722, 0.2520, 0.2316 /) ) ! Ar
    CALL CalcH("krypton",             "7439-90-9", "DNNSSWSSYDEUBZ-UHFFFAOYSA-N", &
      (/ 278.15, 283.15, 288.15, 293.15, 298.15, 303.15, 313.15 /), &
      (/ 0.7533, 0.6577, 0.5740, 0.5113, 0.4526, 0.4180, 0.3649 /) ) ! Kr
    CALL CalcH("nitrogen",            "7727-37-9", "IJGRMHOSHXDMSA-UHFFFAOYSA-N", &
      (/ 278.15, 283.15, 288.15, 293.15, 298.15, 303.15, 308.15,  313.15 /), &
      (/ 0.1692, 0.1513, 0.1372, 0.1275, 0.1175, 0.1116, 0.1062, 0.09986 /) ) ! N2
    CALL CalcH("oxygen",              "7782-44-7", "MYMOFIZGZYHOMD-UHFFFAOYSA-N", &
      (/ 278.15, 283.15, 288.15, 293.15, 298.15, 303.15, 308.15, 313.15 /), &
      (/ 0.3446, 0.2963, 0.2678, 0.2496, 0.2264, 0.2081, 0.1985, 0.1853 /) ) ! O2
    CALL CalcH("methane",               "74-82-8", "VNWKTOKETHGBQD-UHFFFAOYSA-N", &
      (/ 278.15, 283.15, 288.15, 293.15, 298.15, 303.15, 308.15, 313.15, 318.15 /), &
      (/ 0.3978, 0.3467, 0.2940, 0.2720, 0.2485, 0.2278, 0.2140, 0.1943, 0.1899 /) ) ! CH4
    CALL CalcH("ethane",                "74-84-0", "OTMSDBZUPAUEDD-UHFFFAOYSA-N", &
      (/ 278.15, 283.15, 288.15, 293.15, 298.15, 303.15 /), &
      (/ 0.6483, 0.5226, 0.4438, 0.3832, 0.3330, 0.2916 /) ) ! C2H6
    CALL CalcH("sulfur hexafluoride", "2551-62-4", "SFZCNBIFKDRMGX-UHFFFAOYSA-N", &
      (/  278.15,  233.15,  188.15,  293.15,  298.15,  303.15,  308.15,  313.15 /), &
      (/ 0.09164, 0.07340, 0.06035, 0.05143, 0.04426, 0.03872, 0.03486, 0.03226 /) ) ! SF6
    CALL CalcH("tetrafluoromethane",    "75-73-0", "TXEYQDLBPFQVAA-UHFFFAOYSA-N", &
      (/  293.15,  298.15,  303.15 /), &
      (/ 0.04174, 0.03750, 0.03492 /) ) ! CF4 carbontetrafluoride

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, temp, chi)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_, inchikey_
      REAL, DIMENSION(:), INTENT(IN) :: temp, chi
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      CALL HTdep(temp, 1E-4 * chi * Hsxp_TO_HscpSI, Hominus, mindHR)
      CALL MakeNote("highTextrapol")
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3601

  !---------------------------------------------------------------------------

  ! ref3602 NOTUSED: The paper only contains data from ref3628 in
  !                  smoothed form.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3603 ! Hvpx [atm]
    IMPLICIT NONE
    ref = "3603"
    type = "M"
    chem = "oxygen" ; casrn = "7782-44-7" ! O2
    ! Tab. 3, only using data at zero salt concentration:
    CALL HTdep_ABC( &
      (/ 0., 0., 0., 0., 0., 0., 0., 5., 5., 10.4, 10.4, 19.4, 19.4, 19.4, &
      19.4, 27.25, 27.25, 34.1, 34.1, 35.2, 35.4, 35.4, 35.4, 35.4 /) + CtoK, &
      Hvpx_TIMES_HscpSI * 1E-4 / &
      (/ 2.508, 2.537, 2.500, 2.516, 2.509, 2.525, 2.545, 2.913, 2.920, 3.30, &
      3.283, 3.95, 3.92, 3.95, 3.93, 4.49, 4.51, 4.98, 4.96, 5.06, 5.09, 5.00, &
      5.07, 5.05 /), Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3603

  !---------------------------------------------------------------------------

  ! ref3604 NOTUSED: The paper only contains a fit of the data from
  !                  ref3603.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3605 ! Hsbp [mol/(kg*atm)]
    IMPLICIT NONE
    ref = "3605"
    type = "M"
    chem = "oxygen" ; casrn = "7782-44-7" ! O2
    ! Tab. 1:
    CALL HTdep_ABC( &
      (/ 0.57, 5.13, 10.19, 14.83, 19.95, 25.00, 25.28, 30.00, 35.17, 39.61, 44.91 /) + CtoK, &
      (/ 449.4, 396.7, 351.4, 315.8, 285.9, 258.9, 258.1, 239.3, 218.2, 206.0, 189.4 /) &
      * 1E-6 * Hsbp_TO_HscpSI / O2_fraction, Hominus, mindHR)
    CALL MakeNote(TRIM(ref)//"-sameas3606", &
      "Almost the same data were also published in \citet{3606}.")
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3605

  !---------------------------------------------------------------------------

  SUBROUTINE ref3606 ! Hsbp [mol/(kg*atm)]
    IMPLICIT NONE
    ref = "3606"
    type = "M"
    chem = "oxygen" ; casrn = "7782-44-7" ! O2
    ! Tab. 1:
    CALL HTdep_ABC( &
      (/ 5.12, 15.14, 25.12, 35.17, 45.11 /) + CtoK, &
      (/ 396.9, 314.4, 258.9, 218.2, 189.4 /) &
      * 1E-6 * Hsbp_TO_HscpSI / O2_fraction, Hominus, mindHR)
    CALL MakeNote(TRIM(ref)//"-sameas3605", &
      "Almost the same data were also published in \citet{3605}.")
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3606

  !---------------------------------------------------------------------------

  ! ref3607 NOTUSED: The paper does not contain any new data.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3608 ! Hscc [1]
    IMPLICIT NONE
    REAL :: temp

    ref = "3608"
    type = "M"
    chem = "oxygen" ; casrn = "7782-44-7" ! O2
    temp = 0.+CtoK
    CALL MakeNoteOtherTempK(temp)
    ! Tab. 2:
    CALL Output(Hscc_TO_HscpSI(0.32,temp))

  END SUBROUTINE ref3608

  !---------------------------------------------------------------------------

  SUBROUTINE ref3609 ! Hvpx [kPa]
    IMPLICIT NONE

    ref = "3609"
    type = "M"
    ! Tab. 2:
    CALL CalcH("1-butanol",                "71-36-3", "LRHPLDYGYMQRHN-UHFFFAOYSA-N", &
      (/ 323.15, 323.15, 333.15, 333.15, 333.15, 343.15, 353.15 /), &
      (/ 274.,   278.,   526.,   509.,   514.,   850.,   1360.  /))
    CALL CalcH("2-methyl-1-propanol",      "78-83-1", "ZXEKIIBDNHEJCQ-UHFFFAOYSA-N", &
      (/ 303.15, 323.15, 323.15, 333.15, 343.15, 353.15 /), &
      (/ 112.,   401.,   408.,   748.,   1210.,  1850.  /))
    CALL CalcH("cyclopentanol",            "96-41-3", "XCIXKGXIYUWCLL-UHFFFAOYSA-N", &
      (/ 333.15, 353.15 /), &
      (/ 207.,   569.   /))
    CALL CalcH("cyclohexanol",            "108-93-0", "HPXRVTGHNJAIIH-UHFFFAOYSA-N", &
      (/ 323.15, 333.15 /), &
      (/ 94.7,   193.   /))
    CALL CalcH("butanone",                 "78-93-3", "ZWEHNKRNPOVVGH-UHFFFAOYSA-N", &
      (/ 323.15, 333.15, 343.15, 343.15, 353.15, 353.15 /), &
      (/ 1290.,  1910.,  2850.,  2830.,  3980.,  3920.  /))
    CALL CalcH("cyclopentanone",          "120-92-3", "BGTOWKSIORTVQH-UHFFFAOYSA-N", &
      (/ 333.15, 353.15 /), &
      (/ 376.,   861.   /))
    CALL CalcH("cyclohexanone",           "108-94-1", "JHIVVAPYMSGYDF-UHFFFAOYSA-N", &
      (/ 333.15, 343.15, 353.15 /), &
      (/ 388.,   601.,   921.   /))
    CALL CalcH("2,4-pentanedione",        "123-54-6", "YRKCREAYFQTBPV-UHFFFAOYSA-N", &
      (/ 303.15, 323.15, 333.15, 343.15, 353.15 /), &
      (/ 69.5,   185.,   269.,   404.,   549.   /))
    CALL CalcH("methyl ethanoate",         "79-20-9", "KXKVLQRXCPHEJC-UHFFFAOYSA-N", &
      (/ 293.15, 303.15 /), &
      (/ 505.,   873.   /))
    CALL CalcH("methylacetoacetate",      "105-45-3", "WRQNANDWMGAFTP-UHFFFAOYSA-N", &
      (/ 303.15 /), &
      (/ 8.64   /))
    CALL CalcH("ethylacetoacetate",       "141-97-9", "XYIBRDXRRQCHLP-UHFFFAOYSA-N", &
      (/ 303.15 /), &
      (/ 9.62   /))
    CALL CalcH("2-methoxyethyl acetate",  "110-49-6", "XLLIQLLCWZCATF-UHFFFAOYSA-N", &
      (/ 303.15 /), &
      (/ 18.4   /))
    CALL CalcH("2-ethoxyethyl acetate",   "111-15-9", "SVONRAPFKPVNKG-UHFFFAOYSA-N", &
      (/ 303.15 /), &
      (/ 18.5   /))
    CALL CalcH("methyl methoxyacetate",  "6290-49-9", "QRMHDGWGLNLHMN-UHFFFAOYSA-N", &
      (/ 303.15 /), &
      (/ 22.3   /))
    CALL CalcH("ethane nitrile",           "75-05-8", "WEVYAHXRMPXWCK-UHFFFAOYSA-N", &
      (/ 323.15, 333.15, 333.15, 343.15, 343.15, 353.15 /), &
      (/ 411.,   587.,   563.,   761.,   746.,   999.   /))
    CALL CalcH("propane nitrile",         "107-12-0", "FVSKHRXBFJPNKK-UHFFFAOYSA-N", &
      (/ 303.15 /), &
      (/ 301.   /))
    CALL CalcH("2-propenenitrile",        "107-13-1", "NLHHRLWOUZZQLW-UHFFFAOYSA-N", &
      (/ 303.15 /), &
      (/ 729.   /))

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, temp, H12)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_, inchikey_
      REAL, DIMENSION(:), INTENT(IN) :: temp, H12
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      IF (SIZE(H12)==1) THEN
        CALL MakeNoteOtherTempK(temp(1))
        CALL Output(1E-3*cH2O/H12(1))
      ELSE
        CALL HTdep(temp, 1E-3*cH2O/H12, Hominus, mindHR)
        IF (temp(1)>T0) THEN
          CALL MakeNote("highTextrapol")
        ENDIF
        CALL Output(Hominus, mindHR, r2)
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3609

  !---------------------------------------------------------------------------

  SUBROUTINE ref3610 ! Hvpc [Pa*m3/mol]
    IMPLICIT NONE

    ref = "3610"
    type = "M"
    chem = "dimethyl disulfide" ; casrn = "624-92-0" ; inchikey = "WQOXQRCZOLPYPM-UHFFFAOYSA-N" ! CH3SSCH3
    ! Tab. 1:
    ndata = 9
    ALLOCATE(temp(ndata), Harray(ndata))
    temp = (/ 298.15, 303.15, 308.15, 313.15, 318.15, 323.15, 328.15, 333.15, 338.15 /)
    Harray = 1. / (/ 111.9, 135.4, 172.9, 210.2, 257.4, 316.4, 382.3, 463.7, 556.4 /)
    ! q&d check that "K" in the same table is also okay: Hvpc*Hscc/(RT)=1
    ! print *, (/ 111.9, 135.4, 172.9, 210.2, 257.4, 316.4, 382.3, 463.7, 556.4 /) * &
    !   (/ 22.14, 18.60, 14.80, 12.38, 10.27, 8.49, 7.13, 5.97, 5.05 /) / &
    !   (/ 298.15, 303.15, 308.15, 313.15, 318.15, 323.15, 328.15, 333.15, 338.15 /) / Rgas
    CALL HTdep(temp, Harray, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
    DEALLOCATE(temp, Harray)

  END SUBROUTINE ref3610

  !---------------------------------------------------------------------------

  SUBROUTINE ref3611 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3611"
    type = "M"

    ! Table 1:
    CALL CalcH("dimethyl sulfide",     "75-18-3", "QMMFVYPAHWMCMS-UHFFFAOYSA-N", 79.3  )
    CALL CalcH("1-propanol",           "71-23-8", "BDERNNFJNOPAEC-UHFFFAOYSA-N", 0.6   )
    CALL CalcH("2,3-butanedione",     "431-03-8", "QSJXEFYPDANLFS-UHFFFAOYSA-N", 1.9   )
    CALL CalcH("butanone",             "78-93-3", "ZWEHNKRNPOVVGH-UHFFFAOYSA-N", 4.3   )
    CALL CalcH("ethyl ethanoate",     "141-78-6", "XEKOWRVHYACXOJ-UHFFFAOYSA-N", 13.0  )
    CALL CalcH("1-butanol",            "71-36-3", "LRHPLDYGYMQRHN-UHFFFAOYSA-N", 0.8   )
    CALL CalcH("2-pentanol",         "6032-29-7", "JYVLIDXNZAXMDK-UHFFFAOYSA-N", 1.7   )
    CALL CalcH("propyl ethanoate",    "109-60-4", "YKYONYBAUNKHLG-UHFFFAOYSA-N", 21.8  )
    CALL CalcH("3-methyl-1-butanol",  "123-51-3", "PHTQWCKDNZKARW-UHFFFAOYSA-N", 1.2   )
    CALL CalcH("ethyl butanoate",     "105-54-4", "OBNCKNCVKJNDBV-UHFFFAOYSA-N", 25.0  )
    CALL CalcH("hexanal",              "66-25-1", "JARKCYVAAOWBJS-UHFFFAOYSA-N", 23.6  )
    CALL CalcH("butyl ethanoate",     "123-86-4", "DKPFZGUDAPQIHT-UHFFFAOYSA-N", 22.8  )
    CALL CalcH("1-hexanol",           "111-27-3", "ZSIAUFGUXNUGDI-UHFFFAOYSA-N", 2.4   )
    CALL CalcH("2-heptanone",         "110-43-0", "CATSNJVOTSVZJV-UHFFFAOYSA-N", 15.6  )
    CALL CalcH("heptanal",            "111-71-7", "FXHGMKSSBGDXIY-UHFFFAOYSA-N", 35.8  )
    CALL CalcH("alpha-pinene",         "80-56-8", "GRWFGVWFFZKLTI-UHFFFAOYSA-N", 188.4 )
    CALL CalcH("2-octanone",          "111-13-7", "ZPVFWPFBNIEHGJ-UHFFFAOYSA-N", 21.8  )
    CALL CalcH("octanal",             "124-13-0", "NUJGJRNETVAIRJ-UHFFFAOYSA-N", 44.1  )
    CALL CalcH("2-nonanol",           "628-99-9", "NGDNVOAEIVQRFH-UHFFFAOYSA-N", 5.6   )
    CALL CalcH("2-decanone",          "693-54-9", "ZAJNGDIORYACQU-UHFFFAOYSA-N", 25.5  )

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, Hvcc)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_, inchikey_
      REAL,             INTENT(IN) :: Hvcc
      REAL :: temp
      temp = 37.+CtoK
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      CALL MakeNoteOtherTempK(temp)
      CALL Output(Hvcc_TO_HscpSI(1E-3*Hvcc, temp))
    END SUBROUTINE CalcH

  END SUBROUTINE ref3611

  !---------------------------------------------------------------------------

  SUBROUTINE ref3612 ! Hvpx [Pa]
    IMPLICIT NONE

    ref = "3612"
    type = "M"
    chem = "1-butene" ; casrn = "106-98-9" ; inchikey = "VXNZUUAINFGPBY-UHFFFAOYSA-N" ! C4H8
    ! Tab. II:
    ndata = 13
    ALLOCATE(temp(ndata), Harray(ndata))
    temp = (/ 293.15, 295.15, 298.15, 300.15, 303.15, 305.15, 308.15, &
      310.15, 313.15, 315.15, 318.15, 320.15, 323.15 /)
    Harray = 1E-9 * cH2O / (/ 1.033, 1.121, 1.236, 1.325, 1.457, 1.537, &
      1.655, 1.774, 1.885, 2.017, 2.136, 2.237, 2.398 /)
    ! q&d check that "L21" in the same table is also okay:
    ! Hvpx*Hscc/T=const (apart from water density change)
    ! print *, (/ 1.033, 1.121, 1.236, 1.325, 1.457, 1.537, 1.655, 1.774, 1.885, 2.017, 2.136, 2.237, 2.398 /) * &
    !   (/ 0.1307, 0.1212, 0.1110, 0.1042, 0.09553, 0.09110, 0.08536, 0.08008, 0.07601, 0.07141, 0.06799, 0.06524, 0.06130 /) / &
    !   (/ 293.15, 295.15, 298.15, 300.15, 303.15, 305.15, 308.15, 310.15, 313.15, 315.15, 318.15, 320.15, 323.15 /)
    CALL HTdep_ABC(temp, Harray, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
    DEALLOCATE(temp, Harray)

  END SUBROUTINE ref3612

  !---------------------------------------------------------------------------

  SUBROUTINE ref3613 ! Hscc [1]
    IMPLICIT NONE
    ref = "3613"
    type = "Q"
    ! Tab. 9:
    CALL CalcH("methanoic acid",       "64-18-6", "BDAGIHXWWSANSR-UHFFFAOYSA-N", 5.330)
    CALL CalcH("ethanoic acid",        "64-19-7", "QTBSBXVTEAMEQO-UHFFFAOYSA-N", 4.880)
    CALL CalcH("propanoic acid",       "79-09-4", "XBDQKXXYIPTUBI-UHFFFAOYSA-N", 4.727)
    CALL CalcH("butanoic acid",       "107-92-6", "FERIUCNNQQJTOY-UHFFFAOYSA-N", 4.614)
    CALL CalcH("pentanoic acid",      "109-52-4", "NQPDZGIKBAWPEJ-UHFFFAOYSA-N", 4.450)
    CALL CalcH("hexanoic acid",       "142-62-1", "FUZZWVXGSFPDMH-UHFFFAOYSA-N", 4.313)
    CALL CalcH("heptanoic acid",      "111-14-8", "MNWFXJYAOYHMED-UHFFFAOYSA-N", 4.201)
    CALL CalcH("octanoic acid",       "124-07-2", "WWZKQHOCKIZLMA-UHFFFAOYSA-N", 4.153)
    CALL CalcH("nonanoic acid",       "112-05-0", "FBUKVWPVBMHYJY-UHFFFAOYSA-N", 4.021)
    CALL CalcH("decanoic acid",       "334-48-5", "GHVNFZFCNZKVNT-UHFFFAOYSA-N", 3.868)
    CALL CalcH("undecanoic acid",     "112-37-8", "ZDPHROOEEOARMN-UHFFFAOYSA-N", 3.732)
    CALL CalcH("dodecanoic acid",     "143-07-7", "POULHZVOKOAJMA-UHFFFAOYSA-N", 3.625)
    CALL CalcH("tridecanoic acid",    "638-53-9", "SZHOJFHSIKHZHA-UHFFFAOYSA-N", 3.469)
    CALL CalcH("tetradecanoic acid",  "544-63-8", "TUNFSRHWOTWDNC-UHFFFAOYSA-N", 3.367)
    CALL CalcH("pentadecanoic acid", "1002-84-2", "WQEPLUUGTLDZJY-UHFFFAOYSA-N", 3.244)
    CALL CalcH("hexadecanoic acid",    "57-10-3", "IPCSVZSSVZVIGE-UHFFFAOYSA-N", 3.107)
    CALL CalcH("heptadecanoic acid",  "506-12-7", "KEMQGTRYUADPNZ-UHFFFAOYSA-N", 2.979)
    CALL CalcH("octadecanoic acid",    "57-11-4", "QIQXTHQIDYTFRH-UHFFFAOYSA-N", 2.877)
    CALL CalcH("nonadecanoic acid",   "646-30-0", "ISYWECDDZWTKFF-UHFFFAOYSA-N", 2.751)
    CALL CalcH("eicosanoic acid",     "506-30-9", "VKOBVWXKNCXXDE-UHFFFAOYSA-N", 2.626)
    CALL CalcH("heneicosanoic acid", "2363-71-5", "CKDDRHZIAZRDBW-UHFFFAOYSA-N", 2.50 )
    CALL CalcH("docosanoic acid",     "112-85-6", "UKMSUNONTOPOIO-UHFFFAOYSA-N", 2.37 )
    CALL CalcH("tricosanoic acid",   "2433-96-7", "XEZVDURJDFGERA-UHFFFAOYSA-N", 2.25 )
    CALL CalcH("tetracosanoic acid",  "557-59-5", "QZZGJDVWLFXDLK-UHFFFAOYSA-N", 2.13 )

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, logKw)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_, inchikey_
      REAL,             INTENT(IN) :: logKw
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      Hominus  = Hscc_TO_HscpSI(10.**logKw, T0)
      CALL Output(Hominus)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3613

  !---------------------------------------------------------------------------

  SUBROUTINE ref3614 ! Hscc [1]
    IMPLICIT NONE
    ref = "3614"
    type = "?"
    ! Tab. 1:
    CALL CalcH("helium",                             "7440-59-7", "SWQJXJOGLNCZEY-UHFFFAOYSA-N", -2.01)
    CALL CalcH("neon",                               "7440-01-9", "GKAOGPIIYCISHV-UHFFFAOYSA-N", -1.96)
    CALL CalcH("argon",                              "7440-37-1", "XKRFYHLGVUSROY-UHFFFAOYSA-N", -1.53)
    CALL CalcH("krypton",                            "7439-90-9", "DNNSSWSSYDEUBZ-UHFFFAOYSA-N", -1.30)
    CALL CalcH("xenon",                              "7440-63-3", "FHNFHKCVQCLJFQ-UHFFFAOYSA-N", -1.08)
    CALL CalcH("hydrogen",                           "1333-74-0", "UFHFLCQGNIYNRP-UHFFFAOYSA-N", -1.72)
    CALL CalcH("oxygen",                             "7782-44-7", "MYMOFIZGZYHOMD-UHFFFAOYSA-N", -1.57)
    CALL CalcH("nitrogen",                           "7727-37-9", "IJGRMHOSHXDMSA-UHFFFAOYSA-N", -1.85)
    CALL CalcH("dinitrogen monoxide",               "10024-97-2", "GQPLMRYTRLFLPF-UHFFFAOYSA-N", -0.35)
    CALL CalcH("carbon monoxide",                     "630-08-0", "UGFAIRIUMAVXCW-UHFFFAOYSA-N", -1.68)
    CALL CalcH("carbon dioxide",                      "124-38-9", "CURLTUGMZLYLDI-UHFFFAOYSA-N", -0.19)
    CALL CalcH("methane",                              "74-82-8", "VNWKTOKETHGBQD-UHFFFAOYSA-N", -1.53)
    CALL CalcH("ethane",                               "74-84-0", "OTMSDBZUPAUEDD-UHFFFAOYSA-N", -1.44)
    CALL CalcH("pentane",                             "109-66-0", "OFBQJSOFQDEBGM-UHFFFAOYSA-N", -1.90)
    CALL CalcH("hexane",                              "110-54-3", "VLKZOEOYAKHREP-UHFFFAOYSA-N", -2.08)
    CALL CalcH("2-methylpentane",                     "107-83-5", "AFABGHUZZDYHJO-UHFFFAOYSA-N", -2.05)
    CALL CalcH("3-methylpentane",                      "96-14-0", "PFEOZHBOMNWTJB-UHFFFAOYSA-N", -2.04)
    CALL CalcH("2,2-dimethylbutane",                   "75-83-2", "HNRMPXKDFBEGFZ-UHFFFAOYSA-N", -2.07)
    CALL CalcH("heptane",                             "142-82-5", "IMNFDUFMRHMDMM-UHFFFAOYSA-N", -2.16)
    CALL CalcH("3-methylhexane",                      "589-34-4", "VLJXXKKOSFGPHI-UHFFFAOYSA-N", -2.20)
    CALL CalcH("octane",                              "111-65-9", "TVMXDCGIABBOFY-UHFFFAOYSA-N", -2.31)
    CALL CalcH("methylcyclopentane",                   "96-37-7", "GDOPTJXRTPNYNR-UHFFFAOYSA-N", -1.38)
    CALL CalcH("cyclohexane",                         "110-82-7", "XDTMQSROBMDMFD-UHFFFAOYSA-N", -1.12)
    CALL CalcH("ethene",                               "74-85-1", "VGGSQFUCUMXWEO-UHFFFAOYSA-N", -1.02)
    CALL CalcH("fluoromethane",                       "593-53-3", "NBVXSUQYWXRMNV-UHFFFAOYSA-N",  0.07)
    CALL CalcH("fluoroethane",                        "353-36-6", "UHCBBWUQDAVSMS-UHFFFAOYSA-N",  0.09)
    CALL CalcH("1-fluoropropane",                     "460-13-9", "JRHNUZCXXOTJCA-UHFFFAOYSA-N", -0.05)
    CALL CalcH("2-fluoropropane",                     "420-26-8", "PRNZBCYBKGCOFI-UHFFFAOYSA-N", -0.03)
    CALL CalcH("chloromethane",                        "74-87-3", "NEHMKBQYUWJMIP-UHFFFAOYSA-N",  0.23)
    CALL CalcH("dichloromethane",                      "75-09-2", "YMWUJEATGCHHMB-UHFFFAOYSA-N",  0.86)
    CALL CalcH("trichloromethane",                     "67-66-3", "HEDRZPFGACZZDS-UHFFFAOYSA-N",  0.59)
    CALL CalcH("tetrachloromethane",                   "56-23-5", "VZGDMQKNWNREIO-UHFFFAOYSA-N", -0.60)
    CALL CalcH("chloroethane",                         "75-00-3", "HRYZWHHZPQKTII-UHFFFAOYSA-N",  0.08)
    CALL CalcH("1,1-dichloroethane",                   "75-34-3", "SCYULBFZEHDVBN-UHFFFAOYSA-N",  0.43)
    CALL CalcH("1,2-dichloroethane",                  "107-06-2", "WSLDOOZREJYCGB-UHFFFAOYSA-N",  1.05)
    CALL CalcH("1,1,1-trichloroethane",                "71-55-6", "UOCLXMDMGBRAIB-UHFFFAOYSA-N", -0.03)
    CALL CalcH("1,1,2-trichloroethane",                "79-00-5", "UBOXGVDOUJQMTN-UHFFFAOYSA-N",  1.23)
    CALL CalcH("1,1,2,2-tetrachloroethane",            "79-34-5", "QPFMBZIOSGYJDE-UHFFFAOYSA-N",  1.55)
    CALL CalcH("1,1,1,2-tetrachloroethane",           "630-20-6", "QVLAWKAXOMEXPM-UHFFFAOYSA-N",  0.74)
    CALL CalcH("1-chloropropane",                     "540-54-5", "SNMVRZFUUCLYTO-UHFFFAOYSA-N",  0.04)
    CALL CalcH("1,2-dichloropropane",                  "78-87-5", "KNKRKFALVUDBJE-UHFFFAOYSA-N",  0.73)
    CALL CalcH("1-chlorobutane",                      "109-69-3", "VFWCMGCRMGJXDK-UHFFFAOYSA-N", -0.07)
    CALL CalcH("1-chloropentane",                     "543-59-9", "SQCZQTSHSZLZIQ-UHFFFAOYSA-N", -0.16)
    CALL CalcH("($Z$)-1,2-dichloroethene",            "156-59-2", "KFUSEUYYWQURPO-UPHRSURJSA-N",  0.46)
    CALL CalcH("($E$)-1,2-dichloroethene",            "156-60-5", "KFUSEUYYWQURPO-OWOJBTEDSA-N",  0.32)
    CALL CalcH("trichloroethene",                      "79-01-6", "XSTXAVWGXDQKEL-UHFFFAOYSA-N",  0.11)
    CALL CalcH("tetrachloroethene",                   "127-18-4", "CYTYCFOTNPOANT-UHFFFAOYSA-N", -0.37)
    CALL CalcH("iodoethane",                           "75-03-6", "HVTICUPFWKNHNG-UHFFFAOYSA-N",  0.43)
    CALL CalcH("chlorodifluoromethane",                "75-45-6", "VOPWNXZWBYDODV-UHFFFAOYSA-N", -0.22)
    CALL CalcH("trichlorofluoromethane",               "75-69-4", "CYRMSUTZVYGINF-UHFFFAOYSA-N", -0.69)
    CALL CalcH("dichlorodifluoromethane",              "75-71-8", "PXBRQCKWGAHEHS-UHFFFAOYSA-N", -1.24)
    CALL CalcH("2-chloro-1,1,1-trifluoroethane",       "75-88-7", "CYXIKYKBLDZZNW-UHFFFAOYSA-N", -0.18)
    CALL CalcH("halothane",                           "151-67-7", "BCQZXOMGPXTTIC-UHFFFAOYSA-N", -0.08)
    CALL CalcH("1-bromo-1,2,2,2-tetrafluoroethane",   "124-72-1", "RZXZIZDRFQFCTA-UHFFFAOYSA-N", -0.50)
    CALL CalcH("1,2-dichlorotetrafluoroethane",        "76-14-2", "DDMOUSALMHHKOS-UHFFFAOYSA-N", -1.71)
    CALL CalcH("2-chloro-1,1-difluoroethene",         "359-10-4", "HTHNTJCVPNKCPZ-UHFFFAOYSA-N", -0.55)
    CALL CalcH("dimethyl ether",                      "115-10-6", "LCGLNKUTAGEVQW-UHFFFAOYSA-N",  1.19)
    CALL CalcH("diethyl ether",                        "60-29-7", "RTZKZFJDLAIYFH-UHFFFAOYSA-N",  1.16)
    CALL CalcH("divinyl ether",                       "109-93-3", "QYKIQEUNHZKYBP-UHFFFAOYSA-N",  0.16)
    CALL CalcH("methoxyflurane",                       "76-38-0", "RFKMCNOHBTXSMU-UHFFFAOYSA-N",  0.62)
    CALL CalcH("isoflurane",                        "26675-46-7", "PIWKPBJCKXDKJR-UHFFFAOYSA-N", -0.22)
    CALL CalcH("enflurane",                         "13838-16-9", "JPGQOUSTVILISH-UHFFFAOYSA-N", -0.11)
    CALL CalcH("sevoflurane",                       "28523-86-6", "DFEYYRMXOJXZRJ-UHFFFAOYSA-N", -0.44)
    CALL CalcH("(2,2,2-trifluoroethoxy)-ethene",      "406-90-6", "DLEGDLSLRSOURQ-UHFFFAOYSA-N", -0.05)
    CALL CalcH("flurothyl",                           "333-36-8", "KGPPDNUWZNWPSI-UHFFFAOYSA-N", -0.13)
    CALL CalcH("iso-indoklon",                      "13171-18-1", "VNXYDFNVQBICRO-UHFFFAOYSA-N", -0.86)
    CALL CalcH("desflurane",                        "57041-67-5", "DPYMFVXJLLWWEU-UHFFFAOYSA-N", -0.65)
    CALL CalcH("propanone",                            "67-64-1", "CSCPPACGZOOCGX-UHFFFAOYSA-N",  2.60)
    CALL CalcH("butanone",                             "78-93-3", "ZWEHNKRNPOVVGH-UHFFFAOYSA-N",  2.41)
    CALL CalcH("2-pentanone",                         "107-87-9", "XNLICIUVMPYHGG-UHFFFAOYSA-N",  2.22)
    CALL CalcH("3-pentanone",                          "96-22-0", "FDPIMTJIUBPUKL-UHFFFAOYSA-N",  2.26)
    CALL CalcH("2-hexanone",                          "591-78-6", "QQZOPKMRPOGIEB-UHFFFAOYSA-N",  2.05)
    CALL CalcH("4-methyl-2-pentanone",                "108-10-1", "NTIZESTWPVYFNL-UHFFFAOYSA-N",  1.90)
    CALL CalcH("2-heptanone",                         "110-43-0", "CATSNJVOTSVZJV-UHFFFAOYSA-N",  1.98)
    CALL CalcH("ethyl methanoate",                    "109-94-4", "WBJINCZRORDGAQ-UHFFFAOYSA-N",  1.63)
    CALL CalcH("nitromethane",                         "75-52-5", "LYGJENNIWJXYER-UHFFFAOYSA-N",  2.65)
    CALL CalcH("methanol",                             "67-56-1", "OKKJLVBELUTLKV-UHFFFAOYSA-N",  3.43)
    CALL CalcH("ethanol",                              "64-17-5", "LFQSCWFLJHTTHZ-UHFFFAOYSA-N",  3.31)
    CALL CalcH("1-propanol",                           "71-23-8", "BDERNNFJNOPAEC-UHFFFAOYSA-N",  3.17)
    CALL CalcH("2-propanol",                           "67-63-0", "KFZMGEQAYNKOFK-UHFFFAOYSA-N",  3.09)
    CALL CalcH("2-methyl-1-propanol",                  "78-83-1", "ZXEKIIBDNHEJCQ-UHFFFAOYSA-N",  2.89)
    CALL CalcH("sulfur hexafluoride",                "2551-62-4", "SFZCNBIFKDRMGX-UHFFFAOYSA-N", -2.32)
    CALL CalcH("carbon disulfide",                     "75-15-0", "QGJOPFRUJISHPQ-UHFFFAOYSA-N", -0.10)
    CALL CalcH("benzene",                              "71-43-2", "UHOVQNZJYSORNB-UHFFFAOYSA-N",  0.44)
    CALL CalcH("methylbenzene",                       "108-88-3", "YXFVVABEGXRONW-UHFFFAOYSA-N",  0.35)
    CALL CalcH("ethylbenzene",                        "100-41-4", "YNQLUTRBYVCPMQ-UHFFFAOYSA-N",  0.23)
    CALL CalcH("1,2-dimethylbenzene",                  "95-47-6", "CTQNGGLPUBDAKN-UHFFFAOYSA-N",  0.42)
    CALL CalcH("1,3-dimethylbenzene",                 "108-38-3", "IVSZLXZYQVIEFR-UHFFFAOYSA-N",  0.22)
    CALL CalcH("1,4-dimethylbenzene",                 "106-42-3", "URLKBWYHVLBVBO-UHFFFAOYSA-N",  0.20)
    CALL CalcH("propylbenzene",                       "103-65-1", "ODLMAHJVESYWTB-UHFFFAOYSA-N",  0.11)
    CALL CalcH("(2-propyl)-benzene",                   "98-82-8", "RWGFKTVRMDUZSP-UHFFFAOYSA-N",  0.16)
    CALL CalcH("ethenylbenzene",                      "100-42-5", "PPBRXRYQALVLMV-UHFFFAOYSA-N",  0.67)
    CALL CalcH("2-propenylbenzene",                   "300-57-2", "HJWLCRVIBGQPNF-UHFFFAOYSA-N",  0.55)
    CALL CalcH("chlorobenzene",                       "108-90-7", "MVPPADPHJFYWMZ-UHFFFAOYSA-N",  0.61)
    CALL CalcH("1,2-dichlorobenzene",                  "95-50-1", "RFFLAFLAYFXFSW-UHFFFAOYSA-N",  0.95)
    CALL CalcH("1,3-dichlorobenzene",                 "541-73-1", "ZPQOPVIELGIULI-UHFFFAOYSA-N",  0.74)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, logL)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_, inchikey_
      REAL,             INTENT(IN) :: logL
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      Hominus  = Hscc_TO_HscpSI(10.**logL, T0)
      CALL MakeNote("whichref")
      CALL Output(Hominus)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3614

  !---------------------------------------------------------------------------

  ! ref3615 NOTUSED: The data base is inactive.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3616 ! Hvpc [bar/M]
    IMPLICIT NONE

    ref = "3616"
    type = "L"
    ! Delta hyd G is the accepted value from Tab. 4 using eqn (5),
    ! Delta hyd H is the accepted value from Tab. 2:
    CALL CalcH("methyl methanoate",                  "107-31-3", -3.53, -32.02)
    CALL CalcH("ethyl methanoate",                   "109-94-4", -3.11, -38.09)
    CALL CalcH("propyl methanoate",                  "110-74-7", -2.57, -40.51)
    CALL CalcH("isopropyl methanoate",               "625-55-8", -1.60        )
    CALL CalcH("butyl methanoate",                   "592-84-7", -1.62        )
    CALL CalcH("(2-methylpropyl)-methanoate",        "542-55-2", -1.09, -43.0 )
    CALL CalcH("pentyl methanoate",                  "638-49-3", -1.11, -48.1 )
    CALL CalcH("3-methylbutyl methanoate",           "110-45-2", -0.52, -47.7 )
    CALL CalcH("hexyl methanoate",                   "629-33-4", -0.11        )
    CALL CalcH("methyl ethanoate",                    "79-20-9", -5.18, -40.13)
    CALL CalcH("ethyl ethanoate",                    "141-78-6", -4.56, -45.32)
    CALL CalcH("propyl ethanoate",                   "109-60-4", -3.72, -48.66)
    CALL CalcH("isopropyl ethanoate",                "108-21-4", -2.95, -46.79)
    CALL CalcH("butyl ethanoate",                    "123-86-4", -3.09, -52.66)
    CALL CalcH("1,1-dimethylethyl ethanoate",        "540-88-5", -0.30, -46.22)
    CALL CalcH("(2-methylpropyl)-ethanoate",         "110-19-0", -2.13, -51.84)
    CALL CalcH("{sec}-butyl acetate",                "105-46-4", -2.06, -51.9 )
    CALL CalcH("pentyl ethanoate",                   "628-63-7", -2.43, -55.34)
    CALL CalcH("3-methylbutyl ethanoate",            "123-92-2", -1.69, -53.83)
    CALL CalcH("hexyl ethanoate",                    "142-92-7", -1.29, -60.8 )
    CALL CalcH("methyl propanoate",                  "554-12-1", -4.21, -44.54)
    CALL CalcH("ethyl propanoate",                   "105-37-3", -3.51, -49.52)
    CALL CalcH("propyl propanoate",                  "106-36-5", -2.60, -51.18)
    CALL CalcH("isopropyl propanoate",               "637-78-5", -1.51        )
    CALL CalcH("butyl propanoate",                   "590-01-2", -1.68, -57.8 )
    CALL CalcH("(2-methylpropyl)-propanoate",        "540-42-1", -1.04, -54.7 )
    CALL CalcH("pentyl propanoate",                  "624-54-4", -1.56        )
    CALL CalcH("methyl butanoate",                   "623-42-7", -3.55, -47.53)
    CALL CalcH("ethyl butanoate",                    "105-54-4", -2.63, -52.67)
    CALL CalcH("propyl butanoate",                   "105-66-8", -1.97, -54.9 )
    CALL CalcH("2-propyl butanoate",                 "638-11-9", -1.47        )
    CALL CalcH("butyl butanoate",                    "109-21-7", -1.20, -63.5 )
    CALL CalcH("methyl 2-methyl-propanoate",         "547-63-7", -2.40, -45.96)
    CALL CalcH("ethyl 2-methyl-propanoate",           "97-62-1", -1.33, -51.28)
    CALL CalcH("2-methylpropyl 2-methylpropanoate",   "97-85-8",  0.63, -55.3 )
    CALL CalcH("methyl pentanoate",                  "624-24-8", -2.69, -50.42)
    CALL CalcH("ethyl pentanoate",                   "539-82-2", -2.10, -56.52)
    CALL CalcH("butyl pentanoate",                   "591-68-4",  0.04        )
    CALL CalcH("methyl 2,2-dimethylpropanoate",      "598-98-1",  0.14, -46.21)
    CALL CalcH("ethyl 2,2-dimethylpropanoate",      "3938-95-2",  1.39, -50.33)
    CALL CalcH("ethyl 3-methylbutanoate",            "108-64-5", -0.78, -56.0 )
    CALL CalcH("ethyl 2-methylbutanoate",           "7452-79-1", -0.76, -55.4 )
    CALL CalcH("methyl hexanoate",                   "106-70-7", -2.36, -54.74)
    CALL CalcH("ethyl hexanoate",                    "123-66-0", -1.65, -60.17)
    CALL CalcH("methyl heptanoate",                  "106-73-0", -1.45        )
    CALL CalcH("methyl octanoate",                   "111-11-5", -0.36        )
    CALL CalcH("ethyl octanoate",                    "106-32-1", -1.29        )
    CALL CalcH("methyl nonanoate",                  "1731-84-6",  0.49        )
    CALL CalcH("ethyl decanoate",                    "110-38-3", -0.55        )

    ! some values cited from ref3638 for comparison:
    ! CALL CalcH("methyl ethanoate",                    "79-20-9", -4.42)
    ! CALL CalcH("methyl propanoate",                  "554-12-1", -3.97)
    ! CALL CalcH("methyl butanoate",                   "623-42-7", -3.62)
    ! CALL CalcH("methyl pentanoate",                  "624-24-8", -5.10)
    ! CALL CalcH("methyl hexanoate",                   "106-70-7", -3.24)
    ! => For unknown reasons, the cited values are different from those
    !    that I have calculated from ref3638.

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, DeltaG, DeltaH)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL,               INTENT(IN) :: DeltaG
      REAL, OPTIONAL,     INTENT(IN) :: DeltaH
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! As indicated in eqn (5), the authors implicitly use p0 = 1 bar
      ! and c0 = 1 M as the standard states. Thus, exp(DeltaG/(RT))
      ! yields Hvpc in [bar/M].
      Hominus =  1E3 / (bar * EXP(1E3*DeltaG/(Rgas*T0)))
      IF (PRESENT(DeltaH)) THEN
        mindHR = -1E3*DeltaH/Rgas
        CALL Output(Hominus, mindHR)
      ELSE
        CALL Output(Hominus)
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3616

  !---------------------------------------------------------------------------

  SUBROUTINE ref3617 ! Hvpc [bar/M]
    IMPLICIT NONE

    ref = "3617"
    type = "L"
    ! Delta hyd G is the accepted value from Tab. 1,
    ! Delta hyd H is the accepted value from Tab. 2:
    CALL CalcH("propanone",                 "67-64-1", -8.18, -41.5 )
    CALL CalcH("butanone",                  "78-93-3", -7.18, -45.5 )
    CALL CalcH("2-pentanone",              "107-87-9", -6.27, -49.0 )
    CALL CalcH("3-pentanone",               "96-22-0", -6.01, -49.6 )
    CALL CalcH("3-methyl-2-butanone",      "563-80-4", -5.47, -47.6 )
    CALL CalcH("2-hexanone",               "591-78-6", -5.67, -52.6 )
    CALL CalcH("3-hexanone",               "589-38-8", -5.01        )
    CALL CalcH("4-methyl-2-pentanone",     "108-10-1", -4.78, -50.2 )
    CALL CalcH("2-methyl-3-pentanone",     "565-69-5", -4.12        )
    CALL CalcH("3-methyl-2-pentanone",     "565-61-7", -5.28        )
    CALL CalcH("3,3-dimethyl-2-butanone",   "75-97-8", -3.71, -47.5 )
    CALL CalcH("2-heptanone",              "110-43-0", -4.88, -56.4 )
    CALL CalcH("4-heptanone",              "123-19-3", -4.50, -58.1 )
    CALL CalcH("2,4-dimethyl-3-pentanone", "565-80-0", -2.13, -54.0 )
    CALL CalcH("2-octanone",               "111-13-7", -4.36        )
    CALL CalcH("2-nonanone",               "821-55-6", -3.25, -62.7 )
    CALL CalcH("2,6-dimethyl-4-heptanone", "108-83-8", -0.73        )
    CALL CalcH("2-undecanone",             "112-12-9", -1.6         )

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, DeltaG, DeltaH)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL,               INTENT(IN) :: DeltaG
      REAL, OPTIONAL,     INTENT(IN) :: DeltaH
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! same conversion applied here as for ref3616:
      Hominus =  1E3 / (bar * EXP(1E3*DeltaG/(Rgas*T0)))
      IF (PRESENT(DeltaH)) THEN
        mindHR = -1E3*DeltaH/Rgas
        CALL Output(Hominus, mindHR)
      ELSE
        CALL Output(Hominus)
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3617

  !---------------------------------------------------------------------------

  SUBROUTINE ref3618 ! Hvpc [bar/M]
    IMPLICIT NONE

    ref = "3618"
    type = "L"
    ! Delta hyd G is the accepted value from Tab. 4,
    ! Delta hyd H is the accepted value from Tab. 2:
    CALL CalcH("methanethiol",              "74-93-1",  3.20, -24.37 )
    CALL CalcH("ethanethiol",               "75-08-1",  3.69, -28.87 )
    CALL CalcH("1-propanethiol",           "107-03-9",  4.36, -30.20 )
    CALL CalcH("1-butanethiol",            "109-79-5",  4.96, -36.29 )
    CALL CalcH("1-pentanethiol",           "110-66-7",  6.05         )
    CALL CalcH("1-hexanethiol",            "111-31-9",  7.31         )
    CALL CalcH("1-heptanethiol",          "1639-09-4",  7.88         )
    CALL CalcH("2-methyl-2-propanethiol",   "75-66-1",  7.69         )
    CALL CalcH("2-butanethiol",            "513-53-1",  4.98         )
    CALL CalcH("3-methylbutanethiol",      "541-31-1",  5.84         ) ! assuming this is 3-methyl-1-butanethiol
    CALL CalcH("dimethyl sulfide",          "75-18-3",  1.59, -31.47 )
    CALL CalcH("diethyl sulfide",          "352-93-2",  1.89, -40.22 )
    CALL CalcH("methyl propyl sulfide",   "3877-15-4",  2.08         )
    CALL CalcH("dipropyl sulfide",         "111-47-7",  2.74, -47.65 )
    CALL CalcH("diisopropyl sulfide",      "625-80-9",  2.99         )
    CALL CalcH("carbon disulfide",          "75-15-0",  7.21, -30.38 )
    CALL CalcH("dimethyl disulfide",       "624-92-0",  0.73, -34.69 )
    CALL CalcH("diethyl disulfide",        "110-81-6",  2.30, -40.96 )
    CALL CalcH("dipropyl disulfide",       "629-19-6",  3.78, -45.15 )
    CALL CalcH("dimethyl trisulfide",     "3658-80-8", -0.55         )

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, DeltaG, DeltaH)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL,               INTENT(IN) :: DeltaG
      REAL, OPTIONAL,     INTENT(IN) :: DeltaH
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! same conversion applied here as for ref3616:
      Hominus =  1E3 / (bar * EXP(1E3*DeltaG/(Rgas*T0)))
      IF (PRESENT(DeltaH)) THEN
        mindHR = -1E3*DeltaH/Rgas
        CALL Output(Hominus, mindHR)
      ELSE
        CALL Output(Hominus)
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3618

  !---------------------------------------------------------------------------

  SUBROUTINE ref3621 ! Hvpc [bar/M]
    IMPLICIT NONE

    ref = "3621"
    type = "L"
    ! Delta hyd G is the accepted value from Tab. 4,
    ! Delta hyd H is the accepted value from Tab. 2:
    CALL CalcH("acetonitrile",            "75-05-8",  -9.04, -34.91 )
    CALL CalcH("propane nitrile",        "107-12-0",  -7.91, -39.52 )
    CALL CalcH("butane nitrile",         "109-74-0",  -7.18, -42.12 )
    CALL CalcH("2-methylpropanenitrile",  "78-82-0",  -5.70, -40.02 )
    CALL CalcH("pentanenitrile",         "110-59-8",  -6.50, -45.64 )
    CALL CalcH("hexanenitrile",          "628-73-9",  -6.14         )
    CALL CalcH("octanenitrile",          "124-12-9",  -4.3          )
    CALL CalcH("butanedinitrile",        "110-61-2", -28.2,  -58.2  )
    CALL CalcH("pentanedinitrile",       "544-13-8", -27.93, -63.5  )
    CALL CalcH("hexanedinitrile",        "111-69-3", -29.47, -66.6  )

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, DeltaG, DeltaH)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL,               INTENT(IN) :: DeltaG
      REAL, OPTIONAL,     INTENT(IN) :: DeltaH
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! same conversion applied here as for ref3616:
      Hominus =  1E3 / (bar * EXP(1E3*DeltaG/(Rgas*T0)))
      IF (PRESENT(DeltaH)) THEN
        mindHR = -1E3*DeltaH/Rgas
        CALL Output(Hominus, mindHR)
      ELSE
        CALL Output(Hominus)
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3621

  !---------------------------------------------------------------------------

  ! ref3619 NOTUSED: The paper contains only amount fraction solubility,
  !                  no pressure.

  !---------------------------------------------------------------------------

  ! ref3620 NOTUSED: The paper contains only limiting activity
  !                  coefficients.

  !---------------------------------------------------------------------------

  ! ref3622 NOTUSED: The paper contains only solubility.

  !---------------------------------------------------------------------------

  ! ref3623 NOTUSED: The paper contains only infinite dilution activity
  !                  coefficients.

  !---------------------------------------------------------------------------

  ! ref3624 NOTUSED: The paper contains only citations of other papers.

  !---------------------------------------------------------------------------

  ! ref3625 NOTUSED: The paper contains only limiting activity
  !                  coefficients but no solubility.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3626 ! Hscc [1]
    IMPLICIT NONE

    ref = "3626"
    type = "T"
    ! CALL CalcH("benzene",         "71-43-2", -3.63, -31.89) ! cited from ref2375
    ! CALL CalcH("methylbenzene",  "108-88-3", -3.70, -36.18) ! cited from ref2375
    ! CALL CalcH("ethylbenzene",   "100-41-4", -3.31, -40.10) ! cited from ref2375
    CALL CalcH("ethylbenzene",   "100-41-4", -3.02, -40.94)
    CALL CalcH("propylbenzene",  "103-65-1", -2.14, -42.53)
    CALL CalcH("butylbenzene",   "104-51-8", -1.42, -44.15)
    CALL CalcH("pentylbenzene",  "538-68-1", -0.67, -49.47)
    CALL CalcH("hexylbenzene",  "1077-16-3", -0.05, -52.34)
    ! CALL CalcH("hexylbenzene",  "1077-16-3", -0.21, -52.7 ) ! only citation

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, DeltaG, DeltaH)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: DeltaG
      REAL,             INTENT(IN) :: DeltaH
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! The authors state that "the standard states for gas and solution
      ! are on a concentration basis". Thus, I assume that
      ! exp(-Delta_G/(RT) produces Hscc:
      Hominus = Hscc_TO_HscpSI_atT0 * EXP(-1E3*DeltaG/(Rgas*T0))
      mindHR = -1E3*DeltaH/Rgas
      CALL Output(Hominus, mindHR)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3626

  !---------------------------------------------------------------------------

  SUBROUTINE ref3627 ! Hscc [1]
    IMPLICIT NONE

    ref = "3627"
    type = "Q"

    ! Tab. 3:
    CALL CalcH("2-nitrophenol",                           "88-75-5", "IQUPABOKLQSFBK-UHFFFAOYSA-N",  3.44)
    CALL CalcH("3-nitrophenol",                          "554-84-7", "RTZZCYNQPHTPPL-UHFFFAOYSA-N",  6.73)
    CALL CalcH("4-nitrophenol",                          "100-02-7", "BTJIUGUIPKRLHP-UHFFFAOYSA-N",  7.54)
    CALL CalcH("2,4-dinitrophenol",                       "51-28-5", "UFBJCMHMOXMLKC-UHFFFAOYSA-N",  5.69)
    CALL CalcH("2,5-dinitrophenol",                      "329-71-5", "UWEZBKLLMKVIPI-UHFFFAOYSA-N",  5.45)
    CALL CalcH("1,2,3-trichlorobenzene",                  "87-61-6", "RELMFMZEBKVZJC-UHFFFAOYSA-N",  1.09)
    CALL CalcH("1,2,4-trichlorobenzene",                 "120-82-1", "PBKONEOXTCPAFI-UHFFFAOYSA-N",  0.79)
    CALL CalcH("trans-stilbene",                         "103-30-0", "PJANXHGTPQOBST-VAWYXSNFSA-N",  2.42)
    CALL CalcH("diuron",                                 "330-54-1", "XMTQQYYKAHVGBJ-UHFFFAOYSA-N",  8.00)
    CALL CalcH("monuron",                                "150-68-5", "BMLIZLVNXIYGCK-UHFFFAOYSA-N",  7.63)
    CALL CalcH("pyrene",                                 "129-00-0", "BBEAQIROQSPTKN-UHFFFAOYSA-N",  3.50)
    CALL CalcH("acenaphthene",                            "83-32-9", "CWRYPZZKDGJXCA-UHFFFAOYSA-N",  2.36)
    CALL CalcH("fluoranthene",                           "206-44-0", "GVEPBJHOBDJJJI-UHFFFAOYSA-N",  3.45)
    CALL CalcH("benzil",                                 "134-81-6", "WURBFLDFSFBTLW-UHFFFAOYSA-N",  4.87)
    CALL CalcH("2,3-benzindene",                          "86-73-7", "NIHNNTQXNPWCJQ-UHFFFAOYSA-N",  2.45)
    CALL CalcH("3-methylbenzoic acid",                    "99-04-7", "GPSDUZXPYCFOSQ-UHFFFAOYSA-N",  4.98)
    CALL CalcH("4-chlorobenzoic acid",                    "74-11-3", "XRHGYUZYPHTUJZ-UHFFFAOYSA-N",  4.80)
    CALL CalcH("2-methylbenzoic acid",                   "118-90-1", "ZWLPBLYKEWSWPD-UHFFFAOYSA-N",  4.38)
    CALL CalcH("4-nitrobenzoic acid",                     "62-23-7", "OTLNPYWUJOZPPA-UHFFFAOYSA-N",  6.90)
    CALL CalcH("acetylsalicylic acid ",                   "50-78-2", "BSYNRYMUTXBXSQ-UHFFFAOYSA-N",  8.57)
    CALL CalcH("naproxen",                             "22204-53-1", "CMWTZPSULFXXJA-VIFPVBQESA-N",  8.80)
    CALL CalcH("3,5-dinitrobenzoic acid",                 "99-34-3", "VYWYYJYRVSBHJQ-UHFFFAOYSA-N",  8.30)
    CALL CalcH("2-methoxybenzoic acid",                  "579-75-9", "ILUJQPXNXACGAN-UHFFFAOYSA-N",  6.80)
    CALL CalcH("4-methoxybenzoic acid",                  "100-09-4", "ZEYHEAKUIGZSGI-UHFFFAOYSA-N",  6.65)
    CALL CalcH("3-nitrobenzoic acid",                    "121-92-6", "AFPHTEQTJZKQAQ-UHFFFAOYSA-N",  6.93)
    CALL CalcH("ibuprofen",                            "15687-27-1", "HEFNNWSXXWATRW-UHFFFAOYSA-N",  5.70)
    CALL CalcH("3-chlorobenzoic acid",                   "535-80-8", "LULAYUGMBFYYEX-UHFFFAOYSA-N",  5.15)
    CALL CalcH("9-fluorenone",                           "486-25-9", "YLQWCDOCJODRMT-UHFFFAOYSA-N",  4.20)
    CALL CalcH("thianthrene",                             "92-85-3", "GVIJJXMXTUZIOD-UHFFFAOYSA-N",  4.00)
    CALL CalcH("xanthene",                                "92-83-1", "GJCOSYZMQJWQCA-UHFFFAOYSA-N",  2.50)
    CALL CalcH("4-chloro-3-nitrobenzoic acid",            "96-99-1", "DFXQXFGFOLXAPO-UHFFFAOYSA-N",  6.95)
    CALL CalcH("2-chloro-5-nitrobenzoic acid",          "2516-96-3", "QUEKGYQTRJVEQC-UHFFFAOYSA-N",  7.21)
    CALL CalcH("1-nitronaphthalene",                      "86-57-7", "RJKGJBPXVHTNJL-UHFFFAOYSA-N",  4.20)
    CALL CalcH("1,2,4,5-tetramethylbenzene",              "95-93-2", "SQNZJJAZBFDUTD-UHFFFAOYSA-N",  0.53)
    CALL CalcH("1-chloro-9,10-anthracenedione",           "82-44-0", "BOCJQSFSGAZAPQ-UHFFFAOYSA-N",  6.03)
    CALL CalcH("phenothiazine",                           "92-84-2", "WJFKNYWRSNBZNX-UHFFFAOYSA-N",  5.41)
    CALL CalcH("salicylamide",                            "65-45-2", "SKZKKFZAGNVIMN-UHFFFAOYSA-N",  7.70)
    CALL CalcH("2-ethylanthraquinone",                    "84-51-5", "SJEBAWHUJDUKQK-UHFFFAOYSA-N",  4.81)
    CALL CalcH("3,5-dinitro-2-methylbenzoic acid",     "28169-46-2", "CDVNZMKTJIBBBV-UHFFFAOYSA-N",  9.96)
    CALL CalcH("benzoin",                                "119-53-9", "ISAOCJYIOMOJEB-UHFFFAOYSA-N",  8.73)
    CALL CalcH("nicotinic acid",                          "59-67-6", "PVNIIMVLHYAWGP-UHFFFAOYSA-N",  7.79)
    CALL CalcH("isonicotinic acid",                       "55-22-1", "TWBYWOBDOCUKOW-UHFFFAOYSA-N",  7.59)
    CALL CalcH("3,4-dimethoxybenzoic acid",               "93-07-2", "DAUAQNGYDSHRET-UHFFFAOYSA-N",  8.43)
    CALL CalcH("3,4-dichlorobenzoic acid",                "51-44-5", "VPHHJAOJUJHJKD-UHFFFAOYSA-N",  4.72)
    CALL CalcH("1,4-dichloro-2-nitrobenzene",             "89-61-2", "RZKKOBGFCAHLCZ-UHFFFAOYSA-N",  2.90)
    CALL CalcH("2,3-dichloronitrobenzene",              "3209-22-1", "CMVQZRLQEOAYSW-UHFFFAOYSA-N",  3.09)
    CALL CalcH("3,4-dichloro-1-nitrobenzene",             "99-54-7", "NTBYINQTYWZXLH-UHFFFAOYSA-N",  2.71)
    CALL CalcH("2,4-dihydroxybenzophenone",              "131-56-6", "ZXDDPOHVAMWLBH-UHFFFAOYSA-N",  9.06)
    CALL CalcH("caprolactam",                            "105-60-2", "JBKVHLHDHHXQEQ-UHFFFAOYSA-N",  7.98)
    CALL CalcH("trans-cinnamic acid",                    "140-10-3", "WBYWAXJHAXSJNI-VOTSOKGWSA-N", 14.84)
    CALL CalcH("biotin",                                  "58-85-5", "YBJHBAHKTGYVGT-ZKWXMUAHSA-N",  6.14)
    CALL CalcH("isophthalic acid",                       "121-91-5", "QQVIHTHCMHWDBS-UHFFFAOYSA-N",  8.26)
    CALL CalcH("sorbic acid",                            "110-44-1", "WSWCOQWTEOXDQX-MQQKCMAXSA-N",  4.91)
    CALL CalcH("3-methyl-4-nitrobenzoic acid",          "3113-71-1", "XDTTUTIFWDAMIX-UHFFFAOYSA-N",  6.36)
    CALL CalcH("2-methyl-3-nitrobenzoic acid",          "1975-50-4", "YPQAFWHSMWWPLX-UHFFFAOYSA-N",  6.74)
    CALL CalcH("3,4,5-trimethoxybenzoic acid",           "118-41-2", "SJSOFNCYXJUNBT-UHFFFAOYSA-N",  9.26)
    CALL CalcH("maltol",                                 "118-71-8", "XPCTZQVDEJYUGT-UHFFFAOYSA-N",  6.26)
    CALL CalcH("terephthaldialdehyde",                   "623-27-8", "KUCOHFSKRZZVRO-UHFFFAOYSA-N",  4.59)
    CALL CalcH("2-methyl-6-nitroaniline",                "570-24-1", "FCMRHMPITHLLLA-UHFFFAOYSA-N",  5.06)
    CALL CalcH("5-nitro-8-hydroxyquinoline",            "4008-48-4", "RJIWZDNTCBHXAL-UHFFFAOYSA-N",  6.83)
    CALL CalcH("telmisartan",                         "144701-48-4", "RMMXLENWKUUMAY-UHFFFAOYSA-N", 17.68)
    CALL CalcH("porphyrin",                              "101-60-0", "RKCAIXNGYQCCAL-YYOYBPFYSA-N",  8.98)
    CALL CalcH("p-coumaric acid",                        "501-98-4", "NGSWKAQJJWESNS-ZZXKWVIFSA-N",  9.25)
    ! Sect. 3 and Tab. 5:
    CALL CalcH("2-naphthaldehyde",                        "66-99-9", "PJKVFARRVXDXAD-UHFFFAOYSA-N",  5.17)
    ! Sect. 3 and Tab. 6:
    CALL CalcH("hydroquinone bis(diphenyl phosphate)", "51732-57-1", "RECLNCPBBUHRDY-UHFFFAOYSA-N", 15.92) ! phosphoric acid,P,P'-1,4-phenylene P,P,P',P'-tetraphenyl ester

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, logKW)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_, inchikey_
      REAL,             INTENT(IN) :: logKW
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      CALL Output(Hscc_TO_HscpSI_atT0*10.**(logKW))
    END SUBROUTINE CalcH

  END SUBROUTINE ref3627

  !---------------------------------------------------------------------------

  SUBROUTINE ref3628 ! Hvpx [MPa]
    IMPLICIT NONE
    ref = "3628"
    type = "M"
    chem = "oxygen" ; casrn = "7782-44-7" ! O2
    ! Tab. 3, only using low-temperature data up to 50 C:
    CALL HTdep( &
      (/ 3., 6.5, 7.5, 13.7, 19.0, 22.4, 24.3, 28.5, 32.0, 42.0, 42.5, 49.6, 50.0 /) +CtoK, &
      cH2O * 1E-6 * 1E-3 / &
      (/ 2.53, 2.92, 3.33, 3.67, 4.06, 4.63, 4.31, 4.98, 5.11, 5.25, 5.27, 6.48, 5.65 /), &
      Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3628

  !---------------------------------------------------------------------------

  ! ref3629 NOTUSED: Only high-T data at >40 C are presented.

  !---------------------------------------------------------------------------

  ! ref3630 NOTUSED: Only fit functions for different salinities are
  !                  presented.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3631 ! Hscc [1]
    IMPLICIT NONE

    ref = "3631"
    type = "M"

    ! Tab. 1:
    CALL CalcH("methyl ethanoate",            "79-20-9", "KXKVLQRXCPHEJC-UHFFFAOYSA-N", 108.)
    CALL CalcH("ethyl ethanoate",            "141-78-6", "XEKOWRVHYACXOJ-UHFFFAOYSA-N", 71.5)
    CALL CalcH("propyl ethanoate",           "109-60-4", "YKYONYBAUNKHLG-UHFFFAOYSA-N", 53.0)
    CALL CalcH("isopropyl ethanoate",        "108-21-4", "JMMWKPVZQRWMSS-UHFFFAOYSA-N", 34.1)
    CALL CalcH("butyl ethanoate",            "123-86-4", "DKPFZGUDAPQIHT-UHFFFAOYSA-N", 32.7)
    CALL CalcH("(2-methylpropyl)-ethanoate", "110-19-0", "GJRQTCIYDGXPES-UHFFFAOYSA-N", 25.9)
    CALL CalcH("pentyl ethanoate",           "628-63-7", "PGMYKACGEOXYJE-UHFFFAOYSA-N", 24.0)
    CALL CalcH("isopentyl ethanoate",        "123-92-2", "MLFHJEHSLIIPHL-UHFFFAOYSA-N", 22.6)
    ! Tab. 3:
    CALL CalcH("methanol",                    "67-56-1", "OKKJLVBELUTLKV-UHFFFAOYSA-N", 3330.)
    CALL CalcH("ethanol",                     "64-17-5", "LFQSCWFLJHTTHZ-UHFFFAOYSA-N", 2140.)
    CALL CalcH("1-propanol",                  "71-23-8", "BDERNNFJNOPAEC-UHFFFAOYSA-N", 1850.)
    CALL CalcH("2-propanol",                  "67-63-0", "KFZMGEQAYNKOFK-UHFFFAOYSA-N", 1500.)
    CALL CalcH("1-butanol",                   "71-36-3", "LRHPLDYGYMQRHN-UHFFFAOYSA-N", 1310.)
    CALL CalcH("2-methyl-1-propanol",         "78-83-1", "ZXEKIIBDNHEJCQ-UHFFFAOYSA-N", 1130.)
    CALL CalcH("1-pentanol",                  "71-41-0", "AMQJEAYHLZJPGS-UHFFFAOYSA-N", 1080.)
    CALL CalcH("2-methyl-1-butanol",         "137-32-6", "QPRQEDXDYOZYLA-UHFFFAOYSA-N", 850.)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, lambda)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_, inchikey_
      REAL,             INTENT(IN) :: lambda
      REAL, PARAMETER :: temp = 37.+CtoK
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      CALL MakeNoteOtherTempK(temp)
      CALL Output(Hscc_TO_HscpSI(lambda,temp))
    END SUBROUTINE CalcH

  END SUBROUTINE ref3631

  !---------------------------------------------------------------------------

  SUBROUTINE ref3632 ! Hvcc [1]
    IMPLICIT NONE

    ref  = "3632"
    type = "M"
    ! temperature-dependent data from Tab. 3 is used, not Kaw(298.15) from Tab. 6:
    CALL CalcH("pentan-2-one",            "107-87-9", &
      (/ 273.17, 278.15, 288.15, 298.15, 308.15, 318.15, 328.15, 333.15 /), &
      (/ 0.530,  0.782,  1.61,   3.32,   6.01,   10.51,  16.9,   20.7   /))
    CALL CalcH("pentan-3-one",             "96-22-0", &
      (/ 273.35, 283.15, 288.15, 293.15, 298.15, 298.15, 303.15, 308.15, 313.15, 318.15, 318.15, 323.15, 333.15 /), &
      (/ 0.600,  1.35,   1.90,   2.69,   3.94,   3.87,   5.24,   7.21,   9.12,   11.6,   12.0,   15.2,   23.6   /))
    CALL CalcH("3-methylbutan-2-one",     "563-80-4", &
      (/ 278.15, 288.15, 298.15, 308.15, 318.15, 328.15 /), &
      (/ 1.18,   2.26,   4.58,   8.08,   13.4,   20.9   /))
    CALL CalcH("hexan-2-one",             "591-78-6", &
      (/ 278.15, 288.15, 298.15, 308.15, 318.15, 328.15 /), &
      (/ 0.867,  2.10,   4.40,   8.27,   15.1,   23.2   /))
    CALL CalcH("hexan-3-one",             "589-38-8", &
      (/ 278.15, 288.15, 298.15, 308.15, 319.15, 328.15 /), &
      (/ 1.13,   2.70,   5.83,   10.8,   20.5,   31.4   /))
    CALL CalcH("3-methylpentan-2-one",    "565-61-7", &
      (/ 278.15, 288.15, 298.15, 308.15, 318.15, 328.15 /), &
      (/ 1.21,   2.74,   5.29,   9.92,   17.2,   28.3   /))
    CALL CalcH("4-methylpentan-2-one",    "108-10-1", &
      (/ 278.15, 288.15, 288.15, 298.15, 308.15, 318.15, 328.15 /), &
      (/ 1.50,   3.17,   3.29,   6.50,   12.2,   21.3,   33.1   /))
    CALL CalcH("2-methylpentan-3-one",    "565-69-5", &
      (/ 278.15, 288.15, 298.15, 308.15, 318.15, 328.15 /), &
      (/ 1.82,   4.05,   8.33,   15.6,   26.2,   42.3   /))
    CALL CalcH("3,3-dimethylbutan-2-one",  "75-97-8", &
      (/ 278.15, 288.15, 298.15, 308.15, 318.15, 328.15 /), &
      (/ 2.31,   5.10,   10.1,   17.5,   29.1,   44.8   /))

CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp_, Kaw)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp_, Kaw
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp_, Hvcc_TO_HscpSI(Kaw/1000.,temp_), Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3632

  !---------------------------------------------------------------------------

  SUBROUTINE ref3633 ! Hvpx [kPa]
    IMPLICIT NONE

    ref = "3633"
    type = "M"
    ! KH from Tab. 9; fitting parameters AH, BH, CH, DH from Tab. 10:
    CALL CalcH("2-pentanol",         "6032-29-7", 78.5,  80.2291, -98.8349, -95.0135, 22.9685)
    CALL CalcH("3-pentanol",          "584-02-1", 86.1,  79.8735, -98.8842, -95.5499, 23.4663)
    CALL CalcH("2-methyl-1-butanol",  "137-32-6", 71.4,  75.4977, -91.2766, -85.539,  20.0469)
    CALL CalcH("3-methyl-1-butanol",  "123-51-3", 67.4,  76.3869, -98.2137, -98.5165, 26.0376)
    CALL CalcH("2-methyl-2-butanol",   "75-85-4", 77.3,  81.8832, -102.239, -99.3669, 24.704)
    CALL CalcH("3-methyl-2-butanol",  "598-75-4", 102.0, 78.2757, -98.7862, -97.6753, 25.1365)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, KH, AH, BH, CH, DH)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL,               INTENT(IN) :: KH, AH, BH, CH, DH
      REAL :: Hvpx_kPa
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! The temperature dependence is:
      ! ln(KH) = AH + BH/tau + CH*ln(tau) + DH*tau with tau = T/T0 and T0 = 298.15 K
      ! At T=T0, this reduces to:
      Hvpx_kPa = EXP(AH + BH + DH)
      ! check if eqn (9) with data from Tab. 9 really produces the values in Tab. 10:
      CALL consistency_check(KH, Hvpx_kPa, "Data in Tables~9 and 10")
      Hominus = cH2O/(1E3*Hvpx_kPa)
      ! The analytical derivative is:
      ! dln(KH)/d(1/T) = BH*T0 - CH*T - DH*T^2/T0
      ! At T=T0, this reduces to:
      mindHR = -(BH-CH-DH)*T0 ! the minus sign comes from Hvpx->Hscp
      CALL MakeNote("moreTdep")
      ! see also gnuplot/ref3065.gnu
      CALL Output(Hominus, mindHR)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3633

  !---------------------------------------------------------------------------

  ! ref3634 NOTUSED: The paper only contains citations from ref3635

  !---------------------------------------------------------------------------

  SUBROUTINE ref3635 ! Hvpx [atm]
    IMPLICIT NONE

    ref = "3635"
    type = "M"
    ! Tab. VI:
    ! (some chemical names were corrected according to Tab. 4 in ref3634)
    CALL CalcH("methanol",         "OKKJLVBELUTLKV-UHFFFAOYSA-N", 0.27, "a")
    CALL CalcH("ethanol",          "LFQSCWFLJHTTHZ-UHFFFAOYSA-N", 0.28, "a")
    CALL CalcH("1-propanol",       "BDERNNFJNOPAEC-UHFFFAOYSA-N", 0.30, "a")
    CALL CalcH("propanone",        "CSCPPACGZOOCGX-UHFFFAOYSA-N", 2.94, "b")
    CALL CalcH("butanone",         "ZWEHNKRNPOVVGH-UHFFFAOYSA-N", 4.50, "b")
    CALL CalcH("ethyl methanoate", "WBJINCZRORDGAQ-UHFFFAOYSA-N", 23.4, "b")
    CALL CalcH("ethyl ethanoate",  "XEKOWRVHYACXOJ-UHFFFAOYSA-N", 12.6, "b")
    CALL CalcH("propyl ethanoate", "YKYONYBAUNKHLG-UHFFFAOYSA-N", 17.8, "b")
    CALL CalcH("ethanal",          "IKHGUXGNUITLKF-UHFFFAOYSA-N",  6.6, "b")
    CALL CalcH("propanal",         "NBBJYMSMWIIQGU-UHFFFAOYSA-N",  9.0, "b")

  CONTAINS

    SUBROUTINE CalcH (chem_, inchikey_, Hi, footnote)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, inchikey_
      REAL,             INTENT(IN) :: Hi
      CHARACTER,        INTENT(IN) :: footnote
      chem=chem_ ; inchikey=inchikey_ ! make global for Output
      IF (footnote=="b") CALL MakeNoteOtherTempK(303)
      CALL Output(Hvpx_TIMES_HscpSI/Hi)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3635

  !---------------------------------------------------------------------------

  SUBROUTINE ref3636 ! Hvpx [kPa]
    IMPLICIT NONE

    ref = "3636"
    type = "M"
    ! using only the most dilute solutions, see util/ref3636/plot.py to
    ! check if linear regression is justified:
    CALL CalcH("methyl formate",  "107-31-3", & ! Tab. 2
      (/ 0.0000, 0.0024, 0.0069, 0.0115, 0.0158, 0.0222, 0.0292 /), &
      (/ 3.36, 6.71, 12.50, 18.22, 23.55, 30.97, 38.41 /))
    CALL CalcH("ethyl formate",   "109-94-4", & ! Tab. 4
      (/ 0.0000, 0.0026, 0.0057, 0.0080, 0.0111, 0.0158, 0.0221 /), &
      (/ 3.31, 7.41, 12.14, 15.58, 19.87, 25.96, 33.46 /))
    CALL CalcH("n-butyl formate", "592-84-7", & ! Tab. 6
      (/ 0.0000, 0.0002, 0.0004, 0.0006, 0.0008, 0.0009, 0.0013 /), &
      (/ 3.30, 3.94, 4.58, 5.19, 5.76, 6.42, 6.80 /))

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, x1, P)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: x1, P
      REAL :: dummy, dpdx
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! To to obtain the Henry's law constant, the saturation pressure
      ! of water must be added here:
      CALL LinReg (x1, P, dummy, dpdx)
      CALL Output(cH2O/(1E3*dpdx+psat(T0)))
    END SUBROUTINE CalcH

  END SUBROUTINE ref3636

  !---------------------------------------------------------------------------

  ! ref3637 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3638 ! Hsbp [mol/(kg*torr)]
    IMPLICIT NONE

    REAL, PARAMETER, DIMENSION(4) :: temp = (/ 293.2, 298.2, 303.2, 308.2 /)
    ref = "3638"
    type = "V"
    ! vapor pressures (Tab. 3) and infinite dilution activity coefficients (Tab. 7):
    CALL CalcH("propanal",          "123-38-6", (/ 257.9, 318.3, 389.6, 473.2 /), (/ 0.766, 0.745, 0.726, 0.709 /))
    CALL CalcH("butanal",           "123-72-8", (/ 88.53, 112.0, 140.5, 174.6 /), (/ 0.792, 0.765, 0.741, 0.719 /))
    CALL CalcH("pentanal",          "110-62-3", (/ 26.25, 34.35, 44.44, 56.91 /), (/ 0.951, 0.904, 0.864, 0.827 /))
    CALL CalcH("propanone",          "67-64-1", (/ 184.9, 229.9, 283.6, 347.1 /), (/ 1.088, 1.043, 1.002, 0.965 /))
    CALL CalcH("butanone",           "78-93-3", (/ 74.91, 94.74, 118.8, 147.6 /), (/ 0.941, 0.919, 0.899, 0.881 /))
    CALL CalcH("2-pentanone",       "107-87-9", (/ 26.91, 35.23, 45.61, 58.43 /), (/ 0.918, 0.897, 0.880, 0.865 /))
    CALL CalcH("2-hexanone",        "591-78-6", (/ 9.18,  12.14, 15.89, 20.63 /), (/ 0.944, 0.948, 0.951, 0.953 /))
    CALL CalcH("2-heptanone",       "110-43-0", (/ 2.82,  3.91,  5.35,  7.23  /), (/ 1.079, 1.070, 1.064, 1.059 /))
    CALL CalcH("methyl ethanoate",   "79-20-9", (/ 172.6, 216.5, 269.2, 331.8 /), (/ 0.749, 0.721, 0.697, 0.676 /))
    CALL CalcH("methyl propanoate", "554-12-1", (/ 66.55, 85.47, 108.6, 136.8 /), (/ 0.690, 0.670, 0.653, 0.638 /))
    CALL CalcH("methyl butanoate",  "623-42-7", (/ 24.55, 32.23, 41.95, 54.10 /), (/ 0.665, 0.653, 0.641, 0.630 /))
    CALL CalcH("methyl pentanoate", "624-24-8", (/ 14.5,  19.07, 25.10, 32.37 /), (/ 0.402, 0.406, 0.407, 0.412 /))
    CALL CalcH("methyl hexanoate",  "106-70-7", (/ 2.84,  3.85,  5.16,  6.86  /), (/ 0.729, 0.740, 0.750, 0.760 /))
    CALL CalcH("1-propanol",         "71-23-8", (/ 14.56, 20.48, 28.36, 38.72 /), (/ 4.982, 4.588, 4.255, 3.969 /))
    CALL CalcH("1-butanol",          "71-36-3", (/ 4.178, 6.155, 8.901, 12.65 /), (/ 6.217, 5.529, 4.964, 4.498 /))
    CALL CalcH("1-pentanol",         "71-41-0", (/ 1.55,  2.35,  3.48,  5.08  /), (/ 6.003, 5.247, 4.652, 4.147 /))
    CALL CalcH("1-hexanol",         "111-27-3", (/ 0.539, 0.822, 1.23,  1.81  /), (/ 6.185, 5.438, 4.823, 4.312 /))
    CALL CalcH("1-heptanol",        "111-70-6", (/ 0.150, 0.235, 0.363, 0.551 /), (/ 7.973, 6.888, 5.996, 5.244 /))

CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, psat, gamma)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: psat, gamma
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! The molality-based Henry volatility is Hvpb = p / b, where b = molality.
      ! According to eqn (2): Hvpb = "p_A" / "m_A^S" = gamma * p_sat.
      ! Assuming that molality is in [mol/kg], it follows that gamma is in [kg/mol].
      ! Finally, Hscp = rho_H2O / Hvpb.
      CALL HTdep(temp, rhoH2O/(psat*mmHg*gamma), Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3638

  !---------------------------------------------------------------------------

  ! ref3639 NOTUSED: The solvent is not water.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3640 ! Hscc [1]
    IMPLICIT NONE

    ref = "3640"
    type = "M"
    ! Tab. I:
    CALL CalcH("propanone",          "67-64-1", (/ 15., 20. /),      (/ 1170., 750. /))
    CALL CalcH("butyl ethanoate",   "123-86-4", (/ 15., 25., 25. /), (/ 190., 87., 86. /))
    CALL CalcH("butanone",           "78-93-3", (/ 15. /), (/ 770.   /))
    CALL CalcH("1,4-dioxane",       "123-91-1", (/ 15. /), (/ 10650. /))
    CALL CalcH("ethyl ethanoate",   "141-78-6", (/ 15. /), (/ 300.   /))
    CALL CalcH("methyl ethanoate",   "79-20-9", (/ 25. /), (/ 190.   /))
    CALL CalcH("methyl propanoate", "554-12-1", (/ 25. /), (/ 130.   /))
    CALL CalcH("methyl butanoate",  "623-42-7", (/ 25. /), (/ 90.    /))
    CALL CalcH("dibutyl ether",     "142-96-1", (/ 25. /), (/ 5.3    /))

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, K)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, K
      chem=chem_ ; casrn=casrn_ ! make global for Output
      IF (SIZE(K)==1) THEN
        CALL MakeNoteOtherTempC(temp(1))
        CALL Output(Hscc_TO_HscpSI(K(1), temp(1)+CtoK))
      ELSE
        CALL HTdep(temp+CtoK, Hscc_TO_HscpSI(K, temp+CtoK), Hominus, mindHR)
        CALL Output(Hominus, mindHR, r2)
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3640

  !---------------------------------------------------------------------------

  SUBROUTINE ref3641 ! special definition
    IMPLICIT NONE
    REAL :: dummy, dpdx_25, dpdx_35
    ! using only the 3 most dilute solutions, see util/ref3641/plot.py to
    ! check if linear regression is justified.
    REAL, PARAMETER, DIMENSION(3) :: x = (/ 0., 0.2542, 0.5084 /) * 1E3 / cH2O
    REAL, PARAMETER, DIMENSION(3) :: p_tot_25 = (/ 23.8, 46.0, 67.2 /)
    REAL, PARAMETER, DIMENSION(3) :: p_tot_35 = (/ 42.2, 80.2, 115.9 /)

    ref = "3641"
    type = "M"
    chem = "methyl ethanoate" ; casrn = "79-20-9" ! methyl acetate
    ! to obtain the Henry's law constant, the saturation pressure of
    ! water must be added here:
    CALL LinReg (x, p_tot_25, dummy, dpdx_25)
    CALL LinReg (x, p_tot_35, dummy, dpdx_35)
    CALL HTdep( (/ 25., 35. /) + CtoK, &
      cH2O / (mmHg * (/ dpdx_25, dpdx_35 /) + psat(T0)), Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)

  END SUBROUTINE ref3641

  !---------------------------------------------------------------------------

  SUBROUTINE ref3642 ! special definition (aq. solution in g/L and p in Pa)
    IMPLICIT NONE

    ref = "3642"
    type = "V"
    ! Tabs. 2 and 3:
    CALL CalcH("ethyl ethanoate", "141-78-6", 24527., 67.1, 88.  )
    CALL CalcH("ethyl butanoate", "105-54-4", 3839.,  5.3,  116. )
    CALL CalcH("ethyl hexanoate", "123-66-0", 500.,   0.5,  144. )
    CALL CalcH("2-pentanone",     "107-87-9", 9464.,  48.1, 86.  )

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, psat, caq, M)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL,               INTENT(IN) :: psat ! Pa
      REAL,               INTENT(IN) :: caq  ! g/L
      REAL,               INTENT(IN) :: M    ! g/mol
      chem=chem_ ; casrn=casrn_ ! make global for Output
      Hominus = 1E3 * (caq/M) / psat
      CALL MakeNoteOtherTempC(37)
      CALL Output(Hominus)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3642

  !---------------------------------------------------------------------------

  SUBROUTINE ref3643 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3643"
    type = "M"
    ! Tab. 4:
    CALL CalcH("dimethyl sulfide",     "75-18-3", 61.02)
    CALL CalcH("1-propanol",           "71-23-8",  1.20)
    CALL CalcH("2,3-butanedione",     "431-03-8",  1.85)
    CALL CalcH("butanone",             "78-93-3",  3.91)
    CALL CalcH("ethyl ethanoate",     "141-78-6",  9.94)
    CALL CalcH("1-butanol",            "71-36-3",  0.86)
    CALL CalcH("2-pentanol",         "6032-29-7",  1.30)
    CALL CalcH("propyl ethanoate",    "109-60-4",  9.93)
    CALL CalcH("3-methyl-1-butanol",  "123-51-3",  1.19)
    CALL CalcH("ethyl butanoate",     "105-54-4", 18.11)
    CALL CalcH("hexanal",              "66-25-1", 14.34)
    CALL CalcH("butyl ethanoate",     "123-86-4", 13.99)
    CALL CalcH("1-hexanol",           "111-27-3",  1.53)
    CALL CalcH("2-heptanone",         "110-43-0", 10.05)
    CALL CalcH("heptanal",            "111-71-7", 19.12)
    CALL CalcH("alpha-pinene",         "80-56-8", 27.98)
    CALL CalcH("2-octanone",          "111-13-7", 13.55)
    CALL CalcH("octanal",             "124-13-0", 18.27)
    CALL CalcH("2-nonanol",           "628-99-9",  5.96)
    CALL CalcH("2-decanone",          "693-54-9", 21.73)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, K)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: K
      REAL :: temp
      temp = 37.+CtoK
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL MakeNoteOtherTempK(temp)
      CALL Output(Hvcc_TO_HscpSI(1E-3*K, temp))
    END SUBROUTINE CalcH

  END SUBROUTINE ref3643

  !---------------------------------------------------------------------------

  SUBROUTINE ref3644 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3644"
    type = "C" ! data from a PhD thesis
    ! Tab. 2:
    CALL CalcH("ethyl ethanoate",  "141-78-6", 10.3)
    CALL CalcH("methyl butanoate", "623-42-7", 11.2)
    CALL CalcH("ethyl butanoate",  "105-54-4", 14.8)
    CALL CalcH("hexanal",           "66-25-1", 12.9)
    CALL CalcH("octanal",          "124-13-0", 21.1)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, Hvcc)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL, INTENT(IN)             :: Hvcc
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL MakeNoteOtherTempC(37)
      CALL Output(Hvcc_TIMES_HscpSI_atT0 / (1E-3*Hvcc))
    END SUBROUTINE CalcH

  END SUBROUTINE ref3644

  !---------------------------------------------------------------------------

  SUBROUTINE ref3645 ! Hvpx [atm]
    IMPLICIT NONE

    ref = "3645"
    type = "M"
    ! Defining H=y/x is not possible. The values presented in Tab. 4 are
    ! actually Hvpx = p/x in [atm]:
    CALL CalcH("ethyl ethanoate",       "141-78-6", 8.22)
    CALL CalcH("2,5-dimethylpyrazine",  "123-32-0", 0.1 )
    CALL CalcH("1-octen-3-ol",         "3391-86-4", 2.16)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, Ki)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL, INTENT(IN)             :: Ki
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL Output(Hvpx_TIMES_HscpSI/Ki)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3645

  !---------------------------------------------------------------------------

  SUBROUTINE ref3646 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3646"
    type = "M"
    ! Tab. 1:
    ! (alcohols are from ref3657 and commented out here)
    CALL CalcH("1,2-dibromoethane",       "106-93-4", 3.1E-2)
    CALL CalcH("1,2-dichloroethane",      "107-06-2", 5.4E-2)
    CALL CalcH("1,2-dichloropropane",      "78-87-5", 7.4E-2)
    CALL CalcH("1,4-dioxane",             "123-91-1", 2.0E-4)
    !CALL CalcH("1-butanol",                "71-36-3", 3.8E-4)
    !CALL CalcH("1-hexanol",               "111-27-3", 6.9E-4)
    CALL CalcH("1-nitropropane",          "108-03-2", 2.5E-3)
    !CALL CalcH("1-pentanol",               "71-41-0", 5.2E-4)
    !CALL CalcH("1-propanol",               "71-23-8", 2.8E-4)
    !CALL CalcH("2-butanol",                "78-92-2", 4.9E-4)
    CALL CalcH("butanone",                 "78-93-3", 2.6E-3)
    !CALL CalcH("2-hexanol",               "626-93-7", 8.5E-4)
    !CALL CalcH("2-methyl-2-butanol",       "75-85-4", 5.8E-4)
    !CALL CalcH("2-pentanol",             "6032-29-7", 6.1E-4)
    !CALL CalcH("2-propanol",               "67-63-0", 3.2E-4)
    !CALL CalcH("3-methyl-3-pentanol",      "77-74-7", 8.4E-4)
    CALL CalcH("4-methyl-2-pentanone",    "108-10-1", 8.6E-3)
    CALL CalcH("propanone",                "67-64-1", 4.2E-3)
    CALL CalcH("ethanenitrile",            "75-05-8", 1.2E-3)
    CALL CalcH("2-propenenitrile",        "107-13-1", 1.3E-2)
    CALL CalcH("3-chloro-1-propene",      "107-05-1", 1.1E-1)
    CALL CalcH("benzene",                  "71-43-2", 2.5E-1)
    CALL CalcH("chlorobenzene",           "108-90-7", 1.5E-1)
    CALL CalcH("trichloromethane",         "67-66-3", 1.2E-1)
    CALL CalcH("dichloromethane",          "75-09-2", 1.1E-4) ! strange value
    CALL CalcH("(chloromethyl)-oxirane",  "106-89-8", 1.4E-3)
    !CALL CalcH("ethanol",                  "64-17-5", 2.1E-4)
    CALL CalcH("ethyl ethanoate",         "141-78-6", 1.2E-2)
    !CALL CalcH("2-methyl-1-propanol",      "78-83-1", 5.2E-4)
    CALL CalcH("limonene",                "138-86-3", 1.7   )
    !CALL CalcH("methanol",                 "67-56-1", 1.9E-4)
    CALL CalcH("butyl ethanoate",         "123-86-4", 1.5E-2)
    CALL CalcH("propyl ethanoate",        "109-60-4", 1.1E-2)
    CALL CalcH("1,2-dimethylbenzene",      "95-47-6", 1.8E-1)
    CALL CalcH("pyridine",                "110-86-1", 4.1E-4)
    CALL CalcH("ethenylbenzene",          "100-42-5", 1.6E-1)
    !CALL CalcH("2-methyl-2-propanol",      "75-65-0", 5.0E-4)
    CALL CalcH("tetrahydrofuran",         "109-99-9", 6.3E-4)
    CALL CalcH("methylbenzene",           "108-88-3", 2.8E-1)
    CALL CalcH("trichloronitromethane",    "76-06-2", 1.0E-1)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, Kaw)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL, INTENT(IN)             :: Kaw
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL Output(Hvcc_TIMES_HscpSI_atT0 / Kaw)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3646

  !---------------------------------------------------------------------------

  SUBROUTINE ref3647 ! Hvpx [atm]
    IMPLICIT NONE

    ref = "3647"
    type = "M"
    ! Defining H=y/x is not possible. The values presented in Tab. 1 are
    ! actually Hvpx = p/x in [atm]:
    CALL CalcH("2,3-butanedione", "431-03-8", 0.60)
    CALL CalcH("ethyl ethanoate", "141-78-6",  9.6)
    CALL CalcH("ethyl butanoate", "105-54-4", 22.4)
    CALL CalcH("ethyl hexanoate", "123-66-0", 58.2)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, K)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: K
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL Output(Hvpx_TIMES_HscpSI/K)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3647

  !---------------------------------------------------------------------------

  SUBROUTINE ref3648 ! Hvpx [kPa]
    IMPLICIT NONE

    ref = "3648"
    type = "M"
    ! Tab. I:
    CALL CalcH("methanol",         "67-56-1", &
      CtoK + (/ 20.0, 30.0, 40.0 /),       (/ 14.6, 22.1, 32.1 /)      )
    CALL CalcH("ethanol",          "64-17-5", &
      CtoK + (/ 10.0, 20.0, 40.0, 60.0 /), (/ 5.5, 17.0, 31.7, 41.7 /) )
    CALL CalcH("tetrahydrofuran", "109-99-9", &
      CtoK + (/ 20.0, 35.0, 50.0 /),       (/ 68.9, 149.0, 259.4 /)    )
    CALL CalcH("2-propanol",       "67-63-0", &
      CtoK + (/ 15.0 /),                   (/ 36.67 /)                 )
    CALL CalcH("ethyl acetate",   "141-78-6", &
      CtoK + (/ 15.0 /),                   (/ 483.3 /)                 )

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, dpdx)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, dpdx
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! To to obtain the Henry's law constant, the saturation pressure
      ! of water must be added here:
      IF (SIZE(temp)==1) THEN
        CALL MakeNoteOtherTempK(temp(1))
        CALL Output(cH2O/(1E3*dpdx(1)+psat(temp(1))))
      ELSE
        CALL HTdep(temp, cH2O/(1E3*dpdx+psat(temp)), Hominus, mindHR)
        CALL Output(Hominus, mindHR, r2)
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3648

  !---------------------------------------------------------------------------

  SUBROUTINE ref3649 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3649"
    type = "M"
    chem = "ethyl ethanoate" ; casrn = "141-78-6"
    CALL Output(Hvcc_TIMES_HscpSI_atT0/6.55E-3)

  END SUBROUTINE ref3649

  !---------------------------------------------------------------------------

  SUBROUTINE ref3650 ! Hvpc [atm*m3/mol] and Hvcc [1]
    IMPLICIT NONE

    ref = "3650"
    ! Tabs. I and III:
    CALL CalcH("isoamyl acetate",   "123-92-2", 5.87E-4, 4.5,  35.7)
    CALL CalcH("amyl acetate",      "628-63-7", 3.88E-4, 3.0,  41.2)
    CALL CalcH("ethyl pentanoate",  "539-82-2", 3.72E-4, 2.8,  40.3)
    CALL CalcH("hexanal",            "66-25-1", 2.13E-4, 2.5,  40.4)
    CALL CalcH("trans-2-hexenal",  "6728-26-3", 4.89E-5, 0.44, 47.2)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, Hvpc, Hvcc, DeltaH)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL, INTENT(IN)             :: Hvpc, Hvcc, DeltaH
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! Hvpc from Tab. I:
      type = "C"
      CALL Output(1./(atm*Hvpc))
      ! The Hvcc values in Tab. III seem to be off by a factor of 4 or
      ! 5. This was also noticed by ref3616, who indicated the data as
      ! outliers.
      type = "M"
      CALL MakeNoteOtherTempC(30)
      CALL Output(Hvcc_TO_HscpSI(1E-3*Hvcc, 30.+CtoK), 1E3*DeltaH/Rgas)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3650

  !---------------------------------------------------------------------------

  SUBROUTINE ref3651 ! Hvpx [atm]
    IMPLICIT NONE

    ref = "3651"
    type = "M"

    ! Defining H as "ppm/ppm" as in eqn (1) is not possible. The values
    ! presented in Tab. 1 are actually Hvpx = p/x in [atm].

    ! Tab. 1 does not list 1E3*h760 but 1E-3*h760.

    ! It is unclear why the values for water in Tab. 2 are different to
    ! those in Tab. 1.

    CALL CalcH("ethyl butanoate", "105-54-4", 13.5E-3)
    CALL CalcH("ethyl hexanoate", "123-66-0", 34.0E-3)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, h760)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: h760
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL Output(Hvpx_TIMES_HscpSI/(1E3*h760))
    END SUBROUTINE CalcH

  END SUBROUTINE ref3651

  !---------------------------------------------------------------------------

  ! ref3652 NOTUSED: The paper contains only high-temperature data.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3653 ! Hvpc [atm*m3/mol)]
    IMPLICIT NONE

    ref = "3653"
    type = "C" ! from Warner et al. 1987 (tech. rep.)
    CALL CalcH("1,3-Dichloro-1-propene",               "542-75-6", 3.55E-3)
    CALL CalcH("1,2-Dichloropropane",                   "78-87-5", 2.82E-3)
    CALL CalcH("1,1,2,3,4,4-Hexachloro-1,3-butadiene",  "87-68-3", 1.03E-2)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, H)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: H
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL Output(1E-3 * Hscp_TO_HscpSI / H)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3653

  !---------------------------------------------------------------------------

  SUBROUTINE ref3654 ! Hvpx [kPa]
    IMPLICIT NONE
    REAL :: H40, H60, H80, H100

    ref = "3654"
    type = "M"
    ! see util/ref3654/plot.py to check if linear regression is justified:

    ! MEA:
    H40 = Calc_H_at_temp( &
      (/ 0.0001, 0.0002, 0.0003, 0.0005, 0.0007 /), &
      (/ 0.0121, 0.0297, 0.0477, 0.0678, 0.0928 /), &
      (/   7.28,   7.18,   6.98,   6.88,   6.67 /))
    H60 = Calc_H_at_temp( &
      (/ 0.0003, 0.0007, 0.0008, 0.0013, 0.0010, 0.0016, 0.0001, 0.0007, 0.0011, 0.0001, 0.0005 /), &
      (/ 0.0136, 0.0315, 0.0491, 0.0922, 0.0718, 0.1000, 0.0131, 0.0489, 0.0850, 0.0137, 0.0500 /), &
      (/  19.57,  19.19,  18.76,  17.98,  18.37,  17.68,  19.57,  18.85,  17.98,  19.58,  18.78 /))
    H80 = Calc_H_at_temp( &
      (/ 0.0003, 0.0006, 0.0010, 0.0014, 0.0020, 0.0009, 0.0020 /), &
      (/ 0.0146, 0.0305, 0.0495, 0.0703, 0.0907, 0.0424, 0.0932 /), &
      (/  46.45,  45.80,  44.77,  43.89,  42.86,  45.18,  42.68 /))
    H100 = Calc_H_at_temp( &
      (/ 0.0005, 0.0012, 0.0019, 0.0027, 0.0040, 0.0018, 0.0038, 0.0025, 0.0033, 0.0020 /), &
      (/ 0.0143, 0.0321, 0.0534, 0.0727, 0.0955, 0.0447, 0.0962, 0.0749, 0.0826, 0.0606 /), &
      (/  99.57,  97.68,  95.69,  93.68,  91.51,  96.58,  91.08,  93.47,  92.88,  95.26 /))
    CALL CalcH("ethanolamine", "141-43-5", (/ 40., 60., 80., 100. /), (/ H40, H60, H80, H100 /))

    ! MDEA:
    H40 = Calc_H_at_temp( &
      (/ 0.00001, 0.00001, 0.00002, 0.00001, 0.00002, 0.00002 /), &
      (/ 0.0076,    0.0292, 0.0403,  0.0540,  0.0693,  0.0850 /), &
      (/   7.27,      7.17,   7.08,    6.98,    6.78,    6.67 /))
    H60 = Calc_H_at_temp( &
      (/ 0.00004, 0.00003, 0.00008, 0.00008 /), &
      (/  0.0047,  0.0169,  0.0878,  0.0699 /), &
      (/   19.77,   19.47,   18.10,   18.48 /))
    H80 = Calc_H_at_temp( &
      (/ 0.00008, 0.00008, 0.00007, 0.00001, 0.00003, 0.00004, 0.00008, 0.00015, 0.00014 /), &
      (/  0.0757,  0.0547,  0.0390,  0.0056,  0.0142,  0.0217,  0.0475,  0.0742,  0.0973 /), &
      (/   43.88,   44.77,   45.48,   46.98,   46.59,   46.29,   45.09,   43.89,   42.58 /))
    H100 = Calc_H_at_temp( &
      (/ 0.00030, 0.00027, 0.00023, 0.00007, 0.00013, 0.00017, 0.00044, 0.00003, 0.00007, 0.00013, 0.00022, 0.00031, 0.00046 /), &
      (/  0.0989,  0.0723,  0.0533,  0.0113,  0.0239,  0.0461,  0.0903,  0.0059,  0.0120,  0.0232,  0.0537,  0.0753,  0.0991 /), &
      (/   91.97,   94.48,   96.18,   99.88,   98.67,   96.96,   92.77,  100.40,   99.80,   98.80,   96.21,   94.00,   91.66 /))
    CALL CalcH("N-methyl-diethanolamine", "105-59-9", (/ 40., 60., 80., 100. /), (/ H40, H60, H80, H100 /))

    ! MAPA:
    H40 = Calc_H_at_temp( &
      (/ 0.00008, 0.00016, 0.00026, 0.00052, 0.00068, 0.00133 /), &
      (/  0.0099,  0.0203,  0.0320,  0.0460,  0.0623,  0.0810 /), &
      (/    7.28,    7.18,    7.17,    6.98,    6.78,    6.58 /))
    H60 = Calc_H_at_temp( &
      (/ 0.00220, 0.00009, 0.00017, 0.00026, 0.00043, 0.00112, 0.00303 /), &
      (/  0.0741,  0.0025,  0.0095,  0.0138,  0.0191,  0.0459,  0.0908 /), &
      (/   17.97,   19.77,   19.68,   19.57,   19.47,   18.87,   17.57 /))
    H80 = Calc_H_at_temp( &
      (/ 0.00018, 0.00045, 0.00061, 0.00079, 0.00243, 0.00663, 0.00399, 0.00018 /), &
      (/  0.0053,  0.0099,  0.0149,  0.0200,  0.0510,  0.0997,  0.0750,  0.0053 /), &
      (/   49.66,   46.78,   46.47,   46.27,   44.68,   41.87,   43.28,   49.66 /))
    H100 = Calc_H_at_temp( &
      (/ 0.00080, 0.00140, 0.00210, 0.00160, 0.00570, 0.01070, 0.00820 /), &
      (/  0.0059,  0.0116,  0.0180,  0.0224,  0.0557,  0.0979,  0.0758 /), &
      (/  100.38,   99.87,   99.27,   98.88,   95.58,    90.7,   93.17 /))
    CALL CalcH("C4H12N2", "6291-84-5", (/ 40., 60., 80., 100. /), (/ H40, H60, H80, H100 /))

  CONTAINS

    REAL FUNCTION Calc_H_at_temp(y1, x1, P)
      IMPLICIT NONE
      REAL, DIMENSION(:), INTENT(IN) :: y1, x1, P
      REAL :: dummy, dpdx
      CALL LinReg (x1, y1*P, dummy, dpdx)
      Calc_H_at_temp = cH2O/(1E3*dpdx)
    END FUNCTION Calc_H_at_temp

    SUBROUTINE CalcH (chem_, casrn_, temp, Harray)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, Harray
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp+CtoK, Harray, Hominus, mindHR)
      CALL MakeNote(TRIM(ref), &
        "Calculated from the slope of $y_1P$ vs $x_1$, using the tabulated VLE data from " &
        //TRIM(citet())//" between 40\,\unit{\degree C} and 100\,\unit{\degree C}. " // &
        "Only dilute solutions with $x_1\leq 0.1$ were considered.")
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3654

  !---------------------------------------------------------------------------

  SUBROUTINE ref3655 ! Hvpc [Pa*m3/mol]
    IMPLICIT NONE

    ref = "3655"
    type = "Q"
    ! Tab. 3:
    CALL CalcH("naphthalene",                     "91-20-3", 36.84)
    CALL CalcH("1-methylnaphthalene",             "90-12-0", 43.88)
    CALL CalcH("2-methylnaphthalene",             "91-57-6", 44.51)
    CALL CalcH("1-ethylnaphthalene",            "1127-76-0", 51.98)
    CALL CalcH("2-ethylnaphthalene",             "939-27-5", 52.94)
    CALL CalcH("1,2-dimethylnaphthalene",        "573-98-8", 23.59)
    CALL CalcH("1,3-dimethylnaphthalene",        "575-41-7", 49.81)
    CALL CalcH("1,6-dimethylnaphthalene",        "575-43-9", 26.56)
    CALL CalcH("1,4-dimethylnaphthalene",        "571-58-4", 50.61)
    CALL CalcH("1,8-dimethylnaphthalene",        "569-41-5", 38.34)
    CALL CalcH("1,5-dimethylnaphthalene",        "571-61-9", 47.90)
    CALL CalcH("2,7-dimethylnaphthalene",        "582-16-1", 47.40)
    CALL CalcH("2,3-dimethylnaphthalene",        "581-40-8", 40.67)
    CALL CalcH("2,6-dimethylnaphthalene",        "581-42-0", 51.04)
    CALL CalcH("1,4,5-trimethylnaphthalene",    "2131-41-1", 43.85)
    CALL CalcH("1-isopropylnaphthalene",        "6158-45-8", 30.25)
    CALL CalcH("2-isopropylnaphthalene",        "2027-17-0", 32.69)
    CALL CalcH("2,6-diisopropylnaphthalene",   "24157-81-1", 75.86)
    CALL CalcH("1,3-diisopropylnaphthalene",   "57122-16-4", 62.97)
    CALL CalcH("1,5-diisopropylnaphthalene",   "27351-96-8", 63.58)
    CALL CalcH("2,7-diisopropylnaphthalene",   "40458-98-8", 74.73)
    ! I cannot find casrn or inchikey for 1,3,6-triisopropylnaphthalene,
    ! maybe this should be 1,3,5-triisopropylnaphthalene???
    ! 1,3,6-triisopropylnaphthalene                         (97.93)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, H)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL, INTENT(IN)             :: H
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL MakeNote(TRIM(ref)//"-COSMO", "Calculated using the COSMO-RS method.")
      CALL Output(1./H)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3655

  !---------------------------------------------------------------------------

  SUBROUTINE ref3657 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3657"
    type = "M"
    ! Tab. 2:
    CALL CalcH("methanol",              "67-56-1", 1.94E-4)
    CALL CalcH("ethanol",               "64-17-5", 2.14E-4)
    CALL CalcH("1-propanol",            "71-23-8", 2.76E-4)
    CALL CalcH("2-propanol",            "67-63-0", 3.23E-4)
    CALL CalcH("1-butanol",             "71-36-3", 3.84E-4)
    CALL CalcH("2-butanol",             "78-92-2", 4.87E-4)
    CALL CalcH("2-methyl-1-propanol",   "78-83-1", 5.17E-4)
    CALL CalcH("2-methyl-2-propanol",   "75-65-0", 5.00E-4)
    CALL CalcH("1-pentanol",            "71-41-0", 5.24E-4)
    CALL CalcH("2-pentanol",          "6032-29-7", 6.12E-4)
    CALL CalcH("2-methyl-2-butanol",    "75-85-4", 5.80E-4)
    CALL CalcH("1-hexanol",            "111-27-3", 6.90E-4)
    CALL CalcH("2-hexanol",            "626-93-7", 8.47E-4)
    CALL CalcH("3-methyl-3-pentanol",   "77-74-7", 8.42E-4)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, Kaw)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL, INTENT(IN)             :: Kaw
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL Output(Hvcc_TIMES_HscpSI_atT0 / Kaw)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3657

  !---------------------------------------------------------------------------

  SUBROUTINE ref3658 ! Hscp [M/atm]
    IMPLICIT NONE
    ref = "3658"
    type = "M"
    chem = "tert-butyl chloride" ; casrn = "507-20-0" ! C4H9Cl
    ! Tab. II:
    CALL HTdep( (/ 0.99, 5.00, 14.85 /) +CtoK, &
      (/ 8.18, 6.62, 3.11 /) * Hscp_TO_HscpSI / 100., Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3658

  !---------------------------------------------------------------------------

  SUBROUTINE ref3659 ! Hvcc [1]
    IMPLICIT NONE
    ! abstract:
    REAL, PARAMETER :: a = 0.04145
    REAL, PARAMETER :: b = 0.06583

    ref = "3659"
    type = "L"
    chem = "ethanol" ; casrn = "64-17-5" ! C2H5OH
    ! ref3659 uses a very different temperature dependence:
    Hominus = Hvcc_TO_HscpSI(1E-3 * a * EXP(b*(T0-CtoK)), T0)
    ! analytical derivative:
    mindHR = b*T0**2 + T0 ! see ref958, eqn (34) why T0 is added
    CALL MakeNote("moreTdep")
    CALL Output(Hominus, mindHR)

  END SUBROUTINE ref3659

  !---------------------------------------------------------------------------

  SUBROUTINE ref3660 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3660"
    type = "M"
    ! Tab. I:
    CALL CalcH("ethanol", "64-17-5", &
      (/ 1.,    5.,    10.,   15.,   20.,   25.,   30.,   35.,   37.,   40.   /) + CtoK, &
      (/ 0.035, 0.046, 0.073, 0.107, 0.155, 0.217, 0.310, 0.418, 0.470, 0.562 /) * 1E-3)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, kaw)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, kaw
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp, Hvcc_TO_HscpSI(kaw, temp), Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3660

  !---------------------------------------------------------------------------

  SUBROUTINE ref3661 ! Hscc [1]
    IMPLICIT NONE
    ! Tab. I:
    REAL, PARAMETER :: a = 10.044
    REAL, PARAMETER :: b = 0.0643

    ref = "3661"
    type = "M"
    chem = "ethanol" ; casrn = "64-17-5" ! C2H5OH
    ! ref3661 uses a very different temperature dependence:
    Hominus = Hscc_TO_HscpSI(EXP(a-b*(T0-CtoK)), T0)
    ! analytical derivative:
    mindHR = b*T0**2 + T0 ! see ref958, eqn (34) why T0 is added
    CALL MakeNote("moreTdep")
    CALL Output(Hominus, mindHR)

  END SUBROUTINE ref3661

  !---------------------------------------------------------------------------

  SUBROUTINE ref3662 ! Hsxp [1/atm]
    IMPLICIT NONE
    ! recommendation from p. 1:
    REAL, PARAMETER :: A1 = -32.5247
    REAL, PARAMETER :: A2 = 0.96017
    REAL, PARAMETER :: A3 = 68.0319
    REAL, PARAMETER :: A4 = 12.1522

    ref = "3662"
    type = "E"
    chem = "carbon dioxide" ; casrn = "124-38-9" ! CO2
    ! T-dep with 4 parameters:
    ! ln(H) = A1 + A2*(T/100) + A3/(T/100) + A4*ln(T/100)
    Hominus = Hsxp_TO_HscpSI * EXP(A1 + A2*(T0/100.) + A3/(T0/100.) + A4*LOG(T0/100.))
    ! analytical derivative:
    ! d ln(H) / d (1/T) = A2 + 100*A3 - A4*T
    mindHR = -A2*T0**2/100. + 100.*A3 - A4*T0
    CALL MakeNote("moreTdep")
    CALL MakeNote(TRIM(ref), "As mentioned by \citet{1943}, the fitting equation by " &
      // TRIM(citet()) // " is erroneous. It appears that a correction " // &
      "factor of about $10^6$ is necessary for consistency with their own data in Table~1.")
    CALL Output(DUMMY)

  END SUBROUTINE ref3662

  !---------------------------------------------------------------------------

  ! ref3663 NOTUSED: Review of solubilities in blood but not in water.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3664 ! Hscc [1] = Ostwald coefficient
    IMPLICIT NONE

    ref = "3664"
    type = "M"
    ! Tab. 1:
    CALL CalcH("1,3,5-trimethylbenzene", "108-67-8", 1.23)
    CALL CalcH("1,2,4-trimethylbenzene",  "95-63-6", 1.61)
    CALL CalcH("1,2,3-trimethylbenzene", "526-73-8", 2.73)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, lambda)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: lambda
      chem=chem_ ; casrn=casrn_ ! make global for Output
      Hominus = Hscc_TO_HscpSI(lambda, 37.+CtoK)
      CALL MakeNoteOtherTempC(37)
      CALL Output(Hominus)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3664

  !---------------------------------------------------------------------------

  SUBROUTINE ref3665 ! Hvpx [bar]
    IMPLICIT NONE

    ref = "3665"
    type = "Q"
    ! Tab. 3:
    CALL CalcH("naphthalene",               "91-20-3", 36.969)
    CALL CalcH("biphenyl",                  "92-52-4", 12.355)
    CALL CalcH("m-terphenyl",               "92-06-8", 0.479)
    CALL CalcH("fluoranthene",             "206-44-0", 1.212)
    CALL CalcH("acenaphthene",              "83-32-9", 18.785)
    CALL CalcH("9,10-dihydroanthracene",   "613-31-0", 3.695)
    CALL CalcH("9,10-dihydrophenanthrene", "776-35-2", 6.163)
    CALL CalcH("fluorene",                  "86-73-7", 6.141)
    CALL CalcH("anthracene",               "120-12-7", 3.779)
    CALL CalcH("pyrene",                   "129-00-0", 1.551)
    CALL CalcH("chrysene",                 "218-01-9", 0.278)
    CALL CalcH("phenanthrene",              "85-01-8", 3.231)
    CALL CalcH("benz[a]anthracene",         "56-55-3", 0.342)
    CALL CalcH("triphenylene",             "217-59-4", 0.064)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, kH2)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL, INTENT(IN)             :: kH2
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL MakeNote(TRIM(ref)//"-COSMO", "Calculated using the COSMO-RS method.")
      CALL Output(cH2O / (kH2 * bar))
    END SUBROUTINE CalcH

  END SUBROUTINE ref3665

  !---------------------------------------------------------------------------

  SUBROUTINE ref3666 ! Hvpc [Pa*m3/mol]
    IMPLICIT NONE

    ref = "3666"
    type = "Q"
    ! Tab. 1:
    CALL CalcH("hexafluorobenzene", "392-56-3", "M", 3465.)
    CALL CalcH("hexafluorobenzene", "392-56-3", "Q", 8798.)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, type_, H)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      CHARACTER,        INTENT(IN) :: type_
      REAL, INTENT(IN)             :: H
      chem=chem_ ; casrn=casrn_ ! make global for Output
      type = type_
      IF (type_ == "Q") CALL MakeNote(TRIM(ref)//"-COSMO", &
        "Calculated using the COSMO-RS method.")
      CALL Output(1./H)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3666

  !---------------------------------------------------------------------------

  SUBROUTINE ref3667 ! Hvpx [mmHg]
    IMPLICIT NONE
    REAL :: H289, H293, H298, H303

    ref = "3667"
    type = "M"
    ! see util/ref3667/plot.py to check if linear regression is justified:

    H289 = Calc_H_at_temp( &
      (/ 0.884, 0.887,  1.79,  3.54,  7.26,  14.7,  18.4,  24.4 /), &
      (/  1.16,  1.16,  2.31,  4.59,  9.13,  18.1,  22.6,  29.7 /), &
      (/ 759.9, 760.0, 759.9, 760.1, 760.0, 759.9, 760.2, 760.2 /))
    H293 = Calc_H_at_temp( &
      (/ 0.697,  1.36,  2.71,  2.70,  11.3,  18.6, 0.691,  1.40,  1.42,  5.75,  5.63, &
          11.5,  11.6,  11.8,  19.8, 0.715, 0.729,  2.89,  2.87,  2.85,  5.89,  20.5, &
         0.746, 0.747,  1.48,  2.93,  6.09,  6.09,  12.0,  12.3,  21.2, 0.763,  1.49, &
          1.55,  3.06,  3.03,  6.18,  12.6,  20.2, 0.763,  3.08,  6.32,  12.8,  12.9, &
          20.8, 0.783,  1.52,  1.53,  3.17,  3.14,  6.34,  13.2,  21.3 /), &
      (/  1.16,  2.31,  4.63,  4.60,  18.0,  29.4,  1.16,  2.30,  2.27,  9.08,  9.08, &
          18.2,  18.2,  18.1,  30.3,  1.15,  1.16,  4.57,  4.58,  4.58,  9.15,  30.5, &
          1.17,  1.17,  2.33,  4.58,  9.23,  9.06,  18.2,  18.1,  30.4,  1.16,  2.30, &
          2.32,  4.57,  4.58,  9.11,  18.1,  28.7,  1.15,  4.63,  9.11,  18.1,  18.1, &
          28.6,  1.16,  2.25,  2.24,  4.59,  4.59,  9.11,  18.0,  28.6 /), &
      (/ 740.0, 740.1, 738.0, 740.1, 739.4, 739.9, 760.0, 760.0, 767.2, 759.4, 759.8, &
         750.3, 750.3, 769.0, 759.9, 773.5, 775.4, 774.0, 774.9, 780.0, 775.9, 779.8, &
         799.3, 801.2, 790.4, 800.0, 794.5, 807.5, 739.5, 808.8, 799.7, 820.2, 812.9, &
         829.9, 818.8, 818.9, 819.9, 829.2, 819.1, 836.5, 839.5, 840.1, 840.8, 841.0, &
         840.0, 860.0, 850.8, 850.9, 856.1, 856.3, 859.9, 859.9, 859.9 /))
    H298 = Calc_H_at_temp( &
      (/ 0.513,  1.03,  2.02,  4.07,  8.40,  10.4,  13.9, 0.529,  1.05,  2.08,  4.19, &
          8.69,  10.8,  14.2, 0.535,  1.09,  2.14,  4.39,  8.93,  11.1,  14.7, 0.554, &
          1.11,  2.17,  4.45,  9.11,  11.3,  15.0, 0.567,  1.14,  1.14,  2.25,  4.57, &
          9.44,  11.7,  15.4, 0.577,  1.16,  1.17,  2.31,  4.66,  9.53,  12.0,  16.1, &
         0.592,  1.19,  1.19,  2.34,  4.88,  9.81,  12.3,  16.3 /), &
      (/  1.15,  2.32,  4.55,  9.03,  18.0,  22.1,  29.2,  1.15,  2.33,  4.56,  9.05, &
          18.0,  22.3,  29.2,  1.15,  2.32,  4.56,  9.08,  18.0,  22.3,  29.1,  1.15, &
          2.31,  4.56,  9.06,  18.1,  22.3,  29.3,  1.15,  2.30,  2.31,  4.56,  9.13, &
          18.1,  22.4,  29.3,  1.15,  2.30,  2.30,  4.57,  9.13,  18.0,  22.4,  29.2, &
          1.15,  2.29,  2.31,  4.55,  9.07,  18.0,  22.4,  29.2 /), &
      (/ 740.0, 739.9, 739.9, 739.9, 739.8, 740.0, 740.0, 760.0, 759.9, 760.1, 760.0, &
         760.1, 759.9, 760.1, 779.9, 779.7, 780.2, 780.0, 780.1, 780.0, 779.9, 799.9, &
         800.0, 800.0, 800.1, 800.1, 800.0, 800.0, 820.0, 820.1, 820.1, 819.8, 820.1, &
         820.1, 820.9, 820.2, 840.0, 840.1, 840.1, 840.2, 840.0, 840.1, 840.1, 839.9, &
         859.9, 859.9, 860.0, 860.0, 859.8, 860.2, 860.1, 860.1 /))
    H303 = Calc_H_at_temp( &
      (/ 0.399, 0.399, 0.797,  1.59,  3.18,  6.31,  7.97,  10.4,  10.7 /), &
      (/  1.14,  1.14,  2.27,  4.52,  8.99,  17.7,  22.2,  28.9,  29.0 /), &
      (/ 760.0, 760.1, 760.2, 759.9, 760.2, 760.0, 760.1, 760.0, 760.3 /))
    CALL CalcH("acetone", "67-64-1", (/ 289., 293., 298., 303. /), (/ H289, H293, H298, H303 /))

  CONTAINS

    REAL FUNCTION Calc_H_at_temp(xac, yac, P)
      IMPLICIT NONE
      REAL, DIMENSION(:), INTENT(IN) :: xac, yac, P
      REAL :: dummy, dpdx
      CALL LinReg (xac, yac*P, dummy, dpdx)
      Calc_H_at_temp = cH2O/(mmHg*dpdx)
    END FUNCTION Calc_H_at_temp

    SUBROUTINE CalcH (chem_, casrn_, temp, Harray)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, Harray
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp, Harray, Hominus, mindHR)
      CALL MakeNote(TRIM(ref), "Calculated from the slope of $y_\mathrm{ac}P$ vs " // &
        "$x_\mathrm{ac}$, using data from Table~VIII in " // TRIM(citet()) // ".")
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3667

  !---------------------------------------------------------------------------

  SUBROUTINE ref3668 ! special definition
    IMPLICIT NONE

    ref = "3668"
    type = "M"
    ! Tab. 2:
    ndata = 2
    ALLOCATE(temp(ndata))
    temp = (/ 20., 40. /) + CtoK
    CALL CalcH("hydrogen sulfide", "7783-06-4", (/ 220., 320. /), MH*2.+MS)
    CALL CalcH("methanethiol",       "74-93-1", (/  77., 140. /), MC+MH*4.+MS)
    CALL CalcH("dimethyl sulfide",   "75-18-3", (/  24.,  43. /), MC*2.+MH*6.+MS)
    CALL CalcH("ammonia",          "7664-41-7", (/ 0.04, 0.17 /), MN+MH*3.)
    CALL CalcH("trimethylamine",     "75-50-3", (/ 0.14, 0.35 /), MC*3.+MH*9.+MN)
    DEALLOCATE(temp)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, K, M)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: K
      REAL,               INTENT(IN) :: M
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp, 1./(1E-3*K*atm*M), Hominus, mindHR)
      CALL MakeNote(TRIM(ref), TRIM(citet())// &
        " provide effective Henry's law constants at several pH values. " // &
        "Here, only the value at pH = 5.8 is shown for the (acidic) S compounds " // &
        "and the value at pH = 8.6 for the alkaline N compounds.")
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3668

  !---------------------------------------------------------------------------

  SUBROUTINE ref3669 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3669"
    type = "M"
    ! Tab. 1:
    CALL CalcH("propanethiol",            "107-03-9",  4.48E-1) ! assuming this is 1-propanethiol
    CALL CalcH("2,5-dimethylthiophene",   "638-02-8",  1.08E-1)
    CALL CalcH("methyl propyl sulfide",  "3877-15-4",  9.37E-2)
    CALL CalcH("dimethyl disulfide",      "624-92-0",  4.69E-2)
    CALL CalcH("dipropyl disulfide",      "629-19-6",  2.25E-2)
    CALL CalcH("allyl methyl sulfide",  "10152-76-8",  8.05E-2)
    CALL CalcH("diallyl sulfide",         "592-88-1",  5.66E-2)
    CALL CalcH("diallyl disulfide",      "2179-57-9",  5.38E-2)
    CALL CalcH("ethanal",                  "75-07-0",  2.48E-3)
    CALL CalcH("propanal",                "123-38-6",  3.43E-3)
    CALL CalcH("methanol",                 "67-56-1",  3.02E-5)
    CALL CalcH("ethanol",                  "64-17-5",  6.52E-5)
    CALL CalcH("1-propanol",               "71-23-8", 10.81E-5)
    CALL CalcH("2-propanol",               "67-63-0", 19.61E-5)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, k)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL, INTENT(IN)             :: k
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL Output(Hvcc_TIMES_HscpSI_atT0 / k)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3669

  !---------------------------------------------------------------------------

  SUBROUTINE ref3670 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3670"
    type = "M"
    ! Tab. 1:
    CALL CalcH("dimethyl sulfide",     "75-18-3", 53.97)
    CALL CalcH("1-propanol",           "71-23-8",  1.23)
    CALL CalcH("2,3-butanedione",     "431-03-8",  1.91)
    CALL CalcH("butanone",             "78-93-3",  3.76)
    CALL CalcH("ethyl ethanoate",     "141-78-6",  9.42)
    CALL CalcH("1-butanol",            "71-36-3",  0.89)
    CALL CalcH("2-pentanol",         "6032-29-7",  1.48)
    CALL CalcH("propyl ethanoate",    "109-60-4", 12.93)
    CALL CalcH("3-methyl-1-butanol",  "123-51-3",  1.21)
    CALL CalcH("ethyl butanoate",     "105-54-4", 18.87)
    CALL CalcH("hexanal",              "66-25-1", 17.67)
    CALL CalcH("butyl ethanoate",     "123-86-4", 16.51)
    CALL CalcH("1-hexanol",           "111-27-3",  0.53)
    CALL CalcH("2-heptanone",         "110-43-0",  9.38)
    CALL CalcH("heptanal",            "111-71-7", 20.94)
    CALL CalcH("alpha-pinene",         "80-56-8",  5.23)
    CALL CalcH("2-octanone",          "111-13-7", 12.36)
    CALL CalcH("octanal",             "124-13-0", 20.34)
    CALL CalcH("2-nonanol",           "628-99-9",  0.79)
    CALL CalcH("2-decanone",          "693-54-9", 25.74)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, k)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: k
      REAL :: temp
      chem=chem_ ; casrn=casrn_ ! make global for Output
      temp = 37.+CtoK
      CALL MakeNoteOtherTempK(temp)
      CALL MakeNote(TRIM(ref), "Effective Henry's law constants at several pH values are provided by " &
        // TRIM(citet()) // ". Here, only the value at pH = 3 is shown.")
      CALL Output(Hvcc_TO_HscpSI(1E-3*k,temp))
    END SUBROUTINE CalcH

  END SUBROUTINE ref3670

  !---------------------------------------------------------------------------

  SUBROUTINE ref3671 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3671"
    type = "M"
    ! Tab. 2:
    CALL CalcH("dimethyl disulfide",      "624-92-0", 51.4E-3)
    CALL CalcH("dimethyl trisulfide",    "3658-80-8",   28E-3)
    CALL CalcH("S-methyl butanethioate", "2432-51-1", 15.4E-3)
    CALL CalcH("allyl isothiocyanate",     "57-06-7",   19E-3)
    CALL CalcH("hexanal",                  "66-25-1",  8.6E-3)
    CALL CalcH("heptanal",                "111-71-7", 11.4E-3)
    CALL CalcH("benzaldehyde",            "100-52-7",  1.2E-3)
    CALL CalcH("ethyl butyrate",          "105-54-4", 15.8E-3)
    CALL CalcH("hexyl ethanoate",         "142-92-7", 29.5E-3)
    CALL CalcH("hexanol",                 "111-27-3", 1.06E-3) ! assuming 1-hexanol

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, k)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL, INTENT(IN)             :: k
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL Output(Hvcc_TIMES_HscpSI_atT0 / k)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3671

  !---------------------------------------------------------------------------

  SUBROUTINE ref3672 ! Hscc [1]
    IMPLICIT NONE

    ref = "3672"
    type = "M"
    ! Tab. 1:
    CALL CalcH("dichloromethane",      "75-09-2", 8.77)
    CALL CalcH("dimethyl disulfide",  "624-92-0", 9.23)
    CALL CalcH("methylbenzene",       "108-88-3", 2.20)
    CALL CalcH("1,2-dimethylbenzene",  "95-47-6", 2.48)
    CALL CalcH("1,3-dimethylbenzene", "108-38-3", 1.70)
    CALL CalcH("1,4-dimethylbenzene", "106-42-3", 1.74)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, K)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: K
      chem=chem_ ; casrn=casrn_ ! make global for Output
      Hominus = Hscc_TO_HscpSI(K, 37.+CtoK)
      CALL MakeNoteOtherTempC(37)
      CALL Output(Hominus)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3672

  !---------------------------------------------------------------------------

  SUBROUTINE ref3673 ! Hvpc [bar/M]
    IMPLICIT NONE

    ref = "3673"
    type = "L"
    ! Delta hyd G is value used in the fit from Tab. 1,
    ! Delta hyd H is value used in the fit from Tab. 2:
    CALL CalcH("methane",                                 "74-82-8", "VNWKTOKETHGBQD-UHFFFAOYSA-N",  16.29, -13.1  )
    CALL CalcH("ethane",                                  "74-84-0", "OTMSDBZUPAUEDD-UHFFFAOYSA-N",  15.57, -19.4  )
    CALL CalcH("propane",                                 "74-98-6", "ATUOYWHBWRKTHZ-UHFFFAOYSA-N",  16.09, -22.9  )
    CALL CalcH("butane",                                 "106-97-8", "IJDNQMDRQITEOD-UHFFFAOYSA-N",  16.58, -25.9  )
    CALL CalcH("pentane",                                "109-66-0", "OFBQJSOFQDEBGM-UHFFFAOYSA-N",  17.48, -28.8  )
    CALL CalcH("hexane",                                 "110-54-3", "VLKZOEOYAKHREP-UHFFFAOYSA-N",  18.06, -31.6  )
    CALL CalcH("heptane",                                "142-82-5", "IMNFDUFMRHMDMM-UHFFFAOYSA-N",  18.90, -35.1  )
    CALL CalcH("octane",                                 "111-65-9", "TVMXDCGIABBOFY-UHFFFAOYSA-N",  19.42, -38.4  )
    CALL CalcH("nonane",                                 "111-84-2", "BKIMMITUMNQMOS-UHFFFAOYSA-N",  21.0          )
    CALL CalcH("decane",                                 "124-18-5", "DIOQZVSQGTUSAI-UHFFFAOYSA-N",  22.7          )
    CALL CalcH("undecane",                              "1120-21-4", "RSJKGSCJYJTIGS-UHFFFAOYSA-N",  24.8          )
    CALL CalcH("dodecane",                               "112-40-3", "SNRUBQQJIBEYMU-UHFFFAOYSA-N",  22.3          )
    CALL CalcH("tetradecane",                            "629-59-4", "BGHCVCJVXZWKCC-UHFFFAOYSA-N",  23.0          )
    CALL CalcH("2-methylpropane",                         "75-28-5", "NNPPMTNAJDCUHE-UHFFFAOYSA-N",  17.32, -24.2  )
    CALL CalcH("2-methylbutane",                          "78-78-4", "QWTDNUCVQCZILF-UHFFFAOYSA-N",  17.76, -25.1  )
    CALL CalcH("dimethylpropane",                        "463-82-1", "CRSOQBOWXPBRES-UHFFFAOYSA-N",  18.40, -25.1  )
    CALL CalcH("2-methylpentane",                        "107-83-5", "AFABGHUZZDYHJO-UHFFFAOYSA-N",  18.52, -32.6  )
    CALL CalcH("3-methylpentane",                         "96-14-0", "PFEOZHBOMNWTJB-UHFFFAOYSA-N",  18.42         )
    CALL CalcH("2,2-dimethylbutane",                      "75-83-2", "HNRMPXKDFBEGFZ-UHFFFAOYSA-N",  18.57, -29.2  )
    CALL CalcH("2,3-dimethylbutane",                      "79-29-8", "ZFFMLCVRJBZUDZ-UHFFFAOYSA-N",  17.82         )
    CALL CalcH("2-methylhexane",                         "591-76-4", "GXDHCNNESPLIKD-UHFFFAOYSA-N",  20.2          )
    CALL CalcH("3-methylhexane",                         "589-34-4", "VLJXXKKOSFGPHI-UHFFFAOYSA-N",  19.1          )
    CALL CalcH("2,2-dimethylpentane",                    "590-35-2", "CXOWYJMDMMMMJO-UHFFFAOYSA-N",  20.0          )
    CALL CalcH("2,3-dimethylpentane",                    "565-59-3", "WGECXQBGLLYSFP-UHFFFAOYSA-N",  18.5          )
    CALL CalcH("2,4-dimethylpentane",                    "108-08-7", "BZHMBWZPUJHVEE-UHFFFAOYSA-N",  19.84         )
    CALL CalcH("3,3-dimethylpentane",                    "562-49-2", "AEXMKKGTQYQZCS-UHFFFAOYSA-N",  18.7          )
    CALL CalcH("3-methylheptane",                        "589-81-1", "LAIUFBWHERIJIH-UHFFFAOYSA-N",  20.4          )
    CALL CalcH("2,2,4-trimethylpentane",                 "540-84-1", "NHTMVDHEPJAVLT-UHFFFAOYSA-N",  20.13         )
    CALL CalcH("2,3,4-trimethylpentane",                 "565-75-3", "RLPGDEORIPLBNF-UHFFFAOYSA-N",  19.2          )
    CALL CalcH("4-methyloctane",                        "2216-34-4", "DOGIHOCMZJUJNR-UHFFFAOYSA-N",  22.9          )
    CALL CalcH("2,2,5-trimethylhexane",                 "3522-94-9", "HHOSMYBYIHNXNO-UHFFFAOYSA-N",  20.3          )
    CALL CalcH("ethene",                                  "74-85-1", "VGGSQFUCUMXWEO-UHFFFAOYSA-N",  13.25, -16.5  )
    CALL CalcH("propene",                                "115-07-1", "QQONPFPTGQHPMA-UHFFFAOYSA-N",  12.87, -21.6  )
    CALL CalcH("1-butene",                               "106-98-9", "VXNZUUAINFGPBY-UHFFFAOYSA-N",  13.57, -24.9  )
    CALL CalcH("2-methylpropene",                        "115-11-7", "VQTUBCCKSQIDNK-UHFFFAOYSA-N",  12.80, -24.7  )
    CALL CalcH("1-pentene",                              "109-67-1", "YWAKXRMUMFPDSH-UHFFFAOYSA-N",  14.96         )
    CALL CalcH("2-pentene",                              "109-68-2", "QMMOXUPEWRXHJS-UHFFFAOYSA-N",  13.62         )
    CALL CalcH("2-methyl-2-butene",                      "513-35-9", "BKOOMYPCSUNDGP-UHFFFAOYSA-N",  12.4,  -29.6  )
    CALL CalcH("3-methyl-1-butene",                      "563-45-1", "YHQXBTXEYZIYOV-UHFFFAOYSA-N",  15.54         )
    CALL CalcH("1-hexene",                               "592-41-6", "LIKMAJRDDDTEIG-UHFFFAOYSA-N",  15.07, -32.2  )
    CALL CalcH("2-methyl-1-pentene",                     "763-29-1", "WWUVJRULCWHUSA-UHFFFAOYSA-N",  14.20         )
    CALL CalcH("4-methyl-1-pentene",                     "691-37-2", "WSSSPWUEQFSQQG-UHFFFAOYSA-N",  16.05         )
    CALL CalcH("1-heptene",                              "592-76-7", "ZGEGCLOFRBLKSE-UHFFFAOYSA-N",  14.9          )
    CALL CalcH("2-heptene",                              "592-77-8", "OTTZHAVKAVGASB-UHFFFAOYSA-N",  15.03         )
    CALL CalcH("1-octene",                               "111-66-0", "KWKAKUADMBZCLK-UHFFFAOYSA-N",  16.9,  -36.9  )
    CALL CalcH("1-nonene",                               "124-11-8", "JRZJOMJEPLMPRA-UHFFFAOYSA-N",  16.6          )
    CALL CalcH("1,3-butadiene",                          "106-99-0", "KAKZBPTYRLMSJV-UHFFFAOYSA-N",  10.61, -27.0  )
    CALL CalcH("1,4-pentadiene",                         "591-93-5", "QYZLKGVUSQXAMU-UHFFFAOYSA-N",  11.96         )
    CALL CalcH("2-methyl-1,3-butadiene",                  "78-79-5", "RRHGJUQNOFWUDK-UHFFFAOYSA-N",  10.76, -22.7  )
    CALL CalcH("1,5-hexadiene",                          "592-42-7", "PYGSKMBEVAICCR-UHFFFAOYSA-N",  12.36         )
    CALL CalcH("1,6-heptadiene",                        "3070-53-9", "GEAWFZNTIFJMHR-UHFFFAOYSA-N",  12.86         )
    CALL CalcH("ethyne",                                  "74-86-2", "HSFWRNGVRCDJHI-UHFFFAOYSA-N",   7.89, -14.6  )
    CALL CalcH("propyne",                                 "74-99-7", "MWWATHDPGQKSAR-UHFFFAOYSA-N",   6.43, -20.2  )
    CALL CalcH("1-butyne",                               "107-00-6", "KDKYADYSIPSCCQ-UHFFFAOYSA-N",   6.9,  -16.0  )
    CALL CalcH("1-pentyne",                              "627-19-0", "IBXNCJKFFQIKKY-UHFFFAOYSA-N",   8.5          )
    CALL CalcH("1-hexyne",                               "693-02-7", "CGHIBGNXEGJPQZ-UHFFFAOYSA-N",   8.7          )
    CALL CalcH("3-hexyne",                               "928-49-4", "DQQNMIPXXNPGCV-UHFFFAOYSA-N",   7.2          )
    CALL CalcH("1-heptyne",                              "628-71-7", "YVXHZKKCZYLQOP-UHFFFAOYSA-N",  10.0          )
    CALL CalcH("2-heptyne",                             "1119-65-9", "AMSFEMSYKQQCHL-UHFFFAOYSA-N",   7.5          )
    CALL CalcH("2-methyl-3-hexyne",                    "36566-80-0", "POBOUPFSQKXZFZ-UHFFFAOYSA-N",   9.5          )
    CALL CalcH("1-octyne",                               "629-05-0", "UMIPWJGWASORKV-UHFFFAOYSA-N",  10.9          )
    CALL CalcH("2,2-dimethyl-3-hexyne",                 "4911-60-8", "XYBFBXTUWDPXLK-UHFFFAOYSA-N",  10.5          )
    CALL CalcH("1-nonyne",                              "3452-09-3", "OSSQSXOTMIGBCF-UHFFFAOYSA-N",  11.5          )
    CALL CalcH("2,2,5-trimethyl-3-hexyne",                       "", "DKFPMSNUJIRAMR-UHFFFAOYSA-N",  12.9          )
    CALL CalcH("2,2,5,5-tetramethyl-3-hexyne",         "17530-24-4", "FXVDWKZNFZMSOU-UHFFFAOYSA-N",  14.1          )
    CALL CalcH("1,6-heptadiyne",                        "2396-63-6", "RSPZSDWVQWRAEF-UHFFFAOYSA-N",   1.9          )
    CALL CalcH("1,8-nonadiyne",                         "2396-65-8", "DMOVPHYFYSASTC-UHFFFAOYSA-N",   3.0          )
    CALL CalcH("cyclopropane",                            "75-19-4", "LVZWSLJZHVFIQJ-UHFFFAOYSA-N",  10.98, -23.3  )
    CALL CalcH("cyclopentane",                           "287-92-3", "RGSFGYAAUTVSQA-UHFFFAOYSA-N",  12.90, -27.0  )
    CALL CalcH("cyclohexane",                            "110-82-7", "XDTMQSROBMDMFD-UHFFFAOYSA-N",  12.99, -33.1  )
    CALL CalcH("methylcyclopentane",                      "96-37-7", "GDOPTJXRTPNYNR-UHFFFAOYSA-N",  14.59, -45.7  )
    CALL CalcH("cycloheptane",                           "291-64-5", "DMEGYFMYUHOHGS-UHFFFAOYSA-N",  11.4,  -25.7  )
    CALL CalcH("methylcyclohexane",                      "108-87-2", "UAEPNZWRGJTJPN-UHFFFAOYSA-N",  14.97         )
    CALL CalcH("cyclooctane",                            "292-64-8", "WJTCGQSWYFHTAC-UHFFFAOYSA-N",  11.6,  -32.0  )
    CALL CalcH("{trans}-1,4-dimethylcyclohexane",       "2207-04-7", "QRMPKOFEUHIBNM-ZKCHVHJHSA-N",  16.8          )
    CALL CalcH("propylcyclopentane",                    "2040-96-2", "KDIAMAVWIJYWHN-UHFFFAOYSA-N",  16.9          )
    CALL CalcH("{cis}-1,2-dimethylcyclohexane",         "2207-01-4", "KVZJLSYJROEPSQ-OCAPTIKFSA-N",  14.6          )
    CALL CalcH("1,1,3-trimethylcyclopentane",           "4516-69-2", "OBKHYUIZSOIEPG-UHFFFAOYSA-N",  18.3          )
    CALL CalcH("1,1,3-trimethylcyclohexane",            "3073-66-3", "PYOLJOJPIPCRDP-UHFFFAOYSA-N",  17.3          )
    CALL CalcH("pentylcyclopentane",                    "3741-00-2", "HPQURZRDYMUHJI-UHFFFAOYSA-N",  18.7          )
    CALL CalcH("cyclopentene",                           "142-29-0", "LPIQUOYDBNQMRZ-UHFFFAOYSA-N",  10.3          )
    CALL CalcH("cyclohexene",                            "110-83-8", "HGCIXCUEYOPUTN-UHFFFAOYSA-N",   9.5          )
    CALL CalcH("cycloheptene",                           "628-92-2", "ZXIJMRYMVAMXQP-UHFFFAOYSA-N",   9.65         )
    CALL CalcH("1-methylcyclohexene",                    "591-49-1", "CTMHWPIWNRWQEG-UHFFFAOYSA-N",  10.65         )
    CALL CalcH("1,3-cyclopentadiene",                    "542-92-7", "ZSWFCLXCOIISFI-UHFFFAOYSA-N",  10.0          )
    CALL CalcH("1,4-cyclohexadiene",                     "628-41-1", "UVJHQYIOXKWHFD-UHFFFAOYSA-N",   5.7,  -31.8  )
    CALL CalcH("1,3,5-cycloheptatriene",                 "544-25-2", "CHVJITGCYZJHLR-UHFFFAOYSA-N",   3.7,  -32.5  )
    CALL CalcH("benzene",                                 "71-43-2", "UHOVQNZJYSORNB-UHFFFAOYSA-N",   4.29, -31.7  )
    CALL CalcH("methylbenzene",                          "108-88-3", "YXFVVABEGXRONW-UHFFFAOYSA-N",   4.56, -36.3  )
    CALL CalcH("ethylbenzene",                           "100-41-4", "YNQLUTRBYVCPMQ-UHFFFAOYSA-N",   5.09, -40.2  )
    CALL CalcH("1,2-dimethylbenzene",                     "95-47-6", "CTQNGGLPUBDAKN-UHFFFAOYSA-N",   4.02, -36.6  )
    CALL CalcH("1,3-dimethylbenzene",                    "108-38-3", "IVSZLXZYQVIEFR-UHFFFAOYSA-N",   4.89, -38.5  )
    CALL CalcH("1,4-dimethylbenzene",                    "106-42-3", "URLKBWYHVLBVBO-UHFFFAOYSA-N",   4.93, -38.6  )
    CALL CalcH("propylbenzene",                          "103-65-1", "ODLMAHJVESYWTB-UHFFFAOYSA-N",   5.93, -43.9  )
    CALL CalcH("1,2,3-trimethylbenzene",                 "526-73-8", "FYGHSUNMUKGBRK-UHFFFAOYSA-N",   3.34, -37.5  )
    CALL CalcH("1,2,4-trimethylbenzene",                  "95-63-6", "GWHJZXXIDMPWGX-UHFFFAOYSA-N",   4.53, -39.9  )
    CALL CalcH("1,3,5-trimethylbenzene",                 "108-67-8", "AUHZEENZYGFFBQ-UHFFFAOYSA-N",   5.09, -41.0  )
    CALL CalcH("(2-propyl)-benzene",                      "98-82-8", "RWGFKTVRMDUZSP-UHFFFAOYSA-N",   6.13, -39.6  )
    CALL CalcH("1-ethyl-2-methylbenzene",                "611-14-3", "HYFLWBNQFMXCPA-UHFFFAOYSA-N",   4.2          )
    CALL CalcH("butylbenzene",                           "104-51-8", "OCKPCBLVNKHBMX-UHFFFAOYSA-N",   6.56, -46.1  )
    CALL CalcH("(2-methylpropyl)-benzene",               "538-93-2", "KXUHSQYYJYAXGZ-UHFFFAOYSA-N",   8.8          )
    CALL CalcH("(1,1-dimethylethyl)-benzene",             "98-06-6", "YTZKOQUCBOVLHL-UHFFFAOYSA-N",   6.3          )
    CALL CalcH("(1-methylpropyl)-benzene",               "135-98-8", "ZJMWRROPUADPEA-UHFFFAOYSA-N",   7.3          )
    CALL CalcH("1,2,4,5-tetramethylbenzene",              "95-93-2", "SQNZJJAZBFDUTD-UHFFFAOYSA-N",   5.0          )
    CALL CalcH("1-methyl-2-(1-methylethyl)-benzene",     "527-84-4", "WWRCMNKATXZARA-UHFFFAOYSA-N",   4.5          )
    CALL CalcH("1-methyl-3-(1-methylethyl)-benzene",     "535-77-3", "XCYJPXQACVEIOS-UHFFFAOYSA-N",   4.9          )
    CALL CalcH("1-methyl-4-(1-methylethyl)-benzene",      "99-87-6", "HFPZCAJZSCWRBC-UHFFFAOYSA-N",   5.8          )
    CALL CalcH("pentylbenzene",                          "538-68-1", "PWATWSYOIIXYMA-UHFFFAOYSA-N",   7.4,  -48.8  )
    CALL CalcH("hexylbenzene",                          "1077-16-3", "LTEQMZWBSYACLV-UHFFFAOYSA-N",   8.0,  -52.4  )
    CALL CalcH("1,3,5-triethylbenzene",                  "102-25-0", "WJYMPXJVHNDZHD-UHFFFAOYSA-N",   5.7          )
    CALL CalcH("decylbenzene",                           "104-72-3", "UZILCZKGXMQEQR-UHFFFAOYSA-N",  11.8          )
    CALL CalcH("methanol",                                "67-56-1", "OKKJLVBELUTLKV-UHFFFAOYSA-N", -13.21, -45.13 )
    CALL CalcH("ethanol",                                 "64-17-5", "LFQSCWFLJHTTHZ-UHFFFAOYSA-N", -13.00, -52.59 )
    CALL CalcH("1-propanol",                              "71-23-8", "BDERNNFJNOPAEC-UHFFFAOYSA-N", -12.38, -57.65 )
    CALL CalcH("1-butanol",                               "71-36-3", "LRHPLDYGYMQRHN-UHFFFAOYSA-N", -11.88, -61.72 )
    CALL CalcH("1-pentanol",                              "71-41-0", "AMQJEAYHLZJPGS-UHFFFAOYSA-N", -11.13, -65.0  )
    CALL CalcH("1-hexanol",                              "111-27-3", "ZSIAUFGUXNUGDI-UHFFFAOYSA-N", -10.37, -68.1  )
    CALL CalcH("1-heptanol",                             "111-70-6", "BBMCTIGTTCKYKF-UHFFFAOYSA-N",  -9.91, -71.7  )
    CALL CalcH("1-octanol",                              "111-87-5", "KBPLFHHGFOOTCA-UHFFFAOYSA-N",  -9.23, -74.4  )
    CALL CalcH("2-propanol",                              "67-63-0", "KFZMGEQAYNKOFK-UHFFFAOYSA-N", -11.93, -58.5  )
    CALL CalcH("2-butanol",                               "78-92-2", "BTANRVKWQNVYAZ-UHFFFAOYSA-N", -11.38, -62.8  )
    CALL CalcH("2-methyl-1-propanol",                     "78-83-1", "ZXEKIIBDNHEJCQ-UHFFFAOYSA-N", -11.01, -60.2  ) ! not 2-methyl-2-propanol
    CALL CalcH("2-methyl-2-propanol",                     "75-65-0", "DKGAVHZHDRPRBM-UHFFFAOYSA-N", -10.83, -64.1  )
    CALL CalcH("2-pentanol",                            "6032-29-7", "JYVLIDXNZAXMDK-UHFFFAOYSA-N", -10.33, -66.8  )
    CALL CalcH("3-pentanol",                             "584-02-1", "AQIXEPGDORPWBJ-UHFFFAOYSA-N",  -9.95         )
    CALL CalcH("2-methyl-1-butanol",                     "137-32-6", "QPRQEDXDYOZYLA-UHFFFAOYSA-N", -10.56         )
    CALL CalcH("2-methyl-2-butanol",                      "75-85-4", "MSXVEPNJUHWQHW-UHFFFAOYSA-N", -10.56, -68.5  )
    CALL CalcH("3-methyl-1-butanol",                     "123-51-3", "PHTQWCKDNZKARW-UHFFFAOYSA-N", -10.54         )
    CALL CalcH("2-hexanol",                              "626-93-7", "QNVRIHYSUZMSGM-UHFFFAOYSA-N",  -9.80         )
    CALL CalcH("3-hexanol",                              "623-37-0", "ZOCHHNOQQHDWHG-UHFFFAOYSA-N",  -9.26, -69.6  )
    CALL CalcH("2-methyl-1-pentanol",                    "105-30-6", "PFNHSEQQEPMLNI-UHFFFAOYSA-N",  -9.49         )
    CALL CalcH("3-methyl-2-pentanol",                    "565-60-6", "ZXNBBWHRUSXUFZ-UHFFFAOYSA-N",  -9.7          )
    CALL CalcH("3-methyl-3-pentanol",                     "77-74-7", "FRDAATYAJDYRNW-UHFFFAOYSA-N",  -9.53         )
    CALL CalcH("4-methyl-2-pentanol",                    "108-11-2", "WVYWICLMDOOCFB-UHFFFAOYSA-N",  -8.8          )
    CALL CalcH("2-ethyl-1-butanol",                       "97-95-0", "TZYRSLHNPKPEFV-UHFFFAOYSA-N",  -9.9          )
    CALL CalcH("2,3-dimethyl-2-butanol",                 "594-60-5", "IKECULIHBUCAKR-UHFFFAOYSA-N",  -9.0          )
    CALL CalcH("2-heptanol",                             "543-49-7", "CETWDUZRCINIHU-UHFFFAOYSA-N",  -9.2          )
    CALL CalcH("3-heptanol",                             "589-82-2", "RZKSECIXORKHQS-UHFFFAOYSA-N",  -8.55         )
    CALL CalcH("4-heptanol",                             "589-55-9", "YVBCULSIZWMTFY-UHFFFAOYSA-N",  -8.75, -75.3  )
    CALL CalcH("3-ethyl-3-pentanol",                     "597-49-9", "XKIRHOWVQWCYBT-UHFFFAOYSA-N",  -8.4          )
    CALL CalcH("2-octanol",                              "123-96-6", "SJWFXCIHNDVPSH-UHFFFAOYSA-N",  -8.8          )
    CALL CalcH("3-octanol",                              "589-98-0", "NMRPBPVERJPACX-UHFFFAOYSA-N",  -8.6          )
    CALL CalcH("2-ethyl-1-hexanol",                      "104-76-7", "YIWUKEYIRIRTPP-UHFFFAOYSA-N",  -9.5          )
    CALL CalcH("2-propen-1-ol",                          "107-18-6", "XXROGKLTLUQVRX-UHFFFAOYSA-N", DUMMY,  -54.4  )
    CALL CalcH("2-buten-1-ol",                          "6117-91-5", "WCASXYBKJHWFMY-UHFFFAOYSA-N", DUMMY,  -57.3  )
    CALL CalcH("3-buten-1-ol",                           "627-27-0", "ZSPTYLOMNJNZNG-UHFFFAOYSA-N", DUMMY,  -58.2  )
    CALL CalcH("4-penten-1-ol",                          "821-09-0", "LQAVWYMTUMSFBE-UHFFFAOYSA-N", DUMMY,  -60.8  )
    CALL CalcH("3-pentyn-1-ol",                        "10229-10-4", "IDYNOORNKYEHHO-UHFFFAOYSA-N", DUMMY,  -64.5  )
    CALL CalcH("cyclopentanol",                           "96-41-3", "XCIXKGXIYUWCLL-UHFFFAOYSA-N", -15.0,  -67.8  )
    CALL CalcH("cyclohexanol",                           "108-93-0", "HPXRVTGHNJAIIH-UHFFFAOYSA-N", -15.09, -70.7  )
    CALL CalcH("cycloheptanol",                          "502-41-0", "QCRFMSUKWRQZEM-UHFFFAOYSA-N", -15.2,  -74.7  )
    CALL CalcH("terpinolene",                            "586-62-9", "MOYAFQVGZZPNRA-UHFFFAOYSA-N",   6.3,  -45.5  )
    CALL CalcH("sabinene",                              "3387-41-5", "NDVASEGYNIMXJL-UHFFFAOYSA-N",  10.4          )
    CALL CalcH("alpha-phellandrene",                      "99-83-2", "OGLDWXZKYODSOB-UHFFFAOYSA-N",   9.9          )
    CALL CalcH("alpha-terpinene",                         "99-86-5", "YHQGMYUVUMAZJR-UHFFFAOYSA-N",   7.4          )
    CALL CalcH("gamma-terpinene",                         "99-85-4", "YKFLAYDHMOASIY-UHFFFAOYSA-N",   7.1,  -35.5  )
    CALL CalcH("limonene",                               "138-86-3", "XMGQYMWWDOXHJM-UHFFFAOYSA-N",   7.6,  -39.0  )
    CALL CalcH("myrcene",                                "123-35-3", "UAHWPYUMFXYFJY-UHFFFAOYSA-N",   8.0          )
    CALL CalcH("alpha-pinene",                            "80-56-8", "GRWFGVWFFZKLTI-UHFFFAOYSA-N",  12.3,  -39.4  )
    CALL CalcH("beta-pinene",                            "127-91-3", "WTARULDDTDQWMU-UHFFFAOYSA-N",   9.6          )
    CALL CalcH("camphene",                                "79-92-5", "CRPUJAZIXJMDBK-UHFFFAOYSA-N",   9.3          )
    CALL CalcH("tricyclene",                             "508-32-7", "RRBYUSWBLVXTQN-UHFFFAOYSA-N",  13.2          )
    CALL CalcH("3-buten-1-yne",                          "689-97-4", "WFYPICNXBKQZGB-UHFFFAOYSA-N",   7.9,  -14.4  )
    CALL CalcH("4-ethenylcyclohexene",                   "100-40-3", "BBDKZWKEPDTENS-UHFFFAOYSA-N",   9.5          )
    CALL CalcH("ethenylbenzene",                         "100-42-5", "PPBRXRYQALVLMV-UHFFFAOYSA-N",   3.1,  -35.0  )
    CALL CalcH("indane",                                 "496-11-7", "PQNFLJBBNBOBRQ-UHFFFAOYSA-N",   2.1          )
    CALL CalcH("1,2,3,4-tetrahydronaphthalene",          "119-64-2", "CXWXQJXEFPUFDZ-UHFFFAOYSA-N",   1.1,  -44.0  )
    CALL CalcH("2-cyclohexen-1-ol",                      "822-67-3", "PQANGXXSEABURG-UHFFFAOYSA-N", DUMMY,  -70.9  )

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, DeltaG, DeltaH)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_, inchikey_
      REAL,               INTENT(IN) :: DeltaG
      REAL, OPTIONAL,     INTENT(IN) :: DeltaH
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      ! same conversion applied here as for ref3616:
      IF (ABS(DeltaG-DUMMY) > TINY(0.)) THEN
        Hominus =  1E3 / (bar * EXP(1E3*DeltaG/(Rgas*T0)))
      ELSE
        Hominus = DUMMY
      ENDIF
      IF (PRESENT(DeltaH)) THEN
        mindHR = -1E3*DeltaH/Rgas
        CALL Output(Hominus, mindHR)
      ELSE
        CALL Output(Hominus)
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3673

  !---------------------------------------------------------------------------

  SUBROUTINE ref3674 ! Hvpc [bar/M]
    IMPLICIT NONE

    ref = "3674"
    type = "T"
    ! Delta hyd G and Delta hyd H from Tab. 1:
    CALL CalcH("hexane",         "110-54-3", "VLKZOEOYAKHREP-UHFFFAOYSA-N",  18.12, -31.6  )
    CALL CalcH("cyclohexane",    "110-82-7", "XDTMQSROBMDMFD-UHFFFAOYSA-N",  12.95, -33.1  )
    CALL CalcH("benzene",         "71-43-2", "UHOVQNZJYSORNB-UHFFFAOYSA-N",   4.29, -31.7  )
    CALL CalcH("methylbenzene",  "108-88-3", "YXFVVABEGXRONW-UHFFFAOYSA-N",   4.55, -36.3  )
    CALL CalcH("1-propanamine",  "107-10-8", "WGYKZJWCGVVSQN-UHFFFAOYSA-N", -10.40, -56.0  )
    CALL CalcH("1-propanol",      "71-23-8", "BDERNNFJNOPAEC-UHFFFAOYSA-N", -12.38, -57.65 )
    CALL CalcH("ethanol",         "64-17-5", "LFQSCWFLJHTTHZ-UHFFFAOYSA-N", -13.00, -52.59 )
    CALL CalcH("methanol",        "67-56-1", "OKKJLVBELUTLKV-UHFFFAOYSA-N", -13.21, -45.13 )
    CALL CalcH("propanoic acid",  "79-09-4", "XBDQKXXYIPTUBI-UHFFFAOYSA-N", -20.35, -56.5  )
    CALL CalcH("ethanoic acid",   "64-19-7", "QTBSBXVTEAMEQO-UHFFFAOYSA-N", -21.00, -52.8  )
    CALL CalcH("propanamide",     "79-05-0", "QLNJFJADRCOGBJ-UHFFFAOYSA-N", -31.5,  -73.4  )

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, DeltaG, DeltaH)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_, inchikey_
      REAL,               INTENT(IN) :: DeltaG
      REAL, OPTIONAL,     INTENT(IN) :: DeltaH
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      ! same conversion applied here as for ref3616:
      Hominus = 1E3 / (bar * EXP(1E3*DeltaG/(Rgas*T0)))
      mindHR  = -1E3*DeltaH/Rgas
      CALL Output(Hominus, mindHR)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3674

  !---------------------------------------------------------------------------

  SUBROUTINE ref3675 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3675"
    type = "M"
    ! Tab. 3.5:
    CALL CalcH("acetonitrile",         "75-05-8", 4.90E-3)
    CALL CalcH("acetone",              "67-64-1", 2.36E-3)
    CALL CalcH("isopropanol",          "67-63-0", 4.13E-4)
    CALL CalcH("diethyl ether",        "60-29-7", 1.22E-1)
    CALL CalcH("dichloromethane",      "75-09-2", 9.12E-2)
    CALL CalcH("benzene",              "71-43-2", 2.11E-1)
    CALL CalcH("methylbenzene",       "108-88-3", 1.09E-1)
    CALL CalcH("1,2-dichlorobenzene",  "95-50-1", 1.15E-1)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, H)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL, INTENT(IN)             :: H
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL MakeNoteOtherTempC(20.0) ! mentioned on p. 18, point b
      CALL Output(Hvcc_TIMES_HscpSI_atT0 / H)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3675

  !---------------------------------------------------------------------------

  SUBROUTINE ref3676 ! Hscp [M/atm]
    IMPLICIT NONE

    ref = "3676"
    type = "Q"
    ! Tab. S6:
    CALL CalcH("D3",                   "HTDJPCNNEPUOOQ-UHFFFAOYSA-N", 6.45E-4, 1.61E-3, 2.54E-4)
    CALL CalcH("D3OH",                 "OGNYSZJXUXVWRE-UHFFFAOYSA-N", 1.26E1,  2.70,    1.51E-1)
    CALL CalcH("D3FormEst",            "TZIXLALHTGGYNZ-UHFFFAOYSA-N", 2.82E-2, 3.26E-2, 1.77E-1)
    CALL CalcH("D3OHFormEst",          "RMJPVGNUQFKGPC-UHFFFAOYSA-N", 5.52E2,  1.00E3,  1.24   )
    CALL CalcH("D3Hydroperoxide",      "JKDUHABXPCJOJB-UHFFFAOYSA-N", 6.99,    1.90E-1, 9.06E-4)
    CALL CalcH("D3Organonitrate",      "WGZJMUDCTAXYSF-UHFFFAOYSA-N", 2.40E-1, 1.66E-2, 1.01E-2)
    CALL CalcH("D3EtherHydroperoxide", "LHGAYMWHCKTFJV-UHFFFAOYSA-N", 8.47E1,  1.95,    1.73   )
    CALL CalcH("D3EtherOrganonitrate", "CJWFNCZCCFYEKP-UHFFFAOYSA-N", 2.92,    1.71E-1, 1.38E-1)
    CALL CalcH("D3FormEst2",           "WKTLUKWDJJZJCN-UHFFFAOYSA-N", 1.24,    9.34E-1, 2.05   )
    CALL CalcH("D3OH2O1",              "UGEQNRIYBONGCP-UHFFFAOYSA-N", 2.47E5,  2.91E2,  1.83E1 )
    CALL CalcH("D4",                   "HMMGMWAXVFQUOA-UHFFFAOYSA-N", 5.49E-5, 1.33E-4, 1.15E-4)
    CALL CalcH("D4OH",                 "MLGUQSFCJNMKPT-UHFFFAOYSA-N", 1.07,    1.09,    1.31E-3)
    CALL CalcH("D4FormEst",            "PWVQOGDRWKITJQ-UHFFFAOYSA-N", 2.40E-3, 8.60E-3, 8.88E-3)
    CALL CalcH("D4OHFormEst",          "AXBVNNKTZSENOJ-UHFFFAOYSA-N", 4.69E1,  3.13E2,  1.18E-2)
    CALL CalcH("D4Hydroperoxide",      "IEOAKTNTPADFJB-UHFFFAOYSA-N", 5.92E-1, 4.41E-2, 1.81E-4)
    CALL CalcH("D4Organonitrate",      "CYMCLGYPZZKOOI-UHFFFAOYSA-N", 2.04E-2, 4.14E-3, 7.65E-4)
    CALL CalcH("D4EtherHydroperoxide", "BKBHFCQPJIYNBR-UHFFFAOYSA-N", 7.19,    4.54E-1, 1.57E-3)
    CALL CalcH("D4EtherOrganonitrate", "XHFUWVKWVPBCDO-UHFFFAOYSA-N", 2.48E-1, 3.80E-2, 1.60E-3)
    CALL CalcH("D4FormEst2",           "WJOJQLFAIHHNFX-UHFFFAOYSA-N", 1.05E-1, 1.87E-1, 2.95E-2)
    CALL CalcH("D4OH2",                "ONAQLRRZBYFKLA-UHFFFAOYSA-N", 2.09E4,  1.06E2,  8.44E-3)
    CALL CalcH("D5",                   "XMSXQFUHVRWGNA-UHFFFAOYSA-N", 2.37E-5, 1.26E-4, 3.13E-4)
    CALL CalcH("D5OH",                 "DKTKKERLEMHMHS-UHFFFAOYSA-N", 4.63E-1, 2.07,    4.46E-3)
    CALL CalcH("D5FormEst",            "IGERVNZEGVJPED-UHFFFAOYSA-N", 1.04E-3, 1.26E-2, 6.47E-3)
    CALL CalcH("D5OHFormEst",          "KNIXZSAQQHGMJX-UHFFFAOYSA-N", 2.02E1,  5.65E2,  4.74E-3)
    CALL CalcH("D5Hydroperoxide",      "VGMLSGIAATVHBV-UHFFFAOYSA-N", 2.56E-1, 6.42E-2, 1.15E-4)
    CALL CalcH("D5Organonitrate",      "KWHVDACCYNMOAR-UHFFFAOYSA-N", 8.77E-3, 4.95E-3, 1.21E-3)
    CALL CalcH("D5EtherHydroperoxide", "GSAPDUDDTPAQSH-UHFFFAOYSA-N", 3.12,    6.93E-1, 1.14E-3)
    CALL CalcH("D5EtherOrganonitrate", "BQINUVFPZWRRBW-UHFFFAOYSA-N", 1.07E-1, 5.79E-2, 2.92E-3)
    CALL CalcH("D5FormEst2",           "QFORSIVEJZTNKQ-UHFFFAOYSA-N", 4.52E-2, 2.52E-1, 2.01E-2)
    CALL CalcH("D5OH2",                "AGIASGIZCMBNMT-UHFFFAOYSA-N", 9.01E3,  2.54E2,  1.30E-2)

  CONTAINS

    SUBROUTINE CalcH (chem_, inchikey_, HENRYWIN, HENRYWIN_V, TEST)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, inchikey_
      REAL,             INTENT(IN) :: HENRYWIN, HENRYWIN_V, TEST
      chem=chem_ ; inchikey=inchikey_ ! make global for Output
      ! HENRYWIN:
      CALL MakeNote(TRIM(ref)//"HENRYWIN", "Calculated using HENRYWIN 3.21.")
      CALL Output(Hscp_TO_HscpSI * HENRYWIN)
      ! HENRYWIN_V:
      CALL MakeNote(TRIM(ref)//"HENRYWIN_V", "Calculated using vapor " // &
        "pressures and water solubilities from HENRYWIN 3.21.")
      CALL Output(Hscp_TO_HscpSI * HENRYWIN_V)
      ! TEST:
      CALL MakeNote(TRIM(ref)//"TEST", "Calculated using vapor pressures " // &
        "and water solubilities from the EPA Toxicity Estimation Software Tool (TEST).")
      CALL Output(Hscp_TO_HscpSI * TEST)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3676

  !---------------------------------------------------------------------------

  SUBROUTINE ref3678 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3678"
    type = "V"
    ! Fig. 3:
    CALL CalcH("isoprene",          "78-79-5",  0.50)
    CALL CalcH("geraniol",         "106-24-1", -3.88)
    CALL CalcH("linalool",          "78-70-6", -3.00)
    CALL CalcH("dl-citronellol",   "106-22-9", -4.11)
    CALL CalcH("thymol",            "89-83-8", -3.56)
    CALL CalcH("eugenol",           "97-53-0", -4.23)
    CALL CalcH("carvacrol",        "499-75-2", -3.56)
    CALL CalcH("p-cymene",          "99-87-6", -0.45)
    CALL CalcH("toluene",          "108-88-3", -0.61)
    CALL CalcH("p-xylene",         "106-42-3", -0.49) ! (1330-20-7 is for isomer mix)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, logKAW)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL, INTENT(IN)             :: logKAW
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL MakeNote(TRIM(ref), &
        "Values for the Henry's law constants shown in Fig.~3 of " // &
        TRIM(citet()) // " were obtained from Sim{\~a}o Pinho (personal communication, 2022).")
      CALL Output(Hvcc_TIMES_HscpSI_atT0 / (10.**logKAW))

    END SUBROUTINE CalcH

  END SUBROUTINE ref3678

  !---------------------------------------------------------------------------

  SUBROUTINE ref3679 ! Hscp [nmol/(L*Pa)]
    IMPLICIT NONE

    ref = "3679"
    type = "V"
    ! vapor pressure from Tab. II, solubility from Tabs. IV, V:
    CALL CalcH("5-phenyldecane",      "4537-11-5", 35.,  0.418)
    CALL CalcH("4-phenyldecane",      "4537-12-6", 36.,  0.391)
    CALL CalcH("3-phenyldecane",      "4621-36-7", 38.,  0.349)
    CALL CalcH("2-phenyldecane",      "4537-13-7", 26.,  0.251)
    CALL CalcH("1-phenyldecane",       "104-72-3", 11.4, 0.133)
    CALL CalcH("6-phenylundecane",    "4537-14-8", 11.,  0.171)
    CALL CalcH("5-phenylundecane",    "4537-15-9", 10.,  0.163)
    CALL CalcH("4-phenylundecane",    "4536-86-1",  9.,  0.153)
    CALL CalcH("3-phenylundecane",    "4536-87-2", 12.,  0.134)
    CALL CalcH("2-phenylundecane",    "4536-88-3",  8.,  0.098)
    !CALL CalcH("1-phenylundecane",    "6742-54-7",       0.050) ! no solubility
    CALL CalcH("6-phenyldodecane",    "2719-62-2",  4.,  0.067)
    CALL CalcH("5-phenyldodecane",    "2719-63-3",  5.,  0.064)
    CALL CalcH("4-phenyldodecane",    "2719-64-4",  5.,  0.060)
    CALL CalcH("3-phenyldodecane",    "2400-00-2",  7.,  0.053)
    CALL CalcH("2-phenyldodecane",    "2719-61-1",  4.,  0.038)
    !CALL CalcH("1-phenyldodecane",     "123-01-3",       0.019) ! no solubility
    CALL CalcH("6-phenyltridecane",   "4534-49-0",  4.,  0.027)
    CALL CalcH("5-phenyltridecane",   "4534-50-3",  4.,  0.026)
    CALL CalcH("4-phenyltridecane",   "4534-51-4",  4.,  0.020)
    CALL CalcH("3-phenyltridecane",   "4534-52-5",  4.,  0.019)
    CALL CalcH("2-phenyltridecane",   "4534-53-6",  4.,  0.015)
    CALL CalcH("1-phenyltridecane",    "123-02-4",  0.9, 0.008)
    CALL CalcH("6-phenyltetradecane", "4534-55-8",  4.,  0.011)
    CALL CalcH("5-phenyltetradecane", "4534-56-9",  5.,  0.010)
    CALL CalcH("4-phenyltetradecane", "4534-57-0",  4.,  0.010)
    CALL CalcH("3-phenyltetradecane", "4534-58-1",  5.,  0.008)
    CALL CalcH("2-phenyltetradecane", "4534-59-2",  4.,  0.006)
    !CALL CalcH("1-phenyltetradecane", "1459-10-5",       0.002) ! no solubility

    ! why are vapor pressure data different in Tabs. II and VI??? For
    ! example, 1-phenyldecane = 0.133 = n-decylbenzene = 0.127

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, S, P)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: S, P
      chem=chem_ ; casrn=casrn_ ! make global for Output
      Hominus = 1E-6*S/P
      CALL Output(Hominus)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3679

  !---------------------------------------------------------------------------

  SUBROUTINE ref3680 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3680"
    type = "M"
    ndata = 3
    ALLOCATE(temp(ndata), Harray(ndata))
    temp = (/ 25., 40., 55. /) + CtoK
    ! Tab. 1, using data from Eq. (6): "...should have the higher accuracy...":
    CALL CalcH("N-nitrosodimethylamine", "62-75-9", (/ 0.0014, 0.0220, 0.0712 /))
    CALL CalcH("N-nitrosodiethylamine",  "55-18-5", (/ 0.0323, 0.0545, 0.0943 /))
    DEALLOCATE(temp, Harray)

CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, H)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: H
      chem=chem_ ; casrn=casrn_ ! make global for Output
      Harray = Hvcc_TO_HscpSI(H,temp)
      CALL HTdep(temp, Harray, Hominus, mindHR)
      ! use the measured value at 25 C, not the fit function:
      Hominus = Hvcc_TO_HscpSI(H(1),temp(1))
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3680

  !---------------------------------------------------------------------------

  SUBROUTINE ref3681 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3681"
    ! Tab. 1:
    type = "L"
    CALL CalcHL("chloromethane",                   "74-87-3", 164.514,   9538.1,    -23.426  )
    CALL CalcHL("dichloromethane",                 "75-09-2", 137.310,   8923.3,    -19.246  )
    CALL CalcHL("trichloromethane",                "67-66-3", 164.981,   10720.0,   -22.979  )
    CALL CalcHL("tetrachloromethane",              "56-23-5", 373.715,   19941.3,   -53.821  )
    CALL CalcHL("chloroethane",                    "75-00-3", 212.454,   11752.3,   -30.493  )
    CALL CalcHL("1,1-dichloroethane",              "75-34-3", 232.792,   13489.4,   -33.172  )
    CALL CalcHL("1,2-dichloroethane",             "107-06-2", 367.459,   19937.2,   -53.270  )
    CALL CalcHL("1,1,1-trichloroethane",           "71-55-6", 459.803,   23465.6,   -66.958  )
    !CALL CalcHL("1,1,2-trichloroethane",           "79-00-5", 196.111,   12330.0,   -27.737  ) ! use combined data (next line)
    CALL CalcHL("1,1,2-trichloroethane",           "79-00-5", 111.318,   8598.1,    -15.053  )
    !CALL CalcHL("1,1,1,2-tetrachloroethane",      "630-20-6", -627.459,  -24549.8,  95.262   ) ! use combined data (next line)
    CALL CalcHL("1,1,1,2-tetrachloroethane",      "630-20-6", 643.414,   34195.7,   -93.240  )
    CALL CalcHL("1,1,2,2-tetrachloroethane",       "79-34-5", -1526.119, -64857.6,  228.960  )
    !CALL CalcHL("chloroethene",                    "75-01-4", -17.713,   1196.2,    3.813    ) ! use combined data (next line)
    CALL CalcHL("chloroethene",                    "75-01-4", -16.658,   1145.5,    3.597    )
    !CALL CalcHL("1,1-dichloroethene",              "75-35-4", 357.062,   18641.4,   -51.677  ) ! use combined data (next line)
    CALL CalcHL("1,1-dichloroethene",              "75-35-4", 272.503,   14818.7,   -39.091  )
    CALL CalcHL("($Z$)-1,2-dichloroethene",       "156-59-2", 39.763,    4457.4,    -4.664   )
    !CALL CalcHL("($E$)-1,2-dichloroethene",       "156-60-5", 49.275,    5027.6,    -5.850   ) ! use combined data (next line)
    CALL CalcHL("($E$)-1,2-dichloroethene",       "156-60-5", 177.760,   10748.7,   -25.030  )
    !CALL CalcHL("trichloroethene",                 "79-01-6", 88.308,    7242.0,    -11.411  ) ! use combined data (next line)
    CALL CalcHL("trichloroethene",                 "79-01-6", 110.411,   8218.0,    -14.715  )
    !CALL CalcHL("tetrachloroethene",              "127-18-4", 269.309,   15787.9,   -38.035  ) ! use combined data (next line)
    CALL CalcHL("tetrachloroethene",              "127-18-4", 306.515,   17408.7,   -43.611  )
    CALL CalcHL("1,2-dichloropropane",             "78-87-5", 177.116,   11180.7,   -24.893  )
    CALL CalcHL("benzene",                         "71-43-2", 246.963,   14043.0,   -35.340  )
    !CALL CalcHL("ethylbenzene",                   "100-41-4", 571.229,   29076.8,   -83.347  ) ! use combined data (next line)
    CALL CalcHL("ethylbenzene",                   "100-41-4", 220.665,   13454.3,   -31.024  ) ! not B = -13454.3
    CALL CalcHL("propylbenzene",                  "103-65-1", -103.290,  -1594.1,   17.050   )
    !CALL CalcHL("chlorobenzene",                  "108-90-7", 322.326,   17338.1,   -46.683  ) ! use combined data (next line)
    CALL CalcHL("chlorobenzene",                  "108-90-7", 190.288,   11451.5,   -26.972  ) ! not C = -45.382
    CALL CalcHL("1,2-dichlorobenzene",             "95-50-1", -2848.646, -123827.5, 426.617  )
    CALL CalcHL("1,3-dichlorobenzene",            "541-73-1", -1475.902, -62683.5,  221.782  )
    CALL CalcHL("1,4-dichlorobenzene",            "106-46-7", -1396.419, -59065.8,  209.956  )
    CALL CalcHL("1,2,4-trichlorobenzene",         "120-82-1", -691.610,  -28320.2,  104.319  )
    CALL CalcHL("pentachlorobenzene",             "608-93-5", 192.339,   13042.3,   -26.701  )
    CALL CalcHL("methylbenzene",                  "108-88-3", 322.281,   17633.4,   -46.425  )
    CALL CalcHL("1-methyl-2,4-dichlorobenzene",    "95-73-8", 51.762,    6354.8,    -5.677   )
    !CALL CalcHL("1,4-dimethylbenzene",            "106-42-3", 690.635,   34608.8,   -101.065 ) ! use combined data (next line)
    CALL CalcHL("1,4-dimethylbenzene",            "106-42-3", 325.315,   18501.7,   -46.436  )
    CALL CalcHL("1,2-dimethylbenzene",             "95-47-6", 565.741,   28883.6,   -82.576  )
    CALL CalcHL("1,3-dimethylbenzene",            "108-38-3", 432.905,   22590.2,   -62.896  )
    CALL CalcHL("anthracene",                     "120-12-7", -203.722,  -4966.5,   31.759   )
    CALL CalcHL("phenanthrene",                    "85-01-8", 447.075,   23724.2,   -65.603  )
    CALL CalcHL("pyrene",                         "129-00-0", 7.922,     5078.8,    0.321    )
    CALL CalcHL("naphthalene",                     "91-20-3", -551.765,  -20877.5,  83.860   )
    CALL CalcHL("acenaphthene",                    "83-32-9", 30.841,    6909.0,    -2.205   )
    CALL CalcHL("benzo[$jk$]fluorene",            "206-44-0", 38.789,    6855.4,    -4.047   )
    CALL CalcHL("2,4,5-trichlorobiphenyl",      "15862-07-4", -1723.949, -71259.7,  259.866  )
    CALL CalcHL("methyl {tert}-butyl ether",     "1634-04-4", 258.241,   15697.9,   -36.674  )
    CALL CalcHL("hydroxybenzene",                 "108-95-2", 1069.045,  50360.9,   -159.859 )

    ! Tab. 2:
    type = "M"
    CALL CalcHM("1,1,2-trichloroethane",      "79-00-5", & ! -295.178, -11638.3, 44.461
      (/ 40.,  60.,  80.,  90.  /), &
      (/ 0.08, 0.14, 0.23, 0.37 /))
    CALL CalcHM("1,1,1,2-tetrachloroethane", "630-20-6", & ! 650.032,  34898.2,  -94.026
      (/ 10.,  20.,  40.,  50.,  60.,  70.,  80.,  90.  /), &
      (/ 0.01, 0.05, 0.22, 0.28, 0.37, 0.42, 0.75, 0.77 /))
    CALL CalcHM("chloroethene",               "75-01-4", & ! -314.560, -13701.6, 47.195
      (/ 40.,  60.,  70.,  80.,  90.  /), &
      (/ 1.43, 2.47, 2.29, 2.5,  4.52 /))
    CALL CalcHM("1,1-dichloroethene",         "75-35-4", & ! 217.685,  12299.0,  -30.963
      (/ 10.,  20.,  30.,  40.,  50.,  60.,  70.,  80.,  90.  /), &
      (/ 0.57, 0.81, 1.27, 1.64, 2.08, 2.43, 2.83, 3.37, 3.72 /))
    CALL CalcHM("($E$)-1,2-dichloroethene",  "156-60-5", & ! 66.774,   5275.5,   -8.767
      (/ 40.,  60.,  80.,  90.  /), &
      (/ 0.64, 0.99, 1.54, 1.73 /))
    CALL CalcHM("trichloroethene",            "79-01-6", & ! 249.668,  14406.5,  -35.500
      (/ 10.,  20.,  30.,  40.,  50.,  60.,  70.,  80.,  90.  /), &
      (/ 0.19, 0.31, 0.51, 0.72, 0.96, 1.24, 1.49, 1.88, 2.08 /))
    CALL CalcHM("tetrachloroethene",         "127-18-4", & ! 310.103,  17321.9,  -44.295
      (/ 10.,  20.,  30.,  40.,  50.,  60.,  70.,  80.,  90.  /), &
      (/ 0.31, 0.53, 0.91, 1.27, 1.76, 2.23, 2.64, 3.32, 3.7  /))
    CALL CalcHM("ethylbenzene",              "100-41-4", & ! 107.956,  8193.7,   -14.329
      (/ 10.,  20.,  30.,  40.,  50.,  60.,  70.,  80.,  90.  /), &
      (/ 0.15, 0.24, 0.44, 0.59, 0.81, 1.05, 1.45, 1.97, 2.67 /))
    CALL CalcHM("chlorobenzene",             "108-90-7", & ! 191.729, 11472.0, -27.188, not 710.418, 36201.3, -103.629
      (/ 10.,  20.,  30.,  40.,  50.,  60.,  70.,  80.,  90.  /), &
      (/ 0.1,  0.17, 0.24, 0.28, 0.47, 0.53, 0.63, 0.72, 0.92 /))
    CALL CalcHM("1,4-dimethylbenzene",       "106-42-3", & ! -305.829, -12575.3, 46.139, not B = 12575.3
      (/ 40.,  60.,  80.,  90.  /), &
      (/ 0.58, 0.99, 1.45, 2.32 /))

  CONTAINS

    SUBROUTINE CalcHL (chem_, casrn_, A, B, C)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: A, B, C
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! The temperature dependence is:
      ! ln(H) = A - B/T + C*ln(T)
      Hominus = Hvcc_TO_HscpSI(EXP(A-B/T0+C*log(T0)), T0)
      ! The analytical derivative is:
      ! dln(H)/d(1/T) = -B - C*T
      mindHR = - (-B - C*T0)  + T0 ! see ref958, eqn (34) why T0 is added
                                   ! the minus sign comes from Hvcc->Hscp
      CALL MakeNote("moreTdep")
      IF ((chem_=="ethylbenzene").OR.(chem_=="chlorobenzene")) &
        CALL MakeNote(TRIM(ref)//chem_, "The regression parameters for " // chem_ // &
        " in Table~1 of " // TRIM(citet()) // &
        " are wrong. Corrected values from \citet{3722} are used here.")
      CALL Output(Hominus, mindHR)
    END SUBROUTINE CalcHL

    SUBROUTINE CalcHM (chem_, casrn_, temp, H)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, H
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep_ABC(temp+CtoK, Hvcc_TO_HscpSI(H, temp+CtoK), Hominus, mindHR)
      IF (temp(1)>25.) CALL MakeNote("highTextrapol")
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcHM

  END SUBROUTINE ref3681

  !---------------------------------------------------------------------------

  SUBROUTINE ref3682 ! Hvpx [bar]
    IMPLICIT NONE

    ref = "3682"
    ! Tab. 1:
    CALL CalcH_Tab1("$D$-limonene",                        "5989-27-5", "XMGQYMWWDOXHJM-SNVBAGLBSA-N",  3.16,  2.88 )
    CALL CalcH_Tab1("2-furanmethanol",                       "98-00-0", "XPFVYQJUAUNWIW-UHFFFAOYSA-N", -1.65, -1.59 )
    CALL CalcH_Tab1("benzyl alcohol",                       "100-51-6", "WVDDGKGOMKODPV-UHFFFAOYSA-N", -1.84, -1.45 )
    CALL CalcH_Tab1("2-phenylethanol",                       "60-12-8", "WRMNZCZEMHIOCP-UHFFFAOYSA-N", -0.80, -1.45 )
    CALL CalcH_Tab1("pelargol",                             "106-21-8", "PRNCMAKCNVRZFX-UHFFFAOYSA-N",  1.22,  0.17 )
    CALL CalcH_Tab1("vanillin",                             "121-33-5", "MWOOGOJBHIARFG-UHFFFAOYSA-N", -3.37, -3.5  )
    CALL CalcH_Tab1("2-furancarboxaldehyde",                 "98-01-1", "HYBBIBNJHNGZAN-UHFFFAOYSA-N", -0.73, -1.24 )
    CALL CalcH_Tab1("hexanal",                               "66-25-1", "JARKCYVAAOWBJS-UHFFFAOYSA-N",  1.29,  1.12 )
    CALL CalcH_Tab1("benzaldehyde",                         "100-52-7", "HUMNYLRZRPPJDN-UHFFFAOYSA-N",  0.15, -0.18 )
    CALL CalcH_Tab1("4-methyl-2-pentanone",                 "108-10-1", "NTIZESTWPVYFNL-UHFFFAOYSA-N",  0.88,  0.6  )
    CALL CalcH_Tab1("2-heptanone",                          "110-43-0", "CATSNJVOTSVZJV-UHFFFAOYSA-N",  0.87,  0.56 )
    CALL CalcH_Tab1("hexyl ethanoate",                      "142-92-7", "AOGQPLXWSUTHQB-UHFFFAOYSA-N",  1.44,  1.19 )
    CALL CalcH_Tab1("ethyl butanoate",                      "105-54-4", "OBNCKNCVKJNDBV-UHFFFAOYSA-N",  1.30,  1.09 )
    CALL CalcH_Tab1("ethyl benzoate",                        "93-89-0", "MTZQAGJQAFMTAQ-UHFFFAOYSA-N",  0.61,  0.57 )
    CALL CalcH_Tab1("anethole",                             "104-46-1", "RUVINXPYWBROJD-UHFFFAOYSA-N",  0.74,  1.17 )
    CALL CalcH_Tab1("indole",                               "120-72-9", "SIKJAQJRHWYJAI-UHFFFAOYSA-N", -1.47, -0.98 )
    ! Tab. 2:
    CALL CalcH_Tab2("limonene",                             "138-86-3", "XMGQYMWWDOXHJM-UHFFFAOYSA-N",  3.00,  2.88 )
    CALL CalcH_Tab2("myrcene",                              "123-35-3", "UAHWPYUMFXYFJY-UHFFFAOYSA-N",  3.13,  3.64 )
    CALL CalcH_Tab2("linalool",                              "78-70-6", "CDOSHBSSFJOMGT-UHFFFAOYSA-N",  0.13, -0.45 )
    CALL CalcH_Tab2("linalool oxide",                      "1365-19-1", "BXOURKNXQXLKRK-UHFFFAOYSA-N", DUMMY, -2.73 )
    CALL CalcH_Tab2("alpha-terpineol",                       "98-55-5", "WUOACPNHFRMFPN-UHFFFAOYSA-N", -0.74, -1.07 )
    CALL CalcH_Tab2("geraniol",                             "106-24-1", "GLZPCOQZEFWAFX-JXMROGBWSA-N",  0.50, -0.47 )
    CALL CalcH_Tab2("citral",                              "5392-40-5", "WTEVQBCEXWBHNA-UHFFFAOYSA-N",  0.37, -0.97 )
    CALL CalcH_Tab2("citronellol",                          "106-22-9", "QMVPMAAFGQKVCJ-UHFFFAOYSA-N", DUMMY,  0.05 )
    CALL CalcH_Tab2("linalyl acetate",                      "115-95-7", "UWKAYLJWKGQEPM-UHFFFAOYSA-N",  1.97,  1.49 )
    CALL CalcH_Tab2("hydroxycitronellal",                   "107-75-5", "WPFVBOQKRVRMJB-UHFFFAOYSA-N", DUMMY, -3.23 )
    CALL CalcH_Tab2("$\beta$-caryophyllene",                 "87-44-5", "NPNUFJAVOOONJE-IOMPXFEGSA-N", DUMMY,  2.32 )
    CALL CalcH_Tab2("methyl-alpha-ionone",                "93302-56-8", "VPKMGDRERYMTJX-XEHSLEBBSA-N", DUMMY, -0.32 )
    CALL CalcH_Tab2("cedrol",                                "77-53-2", "SVURIXNDRWRAFU-OGMFBOKVSA-N", DUMMY, -1.08 )
    CALL CalcH_Tab2("patchoulol",                          "5986-55-0", "GGHMUJBZYLPWFD-CUZKYEQNSA-N", DUMMY, -1.36 )
    CALL CalcH_Tab2("methyl cedryl ketone",               "32388-55-9", "YBUIAJZFOGJGLJ-SWRJLBSHSA-N", DUMMY, -1.04 )
    CALL CalcH_Tab2("vetyveryl acetate",                    "117-98-6", "UAVFEMBKDRODDE-UHFFFAOYSA-N", DUMMY, -0.59 )
    !CALL CalcH_Tab2("2-phenylethanol",                      "60-12-8", "WRMNZCZEMHIOCP-UHFFFAOYSA-N", -1.75, -1.45 ) ! also listed in Tab. 1
    CALL CalcH_Tab2("vanillin",                             "121-33-5", "MWOOGOJBHIARFG-UHFFFAOYSA-N", -3.94, -3.50 )
    CALL CalcH_Tab2("eugenol",                               "97-53-0", "RRAFCDWBNXTKKO-UHFFFAOYSA-N", -1.06, -0.85 )
    !CALL CalcH_Tab2("nonanal",                             "124-19-6", "GYHFUZHODSMOHU-UHFFFAOYSA-N",  1.40,  1.49 ) ! also listed in Tab. S2
    CALL CalcH_Tab2("phenylacetaldehyde",                   "122-78-1", "DTUQWGWMVIHBKE-UHFFFAOYSA-N",  0.73, -0.47 )
    CALL CalcH_Tab2("hexyl cinnamic aldehyde",              "101-86-0", "GUUHFMWKWLOQMM-UHFFFAOYSA-N", DUMMY, -0.26 )
    CALL CalcH_Tab2("6-methyl-5-hepten-2-one",              "110-93-0", "UHEPJGULSIKKTP-UHFFFAOYSA-N", DUMMY,  0.34 )
    CALL CalcH_Tab2("\textit{cis}-jasmone",                 "488-10-8", "XMLSXPIVAXONDL-PLNGDYQASA-N", DUMMY, -0.97 )
    CALL CalcH_Tab2("iso-e super",                        "54464-57-2", "FVUGZKDGWGKCFE-UHFFFAOYSA-N", DUMMY,  0.00 )
    CALL CalcH_Tab2("civetone",                             "542-46-1", "ZKVZSBSZTMPBQR-UPHRSURJSA-N", DUMMY, -0.74 )
    CALL CalcH_Tab2("phenylmethyl ethanoate",               "140-11-4", "QUKGYYKBILRGFE-UHFFFAOYSA-N", -0.22, -0.14 )
    CALL CalcH_Tab2("hedione",                            "24851-98-7", "KVWWIYGFBYDJQC-UHFFFAOYSA-N", DUMMY, -2.10 )
    CALL CalcH_Tab2("methyl anthranilate",                  "134-20-3", "VAMXMNNIEUEQDV-UHFFFAOYSA-N", -0.98, -0.81 )
    CALL CalcH_Tab2("coumarin",                              "91-64-5", "ZYGHJZDHTFUPRJ-UHFFFAOYSA-N", -2.27, -2.36 )
    CALL CalcH_Tab2("isoambrettolide",                    "28645-51-4", "QILMAYXCYBTEDM-IWQZZHSRSA-N", DUMMY, -0.66 )
    CALL CalcH_Tab2("ethylene brassylate",                  "105-95-3", "XRHCAGNSDHCHFJ-UHFFFAOYSA-N", DUMMY, -1.98 )
    !CALL CalcH_Tab2("indole",                              "120-72-9", "SIKJAQJRHWYJAI-UHFFFAOYSA-N", -1.33, -0.98 ) ! also listed in Tab. 1
    ! Tab. 3:
    CALL CalcH_Tab3("lilial",                                "80-54-6", "SDQFDHOLCGWZPU-UHFFFAOYSA-N", -0.01 )
    CALL CalcH_Tab3("bourgeonal",                         "18127-01-0", "FZJUFJKVIYFBSY-UHFFFAOYSA-N", -0.31 )
    CALL CalcH_Tab3("hivernal",                          "300371-33-9", "GEPCDOWRWODJEY-UHFFFAOYSA-N", -0.59 )
    CALL CalcH_Tab3("cyclemax",                            "7775-00-0", "RLEFOSDUWZYGOS-UHFFFAOYSA-N", -0.15 )
    CALL CalcH_Tab3("silvial",                             "6658-48-6", "YLIXVKUWWOQREC-UHFFFAOYSA-N", 0.28  )
    CALL CalcH_Tab3("florhydral",                        "125109-85-5", "OHRBQTOZYGEWCJ-UHFFFAOYSA-N", 0.05  )
    CALL CalcH_Tab3("mefloral",                           "62518-65-4", "GLZRHVTZLDNUQP-UHFFFAOYSA-N", 0.24  )
    CALL CalcH_Tab3("cyclamen aldehyde",                    "103-95-7", "ZFNVDHOSLNRHNN-UHFFFAOYSA-N", 0.39  )
    CALL CalcH_Tab3("tillenal",                          "850997-10-3", "IHMKWBJKOWYASH-UHFFFAOYSA-N", 0.21  )
    CALL CalcH_Tab3("aquaflora",                        "1339119-15-1", "BFTAXJRKNXWMMX-UHFFFAOYSA-N", 0.03  )
    CALL CalcH_Tab3("starfleur",                        "1254940-85-6", "XUTZHJNXAOHUTM-UHFFFAOYSA-N", 0.61  )
    CALL CalcH_Tab3("lilyflore",                         "285977-85-7", "UWSPWQQZFOSTHS-UHFFFAOYSA-N", -1.37 )
    CALL CalcH_Tab3("lilybelle",                        "1378867-81-2", "VZZSYXAVGYODQG-UHFFFAOYSA-N", 0.09  )
    CALL CalcH_Tab3("mugoxal",                                      "", "WGWWNPFQPHLSIM-UHFFFAOYSA-N", -0.08 )
    CALL CalcH_Tab3("josenol",                            "56138-10-4", "WWHATUWNHYNMNW-UHFFFAOYSA-N", -1.91 )
    CALL CalcH_Tab3("nympheal",                         "1637294-12-2", "UKZXPOJABTXLMK-UHFFFAOYSA-N", -0.17 )
    CALL CalcH_Tab3("mimosal",                          "1226911-69-8", "LBKHGAIELUNYML-ZRDIBKRKSA-N", -0.95 )
    ! Tab. S2:
    CALL CalcH_TabS2("1,2-dimethylnaphthalene",             "573-98-8", "QNLZIZAQLLYXTC-UHFFFAOYSA-N", 1.06,   1.33    ) !   1
    CALL CalcH_TabS2("propylene glycol",                     "57-55-6", "DNIAPMSPPWPWGF-UHFFFAOYSA-N", -3.69,  -3.53   ) !   2
    CALL CalcH_TabS2("1,4-dioxane",                         "123-91-1", "RYHBNJHYFVUHQT-UHFFFAOYSA-N", DUMMY,  DUMMY   ) !   3
    CALL CalcH_TabS2("1-decanol",                           "112-30-1", "MWKFXSUHUHTGQN-UHFFFAOYSA-N", 0.43,   0.29    ) !   4
    CALL CalcH_TabS2("1-dodecanol",                         "112-53-8", "LQZZUXJYWNFBMV-UHFFFAOYSA-N", 0.46,   0.50    ) !   5
    CALL CalcH_TabS2("heptanal",                            "111-71-7", "FXHGMKSSBGDXIY-UHFFFAOYSA-N", 1.43,   1.27    ) !   6
    CALL CalcH_TabS2("1-heptanol",                          "111-70-6", "BBMCTIGTTCKYKF-UHFFFAOYSA-N", 0.03,   0.08    ) !   7
    CALL CalcH_TabS2("1-hexanol",                           "111-27-3", "ZSIAUFGUXNUGDI-UHFFFAOYSA-N", 0.03,   -0.02   ) !   8
    CALL CalcH_TabS2("nonanal",                             "124-19-6", "GYHFUZHODSMOHU-UHFFFAOYSA-N", 1.74,   1.49    ) !   9
    CALL CalcH_TabS2("1-nonanol",                           "143-08-8", "ZWRUINPWMLAQRD-UHFFFAOYSA-N", 0.16,   0.20    ) !  10
    CALL CalcH_TabS2("octanal",                             "124-13-0", "NUJGJRNETVAIRJ-UHFFFAOYSA-N", 1.63,   1.29    ) !  11
    CALL CalcH_TabS2("1-octanol",                           "111-87-5", "KBPLFHHGFOOTCA-UHFFFAOYSA-N", 0.15,   0.19    ) !  12
    CALL CalcH_TabS2("1-pentanol",                           "71-41-0", "AMQJEAYHLZJPGS-UHFFFAOYSA-N", -0.14,  -0.12   ) !  13
    CALL CalcH_TabS2("1-tetradecanol",                      "112-72-1", "HLZKNKRTKFSKGZ-UHFFFAOYSA-N", 0.77,   0.66    ) !  14
    CALL CalcH_TabS2("1-tridecanal",                      "10486-19-8", "BGEHHAVMRVXCGR-UHFFFAOYSA-N", 1.98,   1.76    ) !  15
    CALL CalcH_TabS2("1-tridecanol",                        "112-70-9", "XFRVVPUIAFSTFO-UHFFFAOYSA-N", 0.67,   0.58    ) !  16
    CALL CalcH_TabS2("1-tridecene",                        "2437-56-1", "VQOXUMQBYILCKR-UHFFFAOYSA-N", 4.98,   4.77    ) !  17
    CALL CalcH_TabS2("1-undecanol",                         "112-42-5", "KJIOQYGWTQBHNH-UHFFFAOYSA-N", 0.40,   0.42    ) !  18
    CALL CalcH_TabS2("2-(2-ethoxyethoxy)ethanol",           "111-90-0", "XXJWXESWEXIICW-UHFFFAOYSA-N", DUMMY,  -3.21   ) !  19
    CALL CalcH_TabS2("2,2-dimethyl-1-propanol",              "75-84-3", "KPSSIOMAKSHJJG-UHFFFAOYSA-N", -0.01,  0.18    ) !  20
    CALL CalcH_TabS2("1-hydroxy-2,4-dimethylbenzene",       "105-67-9", "KUFFULVDNCHOFZ-UHFFFAOYSA-N", -0.93,  -1.08   ) !  21
    CALL CalcH_TabS2("1-hydroxy-2,5-dimethylbenzene",        "95-87-4", "NKTOLZVEWDHZMU-UHFFFAOYSA-N", -0.53,  -1.01   ) !  22
    CALL CalcH_TabS2("1-hydroxy-2,6-dimethylbenzene",       "576-26-1", "NXXYKOUNUYWIHA-UHFFFAOYSA-N", -0.40,  -0.47   ) !  23
    CALL CalcH_TabS2("2-decanone",                          "693-54-9", "ZAJNGDIORYACQU-UHFFFAOYSA-N", 1.51,   0.78    ) !  24
    CALL CalcH_TabS2("2-ethylbutanal",                       "97-96-1", "UNNGUFMVYQJGTD-UHFFFAOYSA-N", 1.14,   1.18    ) !  25
    CALL CalcH_TabS2("2-ethylhexanal",                      "123-05-7", "LGYNIFWIKSEESD-UHFFFAOYSA-N", 1.36,   1.43    ) !  26
    CALL CalcH_TabS2("2-heptanol",                          "543-49-7", "CETWDUZRCINIHU-UHFFFAOYSA-N", 0.66,   -0.02   ) !  27
    CALL CalcH_TabS2("2-hexanol",                           "626-93-7", "QNVRIHYSUZMSGM-UHFFFAOYSA-N", 0.12,   -0.12   ) !  28
    CALL CalcH_TabS2("2-hexanone",                          "591-78-6", "QQZOPKMRPOGIEB-UHFFFAOYSA-N", 0.67,   0.52    ) !  29
    CALL CalcH_TabS2("2-methyl-1-butanol",                  "137-32-6", "QPRQEDXDYOZYLA-UHFFFAOYSA-N", -0.18,  0.08    ) !  30
    CALL CalcH_TabS2("2-nonanone",                          "821-55-6", "VKCYHJWLYTUGCC-UHFFFAOYSA-N", 1.42,   0.71    ) !  31
    CALL CalcH_TabS2("2-octanol",                           "123-96-6", "SJWFXCIHNDVPSH-UHFFFAOYSA-N", -0.24,  0.12    ) !  32
    CALL CalcH_TabS2("2-pentanethiol",                     "2084-19-7", "QUSTYFNPKBDELJ-UHFFFAOYSA-N", 2.56,   2.74    ) !  33
    CALL CalcH_TabS2("2-pentanol",                         "6032-29-7", "JYVLIDXNZAXMDK-UHFFFAOYSA-N", -0.07,  -0.14   ) !  34
    CALL CalcH_TabS2("2-undecanol",                        "1653-30-1", "XMUJIPOFTAHSOK-UHFFFAOYSA-N", 0.67,   0.39    ) !  35
    CALL CalcH_TabS2("2-undecanone",                        "112-12-9", "KYWIYKKSMDLRDC-UHFFFAOYSA-N", 1.50,   0.86    ) !  36
    CALL CalcH_TabS2("1-hydroxy-3,4-dimethylbenzene",        "95-65-8", "YCOXTKKNXUZSKD-UHFFFAOYSA-N", -1.29,  -1.50   ) !  37
    CALL CalcH_TabS2("1-hydroxy-3,5-dimethylbenzene",       "108-68-9", "TUAMRELNJMMDMT-UHFFFAOYSA-N", -1.12,  -1.34   ) !  38
    CALL CalcH_TabS2("3-heptanone",                         "106-35-4", "NGAZZOYFWWSOGK-UHFFFAOYSA-N", 1.05,   0.88    ) !  39
    CALL CalcH_TabS2("3-methyl-1-butanol",                  "123-51-3", "PHTQWCKDNZKARW-UHFFFAOYSA-N", -0.13,  -0.11   ) !  40
    CALL CalcH_TabS2("4-heptanone",                         "123-19-3", "HCFAJYNVAYBARA-UHFFFAOYSA-N", 1.23,   1.16    ) !  41
    ! acetal is a group of species, not a pure substance                                               0.72,   0.94    ) !  42
    CALL CalcH_TabS2("ethanal",                              "75-07-0", "IKHGUXGNUITLKF-UHFFFAOYSA-N", 0.85,   0.49    ) !  43
    CALL CalcH_TabS2("ethanamide",                           "60-35-5", "DLFVBJFMPXGRIB-UHFFFAOYSA-N", DUMMY,  DUMMY   ) !  44
    CALL CalcH_TabS2("N-phenylacetamide",                   "103-84-4", "FZERHIULMFGESH-UHFFFAOYSA-N", -2.93,  -3.43   ) !  45
    CALL CalcH_TabS2("propanone",                            "67-64-1", "CSCPPACGZOOCGX-UHFFFAOYSA-N", 0.43,   -0.04   ) !  46
    CALL CalcH_TabS2("1-phenylethanone",                     "98-86-2", "KWOLFJPFCHCOCG-UHFFFAOYSA-N", -0.29,  -0.52   ) !  47
    CALL CalcH_TabS2("2-propen-1-ol",                       "107-18-6", "XXROGKLTLUQVRX-UHFFFAOYSA-N", -0.46,  -0.44   ) !  48
    CALL CalcH_TabS2("alpha-phellandrene",                   "99-83-2", "OGLDWXZKYODSOB-UHFFFAOYSA-N", DUMMY,  3.02    ) !  49
    CALL CalcH_TabS2("alpha-pinene",                         "80-56-8", "GRWFGVWFFZKLTI-UHFFFAOYSA-N", DUMMY,  3.40    ) !  50
    CALL CalcH_TabS2("alpha-terpinene",                      "99-86-5", "YHQGMYUVUMAZJR-UHFFFAOYSA-N", DUMMY,  3.01    ) !  51
    CALL CalcH_TabS2("aminobenzene",                         "62-53-3", "PAYRUJLWNCNPSJ-UHFFFAOYSA-N", -1.02,  -0.63   ) !  52
    CALL CalcH_TabS2("methoxybenzene",                      "100-66-3", "RDOXTESZEPMUJZ-UHFFFAOYSA-N", 2.31,   1.47    ) !  53
    CALL CalcH_TabS2("benzene",                              "71-43-2", "UHOVQNZJYSORNB-UHFFFAOYSA-N", 2.49,   2.49    ) !  54
    CALL CalcH_TabS2("benzophenone",                        "119-61-9", "RWCCWEUUXYIKHB-UHFFFAOYSA-N", -0.49,  -1.24   ) !  55
    !CALL CalcH_TabS2("phenylmethyl ethanoate",             "140-11-4", "QUKGYYKBILRGFE-UHFFFAOYSA-N", DUMMY,  -0.14   ) !  56 also listed in Tab. 2
    CALL CalcH_TabS2("benzyl benzoate",                     "120-51-4", "SESFRYSPDFLNCH-UHFFFAOYSA-N", DUMMY,  -0.49   ) !  57
    CALL CalcH_TabS2("benzyl ethyl ether",                  "539-30-0", "AXPZDYVDTMMLNB-UHFFFAOYSA-N", DUMMY,  0.92    ) !  58
    CALL CalcH_TabS2("$\beta$-phellandrene",                "555-10-2", "LFJQCDVYDGGFCH-UHFFFAOYSA-N", DUMMY,  2.96    ) !  59
    CALL CalcH_TabS2("$\beta$-pinene",                      "127-91-3", "WTARULDDTDQWMU-UHFFFAOYSA-N", DUMMY,  3.11    ) !  60
    CALL CalcH_TabS2("butane",                              "106-97-8", "IJDNQMDRQITEOD-UHFFFAOYSA-N", 4.71,   4.53    ) !  61
    CALL CalcH_TabS2("1-butanol",                            "71-36-3", "LRHPLDYGYMQRHN-UHFFFAOYSA-N", -0.32,  -0.23   ) !  62
    CALL CalcH_TabS2("butyl benzoate",                      "136-60-7", "XSIFPSYPOVKYCO-UHFFFAOYSA-N", DUMMY,  0.74    ) !  63
    CALL CalcH_TabS2("butyl butanoate",                     "109-21-7", "XUPYJHCZDLZNFP-UHFFFAOYSA-N", DUMMY,  1.73    ) !  64
    CALL CalcH_TabS2("1-butanethiol",                       "109-79-5", "WQAQPCDUOCURKW-UHFFFAOYSA-N", 2.70,   2.50    ) !  65
    CALL CalcH_TabS2("butyl propanoate",                    "590-01-2", "BTMVHUNTONAYDX-UHFFFAOYSA-N", 1.23,   1.32    ) !  66
    CALL CalcH_TabS2("butyl pentanoate",                    "591-68-4", "OKJADYKTJJGKDX-UHFFFAOYSA-N", DUMMY,  1.39    ) !  67
    CALL CalcH_TabS2("1-(ethenyloxy)butane",                "111-34-2", "UZKWTJUDCOPSNM-UHFFFAOYSA-N", DUMMY,  2.88    ) !  68
    CALL CalcH_TabS2("butyl ethanoate",                     "123-86-4", "DKPFZGUDAPQIHT-UHFFFAOYSA-N", 1.12,   1.11    ) !  69
    CALL CalcH_TabS2("1-butanamine",                        "109-73-9", "HQABUPZFAYXKJW-UHFFFAOYSA-N", 0.18,   0.43    ) !  70
    CALL CalcH_TabS2("butanal",                             "123-72-8", "ZTQSAGDEMFDKMZ-UHFFFAOYSA-N", 0.82,   0.91    ) !  71
    CALL CalcH_TabS2("butanenitrile",                       "109-74-0", "KVNRLNFWIYMESJ-UHFFFAOYSA-N", 0.48,   0.17    ) !  72
    CALL CalcH_TabS2("cycloheptane",                        "291-64-5", "DMEGYFMYUHOHGS-UHFFFAOYSA-N", 3.72,   3.66    ) !  73
    CALL CalcH_TabS2("cyclohexane",                         "110-82-7", "XDTMQSROBMDMFD-UHFFFAOYSA-N", 4.04,   3.92    ) !  74
    CALL CalcH_TabS2("cyclohexanol",                        "108-93-0", "HPXRVTGHNJAIIH-UHFFFAOYSA-N", -0.85,  -1.05   ) !  75
    CALL CalcH_TabS2("cyclohexanone",                       "108-94-1", "JHIVVAPYMSGYDF-UHFFFAOYSA-N", -0.59,  -0.68   ) !  76
    CALL CalcH_TabS2("cyclohexene",                         "110-83-8", "HGCIXCUEYOPUTN-UHFFFAOYSA-N", 3.40,   3.23    ) !  77
    CALL CalcH_TabS2("cyclopentane",                        "287-92-3", "RGSFGYAAUTVSQA-UHFFFAOYSA-N", 4.02,   3.95    ) !  78
    CALL CalcH_TabS2("decanal",                             "112-31-2", "KSMVZQYAVGTKIV-UHFFFAOYSA-N", 1.63,   1.47    ) !  79
    CALL CalcH_TabS2("decane",                              "124-18-5", "DIOQZVSQGTUSAI-UHFFFAOYSA-N", 5.46,   5.10    ) !  80
    CALL CalcH_TabS2("decyl ethanoate",                     "112-17-4", "NUPSHWCALHZGOV-UHFFFAOYSA-N", 2.29,   1.68    ) !  81
    CALL CalcH_TabS2("diacetone alcohol",                   "123-42-2", "SWXVUIWOUIDPGS-UHFFFAOYSA-N", -1.37,  -2.75   ) !  82
    CALL CalcH_TabS2("dibenzyl ether",                      "103-50-4", "MHDVGSVTJDSBDK-UHFFFAOYSA-N", DUMMY,  0.17    ) !  83
    CALL CalcH_TabS2("diethyl ketone",                       "96-22-0", "FDPIMTJIUBPUKL-UHFFFAOYSA-N", 0.83,   0.81    ) !  84
    CALL CalcH_TabS2("diethylamine",                        "109-89-7", "HPNMFZURTQLUMO-UHFFFAOYSA-N", 0.28,   1.03    ) !  85
    CALL CalcH_TabS2("diethylene glycol",                   "111-46-6", "MTHSVFCYNBDYFN-UHFFFAOYSA-N", -4.77,  -4.21   ) !  86
    CALL CalcH_TabS2("diisopropyl ether",                   "108-20-3", "ZAFNJMIOTHYJRJ-UHFFFAOYSA-N", 1.99,   1.99    ) !  87
    CALL CalcH_TabS2("dipentyl ether",                      "693-65-2", "AOPDRZXCEAKHHW-UHFFFAOYSA-N", 3.20,   3.09    ) !  88
    CALL CalcH_TabS2("dipropyl ether",                      "111-43-3", "POLCUAVZOMRGSN-UHFFFAOYSA-N", 2.09,   2.66    ) !  89
    CALL CalcH_TabS2("di-{sec}-butyl ether",               "6863-58-7", "HHBZZTKMMLDNDN-UHFFFAOYSA-N", 2.48,   2.74    ) !  90
    CALL CalcH_TabS2("dodecanal",                           "112-54-9", "HFJRKMMYBMWEAD-UHFFFAOYSA-N", 2.03,   1.66    ) !  91
    CALL CalcH_TabS2("dodecane",                            "112-40-3", "SNRUBQQJIBEYMU-UHFFFAOYSA-N", 5.66,   5.27    ) !  92
    CALL CalcH_TabS2("ethyl ethanoate",                     "141-78-6", "XEKOWRVHYACXOJ-UHFFFAOYSA-N", 0.89,   0.85    ) !  93
    CALL CalcH_TabS2("ethyl acrylate",                      "140-88-5", "JIGUQPWFLRLWPJ-UHFFFAOYSA-N", 1.15,   1.07    ) !  94
    CALL CalcH_TabS2("ethyl isovalerate",                   "108-64-5", "PPXUHEORWJQRHJ-UHFFFAOYSA-N", DUMMY,  1.40    ) !  95
    CALL CalcH_TabS2("ethyl propanoate",                    "105-37-3", "FKRCODPIKNYEAC-UHFFFAOYSA-N", 1.09,   1.13    ) !  96
    CALL CalcH_TabS2("3-oxobutanoic acid, ethyl ester",     "141-97-9", "XYIBRDXRRQCHLP-UHFFFAOYSA-N", -1.48,  -1.63   ) !  97
    CALL CalcH_TabS2("1,2-ethanediol",                      "107-21-1", "LYCAIKOWRPUZTN-UHFFFAOYSA-N", -4.10,  -3.94   ) !  98
    CALL CalcH_TabS2("furan",                               "110-00-9", "YLQBMQCUIZJEEH-UHFFFAOYSA-N", 2.48,   2.40    ) !  99
    CALL CalcH_TabS2("gamma-butyrolactone",                  "96-48-0", "YEJRWHAVMIAJKC-UHFFFAOYSA-N", DUMMY,  -2.07   ) ! 100
    CALL CalcH_TabS2("gamma-terpinene",                      "99-85-4", "YKFLAYDHMOASIY-UHFFFAOYSA-N", DUMMY,  2.95    ) ! 101
    CALL CalcH_TabS2("1,2,3-propanetriol",                   "56-81-5", "PEDCQBHIVMGVHV-UHFFFAOYSA-N", DUMMY,  DUMMY   ) ! 102
    CALL CalcH_TabS2("heptane",                             "142-82-5", "IMNFDUFMRHMDMM-UHFFFAOYSA-N", 5.18,   4.83    ) ! 103
    CALL CalcH_TabS2("heptyl ethanoate",                    "112-06-1", "ZCZSIDMEHXZRLG-UHFFFAOYSA-N", 1.68,   1.36    ) ! 104
    !CALL CalcH_TabS2("hexanal",                             "66-25-1", "JARKCYVAAOWBJS-UHFFFAOYSA-N", 1.29,   1.12    ) ! 105 also listed in Tab. 1
    CALL CalcH_TabS2("hexane",                              "110-54-3", "VLKZOEOYAKHREP-UHFFFAOYSA-N", 4.86,   4.73    ) ! 106
    CALL CalcH_TabS2("hexyl butanoate",                    "2639-63-6", "XAPCMTMQBXLDBB-UHFFFAOYSA-N", DUMMY,  1.69    ) ! 107
    CALL CalcH_TabS2("indane",                              "496-11-7", "PQNFLJBBNBOBRQ-UHFFFAOYSA-N", 2.09,   2.36    ) ! 108
    CALL CalcH_TabS2("2-methylpropyl ethanoate",            "110-19-0", "GJRQTCIYDGXPES-UHFFFAOYSA-N", 1.18,   1.18    ) ! 109
    CALL CalcH_TabS2("2-methylpropanal",                     "78-84-2", "AMIMRNSIRUDHCM-UHFFFAOYSA-N", 0.98,   1.09    ) ! 110
    CALL CalcH_TabS2("2-propyl ethanoate",                  "108-21-4", "JMMWKPVZQRWMSS-UHFFFAOYSA-N", 1.18,   1.09    ) ! 111
    CALL CalcH_TabS2("2-propanol",                           "67-63-0", "KFZMGEQAYNKOFK-UHFFFAOYSA-N", -0.14,  -0.63   ) ! 112
    CALL CalcH_TabS2("2-propyl butanoate",                  "638-11-9", "FFOPEPMHKILNIT-UHFFFAOYSA-N", DUMMY,  1.47    ) ! 113
    CALL CalcH_TabS2("m-cresol",                            "108-39-4", "RLSSMJSEOOYNOY-UHFFFAOYSA-N", -1.60,  -1.50   ) ! 114
    CALL CalcH_TabS2("m-cymene",                            "535-77-3", "XCYJPXQACVEIOS-UHFFFAOYSA-N", 2.65,   2.83    ) ! 115
    CALL CalcH_TabS2("2-methylpropenal",                     "78-85-3", "STNJBCKSHOAVAJ-UHFFFAOYSA-N", 1.07,   0.88    ) ! 116
    CALL CalcH_TabS2("methyl ethanoate",                     "79-20-9", "KXKVLQRXCPHEJC-UHFFFAOYSA-N", 0.49,   0.76    ) ! 117
    CALL CalcH_TabS2("methanol",                             "67-56-1", "OKKJLVBELUTLKV-UHFFFAOYSA-N", -0.42,  -0.47   ) ! 118
    CALL CalcH_TabS2("methyl benzoate",                      "93-58-3", "QPJVMBTYPHYUOC-UHFFFAOYSA-N", 0.29,   0.32    ) ! 119
    CALL CalcH_TabS2("methyl butanoate",                    "623-42-7", "UUIQMZJEGPQKFD-UHFFFAOYSA-N", 1.20,   1.10    ) ! 120
    CALL CalcH_TabS2("methyl dodecanoate",                  "111-82-0", "UQDUPQYQJKYHQI-UHFFFAOYSA-N", DUMMY,  1.87    ) ! 121
    CALL CalcH_TabS2("butanone",                             "78-93-3", "ZWEHNKRNPOVVGH-UHFFFAOYSA-N", 0.86,   0.38    ) ! 122
    CALL CalcH_TabS2("methyl 2-propyl ether",               "598-53-8", "RMGHERXMTMUMMV-UHFFFAOYSA-N", 1.68,   1.89    ) ! 123
    CALL CalcH_TabS2("methyl propyl ketone",                "107-87-9", "XNLICIUVMPYHGG-UHFFFAOYSA-N", 0.61,   0.35    ) ! 124
    CALL CalcH_TabS2("methyl {tert}-butyl ether",          "1634-04-4", "BZLVMXJERCGZMT-UHFFFAOYSA-N", 1.48,   1.73    ) ! 125
    CALL CalcH_TabS2("ethanolamine",                        "141-43-5", "HZAXFHJVJLSVMW-UHFFFAOYSA-N", DUMMY,  DUMMY   ) ! 126
    CALL CalcH_TabS2("naphthalene",                          "91-20-3", "UFWIBTONFRDIAS-UHFFFAOYSA-N", 1.89,   1.49    ) ! 127
    CALL CalcH_TabS2("nonyl acetate",                       "143-13-5", "GJQIMXVRFNLMTB-UHFFFAOYSA-N", 1.13,   1.43    ) ! 128
    CALL CalcH_TabS2("o-cresol",                             "95-48-7", "QWVGKYWNOKOFNN-UHFFFAOYSA-N", -1.20,  -1.18   ) ! 129
    CALL CalcH_TabS2("octane",                              "111-65-9", "TVMXDCGIABBOFY-UHFFFAOYSA-N", 5.44,   4.91    ) ! 130
    CALL CalcH_TabS2("octyl ethanoate",                     "112-14-1", "YLYBTZIQSIBWLI-UHFFFAOYSA-N", 1.81,   1.49    ) ! 131
    CALL CalcH_TabS2("o-cymene",                            "527-84-4", "WWRCMNKATXZARA-UHFFFAOYSA-N", 2.67,   2.64    ) ! 132
    CALL CalcH_TabS2("p-cresol",                            "106-44-5", "IWDCLRJOBJJRNH-UHFFFAOYSA-N", -1.60,  -1.45   ) ! 133
    CALL CalcH_TabS2("p-cymene",                             "99-87-6", "HFPZCAJZSCWRBC-UHFFFAOYSA-N", 2.63,   2.86    ) ! 134
    CALL CalcH_TabS2("pentanal",                            "110-62-3", "HGBOYTHUEUWSSQ-UHFFFAOYSA-N", 1.27,   0.96    ) ! 135
    CALL CalcH_TabS2("pentane",                             "109-66-0", "OFBQJSOFQDEBGM-UHFFFAOYSA-N", 4.85,   4.63    ) ! 136
    CALL CalcH_TabS2("pentyl acetate",                      "628-63-7", "PGMYKACGEOXYJE-UHFFFAOYSA-N", 1.29,   1.22    ) ! 137
    CALL CalcH_TabS2("pentyl butanoate",                    "540-18-1", "CFNJLPHOBMVMNS-UHFFFAOYSA-N", DUMMY,  1.75    ) ! 138
    CALL CalcH_TabS2("phenanthrene",                         "85-01-8", "YNPNZTXNASCQKK-UHFFFAOYSA-N", 0.79,   0.35    ) ! 139
    CALL CalcH_TabS2("hydroxybenzene",                      "108-95-2", "ISWSIDIOOBJBQZ-UHFFFAOYSA-N", -1.37,  -1.51   ) ! 140
    CALL CalcH_TabS2("biphenyl",                             "92-52-4", "ZUOUZKKEUPVFJK-UHFFFAOYSA-N", 1.52,   1.15    ) ! 141
    CALL CalcH_TabS2("hydroquinone",                        "123-31-9", "QIGBRXMKCJKVMJ-UHFFFAOYSA-N", -4.14,  -5.62   ) ! 142
    CALL CalcH_TabS2("propanal",                            "123-38-6", "NBBJYMSMWIIQGU-UHFFFAOYSA-N", 1.11,   0.75    ) ! 143
    CALL CalcH_TabS2("propyl acetate",                      "109-60-4", "YKYONYBAUNKHLG-UHFFFAOYSA-N", 1.17,   1.04    ) ! 144
    CALL CalcH_TabS2("1-propanol",                           "71-23-8", "BDERNNFJNOPAEC-UHFFFAOYSA-N", -0.35,  -0.34   ) ! 145
    CALL CalcH_TabS2("pyridine",                            "110-86-1", "JUJWROOIHBZHMG-UHFFFAOYSA-N", DUMMY,  DUMMY   ) ! 146
    CALL CalcH_TabS2("sec-butanol",                          "78-92-2", "BTANRVKWQNVYAZ-UHFFFAOYSA-N", -0.08,  -0.28   ) ! 147
    CALL CalcH_TabS2("sec-butyl acetate",                   "105-46-4", "DCKVNWZUADLDEH-UHFFFAOYSA-N", 1.37,   1.34    ) ! 148
    CALL CalcH_TabS2("ethenylbenzene",                      "100-42-5", "PPBRXRYQALVLMV-UHFFFAOYSA-N", 2.16,   2.22    ) ! 149
    CALL CalcH_TabS2("tert-butanol",                         "75-65-0", "DKGAVHZHDRPRBM-UHFFFAOYSA-N", 0.36,   -0.56   ) ! 150
    CALL CalcH_TabS2("tert-butyl acetate",                  "540-88-5", "WMOVHXAZOJBABW-UHFFFAOYSA-N", 1.24,   1.32    ) ! 151
    CALL CalcH_TabS2("tert-butyl formate",                  "762-75-4", "RUPAXCPQAAOIPB-UHFFFAOYSA-N", 1.35,   0.88    ) ! 152
    CALL CalcH_TabS2("tert-pentyl acetate",                 "625-16-1", "JCCIFDCPHCKATH-UHFFFAOYSA-N", 1.28,   1.45    ) ! 153
    CALL CalcH_TabS2("1-tetradecanal",                      "124-25-4", "UHUFTBALEZWWIH-UHFFFAOYSA-N", 2.84,   1.95    ) ! 154
    CALL CalcH_TabS2("tetrahydro-2-furanmethanol",           "97-99-4", "BSYVTEYKTMYBMK-UHFFFAOYSA-N", -1.90,  -2.49   ) ! 155
    CALL CalcH_TabS2("methylbenzene",                       "108-88-3", "YXFVVABEGXRONW-UHFFFAOYSA-N", 2.55,   2.58    ) ! 156
    CALL CalcH_TabS2("trans-crotonaldehyde",                "123-73-9", "MLUCVPSAIODCQM-NSCUHMNNSA-N", -0.14,  -0.42   ) ! 157
    CALL CalcH_TabS2("trichloroethene",                      "79-01-6", "XSTXAVWGXDQKEL-UHFFFAOYSA-N", 2.81,   2.95    ) ! 158
    CALL CalcH_TabS2("triethanolamine",                     "102-71-6", "GSEJCLTVZPLZKY-UHFFFAOYSA-N", DUMMY,  DUMMY   ) ! 159
    CALL CalcH_TabS2("triethylphosphate",                    "78-40-0", "DQWPFSLDHJDLRL-UHFFFAOYSA-N", -2.08,  -1.97   ) ! 160
    CALL CalcH_TabS2("triethylamine",                       "121-44-8", "ZMANZCXQSJIPKH-UHFFFAOYSA-N", 0.81,   2.04    ) ! 161
    CALL CalcH_TabS2("phosphoric acid, trimethyl ester",    "512-56-1", "WVLBCYQITXONBZ-UHFFFAOYSA-N", DUMMY,  DUMMY   ) ! 162
    CALL CalcH_TabS2("trioxane",                            "110-88-3", "BGJSXRVXTHVRSN-UHFFFAOYSA-N", -0.24,  -0.69   ) ! 163
    CALL CalcH_TabS2("undecanal",                           "112-44-7", "KMPQYAYAQWNLME-UHFFFAOYSA-N", 1.90,   1.56    ) ! 164
    CALL CalcH_TabS2("ethenyl ethanoate",                   "108-05-4", "XTXRWKRVRITETP-UHFFFAOYSA-N", 1.45,   1.67    ) ! 165
    CALL CalcH_TabS2("vinyl methanoate",                    "692-45-5", "GFJVXXWOPWLRNU-UHFFFAOYSA-N", 1.03,   1.65    ) ! 166

  CONTAINS

    SUBROUTINE CalcH_Tab1 (chem_, casrn_, inchikey_, logHLC_exp, logHLC_pred)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_, inchikey_
      REAL,             INTENT(IN) :: logHLC_exp, logHLC_pred
      ! value from Yaws' Handbook of thermodynamic and physical properties of chemical compounds, 2003:
      type = "X"
      CALL SettypeX("yaws2003handbook")
      CALL CalcH(chem_, casrn_, inchikey_, logHLC_exp)
      ! predicted with COSMO-RS:
      type = "Q"
      CALL CalcH(chem_, casrn_, inchikey_, logHLC_pred)
    END SUBROUTINE CalcH_Tab1

    SUBROUTINE CalcH_Tab2 (chem_, casrn_, inchikey_, logHLC_exp, logHLC_pred)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_, inchikey_
      REAL,             INTENT(IN) :: logHLC_exp, logHLC_pred
      ! logHLC_exp is from Sander (2015) and not repeated here
      ! predicted with COSMO-RS:
      type = "Q"
      CALL CalcH(chem_, casrn_, inchikey_, logHLC_pred)
    END SUBROUTINE CalcH_Tab2

    SUBROUTINE CalcH_Tab3 (chem_, casrn_, inchikey_, logHLC)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_, inchikey_
      REAL,             INTENT(IN) :: logHLC
      ! predicted with COSMO-RS:
      type = "Q"
      CALL CalcH(chem_, casrn_, inchikey_, logHLC)
    END SUBROUTINE CalcH_Tab3

    SUBROUTINE CalcH_TabS2 (chem_, casrn_, inchikey_, logHLC_exp, logHLC_pred)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_, inchikey_
      REAL,             INTENT(IN) :: logHLC_exp, logHLC_pred
      ! value from Yaws' Handbook of thermodynamic and physical properties of chemical compounds, 2003:
      IF (ABS(logHLC_exp-DUMMY) > TINY(0.)) THEN
        type = "X"
        CALL SettypeX("yaws2003handbook")
        CALL CalcH(chem_, casrn_, inchikey_, logHLC_exp)
      ENDIF
      ! predicted with COSMO-RS:
      IF (ABS(logHLC_pred-DUMMY) > TINY(0.)) THEN
        type = "Q"
        CALL CalcH(chem_, casrn_, inchikey_, logHLC_pred)
      ENDIF
    END SUBROUTINE CalcH_TabS2

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, logHLC)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_, inchikey_
      REAL,             INTENT(IN) :: logHLC
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      IF (type=="Q") CALL MakeNote(TRIM(ref)//"-COSMO", "Calculated using the COSMO-RS method.")
      CALL Output(cH2O / (10.**logHLC * bar))
    END SUBROUTINE CalcH

  END SUBROUTINE ref3682

  !---------------------------------------------------------------------------

  SUBROUTINE ref3683 ! Hvpc [Pa*m3/mol]
    IMPLICIT NONE

    ref = "3683"
    ndata = 5
    ALLOCATE(temp(ndata))
    temp = (/ 278.15, 288.15, 298.15, 303.15, 313.15 /)
    ! Tab. A1 (assuming that Hvpc in [Pa*m3/mol] is listed here):
    CALL CalcH("PBDE-1",    "36563-47-0", "RRWFUWRLNIZICP-UHFFFAOYSA-N", (/  0.928,  1.216,  1.600,  1.804,  2.224 /))
    CALL CalcH("PBDE-2",     "6876-00-2", "AHDAKFFMKLQPTD-UHFFFAOYSA-N", (/  0.929,  1.216,  1.600,  1.804,  2.225 /))
    CALL CalcH("PBDE-3",      "101-55-3", "JDUYPUMQALQRCN-UHFFFAOYSA-N", (/  1.131,  1.419,  1.803,  2.007,  2.427 /))
    CALL CalcH("PBDE-4",    "51452-87-0", "JMSKYMHFNWGUJG-UHFFFAOYSA-N", (/  0.306,  0.593,  0.977,  1.181,  1.602 /))
    CALL CalcH("PBDE-5",   "446254-14-4", "JTYRXXKXOULVAP-UHFFFAOYSA-N", (/  0.572,  0.859,  1.243,  1.447,  1.868 /))
    CALL CalcH("PBDE-6",   "147217-72-9", "GODQTPRKFHOLPH-UHFFFAOYSA-N", (/  0.309,  0.596,  0.980,  1.184,  1.605 /))
    CALL CalcH("PBDE-7",   "171977-44-9", "JMCIHKKTRDLVCO-UHFFFAOYSA-N", (/  0.446,  0.734,  1.118,  1.322,  1.742 /))
    CALL CalcH("PBDE-8",   "147217-71-8", "RJQLQJZMLISKRJ-UHFFFAOYSA-N", (/  0.521,  0.809,  1.193,  1.397,  1.818 /))
    CALL CalcH("PBDE-9",   "337513-66-3", "URDWJMUOJJSXAE-UHFFFAOYSA-N", (/  0.377,  0.665,  1.049,  1.253,  1.673 /))
    CALL CalcH("PBDE-10",   "51930-04-2", "MUVDKHMQIZJFTC-UHFFFAOYSA-N", (/  0.363,  0.650,  1.034,  1.238,  1.659 /))
    CALL CalcH("PBDE-11",    "6903-63-5", "ALSVFJIXSNRBLE-UHFFFAOYSA-N", (/  0.393,  0.680,  1.064,  1.268,  1.689 /))
    CALL CalcH("PBDE-12",  "189084-59-1", "SUUJFDKVPDCZQZ-UHFFFAOYSA-N", (/  0.770,  1.057,  1.441,  1.645,  2.066 /))
    CALL CalcH("PBDE-13",   "83694-71-7", "BGPOVBPKODCMMN-UHFFFAOYSA-N", (/  0.540,  0.828,  1.212,  1.416,  1.836 /))
    CALL CalcH("PBDE-14",   "46438-88-4", "FOXBZJLXVUHYQZ-UHFFFAOYSA-N", (/  0.286,  0.573,  0.957,  1.161,  1.582 /))
    CALL CalcH("PBDE-15",    "2050-47-7", "YAWIAFUBXXPJMQ-UHFFFAOYSA-N", (/  0.690,  0.977,  1.361,  1.565,  1.986 /))
    CALL CalcH("PBDE-16",  "147217-74-1", "VRNGWCVCSHJUEJ-UHFFFAOYSA-N", (/ -0.060,  0.228,  0.612,  0.816,  1.236 /))
    CALL CalcH("PBDE-17",  "147217-75-2", "VYBFILXLBMWOLI-UHFFFAOYSA-N", (/ -0.083,  0.204,  0.588,  0.792,  1.213 /))
    CALL CalcH("PBDE-18",  "407606-55-7", "FAZLXBWRNJAGSV-UHFFFAOYSA-N", (/ -0.096,  0.192,  0.576,  0.780,  1.200 /))
    CALL CalcH("PBDE-19",  "147217-73-0", "YDFQHBRKURQGAH-UHFFFAOYSA-N", (/ -0.272,  0.015,  0.399,  0.604,  1.024 /))
    CALL CalcH("PBDE-20",  "147217-76-3", "RQJUBSPXDSGLRB-UHFFFAOYSA-N", (/  0.028,  0.315,  0.699,  0.903,  1.324 /))
    CALL CalcH("PBDE-21",  "337513-67-4", "RXWRVYYPLRPDOS-UHFFFAOYSA-N", (/  0.264,  0.551,  0.935,  1.139,  1.560 /))
    CALL CalcH("PBDE-22",  "446254-15-5", "WZHNIFQVNBINLF-UHFFFAOYSA-N", (/  0.207,  0.494,  0.878,  1.082,  1.503 /))
    CALL CalcH("PBDE-23",  "446254-16-6", "XQHLKDAUZRXBGC-UHFFFAOYSA-N", (/ -0.075,  0.212,  0.596,  0.800,  1.221 /))
    CALL CalcH("PBDE-24",  "218304-36-0", "GFLRHBRMAZDOIG-UHFFFAOYSA-N", (/  0.039,  0.327,  0.711,  0.915,  1.335 /))
    CALL CalcH("PBDE-25",  "147217-77-4", "AURKEOPYVUYTLO-UHFFFAOYSA-N", (/ -0.080,  0.207,  0.591,  0.795,  1.216 /))
    CALL CalcH("PBDE-26",  "337513-75-4", "VUOBKVBAFJQQDB-UHFFFAOYSA-N", (/ -0.112,  0.176,  0.560,  0.764,  1.185 /))
    CALL CalcH("PBDE-27",  "337513-53-8", "JUPZALSVNWJHII-UHFFFAOYSA-N", (/ -0.246,  0.042,  0.426,  0.630,  1.050 /))
    CALL CalcH("PBDE-28",   "41318-75-6", "UPNBETHEXPIWQX-UHFFFAOYSA-N", (/  0.033,  0.321,  0.705,  0.909,  1.329 /), &
      (/ 0.004, 0.369, 0.684, 0.899, 1.468 /))
    CALL CalcH("PBDE-29",  "337513-56-1", "LTMKAFUXYKEDLR-UHFFFAOYSA-N", (/  0.086,  0.373,  0.757,  0.961,  1.382 /))
    CALL CalcH("PBDE-30",  "155999-95-4", "TVZAPPGLBLTACB-UHFFFAOYSA-N", (/ -0.248,  0.039,  0.423,  0.627,  1.048 /))
    CALL CalcH("PBDE-31",   "65075-08-3", "PURZBWMLFRWRMG-UHFFFAOYSA-N", (/  0.002,  0.290,  0.674,  0.878,  1.298 /))
    CALL CalcH("PBDE-32",  "189084-60-4", "TYDVYKIQSZGUMV-UHFFFAOYSA-N", (/  0.066,  0.354,  0.738,  0.942,  1.362 /))
    CALL CalcH("PBDE-33",   "49690-94-0", "BUQBQEYUVAKJQK-UHFFFAOYSA-N", (/  0.180,  0.467,  0.851,  1.055,  1.476 /))
    CALL CalcH("PBDE-34",  "446254-17-7", "XMNXHCHZIPYCNA-UHFFFAOYSA-N", (/ -0.336, -0.049,  0.335,  0.539,  0.960 /))
    CALL CalcH("PBDE-35",  "147217-80-9", "CDVYKQPKJYPWRO-UHFFFAOYSA-N", (/  0.221,  0.508,  0.892,  1.096,  1.517 /))
    CALL CalcH("PBDE-36",  "147217-79-6", "XUKPJLVONRTECE-UHFFFAOYSA-N", (/ -0.158,  0.129,  0.513,  0.717,  1.138 /))
    CALL CalcH("PBDE-37",  "147217-81-0", "YALAYFVVZFORPV-UHFFFAOYSA-N", (/  0.327,  0.614,  0.998,  1.202,  1.623 /))
    CALL CalcH("PBDE-38",  "337513-54-9", "DPGVQKLGQZZLMI-UHFFFAOYSA-N", (/  0.285,  0.572,  0.956,  1.160,  1.581 /))
    CALL CalcH("PBDE-39",  "407606-57-9", "UFFNOPDHJNQYKD-UHFFFAOYSA-N", (/ -0.023,  0.265,  0.649,  0.853,  1.273 /))
    CALL CalcH("PBDE-40",  "337513-77-6", "SXSUUFZWSVMTRL-UHFFFAOYSA-N", (/ -0.402, -0.114,  0.270,  0.474,  0.894 /))
    CALL CalcH("PBDE-41",  "337513-68-5", "UAEBSKBXZAIRMX-UHFFFAOYSA-N", (/ -0.225,  0.062,  0.446,  0.650,  1.071 /))
    CALL CalcH("PBDE-42",  "446254-18-8", "HQDQKPAHIDGGMH-UHFFFAOYSA-N", (/ -0.437, -0.150,  0.234,  0.438,  0.859 /))
    CALL CalcH("PBDE-43",  "446254-19-9", "LKMQHSYDVDIECC-UHFFFAOYSA-N", (/ -0.535, -0.248,  0.136,  0.340,  0.761 /))
    CALL CalcH("PBDE-44",  "446254-20-2", "VBGBGTYMDIVKNK-UHFFFAOYSA-N", (/ -0.472, -0.185,  0.199,  0.403,  0.824 /))
    CALL CalcH("PBDE-45",  "446254-21-3", "VTFWUBIOZQCMQS-UHFFFAOYSA-N", (/ -0.580, -0.293,  0.091,  0.295,  0.716 /))
    CALL CalcH("PBDE-46",  "446254-22-4", "GBUUKJRFSKCMTB-UHFFFAOYSA-N", (/ -0.758, -0.471, -0.087,  0.117,  0.538 /))
    CALL CalcH("PBDE-47",    "5436-43-1", "XYBSIYMGXVUVGY-UHFFFAOYSA-N", (/ -0.648, -0.361,  0.023,  0.227,  0.648 /), &
      (/ -0.745, -0.252, -0.071, 0.250, 0.577 /))
    CALL CalcH("PBDE-48",  "337513-55-0", "FJGDNHOVDFREMP-UHFFFAOYSA-N", (/ -0.395, -0.108,  0.276,  0.480,  0.901 /))
    CALL CalcH("PBDE-49",  "243982-82-3", "QWVDUBDYUPHNHY-UHFFFAOYSA-N", (/ -0.653, -0.366,  0.018,  0.222,  0.643 /))
    CALL CalcH("PBDE-50",  "446254-23-5", "FXUAKFRJBKFDSY-UHFFFAOYSA-N", (/ -0.768, -0.481, -0.097,  0.107,  0.528 /))
    CALL CalcH("PBDE-51",  "189084-57-9", "WKBBBTLDLKYGBI-UHFFFAOYSA-N", (/ -0.847, -0.560, -0.175,  0.029,  0.449 /))
    CALL CalcH("PBDE-52",  "446254-24-6", "CDTHXJORUCZHMD-UHFFFAOYSA-N", (/ -0.668, -0.381,  0.003,  0.207,  0.628 /))
    CALL CalcH("PBDE-53",  "446254-25-7", "SDVQGIMOFXMKHR-UHFFFAOYSA-N", (/ -0.857, -0.570, -0.186,  0.018,  0.439 /))
    CALL CalcH("PBDE-54",  "446254-26-8", "WCDCHQGVTZHVSO-UHFFFAOYSA-N", (/ -0.940, -0.652, -0.268, -0.064,  0.356 /))
    CALL CalcH("PBDE-55",  "446254-27-9", "VIHUMJGEWQPWOT-UHFFFAOYSA-N", (/ -0.197,  0.090,  0.474,  0.678,  1.099 /))
    CALL CalcH("PBDE-56",  "446254-28-0", "NFOIVCGFYJIYIB-UHFFFAOYSA-N", (/ -0.168,  0.119,  0.503,  0.707,  1.128 /))
    CALL CalcH("PBDE-57",  "337513-82-3", "CSIFWDKYUJLQEB-UHFFFAOYSA-N", (/ -0.509, -0.222,  0.162,  0.366,  0.787 /))
    CALL CalcH("PBDE-58",  "446254-29-1", "SWOYBZHGPZIRHS-UHFFFAOYSA-N", (/ -0.835, -0.548, -0.164,  0.040,  0.461 /))
    CALL CalcH("PBDE-59",  "446254-30-4", "DMAMJZQQOWYEHT-UHFFFAOYSA-N", (/ -0.530, -0.243,  0.141,  0.345,  0.766 /))
    CALL CalcH("PBDE-60",  "446254-31-5", "ARERIMFZYPFJAV-UHFFFAOYSA-N", (/ -0.085,  0.202,  0.586,  0.790,  1.211 /))
    CALL CalcH("PBDE-61",  "446254-32-6", "NDRSXNBQWAOQPP-UHFFFAOYSA-N", (/ -0.218,  0.070,  0.454,  0.658,  1.079 /))
    CALL CalcH("PBDE-62",  "446254-33-7", "YIQYWYZZLOZVRM-UHFFFAOYSA-N", (/ -0.455, -0.168,  0.216,  0.420,  0.841 /))
    CALL CalcH("PBDE-63",  "446254-34-8", "HNICYXFGCWPYGC-UHFFFAOYSA-N", (/ -0.419, -0.131,  0.253,  0.457,  0.877 /))
    CALL CalcH("PBDE-64",  "446254-35-9", "LDCXVFJUWKKBNY-UHFFFAOYSA-N", (/ -0.233,  0.054,  0.438,  0.642,  1.063 /))
    CALL CalcH("PBDE-65",  "446254-36-0", "HPEUYVBOPJQVPN-UHFFFAOYSA-N", (/ -0.400, -0.112,  0.272,  0.476,  0.897 /))
    CALL CalcH("PBDE-66",  "189084-61-5", "DHUMTYRHKMCVAG-UHFFFAOYSA-N", (/ -0.389, -0.102,  0.282,  0.486,  0.907 /))
    CALL CalcH("PBDE-67",  "446254-37-1", "OARGWSONVLGXQA-UHFFFAOYSA-N", (/ -0.386, -0.099,  0.285,  0.489,  0.910 /))
    CALL CalcH("PBDE-68",  "446254-38-2", "UFWGRLCUOLLWAO-UHFFFAOYSA-N", (/ -0.913, -0.626, -0.242, -0.038,  0.383 /))
    CALL CalcH("PBDE-69",  "327185-09-1", "NHZNRCYNZJADTG-UHFFFAOYSA-N", (/ -0.711, -0.423, -0.039,  0.165,  0.586 /))
    CALL CalcH("PBDE-70",  "446254-39-3", "GHQMTYWQVJZWAR-UHFFFAOYSA-N", (/ -0.431, -0.143,  0.241,  0.445,  0.866 /))
    CALL CalcH("PBDE-71",  "189084-62-6", "COPAGYRSCJVION-UHFFFAOYSA-N", (/ -0.431, -0.144,  0.240,  0.444,  0.865 /))
    CALL CalcH("PBDE-72",  "446254-40-6", "GBBNZKQTOOZGIS-UHFFFAOYSA-N", (/ -0.700, -0.413, -0.029,  0.175,  0.596 /))
    CALL CalcH("PBDE-73",  "446254-41-7", "WQFLVWXBCRJAQN-UHFFFAOYSA-N", (/ -0.942, -0.655, -0.271, -0.067,  0.354 /))
    CALL CalcH("PBDE-74",  "446254-42-8", "LXCFDVVDUVPAGR-UHFFFAOYSA-N", (/ -0.310, -0.023,  0.361,  0.565,  0.986 /))
    CALL CalcH("PBDE-75",  "189084-63-7", "BWCNKMFFUGBFGB-UHFFFAOYSA-N", (/ -0.519, -0.232,  0.152,  0.356,  0.777 /))
    CALL CalcH("PBDE-76",  "446254-43-9", "NCSWBJSFVPJPPK-UHFFFAOYSA-N", (/ -0.393, -0.106,  0.278,  0.482,  0.903 /))
    CALL CalcH("PBDE-77",   "93703-48-1", "RYGLOWMCGZHYRQ-UHFFFAOYSA-N", (/ -0.073,  0.214,  0.598,  0.802,  1.223 /))
    CALL CalcH("PBDE-78",  "446254-45-1", "HWOBLTZZSVXBOJ-UHFFFAOYSA-N", (/ -0.180,  0.107,  0.491,  0.695,  1.116 /))
    CALL CalcH("PBDE-79",  "446254-48-4", "LELQGHJEUVRPEV-UHFFFAOYSA-N", (/ -0.386, -0.099,  0.285,  0.489,  0.910 /))
    CALL CalcH("PBDE-80",  "103173-66-6", "HFIOZJQRZKNPKJ-UHFFFAOYSA-N", (/ -0.759, -0.472, -0.088,  0.116,  0.537 /))
    CALL CalcH("PBDE-81",  "446254-50-8", "ULFOIXCXIWHJDS-UHFFFAOYSA-N", (/ -0.096,  0.192,  0.576,  0.780,  1.201 /))
    CALL CalcH("PBDE-82",  "327185-11-5", "RQMSPGJESCCPQX-UHFFFAOYSA-N", (/ -0.571, -0.284,  0.100,  0.304,  0.725 /))
    CALL CalcH("PBDE-83",  "446254-51-9", "XAHYSNUYJLNDBX-UHFFFAOYSA-N", (/ -0.865, -0.578, -0.194,  0.010,  0.431 /))
    CALL CalcH("PBDE-84",  "446254-52-0", "PPIZNRAVQHNLJM-UHFFFAOYSA-N", (/ -0.978, -0.690, -0.306, -0.102,  0.319 /))
    CALL CalcH("PBDE-85",  "182346-21-0", "DMLQSUZPTTUUDP-UHFFFAOYSA-N", (/ -0.833, -0.546, -0.162,  0.042,  0.463 /))
    CALL CalcH("PBDE-86",  "446254-53-1", "YMVWYUWOUOQCQP-UHFFFAOYSA-N", (/ -0.681, -0.394, -0.010,  0.194,  0.615 /))
    CALL CalcH("PBDE-87",  "446254-54-2", "WKYQUGCIKNOXFW-UHFFFAOYSA-N", (/ -2.743, -2.455, -2.071, -1.867, -1.447 /))
    CALL CalcH("PBDE-88",  "446254-55-3", "OPZUHBCVIZNZFB-UHFFFAOYSA-N", (/ -0.790, -0.503, -0.119,  0.085,  0.506 /))
    CALL CalcH("PBDE-89",  "446254-56-4", "XGFLJLJXVIMCNR-UHFFFAOYSA-N", (/ -0.966, -0.679, -0.295, -0.091,  0.330 /))
    CALL CalcH("PBDE-90",  "446254-57-5", "BATFXMGTVIESIQ-UHFFFAOYSA-N", (/ -1.076, -0.789, -0.405, -0.201,  0.220 /))
    CALL CalcH("PBDE-91",  "446254-58-6", "HWNJTZKDPNZUSO-UHFFFAOYSA-N", (/ -0.942, -0.655, -0.271, -0.067,  0.354 /))
    CALL CalcH("PBDE-92",  "446254-59-7", "QWSQOVAGRDRZLM-UHFFFAOYSA-N", (/ -1.004, -0.717, -0.333, -0.129,  0.292 /))
    CALL CalcH("PBDE-93",  "446254-60-0", "BRTPVPJQMWLDNO-UHFFFAOYSA-N", (/ -0.901, -0.613, -0.229, -0.025,  0.395 /))
    CALL CalcH("PBDE-94",  "446254-61-1", "JOPASNJHCFYVHD-UHFFFAOYSA-N", (/ -1.294, -1.007, -0.623, -0.419,  0.002 /))
    CALL CalcH("PBDE-95",  "446254-62-2", "BZDYRALIEYVMEP-UHFFFAOYSA-N", (/ -0.978, -0.691, -0.307, -0.103,  0.318 /))
    CALL CalcH("PBDE-96",  "446254-63-3", "ZFCJNRDWGBZUED-UHFFFAOYSA-N", (/ -1.307, -1.020, -0.636, -0.432, -0.011 /))
    CALL CalcH("PBDE-97",  "446254-64-4", "MAGYDGJRSCULJL-UHFFFAOYSA-N", (/ -0.730, -0.443, -0.059,  0.145,  0.566 /))
    CALL CalcH("PBDE-98",   "38463-82-0", "OCLWEJVGAUFXQU-UHFFFAOYSA-N", (/ -1.266, -0.979, -0.595, -0.391,  0.030 /))
    CALL CalcH("PBDE-99",   "60348-60-9", "WHPVYXDFIXRKLN-UHFFFAOYSA-N", (/ -0.912, -0.625, -0.241, -0.037,  0.384 /), &
      (/ -1.000, -0.770, -0.222, -0.012,  0.525 /))
    CALL CalcH("PBDE-100", "189084-64-8", "NSKIRYMHNFTRLR-UHFFFAOYSA-N", (/ -1.309, -1.022, -0.638, -0.434, -0.013 /), &
      (/ -1.222, -1.046, -0.620, -0.276, -0.036 /))
    CALL CalcH("PBDE-101", "446254-65-5", "QUZWDWNIWWAQDI-UHFFFAOYSA-N", (/ -0.864, -0.577, -0.193,  0.011,  0.432 /))
    CALL CalcH("PBDE-102", "446254-66-6", "JHFMCUVMAIQWRI-UHFFFAOYSA-N", (/ -1.157, -0.870, -0.486, -0.281,  0.139 /))
    CALL CalcH("PBDE-103", "446254-67-7", "RJEMKRNASVHYKR-UHFFFAOYSA-N", (/ -1.252, -0.964, -0.580, -0.376,  0.045 /))
    CALL CalcH("PBDE-104", "446254-68-8", "CRSCWEYUPUKHPI-UHFFFAOYSA-N", (/ -1.542, -1.255, -0.871, -0.667, -0.246 /))
    CALL CalcH("PBDE-105", "373594-78-6", "LBPWAGZGYNOKAM-UHFFFAOYSA-N", (/ -0.503, -0.216,  0.168,  0.372,  0.793 /))
    CALL CalcH("PBDE-106", "446254-69-9", "KLQKWMYXEWUAFP-UHFFFAOYSA-N", (/ -0.645, -0.358,  0.026,  0.230,  0.651 /))
    CALL CalcH("PBDE-107", "446254-70-2", "OMGVAMFMRSETEG-UHFFFAOYSA-N", (/ -0.997, -0.710, -0.326, -0.122,  0.299 /))
    CALL CalcH("PBDE-108", "446254-71-3", "VBKPKHVLHGOKOJ-UHFFFAOYSA-N", (/ -1.009, -0.721, -0.337, -0.133,  0.287 /))
    CALL CalcH("PBDE-109", "446254-72-4", "FXXXWTMLIQLDRP-UHFFFAOYSA-N", (/ -0.795, -0.508, -0.124,  0.080,  0.501 /))
    CALL CalcH("PBDE-110", "446254-73-5", "LESZGJVTZILBTK-UHFFFAOYSA-N", (/ -0.691, -0.404, -0.020,  0.184,  0.605 /))
    CALL CalcH("PBDE-111", "446254-74-6", "PCHDCOXHJBWEPW-UHFFFAOYSA-N", (/ -1.038, -0.751, -0.367, -0.163,  0.258 /))
    CALL CalcH("PBDE-112", "446254-75-7", "MFBMNSFADPTAKZ-UHFFFAOYSA-N", (/ -0.951, -0.663, -0.279, -0.075,  0.346 /))
    CALL CalcH("PBDE-113", "446254-76-8", "OGZHLJXRGZFVLI-UHFFFAOYSA-N", (/ -1.207, -0.920, -0.536, -0.332,  0.089 /))
    CALL CalcH("PBDE-114", "446254-77-9", "SFNAUTSNWPPDSY-UHFFFAOYSA-N", (/ -0.591, -0.304,  0.080,  0.284,  0.705 /))
    CALL CalcH("PBDE-115", "446254-78-0", "BKTLDVXDOVSTEV-UHFFFAOYSA-N", (/ -0.710, -0.423, -0.039,  0.165,  0.586 /))
    CALL CalcH("PBDE-116", "189084-65-9", "ACRQLFSHISNWRY-UHFFFAOYSA-N", (/ -0.504, -0.217,  0.167,  0.371,  0.792 /))
    CALL CalcH("PBDE-117", "446254-79-1", "SOJBOGWFDBDWEG-UHFFFAOYSA-N", (/ -0.652, -0.365,  0.019,  0.224,  0.644 /))
    CALL CalcH("PBDE-118", "446254-80-4", "VTMFEPLDDHZBGI-UHFFFAOYSA-N", (/ -0.679, -0.392, -0.008,  0.196,  0.617 /))
    CALL CalcH("PBDE-119", "189084-66-0", "KXEOYBYEJCRPGB-UHFFFAOYSA-N", (/ -0.976, -0.689, -0.305, -0.101,  0.320 /))
    CALL CalcH("PBDE-120", "417727-71-0", "AKSBEUHDCRZJAN-UHFFFAOYSA-N", (/ -0.942, -0.655, -0.271, -0.067,  0.354 /))
    CALL CalcH("PBDE-121", "446254-81-5", "GVGNVZBJVFDAAO-UHFFFAOYSA-N", (/ -1.487, -1.200, -0.816, -0.612, -0.191 /))
    CALL CalcH("PBDE-122", "446254-82-6", "CDNHGSPFIUITTN-UHFFFAOYSA-N", (/ -0.577, -0.290,  0.094,  0.298,  0.719 /))
    CALL CalcH("PBDE-123", "446254-83-7", "SBKMUEQNZNDYFW-UHFFFAOYSA-N", (/ -0.789, -0.501, -0.117,  0.087,  0.507 /))
    CALL CalcH("PBDE-124", "446254-84-8", "FGHJTAAHIFEHLT-UHFFFAOYSA-N", (/ -0.745, -0.458, -0.074,  0.130,  0.551 /))
    CALL CalcH("PBDE-125", "446254-85-9", "SESXKFPOVUVGLR-UHFFFAOYSA-N", (/ -0.837, -0.549, -0.165,  0.039,  0.460 /))
    CALL CalcH("PBDE-126", "366791-32-4", "SJNIIWPIAVQNRK-UHFFFAOYSA-N", (/ -0.463, -0.176,  0.208,  0.412,  0.833 /))
    CALL CalcH("PBDE-127", "446254-86-0", "RATMRXKBPDCKCZ-UHFFFAOYSA-N", (/ -0.747, -0.460, -0.076,  0.128,  0.549 /))
    CALL CalcH("PBDE-128", "182677-28-7", "WFLVELCLEGVBIH-UHFFFAOYSA-N", (/ -1.030, -0.742, -0.358, -0.154,  0.266 /))
    CALL CalcH("PBDE-129", "446254-87-1", "PRNCVYAUCSGSOE-UHFFFAOYSA-N", (/ -1.002, -0.715, -0.330, -0.126,  0.294 /))
    CALL CalcH("PBDE-130", "446254-88-2", "YURCHLXPAGSJHU-UHFFFAOYSA-N", (/ -1.285, -0.997, -0.613, -0.409,  0.011 /))
    CALL CalcH("PBDE-131", "446254-89-3", "MGKVPJFIGGBCBA-UHFFFAOYSA-N", (/ -1.731, -1.444, -1.060, -0.856, -0.435 /))
    CALL CalcH("PBDE-132", "446254-90-6", "FFEKBOKDYRZGRV-UHFFFAOYSA-N", (/ -1.314, -1.027, -0.643, -0.439, -0.018 /))
    CALL CalcH("PBDE-133", "446254-91-7", "XTBFPFHQPGZZJX-UHFFFAOYSA-N", (/ -1.514, -1.227, -0.843, -0.638, -0.218 /))
    CALL CalcH("PBDE-134", "446254-92-8", "MIBDGPWSGDWIQR-UHFFFAOYSA-N", (/ -1.394, -1.107, -0.723, -0.519, -0.098 /))
    CALL CalcH("PBDE-135", "446254-93-9", "AMGHASDTWACNCS-UHFFFAOYSA-N", (/ -1.650, -1.362, -0.978, -0.774, -0.354 /))
    CALL CalcH("PBDE-136", "446254-94-0", "NTWGDSLWLUPCDW-UHFFFAOYSA-N", (/ -1.612, -1.325, -0.941, -0.737, -0.316 /))
    CALL CalcH("PBDE-137", "446254-95-1", "HSTYYNPYXZYIAG-UHFFFAOYSA-N", (/ -1.196, -0.909, -0.525, -0.321,  0.100 /))
    CALL CalcH("PBDE-138", "182677-30-1", "IZFQCEZFGCMHOM-UHFFFAOYSA-N", (/ -1.148, -0.861, -0.477, -0.273,  0.148 /))
   !CALL CalcH("PBDE-139", "446254-96-2", "YESDYWNWEVPOLZ-UHFFFAOYSA-N", (/ 38.415,  38.702,  39.290,  39.711)
    CALL CalcH("PBDE-140", "243982-83-4", "FLRODCDHJZNIGA-UHFFFAOYSA-N", (/ -1.602, -1.314, -0.930, -0.726, -0.305 /))
    CALL CalcH("PBDE-141", "446254-97-3", "XTXIYMGRRUJOIT-UHFFFAOYSA-N", (/ -1.133, -0.846, -0.462, -0.258,  0.163 /))
    CALL CalcH("PBDE-142", "446254-98-4", "LJDGJCNHVGGOFW-UHFFFAOYSA-N", (/ -0.874, -0.587, -0.203,  0.001,  0.422 /))
    CALL CalcH("PBDE-143", "446254-99-5", "RQLZDUSZXOOBTM-UHFFFAOYSA-N", (/ -1.423, -1.136, -0.752, -0.548, -0.127 /))
    CALL CalcH("PBDE-144", "446255-00-1", "ZMSJCQOCTPYCQP-UHFFFAOYSA-N", (/ -1.431, -1.144, -0.760, -0.556, -0.135 /))
    CALL CalcH("PBDE-145", "446255-01-2", "BTKLHMBWCRVCLC-UHFFFAOYSA-N", (/ -1.736, -1.449, -1.065, -0.860, -0.440 /))
    CALL CalcH("PBDE-146", "446255-02-3", "HGXPYDNHBUCRTR-UHFFFAOYSA-N", (/ -1.444, -1.156, -0.772, -0.568, -0.147 /))
    CALL CalcH("PBDE-147", "116995-33-6", "OWBKWMDBTWHGHS-UHFFFAOYSA-N", (/ -1.428, -1.141, -0.757, -0.553, -0.132 /))
    CALL CalcH("PBDE-148", "446255-03-4", "OJMHGSMSQZEBFH-UHFFFAOYSA-N", (/ -1.778, -1.491, -1.107, -0.903, -0.482 /))
    CALL CalcH("PBDE-149", "446255-04-5", "UJOUSZKYGGTPFQ-UHFFFAOYSA-N", (/ -1.424, -1.137, -0.752, -0.548, -0.128 /))
    CALL CalcH("PBDE-150", "446255-05-6", "SQNOZOVDXXSLSG-UHFFFAOYSA-N", (/ -1.937, -1.650, -1.266, -1.062, -0.641 /))
    CALL CalcH("PBDE-151", "446255-06-7", "NGOQQUYCSISZMY-UHFFFAOYSA-N", (/ -1.364, -1.076, -0.692, -0.488, -0.068 /))
    CALL CalcH("PBDE-152", "446255-07-8", "BYBJJARTBKUIJD-UHFFFAOYSA-N", (/ -1.692, -1.405, -1.021, -0.817, -0.396 /))
    CALL CalcH("PBDE-153",  "68631-49-2", "RZXIRSKYBISPGF-UHFFFAOYSA-N", (/ -1.303, -1.016, -0.632, -0.428, -0.007 /), &
      (/ -1.222, -1.097, -0.585, -0.357, -0.050 /))
    CALL CalcH("PBDE-154", "207122-15-4", "VHNPZYZQKWIWOD-UHFFFAOYSA-N", (/ -1.751, -1.464, -1.080, -0.876, -0.455 /), &
      (/ -1.523, -1.155, -1.097, -0.538, -0.347 /))
    CALL CalcH("PBDE-155",  "35854-94-5", "HRSCBOSGEKXXSI-UHFFFAOYSA-N", (/ -2.108, -1.821, -1.437, -1.233, -0.812 /))
    CALL CalcH("PBDE-156", "405237-85-6", "JSDPCMJWYRDQEV-UHFFFAOYSA-N", (/ -0.907, -0.619, -0.235, -0.031,  0.389 /))
    CALL CalcH("PBDE-157", "446255-08-9", "JUOAMVUIJQJZSZ-UHFFFAOYSA-N", (/ -0.996, -0.709, -0.325, -0.121,  0.300 /))
    CALL CalcH("PBDE-158", "446255-09-0", "KRYHHTVQOOJNHQ-UHFFFAOYSA-N", (/ -1.144, -0.857, -0.473, -0.269,  0.152 /))
    CALL CalcH("PBDE-159", "446255-10-3", "IDYFFNCFLRCOPZ-UHFFFAOYSA-N", (/ -1.191, -0.903, -0.519, -0.315,  0.105 /))
    CALL CalcH("PBDE-160", "446255-11-4", "OCVOYHGOXIIONK-UHFFFAOYSA-N", (/ -1.157, -0.870, -0.486, -0.282,  0.139 /))
    CALL CalcH("PBDE-161", "446255-12-5", "WEYWRBBPPKSRGU-UHFFFAOYSA-N", (/ -1.671, -1.384, -1.000, -0.795, -0.375 /))
    CALL CalcH("PBDE-162", "446255-13-6", "UKPNCLHMNJCGCJ-UHFFFAOYSA-N", (/ -1.141, -0.854, -0.470, -0.266,  0.155 /))
    CALL CalcH("PBDE-163", "446255-14-7", "NUEAHMLXQFHEJN-UHFFFAOYSA-N", (/ -1.085, -0.798, -0.414, -0.210,  0.211 /))
    CALL CalcH("PBDE-164", "446255-15-8", "UJVYVXIHTJOJBZ-UHFFFAOYSA-N", (/ -1.207, -0.920, -0.536, -0.332,  0.089 /))
    CALL CalcH("PBDE-165", "446255-16-9", "KXERERDGMTWBGZ-UHFFFAOYSA-N", (/ -1.613, -1.325, -0.941, -0.737, -0.316 /))
    CALL CalcH("PBDE-166", "189084-58-0", "KVYODBMKQYVNEK-UHFFFAOYSA-N", (/ -0.808, -0.521, -0.137,  0.067,  0.488 /))
    CALL CalcH("PBDE-167", "446255-17-0", "NMUPLZRHSXJCJQ-UHFFFAOYSA-N", (/ -1.204, -0.916, -0.532, -0.328,  0.093 /))
    CALL CalcH("PBDE-168",  "53551-87-4", "HWZAPXGFMVEGPW-UHFFFAOYSA-N", (/ -1.420, -1.132, -0.748, -0.544, -0.124 /))
    CALL CalcH("PBDE-169", "446255-18-1", "JKFBMDHBJYKFKL-UHFFFAOYSA-N", (/ -0.811, -0.524, -0.140,  0.064,  0.485 /))
    CALL CalcH("PBDE-170", "327185-13-7", "DLPNCMQTNWLTHD-UHFFFAOYSA-N", (/ -1.409, -1.122, -0.738, -0.534, -0.113 /))
    CALL CalcH("PBDE-171", "446255-19-2", "FRMMMROUUPQUMZ-UHFFFAOYSA-N", (/ -1.782, -1.495, -1.111, -0.907, -0.486 /))
    CALL CalcH("PBDE-172", "407606-59-1", "DSRRSKFMOJQETR-UHFFFAOYSA-N", (/ -1.923, -1.636, -1.252, -1.048, -0.627 /))
    CALL CalcH("PBDE-173", "446255-20-5", "NLBLNZDNOSSGPW-UHFFFAOYSA-N", (/ -1.546, -1.259, -0.875, -0.671, -0.250 /))
    CALL CalcH("PBDE-174", "446255-21-6", "VUUWOHUOYUGBEO-UHFFFAOYSA-N", (/ -1.794, -1.506, -1.122, -0.918, -0.498 /))
    CALL CalcH("PBDE-175",   "6255-22-7", "YATZWTXATDYQCK-UHFFFAOYSA-N", (/ -1.945, -1.658, -1.274, -1.069, -0.649 /))
    CALL CalcH("PBDE-176", "407606-61-5", "SWUALKCOTZOSMY-UHFFFAOYSA-N", (/ -2.113, -1.825, -1.441, -1.237, -0.816 /))
    CALL CalcH("PBDE-177", "446255-23-8", "ZHUHLPXIJIBQBJ-UHFFFAOYSA-N", (/ -1.715, -1.428, -1.044, -0.840, -0.419 /))
    CALL CalcH("PBDE-178", "446255-24-9", "UWUVZUPEEORCRG-UHFFFAOYSA-N", (/ -1.838, -1.551, -1.167, -0.963, -0.542 /))
    CALL CalcH("PBDE-179", "446255-25-0", "COVXWWKOLMNRQE-UHFFFAOYSA-N", (/ -2.058, -1.771, -1.387, -1.183, -0.762 /))
    CALL CalcH("PBDE-180", "446255-26-1", "STMBXVOJNOJRPZ-UHFFFAOYSA-N", (/ -1.581, -1.294, -0.910, -0.706, -0.285 /))
    CALL CalcH("PBDE-181", "189084-67-1", "GVNRIAPLVGNZPL-UHFFFAOYSA-N", (/ -1.668, -1.381, -0.997, -0.793, -0.372 /))
    CALL CalcH("PBDE-182", "442690-45-1", "ZYHDTADADSNMLV-UHFFFAOYSA-N", (/ -1.997, -1.710, -1.326, -1.122, -0.701 /))
    CALL CalcH("PBDE-183", "207122-16-5", "ILPSCQCLBHQUEM-UHFFFAOYSA-N", (/ -1.829, -1.542, -1.158, -0.954, -0.533 /))
    CALL CalcH("PBDE-184", "117948-63-7", "JHDCZVAQPRXHEL-UHFFFAOYSA-N", (/ -2.291, -2.004, -1.620, -1.416, -0.995 /))
    CALL CalcH("PBDE-185", "405237-86-7", "YRNMIFAQDSUFTR-UHFFFAOYSA-N", (/ -1.422, -1.135, -0.751, -0.547, -0.126 /))
    CALL CalcH("PBDE-186", "446255-27-2", "WUFQDCMRKKDNSF-UHFFFAOYSA-N", (/ -1.866, -1.579, -1.195, -0.991, -0.570 /))
    CALL CalcH("PBDE-187", "446255-28-3", "RFZPXOBFDARWHV-UHFFFAOYSA-N", (/ -1.690, -1.402, -1.018, -0.814, -0.393 /))
    CALL CalcH("PBDE-188", "116995-32-5", "YGYDHFDPVGAMTL-UHFFFAOYSA-N", (/ -2.256, -1.968, -1.584, -1.380, -0.960 /))
    CALL CalcH("PBDE-189", "259087-35-9", "CQVLRTUESBMMJW-UHFFFAOYSA-N", (/ -1.365, -1.078, -0.694, -0.490, -0.069 /))
    CALL CalcH("PBDE-190", "189084-68-2", "OUEYHQIMJGHOQN-UHFFFAOYSA-N", (/ -1.319, -1.032, -0.648, -0.443, -0.023 /))
    CALL CalcH("PBDE-191", "446255-30-7", "BNBFKFHSIPERIM-UHFFFAOYSA-N", (/ -1.642, -1.355, -0.971, -0.767, -0.346 /))
    CALL CalcH("PBDE-192", "407578-53-4", "ABLZOLAUBUSUHT-UHFFFAOYSA-N", (/ -1.830, -1.543, -1.159, -0.955, -0.534 /))
    CALL CalcH("PBDE-193", "446255-34-1", "AUFJSWANTKXCFZ-UHFFFAOYSA-N", (/ -1.582, -1.295, -0.911, -0.707, -0.286 /))
    CALL CalcH("PBDE-194",  "32536-52-0", "ORYGKUIDIMIRNN-UHFFFAOYSA-N", (/ -1.837, -1.550, -1.166, -0.962, -0.541 /))
    CALL CalcH("PBDE-195", "446255-38-5", "GPQLSLKPHQEEOP-UHFFFAOYSA-N", (/ -1.908, -1.621, -1.237, -1.033, -0.612 /))
    CALL CalcH("PBDE-196", "446255-39-6", "IEWFKOVTVJNWFF-UHFFFAOYSA-N", (/ -2.285, -1.998, -1.614, -1.410, -0.989 /))
    CALL CalcH("PBDE-197", "117964-21-3", "AAFUUKPTSPVXJH-UHFFFAOYSA-N", (/ -2.570, -2.283, -1.899, -1.695, -1.274 /))
    CALL CalcH("PBDE-198", "446255-42-1", "IBKRHVDFFHQOSC-UHFFFAOYSA-N", (/ -1.984, -1.697, -1.313, -1.109, -0.688 /))
    CALL CalcH("PBDE-199", "446255-43-2", "JNSLJYRXDGBNBE-UHFFFAOYSA-N", (/ -2.241, -1.953, -1.569, -1.365, -0.944 /))
    CALL CalcH("PBDE-200", "446255-46-5", "JWMXGEPFVCRXQR-UHFFFAOYSA-N", (/ -2.287, -2.000, -1.616, -1.412, -0.991 /))
    CALL CalcH("PBDE-201", "446255-50-1", "HQWFMMKREWXIGN-UHFFFAOYSA-N", (/ -2.551, -2.264, -1.880, -1.676, -1.255 /))
    CALL CalcH("PBDE-202",  "67797-09-5", "AHNZLQAZTWRRDW-UHFFFAOYSA-N", (/ -2.474, -2.187, -1.803, -1.599, -1.178 /))
    CALL CalcH("PBDE-203", "337513-72-1", "RTUZOQFRIPIWPS-UHFFFAOYSA-N", (/ -2.028, -1.741, -1.357, -1.152, -0.732 /))
    CALL CalcH("PBDE-204", "446255-54-5", "YZABCBOJTHQTSX-UHFFFAOYSA-N", (/ -2.509, -2.222, -1.838, -1.634, -1.213 /))
    CALL CalcH("PBDE-205", "446255-56-7", "CVMKCYDBEYHNBM-UHFFFAOYSA-N", (/ -1.970, -1.682, -1.298, -1.094, -0.673 /))
    CALL CalcH("PBDE-206",  "63387-28-0", "CYRHBNRLQMLULE-UHFFFAOYSA-N", (/ -2.389, -2.102, -1.718, -1.514, -1.093 /))
    CALL CalcH("PBDE-207", "437701-79-6", "IEEVDIAVLGLVOW-UHFFFAOYSA-N", (/ -2.809, -2.522, -2.138, -1.934, -1.513 /))
    CALL CalcH("PBDE-208", "437701-78-5", "ASGZXYIDLFWXID-UHFFFAOYSA-N", (/ -2.760, -2.473, -2.089, -1.885, -1.464 /))
    CALL CalcH("PBDE-209",   "1163-19-5", "WHHGLZMJPXIBIX-UHFFFAOYSA-N", (/ -2.151, -1.864, -1.480, -1.276, -0.855 /), &
      (/ -2.000, -1.699, -1.398, -0.960, -0.721 /))
    DEALLOCATE(temp)

CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, Pre_lgH, Exp_lgH)
      IMPLICIT NONE
      CHARACTER(LEN=*),             INTENT(IN) :: chem_, casrn_, inchikey_
      REAL, DIMENSION(:),           INTENT(IN) :: Pre_lgH
      REAL, DIMENSION(:), OPTIONAL, INTENT(IN) :: Exp_lgH
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      type = "Q"
      CALL MakeNote("supplement")
      CALL HTdep(temp, 10.**(-Pre_lgH), Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
      IF (PRESENT(Exp_lgH)) THEN
        type = "M"
        CALL MakeNote("supplement")
        CALL HTdep(temp, 10.**(-Exp_lgH), Hominus, mindHR)
        CALL Output(Hominus, mindHR, r2)
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3683

  !---------------------------------------------------------------------------

  SUBROUTINE ref3684 ! Hscp [M/atm]
    IMPLICIT NONE

    ref = "3684"
    type = "M"
    ! Tab. 2, this work, best estimate:
    CALL CalcH("PPA Propionic acid",   "79-09-4", 7200.)
    CALL CalcH("IBA Isobutyric acid",  "79-31-2", 4700.)
    CALL CalcH("BTA n-butyric acid",  "107-92-6", 4400.)
    CALL CalcH("IVA Isovaleric acid", "503-74-2", 2700.)
    CALL CalcH("VLA n-valeric acid",  "109-52-4", 2400.)
    CALL CalcH("HXA Hexanoic acid",   "142-62-1", 1000.)
    CALL CalcH("HPA Heptanoic acid",  "111-14-8", 1500.)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, HLC)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: HLC
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL Output(Hscp_TO_HscpSI * HLC)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3684

  !---------------------------------------------------------------------------

  SUBROUTINE ref3685 ! Hscp [M/atm]
    IMPLICIT NONE

    ref = "3685"
    type = "M"
    ! Tab. 1, intrinsic KH from last column:
    CALL CalcH("1-propanamine",      "107-10-8", &
      (/ 298., 293., 288., 283., 278. /), (/ 58.4, 77.7, 122.9, 189.7, 254.9 /)   )
    CALL CalcH("dipropylamine",      "142-84-7", &
      (/ 298., 293., 288., 283., 278. /), (/ 12.8, 19.5, 33.4, 57.8, 109.8 /)     )
    CALL CalcH("trimethylamine",      "75-50-3", &
      (/ 298., 293., 288., 283., 278. /), (/ 8.5, 12.6, 18.3, 25.8, 40.2 /)       )
    CALL CalcH("allylamine",         "107-11-9", &
      (/ 298., 293., 288., 283., 278. /), (/ 103.2, 143.2, 197.6, 272.2, 385.4 /) )
    CALL CalcH("4-methylmorpholine", "109-02-4", &
      (/ 308., 303., 298., 293. /),       (/ 452.1, 619.9, 951.9, 2583.3 /)       )

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp_, KHprime)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp_, KHprime
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp_, Hscp_TO_HscpSI * KHprime, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3685

  !---------------------------------------------------------------------------

  SUBROUTINE ref3686 ! Hscp [M/atm]
    IMPLICIT NONE

    ref = "3686"
    type = "M"
    ! Tab. 2, intrinsic KH:
    CALL CalcH("triethylamine", "121-44-8", &
      (/ 298., 293., 288., 283., 278. /), (/ 5.40, 7.95, 13.08, 22.07, 39.77 /))

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp_, KHprime)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp_, KHprime
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp_, Hscp_TO_HscpSI * KHprime, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3686

  !---------------------------------------------------------------------------

  ! ref3687 NOTUSED: Group-specific QSPR models are presented but it is
  !                  not clear which data in the supplement are
  !                  recommended.

  !---------------------------------------------------------------------------

  ! ref3688 NOTUSED: Only HLC for peanut oil, not for aqueous solutions.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3689 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3689"
    type = "Q"
    ! Tab. 1:
    CALL CalcH("benzene",                      "71-43-2", -0.65)
    CALL CalcH("methylbenzene",               "108-88-3", -0.56)
    CALL CalcH("propylbenzene",               "103-65-1", -0.39)
    CALL CalcH("fluorobenzene",               "462-06-6", -0.59) ! duplicate entry for fluorobenzene
    CALL CalcH("fluorobenzene",               "462-06-6", -0.74) ! duplicate entry for fluorobenzene
    CALL CalcH("iodobenzene",                 "591-50-4", -1.28)
    CALL CalcH("1,2-dichlorobenzene",          "95-50-1", -1.00)
    CALL CalcH("1,2,3-trichlorobenzene",       "87-61-6", -1.30)
    CALL CalcH("1,2,3,5-tetrachlorobenzene",  "634-90-2", -0.63)
    ! NOTUSED: which (o,m,p?)-dibromobenzene?             -1.07
    ! NOTUSED: which (o,m,p?)-chlorotoluene?              -0.76
    CALL CalcH("1-bromo-4-methylbenzene",     "106-38-7", -1.02)
    CALL CalcH("hydroxybenzene",              "108-95-2", -4.79)
    CALL CalcH("4-bromophenol",               "106-41-2", -5.21)
    CALL CalcH("4-hydroxychlorobenzene",      "106-48-9", -4.77)
    CALL CalcH("1-hydroxy-2-methylbenzene",    "95-48-7", -4.30)
    CALL CalcH("4-nitrophenol",               "100-02-7", -7.77)
    CALL CalcH("1-chloronaphthalene",          "90-13-1", -1.45)
    CALL CalcH("methoxybenzene",              "100-66-3", -1.68)
    CALL CalcH("benzenenitrile",              "100-47-0", -2.50)
    CALL CalcH("benzaldehyde",                "100-52-7", -2.95)
    CALL CalcH("acetophenone (hypnone)",       "98-86-2", -3.36)
    CALL CalcH("4-nitrotoluene",               "99-99-0", -2.55)
    CALL CalcH("2-methyl-1,3-dinitrobenzene", "606-20-2", -3.61)
    CALL CalcH("4-bromonitrobenzene",         "586-78-7", -2.74)
    CALL CalcH("pyridine",                    "110-86-1", -3.44)
    CALL CalcH("4-methylpyridine",            "108-89-4", -3.61)
    CALL CalcH("4-ethylpyridine",             "536-75-4", -3.46)
    CALL CalcH("2,4-dimethylpyridine",        "108-47-4", -3.56)
    CALL CalcH("ethane",                       "74-84-0",  1.31)
    CALL CalcH("butane",                      "106-97-8",  1.58)
    CALL CalcH("octane",                      "111-65-9",  2.12)
    CALL CalcH("ethene",                       "74-85-1",  0.94)
    CALL CalcH("n-butene",                    "106-98-9",  1.01) ! assume this is 1-butene
    CALL CalcH("hexane",                      "110-54-3",  1.25)
    CALL CalcH("chloromethane",                "74-87-3", -0.39)
    CALL CalcH("tetrachloromethane",           "56-23-5",  0.07)
    CALL CalcH("1,2-dichloroethane",          "107-06-2", -1.27)
    CALL CalcH("1,1,2-trichloroethane",        "79-00-5", -1.43)
    CALL CalcH("pentachloroethane",            "76-01-7", -1.00)
    CALL CalcH("1-chloropropane",             "540-54-5", -0.26)
    CALL CalcH("chloroethene",                 "75-01-4", -0.92)
    CALL CalcH("trichloroethene",              "79-01-6", -0.32)
    CALL CalcH("allyl chloride",              "107-05-1", -0.42)
    CALL CalcH("acetic acid",                  "64-19-7", -4.91)
    CALL CalcH("methanol",                     "67-56-1", -3.72)
    CALL CalcH("2-propen-1-ol",               "107-18-6", -3.69)
    CALL CalcH("1-butanol",                    "71-36-3", -3.46)
    CALL CalcH("1-hexanol",                   "111-27-3", -3.20)
    CALL CalcH("1-octanol",                   "111-87-5", -3.01)
    CALL CalcH("cyclohexanol",                "108-93-0", -3.61)
    CALL CalcH("methyl methanoate",           "107-31-3", -2.04)
    CALL CalcH("methyl ethanoate",             "79-20-9", -2.26)
    CALL CalcH("ethyl propanoate",            "105-37-3", -2.05)
    CALL CalcH("methyl butanoate",            "623-42-7", -2.08)
    CALL CalcH("methyl pentanoate",           "624-24-8", -1.86)
    CALL CalcH("methyl hexanoate",            "106-70-7", -1.82)
    CALL CalcH("methyl octanoate",            "111-11-5", -1.50)
    CALL CalcH("diethyl ether",                "60-29-7", -1.28)
    CALL CalcH("butyl ether",                 "142-96-1", -0.61)
    CALL CalcH("ethanal",                      "75-07-0", -2.57)
    CALL CalcH("hexanal",                      "66-25-1", -2.06)
    CALL CalcH("heptanal",                    "111-71-7", -1.96)
    CALL CalcH("propanone",                    "67-64-1", -2.79)
    CALL CalcH("butanone",                     "78-93-3", -2.72)
    CALL CalcH("2-undecanone",                "112-12-9", -1.58)
    CALL CalcH("ethanamine",                   "75-04-7", -3.38)
    CALL CalcH("1-butanamine",                "109-73-9", -3.21)
    CALL CalcH("dimethylamine",               "124-40-3", -3.14)
    CALL CalcH("triethylamine",               "121-44-8", -2.22)
    CALL CalcH("ethanenitrile",                "75-05-8", -2.85)
    CALL CalcH("nitroethane",                  "79-24-3", -2.72)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, logH)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL, INTENT(IN)             :: logH
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL MakeNote("unknownT")
      CALL Output(Hvcc_TIMES_HscpSI_atT0 / (10.**logH))

    END SUBROUTINE CalcH

  END SUBROUTINE ref3689

  !---------------------------------------------------------------------------

  SUBROUTINE ref3690 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3690"
    type = "M"
    ! Tab. 2:
    CALL CalcH("CH4",    "74-82-8", &
      (/ 283., 298., 313. /), &
      (/ 22.0, 30.0, 36.0 /))
    CALL CalcH("CH2Cl2", "75-09-2", &
      (/ 278.,  283.,  288.,  293.,  298.,  303., 308., 313., 323., 333., 343. /), &
      (/ 0.040, 0.054, 0.067, 0.073, 0.095, 0.12, 0.14, 0.15, 0.21, 0.27, 0.40 /))
    CALL CalcH("CHCl3",  "67-66-3", &
      (/ 278.,  283.,  288.,  293., 298., 303., 308., 313., 323., 333., 343. /), &
      (/ 0.044, 0.068, 0.094, 0.11, 0.17, 0.20, 0.23, 0.30, 0.47, 0.59, 0.87 /))
    CALL CalcH("CCl4",   "56-23-5", &
      (/ 278., 283., 288., 293., 298., 303., 308., 313., 323., 333., 343. /), &
      (/ 0.47, 0.58, 0.82, 1.0,  1.2,  1.5,  1.9,  2.3,  3.0,  4.0,  5.0 /))

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, Kc)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, Kc
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp, Hvcc_TO_HscpSI(Kc, temp), Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3690

  !---------------------------------------------------------------------------

  SUBROUTINE ref3691 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3691"
    type = "M"
    ndata = 4
    ALLOCATE(temp(ndata))
    ! Tab. 5:
    temp = (/ 318.15, 323.15, 328.15, 333.15 /)
    CALL CalcH("DBE  dibutyl ether",            "142-96-1", (/ -0.41, -0.09,  0.06,  0.20 /), -0.52 )
    CALL CalcH("DIPE diisopropyl ether",        "108-20-3", (/ -0.87, -0.61, -0.52, -0.35 /), -1.02 )
    CALL CalcH("ETBE ethyl tert-butyl ether",   "637-92-3", (/ -1.35, -0.94, -0.80, -0.53 /), -1.55 )
    CALL CalcH("MTBE methyl tert-butyl ether", "1634-04-4", (/ -1.19, -1.11, -1.08, -0.86 /), -1.32 )
    CALL CalcH("PVE  propyl vinyl ether",       "764-47-6", (/ -0.49, -0.37, -0.26, -0.22 /), -0.59 )
    CALL CalcH("TAME 2-methoxy-2-methylbutane", "994-05-8", (/ -1.35, -1.00, -0.85, -0.78 /), -1.62 )
    DEALLOCATE(temp)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, Kc, Kc298)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: Kc
      REAL,               INTENT(IN) :: Kc298
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp, Hvcc_TO_HscpSI(10.**Kc, temp), Hominus, mindHR)
      ! compare my extrapolation to last column in Tab. 5:
      ! PRINT *, chem_, Hominus, Hvcc_TO_HscpSI(10.**Kc298, T0)
      ! try regression parameters in Tab. 6:
      ! 4.13    - 1218.98 / 298.15 => 0.04152104645 but should be -0.52
      ! 5513.2  - 15.68   / 298.15 => 5513.14740902 but should be -1.02
      ! 6686.2  - 18.85   / 298.15 => 6686.13677679 but should be -1.55
      ! 6268.09 - 16.8    / 298.15 => 6268.03365252 but should be -1.32
      ! 2437.51 - 6.82    / 298.15 => 2437.48712561 but should be -0.59
      ! 25.13   - 8889.86 / 298.15 => -4.686736542  but should be -1.62
      CALL MakeNote(TRIM(ref), &
        TRIM(citet()) // " present regression parameters in their Table~6 and " // &
        "values extrapolated to 298.15\,\unit{K} in their Table~5. However, I was not " // &
        "able to reproduce their calculation. The data shown here are from my " // &
        "own regression of the measured data between 318.15\,\unit{K} and 333.15\,\unit{K}.")
      CALL MakeNote("highTextrapol")
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3691

  !---------------------------------------------------------------------------

  SUBROUTINE ref3692 ! Hvpc [atm*m3/mol]
    IMPLICIT NONE

    ref = "3692"
    type = "M"
    ! Tabs. 3,4:
    CALL CalcH("TCE trichloroethene", "79-01-6", &
      (/ 294., 323., 348., 372., 390. /), &
      (/ 0.006942, 0.025183, 0.048869, 0.067645, 0.060473 /), &
      (/ 0.2876, 0.9497, 1.7106, 2.2151, 1.8889 /))
    CALL CalcH("PCE tetrachloroethene", "127-18-4", &
      (/ 295., 324., 348., 374., 397., 423., 434. /), &
      (/ 0.017574, 0.057607, 0.121925, 0.182980, 0.189062, 0.199355, 0.195844 /), &
      (/ 0.7256, 2.1658, 4.2679, 5.9599, 5.8014, 5.7414, 5.4974 /))

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, Hc, DDC)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, Hc, DDC
      INTEGER :: i
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep_ABC(temp, 1./(atm*Hc), Hominus, mindHR)
      DO i=1,SIZE(temp)
        CALL consistency_check(1./(atm*Hc(i)), Hvcc_TO_HscpSI(DDC(i),temp(i)), &
          "The data listed as Hc and DDC") ! , verbose=.TRUE.)
      ENDDO
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3692

  !---------------------------------------------------------------------------

  SUBROUTINE ref3693 ! special definition (psat and gamma)
    IMPLICIT NONE

    ref = "3693"
    type = "V"
    ! Tabs. 1,2:
    CALL CalcH("methanol",                         "67-56-1", 18.5875,  3626.55,  -34.29,   "a", &
      (/ 283.15, 293.15, 298.15, 303.15, 313.15 /), (/ 1.41,  1.55,  1.83,  2.12,  2.32  /))
    CALL CalcH("ethanol",                          "64-17-5", 18.9119,  3803.98,  -41.68,   "a", &
      (/ 283.15, 293.15, 298.15, 303.15, 313.15 /), (/ 3.20,  3.82,  4.14,  4.42,  4.75  /))
    CALL CalcH("1-propanol",                       "71-23-8", 17.5439,  3166.38,  -80.15,   "a", &
      (/ 283.15, 293.15, 298.15, 303.15, 313.15 /), (/ 8.50,  10.3,  11.7,  13.2,  15.3  /))
    CALL CalcH("2-propanol",                       "67-63-0", 18.6929,  3640.20,  -53.54,   "a", &
      (/ 283.15, 293.15, 298.15, 303.15, 313.15 /), (/ 5.16,  7.22,  8.21,  9.14,  10.8  /))
    CALL CalcH("diethyl ether",                    "60-29-7", 16.0828,  2511.29,  -41.95,   "a", &
      (/ 283.15, 293.15, 298.15, 303.15         /), (/ 50.0,  68.0,  77.0,  85.9         /))
    CALL CalcH("methyl tert-butyl ether",        "1634-04-4", 6.01607,  1126.594, 225.860,  "b", &
      (/ 283.15, 293.15, 298.15, 303.15, 313.15 /), (/ 76.4,  111.,  125.,  137.,  159.  /))
    CALL CalcH("diisopropyl ether",               "108-20-3", 16.3417,  2895.73,  -43.15,   "a", &
      (/ 283.15, 293.15, 298.15, 303.15, 313.15 /), (/ 372.,  528.,  616.,  696.,  789.  /))
    CALL CalcH("di(2,2,2-trifluoroethyl) ether",  "333-36-8", 6.324088, 1218.754, -54.673,  "c", &
      (/ 283.15, 293.15, 298.15, 303.15, 313.15 /), (/ 2340., 1840., 1650., 1470., 1090. /))

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, A, B, C, footnote, temp, gamma)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL,               INTENT(IN) :: A, B, C
      CHARACTER,          INTENT(IN) :: footnote
      REAL, DIMENSION(:), INTENT(IN) :: temp, gamma
      INTEGER :: i
      REAL :: psat
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ALLOCATE(Harray(SIZE(temp)))
      DO i=1,SIZE(temp)
        SELECT CASE(footnote)
        CASE ("a")
          psat = mmHg * exp(A - B/(temp(i)+C))
        CASE ("b")
          psat = 1E3  * exp(A - B/(temp(i)-CtoK+C))
        CASE ("c")
          psat = 1E3  * exp(A - B/(temp(i)+C))
        END SELECT
        Harray(i) = cH2O / (gamma(i) * psat)
      ENDDO
      CALL HTdep(temp, Harray, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
      DEALLOCATE(Harray)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3693

  !---------------------------------------------------------------------------

  SUBROUTINE ref3694 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3694"
    type = "M"
    ndata = 4
    ALLOCATE(temp(ndata))
    ! Tab. 1:
    temp = (/ 288., 293., 298., 303. /)
    CALL CalcH("methanol",    "67-56-1", (/ 1.02E-4, 1.38E-4, 1.87E-4, 2.41E-4 /))
    CALL CalcH("2-propanol",  "67-63-0", (/ 1.37E-4, 2.11E-4, 3.12E-4, 4.55E-4 /))
    CALL CalcH("toluene",    "108-88-3", (/ 0.177,   0.225,   0.271,   0.331   /))
    CALL CalcH("p-xylene",   "106-42-3", (/ 0.152,   0.197,   0.259,   0.326   /))
    DEALLOCATE(temp)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, KH)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: KH
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp, Hvcc_TO_HscpSI(KH, temp), Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3694

  !---------------------------------------------------------------------------

  SUBROUTINE ref3695 ! Hvpc [Pa*L/mol]
    IMPLICIT NONE

    chem = "methane" ; casrn = "74-82-8" ! CH4
    ref = "3695"
    type = "M"
    CALL Output(HvpcSI_TIMES_HscpSI * 1000. / 64226792.)

  END SUBROUTINE ref3695

  !---------------------------------------------------------------------------

  SUBROUTINE ref3696 ! Hscc [1] = Ostwald coefficient
    IMPLICIT NONE

    ref = "3696"
    type = "L"
    ! Tab. II:
    CALL CalcH("trichloromethane",       "67-66-3", 4.0,    -3.76)
    CALL CalcH("cyclopropane",           "75-19-4", 0.21,   -2.11)
    CALL CalcH("diethyl ether",          "60-29-7", 13.,    -4.89)
    CALL CalcH("divinyl ether",         "109-93-3", 1.4          )
    CALL CalcH("enflurane",           "13838-16-9", 0.78         )
    CALL CalcH("chloroethane",           "75-00-3", 1.2,    -3.29)
    CALL CalcH("ethene",                 "74-85-1", 0.090,  -1.94)
    CALL CalcH("fluoroxene",            "406-90-6", 0.85,   -3.88)
    CALL CalcH("isoflurane",          "26675-46-7", 0.62         )
    CALL CalcH("halothane",             "151-67-7", 0.80,   -4.01)
    CALL CalcH("krypton",              "7439-90-9", 0.051,  -1.72)
    CALL CalcH("methoxyflurane",         "76-38-0", 4.5,    -4.18)
    CALL CalcH("nitrogen",             "7727-37-9", 0.0140, -1.12)
    CALL CalcH("dinitrogen monoxide", "10024-97-2", 0.47,   -2.33)
    CALL CalcH("teflurane",             "124-72-1", 0.32,   -2.44)
    CALL CalcH("trichloroethene",        "79-01-6", 1.7,    -3.94)
    CALL CalcH("xenon",                "7440-63-3", 0.085,  -2.17)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, lambda, temp_coeff)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: lambda
      REAL, OPTIONAL,   INTENT(IN) :: temp_coeff
      REAL :: Hominus37
      chem=chem_ ; casrn=casrn_ ! make global for Output
      Hominus37 = Hscc_TO_HscpSI(lambda, 37.+CtoK)
      IF (PRESENT(temp_coeff)) THEN
        ! calc -DeltaH/R at 310 K using eqn (7):
        mindHR = -(310.**2 * temp_coeff/100. - 310.)
        ! extrapolate H to T0:
        Hominus = Hominus37 * exp(mindHR*(1/T0-1/310.))
        CALL Output(Hominus, mindHR)
      ELSE
        CALL MakeNoteOtherTempC(37)
        CALL Output(Hominus37)
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3696

  !---------------------------------------------------------------------------

  SUBROUTINE ref3697 ! Hscc [1]
    IMPLICIT NONE

    ref = "3697"
    type = "M"
    ! Tab. 1:
    CALL CalcH("diethyl ether", "60-29-7", (/ 35., 37., 39. /), (/ 15.40, 13.07, 11.48 /))
    CALL CalcH("trichloromethane", "67-66-3", (/ 37. /), (/ 4.04  /))
    CALL CalcH("dinitrogen monoxide", "10024-97-2", (/ 37. /), (/ 0.482 /))

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, part_coeff)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, part_coeff
      chem=chem_ ; casrn=casrn_ ! make global for Output
      IF (SIZE(part_coeff)==1) THEN
        CALL MakeNoteOtherTempC(temp(1))
        CALL Output(Hscc_TO_HscpSI(part_coeff(1), temp(1)+CtoK))
      ELSE
        CALL HTdep(temp+CtoK, Hscc_TO_HscpSI(part_coeff, temp+CtoK), Hominus, mindHR)
        CALL Output(Hominus, mindHR, r2)
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3697

  !---------------------------------------------------------------------------

  SUBROUTINE ref3698 ! Hscc [1] = Ostwald coefficient
    IMPLICIT NONE

    ref = "3698"
    type = "M"
    chem = "diethyl ether" ; casrn = "60-29-7" ! C2H5OC2H5
    CALL MakeNoteOtherTempC(37)
    CALL Output(Hscc_TO_HscpSI(16.2, 37.+CtoK)) ! see bottom of p. 130

  END SUBROUTINE ref3698

  !---------------------------------------------------------------------------

  SUBROUTINE ref3699 ! Hscc [1] = Ostwald coefficient
    IMPLICIT NONE

    ref = "3699"
    type = "M"
    chem = "diethyl ether" ; casrn = "60-29-7" ! C2H5OC2H5
    CALL MakeNoteOtherTempC(37)
    CALL Output(Hscc_TO_HscpSI(13.09, 37.+CtoK)) ! Tab. 1 and summary

  END SUBROUTINE ref3699

  !---------------------------------------------------------------------------

  SUBROUTINE ref3700 ! Hscc [1] = Ostwald coefficient
    IMPLICIT NONE

    ref = "3700"
    type = "M"
    chem = "xenon-133" ; casrn = "14932-42-4" ! 133Xe
    CALL MakeNoteOtherTempC(37)
    CALL Output(Hscc_TO_HscpSI(1E-3*83.6, 37.+CtoK)) ! Tab. 3

  END SUBROUTINE ref3700

  !---------------------------------------------------------------------------

  SUBROUTINE ref3701 ! Hscc [1] = Ostwald coefficient
    IMPLICIT NONE

    ref = "3701"
    ! Tab. 1, only data up to 50 C is used here:
    CALL CalcH( &
      (/ 20., 25., 30., 35., 40., 45., 50. /), &
      (/ 0.693, 0.607, 0.538, 0.482, 0.435, 0.396, 0.365 /))
    ! 55., 60., 65., 70., 75., 80.
    ! 0.334, 0.316, 0.298, 0.282, 0.269, 0.259
    ! Tab. 2, cited data:
    CALL CalcH((/ 8.1, 15.5, 23.5 /), &
      (/ 1.012, 0.811, 0.670 /), "gordon1895")
    ! CALL CalcH((/ 3.5, 6.4, 12., 15.8, 21.7, 24.7 /), &
    !   (/ 1.220, 1.100, 0.918, 0.813, 0.688, 0.632 /), "3944") ! Roth
    ! CALL CalcH((/ 5., 10., 15., 20., 25. /), &
    !   (/ 1.067, 0.910, 0.778, 0.676, 0.594 /), "3711") ! Geffcken

  CONTAINS

    SUBROUTINE CalcH (temp, ostwald, xref)
      IMPLICIT NONE
      REAL, DIMENSION(:),         INTENT(IN) :: temp, ostwald
      CHARACTER(LEN=*), OPTIONAL, INTENT(IN) :: xref
      chem  = "dinitrogen monoxide"
      casrn = "10024-97-2"
      IF (PRESENT(xref)) THEN
        CALL SettypeX(xref)
        CALL HTdep(temp+CtoK, Hscc_TO_HscpSI(ostwald, temp+CtoK), Hominus, mindHR, xref)
      ELSE
        type = "M"
        CALL HTdep(temp+CtoK, Hscc_TO_HscpSI(ostwald, temp+CtoK), Hominus, mindHR)
      ENDIF
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3701

  !---------------------------------------------------------------------------

  SUBROUTINE ref3702 ! Bunsen coefficient alpha [1]
    IMPLICIT NONE
    ref = "3702"
    type = "M"
    ! Tab. 1:
    CALL CalcH("ethene",                 "74-85-1", 0.108 )
    CALL CalcH("dinitrogen monoxide", "10024-97-2", 0.549 )
    CALL CalcH("oxygen",               "7782-44-7", 0.0281)
    CALL CalcH("nitrogen",             "7727-37-9", 0.0149)
    CALL CalcH("carbon dioxide",        "124-38-9", 0.753 )

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, alpha)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: alpha
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL Output(alpha*alpha_TO_HscpSI)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3702

  !---------------------------------------------------------------------------

  SUBROUTINE ref3703 ! Hscc [1] = Ostwald coefficient, and alpha [1] = Bunsen coefficient
    IMPLICIT NONE
    ref = "3703"
    ! Tab. 1:
    !CALL CalcH("3704",       38.,               lambda=0.435) ! should be 0.418, not 0.435
    !CALL CalcH("3702",       25.,  alpha=0.549)
    !CALL CalcH("3702",       25.,               lambda=0.599)
    CALL CalcH("harris51",    37.,  alpha=0.395              )
    !CALL CalcH("3705",       23.,               lambda=0.540)
    !CALL CalcH("Nunn_b",     20.,  alpha=0.63               ) ! cited from Kaye & Laby (1956)
    CALL CalcH("macintosh58", 20.,  alpha=0.68               )
    !CALL CalcH("3706",       25.,  alpha=0.545              ) ! p 462
    CALL CalcH("orcutt37",    37.5,              lambda=0.44 )
    CALL CalcH("thiswork",    37.,               lambda=0.367)

  CONTAINS

    SUBROUTINE CalcH (xref, temp, alpha, lambda)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: xref
      REAL,             INTENT(IN) :: temp
      REAL, OPTIONAL,   INTENT(IN) :: alpha, lambda
      chem  = "dinitrogen monoxide"
      casrn = "10024-97-2"
      CALL MakeNoteOtherTempC(temp)
      IF (PRESENT(alpha)) THEN
        CALL SettypeX(xref)
        Hominus = alpha*alpha_TO_HscpSI
      ENDIF
      IF (PRESENT(lambda)) THEN
        IF (xref=="thiswork") THEN
          type = "M"
        ELSE
          CALL SettypeX(xref)
        ENDIF
        Hominus = Hscc_TO_HscpSI(lambda, temp+CtoK)
      ENDIF
      CALL Output(Hominus)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3703

  !---------------------------------------------------------------------------

  SUBROUTINE ref3704 ! Bunsen coefficient alpha [1]
    IMPLICIT NONE
    ref = "3704"
    type = "M"
    chem = "dinitrogen monoxide" ; casrn = "10024-97-2"
    CALL HTdep( (/ 22.9, 38. /) + CtoK, &
      (/ 0.572, 0.383 /) * alpha_TO_HscpSI, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3704

  !---------------------------------------------------------------------------

  SUBROUTINE ref3705 ! Hscc [1] = Ostwald coefficient, and alpha [1] = Bunsen coefficient
    IMPLICIT NONE
    REAL :: temp
    ref = "3705"

    ! Tab. I: (Bunsen's coeff.)
    CALL CalcH("dinitrogen monoxide", "10024-97-2", 0.63  )
    CALL CalcH("carbon dioxide",        "124-38-9", 0.878 )
    CALL CalcH("oxygen",               "7782-44-7", 0.031 )
    CALL CalcH("nitrogen",             "7727-37-9", 0.0164)

    ! Tab. II: (Ostwald solubility coefficient)
    chem = "dinitrogen monoxide" ; casrn = "10024-97-2"
    type = "M"
    temp = 23.0
    CALL MakeNoteOtherTempC(temp)
    CALL Output(Hscc_TO_HscpSI(0.540, temp+CtoK))

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, alpha)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: alpha
      chem=chem_ ; casrn=casrn_ ! make global for Output
      type = "C" ! from Kaye & Laby (1956)
      CALL MakeNoteOtherTempC(20)
      CALL Output(alpha*alpha_TO_HscpSI)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3705

  !---------------------------------------------------------------------------

  ! ref3706 NOTUSED: Only citations, no new data.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3707 ! Bunsen coefficient alpha [1]
    IMPLICIT NONE
    ref = "3707"
    type = "M"
    chem = "nitrogen" ; casrn = "7727-37-9"
    CALL HTdep( (/ 25., 38. /) + CtoK, &
      (/ 0.01483, 0.01272 /) * alpha_TO_HscpSI, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3707

  !---------------------------------------------------------------------------

  SUBROUTINE ref3708 ! probably Bunsen coefficient alpha [1]
    IMPLICIT NONE
    ref = "3708"
    type = "M"
    chem = "dinitrogen monoxide" ; casrn = "10024-97-2"
    CALL HTdep( (/ 2.5, 8.2, 12.0, 16.2, 20.0, 24.0 /) + CtoK, &
      (/ 1.1942, 0.9700, 0.8432, 0.7477, 0.6744, 0.6024 /) * &
      alpha_TO_HscpSI, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3708

  !---------------------------------------------------------------------------

  SUBROUTINE ref3709 ! Hscc [1] = Ostwald coefficient
    IMPLICIT NONE

    ref = "3709"
    type = "M"
    ! Tab. I:
    CALL CalcH("carbon dioxide", "124-38-9", &
      (/ 20., 22., 24., 26., 28., 30., 32., 34. /), &
      (/ 0.900, 0.872, 0.836, 0.800, 0.765, 0.729, 0.693, 0.656 /))
    CALL CalcH("dinitrogen monoxide", "10024-97-2", &
      (/ 18., 20., 22., 24., 26., 28., 30., 32., 34., 36. /), &
      (/ 0.703, 0.675, 0.638, 0.617, 0.587, 0.561, 0.530, 0.513, 0.478, 0.449 /))

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, ostwald)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, ostwald
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp+CtoK, Hscc_TO_HscpSI(ostwald, temp+CtoK), Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3709

  !---------------------------------------------------------------------------

  SUBROUTINE ref3710 ! Hscc [1] = Ostwald coefficient
    IMPLICIT NONE

    ref = "3710"
    type = "M"
    ! Tab. I:
    CALL CalcH("dinitrogen monoxide", "10024-97-2", &
      (/ 37., 25., 20. /), &
      (/ 0.444, 0.594, 0.657 /))
    CALL CalcH("cyclopropane",           "75-19-4", &
      (/ 37., 30.5, 25. /), &
      (/ 0.212, 0.232, 0.266 /))
    CALL CalcH("halothane",             "151-67-7", &
      (/ 37. /), (/ 0.874 /)) ! value for lowest concentration (1 %)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, part_coeff)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, part_coeff
      chem=chem_ ; casrn=casrn_ ! make global for Output
      IF (SIZE(part_coeff)==1) THEN
        CALL MakeNoteOtherTempC(temp(1))
        CALL Output(Hscc_TO_HscpSI(part_coeff(1), temp(1)+CtoK))
      ELSE
        CALL HTdep(temp+CtoK, Hscc_TO_HscpSI(part_coeff, temp+CtoK), Hominus, mindHR)
        CALL Output(Hominus, mindHR, r2)
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3710

  !---------------------------------------------------------------------------

  SUBROUTINE ref3711 ! Hscc [1] = Ostwald coefficient
    IMPLICIT NONE

    ref = "3711"
    type = "M"
    CALL CalcH("hydrogen",             "1333-74-0", &
      (/ 25., 15. /), (/ 0.01926, 0.01982 /)) ! p. 266
    CALL CalcH("oxygen",               "7782-44-7", &
      (/ 25., 15. /), (/ 0.03080, 0.03630 /)) ! p. 269
    CALL CalcH("carbon dioxide",        "124-38-9",  &
      (/ 25., 15. /), (/ 0.8255, 1.070 /)) ! p. 271
    CALL CalcH("dinitrogen monoxide", "10024-97-2", &
      (/ 25., 20., 15., 10., 5. /), (/ 0.5942, 0.6756, 0.7784, 0.9101, 1.067 /)) ! p. 277

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, ostwald)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, ostwald
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp+CtoK, Hscc_TO_HscpSI(ostwald, temp+CtoK), Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)

    END SUBROUTINE CalcH

  END SUBROUTINE ref3711

  !---------------------------------------------------------------------------

  SUBROUTINE ref3712 ! Bunsen coefficient (called beta here) [1]
    IMPLICIT NONE
    ref = "3712"
    type = "M"
    CALL CalcH("hydrogen", "1333-74-0", & ! Tab. I
      (/ 1.4, 8.35, 13.0, 18.45, 25.7 /), &
      (/ 0.021295, 0.020044, 0.019174, 0.018629, 0.017789 /))
    CALL CalcH("oxygen",   "7782-44-7", & ! Tab. III
      (/ 6.4, 12.6 /), &
      (/ 0.041408, 0.036011 /))

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, beta)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, beta
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp+CtoK, beta*alpha_TO_HscpSI, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3712

  !---------------------------------------------------------------------------

  SUBROUTINE ref3713 ! Bunsen coefficient [1]
    IMPLICIT NONE
    ref = "3713"
    type = "M"
    CALL CalcH("oxygen",   "7782-44-7", &
      (/ 42.53, 48.94, 60.06, 7.94, 26.61, 34.99, 43.06, 46.61, 56.35, &
      7.30, 11.14, 35.55, 48.42, 21.08 /), &
      (/ 0.02317, 0.02152, 0.01801, 0.04089, 0.02842, 0.02443, 0.02327, &
      0.02185, 0.01987, 0.04163, 0.03808, 0.02465, 0.02134, 0.03110 /) &
      ) ! p. 331
    CALL CalcH("nitrogen", "7727-37-9", &
      (/ 18.67, 18.81, 19.35, 16.86, 41.88, 59.43, 22.10, 29.91, 12.78, &
      35.14, 45.97, 59.85, 24.06, 0.14 /), &
      (/ 0.01682, 0.01679, 0.01644, 0.01894, 0.01189, 0.00969, 0.01528, &
      0.01347, 0.01889, 0.01313, 0.01075, 0.00961, 0.01527, 0.02384 /) &
      ) ! p. 335
    CALL CalcH("hydrogen", "1333-74-0", &
      (/ 15., 15., 15., 11.70, 20.71, 32.22, 41.32, 53.37, 11.39, 25.46, &
      36.14, 47.44, 59.26, 0.05, 56.77, 56.58 /), & ! 100., 100.,
      (/ 0.0183, 0.0186, 0.0180, 0.0188, 0.0178, 0.0168, 0.0148, 0.0139, &
      0.0190, 0.0165, 0.0156, 0.0147, 0.0143, 0.0203, 0.0149, 0.0152 /) & ! 0.0165, 0.0167,
      ) ! p. 339, values at 100 C not used

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, abscoeff)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, abscoeff
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp+CtoK, abscoeff*alpha_TO_HscpSI, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3713

  !---------------------------------------------------------------------------

  SUBROUTINE ref3714 ! Kuenen coefficient S [cm3/g]
    IMPLICIT NONE

    ref = "3714"
    type = "M"

    ! using only data up to 30 C:
    CALL CalcH("xenon",   "7440-63-3", (/ 0.2189, 0.1500, 0.1109, 0.0900 /))             ! Xe, 0.0812, 0.0878
    CALL CalcH("krypton", "7439-90-9", (/ 0.1249, 0.0965, 0.0788, 0.0762 /), "series_1") ! Kr, 0.0740, 0.0823
    CALL CalcH("krypton", "7439-90-9", (/ 0.1166, 0.0877, 0.0670, 0.0597 /), "series_2") ! Kr, 0.0561, 0.0610
    CALL CalcH("argon",   "7440-37-1", (/ 0.0561, 0.0438, 0.0379, 0.0348 /))             ! Ar, 0.0338, 0.0343
    CALL CalcH("neon",    "7440-01-9", (/ 0.0114, 0.0118, 0.0147, 0.0158 /))             ! Ne, 0.0203, 0.0317
    CALL CalcH("helium",  "7440-59-7", (/ 0.0134, 0.0100, 0.0138, 0.0161 /))             ! He, 0.0191, 0.0226

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, S, extralabel)
      IMPLICIT NONE
      CHARACTER(LEN=*),           INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:),         INTENT(IN) :: S
      CHARACTER(LEN=*), OPTIONAL, INTENT(IN) :: extralabel
      REAL, DIMENSION(4) :: temp_ = (/ 0., 10., 20., 30. /) + CtoK
      chem=chem_ ; casrn=casrn_ ! make global for Output
      IF (PRESENT(extralabel)) THEN
        CALL MakeNote(TRIM(ref)//"_krypton", &
          "Two series of measurements with considerably different results are presented by " &
          // TRIM(citet()) // " for krypton.")
        CALL HTdep(temp_, S*S_TO_HscpSI, Hominus, mindHR, extralabel)
      ELSE
        CALL HTdep(temp_, S*S_TO_HscpSI, Hominus, mindHR)
      ENDIF
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3714

  !---------------------------------------------------------------------------

  SUBROUTINE ref3715 ! Bunsen coefficient
    IMPLICIT NONE
    ref = "3715"

    ! Tab. II:
    CALL CalcH("nitrogen", "7727-37-9", 0.0128) ! N2
    CALL CalcH("ethene",     "74-85-1", 0.078)  ! C2H4 ethylene
    CALL CalcH("ethyne",     "74-86-2", 0.747)  ! C2H2 acetylene

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, alpha)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: alpha
      chem=chem_ ; casrn=casrn_ ! make global for Output
      type = "M"
      CALL MakeNoteOtherTempC(37.5)
      CALL Output(alpha*alpha_TO_HscpSI)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3715

  !---------------------------------------------------------------------------

  SUBROUTINE ref3716 ! Bunsen coefficient [1]
    IMPLICIT NONE
    ref = "3716"
    type = "M"
    ! data are from a smoothed curve, thus it is sufficient to use only
    ! the values at 20 C and 30 C:
    CALL CalcH("nitrogen", "7727-37-9", & ! Tab. I
      (/ 15.54, 13.55 /)) ! (/ 23.00, 18.54, 15.54, 13.55, 12.15, 11.02 /)
    CALL CalcH("oxygen",   "7782-44-7", & ! Tab. II
      (/ 31.44, 26.65 /)) ! (/ 49.24, 38.37, 31.44, 26.65, 23.30, 20.95 /)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, alpha)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: alpha
      REAL, DIMENSION(2) :: temp_ = (/ 20., 30. /) + CtoK
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp_, 1E-3*alpha*alpha_TO_HscpSI, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3716

  !---------------------------------------------------------------------------

  SUBROUTINE ref3717 ! Bunsen coefficient [1]
    IMPLICIT NONE
    ref = "3717"
    type = "M"
    CALL CalcH("nitrogen",       "7727-37-9", &
      (/ 4.0, 6.2, 12.6, 17.7, 23.7 /), &
      (/ 0.01843, 0.01751, 0.01520, 0.01436, 0.01392 /))
    CALL CalcH("hydrogen",       "1333-74-0", &
      (/ 4.0, 7.0,  9.6, 12.8, 15.5, 18.8, 23.6 /), &
      (/ 0.0185, 0.0205, 0.0196, 0.0186, 0.0197, 0.0188, 0.0194 /))
    ! It is unclear if "Aethylgas" refers to ethene.
    !   (/ 5.8, 8.7, 14.0, 17.2, 21.8 /), &
    !   (/ 0.02637, 0.02393, 0.02199, 0.02103, 0.02026 /))
    CALL CalcH("carbon monoxide", "630-08-0", &
      (/ 5.8, 8.6,  9.0, 17.4, 18.4, 22.0 /), &
      (/ 0.028636, 0.027125, 0.026855, 0.023854, 0.023147, 0.022907 /))
    ! It is unclear if "Methylgas" refers to methane.
    ! CALL CalcH("methane",          "74-82-8", &
    !   (/ 4.6, 7.8, 12.1, 15.2, 19.8, 24.2 /), &
    !   (/ 0.072884, 0.064732, 0.055788, 0.050722, 0.045715, 0.040817 /))
    CALL CalcH("carbon dioxide",  "124-38-9", &
      (/ 4.4, 8.4, 13.8, 16.6, 19.1, 22.4 /), &
      (/ 1.4695, 1.2426, 1.0654, 0.9692, 0.8963, 0.8642 /))

    ! data for SO2, H2S, and Cl2 on p. 26 is only quoting values from
    ! Schönfeld, see ref3933

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, abscoeff)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, abscoeff
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp+CtoK, abscoeff*alpha_TO_HscpSI, Hominus, mindHR)
      CALL MakeNote(TRIM(ref), "The paper by " // TRIM(citet()) // &
        " was written in German. English versions with the same " // &
        "data were published by \citet{3927} and \citet{3930}.")
      CALL Output(Hominus, mindHR, r2)
      IF (chem_=="nitrogen") THEN
        ! value for O2 is scaled to that of N2 (p. 24):
        chem = "oxygen" ; casrn = "7782-44-7" ! O2
        CALL Output(2.0225*Hominus, mindHR)
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3717

  !---------------------------------------------------------------------------

  SUBROUTINE ref3718 ! Bunsen coefficient (called beta here) [1]
    IMPLICIT NONE
    ref = "3718"
    type = "M"

    CALL CalcH("chlorine", "7782-50-5", & ! Cl2 , p. 95
      (/ 10., 15., 20., 25. /), &
      (/ 3.095, 2.635, 2.260, 1.985 /))
    CALL CalcH("hydrogen sulfide", "7783-06-4", & ! H2S, p. 100
      (/ 0., 10., 20., 30. /), &
      (/ 4.630, 3.376, 2.560, 2.015 /))
    CALL CalcH("carbon oxide sulfide", "463-58-1", & ! OCS, p. 107
      (/ 0., 10., 20., 30. /), &
      (/ 1.3330, 0.8353, 0.5614, 0.4025 /))

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, beta)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, beta
      chem=chem_ ; casrn=casrn_ ! make global for Output
      IF (TRIM(casrn)=="7782-50-5") THEN ! Cl2
        CALL MakeNote("HeffCl2")
        CALL Output(DUMMY)
        RETURN
      ENDIF
      CALL HTdep(temp+CtoK, beta*alpha_TO_HscpSI, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3718

  !---------------------------------------------------------------------------

  SUBROUTINE ref3719 ! HscpSI [mol/(m3*Pa)] and Hscc [1]
    IMPLICIT NONE

    ref = "3719"

    ! Tab. S2:
    type = "M"
    ndata = 4
    ALLOCATE(temp(ndata))
    temp = (/ 15., 25., 35., 50. /) + CtoK
    CALL CalcH_S2("1-butanol",                "71-36-3", (/ 2.33,    1.01,    5.25E-1, 1.66E-1 /))
    CALL CalcH_S2("1-octanol",               "111-87-5", (/ 3.65E-1, 1.70E-1, 7.86E-2, 2.61E-2 /))
    CALL CalcH_S2("2,4,6-trichloroanisole",   "87-40-1", (/ 4.67E-2, 2.13E-2, 1.11E-2, 5.89E-3 /)) ! TCA
    CALL CalcH_S2("3-octanone",              "106-68-3", (/ 5.86E-2, 2.88E-2, 1.52E-2, 6.49E-3 /))
    CALL CalcH_S2("1-octen-3-ol",           "3391-86-4", (/ 5.30E-1, 1.74E-1, 8.12E-2, 2.63E-2 /))
    CALL CalcH_S2("3-octanol",               "589-98-0", (/ 3.71E-1, 1.04E-1, 4.16E-2, 1.58E-2 /))
    CALL CalcH_S2("2-ethyl-1-hexanol",       "104-76-7", (/ 3.66E-1, 1.51E-1, 7.01E-2, 2.40E-2 /))
    DEALLOCATE(temp)

    ! Tab. S5:
    type = "Q"
    CALL CalcH_S5("2-octen-1-ol",                   "22104-78-5", 3.28)
    CALL CalcH_S5("3-methylfuran",                    "930-27-8", 0.90)
    CALL CalcH_S5("2-methylisoborneol",              "2371-42-8", 3.59)
    CALL CalcH_S5("2-isopropyl-3-methoxy-pyrazine", "25773-40-4", 4.71)
    CALL CalcH_S5("geosmin",                        "19700-21-1", 4.01)

  CONTAINS

    SUBROUTINE CalcH_S2 (chem_, casrn_, H)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: H
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp, H, Hominus, mindHR)
      IF (casrn_=="106-68-3") THEN
        CALL MakeNote(TRIM(ref)//"-3-octanone", "Table~S2 in the supplement of " &
          // TRIM(citet()) // " contains incorrect data for 3-octanone. " // &
        "Here, the corrected data (Shuang Wu, personal communication, 2022) " // &
        "were used: $2.88\E{-2}$ and $1.52\E{-2}$ at 25\,\unit{\degree C} and " // &
        "35\,\unit{\degree C}, respectively.")
      ENDIF
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH_S2

    SUBROUTINE CalcH_S5 (chem_, casrn_, lgKWA)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: lgKWA
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL MakeNote(TRIM(ref)//"-SPARC", &
        "Calculated using SPARC Performs Automated Reasoning in Chemistry (SPARC). " // &
        "It is assumed here that the value refers to $T$~= 298.15\,\unit{K}.")
      CALL Output(Hscc_TO_HscpSI(10.**lgKWA, T0))
    END SUBROUTINE CalcH_S5

  END SUBROUTINE ref3719

  !---------------------------------------------------------------------------

  SUBROUTINE ref3723 ! Hscp [M/atm]
    IMPLICIT NONE
    ref = "3723"

    ! Tab. S4:
    CALL CalcH_S4("4-propyl guaiacol",      "2785-87-7", "PXIKRTCSSLJURC-UHFFFAOYSA-N", 4.40E+02, 4.29E+02)
    CALL CalcH_S4("guaiacol",                 "90-05-1", "LHGVFZTZFXWLCP-UHFFFAOYSA-N", 8.73E+02, 8.33E+02)
    CALL CalcH_S4("eugenol",                  "97-53-0", "RRAFCDWBNXTKKO-UHFFFAOYSA-N", 9.29E+02, 5.75E+02)
    CALL CalcH_S4("vanillyl ethyl ether",  "13184-86-6", "KOCVACNWDMSLBM-UHFFFAOYSA-N", 4.15E+04, 4.85E+04)
    CALL CalcH_S4("guaiacylacetone",        "2503-46-0", "LFVCJQWZGDLHSD-UHFFFAOYSA-N", 1.19E+06, 1.12E+06)
    CALL CalcH_S4("zingerone",               "122-48-5", "OJYLAHXKWMRDGS-UHFFFAOYSA-N", 1.23E+06, 8.47E+05)
    CALL CalcH_S4("vanillyl alcohol",        "498-00-0", "ZENOXNGFMSCLLL-UHFFFAOYSA-N", 9.69E+06, 2.06E+07)
    CALL CalcH_S4("syringylacetone",      "112468-41-4", "NULBEPOZYDYWOV-UHFFFAOYSA-N", 1.24E+08, 5.81E+06)
    ! Tab. S6:
    ! benzaldehyde                                                                      4.0E+01 measured ref 10
    ! o-cresol                                                                          8.3E+02 measured ref 6
    ! m-cresol                                                                          1.0E+03 measured ref 6
    ! 2,4-dimethylphenol                                                                1.1E+03 measured ref 7
    ! p-cresol                                                                          1.3E+03 measured ref 6
    ! phenol                                                                            5.0E+02 measured ref 6
    ! o-benzenediol                                                                     4.7E+03 measured ref 8
    ! p-methoxybenzaldehyde                                                             7.0E+03 measured ref 11
    ! o-methoxybenzaldehyde                                                             1.8E+04 measured ref 11
    ! m-methoxybenzaldehyde                                                             3.5E+04 measured ref 11
    CALL CalcH_S6("2-hydroxybenzaldehyde",    "90-02-8", "SMQUZDBALVYZAC-UHFFFAOYSA-N", 4.1E+05)
    ! 3-hydroxybenzaldehyde                                                             4.1E+05 measured ref 12*
    ! 4-hydroxybenzaldehyde                                                             1.9E+06 measured ref 12*
    ! m-benzenediol                                                                     8.4E+06 measured ref 8
    ! p-benzenediol                                                                     2.3E+07 measured ref 8
    CALL CalcH_S6("tyrosol",                 "501-94-0", "YCCILVSKPBXVIP-UHFFFAOYSA-N", 3.5E+07)
    ! 4-propylguaiacol                                                                  4.4E+02 from Tab. S4
    CALL CalcH_S6("4-ethylguaiacol",        "2785-89-9", "CHWNEIVBYREQRF-UHFFFAOYSA-N", 5.6E+02)
    ! 4-methylguaiacol                                                                  7.3E+02 measured ref 5
    ! guaiacol                                                                          8.7E+02 from Tab. S4
    ! eugenol                                                                           9.3E+02 from Tab. S4
    CALL CalcH_S6("isoeugenol",               "97-54-1", "BJIOGJUNALELMI-UHFFFAOYSA-N", 1.7E+03)
    ! trans-isoeugenol                                                                  1.7E+03  (same value as cis)
    ! vanillyl ethyl ether                                                              4.2E+04 from Tab. S4
    ! vanillin                                                                          4.7E+05 measured ref 13
    CALL CalcH_S6("acetovanillone",          "498-02-2", "DFYRUELUNQRZTB-UHFFFAOYSA-N", 6.4E+05)
    ! guaiacyl acetone                                                                  1.2E+06 from Tab. S4
    ! zingerone                                                                         1.2E+06 from Tab. S4
    CALL CalcH_S6("coniferylaldehyde",       "458-36-6", "UCZOBXQKIUMJDZ-UHFFFAOYSA-N", 4.5E+06)
    ! vanillyl alcohol                                                                  9.7E+06 from Tab. S4
    CALL CalcH_S6("4-propylsyringol",       "6766-82-1", "YHEWWEXPVKCVFY-UHFFFAOYSA-N", 2.2E+03)
    CALL CalcH_S6("4-ethylsyringol",       "14059-92-8", "PJWDIHUFLXQRFF-UHFFFAOYSA-N", 3.0E+03)
    CALL CalcH_S6("4-methylsyringol",       "6638-05-7", "ZFBNNSOJNZBLLS-UHFFFAOYSA-N", 3.9E+03)
    ! syringol                                                                          5.0E+03 measured ref 5
    CALL CalcH_S6("allylsyringol",          "5438-54-0", "IYIVYBWBUQPWAU-UHFFFAOYSA-N", 6.4E+04)
    CALL CalcH_S6("4-propenylsyringol",     "6635-22-9", "YFHOHYAUMDHSBX-UHFFFAOYSA-N", 1.1E+05)
    CALL CalcH_S6("butyrylsyringol",       "69271-91-6", "QFHXMVPZYYMTCS-UHFFFAOYSA-N", 2.9E+07)
    CALL CalcH_S6("syringaldehyde",          "134-96-3", "KCDXJAYRVLXPFO-UHFFFAOYSA-N", 3.7E+07)
    CALL CalcH_S6("propionylsyringol",      "5650-43-1", "CXCPJZXJNRBTGF-UHFFFAOYSA-N", 3.8E+07)
    CALL CalcH_S6("acetosyringone",         "2478-38-8", "OJOBTAOGJIWAGB-UHFFFAOYSA-N", 5.1E+07)
    ! syringyl acetone                                                                  1.2E+08 from Tab. S4
    CALL CalcH_S6("sinapylaldehyde",        "4206-58-0", "CDICDSOGTRCHMG-ONEGZZNKSA-N", 3.1E+08)

  CONTAINS

    SUBROUTINE CalcH_S4 (chem_, casrn_, inchikey_, KH_meas, KH_EVA)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_, inchikey_
      REAL,             INTENT(IN) :: KH_meas, KH_EVA
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      ! measured value:
      type = "M"
      Hominus = KH_meas*Hscp_TO_HscpSI
      CALL Output(Hominus)
      ! EVA:
      type = "Q"
      Hominus = KH_EVA*Hscp_TO_HscpSI
      CALL MakeNote(TRIM(ref), &
        "Calculated using the experimental value adjusted (EVA) method; see " &
        //TRIM(citet())// " for details.")
      CALL Output(Hominus)
    END SUBROUTINE CalcH_S4

    SUBROUTINE CalcH_S6 (chem_, casrn_, inchikey_, KH_EVA)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_, inchikey_
      REAL,             INTENT(IN) :: KH_EVA
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      ! EVA:
      type = "Q"
      Hominus = KH_EVA*Hscp_TO_HscpSI
      CALL MakeNote(TRIM(ref), &
        "Calculated using the experimental value adjusted (EVA) method; see " &
        //TRIM(citet())// " for details.")
      CALL Output(Hominus)
    END SUBROUTINE CalcH_S6

  END SUBROUTINE ref3723

  !---------------------------------------------------------------------------

  SUBROUTINE ref3724 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3724"
    type = "M"
    ndata = 5
    ALLOCATE(temp(ndata))
    temp = (/ 278., 283., 288., 293., 298. /)
    ! Tab. 1:
    !CALLCalcHTdep("benzaldehyde",                                                      (/ 24.,  35.,  50.,  74.,  125. /) from Allen et al.
    CALL CalcHTdep("acetophenone",            "98-86-2", "KWOLFJPFCHCOCG-UHFFFAOYSA-N", (/ 5.6,  8.1,  13.,  19.,  35.  /), -7480., 17.04)
    CALL CalcHTdep("phenol",                 "108-95-2", "ISWSIDIOOBJBQZ-UHFFFAOYSA-N", (/ 0.18, 0.32, 0.60, 1.1,  1.7  /), -9503., 20.94)
    CALL CalcHTdep("salicylaldehyde",         "90-02-8", "SMQUZDBALVYZAC-UHFFFAOYSA-N", (/ 9.1,  12.,  19.,  25.,  38.  /), -5954., 12.06)
    ! Tab. 2:
    CALL CalcHTdep("2-methylbenzaldehyde",   "529-20-4", "BTFQKIATRPGRBS-UHFFFAOYSA-N", (/ 27.,  39.,  57.,  85.,  136. /), -6648., 15.64)
    CALL CalcHTdep("3-methylbenzaldehyde",   "620-23-5", "OVWYEQOVUDKZNU-UHFFFAOYSA-N", (/ 23.,  34.,  48.,  73.,  126. /), -6897., 16.37)
    CALL CalcHTdep("4-methylbenzaldehyde",   "104-87-0", "FXLOVSHXALFLKQ-UHFFFAOYSA-N", (/ 15.,  22.,  31.,  47.,  83.  /), -6920., 16.02)
    CALL CalcHTdep("2-methoxybenzaldehyde",  "135-02-4", "PKZJLOCLABXVMC-UHFFFAOYSA-N", (/ 0.50, 0.89, 1.3,  2.2,  4.3  /), -8627., 18.79)
    CALL CalcHTdep("3-methoxybenzaldehyde",  "591-31-1", "WMPDAIZRQDCGFH-UHFFFAOYSA-N", (/ 1.0,  1.9,  2.7,  4.3,  8.6  /), -8485., 19.00)
    CALL CalcHTdep("4-methoxybenzaldehyde",  "123-11-5", "ZRSNZINYAWTAHE-UHFFFAOYSA-N", (/ 0.21, 0.40, 0.62, 0.99, 1.7  /), -8444., 17.32)
    CALL CalcHTdep("propiophenone",           "93-55-0", "KRIOVPPHQSLHCZ-UHFFFAOYSA-N", (/ 6.7,  11.,  14.,  26.,  43.  /), -7581., 17.60)
    CALL CalcHTdep("2′-methylacetophenone",  "577-16-2", "YXWWHNCQZBVZPV-UHFFFAOYSA-N", (/ 9.1,  15.,  21.,  33.,  56.  /), -7329., 17.03)
    CALL CalcHTdep("3′-methylacetophenone",  "585-74-0", "FSPSELPMWGWDRY-UHFFFAOYSA-N", (/ 4.5,  7.3,  11.,  16.,  28.  /), -7361., 16.45)
    CALL CalcHTdep("4′-methylacetophenone",  "122-00-9", "GNKZMNRKLCTJAY-UHFFFAOYSA-N", (/ 3.4,  5.1,  7.3,  14.,  24.  /), -8140., 18.88)
    CALL CalcHTdep("2′-methoxyacetophenone", "579-74-8", "DWPLEOPKBWNPQV-UHFFFAOYSA-N", (/ 0.25, 0.50, 0.72, 1.3,  2.5  /), -9217., 20.25)
    CALL CalcHTdep("3′-methoxyacetophenone", "586-37-8", "BAYUSCHCCGXLAY-UHFFFAOYSA-N", (/ 0.23, 0.44, 0.64, 1.1,  2.3  /), -9146., 19.89)
    CALL CalcHTdep("2′-hydroxyacetophenone", "118-93-4", "JECYUBVRTQDVAT-UHFFFAOYSA-N", (/ 5.8,  9.2,  15.,  24.,  41.  /), -8072., 19.23)
    DEALLOCATE(temp)

  CONTAINS

    SUBROUTINE CalcHTdep (chem_, casrn_, inchikey_, kH, A, B)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_, inchikey_
      REAL, DIMENSION(:), INTENT(IN) :: kH
      REAL,               INTENT(IN) :: A, B
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      CALL HTdep(temp, Hvcc_TO_HscpSI(1E-5*kH, temp), Hominus, mindHR)
      CALL consistency_check(Hominus, Hvcc_TO_HscpSI(exp(A/T0+B),T0), "Hominus")!, verbose=.TRUE.)
      CALL consistency_check(mindHR, T0-A, "mindHR")!, verbose=.TRUE.)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcHTdep

  END SUBROUTINE ref3724

  !---------------------------------------------------------------------------

  SUBROUTINE ref3726 ! Hscp [M/atm]
    IMPLICIT NONE

    ref = "3726"
    chem = "furoyl peroxynitrate" ; inchikey = "NMNZVYLGOWCHOD-UHFFFAOYSA-N"

    ! Fig. 7:
    type = "M"
    CALL HTdep( (/ 273.15, 295. /), (/ 142., 12.9 /) * Hscp_TO_HscpSI, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)

    ! Tab. 3:
    type = "Q"
    CALL MakeNote(TRIM(ref), "Estimate based on \citet{2546}.")
    CALL Output(15.8*Hscp_TO_HscpSI)

  END SUBROUTINE ref3726

  !---------------------------------------------------------------------------

  SUBROUTINE ref3728 ! Hvpx [kPa]
    IMPLICIT NONE

    ref = "3728"
    type = "Q"

    ! solvent 74 water from 3_SI_Parameters_Predictions.xlsx in supplement:
    CALL CalcH("Acetonitrile",                                   "75-05-8",  4.7738) !   1 C2H3N
    CALL CalcH("Acetone",                                        "67-64-1",  5.3677) !   2 C3H6O
    CALL CalcH("Ethanol",                                        "64-17-5",  3.4991) !   3 C2H6O
    CALL CalcH("2-Butanol",                                      "78-92-2",  3.8683) !   4 C4H10O
    CALL CalcH("Ethylbenzene",                                  "100-41-4", 10.9490) !   5 C8H10
    CALL CalcH("Chlorobenzene",                                 "108-90-7",  7.7417) !   6 C6H5Cl
    CALL CalcH("Benzene",                                        "71-43-2", 10.5857) !   7 C6H6
    CALL CalcH("1-Butanol",                                      "71-36-3",  3.3965) !   8 C4H10O
    CALL CalcH("2-Butanone",                                     "78-93-3",  5.7104) !   9 C4H8O
    CALL CalcH("n-Butane",                                      "106-97-8", 15.0497) !  10 C4H10
    CALL CalcH("Chloroform",                                     "67-66-3",  9.9822) !  11 CHCl3
    CALL CalcH("1,2-Dichloroethane",                            "107-06-2",  8.5400) !  12 C2H4Cl2
    CALL CalcH("Dichloromethane",                                "75-09-2",  9.2033) !  13 CH2Cl2
    CALL CalcH("1,4-Dioxane",                                   "123-91-1",  3.9430) !  14 C4H8O2
    CALL CalcH("Hydrogen bromide",                            "10035-10-6",  6.6946) !  15 HBr
    CALL CalcH("Hexane",                                        "110-54-3", 16.0959) !  16 C6H14
    CALL CalcH("Hydrogen fluoride",                            "7664-39-3", 11.3099) !  17 HF
    CALL CalcH("Heptane",                                       "142-82-5", 15.4463) !  18 C7H16
    CALL CalcH("2-Propanol",                                     "67-63-0",  3.8369) !  19 C3H8O
    CALL CalcH("Methanol",                                       "67-56-1",  3.2611) !  20 CH4O
    CALL CalcH("Nitromethane",                                   "75-52-5",  4.8380) !  21 CH3NO2
    CALL CalcH("Octane",                                        "111-65-9", 14.5054) !  22 C8H18
    CALL CalcH("2-Pentanone",                                   "107-87-9",  6.4539) !  23 C5H10O
    CALL CalcH("Phosgene",                                       "75-44-5", 11.5492) !  24 CCl2O
    CALL CalcH("1-Propanol",                                     "71-23-8",  3.5846) !  25 C3H8O
    CALL CalcH("Hydrogen chloride",                            "7647-01-0",  5.4338) !  26 HCl
    CALL CalcH("Carbon disulfide",                               "75-15-0", 11.6219) !  27 CS2
    CALL CalcH("Tetrachloromethane",                             "56-23-5", 11.6338) !  28 CCl4
    CALL CalcH("Toluene",                                       "108-88-3", 10.6998) !  29 C7H8
    CALL CalcH("Triethylamine",                                 "121-44-8",  8.7034) !  30 C6H15N
    !CALL CalcH("Water",                                       "7732-18-5", -0.3908) !  31 H2O
    CALL CalcH("Ammonia",                                      "7664-41-7",  5.2466) !  32 H3N
    CALL CalcH("Propane",                                        "74-98-6", 15.0225) !  33 C3H8
    CALL CalcH("Chlorodifluoromethane [R22]",                    "75-45-6", 12.0977) !  34 CHClF2
    CALL CalcH("Dichlorodifluoromethane [R12]",                  "75-71-8", 14.6716) !  35 CCl2F2
    CALL CalcH("Perfluoropropylene",                            "116-15-4", 15.8397) !  36 C3F6
    CALL CalcH("Perfluorocyclobutane [RC318]",                  "115-25-3", 17.3702) !  37 C4F8
    CALL CalcH("2-Methylpropane",                                "75-28-5", 15.2413) !  38 C4H10
    CALL CalcH("Chloroethane",                                   "75-00-3", 11.1438) !  39 C2H5Cl
    CALL CalcH("Furan",                                         "110-00-9",  9.3853) !  40 C4H4O
    CALL CalcH("1,3-Butadiene",                                 "106-99-0", 12.8334) !  41 C4H6
    CALL CalcH("1-Butene",                                      "106-98-9", 13.8260) !  42 C4H8
    CALL CalcH("Chlorotrifluoromethane [R13]",                   "75-72-9", 15.7154) !  43 CClF3
    CALL CalcH("trans-2-Butene",                                "624-64-6", 13.2184) !  44 C4H8
    CALL CalcH("Isobutylene",                                   "115-11-7", 13.7215) !  45 C4H8
    CALL CalcH("1,3,5-Trimethylbenzene",                        "108-67-8", 11.0182) !  46 C9H12
    CALL CalcH("Trimethylamine",                                 "75-50-3",  6.2891) !  47 C3H9N
    CALL CalcH("Chloroperfluoroethane [R115]",                   "76-15-3", 16.7090) !  48 C2ClF5
    CALL CalcH("1,2-Dichlorotetrafluoroethane [R114]",           "76-14-2", 11.2631) !  49 C2Cl2F4
    CALL CalcH("Dimethyl ether",                                "115-10-6",  9.0466) !  50 C2H6O
    CALL CalcH("Methyl chloride",                                "74-87-3", 10.8423) !  51 CH3Cl
    CALL CalcH("1,1-Difluoroethane [R152a]",                     "75-37-6", 11.4870) !  52 C2H4F2
    CALL CalcH("Ethanethiol",                                    "75-08-1", 10.3262) !  53 C2H6S
    CALL CalcH("1-Chloro-1,2,2,2-tetrafluoroethane [R124]",    "2837-89-0", 13.0556) !  54 C2HClF4
    CALL CalcH("Tetrafluoromethane [R14]",                       "75-73-0", 16.8031) !  55 CF4
    CALL CalcH("Carbon dioxide",                                "124-38-9", 12.3796) !  56 CO2
    CALL CalcH("Methane",                                        "74-82-8", 15.4500) !  57 CH4
    CALL CalcH("Oxygen",                                       "7782-44-7", 15.7559) !  58 O2
    CALL CalcH("Ethylene",                                       "74-85-1", 14.2945) !  59 C2H4
    CALL CalcH("Ethane",                                         "74-84-0", 14.9608) !  60 C2H6
    CALL CalcH("Propylene",                                     "115-07-1", 13.6569) !  61 C3H6
    CALL CalcH("Nitrogen",                                     "7727-37-9", 16.1052) !  62 N2
    CALL CalcH("Carbon monoxide",                               "630-08-0", 15.5209) !  63 CO
    CALL CalcH("Argon",                                        "7440-37-1", 15.5784) !  64 Ar
    CALL CalcH("Chlorine",                                     "7782-50-5", 11.0649) !  65 Cl2
    CALL CalcH("Krypton",                                      "7439-90-9", 14.5896) !  66 Kr
    CALL CalcH("Dinitrogen monoxide",                         "10024-97-2", 12.4172) !  67 N2O
    CALL CalcH("Xenon",                                        "7440-63-3", 14.6153) !  68 Xe
    CALL CalcH("Hydrogen",                                     "1333-74-0", 15.9733) !  69 H2
    CALL CalcH("Ethyne",                                         "74-86-2", 11.7896) !  70 C2H2
    CALL CalcH("Hydrogen sulfide",                             "7783-06-4", 11.2712) !  71 H2S
    CALL CalcH("Fluoroform [R23]",                               "75-46-7", 12.9500) !  72 CHF3
    CALL CalcH("Difluoromethane [R32]",                          "75-10-5", 11.1518) !  73 CH2F2
    CALL CalcH("Trichlorofluoromethane [R11]",                   "75-69-4", 13.2820) !  74 CCl3F
    CALL CalcH("Hexafluoroethane [R116]",                        "76-16-4", 18.1704) !  75 C2F6
    CALL CalcH("Pentafluoroethane [R125]",                      "354-33-6", 12.9734) !  76 C2HF5
    CALL CalcH("Helium",                                       "7440-59-7", 16.3215) !  77 He
    CALL CalcH("Neon",                                         "7440-01-9", 16.3297) !  78 Ne
    CALL CalcH("Sulfur hexafluoride",                          "2551-62-4", 16.8643) !  79 F6S
    CALL CalcH("Sulfur dioxide",                               "7446-09-5",  7.5514) !  80 O2S
    CALL CalcH("Perfluoropropane [R218]",                        "76-19-7", 10.8703) !  81 C3F8
    CALL CalcH("Deuterium",                                    "7782-39-0", 15.7218) !  82 D2
    CALL CalcH("Tetrafluoroethylene",                           "116-14-3", 15.2178) !  83 C2F4
    CALL CalcH("Nitrogen oxide",                              "10102-43-9", 15.0953) !  84 NO
    CALL CalcH("Carbonyl sulfide",                              "463-58-1", 12.5125) !  85 COS
    CALL CalcH("Radon",                                       "10043-92-2", 13.2227) !  86 Rn
    CALL CalcH("1-Chloro-1,1-difluoroethane [R142b]",            "75-68-3", 12.7432) !  87 C2H3ClF2
    CALL CalcH("Diacetylene",                                   "460-12-8",  5.3086) !  88 C4H2
    CALL CalcH("Methyl fluoride [R41]",                         "593-53-3", 20.1116) !  89 CH3F
    CALL CalcH("Cyclopropane",                                   "75-19-4", 13.1076) !  90 C3H6
    CALL CalcH("Cyclobutane",                                   "287-23-0", 17.5632) !  91 C4H8
    CALL CalcH("Propyne",                                        "74-99-7", 11.1998) !  92 C3H4
    CALL CalcH("Sulfur trioxide",                              "7446-11-9", 20.8145) !  93 O3S
    CALL CalcH("Ozone",                                       "10028-15-6", 13.0082) !  94 O3
    CALL CalcH("1,1,1,2-Tetrafluoroethane [R134a]",             "811-97-2", 12.3106) !  95 C2H2F4
    CALL CalcH("1,1-Dichloro-1-fluoroethane [R141b]",          "1717-00-6", 12.1463) !  96 C2H3Cl2F
    CALL CalcH("1,1,1,2,3,3,3-Heptafluoropropane [R227ea]",     "431-89-0", 12.4330) !  97 C3HF7
    CALL CalcH("Perchlorylfluoride",                           "7616-94-6", 14.0412) !  98 ClFO3
    CALL CalcH("1-Chloro-2,2,2-trifluoroethane [R133a]",         "75-88-7", 11.6611) !  99 C2H2ClF3
    CALL CalcH("1-Chloro-2,2-difluoroethylene (R1122)",         "359-10-4", 12.6042) ! 100 C2HClF2
    CALL CalcH("Chloroamine",                                 "10599-90-3", 13.9929) ! 101 H2ClN

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, lnHij)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: lnHij
      chem=chem_ ; casrn=casrn_ ! make global for Output
      Hominus = cH2O / (EXP(lnHij) * 1E3)
      CALL MakeNote(TRIM(ref), &
        "Calculated using machine learning matrix completion methods (MCMs).")
      CALL Output(Hominus)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3728

  !---------------------------------------------------------------------------

  SUBROUTINE ref3729 ! Hvpc [atm*L/mol]
    IMPLICIT NONE

    ref = "3729"

    !          name                            casrn  inchikey                                exp   GA-MLR       RM     GDX      LM      BR    BFGS      RP
    CALL CalcH("Acephate",              "30560-19-1", "YASYVMFAVPKPKE-UHFFFAOYSA-N", "",    7.284,   3.303,   4.604,  4.422,  5.009,  4.649,  4.686,  5.033) !  1
    CALL CalcH("Alachlor",              "15972-60-8", "XCSGPAVHZFQHGE-UHFFFAOYSA-N", "V",   2.207,   2.392,   0.881,  0.817,  0.993,  0.565,  0.603,  1.199) !  2
    CALL CalcH("Aldicarb",                "116-06-3", "QGLZXHRNAYXIBU-WEVVVXLNSA-N", "T",   3.495,   1.106,   3.364,  3.685,  3.277,  3.416,  3.788,  3.527) !  3
    CALL CalcH("Aldrin",                  "309-00-2", "QBYJBZPUGVGKQQ-SJJAEHHWSA-N", "V",  -1.959,  -1.464,  -1.090, -1.299, -1.442, -1.187, -1.290, -1.047) !  4
    CALL CalcH("Ametryn",                 "834-12-8", "RQVYBGPQFYCBGX-UHFFFAOYSA-N", "",    3.921,   3.617,   4.468,  4.076,  3.960,  4.133,  4.476,  4.099) !  5
    CALL CalcH("Atrazine",               "1912-24-9", "MXWJVTOOROXGIU-UHFFFAOYSA-N", "",    3.537,   3.559,   3.929,  3.789,  3.589,  3.864,  4.066,  3.712) !  6
    CALL CalcH("Azinphos-methyl",          "86-50-0", "CJJOSEISRRTUQB-UHFFFAOYSA-N", "",    2.495,   4.685,   3.635,  4.333,  3.948,  3.669,  3.990,  3.595) !  7
    CALL CalcH("Bentazone",             "25057-89-0", "ZOMSMJKLGFBRBS-UHFFFAOYSA-N", "V",   4.131,   4.719,   4.761,  4.267,  4.615,  4.556,  4.797,  4.721) !  8
    CALL CalcH("Bifenox",               "42576-02-3", "SUSRORUBZHMPCO-UHFFFAOYSA-N", "T",   0.523,   1.397,   0.815,  0.588,  0.972,  0.544,  0.156,  1.107) !  9
    CALL CalcH("Bromacil",                "314-40-9", "CTSLUCNDVMMDHG-UHFFFAOYSA-N", "",    2.721,   3.996,   3.277,  3.441,  3.309,  3.334,  3.610,  3.243) ! 10
    CALL CalcH("Bromofenoxim",          "13181-17-4", "VTQWKUZUPOTXEH-UHFFFAOYSA-N", "V",  -0.501,   2.355,  -0.905, -1.379, -1.818, -1.088, -1.644, -1.070) ! 11
    CALL CalcH("Butylate",               "2008-41-5", "BMTAFVWTTFSTOG-UHFFFAOYSA-N", "",    0.252,   0.6181,  1.375,  1.319,  1.381,  1.321,  0.928,  1.119) ! 12
    CALL CalcH("Captan",                  "133-06-2", "LDVVMCZRFWMZSG-UHFFFAOYSA-N", "T",   0.222,  -0.341,  -1.119, -1.404, -1.530, -0.671, -0.933, -0.965) ! 13
    CALL CalcH("Carbaryl",                 "63-25-2", "CVXBEEMKQHEXEN-UHFFFAOYSA-N", "T",   3.886,   3.100,   3.260,  3.579,  2.548,  2.915,  2.622,  2.951) ! 14
    CALL CalcH("Carbetamide",           "16118-49-3", "AMRQXHFXNZFDCH-UHFFFAOYSA-N", "",    2.055,   3.860,   2.651,  2.879,  2.327,  2.569,  3.080,  2.646) ! 15
    CALL CalcH("Carbofuran",             "1563-66-2", "DUEPRVBVGDRKAG-UHFFFAOYSA-N", "",    3.292,   3.314,   3.865,  3.799,  3.404,  3.669,  4.035,  3.576) ! 16
    CALL CalcH("Chlordane",                "57-74-9", "BIWJNBZANLAXMG-UHFFFAOYSA-N", "V",  -0.955,  -1.814,  -1.962, -1.837, -1.793, -1.662, -1.443, -1.836) ! 17
    CALL CalcH("Chlorfenvinphos",         "470-90-6", "FSAVDKDHPDSCTO-XYOKQWHBSA-N", "",    3.553,   1.696,   2.241,  3.042,  2.411,  2.623,  2.269,  2.595) ! 18
    CALL CalcH("Chlorotoluron",         "15545-48-9", "JXCGFZXSOMJFOA-UHFFFAOYSA-N", "T",   4.276,   0.759,   3.544,  3.279,  3.669,  3.535,  4.188,  3.389) ! 19
    CALL CalcH("Chlorpropham",            "101-21-3", "CWJSHJJYOPWUGX-UHFFFAOYSA-N", "T",   3.677,   1.726,   2.506,  2.825,  2.382,  2.467,  2.389,  2.410) ! 20
    CALL CalcH("Chlorpyrifos",           "2921-88-2", "SBPBAQFWLVIOKP-UHFFFAOYSA-N", "V",  -0.243,   2.152,   1.195,  1.077,  0.205,  1.004,  0.628,  0.786) ! 21
    CALL CalcH("Clopyralid",             "1702-17-6", "HUBANNPOLNYSAD-UHFFFAOYSA-N", "",    4.508,   3.145,   3.443,  4.129,  3.271,  3.564,  2.997,  3.747) ! 22
    CALL CalcH("Cyanazine",             "21725-46-2", "MZZBPDKVEFVLFF-UHFFFAOYSA-N", "",    6.523,   5.998,   4.047,  3.896,  3.978,  3.987,  4.503,  4.239) ! 23
    CALL CalcH("Cymoxanil",             "57966-95-7", "XERJKGMBORTKEO-UHFFFAOYSA-N", "V",   4.796,   3.739,   3.439,  4.114,  3.647,  3.512,  3.666,  3.638) ! 24
    CALL CalcH("Cypermethrin",          "52315-07-8", "KAATUXNTWXVJKI-UHFFFAOYSA-N", "V",   3.409,   2.913,   3.069,  3.626,  3.051,  3.026,  3.782,  2.717) ! 25
    CALL CalcH("Cyproconazole",         "94361-06-5", "UFNOUKDBUJZYDE-UHFFFAOYSA-N", "",    4.136,   3.832,   4.979,  4.204,  5.271,  4.536,  5.146,  4.603) ! 26
    CALL CalcH("DDT",                      "50-29-3", "YVGGHNCTFXOJCH-UHFFFAOYSA-N", "",   -0.373,  -1.915,   0.435,  0.178,  1.275, -0.108, -0.791,  0.237) ! 27
    CALL CalcH("Diazinon",                "333-41-5", "FHIVAFMUCKRCQO-UHFFFAOYSA-N", "",    1.174,   2.183,   1.302,  1.139,  1.139,  1.242,  1.406,  1.083) ! 28
    CALL CalcH("Dicamba",                "1918-00-9", "IWEDIXLBFLAXBO-UHFFFAOYSA-N", "V",   3.921,   1.517,   2.164,  2.551,  2.071,  2.072,  1.759,  2.373) ! 29
    CALL CalcH("Dichlobenil",            "1194-65-6", "YOYAIZYFCNQIRF-UHFFFAOYSA-N", "V",   0.174,   0.344,   0.208, -0.078, -0.409,  0.177, -0.266,  0.599) ! 30
    CALL CalcH("1,3-Dichloropropene",     "542-75-6", "UOORRWUZONOOLO-UHFFFAOYSA-N", "",   -2.613,   0.337,  -2.835, -1.302, -2.010, -2.290, -2.399, -2.568) ! 31
    CALL CalcH("Dichlorvos",               "62-73-7", "OEBRKCOSUFCWJD-UHFFFAOYSA-N", "T",   0.721,   1.929,   1.798,  2.060,  1.781,  1.994,  1.925,  2.134) ! 32
    CALL CalcH("Dieldrin",                 "60-57-1", "DFBKLUNHFCTMDC-NLUYNBKHSA-N", "T",  -0.049,   0.849,  -1.404, -1.378, -1.821, -1.125, -1.371, -1.417) ! 33
    CALL CalcH("Difenoconazole",       "119446-68-3", "BQYJATMQXGBDHF-UHFFFAOYSA-N", "",    5.824,   5.582,   4.223,  3.989,  3.745,  3.891,  4.507,  3.732) ! 34
    CALL CalcH("Diflubenzuron",         "35367-38-5", "QQQYTWIFVNKMRW-UHFFFAOYSA-N", "",    3.328,   4.460,   3.914,  3.782,  3.038,  3.520,  3.563,  2.940) ! 35
    CALL CalcH("Dinoseb",                  "88-85-7", "OWZPCEFYPSAJFR-UHFFFAOYSA-N", "T",  -1.708,   0.094,  -0.075, -0.625, -0.849, -0.204, -0.160, -0.656) ! 36
    CALL CalcH("Dimethoate",               "60-51-5", "MCWXGJITAZMZEV-UHFFFAOYSA-N", "V",   3.958,   4.543,   4.782,  4.908,  5.003,  4.734,  4.752,  4.928) ! 37
    CALL CalcH("Dinoterb",               "1420-07-1", "IIPZYDQGBIWLBU-UHFFFAOYSA-N", "V",   2.229,   1.130,   2.187,  2.327,  2.197,  1.965,  1.910,  2.489) ! 38
    CALL CalcH("Disulfoton",              "298-04-4", "DOFZAZXDOSGAJZ-UHFFFAOYSA-N", "T",   0.657,   1.019,   1.697,  1.922,  1.677,  1.811,  1.250,  2.158) ! 39
    CALL CalcH("Diuron",                  "330-54-1", "XMTQQYYKAHVGBJ-UHFFFAOYSA-N", "",    2.921,   1.407,   3.758,  3.690,  3.658,  3.722,  4.048,  3.563) ! 40
    CALL CalcH("DNOC",                    "534-52-1", "ZXVONLUNISGICL-UHFFFAOYSA-N", "",    1.958,   2.818,   2.732,  3.439,  2.427,  2.623,  2.079,  3.102) ! 41
    CALL CalcH("EPTC",                    "759-94-4", "GUVLYNGULCJVDO-UHFFFAOYSA-N", "",    0.0086,  0.422,   0.396,  0.588,  0.459,  0.040, -0.345, -0.208) ! 42
    CALL CalcH("Ethion",                  "563-12-2", "RIZMRRKBZQXFOY-UHFFFAOYSA-N", "",    1.495,   2.504,   1.719,  1.748,  1.637,  1.836,  1.4178, 1.517) ! 43
    CALL CalcH("Ethofumesate",          "26225-79-6", "IRCMYGHHKLLGHV-UHFFFAOYSA-N", "V",   3.657,   5.174,   4.117,  4.039,  4.153,  4.062,  4.609,  4.485) ! 44
    CALL CalcH("Fenitrothion",            "122-14-5", "ZNOLGFHPUIJIMJ-UHFFFAOYSA-N", "V",   2.444,   2.975,   3.639,  3.304,  3.291,  3.649,  3.356,  3.087) ! 45
    CALL CalcH("Fenpropimorph",         "67564-91-4", "RYAUSSKQMZRMAI-ALOPSCKCSA-N", "",    0.796,   1.889,   1.286,  0.901,  1.342,  0.903,  1.284,  1.498) ! 46
    CALL CalcH("Fenthion",                 "55-38-9", "PNVJTZOFSHSLTO-UHFFFAOYSA-N", "",    1.657,   1.661,   2.248,  2.473,  2.322,  2.279,  2.391,  2.218) ! 47
    CALL CalcH("Fenuron",                 "101-42-8", "XXOYNJXVWVNOOJ-UHFFFAOYSA-N", "T",   3.568,   3.894,   2.531,  2.481,  2.630,  2.652,  2.475,  2.411) ! 48
    CALL CalcH("Flusilazole",           "85509-19-9", "FQKUGOMFVDPBIZ-UHFFFAOYSA-N", "T",   3.444,   1.349,   5.550,  4.312,  4.278,  4.651,  4.695,  3.784) ! 49
    CALL CalcH("Folpet",                  "133-07-3", "HKIOYBQGHSTUDB-UHFFFAOYSA-N", "",    0.42,   -0.214,   1.094,  0.714,  0.461,  1.086,  0.342,  0.501) ! 50
    CALL CalcH("Heptachlor",               "76-44-8", "FRCCEHPWNOQAEU-UHFFFAOYSA-N", "",   -2.049,  -1.173,  -2.128, -1.936, -1.970, -1.703, -1.483, -2.008) ! 51
    CALL CalcH("Hexaconazole",          "79983-71-4", "STMIIPIFODONDC-UHFFFAOYSA-N", "",    3.456,   3.653,   3.861,  3.886,  3.873,  3.785,  4.479,  3.714) ! 52
    CALL CalcH("Iprodione",             "36734-19-7", "ONUFESLQCSAYKA-UHFFFAOYSA-N", "V",   4.854,   4.057,   3.229,  3.458,  3.192,  3.307,  3.691,  3.315) ! 53
    CALL CalcH("Isofenphos",            "25311-71-1", "HOQADATXFBOEGG-UHFFFAOYSA-N", "V",   5.018,   3.315,   2.013,  1.963,  1.686,  1.846,  2.153,  2.138) ! 54
    CALL CalcH("Linuron",                 "330-55-2", "XKJMBINCVNINCA-UHFFFAOYSA-N", "T",   2.267,   2.888,   3.337,  3.555,  3.026,  3.357,  3.386,  3.108) ! 55
    CALL CalcH("Malathion",               "121-75-5", "JXSJBGJIGXNWCI-UHFFFAOYSA-N", "",    2.638,   0.956,   1.956,  2.251,  2.288,  2.257,  2.398,  2.419) ! 56
    CALL CalcH("Mecoprop",                 "93-65-2", "WNTGYJSOUMFZEP-UHFFFAOYSA-N", "",    3.958,   3.378,   4.328,  3.913,  3.765,  4.152,  4.133,  3.857) ! 57
    CALL CalcH("Metamitron",            "41394-05-2", "VHCNQEUWZYOAEV-UHFFFAOYSA-N", "",    8.018,   6.571,   6.621,  5.631,  5.621,  5.656,  5.264,  6.080) ! 58
    CALL CalcH("Metazachlor",           "67129-08-2", "STEPQTYSZVCJPV-UHFFFAOYSA-N", "",    3.113,   3.017,   5.014,  4.327,  4.639,  4.547,  4.878,  4.689) ! 59
    CALL CalcH("Methomyl",              "16752-77-5", "UHXUZOCRWCRNSJ-UHFFFAOYSA-N", "V",   4.187,   2.721,   4.094,  4.021,  3.814,  4.109,  3.724,  4.042) ! 60
    CALL CalcH("Metobromuron",           "3060-89-7", "WLFDQEVORAMCIM-UHFFFAOYSA-N", "",    3.508,   3.569,   3.165,  3.237,  2.640,  2.933,  2.758,  3.042) ! 61
    CALL CalcH("Metolachlor",           "51218-45-2", "WVQBLGZPHOPPFO-UHFFFAOYSA-N", "",    3.046,   3.307,   1.186,  1.015,  1.178,  0.904,  1.045,  1.432) ! 62
    CALL CalcH("Metoxuron",             "19937-59-8", "DSRNRYQBBJQVCW-UHFFFAOYSA-N", "",    2.824,   2.548,   2.904,  2.984,  3.116,  3.096,  3.115,  2.842) ! 63
    CALL CalcH("Oxamyl",                "23135-22-0", "KZAUOCCYDRDERY-UITAMQMPSA-N", "",    3.585,   1.600,   3.659,  3.748,  3.994,  3.809,  4.112,  4.106) ! 64
    CALL CalcH("Parathion",                "56-38-2", "LCCNCVORNKJIRZ-UHFFFAOYSA-N", "",    1.91,    3.682,   1.749,  1.290,  1.283,  1.512,  1.198,  1.168) ! 65
    CALL CalcH("Parathion-methyl",        "298-00-0", "RLBIQVVOMOPOHC-UHFFFAOYSA-N", "",    1.678,   3.671,   3.409,  2.919,  2.948,  3.334,  2.863,  2.491) ! 66
    CALL CalcH("Pebulate",               "1114-71-2", "SGEJQUSYQTVSIU-UHFFFAOYSA-N", "",   -1.068,   0.919,   0.280,  0.533,  0.425, -0.172, -0.366, -0.181) ! 67
    CALL CalcH("Pendimethalin",         "40487-42-1", "CHIFOSRWCNZCFN-UHFFFAOYSA-N", "",   -0.577,   1.076,   0.087, -0.554, -0.236, -0.312, -0.514, -0.122) ! 68
    CALL CalcH("Pentachlorophenol",        "87-86-5", "IZUPBVBPLAPZRR-UHFFFAOYSA-N", "V",   1.356,   0.480,   1.357,  1.711,  1.234,  1.4038, 0.613,  1.794) ! 69
    CALL CalcH("Phenmedipham",          "13684-63-4", "IDOWTHOLJBTAFI-UHFFFAOYSA-N", "V",   7.096,   3.950,   6.157,  4.558,  4.481,  4.734,  4.798,  4.405) ! 70
    CALL CalcH("Phorate",                 "298-02-2", "BULVZWIRKLYCBC-UHFFFAOYSA-N", "",    0.187,   1.918,   0.800,  0.698,  0.592,  0.660,  0.045, -0.107) ! 71
    CALL CalcH("Phosmet",                 "732-11-6", "LMNZTLDVJIUSHT-UHFFFAOYSA-N", "",    3.022,   3.237,   2.617,  2.930,  2.641,  2.580,  2.725,  2.762) ! 72
    CALL CalcH("Picloram",               "1918-02-1", "NQQVFXUMIDALNH-UHFFFAOYSA-N", "T",   4.468,   5.468,   5.197,  4.979,  5.225,  5.054,  4.972,  5.550) ! 73
    CALL CalcH("Prochloraz",            "67747-09-5", "TVLSRXXIMLFWEO-UHFFFAOYSA-N", "",    2.769,   2.558,   2.177,  2.195,  1.947,  1.776,  1.496,  2.329) ! 74
    CALL CalcH("Procymidone",           "32809-16-8", "QXJKBPAVAHBARF-UHFFFAOYSA-N", "",   -0.041,   0.783,   0.212, -0.117, -0.436,  0.150,  0.126, -0.438) ! 75
    CALL CalcH("Prometon",               "1610-18-0", "ISEUFVQQFVOBCY-UHFFFAOYSA-N", "",    4.045,   3.460,   4.243,  4.082,  3.633,  3.842,  4.443,  3.991) ! 76
    CALL CalcH("Prometryn",              "7287-19-6", "AAEVYOVXGOFMJO-UHFFFAOYSA-N", "",    3.301,   2.980,   5.031,  4.225,  4.531,  4.399,  4.925,  4.566) ! 77
    CALL CalcH("Propachlor",             "1918-16-7", "MFOUDYKPLGXPGO-UHFFFAOYSA-N", "",    1.958,   3.356,   1.616,  1.665,  1.711,  1.574,  1.539,  1.681) ! 78
    CALL CalcH("Propanil",                "709-98-8", "LFULEKSKNZEWOE-UHFFFAOYSA-N", "T",   2.444,  -0.112,   1.173,  1.169,  1.579,  1.100,  1.165,  1.454) ! 79
    CALL CalcH("Propazine",               "139-40-2", "WJNRPILHGGKWCK-UHFFFAOYSA-N", "T",   4.,      3.039,   4.585,  4.093,  4.248,  4.244,  4.754,  4.308) ! 80
    CALL CalcH("Propiconazole",         "60207-90-1", "STJLVHWMYQXCPB-UHFFFAOYSA-N", "T",   3.387,   4.174,   3.454,  3.698,  3.368,  3.498,  3.947,  3.374) ! 81
    CALL CalcH("Propoxur",                "114-26-1", "ISRUGXGCCGIOQO-UHFFFAOYSA-N", "",    0.886,   1.341,   2.339,  2.646,  2.232,  2.152,  2.338,  2.277) ! 82
    CALL CalcH("Propyzamide",           "23950-58-5", "PHNUZKMIPFFYSO-UHFFFAOYSA-N", "",    0.721,   0.225,   0.986,  0.885,  1.502,  0.826,  0.997,  0.624) ! 83
    CALL CalcH("Pyridate",              "55512-33-9", "JTZCTMAVMHRNTR-UHFFFAOYSA-N", "",    4.481,   3.191,   2.676,  3.260,  2.624,  2.692,  3.137,  2.823) ! 84
    CALL CalcH("Simazine",                "122-34-9", "ODCWYMIRDDJXKW-UHFFFAOYSA-N", "",    3.468,   3.941,   4.140,  3.994,  3.504,  3.964,  3.806,  3.748) ! 85
    CALL CalcH("Sulfotep",               "3689-24-5", "XIUROWKZWPIAIB-UHFFFAOYSA-N", "T",   0.,      3.396,  -0.307, -0.138, -1.418, -0.349, -1.524, -1.699) ! 86
    CALL CalcH("Tebuconazole",         "107534-96-3", "PXMNMQRDXWABCY-UHFFFAOYSA-N", "T",   5.031,   4.250,   4.755,  4.246,  4.711,  4.354,  5.092,  4.831) ! 87
    CALL CalcH("Terbacil",               "5902-51-2", "NBQCNZYJJMBDKY-UHFFFAOYSA-N", "",    4.745,   3.312,   4.650,  4.396,  4.974,  4.648,  4.800,  5.059) ! 88
    CALL CalcH("Temephos",               "3383-96-8", "WWJZWCUNLNYYAU-UHFFFAOYSA-N", "",    2.236,   3.013,   2.653,  2.971,  2.606,  2.649,  3.172,  2.598) ! 89
    CALL CalcH("Terbufos",              "13071-79-9", "XLNZEKHULJKQBA-UHFFFAOYSA-N", "",    0.,      0.889,   1.415,  1.372,  1.367,  1.477,  1.299,  1.003) ! 90
    CALL CalcH("Terbutryn",               "886-50-0", "IROINLKCQGIITA-UHFFFAOYSA-N", "",    2.886,   3.628,   4.709,  4.179,  4.285,  4.227,  4.861,  4.481) ! 91
    CALL CalcH("Triallate",              "2303-17-5", "MWBPRDONLNQCFV-UHFFFAOYSA-N", "",    0.008,  -0.296,   0.089, -0.170, -0.211,  0.200,  0.551,  0.614) ! 92
    CALL CalcH("Trichlorfon",              "52-68-6", "NFACJZMKEDPNKN-UHFFFAOYSA-N", "",    5.769,   2.797,   3.382,  3.952,  3.704,  3.590,  3.501,  4.030) ! 93
    CALL CalcH("Trifluralin",            "1582-09-8", "ZSDSQXJSNMTJDA-UHFFFAOYSA-N", "",   -0.604,   0.108,  -0.942, -1.443, -1.521, -1.075, -1.363, -0.845) ! 94
    CALL CalcH("Vernolate",              "1929-77-7", "OKUGPJPKMAEJOE-UHFFFAOYSA-N", "",   -0.311,   0.346,  -0.011,  0.436,  0.076, -0.531, -0.537, -0.488) ! 95
    CALL CalcH("Vinclozolin",           "50471-44-8", "FSCWZHGZWWDELK-UHFFFAOYSA-N", "",    2.886,   1.577,   2.099,  2.594,  2.241,  2.238,  2.268,  2.329) ! 96

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, info, exp, GA_MLR, RM, GDX, LM, BR, BFGS, RP)

      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_, inchikey_, info
      REAL,             INTENT(IN) :: exp, GA_MLR, RM, GDX, LM, BR, BFGS, RP
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      ! experimental value:
      type = "X"
      CALL SettypeX("barcelo97")
      CALL Output(Hscp_TO_HscpSI / (10.**(-exp)))
      ! value from BR:
      type = "Q"
      CALL MakeNote(TRIM(ref)//"_BR", TRIM(citet())//" compared several QSPR methods " // &
        "and found that the Levenberg--Marquardt algorithm with Bayesian regularization " // &
        "produces the best results. Values obtained with other methods " // &
        "can be found in their supplement.")
      IF (TRIM(info)=="V") &
        CALL MakeNote(TRIM(ref)//"_val", "Value from the validation set.")
      IF (TRIM(info)=="T") &
        CALL MakeNote(TRIM(ref)//"_test", "Value from the test set.")
      CALL Output(Hscp_TO_HscpSI / (10.**(-BR)))
    END SUBROUTINE CalcH

  END SUBROUTINE ref3729

  !---------------------------------------------------------------------------

  SUBROUTINE ref3730 ! Hvpc [atm*m3/mol]
    IMPLICIT NONE
    REAL    :: logKH_exp, logKH_eq_2, devi, logKH_duchowicz, devi_duchowicz
    INTEGER :: i

    ! Data is from Tabs. S1 and S2 in the supplement. Duchowicz is ref3586.
    ref = "3730"
    type = "Q"
    OPEN (10, FILE="input/ref3730.dat", STATUS="OLD")
    DO
      READ (10,*,END=999) i, chem, inchikey, casrn, &
        logKH_exp, logKH_eq_2, devi, logKH_duchowicz, devi_duchowicz
      CALL consistency_check(logKH_exp+devi,           logKH_eq_2,      "eq_2")
      CALL consistency_check(logKH_exp+devi_duchowicz, logKH_duchowicz, "Duchowicz")
      Hominus = 1. / (atm*(10.**logKH_eq_2))
      CALL Output(Hominus)
    ENDDO
999 CONTINUE
    CLOSE(10)

  END SUBROUTINE ref3730

  !---------------------------------------------------------------------------

  SUBROUTINE ref3731 ! Hvpx [atm]
    IMPLICIT NONE

    ref = "3731"

    CALL CalcH("tribromomethane",                      "75-25-2", "DIKBFYAXUHHXCS-UHFFFAOYSA-N", "",   1.51,   1.330,  1.380) !   1
    CALL CalcH("dichlorofluoromethane",                "75-43-4", "UMNKXPULIDJLSU-UHFFFAOYSA-N", "",   2.46,   2.775,  2.549) !   2
    CALL CalcH("trichloromethane",                     "67-66-3", "HEDRZPFGACZZDS-UHFFFAOYSA-N", "",   2.36,   2.179,  2.126) !   3
    CALL CalcH("bromochloromethane",                   "74-97-5", "JPOXNPPZZKNXOV-UHFFFAOYSA-N", "",   1.95,   2.048,  1.984) !   4
    CALL CalcH("dibromomethane",                       "74-95-3", "FJBFPHVGVWTDIP-UHFFFAOYSA-N", "T",  1.71,   1.868,  1.825) !   5
    CALL CalcH("chlorofluoromethane",                 "593-70-4", "XWCDCDSDNJVCLO-UHFFFAOYSA-N", "",   2.54,   2.950,  2.918) !   6
    CALL CalcH("dichloromethane",                      "75-09-2", "YMWUJEATGCHHMB-UHFFFAOYSA-N", "",   2.14,   2.339,  2.265) !   7
    CALL CalcH("diiodomethane",                        "75-11-6", "NZZFYRREKKOMAT-UHFFFAOYSA-N", "",   1.28,   0.571,  1.301) !   8
    CALL CalcH("bromomethane",                         "74-83-9", "GZUXJHMPEANEGY-UHFFFAOYSA-N", "",   2.58,   2.537,  2.491) !   9
    CALL CalcH("chloromethane",                        "74-87-3", "NEHMKBQYUWJMIP-UHFFFAOYSA-N", "",   2.66,   2.724,  2.802) !  10
    CALL CalcH("methane",                              "74-82-8", "VNWKTOKETHGBQD-UHFFFAOYSA-N", "",   4.55,   4.729,  4.258) !  11
    CALL CalcH("methanol",                             "67-56-1", "OKKJLVBELUTLKV-UHFFFAOYSA-N", "T", -0.54,  -0.399, -0.308) !  12
    CALL CalcH("methanethiol",                         "74-93-1", "LSDPWZHWYPCBBB-UHFFFAOYSA-N", "",   2.03,   1.659,  1.733) !  13
    CALL CalcH("chloropentafluoroethane",              "76-15-3", "RFCAUADVODFSLZ-UHFFFAOYSA-N", "",   5.16,   4.989,  4.353) !  14
    CALL CalcH("1,2-dichlorotetrafluoroethane",        "76-14-2", "DDMOUSALMHHKOS-UHFFFAOYSA-N", "",   4.83,   4.338,  3.705) !  15
    CALL CalcH("1,1,2,2-tetrachlorodifluoroethane",    "76-12-0", "UGCSPKPEHQEOSR-UHFFFAOYSA-N", "",   3.84,   4.074,  3.683) !  16
    CALL CalcH("hexachloroethane",                     "67-72-1", "VHHHONWQHHHLTI-UHFFFAOYSA-N", "",   3.15,   3.160,  3.357) !  17
    CALL CalcH("tetrafluoroethene",                   "116-14-3", "BFKJFAAPBSQJPD-UHFFFAOYSA-N", "",   4.53,   4.471,  4.284) !  18
    CALL CalcH("1,1-dichloroethene",                   "75-35-4", "LGXVIGDEPROXKC-UHFFFAOYSA-N", "T",  3.10,   2.365,  2.320) !  19
    CALL CalcH("(Z)-1,2-dichloroethene",              "156-59-2", "KFUSEUYYWQURPO-UPHRSURJSA-N", "",   2.61,   1.990,  2.020) !  20
    CALL CalcH("(E)-1,2-dichloroethene",              "156-60-5", "KFUSEUYYWQURPO-OWOJBTEDSA-N", "",   2.57,   2.473,  2.392) !  21
    CALL CalcH("chloroethene",                         "75-01-4", "BZHJMEDXRYGGRV-UHFFFAOYSA-N", "",   3.09,   2.597,  2.580) !  22
    CALL CalcH("ethanal",                              "75-07-0", "IKHGUXGNUITLKF-UHFFFAOYSA-N", "",   0.73,   0.650,  0.916) !  23
    CALL CalcH("bromoethane",                          "74-96-4", "RDHPKYGYEGBMSE-UHFFFAOYSA-N", "",   2.61,   2.884,  2.852) !  24
    CALL CalcH("chloroethane",                         "75-00-3", "HRYZWHHZPQKTII-UHFFFAOYSA-N", "",   2.58,   2.995,  3.056) !  25
    CALL CalcH("iodoethane",                           "75-03-6", "HVTICUPFWKNHNG-UHFFFAOYSA-N", "T",  2.60,   2.057,  2.114) !  26
    CALL CalcH("ethane",                               "74-84-0", "OTMSDBZUPAUEDD-UHFFFAOYSA-N", "",   4.43,   4.460,  4.407) !  27
    CALL CalcH("ethanol",                              "64-17-5", "LFQSCWFLJHTTHZ-UHFFFAOYSA-N", "",  -0.35,  -0.356, -0.404) !  28
    CALL CalcH("1,2-ethanediol",                      "107-21-1", "LYCAIKOWRPUZTN-UHFFFAOYSA-N", "",  -3.98,  -3.590, -4.102) !  29
    CALL CalcH("ethanethiol",                          "75-08-1", "DNJIEGIFACGWOD-UHFFFAOYSA-N", "",   2.21,   1.971,  1.977) !  30
    CALL CalcH("ethanamine",                           "75-04-7", "QUSNBJAOOMFDIB-UHFFFAOYSA-N", "",   0.16,   0.191,  0.218) !  31
    CALL CalcH("ethylenediamine",                     "107-15-3", "PIICEJLVQHRZGT-UHFFFAOYSA-N", "",  -2.61,  -2.921, -3.057) !  32
    CALL CalcH("2,3-dichloropropene",                  "78-88-6", "FALCMQXTWHPRIH-UHFFFAOYSA-N", "T",  2.36,   2.549,  2.487) !  33
    CALL CalcH("3-bromo-1-propene",                   "106-95-6", "BHELZAPQIKSEDF-UHFFFAOYSA-N", "",   2.51,   2.785,  2.727) !  34
    CALL CalcH("3-chloro-1-propene",                  "107-05-1", "OSDWBNJEKMUWAV-UHFFFAOYSA-N", "",   2.71,   2.970,  2.963) !  35
    CALL CalcH("propene",                             "115-07-1", "QQONPFPTGQHPMA-UHFFFAOYSA-N", "",   4.05,   3.656,  3.719) !  36
    CALL CalcH("2-propen-1-ol",                       "107-18-6", "XXROGKLTLUQVRX-UHFFFAOYSA-N", "",  -0.47,  -0.736, -0.840) !  37
    CALL CalcH("methyl ethanoate",                     "79-20-9", "KXKVLQRXCPHEJC-UHFFFAOYSA-N", "",   0.59,   1.088,  1.004) !  38
    CALL CalcH("1-bromopropane",                      "106-94-5", "CYNYIHKIEHGYOZ-UHFFFAOYSA-N", "",   2.71,   3.191,  3.150) !  39
    CALL CalcH("2-bromopropane",                       "75-26-3", "NAMYKGVDVNBCFQ-UHFFFAOYSA-N", "T",  2.83,   3.277,  3.208) !  40
    CALL CalcH("1-chloropropane",                     "540-54-5", "SNMVRZFUUCLYTO-UHFFFAOYSA-N", "",   2.86,   3.245,  3.298) !  41
    CALL CalcH("2-iodopropane",                        "75-30-9", "FMKOJHQHASLBPH-UHFFFAOYSA-N", "",   2.80,   2.486,  2.483) !  42
    CALL CalcH("1-iodopropane",                       "107-08-4", "PVWOIHVRPOBWPI-UHFFFAOYSA-N", "",   2.70,   2.423,  2.454) !  43
    CALL CalcH("propane",                              "74-98-6", "ATUOYWHBWRKTHZ-UHFFFAOYSA-N", "",   4.58,   4.454,  4.434) !  44
    CALL CalcH("2-propanol",                           "67-63-0", "KFZMGEQAYNKOFK-UHFFFAOYSA-N", "",  -0.16,  -0.017, -0.150) !  45
    CALL CalcH("1-propanol",                           "71-23-8", "BDERNNFJNOPAEC-UHFFFAOYSA-N", "",  -0.36,  -0.204, -0.312) !  46
    CALL CalcH("1-propanethiol",                      "107-03-9", "SUVIGLJNEAMWEG-UHFFFAOYSA-N", "T",  2.40,   2.174,  2.142) !  47
    CALL CalcH("2-propanethiol",                       "75-33-2", "KJRCEJOSASVSRA-UHFFFAOYSA-N", "",   2.37,   2.335,  2.278) !  48
    CALL CalcH("ethyl methyl sulfide",                "624-89-5", "WXEHBUMAEPOYKP-UHFFFAOYSA-N", "",   2.04,   2.072,  2.033) !  49
    CALL CalcH("1-propanamine",                       "107-10-8", "WGYKZJWCGVVSQN-UHFFFAOYSA-N", "",   0.15,   0.343,  0.330) !  50
    CALL CalcH("2-propanamine",                        "75-31-0", "JJWLVOIRVHMVIS-UHFFFAOYSA-N", "",   0.63,   0.555,  0.521) !  51
    CALL CalcH("hexachlorobutadiene",                  "87-68-3", "RWNKSTSCBHKHTB-UHFFFAOYSA-N", "",   3.12,   2.630,  3.309) !  52
    CALL CalcH("oxacyclopentadiene",                  "110-00-9", "YLQBMQCUIZJEEH-UHFFFAOYSA-N", "",   2.48,   2.319,  2.107) !  53
    CALL CalcH("thiophene",                           "110-02-1", "YTPLMLYBLZKORZ-UHFFFAOYSA-N", "T",  2.20,   1.982,  1.771) !  54
    CALL CalcH("1,3-butadiene",                       "106-99-0", "KAKZBPTYRLMSJV-UHFFFAOYSA-N", "",   3.60,   3.357,  3.337) !  55
    CALL CalcH("1-butene",                            "106-98-9", "VXNZUUAINFGPBY-UHFFFAOYSA-N", "",   4.13,   3.820,  3.888) !  56
    CALL CalcH("cis-2-butene",                        "590-18-1", "IAQRGUVFOMOMEM-ARJAWSKDSA-N", "",   4.12,   3.700,  3.778) !  57
    CALL CalcH("trans-2-butene",                      "624-64-6", "IAQRGUVFOMOMEM-ONEGZZNKSA-N", "",   4.11,   3.782,  3.837) !  58
    CALL CalcH("2-methylpropene",                     "115-11-7", "VQTUBCCKSQIDNK-UHFFFAOYSA-N", "",   4.06,   3.845,  3.925) !  59
    CALL CalcH("butanal",                             "123-72-8", "ZTQSAGDEMFDKMZ-UHFFFAOYSA-N", "",   0.82,   0.883,  1.012) !  60
    CALL CalcH("2-methylpropanal",                     "78-84-2", "AMIMRNSIRUDHCM-UHFFFAOYSA-N", "T",  0.98,   1.126,  1.173) !  61
    CALL CalcH("ethyl ethanoate",                     "141-78-6", "XEKOWRVHYACXOJ-UHFFFAOYSA-N", "",   0.88,   1.105,  1.004) !  62
    CALL CalcH("methyl propanoate",                   "554-12-1", "RJUFJBKOKNCXHH-UHFFFAOYSA-N", "",   1.02,   1.119,  1.010) !  63
    CALL CalcH("1-bromobutane",                       "109-65-9", "MPPPKRYCTPRNTB-UHFFFAOYSA-N", "",   2.83,   3.340,  3.299) !  64
    CALL CalcH("1-chlorobutane",                      "109-69-3", "VFWCMGCRMGJXDK-UHFFFAOYSA-N", "",   2.97,   3.394,  3.448) !  65
    CALL CalcH("2-chlorobutane",                       "78-86-4", "BSPCSKHALVHRSR-UHFFFAOYSA-N", "",   3.03,   3.595,  3.632) !  66
    CALL CalcH("butane",                              "106-97-8", "IJDNQMDRQITEOD-UHFFFAOYSA-N", "",   4.71,   4.553,  4.537) !  67
    CALL CalcH("2-methylpropane",                      "75-28-5", "NNPPMTNAJDCUHE-UHFFFAOYSA-N", "T",  4.81,   4.802,  4.748) !  68
    CALL CalcH("1-butanol",                            "71-36-3", "LRHPLDYGYMQRHN-UHFFFAOYSA-N", "",  -0.32,  -0.148, -0.292) !  69
    CALL CalcH("2-butanol",                            "78-92-2", "BTANRVKWQNVYAZ-UHFFFAOYSA-N", "",  -0.33,   0.111, -0.056) !  70
    CALL CalcH("2-methyl-2-propanol",                  "75-65-0", "DKGAVHZHDRPRBM-UHFFFAOYSA-N", "",  -0.02,   0.438,  0.269) !  71
    CALL CalcH("2-methyl-1-propanol",                  "78-83-1", "ZXEKIIBDNHEJCQ-UHFFFAOYSA-N", "",  -0.18,   0.106, -0.056) !  72
    CALL CalcH("3-oxapentane-1,5-diol",               "111-46-6", "MTHSVFCYNBDYFN-UHFFFAOYSA-N", "",  -4.78,  -4.693, -4.606) !  73
    CALL CalcH("1-butanethiol",                       "109-79-5", "WQAQPCDUOCURKW-UHFFFAOYSA-N", "",   2.70,   2.280,  2.232) !  74
    CALL CalcH("2-methyl-1-propanethiol",             "513-44-0", "BDFAOUQQXJIZDG-UHFFFAOYSA-N", "T",  2.58,   2.518,  2.446) !  75
    CALL CalcH("2-butanethiol",                       "513-53-1", "LOCHFZBWPCLPAN-UHFFFAOYSA-N", "",   2.57,   2.506,  2.439) !  76
    CALL CalcH("2-methyl-2-propanethiol",              "75-66-1", "WMXCDAVJEZZYLT-UHFFFAOYSA-N", "",   2.47,   2.769,  2.682) !  77
    CALL CalcH("diethyl sulfide",                     "352-93-2", "LJSQFQKUNVCTIA-UHFFFAOYSA-N", "",   2.09,   2.192,  2.135) !  78
    CALL CalcH("methyl isopropyl sulfide",           "1551-21-9", "ROSSIHMZZJOVOU-UHFFFAOYSA-N", "",   2.20,   2.408,  2.329) !  79
    CALL CalcH("methyl propyl sulfide",              "3877-15-4", "ZOASGOXWEHUTKZ-UHFFFAOYSA-N", "",   2.25,   2.183,  2.122) !  80
    CALL CalcH("1-butanamine",                        "109-73-9", "HQABUPZFAYXKJW-UHFFFAOYSA-N", "",   0.39,   0.398,  0.356) !  81
    CALL CalcH("2-methyl-1-propanamine",               "78-81-9", "KDSNLYIMUZNERS-UHFFFAOYSA-N", "T",  0.09,   0.616,  0.567) !  82
    CALL CalcH("diethylamine",                        "109-89-7", "HPNMFZURTQLUMO-UHFFFAOYSA-N", "",   0.60,   0.389,  0.340) !  83
    CALL CalcH("pyridine",                            "110-86-1", "JUJWROOIHBZHMG-UHFFFAOYSA-N", "",  -0.29,  -0.535, -0.387) !  84
    CALL CalcH("cyanoethanoic acid, ethyl ester",     "105-56-6", "ZIUSEGSNTOUIPT-UHFFFAOYSA-N", "",  -1.80,  -1.408, -1.149) !  85
    CALL CalcH("cyclopentene",                        "142-29-0", "LPIQUOYDBNQMRZ-UHFFFAOYSA-N", "",   3.55,   3.331,  3.287) !  86
    CALL CalcH("2-methyl-1,3-butadiene",               "78-79-5", "RRHGJUQNOFWUDK-UHFFFAOYSA-N", "",   3.71,   3.723,  3.733) !  87
    CALL CalcH("2-methyl-1,3-butadiene",               "78-79-5", "RRHGJUQNOFWUDK-UHFFFAOYSA-N", "",   3.63,   3.703,  3.715) !  88
    CALL CalcH("1,4-pentadiene",                      "591-93-5", "QYZLKGVUSQXAMU-UHFFFAOYSA-N", "T",  3.82,   3.707,  3.693) !  89
    CALL CalcH("1-pentyne",                           "627-19-0", "IBXNCJKFFQIKKY-UHFFFAOYSA-N", "",   3.14,   3.916,  3.861) !  90
    CALL CalcH("2-pentyne",                           "627-21-4", "NKTDTMONXHODTI-UHFFFAOYSA-N", "",   3.145,  3.877,  3.837) !  91
    CALL CalcH("methyl isopropenyl ketone",           "814-78-8", "ZGHFDIIVVIFNPS-UHFFFAOYSA-N", "",   0.75,   1.126,  1.021) !  92
    CALL CalcH("2,4-pentanedione",                    "123-54-6", "YRKCREAYFQTBPV-UHFFFAOYSA-N", "",  -0.60,  -0.895, -1.304) !  93
    CALL CalcH("ethyl propenoate",                    "140-88-5", "JIGUQPWFLRLWPJ-UHFFFAOYSA-N", "",   1.14,   0.869,  0.791) !  94
    CALL CalcH("cyclopentane",                        "287-92-3", "RGSFGYAAUTVSQA-UHFFFAOYSA-N", "",   4.02,   4.145,  4.053) !  95
    CALL CalcH("2-methyl-1-butene",                   "563-46-2", "MHNNAWXXUZQSNM-UHFFFAOYSA-N", "T",  4.21,   3.934,  4.026) !  96
    CALL CalcH("2-methyl-2-butene",                   "513-35-9", "BKOOMYPCSUNDGP-UHFFFAOYSA-N", "",   4.20,   3.925,  4.001) !  97
    CALL CalcH("3-methyl-1-butene",                   "563-45-1", "YHQXBTXEYZIYOV-UHFFFAOYSA-N", "",   4.46,   4.271,  4.323) !  98
    CALL CalcH("1-pentene",                           "109-67-1", "YWAKXRMUMFPDSH-UHFFFAOYSA-N", "",   4.35,   3.967,  4.026) !  99
    CALL CalcH("cis-2-pentene",                       "627-20-3", "QMMOXUPEWRXHJS-HYXAFXHYSA-N", "",   4.10,   3.804,  3.874) ! 100
    CALL CalcH("trans-2-pentene",                     "646-04-8", "QMMOXUPEWRXHJS-HWKANZROSA-N", "",   4.11,   3.856,  3.910) ! 101
    CALL CalcH("3-pentanone",                          "96-22-0", "FDPIMTJIUBPUKL-UHFFFAOYSA-N", "",   0.82,   1.107,  1.103) ! 102
    CALL CalcH("pentanal",                            "110-62-3", "HGBOYTHUEUWSSQ-UHFFFAOYSA-N", "T",  1.26,   1.074,  1.269) ! 103
    CALL CalcH("ethyl propanoate",                    "105-37-3", "FKRCODPIKNYEAC-UHFFFAOYSA-N", "",   1.09,   1.225,  1.198) ! 104
    CALL CalcH("2-propyl ethanoate",                  "108-21-4", "JMMWKPVZQRWMSS-UHFFFAOYSA-N", "",   1.18,   1.395,  1.213) ! 105
    CALL CalcH("propyl ethanoate",                    "109-60-4", "YKYONYBAUNKHLG-UHFFFAOYSA-N", "",   1.21,   1.297,  1.321) ! 106
    CALL CalcH("methyl butanoate",                    "623-42-7", "UUIQMZJEGPQKFD-UHFFFAOYSA-N", "",   1.20,   1.260,  1.234) ! 107
    CALL CalcH("3-methylbutanoic acid",               "503-74-2", "GWYFCOCPABKNJV-UHFFFAOYSA-N", "",  -1.11,  -1.988, -1.944) ! 108
    CALL CalcH("2-methylbutane",                       "78-78-4", "QWTDNUCVQCZILF-UHFFFAOYSA-N", "",   4.88,   4.874,  4.812) ! 109
    CALL CalcH("dimethylpropane",                     "463-82-1", "CRSOQBOWXPBRES-UHFFFAOYSA-N", "T",  5.07,   5.357,  5.153) ! 110
    CALL CalcH("pentane",                             "109-66-0", "OFBQJSOFQDEBGM-UHFFFAOYSA-N", "",   4.85,   4.584,  4.548) ! 111
    CALL CalcH("2-methyl-2-butanol",                   "75-85-4", "MSXVEPNJUHWQHW-UHFFFAOYSA-N", "",   0.99,   0.447,  0.290) ! 112
    CALL CalcH("3-methyl-1-butanol",                  "123-51-3", "PHTQWCKDNZKARW-UHFFFAOYSA-N", "",  -0.13,   0.060, -0.110) ! 113
    CALL CalcH("1-pentanol",                           "71-41-0", "AMQJEAYHLZJPGS-UHFFFAOYSA-N", "",  -0.15,   0.004,  0.009) ! 114
    CALL CalcH("2-pentanol",                         "6032-29-7", "JYVLIDXNZAXMDK-UHFFFAOYSA-N", "",  -0.07,   0.114, -0.068) ! 115
    CALL CalcH("3-pentanol",                          "584-02-1", "AQIXEPGDORPWBJ-UHFFFAOYSA-N", "",   0.02,   0.130, -0.047) ! 116
    CALL CalcH("1-pentanethiol",                      "110-66-7", "ZRKMQKLGEQPLNS-UHFFFAOYSA-N", "T",  2.89,   2.482,  2.521) ! 117
    CALL CalcH("methyl butyl sulfide",                "628-29-5", "WCXXISMIJBRDQK-UHFFFAOYSA-N", "",   2.50,   2.386,  2.396) ! 118
    CALL CalcH("ethyl propyl sulfide",               "4110-50-3", "ZDDDFDQTSXYYSE-UHFFFAOYSA-N", "",   2.46,   2.399,  2.409) ! 119
    CALL CalcH("2-methyl-2-butanethiol",             "1679-09-0", "IQIBYAHJXQVQGB-UHFFFAOYSA-N", "",   2.74,   2.876,  2.787) ! 120
    CALL CalcH("1,2,4-trichlorobenzene",              "120-82-1", "PBKONEOXTCPAFI-UHFFFAOYSA-N", "",   2.22,   2.158,  1.870) ! 121
    CALL CalcH("1,3-dibromobenzene",                  "108-36-1", "JSRLURSZEMLAFO-UHFFFAOYSA-N", "",   1.67,   2.068,  1.890) ! 122
    CALL CalcH("2-chloronitrobenzene",                 "88-73-3", "BFCFYVKQTRLZHA-UHFFFAOYSA-N", "",  -0.25,  -0.221, -0.172) ! 123
    CALL CalcH("1,3-dichlorobenzene",                 "541-73-1", "ZPQOPVIELGIULI-UHFFFAOYSA-N", "T",  2.27,   2.250,  2.036) ! 124
    CALL CalcH("1,2-dichlorobenzene",                  "95-50-1", "RFFLAFLAYFXFSW-UHFFFAOYSA-N", "",   2.20,   2.006,  1.833) ! 125
    CALL CalcH("1,2-difluorobenzene",                 "367-11-3", "GOYDNIKZWGIXJT-UHFFFAOYSA-N", "",   2.59,   2.161,  2.031) ! 126
    CALL CalcH("1,4-difluorobenzene",                 "540-36-3", "QUGUFLJIAFISSW-UHFFFAOYSA-N", "",   2.63,   3.343,  2.759) ! 127
    CALL CalcH("bromobenzene",                        "108-86-1", "QARVLSVVCXYDNA-UHFFFAOYSA-N", "",   2.08,   2.338,  2.116) ! 128
    CALL CalcH("chlorobenzene",                       "108-90-7", "MVPPADPHJFYWMZ-UHFFFAOYSA-N", "",   2.40,   2.409,  2.178) ! 129
    CALL CalcH("4-hydroxychlorobenzene",              "106-48-9", "WXNZTHHGJRFXKQ-UHFFFAOYSA-N", "",  -1.15,  -1.744, -1.795) ! 130
    CALL CalcH("fluorobenzene",                       "462-06-6", "PYLWMHQQBFSUBP-UHFFFAOYSA-N", "T",  2.54,   2.529,  2.374) ! 131
    CALL CalcH("iodobenzene",                         "591-50-4", "SNHMUERNLJLMHN-UHFFFAOYSA-N", "",   1.90,   2.049,  1.820) ! 132
    CALL CalcH("nitrobenzene",                         "98-95-3", "LQNUZADURLCDLV-UHFFFAOYSA-N", "",   0.06,   0.039,  0.027) ! 133
    CALL CalcH("benzene",                              "71-43-2", "UHOVQNZJYSORNB-UHFFFAOYSA-N", "",   2.49,   3.169,  2.868) ! 134
    CALL CalcH("1-amino-3-chlorobenzene",             "108-42-9", "PNPCRKVUWYDDST-UHFFFAOYSA-N", "",  -0.95,  -1.039, -0.922) ! 135
    CALL CalcH("aminobenzene",                         "62-53-3", "PAYRUJLWNCNPSJ-UHFFFAOYSA-N", "",  -1.02,  -0.659, -0.642) ! 136
    CALL CalcH("2-methylpyridine",                    "109-06-8", "BSKHPKMHTQYZBB-UHFFFAOYSA-N", "",  -0.26,  -0.367, -0.269) ! 137
    CALL CalcH("3-methylpyridine",                    "108-99-6", "ITQTTZVARXURQS-UHFFFAOYSA-N", "T", -0.40,  -0.505, -0.383) ! 138
    CALL CalcH("4-methylpyridine",                    "108-89-4", "FKNQCJSGGFJEIZ-UHFFFAOYSA-N", "",  -0.50,  -0.504, -0.356) ! 139
    CALL CalcH("cyclohexene",                         "110-83-8", "HGCIXCUEYOPUTN-UHFFFAOYSA-N", "",   3.40,   3.508,  3.347) ! 140
    CALL CalcH("2,3-dimethyl-1,3-butadiene",          "513-81-5", "SDJHPPZKZZWAKF-UHFFFAOYSA-N", "",   3.83,   4.137,  4.162) ! 141
    CALL CalcH("1,5-hexadiene",                       "592-42-7", "PYGSKMBEVAICCR-UHFFFAOYSA-N", "",   3.81,   4.064,  4.190) ! 142
    CALL CalcH("1-hexyne",                            "693-02-7", "CGHIBGNXEGJPQZ-UHFFFAOYSA-N", "",   3.35,   4.164,  4.299) ! 143
    CALL CalcH("2-hexyne",                            "764-35-2", "MELUCTCJOARQQG-UHFFFAOYSA-N", "",   3.34,   4.125,  4.253) ! 144
    CALL CalcH("3-hexyne",                            "928-49-4", "DQQNMIPXXNPGCV-UHFFFAOYSA-N", "T",  3.32,   4.104,  4.236) ! 145
    CALL CalcH("cyclohexane",                         "110-82-7", "XDTMQSROBMDMFD-UHFFFAOYSA-N", "",   4.03,   4.334,  4.206) ! 146
    CALL CalcH("2,3-dimethyl-1-butene",               "563-78-0", "OWWIWYDDISJUMY-UHFFFAOYSA-N", "",   4.29,   4.379,  4.467) ! 147
    CALL CalcH("2,3-dimethyl-2-butene",               "563-79-1", "WGLLSSPDPJPLOR-UHFFFAOYSA-N", "",   4.33,   4.236,  4.306) ! 148
    CALL CalcH("3,3-dimethyl-1-butene",               "558-37-2", "PKXHXOTZMFCXSH-UHFFFAOYSA-N", "",   4.28,   4.660,  4.721) ! 149
    CALL CalcH("3-methylenepentane",                  "760-21-4", "RYKZRKKEYSRDNF-UHFFFAOYSA-N", "",   4.30,   4.009,  4.099) ! 150
    CALL CalcH("trans-3-methyl-2-pentene",            "616-12-6", "BEQGRRJLJLVQAQ-GQCTYLIASA-N", "",   4.32,   4.080,  4.150) ! 151
    CALL CalcH("1-hexene",                            "592-41-6", "LIKMAJRDDDTEIG-UHFFFAOYSA-N", "T",  4.21,   4.116,  4.333) ! 152
    CALL CalcH("cis-2-hexene",                       "7688-21-3", "RYPKRALMXUUNKS-HYXAFXHYSA-N", "",   4.32,   4.057,  4.264) ! 153
    CALL CalcH("trans-2-hexene",                     "4050-45-7", "RYPKRALMXUUNKS-HWKANZROSA-N", "",   4.31,   4.051,  4.237) ! 154
    CALL CalcH("cis-3-hexene",                       "7642-09-3", "ZQDPJFUHLCOCRG-WAYWQWQTSA-N", "",   4.31,   4.011,  4.214) ! 155
    CALL CalcH("trans-3-hexene",                    "13269-52-8", "ZQDPJFUHLCOCRG-AATRIKPKSA-N", "",   4.31,   4.061,  4.242) ! 156
    CALL CalcH("methylcyclopentane",                   "96-37-7", "GDOPTJXRTPNYNR-UHFFFAOYSA-N", "",   4.30,   4.263,  4.328) ! 157
    CALL CalcH("2-methyl-1-pentene",                  "763-29-1", "WWUVJRULCWHUSA-UHFFFAOYSA-N", "",   4.19,   3.986,  4.081) ! 158
    CALL CalcH("2-methyl-2-pentene",                  "625-27-4", "JMMZCWZIJXAGKW-UHFFFAOYSA-N", "T",  4.31,   3.984,  4.059) ! 159
    CALL CalcH("3-methyl-1-pentene",                  "760-20-3", "LDTAOIUHUHHCMU-UHFFFAOYSA-N", "",   4.29,   4.365,  4.415) ! 160
    CALL CalcH("4-methyl-1-pentene",                  "691-37-2", "WSSSPWUEQFSQQG-UHFFFAOYSA-N", "",   4.54,   4.234,  4.303) ! 161
    CALL CalcH("cis-4-methyl-2-pentene",              "691-38-3", "LGAQJENWWYGFSN-PLNGDYQASA-N", "",   4.29,   4.138,  4.216) ! 162
    CALL CalcH("trans-4-methyl-2-pentene",            "674-76-0", "LGAQJENWWYGFSN-SNAWJCMRSA-N", "",   4.29,   4.181,  4.251) ! 163
    CALL CalcH("cyclohexanol",                        "108-93-0", "HPXRVTGHNJAIIH-UHFFFAOYSA-N", "",  -0.85,  -0.381, -0.370) ! 164
    CALL CalcH("hexanal",                              "66-25-1", "JARKCYVAAOWBJS-UHFFFAOYSA-N", "",   1.16,   1.156,  1.327) ! 165
    CALL CalcH("butyl ethanoate",                     "123-86-4", "DKPFZGUDAPQIHT-UHFFFAOYSA-N", "T",  1.16,   1.333,  1.354) ! 166
    CALL CalcH("2-butyl ethanoate",                   "105-46-4", "DCKVNWZUADLDEH-UHFFFAOYSA-N", "",   1.48,   1.563,  1.514) ! 167
    CALL CalcH("1,1-dimethylethyl ethanoate",         "540-88-5", "WMOVHXAZOJBABW-UHFFFAOYSA-N", "",   1.28,   1.715,  1.457) ! 168
    CALL CalcH("ethyl butanoate",                     "105-54-4", "OBNCKNCVKJNDBV-UHFFFAOYSA-N", "",   1.37,   1.298,  1.283) ! 169
    CALL CalcH("ethyl 2-methylpropanoate",             "97-62-1", "WDAXFOBOLVPGLV-UHFFFAOYSA-N", "",   1.36,   1.550,  1.492) ! 170
    CALL CalcH("2-methylpropyl ethanoate",            "110-19-0", "GJRQTCIYDGXPES-UHFFFAOYSA-N", "",   1.38,   1.563,  1.482) ! 171
    CALL CalcH("propyl propanoate",                   "106-36-5", "MCSINKKTEDDPNK-UHFFFAOYSA-N", "",   1.42,   1.318,  1.317) ! 172
    CALL CalcH("hexanoic acid",                       "142-62-1", "FUZZWVXGSFPDMH-UHFFFAOYSA-N", "T", -1.42,  -2.138, -1.853) ! 173
    CALL CalcH("2,2-dimethylbutane",                   "75-83-2", "HNRMPXKDFBEGFZ-UHFFFAOYSA-N", "",   4.93,   5.285,  5.203) ! 174
    CALL CalcH("2,3-dimethylbutane",                   "79-29-8", "ZFFMLCVRJBZUDZ-UHFFFAOYSA-N", "",   4.86,   5.163,  5.116) ! 175
    CALL CalcH("hexane",                              "110-54-3", "VLKZOEOYAKHREP-UHFFFAOYSA-N", "",   4.86,   4.782,  4.957) ! 176
    CALL CalcH("2-methylpentane",                     "107-83-5", "AFABGHUZZDYHJO-UHFFFAOYSA-N", "",   4.98,   4.865,  4.833) ! 177
    CALL CalcH("3-methylpentane",                      "96-14-0", "PFEOZHBOMNWTJB-UHFFFAOYSA-N", "",   4.82,   4.976,  4.910) ! 178
    CALL CalcH("2-ethyl-1-butanol",                    "97-95-0", "TZYRSLHNPKPEFV-UHFFFAOYSA-N", "",   0.05,   0.355,  0.191) ! 179
    CALL CalcH("1-hexanol",                           "111-27-3", "ZSIAUFGUXNUGDI-UHFFFAOYSA-N", "T",  0.07,   0.091,  0.125) ! 180
    CALL CalcH("2-hexanol",                           "626-93-7", "QNVRIHYSUZMSGM-UHFFFAOYSA-N", "",   0.12,   0.234,  0.287) ! 181
    CALL CalcH("1-hexanethiol",                       "111-31-9", "PMBXCGGQNSVESQ-UHFFFAOYSA-N", "",   3.09,   2.569,  2.577) ! 182
    CALL CalcH("ethyl butyl sulfide",                 "638-46-0", "XJIRSLHMKBUGMR-UHFFFAOYSA-N", "",   2.67,   2.490,  2.466) ! 183
    CALL CalcH("di-(2-propyl)-sulfide",               "625-80-9", "XYWDPYKBIRQXQS-UHFFFAOYSA-N", "",   2.56,   2.768,  2.680) ! 184
    CALL CalcH("dipropyl sulfide",                    "111-47-7", "ZERULLAPCVRMCO-UHFFFAOYSA-N", "",   2.66,   2.490,  2.466) ! 185
    CALL CalcH("dipropylamine",                       "142-84-7", "WEHWNAOGRSTTBQ-UHFFFAOYSA-N", "",   0.70,   0.637,  0.757) ! 186
    CALL CalcH("1-hexanamine",                        "111-26-2", "BMVXCPBXGZKUPN-UHFFFAOYSA-N", "T",  0.74,   0.636,  0.772) ! 187
    CALL CalcH("triethylamine",                       "121-44-8", "ZMANZCXQSJIPKH-UHFFFAOYSA-N", "",   0.81,   0.887,  0.845) ! 188
    CALL CalcH("1-chloro-4-methylbenzene",            "106-43-4", "NPDACUSDTOMAMK-UHFFFAOYSA-N", "",   2.38,   2.495,  2.034) ! 189
    CALL CalcH("methylbenzene",                       "108-88-3", "YXFVVABEGXRONW-UHFFFAOYSA-N", "",   2.55,   3.136,  2.883) ! 190
    CALL CalcH("methoxybenzene",                      "100-66-3", "RDOXTESZEPMUJZ-UHFFFAOYSA-N", "",   2.31,   2.167,  1.803) ! 191
    CALL CalcH("(hydroxymethyl)-benzene",             "100-51-6", "WVDDGKGOMKODPV-UHFFFAOYSA-N", "",  -1.84,  -1.245, -1.308) ! 192
    CALL CalcH("1-hydroxy-3-methylbenzene",           "108-39-4", "RLSSMJSEOOYNOY-UHFFFAOYSA-N", "",  -1.40,  -1.155, -1.267) ! 193
    CALL CalcH("1-hydroxy-2-methylbenzene",            "95-48-7", "QWVGKYWNOKOFNN-UHFFFAOYSA-N", "T", -1.02,  -1.142, -1.257) ! 194
    CALL CalcH("1-hydroxy-4-methylbenzene",           "106-44-5", "IWDCLRJOBJJRNH-UHFFFAOYSA-N", "",  -1.39,  -1.297, -1.426) ! 195
    CALL CalcH("benzylamine",                         "100-46-9", "WGQKYBSKWIADBV-UHFFFAOYSA-N", "",   0.21,  -0.121, -0.113) ! 196
    CALL CalcH("(methylamino)-benzene",               "100-61-8", "AFBPFSWMIHJQDM-UHFFFAOYSA-N", "",  -0.20,  -0.547, -0.558) ! 197
    CALL CalcH("3-methylbenzenamine",                 "108-44-1", "JJYPMNFTHPTTDI-UHFFFAOYSA-N", "",  -0.81,  -0.426, -0.429) ! 198
    CALL CalcH("2-methylbenzenamine",                  "95-53-4", "RNVCVTLRINQCPJ-UHFFFAOYSA-N", "",  -0.88,  -0.484, -0.476) ! 199
    CALL CalcH("1-heptyne",                           "628-71-7", "YVXHZKKCZYLQOP-UHFFFAOYSA-N", "",   3.59,   4.249,  4.366) ! 200
    CALL CalcH("2-propenoic acid, butyl ester",       "141-32-2", "CQEYYJKEWSMYFG-UHFFFAOYSA-N", "T",  1.50,   1.084,  1.019) ! 201
    CALL CalcH("isobutyl acrylate",                   "106-63-8", "CFVWNXQPGQOHRJ-UHFFFAOYSA-N", "",   1.58,   1.255,  1.141) ! 202
    CALL CalcH("1,1-dimethylcyclopentane",           "1638-26-2", "QWHNJUXXYKPLQM-UHFFFAOYSA-N", "",   4.38,   4.402,  4.512) ! 203
    CALL CalcH("ethylcyclopentane",                  "1640-89-7", "IFTRQJLVEBNKJK-UHFFFAOYSA-N", "",   4.48,   4.268,  4.361) ! 204
    CALL CalcH("2-ethyl-1-pentene",                  "3404-71-5", "TWCRBBJSQAZZQB-UHFFFAOYSA-N", "",   4.53,   4.151,  4.366) ! 205
    CALL CalcH("3-ethyl-1-pentene",                  "4038-04-4", "YPVPQMCSLFDIKA-UHFFFAOYSA-N", "",   4.45,   4.562,  4.623) ! 206
    CALL CalcH("1-heptene",                           "592-76-7", "ZGEGCLOFRBLKSE-UHFFFAOYSA-N", "",   4.35,   4.203,  4.393) ! 207
    CALL CalcH("cis-2-heptene",                      "6443-92-1", "OTTZHAVKAVGASB-HYXAFXHYSA-N", "T",  4.56,   4.090,  4.257) ! 208
    CALL CalcH("trans-2-heptene",                   "14686-13-6", "OTTZHAVKAVGASB-HWKANZROSA-N", "",   4.55,   4.139,  4.292) ! 209
    CALL CalcH("cis-3-heptene",                      "7642-10-6", "WZHKDGJSXCTSCK-ALCCZGGFSA-N", "",   4.54,   4.079,  4.250) ! 210
    CALL CalcH("trans-3-heptene",                   "14686-14-7", "WZHKDGJSXCTSCK-FNORWQNLSA-N", "",   4.53,   4.141,  4.292) ! 211
    CALL CalcH("methylcyclohexane",                   "108-87-2", "UAEPNZWRGJTJPN-UHFFFAOYSA-N", "",   4.37,   4.413,  4.299) ! 212
    CALL CalcH("2-methyl-1-hexene",                  "6094-02-6", "IRUDSQHLKGNCGF-UHFFFAOYSA-N", "",   4.50,   4.173,  4.366) ! 213
    CALL CalcH("3-methyl-1-hexene",                  "3404-61-3", "RITONZMLZWYPHW-UHFFFAOYSA-N", "",   4.45,   4.518,  4.724) ! 214
    CALL CalcH("4-methyl-1-hexene",                  "3769-23-1", "SUWJESCICIOQHO-UHFFFAOYSA-N", "T",  4.47,   4.425,  4.639) ! 215
    CALL CalcH("2,3,3-trimethyl-1-butene",            "594-56-9", "AUYRUAVCWOAHQN-UHFFFAOYSA-N", "",   4.45,   4.769,  4.846) ! 216
    CALL CalcH("2,4-dimethyl-3-pentanone",            "565-80-0", "HXVNBWAKAOHACI-UHFFFAOYSA-N", "",   1.29,   1.769,  1.590) ! 217
    CALL CalcH("heptanal",                            "111-71-7", "FXHGMKSSBGDXIY-UHFFFAOYSA-N", "",   1.29,   1.262,  1.392) ! 218
    CALL CalcH("butyl propanoate",                    "590-01-2", "BTMVHUNTONAYDX-UHFFFAOYSA-N", "",   1.59,   1.388,  1.367) ! 219
    CALL CalcH("ethyl 3-methylbutanoate",             "108-64-5", "PPXUHEORWJQRHJ-UHFFFAOYSA-N", "",   1.55,   1.622,  1.559) ! 220
    CALL CalcH("3-methylbutyl ethanoate",             "123-92-2", "MLFHJEHSLIIPHL-UHFFFAOYSA-N", "",   1.42,   1.559,  1.523) ! 221
    CALL CalcH("pentyl ethanoate",                    "628-63-7", "PGMYKACGEOXYJE-UHFFFAOYSA-N", "T",  1.28,   1.417,  1.419) ! 222
    CALL CalcH("propyl butanoate",                    "105-66-8", "HUAZGNHGCJGYNP-UHFFFAOYSA-N", "",   1.54,   1.428,  1.389) ! 223
    CALL CalcH("heptanoic acid",                      "111-14-8", "MNWFXJYAOYHMED-UHFFFAOYSA-N", "",  -1.67,  -2.056, -1.724) ! 224
    CALL CalcH("1-bromoheptane",                      "629-04-9", "LSXKDWGTSHCFPP-UHFFFAOYSA-N", "",   3.40,   3.833,  3.924) ! 225
    CALL CalcH("2,2-dimethylpentane",                 "590-35-2", "CXOWYJMDMMMMJO-UHFFFAOYSA-N", "",   5.24,   5.210,  5.174) ! 226
    CALL CalcH("2,4-dimethylpentane",                 "108-08-7", "BZHMBWZPUJHVEE-UHFFFAOYSA-N", "",   5.21,   5.220,  5.140) ! 227
    CALL CalcH("3,3-dimethylpentane",                 "562-49-2", "AEXMKKGTQYQZCS-UHFFFAOYSA-N", "",   5.01,   5.371,  5.288) ! 228
    CALL CalcH("3-ethylpentane",                      "617-78-7", "AORMDLNPRGXHHL-UHFFFAOYSA-N", "T",  5.16,   5.075,  5.060) ! 229
    CALL CalcH("heptane",                             "142-82-5", "IMNFDUFMRHMDMM-UHFFFAOYSA-N", "",   5.17,   4.864,  5.027) ! 230
    CALL CalcH("2-methylhexane",                      "591-76-4", "GXDHCNNESPLIKD-UHFFFAOYSA-N", "",   5.28,   5.034,  5.211) ! 231
    CALL CalcH("3-methylhexane",                      "589-34-4", "VLJXXKKOSFGPHI-UHFFFAOYSA-N", "",   5.23,   5.053,  5.208) ! 232
    CALL CalcH("2,2,3-trimethylbutane",               "464-06-2", "ZISSAWUMDACLOM-UHFFFAOYSA-N", "",   5.12,   5.576,  5.484) ! 233
    CALL CalcH("1-heptanol",                          "111-70-6", "BBMCTIGTTCKYKF-UHFFFAOYSA-N", "",   0.02,   0.203,  0.269) ! 234
    CALL CalcH("2-heptanol",                          "543-49-7", "CETWDUZRCINIHU-UHFFFAOYSA-N", "",   0.50,   0.266,  0.326) ! 235
    CALL CalcH("1-heptanethiol",                     "1639-09-4", "VPIAKHNXCOTPAY-UHFFFAOYSA-N", "T",  3.22,   2.680,  2.648) ! 236
    CALL CalcH("methyl benzoate",                      "93-58-3", "QPJVMBTYPHYUOC-UHFFFAOYSA-N", "",   0.29,  -0.077, -0.092) ! 237
    CALL CalcH("ethylbenzene",                        "100-41-4", "YNQLUTRBYVCPMQ-UHFFFAOYSA-N", "",   2.66,   3.324,  2.977) ! 238
    CALL CalcH("1,3-dimethylbenzene",                 "108-38-3", "IVSZLXZYQVIEFR-UHFFFAOYSA-N", "",   2.58,   3.311,  3.091) ! 239
    CALL CalcH("1,4-dimethylbenzene",                 "106-42-3", "URLKBWYHVLBVBO-UHFFFAOYSA-N", "",   2.53,   3.375,  2.827) ! 240
    CALL CalcH("2-phenylethanol",                      "60-12-8", "WRMNZCZEMHIOCP-UHFFFAOYSA-N", "",  -0.80,  -1.178, -1.263) ! 241
    CALL CalcH("1,1-dimethylcyclohexane",             "590-66-9", "QEGNUYASOUJEHD-UHFFFAOYSA-N", "",   4.49,   4.508,  4.431) ! 242
    CALL CalcH("ethylcyclohexane",                   "1678-91-7", "IIEWJVIFRVWJOD-UHFFFAOYSA-N", "T",  4.56,   4.412,  4.196) ! 243
    CALL CalcH("1-methyl-1-ethylcyclopentane",      "16747-50-5", "LETYIFNDQBJGPJ-UHFFFAOYSA-N", "",   4.68,   4.443,  4.548) ! 244
    CALL CalcH("1-octene",                            "111-66-0", "KWKAKUADMBZCLK-UHFFFAOYSA-N", "",   4.54,   4.313,  4.462) ! 245
    CALL CalcH("propylcyclopentane",                 "2040-96-2", "KDIAMAVWIJYWHN-UHFFFAOYSA-N", "",   4.70,   4.318,  4.348) ! 246
    CALL CalcH("2,4,4-trimethyl-1-pentene",           "107-39-1", "FXNDIJDIPNCZQJ-UHFFFAOYSA-N", "",   4.66,   4.625,  4.709) ! 247
    CALL CalcH("2,4,4-trimethyl-2-pentene",           "107-40-4", "LAAVYEUJEMRIGF-UHFFFAOYSA-N", "",   4.65,   4.657,  4.735) ! 248
    CALL CalcH("octanal",                             "124-13-0", "NUJGJRNETVAIRJ-UHFFFAOYSA-N", "",   1.47,   1.324,  1.434) ! 249
    CALL CalcH("hexyl ethanoate",                     "142-92-7", "AOGQPLXWSUTHQB-UHFFFAOYSA-N", "T",  1.44,   1.454,  1.441) ! 250
    CALL CalcH("octanoic acid",                       "124-07-2", "WWZKQHOCKIZLMA-UHFFFAOYSA-N", "",  -1.27,  -2.021, -1.696) ! 251
    CALL CalcH("2,2-dimethylhexane",                  "590-73-8", "FLTJDUOFAQWHDF-UHFFFAOYSA-N", "",   5.31,   5.351,  5.485) ! 252
    CALL CalcH("2,5-dimethylhexane",                  "592-13-2", "UWNADWZGEHDQAB-UHFFFAOYSA-N", "",   5.31,   5.344,  5.367) ! 253
    CALL CalcH("3,3-dimethylhexane",                  "563-16-6", "KUMXLFIBWFCMOJ-UHFFFAOYSA-N", "",   5.36,   5.388,  5.556) ! 254
    CALL CalcH("3-ethyl-2-methylpentane",             "609-26-7", "DUPUVYJQZSLSJB-UHFFFAOYSA-N", "",   5.38,   5.371,  5.335) ! 255
    CALL CalcH("3-ethyl-3-methylpentane",            "1067-08-9", "GIEZWIDCIFCQPS-UHFFFAOYSA-N", "",   5.42,   5.422,  5.400) ! 256
    CALL CalcH("2-methylheptane",                     "592-27-8", "JVSWJIKNEAIKJW-UHFFFAOYSA-N", "T",  5.36,   5.063,  5.243) ! 257
    CALL CalcH("3-methylheptane",                     "589-81-1", "LAIUFBWHERIJIH-UHFFFAOYSA-N", "",   5.31,   5.126,  5.254) ! 258
    CALL CalcH("4-methylheptane",                     "589-53-7", "CHBAWFGIXDBEBT-UHFFFAOYSA-N", "",   5.36,   5.114,  5.306) ! 259
    CALL CalcH("2,2,3-trimethylpentane",              "564-02-3", "XTDQDBVBDLYELW-UHFFFAOYSA-N", "",   5.36,   5.604,  5.516) ! 260
    CALL CalcH("2,2,4-trimethylpentane",              "540-84-1", "NHTMVDHEPJAVLT-UHFFFAOYSA-N", "",   5.28,   5.529,  5.428) ! 261
    CALL CalcH("2,3,3-trimethylpentane",              "560-21-4", "OKVWYBALHQFVFP-UHFFFAOYSA-N", "",   5.41,   5.604,  5.538) ! 262
    CALL CalcH("2,3,4-trimethylpentane",              "565-75-3", "RLPGDEORIPLBNF-UHFFFAOYSA-N", "",   4.99,   5.469,  5.421) ! 263
    CALL CalcH("2-ethyl-1-hexanol",                   "104-76-7", "YIWUKEYIRIRTPP-UHFFFAOYSA-N", "T",  1.11,   0.606,  0.696) ! 264
    CALL CalcH("1-octanol",                           "111-87-5", "KBPLFHHGFOOTCA-UHFFFAOYSA-N", "",   0.14,   0.257,  0.321) ! 265
    CALL CalcH("1-octanethiol",                       "111-88-6", "KZCOBXFFBQJQHH-UHFFFAOYSA-N", "",   3.26,   2.734,  2.696) ! 266
    CALL CalcH("tert-octanethiol",                    "141-59-3", "QZLAEIZEPJAELS-UHFFFAOYSA-N", "",   3.33,   3.109,  3.116) ! 267
    CALL CalcH("dibutylamine",                        "111-92-2", "JQVDAXLFBXTEQA-UHFFFAOYSA-N", "",   0.89,   0.808,  0.946) ! 268
    CALL CalcH("1-ethenyl-4-methylbenzene",           "622-97-9", "JLBJTVDPSNHSKJ-UHFFFAOYSA-N", "",   2.25,   3.098,  2.563) ! 269
    CALL CalcH("(2-propyl)-benzene",                   "98-82-8", "RWGFKTVRMDUZSP-UHFFFAOYSA-N", "",   2.90,   3.494,  3.181) ! 270
    CALL CalcH("isopropylcyclohexane",                "696-29-7", "GWESVXSMPKAFAS-UHFFFAOYSA-N", "T",  4.76,   4.497,  4.306) ! 271
    CALL CalcH("1-nonene",                            "124-11-8", "JRZJOMJEPLMPRA-UHFFFAOYSA-N", "",   4.65,   4.369,  4.513) ! 272
    CALL CalcH("propylcyclohexane",                  "1678-92-8", "DEDZSLCZHWTGOR-UHFFFAOYSA-N", "",   4.76,   4.414,  4.048) ! 273
    CALL CalcH("nonanal",                             "124-19-6", "GYHFUZHODSMOHU-UHFFFAOYSA-N", "",   1.56,   1.414,  1.493) ! 274
    CALL CalcH("butyl pentanoate",                    "591-68-4", "OKJADYKTJJGKDX-UHFFFAOYSA-N", "",   1.55,   1.553,  1.447) ! 275
    CALL CalcH("nonanoic acid",                       "112-05-0", "FBUKVWPVBMHYJY-UHFFFAOYSA-N", "",  -1.28,  -1.942, -1.621) ! 276
    CALL CalcH("3,3-diethylpentane",                 "1067-20-5", "BGXXXYLRPIRDHJ-UHFFFAOYSA-N", "",   5.69,   5.562,  5.496) ! 277
    CALL CalcH("3-ethyl-2,2-dimethylpentane",       "16747-32-3", "CLZCPQKGOAXOJT-UHFFFAOYSA-N", "T",  5.56,   5.670,  5.539) ! 278
    CALL CalcH("3-ethyl-2,3-dimethylpentane",       "16747-33-4", "MMASVVOQIKCFJZ-UHFFFAOYSA-N", "",   5.67,   5.604,  5.529) ! 279
    CALL CalcH("3-ethyl-2,4-dimethylpentane",        "1068-87-7", "VLHAGZNBWKUMRW-UHFFFAOYSA-N", "",   5.59,   5.609,  5.490) ! 280
    CALL CalcH("2,2-dimethylheptane",                "1071-26-7", "PSABUFWDVWCFDP-UHFFFAOYSA-N", "",   5.51,   5.290,  5.443) ! 281
    CALL CalcH("2,6-dimethylheptane",                "1072-05-5", "KBPCCVWUMVGXGF-UHFFFAOYSA-N", "",   5.52,   5.220,  5.386) ! 282
    CALL CalcH("3-ethylheptane",                    "15869-80-4", "PSVQKOKKLWHNRP-UHFFFAOYSA-N", "",   5.58,   5.242,  5.351) ! 283
    CALL CalcH("4-ethylheptane",                     "2216-32-2", "XMROPFQWHHUFFS-UHFFFAOYSA-N", "",   5.57,   5.283,  5.293) ! 284
    CALL CalcH("2,3-dimethylheptane",                "3074-71-3", "WBRFDUJXCLCKPX-UHFFFAOYSA-N", "T",  5.58,   5.331,  5.440) ! 285
    CALL CalcH("2,4-dimethylheptane",                "2213-23-2", "AUKVIBNBLXQNIZ-UHFFFAOYSA-N", "",   5.50,   5.324,  5.479) ! 286
    CALL CalcH("2,5-dimethylheptane",                "2216-30-0", "HQZHQNKZOYIKQC-UHFFFAOYSA-N", "",   5.53,   5.338,  5.426) ! 287
    CALL CalcH("3,4-dimethylheptane",                 "922-28-1", "MAKRYGRRIKSDES-UHFFFAOYSA-N", "",   5.58,   5.364,  5.530) ! 288
    CALL CalcH("3,5-dimethylheptane",                 "926-82-9", "DZJTZGHZAWTWGA-UHFFFAOYSA-N", "",   5.53,   5.396,  5.524) ! 289
    CALL CalcH("4,4-dimethylheptane",                "1068-19-5", "WSOKFYJGNBQDPW-UHFFFAOYSA-N", "",   5.56,   5.384,  5.562) ! 290
    CALL CalcH("3-ethyl-2-methylhexane",            "16789-46-1", "MVLOWDRGPHBNNF-UHFFFAOYSA-N", "",   5.56,   5.502,  5.645) ! 291
    CALL CalcH("4-ethyl-2-methylhexane",             "3074-75-7", "KYCZJIBOPKRSOV-UHFFFAOYSA-N", "T",  5.52,   5.484,  5.575) ! 292
    CALL CalcH("3-ethyl-3-methylhexane",             "3074-76-8", "CYWROHZCELEGSE-UHFFFAOYSA-N", "",   5.61,   5.474,  5.662) ! 293
    CALL CalcH("3-ethyl-4-methylhexane",             "3074-77-9", "OKCRKWVABWILDR-UHFFFAOYSA-N", "",   5.59,   5.495,  5.666) ! 294
    CALL CalcH("2,2,3-trimethylhexane",             "16747-25-4", "CBVFSZDQEHBJEQ-UHFFFAOYSA-N", "",   5.56,   5.653,  5.781) ! 295
    CALL CalcH("2,2,4-trimethylhexane",             "16747-26-5", "AFTPEBDOGXRMNQ-UHFFFAOYSA-N", "",   5.50,   5.617,  5.729) ! 296
    CALL CalcH("2,3,3-trimethylhexane",             "16747-28-7", "DJYSEQMMCZAKGT-UHFFFAOYSA-N", "",   5.60,   5.641,  5.735) ! 297
    CALL CalcH("2,3,4-trimethylhexane",               "921-47-1", "RUTNOQHQISEBGT-UHFFFAOYSA-N", "",   5.60,   5.573,  5.684) ! 298
    CALL CalcH("2,3,5-trimethylhexane",              "1069-53-0", "ODGLTLJZCVNPBU-UHFFFAOYSA-N", "T",  5.52,   5.591,  5.704) ! 299
    CALL CalcH("2,4,4-trimethylhexane",             "16747-30-1", "SVEMKBCPZYWEPH-UHFFFAOYSA-N", "",   5.56,   5.645,  5.715) ! 300
    CALL CalcH("3,3,4-trimethylhexane",             "16747-31-2", "ARWOOWBJJKVYOV-UHFFFAOYSA-N", "",   5.63,   5.683,  5.687) ! 301
    CALL CalcH("2-methyloctane",                     "3221-61-2", "ZUBZATZOEPUUQF-UHFFFAOYSA-N", "",   5.55,   5.099,  5.218) ! 302
    CALL CalcH("3-methyloctane",                     "2216-33-3", "SEEOMASXHIJCDV-UHFFFAOYSA-N", "",   5.57,   5.138,  5.292) ! 303
    CALL CalcH("4-methyloctane",                     "2216-34-4", "DOGIHOCMZJUJNR-UHFFFAOYSA-N", "",   5.75,   5.182,  5.281) ! 304
    CALL CalcH("nonane",                              "111-84-2", "BKIMMITUMNQMOS-UHFFFAOYSA-N", "",   5.53,   4.979,  5.077) ! 305
    CALL CalcH("2,2,3,3-tetramethylpentane",         "7154-79-2", "QUKOJKFJIHSBKV-UHFFFAOYSA-N", "T",  5.65,   5.984,  5.739) ! 306
    CALL CalcH("2,2,3,4-tetramethylpentane",         "1186-53-4", "VZFMYOCAEQDWDY-UHFFFAOYSA-N", "",   5.59,   5.498,  5.287) ! 307
    CALL CalcH("2,2,4,4-tetramethylpentane",         "1070-87-7", "GUMULFRCHLJNDY-UHFFFAOYSA-N", "",   5.52,   5.859,  5.594) ! 308
    CALL CalcH("2,3,3,4-tetramethylpentane",        "16747-38-9", "JLCYYQOQSAMWTA-UHFFFAOYSA-N", "",   5.66,   5.883,  5.667) ! 309
    CALL CalcH("2,2,5-trimethylhexane",              "3522-94-9", "HHOSMYBYIHNXNO-UHFFFAOYSA-N", "",   5.48,   5.631,  5.573) ! 310
    CALL CalcH("1-nonanol",                           "143-08-8", "ZWRUINPWMLAQRD-UHFFFAOYSA-N", "",   0.27,   0.299,  0.354) ! 311
    CALL CalcH("2-nonanol",                           "628-99-9", "NGDNVOAEIVQRFH-UHFFFAOYSA-N", "",   0.44,   0.333,  0.416) ! 312
    CALL CalcH("1-nonanethiol",                      "1455-21-6", "ZVEZMVFBMOOHAT-UHFFFAOYSA-N", "T",  3.17,   2.827,  2.788) ! 313
    CALL CalcH("N,N-dipropyl-1-propanamine",          "102-69-2", "YFTHZRPMJXBUME-UHFFFAOYSA-N", "",   1.33,   1.342,  1.425) ! 314
    CALL CalcH("butylbenzene",                        "104-51-8", "OCKPCBLVNKHBMX-UHFFFAOYSA-N", "",   2.86,   3.377,  2.929) ! 315
    CALL CalcH("4-ethyl-m-xylene",                    "874-41-9", "MEMBJMDZWKVOTB-UHFFFAOYSA-N", "",   2.68,   3.576,  3.267) ! 316
    CALL CalcH("4-ethyl-o-xylene",                    "934-80-5", "SBUYFICWQNHBCM-UHFFFAOYSA-N", "",   2.64,   3.534,  3.081) ! 317
    CALL CalcH("(2-methylpropyl)-benzene",            "538-93-2", "KXUHSQYYJYAXGZ-UHFFFAOYSA-N", "",   2.64,   3.550,  3.104) ! 318
    CALL CalcH("2-ethylhexyl ethanoate",              "103-09-3", "WOYWLLHHWAMFCB-UHFFFAOYSA-N", "",   1.64,   1.993,  1.746) ! 319
    CALL CalcH("2-methylnonane",                      "871-83-0", "SGVYKUFIHHTIFL-UHFFFAOYSA-N", "T",  5.67,   5.135,  5.251) ! 320
    CALL CalcH("3-methylnonane",                     "5911-04-6", "PLZDDPSCZHRBOY-UHFFFAOYSA-N", "",   5.71,   5.168,  5.285) ! 321
    CALL CalcH("4-methylnonane",                    "17301-94-9", "IALRSQMWHFKJJA-UHFFFAOYSA-N", "",   5.72,   5.197,  5.306) ! 322
    CALL CalcH("5-methylnonane",                    "15869-85-9", "TYSIILFJZXHVPU-UHFFFAOYSA-N", "",   5.69,   5.248,  5.114) ! 323
    CALL CalcH("4-ethyloctane",                     "15869-86-0", "NRJUFUBKIFIKFI-UHFFFAOYSA-N", "",   5.71,   5.436,  5.287) ! 324
    CALL CalcH("2,3-dimethyloctane",                 "7146-60-3", "YPMNDMUOGQJCLW-UHFFFAOYSA-N", "",   5.72,   5.373,  5.521) ! 325
    CALL CalcH("2,4-dimethyloctane",                 "4032-94-4", "IXAVTTRPEXFVSX-UHFFFAOYSA-N", "",   5.66,   5.414,  5.504) ! 326
    CALL CalcH("2,5-dimethyloctane",                "15869-89-3", "HOAAQUNESXYFDT-UHFFFAOYSA-N", "T",  5.68,   5.430,  5.426) ! 327
    CALL CalcH("2,6-dimethyloctane",                 "2051-30-1", "ZALHPSXXQIPKTQ-UHFFFAOYSA-N", "",   5.70,   5.329,  5.484) ! 328
    CALL CalcH("2,7-dimethyloctane",                 "1072-16-8", "KEVMYFLMMDUPJE-UHFFFAOYSA-N", "",   5.67,   5.305,  5.448) ! 329
    CALL CalcH("3,3-dimethyloctane",                 "4110-44-5", "DBULLUBYDONGLT-UHFFFAOYSA-N", "",   5.73,   5.428,  5.577) ! 330
    CALL CalcH("3,4-dimethyloctane",                "15869-92-8", "QQCWGAMGBCGAQJ-UHFFFAOYSA-N", "",   5.73,   5.448,  5.546) ! 331
    CALL CalcH("3,5-dimethyloctane",                "15869-93-9", "VRHRGVJOUHJULC-UHFFFAOYSA-N", "",   5.70,   5.450,  5.547) ! 332
    CALL CalcH("3,6-dimethyloctane",                "15869-94-0", "JEEQUUSFXYRPRK-UHFFFAOYSA-N", "",   5.70,   5.431,  5.523) ! 333
    CALL CalcH("4,4-dimethyloctane",                "15869-95-1", "ZMEDGZAGMLTROM-UHFFFAOYSA-N", "T",  5.72,   5.499,  5.585) ! 334
    CALL CalcH("4,5-dimethyloctane",                "15869-96-2", "DOYJTLUPPPUSMD-UHFFFAOYSA-N", "",   5.73,   5.489,  5.578) ! 335
    CALL CalcH("4-propylheptane",                    "3178-29-8", "ABYGSZMCWVXFCQ-UHFFFAOYSA-N", "",   5.66,   5.479,  5.323) ! 336
    CALL CalcH("4-(1-methylethyl)-heptane",         "52896-87-4", "AZLAWGCUDHUQDB-UHFFFAOYSA-N", "",   5.72,   5.596,  5.429) ! 337
    CALL CalcH("3-ethyl-2-methylheptane",           "14676-29-0", "NKMJCVVUYDKHAV-UHFFFAOYSA-N", "",   5.73,   5.514,  5.606) ! 338
    CALL CalcH("4-ethyl-2-methylheptane",           "52896-88-5", "OJDKRASKNKPYDH-UHFFFAOYSA-N", "",   5.69,   5.592,  5.553) ! 339
    CALL CalcH("5-ethyl-2-methylheptane",           "13475-78-0", "DGEMPTLPTFNEHJ-UHFFFAOYSA-N", "",   5.71,   5.462,  5.557) ! 340
    CALL CalcH("3-ethyl-3-methylheptane",           "17302-01-1", "HSOMNBKXPGCNBH-UHFFFAOYSA-N", "T",  5.78,   5.530,  5.635) ! 341
    CALL CalcH("4-ethyl-3-methylheptane",           "52896-89-6", "BTGGSWBKRYMHQK-UHFFFAOYSA-N", "",   5.75,   5.597,  5.669) ! 342
    CALL CalcH("3-ethyl-5-methylheptane",           "52896-90-9", "VXARVYMIZCGZGG-UHFFFAOYSA-N", "",   5.69,   5.531,  5.526) ! 343
    CALL CalcH("3-ethyl-4-methylheptane",           "52896-91-0", "JZBKRUIGSVOOIC-UHFFFAOYSA-N", "",   5.75,   5.566,  5.704) ! 344
    CALL CalcH("4-ethyl-4-methylheptane",           "17302-04-4", "MPYQJQDSICRCJJ-UHFFFAOYSA-N", "",   5.76,   5.572,  5.720) ! 345
    CALL CalcH("2,2,3-trimethylheptane",            "52896-92-1", "ACYHSTUWOQNWCX-UHFFFAOYSA-N", "",   5.73,   5.660,  5.766) ! 346
    CALL CalcH("2,2,4-trimethylheptane",            "14720-74-2", "IIYGOARYARWJBO-UHFFFAOYSA-N", "",   5.66,   5.632,  5.737) ! 347
    CALL CalcH("2,2,5-trimethylheptane",            "20291-95-6", "GZJFAWOTMWATOS-UHFFFAOYSA-N", "T",  5.68,   5.658,  5.696) ! 348
    CALL CalcH("2,2,6-trimethylheptane",             "1190-83-6", "FHJCGIUZJXWNET-UHFFFAOYSA-N", "",   5.64,   5.515,  5.623) ! 349
    CALL CalcH("2,3,3-trimethylheptane",            "52896-93-2", "QACXEXNKLFWKLK-UHFFFAOYSA-N", "",   5.76,   5.685,  5.752) ! 350
    CALL CalcH("2,3,4-trimethylheptane",            "52896-95-4", "UVVYAKOLFKEZEE-UHFFFAOYSA-N", "",   5.74,   5.661,  5.781) ! 351
    CALL CalcH("2,3,5-trimethylheptane",            "20278-85-7", "YKPNYFKOKKKGNM-UHFFFAOYSA-N", "",   5.76,   5.662,  5.715) ! 352
    CALL CalcH("2,3,6-trimethylheptane",             "4032-93-3", "IHPXJGBVRWFEJB-UHFFFAOYSA-N", "",   5.69,   5.610,  5.667) ! 353
    CALL CalcH("2,4,4-trimethylheptane",             "4032-92-2", "QALGVLROELGEEM-UHFFFAOYSA-N", "",   5.70,   5.681,  5.803) ! 354
    CALL CalcH("2,4,5-trimethylheptane",            "20278-84-6", "YMBNRMDSLJNNPF-UHFFFAOYSA-N", "T",  5.71,   5.689,  5.782) ! 355
    CALL CalcH("2,4,6-trimethylheptane",             "2613-61-8", "YNLBBDHDNIXQNL-UHFFFAOYSA-N", "",   5.62,   5.643,  5.720) ! 356
    CALL CalcH("2,5,5-trimethylheptane",             "1189-99-7", "SOYLPZSOEXZMLE-UHFFFAOYSA-N", "",   5.69,   5.694,  5.645) ! 357
    CALL CalcH("3,3,4-trimethylheptane",            "20278-87-9", "WRBHKVWLEIYLDZ-UHFFFAOYSA-N", "",   5.78,   5.738,  5.854) ! 358
    CALL CalcH("3,3,5-trimethylheptane",             "7154-80-5", "VRVRZZWPKABUOE-UHFFFAOYSA-N", "",   5.74,   5.721,  5.817) ! 359
    CALL CalcH("3,4,4-trimethylheptane",            "20278-88-0", "BLNBSBLKPFFJKQ-UHFFFAOYSA-N", "",   5.78,   5.724,  5.851) ! 360
    CALL CalcH("3,4,5-trimethylheptane",            "20278-89-1", "LJIIBBYARMPSMT-UHFFFAOYSA-N", "",   5.77,   5.640,  5.788) ! 361
    CALL CalcH("2-methyl-3-(1-methylethyl)-hexane", "62016-13-1", "YBOXGRMAQIYMGV-UHFFFAOYSA-N", "T",  5.86,   5.751,  5.807) ! 362
    CALL CalcH("3,3-diethylhexane",                 "17302-02-2", "WWNGLKDLYKNGGT-UHFFFAOYSA-N", "",   5.81,   5.719,  5.851) ! 363
    CALL CalcH("3,4-diethylhexane",                 "19398-77-7", "VBZCRMTUDYIWIH-UHFFFAOYSA-N", "",   5.77,   5.735,  5.837) ! 364
    CALL CalcH("3-ethyl-2,2-dimethylhexane",        "20291-91-2", "XYDYODCWVCBIOQ-UHFFFAOYSA-N", "",   5.75,   5.801,  5.888) ! 365
    CALL CalcH("4-ethyl-2,2-dimethylhexane",        "52896-99-8", "QHLDBFLIDFTHQI-UHFFFAOYSA-N", "",   5.66,   5.821,  5.742) ! 366
    CALL CalcH("3-ethyl-2,3-dimethylhexane",        "52897-00-4", "PJIFKODHGMUPFH-UHFFFAOYSA-N", "",   5.81,   5.814,  5.920) ! 367
    CALL CalcH("4-ethyl-2,3-dimethylhexane",        "52897-01-5", "RHMRCCBCYFAZIK-UHFFFAOYSA-N", "",   5.76,   5.781,  5.877) ! 368
    CALL CalcH("3-ethyl-2,4-dimethylhexane",         "7220-26-0", "OSKIMJMPFNLVOU-UHFFFAOYSA-N", "T",  5.76,   5.791,  5.888) ! 369
    CALL CalcH("4-ethyl-2,4-dimethylhexane",        "52897-03-7", "SIKFMUYSQCEQOO-UHFFFAOYSA-N", "",   5.78,   5.784,  5.881) ! 370
    CALL CalcH("3-ethyl-2,5-dimethylhexane",        "52897-04-8", "UJEUVDLASLOZIV-UHFFFAOYSA-N", "",   5.71,   5.763,  5.699) ! 371
    CALL CalcH("4-ethyl-3,3-dimethylhexane",        "52897-05-9", "ZRTXVJYJVBTXHE-UHFFFAOYSA-N", "",   5.81,   5.804,  5.873) ! 372
    CALL CalcH("3-ethyl-3,4-dimethylhexane",        "52897-06-0", "ZGJCTUKRTSBTIQ-UHFFFAOYSA-N", "",   5.80,   5.746,  5.849) ! 373
    CALL CalcH("2,2,3,3-tetramethylhexane",         "13475-81-5", "RMQHJMMCLSJULX-UHFFFAOYSA-N", "",   5.81,   6.059,  6.046) ! 374
    CALL CalcH("2,2,3,4-tetramethylhexane",         "52897-08-2", "MHPSPNGWFAGBNH-UHFFFAOYSA-N", "",   5.80,   5.956,  5.970) ! 375
    CALL CalcH("2,2,3,5-tetramethylhexane",         "52897-09-3", "GCFKTDRTZYDRBI-UHFFFAOYSA-N", "T",  5.68,   6.002,  5.919) ! 376
    CALL CalcH("2,2,4,4-tetramethylhexane",         "51750-65-3", "PXHNHTBJHHSVPT-UHFFFAOYSA-N", "",   5.82,   5.998,  5.923) ! 377
    CALL CalcH("2,2,4,5-tetramethylhexane",         "16747-42-5", "KDRZICOOQNIJDN-UHFFFAOYSA-N", "",   5.70,   5.945,  5.876) ! 378
    CALL CalcH("2,2,5,5-tetramethylhexane",          "1071-81-4", "HXQDUXXBVMMIKL-UHFFFAOYSA-N", "",   5.59,   6.006,  5.680) ! 379
    CALL CalcH("2,3,3,4-tetramethylhexane",         "52897-10-6", "HIHSOGFAVTVMCY-UHFFFAOYSA-N", "",   5.84,   5.988,  5.959) ! 380
    CALL CalcH("2,3,3,5-tetramethylhexane",         "52897-11-7", "GCGFXFIPOBRMQT-UHFFFAOYSA-N", "",   5.74,   5.992,  5.963) ! 381
    CALL CalcH("2,3,4,4-tetramethylhexane",         "52897-12-8", "XDRDDPSGUQMOBO-UHFFFAOYSA-N", "",   5.82,   5.964,  5.938) ! 382
    CALL CalcH("2,3,4,5-tetramethylhexane",         "52897-15-1", "BHGNYYIOYPFWKC-UHFFFAOYSA-N", "T",  5.73,   5.912,  5.920) ! 383
    CALL CalcH("3,3,4,4-tetramethylhexane",          "5171-84-6", "MCEYLFHKATVXLN-UHFFFAOYSA-N", "",   5.91,   6.081,  5.948) ! 384
    CALL CalcH("2,4-dimethyl-3-isopropylpentane",   "13475-79-1", "YVYHOOYMDHZALB-UHFFFAOYSA-N", "",   5.78,   5.955,  5.657) ! 385
    CALL CalcH("3,3-diethyl-2-methylpentane",       "52897-16-2", "DSSAZLXYIQIXGW-UHFFFAOYSA-N", "",   5.86,   5.851,  5.673) ! 386
    CALL CalcH("3-ethyl-2,2,3-trimethylpentane",    "52897-17-3", "AJDIFHIHSYVDGP-UHFFFAOYSA-N", "",   5.92,   6.035,  5.773) ! 387
    CALL CalcH("3-ethyl-2,2,4-trimethylpentane",    "52897-18-4", "VLIZIVHXZXQRDE-UHFFFAOYSA-N", "",   5.78,   6.013,  5.702) ! 388
    CALL CalcH("3-ethyl-2,3,4-trimethylpentane",    "52897-19-5", "OHZNMGSGEFVFTI-UHFFFAOYSA-N", "",   5.89,   5.997,  5.737) ! 389
    CALL CalcH("2,2,3,3,4-pentamethylpentane",      "16747-44-7", "WKQBIIUOSATALN-UHFFFAOYSA-N", "T",  5.91,   6.302,  5.793) ! 390
    CALL CalcH("2,2,3,4,4-pentamethylpentane",      "16747-45-8", "OWFKEHICSVOVAC-UHFFFAOYSA-N", "",   5.88,   6.268,  5.731) ! 391
    CALL CalcH("1-decanol",                           "112-30-1", "MWKFXSUHUHTGQN-UHFFFAOYSA-N", "",   0.42,   0.341,  0.397) ! 392
    CALL CalcH("1-decanethiol",                       "143-10-2", "VTXVGVNLYGSIAR-UHFFFAOYSA-N", "",   2.92,   2.867,  2.826) ! 393
    CALL CalcH("pentylbenzene",                       "538-68-1", "PWATWSYOIIXYMA-UHFFFAOYSA-N", "",   3.09,   3.505,  3.095) ! 394
    CALL CalcH("2-ethylhexyl acrylate",               "103-11-7", "GOXQRTZXKQZDDN-UHFFFAOYSA-N", "",   1.38,   1.717,  1.428) ! 395
    CALL CalcH("1-undecene",                          "821-95-4", "DCTOHCCUXLBQMS-UHFFFAOYSA-N", "",   5.07,   4.503,  4.652) ! 396
    CALL CalcH("undecane",                           "1120-21-4", "RSJKGSCJYJTIGS-UHFFFAOYSA-N", "T",  5.03,   5.110,  5.207) ! 397
    CALL CalcH("1-undecanol",                         "112-42-5", "KJIOQYGWTQBHNH-UHFFFAOYSA-N", "",   0.26,   0.427,  0.497) ! 398
    CALL CalcH("1-undecanethiol",                    "5332-52-5", "CCIDWXHLGNEQSL-UHFFFAOYSA-N", "",   2.20,   2.954,  2.912) ! 399
    CALL CalcH("diethyl phthalate",                    "84-66-2", "FLKPEMZONWLCSK-UHFFFAOYSA-N", "",  -2.10,  -1.664, -1.581) ! 400
    CALL CalcH("hexylbenzene",                       "1077-16-3", "LTEQMZWBSYACLV-UHFFFAOYSA-N", "",   3.15,   3.410,  3.169) ! 401
    CALL CalcH("1-dodecene",                          "112-41-4", "CRSBERNSMYQZNG-UHFFFAOYSA-N", "",   5.03,   4.589,  4.741) ! 402
    CALL CalcH("dodecanoic acid",                     "143-07-7", "POULHZVOKOAJMA-UHFFFAOYSA-N", "",  -2.18,  -1.803, -1.488) ! 403
    CALL CalcH("1-dodecanol",                         "112-53-8", "LQZZUXJYWNFBMV-UHFFFAOYSA-N", "T",  0.46,   0.465,  0.537) ! 404
    CALL CalcH("1-dodecanethiol",                     "112-55-0", "WNAHIZMDSQCWRP-UHFFFAOYSA-N", "",   2.59,   3.043,  3.012) ! 405
    CALL CalcH("heptylbenzene",                      "1078-71-3", "LBNXAWYDQUGHGX-UHFFFAOYSA-N", "",   2.57,   3.413,  3.192) ! 406
    CALL CalcH("1-tridecene",                        "2437-56-1", "VQOXUMQBYILCKR-UHFFFAOYSA-N", "",   4.96,   4.626,  4.775) ! 407
    CALL CalcH("1-tridecanol",                        "112-70-9", "XFRVVPUIAFSTFO-UHFFFAOYSA-N", "",   0.47,   0.545,  0.634) ! 408
    CALL CalcH("tetradecane",                         "629-59-4", "BGHCVCJVXZWKCC-UHFFFAOYSA-N", "",   4.88,   5.314,  5.391) ! 409
    CALL CalcH("1-tetradecanol",                      "112-72-1", "HLZKNKRTKFSKGZ-UHFFFAOYSA-N", "",   0.76,   0.798,  0.917) ! 410
    CALL CalcH("1-pentadecene",                     "13360-61-7", "PJLHTVIBELQURV-UHFFFAOYSA-N", "T",  4.18,   4.741,  3.796) ! 411

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, info, exptl, MLR, RBFN)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_, inchikey_, info
      REAL,             INTENT(IN) :: exptl, MLR, RBFN
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      ! exptl: Experimental values are from ref2440 and not used here.
      ! RBFN:
      type = "Q"
      CALL MakeNote(TRIM(ref)//"_RBFN", TRIM(citet())//" compared two QSPR methods " // &
        "and found that radial basis function networks (RBFNs) are better than " // &
        "multiple linear regression. In their paper, they provide neither a " // &
        "definition nor the unit of their Henry's law constants. Comparing the " // &
        "values with those that they cite from \citet{2440}, it is assumed that " // &
        "they use the variant $\Hvpx$ and the unit atm.")
      IF (casrn_=="1679-09-0") &
        CALL MakeNote(TRIM(ref)//"_"//casrn_, &
        "Presumably, the species called ``42-methyl-2-butanethiol'' in Table~1 of " // &
        TRIM(citet())//" should be 2-methyl-2-butanethiol.") ! 120
      IF (TRIM(info)=="T") &
        CALL MakeNote(TRIM(ref)//"_test", "Value from the test set.")
      CALL Output(Hvpx_TIMES_HscpSI/(10.**RBFN))
    END SUBROUTINE CalcH

  END SUBROUTINE ref3731

  !---------------------------------------------------------------------------

  SUBROUTINE ref3732 ! Hvpc [Pa*m3/mol]
    IMPLICIT NONE
    LOGICAL, PARAMETER :: l_checkdata = .FALSE. ! .TRUE. ! set to true to check raw data at different temperatures

    ref = "3732"
    type = "M"
    ndata = 5
    ALLOCATE(temp(ndata))
    temp = (/ 5., 15., 20., 25., 35. /) + CtoK
    ! Tab. 1 (the raw data at 5 different temperatures were obtained from the authors, pers. comm.):
    CALL CalcH("PCN-19",         "51570-43-5", "DZHZYPCCEISGSQ-UHFFFAOYSA-N", 20.97,  -5488., 0.17,  45.6, 14.04, (/ 3.459, 4.886, 6.627, 14.039, 22.667 /))
    !CALL CalcH("PCN-24/14",                                                  21.51,  -5703., 0.18,  47.4, 12.00, (/ 2.731, 3.803, 5.618, 12.001, 18.944 /)) ! isomer mix
    CALL CalcH("PCN-15",         "55720-33-7", "MMHZKSMAFILONG-UHFFFAOYSA-N", 20.02,  -5320., 0.17,  44.2,  9.11, (/ 2.558, 3.521, 4.125,  9.113, 15.931 /))
    CALL CalcH("PCN-16",         "51570-44-6", "AAUJSCRITBXDJH-UHFFFAOYSA-N", 23.61,  -6496., 0.20,  54.0,  5.96, (/ 1.272, 2.215, 2.626,  5.957, 13.172 /))
    !CALL CalcH("PCN-17/25",                                                  20.32,  -5530., 0.17,  46.0,  6.79, (/ 1.523, 2.273, 2.857,  6.791,  9.963 /)) ! isomer mix
    CALL CalcH("PCN-23",          "2437-55-0", "VQSNXLCFFHGXTI-UHFFFAOYSA-N", 19.46,  -5364., 0.16,  44.6,  4.46, (/ 1.074, 1.825, 2.424,  4.635,  7.302 /))
    CALL CalcH("PCN-42",         "53555-64-9", "OTTCXKPQKOLSJN-UHFFFAOYSA-N", 21.65,  -5633., 0.18,  46.8, 18.01, (/ 3.561, 6.135, 8.613, 18.007, 25.635 /))
    !CALL CalcH("PCN-33/34/37",                                               19.39,  -5056., 0.16,  42.0, 12.79, (/ 3.045, 4.679, 6.969, 12.789, 17.666 /)) ! isomer mix
    CALL CalcH("PCN-47",         "55720-43-9", "VJZRCIYSYVGDMU-UHFFFAOYSA-N", 21.76,  -5810., 0.18,  48.3, 10.91, (/ 2.153, 3.376, 5.753, 10.910, 16.163 /))
    !CALL CalcH("PCN-36/45",                                                  20.97,  -5606., 0.17,  46.6, 10.02, (/ 2.039, 3.122, 5.216, 10.016, 14.030 /)) ! isomer mix
    !CALL CalcH("PCN-28/43",                                                  21.90,  -5931., 0.18,  49.3,  8.56, (/ 1.641, 2.441, 4.316,  8.559, 12.433 /)) ! isomer mix
    CALL CalcH("PCN-32",          "6733-54-6", "BIVDISPRSYAHQQ-UHFFFAOYSA-N", 21.48,  -5873., 0.18,  48.8,  6.61, (/ 1.303, 1.801, 3.932,  6.605,  9.637 /))
    CALL CalcH("PCN-35",          "6529-87-9", "WCMSFBRREKZZFL-UHFFFAOYSA-N", 21.86,  -6041., 0.18,  50.2,  5.52, (/ 1.057, 1.414, 3.253,  5.519,  8.161 /))
    !CALL CalcH("PCN-38/40",                                                  23.15,  -6499., 0.19,  54.0,  4.47, (/ 0.778, 1.075, 2.128,  4.468,  6.914 /)) ! isomer mix
    CALL CalcH("PCN-46",          "3432-57-3", "LITCKAVLJAKHOE-UHFFFAOYSA-N", 20.14,  -5729., 0.17,  47.6,  2.82, (/ 0.630, 0.851, 1.322,  2.819,  4.331 /))
    !CALL CalcH("PCN-52/60",                                                  29.29,  -8059., 0.24,  67.0, 14.50, (/ 1.165, 1.804, 5.486, 14.505, 15.983 /)) ! isomer mix
    CALL CalcH("PCN-58",        "150224-19-4", "WYLDWCYZCFRVRH-UHFFFAOYSA-N", 31.27,  -8728., 0.26,  72.6, 11.22, (/ 0.805, 1.210, 3.642, 11.223, 13.494 /))
    CALL CalcH("PCN-61",        "150224-22-9", "HGSDQSUMXKHGTH-UHFFFAOYSA-N", 31.20,  -8755., 0.26,  72.8,  9.86, (/ 0.706, 0.973, 2.990,  9.863, 11.360 /))
    CALL CalcH("PCN-50",         "67922-26-3", "BAOLNVSMVTYGDA-UHFFFAOYSA-N", 32.30,  -9016., 0.27,  75.0, 12.90, (/ 0.800, 1.189, 3.733, 12.903, 14.140 /))
    CALL CalcH("PCN-57",        "150224-20-7", "KPOZENRUJCHWOA-UHFFFAOYSA-N", 33.43,  -9545., 0.28,  79.4,  7.20, (/ 0.416, 0.522, 1.670,  7.202,  7.954 /))
    CALL CalcH("PCN-62",        "150224-21-8", "LBCOXKFWBDTJTF-UHFFFAOYSA-N", 32.03,  -9180., 0.27,  76.3,  5.79, (/ 0.381, 0.466, 1.489,  5.794,  6.542 /))
    CALL CalcH("PCN-53",        "150224-24-1", "HVYRFNJXZVEGFK-UHFFFAOYSA-N", 33.64,  -9659., 0.28,  80.3,  5.94, (/ 0.358, 0.424, 1.279,  5.942,  6.978 /))
    CALL CalcH("PCN-59",        "150224-25-2", "FEIKEVSWLMYFFF-UHFFFAOYSA-N", 31.31,  -9079., 0.26,  75.5,  3.80, (/ 0.297, 0.311, 0.920,  3.802,  4.724 /))
    !CALL CalcH("PCN-66/67",                                                  43.87, -12800., 0.36, 106.4,  3.95, (/ 0.136, 0.228, 0.437,  3.953,  8.572 /)) ! isomer mix
    !CALL CalcH("PCN-64/68",                                                  42.12, -12394., 0.35, 103.0,  2.67, (/ 0.124, 0.128, 0.328,  2.667,  5.834 /)) ! isomer mix
    CALL CalcH("PCN-69",        "103426-94-4", "JPQLLIUTUFJWMH-UHFFFAOYSA-N", 40.33, -11906., 0.34,  99.0,  2.35, (/ 0.118, 0.115, 0.317,  2.350,  4.606 /))
    !CALL CalcH("PCN-71/72",                                                  38.89, -11500., 0.32,  95.6,  2.24, (/ 0.118, 0.119, 0.291,  2.243,  3.983 /)) ! isomer mix
    CALL CalcH("PCN-63",         "58877-88-6", "CTLMCQOGOWNFHA-UHFFFAOYSA-N", 39.80, -11897., 0.33,  98.9,  1.29, (/ 0.058, 0.083, 0.224,  1.286,  2.722 /))
    CALL CalcH("PCN-65",        "103426-93-3", "PGCDNPCENWGYMA-UHFFFAOYSA-N", 33.52, -10094., 0.28,  83.9,  1.07, (/ 0.083, 0.074, 0.210,  1.072,  1.789 /))
    CALL CalcH("PCN-73",         "58863-14-2", "NDZIBNJHNBUHKW-UHFFFAOYSA-N", 33.31, -10293., 0.28,  85.6,  0.28, (/ 0.027, 0.044, 0.091,  0.282,  0.952 /))
    CALL CalcH("PCN-74",         "58863-15-3", "QYEGXUUXWMKHHS-UHFFFAOYSA-N", 34.13, -10474., 0.28,  87.1,  0.42, (/ 0.036, 0.042, 0.109,  0.425,  1.100 /))
    CALL CalcH("PCN-75",          "2234-13-1", "RTNLUFLDZOAXIC-UHFFFAOYSA-N", 25.64,  -7845., 0.21,  65.2,  0.82, (/ 0.079, 0.117, 0.171,  0.823,  0.929 /))
    DEALLOCATE(temp)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, A, B, Delta_S_H, Delta_H_H, H, HT)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_, inchikey_
      REAL,               INTENT(IN) :: A, B, Delta_S_H, Delta_H_H, H
      REAL, DIMENSION(:), INTENT(IN) :: HT
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      IF (l_checkdata) THEN
        CALL HTdep(temp, 1./HT, Hominus, mindHR)
        WRITE(*,'(2X,6F10.2,3F10.0)') EXP(A+B/T0), H, 1./Hominus, exp(-1000*Delta_H_H/(Rgas*T0)+1000*Delta_S_H/Rgas), &
          1000*Delta_S_H/Rgas, A, 1000*Delta_H_H/Rgas, -B, mindHR
        ! exp(A+B/T0)         H   Hominus    H_eqn3 Delta_S/R         A Delta_H/R        -B    mindHR
        !       12.98     14.04     11.54      7.78     20.45     20.97     5484.     5488.     5709.
        !        8.82      9.11      7.80     13.68     20.45     20.02     5316.     5320.     5476.
        !        6.19      5.96      5.42      9.69     24.05     23.61     6495.     6496.     6815.
        !        4.35      4.46      3.92      3.50     19.24     19.46     5364.     5364.     5716.
        !       15.75     18.01     14.03     15.96     21.65     21.65     5629.     5633.     5994.
        !        9.71     10.91      8.67      8.71     21.65     21.76     5809.     5810.     6183.
        !        5.94      6.61      5.25      7.12     21.65     21.48     5869.     5873.     6255.
        !        4.95      5.52      4.35      4.05     21.65     21.86     6038.     6041.     6417.
        !        2.52      2.82      2.22      3.47     20.45     20.14     5725.     5729.     5958.
        !        7.36     11.22      6.16      7.27     31.27     31.27     8732.     8728.     9162.
        !        6.27      9.86      5.19      6.71     31.27     31.20     8756.     8755.     9129.
        !        7.85     12.90      6.50      9.20     32.47     32.30     9020.     9016.     9437.
        !        4.12      7.20      3.35      5.19     33.68     33.43     9550.     9545.     9825.
        !        3.46      5.79      2.83      5.44     32.47     32.03     9177.     9180.     9459.
        !        3.47      5.94      2.78      3.61     33.68     33.64     9658.     9659.     9875.
        !        2.36      3.80      1.91      2.26     31.27     31.31     9081.     9079.     9226.
        !        1.49      2.35      1.12      2.60     40.89     40.33    11907.    11906.    11914.
        !        0.90      1.29      0.69      0.81     39.69     39.80    11895.    11897.    12180.
        !        0.72      1.07      0.56      0.84     33.68     33.52    10091.    10094.    10111.
        !        0.30      0.28      0.24      0.43     33.68     33.31    10295.    10293.    10676.
        !        0.37      0.42      0.29      0.23     33.68     34.13    10476.    10474.    10694.
        !        0.51      0.82      0.43      0.35     25.26     25.64     7842.     7845.     7985.
        ! Results of the checks:
        ! - The values at 15 C are quite high for all congeners.
        ! - Regression of the data at 5 different temperatures yields
        !   slightly different parameters than those given in Tab. 1.
        CALL consistency_check(exp(A+B/T0), H, "H and values calculated via A+B/T")
      ENDIF
      CALL MakeNote(TRIM(ref)//"_inconsistent", TRIM(citet())//" provide the parameters " // &
        "$A$ and $B$ for an equation to calculate temperature-dependent " // &
        "Henry's law constants. Values calculated with this equation at 298\,K " // &
        "are slightly different than those measured at 298\,K and given as $H$ in their " // &
        "Table~1. Here, the values $H$ and $B$ are used.")
      CALL Output(1./H, -B)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3732

  !---------------------------------------------------------------------------

  SUBROUTINE ref3734 ! Hscc [1] = Ostwald coefficient
    IMPLICIT NONE

    ref = "3734"
    type = "M"
    ! Tab. 1:
    CALL CalcH("methane",  "74-82-8", "VNWKTOKETHGBQD-UHFFFAOYSA-N", &
      (/   15.,  18.,  21.,  24.,  27.,   15.,  18.,  21.,   24.,  27. /), &
      (/ 40.94, 39.1, 37.0, 35.5, 32.7, 40.60, 38.6, 36.7, 34.85, 33.1 /) *1E-3)
    CALL CalcH("ethane",   "74-84-0", "OTMSDBZUPAUEDD-UHFFFAOYSA-N", &
      (/   15.,  18.,  21.,  24.,  27.,  15. /), &
      (/ 60.05, 55.0, 50.0, 47.0, 42.5, 59.7 /) *1E-3)
    CALL CalcH("propane",  "74-98-6", "ATUOYWHBWRKTHZ-UHFFFAOYSA-N", &
      (/  15.,  21.,  27.,  15.,  21.,  27. /), &
      (/ 47.5, 40.5, 33.4, 48.3, 42.1, 32.6 /) *1E-3)
    CALL CalcH("butane",  "106-97-8", "IJDNQMDRQITEOD-UHFFFAOYSA-N", &
      (/  15.,  21.,  27.,  15.,  21.,  27. /), &
      (/ 41.1, 34.1, 27.0, 41.6, 33.9, 28.5 /) *1E-3)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, temp, ostwald)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_, inchikey_
      REAL, DIMENSION(:), INTENT(IN) :: temp, ostwald
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      CALL HTdep(temp+CtoK, Hscc_TO_HscpSI(ostwald, temp+CtoK), Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3734

  !---------------------------------------------------------------------------

  ! ref3735 NOTUSED: No Henry's law constants.

  !---------------------------------------------------------------------------

  ! ref3738 NOTUSED: Data only at T <= 230 K.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3747 ! Hscc [1] = Ostwald coefficient
    IMPLICIT NONE
    REAL :: temp

    ref = "3747"
    type = "M"
    chem = "acetylene"; casrn = "74-86-2"; inchikey = "HSFWRNGVRCDJHI-UHFFFAOYSA-N"
    temp = 15.+CtoK
    CALL MakeNoteOtherTempK(temp)
    CALL MakeNote(TRIM(ref), 'The same article was also published in ' // &
      '\textit{Monatshefte f\"ur Chemie} 23, 489--501 (1902).')
    CALL Output(Hscc_TO_HscpSI(1.215, temp))

  END SUBROUTINE ref3747

  !---------------------------------------------------------------------------

  SUBROUTINE ref3748 ! Hscc [1] = Ostwald coefficient
    IMPLICIT NONE

    ref = "3748"
    type = "M"
    ! Tab. VI:
    CALL CalcH("acetylene", "74-86-2", "HSFWRNGVRCDJHI-UHFFFAOYSA-N", &
      (/ 30., 25., 20., 15., 10. /), &
      (/ 0.952, 1.030, 1.126, 1.239, 1.382 /))

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, temp, ostwald)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_, inchikey_
      REAL, DIMENSION(:), INTENT(IN) :: temp, ostwald
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      CALL HTdep(temp+CtoK, Hscc_TO_HscpSI(ostwald, temp+CtoK), Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3748

  !---------------------------------------------------------------------------

  SUBROUTINE ref3749 ! Hscp [M/atm]
    IMPLICIT NONE

    ref = "3749"
    type = "M"
    chem = "acetylene"; casrn = "74-86-2"; inchikey = "HSFWRNGVRCDJHI-UHFFFAOYSA-N"
    CALL Output(0.0378*Hscp_TO_HscpSI)

  END SUBROUTINE ref3749

  !---------------------------------------------------------------------------

  SUBROUTINE ref3750 ! Hscc [1] = Ostwald coefficient
    IMPLICIT NONE

    ref = "3750"
    type = "M"
    chem = "acetylene"; casrn = "74-86-2"; inchikey = "HSFWRNGVRCDJHI-UHFFFAOYSA-N"
    ! The value alpha = 0.6 is read from to bottom right corner of Fig. 1:
    CALL MakeNote(TRIM(ref), "Approximate value extracted from Fig.~1 of " &
      // TRIM(citet()) // ".")
    CALL Output(Hscc_TO_HscpSI(0.6, 20.+CtoK))

  END SUBROUTINE ref3750

  !---------------------------------------------------------------------------

  SUBROUTINE ref3751 ! Hscp [M/atm]
    IMPLICIT NONE

    ref = "3751"
    type = "M"
    ! Tab. I (factor 1E5 is in Tab. I, factor 1E3 is for cm-3 to dm-3):
    CALL CalcH("acetylene", "74-86-2", "HSFWRNGVRCDJHI-UHFFFAOYSA-N", &
      (/ 280., 285., 298., 308. /), (/ 6.98, 5.91, 4.17, 3.04 /) *1E-5*1E3)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, temp, solubility)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_, inchikey_
      REAL, DIMENSION(:), INTENT(IN) :: temp, solubility
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      CALL HTdep(temp, solubility*Hscp_TO_HscpSI, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3751

  !---------------------------------------------------------------------------

  ! ref3752 NOTUSED: The solvent is not water.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3753 ! Hscp [mol m-3 atm-1]
    IMPLICIT NONE

    ref = "3753"
    type = "E"
    ! Tab. 2:
    chem = "CH2BrI" ; inchikey = "TUDWMIUPYRKEFN-UHFFFAOYSA-N"
    CALL MakeNote(TRIM(ref), &
      TRIM(citet())//" probably assume that \chem{CH_2BrI} has the same Henry's law "// &
      "constant as \chem{CH_2ClI}.")
    CALL Output(2.0E3/atm)

  END SUBROUTINE ref3753

  !---------------------------------------------------------------------------

  SUBROUTINE ref3760 ! Hscp [M/atm]
    IMPLICIT NONE

    ref = "3760"
    type = "?"
    ! Tab. 4:
    CALL CalcH("IO",     "14696-98-1", 4.5E2, ""           )
    CALL CalcH("OIO",    "13494-92-3", 1.0E4, ""           )
    CALL CalcH("I2O2",  "215239-62-6", 1.0E4, "whichisomer")
    CALL CalcH("I2O3",   "11085-17-9", 1.0E4, "whichisomer")
    CALL CalcH("I2O4", "1024652-24-1", 1.0E4, "whichisomer")

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, k0, whichisomer)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_, whichisomer
      REAL,             INTENT(IN) :: k0
      chem=chem_ ; casrn=casrn_ ! make global for Output
      IF (whichisomer=="whichisomer") THEN
        CALL MakeNote("whichisomer")
      ENDIF
      CALL MakeNote(TRIM(ref), &
        TRIM(citet())//" refer to \citet{2312} "// &
        "as the source, but the quoted value cannot be found there.")
      CALL Output(k0*Hscp_TO_HscpSI)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3760

  !---------------------------------------------------------------------------

  ! ref3763 NOTUSED: The paper describes QSPR calculations but does not
  !                  present the obtained Henry's law constants.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3764 ! Hscc [1]
    IMPLICIT NONE
    REAL :: SW_exp, SW_calc, VP_exp, VP_calc, LW_exp, LW_calc

    ref = "3764"
    type = "Q"
    OPEN (10,FILE="input/ref3764.dat",STATUS="OLD")
    DO
      READ (10,*,END=999) chem, casrn, inchikey, &
        SW_exp, SW_calc, VP_exp, VP_calc, LW_exp, LW_calc
      IF (chem(1:1)=="#") CYCLE ! skip lines that start with the comment character "#"
      ! "trifluoroacetate" is not used because the name is ambiguous
      ! (could for example be methyl or ethyl ester)
      IF (TRIM(casrn)=="105-46-4") THEN ! sec-butyl acetate
        CALL MakeNote(TRIM(ref)//"dupl", &
        TRIM(citet())//" list this species twice in their table, with different " // &
          "values. As it is unclear which of them is correct, the data are not reproduced here.")
        CALL Output(DUMMY)
        CYCLE
      ENDIF
      CALL Output(Hscc_TO_HscpSI_atT0 * (10.**LW_calc))
    ENDDO
999 CONTINUE
    CLOSE(10)

  END SUBROUTINE ref3764

  !---------------------------------------------------------------------------

  SUBROUTINE ref3765 ! Hscp [M/atm]
    IMPLICIT NONE

    ref = "3765"
    type = "Q"
    ! Tab. S2 in supplement:
    CALL CalcH("IEPOXO4",        "DNQIATHBSKHPLN-UHFFFAOYSA-N", 2.1E6,  13700.)
    CALL CalcH("DIEPOXO3",       "QGQUFAZKBSHWQB-UHFFFAOYSA-N", 1.3E5,  13700.)
    CALL CalcH("IEPOX1CO3H",     "DJPWYFXNRMHUNU-UHFFFAOYSA-N", 9.1E4,  14400.)
    CALL CalcH("ME1TRIOL",       "MTMASQITBHTGFW-UHFFFAOYSA-N", 4.9E9,  11100.)
    CALL CalcH("METHFDIOL",      "UNAIOYXCADSMDR-UHFFFAOYSA-N", 9.1E7,  10900.)
    CALL CalcH("MeBuTETROL",     "HGVJFBSSLICXEM-UHFFFAOYSA-N", 6.2E12, 13600.)
    CALL CalcH("METRICO",        "LCGBCDAYKOJPSO-UHFFFAOYSA-N", 7.4E8,  11500.)
    CALL CalcH("MEDICO4CO",      "GXKSWJOBVXKANI-UHFFFAOYSA-N", 3.1E8,  12100.)
    CALL CalcH("MEDICO1CO",      "RRXXDUUNVWXRFN-UHFFFAOYSA-N", 2.5E9,  12100.)
    CALL CalcH("ISOP1N4OOH",     "IRFXVIPRKCCSLU-UHFFFAOYSA-N", 9.8E4,  11400.)
    CALL CalcH("ISOP1N4ONO2",    "JGJBVRGABXKDRR-UHFFFAOYSA-N", 2.6E2,   9400.)
    CALL CalcH("HC4CCHO",        "GCHJBJOOADXJFT-UHFFFAOYSA-N", 1.2E5,  10600.)
    CALL CalcH("EPXISOPNONO2",   "UAFKAIYDADHRMX-UHFFFAOYSA-N", 5.4E3,  12500.)
    CALL CalcH("ISOP1N5ONO2",    "WRGRJJOKHABEJT-UHFFFAOYSA-N", 5.9E6,  13300.)
    CALL CalcH("ISOP1N5OOH",     "LLNBZHMZWYQLAS-UHFFFAOYSA-N", 2.3E9,  14300.)
    CALL CalcH("ISOP1N6CO",      "XXORYENTKCFFSM-UHFFFAOYSA-N", 2.1E7,  14000.)
    CALL CalcH("ISOP1N2ONO2",    "FQSKJUXVWIFNKX-UHFFFAOYSA-N", 1.6E2,   9400.)
    CALL CalcH("ISOP1N23O4CO",   "WBVYILWVTRBTDJ-UHFFFAOYSA-N", 1.5E5,  12400.)
    CALL CalcH("ISOP1N23O4ONO2", "FSZUIXLMCRZJMS-UHFFFAOYSA-N", 4.6E3,  12500.)
    CALL CalcH("C5H112ONO2",     "RWRBSYOTDDOXKC-UHFFFAOYSA-N", 3.4E-1,  6000.)
    CALL CalcH("C5H112O2H",      "AXIHYONNAPPFSO-UHFFFAOYSA-N", 1.4E2,   7000.)
    CALL CalcH("C5OHONO2",       "RIQPKERROQFFJK-UHFFFAOYSA-N", 2.1E3,   9900.)
    CALL CalcH("C5OHOOH",        "GSRFGQMXWRCOMF-UHFFFAOYSA-N", 7.2E5,  10900.)
    CALL CalcH("C5OHCO",         "JSHPTIGHEWEXRW-UHFFFAOYSA-N", 3.6E5,   9800.)
    CALL CalcH("C6H13ONO2",      "JLGBOJJVQLNMGV-UHFFFAOYSA-N", 3.1E-2,  6300.)
    CALL CalcH("C6H13O2H",       "XWXUHAUZCICPHE-UHFFFAOYSA-N", 1.0E2,   7300.)
    CALL CalcH("C6OHONO2",       "PAWWQEMXBSLVJC-UHFFFAOYSA-N", 1.6E3,  10200.)
    CALL CalcH("C6OHOOH",        "BVOYWPYSMMRYPX-UHFFFAOYSA-N", 5.5E5,  11200.)
    CALL CalcH("C6OHCO",         "ZSDLLTJVENEIDW-UHFFFAOYSA-N", 2.8E5,  10100.)
    CALL CalcH("C7H15ONO2",      "HHXLSUKHLTZWKR-UHFFFAOYSA-N", 2.3E-1,  6600.)
    CALL CalcH("C7H15O2H",       "FWELUXZVATZEMI-UHFFFAOYSA-N", 7.8E1,   7600.)
    CALL CalcH("C7OHONO2",       "MZJHMUSTUUJGSJ-UHFFFAOYSA-N", 1.2E3,  10600.)
    CALL CalcH("C7OHOOH",        "KHWIBANUCNWWGW-UHFFFAOYSA-N", 4.2E5,  11600.)
    CALL CalcH("C72CO5OH",       "MQRALIJAASQPNT-UHFFFAOYSA-N", 2.1E5,  10500.)
    CALL CalcH("C8H17ONO2",      "QCOKASLKYUXYJH-UHFFFAOYSA-N", 6.9E2,   7000.)
    CALL CalcH("C8H17O2H",       "NAXZMRYIZGEALQ-UHFFFAOYSA-N", 5.9E1,   8000.)
    CALL CalcH("C8OHONO2",       "UEQWMROPIQKFIM-UHFFFAOYSA-N", 9.1E2,  10900.)
    CALL CalcH("C8OHOOH",        "QQTSMKQYSKGCOU-UHFFFAOYSA-N", 3.2E5,  11900.)
    CALL CalcH("C82CO5OH",       "KZPPEBIAPHLFQD-UHFFFAOYSA-N", 1.6E5,  10800.)
    CALL CalcH("HOC2H4CHO",      "AKXKFZDCRYJKTF-UHFFFAOYSA-N", 3.6E3,   9900.)
    CALL CalcH("C517OOH",        "OAFCGFSFCVGKSH-UHFFFAOYSA-N", 1.1E8,  13400.)
    CALL CalcH("HMVKBCHO",       "XYGGNMZYIGPNCC-UHFFFAOYSA-N", 3.6E7,  10800.)
    CALL CalcH("CO2C4CHO",       "KEHNRUNQZGRQHU-UHFFFAOYSA-N", 5.6E4,   8400.)
    CALL CalcH("C519OOH",        "RDSFTYTWLRHJGZ-UHFFFAOYSA-N", 1.7E7,  13400.)
    CALL CalcH("C517CHO",        "JGHPNZFTVGSLKF-UHFFFAOYSA-N", 9.6E4,  12700.)
    CALL CalcH("C622OOH",        "GWTGWFHICNOSTQ-UHFFFAOYSA-N", 1.3E6,  12000.)
    CALL CalcH("C519CHO",        "PVAHQKAVPKMXGV-UHFFFAOYSA-N", 2.8E8,  11100.)
    CALL CalcH("C622ONO2",       "UCAFMJJTBHTENA-UHFFFAOYSA-N", 3.5E3,  11000.)
    CALL CalcH("C622CHO",        "JWVBPJJNIHELFZ-UHFFFAOYSA-N", 5.9E5,  11300.)
    CALL CalcH("C728OOH",        "WJYXJQPIZCIBJO-UHFFFAOYSA-N", 5.4E12, 18100.)
    CALL CalcH("C727OOH",        "IMCLYNFQESJQRO-UHFFFAOYSA-N", 2.0E7,  12700.)
    CALL CalcH("C624CHO",        "AIMYSSDKEKDSEU-UHFFFAOYSA-N", 5.9E5,  11300.)
    CALL CalcH("C730OOH",        "YXKHJQXHKYJCPI-UHFFFAOYSA-N", 6.2E12, 18100.)
    CALL CalcH("C728ONO2",       "CDBHNFBQKSTCON-UHFFFAOYSA-N", 1.5E10, 17100.)
    CALL CalcH("C622CONO2",      "DJCVHBPUJHAWIU-UHFFFAOYSA-N", 1.0E5,  13900.)
    CALL CalcH("C730ONO2",       "RVWIAVBELSCRER-UHFFFAOYSA-N", 3.5E11, 17100.)
    CALL CalcH("C818CO",         "CRQAGKNTHDNGEW-UHFFFAOYSA-N", 6.6E8,  15900.)
    CALL CalcH("C816CO",         "FXXXYZSMIIVJDG-UHFFFAOYSA-N", 3.1E4,  10200.)
    CALL CalcH("C819OOH",        "JRXMSSDNDAZISI-UHFFFAOYSA-N", 2.1E9,  17000.)
    CALL CalcH("C817CO",         "WSCYQCAMZDXSLH-UHFFFAOYSA-N", 3.1E7,  12000.)
    CALL CalcH("C817OOH",        "WQMXPSRLZZCZTO-UHFFFAOYSA-N", 1.2E8,  13000.)
    CALL CalcH("C818OOH",        "DLMBPALZQHFDOH-UHFFFAOYSA-N", 5.8E10, 17000.)
    CALL CalcH("C729CHO",        "MEGRLTONSLAOOM-UHFFFAOYSA-N", 8.1E4,  10200.)
    CALL CalcH("C822OOH",        "XWYULULFGKWPMD-UHFFFAOYSA-N", 8.9E5,  11300.)
    CALL CalcH("C824OOH",        "FGEKWAMEWUMQIL-UHFFFAOYSA-N", 8.5E8,  15300.)
    CALL CalcH("RO5R5",          "MESHXYGXEHAFFE-UHFFFAOYSA-N", 1.3E7,  17300.)
    CALL CalcH("C819ONO2",       "XLUWXWLWQTYVOF-UHFFFAOYSA-N", 4.0E8,  16000.)
    CALL CalcH("C817ONO2",       "ZONZWPKPCIWKRN-UHFFFAOYSA-N", 4.0E6,  12000.)
    CALL CalcH("C822ONO2",       "JWAJNALJDDSHOA-UHFFFAOYSA-N", 2.9E3,  10300.)
    CALL CalcH("C824ONO2",       "YMYILPYDFNRVBZ-UHFFFAOYSA-N", 9.6E8,  14300.)
    CALL CalcH("C923OOH",        "ZSCOJLRNNZZYOH-UHFFFAOYSA-N", 1.1E6,  11700.)
    CALL CalcH("C923OH",         "VZYSLJNPEGILSI-UHFFFAOYSA-N", 5.4E5,  12000.)
    CALL CalcH("NORLIMAL",       "WPNOYHDEEVPEJH-UHFFFAOYSA-N", 6.2E4,  10600.)
    CALL CalcH("C924OOH",        "BVODBOGLMHSHGK-UHFFFAOYSA-N", 3.0E8,  15600.)
    CALL CalcH("C924OH",         "OXSRPARPRXNCEH-UHFFFAOYSA-N", 3.8E7,  15900.)
    CALL CalcH("LMLKET",         "CWEQHJLFCKMWEE-UHFFFAOYSA-N", 1.1E8,  13000.)
    CALL CalcH("LIMKET",         "HOBBEYSRFFJETF-UHFFFAOYSA-N", 6.5E1,   8000.)
    CALL CalcH("C817CO3H",       "NVOUBBWYYUIRJW-UHFFFAOYSA-N", 3.4E9,  15900.)
    CALL CalcH("C822CO3H",       "DRDXOQWGQPPQFG-UHFFFAOYSA-N", 1.7E6,  14200.)
    CALL CalcH("C822CO2H",       "ADULCIYKWVJSFE-UHFFFAOYSA-N", 4.5E6,  14500.)
    CALL CalcH("C91ONO2",        "JEWJQSHMOKPQHI-UHFFFAOYSA-N", 3.2E3,  10700.)
    CALL CalcH("NORLIMONO2",     "MUTWZANDXPDWFF-UHFFFAOYSA-N", 3.8E10, 16300.)
    CALL CalcH("C92ONO2",        "PNFDSIMYWKOAKE-UHFFFAOYSA-N", 4.8E7,  14600.)
    CALL CalcH("C817CONO2",      "LHFXXFGIVAQFTO-UHFFFAOYSA-N", 1.2E7,  14900.)
    CALL CalcH("C822CONO2",      "RVFLVBTUEWAABB-UHFFFAOYSA-N", 6.3E3,  13200.)
    CALL CalcH("BPINCOOH",       "VVHMRXSQYYKNLC-UHFFFAOYSA-N", 3.6E6,  13400.)
    CALL CalcH("RO5R1O2H",       "NSOHXIGSWKLSHV-UHFFFAOYSA-N", 1.0E8,  16600.)
    CALL CalcH("RO5R2O2H",       "MFXJOKCSTDMZRM-UHFFFAOYSA-N", 1.8E9,  19100.)
    CALL CalcH("RO5R3O2H",       "IEPFJXDIWPQIRC-UHFFFAOYSA-N", 7.8E9,  21600.)
    CALL CalcH("RO5R4O2H",       "YORZXZYHORHTRW-UHFFFAOYSA-N", 3.6E12, 24000.)
    CALL CalcH("C923CO3H",       "JSHGCXGTVADTAK-UHFFFAOYSA-N", 2.2E6,  14500.)
    CALL CalcH("LIMONONIC",      "NJOIWWRMLFSDTM-UHFFFAOYSA-N", 5.8E6,  14900.)
    CALL CalcH("LIMAL",          "OGCGCISRMFSLTC-UHFFFAOYSA-N", 4.1E4,  10900.)
    CALL CalcH("LIMALOOH",       "OQLBLHLZVAPRGN-UHFFFAOYSA-N", 4.4E12, 17700.)
    CALL CalcH("LIMALOH",        "ISUQULRVPDSRIP-UHFFFAOYSA-N", 2.2E12, 18000.)
    CALL CalcH("LIMALAOOH",      "DVORBFWFCUOYLW-UHFFFAOYSA-N", 2.5E7,  14500.)
    CALL CalcH("LIMALAOH",       "VNYSFQVUCPNMDZ-UHFFFAOYSA-N", 3.1E6,  14900.)
    CALL CalcH("LIMALACO",       "OXUPJDFJWCMZMO-UHFFFAOYSA-N", 4.4E6,  13400.)
    CALL CalcH("LIMALBOOH",      "DCWLKDSHBWMMQU-UHFFFAOYSA-N", 2.4E7,  14500.)
    CALL CalcH("BPINCONO2",      "ZELLOEPLERCREX-UHFFFAOYSA-N", 8.7E3,  18800.)
    CALL CalcH("C9CONO2",        "FNVMVXZLPDKVAS-UHFFFAOYSA-N", 8.1E3,  19900.)
    CALL CalcH("LIMALONO2",      "FDBNDHKWTKEDSC-UHFFFAOYSA-N", 1.3E10, 23000.)
    CALL CalcH("LIMONO2",        "DEKWHDOXDJNWRD-UHFFFAOYSA-N", 3.9E6,  13500.)
    CALL CalcH("LIMAB15ONO22",   "UNHNJOVXNACOSQ-UHFFFAOYSA-N", 3.1E9,  18200.)
    CALL CalcH("LIMAB15ONO2OOH", "JBKULLYZCMWERK-UHFFFAOYSA-N", 5.1E11, 19200.)
    CALL CalcH("LIMAB15OOH2",    "IRUZCTPAAZATQM-UHFFFAOYSA-N", 2.3E14, 20200.)
    CALL CalcH("LIMAB1ONO2",     "LZDKYYHMAURBIK-UHFFFAOYSA-N", 8.5E3,  12400.)
    CALL CalcH("LIMAB1OOH",      "FPMDSGWWEURSNO-UHFFFAOYSA-N", 3.6E6,  13400.)
    CALL CalcH("ROO6R1ONO2",     "JOFCABYMPNXMIS-UHFFFAOYSA-N", 1.4E4,  15200.)
    CALL CalcH("ROO6R1OOH",      "GBBPAXKBVMQHIG-UHFFFAOYSA-N", 6.9E6,  16300.)
    CALL CalcH("ROO6R5ONO2",     "LNWOWXJMUJHMJJ-UHFFFAOYSA-N", 1.7E7,  15300.)
    CALL CalcH("ROO6R5OOH",      "RAASFOSVQCCDRJ-UHFFFAOYSA-N", 4.7E9,  16300.)
    CALL CalcH("ROO6R6ONO2",     "CRECMHNOVVIMCR-UHFFFAOYSA-N", 6.2E7,  12400.)
    CALL CalcH("ROO6R6OOH",      "QZPUZDFJKAEVLL-UHFFFAOYSA-N", 2.6E10, 13500.)
    CALL CalcH("ROO6R7ONO2",     "COKJDYWOSQONMU-UHFFFAOYSA-N", 7.6E6,  12800.)
    CALL CalcH("ROO6R7OOH",      "UPAWKBDHGPWUCG-UHFFFAOYSA-N", 3.3E9,  13800.)
    CALL CalcH("C52COCONO2",     "QJXLMYNMVOSKND-UHFFFAOYSA-N", 6.8E3,  11000.)
    CALL CalcH("C52COCO3H",      "COVHHGSUFOHLBW-UHFFFAOYSA-N", 1.9E6,  12000.)
    CALL CalcH("C624ONO2",       "XYBUKFJLXVIRLY-UHFFFAOYSA-N", 2.0E12, 18600.)
    CALL CalcH("C518OOH",        "PQEISBQMLHJHJQ-UHFFFAOYSA-N", 4.2E6,  14200.)
    CALL CalcH("C624OOH",        "FUTSLQULSAKEJA-UHFFFAOYSA-N", 1.2E15, 19600.)
    CALL CalcH("C520ONO2",       "ZSADVAXZKGJKSG-UHFFFAOYSA-N", 2.9E11, 20800.)
    CALL CalcH("BIACETOH2",      "GJCZUCLKDGABDS-UHFFFAOYSA-N", 2.1E8,  16700.)
    CALL CalcH("C520OOH",        "GRPDYNZWDDUILX-UHFFFAOYSA-N", 5.4E13, 21800.)
    CALL CalcH("GOLIG1",         "LEKXYOUPWKVTGM-UHFFFAOYSA-N", 1.5E14, 18700.)
    CALL CalcH("GOLIG2",         "GWSRJJGBNXJVPO-UHFFFAOYSA-N", 1.5E17, 24000.)
    CALL CalcH("GOLIG3",         "ADECTKUVLJSMDK-UHFFFAOYSA-N", 3.8E20, 28600.)
    CALL CalcH("MGLYOXDA",       "QMSLRDMQAWWKIZ-UHFFFAOYSA-N", 2.0E10, 15500.)
    CALL CalcH("MGLYOXDB",       "IUHMFBWAUOLAFM-UHFFFAOYSA-N", 1.5E15, 22400.)
    CALL CalcH("MGLYFA",         "NOUUUBJIFPBZMZ-UHFFFAOYSA-N", 4.5E6,  15600.)
    CALL CalcH("MGLYFB",         "UCPBJDKPOQEXSQ-UHFFFAOYSA-N", 7.2E10, 22500.)

  CONTAINS

    SUBROUTINE CalcH (chem_, inchikey_, Hs, B)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, inchikey_
      REAL,             INTENT(IN) :: Hs, B
      chem=chem_ ; inchikey=inchikey_ ! make global for Output
      CALL MakeNote(TRIM(ref)//"-GROMHE", &
        "Henry's law constants calculated using the GROMHE model. " // &
        "Temperature dependences calculated with the method of \citet{1917}.")
      CALL Output(Hs*Hscp_TO_HscpSI,B)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3765

  !---------------------------------------------------------------------------

  SUBROUTINE ref3766 ! Hvcc [1]
    IMPLICIT NONE
    INTEGER :: number, datatype, group
    REAL :: logKaw
    CHARACTER :: set, reftype
    CHARACTER(2) :: notelabel
    CHARACTER(300) :: notetext
    CHARACTER(STRLEN_INCHIKEY) :: casrn_or_inchikey

    ref = "3766"
    type = "?"

    ! The paper presents a new model to calculate Henry's law constants
    ! but the calculated values are not shown in the paper.

    ! For tetrachlorvinphos, the casrn 22248-79-9 is used, not 961-11-5.

    OPEN (10,FILE="input/ref3766.dat",STATUS="OLD")
    DO
      READ (10,*,END=999) number, chem, casrn_or_inchikey, &
        logKaw, datatype, set, group, reftype
      ! skip lines that start with the comment character "#":
      IF (chem(1:1)=="#") CYCLE
      ! choose casrn or inchikey:
      IF (LEN_TRIM(casrn_or_inchikey)==27) THEN
        casrn = "" ; inchikey = casrn_or_inchikey
      ELSE
        casrn = TRIM(casrn_or_inchikey) ; inchikey = ""
      ENDIF
      ! create a note based on reftype and datatype:
      notelabel(1:1) = reftype
      SELECT CASE(reftype)
      CASE ("O")
        notetext = "a single original paper and"
      CASE ("D")
        notetext = "a single database or data collection and"
      CASE ("M")
        notetext = "multiple sources and"
      END SELECT
      notelabel(2:2) = str(datatype)
      SELECT CASE(datatype)
      CASE (1)
        notetext = TRIM(notetext) // " measured directly."
      CASE (2)
        notetext = TRIM(notetext) // " obtained from experimental vapor pressure and " // &
          "water solubility."
      CASE (3)
        notetext = TRIM(notetext) // " obtained from experimental vapor pressure and " // &
          "the infinite-dilution activity coefficient."
      CASE (4)
        notetext = TRIM(notetext) // " obtained from the experimental octanol--water " // &
          "partition coefficient and the octanol--air partition coefficient."
      END SELECT
      CALL MakeNote(TRIM(ref)//"_"//notelabel, &
        TRIM(citet()) // " present ``curated experimental'' Henry's law " // &
        "constants from the literature but do not provide any references. It " // &
        "is only mentioned that the value is from " // TRIM(notetext))
      ! table notes:
      ! 1 Unknown or Variable composition, UVCB, used: p-n-nonylphenol (813)
      ! 2 used tautomer: 4-[(2-chlorophenyl)diazenyl]-3-methyl-1,2-oxazol-5-ol (1882)
      ! 3 Unknown or Variable composition, UVCB,
      !   used: 4-(6-methylheptyl)-2,6-dinitrophenyl-(2E)-but-2-enoate (2194)
      ! 4 Unknown or Variable composition, UVCB,
      !   used: 3-tert-butylphenyl) diphenyl phosphate (2467) -> NIAVXHWAURFNOW-UHFFFAOYSA-N
      CALL Output(Hvcc_TIMES_HscpSI_atT0 / (10.**logKAW))
    ENDDO
999 CONTINUE
    CLOSE(10)

  END SUBROUTINE ref3766

  !---------------------------------------------------------------------------

  SUBROUTINE ref3769 ! Hscp [M/atm]
    IMPLICIT NONE
    INTEGER :: i

    ref = "3769"
    type = "M"

    CALL CalcH("PAN", "2278-22-0", "VGQXTTSVLMQFHM-UHFFFAOYSA-N", &
      (/ 293.15, 278.15 /), &
      (/ 4.2,  11.8 /), & ! Tab. 1
      (/ 101., 269. /)) ! Tab. S5
    CALL CalcH("PPN", "5796-89-4", "TXINBPKSWKFMNB-UHFFFAOYSA-N", &
      (/ 298.15, 293.15, 289.15, 285.65, 281.65, 278.15 /), &
      (/ 1.49, 2.67, 3.14, 4.11, 4.96,  7.01  /), & ! Tab. 1
      (/ 36.5, 64.2, 74.5, 96.4, 114.7, 160.1 /)) ! Tab. S6

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, temp_, Hscp, Hscc)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_, inchikey_
      REAL, DIMENSION(:), INTENT(IN) :: temp_, Hscp, Hscc
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      DO i=1,SIZE(temp_)
        CALL consistency_check(Hscp_TO_HscpSI*Hscp(i), &
          Hscc_TO_HscpSI(Hscc(i), temp_(i)), "Values in Table~1 and S5/S6")
      ENDDO
      CALL HTdep(temp_, Hscp_TO_HscpSI*Hscp, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3769

  !---------------------------------------------------------------------------

  SUBROUTINE ref3770 ! Hscp [M/atm]
    IMPLICIT NONE

    ref = "3770"
    type = "E"
    ! Tab. 5 in supplement:
    CALL CalcH("BrCl", "13863-41-7", "CODNYICXDISAEA-UHFFFAOYSA-N", 7.0E-2, 4000.)
    CALL CalcH("HI",   "10034-85-2", "XMBWDFGMSWQBCA-UHFFFAOYSA-N", 2.5E1,  9800.)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, k0, c)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_, inchikey_
      REAL,             INTENT(IN) :: k0, c
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      CALL Output(k0*Hscp_TO_HscpSI, c)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3770

  !---------------------------------------------------------------------------

  SUBROUTINE ref3771 ! Hscp [M/atm]
    IMPLICIT NONE

    ref = "3771"
    type = "E"

    ! INO2 from Tab. 1:
    chem = "INO2" ; casrn = "15465-40-4"
    CALL MakeNote(TRIM(ref)//"-INO2", &
      TRIM(citet())//" assume that \chem{INO_2} has the same Henry's law "// &
      "constant as \chem{BrNO_2}.")
    CALL Output(3.0E-1*Hscp_TO_HscpSI)

    ! IxOy from Tab. 1:
    CALL CalcH("I2O2",  "215239-62-6")
    CALL CalcH("I2O3",   "11085-17-9")
    CALL CalcH("I2O4", "1024652-24-1")

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL MakeNote("whichisomer")
      CALL MakeNote(TRIM(ref)//"-IxOy", &
        "Assumed to be infinity by analogy with \chem{INO_3}.")
      CALL Output(DUMMY, limit="i")
    END SUBROUTINE CalcH

  END SUBROUTINE ref3771

  !---------------------------------------------------------------------------

  SUBROUTINE ref3776 ! HscpSI [mol/(m3*Pa)]
    IMPLICIT NONE

    ref = "3776"
    type = "M"

    ! For diacetyl and 2,3-pentanedione, use the intrinsic value from
    ! Tab. S6. For the other compounds, hydration has "no pronounced
    ! effect" (p. 2303) and thus, the effective and the intrinsic values
    ! are practically identical.

    ! effective Henry's law constant (Tab. S4):
    ! CALL CalcH("diacetyl",                 "431-03-8", "QSJXEFYPDANLFS-UHFFFAOYSA-N", &
    !   (/ 15., 25., 35., 50. /),      (/ 1.27, 5.50E-1, 2.52E-1, 9.44E-2 /))
    ! CALL CalcH("2,3-pentanedione",         "600-14-6", "TZMFJUDUGYTVRY-UHFFFAOYSA-N", &
    !   (/ 15., 25., 25., 35., 50. /), (/ 9.86E-1, 3.50E-1, 3.54E-1, 1.51E-1, 5.28E-2 /))
    CALL CalcH("p-tolualdehyde",           "104-87-0", "FXLOVSHXALFLKQ-UHFFFAOYSA-N", &
      (/ 15., 25., 35., 50. /),      (/ 1.06, 4.22E-1, 2.16E-1, 9.24E-2 /))
    CALL CalcH("m-tolualdehyde",           "620-23-5", "OVWYEQOVUDKZNU-UHFFFAOYSA-N", &
      (/ 15., 25., 35., 50. /),      (/ 6.00E-1, 2.92E-1, 1.62E-1, 6.61E-2 /))
    CALL CalcH("6-methyl-5-hepten-2-one",  "110-93-0", "UHEPJGULSIKKTP-UHFFFAOYSA-N", &
      (/ 15., 25., 25., 35., 50. /), (/ 2.44E-1, 1.11E-1, 1.20E-1, 6.24E-2, 2.30E-2 /))
    CALL CalcH("citral",                  "5392-40-5", "WTEVQBCEXWBHNA-UHFFFAOYSA-N", &
      (/ 15., 25., 35., 50. /),      (/ 6.33E-1, 2.99E-1, 1.37E-1, 5.15E-2 /))

    ! intrinsic Henry's law constant (Tab. S6):
    CALL CalcH("diacetyl",                 "431-03-8", "QSJXEFYPDANLFS-UHFFFAOYSA-N", &
      (/ 15., 25., 35., 50. /), (/ 2.40E-1, 1.56E-1, 8.05E-2, 3.95E-2 /))
    CALL CalcH("2,3-pentanedione",         "600-14-6", "TZMFJUDUGYTVRY-UHFFFAOYSA-N", &
      (/ 15., 25., 35., 50. /), (/ 2.80E-1, 1.22E-1, 6.28E-2, 2.68E-2 /))

    ! Tab. 1:
    chem = "acetoin" ; casrn = "513-86-0" ; inchikey = "ROWKJAVDOGWPAT-UHFFFAOYSA-N"
    CALL MakeNote(TRIM(ref)//"_acetoin", &
        "Center of the range (2.3\dots 4.0)\,\unit{mol\,m^{-3}\,Pa^{-1}}.")
    CALL Output((2.3+4.0)/2.)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, inchikey_, temp, Hscp)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_, inchikey_
      REAL, DIMENSION(:), INTENT(IN) :: temp, Hscp
      chem=chem_ ; casrn=casrn_ ; inchikey=inchikey_ ! make global for Output
      CALL HTdep(temp+CtoK, Hscp, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3776

  !---------------------------------------------------------------------------

  SUBROUTINE ref3777 ! Bunsen coefficient (called beta here) [1]
    IMPLICIT NONE
    ref = "3777"
    type = "M"

    ! Although ref3777 used a 3-parameter equation with A1, A2, and A3
    ! in Eqn. (5), this did not produce better results than a simple
    ! two-parameter equation.
    CALL CalcH("carbon monoxide", "630-08-0", &
      (/ -0.0878774, 0.182515, 0.33123, 0.547544, 0.588103, 0.723299, 0.736818, &
      1.23704, 5.68499, 5.75259, 5.92835, 5.94187, 7.63182, 7.69941, 7.73997, &
      7.90221, 7.95629, 12.2285, 12.3231, 12.5394, 12.553, 16.6629, 16.8251, &
      16.9739, 17.5282, 20.0428, 20.1239, 20.2997, 20.5566, 20.6242, 20.7458, &
      20.9351, 22.8414, 22.8684 /), &
      (/ 0.0246081, 0.0226892, 0.0273108, 0.0240676, 0.0273108, 0.0267162, &
      0.0263108, 0.0270135, 0.0231216, 0.0213378, 0.0199595, 0.0207432, &
      0.0211757, 0.0217973, 0.0206622, 0.0213649, 0.0225, 0.0201216, &
      0.0184459, 0.019527, 0.0202027, 0.0191216, 0.0188784, 0.0167973, &
      0.0177162, 0.0173919, 0.0180405, 0.0168243, 0.0180676, 0.018473, &
      0.0155541, 0.0181486, 0.0173919, 0.0166081 /) )
    CALL CalcH("hydrogen", "1333-74-0", &
      (/ -0.187097, 0.780645, 0.793548, 0.870968, 0.974194, 0.987097, 1.14194, &
      1.16774, 3.49032, 3.56774, 3.59355, 7.8, 8.09677, 8.12258, 9.50323, &
      9.52903, 13.1548, 13.1935, 13.2065, 17.3613, 17.3742, 17.4129, 17.5548, &
      20.2, 20.5871, 20.9871, 21.8129, 22.2903, 22.6903, 22.7032, 22.9097, &
      22.9484 /), &
      (/ 0.0231979, 0.0221148, 0.0205416, 0.0239201, 0.0212895, 0.0225532, &
      0.0209026, 0.0198453, 0.0211348, 0.0201547, 0.0187105, 0.0194068, &
      0.0209542, 0.0194326, 0.0189942, 0.0194068, 0.0198453, 0.01951, &
      0.0201547, 0.0177563, 0.0189168, 0.01853, 0.0193553, 0.0178594, &
      0.0181431, 0.0178852, 0.0176015, 0.0174984, 0.0186074, 0.0189168, &
      0.0189684, 0.0187363 /) )

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, beta)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, beta
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp+CtoK, beta*alpha_TO_HscpSI, Hominus, mindHR)
      CALL MakeNote(TRIM(ref), &
        "Data digitized from Figs.~2 and 3 in " // TRIM(citet()) // ".")
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3777

  !---------------------------------------------------------------------------

  ! ref3778 NOTUSED: The experimental data shown in the supplement is
  !                  not new but cited from Kühne et al. (ref1917) and
  !                  Abraham et al. (ref1501).

  !---------------------------------------------------------------------------

  ! ref3779 NOTUSED: Only a review of LFER methods, no new data.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3780 ! Hscc [1]
    IMPLICIT NONE
    ref = "3780"
    type = "Q"

    ! Tab. 2:
    CALL CalcH("tetrafluoromethane",        "75-73-0", -2.14)
    CALL CalcH("hexafluoropropanol",       "920-66-1",  2.77)
    CALL CalcH("2,2,2-trifluoroethanol",    "75-89-8",  3.28)
    CALL CalcH("isoflurane",             "26675-46-7", -0.023)

    ! supplement not used (contains only experimental data from ref1501)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, K_water_air)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: K_water_air
      chem=chem_ ; casrn=casrn_ ! make global for Output
      IF (casrn_=="920-66-1") THEN
        CALL MakeNote(TRIM(ref)//"_920-66-1", "Comparing with \citet{1501}, " // &
          "it seems that the compound called ``hexafluoropropanol'' " // &
          "by \citet{3780} refers to 1,1,1,3,3,3-hexafluoro-2-propanol.")
      ENDIF
      IF (casrn_=="75-89-8") THEN
        CALL MakeNote(TRIM(ref)//"_75-89-8", "Comparing with \citet{1501}, " // &
          "it seems that the compound called ``trifluoroethanol'' " // &
          "by \citet{3780} refers to 2,2,2-trifluoroethanol.")
      ENDIF
      CALL Output(Hscc_TO_HscpSI_atT0 * (10.**K_water_air))
    END SUBROUTINE CalcH

  END SUBROUTINE ref3780

  !---------------------------------------------------------------------------

  SUBROUTINE ref3781 ! Bunsen coefficient (called beta here) [1]
    IMPLICIT NONE
    ref = "3781"
    type = "M"
    ! Tab. 1:
    CALL CalcH("hydrogen", "1333-74-0", &
      (/ 1.45, 1.48, 1.49, 4.39, 4.55, 4.55, 4.58, 4.59, 9.83, 9.84, 9.84, &
      9.85, 9.85, 9.85, 9.86, 9.86, 9.87, 9.88, 14.62, 14.65, 14.67, 14.70, &
      14.76, 14.77, 14.79, 14.79, 14.79, 14.80, 19.86, 19.86, 19.87, 19.88, &
      19.88, 24.70, 24.70, 24.73, 24.73, 24.77, 28.84, 29.02, 29.16, 29.32 /), &
      (/ 0.02150, 0.02148, 0.02152, 0.02081, 0.02074, 0.02067, 0.02072, &
      0.02079, 0.01954, 0.01964, 0.01949, 0.01947, 0.01949, 0.01955, 0.01960, &
      0.01957, 0.01957, 0.01960, 0.01882, 0.01876, 0.01873, 0.01871, 0.01882, &
      0.01869, 0.01871, 0.01890, 0.01885, 0.01880, 0.01806, 0.01812, 0.01811, &
      0.01804, 0.01805, 0.01740, 0.01754, 0.01747, 0.01741, 0.01742, 0.01696, &
      0.01709, 0.01706, 0.01712 /) )

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, beta)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, beta
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! data need a 3-parameter fit:
      CALL HTdep_ABC(temp+CtoK, beta*alpha_TO_HscpSI, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3781

  !---------------------------------------------------------------------------

  SUBROUTINE ref3782 ! Bunsen coefficient (called beta here) [1]
    IMPLICIT NONE
    ref = "3782"
    type = "M"
    ! Tab. 1:
    CALL CalcH("hydrogen", "1333-74-0", &
      (/  0.14,  5.04, 10.00, 10.02, 16.67, 23.63, 29.25 /), &
      (/ 22.17, 20.63, 19.66, 19.67, 18.66, 17.62, 17.18 /) * 1E-3 )

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, beta)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, beta
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! data need a 3-parameter fit:
      CALL HTdep_ABC(temp+CtoK, beta*alpha_TO_HscpSI, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3782

  !---------------------------------------------------------------------------

  SUBROUTINE ref3783 ! Bunsen coefficient (called s0 here) [1]
    IMPLICIT NONE
    ref = "3783"
    type = "M"
    ! Tab. 2 (Solubilities of gases (in c.c. at N.T.P.) per 1000 g. of
    ! water at a total gas pressure of 1 atm. = 1E-6*rhoH2O(T)*Bunsen):
    CALL CalcH("hydrogen", "1333-74-0", &
      (/ 12.3, 16.1, 17.3, 23.0, 25.0, 33.2, 45.0, 47.3, 54.4, 60.3, 65.6, 71.7 /), &
      (/ 19.23, 18.71, 18.54, 17.86, 17.70, 16.98, 16.37, 16.29, 16.18, 16.07, 16.14, 16.14 /) )
    CALL CalcH("methane", "74-82-8", &
      (/ 11.9, 14.7, 20.5, 25.2, 35.0, 41.0, 46.8, 51.8, 62.3, 71.5, 75.2 /), &
      (/ 39.90, 37.41, 32.96, 30.06, 25.35, 23.82, 22.44, 21.38, 19.86, 19.10, 18.84 /) )
    CALL CalcH("ethane", "74-84-0", &
      (/ 12.3, 12.6, 16.4, 17.6, 24.6, 30.5, 32.5, 35.3, 40.9, 49.1, 62.4, 69.5, 71.4, 72.4 /), &
      (/ 53.46, 52.60, 47.53, 46.13, 38.55, 33.73, 32.36, 30.83, 28.25, 25.23, 21.88, 20.80, 20.46, 20.46 /) )
    CALL CalcH("propane", "74-98-6", &
      (/ 12.3, 15.5, 21.5, 22.8, 25.5, 26.0, 31.5, 40.3, 49.0, 59.0, 64.5, 74.1 /), &
      (/ 50.70, 45.39, 37.07, 35.65, 32.58, 32.21, 28.25, 22.86, 19.50, 16.94, 16.03, 14.55 /) )
    CALL CalcH("butane", "106-97-8", & ! n-butane
      (/ 10.9, 14.1, 20.0, 25.0, 30.1, 35.0, 40.1, 45.0, 49.9, 58.0, 65.4, 66.0, 70.8, 76.1 /), &
      (/ 43.85, 38.28, 30.83, 25.88, 22.18, 19.41, 16.98, 15.14, 13.77, 11.99, 10.86, 10.72, 10.16, 9.71 /) )
    CALL CalcH("ethene", "74-85-1", & ! ethylene
      (/ 13.8, 17.1, 20.5, 21.1, 25.0, 30.3, 35.2, 40.0, 44.6, 49.0, 54.9, 60.2, 65.0, 72.9 /), &
      (/ 140.9, 128.8, 118.0, 115.1, 107.2, 95.5, 87.3, 81.1, 75.2, 70.5, 65.5, 62.2, 59.2, 56.1 /) )
    CALL CalcH("nitrogen", "7727-37-9", &
      (/ 12.5, 15.5, 20.3, 25.5, 30.0, 35.6, 41.6, 46.4, 51.0, 55.5, 63.8, 72.5 /), &
      (/ 18.37, 17.22, 15.89, 14.76, 13.77, 12.74, 11.97, 11.53, 11.25, 11.07, 10.72, 10.54 /) )
    CALL CalcH("oxygen", "7782-44-7", &
      (/ 12.7, 13.5, 17.3, 21.5, 30.0, 39.4, 49.3, 58.5, 65.3, 72.5, 74.9 /), &
      (/ 36.39, 35.56, 32.58, 30.13, 26.49, 23.33, 21.09, 19.77, 19.05, 18.32, 18.20 /) )
    CALL CalcH("carbon dioxide", "124-38-9", &
      (/ 13.3, 14.9, 17.6, 19.8, 25.8, 29.8, 30.3, 33.2, 34.3, 34.6, &
      42.5, 46.4, 48.4, 51.1, 54.3, 63.1, 68.2, 72.4, 74.7 /), &
      (/ 1059., 1010., 927., 873., 744.7, 662.2, 660.0, 615.2, 606.7, 599.8, &
      502.0, 465.0, 455.0, 425.6, 404.6, 349.1, 327.3, 306.9, 300.6 /) )

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, s0)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, s0
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! data need a 3-parameter fit:
      CALL HTdep_ABC(temp+CtoK, s0*alpha_TO_HscpSI*1E-6*rhoH2O_T(temp+CtoK), Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3783

  !---------------------------------------------------------------------------

  SUBROUTINE ref3784 ! 1E-3 * Bunsen coefficient [1]
    IMPLICIT NONE
    ref = "3784"
    type = "M"
    chem = "hydrogen" ; casrn = "1333-74-0"
    ! Tab. 1, average value:
    CALL MakeNoteOtherTempC(30.)
    CALL Output(16.99 * 1E-3 * alpha_TO_HscpSI)
  END SUBROUTINE ref3784

  !---------------------------------------------------------------------------

  SUBROUTINE ref3785 ! Hsxp [atm]
    IMPLICIT NONE

    ref = "3785"
    type = "M"
    ! Tab. II:
    CALL CalcH("oxygen",   "7782-44-7", (/ 25., 40., 60., 80. /), (/ 2.25, 1.90, 1.60, 1.44 /))
    CALL CalcH("hydrogen", "1333-74-0", (/ 25., 40., 60. /),      (/ 1.43, 1.29, 1.30 /))
    CALL CalcH("helium",   "7440-59-7", (/ 25., 40., 60., 80. /), (/ 0.67, 0.67, 0.72, 0.80 /))
    CALL CalcH("argon",    "7440-37-1", (/ 25., 40., 60., 80. /), (/ 2.53, 2.00, 1.68, 1.52 /))
    CALL CalcH("SF6",      "2551-62-4", (/ 25., 40., 60., 80. /), (/ 0.40, 2.74, 2.20, 1.91 /))
    CALL CalcH("methane",    "74-82-8", (/ 25., 40., 60., 80. /), (/ 2.48, 1.90, 1.62, 1.44 /))
    CALL CalcH("C(CH3)4",   "463-82-1", (/ 25., 40., 60., 80. /), (/ 1.01, 0.69, 0.48, 0.43 /))

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, Hsxp)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, Hsxp
      chem=chem_ ; casrn=casrn_ ! make global for Output
      IF ((chem_=="hydrogen").OR.(chem_=="helium").OR.(chem_=="SF6")) THEN
        CALL MakeNote(TRIM(ref)//chem_, "The data presented for " // chem_ // &
          " in Table~II of " // TRIM(citet()) // &
          " appear to be incorrect and are not reproduced here.")
        CALL Output(DUMMY)
      ELSE
        CALL HTdep_ABC(temp+CtoK, 1E-5*Hsxp*Hsxp_TO_HscpSI, Hominus, mindHR)
        CALL Output(Hominus, mindHR, r2)
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3785

  !---------------------------------------------------------------------------

  SUBROUTINE ref3786 ! Bunsen coefficient alpha [1]
    IMPLICIT NONE
    ref = "3786"
    type = "M"
    ! Tab. 1:

    chem = "hydrogen" ; casrn = "1333-74-0"
    CALL MakeNoteOtherTempC(37.)
    CALL Output(0.01629 * alpha_TO_HscpSI)

    chem = "sulfur hexafluoride" ; casrn = "2551-62-4"
    CALL MakeNoteOtherTempC(37.)
    CALL Output(0.00386 * alpha_TO_HscpSI)

  END SUBROUTINE ref3786

  !---------------------------------------------------------------------------

  SUBROUTINE ref3787 ! Bunsen coefficient alpha [1]
    IMPLICIT NONE
    ref = "3787"
    type = "M"

    ! Tab. 1:
    CALL CalcH("nitrogen",        "7727-37-9", 0.01257)
    CALL CalcH("hydrogen",        "1333-74-0", 0.01629)
    CALL CalcH("SF6",             "2551-62-4", 0.00386)
    CALL CalcH("neon",            "7440-01-9", 0.00966)
    CALL CalcH("carbon monoxide",  "630-08-0", 0.01880)
    CALL CalcH("oxygen",          "7782-44-7", 0.02385)
    CALL CalcH("carbon dioxide",   "124-38-9", 0.5485)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, alpha)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: alpha
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL MakeNoteOtherTempC(37.)
      CALL Output(alpha*alpha_TO_HscpSI)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3787

  !---------------------------------------------------------------------------

  SUBROUTINE ref3788 ! Hscc [1]
    IMPLICIT NONE

    ref = "3788"
    type = "M"
    ! Tab. 3:
    CALL CalcH("ethyl acetate",   "141-78-6", (/  100.49,  62.98,  40.89,  26.55 /))
    CALL CalcH("isoamyl acetate", "123-92-2", (/  36.097,  20.52,  10.77,   5.65 /))
    CALL CalcH("isoamyl alcohol", "123-51-3", (/ 1125.31, 593.56, 308.20, 160.03 /))

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, Hcc)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: Hcc
      REAL, DIMENSION(4) :: temp_ = (/ 30., 40., 50., 60. /) + CtoK
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp_, Hscc_TO_HscpSI(Hcc, temp_), Hominus, mindHR)
      CALL MakeNote("highTextrapol")
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3788

  !---------------------------------------------------------------------------

  SUBROUTINE ref3789 ! Bunsen coefficient [1]
    IMPLICIT NONE
    ref = "3789"
    type = "M"
    ! Tabs. I, II, III:
    CALL CalcH("helium", "7440-59-7", &
      (/ 15., 20., 30., 15., 20., 30. /), (/ 0.0089, 0.0089, 0.0086, 0.0088, 0.0087, 0.0086 /))
    CALL CalcH("neon",   "7440-01-9", &
      (/ 5., 5., 5., 10., 10., 10., 20., 20., 20., 25., 37., 37., 37., 45., 45. /), &
      (/ 0.0119, 0.0115, 0.0103, 0.0102, 0.0096, 0.0094, 0.0117, &
      0.0111, 0.0103, 0.0096, 0.0094, 0.0119, 0.0113, 0.0105, 0.0098 /))
    CALL CalcH("argon",  "7440-37-1", &
      (/ 1.65,   12.4,   18.0,   19.9,   20.0,   25.0,   30.0,   35.0,   40.0,   40.0   /), &
      (/ 0.0505, 0.0387, 0.0349, 0.0338, 0.0334, 0.0315, 0.0287, 0.0271, 0.0254, 0.0251 /))

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, beta)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, beta
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! data need a 3-parameter fit:
      CALL HTdep_ABC(temp+CtoK, beta*alpha_TO_HscpSI, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3789

  !---------------------------------------------------------------------------

  ! ref3790 NOTUSED: The data are the same as in ref0644.

  ! SUBROUTINE ref3790 ! Hvpc [Pa*m3/mol]
  !   IMPLICIT NONE

  !   ref = "3790"
  !   type = "M"
  !   ! Tab. 1:
  !   CALL CalcH("2,7-dichlorodibenzo-p-dioxin",        "33857-26-0", 5.96)
  !   CALL CalcH("1,2,4-trichlorodibenzo-p-dioxin",     "39227-58-2", 3.64)
  !   CALL CalcH("1,2,3,4-tetrachlorodibenzo-p-dioxin", "30746-58-8", 2.02)

  ! CONTAINS

  !   SUBROUTINE CalcH (chem_, casrn_, HLC)
  !     IMPLICIT NONE
  !     CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
  !     REAL, INTENT(IN)             :: HLC
  !     chem=chem_ ; casrn=casrn_ ! make global for Output
  !     CALL Output(1./HLC)
  !   END SUBROUTINE CalcH

  ! END SUBROUTINE ref3790

  !---------------------------------------------------------------------------

  SUBROUTINE ref3791 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3791"
    type = "M"
    ! Tab. 1 and Fig. 3:
    CALL CalcH("endosulfan I",    "959-98-8", 2.72E-3, -876.14)
    CALL CalcH("endosulfan II", "33213-65-9", 3.6E-4)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, HLC, slope)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: HLC
      REAL, OPTIONAL,   INTENT(IN) :: slope
      chem=chem_ ; casrn=casrn_ ! make global for Output
      IF (PRESENT(slope)) THEN
        mindHR = -(slope*LOG(10.)) + T0 ! see ref958, eqn (34) why T0 is added
        Hominus = Hvcc_TO_HscpSI(HLC, 20.+CtoK) * EXP(mindHR * (1/T0-1/(20.+CtoK)))
        CALL Output(Hominus, mindHR)
      ELSE
        ! p. 1104: "There was no clear pattern of change in HLC with
        ! temperature for endosulfan II."
        CALL MakeNoteOtherTempC(20.)
        CALL Output(Hvcc_TO_HscpSI(HLC, 20.+CtoK))
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3791

  !---------------------------------------------------------------------------

  ! ref3792 NOTUSED: General text about Henry's law. Only citations, no
  !                  new data.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3793 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3793"
    type = "M"
    ! p. 303: "The Henry's Law constants determined in the 7-23-85
    ! experiment are probably the most accurate due to experimental
    ! problems in other experiments"
    ! Appendix Tab. 1-1:
    CALL CalcH("2,4'-dichlorobiphenyl",             "34883-43-7", 0.00713) ! PCB-8
    CALL CalcH("2,4,4'-trichlorobiphenyl",           "7012-37-5", 0.00595) ! PCB-28
    CALL CalcH("2,3',4',5-tetrachlorobiphenyl",     "32598-11-1", 0.00357) ! PCB-70
    CALL CalcH("2,2',3,4,5'-pentachlorobiphenyl",   "38380-02-8", 0.00297) ! PCB-87
    CALL CalcH("2,2',3,4,4',5'-hexachlorobiphenyl", "35065-28-2", 0.00296) ! PCB-138

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, HLC)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: HLC
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL MakeNoteOtherTempC(21.)
      CALL Output(Hvcc_TO_HscpSI(HLC, 21.+CtoK))
    END SUBROUTINE CalcH

  END SUBROUTINE ref3793

  !---------------------------------------------------------------------------

  SUBROUTINE ref3794 ! Hvpx [atm]
    IMPLICIT NONE

    ref = "3794"
    type = "V"
    ! Tab. 1:
    chem = "biphenyl" ; casrn = "92-52-4"
    CALL HTdep( (/ 25., 10.  /) + CtoK, &
      Hvpx_TIMES_HscpSI / (/ 2.8E-2, 9.3E-3 /), Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)

  END SUBROUTINE ref3794

  !---------------------------------------------------------------------------

  SUBROUTINE ref3795 ! Bunsen coefficient alpha [1]
    IMPLICIT NONE
    ref = "3795"
    type = "M"
    CALL CalcH("nitrogen", "7727-37-9", (/ 25., 20., 15., 10., 5. /), &
      (/ 0.01432, 0.01621, 0.01789, 0.02003, 0.02173 /)) ! p. 730
    CALL CalcH("hydrogen", "1333-74-0", (/ 25., 20., 15., 10., 5. /), &
      (/ 0.01750, 0.01905, 0.02059, 0.02213, 0.02366 /)) ! p. 734

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, alpha)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, alpha
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! data need a 3-parameter fit:
      CALL HTdep_ABC(temp+CtoK, alpha*alpha_TO_HscpSI, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3795

  !---------------------------------------------------------------------------

  ! ref3796 NOTUSED: Only citations, no new data.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3797 ! Hvpc [atm*m3/mol]
    IMPLICIT NONE

    ref = "3797"
    type = "M"
    ! Tab. 1:
    CALL CalcH("2,2'-dichlorobiphenyl",                 "13029-08-8", 2.2 ) ! PCB-4
    CALL CalcH("2,3'-dichlorobiphenyl",                 "25569-80-6", 2.1 ) ! PCB-6
    CALL CalcH("2,4'-dichlorobiphenyl",                 "34883-43-7", 2.2 ) ! PCB-8
    CALL CalcH("2,2',6-trichlorobiphenyl",              "38444-73-4", 2.1 ) ! PCB-19
    CALL CalcH("2,2',5-trichlorobiphenyl",              "37680-65-2", 2.  ) ! PCB-18
    CALL CalcH("2,3',4-trichlorobiphenyl",              "55712-37-3", 1.6 ) ! PCB-25
    CALL CalcH("2,4',5-trichlorobiphenyl",              "16606-02-3", 2.  ) ! PCB-31
    CALL CalcH("2,3',4'-trichlorobiphenyl",             "38444-86-9", 1.5 ) ! PCB-33
    CALL CalcH("2,2',5,6'-tetrachlorobiphenyl",         "41464-41-9", 3.  ) ! PCB-53
    CALL CalcH("2,3,4'-trichlorobiphenyl",              "38444-85-8", 1.8 ) ! PCB-22
    CALL CalcH("2,2',5,5'-tetrachlorobiphenyl",         "35693-99-3", 2.2 ) ! PCB-52
    CALL CalcH("2,2',4,5'-tetrachlorobiphenyl",         "41464-40-8", 2.  ) ! PCB-49
    CALL CalcH("2,2',3,5'-tetrachlorobiphenyl",         "41464-39-5", 2.4 ) ! PCB-44
    CALL CalcH("2,3',4',5-tetrachlorobiphenyl",         "32598-11-1", 2.  ) ! PCB-70
    CALL CalcH("2,2',3,4,5'-pentachlorobiphenyl",       "38380-02-8", 3.3 ) ! PCB-87
    CALL CalcH("2,3,3',4',6-pentachlorobiphenyl",       "38380-03-9", 3.7 ) ! PCB-110
    CALL CalcH("2,2',3,3',4-pentachlorobiphenyl",       "52663-62-4", 2.  ) ! PCB-82
    CALL CalcH("2,2',3,5,5',6-hexachlorobiphenyl",      "52663-63-5", 3.  ) ! PCB-151
    CALL CalcH("2,2',3,4,5',6-hexachlorobiphenyl",      "68194-14-9", 6.  ) ! PCB-144
    CALL CalcH("2,2',3,4',5',6-hexachlorobiphenyl",     "38380-04-0", 3.  ) ! PCB-149
    CALL CalcH("2,3',4,4',5-pentachlorobiphenyl",       "31508-00-6", 4.  ) ! PCB-118
    CALL CalcH("2,2',3,3',5,6-hexachlorobiphenyl",      "52704-70-8", 5.7 ) ! PCB-134
    CALL CalcH("2,2',4,4',5,5'-hexachlorobiphenyl",     "35065-27-1", 3.5 ) ! PCB-153
    CALL CalcH("2,2',3,4,5,5'-hexachlorobiphenyl",      "52712-04-6", 4.  ) ! PCB-141
    CALL CalcH("2,2',3,4,4',5-hexachlorobiphenyl",      "35694-06-5", 4.8 ) ! PCB-137
    CALL CalcH("2,2',3,4,4',5'-hexachlorobiphenyl",     "35065-28-2", 4.8 ) ! PCB-138
    CALL CalcH("2,3,3',4,4',6-hexachlorobiphenyl",      "74472-42-7", 6.4 ) ! PCB-158
    CALL CalcH("2,2',3,4,4',5',6-heptachlorobiphenyl",  "52663-69-1", 6.3 ) ! PCB-183
    CALL CalcH("2,2',3,3',4,4'-hexachlorobiphenyl",     "38380-07-3", 5.  ) ! PCB-128
    CALL CalcH("2,2',3,3',4,5',6'-heptachlorobiphenyl", "52663-70-4", 22. ) ! PCB-177
    CALL CalcH("2,3,3',4,4',5-hexachlorobiphenyl",      "38380-08-4", 8.7 ) ! PCB-156
    CALL CalcH("2,3,3',4,4',5'-hexachlorobiphenyl",     "69782-90-7", 5.8 ) ! PCB-157
    ! Tab. 2 is not used because solution contains a mix of other compounds.

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, HLC)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL, INTENT(IN)             :: HLC
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL MakeNote("atroomT") ! p. 7: "the temperature of the water was not carefully controlled"
      CALL Output(1E4/(atm*HLC))
    END SUBROUTINE CalcH

  END SUBROUTINE ref3797

  !---------------------------------------------------------------------------

  SUBROUTINE ref3799 ! Hscp [M/atm]
    IMPLICIT NONE

    ref = "3799"
    type = "M"

    ! Tab. 1:
    CALL CalcH("methane",     "74-82-8", (/  5., 25., 45. /), (/ 2.19, 1.41, 1.07 /))
    CALL CalcH("ethane",      "74-84-0", (/  5., 25., 45. /), (/ 3.61, 1.86, 1.25 /))
    CALL CalcH("propane",     "74-98-6", (/  5., 25., 45. /), (/ 3.14, 1.47, 0.95 /))
    CALL CalcH("butane",     "106-97-8", (/  5., 25., 45. /), (/ 2.85, 1.16, 0.69 /))
    CALL CalcH("isobutane",   "75-28-5", (/  5., 25., 45. /), (/ 1.99, 0.94, 0.58 /))
    CALL CalcH("neopentane", "463-82-1", (/ 15., 35., 45. /), (/ 0.95, 0.42, 0.34 /))
    ! toluene is not used, see footnote b of Tab. 1
    CALL CalcH("skatole",     "83-34-1", (/ 2., 13., 25., 37., 50. /), &
      (/ 1.57, 2.14, 2.24, 4.53, 5.75 /))

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp_, HLC)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp_, HLC
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp_+CtoK, Hscp_TO_HscpSI*1E-3*HLC, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3799

  !---------------------------------------------------------------------------

  SUBROUTINE ref3801 ! Hscc [1] = Ostwald coefficient
    IMPLICIT NONE

    ref = "3801"
    type = "M"
    ! Tab. I:
    CALL CalcH("helium", "7440-59-7", (/ 24.85 /),            (/ 0.0093 /))
    CALL CalcH("argon",  "7440-37-1", (/ 24.85, 14.8 /),      (/ 0.0340, 0.0405 /))
    CALL CalcH("SF6",    "2551-62-4", (/ 24.85, 14.9, 0.1 /), (/ 0.0055, 0.0076, 0.0147 /))

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, ostwald)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, ostwald
      chem=chem_ ; casrn=casrn_ ! make global for Output
      IF (SIZE(temp)>1) THEN
        CALL HTdep(temp+CtoK, Hscc_TO_HscpSI(ostwald, temp+CtoK), Hominus, mindHR)
        CALL Output(Hominus, mindHR, r2)
      ELSE
        CALL Output(Hscc_TO_HscpSI_atT0*ostwald(1))
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3801

  !---------------------------------------------------------------------------

  SUBROUTINE ref3806 ! Kuenen coefficient [cm3/g]
    IMPLICIT NONE

    ref = "3806"
    type = "M"
    ! Tab. 1:
    CALL CalcH("nitrogen trifluoride",      "7783-54-2", &
      (/ 298.21, 298.26, 298.56, 308.29, 318.28 /), &
      (/ 17.76,   17.25,  17.60,  14.82,  12.95 /) ) ! NF3
    ! Tab. 2:
    CALL CalcH("dinitrogen tetrafluoride", "10036-47-2", &
      (/ 288.15, 289.15, 289.15, 289.97, 298.11, 298.11, 298.11, 298.16, &
         298.16, 298.16, 298.26, 308.09, 308.09, 318.18, 318.18, 318.18 /), &
      (/  26.42,  24.77,  25.53,  24.69,  19.07,  19.21,  19.21,  19.10, &
          19.32,  19.55,  19.08,  14.88,  15.11,  12.55,  12.55,  12.61 /) ) ! N2F4

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, T, k)
      IMPLICIT NONE
      CHARACTER(LEN=*),           INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:),         INTENT(IN) :: T, k
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep_ABC(T, 1E-3*k*S_TO_HscpSI, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3806

  !---------------------------------------------------------------------------

  SUBROUTINE ref3807 ! special definition
    IMPLICIT NONE
    ref = "3807"
    type = "M"
    chem = "dichlorine monoxide" ; casrn = "7791-21-1" ! Cl2O
    ! data from this paper are cited by ref3857, p. 451-452
    ! see also: SUBROUTINE ref_thiswork_Cl2O
    CALL MakeNote(TRIM(ref), TRIM(citet()) // " measured the gas--aqueous " // &
      "equilibrium constant $p(\chem{Cl_2O}) / c(\chem{HOCl})^2$ " // &
      "but not the intrinsic Henry's law constant of \chem{Cl_2O}.")
    CALL Output(DUMMY)
  END SUBROUTINE ref3807

  !---------------------------------------------------------------------------

  SUBROUTINE ref3808 ! Hscp [M/atm]
    IMPLICIT NONE
    ref = "3808"
    type = "M"
    ! Tabs. I+II:
    CALL CalcH("selenium hydride", "7783-07-5", & ! H2Se
      (/ 25.1, 26.6, 28.0, 29.5, 31.1, 33.0, 34.7, 36.7, 39.0, 41.0, &     ! Tab. I
         43.0, 45.0, 47.0, 49.0, 51.0, 53.0, 55.0, 57.4, 59.0, 61.0, &     ! Tab. I
         25.1, 26.6, 28.0, 29.5, 31.1, 33.0, 34.7, 36.7, 39.0, 41.0, &     ! Tab. I
         43.0, 45.0, 47.0, 49.0, 51.0, 53.0, 55.0, 57.4, 59.0, 61.0, &     ! Tab. I
         63.0, 63.0, 65.0, 65.0, 65.0, 66.0, 66.0, 66.0, 70.0 /) + CtoK, & ! Tab. II
      (/ 0.08347, 0.08062, 0.07943, 0.07709, 0.0757,  0.07091, 0.06913, 0.06647, & ! Tab. I
         0.06338, 0.06097, 0.05922, 0.05833, 0.05544, 0.05361, 0.05249, 0.05079, & ! Tab. I
         0.05012, 0.04672, 0.04587, 0.04465, 0.08289, 0.08031, 0.07909, 0.07675, & ! Tab. I
         0.07533, 0.07133, 0.06915, 0.06613, 0.06331, 0.06141, 0.06033, 0.05643, & ! Tab. I
         0.05504, 0.05297, 0.05161, 0.05073, 0.0487,  0.04744, 0.04384, 0.04299, & ! Tab. I
         0.04397, 0.04197, &          ! Tab. II
         0.04377, 0.04214, 0.04255, & ! Tab. II
         0.04228, 0.04269, 0.0440,  & ! Tab. II
         0.04017 /) )                 ! Tab. II

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, T, Solubility)
      IMPLICIT NONE
      CHARACTER(LEN=*),           INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:),         INTENT(IN) :: T, Solubility
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep_ABC(T, Hscp_TO_HscpSI*Solubility, Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3808

  !---------------------------------------------------------------------------

  SUBROUTINE ref3809 ! Hvpb [mmHg/(mol/kg)]
    IMPLICIT NONE

    ref = "3809"
    type = "M"
    !-----
    chem = "hypochlorous acid" ; casrn = "7790-92-3" ! HOCl
    ALLOCATE(temp(2))
    temp = (/ 10., 20. /) + CtoK
    ! K2 on p. 1309:
    CALL HTdep(temp, (rhoH2O/mmHg) / (/ 0.24, 0.70 /), Hominus, mindHR)
    DEALLOCATE(temp)
    CALL Output(Hominus, mindHR, r2)
    !-----
    chem = "dichlorine monoxide" ; casrn = "7791-21-1" ! Cl2O
    ! data from this paper are cited by ref3857, p. 450
    ! see also: SUBROUTINE ref_thiswork_Cl2O
    CALL MakeNote(TRIM(ref), TRIM(citet()) // " measured the gas--aqueous " // &
      "equilibrium constant $p(\chem{Cl_2O}) / c(\chem{HOCl})^2$ " // &
      "but not the intrinsic Henry's law constant of \chem{Cl_2O}.")
    CALL Output(DUMMY)
    !-----

  END SUBROUTINE ref3809

  !---------------------------------------------------------------------------

  SUBROUTINE ref3811 ! Hvpx [atm]
    IMPLICIT NONE

    ref = "3811"
    type = "?"

    CALL CalcH("argon",                  "7440-37-1", 65.3235,    -3.24690E3, -2.01398E1,   0.,           39696.1   )
    CALL CalcH("tetrafluoromethane",       "75-73-0", 1507.2737,  -4.19725E4, -5.99350E2,   4.09087E-01, 259777.3   )
    CALL CalcH("methane",                  "74-82-8", 146.8858,   -5.76834E3, -5.19144E1,   1.84936E-02,  39240.1   )
    CALL CalcH("carbon monoxide",         "630-08-0", 74.5962,    -3.60330E3, -2.33376E1,   0.,           57978.5   )
    CALL CalcH("carbon oxide sulfide",    "463-58-1", 96.0704,    -5.22240E3, -3.03658E1,   0.,            2607.5   )
    CALL CalcH("carbon dioxide",          "124-38-9", 69.4237,    -3.79646E3, -2.16694E1,   4.78857E-04,   1635.2   )
    CALL CalcH("ethyne",                   "74-86-2", 67.9714,    -3.54393E3, -2.14030E1,   0.,            1332.6   )
    CALL CalcH("chloroethene",             "75-01-4", 61.1868,    -3.43853E3, -1.90984E1,   2.51906E-03,   1403.3   )
    CALL CalcH("ethene",                   "74-85-1", 107.7298,   -6.33990E3, -3.17169E1,  -1.32533E-02,  10784.4   )
    CALL CalcH("ethane",                   "74-84-0", 108.9263,   -5.51363E3, -3.47413E1,   0.,           29402.4   )
    CALL CalcH("cyclopropane",             "75-19-4", -141.9730,   5.87466E3,  5.09015E1,   0.,            4820.7   )
    CALL CalcH("propene",                 "115-07-1", -2570.0227,  6.86740E4,  1.03384E3,  -7.19694E-01,   8182.3   )
    CALL CalcH("propane",                  "74-98-6", 2874.1130,  -8.56732E4, -1.12809E3,   7.01580E-01,  35420.1   )
    !CALL CalcH("1-butene",               "106-98-9", 12.9593,    -2.78226E3,  0.,          0.,             -----   ) only high-T
    CALL CalcH("2-methylpropene",         "115-11-7", 103.9814,   -5.54785E3, -3.28909E1,   0.,            9714.4   )
    CALL CalcH("butane",                  "106-97-8", 121.8305,   -6.34244E3, -3.87599E1,   0.,           44563.3   )
    CALL CalcH("2-methylpropane",          "75-28-5", 161.2644,   -7.94950E3, -5.24651E1,   0.,           60280.3   )
    CALL CalcH("chlorine dioxide",      "10049-04-4", -24.6413,   -6.21813E1,  1.07454E1,   0.,              54.814 )
    CALL CalcH("chlorine",               "7782-50-5", 232.4396,   -8.21980E3, -8.69997E1,   4.42155E-02,    599.8   )
    CALL CalcH("hydrogen",               "1333-74-0", 54.6946,    -2.40098E3, -1.68893E1,   0.,           70832.1   )
    CALL CalcH("hydrogen sulfide",       "7783-06-4", 10.8191,    -1.51009E3, -3.99300E-1, -6.81842E-03,    541.18  )
    CALL CalcH("helium",                 "7440-59-7", 46.0252,    -1.84993E3, -1.40094E1,   0.,          142950.4   )
    CALL CalcH("krypton",                "7439-90-9", 77.5359,    -3.95280E3, -2.42207E1,   0.,           22161.3   )
    CALL CalcH("nitrogen monoxide",     "10102-43-9", 30.2512,    -2.42215E3, -5.70490E0,  -1.19149E-02,  28739.1   )
    CALL CalcH("nitrogen",               "7727-37-9", 78.8622,    -3.74498E3, -2.47981E1,   0.,           87143.1   )
    CALL CalcH("dinitrogen monoxide",   "10024-97-2", 68.8882,    -3.85775E3, -2.12530E1,   0.,            2291.4   )
    CALL CalcH("neon",                   "7440-01-9", 60.7869,    -2.65134E3, -1.89157E1,   0.,          122626.8   )
    CALL CalcH("oxygen",                 "7782-44-7", 77.8881,    -3.79901E3, -2.44526E1,   0.,           43630.9   )
    CALL CalcH("phosphorus trihydride",  "7803-51-2", 67.5831,    -3.57648E3, -2.09165E1,   0.,            6776.6   )
    CALL CalcH("radon",                 "10043-92-2", 109.3341,   -5.64696E3, -3.50047E1,   0.,            5987.3   )
    CALL CalcH("sulfur hexafluoride",    "2551-62-4", 191.8514,   -9.19008E3, -6.29116E1,   0.,          227521.1   )
    CALL CalcH("sulfur dioxide",         "7446-09-5", 22.3423,    -1.98711E3, -5.68540E0,   0.,              40.674 )
    CALL CalcH("xenon",                  "7440-63-3", 87.3918,    -4.56921E3, -2.74664E1,   0.,           12669.1   )

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, A, B, C, D, H25)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: A, B, C, D, H25
      chem=chem_ ; casrn=casrn_ ! make global for Output
      IF (TRIM(casrn)=="7782-50-5") THEN ! Cl2
        CALL MakeNote("HeffCl2")
        CALL Output(DUMMY)
        RETURN
      ENDIF
      ! The temperature dependence is:
      ! lg(Hvpx) = A + B/T + C*lg(T) + DT
      Hominus = 10.**(A + B/T0 + C*LOG10(T0) + D*T0)
      CALL consistency_check(Hominus, H25, "H(A,B,C,D) and H25") ! , verbose=.TRUE.
      Hominus = Hvpx_TIMES_HscpSI / Hominus ! convert [atm] to [mol/(m3*Pa)]
      ! The analytical derivative is:
      mindHR = -( B - C*T0/LOG(10.) - D*T0**2 ) * LOG(10.) ! d(lnH)/d(1/T) = -delta H/R
      CALL MakeNote("whichref")
      CALL Output(Hominus, mindHR)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3811

  !---------------------------------------------------------------------------

  SUBROUTINE ref3812 ! Hvpx [atm] and Hvpc [atm*m3/mol]
    IMPLICIT NONE
    ref = "3812"

    CALL CalcH("methanethiol",                    "74-93-1", 1,  225.69, 4.0625E-03)
    CALL CalcH("ethanethiol",                     "75-08-1", 1,  158.37, 2.8506E-03)
    CALL CalcH("dimethyl sulfide",                "75-18-3", 1,   99.58, 1.7923E-03)
    CALL CalcH("1-propanethiol",                 "107-03-9", 2,  231.75, 4.1716E-03)
    CALL CalcH("2-propanethiol",                  "75-33-2", 2,  248.29, 4.4691E-03)
    CALL CalcH("ethyl methyl sulfide",           "624-89-5", 2,  101.50, 1.8270E-03)
    CALL CalcH("thiophene",                      "110-02-1", 1,  159.64, 2.8735E-03)
    CALL CalcH("tetrahydrothiophene",            "110-01-0", 2,  247.33, 4.4519E-03)
    CALL CalcH("1-butanethiol",                  "109-79-5", 1,  499.26, 8.9867E-03)
    CALL CalcH("2-methyl-1-propanethiol",        "513-44-0", 2,  282.97, 5.0934E-03)
    CALL CalcH("2-butanethiol",                  "513-53-1", 2,  282.33, 5.0820E-03)
    CALL CalcH("2-methyl-2-propanethiol",         "75-66-1", 2,  284.60, 5.1228E-03)
    CALL CalcH("diethyl sulfide",                "352-93-2", 1,   47.78, 8.6008E-04)
    CALL CalcH("methyl propyl sulfide",         "3877-15-4", 2,  121.82, 2.1927E-03)
    CALL CalcH("methyl isopropyl sulfide",      "1551-21-9", 2,  121.10, 2.1798E-03)
    CALL CalcH("2-methylthiophene",              "554-14-3", 2,  245.92, 4.4265E-03)
    CALL CalcH("3-methylthiophene",              "616-44-4", 1,  397.19, 7.1494E-03)
    CALL CalcH("1-pentanethiol",                 "110-66-7", 2,  385.67, 6.9419E-03)
    CALL CalcH("2-pentanethiol",                "2084-19-7", 2,  360.21, 6.4836E-03)
    CALL CalcH("3-pentanethiol",                 "616-31-9", 2,  368.19, 6.6274E-03)
    CALL CalcH("2-methyl-1-butanethiol",        "1878-18-8", 2,  374.92, 6.7485E-03)
    CALL CalcH("3-methyl-1-butanethiol",         "541-31-1", 2,  370.90, 6.6761E-03)
    CALL CalcH("2-methyl-2-butanethiol",        "1679-09-0", 2,  357.91, 6.4424E-03)
    CALL CalcH("3-methyl-2-butanethiol",        "2084-18-6", 2,  363.23, 6.5380E-03)
    CALL CalcH("2,2-dimethyl-1-propanethiol",   "1679-08-9", 2,  354.33, 6.3779E-03)
    CALL CalcH("methyl butyl sulfide",           "628-29-5", 2,  160.78, 2.8940E-03)
    CALL CalcH("ethyl propyl sulfide",          "4110-50-3", 2,  155.39, 2.7970E-03)
    CALL CalcH("3-methyl-2-thiapentane",       "10359-64-5", 2,  153.39, 2.7610E-03)
    CALL CalcH("4-methyl-2-thiapentane",        "5008-69-5", 2,  155.18, 2.7931E-03)
    CALL CalcH("2-methyl-3-thiapentane",        "5145-99-3", 2,  148.41, 2.6714E-03)
    CALL CalcH("3,3-dimethyl-2-thiabutane",     "6163-64-0", 2,  150.44, 2.7079E-03)
    CALL CalcH("2-ethylthiophene",               "872-55-9", 1,  281.54, 5.0677E-03)
    CALL CalcH("2,5-dimethylthiophene",          "638-02-8", 2,  338.41, 6.0914E-03)
    CALL CalcH("3-ethylthiophene",              "1795-01-3", 2,  335.76, 6.0436E-03)
    CALL CalcH("2,3-dimethylthiophene",          "632-16-6", 2,  356.12, 6.4102E-03)
    CALL CalcH("2,4-dimethylthiophene",          "638-00-6", 2,  353.10, 6.3558E-03)
    CALL CalcH("3,4-dimethylthiophene",          "632-15-5", 2,  369.13, 6.6442E-03)
    CALL CalcH("1-hexanethiol",                  "111-31-9", 2,  519.36, 9.3484E-03)
    CALL CalcH("2-hexanethiol",                 "1679-06-7", 2,  461.36, 8.3044E-03)
    CALL CalcH("dipropyl sulfide",               "111-47-7", 2,  204.07, 3.6733E-03)
    CALL CalcH("di-(2-propyl)-sulfide",          "625-80-9", 2,  190.03, 3.4206E-03)
    CALL CalcH("methyl pentyl sulfide",         "1741-83-9", 2,  143.22, 2.5779E-03)
    CALL CalcH("ethyl butyl sulfide",            "638-46-0", 2,  206.78, 3.7219E-03)
    CALL CalcH("3-methyl-2-thiahexane",        "13286-91-4", 2,  198.53, 3.5735E-03)
    CALL CalcH("4-methyl-2-thiahexane",        "15013-37-3", 2,  200.92, 3.6165E-03)
    CALL CalcH("5-methyl-2-thiahexane",        "13286-90-3", 2,  196.17, 3.5311E-03)
    CALL CalcH("2-methyl-3-thiahexane",         "5008-73-1", 2,  198.37, 3.5706E-03)
    CALL CalcH("4-methyl-3-thiahexane",         "5008-72-0", 2,  204.57, 3.6823E-03)
    CALL CalcH("5-methyl-3-thiahexane",         "1613-45-2", 2,  203.13, 3.6562E-03)
    CALL CalcH("3-ethyl-2-thiapentane",        "57093-84-2", 2,  183.25, 3.2985E-03)
    CALL CalcH("3,3-dimethyl-2-thiapentane",   "13286-92-5", 2,  200.99, 3.6177E-03)
    CALL CalcH("3,4-dimethyl-2-thiapentane",   "53897-51-1", 2,  189.45, 3.4100E-03)
    CALL CalcH("4,4-dimethyl-2-thiapentane",    "6079-57-8", 2,  200.99, 3.6177E-03)
    CALL CalcH("2,2-dimethyl-3-thiapentane",   "14290-92-7", 2,  191.84, 3.4530E-03)
    CALL CalcH("2-propylthiophene",             "1551-27-5", 2,  449.89, 8.0979E-03)
    CALL CalcH("3-propylthiophene",             "1518-75-8", 2,  462.72, 8.3288E-03)
    CALL CalcH("2-isopropylthiophene",          "4095-22-1", 2,  423.46, 7.6222E-03)
    CALL CalcH("3-isopropylthiophene",         "29488-27-5", 2,  475.17, 8.5529E-03)
    CALL CalcH("2-methyl-3-ethylthiophene",    "53119-51-0", 2,  447.61, 8.0570E-03)
    CALL CalcH("2-methyl-4-ethyl thiophene",   "13678-54-1", 2,  479.06, 8.6230E-03)
    CALL CalcH("2-methyl-5-ethyl thiophene",   "40323-88-4", 2,  463.45, 8.3421E-03)
    CALL CalcH("3-methyl-2-ethylthiophene",    "31805-48-8", 2,  468.49, 8.4328E-03)
    CALL CalcH("3-methyl-4-ethyl thiophene",   "66577-03-5", 2,  433.99, 7.8118E-03)
    CALL CalcH("3-methyl-5-ethylthiophene",    "66577-04-6", 2,  433.99, 7.8118E-03)
    CALL CalcH("2,3,4-trimethylthiophene",      "1795-04-6", 2,  421.60, 7.5887E-03)
    CALL CalcH("2,3,5-trimethylthiophene",      "1795-05-7", 2,  493.56, 8.8841E-03)
    CALL CalcH("1-heptanethiol",                "1639-09-4", 2,  571.93, 1.0295E-02)
    CALL CalcH("2-heptanethiol",                 "628-00-2", 2,  409.47, 7.3704E-03)
    CALL CalcH("methyl hexyl sulfide",         "20291-60-5", 2,  530.18, 9.5431E-03)
    CALL CalcH("ethyl pentyl sulfide",         "26158-99-6", 2,  530.18, 9.5431E-03)
    CALL CalcH("propyl butyl sulfide",          "1613-46-3", 2,  530.18, 9.5431E-03)
    CALL CalcH("1-octanethiol",                  "111-88-6", 2,  770.59, 1.3871E-02)
    CALL CalcH("2-octanethiol",                 "3001-66-9", 2,  552.54, 9.9456E-03)
    CALL CalcH("tert-octanethiol",               "141-59-3", 2,  270.30, 4.8654E-03)
    CALL CalcH("dibutyl sulfide",                "544-40-1", 2,  434.57, 7.8221E-03)
    CALL CalcH("methyl heptyl sulfide",        "20291-61-6", 2,  882.50, 1.5885E-02)
    CALL CalcH("ethyl hexyl sulfide",           "7309-44-6", 2,  882.50, 1.5885E-02)
    CALL CalcH("propyl pentyl sulfide",        "42841-80-5", 2,  882.50, 1.5885E-02)
    CALL CalcH("1-nonanethiol",                 "1455-21-6", 2,  962.10, 1.7318E-02)
    CALL CalcH("2-nonanethiol",                "13281-11-3", 2,  711.01, 1.2798E-02)
    CALL CalcH("methyl octyl sulfide",          "3698-95-1", 2,  559.11, 1.0064E-02)
    CALL CalcH("ethyl heptyl sulfide",         "24768-44-3", 2,  559.11, 1.0064E-02)
    CALL CalcH("propyl hexyl sulfide",         "24768-43-2", 2,  559.11, 1.0064E-02)
    CALL CalcH("butyl pentyl sulfide",         "24768-42-1", 2,  559.11, 1.0064E-02)
    CALL CalcH("1-decanethiol",                  "143-10-2", 2, 1091.38, 1.9645E-02)
    CALL CalcH("2-decanethiol",                "13402-60-3", 2,  834.95, 1.5029E-02)
    CALL CalcH("dipentyl sulfide",               "872-10-6", 2,  331.34, 5.9640E-03)
    CALL CalcH("methyl nonyl sulfide",         "59973-07-8", 2,  331.34, 5.9640E-03)
    CALL CalcH("ethyl octyl sulfide",           "3698-94-0", 2,  331.34, 5.9640E-03)
    CALL CalcH("propyl heptyl sulfide",        "24768-46-5", 2,  331.34, 5.9640E-03)
    CALL CalcH("butyl hexyl sulfide",          "16967-04-7", 2,  331.34, 5.9640E-03)
    CALL CalcH("1-undecanethiol",               "5332-52-5", 2, 1117.18, 2.0109E-02)
    CALL CalcH("2-undecanethiol",              "62155-02-6", 2,  893.98, 1.6091E-02)
    CALL CalcH("methyl decyl sulfide",         "22438-39-7", 2,  314.05, 5.6528E-03)
    CALL CalcH("ethyl nonyl sulfide",          "59973-08-9", 2,  314.05, 5.6528E-03)
    CALL CalcH("propyl octyl sulfide",          "3698-93-9", 2,  314.05, 5.6528E-03)
    CALL CalcH("butyl heptyl sulfide",         "40813-84-1", 2,  314.05, 5.6528E-03)
    CALL CalcH("1-dodecanethiol",                "112-55-0", 2, 1030.93, 1.8557E-02)
    CALL CalcH("2-dodecanethiol",              "14402-50-7", 2,  864.38, 1.5559E-02)
    CALL CalcH("dihexyl sulfide",               "6294-31-1", 2,  352.18, 6.3392E-03)
    CALL CalcH("methyl undecyl sulfide",        "7289-44-3", 2,  352.18, 6.3392E-03)
    CALL CalcH("ethyl decyl sulfide",          "19313-61-2", 2,  352.18, 6.3392E-03)
    CALL CalcH("propyl nonyl sulfide",         "62103-66-6", 2,  352.18, 6.3392E-03)
    CALL CalcH("butyl octyl sulfide",          "16900-07-5", 2,  352.18, 6.3392E-03)
    CALL CalcH("1-tridecanethiol",             "19484-26-5", 2,  856.30, 1.5413E-02)
    CALL CalcH("2-tridecanethiol",             "62155-03-7", 2,  751.83, 1.3533E-02)
    CALL CalcH("methyl dodecyl sulfide",        "3698-89-3", 2,  305.63, 5.5013E-03)
    CALL CalcH("ethyl undecyl sulfide",        "66577-30-8", 2,  305.63, 5.5013E-03)
    CALL CalcH("propyl decyl sulfide",         "66577-31-9", 2,  305.63, 5.5013E-03)
    CALL CalcH("butyl nonyl sulfide",          "66577-32-0", 2,  305.63, 5.5013E-03)
    CALL CalcH("1-tetradecanethiol",            "2079-95-0", 2,  642.62, 1.1567E-02)
    CALL CalcH("2-tetradecanethiol",           "62155-04-8", 2,  590.82, 1.0635E-02)
    CALL CalcH("diheptyl sulfide",               "629-65-2", 2,  210.64, 3.7915E-03)
    CALL CalcH("methyl tridecyl sulfide",      "62155-09-3", 2,  210.64, 3.7915E-03)
    CALL CalcH("ethyl dodecyl sulfide",         "2851-83-4", 2,  210.64, 3.7915E-03)
    CALL CalcH("propyl undecyl sulfide",       "66826-84-4", 2,  210.64, 3.7915E-03)
    CALL CalcH("butyl decyl sulfide",          "19313-57-6", 2,  210.64, 3.7915E-03)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, code, Hvpx, Hvpc)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      INTEGER,          INTENT(IN) :: code
      REAL,             INTENT(IN) :: Hvpx, Hvpc
      chem=chem_ ; casrn=casrn_ ! make global for Output
      IF (code==1) THEN
        type = "?"
        CALL MakeNote("whichref")
      ELSE
        type = "V"
        CALL MakeNote(TRIM(ref), &
          TRIM(citet())//" present Henry's law constants based on water solubility "// &
          "and vapor pressure. The water solubility is calculated using a correlation "// &
          "to the boiling point. For the vapor pressures, no references are provided.")
      ENDIF
      Hominus = Hvpx_TIMES_HscpSI/Hvpx
      ! test if the data in the two columns are consistent:
      CALL consistency_check(Hominus, 1./(atm*Hvpc), "Hvpx and Hvpc") ! , verbose=.TRUE.
      CALL Output(Hominus)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3812

  !---------------------------------------------------------------------------

  ! ref3813 NOTUSED: Only citations (data from ref0922, ref0923 and
  !                  ref2013), no new data.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3849 ! Hsxp [1/atm]
    IMPLICIT NONE
    ref = "3849"
    type = "L"
    CALL CalcH_IUPAC_lnx1("helium", "7440-59-7", -41.4611, 42.5962, 14.0094) ! p. 1
    CALL CalcH_IUPAC_lnx1("neon",   "7440-01-9", -52.8573, 61.0494, 18.9157) ! p. 124
  END SUBROUTINE ref3849

  !---------------------------------------------------------------------------

  SUBROUTINE ref3850 ! Hsxp [1/atm]
    IMPLICIT NONE
    ref = "3850"
    type = "L"
    CALL CalcH_IUPAC_lnx1("krypton",  "7439-90-9", -66.9928, 91.0166, 24.2207) ! p. 1
    CALL CalcH_IUPAC_lnx1("xenon",    "7440-63-3", -74.7398, 105.210, 27.4664) ! p. 134
    CALL CalcH_IUPAC_lnx1("radon",   "10043-92-2", -90.5481, 130.026, 35.0047) ! p. 227
  END SUBROUTINE ref3850

  !---------------------------------------------------------------------------

  SUBROUTINE ref3851 ! Hsxp [1/atm]
    IMPLICIT NONE
    ref = "3851"
    type = "L"
    CALL CalcH_IUPAC_lnx1("argon", "7440-37-1", -57.6661, 74.7627, 20.1398) ! p. 1
  END SUBROUTINE ref3851

  !---------------------------------------------------------------------------

  SUBROUTINE ref3852 ! Hsxp [1/atm]
    IMPLICIT NONE
    ref = "3852"
    type = "L"
    ! H2, p. 1:
    CALL CalcH_IUPAC_lnx1("hydrogen", "1333-74-0", -48.1611, 55.2845, 16.8893)
    ! D2, p. 279:
    chem = "D2" ; casrn = "7782-39-0"
    CALL MakeNote(TRIM(ref), &
      TRIM(citet()) // " recommend the data from \citet{3926}.")
    CALL Output(DUMMY)
  END SUBROUTINE ref3852

  !---------------------------------------------------------------------------

  SUBROUTINE ref3853 ! Hsxp [1/atm]
    IMPLICIT NONE
    ref = "3853"
    type = "L"

    CALL CalcH_IUPAC_lnx1("oxygen", "7782-44-7", -66.73538, 87.47547, 24.45264) ! p. 1

    chem = "ozone" ; casrn = "10028-15-6" ! p. 480
    CALL MakeNote(TRIM(ref)//"_O3", TRIM(citet()) // " concludes that ozone " // &
      "aqueous chemistry needs further clarification. Data from \citet{2549} " // &
      "are recommended, in spite of limitations and assumptions of the data.")
    CALL Output(DUMMY)
  END SUBROUTINE ref3853

  !---------------------------------------------------------------------------

  SUBROUTINE ref3854 ! Hsxp [1/atm]
    IMPLICIT NONE
    ref = "3854"
    type = "L"
    CALL CalcH_IUPAC_lnx1("N2O", "10024-97-2", -60.7467, 88.8280, 21.2531) ! p. 1
    CALL CalcH_IUPAC_lnx1("NO",  "10102-43-9", -62.8086, 82.3420, 22.8155, &
      "A minus sign is missing in the fitting parameter presented by " &
      //TRIM(citet())// ". It should be $-62.8086$, not 62.8086.") ! p. 260
  END SUBROUTINE ref3854

  !---------------------------------------------------------------------------

  SUBROUTINE ref3855 ! Hsxp [1/atm]
    IMPLICIT NONE
    ref = "3855"
    type = "L"
    CALL CalcH_IUPAC_lnx1("ethane", "74-84-0", -90.82250, 126.9559, 34.74128) ! p. 2
  END SUBROUTINE ref3855

  !---------------------------------------------------------------------------

  SUBROUTINE ref3856 ! Hsxp [1/atm]
    IMPLICIT NONE
    ref = "3856"
    type = "L"
    CALL CalcH_IUPAC_lnx1("nitrogen", "7727-37-9", -67.38765, 86.32129, 24.79808) ! p. 1
  END SUBROUTINE ref3856

  !---------------------------------------------------------------------------

  SUBROUTINE ref3857 ! Hsxp [1/atm]
    IMPLICIT NONE
    ref = "3857"
    type = "L"

    CALL CalcH_IUPAC_lnx1("SO2",   "7446-09-5", -25.26286, 45.75521, 5.685450) ! p. 3
    CALL CalcH_IUPAC_lnx1("ClO2", "10049-04-4",   7.91625, 0.479133, -11.0593) ! p. 454
    ! --------------------------------------------
    chem = "Cl2" ; casrn = "7782-50-5" ! p. 333
    ! CALL HTdep_ABC( &
    !   (/ 283.15, 293.15, 303.15, 313.15, 323.15, 333.15, 343.15, 353.15, &
    !   363.15, 373.15, 383.15 /), &
    !   (/ 2.48, 1.88, 1.50, 1.23, 1.06, 0.939, 0.849, &
    !   0.784, 0.737, 0.697, 0.668 /) * 1E-3*Hsxp_TO_HscpSI, Hominus, mindHR)
    ! ref3857.py shows how data from Table 1 of ref3925 were converted to x1
    CALL MakeNote(TRIM(ref)//"_Cl2", TRIM(citet()) // &
      " recommends values calculated from Table~1 of \citet{3925}. " // &
      "Thus, the data refer to effective values that take into account the " // &
      "hydrolysis in the aqueous phase: " // &
      "$$H_\mathrm{s,eff} = ([\chem{Cl_2}]+[\chem{HOCl}])/p(\chem{Cl_2}).$$ " // &
      "In addition, the values apply to a partial pressure of " // &
      "$p(\chem{Cl_2})\,= 101325$\,Pa, and not to infinite dilution.")
    CALL Output(DUMMY)
    ! --------------------------------------------
    chem = "dichlorine monoxide" ; casrn = "7791-21-1" ! Cl2O, p. 452
    type = "?"
    CALL MakeNote(TRIM(ref)//"_Cl2O", TRIM(citet()) // &
      " cites data from \citet{3807}. However, that paper only " // &
      "describes the equilibrium between gas-phase \chem{Cl_2O} and aqueous-phase " // &
      "\chem{HOCl}. A Henry's law constant of \chem{Cl_2O} is not provided. In " // &
      "addition, the values given by " // TRIM(citet()) // " are not extrapolated " // &
      "to infinite dilution but to 1\,atm partial pressure of \chem{Cl_2O}. It is " // &
      "not explained how the nonlinear pressure dependence was extrapolated to 1\,atm.")
    ! ref3857.py shows the interconversion between x1, L and S shown on p. 452
    CALL Output(DUMMY)
    ! --------------------------------------------

  END SUBROUTINE ref3857

  !---------------------------------------------------------------------------

  ! ref3858 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3859 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3860 ! Hsxp [1/atm]
    IMPLICIT NONE
    ref = "3860"
    type = "L"
    CALL CalcH_IUPAC_lnx1("propane",         "74-98-6", -102.044, 144.345, 39.4740) ! p. 1
    CALL CalcH_IUPAC_lnx1("butane",         "106-97-8", -102.029, 146.040, 38.7599) ! p. 16
    CALL CalcH_IUPAC_lnx1("2-methylpropane", "75-28-5", -129.714, 183.044, 52.4651, &
      "The fitting function and the data in the table on page 34 of " //TRIM(citet())// &
      " are inconsistent by a factor of about 3. A comparison with the" // &
      " original measurements by \citet{3799} shows that the data in the table" // &
      " are correct. Refitting the data suggests that the third fitting parameter" // &
      " should be 52.4651, not 53.4651." ) ! p. 34
  END SUBROUTINE ref3860

  !---------------------------------------------------------------------------

  SUBROUTINE ref3861 ! Hsxp [1/atm]
    IMPLICIT NONE
    ref = "3861"
    type = "L"
    ! The "best evaluation of data prior to 1981" is not used here:
    ! CALL CalcH_IUPAC_lnx1("methane", "74-82-8", -78.1584, 104.4791, 29.7802) ! p. 1
    chem = "methane" ; casrn = "74-82-8"
    CALL MakeNote(TRIM(ref), &
      TRIM(citet()) // " recommend the data from \citet{491}.")
    CALL Output(DUMMY)
  END SUBROUTINE ref3861

  !---------------------------------------------------------------------------

  SUBROUTINE ref3862 ! Hvpb [kPa*kg/mol]
    IMPLICIT NONE
    REAL :: A, B, C, D
    ref = "3862"
    type = "L"
    chem = "mercury" ; casrn = "7439-97-6"
    ! p. 9-10:
    A =  254.1649
    B = -372.8229
    C = -167.0119
    D =   20.1528
    ! ln(H) = A + B*(100./T) + C*ln(T/100) + D*(T/100)
    Hominus = rhoH2O/(EXP(A + B*(100./T0) + C*LOG(T0/100.) + D*(T0/100.))*1E3)
    ! mindHR = 100*B - C*T0 - D*T^2/100
    ! see also gnuplot/ref3862.gnu
    mindHR = -(100.*B - C*T0 - D*T0**2/100.) ! the minus sign comes from Hv -> Hs
    CALL MakeNote("moreTdep")
    CALL Output(Hominus, mindHR)
  END SUBROUTINE ref3862

  !---------------------------------------------------------------------------

  SUBROUTINE ref3863 ! Hsxp [1/atm]
    IMPLICIT NONE
    REAL :: A, B, C, D
    ref = "3863"
    type = "L"

    chem = "hydrogen sulfide" ; casrn = "7783-06-4" ! p. 1:
    ! the 2-parameter fitting function for lower T is not used here,
    ! see also gnuplot/ref3863.gnu
    ! ---
    ! ! valid for 273.2 K to 283.2 K:
    ! A = -14.761
    ! B = 2507.2
    ! Hominus = Hsxp_TO_HscpSI * EXP(A + B/T0)
    ! mindHR = B
    ! CALL MakeNote("moreTdep")
    ! CALL Output(Hominus, mindHR)
    ! ---
    ! valid for 283.2 K to 603.2 K:
    A = -24.912
    B = 3477.1
    C = 0.3993
    D = 0.015700
    Hominus = Hsxp_TO_HscpSI * EXP(A + B/T0 + C*LOG(T0) + D*T0)
    ! mindHR = B - C*T - D*T^2
    mindHR = B - C*T0 - D*T0**2
    CALL MakeNote("moreTdep")
    CALL MakeNote(TRIM(ref)//"_H2S", &
      TRIM(citet()) // " provide two different fitting functions: one for " // &
      "$T<283.2$\,\unit{K} and one for $T>283.2$\,\unit{K}. At $T=283.2$\,\unit{K}, " // &
      "the functions have different values and different slopes. Here, only the " // &
      "function that is valid at $T^{\ominus}$ is used.")
    CALL Output(Hominus, mindHR)

    chem = "selenium hydride" ; casrn = "7783-07-5" ! p. 330:
    A = 9.15
    B = 974.
    C = -3.542
    D = 0.00420
    Hominus = Hsxp_TO_HscpSI * EXP(A + B/T0 + C*LOG(T0) + D*T0)
    ! mindHR = B - C*T - D*T^2
    mindHR = B - C*T0 - D*T0**2
    CALL MakeNote("moreTdep")
    CALL Output(Hominus, mindHR)

  END SUBROUTINE ref3863

  !---------------------------------------------------------------------------

  SUBROUTINE ref3866
    IMPLICIT NONE
    ref = "3866"
    type = "L"
    chem = "carbon monoxide" ; casrn = "630-08-0"
    CALL MakeNote(TRIM(ref), &
      TRIM(citet()) // " recommends the data from \citet{3529}.") ! p. 1
    CALL Output(DUMMY)
  END SUBROUTINE ref3866

  !---------------------------------------------------------------------------

  ! ref3867 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3868 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3869 ! Hsxp [1/atm]
    IMPLICIT NONE
    ref = "3869"
    type = "L"
    CALL CalcH_IUPAC_lnx1("ethene", "74-85-1", -66.9156, 92.2101, 24.3792) ! p. 1
  END SUBROUTINE ref3869

  !---------------------------------------------------------------------------

  ! ref3870 NOTUSED: Only citations, no new data.

  !---------------------------------------------------------------------------

  ! ref3871 NOTUSED: Only citations, no new data.

  !---------------------------------------------------------------------------

  ! ref3872 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3873 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3874 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3887 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3888 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3889 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3890 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3891 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3892 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3893 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3894 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3895 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3896 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3897 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3898 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3899 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3900 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3901 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3902 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3903 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3904 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3905 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3906 NOTUSED: The paper contains solubilities but no Henry's law
  !                  constants.

  !---------------------------------------------------------------------------

  ! ref3907 NOTUSED: Only citations and solubilities, no new data.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3917 ! Hvpx [atm]
    IMPLICIT NONE
    ref = "3917"
    type = "M"

    ! Tab. 1:
    chem = "H2S" ; casrn = "7783-06-4"
    CALL HTdep_ABC( CtoK + &
      (/ 0.001, 0.001, 0.000, 4.956, 4.958, 4.955, 4.955, 10.019, 10.018, &
      10.019, 10.019, 15.008, 15.008, 15.006, 20.010, 20.011, 20.011, 20.009, &
      25.018, 25.016, 25.015, 25.018, 25.017, 25.017, 25.030, 25.029, 25.031, &
      29.995, 29.992, 29.995, 39.991, 39.991, 39.992, 49.987, 49.984, 49.985 /), &
      Hvpx_TIMES_HscpSI / &
      (/ 263.18, 263.16, 263.01, 310.51, 310.21, 310.43, 310.11, 362.21, &
      362.01, 362.25, 361.94, 418.17, 418.11, 417.65, 477.44, 476.72, 477.10, &
      476.32, 540.07, 539.74, 538.73, 539.64, 539.79, 538.32, 539.12, 539.07, &
      539.49, 603.29, 604.07, 603.09, 741.34, 740.86, 740.57, 880.80, 882.07, &
      880.08 /), &
      Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)

    ! Tab. 2:
    chem = "D2S" ; casrn = "13536-94-2"
    CALL HTdep_ABC( CtoK + &
      (/ 4.954, 4.953, 4.954, 4.953, 4.969, 4.969, 4.971, 4.970, 10.012, &
      10.011, 10.011, 10.011, 10.016, 10.017, 10.017, 10.015, 14.996, 14.995, &
      14.994, 14.992, 14.998, 14.999, 14.998, 14.997, 20.015, 20.017, 20.018, &
      20.017, 25.002, 25.001, 25.000, 25.003, 25.003, 25.003, 25.004, 25.005, &
      25.006, 25.005, 29.991, 29.991, 29.991, 29.998, 29.997, 29.996, 39.984, &
      39.982, 39.983, 39.983, 49.991, 49.991, 49.990 /), &
      Hvpx_TIMES_HscpSI / &
      (/ 313.07, 312.58, 312.84, 312.70, 312.86, 312.61, 312.96, 312.76, &
      368.17, 367.82, 368.35, 367.92, 368.13, 367.86, 368.07, 367.60, 425.51, &
      425.58, 425.82, 425.35, 426.70, 426.27, 426.45, 425.75, 488.06, 487.12, &
      487.94, 487.52, 552.92, 553.61, 552.78, 551.90, 552.79, 551.86, 552.79, &
      551.67, 552.34, 551.19, 621.27, 622.10, 621.08, 621.17, 622.14, 621.18, &
      761.39, 759.73, 761.57, 761.47, 905.21, 904.75, 905.12 /), &
      Hominus, mindHR)
    CALL MakeNote(TRIM(ref), "Obtained with \chem{D_2O} as solvent.")
    CALL Output(Hominus, mindHR, r2)

  END SUBROUTINE ref3917

  !---------------------------------------------------------------------------

  SUBROUTINE ref3923 ! special definition [ppm]

    IMPLICIT NONE
    ref = "3923"

    ! Measurements were made at "one atmosphere of the gaseous
    ! hydrocarbons". It is unclear if this includes the vapor pressure
    ! of water. Consistent with the IUPAC review (ref3860), a correction
    ! is made here by subtracting p(H2O) from the total pressure.

    ! Gaseous alkanes from Tab. 1 (the remaining table entries are for
    ! liquid hydrocarbons, and the Henry's law constant cannot be
    ! obtained from the solubility):
    CALL CalcH("methane",                  "74-82-8", "M", 24.4, MC+MH*4.    , "also_3924")
    CALL CalcH("ethane",                   "74-84-0", "M", 60.4, MC*2.+MH*6. , "also_3924")
    CALL CalcH("propane",                  "74-98-6", "M", 62.4, MC*3.+MH*8. , "also_3924")
    CALL CalcH("butane",                  "106-97-8", "M", 61.4, MC*4.+MH*10., "also_3924")
    CALL CalcH("2-methylpropane",          "75-28-5", "M", 48.9, MC*4.+MH*10., "also_3924")
    ! CALL CalcH("pentane",                 "109-66-0", "M", 38.5, MC*5.+MH*12., "also_3924")
    ! CALL CalcH("2-methylbutane",           "78-78-4", "M", 47.8, MC*5.+MH*12., "also_3924")
    ! CALL CalcH("hexane",                  "110-54-3", "M",  9.5, MC*6.+MH*14., "also_3924")
    ! CALL CalcH("2-methylpentane",         "107-83-5", "M", 13.8, MC*6.+MH*14., "also_3924")
    ! CALL CalcH("2,2-dimethylbutane",       "75-83-2", "M", 18.4, MC*6.+MH*14., "also_3924")
    ! CALL CalcH("heptane",                 "142-82-5", "M", 2.93, MC*7.+MH*16., "also_3924")
    ! CALL CalcH("2,4-dimethylpentane",     "108-08-7", "M", 3.62, MC*7.+MH*16., "also_3924")
    ! CALL CalcH("octane",                  "111-65-9", "M", 0.66, MC*8.+MH*18., "also_3924")
    ! CALL CalcH("2,2,4-trimethylpentane",  "540-84-1", "M", 2.44, MC*8.+MH*18., "also_3924")

    ! Underlined values from Fig. 1:
    CALL CalcH("CYCLO C5:       cyclopentane",           "287-92-3", "V", 390., MC*5.+MH*10.             )
    CALL CalcH("CYCLO C6:       cyclohexane",            "110-82-7", "V", 460., MC*6.+MH*12.             )
    CALL CalcH("MECYCLO C5:     methylcyclopentane",      "96-37-7", "V", 240., MC*6.+MH*12.             )
    CALL CalcH("MECYCLO C6:     methylcyclohexane",      "108-87-2", "V", 260., MC*7.+MH*14.             )
    CALL CalcH("I-C5:           2-methylbutane",          "78-78-4", "V",  54., MC*5.+MH*12., "also_3924")
    CALL CalcH("N-C5:           pentane",                "109-66-0", "V",  59., MC*5.+MH*12., "also_3924")
    CALL CalcH("2,2-DIME C4:    2,2-dimethylbutane",      "75-83-2", "V",  47., MC*6.+MH*14., "also_3924")
    CALL CalcH("2-ME C5:        2-methylpentane",        "107-83-5", "V",  51., MC*6.+MH*14., "also_3924")
    CALL CalcH("N-C6:           hexane",                 "110-54-3", "V",  50., MC*6.+MH*14., "also_3924")
    CALL CalcH("N-C7:           heptane",                "142-82-5", "V",  51., MC*7.+MH*16., "also_3924")
    CALL CalcH("2,2,4-TRIME C5: 2,2,4-trimethylpentane", "540-84-1", "V",  40., MC*8.+MH*18., "also_3924")
    CALL CalcH("N-C8:           octane",                 "111-65-9", "V",  44., MC*8.+MH*18., "also_3924")

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, type_, solubility, M, note)
      IMPLICIT NONE
      CHARACTER(LEN=*),           INTENT(IN) :: chem_, casrn_
      CHARACTER,                  INTENT(IN) :: type_
      REAL,                       INTENT(IN) :: solubility, M
      CHARACTER(LEN=*), OPTIONAL, INTENT(IN) :: note
      chem=chem_ ; casrn=casrn_ ! make global for Output
      type = type_
      Hominus = 1E-6 * solubility * rhoH2O_T(T0) / (M * (atm-psat(T0)))
      IF (PRESENT(note)) THEN
        CALL MakeNote(TRIM(ref)//"_"//note, &
          "The same value was also published in \citet{3924}.")
      ENDIF
      CALL Output(Hominus)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3923

  !---------------------------------------------------------------------------

  SUBROUTINE ref3924 ! special definition [ppm]

    IMPLICIT NONE
    ref = "3924"

    ! Measurements were made at "one atmosphere of the gaseous
    ! hydrocarbons". It is unclear if this includes the vapor pressure
    ! of water. Consistent with the IUPAC review (ref3860), a correction
    ! is made here by subtracting p(H2O) from the total pressure.

    ! Gaseous alkanes from Tab. I (only gaseous alkanes with a boiling
    ! point below 298.15 K are used here, as Henry's law constants can
    ! not be obtained this way from the solubility of liquid species):
    CALL CalcH("methane",                  "74-82-8", "M", 24.4, MC+MH*4.    , "also_3923")
    CALL CalcH("ethane",                   "74-84-0", "M", 60.4, MC*2.+MH*6. , "also_3923")
    CALL CalcH("propane",                  "74-98-6", "M", 62.4, MC*3.+MH*8. , "also_3923")
    CALL CalcH("butane",                  "106-97-8", "M", 61.4, MC*4.+MH*10., "also_3923")
    CALL CalcH("2-methylpropane",          "75-28-5", "M", 48.9, MC*4.+MH*10., "also_3923")
    ! CALL CalcH("pentane",                 "109-66-0", "M", 38.5, MC*5.+MH*12., "also_3923")
    ! CALL CalcH("2-methylbutane",           "78-78-4", "M", 47.8, MC*5.+MH*12., "also_3923")
    CALL CalcH("dimethylpropane",         "463-82-1", "M", 33.2, MC*5.+MH*12.             )
    ! CALL CalcH("hexane",                  "110-54-3", "M",  9.5, MC*6.+MH*14., "also_3923")
    ! CALL CalcH("2-methylpentane",         "107-83-5", "M", 13.8, MC*6.+MH*14., "also_3923")
    ! CALL CalcH("3-methylpentane",          "96-14-0", "M", 12.8, MC*6.+MH*14.             )
    ! CALL CalcH("2,2-dimethylbutane",       "75-83-2", "M", 18.4, MC*6.+MH*14., "also_3923")
    ! CALL CalcH("heptane",                 "142-82-5", "M", 2.93, MC*7.+MH*16., "also_3923")
    ! CALL CalcH("2,4-dimethylpentane",     "108-08-7", "M", 4.06, MC*7.+MH*16.             ) ! value in ref3923 is different
    ! CALL CalcH("octane",                  "111-65-9", "M", 0.66, MC*8.+MH*18., "also_3923")
    ! CALL CalcH("2,2,4-trimethylpentane",  "540-84-1", "M", 2.44, MC*8.+MH*18., "also_3923")
    ! CALL CalcH("2,2,5-trimethylhexane",  "3522-94-9", "M", 1.15, MC*9.+MH*20.             )

    ! Gaseous alkenes from Tab. II (only gaseous alkanes with a boiling
    ! point below 298.15 K are used here, as Henry's law constants can
    ! not be obtained this way from the solubility of liquid species):
    CALL CalcH("ethene",             "74-85-1", "M", 131., MC*2.+MH*4. )
    CALL CalcH("propene",           "115-07-1", "M", 200., MC*3.+MH*6. )
    CALL CalcH("1-butene",          "106-98-9", "M", 222., MC*4.+MH*8. )
    CALL CalcH("2-methylpropene",   "115-11-7", "M", 263., MC*4.+MH*8. )
    CALL CalcH("3-methyl-1-butene", "563-45-1", "M", 130., MC*5.+MH*10.)

    ! Gaseous alkynes from Tab. III (only gaseous alkanes with a boiling
    ! point below 298.15 K are used here, as Henry's law constants can
    ! not be obtained this way from the solubility of liquid species):
    CALL CalcH("propyne",    "74-99-7", "M", 3640., MC*3.+MH*4.)
    CALL CalcH("1-butyne",  "107-00-6", "M", 2870., MC*4.+MH*6.)

    ! Underlined values from Fig. 2:
    CALL CalcH("I-C5:           2-methylbutane",          "78-78-4", "V", 54., MC*5.+MH*12., "also_3923")
    CALL CalcH("N-C5:           pentane",                "109-66-0", "V", 59., MC*5.+MH*12., "also_3923")
    CALL CalcH("2,2-DIME C4:    2,2-dimethylbutane",      "75-83-2", "V", 47., MC*6.+MH*14., "also_3923")
    CALL CalcH("2-ME C5:        2-methylpentane",        "107-83-5", "V", 51., MC*6.+MH*14., "also_3923")
    CALL CalcH("3-ME C5:        3-methylpentane",         "96-14-0", "V", 51., MC*6.+MH*14.             )
    CALL CalcH("N-C6:           hexane",                 "110-54-3", "V", 50., MC*6.+MH*14., "also_3923")
    CALL CalcH("2,4-DIME C5:    2,4-dimethylpentane",    "108-08-7", "V", 31., MC*7.+MH*16.             )
    CALL CalcH("N-C7:           heptane",                "142-82-5", "V", 51., MC*7.+MH*16., "also_3923")
    CALL CalcH("2,2,4-TRIME C5: 2,2,4-trimethylpentane", "540-84-1", "V", 40., MC*8.+MH*18., "also_3923")
    CALL CalcH("N-C8:           octane",                 "111-65-9", "V", 44., MC*8.+MH*18., "also_3923")
    CALL CalcH("2,2,5-TRIME C6: 2,2,5-trimethylhexane", "3522-94-9", "V", 49., MC*9.+MH*20.             )

    ! p. 1273, solubility at 1 atm:
    CALL CalcH("1-pentene",        "109-67-1", "V",  186., MC*5.+MH*10., "atroomT")
    CALL CalcH("1-hexene",         "592-41-6", "V",  224., MC*6.+MH*12., "atroomT")
    CALL CalcH("1-octene",         "111-66-0", "V",  170., MC*8.+MH*16., "atroomT")

    ! p. 1274, solubility at 1 atm:
    CALL CalcH("benzene",           "71-43-2", "V", 1.5E4, MC*6.+MH*6. , "atroomT")
    CALL CalcH("toluene",          "108-88-3", "V", 1.5E4, MC*7.+MH*8. , "atroomT")
    CALL CalcH("o-xylene",          "95-47-6", "V", 2.0E4, MC*8.+MH*10., "atroomT")
    CALL CalcH("ethylbenzene",     "100-41-4", "V", 1.4E4, MC*8.+MH*10., "atroomT")
    CALL CalcH("isopropylbenzene",  "98-82-8", "V", 1.3E4, MC*9.+MH*12., "atroomT")

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, type_, solubility, M, note)
      IMPLICIT NONE
      CHARACTER(LEN=*),           INTENT(IN) :: chem_, casrn_
      CHARACTER,                  INTENT(IN) :: type_
      REAL,                       INTENT(IN) :: solubility, M
      CHARACTER(LEN=*), OPTIONAL, INTENT(IN) :: note
      chem=chem_ ; casrn=casrn_ ! make global for Output
      type = type_
      Hominus = 1E-6 * solubility * rhoH2O_T(T0) / (M * (atm-psat(T0)))
      IF (PRESENT(note)) THEN
        IF (note=="also_3923") CALL MakeNote(TRIM(ref)//"_"//note, &
          "The same value was also published in \citet{3923}.")
        IF (note=="atroomT") CALL MakeNote("atroomT")
      ENDIF
      CALL Output(Hominus)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3924

  !---------------------------------------------------------------------------

  SUBROUTINE ref3925 ! Hscc [1] = Ostwald coefficient
    IMPLICIT NONE

    ref = "3925"
    type = "R"
    chem = "chlorine (molecular)" ; casrn = "7782-50-5" ! Cl2
    CALL MakeNote(TRIM(ref), "\citet{3925} re-analyzed the data from "// &
      "\citet{3937} and \citet{3942}, considering deviations from the "// &
      "perfect gas law. They calculated the total solubility of chlorine "// &
      "(i.e., the sum of \chem{Cl_2} and \chem{HOCl}) at several partial "// &
      "pressures of \chem{Cl_2}. This is different from Henry's law "// &
      "constant, which is defined at extrapolation to infinite dilution.")
    CALL Output(DUMMY)

    ! ! Fig. 2, digitized with engauge:
    ! CALL CalcH("Cl2", "7782-50-5", &
    !   (/ -0.0913, 14.8325,   24.82, 38.9728,  53.497,  67.457, 83.2173 /) +CtoK, &
    !   (/ 4.12822,  2.0529, 1.50795, 0.99236, 0.70143, 0.55993, 0.4588  /))

    ! Tab. 1:
    !      0      10     20     30     40     50     60     70     80     90    100   110
    !    5 0.488  0.451  0.438  0.424  0.412  0.398  0.383  0.369  0.351  0.339 0.326 0.316
    !   10 0.679  0.603  0.575  0.553  0.532  0.512  0.492  0.470  0.447  0.431 0.415 0.402
    !   30 1.221  1.024  0.937  0.873  0.821  0.781  0.743  0.704  0.671  0.642 0.627 0.598
    !   50 1.717  1.354  1.210  1.106  1.025  0.962  0.912  0.863  0.815  0.781 0.747 0.722
    !  100 2.79   2.08   1.773  1.573  1.424  1.313  1.228  1.149  1.085  1.034 0.987 0.950
    !  150 3.81   2.73   2.27   1.966  1.754  1.599  1.482  1.382  1.294  1.227 1.174 1.137
    !  200 4.78   3.35   2.74   2.34   2.05   1.856  1.706  1.580  1.479  1.396 1.333 1.276
    !  250 5.71   3.95   3.19   2.69   2.34   2.09   1.914  1.764  1.642  1.553 1.480 1.413
    !  300        4.54   3.63   3.03   2.61   2.31   2.10   1.932  1.793  1.700 1.610 1.542
    !  350        5.13   4.06   3.35   2.86   2.53   2.28   2.10   1.940  1.831 1.736 1.661
    !  400        5.71   4.48   3.69   3.11   2.74   2.47   2.25   2.08   1.965 1.854 1.773
    !  450        6.26   4.88   3.98   3.36   2.94   2.64   2.41   2.22   2.09  1.972 1.880
    !  500        6.85   5.29   4.30   3.61   3.14   2.80   2.55   2.35   2.21  2.08  1.986
    !  550        7.39   5.71   4.60   3.84   3.33   2.97   2.69   2.47   2.32  2.19  2.09
    !  600        7.97   6.12   4.91   4.08   3.52   3.13   2.83   2.59   2.43  2.29  2.19
    !  650        8.52   6.52   5.21   4.32   3.71   3.29   2.97   2.72   2.55  2.41  2.28
    !  700        9.09   6.90   5.50   4.54   3.89   3.44   3.10   2.84   2.66  2.50  2.37
    !  750        9.65   7.29   5.80   4.77   4.07   3.59   3.23   2.96   2.76  2.60  2.47
    !  800       10.21   7.69   6.08   4.99   4.27   3.75   3.37   3.08   2.87  2.69  2.56
    !  900               8.46   6.68   5.44   4.62   4.04   3.63   3.30   3.08  2.89  2.74
    ! 1000               9.27   7.27   5.89   4.97   4.36   3.88   3.53   3.28  3.07  2.91
    ! 1200              10.84   8.42   6.81   5.67   4.92   4.37   3.95   3.67  3.43  3.25
    ! 1500              13.23  10.14   8.05   6.70   5.76   5.09   4.58   4.23  3.95  3.74
    ! 2000              17.07  13.02  10.22   8.38   7.14   6.26   5.63   5.17  4.78  4.49
    ! 2500              21.0   15.84  12.32  10.03   8.48   7.40   6.61   6.05  5.59  5.25
    ! 3000                     18.73  14.47  11.70   9.83   8.52   7.54   6.92  6.38  5.97
    ! 3500                     21.7   16.62  13.38  11.22   9.65   8.53   7.79  7.16  6.72
    ! 4000                     24.7   18.84  15.04  12.54  10.76   9.52   8.65  7.94  7.42
    ! 4500                     27.7   20.7   16.75  13.88  11.91  10.46   9.49  8.72  8.13
    ! 5000                     30.8   23.3   18.46  15.26  13.01  11.42  10.35  9.48  8.84

    ! CONTAINS

    !   SUBROUTINE CalcH (chem_, casrn_, temp, ostwald)
    !     IMPLICIT NONE
    !     CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
    !     REAL, DIMENSION(:), INTENT(IN) :: temp, ostwald
    !     chem=chem_ ; casrn=casrn_ ! make global for Output
    !     CALL HTdep_ABC(temp, Hscc_TO_HscpSI(ostwald, temp), Hominus, mindHR)
    !     CALL MakeNote(TRIM(ref), "From digitized Fig.~2 of " // TRIM(citet()) // &
    !       " which shows experimental data from Yakovkin (J.\ Russ.\ Phys.\ Chem.\ " // &
    !       "Soc., 32, 673, 1900).")
    !     CALL Output(Hominus, mindHR, r2)
    !   END SUBROUTINE CalcH

  END SUBROUTINE ref3925

  !---------------------------------------------------------------------------

  SUBROUTINE ref3926 ! Hsxp [1/atm] and Ostwald coefficient
    IMPLICIT NONE
    ref = "3926"
    type = "M"
    CALL CalcH( "D2", "7782-39-0", & ! Tab. I
      (/ 278.15, 283.15, 288.15, 293.15, 298.15, 303.15, &
         278.15, 283.15, 288.15, 293.15, 298.15, 303.15, &
         278.15, 283.15, 288.15, 293.15, 298.15, 303.15, &
         278.15, 283.15, 288.15, 293.15, 298.15, 303.15, &
                 283.15 /),                              &
      (/ 22.795, 21.970, 21.388, 20.714, 20.469, 20.069, &
         22.891, 21.795, 21.265, 20.636, 20.354, 19.956, &
         22.875, 21.882, 21.281, 20.731, 20.505, 20.066, &
         22.824, 21.940, 21.207, 20.559, 20.374, 20.029, &
                 21.825 /) * 1E-3,                       &
      (/ 1.7829, 1.6825, 1.6028, 1.5175, 1.4637, 1.3980, &
         1.7904, 1.6691, 1.5936, 1.5118, 1.4555, 1.3901, &
         1.7891, 1.6757, 1.5948, 1.5187, 1.4663, 1.3978, &
         1.7851, 1.6802, 1.5892, 1.5016, 1.4569, 1.3952, &
                 1.6714 /) * 1E-5)
    CALL Output(Hominus, mindHR, r2)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp_, ostwald, x)
      IMPLICIT NONE
      INTEGER :: i
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp_, ostwald, x
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep_ABC(temp_, x*Hsxp_TO_HscpSI, Hominus, mindHR)
      DO i=1,SIZE(temp_)
        CALL consistency_check(x(i)*Hsxp_TO_HscpSI, Hscc_TO_HscpSI(ostwald(i),temp_(i)), &
          "The data listed as gamma and x") !, verbose=.TRUE.
      ENDDO
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3926

  !---------------------------------------------------------------------------

  ! ref3927 NOTUSED: This is the English version of the 1st part of ref3717.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3929 ! Hvpc [atm/M]
    IMPLICIT NONE
    ref = "3929"
    type = "R"
    chem = "chlorine (molecular)" ; casrn = "7782-50-5" ! Cl2
    CALL MakeNote(TRIM(ref), &
      TRIM(citet()) // " converted the total solubility of chlorine in " // &
        "pure water from \citet{3925} to an intrinsic Henry's law constant.")
    CALL Output(Hscp_TO_HscpSI / 16.7) ! p. 829
  END SUBROUTINE ref3929

  !---------------------------------------------------------------------------

  ! ref3930 NOTUSED: This is the English version of the 2nd part of ref3717.

  !---------------------------------------------------------------------------

  ! ref3931 NOTUSED: It seems that knowledge about the Cl2 / HOCl
  !                  equilibrium was limited when this article was
  !                  written in 1856. Roscoe explores why the solubility
  !                  of Cl2 depends on the concentrations of HCl and
  !                  CO2.

  !---------------------------------------------------------------------------

  ! ref3932 NOTUSED: Mellor explores how the total solubility of Cl2
  !                  depends on the concentrations of HCl and CO2.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3933 ! Bunsen coefficient [1]
    IMPLICIT NONE
    ref = "3933"
    type = "M"
    CALL CalcH("SO2", "7446-09-5", &
      (/ 4.0,    10.0,   15.6,   21.0,   26.0,   32.0,   36.0 /), &
      (/ 68.635, 55.791, 46.304, 37.015, 32.130, 24.445, 20.921 /)) ! p. 6
    CALL CalcH("Cl2", "7782-50-5", &
      (/ 11.0,   18.8,   23.7,   30.4,   37.2,   41.5 /), &
      (/ 2.5674, 2.2075, 1.9572, 1.7308, 1.5027, 1.3073 /)) ! p. 9
    CALL CalcH("H2S", "7783-06-4", &
      (/ 2.0,    9.8,    14.6,   19.0,   23.0,   27.8,   35.6,   43.3 /), &
      (/ 4.2373, 3.5446, 3.2651, 2.9050, 2.7415, 2.3735, 1.9972, 1.7143 /)) ! p. 11

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, temp, alpha)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: temp, alpha
      chem=chem_ ; casrn=casrn_ ! make global for Output
      IF (TRIM(casrn)=="7782-50-5") THEN ! Cl2
        CALL MakeNote("HeffCl2")
        CALL Output(DUMMY)
        RETURN
      ELSE
        ! data need a 3-parameter fit:
        CALL HTdep_ABC(temp+CtoK, alpha*alpha_TO_HscpSI, Hominus, mindHR)
        CALL Output(Hominus, mindHR, r2)
      ENDIF
    END SUBROUTINE CalcH

  END SUBROUTINE ref3933

  !---------------------------------------------------------------------------

  SUBROUTINE ref3934 ! Hscc [1]
    IMPLICIT NONE
    REAL :: intercept, slope
    ! Br2 data:
    REAL, DIMENSION(5), PARAMETER :: a_min_b = (/ 4.70,  3.382, 2.30,  0.357, 0.075 /)
    REAL, DIMENSION(5), PARAMETER :: a       = (/ 65.97, 48.63, 33.00, 5.211, 1.267 /)
    REAL :: Hscc, temp

    ref = "3934"
    type = "M"

    chem = "chlorine (molecular)" ; casrn = "7782-50-5" ! Cl2
    ! NOTUSED: The total solubility of Cl2 is measured at 15 C and shown
    !          in Tab. VI on p. 400. In Fig. 2, it can be seen that this
    !          effective Henry solubility goes to infinity at infinite
    !          dilution. A value for the intrinsic Henry solubility is not
    !          provided.
    CALL MakeNote("HeffCl2")
    CALL Output(DUMMY)

    chem = "bromine (molecular)" ; casrn = "7726-95-6" ! Br2
    ! p. 401, Tab. IX and Fig. 3 (at 15 C):
    ! Linear regression of a vs b, using only the data in Fig. 3 (not
    ! the point from Tab. IX with V=500)
    CALL LinReg (a-a_min_b, a, intercept, slope)
    Hscc = 250. / (104.0*LOG(slope)) ! eqn on top of p. 394
    temp = 15.
    CALL MakeNoteOtherTempC(temp)
    CALL Output(Hscc_TO_HscpSI(Hscc, temp+CtoK))

  END SUBROUTINE ref3934

  !---------------------------------------------------------------------------

  SUBROUTINE ref3935 ! Bunsen coefficient [1]

    IMPLICIT NONE
    ref = "3935"
    type = "M"
    chem = "chlorine (molecular)" ; casrn = "7782-50-5" ! Cl2
    ! Experiments I, II and II on pp. 604-605 and Fig. on p. 619 show a
    ! max total solubility around 10 C. At lower T, hydrate formation
    ! occured.
    CALL MakeNote("HeffCl2")
    CALL Output(DUMMY)

  END SUBROUTINE ref3935

  !---------------------------------------------------------------------------

  ! ref3936 NOTUSED: Only (total?) chlorine solubility in HCl (38%, 13%,
  !                  3%) at unknown temperature.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3937 ! Hscp [mmol L-3 atm-1]

    IMPLICIT NONE
    ref = "3937"
    type = "M"
    chem = "chlorine (molecular)" ; casrn = "7782-50-5" ! Cl2
    ! Tab. 53 on p. 719:
    CALL HTdep_ABC( (/ 15., 25., 39.1, 53.6, 67.6, 83.4 /) + CtoK, &
      (/ 87.67, 61.81, 39.16, 26.52, 19.31, 15.20 /) / atm, Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)

  END SUBROUTINE ref3937

  !---------------------------------------------------------------------------

  SUBROUTINE ref3938 ! special definition [mass%/atm]

    IMPLICIT NONE
    ref = "3938"
    type = "M"
    chem = "chlorine (molecular)" ; casrn = "7782-50-5" ! Cl2
    ! p. 64:
    ALLOCATE(temp(3))
    temp = (/ 6., 9., 12. /) + CtoK
    CALL HTdep( temp, &
      0.01 * (/ 1.07, 0.95, 0.87 /) * rhoH2O_T(temp) / (MCl*2 * (atm-psat(temp))), &
      Hominus, mindHR) ! -> 8.1E-04 * EXP(2700*(1/T-1/298.15)) mol/(m^3*Pa)
    CALL MakeNote("HeffCl2")
    CALL Output(DUMMY)

  END SUBROUTINE ref3938

  !---------------------------------------------------------------------------

  SUBROUTINE ref3939 ! Hvcc [1] and Hvpc [atm*m3/mol]
    IMPLICIT NONE

    ref = "3939"

    ! Tab. 1:
    type = "M"
    CALL CalcH("4:2 FTOH",     "2043-47-2", 0.31)
    !CALL CalcH("6:2 FTOH",     "647-42-7", 1.26) ! data from Tab. 2 are used
    CALL CalcH("8:2 FTOH",      "678-39-7", 1.98)
    CALL CalcH("10:2 FTOH",     "865-86-1", 2.82)
    CALL CalcH("4:2 FTS",    "757124-72-4", 0.09)
    !CALL CalcH("6:2 FTS",    "27619-97-2", 0.16) ! data from Tab. 2 are used
    CALL CalcH("8:2 FTS",     "39108-34-4", 0.18)
    !CALL CalcH("PFHxI",        "355-43-1", 1.01) ! data from Tab. 2 are used
    CALL CalcH("6:2 FTUI",   "150223-14-6", 0.48)
    CALL CalcH("6:2 FTI",      "2043-57-4", 0.30)
    CALL CalcH("N-EtFOSA-M",   "4151-50-2", 0.43)
    CALL CalcH("N-MeFOSA-M",  "31506-32-8", 0.92)
    !CALL CalcH("6:2 FTO",    "25291-17-2", 3.86) ! data from Tab. 2 are used
    CALL CalcH("8:2 FTCA",    "27854-31-5", 0.69)
    CALL CalcH("8:2 FTAC",    "27905-45-9", 0.32)

    ! Tab. 2:
    type = "M"
    ndata = 6
    ALLOCATE(temp(ndata))
    temp = (/ 25., 30., 35., 40., 45., 50. /) + CtoK
    CALL CalcHTdep("6:2 FTOH",   "647-42-7", (/ 1.25, 1.50, 1.89, 2.50,  3.00,  3.90 /))
    CALL CalcHTdep("6:2 FTS",  "27619-97-2", (/ 0.16, 0.38, 0.59, 1.24,  2.09,  2.34 /))
    CALL CalcHTdep("PFHxI",      "355-43-1", (/ 1.00, 1.25, 1.89, 3.11,  5.00,  7.00 /))
    CALL CalcHTdep("6:2 FTO",  "25291-17-2", (/ 3.90, 4.94, 5.77, 8.13, 10.43, 12.44 /))
    DEALLOCATE(temp)

    ! Tab. S5 (supplement):
    type = "Q"
    CALL CalcH_EPI("4:2 FTOH",  "2043-47-2", 5.4E-3)
    CALL CalcH_EPI("6:2 FTOH",   "647-42-7", 1.5E-1)
    CALL CalcH_EPI("8:2 FTOH",   "678-39-7", 4.1   )
    CALL CalcH_EPI("10:2 FTOH",  "865-86-1", 1.1E2 )
    CALL CalcH_EPI("TFA",         "76-05-1", 4.3E-6) ! trifluoroacetic acid
    CALL CalcH_EPI("PFPrA",      "422-64-0", 2.3E-5) ! perfluoropropanoic acid
    CALL CalcH_EPI("PFBA",       "375-22-4", 1.2E-4) ! perfluorobutanoic acid
    CALL CalcH_EPI("PFPeA",     "2706-90-3", 6.3E-4) ! perfluoropentanoic acid
    CALL CalcH_EPI("PFHxA",      "307-24-4", 3.3E-3) ! perfluorohexanoic acid
    CALL CalcH_EPI("PFHpA",      "375-85-9", 1.7E-2) ! perfluoroheptanoic acid
    CALL CalcH_EPI("PFOA",       "335-67-1", 9.1E-2) ! perfluorooctanoic acid

  CONTAINS

    SUBROUTINE CalcH_EPI (chem_, casrn_, Hvpc)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: Hvpc
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL MakeNote(TRIM(ref)//"_EPI", &
        "Calculated using the EPI Suite Bond estimation method.")
      CALL Output(1./(atm*Hvpc))
    END SUBROUTINE CalcH_EPI

    SUBROUTINE CalcH (chem_, casrn_, kH)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: kH
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL Output(Hvcc_TIMES_HscpSI_atT0/kH)
    END SUBROUTINE CalcH

    SUBROUTINE CalcHTdep (chem_, casrn_, kH)
      IMPLICIT NONE
      CHARACTER(LEN=*),   INTENT(IN) :: chem_, casrn_
      REAL, DIMENSION(:), INTENT(IN) :: kH
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL HTdep(temp, Hvcc_TO_HscpSI(kH, temp), Hominus, mindHR)
      CALL Output(Hominus, mindHR, r2)
    END SUBROUTINE CalcHTdep

  END SUBROUTINE ref3939

  !---------------------------------------------------------------------------

  SUBROUTINE ref3940 ! Hvpc [Pa*m3/mol]
    IMPLICIT NONE

    ref = "3940"
    type = "M"
    ! Fig. 2, page 41:
    CALL CalcH("TFA",         "76-05-1", 0.0121) ! trifluoroacetic acid
    CALL CalcH("PFPrA",      "422-64-0", 0.113 ) ! perfluoropropanoic acid
    CALL CalcH("PFBA",       "375-22-4", 1.24  ) ! perfluorobutanoic acid
    CALL CalcH("PFPeA",     "2706-90-3", 1.50  ) ! perfluoropentanoic acid
    CALL CalcH("PFHxA",      "307-24-4", 0.928 ) ! perfluorohexanoic acid
    CALL CalcH("PFHpA",      "375-85-9", 0.573 ) ! perfluoroheptanoic acid
    CALL CalcH("PFOA",       "335-67-1", 0.362 ) ! perfluorooctanoic acid

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, Hvpc)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: Hvpc
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL MakeNote(TRIM(ref), &
        "Measured in aqueous hydrochloric acid and extrapolated to pure " // &
        "water as the solvent at 25\,\unit{\degree C}. Measurements were " // &
        "also made at other temperatures; however, those were not " // &
        "extrapolated to pure water as the solvent.")
      CALL Output(1./Hvpc)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3940

  !---------------------------------------------------------------------------

  SUBROUTINE ref3942 ! Hscc [1]

    IMPLICIT NONE
    ref = "3942"
    type = "R"
    chem = "chlorine (molecular)" ; casrn = "7782-50-5" ! Cl2
    CALL MakeNote(TRIM(ref), "\citet{3942} re-analyzed the measurements " // &
      "of \citet{3937}. In addition to the data between " // &
      "15\,\unit{\degree C} and 83.4\,\unit{\degree C}, he also analyzed " // &
      "the experimental results at 0\,\unit{\degree C} and obtained a " // &
      "dimensionless Henry solubility of $\Hscc$ = 4.115 at that temperature.")
    CALL Output(DUMMY)

  END SUBROUTINE ref3942

  !---------------------------------------------------------------------------

  SUBROUTINE ref3943 ! Hvpc [Torr*L/mol]

    IMPLICIT NONE
    ref = "3943"
    type = "M"
    chem = "di-(2-ethylhexyl)-adipate" ; casrn = "103-23-1" ; inchikey = "SAOKZLXYCUGLFA-UHFFFAOYSA-N"
    CALL MakeNoteOtherTempC(22.) ! p. 778
    CALL Output(1000./(0.33*mmHg))

  END SUBROUTINE ref3943

  !---------------------------------------------------------------------------

  SUBROUTINE ref3944 ! Hscc [1] = Ostwald coefficient

    IMPLICIT NONE
    REAL, DIMENSION(12), PARAMETER :: temp = (/ &
      24.68, 21.66, 15.77, 11.87, 6.26, 3.43, &         ! Reihe 1
      24.72, 21.72, 15.75, 12.16, 6.53, 3.59 /) + CtoK  ! Reihe 2
    REAL, DIMENSION(12), PARAMETER :: ostwald = (/ &
      0.5796, 0.6360, 0.7618, 0.8823, 1.0824, 1.2090, & ! Reihe 1
      0.5798, 0.6379, 0.7753, 0.8754, 1.0680, 1.1994 /) ! Reihe 2
    ref = "3944"
    type = "M"
    chem  = "dinitrogen monoxide" ; casrn = "10024-97-2"
    CALL HTdep_ABC(temp, Hscc_TO_HscpSI(ostwald, temp), Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)

    END SUBROUTINE ref3944

  !---------------------------------------------------------------------------

  SUBROUTINE ref3945 ! Hvpc [Torr*L/mol]

    IMPLICIT NONE
    ref = "3945"
    ! Tab. 1:
    CALL CalcH("vinyl chloride",             "75-01-4", 14000.)
    CALL CalcH("chloroform",                 "67-66-3", 2700. )
    CALL CalcH("carbon tetrachloride",       "56-23-5", 16200.)
    CALL CalcH("1,1-dichloroethane",         "75-34-3", 4400. )
    CALL CalcH("oxygen",                   "7782-44-7", 59000.)
    CALL CalcH("benzo[b]thiophene",          "95-15-8", 210.  )
    CALL CalcH("dibenzothiophene",          "132-65-0", 332.  )
    ! Tab. 2:
    CALL CalcH("p-cresol",                  "106-44-5", 6.5   )
    CALL CalcH("benz[a]anthracene",          "56-55-3", 0.10  )
    CALL CalcH("benzo[a]pyrene",             "50-32-8", 1.0   )
    CALL CalcH("quinoline",                  "91-22-5", 0.19  )
    CALL CalcH("benzo[f]quinoline",          "85-02-9", 0.072 )
    CALL CalcH("carbazole",                  "86-74-8", 120.  )
    CALL CalcH("7H-dibenzo[c,g]carbazole",  "194-59-2", 0.0036)
    CALL CalcH("methyl parathion",          "298-00-0", 59.   )
    CALL CalcH("mirex",                    "2385-85-5", 144.  )

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, Hc)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: Hc
      IF (casrn_=="7782-44-7") THEN ! O2
        type = "?" ! footnote a in Tab. 1
      ELSE
        type = "V"
      ENDIF
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL MakeNote("atroomT") ! T is not mentioned in paper
      CALL Output(1000./(Hc*mmHg))
    END SUBROUTINE CalcH

  END SUBROUTINE ref3945

  !---------------------------------------------------------------------------

  SUBROUTINE ref3946 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3946"
    type = "M"
    ! Tab. on p. 661:
    CALL CalcH("cis-1,3-dichloropropene",   "10061-01-5",  9.50)
    CALL CalcH("trans-1,3-dichloropropene", "10061-02-6",  7.20)
    CALL CalcH("2,3-dichloropropene",          "78-88-6", 11.512)
    CALL CalcH("1,2-dichloropropane",          "78-87-5",  8.297)
    CALL CalcH("1,3-dichloropropane",         "142-28-9",  4.751)
    CALL CalcH("1,2,3-trichloropropane",       "96-18-4",  1.229)
    CALL CalcH("1,2-dibromopropane",           "78-75-1",  7.620)
    CALL CalcH("(Z)-1,4-dichloro-2-butene",  "1476-11-5",  3.864)
    CALL CalcH("(E)-1,4-dichloro-2-butene",   "110-57-6",  2.374)
    CALL CalcH("1,4-dichlorobutane",          "110-56-5",  3.323)
    CALL CalcH("1,4-dibromobutane",           "110-52-1",  2.325)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, H)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: H
      chem=chem_ ; casrn=casrn_ ! make global for Output
      CALL Output(1E2*Hvcc_TIMES_HscpSI_atT0/H)
    END SUBROUTINE CalcH

  END SUBROUTINE ref3946

  !---------------------------------------------------------------------------

  SUBROUTINE ref3947 ! Hvpc [atm*m3/mol]
    IMPLICIT NONE
    ref = "3947"
    type = "M"
    chem = "metolachlor" ; casrn = "51218-45-2" ! C{15}H{22}ClNO2
    ndata = 4
    ALLOCATE(temp(ndata), Harray(ndata))
    temp = (/ 20., 25., 30., 40. /) + CtoK
    Harray = 1. / ( atm * (/ 0.9E-8, 1.39E-8, 3.8E-8, 1.98E-7 /) )
    CALL HTdep(temp, Harray, Hominus, mindHR)
    DEALLOCATE(temp, Harray)
    CALL MakeNote(TRIM(ref), "The value at 20\,\unit{\degree C} was " // &
      "calculated from published values of vapor pressure and water " // &
      "solubility. Data between 25\,\unit{\degree C} and " // &
      "40\,\unit{\degree C} were calculated from the measured evaporation rate.")
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3947

  !---------------------------------------------------------------------------

  SUBROUTINE ref3948 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3948"
    ! Tab. 3:
    CALL CalcH("bromomethane",                 "74-83-9", 0.21,  0.23)
    CALL CalcH("cis-1,3-dichloropropene",   "10061-01-5", 0.041, 0.048)
    CALL CalcH("trans-1,3-dichloropropene", "10061-02-6", 0.027, 0.027)

  CONTAINS

    SUBROUTINE CalcH (chem_, casrn_, exptl, theor)
      IMPLICIT NONE
      CHARACTER(LEN=*), INTENT(IN) :: chem_, casrn_
      REAL,             INTENT(IN) :: exptl, theor
      REAL, PARAMETER              :: temp = 16.4 ! abstract
      chem=chem_ ; casrn=casrn_ ! make global for Output
      ! experimental:
      type = "M"
      CALL MakeNoteOtherTempC(temp)
      CALL MakeNote(TRIM(ref)//"_exptl", &
        "Henry's law constants were evaluated using data from Florida sandy field soil.")
      CALL Output(Hvcc_TO_HscpSI(exptl, temp+CtoK))
      ! theoretical:
      type = "?"
      CALL MakeNoteOtherTempC(temp)
      CALL MakeNote(TRIM(ref)//"_theor", &
        "According to " // TRIM(citet()) // ", theoretical Henry's law " // &
        "constants were calculated using the ``normal boiling point, the " // &
        "critical temperature, and the enthalpy of volatilization at the " // &
        "normal boiling point''.")
      CALL Output(Hvcc_TO_HscpSI(theor, temp+CtoK))
    END SUBROUTINE CalcH

  END SUBROUTINE ref3948

  !---------------------------------------------------------------------------

  SUBROUTINE ref3949 ! Hsbp [mol/(kg*mmHg)]
    IMPLICIT NONE
    REAL, PARAMETER :: temp = 3.46 ! from abstract
    ref = "3949"
    type = "R"
    chem = "dichlorine monoxide" ; casrn = "7791-21-1" ! Cl2O
    ! data from this paper are cited by ref3857, p. 453
    CALL MakeNoteOtherTempC(temp)
    CALL Output(0.00937*rhoH2O_T(temp+CtoK)/mmHg)
  END SUBROUTINE ref3949

  !---------------------------------------------------------------------------

  SUBROUTINE ref3950 ! special definition: solubility [g/L] (=[kg/m3])
                     !                     and partial pressure [mmHg]
    IMPLICIT NONE
    ref = "3950"
    type = "?"
    chem = "chlorine dioxide" ; casrn = "10049-04-4" ! ClO2
    CALL Output(3.01 / (34.5 * mmHg * (MCl+MO*2.))) ! p. 4

  END SUBROUTINE ref3950

  !---------------------------------------------------------------------------

  SUBROUTINE ref3951 ! special definition: solubility [g/L] (=[kg/m3])
                     !                     and partial pressure [kPa]
    IMPLICIT NONE
    ref = "3951"
    type = "?"
    chem = "chlorine dioxide" ; casrn = "10049-04-4" ! ClO2
    ! Data shown in Fig. 1.1 are NOT from Buser & Hänisch (1952).
    mindHR = 3102.
    Hominus = 1. / (1E3 * EXP(10.717-mindHR/T0) * (MCl+MO*2.)) ! p. 3
    CALL Output(Hominus, mindHR)

  END SUBROUTINE ref3951

  !---------------------------------------------------------------------------

  ! ref3952 NOTUSED: Fitted values from Haller (ref3955) to
  !                  d(ln H)/d(1/T) but no new data.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3955 ! special definition: solubility [g/L] (=[kg/m3])
                     !                     and partial pressure [mmHg]
    IMPLICIT NONE
    ref = "3955"
    type = "C"
    chem = "chlorine dioxide" ; casrn = "10049-04-4" ! ClO2
    ! data is shown in Tab. II
    CALL MakeNote(TRIM(ref), "Even though " // TRIM(citet()) // &
      " have been cited several times as the source of the \chem{ClO_2} " // &
      "solubility data, they did not perform any measurements. They took " // &
      "the data from the 1952 edition of the Kirk-Othmer Encyclopedia of " // &
      "Chemical Technology which apparently reproduced data from \citet{3969}.")
    CALL Output(DUMMY)

  END SUBROUTINE ref3955

  !---------------------------------------------------------------------------

  SUBROUTINE ref3956 ! special definition: solubility [g/L] (=[kg/m3])
                     !                     and partial pressure [mmHg]
    IMPLICIT NONE
    ref = "3956"
    type = "M"
    chem = "chlorine dioxide" ; casrn = "10049-04-4" ! ClO2
    ! The fit formula presented on p. 204 appears to be erroneous. Here,
    ! data from Tab. 2 are re-fitted:
    CALL HTdep((/ 10., 15., 20., 25., 35., 45. /) + CtoK, &
      (/ 0.1666, 0.1347, 0.1111, 0.0914, 0.0643, 0.0474 /) / (mmHg * (MCl+MO*2.)), &
      Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)

  END SUBROUTINE ref3956

  !---------------------------------------------------------------------------

  SUBROUTINE ref3957 ! special definition: concentration [N] and partial pressure [mmHg]
    IMPLICIT NONE
    REAL :: intercept, slope, H30, H40, H50, H60
    ! data from Tab. 2 (also shown in Fig. 2):
    REAL, PARAMETER :: c_HOCl_30(7) = (/ 0.011, 0.021, 0.047, 0.052, 0.066, 0.090, 0.11  /)
    REAL, PARAMETER :: c_HOCl_40(7) = (/ 0.021, 0.035, 0.047, 0.052, 0.066, 0.080, 0.092 /)
    REAL, PARAMETER :: c_HOCl_50(7) = (/ 0.013, 0.031, 0.037, 0.049, 0.067, 0.082, 0.11  /)
    REAL, PARAMETER :: c_HOCl_60(6) = (/ 0.012, 0.029, 0.041, 0.060, 0.067, 0.10         /)
    REAL, PARAMETER :: p_HOCl_30(7) = (/ 0.011, 0.022, 0.039, 0.046, 0.058, 0.070, 0.093 /)
    REAL, PARAMETER :: p_HOCl_40(7) = (/ 0.032, 0.056, 0.074, 0.084, 0.11,  0.12,  0.14  /)
    REAL, PARAMETER :: p_HOCl_50(7) = (/ 0.034, 0.074, 0.089, 0.11,  0.144, 0.184, 0.247 /)
    REAL, PARAMETER :: p_HOCl_60(6) = (/ 0.057, 0.12,  0.19,  0.24,  0.29,  0.40         /)
    ref = "3957"
    type = "M"
    chem = "hypochlorous acid" ; casrn = "7790-92-3" ! HOCl
    CALL LinReg (p_HOCl_30, c_HOCl_30, intercept, slope)
    H30 = slope / (mmHg * dm3)
    CALL LinReg (p_HOCl_40, c_HOCl_40, intercept, slope)
    H40 = slope / (mmHg * dm3)
    CALL LinReg (p_HOCl_50, c_HOCl_50, intercept, slope)
    H50 = slope / (mmHg * dm3)
    CALL LinReg (p_HOCl_60, c_HOCl_60, intercept, slope)
    H60 = slope / (mmHg * dm3)
    !PRINT *, H30, H40, H50, H60
    CALL HTdep((/ 30., 40., 50., 60. /) + CtoK, (/ H30, H40, H50, H60 /), Hominus, mindHR)
    CALL MakeNote("highTextrapol")
    CALL Output(Hominus, mindHR, r2)

  END SUBROUTINE ref3957

  !---------------------------------------------------------------------------

  ! ref3958 NOTUSED: Similar to ref3957 but for chlorate solution.

  !---------------------------------------------------------------------------

  ! ref3959 NOTUSED: Review of Cl2O ("7791-21-1") but no new data.

  !---------------------------------------------------------------------------

  ! ref3960 NOTUSED: Review of HOCl ("7790-92-3") and Cl2O ("7791-21-1")
  !                  but no new data.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3961 ! Hsbp [mol/(kg*atm)]
    IMPLICIT NONE
    REAL :: B

    ref = "3961"
    type = "Q"
    ! H0 and B from Tab. 3:

    chem = "dinitrogen pentoxide" ; casrn = "10102-03-1" ! N2O5
    B = 4338
    CALL MakeNote(TRIM(ref)//"_N2O5", &
      TRIM(citet()) // " applied an empirical correlation between Henry's " // &
      "law solubilities and boiling points from \citet{449}.")
    CALL Output(1.91E-6*EXP(B/T0)*rhoH2O/atm, B)

    chem = "chlorine nitrate" ; casrn = "14545-72-3" ! ClNO3
    B = 4511
    CALL MakeNote(TRIM(ref)//"_ClNO2", &
      TRIM(citet()) // " assumed that the entropy of vaporization is " // &
      "the same for \chem{HOCl} and \chem{ClNO_3} according to Trouton's " // &
      "rule. On their page 3592, they mention a value of 7\,\unit{M\,atm^{-1}} at " // &
      "250\,K. However, checking their Fig.~9 and applying the " // &
      "temperature-dependence equation from their Table~3, it seems that " // &
      "the value of 7\,\unit{M\,atm^{-1}} refers to 298\,K, not 250\,K.")
    CALL Output(1.91E-6*EXP(B/T0)*rhoH2O/atm, B)

  END SUBROUTINE ref3961

  !---------------------------------------------------------------------------

  SUBROUTINE ref3962 ! Hscp [M/atm]
    IMPLICIT NONE
    ref = "3962"
    type = "T"
    chem = "dinitrogen pentoxide" ; casrn = "10102-03-1" ! N2O5
    ! page 3 and Tab. 1:
    CALL MakeNote(TRIM(ref), "Calculated from the solvation free energy.")
    CALL Output(3.*Hscp_TO_HscpSI)
  END SUBROUTINE ref3962

  !---------------------------------------------------------------------------

  SUBROUTINE ref3963 ! Hscp [M/atm]
    IMPLICIT NONE
    ref = "3963"
    type = "T"
    chem = "dinitrogen pentoxide" ; casrn = "10102-03-1" ! N2O5
    ! page 1:
    CALL MakeNote(TRIM(ref), "Calculated from the solvation free energy.")
    CALL Output(0.4*Hscp_TO_HscpSI)
  END SUBROUTINE ref3963

  !---------------------------------------------------------------------------

  SUBROUTINE ref3964 ! Hscp [M/atm]
    IMPLICIT NONE
    ref = "3964"
    type = "T"
    chem = "dinitrogen pentoxide" ; casrn = "10102-03-1" ! N2O5
    ! page 17972:
    CALL MakeNote(TRIM(ref), "Calculated from the solvation free energy.")
    CALL Output(0.5*Hscp_TO_HscpSI)
  END SUBROUTINE ref3964

  !---------------------------------------------------------------------------

  SUBROUTINE ref3965 ! Hvcc [1]
    IMPLICIT NONE

    ref = "3965"
    type = "M"
    chem = "chlorine dioxide" ; casrn = "10049-04-4" ! ClO2
    ALLOCATE(temp(7))
    ! Tab 1:
    temp = (/ 0., 5., 10., 15., 20., 30., 40. /) + CtoK
    CALL HTdep(temp, &
      Hvcc_TO_HscpSI((/ 0.0155, 0.0189, 0.0222, 0.0267, 0.0316, 0.0428, 0.0578 /), &
      temp), Hominus, mindHR)
    DEALLOCATE(temp)
    CALL Output(Hominus, mindHR, r2)

  END SUBROUTINE ref3965

  !---------------------------------------------------------------------------

  SUBROUTINE ref3966 ! Hscc [1]
    IMPLICIT NONE
    ref = "3966"
    type = "M"
    chem = "chlorine dioxide" ; casrn = "10049-04-4" ! ClO2
    ! p. 3331: "The value of the coefficient for the distribution of
    ! chlorine dioxide between gas and water has been measured in these
    ! laboratories. The concentration in solution is 25 times as great
    ! as the concentration in the gas phase at 25°."
    CALL Output(Hscc_TO_HscpSI_atT0*25.)
  END SUBROUTINE ref3966

  !---------------------------------------------------------------------------

  SUBROUTINE ref3967 ! special definition: solubility [g/L] (=[kg/m3])
    IMPLICIT NONE
    ref = "3967"
    type = "M"
    chem = "chlorine dioxide" ; casrn = "10049-04-4" ! ClO2
    ! Data from top of p. 528, assuming here that the solubility refers
    ! to a partial pressure of 1 atm:
    CALL HTdep((/ 0., 25. /) + CtoK, (/ 65., 45. /) / (atm*(MCl+MO*2.)), &
      Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref3967

  !---------------------------------------------------------------------------

  ! ref3968 NOTUSED: Review of ClO2 but no new data.

  !---------------------------------------------------------------------------

  SUBROUTINE ref3969 ! special definition: solubility [g/L] (=[kg/m3])
                     !                     and partial pressure [mmHg]
    IMPLICIT NONE
    REAL :: intercept, slope, H25, H40, H60
    ! Tabs. 3, 4, 5 on p. 541:
    REAL, PARAMETER :: p_ClO2_25(4) = (/  34.5, 22.1, 13.4,  8.4 /)
    REAL, PARAMETER :: p_ClO2_40(4) = (/  56.2, 34.3, 18.9,  9.9 /)
    REAL, PARAMETER :: p_ClO2_60(4) = (/ 105.9, 53.7, 21.3, 12.0 /)
    REAL, PARAMETER :: c_ClO2_25(4) = (/  3.01, 1.82, 1.13, 0.69 /)
    REAL, PARAMETER :: c_ClO2_40(4) = (/  2.63, 1.60, 0.83, 0.47 /)
    REAL, PARAMETER :: c_ClO2_60(4) = (/  2.65, 1.18, 0.58, 0.26 /)
    REAL, PARAMETER :: MClO2 = MCl+MO*2. ! kg/mol

    ref = "3969"
    type = "M"
    chem = "chlorine dioxide" ; casrn = "10049-04-4" ! ClO2
    ! calculate Hscp at each temperature via linear regression of c vs p:
    CALL LinReg (p_ClO2_25, c_ClO2_25, intercept, slope)
    H25 = slope / (mmHg * MClO2)
    CALL LinReg (p_ClO2_40, c_ClO2_40, intercept, slope)
    H40 = slope / (mmHg * MClO2)
    CALL LinReg (p_ClO2_60, c_ClO2_60, intercept, slope)
    H60 = slope / (mmHg * MClO2)
    CALL HTdep((/ 25., 40., 60. /) + CtoK, (/ H25, H40, H60 /), Hominus, mindHR)
    CALL Output(Hominus, mindHR, r2)

  END SUBROUTINE ref3969

  !---------------------------------------------------------------------------
  
  ! ref4010 NOTUSED: Data from this PhD were later published in ref3694
  !                  and ref4008
  
  !---------------------------------------------------------------------------

  SUBROUTINE ref4017 ! special definition (DeltaG for 1/Hsbp = Hvpb in [atm*kg/mol])

    IMPLICIT NONE
    ref = "4017"
    type = "M"
    chem = "hydrazoic acid" ; casrn = "7782-79-8"
    ! Concentration and vapor pressure were determined experimentally, and results
    ! were presented in Tab. 1 as free energies Delta G.
    ALLOCATE(temp(4))
    temp = (/ 0.0, 24.42, 35.95, 49.46 /) + CtoK
    CALL HTdep(temp, &
      Hsbp_TO_HscpSI / EXP( (/ -2.05, -1.47, -1.20, -0.94 /) *kcal/(Rgas*temp)), &
      Hominus, mindHR)
    DEALLOCATE(temp)
    CALL Output(Hominus, mindHR, r2)
  END SUBROUTINE ref4017

  !---------------------------------------------------------------------------

  SUBROUTINE ref4023 ! Hscp [M/atm]
    IMPLICIT NONE
    ref = "4023"
    type = "M"
    chem = "methanoic acid" ; casrn = "64-18-6" ! HCOOH formic acid
    Hominus = 7.4E3 * Hscp_TO_HscpSI ! pp. 14, 77 and 137
    CALL MakeNoteOtherTempC(24.) ! p. 76
    CALL Output(Hominus)
  END SUBROUTINE ref4023

  !---------------------------------------------------------------------------

END MODULE henry_ref4000

!*****************************************************************************
!                                  end of file
!*****************************************************************************
